-- 参考 cocos 的事件分发的 lua 代码

return class {
    typename = "Event", 

    ctor = function (self)
        self.next_id = 0
        self.listeners = {}
    end, 

    on = function (self, name, listener)
        return self:add(name, listener)
    end, 

    off = function (self, name, listener)
        return self:remove(name, listener)
    end, 

    add = function (self, name, listener)
        local list = self.listeners[name]
        if list == nil then 
            list = {}
            self.listeners[name] = list
        end

        local id = tostring(self.next_id)
        self.next_id = self.next_id + 1
        list[id] = listener
        return id
    end, 

    remove = function (self, name, del_id)
        local list = self.listeners[name]
        if list == nil then 
            return false 
        end

        if type(del_id) == "string" then 
            for id, listener in pairs(list) do
                if id == del_id then 
                    list[id] = nil
                    return true
                end
            end
        else
            for id, listener in pairs(list) do
                if listener == del_id then 
                    list[id] = nil
                    return true
                end
            end
        end
        return false
    end, 

    removeAll = function(self, name)
        self.listeners[name] = nil
    end,

    clear = function (self)
        self.listeners = {}
    end, 

    dispatch = function (self, name, ...)
        local list = self.listeners[name]
        if list == nil then return end

        for id, listener in pairs(list) do
            listener(...)
        end
    end, 
}
