--[[
    使用范例
    print(locale:get("TEST"))  -- 从默认文件base 加载
    print(locale:get("TEST", "gui")) -- 从指定文件gui 加载
    简写方法
    print(L"TEST")          -- 等价于 locale:get("TEST")
    print(L.TEST)           -- 等价于 locale:get("TEST")
    print(L("TEST", "gui")) -- 等价于 locale:get("TEST", "gui")
]]
return setmetatable({
    languages = {"zhCN", "zhTW", "enUS", "frFR"},
    language = "zhCN", 
    relativePath = "localizable/", 
    bundles = {},

    reload = function (self)
        for k, v in pairs(self.bundles) do 
            self:loadBundle(k)
        end
    end, 

    get = function (self, strid, bundleName)
        if bundleName == nil then 
            bundleName = "base"
        end

        local bundle = self.bundles[bundleName]
        if bundle == nil then 
            bundle = self:loadBundle(bundleName)
        end
        return bundle[strid] or strid
    end,

    loadBundle = function (self, bundleName)
        local fullPath = self.relativePath .. bundleName .. "-" .. self.language .. ".lua"
        local fileData = dofile(fullPath) or {}
        self.bundles[bundleName] = fileData 
        return fileData
    end,

    foreach = function (self, callback)
        for k, v in ipairs(self.languages) do
            callback(v)
        end
    end,
}, {
    __call = function (self, name, bundle)
        return self:get(name, bundle)
    end, 
    __index = function (self, key)
        return self:get(key)
    end, 
})
