--[[
	local tcp = class {
		super = require("base.tcp"), 

		onInit = function (self, args)
			self.map = {
				-- 协议类型映射
			}
		end, 

		createInstance = function (self, cmd)
			local pb = self.map and self.map[cmd]
			return pb and pb()
		end, 
	}.new()

	tcp:on("connect", function ()
		print("连接成功")
	end)

	tcp:on("close", function ()
		print("连接关闭")
	end)

	tcp:on("connect", function ()
		print("连接成功")
	end)

	tcp:on("message", function (cmd, msg)
		print("收到协议", cmd, msg)
	end)

	tcp:connect("127.0.0.1", 9192)

	-- !! protobuf 协议 tips !!
	-- 通过 getmetatable(msg)._descriptor.name 可以得到协议对象的协议名字
]]

return class {
	ctor = function (self, args)
		local functor = require("base.functor")
		self._reconnect_list = {}
		self.state = "close"
		self.events = require("base.event").new()
		self.tcp = Fenix.Net.TcpConnection()
		self.tcp:Register(Fenix.App.GetInstance())
		self.tcp.MessageReceived = functor.make(self.onReceive, self)
		self.tcp.Connected = functor.make(self.onConnect, self)
		self.tcp.Closed = functor.make(self.onClose, self)
		self:onInit(args)

		if args ~= nil then 
			if args.ip ~= nil and args.port ~= nil then 
				self:connect(args.ip, args.port)
			end
		end
	end,

	on = function (self, event, handler)
		return self.events:add(event, handler)
	end, 

	off = function (self, event, handler)
		return self.events:remove(event, handler)
	end, 

	onInit = function (self, args)
	end, 

	onConnect = function (self)
		assert(self.state == "begin")
		self.state = "connect"
		self.events:dispatch("connect")
	end,

	onClose = function (self)
		if self.state == "connect" then 
			self.events:dispatch("disconnect")
		end

		local refused = self.state == "begin"
		self.state = "close"
		self.events:dispatch("close", refused)
	end,

	connect = function (self, ip, port)

		print('tcp connect!')
		
		if self.state ~= "close" then 
			return false
		end
		self.state = "begin"
		self.tcp:BeginConnect(ip, port)
		return true
	end,

	close = function (self)
		if self.state ~= "close" then 
			self.tcp:Close()
		end
	end,

	onReceive = function (self, cmd, data)
		--TODO: 通过协议id创建对象
		 --print("onReceive:"..cmd)
		self._reconnect_list[cmd] = nil
		local obj = self:createInstance(cmd)
		if obj ~= nil then 
			obj:ParseFromString(Slua.ToString(data))
			self.events:dispatch(cmd, obj)
			self.events:dispatch("message", cmd, obj)
		else
			print("fail to parse", cmd, data and #data)
		end
	end,

	createInstance = function (self, cmd)
	end, 

	send = function (self, cmd, obj, is_resend)
		if cmd then 
			if self.state == "connect" then 
				self._reconnect_list[cmd] = {cmd = cmd, obj = obj}
				if obj then 
					local data = obj:SerializeToString()
					self.tcp:Send(cmd, data and Slua.ToBytes(data), is_resend)
				else
					self.tcp:Send(cmd, nil, is_resend)
				end
			end
		end
	end,

	onReconnect = function(self)
		for cmd, unit in pairs(self._reconnect_list) do
			self:send(unit.cmd, unit.obj)
		end
	end,
}
