local __tests = {}
local anonymous_str = "<anonymous-class>"
local log = logger.get("Debug", "UnitTest")

local __traceback__ = function (...)

end

return {
    testable = function (methods)
        return function (cls)
            table.insert(__tests, { class = cls, methods = methods })
        end
    end,

    runtests = function (cls_only)
        for k, v in ipairs(__tests) do 
            if cls_only == nil or v.class == cls_only then 
                local class_typename = v.class.typename or anonymous_str
                log.info("===== TESTS FOR:", class_typename, "=====")
                local passed = 0
                local total = 0
                local inst = v.class.new()
                for method_name, method in pairs(v.methods) do 
                    local time = Time.realtimeSinceStartup
                    local status, result = xpcall(function () method(inst) end, function (errmsg)
                        local time_str = string.format("[%.3fms]", (Time.realtimeSinceStartup - time) * 1000)
                        log.error(time_str, class_typename .. "." .. method_name, ":fail **", errmsg, "\n", debug.traceback())
                    end)
                    total = total + 1
                    if status then 
                        passed = passed + 1
                        local time_str = string.format("[%.3fms]", (Time.realtimeSinceStartup - time) * 1000)
                        log.info(time_str, class_typename .. "." .. method_name, ":ok")
                    end
                end
                log.info("===== TESTS END:", class_typename, "(" .. passed .. "/" .. total .. ")", "=====")
            end
        end
    end, 

    deprecated = function (arg1, arg2) 
        local tp = type(arg1)
        if tp == "table" then 
            return function (cls)
                for k, v in pairs(arg1) do 
                    cls[k] = function (...)
                        log.warn((cls.typename or anonymous_str) .. "." .. k, " is deprecated")
                        v(...)
                    end
                end
            end
        else
            return function (cls)
                local v = cls[arg1]
                cls[arg1] = function (...)
                    log.warn((cls.typename or anonymous_str) .. "." .. arg1, " is deprecated.", arg2 or "")
                    v(...)
                end
            end
        end
    end, 
}
