local log = logger.get("Game", "BT")
local super = require("bt.sequence")
local states = super.states

-- 依次执行子结点，发现有失败就返回失败。执行完成后返回成功
return class {
    typename = "循环结点", 
    super = super, 

    ctor = function (self, ...)
        local log_id = select(2, ...)
        super.ctor(self, log_id)
        self.inited = false
        self.loop_count_param = select(1, ...)
        self.loop_count = nil
        self.loop_idx = 1
    end,

    setContext = function (self, ctx)
        self.ctx = ctx
        super.setContext(self, ctx)

        if type(self.loop_idx) == "string" and ctx.skill_cfg then
            self.loop_idx =  ctx.skill_cfg[self.loop_idx] or self.loop_idx
        end
    end, 

    update = function (self, dt)
        if not self.inited then
            self.inited = true
            self.loop_idx = 1

            if type(self.loop_count_param) == 'function' then
                self.loop_count = self.loop_count_param()
            else
                self.loop_count = self.loop_count_param
            end
        end

        -- 执行 sequence 的 update
        local rst = super.update(self, dt)
        if rst == states.FAILURE then
            self.loop_idx = 1
            return states.FAILURE
        elseif rst == states.RUNNING then
            return states.RUNNING
        elseif rst == states.SUCCESS then
            local loop_count = self.loop_count
            if type(loop_count) == 'function' then
                loop_count = loop_count()
            end

            if self.loop_idx == loop_count then
                self.loop_idx = 1
                return states.SUCCESS
            else
                self.loop_idx = self.loop_idx + 1
                return states.RUNNING
            end
        end

    end, 

    clone = function (self)
        local inst = self.class.new()
        for _, child in ipairs(self.children) do 
            inst:add(child:clone())
        end
        return inst
    end, 
}