local log = logger.get("Game", "BT")
local super = require("bt.node")
local states = super.states

return class {
    typename = "根节点", 
    super = super, 

    enter = function (self)
        self.child = nil    -- 唯一子结点
        self.state = states.SUCCESS -- 当前执行状态
    end, 

    setChild = function(self, child)
        self.child = child
        return self
    end,

    setContext = function (self, ctx)
       self.ctx = ctx
       if self.child ~= nil then
        self.child:setContext(ctx)
       end
    end, 

    -- 重置
    reset = function(self)
        self.child:reset()
    end,

    -- 执行一次逻辑
    update = function(self, dt, steps)
        --log.debug('root update')

        if not self.child then
            log.error('行为树根结点必须要指定一个子结点')
            return states.FAILURE
        end

        steps = steps or 1
        dt = dt / steps
        for i = 1, steps, 1 do
            self.state = self.child:update(dt)
            if self.state ~= states.RUNNING then 
                return self.state
            end
        end
        return self.state
    end,
}