return {

    RegisterClick = function (self, name, callback)
        local handler = self:_getWrapper(name, UnityEngine.UI.Button) 
        if handler then 
            self.rec_events.click[name] = true  -- 添加到事件列表

            handler.onClick:AddListener(function ()
                -- 通知新手引导，有界面元素被按下
                if guide_helper then
                    guide_helper:onClick(name)
                end
                SoundEngine.PostEvent("Play_ui_click", app.gameObject)

                callback(self, handler)
            end)
        end
        return handler
    end, 

    UnregisterClick = function (self, name)
        local handler = self:_getWrapper(name, UnityEngine.UI.Button) 
        if handler then 
            handler.onClick:RemoveAllListeners()
        end
        self.rec_events.click[name] = false
        return handler
    end, 

    RegisterDrag = function (self, name, callback)
        local handler = self:_getWrapper(name, Fenix.UI.DragHandler, true)
        if handler then 
            self.rec_events.drag[name] = true

            handler:AddBeginListener(function (eventData)
                callback(self, handler, eventData, "begin")
            end)
            handler:AddDragListener(function (eventData)
                callback(self, handler, eventData, "drag")
            end)
            handler:AddEndListener(function (eventData)
                callback(self, handler, eventData, "end")
            end)
        end
        return handler
    end,

    UnregisterDrag = function (self, name, callback)
        local handler = self:_getWrapper(name, Fenix.UI.DragHandler, true)
        if handler then 
            handler:ClearBeginListener()
            handler:ClearDragListener()
            handler:ClearEndListener()
        end
        self.rec_events.drag[name] = false
        return handler
    end,

    RegisterDrop = function (self, name, callback)
        local handler = self:_getWrapper(name, Fenix.UI.DropHandler, true)
        if handler then 
            self.rec_events.drop[name] = true
            handler:AddDropListener(function (eventData)
                callback(self, handler, eventData)
            end)
        end
        return handler
    end,

    UnregisterDrop = function (self, name, callback)
        local handler = self:_getWrapper(name, Fenix.UI.DropHandler, true)
        if handler then 
            handler:ClearDropListener()
        end
        self.rec_events.drop[name] = false
        return handler
    end,

    RegisterUpDown = function (self, name, callback)
        local handler = self:_getWrapper(name, Fenix.UI.PointerHandler, true)
        if handler then 
            self.rec_events.updown[name] = true
            handler:AddUpListener(function (eventData)
                callback(self, handler, eventData, "up")
            end)

            handler:AddDownListener(function (eventData)
                callback(self, handler, eventData, "down")
            end)
        end
        return handler
    end,

    UnregisterUpDown = function (self, name, callback)
        local handler = self:_getWrapper(name, Fenix.UI.PointerHandler, true)
        if handler then 
            handler:ClearUpListener()
            handler:ClearDownListener()
        end
        self.rec_events.updown[name] = false
        return handler
    end,

    RegisterSingleClick = function (self, name, callback)
        local handler = self:_getWrapper(name, Fenix.UI.PointerHandler, true)
        if handler then 
            self.rec_events.singleclick[name] = true
            handler:AddClickListener(function (eventData)
                if guide_helper then
                    guide_helper:onClick(name)
                end
                SoundEngine.PostEvent("Play_ui_click", app.gameObject)

                callback(self, handler, eventData)
            end)
        end
        return handler
    end,

    UnregisterSingleClick = function (self, name, callback)
        local handler = self:_getWrapper(name, Fenix.UI.PointerHandler, true)
        if handler then 
            handler:ClearClickListener()
        end
        self.rec_events.singleclick[name] = false
        return handler
    end,

    RegisterDoubleClick = function (self, name, callback)
        local handler = self:_getWrapper(name, Fenix.UI.PointerHandler, true)
        if handler then 
            self.rec_events.doubleclick[name] = true
            handler:AddDoubleClickListener(function (eventData)
                callback(self, handler, eventData)
            end)
        end
        self.rec_events.singleclick[name] = false
        return handler
    end,

    UnregisterDoubleClick = function (self, name, callback)
        local handler = self:_getWrapper(name, Fenix.UI.PointerHandler, true)
        if handler then 
            handler:ClearDoubleClickListener()
        end
        self.rec_events.doubleclick[name] = false
        return handler
    end,

    RegisterHover = function (self, name, callback)
        local handler = self:_getWrapper(name, Fenix.UI.HoverHandler, true)
        if handler then 
            self.rec_events.hover[name] = true
            handler:AddEnterListener(function (eventData)
                callback(self, handler, eventData, "enter")
            end)
            handler:AddExitListener(function (eventData)
                callback(self, handler, eventData, "exit")
            end)
        end
        return handler
    end, 

    UnregisterHover = function (self, name, callback)
        local handler = self:_getWrapper(name, Fenix.UI.HoverHandler, true)
        if handler then 
            handler:ClearEnterListener()
            handler:ClearExitListener()
        end
        self.rec_events.hover[name] = false
        return handler
    end, 

    RegisterScroll = function (self, name, callback)
        local handler = self:_getWrapper(name, Fenix.UI.ScrollHandler, true)
        if handler then 
            self.rec_events.scroll[name] = true
            handler:AddScrollListener(function (eventData)
                callback(self, handler, eventData)
            end)
        end
        return handler
    end, 

    UnregisterScroll = function (self, name, callback)
        local handler = self:_getWrapper(name, Fenix.UI.ScrollHandler, true)
        if handler then 
            handler:ClearScrollListener()
        end
        self.rec_events.scroll[name] = false
        return handler
    end, 
}