local log = logger.get("Game", "Actor State")
local super = require("common.state")
local actor_types = require('RTS.enums').actor_types

-- 角色状态基类
return class {
    super = super,
    ctor = function(self, ...)
        super.ctor(self, ...)

        self.cs_inst = self.owner.cs_inst
        self.animation = self.owner.animation    -- 动画组件
        self.anim_names = self.owner.anim_names  -- 动画名称表

        self.check_postfix = true   -- 是否检查 postfix 发生了变化
        self.anim_postfix = nil
        self.check_anim_elapsed = 0.0
        self.marks = {}
    end,

    onUpdate = function(self, dt)
        if self.owner.anim_loop and self.owner.anim_postfix then
            self.check_anim_elapsed = self.check_anim_elapsed + dt
            if self.check_anim_elapsed > 0.5 then
                local postfix = self.owner:getAnimPostfix()
                if postfix ~= self.owner.anim_postfix then
                    self:playAnimation(self.owner.orig_anim_name, true)
                end
                self.check_anim_elapsed = 0.0
            end
        end
    end,

    -- 播放动画
    playAnimation = function(self, anim, loop, random_postfix)
        --[[
        if self.owner.team == 1 then
            log.info("#### play anim:"..anim)
            log.info(debug.traceback())
        end]]

        local _anim_name = anim
        local postfix = self.owner:getAnimPostfix(random_postfix)
        if postfix then
            _anim_name = postfix .. _anim_name 
            self.owner.anim_postfix = postfix
        end
        if self.owner.anim_loop and self.owner.anim_name == _anim_name then
            return
        end
        self.owner.cs_inst:PlayAnimation(_anim_name)

        self.owner.orig_anim_name = anim
        self.owner.anim_name = _anim_name
        self.owner.anim_loop = loop
    end,

    -- 移动到目标
    moveTo = function(self, pos)
        self.cs_inst:SetDestination(pos)
    end,

    -- 停止移动
    stopMove = function(self)
        self.cs_inst:StopMove()
    end,

}