local log = logger.get("Game", "Actor State")
local super = require('RTS.actor_states.actor_state_base')
local actor_types = require('RTS.enums').actor_types

-- 死亡状态
return class {
    super = super,
    onEnter = function(self, args)
        self.state_elapsed = 0.0
         -- 死亡动画
        if self.owner.actor_type == actor_types.SOLDIER then
            self:playAnimation(self.anim_names.die, false, true)
        elseif self.owner.actor_type == actor_types.CANNON then
            log.info("cannon enter state die, pos:"..tostring(self.owner:getPosition()))

            self:playAnimation(self.anim_names.die, false, true)
            -- 受击特效
            battle_helper:playFXOnActorPos(self.owner.cfg.die_effect, self.owner, self.owner:getPosition(), 5, true)
        else
            -- 受击特效
            battle_helper:playFXOnActorPos(self.owner.cfg.die_effect, self.owner, self.owner:getPosition(), 5, true)
        end

        self:stopMove()
        self.owner.cs_inst:Dead()
        self.die_delay = self.owner.cfg.die_delay / 1000
    end,

    onUpdate = function(self, dt)
        if self.state_elapsed < self.die_delay then
            self.state_elapsed = self.state_elapsed + dt
            if self.state_elapsed >= self.die_delay then
                -- 将 damage text 脱离结点
                for k,v in ipairs(self.owner.damage_text) do
                    v.transform:SetParent(nil)
                end

                self.owner:hideGameObject()
                self.state_elapsed = -10.0
            end
        end
    end,

    canChangeTo = function(self, next_state_name, ...)
        return false
    end,

    onExit = function(self)
        --log.error("state die onExit!")
    end,
}