local log = logger.get("Game", "Actor State skill")
local super = require('RTS.actor_states.actor_state_base')
local bullet_class = require('RTS.rts_bullet')
local actor_types = require('RTS.enums').actor_types
local bt_states = require('bt.node').states

-- 技能施法阶段,time scale 为 0
return class {
    super = super,
    onEnter = function(self, args)
        self.skill_id = args.id
        log.info('Enter state preskill:'..self.skill_id)
        self.skill_cfg = config.skill_skills[self.skill_id]
        if not self.skill_cfg then
            log.error('Failed to find skill cfg:'..tostring(self.skill_id))
        end

        self.owner.skill_cfg = self.skill_cfg   -- 将当前技能记录到角色上
        local bt_id = self.skill_cfg.pre_bt_id or -101 -- 行为树 id

        self.is_finished = false

        -- 创建技能行为树
        local bt_fun = require('RTS.skill_lib._skill_bt')[bt_id]
        if not bt_fun then 
            log.error('failed to find skill_bt:'..tostring(bt_id)..'in skill:'..self.skill_id)
        end
        self.bt = bt_fun(self.owner)

        if self.bt == nil then
            log.error('failed to find skill bt:'..tostring(self.skill_id))
        end

   end,

    -- 播放动画
    playAnimation = function(self, anim, loop)
        self.owner.cs_inst:PlayUnscaledTimeAnimation(anim)
    end,

    onUpdate = function(self, _)
        local dt = Time.unscaledDeltaTime
        if not self.is_finished then 
            local rst = self.bt:update(dt, 2)
            if rst == bt_states.SUCCESS or rst == bt_states.FAILURE then
                self:finished()
            end
        end
    end,

    canChangeTo = function(self, next_state_name, args) 
        return true
    end,

    finished = function(self)
        self.is_finished = true
        self.owner:changeState('idle')
    end,

    setContext = function (self, ctx)
       self.ctx = ctx
       if self.child ~= nil then
        self.child:setContext(ctx)
       end
    end, 

    -- 重置
    reset = function(self)
        self.child:reset()
    end,

    onExit = function(self)
        self.owner.cs_inst:ReleaseUnscaledVFX()
        self.skill_id = 0
        self.owner.skill_cfg = nil

        if self.bt then
            self.bt:reset()
            self.bt = nil
        end

        battle_helper:setCameraSizeFactor(1.0, 1.0)
    end,
}