local log = logger.get("Game", "Actor State skill")
local super = require('RTS.actor_states.actor_state_base')
local bullet_class = require('RTS.rts_bullet')
local actor_types = require('RTS.enums').actor_types
local bt_states = require('bt.node').states

-- 技能状态
return class {
    super = super,
    onEnter = function(self, args)

        self.skill_id = args.id

        -- 存在一个或多个目标
        self.target_actors = args.ta

        self.skill_cfg = config.skill_skills[self.skill_id]
        if not self.skill_cfg then
            log.error("找不到技能配置文件", self.skill_id)
        end

        self.owner.skill_cfg = self.skill_cfg   -- 将当前技能记录到角色上
        local bt_id = self.skill_cfg.bt_id or 1 -- 行为树 id

        self.is_finished = false
        self.state_elapsed = 0.0

        -- 创建技能行为树
        local bt_fun = require('RTS.skill_lib._skill_bt')[bt_id]
        if not bt_fun then 
            log.error("找不到技能行为树#", bt_id, self.skill_id)
        end
        self.bt = bt_fun(self.owner)

        if self.bt == nil then
            log.error("找不到技能行为树", self.skill_id)
        end

        self:stopMove()
        self.casting_complete = false
    end,

    onUpdate = function(self, dt)
        self.state_elapsed = self.state_elapsed + dt
        if self.state_elapsed > 30 then
            log.info('释放技能超时 30scd !!')
        end

        if not self.is_finished then 
            local rst = self.bt:update(dt, 2)
            if rst == bt_states.SUCCESS or rst == bt_states.FAILURE then
                self:finished()
            end
        end
    end,

    canChangeTo = function(self, next_state_name, args) 
        if next_state_name == "skill_tb" then
            return true
        else
            return true
        end
    end,

    finished = function(self)
        self.is_finished = true
        self.owner:changeState('idle')
    end,

    setContext = function (self, ctx)
       self.ctx = ctx
       if self.child ~= nil then
        self.child:setContext(ctx)
       end
    end, 

    -- 重置
    reset = function(self)
        self.child:reset()
    end,

    onExit = function(self)
        self.skill_id = 0
        self.target_actors = nil
        self.skill_position = nil
        self.skill_rotation = nil
        self.owner.skill_cfg = nil

        if self.bt then
            self.bt:reset()
            self.bt = nil
        end
    end,
}