local log = logger.get("Game", "Group State")
local super = require('RTS.actor_states.actor_state_base')

-- 士兵组 跟随目标，按照 cd 使用攻击技能 (目标可能是宿主坦克，攻击就是加血技能)
return class {
    super = super,

    onEnter = function(self, args)
        if not args then
            info.error('move to and attack, target is nil')
            return
        end

        if self.owner.target_actor ~= args then
            self.owner:ai_setTargetActor(args)
        end

        self.target_actor = self.owner.target_actor
        self.soldiers = self.owner.soldiers

        -- 会反复执行 onEnter, 防止数值被重复初始化
        if not self.check_elapsed then
            self.check_elapsed = 0.1
            self.check_time = math.random() * 0.1 + 0.1
            self.check_idx = 1
        end
    end,

    onUpdate = function(self, dt)
        self.check_elapsed = self.check_elapsed + dt
        if self.check_elapsed >= self.check_time then 
            -- 判断 soldier 的 target 是否合法
            if self.target_actor:isEscape() then 
                self.owner:clearTargetActor()
                return
            end

            if self.check_idx > #self.soldiers then
                self.check_idx = 1
            end

            -- 判断 每个 soldier 应该'向敌人移动'还是'进行攻击'
            local soldier = self.soldiers[self.check_idx]
            if soldier:isAlive() then
                
               local is_attack = false 
               if not soldier:isInSkillCD(soldier:getCommonAtkID()) then 
                    if soldier:castSkill(soldier:getCommonAtkID(), soldier.target_actor) then
                        is_attack = true
                    end
                end
                if not is_attack and soldier.target_actor then
                    local offset_dir = soldier.offset
                    offset_dir:Normalize()
                    offset_dir = offset_dir * (soldier.attack_range + (soldier.target_actor.radius or 1))
                    local target = soldier.target_actor:getPosition() + offset_dir
                    if Vector3.Distance(target,soldier:getPosition()) > (soldier.radius + 0.2) then
                        soldier:changeState('move', soldier.target_actor:getPosition() + soldier.offset)
                    else
                        --log.info('############# idle!!')
                        soldier:changeState('idle')
                    end
                end
            end

            self.check_elapsed = 0
            self.check_time = math.random() * 0.1 + 0.1
            self.check_idx = self.check_idx + 1
        end
    end,

    onExit = function(self)
    end,
}