local log = logger.get("Game", "AI BT")
local actor_types = require('RTS.enums').actor_types
local bt_manager = require('bt.manager')
local root = require('bt.root')
local sequence = require("bt.sequence")
local loop = require("bt.loop")
local loop_until = require("bt.loop_until")
local select = require("bt.select")
local parallel = require("bt.parallel")
local delay = require("bt.delay")
local action = require("bt.action")
local condition = require("bt.condition")
local inverse = require("bt.inverse")
local random_select = require("bt.random_select")
local random_node = require("bt.random_node")

-- only_attackable-是否只选择可攻击的敌人, keep_old_if_nil-如果没选择到新的，是否保持原敌人
local select_enemy = require('RTS.ai_lib.select_enemy')
local move_to_enemy = require('RTS.ai_lib.move_to_enemy')
local attack_enemy = require('RTS.ai_lib.attack_enemy') -- 使用普攻技能
local move_to_enemy_and_attack = require('RTS.ai_lib.move_to_enemy_and_attack')
local idle = require('RTS.ai_lib.idle')
local delay_attack_cd = require('RTS.ai_lib.delay_attack_cd')
--local escape = require('RTS.ai_lib.escape')
--local follow_owner_and_attack = require('RTS.ai_lib.follow_owner_and_attack')
local is_enemy_in_attack_range = require('RTS.ai_lib.is_enemy_in_attack_range')
local is_enemy_valid = require('RTS.ai_lib.is_enemy_valid')
local is_owner_dead = require('RTS.ai_lib.is_owner_dead')
local cast_skill = require('RTS.ai_lib.cast_skill') -- 释放技能，参数 skil_id 不填是默认技能
local cast_opening_skill = require('RTS.ai_lib.cast_opening_skill')
local add_to_camera_view = require('RTS.ai_lib.add_to_camera_view')
local remove_from_camera_view = require('RTS.ai_lib.remove_from_camera_view')
local is_enemy_in_min_attack_range = require('RTS.ai_lib.is_enemy_in_min_attack_range')
local is_enemy_out_safe_range = require('RTS.ai_lib.is_enemy_out_safe_range')
local move_away_from_enemy = require('RTS.ai_lib.move_away_from_enemy')
local facing_target = require('RTS.ai_lib.facing_target')
local is_auto = require('RTS.ai_lib.is_auto')   -- 是否自动战斗
local timer_begin = require('RTS.ai_lib.timer_begin')   -- 开始计时器 init_value - 初始值
local is_timer_bigger_than = require('RTS.ai_lib.is_timer_bigger_than') -- 判断计时器是否超过某值 time_value 比较值
-- 标记操作
local add_mark = require('RTS.ai_lib.mark.add_mark')
local del_mark = require('RTS.ai_lib.mark.del_mark')
local inc_mark = require('RTS.ai_lib.mark.inc_mark')
local dec_mark = require('RTS.ai_lib.mark.dec_mark')
local set_mark = require('RTS.ai_lib.mark.set_mark')
local is_mark_exist = require('RTS.ai_lib.mark.is_mark_exist')
local is_mark_equal = require('RTS.ai_lib.mark.is_mark_equal')
local is_mark_greater_than = require('RTS.ai_lib.mark.is_mark_greater_than')
local is_mark_less_than = require('RTS.ai_lib.mark.is_mark_less_than')
local print_marks = require('RTS.ai_lib.mark.print_marks')

return {

    -- key 配置在 actor 表上
    -- 通用行为树
    common = function(agent)
        if not agent then 
            log.error('agent is nil!!')
            return nil
        end

        local bt = root():setChild(
                select()   -- 选择
                    :add(sequence() -- 存在合法敌人
                        :add(is_enemy_valid())
                        :add(select()
                            :add(sequence() -- 敌人在攻击范围内
                                :add(is_enemy_in_attack_range())
                                :add(attack_enemy()))
                            :add(sequence() -- 敌人不在范围内
                                :add(move_to_enemy())
                                :add(delay(0.1, 0.3)))))
                    :add(sequence() -- 不存在合法敌人
                        :add(idle())  -- 待机
                        :add(select_enemy(false)) -- 只选择车辆类型敌人
                        :add(delay(0.1, 0.3))))

        bt:setContext(agent)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,

    -- 坦克型战车行为树
    tank = function(agent)
         if not agent then 
            log.error('agent is nil!!')
            return nil
        end

        local bt = root():setChild(
                select()   -- 选择
                    :add(sequence() -- 存在合法敌人
                        :add(is_enemy_valid())
                        :add(select()
                            :add(sequence() -- 敌人在攻击范围内
                                :add(is_enemy_in_attack_range())
								:add(select()
									:add(sequence() --是否是自动战斗
										:add(is_auto())
                                        :add(cast_skill())
                                        :add(delay(1.0)))
									:add(select()
										:add(sequence()
											:add(is_enemy_valid())
											:add(attack_enemy()))
										:add(sequence()	
											:add(idle())))))
							:add(select()
								:add(sequence() -- 敌人不在范围内
									:add(inverse(is_mark_exist('move_shooten', false)))
									:add(delay(1))
									:add(cast_opening_skill())
									:add(add_mark('move_shooten'), false))
								:add(sequence()
									:add(move_to_enemy())
									:add(delay(0.1, 0.3)))
									)))
                    :add(sequence() -- 不存在合法敌人
                        :add(idle())  -- 待机
                        :add(select_enemy(false)) -- 只选择车辆类型敌人
                        :add(delay(0.1, 0.3))))

        bt:setContext(agent)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt       
    end,

    --肉盾型战车行为树
    heavy = function(agent)
         if not agent then 
            log.error('agent is nil!!')
            return nil
        end

        local bt = root():setChild(
                select()   -- 选择
                    :add(sequence() -- 存在合法敌人
                        :add(is_enemy_valid())
                        :add(select()
                            :add(sequence() -- 敌人在攻击范围内
                                :add(is_enemy_in_attack_range())
								:add(select()
									:add(sequence() --是否是自动战斗
										:add(is_auto())
                                        :add(cast_skill())
                                        :add(delay(1.0)))
									:add(select()
										:add(attack_enemy())
										--:add(idle()))))	
										:add(facing_target()))))	
                            :add(select()
                                :add(sequence()
                                    :add(inverse(is_mark_exist('heavy_rushed', false)))
                                    :add(delay(1))
                                    :add(select_enemy(false)) -- 只选择车辆类型敌人
                                    :add(cast_opening_skill())
                                    :add(add_mark('heavy_rushed'), false))
                                :add(sequence()
                                    :add(move_to_enemy())
                                    :add(delay(0.1, 0.1))))))
                    :add(sequence() -- 不存在合法敌人
                        :add(idle())  -- 待机
                        :add(select_enemy(false)) -- 只选择车辆类型敌人
                        :add(delay(0.1, 0.1))))

        bt:setContext(agent)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt       
    end,

    -- 突击型战车行为树
    assult = function(agent)
         if not agent then 
            log.error('agent is nil!!')
            return nil
        end

        local bt = root():setChild(
                select()   -- 选择
                    :add(sequence() -- 存在合法敌人
                        :add(is_enemy_valid())
                        :add(select()
                           :add(sequence() -- 敌人在最大攻击范围内
                                :add(is_enemy_in_attack_range())
								:add(select() 
									:add(sequence() --敌人在最小射程之内
										:add(is_enemy_in_min_attack_range())									
											:add(select()
												:add(sequence() --攻击上下限范围内是否有敌人
													:add(select_enemy(true,true)))
												:add(sequence()	
													:add(move_away_from_enemy())    --远离目标	
													:add(loop_until(is_enemy_out_safe_range())  --直到超出了安全距离
														:add(move_away_from_enemy()))
													:add(delay(0.1))))													
										:add(select()
											:add(sequence() --是否是自动战斗
												:add(is_auto())
												:add(cast_skill())
												:add(delay(1.0)))
											:add(select()
												:add(sequence()
													:add(is_enemy_valid())
													:add(attack_enemy()))
												:add(sequence()	
													:add(idle())))))	
									:add(sequence()--敌人不在最小射程之内，即在有效射程
										:add(select()
											:add(sequence() --是否是自动战斗
												:add(is_auto())
												:add(cast_skill())
												:add(delay(1.0)))
											:add(select()
												:add(sequence()
													:add(is_enemy_valid())
													:add(attack_enemy()))
												:add(sequence()	
													:add(idle())))))))							
                            :add(select()
                                :add(sequence()
                                    :add(inverse(is_mark_exist('assult_rushed', false)))
                                    :add(delay(1))
                                    :add(select_enemy(false)) -- 只选择车辆类型敌人
                                    :add(cast_opening_skill())
                                    :add(add_mark('assult_rushed'), false))
                                :add(sequence()
                                    :add(move_to_enemy())
                                    :add(delay(0.1,0.1))))))
                    :add(sequence() -- 不存在合法敌人
                        :add(idle())  -- 待机
                        :add(select_enemy(false)) -- 只选择车辆类型敌人
                        :add(delay(0.1, 0.3))))

        bt:setContext(agent)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt       
    end,


    -- 火炮型战车行为树
    artillery = function(agent)
         if not agent then 
            log.error('agent is nil!!')
            return nil
        end

        local bt = root():setChild(
                select()   -- 选择
                    :add(sequence()
                        :add(inverse(is_mark_exist('artillery_fired', false)))
                        :add(select_enemy(false))
                        :add(cast_opening_skill())
						:add(delay(0.2, 0.8)) 
						:add(delay(1))  
                        :add(add_mark('artillery_fired'), false)
                        :add(print_marks())
                        :add(remove_from_camera_view())
                        :add(delay(1))
                        :add(add_to_camera_view()))
                    :add(sequence() -- 存在合法敌人
                        :add(is_enemy_valid())
                        :add(select()
                            :add(sequence() -- 敌人在最大攻击范围内
                                :add(is_enemy_in_attack_range())
								:add(select() 
									:add(sequence() --敌人在最小射程之内
										    :add(is_enemy_in_min_attack_range())									
											:add(select()
												--:add(sequence() --攻击上下限范围内是否有敌人
												:add(sequence() --攻击上下限范围内是否有敌人
													:add(select_enemy(true,true)))
												:add(sequence()	
													:add(move_away_from_enemy())    --远离目标	
													:add(loop_until(is_enemy_out_safe_range())  --直到超出了安全距离
														:add(move_away_from_enemy()))
													:add(delay(0.1))))													
											:add(select()
												:add(sequence() --是否是自动战斗
													:add(is_auto())
													:add(cast_skill())
													:add(delay(1.0)))
												:add(select()
													:add(sequence()
														:add(is_enemy_valid())
														:add(attack_enemy()))
													:add(sequence()	
														:add(idle())))))	
									:add(sequence()--敌人不在最小射程之内，即在有效射程
										:add(select()
											:add(sequence() --是否是自动战斗
												:add(is_auto())
												:add(cast_skill())
												:add(delay(1.0)))
											:add(select()
												:add(sequence()
													:add(is_enemy_valid())
													:add(attack_enemy()))
												:add(sequence()	
													:add(idle())))))))	
                            :add(sequence() --敌人在最大攻击范围外
								:add(move_to_enemy())
                                :add(delay(0.1,0.1)))))
                                --:add(delay(0.5,1))
                    :add(sequence() -- 不存在合法敌人
                        :add(idle())  -- 待机
                        :add(select_enemy(false)) --不判断攻击范围索敌 
                        :add(delay(0.1, 0.3))))

        bt:setContext(agent)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt       
    end,

	
    -- 士兵组行为树
    soldier = function(agent)
        if not agent then 
            log.error('agent is nil!!')
            return nil
        end

        local bt = root():setChild(
                select()   -- 选择
                    :add(sequence() -- 存在合法敌人

                        :add(is_enemy_valid())
                        :add(move_to_enemy_and_attack())
                        :add(delay(0.1, 0.1))
                        :add(select_enemy(false)) -- 只选择车辆类型敌人
                        :add(delay(0.2, 0.2))
                        --[[
                        :add(select()
                            :add(sequence() -- 敌人在攻击范围内
                                :add(is_enemy_in_attack_range())
                                :add(attack_enemy()))
                            :add(sequence() -- 敌人不在范围内
                                :add(move_to_enemy())
                                :add(delay(0.5,1))))]]
                        )
                    :add(sequence() -- 不存在合法敌人
                        :add(idle())  -- 待机
                        :add(select_enemy(false)) -- 只选择车辆类型敌人
                        :add(delay(3.0, 15.0))))

        bt:setContext(agent)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,


       -- 医疗兵组行为树
    engineer = function(agent)
        if not agent then 
            log.error('agent is nil!!')
            return nil
        end

        local bt = root():setChild(
                select()   -- 选择
                    --:add(sequence() -- 是否所属坦克死亡
                     --   :add(is_owner_dead())
                        --:add(escape())
                     --   :add(delay(100, 100)))
                    :add(sequence() -- 存在合法敌人

                        --:add(follow_owner_and_attack())
                        :add(delay(1, 2))
                        --[[
                        :add(select()
                            :add(sequence() -- 敌人在攻击范围内
                                :add(is_enemy_in_attack_range())
                                :add(attack_enemy()))
                            :add(sequence() -- 敌人不在范围内
                                :add(move_to_enemy())
                                :add(delay(0.5,1))))]]
                        ))

        bt:setContext(agent)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end, 

    -- 蜘蛛行为树
    Boss_spider = function(agent)
        if not agent then 
            log.error('agent is nil!!')
            return nil
        end

        local bt = root():setChild(
                select()   -- 选择
                    :add(sequence()
                        :add(inverse(is_mark_exist('Boss_spider_fired', false)))
                        :add(timer_begin()) --计时开始
                        :add(select_enemy(false))
                        :add(cast_opening_skill(true))  --释放开场技能
                        :add(delay(3)) 
                        :add(add_mark('Boss_spider_fired'), false)
                        --:add(print_marks())
                       -- :add(remove_from_camera_view())
                        --:add(delay(1))
                       -- :add(add_to_camera_view()
                        )
                    :add(sequence() -- 存在合法敌人
                        :add(is_enemy_valid())
                        :add(select()
                            :add(sequence() -- 敌人在攻击范围内
                                :add(is_timer_bigger_than(15))  --计时大于15秒
                                :add(cast_skill(101105))    --开始释放技能
                                :add(timer_begin()))    --计时开始
                            :add(random_select()       --进行行为随机
                                :add(random_node(50)    --50%几率释放以下技能
                                    --:add(is_enemy_in_attack_range())
                                    :add(cast_skill(101104)))
                                :add(random_node(50)    --50%几率释放以下技能
                                    --:add(is_enemy_in_attack_range())
                                    :add(cast_skill(101103))))))

                    :add(sequence() -- 不存在合法敌人
                        :add(idle())  -- 待机
                        :add(select_enemy(false)) -- 只选择车辆类型敌人
                        :add(delay(2.1, 2.3))))

        bt:setContext(agent)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,

    PVPtank = function(agent)
        if not agent then 
            log.error('agent is nil!!')
            return nil
        end

        local bt = root():setChild(
                select()   -- 选择
                    :add(sequence() -- 存在合法敌人
                        :add(is_enemy_valid())
                        :add(attack_enemy())
                        :add(delay(1.5, 2.5)))
                    :add(sequence() -- 不存在合法敌人
                        :add(idle())  -- 待机
                        :add(select_enemy(false)) -- 只选择车辆类型敌人
                        :add(delay(0.1, 0.3))))

        bt:setContext(agent)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,


    Tower = function(agent)
        if not agent then 
            log.error('agent is nil!!')
            return nil
        end

        local bt = root():setChild(
                select()   -- 选择
                    :add(sequence() -- 存在合法敌人
                        :add(is_enemy_valid())
                        :add(attack_enemy())
                        :add(delay(1.5, 2.5)))
                    :add(sequence() -- 不存在合法敌人
                        :add(idle())  -- 待机
                        :add(select_enemy(false)) -- 只选择车辆类型敌人
                        :add(delay(0.1, 0.3))))

        bt:setContext(agent)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,
    
}