local log = logger.get("Game", "Attachment")
local bt_states = require('bt.node').states
local actor_types = require('RTS.enums').actor_types
local bullet_class = require('RTS.rts_bullet')
local penetration_bullet_class = require('RTS.rts_bullet_penetration')
local damage_calculator = require('RTS.prop_sys.damage_calculator')

-- 挂载物，包括副武器，装饰物, 轮胎
return class
{
    ctor = function(self, owner, attach_point, cfg_id, loadedCallback, weapon_pos)
        self.owner = owner
        self.id = owner.id
        self.team = owner.team
        self.bullets = owner.bullets
        self.props = owner.props
        self.attach_point = attach_point
        self.is_wheel = (attach_point == "Wheel")
        self.cfg_id = cfg_id
        self.cfg = config.weapon_weapons[cfg_id]
        self.loadedCallback = loadedCallback
        -- 表示副武器的槽位，如果是轮胎值为nil
        self.weapon_pos = weapon_pos
        if not self.cfg then
            log.error('Failed to find attachment weapon:'..cfg_id)
        else
            -- 加载副武器资源
            self:loadResource(attach_point)
        end

        -- 技能相关
        if not self.is_wheel then
            self.skill_cfg = config.skill_skills[self.cfg.commonatk_id]
            self.bt = nil           -- 技能行为树
            self.skill_elapsed = 99
            self.in_skill = false   -- 是否在技能中

            self.shoot_point = self.attach_point    -- 射击点
            self.attack_range = self.cfg.atk_range  -- 射程
        end

    end,

    --[[
    loadMaterial = function(self)
        local wheel_trans = self.owner.cs_inst:FindChild("Wheel")
        if not wheel_trans then
            log.error("Failed to find Wheel node, tank id:"..self.owner.cfg.id)
            self.is_initialized = true
            self:init()
            return
        end

        local wheel_go = wheel_trans.gameObject

        local callback = function(material)
            -- callback
            if material ~= nil then
                local renderer = wheel_go:GetComponent(UnityEngine.Renderer)
                if renderer then
                    --renderer.material = material
                    renderer.material = material
                else
                    log.error("Failed to find renderer in wheel node:"..self.id)
                end
                self:init()
            else
                log.error('load material failed:'..self.cfg.material)
            end
        end

        local path = self.cfg.material
        if path == "" then
            log.error('Failed to load wheel material:'..self.cfg_id)
        else
           -- log.error("Load Material:"..self.cfg_id..","..tostring(path))
            Fenix.Assets.Helper.LoadAsset(path, function(asset) 
                local obj = asset:Get()
                if not obj then
                    log.error("Failed to load wheel material: "..path)
                end
                local mat = UnityEngine.Material(obj)

                if self.package then
                    self.package:RemoveRef()
                end
                self.package = asset.package
                if self.package then
                    self.package:AddRef()
                end
                callback(mat)
            end)
        end
    end,]]

    -- 加载资源
    loadResource = function(self, root)
        if self.is_wheel then
            --self:loadMaterial()
            self.gameObject = self.owner.gameObject
            --self:init()
        else

            local callback = function(go)
                -- callback
                if go ~= nil then
                    self.gameObject = go
                    self.transform = go.transform
                    self.transform:SetParent(root)
                    self.transform.localPosition = Vector3.zero
                    self.transform.localScale = Vector3.one
                    self.transform.localRotation = Quaternion.identity
    
                    self:init()
                else
                    log.error('loadResource go is nil')
                end
            end
    
            local prefab_path = self.cfg.model_resource
            if prefab_path == "" then
                log.error('Failed to load minor weapon res:'..self.cfg_id)
            else
                if war_helper then
                    if not war_helper:loadPrefab(prefab_path, callback) then
                        self.is_initialized = true
                    end
                elseif battle_helper then
                    --log.info("### loadResource attachment from  battle_helper:loadPrefab(prefab_path, callback)")
                    if not battle_helper:loadPrefab(prefab_path, callback) then
                        self.is_initialized = true
                    end
                else
                    app:startCoroutine(function()
                        coroutine.yield(nil)
                        coroutine.yield(WaitForInstantiate(prefab_path, root, callback))
                    end)
                end
            end
        end
    end,

    init = function(self)
        -- 如果有轮胎槽位表示该类为副武器
        if self.weapon_pos then
            -- print("副武器添加:", self.owner.id, self.weapon_pos)
            self.animation = self.gameObject:GetComponent(UnityEngine.Animation)
            self.owner.cs_inst:addMinorAnimation(self.weapon_pos, self.animation) 
        else
            -- 轮胎，解析出轮胎 Y 轴移动的结点 
            if self.owner.cs_inst then
                self.left_wheel = self.owner.cs_inst:FindChild('luntai_left')
                self.right_wheel = self.owner.cs_inst:FindChild('luntai_right')

            end
            
        end

        if self.cfg.commonatk_id > 0 and self.skill_cfg then
            
            local bt_id = self.skill_cfg.bt_id

            -- 创建技能行为树
            local bt_fun = require('RTS.skill_lib._skill_bt')[bt_id]
            if not bt_fun then 
                log.error('failed to find skill_bt:'..tostring(bt_id)..'in skill:'..self.skill_id)
            end
            self.bt = bt_fun(self)

            if self.bt == nil then
                log.error('failed to find skill bt:'..tostring(self.skill_id))
            else
                --log.info('##### attachment inited!!!')
            end

        end

        self.is_initialized = true
        if self.loadedCallback then
            self.loadedCallback(self.gameObject)
        end
    end,

    getTargetActor = function(self)
        self.target_actor = self.owner.target_actor
        return self.target_actor
    end,

    getPosition = function(self)
        return self.transform.position
    end,

    beginCD = function(self)
        self.skill_elapsed = 0.0
    end,

    update = function(self, dt)
        if self.left_wheel or self.right_wheel then
            self:updateWheel(dt)
        end

        if not self.bt or not self.owner:isAlive() then
            return
        end

        self.skill_elapsed = self.skill_elapsed + dt
        if  self.skill_elapsed >= self.skill_cfg.skill_cd then
            self.in_skill = true
        end

        if self.in_skill then
            local rst = self.bt:update(dt, 2)
            if rst == bt_states.SUCCESS or rst == bt_states.FAILURE then
                self.in_skill = false
                self.bt:reset()
            end
        end
    end,

    updateWheel = function(self, dt)
        if not self.left_wheel and not self.right_wheel then
            return
        end

        local expect_angle = self.owner.cs_inst.m_fSteeringAngular * 90
        local angle, axis = self.owner.transform.rotation:ToAngleAxis() -- to angle
        if axis.y < 0 then
            angle = -angle
        end
        angle = angle *  57.29578
        angle = angle + expect_angle

        local r = UnityEngine.Quaternion.AngleAxis(angle, Vector3.up)
        if self.left_wheel then
            self.left_wheel.rotation = r
        end
        if self.right_wheel then
            self.right_wheel.rotation = r
        end

    end,

    sk_playAnimation = function(self)
    end,

    -- 绑定 obj 到射击点
    bindToShootPoint = function (self, obj, rot)
        return self.cs_inst:BindToShoot(obj, Vector3(rot[1], rot[2], rot[3]))
    end, 

    -- 绑定 obj 到头顶
    bindToHeadPoint =  function (self, obj, rot)
        local rot = rot or {0,0,0}
        return self.cs_inst:BindToHead(obj, Vector3(rot[1], rot[2], rot[3]))
    end, 

    -- 绑定 obj 到任意 bone
    bind = function (self, bone, obj, rot)
        local rot = rot or {0,0,0}
        return self.cs_inst:Bind(bone, obj, Vector3(rot[1], rot[2], rot[3]))
    end, 

    getActorType = function(self)
        return self.owner.actor_type
    end,

    -- 发射一颗子弹
    sk_shootBullet = function(self, target, loop_effect, loop_body, hit_effect, stop_effect, from, to, speed, curve_name, damage_width, penetration, max_dist, curve_height, cam_anim) -- 参数的 伤害弃用 
        to.y = from.y
        local rot = Quaternion.LookRotation(to-from, Vector3.up)
        local skill_cfg = self.skill_cfg

        local loop_body_path = nil
        if loop_body then loop_body_path = loop_body.effect_path end
        if penetration then
            --ctor = function (self, owner, target, prefab_path, pos, destination, speed, hit_callback)
            local bullet = penetration_bullet_class(self.owner, rot, loop_effect.effect_path, loop_body_path, from, to, speed, max_dist, damage_width,
                function(b, target_actor) -- hit callback
                    -- 目标减血
                    if target_actor then
                        -- 调用伤害计算
                        local dmg, is_hit, is_crit = damage_calculator.calculate(damage_calculator, self.owner, target_actor, skill_cfg)
                        if is_hit then
                            target_actor:beAttacked(self.owner, skill_cfg, dmg, is_hit, is_crit)
                        end
                        -- 播放击中特效
                        --[[
                        if hit_effect then
                            battle_helper:playFXOnPos(hit_effect.effect_path, target_actor:getPosition(), hit_effect.effect_time)
                        end]]

                    end
                end,
                function(b) -- stop callback
                    self.bullets:remove(b)
                    if stop_effect and stop_effect.effect_path then
                        battle_helper:playFXOnActorPos(stop_effect.effect_path, target, to, stop_effect.effect_time)
                    end

                    if cam_anim then
                        battle_helper:playCamAnim(cam_anim)
                    end
                end           
                )
            bullet.curve_height = curve_height
            self.bullets:add(bullet)  
        else
            local curve = nil
            if curve_name then
                --log.info(curve_name)
                curve = battle_helper:loadCurve(curve_name)
            end
            --ctor = function (self, owner, target, prefab_path, pos, destination, speed, hit_callback)
            if loop_effect then
                local bullet = bullet_class(self.owner, target, rot, loop_effect.effect_path, loop_body_path, from, to, speed, curve,
                function(b) 
                    --log.info('bullet hit target_actor!') 
                    self.bullets:remove(b)
                    -- 目标减血
                    if b.target then
                        -- 调用伤害计算
                        local dmg, is_hit, is_crit = damage_calculator.calculate(damage_calculator, self.owner, target, skill_cfg)
                        if is_hit then
                            b.target:beAttacked(self.owner, skill_cfg, dmg, is_hit, is_crit)
                        end
                    end
                    -- 播放击中特效
                    if stop_effect then
                        battle_helper:playFXOnActorPos(stop_effect.effect_path, target, to, stop_effect.effect_time)

                        if cam_anim then
                            battle_helper:playCamAnim(cam_anim)
                        end

                    end
                end)
                bullet.curve_height = curve_height
                self.bullets:add(bullet)     
            else
                log.error("Failed to shoot bullet, minor weapon id:"..self.id)
            end         
        end
    end,

    destroy = function(self)
        if self.is_wheel then
            if self.package then
                self.package:RemoveRef()
                self.package = nil
            end
        else
            if self.gameObject  then
                self.gameObject:destroy()
            else
                log.error("### no self.gameObject")
            end
        end
    end,
}