local log = logger.get("Game", "Main Weapon")
require("config.tankzoom")

-- 主武器
return class
{
 ctor = function(self, owner, attach_point, cfg_id, loadedCallback)
        self.owner = owner
        self.attach_point = attach_point
        self.cfg_id = cfg_id
        self.cfg = config.weapon_weapons[cfg_id]
        self.is_turning = true

        self.loadedCallback = loadedCallback
        self.is_initialized = false
        if not self.cfg then
            log.error('Failed to find weapon:'..cfg_id)
        else
            -- 加载副武器资源
            self:loadResource(attach_point)
            if self.cfg.Atk_Ani_Mix == 0 then   -- 技能时候能否混合受击动画
                owner.play_hit_in_skill = false
            end
        end
    end,

    -- 加载资源
    loadResource = function(self, root)
        local callback = function(go)
            -- callback
            if go ~= nil then
                self.gameObject = go
                self.transform = go.transform
                self.transform:SetParent(root)
                self.transform.localPosition = Vector3.zero
                
                local rts_helper = require('RTS.rts_helper')

                local scale = rts_helper:getWeaponScale(self.owner.cfg.prefab1, self.cfg_id)

                self.transform.localScale = Vector3(scale, scale, scale)
                self.transform.localRotation = Quaternion.identity

                self:init()
            else
                log.error('loadResource go is nil')
            end
        end

        local prefab_path = self.cfg.model_resource
        if prefab_path == "" then
            log.error('加载主武器模型失败:', self.cfg_id)
        else
            if war_helper then
                war_helper:loadPrefab(prefab_path, callback)
            elseif battle_helper then
                battle_helper:loadPrefab(prefab_path, callback)
            else
                app:startCoroutine(function()
                    coroutine.yield(nil)
                    coroutine.yield(WaitForInstantiate(prefab_path, root, callback))
                end)
            end
        end
    end,

    init = function(self)
        self.owner.shoot_point = self.owner.cs_inst:FindInTransform(self.transform, 'shoot')
        if not self.owner.shoot_point then
            log.error("Failed to find shoot node in weapon:"..self.cfg_id)
        end

        self.animation = self.gameObject:GetComponentInChildren(UnityEngine.Animation, true)
        self.owner.cs_inst:AddAnimationComp(self.animation)

        -- self.owner:initMinorWeapons()
        self.is_initialized = true

        if self.loadedCallback then
            self.loadedCallback(self.gameObject)
        end
    end,

    -- 获取主武器的旋转角度
    getRotationAngle = function(self)
        local a = self.transform.forward
        local b = self.owner.transform.forward

        local angle = Vector3.Angle(a, b)
        local axis = Vector3.Cross(b, a)

        if axis.y < 0 then
            angle = 360 - angle
        end

        return angle
    end,

    -- 设置是否旋转武器
    setTurning = function(self, is_turning)
        self.is_turning = is_turning
    end,

    facingTarget = function(self, target_transform)
        self.look_target = target_transform
    end,

    --local ToAngle = 57.29578
    --local ToRad = 0.01745329
    update = function(self, dt)
        if self.is_turning and self.look_target and self.cfg.rotate_angle > 0 then
            -- 当前的朝向
            local forward = self.transform.forward
            forward.y = 0
            forward:Normalize()

            -- 期待朝向
            local expect = self.look_target.position-self.transform.position
            expect.y = 0
            expect:Normalize()

            local angle = Vector3.Angle(forward, expect)

            if angle <= self.cfg.rotate_angle  and angle > 5 then

                local new_forward = Vector3.Slerp(forward, expect, self.cfg.rotate_speed / angle * dt)

                --log.info("#### new_forward:"..tostring(new_forward))
                local r = Quaternion.New()
                r:SetLookRotation(new_forward)
                self.transform.rotation = r

            end
        end
    end,

    destroy = function(self)
        if self.gameObject then
            self.gameObject:destroy()
        else
            log.error("### no self.gameObject!!")
        end
    end,
}