-- 一个背包
local log = logger.get("Game", "bag")
local item_cls = require('RTS.bag_sys.item')

return class {
    ctor = function(self, bag_type)
        self.bag_type = bag_type      
        self.items = {} -- 背包中的道具 以 instId 为下标的 item_info
    end,

    print = function(self)
        local str = '===== bag:'..self.bag_type..' ===== \n'
        for k,v in pairs(self.items) do
            str = str..'['..k..'] cfg_id:'..v.cfg_id..', inst_id:'..v.inst_id..', num:'..
                v.count..', level:'..tostring(v.level)..', exp:'..tostring(v.exp)..', totalexp'..tostring(v.total_exp)..'\n'
            if v.props then
                str = str..'props:\n'
                for _, prop in ipairs(v.props) do
                    str = str ..'['..prop.PropId..'] = '..prop.PropValue..'\n'
                end
            end
        end

        log.info(str)
    end,

    -- 添加道具到背包第 idx 个格子
    setItem = function(self, item_info)
        log.info('#### setItem cfg_id'..item_info.cfgId..', inst_id:'..item_info.instId..', num:'..item_info.count..', status:'..item_info.status)

        local idx = item_info.instId
        if not self.items[idx] then
            -- 背包格子是空的
            self.items[idx] = item_cls(item_info)
        else
            -- 背包格子已经存在道具
            local item = self.items[idx]
            if item.cfg_id == item_info.cfgId then
                item:refresh(item_info)
            else
                log.error('Add item error, not same cfg_id:'..cfg_id)
            end
        end
        self.items[idx].slot = 0
        
        return self.items[idx]
    end,

    getItem = function(self, inst_id)
        return self.items[inst_id]
    end,

    -- 获取道具数量
    getItemNum = function(self, cfg_id)
        local num = 0
        local old_num = 0
        for k,v in pairs(self.items) do
            if v.cfg_id == cfg_id then
                num = num + v.count
                old_num = old_num + v.old_count
            end
        end
        return num, old_num
    end,

    -- 根据类型获取道具
    getItemsByType = function(self, type, items)
        for _,item in pairs(self.items) do
            if item.cfg.type == type then
                table.insert(items, item)
            end
        end
    end,

    -- 根据cfg_id获取所有道具实例
    getItemsByCfgId = function(self, cfg_id)
        local rst = {}
        for id, item in pairs(self.items) do
            if item.cfg_id == cfg_id then
                table.insert(rst, item)
            end
        end
        return rst
    end,

    getItemsByQuality = function(self, quality, items)
        for _,item in pairs(self.items) do
            if item.cfg.color == quality then
                table.insert(items, item)
            end
        end
    end,

    getItemsByQualityFromList = function(self, quality, items, list)
        for _,item in pairs(list) do
            if item.cfg.color == quality then
                table.insert(items, item)
            end
        end
    end,

    --根据武器family获取武器
    getItemsByFamily = function(self, family, items)
        for _,item in pairs(self.items) do
            if item.cfg.type == 8  and 
                family == config.weapon_weapons[item.cfg_id].family then
                table.insert(items, item)
            end
        end
    end,

    removeItem = function(self, inst_id)
        --log.info('#### removeItem  inst_id:'..inst_id)

        local item = self.items[inst_id]
        if not item then
            log.error('failed to find item to remove! bag:'..self.bag_type..', inst_id:'..inst_id)
            return nil
        end

        self.items[inst_id] = nil

        return item
    end,

    -- 获得所有道具，用 comp 函数排序
    getItems = function(self, item_type, comp)
        local items = {}
        for _,item in pairs(self.items) do
            if not item_type  or item.cfg.type == item_type then
                table.insert(items, item)
            end
        end

        if #items > 0 and comp then
            table.sort(items, comp)
        end

        return items
    end,

    -- 根据使用等级排序获得道具
    getItemsSortByUseLevel = function(self, item_type)
        local sort_func = function(a, b)
            if a.cfg and b.cfg and a.cfg.uselevel and b.cfg.uselevel then
                if a.cfg.uselevel == b.cfg.uselevel then
                    return false
                else
                    return a.cfg.uselevel > b.cfg.uselevel
                end
            else
                return false
            end
        end

        return self:getItems(item_type, sort_func)
    end,

    -- 根据品质排序获得道具
    getItemsSortByQuality = function(self, item_type)
        local sort_func = function(a, b)
            if a.cfg and b.cfg and a.cfg.color and b.cfg.color then
                if a.cfg.color == b.cfg.color then
                    return false
                else
                    return a.cfg.color > b.cfg.color
                end
            else
                return false
            end
        end

        return self:getItems(item_type, sort_func)
    end,

    -- 根据强化等级排序获得道具
    getItemsSortByEnhanceLevel = function(self, item_type)
        local sort_func = function(a, b)
            if a.level and b.level then
                if a.level == b.level then
                    return false
                else
                    return a.level > b.level
                end
            else
                return false
            end
        end

        return self:getItems(item_type, sort_func)
    end,

}
