-- 背包管理
local log = logger.get("Game", "bag_manager")
local bag_cls = require('RTS.bag_sys.bag')
local hero_bag_cls = require('RTS.bag_sys.hero_bag')
local item_cls = require('RTS.bag_sys.item')

require ('RTS.bag_sys.item_helper')
--[[
1 装备背包：
ID 1-30000
主副武器，装饰物，轮胎等

2.素材背包：
ID： 30001 - 60000
包含所有不可以使用的非碎片，非图纸类

3.消耗品背包：
ID:60001-100000
所有可以使用的物品

4.图纸背包：
ID:100001 - 120000

5.指挥官碎片背包
ID:120001 - 130000


不会显示在仓库里的特殊物品放置的背包编号从11开始：
11.船员背包
ID:130001 - 150000
12.关卡永久背包（出关卡依旧保留该背包内所有物品）
13.关卡临时背包（出关卡会清空该背包）
]]
local bag_manager_cls = class {
    ctor = function(self)
        -- 常量定义
        self.NUM_BAGS = 20   -- 背包数量,11 开始为不可见背包
        
        self.bags = {}      -- 所有背包
        for i=0,self.NUM_BAGS,1 do
            self.bags[i] = bag_cls(i)
        end

        self.hero_bags = {} -- 装备栏背包数组，每个英雄/战车 对应一个装备背包
        self.weapon_set_cfg = item_helper:getSetConfig()
        self.events = require('base.event').new()

    end,

    -- 获取某类型的背包
    getBag = function(self, bagtype)
        return self.bags[bagtype]
    end,

    -- 添加道具到背包
    setItem = function(self, item_info)
        local cfg_id = item_info.cfgId
        if cfg_id == 0 then return end

        local cfg = item_helper:getItemCfg(cfg_id)
        if not cfg then
            --log.info(debug.traceback())
            log.error('setItem Failed to find item cfg:'..cfg_id)
            return nil
        end

        local bag = self.bags[cfg.bagtype]
        if not bag then
            
            log.error('Failed to find bag:'..item_info.cfgId)
            log.error('Failed to find bag:'..cfg.bagtype)
        end

        log.info('### bag manager set item:'..cfg_id..' num:'..item_info.count)
        local item = bag:setItem(item_info)

        self.events:dispatch("onBagChanged", item)

        return item
        --[[
        -- 如果是资源类道具发生变化，更新界面
        if cfg_id == self.STEEL_ID or cfg_id == self.WOOD_ID or
             cfg_id == self.BRICK_ID or cfg_id == self.URANIUM_ID then
            if world_map_info.ui then world_map_info.ui:refreshResourceNum() end
        end]]

    end,

    getItemById = function(self, id)
        for __, bag in pairs(self.bags) do
            local item_lua = bag:getItem(id)
            if item_lua then
                return item_lua
            end
        end
        return nil
    end,

    -- 获取某道具数量
    getItemNum = function (self, cfg_id)
         local cfg = item_helper:getItemCfg(cfg_id)
         if not cfg then return 0 end
         
         local bag = self.bags[cfg.bagtype]
         if bag then
            return bag:getItemNum(cfg_id)
         else
            return 0
         end
    end,

    getItemsByCfgId = function(self, cfg_id)
        local cfg = item_helper:getItemCfg(cfg_id)
        if not cfg then return {} end

        local bag = self.bags[cfg.bagtype]
        if bag then
            return bag:getItemsByCfgId(cfg_id)
        else
            return {}
        end
    end,

    -- 从背包删除道具
    removeItem = function(self, cfg_id, inst_id, num)
        local cfg = item_helper:getItemCfg(cfg_id)
        if not cfg then
            log.error('removeItem Failed to find item cfg:'..cfg_id)
            log.info(debug.traceback())
            return nil
        end

        log.info('### bag manager removeItem:'..cfg_id..', instId:'..inst_id..' num:'..num)

        local bag = self.bags[cfg.bagtype]
        if not bag then
            log.error('removeItem Failed to find bag:'..cfg.bagtype)
        end

        local item = bag:removeItem(inst_id, num)

        self.events:dispatch("onBagChanged", item)
        return item
    end,

    -- 根据类型获取道具
    getItemsByType = function(self, type)
        local items = {}
        for _,bag in pairs(self.bags) do
            bag:getItemsByType(type, items)
        end
        return items
    end,

    -- 初始化英雄背包
    initHeroBag = function(self, hero_id)
        if self.hero_bags[hero_id] then
            log.error('hero bag already existed!! '..hero_id)
        else
            self.hero_bags[hero_id] = hero_bag_cls(hero_id)
        end

        return self.hero_bags[hero_id]
    end,

    -- 获取 hero bag
    getHeroBag = function(self, hero_id)
        return self.hero_bags[hero_id]
    end,

    getAllHeroBagItem = function(self)
        local items = {}
        for _, bag in pairs(self.hero_bags) do
            for _, item in pairs(bag.slots) do
                table.insert( items, item )
            end
        end
        return items
    end,

    refreshItem = function(self, item_info)
        local items = self:getAllHeroBagItem()
        for k, v in ipairs(items) do
            if item_info.instId == v.inst_id then
                v:refresh(item_info)
                return
            end
        end

        self:setItem(item_info)
    end,

    --[[
    //回应英雄操作装备
    message SCHeroEquip	
    {
        required uint32        HeroCfgId     = 1;        //英雄Id
        optional MItemChange   equipItem     = 2;        //装备的装备
        optional MItemChange	 unequipItem   = 3;        //卸下的装备
    }
    message MItemChange
    {
        required ITEM_UPDATE  ChangeType      = 1;        //改变类型
        required MItemInfo    ItemInfo        = 2;        //道具信息
    }
    message MItemInfo
    {
        required string         instId      = 1;  //道具唯一ID
        optional uint32        cfgId      = 2;  //道具模板ID
        optional EQUIP_SLOT    status      = 3;  //战车装备槽位
        optional uint32        count       = 4;  //道具个数
    }  
]]
    -- 处理 hero euqip 消息
    onHeroEquip = function(self, hero_equip)
        -- Todo: 检查 hero_equip 的 respCode 是否为 0

        --[[local hero_bag = self:getHeroBag(hero_equip.HeroCfgId)
        if not hero_bag then 
            log.error('Failed to find hero bag:'..hero_equip.HeroCfgId)
            return
        end]]

       
        self:unequipItem(hero_equip.unequipItem)
        local item = self:equipItem(hero_equip.equipItem)

        --self.events:dispatch("onBagChanged", item)

        return item
    end,

    equipItem = function(self, item_change)

        if not item_change or item_change.ItemInfo.cfgId == 0 then
            log.info('#### equipItem item is nil')
            return
        end

        local hero_bag = self:getHeroBag(item_change.heroId)
        if not hero_bag then 
            log.error('equipItem failed to find hero bag:'..item_change.heroId)
            return
        end

        local item_info = item_change.ItemInfo

        -- 从原背包中删除
        --log.info('####### remove item:'..item_info.cfgId..', instId:'..item_info.instId)
        self:removeItem(item_info.cfgId, item_info.instId, 1)

        -- 添加到英雄装备背包
        local item = hero_bag:equip(item_info)
        --self.events:dispatch("onBagChanged", item)

        return item
    end,

    unequipItem = function(self, item_change)
        if not item_change or #item_change == 0 then
            return
        end
        
        for _, v in ipairs(item_change) do

            local hero_bag = self:getHeroBag(v.heroId)
            if not hero_bag then 
                log.error('unequipItem failed to find hero bag:'..v.heroId)
                return
            end
            
            local item_info = v.ItemInfo

            -- 从英雄背包删除
            hero_bag:unequip(item_info.status)

            -- 添加回背包
            local item = self:setItem(item_info)
        end

        --self.events:dispatch("onBagChanged", item)
    end,

    -- 道具升级
    upgradeItem = function(self, hero_id, item_change)
        --log.info('### upgradeItem received!!')
        local item_info =item_change.ItemInfo
        local orig_level = 0
        local orig_props = nil
        local item = nil

        if hero_id and hero_id > 0 then
            local hero_bag = self:getHeroBag(hero_id)
            if not hero_bag then
                log.error('Failed to find hero bag:'..hero_id)
                return
            end
            item = hero_bag:get(item_info.status)
            if item and item.inst_id == item_info.instId then
                orig_level = item.level
                orig_props = item.props
                hero_bag:equip(item_info)
            else
                -- 道具信息出错，英雄身上 equipSlot (status) 不存在道具
                log.error('handle upgradeItem Failed!! hero:'..hero_id..
                ' have no item:'..item_info.cfgId..' on slot:'..item_info.status)
            end
            
        else
            local item_cfg = item_helper:getItemCfg(item_info.cfgId)
            if not item_cfg then
                log.error('upgradeItem Failed to find item cfg:'..item_info.cfgId)
                return
            end
            local bag = self.bags[item_cfg.bagtype]
            item = bag:getItem(item_info.instId)
            orig_level = item.level
            orig_props = item.props

            --bag:removeItem(item_info.instId)
            bag:setItem(item_info)
        end

        self.events:dispatch("onBagChanged", item)
        self.events:dispatch("onItemUpgrade", item, orig_level, orig_props)
    end,

    -- 获取背包装备数量
    getEquipNum = function(self)
        local bag = self:getBag(1)

        local cnt = 0
        for k, v in pairs(bag.items) do
            cnt = cnt + 1
        end

        return cnt
    end,

    print = function(self)
        for k,v in pairs(self.bags) do 
            v:print()
        end

        for k,v in pairs(self.hero_bags) do 
            v:print()
        end

        local local_player = require('game.player')
        local_player:print()
        
    end,

}

bag_manager = bag_manager or bag_manager_cls()

return bag_manager
