-- 英雄背包，相当于英雄装备槽
local log = logger.get("Game", "Hero Bag")
local item_cls = require('RTS.bag_sys.item')

return class
{
    ctor = function(self, hero_id)
        self.hero_id = hero_id  -- 所属英雄的 cfg_id
        self.slots = {} -- 装备的槽位
        self.is_dirty = false
        self.sets = {}
    end,

    -- 在槽位上装备
    equip = function(self, item_info)
       -- log.info('##### Hero bag equip cfg_id:'..item_info.cfgId..', inst_id:'..item_info.instId..', slot_id:'..tostring(item_info.status))
        if item_info.status then
            local orig_item = self.slots[item_info.status]
            if orig_item then
                orig_item:refresh(item_info)
            else
                local item = item_cls(item_info)
                item.owner = self
                item.hero_id = self.hero_id
                self.slots[item_info.status] = item
            end
            self.sets = self:activedSet(self.slots)
            bag_manager.events:dispatch("onEquipChanged", self.hero_id)
        else
            log.error('Failed to find item_info.status!!')
        end
    end,

    -- 解除槽位装备
    unequip = function(self, slot_id)
        log.info('##### Hero bag unequip slot_id:'..tostring(slot_id))
        local item
        if slot_id then 
            item = self.slots[slot_id]
            item.owner = nil
            item.hero_id = nil
            self.slots[slot_id] = nil 
            self.sets = self:activedSet(self.slots)
            bag_manager.events:dispatch("onEquipChanged", self.hero_id)
        end
        return item
    end,

    activedSet = function(self, slots)
        local sets = {}
        local tb = {}
        local func = function(cfg_id, id)
            if tb[cfg_id] then
                return
            else
                for k, v in pairs(sets) do
                    if id == k then
                        sets[k] = sets[k] + 1
                        return
                    end
                end
                sets[id] = sets[id] or 0
            end
        end
        for k, v in pairs(slots) do
            if v.cfg.set ~= 0 then
                func(v.cfg_id, v.cfg.set)
                tb[v.cfg_id] = true
            end
        end

        return sets
    end,

    getSetActived = function(self, slot, item)
        local weapons = {}
        for k, v in pairs(self.slots) do
            weapons[k] = v
        end
        weapons[slot] = item

        local sets = self:activedSet(weapons)

        return sets
    end,

    --[[
    //战车装备槽位
    enum EQUIP_SLOT
    { 	
        MAIN_WEAPON_SLOT       = 0; //主武器
        MAIN_WEAPON_OTHER_SLOT = 1; //
        MINOR_WEAPON_SLOT_1    = 2; //副武器1
        MINOR_WEAPON_SLOT_2	   = 3; //
        MINOR_WEAPON_SLOT_3	   = 4; //
        MINOR_WEAPON_SLOT_4    = 5;
        MINOR_WEAPON_SLOT_5    = 6;        
        DECORATION_SLOT        = 7; //装饰物
        WHEEL_SLOT             = 8; //轮胎
        FASHION_SLOT           = 9; //时装
        OTHER_SLOT_1           =10; //预留槽位
        OTHER_SLOT_2           =11;
        MAX_SLOT               =12;
    }
    ]]
    -- 获取槽位的装备
    get = function(self, slot_id)
        return self.slots[slot_id]
    end,

    print = function(self)
        local str = '===== hero bag:'..self.hero_id..' ===== \n'
        for k,v in pairs(self.slots) do
            str = str..'['..k..'] cfg_id:'..v.cfg_id..', inst_id:'..v.inst_id..', level:'..tostring(v.level)..', exp:'..tostring(v.exp)..'\n'
            str = str..'props:\n'
            for _, prop in ipairs(v.props) do
                str = str ..'['..prop.PropId..'] = '..prop.PropValue..'\n'
            end
        end

        log.info(str)
    end,

}