local item_helper = require('RTS.bag_sys.item_helper')
local log = logger.get("Game", "Item")

-- item 的顺序流水号
global_item_seq_id = 0
genItemSeqId = function()
    global_item_seq_id = global_item_seq_id + 1
    return global_item_seq_id
end

-- 道具类
return class {
    ctor = function(self, item_info)
        --self.info = item_info
        self.inst_id = item_info.instId
        self.cfg_id = item_info.cfgId
        self.count = item_info.count
        self.props = item_info.itemProp
        self.level = item_info.level
        self.exp = item_info.experience
        self.total_exp = item_info.totalExp
        self.slot = item_info.status    -- 装备的槽位
        self.cfg = item_helper:getItemCfg(self.cfg_id)
        self.hero_id = nil  -- 所属的英雄 id
        self.owner = nil    -- 所属的背包
        self.locked = item_info.lockStatus  -- 锁定状态 0 未上锁  1 锁定
        self.rand_props = item_info.randProp
        self.rand_skill = item_info.randSkill
        
        self.seq_id = genItemSeqId()    -- 顺序 id, 用于以获得先后顺序排序
        self.is_new = false             -- 是否是新获得的道具
        self.old_count = 0      --更新前的数量

    end,

    refresh = function(self, item_info)
        self.inst_id = item_info.instId
        self.cfg_id = item_info.cfgId
        self.old_count = self.count
        self.count = item_info.count
        self.props = item_info.itemProp
        self.level = item_info.level
        self.exp = item_info.experience
        self.total_exp = item_info.totalExp
        self.cfg = item_helper:getItemCfg(self.cfg_id)
        self.locked = item_info.lockStatus
    end,

    getProp = function(self, prop_id)
        for k,v in ipairs(self.props) do
            if v.PropId == prop_id then
                return v.PropValue
            end
        end

        return 0
    end,

    -- 是否已经装备
    isEquiped = function(self)  
        return self.slot and self.slot > 0
    end,

    copy = function(self, item)
        item.inst_id = self.inst_id
        item.cfg_id = self.cfg_id
        item.count = self.count
        item.props = self.props
        item.level = self.level
        item.exp = self.exp
        item.total_exp = self.total_exp
        item.slot = self.slot     -- 装备的槽位
        item.cfg = self.cfg
        item.hero_id = self.hero_id   -- 所属的英雄 id
        item.owner = self.owner    -- 所属的背包
        item.locked = self.locked
    end,

    -- 是否锁定
    isLocked = function(self)
        return self.locked and self.locked > 0
    end,
}

--[[
ID	道具类型
1	钻石
2	金币（游戏币）
3	量子模块组件
4	关卡挑战次数
5	关卡行动力
6	关卡内资源
7	Boost点数
8	武器
9	宝箱类（调用dropbag）
10	通用加速道具
11	钢材
12	木头
13	钛
14	食物
15	伴生资源
16	指挥官晋升素材
17	指挥官碎片
18	个人行军加速道具
19	个人行军召回道具
20	量子装备打造材料
21	建造加速道具
22	个人基地训练项目
23	日常活跃值
24	武器经验素材
25	装甲经验素材
26	获得后会立即使用的道具
27	赌博币
28	旅行商人声望
29	玩家经验
30	一次性指挥官（获取后自动使用并获得一个指挥官，如果已经有该指挥官，则不给任何东西）
31	重复性指挥官（获取后自动使用并获得一个指挥官，如果已经有该指挥官，则会给予其他物品）
32	修改联盟名字
33	修改联盟缩写
34	联盟币
35	公会资源
36	随机指挥官（获取后自动使用并获得一个随机指挥官，根据配置的物品参数来限定随机指挥官的品质和类型
37	需要玩家在仓库里手动点击使用的物品，使用后获得MAP格式的物品（读取物品对应的reward_item字段）
38	钢材资源包：使用后获得MAP格式的物品（读取物品对应的reward_item字段）
39	齿轮资源包：使用后获得MAP格式的物品（读取物品对应的reward_item字段）
40	电能资源包：使用后获得MAP格式的物品（读取物品对应的reward_item字段）
41	钛结晶资源包：使用后获得MAP格式的物品（读取物品对应的reward_item字段）
42	通用固定时间加速道具
43	个人基地建造/升级加速道具
44	科研加速道具
45	生产加速道具
46	修理加速道具
47	车体改装加速道具
48	通用百分比时间加速道具（行军加速）
49	运输加速道具
50	集结行军加速道具
51	紧急修理包
52	修理包制造加速道具
53	普通素材
54	侦查队列的移动加速道具
55	采集队列的移动加速道具
56	出征队列的移动加速道具
57	普通物品
58	燃料
59	行动力
60	随飞道具
61	城墙修复加速道具
62	量子装备制造加速
64	陷阱生产加速
65	陷阱修复加速
66	医疗士兵加速
68	基地buff道具
69	皮肤道具
72	行动力资源包：使用后获得MAP格式的物品（读取物品对应的reward_item字段）
76	食物资源包：使用后获得MAP格式的物品（读取物品对应的reward_item字段）
77	钻石资源包：使用后获得MAP格式的物品（读取物品对应的reward_item字段）
78	士兵训练加速道具
79	指挥官经验物品，用于指挥官升级
80	城墙灭火道具
81	领主天赋重置道具
82	指挥官招募卷轴
83	领主经验道具
84	船员（量子装备）
]]