local log = logger.get("Game", "Item Helper")

item_helper = {

    WEAPEN_BAG_ID = 1,

    ALL_ITEM_TAB = {config.item_resources, config.item_useables, config.item_pieces, config.item_blueprints, config.quantummodule_blocks},

    -- 根据 id 分段到
    getItemCfg = function(self, cfg_id)
        local item = nil
        if cfg_id <= 30000 then
            -- 武器类，装饰物类
            item = config.weapon_weapons[cfg_id]
        elseif cfg_id <= 60000 then
            -- 资源类
            item = config.item_resources[cfg_id]

        elseif cfg_id <= 66000 then
            item = config.item_useables[cfg_id]
        elseif cfg_id <= 90000 then
            item = config.item_resources[cfg_id]
        elseif cfg_id <= 100000 then
            -- 消耗品， useable
            item = config.item_useables[cfg_id]
        elseif cfg_id <= 120000 then
            -- 图纸 blueprint
            item = config.item_blueprints[cfg_id]
        elseif cfg_id <= 130000 then
            -- 指挥官碎片 piece
            item = config.item_pieces[cfg_id]
        elseif cfg_id <= 150000 then
            -- 船员装备
            item= config.crew_blocks[cfg_id]
        elseif cfg_id <= 180000 then
            -- 装备材料
            item= config.crew_materials[cfg_id]
        elseif cfg_id <= 220000 then
            -- 活动预留
        elseif cfg_id <= 300000 then
        
        elseif cfg_id <= 300500 then
            -- 联盟基本道具
            item = config.item_resources[cfg_id]
        elseif cfg_id <= 301000 then
            -- 联盟礼物
            item = config.item_useables[cfg_id]
        end

        if not item then
            -- 没有找到 item, 遍历所有表找一遍

            -- 遍历所有 item 表找该道具
            for _,tab in ipairs(self.ALL_ITEM_TAB) do
                item = tab[cfg_id]
                if item then return item end
            end
            
            log.error('Failed to find item config in planned table!! item id:'..cfg_id)
        end

        return item;
    end,

    -- 道具升级
    handleUpgradeItem = function(self, msg)
        if msg.respCode == 0 then
            bag_manager:upgradeItem(msg.HeroCfgId, msg.upgradeItem)

            -- 删除被消耗
            local bag = bag_manager:getBag(self.WEAPEN_BAG_ID)   -- 武器背包 id
            for _,v in ipairs(msg.consumeItemIds) do
                bag:removeItem(v)
            end
        else
            log.info('upgrade item failed:'..msg.respCode)
        end
    end,

    -- 筛选未锁定的道具
    getUnlockedItems = function(self, items)
        local tab = {}
        for k, item in ipairs(items) do
            if not item:isLocked() then
                table.insert( tab, item )
            end
        end
        return tab
    end,

    -- 武器素材配置
    getWeaponExpMaterialCfg = function(self, type)
        local tab = {}
        if type ~= 24 and type ~= 25 then
            log.info("weapon exp material type error ! type: ",type)
        else
            config.item_resources:foreach(
                function (k, v)
                    if v.type == type then
                        table.insert( tab, v )
                    end
                end
            )
            table.sort( tab, function(a, b) return a.para1 > b.para1 end )
        end

        return tab
    end,

    sortItemByUseLevel = function(self, items)
        local sort_func = function(a, b)
            if a.cfg and b.cfg and a.cfg.uselevel and b.cfg.uselevel then
                if a.cfg.uselevel == b.cfg.uselevel then
                    return false
                else
                    return a.cfg.uselevel > b.cfg.uselevel
                end
            else
                return false
            end
        end

        table.sort( items, sort_func )
    end,

    sortItemByQuality = function(self, items)
        local sort_func = function(a, b)
            if a.cfg and b.cfg and a.cfg.color and b.cfg.color then
                if a.cfg.color == b.cfg.color then
                    return false
                else
                    return a.cfg.color > b.cfg.color
                end
            else
                return false
            end
        end

        table.sort( items, sort_func )
    end,

    sortItemByEnhanceLevel = function(self, items)
        local sort_func = function(a, b)
            if a.level and b.level then
                if a.level == b.level then
                    return false
                else
                    return a.level > b.level
                end
            else
                return false
            end
        end

        table.sort( items, sort_func )
    end,

    -- 武器套装配置
    getSetConfig = function(self)
        local tab = {}
        config.weapon_setconfigs:foreach(
            function(k, v)
                tab[k] = {}
            end
        )

        config.weapon_sets:foreach(
            function(k, v)
                if tab[v.set_id] then
                    local lv = v.set_num - 1
                    tab[v.set_id][lv] = v
                end
            end
        )

        return tab
    end,

    -- 获得某道具使用 money 作为货币的价格
    getItemBuyPrice = function(self, item_cfg, money)
        if item_cfg.buyprice == "" then
            log.error("Failed to find buyprice:"..item_cfg.id)
            return 0
        end
        local tab = loadstring("return ".. item_cfg.buyprice)()
        return tab[money or 30002] or 0
    end,

    -- 根据类型拿到所有配置
    getItemCfgByType = function(self, item_type)
        local rst = {}
        for __,tab in ipairs(self.ALL_ITEM_TAB) do
            tab:foreach(function(id, cfg)
                if cfg.type == item_type then
                    table.insert(rst, cfg)
                end
            end)
        end
        return rst
    end,

    isOfficerPiece = function(self, cfg_id)
        return cfg_id > 120000 and cfg_id <= 130000
    end,
}

return item_helper