local log = logger.get("Game", "BattleHelper")
local rect_area = require('RTS.areas.rect')
local circle_area = require('RTS.areas.circle')
local damage_calculator = require('RTS.prop_sys.damage_calculator')
local buff_targets = require('RTS.enums').buff_targets
local actor_types = require('RTS.enums').actor_types
local record_types = require('RTS.enums').record_types
local battle_types = require('RTS.enums').battle_types
--local fan_area = require('RTS.areas.fan')

soldier_pool = nil

-- 战斗辅助函数, 相当于单件
battle_helper_inst =
{
    -- 初始化士兵
    initSoldierPool = function(self)
        if not soldier_pool  then
            soldier_pool = require("base.pool") {
                size = 1, 
                transform = game_director.gameObject.transform:CreateChild("soldier_pool"),
            }
            app:startCoroutine( function() 
                --log.error("#### begin preload soldier")
                soldier_pool:wait_prepare('Assets/Prefab/Soldiers/Prefab_blue/soldier_jqb_blue.prefab', 1)
                soldier_pool:wait_prepare('Assets/Prefab/Soldiers/Prefab_blue/soldier_gcb_blue.prefab', 1)
                soldier_pool:wait_prepare('Assets/Prefab/Soldiers/Prefab_blue/soldier_hjb_blue.prefab', 1)
                soldier_pool:wait_prepare('Assets/Prefab/Soldiers/Prefab_blue/soldier_ldz_blue.prefab', 1)

                soldier_pool:wait_prepare('Assets/Prefab/Soldiers/Prefab_red/soldier_gcb_red.prefab', 1)
                soldier_pool:wait_prepare('Assets/Prefab/Soldiers/Prefab_red/soldier_hjb_red.prefab', 1)
                soldier_pool:wait_prepare('Assets/Prefab/Soldiers/Prefab_red/soldier_jqb_red.prefab', 1)
                soldier_pool:wait_prepare('Assets/Prefab/Soldiers/Prefab_red/soldier_ldz_red.prefab', 1)
                --log.error("#### end preload soldier")

                self.is_soldier_preloaded = true    -- 士兵是否预加载过了
            end)
        end
    end,

    -- 初始化
    init = function(self, stage, callback)
        self.pool = game_director.pool
        --[[
                    require("base.pool") {
            size = 5, 
            transform = app.gameObject.transform:CreateChild("pool"),
        }]]

        self.inited = fales
        self.cast_inst_id = 0
        self.curves = {}

        -- 预加载资源
        app:startCoroutine( function() 

            local preloadSkillEffects = function(skills)
                -- 统计 effect 数量
                local effect_num = {}
                for _, skill_id in ipairs(skills) do
                    if skill_id ~= 0 then
                        local skill_cfg = config.skill_skills[skill_id]
                        if not skill_cfg then
                            log.error("Failed to find skill id:"..tostring(skill_id))
                        else
                            local tab = loadstring("return "..skill_cfg.applied_effect)()
                            for k, effect_id in ipairs(tab) do
                                --perload_effect(effect_id)
                                local cfg = config.shoot_effect_effect_libs[effect_id]
                                if cfg then
                                    if effect_num[cfg.effect_path] then
                                        effect_num[cfg.effect_path] = effect_num[cfg.effect_path] + 1
                                    else
                                        effect_num[cfg.effect_path] = 1
                                    end
                                else
                                    log.error("Failed to perload effect:"..tostring(effect_id))
                                end
                            end
                        end
                    end
                end

                for path, num in pairs(effect_num) do
                    self.pool:wait_prepare(path, math.ceil(num*0.5))
                end
            end

           -- self.pool:wait_prepare(consts.assets.battleui, 1)
            if stage.eType == battle_types.REPLAY then
                
                -- 战报模式，预加载小兵弹道
               -- log.error("## 11111")
                self.pool:wait_prepare('Assets/Arts/fx/prefab/loop_soldier_test.prefab', 160)--80
                --log.error("## 222222")

                
               -- log.error("## 3333333")
                self.pool:wait_prepare(consts.assets.foot_circle_T0, 5)
                self.pool:wait_prepare(consts.assets.foot_circle_T1, 5)
                self.pool:wait_prepare(consts.assets.blob_shadow, 10)

                --[[
                log.error("## 444444444444")
                local skills = {}
                for _,tank_cfg_id in ipairs(stage.battle_report.T0.cfg_id) do
                    local weapon_id = config.tank_basic_ids[tank_cfg_id].initial_weapon_id
                    local weapon_cfg = config.weapon_weapons[weapon_id]
                    if weapon_cfg then
                       -- table.insert(skills, weapon_cfg.skill_id)
                        table.insert(skills, weapon_cfg.commonatk_id)
                    end
                end
                for _,tank_cfg_id in ipairs(stage.battle_report.T1.cfg_id) do
                    local weapon_id = config.tank_basic_ids[tank_cfg_id].initial_weapon_id
                    local weapon_cfg = config.weapon_weapons[weapon_id]
                    if weapon_cfg then
                       -- table.insert(skills, weapon_cfg.skill_id)
                        table.insert(skills, weapon_cfg.commonatk_id)
                    end
                end

                preloadSkillEffects(skills)
                ]]

                --log.error("## 55555555555555")
                
            else

                self.pool:wait_prepare(consts.assets.hp_slider_T0, 8)
                self.pool:wait_prepare(consts.assets.hp_slider_T1, 8)
                self.pool:wait_prepare(consts.assets.foot_circle_T0, 8)
                self.pool:wait_prepare(consts.assets.foot_circle_T1, 8)
                self.pool:wait_prepare(consts.assets.blob_shadow, 16)

                -- 非战报模式会有伤害跳字
                self.pool:wait_prepare(consts.assets.damage_text, 50, { active = false })       
                        
                local skills = {}
                local collectHeroSkill = function(hero)
                    local weapon_id
                    if hero.weapon and #hero.weapon > 0 then
                        weapon_id = hero.weapon[1]
                    else
                        if not hero.tank_cfg_id then
                            return
                        end
                        weapon_id = config.tank_basic_ids[hero.tank_cfg_id].default_weapon_id

                    end
                    if weapon_id then
                        local weapon_cfg = config.weapon_weapons[weapon_id]
                        if weapon_cfg then
                            table.insert(skills, weapon_cfg.skill_id)
                            table.insert(skills, weapon_cfg.commonatk_id)
                        else
                            log.error("Failed to find weapon_cfg:"..tostring(weapon_id)..", tank_cfg_id:"..hero.tank_cfg_id)
                        end
                    end
                end

                -- 找到出战的英雄
                for k,hero in ipairs(stage.stage_cfg.T0) do
                    collectHeroSkill(hero)
                end
                for k,hero in ipairs(stage.stage_cfg.T1) do
                    collectHeroSkill(hero)
                end

                preloadSkillEffects(skills)
            end

            
            -- 预加载 curve
            self.curves = consts.curve_insts
            --[[
            for name, path in pairs(consts.assets.curves) do
                coroutine.yield(Fenix.Assets.Helper.WaitForAsset(path, UnityEngine.ScriptableObject, function(asset) 
                    local obj = asset:Get()
                    if asset.package then
                        asset.package:AddRef() 
                    end
                    if obj then
                        self.curves[path] = obj.curve
                    else
                        log.info('failed to load curve:'..path)
                    end
                end))
            end ]]


            --[[
            self.pool:wait_prepare(consts.assets.skill_indicator_rect, 1)
            self.pool:wait_prepare(consts.assets.skill_indicator_circle, 1)
            self.pool:wait_prepare(consts.assets.skill_indicator_single, 1)
            self.pool:wait_prepare(consts.assets.skill_indicator_line, 1)
            self.pool:wait_prepare(consts.assets.skill_indicator_target, 1)]]

           --[[
            for k,v in pairs(gConfig.shoot_effect_effect_libs) do
                if type(k) == 'number' and type(v) == 'table' and type(v[1]) == 'string' then
                    self.pool:wait_prepare(v[1], 5)
                end
            end]]


            self.inited = true
            if callback then
                callback()
            end
        end )

        self.scheduler = require("base.scheduler").new()
        self.stage = stage  -- cache stage
        self.stage_cfg = stage.stage_cfg
        self.is_replay = stage.is_replay
        --self.battle_records = {}    -- 战斗记录 

    end,

    getCameraForward = function(self)
        if self.cam_forward == nil then
            local cam = Camera.main
            self.cam_forward = cam.transform.forward
            self.cam_forward.y = 0.0
            self.cam_forward:Normalize()
        end

        return self.cam_forward
    end,

    -- 是否是重放模式 
    isReplay = function(self)
        return self.is_replay
    end,

    genSkillCastInstID = function(self)
        self.cast_inst_id = self.cast_inst_id + 1
        return self.cast_inst_id
    end,

    -- 在 actor 上播放特效
    playFXOnActor = function(self, effect_id, actor, bone)
        -- 判断特效是否满足播放条件
        local effect_cfg = config.shoot_effect_effect_libs[effect_id]
        local path = effect_cfg.effect_path
        local fx_num = nil 
        local fx_frame = nil 
        local should_play = false
        should_play, fx_num, fx_frame = self:shouldPlayVFXOnActor(actor, path)
        if not should_play then
            return nil
        end

        if effect_cfg.effect_important > 0 then
            should_play = true
        end

        -- 确定要播放该特效,记录特效播放次数和帧数
        if fx_num then
            actor.active_vfx[path] = fx_num + 1
        else
            actor.active_vfx[path] = 1
        end
        actor.active_vfx_frame[path] = actor.frame

        local fx = self.pool:instantiate(effect_cfg.effect_path)

        if fx then 
            if bone == 'shoot' then
                 actor:bindToShootPoint(fx, {0, 0, 0})
            else
                actor:bind(bone, fx, {0, 0, 0})
            end

            self:destroyFxFromActorAfter(fx, path, actor, effect_cfg.effect_time or 3)

        else
            log.error('fx:'..effect_cfg.effect_path.." is nil!")
        end

        return fx
    end,

    -- 定时销毁指定对象
    destroyFxFromActorAfter = function (self, obj, path, actor, time)
        self.scheduler:create(time, 1, 
            function ()
                obj:destroy()
                if actor and actor.active_vfx[path] then
                    actor.active_vfx[path] = actor.active_vfx[path] - 1
                end
            end)
    end, 

    -- 是否应该在角色上播放特效
    shouldPlayVFXOnActor = function(self, actor, path)
        if not actor then
            return false, 0, 0
        end

        if not actor.active_vfx then
            return false, 0, 0
        end

        local fx_num = actor.active_vfx[path]
        local fx_frame = actor.active_vfx_frame[path]

        if fx_num and fx_num > consts.MAX_SAME_FX_ACTOR then  -- 超过相同特效最大存在量
            return false, fx_num, fx_frame
        end

        if fx_frame and actor.frame - fx_frame <= consts.MIN_SAME_FX_FRAME then -- 不满足最小帧间隔
            return false, fx_num, fx_frame
        end

        return true, fx_num, fx_frame
    end,

    -- 播放特效并 attach 到 actor 骨骼上
    playFXAttached = function(self, path, actor, bone, duration, force, ignore_if_no_bone)

        local fx_num = nil --actor.active_vfx[path]
        local fx_frame = nil --actor.active_vfx_frame[path]
        local should_play = false
        should_play, fx_num, fx_frame = self:shouldPlayVFXOnActor(actor, path)
        if force then
            should_play = true
        end

        if not should_play then
            return nil
        end

        -- 确定 bone 是否存在
        if ignore_if_no_bone then
            if not actor.cs_inst:FindChild(bone) then
                return
            end
        end

        -- 确定要播放该特效,记录特效播放次数和帧数
        if actor.active_vfx then
            if fx_num then
                actor.active_vfx[path] = fx_num + 1
            else
                actor.active_vfx[path] = 1
            end
            actor.active_vfx_frame[path] = actor.frame
        end

        local fx = self.pool:instantiate(path)

        if fx then 
            if bone == 'shoot' then
                actor:bindToShootPoint(fx, {0, 0, 0})
            else
                actor:bind(bone, fx, {0, 0, 0})
            end

            self:destroyFxFromActorAfter(fx, path, actor, duration or 3)

            self:playSfx4Vfx(path, actor.gameObject)
        else
            log.error('fx:'..path.." is nil!")
        end

        return fx
    end,

    playSfx4Vfx = function(self, vfx_path, go)
        local sfx =  vfx_sfx_map[vfx_path]
        if sfx and sfx ~= "" then
            log.info("特效播放音效:"..sfx)
            SoundEngine.PostEvent(sfx, go or app.gameObject)
        end
    end,

    -- 播放音效
    playSfx = function(self, sfx_path, go)
        SoundEngine.PostEvent(sfx_path, go)
    end,

    -- 在某位置播放特效
    playFXOnPos = function(self, prefab_path, pos, duration)

        local fx = self.pool:instantiate(prefab_path)

        if fx then 
            fx.transform.position = pos
            self:destroyObjectAfter(fx, duration  or 3)
            self:playSfx4Vfx(prefab_path)
        else
            log.error('fx:'..prefab_path.." is nil!")
        end

        return fx
    end,

    -- 在某位置播放特效，记录在 actor 上
    playFXOnActorPos = function(self, path, actor, pos, duration, force)

        local fx_num = nil --actor.active_vfx[path]
        local fx_frame = nil --actor.active_vfx_frame[path]
        local should_play = false
        if not force then
            should_play, fx_num, fx_frame = self:shouldPlayVFXOnActor(actor, path)
            if not should_play then
                --log.error("#### skip fx:"..path)
                return nil
            end
        else
            should_play = true
        end

        local fx = self.pool:instantiate(path)

        if fx then 
                    -- 确定要播放该特效,记录特效播放次数和帧数
            if fx_num then
                actor.active_vfx[path] = fx_num + 1
            else
                actor.active_vfx[path] = 1
            end
            actor.active_vfx_frame[path] = actor.frame

            fx.transform.position = pos
            self:destroyFxFromActorAfter(fx, path, actor, duration or 3)
            self:playSfx4Vfx(path, actor.gameObject)
        else
            log.error('fx:'..path.." is nil!")
        end
    end,

    -- 读取一个资源
    loadRes = function(self, prefab_path, need_reset_vfx)
        if not prefab_path then return nil end
        
        local res = self.pool:instantiate(prefab_path)
        if not res then
            log.error('pool:instantiate got nil:'..tostring(prefab_path))
        end

        if need_reset_vfx then
            local vfx = res:GetComponent('VFX')
            if vfx then
                vfx:Reset()
            end
        end

        return res
    end,

    loadCurve = function(self, curve_path)
        return self.curves[curve_path]
    end,
    
    -- 加载一个prefab
    loadPrefab = function(self, prefab_path, callback)
        local uobject = self.pool:instantiate(prefab_path)
        if not uobject then
            log.error('Failed to load prefab:'..prefab_path)
            return nil
        end

        if callback then
            if uobject.isLoaded then 
                callback(uobject)
            else 
                uobject.onLoaded = callback
            end
        end
        return uobject
    end,

    -- 受伤跳字
    flyDamageText = function (self, actor, value, time, color, size)
        local dmg = self.pool:instantiate(consts.assets.damage_text)

        dmg:Reset()
        dmg:ScaleTo(0.2)
        if type(value) == "number" then
            dmg:SetText(value or 0)
        else
            dmg:SetStringText(tostring(value))
        end
        dmg:SetColor(color or Color.red)
        dmg:SetTextFontSize(size)
        dmg:Play(string.cache("damage_text_jump"), 0)
        actor:bindToHeadPoint(dmg)

        self.damageList = self.damageList or {}
        table.insert(self.damageList, dmg)

        actor:addDamageText(dmg)
        self:destroyDamageTextAfter(actor, dmg, time or 3)
    end, 

    -- 设置受伤数字的显示和隐藏
    setDamageTextVsb = function(self, vsb)
        if self.damageList then
            for k,obj in pairs(self.damageList) do
                if obj and obj.transform then
                    obj.transform:GetComponentInChildren(UnityEngine.UI.Text).enabled = not vsb
                end
            end
        end
    end,

    -- 定时销毁指定对象
    destroyObjectAfter = function (self, obj, time)
        self.scheduler:create(time, 1, functor.make(obj.destroy, obj))
    end, 

    -- 定时销毁指定对象
    destroyDamageTextAfter = function (self, actor, obj, time)
        self.scheduler:create(time, 1, 
            function ()
                actor:removeDamageText(obj)
                obj:destroy()

                if self.damageList then
                    for k,v in pairs(self.damageList) do
                        if v == obj then
                            table.remove(self.damageList, k)
                        end

                        if v == nil then
                            table.remove(self.damageList, k)
                        end
                    end
                end
            end)
    end, 

    -- 从 soldier group 中选择一个最近的敌人 
    selectNearestFromGroup = function(self, orig, soldier_group)
        local min_d_enemy = nil
        local min_d = 999999999

        for k,v in pairs(soldier_group.soldiers) do
            if v:isAlive() then
                local d = Vector3.SqrDistance(orig, v:getPosition())
                if d < min_d then
                    min_d = d
                    min_d_enemy = v
                end
            end
        end
        return min_d_enemy
    end,

    -- 从 soldier 种随机选择敌人
    selectRandomFromGroup = function(self, soldier_group)
        local soldiers = {}
        for k,v in pairs(soldier_group.soldiers) do
            if v:isAlive() then
                table.insert(soldiers, v)
            end
        end
        local idx = math.random(#soldiers)
        return soldiers[idx]
     end,

    -- 随机选择一个范围内的 vehicle actor
    selectRandomVehicleInRange = function(self, orig, range, team)
        for k,v in pairs(self.stage.actors) do
            if v:isAlive() and v.team == team then
                local d = Vector3.SqrDistance(orig, v:getPosition())
                if d < range * range then
                    return v
                end
            end
        end
        return nil
    end,

    -- 选择最近的战车 actor
    selectNearestVehicle = function(self, orig, team)
        local min_d = 99999999
        local min_d_actor = nil
        for k,v in pairs(self.stage.actors) do
            if v:isAlive() and v.team == team and v:getActorType() == actor_types.VEHICLE then
                local d = Vector3.SqrDistance(orig, v:getPosition())
                if d < min_d then
                    min_d = d
                    min_d_actor = v
                end
            end
        end
        return min_d_actor
    end,

    -- 选择某 team 阵营的随机位置
    selectRandomPosOfTeam = function(self, team)
        -- 取 stage cfg 中阵型位置的随机位置
        local area
        if team == 0 then
            area = self.stage.stage_cfg.areaT0
        else
            area = self.stage.stage_cfg.areaT1
        end
        
        return Vector3(math.random(area[1], area[3]), 0, math.random(area[2], area[4]))

        --[[
        local select_num = math.random(2, 5)
        local pos_count = 0
        local pos_sum = Vector3.zero
        local max_limit = 20
        while (select_num > 0) do
            for k,v in pairs(self.stage.actors) do
                if v.team == team and v:getActorType() == actor_types.VEHICLE then
                    if math.random(0, 1) > 0.75 then
                        pos_sum = pos_sum + v:getPosition()
                        pos_count = pos_count + 1
                        select_num = select_num - 1
                    end
                end
            end

            max_limit = max_limit - 1
            if max_limit <= 0 then select_num = 0 end
        end

        if pos_count > 0 then
            return pos_sum / pos_count
        else
            return nil
        end]]
    end,

    -- 隐藏角色头顶 UI
    setVehicleHeadUIEnable = function(self, team, enabled)
        for k,v in pairs(self.stage.actors) do
            if v.team == team and v:getActorType() == actor_types.VEHICLE and v.head_info then
                v.head_info.gameObject:SetActive(enabled)
            end
        end
    end,

    -- 用户点击坦克，游戏暂停，进入施放必杀技态
    tryCastUltimateSkill = function(self, actor_id)
        --log.info('User try case ultimate skill:'..actor_id)
        self.stage:tryCastUltimateSkill(actor_id)
    end,

    -- 释放必杀技
    castUltimateSkill = function(self)
        --log.info('battle_helper: User case ultimate skill')
        self.stage:castUltimateSkill()
    end,

    -- 对 team 执行范围伤害
    doRectDamage = function(self, attacker, team, center, rot, length, width, skill_cfg)
        local half_extents = Vector3.New(width*0.5, 1.0, length*0.5)
        local rect = rect_area(team, center, rot, half_extents)

        --log.info('bh doRectDamage:'..attacker.name..', damage:'..damage..', c:'..tostring(center)..', r:'..tostring(rot)..','..length..','..width)
        local actors = rect:intersectActors(self.stage.actors)
        if not actors then return end
        
        --log.info('rect intersect actors:'..#actors)
        for _, actor in pairs(actors) do
            -- todo: 从技能表获取伤害计算函数名
            local damage, is_hit, is_crit, curr_hp = attacker:calculateDamage(actor, skill_cfg, attacker.cast_inst_id)
            actor:beAttacked(attacker, skill_cfg, damage, is_hit, is_crit, attacker.cast_inst_id, battle_helper.is_replay, curr_hp)

        end
    end,

    doCircleDamage = function(self, attacker, team, center, radius, skill_cfg)
        if radius == 0 then
            log.error('doCircleDamage, radius is 0!')
            return
        end

        local actors = self:getIntersectActorsCircle(team, center, radius)
        if actors then
            for _,actor in pairs(actors) do 
                -- todo: 从技能表获取伤害计算函数名
                local damage, is_hit, is_crit, curr_hp = attacker:calculateDamage(actor, skill_cfg, attacker.cast_inst_id)
                actor:beAttacked(attacker, skill_cfg, damage, is_hit, is_crit, attacker.cast_inst_id, battle_helper.is_replay, curr_hp)
            end
        end
    end,

    getIntersectActorsCircle = function(self, team, center, r)
        local circle = circle_area(team, center, r)
        local actors = circle:intersectActors(self.stage.actors)
        return actors
    end,

    -- 获取buff目标角色列表
    getBuffTargetActors = function(self, actor, target)  
        local rst = {}
        local self_team = actor.team
        if target == buff_targets.SELF then
            table.insert(rst, actor)

        elseif target == buff_targets.ENEMY then
            table.insert(rst, actor.target_actor)

        elseif target == buff_targets.ALL_ENEMY_VEHICLE then
            for k,v in pairs(self.stage.actors) do
                if self_team ~= v.team and v:getActorType() == actor_types.VEHICLE then
                    table.insert(rst, v)
                end
            end

        elseif target == buff_targets.ALL_ENEMY_SOLDIER then
            for k,v in pairs(self.stage.actors) do
                if self_team ~= v.team and v:getActorType() == actor_types.SOLDIER then
                    table.insert(rst, v)
                end
            end

        elseif target == buff_targets.ALL_TEAMATE_VEHICLE then
            for k,v in pairs(self.stage.actors) do
                if self_team == v.team and v:getActorType() == actor_types.VEHICLE then
                    table.insert(rst, v)
                end
            end

        elseif target == buff_targets.ALL_TEAMATE_SOLDIER then
            for k,v in pairs(self.stage.actors) do
                if self_team == v.team and v:getActorType() == actor_types.SOLDIER then
                    table.insert(rst, v)
                end
            end

        end

        return rst
    end,

    -- 重置 actor 的 ai
    resetAI = function(self, actor)
        self.stage.ai_world:resetAI(actor)
    end,

    update = function(self, dt)
        self.scheduler:update(dt)
    end,

    addSelectEnemyRecord = function(self, actor, enemy)
        --[[
        if not self.is_replay then
            local enemy_id = nil
            if enemy then enemy_id = enemy.id end
            local rec = {
                record_type = record_types.SELECT_ENEMY,
                time = Time.time,
                actor_id = actor.id,
                enemy_id = enemy_id,
            }
            table.insert(self.battle_records, rec)
        end
        ]]
    end,

    playCamAnim = function(self, anim)
        self.stage.level_manager:PlayCameraAnimation(anim)
    end,

    addToCameraView = function(self, actor)
        if actor then
            self.stage.level_manager:AddToCameraView(actor.transform);
        end
    end,

    setCameraSizeFactor = function(self, size_factor, smooth_factor)
         self.stage.level_manager:SetCameraSizeFactor(size_factor, smooth_factor)
    end,

    removeFromCameraView = function(self, actor)
        if actor then
            self.stage.level_manager:RemoveFromCameraView(actor.transform);
        end
    end,

    -- 获取战斗经过的时间
    getBattleTime = function(self)
        local t = self.stage.battle_time
        return math.floor(t * 100 + 0.5) * 0.01   -- 精确到 10毫秒
    end,

    -- 解析 1-2-3-4-5-6
    split = function(self, s)
        if type(s) ~= "string" then
            log.error("battle_helper 解析失败...", s, type(s))
        end

        if not self.frame_helper then
            self.frame_helper = require("gui.tool.frame_helper")
        end

        s = string.sub(s, 2, #s - 2)

        return self.frame_helper:split(s, '-')
    end,

    
    -- 1~3 肉盾
    -- 4~6 突击
    -- 7~9 坦克
    -- 10~12 火炮
    -- 13 无阵
    -- return 1-克制 0-平等  -1-被克制
    getFormationEffective = function(self, team)
        local standard = function(id)
            if id >=1 and id <= 3 then
                return 1
            elseif id >=4 and id <= 6 then
                return 4
            elseif id >=7 and id <= 9 then
                return 7
            elseif id >=10 and id <= 12 then
                return 10
            else
                return 13
            end
        end
    
        local t0 = standard(self.stage_cfg.formationT0)
        local t1 = standard(self.stage_cfg.formationT1)
        local result = 0

        if t0 == 1 then
            if t1 == 1 then result = 0
            elseif t1 == 4 then result = 1 
            elseif t1 == 7 then result = -1 
            elseif t1 == 10 then result = -1
            elseif t1 == 13 then result = 1
            end
        elseif t0 == 4 then
            if t1 == 1 then result = -1
            elseif t1 == 4 then result = 0 
            elseif t1 == 7 then result = -1 
            elseif t1 == 10 then result = 1
            elseif t1 == 13 then result = 1
            end
        elseif t0 == 7 then
            if t1 == 1 then result = 1
            elseif t1 == 4 then result = 1 
            elseif t1 == 7 then result = 0 
            elseif t1 == 10 then result = 1
            elseif t1 == 13 then result = 1
            end
        elseif t0 == 10 then
            if t1 == 1 then result = 1
            elseif t1 == 4 then result = -1 
            elseif t1 == 7 then result = -1 
            elseif t1 == 10 then result = 0
            elseif t1 == 13 then result = 1
            end
        else
            if t1 == 1 then result = -1
            elseif t1 == 4 then result = -1 
            elseif t1 == 7 then result = -1 
            elseif t1 == 10 then result = -1
            elseif t1 == 13 then result = 0
            end
        end
        
        if team == 0 then
            return result
        else
            return -result
        end
    end,

    -- 销毁
    destroy = function(self)
        log.info('battle_helper.destroy')
        if not self:isReplay() then
            log.info('battle_helper add stage battle record!!')
           --battle_record_manager:addBattleRecord(self.stage.battle_id, self.battle_records)
        end
        
        self.scheduler:destroy()
       -- self.pool:destroy()
    end,

}

return battle_helper_inst