local log = logger.get("Game", "buff")

--[[
buff cfg 说明：
buff_type 
1. 数值属性的修正：生命、攻击，防御、CD、能量、命中、闪避、暴击、免爆
2. 瘫痪状态
3. 燃烧状态
4. 减速状态
5. 护盾
6. 中毒
7. 潜行

work_condition， work_para:
1. 立刻生效，全程不变
2. 不立刻生效，时间触发
3. 不立刻生效，行为触发
4. 不立刻生效，属性判定触发
clean_condition, clean_para
1. 时间移除
2. 触发次数移除
3. 死亡移除


]]


return class {
    ctor = function(self, owner, caster, prop_group, cfg, inst_id)
        self.owner = owner -- buff handler
        self.owner_actor = owner.owner_actor
        self.caster = caster -- 技能施放者
        self.prop = prop_group -- buff prop base
        self.cfg = cfg
        if not self.cfg then 
            log.error('Failed to find buff cfg:'..tostring(cfg_id))
            return nil
        end

        self.inst_id = inst_id                  -- 实例 id

        if self.cfg.clean_condition == 1 then
            self.life = self.cfg.clean_para         -- buff 持续时间
        elseif self.cfg.clean_condition == 2 then
            self.times = 0                          -- 激活次数
        end

        -- 事件触发
        if self.cfg.work_condition == 2 then
            self.active_time = self.cfg.work_para   -- 每隔多久激活一次，0 就只开始激活一次
        end

        self.actived = fales                    -- 是否激活过(至少一次)了
        
        self.values ={}                         -- 记录 buff 改变的属性
        for k,v in pairs(cfg.prop_tab) do
            self.values[k] = 0.0
        end

        self.effect = nil       -- buff 在角色上播放的特效
    end,

    -- 是否是每隔一段时间激活一次，共激活多次效果的 buff
    isEffectOfTime = function(self)
        return self.cfg.work_condition == 2 
    end,

    doActive = function(self)
        if self.actived then
            log:error('buff already actived!! :'..cfg.name)
            return false
        end

        if self.cfg.buff_type == 1 then
            self.prop:appendFromPropTab(self.cfg.prop_tab)
            -- 记录 buff 增加的属性
            for k,v in pairs(self.cfg.prop_tab) do
                self.values[k] = self.values[k] + v
            end            
        elseif self.cfg.buff_type == 5 then
            -- 护盾状态
            if self.cfg.prop_tab[1] then
                self.owner_actor.shield = self.cfg.prop_tab[1]
            elseif self.cfg.prop_tab[2] then
                self.owner_actor.shield = 
                    self.owner_actor.props:getProp(prop_types.MAX_HP) * self.cfg.prop_tab[2]
            end
        elseif self.cfg.buff_type == 7 then
            -- 潜行
            self.owner_actor.is_sneak = true
        end

        self.actived = true

        if self.times then
            self.times = self.times + 1
        end

        return self.cfg.prop_tab
    end,

    doDeactive = function(self)
        if self.cfg.buff_type == 1 and self.actived then
            self.prop:subtractFromPropTab(self.values)
        end

        if self.cfg.buff_type == 5 then
            self.owner_actor.shield = 0
            --log.info("#### shield doDeactive!!")
        elseif self.cfg.buff_type == 7 then
            -- 潜行
            self.owner_actor.is_sneak = false
        end

        if self.effect then
            self.effect:destroy()
        end

        return self.cfg.prop_tab
    end,

    -- 返回是否还存在
    update = function(self, dt)
        if self.life then
            self.life = self.life - dt
        end

        -- 时间移除的
        if self.actived and self.active_time and self.active_time > 0.0 then
            self.active_time = self.active_time - dt
            if self.active_time <= 0 then
                self.actived = false
                self.owner:onActiveTime(self) -- 通知 buff handler 触发时间到了
                self.active_time = self.cfg.work_para
            end
        end

        if self.actived and self.cfg.buff_type == 5 and self.owner_actor.shield <= 0 then
            -- 盾被打爆
            self.actived = false
            self.owner:onActiveTime(self) -- 通知 buff handler 触发时间到了
            self.active_time = self.cfg.work_para
        end

        if self.life then
            return self.life > 0.0
        elseif self.times then
            return self.times >= self.cfg.clean_para
        else
            return true
        end
    end,
}