local log = logger.get("Game", "buff")
local buff_class = require("RTS.buff.buff")
require("RTS.prop_sys.prop_enum")

return class {
    ctor = function(self, owner_actor)
        self.owner_actor = owner_actor  -- 拥有者 actor
        self.props = owner_actor.props
        self.buff_group = self.props:getGroup(prop_group_types.BUFF)

        self.buffs = {} -- 容器
        self.dirty = false -- 是否buff发生了变化
        self.buff_guid = 0
        self.frame_prop_tab = {}    -- 每帧里变化的属性集合
        self.hp_or_energy_changed = false
    end,

    -- 生成每个 buff 的唯一 id
    genBuffGuid = function(self)
        self.buff_guid = self.buff_guid + 1
        return self.buff_guid
    end,

    -- 添加一个 buff
    addBuff = function(self, caster, cfg_id)

        local buff_cfg = config.skill_buffs[cfg_id]
        
        -- 判断是否能添加这个 buff
        if not game_info.is_replay then
            local can_add, remove_idx =  self:canAddBuff(buff_cfg)
            if can_add then
                if remove_idx then
                    self.buffs[remove_idx]:doDeactive()
                    table.remove(self.buffs, remove_idx)
                end
            else
                return
            end
        end

        local inst_id = self:genBuffGuid()
        local buff = buff_class(self, caster, self.buff_group, buff_cfg, inst_id)
        table.insert(self.buffs, buff)

        -- 立刻生成
        if buff.cfg.work_condition == 1 then
            self:active(buff)   
        end

        -- 播放特效
        if buff.cfg.buff_effect > 0 then
            local effect = config.shoot_effect_effect_libs[buff.cfg.buff_effect]
            buff.effect = battle_helper:playFXAttached(effect.effect_path, self.owner_actor, buff.cfg.buff_effect_bone, effect.effect_time)
        end
     end,

    -- 是否可以添加 buff, 返回 bool 和 被顶退的 buff_idx
    canAddBuff = function(self, buff_cfg)
        if not buff_cfg then
            return false
        end

        if buff_cfg.buff_add_type == 0 then
            return true
        end

        -- 具有相同的 buff_add_type, 并且 strength 比原来的大或相等就顶退
        for idx, buff in pairs(self.buffs) do
            if buff.cfg.buff_add_type == buff_cfg.buff_add_type then
                if buff_cfg.buff_add_strength >= buff.cfg.buff_add_strength then
                    return true, idx
                else
                    return false
                end
            end
        end

        return true
    end,

    update = function(self, dt)
        self.hp_or_energy_changed = false

        local buffs = self.buffs
        -- 遍历更新所有 buff, 删除生命结束的
        for i=#buffs,1,-1 do
            if not buffs[i]:update(dt) then                               
                local buff = buffs[i]
                self:deactive(buffs[i])
                table.remove(buffs, i)
            end
        end

        -- 属性 dirty, 重新计算 final
        if self.dirty then
            local orig_max_hp = self.props:getProp(MAX_HP_IDX)
            local orig_max_energy = self.props:getProp(MAX_ENERGY_IDX)
            local orig_hp = self.props:getProp(HP_IDX)

            self.props:computeFinal()

            -- HP, Energy 发生变化， 需要改变 actor 的值
            if orig_max_hp ~= self.props:getProp(MAX_HP_IDX) or 
                orig_max_energy ~= self.props:getProp(MAX_ENERGY_IDX) then
                -- 刷新界面
                self.owner_actor.max_hp = self.props:getProp(MAX_HP_IDX)
                self.owner_actor.max_energy = self.props:getProp(MAX_ENERGY_IDX)
            end

            local dHP = self.props:getProp(HP_IDX) - orig_hp
            if dHP ~= 0.0 then
                self.owner_actor:addHP(dHP)
            end

            self.dirty = false
        end
    end,

    active = function(self, buff)
        buff:doActive()
        self.dirty = true
    end,

    deactive = function(self, buff)
       -- if buff.cfg.cleaning == 1 then    -- buff 结束时是否还原属性
            buff:doDeactive()
            self.dirty = true
        --end
    end,

    onActiveTime = function(self, buff)
        self:active(buff)
    end,



}