local log = logger.get("Game", "Cheat")
local gui_helper = require('gui.gui_helper')
return {
    stage = nil,
    debug = false,
    showActors = function(self)
        log.info('show actors!!!')
        for k,v in pairs(self.stage.actors) do 
            
            log.info('actor:'..v.name..' state:'..v.curr_state.name)

        end

        self.debug = true
    end,

    -- 打印某角色属性
    printProp = function(self, id)
        local actor = self.stage.actors[id]
        log.info(actor.name..' cfg:'..actor.cfg_id..' type:'..tostring(actor.actor_type))
        if actor and actor.props then
            actor.props:log()
        end
    
    end,

    -- 打印某 actor 属性 
    prop = function(self, id)
        self:printProp(id)
    end,

    -- 模拟伤害计算
    attack = function(self, a_id, d_id)
        local a = self.stage.actors[a_id]
        local d = self.stage.actors[d_id]
        local skill_id = a.getCommonAtkID()
        local skill_cfg = config.skill_skills[skill_id]
        local damage_calculator = require('RTS.prop_sys.damage_calculator')
        local dmg, is_hit, is_crit = damage_calculator.calculate(damage_calculator, a, d, skill_cfg)

        log.info(a.name..' attack '..d.name.. ' hit:'..tostring(is_hit)..' crit:'..tostring(is_crit)..' dmg:'..dmg)
    end,

    -- 给一个 actor 加 buff
    buff = function(self, actor_id, buff_id)
        local actor = self.stage.actors[actor_id]
        actor.buff_handler:addBuff(actor, buff_id)
        
    end,

    -- 设置时间速率
    setTimeScale = function(self, scale)
        Time.timeScale = scale
    end,

    -- 所有角色能量加满
    fullEnergy = function(self)
        for _,actor in pairs(self.stage.actors) do
            actor.energy = 999999
        end
    end,

    -- 给一个角色装备道具
    equip = function(self, hero_id, item_id, slot_id)
        log.info('## equip hero_id:'..hero_id ..' inst_id:'..item_id..' slot:'..slot_id)
        network_sender:heroEquip(hero_id, tostring(item_id), slot_id, protocol_pb.IU_EQUIP)
    end,

    -- 解除装备
    unequip = function(self, hero_id, item_id, slot_id)
        log.info('## unequip hero_id:'..hero_id..' inst_id:'..item_id..' slot:'..slot_id)
        network_sender:heroEquip(hero_id, tostring(item_id), slot_id, protocol_pb.IU_UNEQUIP)
    end,

    -- 道具升级
    upgradeItem = function(self, hero_id, item_id, consume_items)
    --[[
        local hero_bag = bag_manager:getHeroBag(1)
        local slot_id = 1
        local upgrade_item = hero_bag:get(slot_id)

        local consume_items = {}

        local bag = bag_manager:getBag(0)
        for _,v in pairs(bag.items) do
            if v.cfg.type == upgrade_item.cfg.type then
                table.insert(consume_items, v.instId)
            end
        end]]
        local tab = {}
        for _,v in ipairs(consume_items) do
            table.insert(tab, tostring(v))
        end

        network_sender:upgradeItem(hero_id, tostring(item_id), tab)
    end,

    -- 指挥官晋升
    promote = function(self, officer_id)
        network_sender:officerPromote(officer_id)
    end,

    -- 指挥官觉醒
    awake = function(self, officer_id)
        network_sender:officerAwake(officer_id)
    end,

    -- 打印背包
    printBag = function(self)
        bag_manager:print()
    end,

    -- 打印指挥官
    printOfficer = function(self)
        hero_manager:printOfficer()
    end,

    -- 打印所有收藏
    printCollection = function(self)
        local collections = game_director.curr_state.collections
        local str = ""
        for k,v in pairs(collections) do
            str = str.."["..v.id.."] ".." tab:"..v.eTable.." desc:"..v.strDesc.." X:"..
                v.posX.." Y:"..v.posY.." dist:"..v.marchDist..' time:'..v.marchTime.."\n"
        end
        log.info(str)
    end,

    -- 添加收藏
    addCollection = function(self, eTab, desc, iconId, x, y)
        network_sender:worldMapCollectionAdd(eTab, desc, iconId, x, y)
    end,

    -- 修改收藏
    modCollection = function(self, id, eTab, desc)
        network_sender:worldMapCollectionMod(id, eTab, desc)
    end,

    -- 删除收藏
    delCollection = function(self, id)
        network_sender:worldMapCollectionDel(id)
    end,

    -- 显示背包界面
    showFrame = function(self, frame_name)
        log.info('kkkkk')
        frame_manager:show(frame_name, self)
    end,

    -- 获取某道具数量
    getItemNum = function(self, item_id)
        local num = bag_manager:getItemNum(item_id)
        log.info('item:'..item_id..', num:'..num)
    end,

    -- 选择章节出战英雄
    selectAttendHeros = function(self, chapter_id, level_id, heroes)
        network_sender:selectChapterAttendHeros(chapter_id, level_id, heroes)
    end,

    -- 请求章节出战英雄
    reqAttendHeros = function(self, chapter_id)
        network_sender:reqChapterAttendHeros(chapter_id)
    end,

    -- 布阵
    levelEmbattle = function(self, chapter_id, heros)
        network_sender:levelEmbattle(chapter_id, heros, 0)
    end,

    -- 测试 ui_tank
    testUITank = function(self)
        self.ui_tank = gui_helper:createUITank(1)
        self.ui_tank.transform.localScale = Vector3.New(20,20,20)

    end,

    testUITankWeapon = function(self, weapon_id)
        if self.ui_tank then
            self.ui_tank:setMainWeapon(weapon_id)
        end
    end,

    -- 请求商店信息
    reqShop = function(self, shop_type)
        network_sender:reqShopItemList(shop_type)
    end,

    -- 购买商品
    buyGoods = function(self, shop_type, index)
        network_sender:buyGoods(shop_type, index)
    end,

    -- 打印商店信息
    printShop = function(self, shop_type)
        local shop = shop_manager:getShop(shop_type)
        if shop then
            shop:print()
        else
            log.error("Failed to find shop:"..shop_type)
        end
    end,

    -- (关卡中)应用新的英雄
    performNewHero = function(self)
        local hero_id = 0
        for k,v in pairs(hero_manager.new_heros) do
            hero_id = k
            break
        end

        level_script:performNewHero(hero_id)
    end,

    -- 重连
    reconnect = function(self)
        reconnector:reconnect()
    end,

    -- 重新进关卡
    resetLevel = function(self)
        game_director:changeState('level', {scene_name = level_info.level_cfg.Map, level_id = level_info.map_id})
    end,

    -- 关卡中消失一个 entity
    disappear = function(self, id)
        if game_state_level then
            game_state_level.level:entityDisappear(id)
        end
    end,

    -- 掉落
    dropItem = function(self, id)
        if not game_state_level.level then
            return
        end

        local pos = game_state_level.level.player:getPosition()
        local item = level_helper:dropItem(pos, id, 1)

        table.insert(game_state_level.drop_items, item)
    
    end,

    -- 邀请加入工会
    familyInvite = function(self, player_id)
        network_sender:familyInvite(player_id)
    end,

    -- 清除PlayerPrefs
    clearCache = function(self)
        UnityEngine.PlayerPrefs.DeleteAll()
    end,

    -- 显示商店
    showShop = function(self)
        network_sender:getShopList()
    end,

    -- 本地战斗
    battle = function(self)
        level_info.is_simple_battle = true
        game_director:changeState('battle', {stage_id=1, battle_id=1})
    end,

    -- 战斗回放
    replay = function(self)
        --[[
        level_info.is_simple_battle = true
        local path = ""
        if file_name then
            path = Fenix.App.GetStoragePath()
            path = path .. file_name
        else
            path = Fenix.Assets.Helper.GetStreamingAssetsPath() .. "battle_rec.lua" --"DummyBattleRecord_1.lua"
        end
        ]]

        -- 战报数据
        local battle_report = {

            S0 = {
                cfg_id = {20002, 20003, 20004, 20005},
                total = {1205, 1800, 3010, 2000},  -- 四种士兵参战人数
                left = {600, 1200, 500, 900},
            },
            S1 = {
                cfg_id = {20002, 20003, 20004, 20005},
                total = {800, 1203, 2800, 1001},  -- 四种士兵参战人数
                left = {200, 120, 1000, 500},
            },

            T0 = {
                cfg_id = {32, 33, 41, 42, 44},
            },

            T1 = {
                cfg_id = {10, 14, 15, 22, 30},--, 33,
            },
            win_team = 0,
            win_degree = 1, -- 胜利的程度
            turns = 30,     -- 回合数
            name0 = "NameTeam0",
            name1 = "NameTeam1",
            icon0 = 10,
            icon1 = 20,
            timeOut = true,
        }

        game_director:changeState('loading', {next_state = 'battle_replay', scene_name = 'desert_1', battle_report = battle_report})
    end,

    -- 船战
    shipBattle = function(self)
        
        -- 战报数据
        local battle_report = {

            -- 士兵
            S0 = {
                cfg_id = {20002, 20003, 20004, 20005},
                total = {1205, 1800, 3010,2000},  -- 四种士兵参战人数
                mid_left = {1100, 1600, 1500, 1800}, -- 半程剩余
                left = {600, 1200, 500, 900},
            },

            S1 = {
                cfg_id = {20002, 20003, 20004, 20005},
                total = {800,1203,2800,1001},  -- 四种士兵参战人数
                mid_left = {800, 1203, 2800, 1001},  -- 半程剩余
                left = {200, 120, 1000, 500},
            },

            -- 战车
            T0 = {
                cfg_id = {17, 39, 10, 33, 42},
            },

            T1 = {
                cfg_id = {17, 10, 39, 33, 42},--, 33,
            },

            -- 防守方炮台
            Cannon = {
                cfg_id = {21002,21003,21004},
                total = {4000, 3000, 2000},
                left = {0, 0, 0},
            },

            mid_win_team = 0,
            win_team = 0,
            win_degree = 1, -- 胜利的程度
            defence_total = 1000,
            defence_left = 0, -- 防御值损失
            break_wall_turns = 10,  -- 城战回合数
            occupy_turns = 30,     -- 第二场回合数
            name0 = "NameTeam0",
            name1 = "NameTeam1",
            icon0 = 10,
            icon1 = 20,
            is_ship_battle = true,
        }

        game_director:changeState('loading', {next_state = 'battle_replay', scene_name = 'desert_ship_battle', battle_report = battle_report})
    end,

    -- 设置车头灯是否开启
    enableLight = function(self, enable)
        game_state_level.level.player:setHeadLightEnable(enable)
    end,

    -- 测试巡逻
    testPatrol = function(self)
        local entity = 
            battle_helper.level:getEntity(13)
        entity:beginPatrol({
            Vector3.New(-331, 0, -124),
            Vector3.New(-228, 0, -121),
            Vector3.New(-179, 0, -78),
        }, 30)
    end,

    -- 测试操作区域
    testOpTrigger = function(self)
        level_script:createOpTrigger(1, 1, nil, Vector3.New(-354, 0, 156), 20, 
            function()
               level_script:removeOpTrigger(1)
            end)
    end,

    -- 播放对话
    playDialog = function(self, scenario_id)
        --[[
        level_script:playDialog(id, 
        function ()
            log.info("### dialog finished!!")
        end)]]
        
        dialog_manager:playScenario(scenario_id, 
            function ()
                log.info("### dialog finished!!")
            end)
    end,

    -- 对话表演脚本
    performance = function(self, id)
        dialog_frame.dialog.performance = id
        dialog_frame:playPerformance()
    end,

    -- 关卡结束
    finishLevel = function(self, is_success)
        level_script:finishLevel(is_success)
    end,

--[[
    playRec = function(self, file_name)
        local storagePath = Fenix.App.GetStoragePath()
        local path = "file://"..storagePath .. (file_name or "battle_rec.lua")

         game_director:changeState('battle_replay', {url = path})
    end,]]

    -- 对话设置表情
    setEmotion = function(self, emotion)
        if dialog_frame then
            dialog_frame:setNpcEmotion(dialog_frame.left_npc, emotion)
        end
    end,

    -- 播放摄像机动画
    playCamAnim = function(self)
        level_script:playLevelCameraAnim("scenario-1-2_2")
    end,

    -- 显示聊天界面
    showChat = function(self)
        frame_manager:show('chat')
    end,

    -- 关卡玩家移动到某点
    playerMoveTo = function(self)
        level_script:breakPlayerInput()
        level_script:playerMoveTo(Vector3.New(-400, 0, -219.0), 300, 
            function() 
                level_script:resumePlayerInput()
                log.info("## move callback!!")
            end)
    end,

    -- 中断操作
    breakInput = function(self)
        level_script:breakPlayerInput()
    end,

    -- 打开章节界面
    showChapter = function(self)
        frame_manager:show("chapter")
    end,

    -- 打开章节界面
    showHomeSkin = function(self)
        frame_manager:show("home_skin")
    end,

    -- 显示关卡布阵
    showLevelFormation = function(self)
        print("game_director.curr_state.name:", game_director.curr_state.name)
        if game_director.curr_state.name == "level" then
            game_director.curr_state.level:showFormation()
        end
    end,

    -- 显示关卡布阵设置
    showLevelFormationSet = function(self)
        print("game_director.curr_state.name:", game_director.curr_state.name)
        if game_director.curr_state.name == "level" then
            game_director.curr_state.level:showFormationSet()
        end
    end,

    -- 显示资源补充界面
    showResourceConvert = function(self)
        frame_manager:show('resource_convert', nil, 
        {
            bar_idx = 1,
            need_res = {
                [32001] = 10000,
                -- [30002] = 10000,
                -- [32002] = 10000,
            }
        })
    end,

    -- 显示修理界面
    showHomeRepair = function(self)
        frame_manager:show('home_repair')
    end,
    
    showUseSlider = function(self)
        frame_manager:show('use_slider', nil,
        {
            item_id = 33014,
        })
    end,

    showBuySlider = function(self)
        frame_manager:show('buy_slider')
    end,

    -- 测试对话脚本
    dialogScript = function(self)
        dialog_frame.dialog_script:runScript("dialog:aa('aaaa') dialog:showLeftIcon()")
    end,

    -- 对话显示自定义图片
    customImg = function(self)
        dialog_frame:showCustomImage("Assets/Arts/ui/atlas/icon/union/guild_head_3.png", Vector3.zero, 0.3, 2.0, 0.1)
    end,

    -- 某 id 角色被击飞
    hitFly = function(self, id)
        local actor = self.stage.actors[id]
        if actor then
            actor:changeState("hit_fly")
        end
    end,

    -- 选择多个敌人
    selectMultiple = function(self, id)
        local world = self.stage.ai_world
        local agent = world.agents[id]
        local enemys = 
            self.stage.ai_world:selectMultipleEnemyByRange(agent, 3, false)
        for _,e in ipairs(enemys) do
            log.info("## Selet:", e.id)
        end

    end,

    -- 战斗胜利
    win = function(self)
        for _,actor in pairs(self.stage.actors) do
            if actor.team ~= 0 then
                actor:addHP(-(actor.hp + 999))
            end
        end
    end,

    -- 战斗失败
    lose = function(self)
        for _,actor in pairs(self.stage.actors) do
            if actor.team == 0 then
                actor:addHP(-(actor.hp+999))
            end
        end
    end,

    -- 己方剩余 20% 血
    half = function(self)
        for _,actor in pairs(self.stage.actors) do
            if actor.team == 0 then
                actor:addHP(-(actor.hp*0.8))
            end
        end
    end,

    -- 设置目标帧数
    setFPS = function(self, fps)
        UnityEngine.Application.targetFrameRate = fps

    end,

    -- 怪物移动到路点
    monsterMove = function(self)
        for k,entity in pairs(level.entities) do
            if entity then
                if entity.entity_type == common_pb.MET_MONSTER then
                    entity:beginPatrol({1,2,3,4,5,6,7}, 200)
                end
            end
        end
    end,

    testProp = function(self)
        network_sender:levelChangeHeroProp(0, -1)
    end,

    hideEntity = function(self, id, is_hide)
        if is_hide then
            level_script:hideEntity(id)
        else
            level_script:showEntity(id)
        end
    end,

    option = function(self)
        level_script:showDialogOption('level_script_96', {'level_script_91','level_script_92'},
        function(idx)
            log.info(idx)
        end)
    end,   

    orderPatrol = function(self, id)
        local monster = level_script.level:getEntity(id)
        monster:changeState("order_patrol", {points = {2,3,4,5,6,5,4,3}, speed = 100})
    end,

    -- 模拟战斗
    simBattle = function(self, level)
        local msg = protocol_pb.CSTestFightProp()
        local cfg = gConfig.stages[1]
        -- T0
        local t0 = msg.playerFormation
        t0.formationId = cfg.formationT0
        t0.teamId = 0
        for idx, line in ipairs(cfg.T0) do
            local hero = t0.formationHero:add()
            hero.heroId = idx
            hero.tankId = line.tank_cfg_id
            hero.officerId = line.officer_cfg_id
            hero.location = line.slot_pos
            hero.fightProp.HeroId = idx
            hero.fightProp.tankId = line.tank_cfg_id
            hero.fightProp.officeId = line.officer_cfg_id
            hero.fightProp.HeroLevel = level or 1
            hero.fightProp.pos = line.slot_pos
            hero.fightProp.monsterStatId = line.monster_stat_id
        end
     
        -- T1
        local t1 = msg.enemyFormation
        t1.formationId = cfg.formationT1
        t1.teamId = 1
        for idx, line in ipairs(cfg.T1) do
            local hero = t1.formationHero:add()
            hero.heroId = idx
            hero.tankId = line.tank_cfg_id
            hero.officerId = line.officer_cfg_id
            hero.location = line.slot_pos
            hero.fightProp.HeroId = idx
            hero.fightProp.tankId = line.tank_cfg_id
            hero.fightProp.officeId = line.officer_cfg_id
            hero.fightProp.HeroLevel = level or 1
            hero.fightProp.pos = line.slot_pos
            hero.fightProp.monsterStatId = line.monster_stat_id
        end

        network_sender:simBattle(msg)
    end,

    setResUsed = function(self)
        UsingResourceChecker.setResUsed("Assets/Shader/", 1)
    end,

    levelOver = function(self)
        level_info.level_over = true
        game_state_level.gui:levelOver()
    end,

    showIcon = function(self)
        game_state_level.level.player:showIcon(
            "Assets/Arts/ui/atlas/icon/resources/resources_4.png", 
            "+1", true)
    end,

    showMissionProgress = function(self, str)
        game_state_level.gui:showMissionProgress(str or "任务名称 1/100")
    end,

    highLightItem = function(self)
        game_state_level.gui:highLightItem(31005)
    end,

    emptyGas = function(self)
        level_info.gasoline = 1
    end,

    -- 显示顶部的技能打断条
    bar = function(self)
        battle_helper.stage.gui:showTopBar(999, true, true)
        battle_helper.stage.gui:showTopSkillList(true, {1,2,3})
    end,

    mulKillsEffect = function(self)
        print('fuck')
        battle_helper.stage.gui:showDamage(1, 100)
    end,

    bt = function(self)
        self:battle()
    end,

    rm = function(self, type)
        battle_helper.stage.gui:checkTopSkillIcon(type)
    end,

    laser = function(self)
        local a = nil
        local b = nil

        for k, actor in ipairs(battle_helper.stage.actors) do
            if not a and actor.team == 0 then
                a = actor
            end

            if not b and actor.team == 1 then
                b = actor
            end
        end

        a:sk_shootLaser(b, "shoot", config.shoot_effect_effect_libs[1], 
            config.shoot_effect_effect_libs[2], a.cs_inst:FindChild("shoot"), b.cs_inst:FindChild("hit"))

    end,

    toEmpty = function(self)
        game_director:changeState("empty")
        SceneManager.LoadScene("empty")
        frame_manager:hide("loading")
        
        Fenix.Game.GameApp.GC()

        app:startCoroutine(
            function()
                coroutine.yield(WaitForSeconds(1.0))
                collectgarbage("collect")
                coroutine.yield(WaitForSeconds(1.0))
                collectgarbage("collect")
                coroutine.yield(WaitForSeconds(1.0))
                collectgarbage("collect")
            end
        )

    end,

    checkOC = function(self)
        Fenix.Game.GameApp.CheckObjectCache()
    end,

    a = function(self)
        self:toEmpty()
    end,

    b = function(self)
        game_director:changeState('loading', {next_state = 'world_map', scene_name = 'hex_map', level_id = 2001})
    end,

    c = function(self)
        if self.c_state == nil then
            self.c_state = false
        end
        self.c_state = not self.c_state
        if self.c_state then
            game_director:changeState('loading', {next_state = 'world_map', scene_name = 'hex_map'})
        else
            game_director:changeState('loading', {scene_name = 'main', next_state = 'login'})
        end
    end,

    -- 加载 spine 立绘
    spine = function(self)
        local path = "Assets/Arts/spines/17_moxi/17_moxi_SkeletonData.asset"
        --[[
        app:startCoroutine(
            function()
                local path = "Assets/Arts/spines/17_moxi/17_moxi_SkeletonData.asset"
                coroutine.yield(Fenix.Assets.Helper.WaitForAsset(path, UnityEngine.ScriptableObject, function(asset) 
                    local sda = asset:Get() -- skeleton data asset
                    if sda then
                        --local sa = Spine.Unity.SkeletonAnimation.NewSkeletonAnimationGameObject(obj)
                        --sa:Initialize(false)

                        Spine.Unity.SkeletonGraphic.NewSkeletonGraphicGameObject()
                    else
                        log.info('failed to load spine:'..path)
                    end
                end)) 
            end
        )]]
        dialog_frame:changeNpcToSpine(0, path)

    end,

    rect = function(self)
        game_state_world_map.world_map.cs_inst:DrawRect(Vector3.zero, Vector3(2000, 0, 1000))
    end,

    levelFailed = function(self)
        level_script:finishLevel(false)
    end,

    postError = function(self)
        log.error("报错了报错了！")
    end,

    setMoveSpeed = function(self, speed)
        game_director.curr_state.world_map.cs_inst:SetCameraMoveSpeed(speed, speed)
    end,

    fps = function(self, is_fps)
        game_state_level:setFPSMode(is_fps)
    end,

    -- 运行一个引导
    guide = function(self, id)
        require ('guide.guide_manager')

        app:startCoroutine(
            function()
                coroutine.yield(nil)
                if not id then
                    guide_manager:tryStartGuide()
                else
                    guide_manager:_doGuide(id)
                end
            end
        )

    end,

    chapter = function(self)
        frame_manager:show("chapter_achievement")
        network_sender:reqChapterAchievementList()
    end,

    packGuide = function(self)
        local packed = GuideInfoPacker.pack({1,2,32,33,65,66,67,99,127})
        local unpacked = GuideInfoPacker.unpack(packed)

        for k,v in pairs(unpacked.Table) do
            log.info("##"..k..':'..v)
        end

    end,

    -- 测试保存引导
    saveGuide = function(self)
        app:startCoroutine(
            function()
                local tab = {}
                for i=10001,10127 do
                    tab[i] = true
                end
                network_sender:reqSaveGuide(100, tab)
                network_sender:reqNotifyGuide()
                coroutine.yield(WaitForSeconds(1.0))
                for i=10001, 10127 do
                    log.info(i..":" .. tostring(guide_helper:isGuideComplete(i)))
                end
            end)
    end,

    noGuide = function(self)
        local tab = {}
        for i=10001,10127 do
            tab[i] = true
        end
        network_sender:reqSaveGuide(9999, tab)
        network_sender:reqNotifyGuide()
    end,
    
    changeLanguage = function(self, id)
        localization_manager:set(id)
    end,

    -- 测试创建世界地图区域
    region = function(self)
        local hex_map_lua = game_state_world_map.world_map.cs_inst
        local color = Color(12/255, 243/255, 110/255, 10/255)
        hex_map_lua:DrawRaingRegion(20, 10, 50, 20, 30, 12, 40, 18, color)
    end,

    seekLeak = function(self)
        require('global_check')

        -- leak_ui = {}
        -- m0 = {}
        -- m1 = {}
        -- m2 = {}

        -- m0.a = leak_ui
        -- m1.b = leak_ui
        -- m2.c = leak_ui

        findObjectInGlobal(leak_ui)

        -- log.error('界面数量:', #frame_manager.visibleList)
        -- for k,v in ipairs(frame_manager.visibleList) do
        --     log.error('当前显示的界面:', v.key)
        -- end
    end,

    -- 章节成就过场
    cutscene = function(self, id)
        frame_manager:show("chapter_achievement_cutscene", nil, id or 1)
    end,

    chapterStart = function(self, id)
        frame_manager:show("chapter_start", nil, id or 1)
    end,

    upgradeTest = function(self, count)
        app:startCoroutine(
            function()
                local i = 1
                while true do
                    if i > count then
                        break
                    end
                    local msg = protocol_pb.CSUpGradeItem()
                    msg.HeroCfgId = 41
                    msg.instId = "11"
                    msg.consumeItems:append("19")
                    msg.consumeItems:append("21")
                    msg.consumeItems:append("15")
                    msg.consumeItems:append("22")
                    msg.consumeItems:append("23")
                    msg.consumeItems:append("17")
                    msg.consumeItems:append("18")
                    msg.consumeItems:append("16")
                    msg.consumeItems:append("20")
                    network_sender:send(protocol_pb.UPGRADEITEM, msg)
                    coroutine.yield(WaitForSeconds(0.5))

                    i = i + 1
                end
            end
        )
    end,

    setMyMoney = function(self, num)
        app:startCoroutine(function()
            network_sender:gm("additem 32001 " .. num)
            network_sender:gm("additem 30002 " .. num)
            network_sender:gm("additem 32002 " .. num)
            network_sender:gm("additem 32003 " .. num)
            network_sender:gm("additem 32004 " .. num)
            -- coroutine.yield(nil)        
        end)
    end,

    iWantMoney = function(self)
        app:startCoroutine(function()
            network_sender:gm("additem 32001 99999990")
            network_sender:gm("additem 30002 99999990")
            network_sender:gm("additem 32002 99999990")
            network_sender:gm("additem 32003 99999990")
            network_sender:gm("additem 32004 99999990")
            network_sender:gm("additem 30001 99999990")
            -- coroutine.yield(nil)        
        end)
    end,

    iWantPoor = function(self)
        app:startCoroutine(function()
            network_sender:gm("additem 32001 -99999990")
            network_sender:gm("additem 30002 -99999990")
            network_sender:gm("additem 32002 -99999990")
            network_sender:gm("additem 32003 -99999990")
            network_sender:gm("additem 32004 -99999990")
            -- network_sender:gm("additem 30001 99999990")
            -- coroutine.yield(nil)        
        end)
    end,

    showExchangeFrame = function(self)
        frame_manager:show('resource_exchange', nil, { 
            item_id = 50001,
            item_num = 1000000,
        })
    end,

    showHomeBuff = function(self)
        frame_manager:show('home_buff')
    end,


    -- 修改出征界面带兵数量上限
    setMaxCount = function(self, value)
        if launch_rally then
            launch_rally:setMaxCount(value)
        end
    end,

    showHomeDefense = function(self)
        frame_manager:show('home_defense')
    end,

    showLordTalent = function(self)
        frame_manager:show('lord_talent')
    end,
    
    drillSoldier = function(self, path_id, speed, num)
        local spd, n
        if speed then
            spd = speed
        else
            speed = 10
        end

        if num then
            n = num
        else
            n = 5
        end

        if path_id > 0 then
            drill_ground_helper:createPathMovingSoldier(1, path_id, spd, n, 0.15)
        else
            app:startCoroutine(
                function()
                    for i = 1, 5 do
                        drill_ground_helper:createPathMovingSoldier(1, i, spd, n, 0.15)
                        coroutine.yield(WaitForSeconds(3.0))
                    end
                end
            )
        end
    end,

    chouka = function(self, weapon_type, count)
        for i = 1, count do
            network_sender:armoryProductEquip(weapon_type, 1)    
        end
    end,
    
    squareSoldier = function(self)
        drill_ground_helper:refreshSquareSoldiers()
    end,

    -- 显示出征界队伍属性加成
    showAllProps = function(self)
        if launch_rally then
            launch_rally:showAllProps()
        end
    end,

    enterMap = function(self, id)
        network_sender:enterMap(id, 0)
    end,

    initChapterResShip = function(self)
        chapter_manager:initChapterResShip()
    end,

    -- 设置章状态
    setChapterState = function(self, chapter_id, state, cur_level)
        --chapter_manager:setChapterInfo(chapter_id, state, curr_level)
        network_sender:setChapterState(chapter_id, state)
    end,

    -- 本地设置 chapter 状态
    setLocalChapter = function(self, chapter_id, state, curr_level)
        chapter_manager:setChapterInfo(chapter_id, state, curr_level)
    end,

    -- 打开关卡选择界面
    levelSelect = function(self, id)
        frame_manager:show('level_select', nil, {chapter_id = id or 1})
    end,

    -- 解锁关卡
    unlockLevel = function(self, chapter_id, level_id)
        network_sender:unlockLevel(chapter_id, level_id)
    end,

    drawCircleRegion = function(self, x, y, radius)
        world_map_helper:getGuildBaseRangeCells(x, y, radius)
    end,

    growthTask = function(self)
        frame_manager:show('growth_task')
    end,


    showActivity = function(self)
        frame_manager:show('activity_main')
    end,

    hideTavernReward = function(self)
        frame_manager:hide('home_tavern_reward')
    end,

    --fps调整开枪子弹
    setFpsBulletEffect = function(self, id)
        game_state_level:updateShootEffect(id)
    end,

    -- 
    findNearestPoint = function(self)
        level.level_script:playerFindNearestPathPoint()
    end,

    redpoints = function(slef)

        require('config.redpoint')
        local points = {}

        config.redpoint_redpoints:foreach(
            function(k,v)
                table.insert(points, {id=v.id, count = v.id})
            end
        )

        local msg = {
            points = points
        }

        red_point_manager:appear(msg)
    end,

    clearRedpoints = function(self)
        local ids = {}

        config.redpoint_redpoints:foreach(
            function(k,v)
                table.insert(ids, v.id)
            end
        )

        local msg = {
            ids = ids
        }

        red_point_manager:disappear(msg)
    end,

    battleSettlement = function(self, win)
        if win then
            game_info.battle_result = 1
        else
            game_info.battle_result = 0
        end

        frame_manager:show('battle_settle')
    end,

    showPowerEffect = function(self)
        frame_manager:show('effect_power_up', nil, {
            pre_power = 12345678,
            cur_power = 123456789,
        })
    end,

    findAvailableGrid = function(self)
        local entity = guide_helper:findAvailableGrid(12)
        if entity then
            log.info("## found entity:"..entity.grid_id)
        else
            log.error("## not found!")
        end
    end,

    unlockResBoat = function(self)

        app:startCoroutine(function()
            network_sender:gm("SetBuildingLevel 1 20")
            coroutine.yield(nil)        
            network_sender:gm("setchapterstate 1 1")
            coroutine.yield(nil)        
            network_sender:gm("setchapterstate 1 2")
            coroutine.yield(nil)        
            network_sender:gm("setchapterstate 1 3")
            coroutine.yield(nil)        
            network_sender:gm("setchapterstate 1 4")
            -- coroutine.yield(nil)        
        end)

    end,

    saveUsedList = function(self)
        ResChecker.ms_instance:SaveUsedRes()
    end,

    refreshRedPoints = function(self)
        red_point_manager:refreshRedPoints()
    end,

    searchGuide = function(self)
        app:startCoroutine(
            function()
                guide_helper:waitForceButtonGuide(
                    "world_map_search", "BG/Monster/Left/Scroll View/Viewport/Content/Res (3)")
            end
        )
    end,

    heroGuide = function(self)
        app:startCoroutine(
            function()
                guide_helper:waitForceButtonGuide(
                    "world_map_search", "BG/Monster/Left/Scroll View/Viewport/Content/Res (3)")
            end
        )
    end,

}