local log = logger.get("Game", "crew manager")
local item_cls = require('RTS.bag_sys.item')

-- 船员系统管理器
local crew_manager_cls = class{
    ctor = function(self)
        self.events = require('base.event').new()
        self.equiped = {}   -- slot - item
    end,

    -- 判断是否有可装备的船员
    canEquipCrew = function(self, slot)
        local bag = bag_manager:getBag(11)
        local can_equip = false
        for _,item in pairs(bag.items) do
            if slot == item.cfg.Slot and local_lord.level >= item.cfg.UserLevel then
                can_equip = true
            end
        end
        return can_equip
    end,

    -- 装备船员
    equip = function(self, item_info)
        local slot = item_info.status
        log.info("crew_manager equip:"..slot)

        if self.equiped[slot] == nil then
            self.equiped[slot] = item_cls(item_info)

        else
            -- 在已存在的槽位上装备船员
            if self.equiped[slot].inst_id == item_info.instId then
                -- 是同一个船员
                self.equiped[slot]:refresh(item_info)
            else
                -- 
                log.error("equip crew, slot already exist, slot:"..slot)
            end
        end
    end,

    -- 解除装备
    unequip = function(self, slot)
        log.info("crew_manager unequip:"..slot)
        if self.equiped[slot] then
            self.equiped[slot] = nil
        else
            log.error('failed to find equiped crew:'..slot)
        end
    end,

    -- 获取每个槽位上装备的船员
    getEquipedCrew = function(self)
        return self.equiped
    end,
    
    -- 根据槽位获取所有船员 （已装备和未装备）
    getCrewBySlot = function(self, slot, to_sort)
        local tab = {}
        if self.equiped[slot] then
            table.insert(tab, self.equiped[slot])
        end

        local bag = bag_manager:getBag(11)
        for _,item in pairs(bag.items) do
            if slot == item.cfg.Slot then
                table.insert(tab, item)
            end
        end

        if to_sort then
            table.sort(tab, function(a, b)
                if a.slot ~= b.slot then
                    return a.slot > b.slot
                end

                if a.cfg.UserLevel ~= b.cfg.UserLevel then
                    return a.cfg.UserLevel > b.cfg.UserLevel
                end

                if a.cfg.color ~= b.cfg.color then
                    return a.cfg.color > b.cfg.color
                end

                return a.cfg.id > b.cfg.id
            end)
        end

        return tab
    end,

    -- 根据槽位获取已装备的
    getEquippedCrewBySlot = function(self, slot)
        return self.equiped[slot]
    end,

    -- 是否装备
    isItemEquipped = function(self, item_inst_id)
        for _,item in pairs(self.equiped) do
            if item.inst_id == item_inst_id then
                return true
            end
        end
        return false
    end,

    -- 获得数量根据道具id
    getItemNumByCfgId = function(self, cfg_id)
        local rst = 0
        for _,item in pairs(self.equiped) do
            if item.cfg_id == cfg_id then
                rst = rst + 1
            end
        end
        return rst
    end,
}

crew_manager = crew_manager or crew_manager_cls()

return crew_manager