local log = logger.get("Game", "Hero")
require 'RTS.bag_sys.bag_manager'
local officer_cls = require('RTS.hero_sys.officer')

--[[
//英雄信息
message MHeroInfo
{ 
	required uint32     Id              = 1;//英雄唯一id 
	optional uint32		tankId			= 2;//坦克配置id
	optional uint32 initAttr			= 3;//初始属性id
	optional MCarBody carBody			= 4;//车体
	optional HERO_STATUS     HeroStatus = 5;//英雄状态 0不在阵中(组合值见枚举HERO_STATUS)
	repeated string     EquipIds        = 6;//所装备的装备Id,存库时使用 
	repeated MItemInfo     Equips       = 7;  //英雄所装备的装备，下发给客户端时使用
	optional uint32 petId               = 8 ; //小兵id
	optional MOfficerInfo officerInfo   = 9; //指挥官信息
	optional uint32 armour				=10; //当前护甲
	optional uint32 isRandOfficer       =11; //是否是随机指挥官0不是，1是
}
]]
-- 指挥官及其对应战车
return class{
    ctor = function(self, hero_info, hero_props, actived, global_props)
        self.id = hero_info.Id  -- 英雄id  
        self.tank_id = hero_info.tankId  --  英雄 id 和 战车 id 分开
        self.tank_cfg = config.tank_basic_ids[self.tank_id]
        self.officer = nil
        self.hero_info = hero_info
        self.hero_props = hero_props
        self.global_props = global_props
        self.is_actived = actived
        self.battle_force = hero_info.sword   -- 战斗力
        self.tankBody_info = hero_info.carBody
        self.strength = hero_info.strength      -- 实力
        self.state = hero_info.HeroStatus

        if not self.tank_cfg then
            log.error("### Failed to find tank cfg:"..self.tank_id)
            log.error(debug.traceback())
        end

        self:init()
    end,

    init = function(self)
        if self.is_actived then
            -- 指挥官
            if not self.hero_bag then
                self.hero_bag = bag_manager:initHeroBag(self.id)
            end
            --[[message MItemInfo
            {
                required string         instId     = 1;  //道具唯一ID
                optional uint32        cfgId       = 2;  //道具模板ID
                optional EQUIP_SLOT    status   = 3;  //战车装备槽位
                optional uint32        count       = 4;  //道具个数
            }]]
            -- 遍历英雄的所有装备，添加到英雄背包
            for k,item in ipairs(self.hero_info.Equips) do
                --log.info('$$$$$ hero equip:'..item.cfgId..', inst id:'..item.instId..', Status:'..tostring(item.status))
                self.hero_bag:equip(item)
            end
        end

        -- 创建 officer
        if self.hero_info.officerInfo then
            if not self.officer then
                self.officer = hero_manager:createOfficer(self.hero_info.officerInfo, self.id, self.is_actived)
            else
                self.officer:refresh(self.hero_info.officerInfo, self.id, self.is_actived)
            end
        end

    end,

    refresh = function(self, hero_info, hero_props, actived, global_props)
        self.id = hero_info.Id
        self.tank_id = hero_info.tankId
        self.hero_info = hero_info
        self.hero_props = hero_props
        self.global_props = global_props
        self.strength = hero_info.strength
        self.battle_force = hero_info.sword   -- 战斗力
        self.tankBody_info = hero_info.carBody
        self.state = hero_info.HeroStatus

        if not self.is_actived then
            self.is_actived = actived
            self:init()
        end


        hero_manager.events:dispatch("HEROREFRESH", self)
    end,

    updateState = function(self, state)
        self.state = state
    end,

    updateStrength = function(self, value)
        self.strength = value
    end,

    updateBattleForce = function(self, value)
        self.battle_force = value
    end,

    -- 根据 插槽 id 获取道具
    getEquipItem = function(self, slot_id)
        if self.hero_bag then
            return self.hero_bag:get(slot_id)
        else
            return nil
        end
    end,

    -- 设置指挥官
    setOfficer = function(self, officer)
        self.officer = officer
    end,

    --获取需要显示的属性
    getHeroShowedProps = function(self)
        if not self.hero_props then
            return {}
        end

        local props = {}
        for k, v in ipairs(self.hero_props) do
            if v.PropValue ~= 0 and config.attribute_config_attrs[v.PropId].show_order ~= 0 then
                table.insert( props, {PropId = v.PropId, PropValue = v.PropValue} )
            end
        end
        
        local sort_fun = function(a, b)
            local cfg = config.attribute_config_attrs
            return cfg[a.PropId].show_order < cfg[b.PropId].show_order
        end

        table.sort(props, sort_fun)

        return props
    end,

    getHeroPorp = function(self, prop_id)
        for k, v in ipairs(self.hero_props) do
            if v.PropId == prop_id then
                return v
            end
        end

        return nil
    end,

    getGlobalProp = function(self, prop_id)
        for k, v in ipairs(self.global_props) do
            if v.PropId == prop_id then
                return v.PropValue
            end
        end

        return 0
    end,

    -- 更新英雄属性
    updateHeroProps = function(self, hero_props)
        for k, v in ipairs(hero_props) do
            local prop = self:getHeroPorp(v.PropId)
            if prop then
                prop.PropValue = v.PropValue
            else
                table.insert(self.hero_props, v)
            end
        end
    end,

    -- 获取解锁的副武器槽位数量
    getMinorWeaponSlotNum = function(self)
        if not self.hero_props then
            return 0
        end
        for k, v in ipairs(self.hero_props) do
            if v.PropId == common_pb.PARTNER_PROP_SECOND_WEAPON_NUM then
                return v.PropValue
            end
        end

        return 0
    end,

    -- 更新英雄车体改造信息
    updateHeroCarBodyInfo = function(self, carBody_Info)
        self.tankBody_info = carBody_Info
    end,

    isWeakness = function(self)
        return self.state == common_pb.ETEAM_WEAK or self.state == common_pb.ETEAM_CAPTURE
    end,

    hpEmpty = function(self)
        local cur_hp = self:getHeroPorp(common_pb.PARTNER_PROP_HP)
        return cur_hp.PropValue <= 0
    end,

}