-- 英雄管理
local log = logger.get("Game", "Hero Manager")
local hero_cls = require('RTS.hero_sys.hero')
local officer_cls = require('RTS.hero_sys.officer')

local hero_manager_cls = class{
    ctor = function(self)
        self:init()
    end,

    init = function(self)
        self.heros = {} -- 所有英雄
        self.officers = {}  -- 所有指挥官
        self.new_heros = {}    -- 服务器通知了新增，但还没有显示给玩家的英雄

        self.events = require('base.event').new()
    end,


    --[[
    //英雄信息
    message MHeroInfo
    { 
        required uint32     Id              = 1;  //英雄唯一id 
        optional uint32		tankId			= 2;  //坦克配置id
        optional HERO_STATUS     HeroStatus = 3;  //英雄状态 0不在阵中(组合值见枚举HERO_STATUS)
        repeated string     EquipIds        = 4;  //所装备的装备Id,存库时使用 
        repeated MItemInfo     Equips       = 5;  //英雄所装备的装备，下发给客户端时使用
        optional uint32 petId               = 6; //小兵id
        optional MOfficerInfo officerInfo   = 7; //指挥官信息
        optional uint32 armour				= 8; //当前护甲
    }]]
    -- 创建英雄
    createHero = function(self, hero_info, hero_props, global_props)
        local hero_id = hero_info.Id -- 英雄 id

        if self.heros[hero_id] then
            self.heros[hero_id]:refresh(hero_info, hero_props, true, global_props)
        else
            self.heros[hero_id] = hero_cls(hero_info, hero_props, true, global_props)
        end

        return self.heros[hero_id]
    end,

    -- 创建未获得的英雄
    createUnactivedHero = function(self, hero_id, tank_id, officer_id)
        local hero_info = {Id = hero_id, tankId = tank_id, officerInfo={officerId = officer_id}}

        self:unactiveHeroRepeat(tank_id)
        self.heros[hero_id] = hero_cls(hero_info, nil, false)

        return self.heros[hero_id]
    end,

    createStoryHero = function(self, hero_id, tank_id, officer_id)
        local hero_info = {Id = hero_id, tankId = tank_id, officerInfo={officerId = officer_id}}
        self.heros[hero_id] = hero_cls(hero_info, nil, false)
 
        return self.heros[hero_id]
    end,

    unactiveHeroRepeat = function(self, tank_id)
        for k, v in pairs(self.heros) do
            if not v.is_actived and v.tank_id == tank_id then
                self.heros[v.id] = nil
            end
        end
    end,

    -- 拥有的指挥官
    getOwnOfficers = function(self)
        local rst = {}
        for k, hero in pairs(self.heros) do
            if hero.is_actived then
                table.insert(rst, hero.officer)
            end
        end
        return rst
    end,

    getHero = function(self, hero_id)
        return self.heros[hero_id]
    end,

    -- 创建指挥官
    createOfficer = function(self, officer_info, hero_id)
        local id = officer_info.officerId
        local officer = officer_cls(officer_info, hero_id)
        self.officers[officer.inst_id] = officer
        return officer
    end,

    -- 获得所有英雄，用 comp 比较函数 对 hero 进行排序
    getHerosSort = function(self, comp)
        local tab = {}
        for _,v in pairs(self.heros) do
            table.insert(tab, v)
        end

        table.sort(tab, comp)

        return tab
    end,

    -- 根据等级比较两个 hero
    compHeroByOfficerLevel = function(a, b)
        if not a.is_actived and not b.is_actived then
            return a.officer.cfg_id > b.officer.cfg_id
        elseif not b.is_actived then
            return true
        elseif not a.is_actived then
            return false
        end

        return a.officer.level > b.officer.level
    end,

    -- 根据 star 比较两个 hero 
    compHeroByOfficerStar = function(a, b)
         if not a.is_actived and not b.is_actived then
            return a.officer.cfg_id > b.officer.cfg_id
        elseif not b.is_actived then
            return true
        elseif not a.is_actived then
            return false
        end

        if a.officer.star_level == b.officer.star_level then
            return a.officer.awake_level > b.officer.awake_level
        else
            return a.officer.star_level > b.officer.star_level
        end
    end,

    -- 根据战斗力比较两个 hero
    compHeroByBattleForce = function(a, b)
        if not a.is_actived and not b.is_actived then
            return a.officer.cfg_id > b.officer.cfg_id
        elseif not b.is_actived then
            return true
        elseif not a.is_actived then
            return false
        end

        return a.strength >= b.strength
    end,

    -- 根据品质比较两个 hero
    compHeroByColor = function(a, b)
        if not a.is_actived and not b.is_actived then
            if a.officer.cfg.color == b.officer.cfg.color then
                return a.officer.cfg_id > b.officer.cfg_id
            else
                return a.officer.cfg.color > b.officer.cfg.color
            end
        elseif not b.is_actived then
            return true
        elseif not a.is_actived then
            return false
        end

        if a.officer.cfg.color == b.officer.cfg.color then
            return a.officer.cfg_id > b.officer.cfg_id
        else
            return a.officer.cfg.color > b.officer.cfg.color
        end
    end,

    getOfficer = function(self, officer_id)
        return self.officers[officer_id]
    end,

    printOfficer = function(self)
        local str = ""
        for _, officer in pairs(self.officers) do
            str = str .. officer:toString() .. "\n"
        end
        log.info('=================officers================')
        log.info(str)
    end,

    getUnDispatchedHeros = function(self)
        local tab = {}
        for _, hero in pairs(self.heros) do
            if hero.is_actived then
                if hero.state == common_pb.ETEAM_NONE then
                    table.insert( tab, hero )
                end
            end
        end
        return tab
    end,

    getNeedRepairHero = function(self)
        local tab = {}
        for _, hero in pairs(self.heros) do
            if hero.is_actived then
                if hero.state == common_pb.ETEAM_NONE or 
                    hero.state == common_pb.ETEAM_FIXING or 
                    hero.state == common_pb.ETEAM_WAITFIXING then
                        table.insert( tab, hero )
                end
            end
        end
        return tab
    end,

    getHeroAmount = function(self)
        local amount = 0

        for k, v in pairs(self.heros) do
            if v.is_actived then
                amount = amount + 1
            end
        end

        return amount
    end,

    heroAmountMax = function(self)
        local amount = self:getHeroAmount()
        local max = local_player:getMaxHeroCnt()

        return amount >= max
    end,

    -- 是否有指挥官可以升星或者招募
    haveStarUpOfficers = function(self)
        local frame_helper = require("gui.tool.frame_helper")
        local count = 0
        for _, hero in pairs(self.heros) do
            if hero and hero.officer.cfg.isshow > 0 then
                --log.info("## hero id:"..hero.id..", "..tostring(hero.is_actived))
                if hero.is_actived then
                    -- 判断是否可以晋升
                    if hero.officer:canStarUp() then
                        --log.info("## can star up")
                        count = count + 1
                    end
                else
                    if hero.officer:canRecruit() then
                        --log.info("## can recruit")
                        count = count + 1
                    end
                end
            end
        end

        if count == 0 then
            log.info("### no StarUp officer")
            return false, 0
        else
            log.info("### have StarUp officer")
            return true, count
        end
    end,

    clear = function(self)
        self.heros = {} -- 所有英雄
        self.officers = {}  -- 所有指挥官
        self.new_heros = {}    -- 服务器通知了新增，但还没有显示给玩家的英雄

    end,
}

-- 玩家英雄管理器
hero_manager = hero_manager_cls()
-- 剧情英雄管理器
story_hero_manager = hero_manager_cls()

return hero_manager