local log = logger.get("Game", "Officer")
local hero_helper = require('RTS.hero_sys.hero_helper')

-- 指挥官
return class
{
--[[
//指挥官信息
message MOfficerInfo
{
  required uint32  officerId                 = 1; //指挥官Id
  required uint32  officerLv                 = 2; //指挥官等级
  required uint32  officerExp                = 3; //经验
  required uint32  qualityLv                 = 4; //军衔等级
  required uint32  starLv                    = 5; //指挥官星级
  required uint32  awakenLv                  = 6; //觉醒等级
  repeated MParPropIdValue  officerProp      = 7; //指挥官属性
  optional SEX_TYPE sex						 = 8; //性别
  optional uint32 resId				         = 9; //资源id
  optional string name				         = 10; //姓名
  optional uint32 skillId				     = 11; //技能id
}]]

    ctor = function(self, officer_info, hero_id, is_actived)
        --log.info("##Create officer, hero_id:"..tostring(hero_id))

        self:refresh(officer_info, hero_id, is_actived)
    end,

    refresh = function(self, officer_info, hero_id, is_actived)
        self.cfg_id = officer_info.officerId
        self.inst_id = hero_id
        self.hero_id = hero_id  -- officer 属于哪个英雄
        self.is_actived = is_actived
        self.cfg = config.commander_basics[self.cfg_id]  -- 指挥官基础配置

        if not self.cfg then
            log.error("### Failed to find officer cfg:"..self.cfg_id)
            log.error(debug.traceback())
            return
        end

        self.promotion_cfg = nil -- 晋升配置
        self.awake_cfg = nil     -- 觉醒/升星 配置

        self.level = officer_info.officerLv or 0  -- 指挥官等级
        self.exp = officer_info.officerExp or 0   -- 指挥官经验
        self.level_exp = 100 -- config.commander_levelups[self.level]   -- 本级的升级经验

        --self.quality = officer_info.qualityLv  or 0      -- 品质(军衔)，通过 promote 晋升

        --self.awake_level = officer_info.awakenLv or 0    -- 觉醒等级    
        self.star_level= officer_info.starLv or 0        -- 星级
        
        --log.info('##### create officer:'..self.cfg_id..' lv:'..self.level)

        --self.promotion_bag_type = 2 --晋升材料背包类型

        if officer_info.name and string.len(officer_info.name) >0 then
            self.name = officer_info.name
        else
            self.name = frame_helper:getLangCfg(self.cfg.commander_name)
        end

        --self.sex = officer_info.sex or self.cfg.gender
        --self.res_id = officer_info.resId or officer_info.officerId
        self.skills = officer_info.skillList or {}
        
        --self:init()
    end,

    updateSkill = function(self, skill)
        self.skills = skill or {}
    end,

    toString = function(self)
        local str = "Officer:"..self.cfg_id..', level:'..self.level..', exp:'..self.exp..', '..
            'quality:'..self.quality..', star:'..self.star_level..', awake:'..self.awake_level..
            ', sex:'..self.sex..', name:'..self.name..', res id:'..self.res_id;
        return str
    end,

    -- 根据军衔获取晋升配置
    getPromotionCfg = function(self, quality)
        return self.promotion_cfg[quality]
    end,

    -- 根据星级和觉醒等级获取觉醒配置
    getAwakeCfg = function(self, star_level, awake_level)
        local key = star_level*10 + awake_level
        return self.awake_cfg[key]
    end,

    -- 获取指挥官最大军衔
    getMaxPromote = function(self)
        local max = 0
        for k, v in pairs(self.promotion_cfg) do
            max = max + 1
        end

        return max
    end,

    -- 获取指挥官的最大星级
    getMaxStar = function(self)
        local max_lv = 0

        for k, v in pairs(self.awake_cfg) do
            local lv = math.floor(k/10)
            if max_lv < lv then
                max_lv = lv
            end
        end

        return max_lv
    end,

    -- 获取对应星级的觉醒次数
    getAwakeNum = function(self, star_level)
        local num = 0

        for k, v in pairs(self.awake_cfg) do
            local lv = math.floor(k/10)
            if star_level == lv then
                local n = k%10
                if num < n then
                    num = n
                end
            end
        end

        return num
    end,

    getSkillUnlockStar = function(self, idx)
        local mods = passive_skill_helper:getSkillMods(self.cfg.skill_mode)

        if mods then
            local tab = {}
            for k, v in pairs(mods) do
                table.insert( tab, v )
            end

            table.sort( tab, function(a, b) return a.starlevel < b.starlevel end )

            for k, v in ipairs(tab) do
                local list = loadstring("return "..v.skill_level)()

                if type(list) == "table" then
                    if list[idx] and list[idx] > 0 then
                        return v.starlevel
                    end
                end
            end
        else
            log.error("########################################### skill mod error ! id", self.cfg.skill_mode)
        end

        return 999
    end,

    -- 获取指挥官对应的战车 id
    getTankId = function(self)
        return self.cfg.tank_id
    end,

    init = function(self)
        self.promotion_cfg = hero_helper:getPromotionCfg(self.cfg.growup)
        self.awake_cfg = hero_helper:getAwakeCfg(self.cfg.growup)
        
    end,

    -- 经验，等级发生变化
    expChange = function(self, exp, level)
        self.exp = exp
        self.level = level
    end,

    -- 晋升
    promote = function(self, quality)
        self.quality = quality
    end,

    -- 觉醒
    awake = function(self, star_level, awake_level)
        self.star_level = star_level       
        self.awake_level = awake_level
    end,

    -- 检查是否达成晋升等级
    checkPromoteLevel = function(self)
        local cfg = self.promotion_cfg[self.quality]
        if self.level >= cfg.level_limit then
            return true, cfg.level_limit
        else
            return false, cfg.level_limit
        end
    end,

    -- 检查晋升道具是否足够
    checkPromoteItem = function(self)
        local cfg = self.promotion_cfg[self.quality]
        local bag = bag_manager:getBag(self.promotion_bag_type)   -- 指挥官养成背包类型
        
        local enough = true
        local item_tab = {}
        for k,v in pairs(cfg.stuff) do
            local num = bag:getItemNum(k)
            if num < v then 
                -- 不满足条件
                enough = false
            end
            item_tab[k] = {num, v}  -- item_id - {curr_num, req_num}
        end

        return enough, item_tab
    end,

    -- 计算指挥官属性
    getProps = function(self, level, quality, awake_level, star_level)
        local promote_cfg = self.promotion_cfg[quality]
        local awake_cfg = self:getAwakeCfg(star_level, awake_level)
        local atk = self.cfg.ATK_INC*level + (promote_cfg.attr[17] or 0) + (awake_cfg.attr[17] or 0)
        local def = self.cfg.DEF_INC*level + (promote_cfg.attr[18] or 0) + (awake_cfg.attr[18] or 0)
        local max_hp = self.cfg.MAX_HP_INC*level + (promote_cfg.attr[19] or 0) + (awake_cfg.attr[19] or 0)
        local props = {}
        table.insert( props, {PropId = 17, PropValue = math.floor(atk)} )
        table.insert( props, {PropId = 18, PropValue = math.floor(def)} )
        table.insert( props, {PropId = 19, PropValue = math.floor(max_hp)} )

        return props
    end,

    getStarUpPiece = function(self)
        local cfg = config.commander_star_ups[self.star_level]

        if cfg then
            return cfg.NeedChip
        end

        return 0
    end,

    getLevelStrength = function(self, level)
        local cfg = config.commander_level_ups[level]
        local val = 0

        if cfg then
            if self.cfg.color == 3 then
                val = cfg.Quality3Power
            elseif self.cfg.color == 4 then
                val = cfg.Quality4Power
            elseif self.cfg.color == 5 then
                val = cfg.Quality5Power
            end
        end

        return val
    end,

    getLevelUpExp = function(self, level)
        local cfg = config.commander_level_ups[level]
        local val = 0

        if cfg then
            val = cfg.NeedExp
        end

        return val
    end,

    getStarUpPieceCnt = function(self, star_lv)
        local cfg = config.commander_star_ups[star_lv]
        local val = 0

        if cfg then
            if self.cfg.color == 3 then
                val = cfg.NeedChip
            elseif self.cfg.color == 4 then
                val = cfg.NeedChip
            elseif self.cfg.color == 5 then
                val = cfg.NeedChip
            end
        end

        return val
    end,

    isMaxStar = function(self)
        local next_lv = self.star_level + 1

        return config.commander_star_ups[next_lv] == nil
    end,

    -- 是否可升星
    canStarUp = function(self)
        local curr_piece = frame_helper:getItemNum(self.cfg.piece_id)
        local need_piece = self:getStarUpPieceCnt(self.star_level)

        return curr_piece >= need_piece and not self:isMaxStar()
    end,

    -- 是否可招募
    canRecruit = function(self)
        local curr_piece = frame_helper:getItemNum(self.cfg.piece_id)
        local need_piece = self.cfg.recruitneed
        return curr_piece >= need_piece
    end,

    levelMax = function(self)
        local next_lv = self.level + 1
        local lv_max = config.commander_level_ups[next_lv] == nil

        return lv_max
    end,
}