local log = logger.get("Game", "drill_ground_helper")
local rts_soldier = require('RTS.rts_soldier')

-- 路径
local path_config = 
{
    [1] = {Vector3(14.8,0,-31.83), Vector3(11.17,0,-31.83), Vector3(11.17,0,-13.05),Vector3(20.53,0,-13.35)},    --顶视图左上兵营
    [2] = {Vector3(15.14,0,-42.81), Vector3(10.33,0,-42.82),Vector3(10.33,0,-13.37),Vector3(20.34,0,-13.37)},             --顶视图右上兵营
    [3] = {Vector3(6.02,0,-30.71), Vector3(-0.2,0,-30.71),Vector3(-0.2,0,-13.09),Vector3(19.99,0,-13.09)},             --顶视图左下兵营
    [4] = {Vector3(3.7,0,-40.68), Vector3(-0.01,0,-40.68),Vector3(-0.01,0,-12.73),Vector3(20.13,0,-12.73)},             --顶视图右下兵营
    [5] = {Vector3(10.52,3.008,2.76),Vector3(10.52,0.06,-7.62),Vector3(10.52,0.06,-12.56),Vector3(19.81,0.06,-12.56)},           --医院
}

-- soldier type 作为 keyVector3(10.52,3.008,-1.29)
local square_config = 
{
    [1] = {dist = 1, dim = 3},   -- dist - 间距  dim - 方阵尺寸 
    [2] = {dist = 1, dim = 3},   -- 
    [3] = {dist = 1, dim = 3},   -- 
    [4] = {dist = 1, dim = 2},   -- 
}

-- 方阵原点
local square_orig_config =
{
    [1] = Vector3(23.65, 0, -18.26),
    [2] = Vector3(19.96, 0, -18.14),
    [3] = Vector3(23.65, 0, -11.92),
    [4] = Vector3(19.89, 0, -11.9),
    [5] = Vector3(23.81, 0, -5.3),
    [6] = Vector3(19.86, 0, -5.38),
    [7] = Vector3(23.81, 0, 1.01),
    [8] = Vector3(19.86, 0, 0.93),
    [9] = Vector3(23.81, 0, 7.03),
    [10] = Vector3(19.86, 0, 6.95),
    [11] = Vector3(23.81, 0, 12.79),
    [12] = Vector3(19.86, 0, 12.71),
    [13] = Vector3(23.81, 0, 18.72),
    [14] = Vector3(19.86, 0, 18.64),

}

local STANDARD_SQUARE_SOLDIER_NUM = 1000   -- 每个方阵代表的士兵数
local SOLDIER_SCALE = 0.4                   -- 缩放
local DEFAULT_ANIM = "E_idle"               -- 默认动画名称

-- 向校场移动的士兵
local drill_moving_soldier = class{
    ctor = function(self, cfg_id, path, speed, call_back)
        self.is_moving = false
        self.is_finished = false
        self.path = path
        self.speed = speed or 10
        self.sub_path = 1 -- 当前的路径段
        self.sub_path_time = 1 -- 当前子路径总时间
        self.elapsed = 0.0 -- 子路径移动经过时间
        self.cfg = config.tank_basic_ids[cfg_id]      
        self.call_back = call_back
        if not self.cfg then
            log.error("Failed to find soldier cfg:"..tostring(cfg_id))
        end  
        self:init()
    end,

    init = function(self)
        local callback = function(go)
            -- callback
            if go ~= nil then
                self.gameObject = go
                self.transform = go.transform
                self.cs_inst = go:GetComponent("Soldier")
                local billboard = 
                    self.cs_inst.m_SpriteAnimator.gameObject:GetComponent(Fenix.Unity.Billboard)
                billboard.m_bOnce = false

                self:beginSubPath(1)
            else

            end
        end

        local prefab_path = self.cfg.prefab1
        local uobject = soldier_pool:instantiate(prefab_path)
        if not uobject then
            log.error('Failed to load prefab:'..prefab_path)
            return nil
        end

        if uobject.isLoaded then 
            callback(uobject)
        else 
            uobject.onLoaded = callback
        end
    end,

    -- 开始子路径段（直线）
    beginSubPath = function(self, sub_path)
        self.sub_path = sub_path
        --log.info("### beginSubPath:"..sub_path)

        if sub_path < #self.path then
            self.transform.position = self.path[sub_path]
            self.sub_from = self.path[sub_path]
            self.sub_to = self.path[sub_path+1]
            local to = self.sub_to- self.sub_from
            self.transform.rotation = Quaternion.LookRotation(to, Vector3.up)
            self.transform.localScale = Vector3(SOLDIER_SCALE, SOLDIER_SCALE, SOLDIER_SCALE)
            self.cs_inst:PlayAnimation(rts_soldier.getAnimPostfix(self).."run")

            self.sub_path_time = to.magnitude / self.speed
            self.elapsed = 0.0
            self.is_moving = true

            --log.info("time:"..self.sub_path_time..", from:"..tostring(self.sub_from)..", to:"..tostring(self.sub_to))
        elseif sub_path == #self.path then
            self.transform.position = self.path[sub_path]
            self.is_finished = true
            self.is_moving = false
            if self.call_back then
                self.call_back()
            end
        end
    end,

    update = function(self, dt)
        if not self.is_moving then
            return
        end

        self.elapsed = self.elapsed + dt
        local percent = self.elapsed / self.sub_path_time
        if percent >= 1.0 then
            self:beginSubPath(self.sub_path + 1)
        else
            self.transform.position = Vector3.Lerp(self.sub_from, self.sub_to, percent)            
        end
    end,

    destroy = function(self)
        if self.gameObject then
            self.gameObject:destroy()
            self.gameObject = nil
        end
    end,
}

-- 校场的一个方阵
local drill_square_soldier = class{
    -- soldier_type - 士兵类型 [1,4]
    -- orig_pos - 方阵原点（第一个士兵位置）
    -- standard_soldier_num - 方阵表示的标准士兵数量，默认 10000
    -- soldier_nums - 这个方阵实际的士兵数量
    ctor = function(self, idx, soldier_type, orig_pos, standard_soldier_num, soldier_nums)
        self.index = idx
        self:refresh(soldier_type, orig_pos, standard_soldier_num, soldier_nums)
    end,

    refresh = function(self, soldier_type, orig_pos, standard_soldier_num, soldier_nums)
        if self.soldier_type ~= soldier_type then
            self:destroy()
            self.soldiers = {}
        end

        self.soldier_type = soldier_type        
        self.square_cfg = square_config[soldier_type]
        self.orig_pos = orig_pos
        local max_num = self.square_cfg.dim * self.square_cfg.dim
        self.soldier_nums = -- 显示的士兵数量
            math.ceil(soldier_nums / standard_soldier_num * max_num)

        --log.info("### "..self.index.. ", type:"..soldier_type..",nums:"..soldier_nums..", max display nums:"..max_num..", display num:"..self.soldier_nums)
        --log.info("### orig:"..tostring(orig_pos))
        local soldier_cfg_id = battle_replay_helper:getSoldierCfgIdByType(soldier_type)
        self.soldier_cfg = config.tank_basic_ids[soldier_cfg_id]  

        -- 出生所有士兵
        local dist = self.square_cfg.dist
        local idx = 1
        for x=1, self.square_cfg.dim do -- 行
            for z=1, self.square_cfg.dim do -- 列
                local one_soldier = self.soldiers[idx]
                if not one_soldier then
                    local pos = orig_pos + Vector3((x-1)*dist, 0, (z-1)*dist)
                    one_soldier = self:spawnOneSoldier(pos)
                    if one_soldier then
                        self.soldiers[idx] = one_soldier
                    end
                    --log.info("### spawn soldier:"..idx)
                else
                    -- soldier already exist
                end
                if idx >= self.soldier_nums then
                    break
                end
                idx = idx + 1
            end
            if idx >= self.soldier_nums then
                break
            end
        end

        for i = idx+1, max_num do
            local soldier = self.soldiers[i]
            if soldier then
                soldier:destroy()
                self.soldiers[i] = false
            end
        end
    end,

    spawnOneSoldier = function(self, pos)
        local prefab_path = self.soldier_cfg.prefab1   
        local callback = function(go)
            -- callback
            if go ~= nil then
                self.gameObject = go
                self.transform = go.transform
                self.transform.localScale = Vector3(SOLDIER_SCALE, SOLDIER_SCALE, SOLDIER_SCALE)
                self.transform.position = pos
                self.cs_inst = go:GetComponent("Soldier")
                self.cs_inst:PlayAnimation(DEFAULT_ANIM)
                --log.info("## load prefab:"..prefab_path)
            else
                log.error("Failed to load soldier:"..prefab_path)
            end
        end

        local uobject = soldier_pool:instantiate(prefab_path)
        if not uobject then
            log.error('Failed to load prefab:'..prefab_path)
            return nil
        end

        if uobject.isLoaded then 
            callback(uobject)
        else 
            uobject.onLoaded = callback
        end
        return uobject
    end,

    -- 销毁所有士兵
    destroy = function(self)
        if self.soldiers then
            for idx, soldier in ipairs(self.soldiers) do
                if soldier then
                    soldier:destroy()
                end
            end
            self.soldiers = nil
        end
    end,
}

-- 校场辅助函数
drill_ground_helper =
{
    init = function(self)
        self.moving_soldiers = {}  -- 向校场移动的士兵
        self.soldier_squares = {}  -- 校场上站立的士兵方阵

        if battle_helper.is_soldier_preloaded then
            drill_ground_helper:refreshSquareSoldiers()
            self.is_square_soldier_inited = true
        end
    end,

    -- 创建一个士兵沿路径移动
    createPathMovingSoldier = function(self, soldier_type, path_id, speed, num, interval)
        local cfg_id = battle_replay_helper:getSoldierCfgIdByType(soldier_type)
        local path = path_config[path_id]
        app:startCoroutine(
            function()
                for i=1,num do
                    local call_back = nil
                    if i == num then
                        call_back = function()
                            self:refreshSquareSoldiers()
                        end
                    end
                    local soldier = drill_moving_soldier(cfg_id, path, speed, call_back)
                    table.insert(self.moving_soldiers, soldier)
                    coroutine.yield(WaitForSeconds(interval))
                end                
            end
        )
    end,

    -- 刷新广场上的士兵
    refreshSquareSoldiers = function(self)
        if not battle_helper.is_soldier_preloaded then
            return
        end

        local max_square_num = #square_orig_config -- 最大方阵数

        -- 按照类型统计士兵数量
        local all_soldiers = home_info:getAvailableSoldiers()
        local soldier_nums = {0,0,0,0}   
        local total = 0 -- 士兵总数量
        for _,s in pairs(all_soldiers) do
            local cfg = config.soldier_soldiers[s.soldier_id]
            local soldier_type = cfg.type
            soldier_nums[soldier_type] = soldier_nums[soldier_type] + s.available_num
            total = total + s.available_num
        end

        local standard_square_soldier_num = STANDARD_SQUARE_SOLDIER_NUM -- 每个方阵代表的士兵数量，默认是10000. 当士兵总数过大，会超过 10000
        if total > max_square_num * standard_square_soldier_num then
            standard_square_soldier_num = total / max_square_num        -- 
        end

        --log.info("### standard_square_soldier_num:"..standard_square_soldier_num)
        -- 计算每个类型的方阵数量
        local square_nums = {0,0,0,0}
        local total_square_num = 0  -- 方阵总数
        for i=1,4 do
            square_nums[i] = math.ceil(soldier_nums[i] / standard_square_soldier_num)
            total_square_num = total_square_num + square_nums[i]
        end

        -- 当方阵总数超过最大方阵数，从最大数量中减去
        while total_square_num > max_square_num do
            local max_idx = 0
            local max_value = 0
            for i=1,4 do
                if square_nums[i] > max_value then
                    max_idx = i
                    max_value = square_nums[i]
                end
            end
            if max_idx > 0 then
               -- log.info("### reduce square num:"..max_idx)
                square_nums[max_idx] = square_nums[max_idx] - 1
                total_square_num = total_square_num - 1
            end
        end

        -- 刷新 soldier square
        local square_idx = 1
        for i=1,4 do    -- 遍历 4 中类型士兵
            local square_num = square_nums[i]   -- 方阵数量
            local left_num = soldier_nums[i]    -- 实际士兵数量

            for j=1,square_num do   -- 遍历一个类型士兵的方阵
                if left_num <= 0 then
                    log.error("### left_num error:"..left_num)
                    break
                end

                local orig = square_orig_config[square_idx]    -- 原点
                local square_soldier = self.soldier_squares[square_idx]     -- 方阵实例
                local square_soldier_num = math.min(standard_square_soldier_num, left_num)  -- 方阵的士兵数量
                if not square_soldier then
                    square_soldier = drill_square_soldier(square_idx, i, orig, standard_square_soldier_num, square_soldier_num)
                    self.soldier_squares[square_idx] = square_soldier
                else
                    square_soldier:refresh(i, orig, standard_square_soldier_num, square_soldier_num)
                end

                left_num = left_num - standard_square_soldier_num
                square_idx = square_idx + 1
            end
        end

        -- 清空剩余 square

    end,

    update = function(self, dt)
        if not self.is_square_soldier_inited then
            if battle_helper.is_soldier_preloaded then
                drill_ground_helper:refreshSquareSoldiers()
                self.is_square_soldier_inited = true
            end
        end

        local to_destroy_idx = nil
        for idx, s in ipairs(self.moving_soldiers) do
            s:update(dt)
            if not to_destroy_idx and s.is_finished then
                to_destroy_idx = idx
            end
        end

        if to_destroy_idx then
            self.moving_soldiers[to_destroy_idx]:destroy()
            table.remove(self.moving_soldiers, to_destroy_idx)
        end
    end,

    destroy = function(self)
        for _,s in ipairs(self.moving_soldiers) do
            s:destroy()
        end
        self.moving_soldiers = {}

        for _,s in ipairs(self.soldier_squares) do
            s:destroy()
        end
        self.soldier_squares = {}
    end,
}


return drill_ground_helper