local super = require("RTS.home_sys.home_entity_grid")

return class
{
    super = super,

    ctor = function(self, owner, grid_id, id, cfg_id, level, root, events)
        super.ctor(self, owner, grid_id, events, root)

        self.id = id
        self.cfg_id = cfg_id
        self.level = level
        
        self.basic_config = config.home_config_lists[cfg_id]
        self.detail_config, self.max_level = self:getDetailCfg(cfg_id, level)

        self:createModel()
        self:checkUpgrading()
    end,

    hasBuild = function(self)
        return true
    end,

    createModel = function(self)
        local model_path = self:getModelPath()
        if model_path == self.model_path then
            return
        else
            self.model_path = model_path
        end
        self.model_path = model_path
        local callback = function (model_go)

            if model_go then
                model_go.transform.parent = self.transform
                frame_helper:resetTransform(model_go.transform)
                -- 销毁旧的模型
                if self.model then
                    self.model:destroy()
                end
                self.model = model_go
                self.uobject = nil
                local bc = model_go:GetComponentInChildren(UnityEngine.Collider)
                if bc then
                    self.mono = bc.gameObject:GetComponent(HomeEntity)
                    if not self.mono then
                        self.mono = bc.gameObject:AddComponent(HomeEntity)
                    end
                    self.mono.id = self.grid_id
                    self.mono.is_group = false
                    frame_helper:setChildLayer(self.model.transform, "Home")
                end
                -- 倒计时建造
                if not self.inited then
                    self.inited = true
                    self.events:dispatch("BUILD_LOADED", self)
                end
            else
                log.error("加载模型失败:", model_path)
            end
        end
        local old_uobject = self.uobject
        if old_uobject then
            old_uobject:destroy()
        end
        local new_uobject = game_director.pool:instantiate(model_path)
        if new_uobject.isLoaded then 
            app:startCoroutine(function()
                coroutine.yield(nil)
                callback(new_uobject)
            end)
        else 
            new_uobject.onLoaded = callback
        end
        self.uobject = new_uobject
    end,

    interactEntity = function(self)
        print("interactEntity:", "建筑")
    end,
    
    -- 获取模型的路径
    getModelPath = function(self)
        if self.detail_config then
            return self.detail_config.prefab_path .. self.detail_config.prefab_id
        else
            local path = nil
            config.home_config_lists:foreach(
                function(k, v)
                    if v.id == self.cfg_id then
                        path = v.level_0_model
                    end
                end)
            return path
        end
    end,

    refresh = function(self)
        local build_data = home_info:getBuildDataByGridId(self.grid_id)
        local is_level_up = build_data.level > self.level
        if is_level_up then
            self.level = build_data.level
            self.detail_config, self.max_level = self:getDetailCfg(build_data.cfgId, build_data.level)
            self:createModel()
            self:playUpgradeFx()
        end
        self:checkUpgrading()
    end,

    playUpgradeFx = function(self)
        if self.fx_object then
            self.fx_object:destroy()
        end
        local model_path = self.basic_config.upgrade_finish_effect
        local callback = function (go)
            if go ~= nil then
                go.transform.parent = self.transform
                go.transform.localPosition = Vector3.zero
                go.transform.localScale = Vector3.one
                go.transform.localRotation = Quaternion.identity
            end
        end
        local uobject = game_director.pool:instantiate(model_path)
        if not uobject then
            log.error("Failed to load :"..tostring(model_path))
        else
            if uobject.isLoaded then 
                callback(uobject)
            else 
                uobject.onLoaded = callback
            end
        end
        self.fx_object = uobject
    end,

    -- 是否升级
    canUpgrade = function(self)
        if self.basic_config.can_upgrade == 1 and self.level < self.max_level then
            local cur_upgrading = home_info:getUpgradingEntity()
            if cur_upgrading and cur_upgrading.id == self.id then
                return false
            end
            return true
        end
    end,

    -- 是否加速
    canSpeedUp = function(self)
        local cur_upgrading = home_info:getUpgradingEntity()
        if cur_upgrading and cur_upgrading.id == self.id then
            return true
        end
        return false
    end,


    -- 获取配置文件
    getDetailCfg = function(self, cfg_id, level)
        local rst = nil
        local max_level = 1
        config.home_config_upgrades:foreach(
            function(k, v)
                if v.building_id == cfg_id then
                    if v.level == level then rst = v end
                    if v.level > max_level then max_level = v.level end
                end
            end)
        return rst, max_level
    end,

    -- -- 资源是否足够用来升级
    isFactorEnoughForUpgrade = function(self)
        self.next_detail_cfg = self:getDetailCfg(self.cfg_id, self.level + 1)
        local dt = home_info:getBuildDataByGridId(self.id)
        if dt and dt.tLevelUp > 0 then
            return false, NativeText.GetLangStr(120012)
        end
        local cost_args = frame_helper:getJsonTab(self.next_detail_cfg.cost)
        for item_id, need_num in pairs(cost_args) do
            local owner_num = frame_helper:getItemNum(tonumber(item_id))
            if need_num > owner_num then
                print("upgrade cost item not enough!!!", item_id)
                return false, NativeText.GetLangStr(120013)
            end
        end
        if local_player:getLevel() < self.next_detail_cfg.player_level then
            return false, NativeText.GetLangStr(120014)
        end
        if self.cfg_id ~= 1 then
            if home_info:getMainEntityLv() < self.next_detail_cfg.base_level then
                return false, NativeText.GetLangStr(120015)
            end
        end
        return true, nil
    end,

    -- 获取功能栏
    getOperators = function(self)
        -- 等级是0级的话没有任何功能
        if self.level <= 0 then
            return {}
        end

        local rst = {}
        local operate_args = frame_helper:getJsonTab(self.basic_config.operate)
        for k, tmp in pairs(operate_args) do
            local id = tonumber(tmp)
            if id == 1 then
                table.insert(rst, id)
            elseif id == 2 then -- 判断是否能升级
                if self:canUpgrade() and self.level > 0 then table.insert(rst, id) end
                -- table.insert(rst, id)
            elseif id == 9 then -- 是否能加速
                if self:canSpeedUp() then table.insert(rst, id) end
            elseif id == 17 then
                if self.level == 0 and not self:canSpeedUp() then 
                    table.insert(rst, id) 
                end
            else
                if self.level > 0 then
                    table.insert(rst, id)
                end
            end
        end
        return rst
    end,

    checkUpgrading = function(self)
        local build_data = home_info:getBuildDataByGridId(self.grid_id)
        if build_data.tLevelUp > 0 then
            if not self.upgrading_trans then
                local newObj = GameObject()
                newObj.name = "upgrading"
                newObj.transform.parent = self.transform
                newObj.transform.localPosition = Vector3.zero
                newObj.transform.localScale = Vector3.one
                newObj.transform.localRotation = Quaternion.identity
                self.upgrading_trans = newObj.transform

                local next_config = self:getDetailCfg(self.cfg_id, self.level + 1)
                local model_path = next_config.prefab_path .. next_config.prefab_building
                -- if self.grid_id == 4 then
                --     log.error("加载完成！！！！", self.grid_id, model_path)
                -- end

                local callback = function (go)
                    if go ~= nil then
                        if self.upgrading_trans then
                            go.transform.parent = self.upgrading_trans
                            go.transform.localPosition = Vector3.zero
                            go.transform.localScale = Vector3.one
                            go.transform.localRotation = Quaternion.identity
                        else
                            go:destroy()
                        end
                    end
                end
                local uobject = game_director.pool:instantiate(model_path)
                if uobject.isLoaded then 
                    callback(uobject)
                else 
                    uobject.onLoaded = callback
                end
            end
        else
            if self.upgrading_trans then
                GameObject.Destroy(self.upgrading_trans.gameObject)
                self.upgrading_trans = nil
            end
        end
    end,

    exit = function(self)
        self.events:dispatch("BUILD_UN_LOADED", self)
        super.exit(self)
    end,
}