local super = require("RTS.home_sys.home_entity_grid")

return class{
    super = super,

    ctor = function(self, owner, grid_id, root, events)
        super.ctor(self, owner, grid_id, events, root)
        --modified by hong feng, 2018/08/21
        self.rot = self:getRot(grid_id)
        self.scale = self:getScale(grid_id)
        --modification end
        self:createModel()
    end,

    hasBuild = function(self)
        return false
    end,

    createModel = function(self)
        local model_path = self:getModelPath()
        local callback = function (model_go)
            if model_go then
                if not model_go.transform then
                    return
                end

                model_go.transform.parent = self.transform
                frame_helper:resetTransform(model_go.transform)
                self.model = model_go
                local bc = model_go:GetComponentInChildren(UnityEngine.Collider)
                if bc then
                    self.mono = bc.gameObject:GetComponent(HomeEntity)
                    if not self.mono then
                        self.mono = bc.gameObject:AddComponent(HomeEntity)
                    end
                    self.mono.id = self.grid_id
                    self.mono.is_group = false
                    frame_helper:setChildLayer(self.model.transform, "Home")
                end
                --modified by hong feng, 2018/08/21
                model_go.transform.localScale = Vector3(self.scale.x, self.scale.y, self.scale.z)
                model_go.transform.localRotation = Quaternion.Euler(0, self.rot.r, 0)
                --modification end
            else
                log.error("加载模型失败:", model_path)
            end

            -- 倒计时建造
            if not self.inited then
                self.inited = true
                self.events:dispatch("EMPTY_LOADED", self)
            end
        end
        local new_uobject = game_director.pool:instantiate(model_path)

        if new_uobject.isLoaded then 
            app:startCoroutine(function()
                coroutine.yield(nil)
                callback(new_uobject)
            end)
        else 
            new_uobject.onLoaded = callback
        end
    end,

    destroyBuild = function(self)
        if self.model then
            self.model:destroy()
            self.model = nil
        end
    end,

    getModelPath = function(self)
        local cfg = config.home_config_grids[self.grid_id]
        return cfg.grid_model
    end,

    --modified by hong feng, 2018/08/21
    getScale = function(self, grid_id)
        local cfg = config.home_config_grids[grid_id]
        return { x = cfg.grid_scale_x, y = cfg.grid_scale_y, z = cfg.grid_scale_z}
    end,

    getRot = function(self, grid_id)
        local cfg = config.home_config_grids[grid_id]
        return {r = cfg.grid_rot}
    end,
    --modification end

    interactEntity = function(self)
        print("interactEntity:", "空地")
    end,
}

