log = logger.get("Game", "home_info")

home_info = {
    events = require("base.event").new(),
    -- 地图信息
    builds = nil,
    -- 主基地id
    main_entity_cfg_id = 1,
    -- 研究所id
    research_entity_cfg_id = 2,

    building_free_time = 0,

    groups = {},

    grids = {},

    builds = {},

    setBuildFreeTime = function(self, time)
        local is_change = self.building_free_time ~= time
        self.building_free_time = time
        if is_change then
            self.events:dispatch("free_time_change")
        end
        log.info("升级免费时间:", self.building_free_time)
    end,
    
    setBuild = function(self, grid_id, entity)
        self.builds = self.builds or {}
        self.builds[grid_id] = self.builds[grid_id] or {}
        self.builds[grid_id].eObjType       = entity.eObjType
        self.builds[grid_id].instId         = entity.instId
        self.builds[grid_id].id             = entity.instId
        self.builds[grid_id].cfgId          = entity.cfgId 
        self.builds[grid_id].level          = entity.level

        local pos_cfg = config.home_config_grids[entity.gridId]
        self.builds[grid_id].x_tips         = pos_cfg.x_tips
        self.builds[grid_id].y_tips         = pos_cfg.y_tips
        self.builds[grid_id].z_tips         = pos_cfg.z_tips
        
        self.builds[grid_id].level = entity.level  
        self.builds[grid_id].tLevelUp = entity.tLevelUp
        self.builds[grid_id].totaTime = entity.totaTime
        self.builds[grid_id].totalTime = entity.totaTime
        self.builds[grid_id].canHelp = entity.canHelp
        self.builds[grid_id].rmbTime = Time.realtimeSinceStartup -- 记下收到消息的时间点
        self.builds[grid_id].gridId = entity.gridId
        self.builds[grid_id].grid_id = entity.gridId

        self.builds[grid_id].barrack = entity.barrack
        self.builds[grid_id].resource = entity.resource
        self.builds[grid_id].studio = entity.studio
        self.builds[grid_id].parade = entity.parade
        self.builds[grid_id].hospital = entity.hospital
        self.builds[grid_id].guardsystem = entity.guardsystem
        self.builds[grid_id].trainCenter = entity.trainCenter
        self.builds[grid_id].armory = entity.armory
        self.builds[grid_id].vault = entity.vault
        self.builds[grid_id].tavern = entity.tavern


        self.builds[grid_id].free_upgrade = entity.tLevelUp <= self.building_free_time

        log.info("网格Id:", grid_id, "建筑配置id", entity.cfgId, "等级:", entity.level)
    end,

    removeBuild = function(self, id)
        for grid_id, build in pairs(self.builds) do
            if build.id == id then
                self.builds[grid_id] = nil
                return
            end
        end
    end,

    setGroup = function(self, group_id, lock)
        self.groups = self.groups or {}
        self.groups[group_id] = lock
        self:setGrid(group_id, lock)
    end,

    setGrid = function(self, group_id, lock)
        self.grids = self.grids or {}
        local cfg = config.home_config_grid_groups[group_id]
        local tab = frame_helper:getJsonTab(cfg.member)
        for __, grid_id in ipairs(tab) do
            self.grids[grid_id] = lock
        end
    end,

    dispatchBuildInfos = function(self, id)
        self.events:dispatch("BUILD_INFO_CHANGE", id)
    end,

    dispatchGroupInfos = function(self)
        self.events:dispatch("GROUP_INFO_CHANGE")
    end,

    dispatchGridInfos = function(self)
        self.events:dispatch("GRID_INFO_CHANGE")
    end,

    isGridHasBuild = function(self, grid_id)
        if self.builds[grid_id] then
            return true
        else
            return false
        end
    end,

    isGridLock = function(self, grid_id)
        return self.grids[grid_id]
    end,

    isGroupLock = function(self, group_id)
        return self.groups[group_id]
    end,

    getUpgradingEntity = function(self)
        for id, build in pairs(self.builds) do
            if build.tLevelUp > 0 then
                return build
            end
        end
    end,

    getEntityByObjType = function(self, eObjType)
        local level = 0
        local rst = nil
        for id, entity in pairs(self.builds) do
            if entity.eObjType == eObjType then
                if entity.level >= level then
                    level = entity.level
                    rst = entity
                end
            end
        end
        return rst
    end,

    getAllBuilds = function(self)
        return self.builds
    end,

    getBuildDataByCfgId = function(self, cfg_id)
        for id, entity in pairs(self.builds) do
            if entity.cfgId == cfg_id then
                return entity
            end
        end
    end,

    getBuildDataByGridId = function(self, grid_id)
        return self.builds[grid_id]
    end,

    getBuildDataById = function(self, id)
        for grid_id, build in pairs(self.builds) do
            if build.id == id then
                return build
            end
        end
    end,

    getBuildNumByCfgId = function(self, cfg_id)
        local num = 0
        for grid_id, build in pairs(self.builds) do
            if build.cfgId == cfg_id then
                num = num + 1
            end
        end
        return num
    end,

    getBuildMaxNumByCfgId = function(self, cfg_id)
        return config.home_config_lists[cfg_id].build_num
    end,

    -- 计算升级的剩余时间
    getBuildUpgradeLeftTime = function(self, id)
        local entity = self:getBuildDataById(id)
        if entity and entity.rmbTime and entity.tLevelUp > 0 then
            local pass_time = Time.realtimeSinceStartup - entity.rmbTime
            local real_left_time = entity.tLevelUp - pass_time
            return real_left_time >= 0 and real_left_time or 0, true
        else
            -- log.error('can not find entity id:', id)
            return 0, false
        end
    end,

    isBuildInFreeTime = function(self, id)
        local build_data = self:getBuildDataById(id)
        if build_data.tLevelUp > 0 then
            local left_time = self:getBuildUpgradeLeftTime(id)
            return left_time <= self.building_free_time 
        end
    end,

    -- 计算升级需要的宝石
    getUpgradeCostGem = function(self, id, time)
        return gui_helper:getSpeedUpCost(43, time)
    end,

    clearEntitys = function(self)
        self.groups = {}
        self.grids = {}
        self.builds = {}
    end,

    -- 建筑的数量
    getEntityNum = function(self, cfg_id)
        local num = 0
        for k, v in pairs(self.builds) do
            if v.cfgId == cfg_id then
                num = num + 1
            end
        end
        return num
    end,

    -- 获得主基地等级
    getMainEntityLv = function(self)
        local build_data = self:getBuildDataByCfgId(1)
        if build_data then
            return build_data.level
        end
        return 0
    end,

    -- 获得研究所等级
    getResearchEntityLv = function(self)
        local dt = nil
        for k, v in pairs(self.builds) do
            if v.cfgId == self.research_entity_cfg_id then
                dt = v
                break
            end
        end
        -- 不存在
        if not dt then
            return 0
        end
        
        return dt.level
    end,

    -- 获得科技等级
    getTechLevel = function(self, tech_cfg_id)
        local cfg_id = 2
        local build_data = self:getBuildDataByCfgId(2)
        if build_data then
            local level = 0
            for __, t in ipairs(build_data.studio.technologys) do
                if t.cfgId == tech_cfg_id then
                    return t.level, true
                end
            end
            return level, true
        else
            -- log.error("研究所尚未建造")
            return 0, false
        end
    end,

    -- 是否有建筑升级帮助
    isUpgradeCanHelp = function(self, entity_data)
        --[[
        if not local_guild:isSelfInGuild() then
            return false
        end
        return entity_data.tLevelUp > 0 and entity_data.canHelp
        ]]
        return entity_data.canHelp
    end,

    -- 获得建筑工人的数量
    getBuilderNum = function(self)
        return 1
    end,

    -- 获得正在升级的建筑
    getAllUpgradingEntity = function(self)
        local rst = {}
        if self.builds then
            for id, build_data in pairs(self.builds) do
                if build_data.tLevelUp > 0 then
                    table.insert(rst, build_data)
                end
            end
        end
        return rst
    end,

    -- 出征士兵数量
    getOutSoldierNum = function(self, soldier_id)
        local build_data = self:getBuildDataByCfgId(10)
        if build_data then
            for __, s in ipairs(build_data.parade.soldiers) do
                if s.cfgId == soldier_id then
                    return s.bInArmy
                end
            end
        end
    end,

    -- 闲置士兵数量
    getAvailableSoldierNum = function(self, soldier_id)
        local build_data = self:getBuildDataByCfgId(10)
        if build_data then
            for __, s in ipairs(build_data.parade.soldiers) do
                if s.cfgId == soldier_id then
                    return s.totalNum - s.bInArmy - s.bHurt - s.bTreat
                end
            end
        end

        local build_data = self:getBuildDataByCfgId(24)
        if build_data then
            for __, s in ipairs(build_data.guardsystem.traps) do
                if s.cfgId == soldier_id then
                    return s.totalNum - s.bInArmy - s.bHurt - s.bTreat
                end
            end
        end

        return 0
    end,

    getAvailableSoldiers = function(self)
        local rst = {}
        local build_data = self:getBuildDataByCfgId(10)
        if build_data then
            for __, s in ipairs(build_data.parade.soldiers) do
                local available_num = s.totalNum - s.bInArmy - s.bHurt - s.bTreat
                if available_num > 0 then
                    table.insert(rst,{
                        soldier_id = s.cfgId,
                        available_num = available_num,
                    })
                end
            end
        end
        return rst
    end,

    -- 受伤士兵数量
    getHurtedSoldierNum = function(self, soldier_id)
        local build_data = self:getBuildDataByCfgId(10)
        if build_data then
            for __, s in ipairs(build_data.parade.soldiers) do
                if s.cfgId == soldier_id then
                    return s.bHurt
                end
            end
        end
        return 0
    end,

    -- 受伤士兵全部数量
    getAllHurtedSoldierNum = function(self)
        local total_num = 0
        local build_data = self:getBuildDataByCfgId(10)
        if build_data then
            for __, s in ipairs(build_data.parade.soldiers) do
                total_num = total_num + s.bHurt
            end
        end
        return total_num
    end,

    -- 治疗中士兵数量
    getCureSoldierNum = function(self, soldier_id)
        local build_data = self:getBuildDataByCfgId(10)
        if build_data then
            for __, s in ipairs(build_data.parade.soldiers) do
                if s.cfgId == soldier_id then
                    return s.bTreat
                end
            end
        end
        return 0
    end,

    -- 治疗中士兵全部数量
    getAllCureSoldierNum = function(self)
        local total_num = 0
        local build_data = self:getBuildDataByCfgId(10)
        if build_data then
            for __, s in ipairs(build_data.parade.soldiers) do
                total_num = s.bTreat + total_num
            end
        end
        return total_num
    end,

    -- 全部士兵数量
    getAllSoldierNum = function(self, soldier_id)
        local build_data = self:getBuildDataByCfgId(10)
        if build_data then
            for __, s in ipairs(build_data.parade.soldiers) do
                if s.cfgId == soldier_id then
                    return s.totalNum
                end
            end
        end
        return 0
    end,

    getRealRes = function(self, expect_res, rate)
        local real_res = expect_res * rate
        real_res = tostring(real_res)
        real_res = tonumber(real_res)
        if real_res <= 0 then
            return 0
        else
            real_res = math.floor(real_res)
            return real_res == 0 and 1 or real_res
        end
    end,

    -- 获得士兵治疗时间
    getSoldierCureTime = function(self, barrack_cfg, num)
        if barrack_cfg then
            local prop = local_player.globale_props
            local rate = 0
            if barrack_cfg.type < 10 then
                rate = local_player:getGlobalProp(common_pb.CORP0_HEAL_SPEED_ADD_PERCENT)
                rate = rate + local_player:getGlobalProp(common_pb["CORP" .. barrack_cfg.type .. "_HEAL_SPEED_ADD_PERCENT"])
            else
                rate = local_player:getGlobalProp(common_pb.DEFENSE_WEAPON_REPAIR_SPEED_PERCENT)
            end

            local total_time = barrack_cfg.heal_time * num
            local real_time = total_time / ( rate / 10000 + 1 )
            if real_time <= 0 then
                return 0
            else
                real_time = math.floor(real_time)
                return real_time == 0 and 1 or real_time
            end
        else
            log.error("获得士兵治疗时间 Error:", soldier_id, num)
            return 0
        end
    end,

    getSoldierCureCost = function(self, barrack_cfg, num)
        local prop = local_player.globale_props
        local rst = {}
        local rate = 0
        if barrack_cfg.type < 10 then
            rate = local_player:getGlobalProp(common_pb.CORP0_HEAL_COST_REDUCE_PERCENT)
            rate = rate + local_player:getGlobalProp(common_pb["CORP" .. barrack_cfg.type .. "_HEAL_COST_REDUCE_PERCENT"])
            rate = 1 / ( 1 + rate / 10000 )
        else
            rate = local_player:getGlobalProp(common_pb.SYS_DEFENSE_WEAPON_PRODUCE_REPAIR_PERCENT)
            rate = 1 / ( 1 + rate / 10000 )
        end
        local cost_table = loadstring("return " .. barrack_cfg.heal_cost)()
        for item_id, need_num in pairs(cost_table) do
            rst[item_id] = self:getRealRes(need_num * num, rate) 
        end
        return rst
    end,

    -- 在有限的资源内，能治疗多少兵
    getSoldierCureNumInLimitRes = function(self, barrack_cfg, res_tab, max_num)
        if barrack_cfg then
            local prop = local_player.globale_props
            local rate = 0
            if barrack_cfg.type < 10 then
                rate = local_player:getGlobalProp(common_pb.CORP0_HEAL_COST_REDUCE_PERCENT)
                rate = rate + local_player:getGlobalProp(common_pb["CORP" .. barrack_cfg.type .. "_HEAL_COST_REDUCE_PERCENT"])
                rate = 1 / ( 1 + rate / 10000 )
            else
                rate = local_player:getGlobalProp(common_pb.SYS_DEFENSE_WEAPON_PRODUCE_REPAIR_PERCENT)
                rate = 1 / ( 1 + rate / 10000 )
            end

            -- 计算能造兵的最大数量
            local cost_table = loadstring("return " .. barrack_cfg.heal_cost)()
            local num = frame_helper.max_number
            for item_id, need_num in pairs(cost_table) do
                local own_num = res_tab[item_id]
                local single_price = need_num * rate
                local floor_num = math.floor(own_num / single_price)
                local ceil_num = math.ceil(own_num / single_price)
                local real_need = self:getRealRes(ceil_num * need_num, rate)
                local rst = 0
                if own_num >= real_need then
                    rst = ceil_num
                else
                    rst = floor_num
                end
                if rst < num then
                    num = rst
                end
            end
            if num == frame_helper.max_number then
                num = 0
            elseif num > max_num then
                num = max_num
            end
            -- 扣除res_table表里面的资源
            for item_id, need_num in pairs(cost_table) do
                local reduction = self:getRealRes(num * need_num, rate) 
                res_tab[item_id] = res_tab[item_id] - reduction
            end
            return num, res_tab
        else
            log.error("barrack_cfg Is Nil Error:")
            return 0, 0
        end
    end,

    -- 获得医院的容量
    getHospitalCapacity = function(self)
        local capacity = local_player:getGlobalProp(common_pb.HOSTPITAL_CAPACITY) * ( 1 +  local_player:getGlobalProp(common_pb.HOSTPITAL_CAPACITY_PERCENT) / 10000) 
        capacity = math.floor(capacity)
        return capacity
    end,

    -- 获得防御系统容量
    getDefenseCapacity = function(self)
        local prop = local_player.globale_props
        local capacity = local_player:getGlobalProp(common_pb.SYS_MAX_DEFENSE_WEAPON_NUM) * ( 1 +  local_player:getGlobalProp(common_pb.SYS_MAX_DEFENSE_WEAPON_NUM_PERCENT) / 10000) 
        capacity = math.floor(capacity)
        return capacity
    end,

    getSoldierTypeList = function(self)
        return {1,2,3,4}
    end,

    getTrapTypeList = function(self)
        return {11,12,13}
    end,

    getSoliderCfgList = function(self)
        local rst = {}
        for __, type in ipairs(self:getSoldierTypeList()) do
            config.soldier_soldiers:foreach(function(id, cfg)
                if cfg.type == type then
                    table.insert(rst, cfg)
                end
            end)
        end
        return rst
    end,

    getTrapCfgList = function(self)
        local rst = {}
        for __, type in ipairs(self:getTrapTypeList()) do
            config.soldier_soldiers:foreach(function(id, cfg)
                if cfg.type == type then
                    table.insert(rst, cfg)
                end
            end)
        end
        return rst
    end,

    isBuildCanTrainSoldier = function(self, build_data)
        -- 训练中
        if build_data.barrack.bTran then
            return false
        end
        for __, cfg in ipairs(home_info:getSoliderCfgList()) do
            local soldier_id = cfg.id
            local barrack_cfg = home_info:getBarrackCfgBySoldierId(soldier_id)
            if barrack_cfg.building == build_data.cfgId then
                if barrack_cfg.unlock > 0 then
                    -- 有解锁条件
                    local tech_cfg = config.home_config_research_details[barrack_cfg.unlock]
                    local tech_cfg_id = tech_cfg.list_id
                    local tech_level = tech_cfg.level
                    local cur_level, is_unlock = home_info:getTechLevel(tech_cfg_id)
                    if cur_level >= tech_level then
                        return true
                    end
                else 
                    -- 没有解锁条件
                    return true
                end
            end
        end
        return false
    end,

    isTrapCanTrainBySoldierId = function(self, soldier_id)
        local build_data = home_info:getBuildDataByCfgId(24)
        local soldier_cfg = config.soldier_soldiers[soldier_id]
        -- 训练中
        if build_data.guardsystem.build.bTran then
            return false
        end
        -- 尚未解锁
        local barrack_cfg = home_info:getBarrackCfgBySoldierId(soldier_id)
        local tech_cfg = config.home_config_research_details[barrack_cfg.unlock]
        local tech_cfg_id = tech_cfg.list_id
        local tech_level = tech_cfg.level
        local cur_level, is_unlock = home_info:getTechLevel(tech_cfg_id)
        if cur_level < tech_level then
            return false
        end
        -- 是否超出上限
        local prop = local_player.globale_props
        local whole_max_num = math.floor(local_player:getGlobalProp(common_pb.SYS_MAX_DEFENSE_WEAPON_NUM) * (1 + local_player:getGlobalProp(common_pb.SYS_MAX_DEFENSE_WEAPON_NUM_PERCENT) / 10000))
        local temp_num = local_player:getGlobalProp(common_pb.SYS_DEFENSE_WEAPON_PRODUCE_NUM) + local_player:getGlobalProp(common_pb["SYS_CORP" .. soldier_cfg.type .. "_PRODUCE_NUM"])
        local once_max_num = math.floor(temp_num * ( 10000 + local_player:getGlobalProp(common_pb.SYS_DEFENSE_WEAPON_PRODUCE_NUM_ADD_PERCENT)) / 10000)
        local own_num = 0
        for k, v in ipairs(build_data.guardsystem.traps) do
            own_num = own_num + v.totalNum
        end
        -- 剩余空间
        local remain_num = whole_max_num - own_num
        return remain_num > 0
    end,

    getBarrackCfgBySoldierId = function(self, soldier_id)
        local rst = nil
        config.home_config_barracks:foreach(function(id, cfg)
            if cfg.soldier_id == soldier_id then
                rst = cfg
            end
        end)
        return rst
    end,

    -- 获得科技升级时间
    getTechUpgradeTime = function(self, research_detail_cfg)
        if research_detail_cfg then
            local rate = local_player:getGlobalProp(common_pb.SYS_TECH_SPEED_PERCENT)
            local total_time = research_detail_cfg.time
            local real_time = total_time / ( rate / 10000 + 1 )
            if real_time <= 0 then
                return 0
            else
                real_time = math.floor(real_time)
                return real_time == 0 and 1 or real_time
            end
        else
            log.error("获得科技升级时间 Error:")
            return 0
        end
    end,

    -- 获得科技消耗
    getTechUpgradeCost = function(self, research_detail_cfg)
        local rst = {}
        if research_detail_cfg then
            local rate = local_player:getGlobalProp(common_pb.SYS_TECH_COST_REDUCE_PERCENT) 
            local cost_table = loadstring("return " .. research_detail_cfg.cost)()
            for item_id, need_num in pairs(cost_table) do
                local value = need_num * ( 1 - rate / 10000)
                if value <= 0 then
                    value = 0
                else
                    value = math.floor(value)
                    value = (value == 0 and 1 or value)
                end
                table.insert(rst, {
                    item_id = item_id,
                    need_num = value
                })
            end
            return rst
        else
            log.error("获得科技消耗 Error:")
            return rst
        end
    end,

    -- 锻造升级
    getForgeUpgradeTime = function(self, cfg)
        if cfg then
            local rate = local_player:getGlobalProp(common_pb.SYS_FORGE_SPEED_UP_PERCENT)
            local total_time = cfg.TrainTime
            local real_time = total_time / ( rate / 10000 + 1 )
            if real_time <= 0 then
                return 0
            else
                real_time = math.floor(real_time)
                return real_time == 0 and 1 or real_time
            end
        else
            log.error("获得锻造升级时间 Error:")
            return 0
        end
    end,

    -- 兵工厂制造时间
    getArsenalMakeTime = function(self, cfg)
        local temp = nil
        if cfg.weapon_type == 1 then
            temp = common_pb.SYS_MAKE_WEAPON_SPEED_PERCENT
        elseif cfg.weapon_type == 2 then
            temp = common_pb.SYS_MAKE_ARMOR_SPEED_PERCENT
        elseif cfg.weapon_type == 3 then
            temp = common_pb.SYS_MAKE_WHEEL_SPEED_PERCENT
        end
        local rate = local_player:getGlobalProp(temp)
        local total_time = cfg.init_time
        local real_time = total_time / ( rate / 10000 + 1 )
        if real_time <= 0 then
            return 0
        else
            real_time = math.floor(real_time)
            return real_time == 0 and 1 or real_time
        end
    end,

    -- 全部tick
    onTick = function(self)
        local entitys = home_info:getAllBuilds()
        if entitys then
            for grid_id, e in pairs(entitys) do
                -- 建筑升级
                if e.tLevelUp > 0 then
                    local elapsed = e.tLevelUp - Time.realtimeSinceStartup + e.rmbTime
                    elapsed = elapsed >= 0 and math.floor(elapsed) or 0
                    home_info.events:dispatch("BUILD_UPGRADE_TICK", e, elapsed, e.totaTime)


                    if not e.free_upgrade then
                        if elapsed <= self.building_free_time then
                            e.free_upgrade = true
                            self.events:dispatch("BUILD_INFO_CHANGE", e.id)
                        end
                    end
                end
                -- 训练士兵
                if e.barrack.reamintime >= 0 and e.barrack.bTran then
                    local elapsed = e.barrack.reamintime - Time.realtimeSinceStartup + e.rmbTime
                    elapsed = elapsed >= 0 and math.floor(elapsed) or 0
                    home_info.events:dispatch("BARRACK_TRAIN_TICK", e, elapsed, e.barrack.totaltime)
                end
                -- 治疗士兵 : 最大剩余时间的士兵tick
                if #e.hospital.soldiers > 0 then
                    local s = nil
                    for __, soldier in ipairs(e.hospital.soldiers) do
                        if soldier.reamintime > 0 then
                            if not s then
                                s = soldier
                            else
                                if soldier.reamintime > s.reamintime then
                                    s = soldier
                                end
                            end
                        end
                    end
                    if s then
                        local elapsed = s.reamintime - Time.realtimeSinceStartup + e.rmbTime
                        home_info.events:dispatch("HOSPITAL_CURE_TICK", e, elapsed, s.totaltime)
                    end
                end
                -- 训练陷阱
                if e.guardsystem and e.guardsystem.build and e.guardsystem.build.bTran then
                    local elapsed = e.guardsystem.build.reamintime - Time.realtimeSinceStartup + e.rmbTime
                    elapsed = elapsed >= 0 and math.floor(elapsed) or 0
                    home_info.events:dispatch("TRAP_TRAIN_TICK", e, elapsed, e.guardsystem.build.totaltime)
                end 
                -- 治疗陷阱 : 最大剩余时间的陷阱tick
                if e.guardsystem and #e.guardsystem.repair > 0 then
                    local t = nil
                    for __, trap in ipairs(e.guardsystem.repair) do
                        if trap.reamintime > 0 then
                            if not t then
                                t = trap
                            else
                                if trap.reamintime > t.reamintime then
                                    t = trap
                                end
                            end
                        end
                    end
                    if t then
                        local elapsed = t.reamintime - Time.realtimeSinceStartup + e.rmbTime
                        home_info.events:dispatch("TRAP_CURE_TICK", e, elapsed, t.totaltime)
                    end
                end
                -- 城墙修理
                if e.guardsystem then
                    if e.guardsystem.bFire then
                        local total_time = config.home_config_configs[1].fire_duration
                        local elapsed = e.guardsystem.fire_remaintime - Time.realtimeSinceStartup + e.rmbTime
                        home_info.events:dispatch("WALL_CURE_TICK", e, elapsed, total_time)
                    elseif e.guardsystem.repairWallRemainTime > 0 then
                        local total_time = math.floor(e.guardsystem.repairWallRemainTime / ( 1 - e.guardsystem.WallHp / e.guardsystem.WallMaxHp ))
                        local elapsed = e.guardsystem.repairWallRemainTime - Time.realtimeSinceStartup + e.rmbTime
                        home_info.events:dispatch("WALL_CURE_TICK", e, elapsed, total_time)
                    end
                end
                -- 研究所科技研发
                if e.cfgId == 2 and e.studio.tLevelUp > 0 then
                    local elapsed = e.studio.tLevelUp - Time.realtimeSinceStartup + e.rmbTime
                    home_info.events:dispatch("TECH_UPGRADE_TICK", e, elapsed, e.studio.totalTime)
                    -- print("科技升级倒计时:", elapsed, e.studio.totalTime)
                end
                -- 锻造厂
                if e.cfgId == 22 and e.trainCenter.bWork then
                    local elapsed = e.trainCenter.remainTime - Time.realtimeSinceStartup + e.rmbTime
                    home_info.events:dispatch("FORGE_UPGRADE_TICK", e, elapsed, e.trainCenter.totaltime)
                end
                -- 兵工厂
                if e.cfgId == 11 and e.armory.state == common_pb.MMAKE_MAKING then
                    local elapsed = e.armory.leftTime - Time.realtimeSinceStartup + e.rmbTime
                    home_info.events:dispatch("ARSENAL_CREATE_TICK", e, elapsed, e.armory.makeTime)
                end
                -- 酒馆
                if e.cfgId == 25 then
                    for __, value in ipairs(e.tavern.drawCardInfo) do
                        if value.leftFreeTime > 0 and value.updateTime > 0 then
                            local elapsed = value.updateTime - Time.realtimeSinceStartup + e.rmbTime
                            if elapsed >= 0 then
                                local total_time = config.tavern_taverns[value.drawType].FreeTimeCD * 60
                                home_info.events:dispatch("TAVERN_COOL_DOWN_TICK", e, elapsed, total_time)
                            else
                                value.updateTime = 0
                                self.events:dispatch("BUILD_INFO_CHANGE", e.id)
                            end
                            break
                        end
                    end
                end
            end
        end
    end,

    getMaxTrainingBuild = function(self)
        local max_build = nil

        for i=3, 6 do
            local build = self:getBuildDataByCfgId(i)
            if not max_build then
                max_build = build
            else
                if max_build.level < build.level then
                    max_build = build
                end
            end
        end

        return max_build
    end,

    getGridIdByBuildCfgId = function(self, build_cfg_id)
        local grid_id = nil
        config.home_config_grids:foreach(function(id, cfg)
            local has_build = home_info:isGridHasBuild(id)
            if not has_build then
                local list = loadstring("return " .. cfg.can_build)()
                for idx, cfg_id in ipairs(list) do
                    if build_cfg_id == cfg_id then 
                        grid_id = id
                    end
                end
            end
        end)
        return grid_id
    end,

    --[[
    isResCanExchange = function(self, item_id)
        local list = {}
        table.insert(list, 32001)
        table.insert(list, 30002)
        table.insert(list, 32002)
        table.insert(list, 32003)
        table.insert(list, 32004)
        table.insert(list, local_lord.EXP_ID)
        table.insert(list, army_queue_manager.normal_speed_up)
        table.insert(list, army_queue_manager.transport_speed_up)
        table.insert(list, army_queue_manager.mass_speed_up)
        for k, v in ipairs(list) do
            if v == item_id then
                return true
            end
        end
        return false
    end,
    ]]
}