local log = logger.get("Game", "Home Map")

-- 建筑
local build_cls = require("RTS.home_sys.home_entity_build")
-- 空地
local empty_cls = require("RTS.home_sys.home_entity_empty")
-- 区域（未解锁）
local lock_group_cls = require("RTS.home_sys.home_entity_group")

local layer_home = "Home"
local layer_ground = "Ground"
local frame_helper = require("gui.tool.frame_helper")
local log = logger.get("Game", "Home")
local operator_path = "Assets/Frames/HomeEntityOperator.prefab"
local rot = Quaternion.identity
local update_interval = 10
local screen_center = Vector3(UnityEngine.Screen.width * 0.5, UnityEngine.Screen.height  * 0.5, 0)
local fov = 55

-- //个人基地加速类型
-- enum EPB_SPEEDUP
-- {
-- EPB_NONE    		= 0;
-- EPB_BUILD			= 1;//建造建筑
-- EPB_TECH_RESEARCH	= 2;//科技研发
-- EPB_PRO_EQUIP		= 3;//装备生产
-- EPB_REPAIR_WALL	= 4;//城墙修复
-- EPB_PRO_SOLDIER	= 5;//生产士兵
-- EPB_PRO_TRAP 		= 6;//生产陷阱
-- EPB_REPAIR_TRAP	= 7;//陷阱修复
-- EPB_CURE_SOLDIER	= 8;//治疗士兵
-- EPB_PRO_QUA_EQUIP	= 9;//生产量子装备
-- }

home_speed_up_type = {
    build_upgrade = 43,
    tech_upgrade = 44,
    equip_create = 45,
    wall_reqair = 61,
    soldier_create = 78,
    trap_create = 64,
    trap_repair = 65,
    soldier_repair = 66,
    quantum_create = 62,
}

-- 基地的管理类
return class
{
    ctor = function(self, owner)
        self.owner = owner              -- game_state_home
        self.camera = Camera.main       -- 主摄像头 
        self.home_camera = self.camera.gameObject:GetComponent(Home.HomeCamera)
        self.events = require("base.event").new()
        self.root = GameObject() 
        self.root.transform.name = "build_root"
        frame_helper:setChildLayer(self.root.transform, "Home")

        self.IS_RES_LOADED = false
    end,

    init = function(self, msg)
        -- 创建对象池
        self.pool_parent = app.gameObject.transform:CreateChild("home_map_pool")
         self.pool = require("base.pool") {
            size = 1, 
            transform = self.pool_parent,
        }

        app:startCoroutine(function() 
            self.pool:wait_prepare(operator_path, 5)

            -- 等对象池加载完成后，再向服务器发送消息（因为在使用对象池的时候，对象尚未加载完成）
            if type(game_info.player_id) == "string" then
                home_info:clearEntitys()
                network_sender:enterPersonBase(game_info.player_id)
                -- 请求活动
                network_sender:getActiveList()
            else
                log.error("玩家id类型不对", game_info.player_id)
            end
        end)

        self.BUILD_LOADED = self.events:on("BUILD_LOADED", function (home_build)   
            local build_data = home_info:getBuildDataByGridId(home_build.grid_id)
            if build_data then
                self:initOperator(build_data.id)
                self:refreshOperator(build_data.id)

                self:setModelLoaded(home_build.grid_id)
            end
        end)

        self.events:on("EMPTY_LOADED", function (home_empty)   
            self:setModelLoaded(home_empty.grid_id)
        end)

        self.events:on("BUILD_UN_LOADED", function(home_build)
            self:destroyOperator(home_build.id)
        end)

        self.BUILD_INFO_CHANGE = home_info.events:on("BUILD_INFO_CHANGE", function(id)
            self:refreshBuildInfo(id)
            self:refreshOperator(id)

            -- 检测士兵的增加
            self:onIncrementSoldiers(id)
        end)

        self.GROUP_INFO_CHANGE = home_info.events:on("GROUP_INFO_CHANGE", function()
            self:refreshGroupInfo()
        end)

        self.GRID_INFO_CHANGE = home_info.events:on("GRID_INFO_CHANGE", function()
            self:refreshEmpty()
        end)

        self.CHAPTER_OVER = chapter_manager.events:on("CHAPTER_OVER", function()
            self:refreshShips()
        end)

        self.tick_event_list = {
            "BUILD_UPGRADE_TICK",
            "BARRACK_TRAIN_TICK",
            "HOSPITAL_CURE_TICK",
            "TRAP_TRAIN_TICK",
            "TRAP_CURE_TICK",
            "WALL_CURE_TICK",
            "TECH_UPGRADE_TICK",
            "FORGE_UPGRADE_TICK",
            "ARSENAL_CREATE_TICK",
        }

        for __, event_name in ipairs(self.tick_event_list) do
            home_info.events:on(event_name, function(build_data, elapsed, total_time)
                self:onBuildTick(event_name, build_data, elapsed, total_time)
            end)
        end

        self:refreshShips()
    end,

    onUpdate = function(self, dt)
        -- self:updateMine(dt)
    end,

    -- 刷新空地，空地 = (没有建筑 and 已解锁)
    refreshEmpty = function(self)
        self.home_entitys = self.home_entitys or {}
        for grid_id, is_lock in pairs(home_info.grids) do
            local is_empty = false
            if is_lock then
                is_empty = true
            else
                local has_build = home_info:isGridHasBuild(grid_id)
                if not has_build then
                    is_empty = true
                end
            end
            if is_empty then
                local cls = self.home_entitys[grid_id]
                if cls and cls:hasBuild() then
                    cls:exit()
                    self.home_entitys[grid_id] = empty_cls(self, grid_id, self:getRootByGridId(grid_id), self.events)
                else
                    if cls then
                        cls:refresh()
                    else
                        self.home_entitys[grid_id] = empty_cls(self, grid_id, self:getRootByGridId(grid_id), self.events)
                    end
                end
            end
        end
    end,

    -- 刷新区域
    refreshGroupInfo = function(self)
        self.home_groups = self.home_groups or {}
        for group_id, is_lock in pairs(home_info.groups) do
            local cls = self.home_groups[group_id]
            if cls then
                if is_lock then
                    cls:refresh()
                else
                    cls:exit()
                    self.home_groups[group_id] = nil
                end
            else
                if is_lock then
                    -- 废弃
                    -- self.home_groups[group_id] = lock_group_cls(self, self.events, self:getRootByGroupId(group_id), group_id)
                end
            end
        end
    end,

    -- 同步建筑
    refreshBuildInfo = function(self, id)
        self.home_entitys = self.home_entitys or {}
        local build_data = home_info:getBuildDataById(id)
        if build_data then
            local grid_id = build_data.grid_id
            local cls = self.home_entitys[grid_id]
            if cls then
                if cls:hasBuild() then
                    cls:refresh()
                else
                    cls:exit()
                    self.home_entitys[grid_id] = build_cls(self, grid_id, build_data.id, build_data.cfgId, build_data.level, self:getRootByGridId(grid_id), self.events)    
                end
            else
                self.home_entitys[grid_id] = build_cls(self, grid_id, build_data.id, build_data.cfgId, build_data.level, self:getRootByGridId(grid_id), self.events)
            end
        end
    end,

    getRootByGridId = function(self, grid_id)
        local rst = self.root
        config.home_config_grid_groups:foreach(function(id, cfg)
            local tab = loadstring("return " .. cfg.member)()
            for __, id in ipairs(tab) do
                if grid_id == id and string.len(cfg.grid_parent) > 0 then
                    local temp = GameObject.Find(cfg.grid_parent)
                    if temp then
                        rst = temp
                        break
                    else
                        log.error("不存在游戏对象:", cfg.grid_parent)
                    end
                end
            end
        end)
        return rst
    end,

    getRootByGroupId = function(self, group_id)
        local rst = self.root
        local cfg = config.home_config_grid_groups[group_id]
        if string.len(cfg.grid_parent) > 0 then
            rst = GameObject.Find(cfg.grid_parent)
        end
        return rst
    end,

    getHomeBuildById = function(self, id)
        for grid_id, he in pairs(self.home_entitys) do
            if he.id == id then
                return he
            end
        end
    end,

    getEntityByGridId = function(self, grid_id)
        return self.home_entitys[grid_id]
    end,

    onClickEntity = function(self, grid_id, force_open_ui)
        local entity_grid = self.home_entitys[grid_id]
        entity_grid:interactEntity()
        if entity_grid:hasBuild() then
            -- 显示功能栏
            self:showOperator(entity_grid)
            print("建筑 Grid Id:", grid_id)
        else
            print("空地 Grid Id:", grid_id)
            self:moveToEmptyOpenUI(grid_id, force_open_ui)
        end
    end,

    -- 点击场景
    click = function(self, point)
        guide_helper:removeTapHere()

        local rst, go = self:raycast(point, layer_home)
        if rst and go then
            local mono = self:getHomeEntityMono(go)
            if mono then
                print("MonoId:", mono.id)
                if mono.is_group then
                    local group_id = mono.id
                    self.home_groups[group_id]:interactEntity()
                else
                    local grid_id = mono.id
                    self:onClickEntity(grid_id)
                end
            else
                -- 点了资源船模型
                local res_ship = go:GetComponent(HomeResShip)
                if res_ship then
                    log.info("### Click ResShip:"..res_ship.id)
                    chapter_manager:onClickResShip(res_ship.id)
                end

                -- 隐藏功能栏
                self:showOperator(nil)
            end
        else
            -- 隐藏功能栏
            self:showOperator(nil)
        end
    end,

    -- 实现点击建筑并显示功能按钮 
    clickEntityByGridId = function(self, grid_id, build_btn_type)
        local build_data = home_info:getBuildDataByGridId(grid_id)
        if build_data then
            local grid_id = build_data.grid_id
            local entity = self.home_entitys[grid_id]
            entity:interactEntity()
            if entity:hasBuild() then
                -- 显示功能栏
                self:showOperator(entity)

                if self.cache_build_btns and self.cache_build_btns[build_btn_type] then
                    local parent = self.cache_build_btns[build_btn_type]
                    guide_helper:showTapHere(parent, "UI", 1, true)
                end
            end
        else
            self:moveToEmptyOpenUI(grid_id)
        end
    end,

    -- 移动到建筑上面显示建筑升级界面
    moveToEmptyOpenUI = function(self, grid_id, force)
        if not grid_id then
            return
        end
        local lock = home_info:isGridLock(grid_id)
        if lock then
            print("该网格尚未解锁!!", grid_id)
            return 
        end

        local home_entity = self:getEntityByGridId(grid_id)
        -- self.home_camera:LookAt(home_entity.transform, screen_center, 55)  --modified by hogn feng 改一下点击grid后的fov值
        -- app:startCoroutine(
        --     function()
        --         coroutine.yield(WaitForSeconds(0.75))
        --         frame_manager:show('home_entity_create', self, grid_id)
        --     end
        -- )

         --modified by hogn feng 改一下点击grid后的fov值

        if force then
            app:startCoroutine(function()
                self.home_camera:LookAt(home_entity.transform, screen_center, 55, nil)
                coroutine.yield(WaitForSeconds(0.5))
                frame_manager:show('home_entity_create', self, grid_id)
            end)
        else
            self.home_camera:LookAt(home_entity.transform, screen_center, 55, function()
                frame_manager:show('home_entity_create', self, grid_id)
            end) 
        end
    end,

    dragStart = function(self, point)
    end,

    drag = function(self, point)
    end,
    
    dragEnd = function(self, point)
    end,

    -- 射线
    raycast = function(self, point, layer)
        local mask = core.mask(layer)
        local r = self.camera:ScreenPointToRay(point)
        local rst, hit = UnityEngine.Physics.Raycast(r, hit, 999, mask)
        if rst then
            local go = hit.transform.gameObject
            return rst, go, hit
        else
            return rst, nil, nil
        end
    end,

    -- 获取场景建筑上脚本
    getHomeEntityMono = function(self, go)
        return go:GetComponent(HomeEntity)
    end,

    -- 显示建筑操作界面
    showOperator = function(self, home_entity, force)
        if home_entity and home_entity.level <= 0 then
            return
        end

        self.cache_build_btns = nil
        if force or self.select_build ~= home_entity then
            local ex = self.select_build
            if ex then
                local obj = self:getOperator(ex.id)
                if obj then
                    self:hideOperatorBtns(obj)
                end
            end

            local cur = home_entity
            if cur then
                local obj = self:getOperator(cur.id)
                if obj then
                    self:showOperatorBtns(cur, obj)
                end
            end
            self.select_build = home_entity
        end
    end,

    -- 显示建筑倒计时
    initOperator = function(self, id)
        local obj = self:getOperator(id)
        if not obj then 
            obj = self:createOperator(id)
            self:hideOperatorBtns(obj)
        end
    end,

    createWorldSpaceCanvas = function(self)
        if not self.operate_root then
            local root = GameObject()
            root.name = "canvas_world"
            root.transform.position = Vector3.zero
            local canvas = root:AddComponent(UnityEngine.Canvas)
            canvas.renderMode = 2
            canvas.sortingLayerName = "UI"
            root:AddComponent(UnityEngine.UI.CanvasScaler);
            root:AddComponent(UnityEngine.UI.GraphicRaycaster);
            frame_helper:setChildLayer(root.transform, "UI")
            self.operate_root = root
        end
        return self.operate_root
    end,

    -- 刷新头顶ui
    refreshOperator = function(self, id)

        self:removeBuildHeadProgress(id)

        local build_data = home_info:getBuildDataById(id)
        local obj = self:getOperator(id)
        if build_data and obj then

            local parent_trans = obj.transform:Find("content/info/detail/layout")
            for i=0, parent_trans.childCount - 1 do
                local child = parent_trans:GetChild(i)
                child:SetActive(false)
            end
            
            local bubble_trans = obj.transform:Find("content/info/detail/bg/bubble layout")
            for i=0, bubble_trans.childCount - 1 do
                local child = bubble_trans:GetChild(i)
                child:SetActive(false)
            end

            local setSliderCount = function(idx, prefix, elapsed, total_time, tag_sprite_name, event_name)
                local child = parent_trans:Find(idx == 0 and "build_upgrade" or "other_upgrade")
                child:SetActive(true)
                local slider = child.transform:Find("icon"):GetComponent(UnityEngine.UI.Image)
                local text = child.transform:Find("Text"):GetComponent(UnityEngine.UI.Text)
                local tag = child.transform:Find("tag"):GetComponent(UnityEngine.UI.Image)
                tag:SetSprite(tag_sprite_name)
                slider.fillAmount = 1 - elapsed / total_time
                text.text = frame_helper:timeFormat(elapsed)
                self:registerBuildHeadProgress(id, event_name, text, slider)
            end

            -- MTechnology
            local setBubble = function(idx, none, clickCallback, cfg_id)
                cfg_id = cfg_id or 1
                local child = self:getChildByIdx(idx, bubble_trans)
                child:SetActive(true)
                local icon = child.transform:Find("animation_root/icon"):GetComponent(UnityEngine.UI.Image)
                local icon_add = child.transform:Find("animation_root/icon_add"):GetComponent(UnityEngine.UI.Image)
                local bg = child.transform:Find("animation_root/bg"):GetComponent(UnityEngine.UI.Image)
                local cfg = config.home_config_bubble_news[cfg_id]
                bg:SetSprite(cfg.frame)
                icon:SetSprite(cfg.icon)
                icon_add:SetSprite(cfg.icon)
                local btn = child:GetComponent(UnityEngine.UI.Button)
                btn.onClick:RemoveAllListeners()
                btn.onClick:AddListener(function()
                    if clickCallback then clickCallback() end
                end)
                return icon,bg
            end

            local bubble_idx = 0

            -- 科技升级
            if build_data and build_data.studio then
                local studio = build_data.studio
                local upgrade_time = studio.tLevelUp - (Time.realtimeSinceStartup - build_data.rmbTime)
                if upgrade_time > 0 then
                    setSliderCount(1, prefix, upgrade_time, studio.totalTime, "Assets/Arts/ui/atlas/common/main_icon_21_2.png", "TECH_UPGRADE_TICK")
                    -- 处于工会可以请求帮助
                    if local_guild:isSelfInGuild() then
                        if studio.canHelp then
                            local icon, bg  = setBubble(bubble_idx, nil, function()
                                network_sender:clickBubblie(common_pb.BT_TECHNOLOGY, studio.cfgId)
                            end, 1)
                        end
                    else
                        local icon, bg  = setBubble(bubble_idx, nil, function()
                            frame_helper:showCommonBox(NativeText.GetLangStr(900015), function()
                                gui_helper:openGuildListUI()
                            end)
                        end, 1)
                    end
                end
            end

            -- 修理包制造
            if build_data and build_data.fixFactoryInfo then
                local f = build_data.fixFactoryInfo
                local upgrade_time = f.leftTime - (Time.realtimeSinceStartup - build_data.rmbTime)
                if upgrade_time > 0 then
                    setSliderCount(1, prefix, upgrade_time, f.makeTime, "Assets/Arts/ui/atlas/common/main_icon_21_1.png")
                end
                if f.state == 3 then
                    local icon, bg = setBubble(bubble_idx, nil, function()
                        network_sender:clickBubblie(common_pb.BT_FIX_PACKAGE, 1)
                    end, 2)
                end
            end

            -- 修理厂图纸制造进度条
            if build_data and build_data.armoryInfo and build_data.armoryInfo.state == common_pb.MMAKE_MAKING then
                local f = build_data.armoryInfo
                local upgrade_time = f.leftTime - (Time.realtimeSinceStartup - build_data.rmbTime)
                if upgrade_time > 0 then
                    setSliderCount(1, prefix, upgrade_time, f.makeTime, "Assets/Arts/ui/atlas/common/main_icon_21_1.png")
                end
            end

            -- 修理厂图纸制造完成气泡
            if build_data and build_data.armoryInfo and build_data.armoryInfo.state == common_pb.MMAKE_MAKE_DONE then
                local icon, bg  = setBubble(bubble_idx, nil, function()
                    network_sender:getArmoryEquip()
                end, 4)
            end

            -- 英雄修理完成提示
            if build_data and build_data.endFixHeroes then
                if #build_data.endFixHeroes > 0 then
                    local dis, frame = frame_manager:isDisplay('repair')
                    if dis then
                        network_sender:clickBubblie(common_pb.BT_REPAIR_HERO, 1)
                    else
                        local icon, bg  = setBubble(bubble_idx, nil, function()
                            network_sender:clickBubblie(common_pb.BT_REPAIR_HERO, 1)
                        end, 3)
                    end
                end
            end

            -- 科技研发帮助按钮提示
            if build_data then
                local rst = build_data.studio.canHelp
                if rst then
                    local icon, bg = setBubble(bubble_idx, nil, function()
                        network_sender:reqHelpFastBuild(build_data.studio.cfgId, common_pb.HT_RES)
                    end, 1)
                end
            end

            -- 兵营造兵进度条
            if build_data and build_data.barrack and build_data.barrack.bTran and build_data.barrack.reamintime > 0 then
                print("兵营造兵进度条:", build_data.barrack.bTran, build_data.barrack.reamintime)
                local f = build_data.barrack
                local left_time = f.reamintime - (Time.realtimeSinceStartup - build_data.rmbTime)
                if left_time > 0 then
                    setSliderCount(1, "", left_time, f.totaltime, "Assets/Arts/ui/atlas/common/main_icon_21_1.png", "BARRACK_TRAIN_TICK")
                end
            end

            -- 兵营领取气泡
            if build_data.cfgId == 3 or build_data.cfgId == 4 or build_data.cfgId == 5 or build_data.cfgId == 6 then
                -- 获得可以训练的士兵类型
                local can_train_soldier_type = home_info:isBuildCanTrainSoldier(build_data)
                -- 可以接收的士兵
                if build_data and build_data.barrack and build_data.barrack.canRecv then
                    local icon, bg  = setBubble(bubble_idx, nil, function()
                        network_sender:receiveSoldier(build_data.id)
                    end, 3)
                -- 可以训练的士兵
                elseif can_train_soldier_type then
                    --[[
                    local icon, bg  = setBubble(bubble_idx, nil, function()
                        frame_manager:show('home_barrack', self, {
                            build_id = build_data.id,
                            soldier_type = can_train_soldier_type
                        })
                    end)
                    local cfg = config.home_config_bubble_news[4]
                    bg:SetSprite(cfg.frame)
                    icon:SetSprite(cfg.icon)
                    bubble_idx = bubble_idx + 1
                    ]]
                end
            end

            -- 医院治疗进度条
            if build_data and #build_data.hospital.soldiers > 0 then
                local soldier = nil
                local temp = 0
                for __, value in ipairs(build_data.hospital.soldiers) do
                    if value.reamintime > temp then
                        temp = value.reamintime
                        soldier = value
                    end
                end
                if soldier then
                    local left_time = soldier.reamintime - (Time.realtimeSinceStartup - build_data.rmbTime)
                    if left_time > 0 then
                        setSliderCount(1, "", left_time, soldier.totaltime, "Assets/Arts/ui/atlas/common/main_icon_21_1.png", "HOSPITAL_CURE_TICK")
                    end
                end
            end

            -- 医院领取气泡
            if build_data and #build_data.hospital.soldiers > 0 then
                local can_rcv = false
                for __, value in ipairs(build_data.hospital.soldiers) do
                    -- 有治疗好的士兵可以领取
                    if value.canRecv then
                        local icon, bg  = setBubble(bubble_idx, nil, function()
                            network_sender:recvTreatedSoldier(0)
                        end, 6)
                        can_rcv = true
                        break
                    end
                end
                if not can_rcv then
                    local cure_num = home_info:getAllCureSoldierNum()
                    local capacity = home_info:getHospitalCapacity()
                    -- 超过医院容量
                    if cure_num >= capacity then
                        local icon, bg  = setBubble(bubble_idx, nil, function()
                            hint:setText(NativeText.GetLangStr(900014))
                        end, 5)
                    else
                        local num = home_info:getAllHurtedSoldierNum()
                        -- 有士兵受伤
                        if num > 0 then
                            local icon, bg  = setBubble(bubble_idx, nil, function()
                                frame_manager:show('home_hospital', nil, {
                                    is_defense = false,
                                })
                            end, 4)
                        end
                    end
                end
            end

            -- 锻造厂升级装备
            if build_data.cfgId == 22 and build_data.trainCenter.bWork and build_data.trainCenter.remainTime > 0 then
                local f = build_data.trainCenter
                local left_time = f.remainTime - (Time.realtimeSinceStartup - build_data.rmbTime)
                if left_time > 0 then
                    setSliderCount(1, "", left_time, f.totaltime, "Assets/Arts/ui/atlas/common/main_icon_21_1.png", "FORGE_UPGRADE_TICK")
                end
            end

            -- 资源矿领取气泡
            if build_data.resource then

                local getCfgId = function()
                    if build_data.cfgId == 12 then 
                        return 19
                    elseif build_data.cfgId == 13  then 
                        return 21
                    elseif build_data.cfgId == 14  then 
                        return 23
                    elseif build_data.cfgId == 15 then
                        return 25
                    elseif build_data.cfgId == 20 then
                        return 27
                    end
                    return 1
                end

                -- log.info("build_data.resource.isBubble:", build_data.id, build_data.resource.isBubble)
                -- log.info("build_data.resource.isStop:", build_data.id, build_data.resource.isStop)

                -- 可领取
                if build_data.resource.isBubble == 1 then
                    local icon, bg  = setBubble(bubble_idx, nil, function()
                        network_sender:getMineFieldMineral(build_data.id)                    
                    end, getCfgId() + 1)
                -- 已经停止
                elseif build_data.resource.isStop == 1 then
                    local icon, bg  = setBubble(bubble_idx, nil, function()
                        hint:setText(NativeText.GetLangStr(999009))
                    end, getCfgId())
                end
            end

            -- 防御系统进度条
            if build_data.cfgId == 24 then
                if build_data.guardsystem.build and build_data.guardsystem.build.bTran then -- 制造中
                    local left_time = build_data.guardsystem.build.reamintime - (Time.realtimeSinceStartup - build_data.rmbTime)
                    if left_time > 0 then
                        setSliderCount(1, "", left_time, build_data.guardsystem.build.totaltime, "Assets/Arts/ui/atlas/common/main_icon_21_1.png",  "TRAP_TRAIN_TICK")
                    end
                elseif build_data.guardsystem.repair and #build_data.guardsystem.repair > 0 then -- 治疗中
                    local temp = nil
                    for __, s in ipairs(build_data.guardsystem.repair) do
                        if temp then
                            if s.reamintime > temp.reamintime then
                                temp = s
                            end
                        else
                            temp = s
                        end
                    end
                    local left_time = temp.reamintime - (Time.realtimeSinceStartup - build_data.rmbTime)
                    if left_time > 0 then
                        setSliderCount(1, "", left_time, temp.totaltime, "Assets/Arts/ui/atlas/common/main_icon_21_1.png", "TRAP_CURE_TICK")
                    end
                elseif build_data.guardsystem.bFire then -- 治疗中
                    -- local left_time = build_data.guardsystem.fire_remaintime - (Time.realtimeSinceStartup - build_data.rmbTime)
                    -- if left_time > 0 then
                    --     setSliderCount(1, "", left_time, config.home_config_configs[1].fire_duration, "Assets/Arts/ui/atlas/common/main_icon_21_1.png", "WALL_CURE_TICK")
                    -- end
                elseif ( not build_data.guardsystem.bFire ) and build_data.guardsystem.repairWallRemainTime > 0 then -- 治疗中
                    local total_time = math.floor(build_data.guardsystem.repairWallRemainTime / ( 1 - build_data.guardsystem.WallHp / build_data.guardsystem.WallMaxHp ))
                    local left_time = build_data.guardsystem.repairWallRemainTime - (Time.realtimeSinceStartup - build_data.rmbTime)
                    if left_time > 0 then
                        setSliderCount(1, "", left_time, total_time, "Assets/Arts/ui/atlas/common/main_icon_21_1.png", "WALL_CURE_TICK")
                    end
                end
            end

            -- 防御系统气泡
            if build_data.cfgId == 24 then
                -- 是否有损坏的陷阱
                local is_hurted = false
                if build_data and build_data.guardsystem then
                    for __, s in ipairs(build_data.guardsystem.traps) do
                        if s.bHurt > 0 then
                            is_hurted = true
                            break
                        end
                    end
                end
                -- 获得可以训练的陷阱类型
                local can_train_soldier_type = nil
                for __, cfg in ipairs(home_info:getTrapCfgList()) do
                    local can_train = home_info:isTrapCanTrainBySoldierId(cfg.id)
                    if can_train then
                        can_train_soldier_type = cfg.type
                        break
                    end
                end

                -- 制造中陷阱可领取
                if build_data.guardsystem.build and build_data.guardsystem.build.canRecv then
                    -- log.error("可领陷阱", build_data.guardsystem.build.canRecv)
                    local icon, bg  = setBubble(bubble_idx, nil, function()
                        network_sender:receiveSoldier(build_data.id)
                    end, 11)
                -- 修理中陷阱可领取
                elseif build_data.guardsystem.repair and #build_data.guardsystem.repair > 0 then
                    for __, s in ipairs(build_data.guardsystem.repair) do
                        if s.canRecv then
                            local icon, bg  = setBubble(bubble_idx, nil, function()
                                network_sender:recvRepairedTrap(s.cfgId)
                            end, 12)
                            break
                        end
                    end
                -- 城墙着火时，气泡提示，点击后跳转至修理城墙界面
                elseif build_data.bFire then 
                    local icon, bg  = setBubble(bubble_idx, nil, function()
                        frame_manager:show('home_wall_repair', self)
                    end, 7)
                -- 击飞值不满，并且修理击飞值的冷却完成，气泡提示，点击后跳转至修理城墙界面
                elseif build_data.guardsystem.repairCoreCd <= 0 and build_data.guardsystem.CoreHp < build_data.guardsystem.CoreMaxHp then 
                    local icon, bg  = setBubble(bubble_idx, nil, function()
                        frame_manager:show('home_wall_repair', self)
                    end, 8)
                -- 有可以修理的陷阱时，气泡提示，点击后跳转至修理陷阱界面
                elseif is_hurted then
                    local icon, bg  = setBubble(bubble_idx, nil, function()
                        frame_manager:show('home_hospital', nil, {
                            is_defense = true,
                        })
                    end, 10)
                -- 城墙武器容量未满时，防御系统上方有气泡提示玩家建造，点击后跳转至建造陷阱界面
                elseif can_train_soldier_type then
                    local icon, bg  = setBubble(bubble_idx, nil, function()
                        frame_manager:show('home_barrack', self, {
                            build_id = build_data.id,
                            soldier_type = can_train_soldier_type
                        })
                    end, 9)
                end
            end

            -- 兵工厂
            if build_data.cfgId == 11 then
                local e = build_data
                if e.armory.state == common_pb.MMAKE_MAKING then
                    local elapsed = e.armory.leftTime - Time.realtimeSinceStartup + e.rmbTime
                    setSliderCount(1, "", elapsed, e.armory.makeTime, "Assets/Arts/ui/atlas/common/main_icon_21_1.png", "ARSENAL_CREATE_TICK")
                elseif e.armory.state == common_pb.MMAKE_MAKE_DONE then
                    local icon, bg  = setBubble(bubble_idx, nil, function()
                        network_sender:getArmoryEquip()
                    end, 13)
                end
            end

            -- 酒馆抽卡提示
            if build_data.cfgId == 25 then
                local list = build_data.tavern.drawCardInfo
                for __, value in ipairs(list) do
                    if value.leftFreeTime > 0 and value.updateTime <= 0 then
                        local icon, bg  = setBubble(bubble_idx, nil, function()
                            frame_manager:show('home_tavern')              
                        end, value.drawType == common_pb.LOW1DRAW and 18 or 17)
                    end
                end
            end

            -- 建筑升级
            if build_data.tLevelUp > 0 then
                local elapsed = build_data.tLevelUp - Time.realtimeSinceStartup + build_data.rmbTime
                setSliderCount(0, prefix, elapsed, build_data.totaTime, "Assets/Arts/ui/atlas/common/main_icon_21_3.png", "BUILD_UPGRADE_TICK")

                if build_data.free_upgrade then
                    local icon, bg = setBubble(bubble_idx, nil, function()
                        network_sender:perSonBaseSpeedUp(build_data.id, common_pb.EPB_BUILD, 1, 0, {})
                    end, 2)
                else
                    -- 处于工会可以请求帮助
                    if local_guild:isSelfInGuild() then
                        if home_info:isUpgradeCanHelp(build_data) then
                            local icon, bg = setBubble(bubble_idx, nil, function()
                                network_sender:reqHelpFastBuild(build_data.id, common_pb.HT_BUILD)
                            end, 1)
                        end
                    else
                        local icon, bg  = setBubble(bubble_idx, nil, function()
                            frame_helper:showCommonBox(NativeText.GetLangStr(900015), function()
                                gui_helper:openGuildListUI()
                            end)
                        end, 1)
                    end
                end
            end
        end
    end,

    -- index start at 0
    getChildByIdx = function(self, index, parent_trans)
        if index >= parent_trans.childCount then
            local template = parent_trans:GetChild(0)
            for i = 1, index - parent_trans.childCount + 1 do
                local inst_obj = Object.Instantiate(template.gameObject)
                inst_obj.transform.name = i
                inst_obj.transform:SetParent(parent_trans)
                inst_obj.transform.localScale = Vector3.one
                inst_obj.transform.localRotation = Quaternion.identity
                local tplt_rect = template.transform:GetComponent(UnityEngine.RectTransform)
                local inst_rect = inst_obj.transform:GetComponent(UnityEngine.RectTransform)
                inst_rect.anchoredPosition = tplt_rect.anchoredPosition
                inst_rect.anchoredPosition3D = tplt_rect.anchoredPosition3D
                inst_rect.anchorMax = tplt_rect.anchorMax
                inst_rect.anchorMin = tplt_rect.anchorMin
                inst_rect.offsetMax = tplt_rect.offsetMax
                inst_rect.offsetMin = tplt_rect.offsetMin
                inst_rect.pivot = tplt_rect.pivot
                inst_rect.sizeDelta = tplt_rect.sizeDelta
            end    
        end
        local child = parent_trans:GetChild(index)
        return child
    end,

    -- 显示操作按钮
    showOperatorBtns = function(self, home_entity, obj)
        if obj then
            local build_data = home_info:getBuildDataById(home_entity.id)
            local cfg_id = build_data.cfgId
            self.home_camera:LookAt(home_entity.transform, screen_center, config.home_config_lists[cfg_id].select_fov, nil)
            local tab = home_entity:getOperators()
            if tab then
                self:hideOperatorBtns(obj)
                local parent_trans = obj.transform:Find("content/layout")
                for __, idx in ipairs(tab) do
                    local btn_trans = frame_helper:getChildPlus(parent_trans, __).transform
                    if btn_trans then
                        btn_trans.gameObject:SetActive(true)
                        btn_trans.gameObject.name = "operator_"..idx    -- 用对应的 operator idx 命名，方便查找按钮 .syz
                        local icon = btn_trans:GetComponent(UnityEngine.UI.Image)
                        icon:SetSprite(config.home_config_operates[idx].path .. config.home_config_operates[idx].name)
                        local name = btn_trans:Find("Text"):GetComponent(UnityEngine.UI.Text)
                        name.text = frame_helper:getLangCfg(config.home_config_operates[idx].operate)
                        local btn = btn_trans:GetComponent(UnityEngine.UI.Button)
                        btn.onClick:RemoveAllListeners()
                        btn.onClick:AddListener(function ()
                            self:click_operator(home_entity, idx)
                        end)
                    end

                    self.cache_build_btns = self.cache_build_btns or {}
                    self.cache_build_btns[idx] = btn_trans
                end
                local circle_layout = obj.transform:Find("content/layout"):GetComponent(CircleLayout)
                circle_layout:Show()
            end
        end
    end,

    -- 隐藏操作按钮
    hideOperatorBtns = function(self, obj)
        if obj then
            local parent_path = "content/layout"
            local parent_trans = obj.transform:Find(parent_path)
            for i=0, parent_trans.childCount - 1 do
                local child = parent_trans:GetChild(i)
                child:SetActive(false)
            end
        end
    end,

    getOperator = function(self, id)
        if self.operator_list and self.operator_list[id] then
            local temp = self.operator_list[id]
            self:setOperatorInfo(id, temp)
            return temp
        end
    end,

    setOperatorInfo = function(self, id, obj)
        local build_data = home_info:getBuildDataById(id)
        if build_data then
            local cfg = config.home_config_lists[build_data.cfgId]
            local des_path = "content/info/detail/bg/des_text"
            local des_text = obj.transform:Find(des_path):GetComponent(UnityEngine.UI.Text)
            local name = frame_helper:getLangCfg(cfg.name_id)
            if cfg.can_upgrade == 1 and build_data.level > 0 then
                name =  "<color=#FFFFFFFF>Lv." .. build_data.level .. "</color>  " .. name
            else
                name = name .. ""
            end
            des_text.text = name
        end
    end,

    -- 创建操作界面
    createOperator = function(self, id)
        self.operator_list = self.operator_list or {}
        if self.operator_list[id] then
            return self.operator_list[id]
        end

        local home_build = self:getHomeBuildById(id)
        local build_data = home_info:getBuildDataById(id)

        local tip_pos = home_build.transform.position + Vector3(0, 9, 0)

        local root = self:createWorldSpaceCanvas()
        local obj = self.pool:instantiate(operator_path)
        if obj then
            obj.transform:SetParent(home_build.transform)
            obj:Reset()
            obj:ScaleTo(0.03)
            obj.transform.localPosition = Vector3(build_data.x_tips, build_data.y_tips, build_data.z_tips)

            local content = obj.transform:Find("content")
            content.rotation = self.camera.transform.rotation
            local distance = Vector3.Distance(self.camera.transform.position, obj.transform.position)
            distance = math.floor(distance)
            local c = obj:GetComponent(UnityEngine.Canvas)
            c.sortingOrder = distance
        end
        self.operator_list[id] = obj
        self:setOperatorInfo(id, obj)
        return obj
    end,

    -- 删除操作界面
    destroyOperator = function(self, id)
        self.operator_list = self.operator_list or {}
        local obj = self.operator_list[id]
        if obj then
            obj:destroy()
            self.operator_list[id] = nil
        end
    end,

    -- 建筑弹出界面事件
    click_operator = function(self, home_entity, operator_types)
        print('操作类型:', operator_types)
        if operator_types == 1 then
            self:openInfoUI(home_entity)
        elseif operator_types == 2 then
            self:openUpgradeUI(home_entity)
        elseif operator_types == 3 then
            self:openResearchUI(home_entity)
        elseif operator_types == 4 then
            self:openTrainUI(home_entity)
        elseif operator_types == 5 then
        elseif operator_types == 6 then
            self:openGarageUI(home_entity)
        elseif operator_types == 7 then
            frame_manager:show('chapter')
        elseif operator_types == 8 then
            self:openDestroyUI(home_entity)
        elseif operator_types == 9 then
            frame_manager:show("home_speed_up", nil, {
                speed_up_type = home_speed_up_type.build_upgrade,
                id = home_entity.id
            })
        elseif operator_types == 10 then
            network_sender:getRepairInfo()
        elseif operator_types == 11 then
            network_sender:getFactoryMakeInfo()
        elseif operator_types == 12 then
            frame_manager:show('home_arsenal_main')
        elseif operator_types == 13 then
            local data = home_info:getBuildDataByGridId(home_entity.id)
            network_sender:getMineFieldMineral(home_entity.id)
        elseif operator_types == 14 then
            network_sender:getShopList()
        elseif operator_types == 15 then
        elseif operator_types == 16 then
            network_sender:getCellInfo()
        elseif operator_types == 17 then
            self:openUpgradeUI(home_entity)
        elseif operator_types == 18 then
            frame_manager:show('home_buff')
        elseif operator_types == 19 then
            network_sender:getSkinInfo()
        elseif operator_types == 20 then
            frame_manager:show('home_barrack', self, {
                build_id =home_entity.id,
            })
        elseif operator_types == 21 then
            frame_manager:show('home_drill_ground')
            print("部队详情")
        elseif operator_types == 22 then
            print("编队管理")
        elseif operator_types == 23 then
            print("治疗")
            frame_manager:show('home_hospital', nil, {
                is_defense = false,
            })
        elseif operator_types == 24 then
            frame_manager:show('home_defense', self)
        elseif operator_types == 25 then
            frame_manager:show('home_wall_repair', self)
        elseif operator_types == 26 then
            frame_manager:show('home_garrison')
        elseif operator_types == 27 then
            frame_manager:show('home_main_forge')
        elseif operator_types == 28 then
            network_sender:getDepotPropList(tostring(home_entity.id))
        elseif operator_types == 29 then
            if local_guild:isSelfInGuild() then
                gui_helper:openGuildMassUI()
            else
                frame_helper:showCommonBox(NativeText.GetLangStr(900013), function()
                    gui_helper:openGuildListUI()
                end)
            end
        elseif operator_types == 30 then
            -- 侦察
            chapter_manager:spyResShip(home_entity.grid_id)
        elseif operator_types == 31 then
            -- 攻打
            chapter_manager:attackResShip(home_entity.grid_id)
        elseif operator_types == 32 then
            frame_manager:show('home_tavern')
        end
        self:showOperator(nil)
    end,

    -- 打开信息界面
    openInfoUI = function(self, home_entity)
        local want_send_msg = false
        local build_data = home_info:getBuildDataById(home_entity.id)
        config.home_config_resource_buildings:foreach(function(id, cfg)
            if cfg.building_id == build_data.cfgId then
                want_send_msg = true
            end
        end)
        if want_send_msg then
            network_sender:getBuildDetails(home_entity.id)
        else
            frame_manager:show('home_entity_info_detail', nil, {
                is_res = false,
                content = build_data,
            })
        end
    end,

    -- 打开升级界面
    openUpgradeUI = function(self, home_entity)
        network_sender:buildingExpend(home_entity.id, common_pb.EBUILDING_LEVELUP)
    end,

    -- 打开研究界面
    openResearchUI = function(self, home_entity)
        local build_id = home_entity.id
        frame_manager:show('home_entity_research', self, build_id)
    end,

    -- 打开训练室界面
    openTrainUI = function(self, home_entity)
        network_sender:personBaseQuery(home_entity.id)
    end,

    -- 打开销毁界面
    openDestroyUI = function(self, home_entity)
    end,

    -- 打开车库界面
    openGarageUI = function(self, home_entity)
        network_sender:personBaseQuery(home_entity.id)
    end,

    -- 移动到建筑上面显示建筑升级界面
    moveToEntityOpenUI = function(self, cfg_id, home_entity)
        local entity_data = home_info:getBuildDataByCfgId(cfg_id)
        local home_entity = self:getHomeBuildById(entity_data.id)
        self.home_camera:LookAt(home_entity.transform, screen_center, fov, function()
            network_sender:buildingExpend(home_entity.id, common_pb.EBUILDING_LEVELUP)
        end)
        -- app:startCoroutine(
        --     function()
        --         coroutine.yield(WaitForSeconds(1))
        --         -- frame_manager:show('home_entity_upgrade', self, home_entity)
        --         network_sender:buildingExpend(home_entity.id, common_pb.EBUILDING_LEVELUP)
        --     end
        -- )
    end,

    -- 摄像机定位到某 transform
    cameraLookAt = function(self, trans)
        self.home_camera:LookAt(trans, screen_center, fov, nil)
    end,

    -- 根据 cast 到的 gameobject 获取 entity
    getCastEntity = function(self, go)
        local home_entity = go:GetComponent(HomeEntity)
        if not home_entity then
            return nil
        end
        return self:getEntityByGridId(home_entity.id)
    end,

    onIncrementSoldiers = function(self, build_id)
        local build_data = home_info:getBuildDataById(build_id)
        if not build_data then
            return
        end

        if build_data.cfgId == 10 and #build_data.parade.adds > 0 then
            local getPathByGridId = function(grid_id)
                if grid_id == 19 then
                    return 2
                elseif grid_id == 20 then
                    return 4
                elseif grid_id == 17 then
                    return 3
                elseif grid_id == 18 then
                    return 1
                elseif grid_id == 13 then
                    return 5
                end
            end

            local cook_data = {}

            for __, value in ipairs(build_data.parade.adds) do
                cook_data[value.instid] = cook_data[value.instid] or {}
                local list = cook_data[value.instid]
                table.insert(list, value)
            end

            for id, list in pairs(cook_data) do
                app:startCoroutine(function()
                    for __, value in ipairs(list) do
                        local max_num = config.home_config_configs[1].display_soldier_max
                        local num = math.ceil(value.count / config.home_config_configs[1].display_soldier_num)
                        if num > max_num then
                            num = max_num
                        end
                        local soldier_cfg = config.soldier_soldiers[value.cfgId]
                        local path_id = getPathByGridId(home_info:getBuildDataById(value.instid).grid_id)
                        drill_ground_helper:createPathMovingSoldier(soldier_cfg.type, path_id, 10, num, 0.25)
                        local wait_time = 1 + num * 0.25
                        coroutine.yield(WaitForSeconds(2.5))
                    end
                end)
            end
        end
    end,

    onBuildTick = function(self, event_name, build_data, elapsed, total_time)
        -- print("onBuildTick:", event_name, build_data.id, elapsed, total_time)
        local build_id = build_data.id
        if self.build_progress and self.build_progress[build_id] then
            local list = self.build_progress[build_id]
            for __, value in ipairs(list) do
                if value.event_name == event_name then
                    value.text.text = frame_helper:timeFormat(elapsed)
                    value.slider.fillAmount = 1 - elapsed / total_time
                end
            end
        end
    end,

    registerBuildHeadProgress = function(self, build_id, event_name, text, slider)
        self.build_progress = self.build_progress or {}
        self.build_progress[build_id] = self.build_progress[build_id] or {}
        local list = self.build_progress[build_id]
        table.insert(list, {
            event_name = event_name,
            text = text,
            slider = slider,
        })
    end,

    removeBuildHeadProgress = function(self, build_id)
        self.build_progress = self.build_progress or {}
        self.build_progress[build_id] = nil
    end,

    setModelLoaded = function(self, grid_id)
        if not self.IS_RES_LOADED then
            self.build_loaded_state = self.build_loaded_state or {}
            self.build_loaded_state[grid_id] = true

            local temp = true
            for grid_id, build in pairs(self.home_entitys) do
                if not self.build_loaded_state[grid_id] then
                    temp = false
                    break
                end
            end
            if temp then
                self.IS_RES_LOADED = true
                game_scheduler:excuteHomeMapEvent(self)
                print("个人基地全部资源加载完成啦！")

                -- 触发引导
                guide_manager:tryStartGuide()
            end
        end
    end,

    refreshShips = function(self)
        config.home_config_grid_groups:foreach(function(id, cfg)
            if cfg.unlock > 0 then
                local trans = GameObject.Find(cfg.grid_parent).transform
                local info = chapter_manager:getChapterInfo(cfg.unlock)
                trans.localScale = info.state == common_pb.ECHAPTER_OVER and Vector3.one or Vector3.zero
            end
        end)
    end,

    -- 退出
    onExit = function(self)
        self.cache_build_btns = nil
        self.build_loaded_state = nil

        home_info.events:off("BUILD_LOADED", self.BUILD_LOADED)
        home_info.events:off("BUILD_INFO_CHANGE", self.BUILD_INFO_CHANGE)
        home_info.events:off("GROUP_INFO_CHANGE", self.GROUP_INFO_CHANGE)
        home_info.events:off("GRID_INFO_CHANGE", self.GRID_INFO_CHANGE)
        chapter_manager.events:off("CHAPTER_OVER", self.CHAPTER_OVER)

        for __, event_name in ipairs(self.tick_event_list) do
            home_info.events:removeAll(event_name)
        end

        if self.pool then
            self.pool:destroy()
        end
        if self.pool_parent then
            UnityEngine.Object.Destroy(self.pool_parent.gameObject)
            self.pool_parent = nil
        end

        if self.home_entitys then
            for id, he in pairs(self.home_entitys) do
                he:exit()
            end
        end
    end,
}