local log = logger.get("Game", "prop")
require('RTS.prop_sys.prop_enum')

local ATK = prop_types.ATK
local DEF = prop_types.DEF
local DAMAGE_ADD = prop_types.DAMAGE_ADD
local DAMAGE_FACTOR = prop_types.DAMAGE_FACTOR
local BE_DAMAGED_REDUCE = prop_types.BE_DAMAGED_REDUCE
local BE_DAMAGED_REDUCE_FACTOR = prop_types.BE_DAMAGED_REDUCE_FACTOR
local TRUE_DAMAGE_ADD  = prop_types.TRUE_DAMAGE_ADD
local TRUE_DAMAGE_ADD_FACTOR = prop_types.TRUE_DAMAGE_ADD_FACTOR
local ACCURACY_RATE = prop_types.ACCURACY_RATE
local DODGE_RATE = prop_types.DODGE_RATE
local CRIT_RATE = prop_types.CRIT_RATE
local CRIT_FACTOR = prop_types.CRIT_FACTOR
local ANTI_CRIT_RATE = prop_types.ANTI_CRIT_RATE


-- 常数
local ATK_FACTOR = 0
local DEF_FACTOR = 0

local BREAK_DEF_FACTOR = 0  -- 破防系数
local BREAK_DEF_FACTOR_INV = 0

local MIN_ACCURACY_RATE = 0    -- 最小命中率
local MIN_CRIT_RATE = 0       -- 最小暴击率

-- 伤害计算公式
return {

    const_inited = false,
    -- 初始化常量
    initConst = function(self)
        ATK_FACTOR = rts_helper:getCommonVar('ATK_FACTOR')
        DEF_FACTOR = rts_helper:getCommonVar('DEF_FACTOR')

        BREAK_DEF_FACTOR = rts_helper:getCommonVar('BREAK_DEF_FACTOR')-- 破防系数
        BREAK_DEF_FACTOR_INV = 1.0 / BREAK_DEF_FACTOR

        MIN_ACCURACY_RATE = rts_helper:getCommonVar('MIN_ACCURACY_RATE')    -- 最小命中率
        MIN_CRIT_RATE = rts_helper:getCommonVar('MIN_CRIT_RATE')       -- 最小暴击率

        self.const_inited = true
    end,

    -- 总入口
    calculate = function(self, attacker, defender, skill_cfg)
        if not self.const_inited then
            self:initConst()
        end

        -- 从 skill_cfg 中获取伤害公式名称，没有就用默认的
        if self[skill_cfg.calc_name] then
            return self[skill_cfg.calc_name](self, attacker, defender, skill_cfg)
        else
            return self:base(attacker, defender, skill_cfg)
        end
    end,

    -- 参数为攻击者 final 属性组，防御者final 属性组
    base = function(self, attacker,  defender, skill_cfg)
        if not attacker.props or not defender.props then
            return 0, false, false
        end

        local atk_prop_gorup = attacker.props:getFinalGroup()   -- final 属性组
        local def_prop_group = defender.props:getFinalGroup()

        local atk_prop_array = atk_prop_gorup.props     -- 属性数组，方便用
        local def_prop_array = def_prop_group.props

        -- 技能伤害系数
        local skill_factor = 1.0
        if skill_cfg then
            skill_factor = skill_cfg.skill_damage_factor
        end

		-- 火力,电力,动力伤害附加系数
        local firepower_add = 0.0
        local electricpower_add = 0.0
        local motionpower_add = 0.0
        if skill_cfg then
            firepower_add = skill_cfg.firepower_add
			electricpower_add = skill_cfg.electricpower_add
			motionpower_add = skill_cfg.motionpower_add
        end
		
        -- 依次返回以下三个参数
        local dmg, is_hit, is_crit

        -- 命中判定
        is_hit = self.isHit(atk_prop_array, def_prop_array, skill_cfg)
        if not  is_hit then
            return 0, false, false
        end

        -- 根据目标车辆类型的系数加成
        local type_add = 0.0
        local type_factor = 0.0

        --log.info('##### firepower_add:'..firepower_add)
		--log.info('##### electricpower_add:'..electricpower_add)
		--log.info('##### motionpower_add:'..motionpower_add)


        if type(skill_cfg.type_damage_add) == 'table' then
            local type_factor_table = skill_cfg.type_damage_add[defender.cfg.type]
            if type_factor_table then
                type_add = type_factor_table[1]
                type_factor = type_factor_table[2]
                --log.info('type add:'..type_add..', '..'type factor:'..type_factor)
            end
        end

        local break_def = true  -- 是否破防
        -- 核心伤害
        dmg, break_def = self:coreDamage(atk_prop_array, def_prop_array, skill_factor)

        --log.info('##### dmg 0:'..dmg)

        -- 附加伤害，减伤
        dmg = self:additionalDamage(dmg, atk_prop_array, def_prop_array)

        --log.info('##### dmg 1:'..dmg)
		-- 各种力的加成
        dmg = self:powerDamage(dmg, atk_prop_array, def_prop_array, firepower_add, electricpower_add, motionpower_add)
	
        --log.info('##### dmg 2:'..dmg)	
        -- 保底伤害
        dmg = self:trueDamage(dmg, atk_prop_array, def_prop_array)

		--log.info('##### dmg 3:'..dmg)	
        -- 暴击
        dmg, is_crit = self:critDamage(dmg, atk_prop_array, def_prop_array)


       -- log.info("###".. attacker.id..", "..defender.id..", type_add"..type_add..", type_factor:"..type_factor)
        dmg = (dmg + type_add) * (1.0 + type_factor)
		
		--log.info('##### dmg 4:'..dmg)	
        return math.ceil(dmg), is_hit, is_crit
    end,

	-- 参数为攻击者 final 属性组，防御者final 属性组
    heal = function(self, attacker,  defender, skill_cfg)

        local atk_prop_gorup = attacker.props:getFinalGroup()   -- final 属性组
        local def_prop_group = defender.props:getFinalGroup()

        local atk_prop_array = atk_prop_gorup.props     -- 属性数组，方便用
        local def_prop_array = def_prop_group.props

        -- 技能伤害系数，直接调用伤害系数作为治疗系数
        local skill_factor = 1.0
        if skill_cfg then
            skill_factor = skill_cfg.skill_damage_factor
        end

		-- 火力,电力,动力伤害附加系数
        local firepower_add = 0.0
        local electricpower_add = 0.0
        local motionpower_add = 0.0
        if skill_cfg then
            firepower_add = skill_cfg.firepower_add
			electricpower_add = skill_cfg.electricpower_add
			motionpower_add = skill_cfg.motionpower_add
        end
		
        -- 依次返回以下三个参数
        local dmg, is_crit
		local is_hit = true     --治疗必中

        -- 根据目标车辆类型的系数加成,治疗需要的话也可使用
        local type_add = 0.0
        local type_factor = 0.0

        if type(skill_cfg.type_damage_add) == 'table' then
            local type_factor_table = skill_cfg.type_damage_add[defender.cfg.type]
            if type_factor_table then
                type_add = type_factor_table[1]
                type_factor = type_factor_table[2]
                --log.info('type add:'..type_add..', '..'type factor:'..type_factor)
            end
        end

        -- 核心治疗
        dmg = self:coreHeal(atk_prop_array, def_prop_array, skill_factor)

		-- 各种力的加成，力可以对治疗进行加成
        dmg = self:powerDamage(dmg, atk_prop_array, def_prop_array, firepower_add, electricpower_add, motionpower_add)
	
        -- 暴击
        dmg, is_crit = self:critHeal(dmg, atk_prop_array, def_prop_array)

        dmg = -(dmg + type_add) * (1.0 + type_factor)  --结算伤害时取负计算
		
        return math.ceil(dmg), is_hit, is_crit
    end,	
	
    hp_judge_base = function(self, attacker,  defender, skill_cfg)		--目标生命值低于多少，受到伤害提高
	
		local damage,is_hit,is_crit = self:base(attacker, defender, skill_cfg)
        if is_hit then
			if  defender.hp/defender.max_hp < skill_cfg["p3"] then
			damage = damage * (1 + skill_cfg["p4"])
			end
		end		
        return math.ceil(damage), is_hit, is_crit
    end,

    hp_judge_base1 = function(self, attacker,  defender, skill_cfg)     --生命低于多少，必定暴击
	
        local atk_prop_gorup = attacker.props:getFinalGroup()   -- final 属性组
        local def_prop_group = defender.props:getFinalGroup()

        local atk_prop_array = atk_prop_gorup.props     -- 属性数组，方便用
        local def_prop_array = def_prop_group.props

        -- 技能伤害系数
        local skill_factor = 1.0
        if skill_cfg then
            skill_factor = skill_cfg.skill_damage_factor
        end
		
		-- 火力,电力,动力伤害附加系数
        local firepower_add = 0.0
        local electricpower_add = 0.0
        local motionpower_add = 0.0
        if skill_cfg then
            firepower_add = skill_cfg.firepower_add
			electricpower_add = skill_cfg.electricpower_add
			motionpower_add = skill_cfg.motionpower_add
        end	
		
				
        -- 依次返回以下三个参数
        local dmg, is_hit, is_crit

        -- 命中判定
        is_hit = self.isHit(atk_prop_array, def_prop_array, skill_cfg)
        if not  is_hit then
            return 0, false, false
        end

        -- 根据目标车辆类型的系数加成
        local type_add = 0.0
        local type_factor = 0.0

        if type(skill_cfg.type_damage_add) == 'table' then
            local type_factor_table = skill_cfg.type_damage_add[defender.cfg.type]
            if type_factor_table then
                type_add = type_factor_table[1]
                type_factor = type_factor_table[2]
                --log.info('type add:'..type_add..', '..'type factor:'..type_factor)
            end
        end

        local break_def = true  -- 是否破防
        -- 核心伤害
        dmg, break_def = self:coreDamage(atk_prop_array, def_prop_array, skill_factor)

        -- 附加伤害，减伤
        dmg = self:additionalDamage(dmg, atk_prop_array, def_prop_array)

        -- 各种力的加成
        dmg = self:powerDamage(dmg, atk_prop_array, def_prop_array, firepower_add, electricpower_add, motionpower_add)
		
        -- 保底伤害
        dmg = self:trueDamage(dmg, atk_prop_array, def_prop_array)
		
        -- 暴击
		if  defender.hp / defender.max_hp < skill_cfg["p2"] then
			is_crit = true
		end
        dmg = dmg * (1.0 + atk_prop_array[CRIT_FACTOR])

        dmg = (dmg + type_add) * (1.0 + type_factor)

        return math.ceil(dmg), is_hit, is_crit
    end,	
		
	

    must_crit_calc = function(self, attacker,  defender, skill_cfg)     --必定暴击，且暴击率转化为伤害率
	
        local atk_prop_gorup = attacker.props:getFinalGroup()   -- final 属性组
        local def_prop_group = defender.props:getFinalGroup()

        local atk_prop_array = atk_prop_gorup.props     -- 属性数组，方便用
        local def_prop_array = def_prop_group.props

        -- 技能伤害系数
        local skill_factor = 1.0
        if skill_cfg then
            skill_factor = skill_cfg.skill_damage_factor
        end
		
		-- 火力,电力,动力伤害附加系数
        local firepower_add = 0.0
        local electricpower_add = 0.0
        local motionpower_add = 0.0
        if skill_cfg then
            firepower_add = skill_cfg.firepower_add
			electricpower_add = skill_cfg.electricpower_add
			motionpower_add = skill_cfg.motionpower_add
        end	
		
				
        -- 依次返回以下三个参数
        local dmg, is_hit, is_crit

        -- 命中判定
        is_hit = self.isHit(atk_prop_array, def_prop_array, skill_cfg)
        if not  is_hit then
            return 0, false, false
        end

        -- 根据目标车辆类型的系数加成
        local type_add = 0.0
        local type_factor = 0.0

        if type(skill_cfg.type_damage_add) == 'table' then
            local type_factor_table = skill_cfg.type_damage_add[defender.cfg.type]
            if type_factor_table then
                type_add = type_factor_table[1]
                type_factor = type_factor_table[2]
                --log.info('type add:'..type_add..', '..'type factor:'..type_factor)
            end
        end

        local break_def = true  -- 是否破防
        -- 核心伤害
        dmg, break_def = self:coreDamage(atk_prop_array, def_prop_array, skill_factor)

        -- 附加伤害，减伤
        dmg = self:additionalDamage(dmg, atk_prop_array, def_prop_array)

        -- 各种力的加成
        dmg = self:powerDamage(dmg, atk_prop_array, def_prop_array, firepower_add, electricpower_add, motionpower_add)
		
        -- 保底伤害
        dmg = self:trueDamage(dmg, atk_prop_array, def_prop_array)
		
        -- 必定暴击
		is_crit = true
		
        dmg = dmg * (1.0 + atk_prop_array[CRIT_FACTOR]) * (1+ atk_prop_array[CRIT_RATE])

        dmg = (dmg + type_add) * (1.0 + type_factor)

        return math.ceil(dmg), is_hit, is_crit
    end,

	
	
    -- 核心伤害计算
    coreDamage = function(self, atk_prop_array, def_prop_array, skill_factor)
        local dmg = 0
        local break_def = true
        if atk_prop_array[ATK] * (1.0 - BREAK_DEF_FACTOR_INV) > def_prop_array[DEF] then
            -- 破防
            dmg = (atk_prop_array[ATK] * ATK_FACTOR - def_prop_array[DEF] * DEF_FACTOR) * skill_factor
            break_def = true
        else
            dmg = atk_prop_array[ATK] * BREAK_DEF_FACTOR_INV
            break_def = false
        end

        return dmg, break_def
    end,

    -- 核心治疗计算
    coreHeal = function(self, atk_prop_array, def_prop_array, skill_factor)
        local dmg = 0
            dmg = atk_prop_array[ATK] * skill_factor
        return dmg
    end,	
	
    -- 附加伤害，减伤
    additionalDamage = function(self, core_dmg, atk_prop_array, def_prop_array)
        local dmg_add = atk_prop_array[DAMAGE_ADD] - def_prop_array[BE_DAMAGED_REDUCE]
        local dmg_factor = atk_prop_array[DAMAGE_FACTOR] - def_prop_array[BE_DAMAGED_REDUCE_FACTOR]
        local dmg = (core_dmg + dmg_add) * (1.0 + dmg_factor)
        return dmg
    end,
	
	-- 火力，电力，动力加成
    powerDamage = function(self, dmg, atk_prop_array, def_prop_array ,firepower_add ,electricpower_add ,motionpower_add)
        local dmg_add_fire = atk_prop_array[FIRE_POWER_IDX] * firepower_add
		local dmg_add_electric = atk_prop_array[ELECTRIC_POWER_IDX] * electricpower_add
		local dmg_add_motion = atk_prop_array[MOTION_POWER_IDX] * motionpower_add
        local dmg = dmg + dmg_add_fire + dmg_add_electric + dmg_add_motion
	    --log.info('##### dmg_add_fire:'..dmg_add_fire)
        --log.info('##### dmg_add_electric:'..dmg_add_electric)
        --log.info('##### dmg_add_motion:'..dmg_add_motion)
        return dmg
    end,
	
    -- 保底伤害
    trueDamage = function(self, dmg, atk_prop_array, def_prop_array)
        local v = dmg + atk_prop_array[TRUE_DAMAGE_ADD] * ( 1.0 + atk_prop_array[TRUE_DAMAGE_ADD_FACTOR] )
        return v
    end,

    -- 暴击伤害
    critDamage = function(self, dmg, atk_prop_array, def_prop_array)
        is_crit = self.isCrit(atk_prop_array, def_prop_array, skill_cfg)
        local v = dmg
        if is_crit then
            v = dmg * (1.0 + atk_prop_array[CRIT_FACTOR])
        end
        return v, is_crit
    end,

    -- 暴击治疗
    critHeal = function(self, dmg, atk_prop_array, def_prop_array)
        is_crit = self.isHealCrit(atk_prop_array, def_prop_array, skill_cfg)
        local v = dmg
        if is_crit then
            v = dmg * (1.0 + atk_prop_array[CRIT_FACTOR])
        end
        return v, is_crit
    end,
	
    isHit = function(atk_prop_array, def_prop_array, skill_cfg)
        --log.info('A:'..attacker[ACCURACY_RATE])
        --log.info('D:'..defender[DODGE_RATE])

        local rate = atk_prop_array[ACCURACY_RATE] - def_prop_array[DODGE_RATE]
        rate = math.max( rate, MIN_ACCURACY_RATE)
        return RTSHelper.RandomDouble() <= rate
    end,

    isCrit = function(atk_prop_array, def_prop_array, skill_cfg)
        local rate = atk_prop_array[CRIT_RATE] - def_prop_array[ANTI_CRIT_RATE]
        rate = math.max(rate, MIN_CRIT_RATE)
        return RTSHelper.RandomDouble() <= rate
    end,
	
	--治疗的暴击判定
    isHealCrit = function(atk_prop_array, def_prop_array, skill_cfg)  
        local rate = atk_prop_array[CRIT_RATE]
        rate = math.max(rate, MIN_CRIT_RATE)
        return RTSHelper.RandomDouble() <= rate
    end,
}       