
require('RTS.prop_sys.prop_enum')
local log = logger.get("Game", "prop")

-- 属性索引
local ATK_INC = prop_types.ATK_INC
local DEF_INC = prop_types.DEF_INC
local MAX_HP_INC = prop_types.MAX_HP_INC
local ATK_RAW = prop_types.ATK_RAW
local DEF_RAW = prop_types.DEF_RAW
local MAX_HP_RAW = prop_types.MAX_HP_RAW
local ATK = prop_types.ATK
local DEF = prop_types.DEF
local MAX_HP = prop_types.MAX_HP
local HP = prop_types.HP
local ATK_FACTOR = prop_types.ATK_FACTOR
local DEF_FACTOR = prop_types.DEF_FACTOR
local MAX_HP_FACTOR = prop_types.MAX_HP_FACTOR
local ACCURACY = prop_types.ACCURACY
local ACCURACY_RATE = prop_types.ACCURACY_RATE
local DODGE = prop_types.DODGE
local DODGE_RATE = prop_types.DODGE_RATE
local CRIT = prop_types.CRIT
local CRIT_RATE = prop_types.CRIT_RATE
local CRIT_FACTOR = prop_types.CRIT_FACTOR
local ANTI_CRIT = prop_types.ANTI_CRIT
local ANTI_CRIT_RATE = prop_types.ANTI_CRIT_RATE


-- 一个属性组
return class {
    ctor = function(self, name, index)
        self.name = name
        self.index = index
        self.props = {}
        self:clear()
    end,

    -- 清空属性组
    clear = function(self)
        -- 设置属性初始值
        for i=1,prop_types.MAX-1, 1 do 
            self.props[i] = 0.0
        end

        self.computed = false
    end,

    -- 从配置中拷贝属性, 不存的就置 0
    copyFromCfg = function(self, cfg)
        for prop_name, prop_idx in pairs(prop_types) do
            if cfg.containsField(prop_name) then
                self.props[prop_idx] = cfg[prop_name] or 0.0
            end
        end
    end,

    -- 将配置中的属性附加进来
    appendFromCfg = function(self, cfg)
        for prop_name, prop_idx in pairs(prop_types) do
            -- 判断 cfg 中是否有 field prop_name
            if cfg.containsField(prop_name) then
                self.props[prop_idx] = self.props[prop_idx] + (cfg[prop_name] or 0.0)
            end
        end
    end,

    -- 从 prop_tab 拷贝属性，不存在的置0
    -- prop_tab 型如 {[id]=value, [id]=value, [id]=value, ...}
    copyFromPropTab = function(self, prop_tab)
        self:clear()
        for id,value in pairs(prop_tab) do
            self.props[id] = value
        end

    end,

    -- 将 prop_tab 的属性附加进来
    appendFromPropTab = function(self, prop_tab)
        for id,value in pairs(prop_tab) do
            self.props[id] = self.props[id] + value
        end

    end,

    -- 将 prop_tab  中的属性减去
    subtractFromPropTab = function(self, prop_tab)
        for id,value in pairs(prop_tab) do
            self.props[id] = self.props[id] - value
        end

    end,

    -- 根据等级重新计算所有属性值
    computeStatic = function(self, level)
        -- 限制只能 compute 一次
        if self.computed then
            log.error('prop_group already computed. Mast be cleared before compute again！！')
            return
        end

        local props = self.props

        -- 根据等级变化的属性值
        props[ATK_RAW] = (props[ATK] + props[ATK_INC] * (level-1))
        props[DEF_RAW] = (props[DEF] + props[DEF_INC] * (level-1))
        props[MAX_HP_RAW] = (props[MAX_HP] + props[MAX_HP_INC] * (level-1))

        self:computeFinal()

        props[HP] = props[MAX_HP]
        self.computed = true
    end,

    -- 将攻防血最终值转换为 raw 值，相应 factor 置 0 
    settleDownStatic = function(self)
        local props = self.props
        
        props[ATK_RAW] = props[ATK]
        props[ATK_FACTOR] = 0
        props[DEF_RAW] = props[DEF]
        props[DEF_FACTOR] = 0
        props[MAX_HP_RAW] = props[MAX_HP]
        props[MAX_HP_FACTOR] = 0
    end,

    -- 计算 Final 属性
    computeFinal = function(self)
        local props = self.props
        props[ATK] = props[ATK_RAW] * (1.0 + props[ATK_FACTOR]) 
        props[DEF] = props[DEF_RAW] * (1.0 + props[DEF_FACTOR])
        props[MAX_HP] = props[MAX_HP_RAW] * (1.0 + props[MAX_HP_FACTOR])

        -- 面板值到几率转换
        props[ACCURACY_RATE] = props[ACCURACY_RATE] + 1 + props[ACCURACY] * rts_helper:getCommonVar('ACCURACY_CONVERTOR')
        props[DODGE_RATE] = props[DODGE_RATE] + props[DODGE] * rts_helper:getCommonVar('DODGE_CONVERTOR')
        props[CRIT_RATE] =  props[CRIT_RATE] + props[CRIT] * rts_helper:getCommonVar('CRIT_CONVERTOR')
        props[ANTI_CRIT_RATE] = props[ANTI_CRIT_RATE] + props[ANTI_CRIT] * rts_helper:getCommonVar('ANTI_CRIT_CONVERTOR')
    end,

    getProp = function (self, prop_idx)
        return self.props[prop_idx]
    end,

    setProp = function(self, prop_idx, value)
        self.props[prop_idx] = value
    end,

    -- 把 other_prop_group 的值累加到自身
    accumulate = function(self, other)
        for i=1,prop_types.MAX-1,1 do
            self.props[i] = self.props[i] + other.props[i]
        end
    end,

    log = function(self)
        local str_log = '------ prop group '..self.name..'----------\n'

        for i=1, prop_types.MAX-1, 1 do
            str_log = str_log..'['..i..'] = '..string.format("%.5f", self.props[i])..'\n'
        end
        log.info(str_log)
    end,

    compair = function(self, other)
        local str = "Group ["..self.name.."]"
        for i=1, prop_types.MAX-1, 1 do
            --if i ~= 4 and i ~= 5 and i ~= 6 then 
                local myValue = self.props[i]
                local otherValue = other.props[i]
                if myValue ~= otherValue then
                    str = str .. '['..i..'] = '..string.format("%.5f", myValue)..', '..string.format("%.5f", otherValue)..'\n'
                end
            --end
        end
        log.info(str)
    end,
}