local log = logger.get('Game', 'red point manager')
require ('RTS.bag_sys.bag_manager')
require ('config.redpoint')

-- 红点管理器
local red_point_manager_cls = class{
    ctor = function( self )
        self.red_points = {}

        bag_manager.events:on("onBagChanged", functor.make(self.onBagChanged, self))

        -- 建立 redpoint path - id 的 map
        self.path2Id = {}

        local addPath2Id = function(path, id)
            if not self.path2Id[path] then
                self.path2Id[path] = {id}
            else
                table.insert(self.path2Id[path], id)
            end
        end

        config.redpoint_redpoints:foreach(function(id, cfg)
            addPath2Id(cfg.frame1_a.."/"..cfg.path1, id)
            addPath2Id(cfg.frame1_b.."/"..cfg.path1, id)
            addPath2Id(cfg.frame2.."/"..cfg.path2, id)
            addPath2Id(cfg.frame3.."/"..cfg.path3, id)
            addPath2Id(cfg.frame4.."/"..cfg.path4, id)

        end)

    end,

    --[[
    message MRedPoint
    {
        optional uint32 id = 1;//Id
        optional uint32 count = 2;//数量
    }

    message SCRedPointAppear
    {
        repeated MRedPoint points = 1;
    }
    
    message SCRedPointDisappear
    {
        repeated uint32 ids = 1;
    }
    ]]

    getOffsetFromStr = function(self, str_tab)
        if not str_tab or str_tab == "{}" then
            return nil
        end

        local tab = loadstring('return '..str_tab)()
        return Vector3(tab[1], tab[2], 0)
    end,

    -- 红点出现
    appear = function(self, msg)
        for _, p in ipairs(msg.points) do
            log.info("## red point appear:"..p.id..", "..p.count)

            self.red_points[p.id] = p.count     -- count 为红点显示数字，0 无数字但有红点

            local cfg = config.redpoint_redpoints[p.id]
            if not cfg then
                log.error("Failed to find red point cfg:"..p.id)
            end

            local addRedPointIfFrameOpen = function(frame_name, path, offset)
                if not frame_name then return end

                if frame_manager:isDisplay(frame_name) then
                    self:addRedPointToUI(frame_manager:getFrame(frame_name), path, offset)
                end
            end

            addRedPointIfFrameOpen(cfg.frame1_a, cfg.path1, self:getOffsetFromStr(cfg.offset1))
            addRedPointIfFrameOpen(cfg.frame1_b, cfg.path1, self:getOffsetFromStr(cfg.offset1))
            addRedPointIfFrameOpen(cfg.frame2, cfg.path2, self:getOffsetFromStr(cfg.offset2))
            addRedPointIfFrameOpen(cfg.frame3, cfg.path3, self:getOffsetFromStr(cfg.offset3))
            addRedPointIfFrameOpen(cfg.frame4, cfg.path4, self:getOffsetFromStr(cfg.offset4))

        end
    end,

    -- 红点消失
    disappear = function(self, msg)
        for _, id in ipairs(msg.ids) do

            log.info("## red point disappear:"..id)

            self.red_points[id] = false
            local cfg = config.redpoint_redpoints[id]
            if not cfg then
                log.error("Failed to find red point cfg:"..p.id)
            end

            local removeRedPointIfFrameOpen = function(frame_name, path)
                if not frame_name then return end

                local relate_ids = self.path2Id[frame_name.."/"..path]
                for _,id in ipairs(relate_ids) do
                    if self.red_points[id] then
                        -- 有红点显示
                        return
                    end
                end

                if frame_manager:isDisplay(frame_name) then
                    self:removeRedPointFromUI(frame_name, path)
                end
            end

            -- log.info("### remove red point:"..id)
            removeRedPointIfFrameOpen(cfg.frame1_a, cfg.path1)
            removeRedPointIfFrameOpen(cfg.frame1_b, cfg.path1)
            removeRedPointIfFrameOpen(cfg.frame2, cfg.path2)
            removeRedPointIfFrameOpen(cfg.frame3, cfg.path3)
            removeRedPointIfFrameOpen(cfg.frame4, cfg.path4)
        end
    end,

    -- 获取界面上需要显示的红点 id
    getFarmeRedPoints = function(self, frame_name)
        local ids = {}
        local found = false
        for id, count in pairs(self.red_points) do
            if count then
                local cfg = config.redpoint_redpoints[id]
                if not cfg then
                    log.error("Failed to find redpoint cfg:"..id)
                end

                if cfg.frame1_a == frame_name or cfg.frame1_b == frame_name or
                    cfg.frame2 == frame_name or cfg.frame3 == frame_name or cfg.frame4 == frame_name then
                    table.insert(ids, id)
                    found = true
                end
            end
        end
        
        if found then
            return ids
        else
            return nil
        end
    end,

    -- 尝试添加红点
    tryAddRedPoint = function(self, frame_name, frame)
        if frame_name == "worldMap" or frame_name == "home" then
            self:checkClientRedPoint()
        end

        require ('config.redpoint')
        local ids = self:getFarmeRedPoints(frame_name)
        if not ids then
            return
        end

        app:startCoroutine(
            function()
                while true do
                    if frame:isLoaded() then
                        coroutine.yield(nil)
                        for _,id in ipairs(ids) do
                            --log.info("### red points:"..id)
                            local cfg = config.redpoint_redpoints[id]
                            if cfg then 
                                if cfg.frame1_a == frame_name or cfg.frame1_b == frame_name then
                                    self:addRedPointToUI(frame, cfg.path1, self:getOffsetFromStr(cfg.offset1))
                                end
                                if cfg.frame2 == frame_name then
                                    self:addRedPointToUI(frame, cfg.path2, self:getOffsetFromStr(cfg.offset2))
                                end
                                if cfg.frame3 == frame_name then
                                    self:addRedPointToUI(frame, cfg.path3, self:getOffsetFromStr(cfg.offset3))
                                end
                                if cfg.frame4 == frame_name then
                                    self:addRedPointToUI(frame, cfg.path4, self:getOffsetFromStr(cfg.offset4))
                                end
                            else
                                log.error("### Failed to find redpoint cfg:"..id)
                            end
                        end
                        break
                    else
                        -- log.info("### wait frame load:"..frame_name)
                        coroutine.yield(nil)
                    end
                end
            end
        )
    end,

    -- 为一个 frame  ui_path 控件添加红点
    addRedPointToUI = function(self, frame, ui_path, offset)
        -- log.info("## addRedPointToUI:"..frame.name..", path:"..ui_path..", "..tostring(offset))
        if not frame.transform then
            return
        end
        local parent = frame.transform:Find(ui_path)

        if not parent then
            log.error("Failed to find ui_path to add red point:"..tostring(frame.name)..","..ui_path)
            return
        end

        if parent:Find("common_red_point") then
            return
        end

        -- 判断按钮是否解锁
        if not unlock_manager:isUnlock(frame.name, ui_path) then
            return
        end

        --log.error("## try add redpoint:"..frame.name.."/"..ui_path)
        --log.error(debug.traceback())

        local red_point = GameObject.Instantiate(game_director.red_point_prefab, parent)
        local rect_trans = red_point.transform
        rect_trans.anchorMin = Vector2(0.5, 0.5)
        rect_trans.anchorMax = Vector2(0.5, 0.5)
        rect_trans.pivot = Vector2(1, 1)
        rect_trans.sizeDelta = Vector2(47, 47)
        red_point.transform.localScale = Vector3.one
        local psize = parent.sizeDelta
        rect_trans.anchoredPosition = Vector3(psize.x * 0.5, psize.y * 0.5) + offset
        --go.transform.localPosition = Vector3.zero

        red_point.name = "common_red_point"
        red_point:SetActive(true)

        --[[
        app:startCoroutine(
            function()
                coroutine.yield(WaitForInstantiate("Assets/Prefab/UI/RedPoint.prefab", parent, function(go)
                    --go.transform:SetAsLastSibling()
                    local rect_trans = go.transform
                    rect_trans.anchorMin = Vector2(0.5, 0.5)
                    rect_trans.anchorMax = Vector2(0.5, 0.5)
                    rect_trans.pivot = Vector2(1, 1)
                    rect_trans.sizeDelta = Vector2(47, 47)
                    go.transform.localScale = Vector3.one
                    local psize = parent.sizeDelta
                    rect_trans.anchoredPosition = Vector3(psize.x * 0.5, psize.y * 0.5) + offset
                    --go.transform.localPosition = Vector3.zero

                    go.name = "common_red_point"
                end))
            end)
            ]]

    end,

    removeRedPointFromUI = function(self, frame_name, ui_path)
        local frame = frame_manager:getFrame(frame_name)
        if not frame  or not frame.transform then
            return
        end

        log.info("## removeRedPointFromUI:"..frame_name..","..ui_path)

        local red_point = frame.transform:Find(ui_path.."/common_red_point")
        if red_point then
            Object.DestroyImmediate(red_point.gameObject)
        end
    end,

    
    onBagChanged = function(self, item)
        -- log.info("### red_point_manager onBagChanged type:"..item.cfg.type)
        
        if item.cfg.type == 17 then
            app:startCoroutine(
                function()
                    coroutine.yield(WaitForSeconds(0.1))
                    self:checkClientRedPoint()   
                    coroutine.yield(WaitForSeconds(1.0))
                    self:checkClientRedPoint()   
                end
            )
        end
    end,

    checkClientRedPoint = function(self)
        local have, count = hero_manager:haveStarUpOfficers()
        -- log.info("## have, count:"..tostring(have)..", "..count)
        if have and not self.red_points[1] then
            local points = {{id=1, count=count}}
            local msg = {points = points}
            self:appear(msg)
        elseif not have and self.red_points[1] then
            local ids = {1}
            local msg = {ids = ids}
            self:disappear(msg)
        end        
    end,

    -- 刷新当前界面红点
    refreshRedPoints = function(self)
        local appear_points = {}
        local disappear_ids = {ids = {}}
        config.redpoint_redpoints:foreach(function(id, cfg)
            if self.red_points[id] then
                -- 有红点
                table.insert(appear_points, {id = id, count = self.red_points[id]})
            else
                -- 没红点
                table.insert(disappear_ids, id)
            end
        end)

        self:appear({points = appear_points})
        self:disappear({ids = disappear_ids})
    end,
}

red_point_manager = red_point_manager or red_point_manager_cls()

return red_point_manager