local log = logger.get("Game", "battle replay helper")

-- 战报播放辅助函数
battle_replay_helper = {
    -- 从战报中获取双方士兵总数量
    getSoldierTotalNums = function(self, report)
        local tmp = report.S0.total
        local t0_total = tmp[1] + tmp[2] + tmp[3] + tmp[4]
        tmp = report.S1.total
        local t1_total = tmp[1] + tmp[2] + tmp[3] + tmp[4]
        
        return t0_total, t1_total
    end,

    -- 士兵剩余数量
    getSoldierLeftNums = function(self, report)
        local tmp = report.S0.left
        local t0_left = tmp[1] + tmp[2] + tmp[3] + tmp[4]
        tmp = report.S1.left
        local t1_left = tmp[1] + tmp[2] + tmp[3] + tmp[4]
        
        return t0_left, t1_left
    end,

    -- 士兵半场数量
    getSoldierMidLeftNums = function(self, report)
        local tmp = report.S0.mid_left
        local t0_left = tmp[1] + tmp[2] + tmp[3] + tmp[4]
        tmp = report.S1.mid_left
        local t1_left = tmp[1] + tmp[2] + tmp[3] + tmp[4]
        
        return t0_left, t1_left
    end,

    -- 火炮总数量
    getCannonTotalNums = function(self, report)
        local tmp = report.Cannon.total
        return tmp[1] + tmp[2] + tmp[3]
    end,

    -- 火炮剩余数量
    getCannonLeftNums = function(self, report)
        local tmp = report.Cannon.left
        return tmp[1] + tmp[2] + tmp[3]
    end,

    -- 士兵阵亡数量
    getSoldierKilledNums = function(self, report, base_num)
        local T = report.S0.total
        local L = report.S0.left
        local k0 = {math.ceil((T[1]-L[1]) / base_num),
                    math.ceil((T[2]-L[2]) / base_num),
                    math.ceil((T[3]-L[3]) / base_num),
                    math.ceil((T[4]-L[4]) / base_num)}

        T = report.S1.total
        L = report.S1.left
        local k1 = {math.ceil((T[1]-L[1]) / base_num),
                    math.ceil((T[2]-L[2]) / base_num),
                    math.ceil((T[3]-L[3]) / base_num),
                    math.ceil((T[4]-L[4]) / base_num)}

        return {[0]=k0, [1]=k1}
    end,

    -- 士兵上半场阵亡数量
    getSoldierFirstPartKilledNums = function(self, report, base_num)
        local T = report.S0.total
        local L = report.S0.mid_left
        local k0 = {math.ceil((T[1]-L[1]) / base_num),
                    math.ceil((T[2]-L[2]) / base_num),
                    math.ceil((T[3]-L[3]) / base_num),
                    math.ceil((T[4]-L[4]) / base_num)}

        T = report.S1.total
        L = report.S1.mid_left
        local k1 = {math.ceil((T[1]-L[1]) / base_num),
                    math.ceil((T[2]-L[2]) / base_num),
                    math.ceil((T[3]-L[3]) / base_num),
                    math.ceil((T[4]-L[4]) / base_num)}

        return {[0]=k0, [1]=k1}
    end,

    -- 士兵下半场阵亡数量
    getSoldierSecondPartKilledNums = function(self, report, base_num)
        local T = report.S0.mid_left
        local L = report.S0.left
        local k0 = {math.ceil((T[1]-L[1]) / base_num),
                    math.ceil((T[2]-L[2]) / base_num),
                    math.ceil((T[3]-L[3]) / base_num),
                    math.ceil((T[4]-L[4]) / base_num)}

        T = report.S1.mid_left
        L = report.S1.left
        local k1 = {math.ceil((T[1]-L[1]) / base_num),
                    math.ceil((T[2]-L[2]) / base_num),
                    math.ceil((T[3]-L[3]) / base_num),
                    math.ceil((T[4]-L[4]) / base_num)}

        return {[0]=k0, [1]=k1}
    end,

    -- 战斗时长
    getBattleTime = function(self, report)
        -- 根据胜利程度判断战斗时长
        if report.win_degree == 1 then
            return 45
        else
            return 30
        end
    end,

    -- 根据4兵种数量安排阵型
    arrangeSoldierFormation = function(self, soldier_nums)
        -- 士兵布阵的结果
        local formation = {{0,0,0,0,0},{0,0,0,0,0},{0,0,0,0,0},{0,0,0,0,0}}
        -- 4 种兵放置行的优先级
        local search_order = {{2,1,3,4},{3,4,1,2},{4,3,2,1},{1,2,3,4}}  
            
        -- 遍历4种士兵
        for i=1, 4 do
            local num = soldier_nums[i] -- 需要分配的士兵数量
            
            local order = search_order[i]   
            for k=1, 4 do -- 遍历 order
                if num <= 0 then
                    break
                end

                local row = order[k]    -- 优先放置的行
                for column = 1,5 do
                    if num > 0 then
                        if formation[row][column] == 0 then
                            if num <= 5 then
                                formation[row][column] = {i, num}
                                    num = 0
                                break
                            else
                                formation[row][column] = {i, 5}
                                num = num - 5
                            end
                        end
                    else
                        break
                    end
                end
            end
        end

        return formation
    end,

    -- 根据士兵类型获取配置 id
    getSoldierCfgIdByType = function(self, type)
        if type == 1 then
            return 20004
        elseif type == 2 then
            return 20002
        elseif type == 3 then
            return 20003
        elseif type == 4 then
            return 20005
        else
            log.error("Failed to find SoldierCfgIdByType:"..tostring(type))
            return -1
        end
    end,

    -- 战车位置排布
    arrangeVehicleFormation = function(self, cfg_ids)
        local formation = {{}, {}, {}}  -- 3 排
        for k, id in ipairs(cfg_ids) do
            local cfg = config.tank_basic_ids[id]
            if not cfg then
                log.error("Failed to find tank cfg:"..id)
            else
                local tab = formation[cfg.position]
                if not tab then
                    log.error("### Wrong tank position:"..id)
                end
                table.insert(tab, id)                
            end

        end
        return formation
    end,

}