local log = logger.get("Game", "RTSReplayActor")
local trace = require('debug.trace')
local bullet_class = require('RTS.rts_bullet')
local list = require('base.list')
local super = require('RTS.rts_actor_base')
local actor_types = require('RTS.enums').actor_types
local actor_props = require('RTS.prop_sys.actor_props')
local buff_handler = require('RTS.buff.buff_handler')
local attachment_class = require('RTS.attachment.attachment')
local main_weapon_class = require('RTS.attachment.main_weapon')
require ('RTS.hero_sys.hero_manager')

-- 战车
return class {
    super = super,

    ctor = function(self, cfg_id, pos, rot, team, id, events, root)
        super.ctor(self, team, id)

        self.cfg_id = cfg_id or 1
        self.cfg = config.tank_basic_ids[cfg_id]

        -- 由服务器下发的信息 info
        self.level = 1 -- 坦克等级

        self.weapon_id = self.cfg.default_weapon_id        -- 主武器 id
        self.weapon_cfg = config.weapon_weapons[self.weapon_id]
        if not self.weapon_cfg then
            log.error("Failed to find weapon cfg:"..tostring(self.weapon_id)..", cfg_id:"..tostring(cfg_id))
        end
        self.wheel_id = self.cfg.initial_wheel_id or 0  -- 轮胎 id

        self.name = 'actor_'..team..'_'..id
        self.actor_type = actor_types.VEHICLE
        self.events = events
        self.cs_inst = nil  -- 对应的 cs 类
        self.animation = nil -- Animation 组件
        self.frame = 0
        self.active_vfx = {}    -- 播放中的特效 [路径] - 数量
        self.active_vfx_frame = {}  -- 该名称特效是在多少帧播放的

        -- cfg
        self.ai_name = self.cfg.PVP_ai or "tank"

        self.anim_names =       -- 动画名称配置 
        {
            idle = 'idle',
            move = 'run',
            stop = 'stop',
            attack = 'attack',
            hit = 'hitten',
            die = 'die',
            win = 'win',
        }   
        self.bullet_prefab = consts.assets.bullet
        self.radius = self.cfg.round
        self.height = self.cfg.scene_height

        -- 轮胎
        self.wheel = nil

        -- props
        self.hp = 0   
        self.energy = 0
        self.shield = 0 -- 护盾，受伤时优先减护盾值
        self.attack_range = self.weapon_cfg.max_atk_range + math.random(-self.weapon_cfg.random_range, self.weapon_cfg.random_range)      -- 最大攻击距离
        self.attack_range_sq = self.attack_range * self.attack_range    
        self.min_attack_range = self.weapon_cfg.min_atk_range   -- 最小攻击距离 
        self.safe_range = self.weapon_cfg.safe_range            -- 逆向敌人移动时，移到多远认为安全

        self.speed = self.cfg.speed
        self.acceleration = self.speed

        -- buff 状态
        self.is_sneak = false   -- 是否在潜行

         -- 容器
        self.bullets = list()
        self.beattacked_skill_ids = {}

        -- 血条显示时间
        self.hp_slider_time = 0.0

        -- 能量恢复时间
        self.energy_recover_elapsed = 0.0

        self.be_attack_count = 0
        self.pool = battle_helper.pool
        --log.info('atk_type:'..self.cfg.atk_type)
        --log.info(self.attack_cfg.start)
        -- {"肉盾型车体1号",1,300,1,1,1,3,4,"Assets/Arts/test/Prefab/kv.prefab","","","","","","", },
        -- ["tank_basic_ids"]={"name","type","hp","atk_type","position","round","width","height","prefab","point1","item1","point2","item2","point3","item3",},
        if self.cfg == nil then
            -- 找不到配置
            log.error('config.tank_basic_ids 中找不到战车配置:'..tostring(cfg_id))
        else
            --log.info('Begin create actor:'..self.cfg.prefab1)
            self:loadResource(pos, rot, 'tank_'..team.."_"..id, root)
        end
    end,

    -- 加载资源
    loadResource = function(self, pos, rot, name, root)
        local callback = function(go)
            -- callback
            if go ~= nil then
                go.name = name
                self.gameObject = go
                go.transform:SetParent(root)
                local scale = self.cfg.battle_scale * 0.0001
                go.transform.localScale = Vector3(scale, scale, scale)

                self:init()
                self.transform.position = pos
                self.transform.rotation = rot

                self.init_rot = rot -- 初始朝向
                self.events:dispatch("actor_loaded", self)
            else
                log.error('loadResource go is nil')
            end
        end

        local prefab_path = self.cfg.prefab1
        if self.team == 1 then
            prefab_path = self.cfg.prefab2
        end

        local uobject = self.pool:instantiate(prefab_path)
        if not uobject then
            log.error('Failed to load prefab:'..prefab_path)
            return nil
        end

        if uobject.isLoaded then 
            callback(uobject)
        else 
            uobject.onLoaded = callback
        end
    end,

    -- 初始化
    init = function(self)
        if self.is_initialized then return end

        -- 初始化 CS 对象
        self.cs_inst = self.gameObject:GetComponentInChildren('Tank')
        if not self.cs_inst then
            log.error("Failed to find Tank component in :"..self.cfg.prefab1)
            return
        end

        --self.cs_inst:SetMoveParam(self.speed, self.acceleration, 90, 1.0)
        --log.error("### actor:"..self.id..", set priority:"..self.cfg.avoid_priority)
       -- self.cs_inst:SetAvoidancePriority(self.cfg.avoid_priority)
        self.cs_inst.m_AttackRange = self.attack_range
        self.transform =  self.cs_inst.transform

        self.transform.localScale = Vector3.one

        self.animation = self.cs_inst.m_Animation
        self.cs_inst:SetAnimationLayer(self.anim_names.hit, 1)

        self.shoot_point = nil --self.cs_inst.m_ShootPoint -- 射击点
        -- 等入场完成后再调用 cs_inst:Init
        --self.cs_inst:Init(self.team, self.id)
        self.cs_inst.enabled = false

        if not self.level or self.level == 0 then
            self.level = 1
        end
        
        -- 初始化主武器
        self:initMainWeapon()

        -- 初始化轮胎

        self:initWheel()
        -- 初始化拖挂
        self:initTrailerBody()

        -- 初始化属性
        self:initProps()

        -- 初始化角色状态机
        self:initStates()

        -- 创建头顶信息
        self:initUI()

        -- 初始化 shadow blob
        self:initBlobShadow()

        -- buff 处理器
        self.buff_handler = buff_handler(self)
        -- 添加初始 buff
        if self.init_buffs then
            for _,buff in pairs(self.init_buffs) do
                self.buff_handler:addBuff(self, buff)
            end
        end
    end,

    initUI = function(self)
  
        -- 脚底光圈
        self.foot_circle = nil
        if self.team == 0 then
            self.foot_circle = battle_helper:loadRes(consts.assets.foot_circle_T0)
        else
            self.foot_circle = battle_helper:loadRes(consts.assets.foot_circle_T1)
        end
        self.foot_circle.transform.localScale = Vector3(self.cfg.round, self.cfg.round, self.cfg.round)
        self:bind("root", self.foot_circle)
        
    end,

    initBlobShadow = function(self)
        self.blob_shadow = battle_helper:loadRes(consts.assets.blob_shadow)
        local zoom = self.cfg.round
        cc = self.cs_inst:GetComponent(UnityEngine.CapsuleCollider)
        self.blob_shadow.transform.localScale = Vector3(zoom*cc.radius*2 ,zoom,zoom*(cc.height))
        self:bind("root", self.blob_shadow)
        local pos = self.blob_shadow.transform.position
        pos.y = 0.1
        self.blob_shadow.transform.position = pos
    end,

    -- 初始化主武器
    initMainWeapon = function(self)
        local attach_point = self.cs_inst:FindChild('z_wq')
        self.main_weapon = main_weapon_class(self, attach_point, self.weapon_id, function()
            self.is_initialized = true    
        end)
    end,

    -- 初始化轮胎
    initWheel = function(self)
        if self.wheel_id == 0 then return end
        local attach_point = "Wheel" --self.cs_inst:FindChild('lt_1')
        -- log.info('########### attach_point:'..tostring(attach_point)..', wheel_id:'..self.wheel_id)
        self.wheel = attachment_class(self, attach_point, self.wheel_id,
            function(obj)
                --local anim = obj:GetComponentInChildren(UnityEngine.Animation)
                --self.cs_inst:AddAnimationComp(anim)
                if self.main_weapon.is_initialized then
                    self.is_initialized = true
                end
            end
        )
    end,

    -- 初始化拖挂
    initTrailerBody = function(self)
        if self.cfg.relatetank and self.cfg.relatetank > 0 then
            self.taiiler_cfg = config.tank_basic_ids[self.cfg.relatetank]

            local prefab_path = self.taiiler_cfg.prefab1
            if self.team == 1 then
                prefab_path = self.taiiler_cfg.prefab2
            end

            local uobject = game_director.pool:instantiate(prefab_path)
            if not uobject then
                log.error('Failed to load TrailerBody:'..prefab_path)
                return
            end

            local callback = function(go)
                self.trailer_body_go = go
                self.trailer_body = go:GetComponentInChildren(TrailerBody)
                if not self.trailer_body then
                    log.error("Failed to find TrailerBody component")
                    return
                end
                if not self.trailer_body:ConnectTo(self.cs_inst) then
                    log.error("Failed to connect trailer, tank id:"..self.cfg_id)
                end
            end

            if uobject.isLoaded then 
                callback(uobject)
            else 
                uobject.onLoaded = callback
            end
        else
            --log.info("# No relatetank:"..tostring(self.cfg_relatetank))
        end
    end,


    -- 初始化角色所有系统的属性
    initProps = function(self)
        self.props = actor_props(self.cfg, self.level)

        if not self.init_props then
            -- 武器属性组
            local weapon_group = self.props:getGroup(prop_group_types.MAIN_WEAPON)
            weapon_group:copyFromPropTab(self.weapon_cfg.basic_attr)

            -- 累加计算最终属性
            self.props:computeStatic()
            self.props.static_group:settleDownStatic()
            self.props:computeFinal()

            self.max_hp = self.props:getProp(MAX_HP_IDX)
            --self.props:getProp(MAX_ENERGY_IDX)
            self.hp = self.max_hp
            self.energy = 0
        else
            --log.info('tank id:'..self.cfg_id)
            for k,v in ipairs(self.init_props) do
                self.props.static_group.props[v.PropId] = v.PropValue
                --log.info('prop['..v.PropId..'] = '..v.PropValue)
            end
            self.props.static_group:settleDownStatic()
            self.props:computeFinal()

            self.max_hp = self.props:getProp(MAX_HP_IDX)
            self.max_energy = self.props:getProp(MAX_ENERGY_IDX)
            self.hp = self.props:getProp(HP_IDX) --self.max_hp
            self.energy =  self.props:getProp(ENERGY_IDX)
            self.events:dispatch("actor_hp_change", self.id, self.hp, self.max_hp)

            --log.info('########## cfg_id:'..self.cfg_id..' hp:'..self.hp..'/'..self.max_hp)


            -- 和本地属性比较
            --[[
            local clientProp = actor_props(self.cfg, self.level)
            local weapon_group = clientProp:getGroup(prop_group_types.MAIN_WEAPON)
            --local weapon_prop = loadstring('return '..self.weapon_cfg.basic_attr)()
            weapon_group:copyFromPropTab(self.weapon_cfg.basic_attr)

            clientProp:computeStatic()
            clientProp:computeFinal()
            log.info('tank id:'..self.cfg_id..', level:'..self.level..', team:'..self.team)
            self.props.static_group:compair(clientProp.static_group)
            clientProp.base_group:log()
            weapon_group:log()
            clientProp.static_group:log()
            ]]
        end
        self.max_energy = self.cfg.MAX_ENERGY
        self.energy_recover = self.cfg.ENERGY_RECOVER * 5   -- 每秒恢复
    end,

    initStates = function(self)
        self.states['idle'] = require('RTS.actor_states.actor_state_idle').new('idle', self)
        self.states['move'] = require('RTS.actor_states.actor_state_move').new('move', self)
        --self.states['preskill'] = require('RTS.actor_states.actor_state_preskill').new('preskill', self)
        self.states['skill'] = require('RTS.actor_states.actor_state_skill').new('skill', self)
        self.states['die'] = require('RTS.actor_states.actor_state_die').new('die', self)
        self.states['entrance'] = require('RTS.actor_states.actor_state_entrance').new('entrance', self)
       -- self.states['hit_fly'] = require('RTS.actor_states.actor_state_hit_fly').new('hit_fly', self)
        self.states['win'] = require('RTS.actor_states.actor_state_win').new('win', self)
        
        self.skill_state = self.states['skill']
        self:changeState('entrance')

        --self.is_initialized = true
    end,

    -- 获取普通攻击技能 id，子类应该重写
    getCommonAtkID = function(self)
        return self.weapon_cfg.commonatk_id
    end,

    -- 获取技能 id, 子类应当重写
    getSkillID = function(self)
        return self.weapon_cfg.skill_id
    end,

    -- 是否可以释放技能
    canCastSkill = function(self)
        return self.energy >= self.max_energy
    end,

    update = function (self, dt)
        self.frame = self.frame + 1

        super.update(self, dt)
        self.bullets:update(dt)

        -- 更新主武器
        if self.main_weapon then
            self.main_weapon:update(dt)
        end

        --[[
        if self.team ~= 0 and self.head_info_go then
            if self.hp_slider_time > 0.0 then
                self.hp_slider_time = self.hp_slider_time - dt
                if self.hp_slider_time < 0.0 then 
                    self.head_info_go:SetActive(false)
                end
            end
        end]]

        if self.energy_recover > 0 then
            self.energy_recover_elapsed = self.energy_recover_elapsed + dt
            if self.energy_recover_elapsed > 1.0 then
                self.energy_recover_elapsed = 0.0
                self:addEnergy(self.energy_recover)
            end
        end
    end,

    -- 能量值改变
    addEnergy = function(self, value)
        self.energy = self.energy + value
        if self.energy > self.max_energy then
            self.energy = self.max_energy
        elseif self.energy < 0 then
            self.energy = 0
        end

        --log.info("# add energy:"..value..", energy:"..self.energy)
    end,

    beAttacked = function(self, attacker, skill_cfg, damage, is_hit, is_crit, cast_inst_id, from_replay, curr_hp)
        self.be_attack_count = self.be_attack_count + 1
    end,

    -- 判断攻击者所在方向
    getAttackAngle = function(self, attack_pos)
        local a = attack_pos - self.transform.position
        local b = self.transform.forward
        a:Normalize()
        local angle = Vector3.Angle(a, b)
        local axis = Vector3.Cross(b, a)

        if axis.y < 0 then
            angle = 360 - angle
        end
        return angle
    end,

    -- 血量变化
    addHP = function (self, hp, skill_type, is_crit, curr_hp)
       
    end,

    kill = function(self)
        super.kill(self)
        if self.head_info then
            self.head_info:SetHPSliderValue(0)
        end
        --log.info("#### kill:"..self.id..', hp:'..self.hp)
        self.events:dispatch("actor_hp_change", self.id, self.hp, self.max_hp)
    end,
}

