local log = logger.get("Game", "RTSReplayStage")
rts_helper = require('RTS.rts_helper')
battle_helper = require('RTS.battle_helper')
local list = require('base.list')
local actor_types = require('RTS.enums').actor_types
local battle_types = require('RTS.enums').battle_types
local soldier_group_class = require('RTS.rts_soldier_group')
local super = require('common.state_machine')

require('RTS.replay_stage.battle_replay_helper')
require('base.timer')

function math.round(value)
    value = tonumber(value) or 0
    return math.floor(value + 0.5)
end

function math.clamp(v, minValue, maxValue)  
    if v < minValue then
        return minValue
    end
    if( v > maxValue) then
        return maxValue
    end
    return v 
end

-- 世界地图战报战斗
-- 服务器计算战斗结果，客户端表现战斗过程
return class {
    inst = nil, -- 唯一实例

   -- extensions = { "RTS/stage/rts_stage+war_mgr.lua"}, 

    ctor = function (self, eType, args)
        -- 父类成员
        self.states = {}
        self.curr_state = nil
        self.result = nil

        -- 检查参数是否合法
        if eType ~= battle_types.REPLAY then 
            log.error("wrong battle type:"..tostring(eType))
            eType = battle_types.REPLAY
        end
        self.eType = eType

        -- 初始化容器
        self.actors = {}
        self.actor_guid = 1 -- actor 唯一 id

        -- 初始化状态变量
        self.paused = true
        self.actor_initialized = false

        self.gameObject = GameObject("scene")
        GameObject.DontDestroyOnLoad(self.gameObject);
        self.transform = self.gameObject.transform

        self.owner = args.owner -- game state battle replay
        local stage_id = args.stage_id
        self.battle_id = args.battle_id
        self.battle_report = args.battle_report

        if not self.battle_report then
            log.error("#### no battle report!!")
        end

        self.stage_cfg = gConfig.stages[stage_id]
        self.stage_cfg = self:cookStageCfg(self.stage_cfg)

        self.is_replay = true
        self.ai_world = require('RTS.ai.world').new()
        game_info.is_replay = self.is_replay

        self.ai_world:setPaused(true)
        self.battle_time = 0.0

        -- 按照生/死 和 受击次数排序的士兵
        self.sorted_soldiers = {[0] = {{},{},{},{}}, [1] = {{},{},{},{}}}
        -- 排序的战车
        self.sorted_vihicles = {[0] = {}, [1] = {}}

        rts_helper:init()
        battle_helper:init(self, function()
            self:init()
        end)

        cheat.stage = self

       -- self:ext_ctor()
        -- 注册事件处理函数
        self.events = require("base.event").new()

        -- 出生 actor 回调
        self.events:on('actor_spawned', function(actor)

            log.info("actor_spawned:"..actor.id..","..actor.name)
            self.actors[actor.id] = actor


            if actor.actor_type == actor_types.SOLDIER then
                local tab = self.sorted_soldiers[actor.team][actor.soldier_type]
                table.insert(tab, actor)
            end
        end)

        -- actor 资源加载完成回调
        self.events:on("actor_loaded", function (actor)
           if actor.actor_type == actor_types.SOLDIER then
               -- log.info("########## actor loaded:"..actor.id)
                actor.cs_inst:Init(actor.team, actor.id, actor.group_id, false)
                --self.level_manager:AddSoldier(actor.cs_inst, actor.group_id)
            end            
        end)

        -- actor 死亡回调
        self.events:on("actor_dead", function (actor)
            if actor.ai_name then
                self.ai_world:removeAI(actor)
            end
        end)

        -- actor 溃逃回调
        self.events:on("actor_escaped", function (actor)
        end)

    end,

    super = super,

    -- 初始化,由 owner 调用
    init = function(self)
        local go = GameObject.Find('LevelManager')
        if go ~= nil then 
            self.level_manager = go:GetComponent('LevelManager')    -- cs 组件，预先缓存关卡中的重要对象
            self.shadow_projector = self.level_manager.shadowProjector
            if self.shadow_projector then
                self.shadow_projector.gameObject:SetActive(false)
            end
            self.tank_root = self.level_manager.battleTankRoot
            self.soldier_root = self.level_manager.battleSoldierRoot

            if not self.soldier_root then
                log.error("## No Soldier root in level_manager!")
            end

            -- 显示界面
            self.gui = frame_manager:show('replay_battle', self)
        else
            log.error('无法在关卡中找到 LevelManager 对象')
        end

        -- 两队 士兵总数
        local n0, n1 = battle_replay_helper:getSoldierTotalNums(self.battle_report)
        self.soldier_total_num_0, self.soldier_total_num_1 = n0, n1   -- 两队的总士兵数量
        self.soldier_left_num_0, self.soldier_left_num_1 =  
                battle_replay_helper:getSoldierLeftNums(self.battle_report)
        -- 两队死亡总数
        self.soldier_kill_num_0, self.soldier_kill_num_1 = 
            self.soldier_total_num_0 - self.soldier_left_num_0, self.soldier_total_num_1 - self.soldier_left_num_1

        self.base_num = 1  -- 每个士兵代表的实际数量
        if n0 > n1 then
            base_num = math.ceil(n0 / 80)    -- 80
        else
            base_num = math.ceil(n1 / 80)
        end


        local t0 = self.battle_report.S0.total
        local t1 = self.battle_report.S1.total
        -- 两队士兵数量
        self.soldier_nums = {
            [0] = {math.ceil(t0[1] / base_num), math.ceil(t0[2] / base_num), math.ceil(t0[3] / base_num), math.ceil(t0[4] / base_num)},
            [1] = {math.ceil(t1[1] / base_num), math.ceil(t1[2] / base_num), math.ceil(t1[3] / base_num), math.ceil(t1[4] / base_num)}
        }
        
        -- 两队士兵死亡数量 按小兵类型分组
        self.killed_nums = battle_replay_helper:getSoldierKilledNums(self.battle_report, base_num)  
        -- 记录两队已经死亡数量
        self.killed_nums_record = {[0]={0,0,0,0},[1]={0,0,0,0}}
        
        log.info("soldier t0 nums:"..self.soldier_nums[0][1]..', '..self.soldier_nums[0][2]..', '..self.soldier_nums[0][3]..', '..self.soldier_nums[0][4])
        log.info("soldier t1 nums:"..self.soldier_nums[1][1]..', '..self.soldier_nums[1][2]..', '..self.soldier_nums[1][3]..', '..self.soldier_nums[1][4])

        log.info("kill t0 nums:"..self.killed_nums[0][1]..', '..self.killed_nums[0][2]..', '..self.killed_nums[0][3]..', '..self.killed_nums[0][4])
        log.info("kill t1 nums:"..self.killed_nums[1][1]..', '..self.killed_nums[1][2]..', '..self.killed_nums[1][3]..', '..self.killed_nums[1][4])

        -- 两队死亡比例
        self.killed_percent = {[0]=0, [1]=0}
        self.killed_percent[0] = 
            (self.killed_nums[0][1] + self.killed_nums[0][2] + self.killed_nums[0][3] + self.killed_nums[0][4]) / 
            (self.soldier_nums[0][1] + self.soldier_nums[0][2] + self.soldier_nums[0][3] + self.soldier_nums[0][4]);
        self.killed_percent[1] = 
            (self.killed_nums[1][1] + self.killed_nums[1][2] + self.killed_nums[1][3] + self.killed_nums[1][4]) / 
            (self.soldier_nums[1][1] + self.soldier_nums[1][2] + self.soldier_nums[1][3] + self.soldier_nums[1][4]);
        self.killed_percent[0] = math.clamp(self.killed_percent[0], 0, 1)
        self.killed_percent[1] = math.clamp(self.killed_percent[1], 0, 1)

        log.info("### t0 kill percent:"..self.killed_percent[0])
        log.info("### t1 kill percent:"..self.killed_percent[1]) 

        local vehicle_num_t0 = #(self.battle_report.T0.cfg_id)
        local vehicle_num_t1 = #(self.battle_report.T1.cfg_id)
        -- 两队战车死亡数量
        self.killed_vehicle_nums = {
            [0] = math.ceil(vehicle_num_t0 * self.killed_percent[0]), 
            [1] = math.ceil(vehicle_num_t1 * self.killed_percent[1])}
        --记录已经死亡战车数量
        self.killed_vehicle_nums_record = {[0]=0, [1]=0}

        log.info("### t0 kill vehicle:"..self.killed_vehicle_nums[0])
        log.info("### t1 kill vehicle:"..self.killed_vehicle_nums[1]) 

        self.turn_time = 0.5
        -- 总回合数
        self.turn_num = self.battle_report.turns --math.ceil(self.battle_duration / 1) 
        self.time_count_down = self.turn_num * self.turn_time

        -- 战斗时长
        self.battle_duration = self.battle_report.turns * self.turn_time
        --battle_replay_helper:getBattleTime(self.battle_report)

        self.elapsed = 0.0          -- 经过的时间
        self.turn = 0               -- 当前回合

           -- 出生士兵
        self:spawnSoldiers()
        -- 出生战车
        self:spawnVehicles()

        self.is_inited = true
        self.is_entrance = false -- 是否在入场
        self.is_battle_finished = false  -- 是否分出了胜负，失败方溃逃
    end,

    genActorGuid = function(self)
        self.actor_guid = self.actor_guid + 1
        return self.actor_guid
    end,

    -- 出生士兵
    spawnSoldiers = function(self, root0, root1)
        -- 安排士兵阵型
        self.formation_S0 = battle_replay_helper:arrangeSoldierFormation(self.soldier_nums[0])
        self.formation_S1 = battle_replay_helper:arrangeSoldierFormation(self.soldier_nums[1])

        local offset_tab = self.stage_cfg.soldier_offset_tab
        local offset_random = self.stage_cfg.offset_random

        -- 根据阵型和配置出生士兵
        local spawnSoldier4Team = function(formation, cfg, team, root)
            for i=1,4 do        -- 行
                for j=1,5 do    -- 列
                    local s = formation[i][j]   -- group
                    if s ~= 0 then
                        local type = s[1]   -- 士兵类型
                        local num = s[2]    -- 士兵数量
                        local id = self:genActorGuid()
                        local cfg_id = battle_replay_helper:getSoldierCfgIdByType(type)

                        local soldier_group = soldier_group_class(cfg_id, team, id, self.events)
                        soldier_group.level = 1
                        soldier_group.init_props = nil

                        local id_tab = {}
                        for k=1,num do
                            local id = self:genActorGuid()
                            table.insert(id_tab, id)
                        end
                        local p = cfg[i].pos[j]
                        local pos = Vector3(p[1], 0, p[2])

                        if root then
                            pos = pos + root.position
                        end

                        local rot =  Quaternion.AngleAxis(cfg[i].yaw, Vector3.up) 

                        soldier_group:spawnSoldiers(cfg_id, pos, rot, offset_tab, offset_random, num, id_tab, root or self.soldier_root, type)
                        
                        self.events:dispatch("actor_spawned", soldier_group)
                    end
                end
            end
        end

        spawnSoldier4Team(self.formation_S0, self.stage_cfg.S0, 0, root0)
        spawnSoldier4Team(self.formation_S1, self.stage_cfg.S1, 1, root1)

        --[[
        -- 检查 formation
        log.info("## soldier formation")
        local str = ""
        for i=1,4 do
            for j=1,5 do
                local s = self.formation_S0[i][j]
                if s == 0 then
                    str = str .. '0,'
                else
                    str = str..'['..s[1]..','..s[2]..'],'
                end
            end
            str = str .. '\r\n'
        end

        log.info(str)

        str = ""
        for i=1,4 do
            for j=1,5 do
                local s = self.formation_S1[i][j]
                if s == 0 then
                    str = str .. '0,'
                else
                    str = str..'['..s[1]..','..s[2]..'],'
                end
            end
            str = str .. '\r\n'
        end
        log.info(str)
        ]]
    end,

    -- 出生战车
    spawnVehicles = function(self, root0, root1)
        self.formation_T0 = battle_replay_helper:arrangeVehicleFormation(self.battle_report.T0.cfg_id)
        self.formation_T1 = battle_replay_helper:arrangeVehicleFormation(self.battle_report.T1.cfg_id)

        -- 出生某 team 的战车
        local spawnVehicle4Team = function(formation, cfg, team, root)
            local vehicle_class = require("RTS.replay_stage.rts_replay_actor")
            for i=1,3 do        -- 遍历 3 行
                for j=1,5 do    -- 遍历 5 列
                    local tank_cfg_id = formation[i][j]
                    if tank_cfg_id then
                        -- 出生这个坦克
                        local p = cfg[i].pos[j]
                        local pos = Vector3(p[1], 0, p[2])
                        if root then
                            pos = pos + root.position
                        end

                        local rot =  Quaternion.AngleAxis(cfg[i].yaw, Vector3.up) 
                        local id = self:genActorGuid()

                        local actor = vehicle_class(tank_cfg_id, pos, rot, team, id, self.events, root or self.tank_root)
                        self.events:dispatch("actor_spawned", actor)

                        table.insert(self.sorted_vihicles[team], actor)
                    end
                end
            end
        end

        -- 检查 formation
        --[[
        log.info("## tank formation")
        local str = ''
        for i=1,3 do        -- 遍历 3 行
            for j=1,5 do    -- 遍历 5 列
                local tank_cfg_id = self.formation_T0[i][j]
                if tank_cfg_id then
                    str = str..tank_cfg_id..','
                else
                    str = str .. 0 ..","
                end
            end
            str = str.."\r\n"
        end

        log.info(str)
        str = ''
        for i=1,3 do        -- 遍历 3 行
            for j=1,5 do    -- 遍历 5 列
                local tank_cfg_id = self.formation_T1[i][j]
                if tank_cfg_id then
                    str = str..tank_cfg_id..','
                else
                    str = str .. 0 .. ","
                end
            end
            str = str.."\r\n"
        end
        log.info(str)
        ]]
        spawnVehicle4Team(self.formation_T0, self.stage_cfg.T0, 0, root0)
        spawnVehicle4Team(self.formation_T1, self.stage_cfg.T1, 1, root1)
    end,

    -- 根据服务器的战斗数据更新 stage 配置
    cookStageCfg = function(self, orig, eType)
        return orig;
    end,
    
    isShipBattle = function(self)
        return false
    end,

    getSoldierTotalNums = function(self)
        return battle_replay_helper:getSoldierTotalNums(self.battle_report)
    end,

    setPaused = function(self, paused)
        self.paused = paused
        self.ai_world:setPaused(paused)
    end,

    -- 是否所有 actor 都初始化完成
    isActorsInitialized = function(self)
        if self.actor_initialized then
            return self.actor_initialized
        end

        local inited = true
        for k,v in pairs(self.actors) do
            if not v.is_initialized then
                inited = false
                break
            end
        end

        return inited
    end,


    -- 开始进场
    beginEntrance = function(self)
        if self.is_entrance then
            return nil
        end

        local v0 = self.stage_cfg.vectorT0
        local v1 = self.stage_cfg.vectorT1
        v0 = Vector3.New(v0[1], v0[2], v0[3])
        v1 = Vector3.New(v1[1], v1[2], v1[3])
        local s0 = self.stage_cfg.speedT0
        local s1 = self.stage_cfg.speedT1

        log.info("#### v0:"..tostring(v0)..", v1:"..tostring(v1)..", s0:"..s0..", s1:"..s1)
        self.entrance_total_time = v0.magnitude / s0
        if self.entrance_total_time <= 0.01 then
            return
        end

        self.is_entrance = true
        self.elapsed = 0.0
        for _,actor in pairs(self.actors) do
            if actor.actor_type == actor_types.SOLDIER or actor.actor_type == actor_types.VEHICLE then
                if actor.team == 0 then
                    actor:changeState('entrance', {offset=v0, speed=s0})
                else 
                    actor:changeState('entrance', {offset=v1, speed=s1})
                end
            end
        end

        return self.entrance_total_time
    end,

    -- 进场结束
    endEntrance = function(self)
        for _,actor in pairs(self.actors) do
            if actor.actor_type == actor_types.SOLDIER or actor.actor_type == actor_types.VEHICLE then
                actor:changeState('idle')
            end
        end
        self.is_entrance = false
        self.elapsed = 0.0
        self:setPaused(false)
    end,

    -- 创建所有 actor AI
    createActorsAI = function(self, team)
        for _,actor in pairs(self.actors) do
            if actor.team == team and actor.ai_name and actor.actor_type ~= actor_types.CANNON then
                --log.info("## createActorAI:"..actor.name..", ai:"..actor.ai_name)
                self.ai_world:createAI(actor)
            elseif not actor.ai_name then
                --log.info("## Actor no ai:"..actor.name)
            end
        end
    end,

    update = function(self, dt)
        if not self.is_inited then
            return
        end
        -- 等待所有士兵初始化完成
        if not self.actor_initialized then
            if self:isActorsInitialized() then
                self.actor_initialized = true
                --self:setPaused(false)
                
                frame_manager:hide('loading')
                self:createActorsAI(0)
                self:createActorsAI(1)

                self:beginEntrance()
            else
                return
            end
        end
        self.elapsed = self.elapsed + dt

        -- 进场
        if self.is_entrance then
            for k,v in pairs(self.actors) do
                v:update(dt)
            end

            if self.elapsed >= self.entrance_total_time then
                self:endEntrance()

                local randMove = self.stage_cfg.randomMove
                local f0 = self.stage_cfg.fixedMoveT0
                local f1 = self.stage_cfg.fixedMoveT1
                local fixedMoveT0 = Vector3(f0[1], f0[2], f0[3])
                local fixedMoveT1 = Vector3(f1[1], f1[2], f1[3])
                self.level_manager:BeginSoldierRandomMove(
                    randMove.moveTime[1], randMove.moveTime[2],
                    randMove.stopTime[1], randMove.stopTime[2],
                    randMove.speed[1], randMove.speed[2],
                    fixedMoveT0, fixedMoveT1)
            end

            return
        end

        -- 战斗结束，失败方溃逃
        if self.is_battle_finished then
            for k,v in pairs(self.actors) do
                v:update(dt)
            end
            --[[
            if self.elapsed > 5.0 then
                game_director:changeState('loading', {next_state = 'home', scene_name = 'home'})
            end]]
            return
        end

        self.time_count_down = self.time_count_down - dt
        if self.time_count_down < 0 then
            self.time_count_down = 0
        end    

        if self.elapsed > self.turn * self.turn_time then
            self:updateTurn()
        end

        for k,v in pairs(self.actors) do
             v:preUpdate(dt)
        end
        
        if self.ai_world then
            self.ai_world:update(dt)
        end

        battle_helper:update(dt)

        for k,v in pairs(self.actors) do
            v:update(dt)
        end

        if self.turn > self.turn_num then
            -- 战斗结束
            --game_director:changeState('loading', {next_state = 'home', scene_name = 'home'})
            self:battleFinish()
        end
    end,

    -- 战斗结束
    battleFinish = function(self) 
        log.info("### battleFinished!")
        --log.info(debug.traceback())

        self.is_battle_finished = true
        self.elapsed = 0.0

        local win_team = self.battle_report.win_team
        local lose_team = 1
        if win_team == 1 then lose_team = 0 end
        self.level_manager:SoldierEscape(lose_team)

        for k, group in pairs(self.actors) do
            if group.actor_type == actor_types.SOLDIER_GROUP then
                if group.team == win_team then
                    --log.info("##### group:changeState('win')")
                    group:changeState('win')   -- 原地欢呼
                else
                    group:changeState('escape') -- 溃逃
                end
            end
        end

        local killed_0 = self.soldier_total_num_0 - self.soldier_left_num_0
        local killed_1 = self.soldier_total_num_1 - self.soldier_left_num_1
        app:startCoroutine(
            function()
                coroutine.yield(WaitForSeconds(1.5))
                self.events:dispatch("battle_finished", self.battle_report.win_team == 0, killed_0, killed_1)
            end
        )


    end,

    -- 回合更新
    updateTurn = function(self)

        --log.error("############### turn:"..self.turn)
        -- 士兵死亡
        local sorter = function(a, b)
            if a.is_alive and b.is_alive then
                return a.be_attack_count > b.be_attack_count
            elseif a.is_alive then
                return true
            elseif b.is_alive then
                return false
            else
                return false
            end
        end
        
        table.sort(self.sorted_soldiers[0][1], sorter)
        table.sort(self.sorted_soldiers[0][2], sorter)
        table.sort(self.sorted_soldiers[0][3], sorter)
        table.sort(self.sorted_soldiers[0][4], sorter)
        table.sort(self.sorted_soldiers[1][1], sorter)
        table.sort(self.sorted_soldiers[1][2], sorter)
        table.sort(self.sorted_soldiers[1][3], sorter)
        table.sort(self.sorted_soldiers[1][4], sorter)

        local percent = self.turn / self.turn_num   -- 到当前回合的死亡比例
        percent = math.clamp(percent, 0, 1)

        -- 杀本回合的士兵
        local killSoldier = function(team)
            --local ids = self.soldier_ids[team]
            for i=1,4 do    -- 遍历 4 种士兵
                local should_kill_nums = math.floor(self.killed_nums[team][i] * percent)   -- 到本回合，应该死掉的数量
                local turn_kill = should_kill_nums - self.killed_nums_record[team][i]   -- 本回合应该杀掉的

                local tab = self.sorted_soldiers[team][i]
                for i=1,turn_kill do
                    local actor = tab[i]
                    if actor and actor:isAlive() then
                        actor:kill()
                    end
                end

                self.killed_nums_record[team][i] = should_kill_nums
            end
        end

        -- 杀本回合的战车
        local killVehicle = function(team)
            local should_kill_nums = math.floor(self.killed_vehicle_nums[team] * percent)
            local turn_kill = should_kill_nums - self.killed_vehicle_nums_record[team]
            if turn_kill > 0 then
                local tab = self.sorted_vihicles[team] 
                table.sort(tab, sorter)

                for i=1, turn_kill do
                    local actor = tab[i]
                    if actor and actor:isAlive() then
                        app:startCoroutine(
                            function()
                                coroutine.yield(1.0)
                                actor:kill()                                
                            end
                        )
                        actor:kill()
                    end
                end

                self.killed_vehicle_nums_record[team] = should_kill_nums
            end
        end

        -- 战车放技能
        vehicleCastSkill = function()
            local caster = nil
            for _,actor in pairs(self.actors) do
                if actor.actor_type == actor_types.VEHICLE and actor:isAlive() then
                    if actor:canCastSkill() and actor.target_actor then
                        caster = actor
                        log.info("### can cast skill:"..actor.id)
                        break
                    end
                end
            end

            if caster then
                local rst = caster:ai_castUltimateSkill(caster.target_actor)

                local id = caster:getSkillID()
                --log.info("##### cast skill rst:"..tostring(rst)..", id:"..id)                
                local skill_cfg = config.skill_skills[id]
                if skill_cfg.skip_turn > 1 then
                    self.turn = self.turn + skill_cfg.skip_turn - 1
                end
            end
        end

        killSoldier(0)
        killSoldier(1)

        killVehicle(0)
        killVehicle(1)

        vehicleCastSkill()

        self:updateTurnUI(percent)

        self.turn = self.turn + 1
    end,

    -- 每回合更新界面
    updateTurnUI = function(self, percent)
        local left_soldier_0 = self.soldier_total_num_0 - self.soldier_kill_num_0*percent
        local left_soldier_1 = self.soldier_total_num_1 - self.soldier_kill_num_1*percent
        local p0 = left_soldier_0 / self.soldier_total_num_0
        local p1 = left_soldier_1 / self.soldier_total_num_1

        --self.events:dispatch("update_turn", self.turn, self.turn_num, left_soldier_0, left_soldier_1, p0, p1, self.turn_time)
        self.events:dispatch("update_turn", self.turn, 30, left_soldier_0, left_soldier_1, p0, p1, self.turn_time, math.ceil(self.time_count_down))
    end,

    -- 战斗是否结束
    isFinished = function(self)
        
        return false
    end,

    destroy = function(self)
        frame_manager:hide('replay_battle')
        for _,actor in pairs(self.actors) do
            actor:destroy()
        end
        self.actors = nil

        battle_helper:destroy()
        Object.DestroyImmediate(self.gameObject)
        
        if self.ai_world then
            self.ai_world:destroy()
        end
    end,
}