local super = require ('RTS.replay_stage.rts_replay_stage')
local list = require('base.list')
local actor_types = require('RTS.enums').actor_types
local battle_types = require('RTS.enums').battle_types
local soldier_group_class = require('RTS.rts_soldier_group')
local log = logger.get("Game", "Ship Battle replay stage")
local state_machine = require("common.state_machine")
return class
{
    inst = nil,     -- 唯一实例
    super = super,


    -- 第一阶段初始化
    initFirstPart = function(self)
        -- 两队 士兵总数
        local n0, n1 = battle_replay_helper:getSoldierTotalNums(self.battle_report)
        self.soldier_total_num_0, self.soldier_total_num_1 = n0, n1   -- 两队的总士兵数量
        self.soldier_left_num_0, self.soldier_left_num_1 =  
                battle_replay_helper:getSoldierMidLeftNums(self.battle_report)  -- 半场士兵人数

        -- 两队死亡总数
        self.soldier_kill_num_0, self.soldier_kill_num_1 = 
            self.soldier_total_num_0 - self.soldier_left_num_0, self.soldier_total_num_1 - self.soldier_left_num_1

        self.base_num = 1  -- 每个士兵代表的实际数量
        if n0 > n1 then
            base_num = math.ceil(n0 / 80)    -- 80
        else
            base_num = math.ceil(n1 / 80)
        end
        self.base_num = base_num    -- 每个表现士兵代表真实士兵数量

        -- 真实士兵数量
        local t0 = self.battle_report.S0.total  
        local t1 = self.battle_report.S1.total
        -- 两队表现士兵数量
        self.soldier_nums = {
            [0] = {math.ceil(t0[1] / base_num), math.ceil(t0[2] / base_num), math.ceil(t0[3] / base_num), math.ceil(t0[4] / base_num)},
            [1] = {math.ceil(t1[1] / base_num), math.ceil(t1[2] / base_num), math.ceil(t1[3] / base_num), math.ceil(t1[4] / base_num)}
        }
        
        -- 两队士兵死亡数量 按小兵类型分组
        self.killed_nums = battle_replay_helper:getSoldierFirstPartKilledNums(self.battle_report, base_num)  
        -- 记录两队已经死亡数量
        self.killed_nums_record = {[0]={0,0,0,0},[1]={0,0,0,0}}
        
        log.info("soldier t0 nums:"..self.soldier_nums[0][1]..', '..self.soldier_nums[0][2]..', '..self.soldier_nums[0][3]..', '..self.soldier_nums[0][4])
        log.info("soldier t1 nums:"..self.soldier_nums[1][1]..', '..self.soldier_nums[1][2]..', '..self.soldier_nums[1][3]..', '..self.soldier_nums[1][4])

        log.info("kill t0 nums:"..self.killed_nums[0][1]..', '..self.killed_nums[0][2]..', '..self.killed_nums[0][3]..', '..self.killed_nums[0][4])
        log.info("kill t1 nums:"..self.killed_nums[1][1]..', '..self.killed_nums[1][2]..', '..self.killed_nums[1][3]..', '..self.killed_nums[1][4])

        -- 两队死亡比例
        self.killed_percent = {[0]=0, [1]=0}
        self.killed_percent[0] = 
            (self.killed_nums[0][1] + self.killed_nums[0][2] + self.killed_nums[0][3] + self.killed_nums[0][4]) / 
            (self.soldier_nums[0][1] + self.soldier_nums[0][2] + self.soldier_nums[0][3] + self.soldier_nums[0][4]);
        self.killed_percent[1] = 
            (self.killed_nums[1][1] + self.killed_nums[1][2] + self.killed_nums[1][3] + self.killed_nums[1][4]) / 
            (self.soldier_nums[1][1] + self.soldier_nums[1][2] + self.soldier_nums[1][3] + self.soldier_nums[1][4]);
        self.killed_percent[0] = math.clamp(self.killed_percent[0], 0, 1)
        self.killed_percent[1] = math.clamp(self.killed_percent[1], 0, 1)

        log.info("### t0 kill percent:"..self.killed_percent[0])
        log.info("### t1 kill percent:"..self.killed_percent[1]) 

        local vehicle_num_t0 = #(self.battle_report.T0.cfg_id)
        local vehicle_num_t1 = #(self.battle_report.T1.cfg_id)
        -- 两队战车死亡数量
        self.killed_vehicle_nums = {
            [0] = math.floor(vehicle_num_t0 * self.killed_percent[0]), 
            [1] = math.floor(vehicle_num_t1 * self.killed_percent[1])}
        --记录已经死亡战车数量
        self.killed_vehicle_nums_record = {[0]=0, [1]=0}
        log.info("### t0 kill vehicle:"..self.killed_vehicle_nums[0])
        log.info("### t1 kill vehicle:"..self.killed_vehicle_nums[1]) 

        -- 防御方火炮数量
        local tmp = self.battle_report.Cannon.total
        local max_cannon_num = math.max(math.max(tmp[1], tmp[2]), tmp[3])
        self.cannon_base_num = math.ceil( max_cannon_num / 5)   -- 表现上一个炮台代表的数量   5-每种炮台最多5个

        if self.cannon_base_num > 0 then
            -- 炮台的表现数量
            self.cannon_nums = {math.floor(tmp[1]/self.cannon_base_num),    
                                math.floor(tmp[2]/self.cannon_base_num), 
                                math.floor(tmp[3]/self.cannon_base_num)}
                                
            local tmp = self.battle_report.Cannon.left      
            -- 炮台的剩余表现数量
            self.cannon_left_nums = {math.floor(tmp[1]/self.cannon_base_num),   
                                math.floor(tmp[2]/self.cannon_base_num), 
                                math.floor(tmp[3]/self.cannon_base_num)}
        else
            self.cannon_nums = {0,0,0}
            self.cannon_left_nums = {0,0,0}
        end

        -- 炮台损失数量
        self.killed_cannon_nums = {self.cannon_nums[1]-self.cannon_left_nums[1], 
                                    self.cannon_nums[2]-self.cannon_left_nums[2],
                                    self.cannon_nums[3]-self.cannon_left_nums[3],}

        self.killed_cannon_sum = self.killed_cannon_nums[1] + self.killed_cannon_nums[2] + self.killed_cannon_nums[3]
        self.killed_cannon_nums_record = {self.killed_cannon_nums[1],self.killed_cannon_nums[2],self.killed_cannon_nums[3]}
        self.killed_cannon_sum_record = 0

        log.info("kill cannon num:"..self.killed_cannon_nums[1]..", "..self.killed_cannon_nums[2]..", "..self.killed_cannon_nums[3])
  
        self.turn_time = 0.5
        -- 总回合数
        self.turn_num = self.battle_report.break_wall_turns --math.ceil(self.battle_duration / 1) 
        self.turn = 0

        -- 战斗时长
        self.battle_duration = self.turn_num * self.turn_time

        self.time_count_down = (self.turn_num  + self.battle_report.occupy_turns) * self.turn_time

        log.info("### tuan_num:"..self.turn_num..", duration:"..self.battle_duration)
    end,

    -- 第二阶段初始化
    initSecondPart = function(self)
        -- 两队 士兵总数
        local n0, n1 = battle_replay_helper:getSoldierTotalNums(self.battle_report)
        self.soldier_total_num_0, self.soldier_total_num_1 = n0, n1   -- 两队的总士兵数量
        self.soldier_left_num_0, self.soldier_left_num_1 =  
                battle_replay_helper:getSoldierLeftNums(self.battle_report)
        -- 中场剩余兵力
        self.soldier_mid_left_num_0, self.soldier_mid_left_num_1 = 
                battle_replay_helper:getSoldierMidLeftNums(self.battle_report)

        -- 两队死亡总数
        self.soldier_kill_num_0, self.soldier_kill_num_1 = 
            self.soldier_mid_left_num_0 - self.soldier_left_num_0, self.soldier_mid_left_num_1 - self.soldier_left_num_1
        
        base_num = self.base_num

        local t0 = self.battle_report.S0.total
        local t1 = self.battle_report.S1.total
        -- 两队士兵数量
        self.soldier_nums = {
            [0] = {math.ceil(t0[1] / base_num), math.ceil(t0[2] / base_num), math.ceil(t0[3] / base_num), math.ceil(t0[4] / base_num)},
            [1] = {math.ceil(t1[1] / base_num), math.ceil(t1[2] / base_num), math.ceil(t1[3] / base_num), math.ceil(t1[4] / base_num)}
        }
        
        -- 两队士兵死亡数量 按小兵类型分组
        self.killed_nums = battle_replay_helper:getSoldierKilledNums(self.battle_report, base_num)  
        -- 记录两队已经死亡数量
        self.killed_nums_record = {[0]={0,0,0,0},[1]={0,0,0,0}}
        
        log.info("soldier t0 nums:"..self.soldier_nums[0][1]..', '..self.soldier_nums[0][2]..', '..self.soldier_nums[0][3]..', '..self.soldier_nums[0][4])
        log.info("soldier t1 nums:"..self.soldier_nums[1][1]..', '..self.soldier_nums[1][2]..', '..self.soldier_nums[1][3]..', '..self.soldier_nums[1][4])

        log.info("kill t0 nums:"..self.killed_nums[0][1]..', '..self.killed_nums[0][2]..', '..self.killed_nums[0][3]..', '..self.killed_nums[0][4])
        log.info("kill t1 nums:"..self.killed_nums[1][1]..', '..self.killed_nums[1][2]..', '..self.killed_nums[1][3]..', '..self.killed_nums[1][4])

        -- 两队死亡比例
        self.killed_percent = {[0]=0, [1]=0}
        self.killed_percent[0] = 
            (self.killed_nums[0][1] + self.killed_nums[0][2] + self.killed_nums[0][3] + self.killed_nums[0][4]) / 
            (self.soldier_nums[0][1] + self.soldier_nums[0][2] + self.soldier_nums[0][3] + self.soldier_nums[0][4]);
        self.killed_percent[1] = 
            (self.killed_nums[1][1] + self.killed_nums[1][2] + self.killed_nums[1][3] + self.killed_nums[1][4]) / 
            (self.soldier_nums[1][1] + self.soldier_nums[1][2] + self.soldier_nums[1][3] + self.soldier_nums[1][4]);
        self.killed_percent[0] = math.clamp(self.killed_percent[0], 0, 1)
        self.killed_percent[1] = math.clamp(self.killed_percent[1], 0, 1)

        log.info("### t0 kill percent:"..self.killed_percent[0])
        log.info("### t1 kill percent:"..self.killed_percent[1]) 

        local vehicle_num_t0 = #(self.battle_report.T0.cfg_id)
        local vehicle_num_t1 = #(self.battle_report.T1.cfg_id)
        -- 两队战车死亡数量
        self.killed_vehicle_nums = {
            [0] = math.round(vehicle_num_t0 * self.killed_percent[0]), 
            [1] = math.round(vehicle_num_t1 * self.killed_percent[1])}
        --记录已经死亡战车数量
        self.killed_vehicle_nums_record = {[0]=0, [1]=0}

        log.info("### t0 kill vehicle:"..self.killed_vehicle_nums[0])
        log.info("### t1 kill vehicle:"..self.killed_vehicle_nums[1]) 

        self.turn_time = 0.5
        -- 总回合数
        self.turn_num = self.battle_report.occupy_turns --math.ceil(self.battle_duration / 1) 

        -- 战斗时长
        self.battle_duration = self.battle_report.occupy_turns * self.turn_time
        --battle_replay_helper:getBattleTime(self.battle_report)

        self.elapsed = 0.0          -- 经过的时间
        self.turn = 0               -- 当前回合
        self.time_count_down = self.turn_num * self.turn_time  -- 倒计时
    end,

    -- 初始化,由 owner 调用
    init = function(self)
        local go = GameObject.Find('LevelManager')
        if go ~= nil then 
            self.level_manager = go:GetComponent('ShipBattleLevelManager')    -- cs 组件，预先缓存关卡中的重要对象
            self.shadow_projector = self.level_manager.shadowProjector
            if self.shadow_projector then
                self.shadow_projector.gameObject:SetActive(false)
            end
            self.root0 = self.level_manager.root0
            self.root1 = self.level_manager.root1

            if not self.root0 or not self.root1 then
                log.error("init failed!!, root0 or root1 is nil!!")
            end

            -- 显示界面
            self.gui = frame_manager:show('replay_battle', self)
        else
            log.error('无法在关卡中找到 LevelManager 对象')
        end

        -- 排序的炮台
        self.sorted_cannons = {}

        -- 初始化状态机
        self:initStates()

        -- 第一阶段初始化
        self:initFirstPart()
        --battle_replay_helper:getBattleTime(self.battle_report)

        self.elapsed = 0.0          -- 经过的时间
        self.turn = 0               -- 当前回合

        -- 出生士兵
        self:spawnSoldiers(self.root0, self.root1)
        -- 出生战车
        self:spawnVehicles(self.root0, self.root1)
        -- 出生炮台
        self:spawnCannons(self.level_manager.cannon1)

        self.is_inited = true
        self.is_battle_finished = false  -- 是否分出了胜负，失败方溃逃
    end,

    initStates = function(self)
        self.states['approach'] = require('RTS.replay_stage.rts_ship_battle_state_approach').new('approach', self);
        self.states['break_wall'] = require('RTS.replay_stage.rts_ship_battle_state_break_wall').new('break_wall', self);
        self.states['landing_boat'] = require('RTS.replay_stage.rts_ship_battle_state_landing_boat').new('landing_boat', self);
        self.states['occupy'] = require('RTS.replay_stage.rts_ship_battle_state_occupy').new('occupy', self); 
        self.states['finished'] = require('RTS.replay_stage.rts_ship_battle_state_finished').new('finished', self);
    end,

    -- 出生炮台
    spawnCannons = function(self, root)
          -- 出生某 team 的战车
        local spawnCannon4Type = function(type, team)
            local cannon_class = require("RTS.rts_cannon")
            local num = self.cannon_nums[type]
            local cfg_id = self.battle_report.Cannon.cfg_id[type]
            if num == 0 or cfg_id == 0 then
                return
            end
            local cfg = config.tank_basic_ids[cfg_id]
            if not cfg then
                log.error("## Failed to find cannon cfg:"..tostring(cfg_id))
            end
            log.info("## Spawn cannon cfg:"..cfg_id..", num:"..num)
            for i=1, num do
                local root_name = self.stage_cfg.C1[type].pos[i]
                log.info("## spawn cannon:"..root_name)

                local base_root = root:Find(root_name)

                if base_root then
                    local pos = base_root.position
                    local rot = base_root.rotation

                    local id = self:genActorGuid()

                    local actor = cannon_class(cfg_id, cfg, pos, rot, team, id, self.events, base_root)
                    actor.cannon_type = type
                    self.events:dispatch("actor_spawned", actor)

                    table.insert(self.sorted_cannons, actor)
                else
                    log.error("Failed to find cannon base root:"..root_name)
                end
            end
        end

        -- 出生 3 种炮台
        spawnCannon4Type(1, 1)
        spawnCannon4Type(2, 1)
        spawnCannon4Type(3, 1)
    end,

    isShipBattle = function(self)
        return true
    end,

    getDefenceTotalNums = function(self)
        return self.battle_report.defence_total
    end,

    -- 创建炮台 AI
    createCannonsAI = function(self)
        for k,actor in pairs(self.actors) do
            if actor.actor_type == actor_types.CANNON then
                self.ai_world:createAI(actor)
            end
        end
    end,

    -- 移除炮台 AI
    removeCannonAI = function(self)
        for k,actor in pairs(self.actors) do
            if actor.actor_type == actor_types.CANNON then
                self.ai_world:removeAI(actor)
            end
        end
    end,

    -- 破墙阶段回合时间更新,返回是否所有回合结束
    updateTurnTimeBreakWall = function(self)
        self.elapsed = self.elapsed + Time.deltaTime
        self.time_count_down = self.time_count_down - Time.deltaTime     
        if self.time_count_down < 0 then
            self.time_count_down = 0
        end 

        if self.elapsed > self.turn * self.turn_time then
            self:updateTurnBreakWall()
        end

        return self.turn > self.turn_num
    end,

    -- 占领阶段会和时间更新,返回是否所有回合结束
    updateTurnTimeOccupy = function(self)
        self.elapsed = self.elapsed + Time.deltaTime
        self.time_count_down = self.time_count_down - Time.deltaTime     
        if self.time_count_down < 0 then
            self.time_count_down = 0
        end    
           
        if self.elapsed > self.turn * self.turn_time then
            self:updateTurn()
        end

        return self.turn > self.turn_num
    end,

    -- 破墙阶段回合更新
    updateTurnBreakWall = function(self)
        local sorter = function(a, b)
            if a.is_alive and b.is_alive then
                return a.be_attack_count > b.be_attack_count
            elseif a.is_alive then
                return true
            elseif b.is_alive then
                return false
            else
                return false
            end
        end
        
        table.sort(self.sorted_soldiers[0][1], sorter)
        table.sort(self.sorted_soldiers[0][2], sorter)
        table.sort(self.sorted_soldiers[0][3], sorter)
        table.sort(self.sorted_soldiers[0][4], sorter)

        local percent = self.turn / self.turn_num   -- 到当前回合的死亡比例
        percent = math.clamp(percent, 0, 1)

        -- 杀本回合的士兵
        local killSoldier = function(team)
            --local ids = self.soldier_ids[team]
            for i=1,4 do    -- 遍历 4 种士兵
                local should_kill_nums = math.floor(self.killed_nums[team][i] * percent)   -- 到本回合，应该死掉的数量
                local turn_kill = should_kill_nums - self.killed_nums_record[team][i]   -- 本回合应该杀掉的

                local tab = self.sorted_soldiers[team][i]
                for i=1,turn_kill do
                    local actor = tab[i]
                    if actor and actor:isAlive() then
                        actor:kill()
                    end
                end

                self.killed_nums_record[team][i] = should_kill_nums
            end
        end

        -- 杀本回合的战车
        local killVehicle = function(team)
            local should_kill_nums = math.floor(self.killed_vehicle_nums[team] * percent)
            local turn_kill = should_kill_nums - self.killed_vehicle_nums_record[team]
            if turn_kill > 0 then
                local tab = self.sorted_vihicles[team] 
                table.sort(tab, sorter)

                for i=1, turn_kill do
                    local actor = tab[i]
                    if actor and actor:isAlive() then
                        app:startCoroutine(
                            function()
                                coroutine.yield(1.0)
                                actor:kill()                                
                            end
                        )
                        actor:kill()
                    end
                end

                self.killed_vehicle_nums_record[team] = should_kill_nums
            end
        end


        -- 找一个该杀掉的类型
        local getKillCannonType = function()
            if self.killed_cannon_nums_record[1] >= self.killed_cannon_nums_record[2] and self.killed_cannon_nums_record[1] >= self.killed_cannon_nums_record[3] then
                return 1
            elseif self.killed_cannon_nums_record[2] >= self.killed_cannon_nums_record[1] and self.killed_cannon_nums_record[2] >= self.killed_cannon_nums_record[3] then
                return 2
            else
                return 3
            end
        end

        local killOneCannon = function(type)
            local tab = self.sorted_cannons
            table.sort(tab, sorter)
            for i=1, #tab do
                local actor = tab[i]
                if actor and actor:isAlive() and actor.cannon_type == type then
                    log.info("kill cannon:"..actor.name)
                    app:startCoroutine(
                        function()
                            coroutine.yield(1.0)
                            actor:kill()                                
                        end
                    )
                    actor:kill()
                    return true
                end
            end
            return false
        end

        -- 杀本回合的炮台
        local killCannon = function()
            local should_kill_nums = math.floor(self.killed_cannon_sum * percent)
            local turn_kill_num = should_kill_nums - self.killed_cannon_sum_record

            local loop_limit = 20

           -- log.info("should_kill_num:"..should_kill_nums..", turn_kill_num:"..turn_kill_num)
            while turn_kill_num > 0 do
                loop_limit = loop_limit - 1
                if loop_limit <= 0 then
                    break
                end
                
                local kill_type = getKillCannonType()     
                if self.killed_cannon_nums_record[kill_type] <= 0 then
                    log.error("### should not be here !!!")
                    break
                else
                    log.info("kill cannon"..", type:"..kill_type)
                end

                if killOneCannon(kill_type) then
                    turn_kill_num = turn_kill_num - 1
                    self.killed_cannon_nums_record[kill_type] = self.killed_cannon_nums_record[kill_type] - 1
                    self.killed_cannon_sum_record = self.killed_cannon_sum_record + 1
                end
            end

            local cannon_num = (self.cannon_nums[1] + self.cannon_nums[2] + self.cannon_nums[3])
            log.info("### cannon_num:"..cannon_num)

            local playFx = function()
                local fx = self.level_manager.defenceWall.transform:Find("jx_zhan_he")
                if fx then
                    fx:SetActive(true)
                else
                    log.error("Failed to find fx jx_zhan_he")
                end                
            end

            if cannon_num <= 0 then
                self.level_manager.defenceWall:SetInteger("wall_break_state", 3)
                playFx()

            else
                local p = should_kill_nums / cannon_num 
                --log.info("### PPPP:"..p)
                if p >= 0.7 then
                    self.level_manager.defenceWall:SetInteger("wall_break_state", 3)
                    playFx()
                end          
            end
        end

        -- 战车放技能
        local vehicleCastSkill = function(team)
            local caster = nil
            for _,actor in pairs(self.actors) do
                if actor.actor_type == actor_types.VEHICLE and actor:isAlive() and actor.team == team then
                    if actor:canCastSkill() and actor.target_actor then
                        caster = actor
                        log.info("### can cast skill:"..actor.id)
                        break
                    end
                end
            end

            if caster then
                local rst = caster:ai_castUltimateSkill(caster.target_actor)

                local id = caster:getSkillID()
                --log.info("##### cast skill rst:"..tostring(rst)..", id:"..id)                
                local skill_cfg = config.skill_skills[id]
                if skill_cfg.skip_turn > 1 then
                    self.turn = self.turn + skill_cfg.skip_turn - 1
                end
            end
        end

        killSoldier(0)
        killVehicle(0)
        killCannon(1)
        vehicleCastSkill(0)

        local left_soldier_0 = self.soldier_total_num_0 - self.soldier_kill_num_0*percent
        local left_soldier_1 = self.soldier_total_num_1 - self.soldier_kill_num_1*percent

        local left_defence = (self.battle_report.defence_total - self.battle_report.defence_left) *(1-percent) + self.battle_report.defence_left

        local p0 = left_soldier_0 / self.soldier_total_num_0
       -- local p1 = left_soldier_1 / self.soldier_total_num_1
        local p1 = left_defence / self.battle_report.defence_total

        self.events:dispatch("update_turn", self.turn, self.turn_num, left_soldier_0, left_defence, p0, p1, self.turn_time, math.round(self.time_count_down))

        self.turn = self.turn + 1
        return self.turn > self.turn_num
    end,

    -- 每回合更新界面
    updateTurnUI = function(self, percent)
        local left_soldier_0 = self.soldier_mid_left_num_0 - self.soldier_kill_num_0*percent
        local left_soldier_1 = self.soldier_mid_left_num_1 - self.soldier_kill_num_1*percent
        local p0 = left_soldier_0 / self.soldier_total_num_0
        local p1 = left_soldier_1 / self.soldier_total_num_1
        self.events:dispatch("update_turn", self.turn, self.turn_num, left_soldier_0, left_soldier_1, p0, p1, self.turn_time, math.round(self.time_count_down))
    end,

    beginRandomMove = function()
        local randMove = self.stage_cfg.randomMove
        local f0 = self.stage_cfg.fixedMoveT0
        local f1 = self.stage_cfg.fixedMoveT1
        local fixedMoveT0 = Vector3(f0[1], f0[2], f0[3])
        local fixedMoveT1 = Vector3(f1[1], f1[2], f1[3])
        self.level_manager:BeginSoldierRandomMove(
            randMove.moveTime[1], randMove.moveTime[2],
            randMove.stopTime[1], randMove.stopTime[2],
            randMove.speed[1], randMove.speed[2],
            fixedMoveT0, fixedMoveT1)        
    end,

    update = function(self, dt)
        -- 状态机更新
        state_machine.update(self, dt)

        if not self.is_inited then
            return
        end

        -- 等待所有士兵初始化完成
        if not self.actor_initialized then
            if self:isActorsInitialized() then
                self.actor_initialized = true
                frame_manager:hide('loading')
                -- 接敌阶段
                self:changeState('approach')
            else
                return
            end
        end
        --self.elapsed = self.elapsed + dt
        
        -- 进场
        if self.is_entrance then
            for k,v in pairs(self.actors) do
                v:update(dt)
            end
            return
        end

        -- 战斗结束，失败方溃逃
        if self.is_battle_finished then
            --[[
            if self.elapsed > 5.0 then
                game_director:changeState('loading', {next_state = 'home', scene_name = 'home'})
            end]]
            return
        end

        for k,v in pairs(self.actors) do
             v:preUpdate(dt)
        end
        
        if self.ai_world then
            self.ai_world:update(dt)
        end

        battle_helper:update(dt)

        for k,v in pairs(self.actors) do
            v:update(dt)
        end

    end,
    
    changeState = function(self, state_name, args)
        if state_name ~= "break_wall" then
            self.elapsed = 0.0
        end

       return  state_machine.changeState(self, state_name, args)
    end,

}