local log = logger.get("Game", "RTSActor")
local actor_types = require('RTS.enums').actor_types

return {

    -- 将 target_actor 做为当前目标
    ai_setTargetActor = function(self, target_actor)
        self:setTargetActor(target_actor)
    end,

    -- 带 ai 前缀的，是给 ai world 调用的接口
    -- group 类型 actor, 移动到目标,如果进入射程就开始攻击
    ai_moveToAndAttack = function(self, target_actor)
        if not self:isAlive() then return end

        if self:getActorType() ~= actor_types.SOLDIER_GROUP then
            log.error('move to enemy and attack only suppor soldier group!!')
            return
        end

        self:setTargetActor(target_actor)
        self:changeState('move_to_and_attack', target_actor)

    end,

    ai_moveToTarget = function(self, target)
        self:changeState('move', target)
    end,

    ai_moveToPos = function(self, pos)
        self:changeState('move', pos)
    end,

    ai_followAndAttack = function(self, target)
        self:changeState('follow_and_attack', target)
    end,

    ai_attackTarget = function(self, target)
        --self.target_actor = target
        local skill_id = self:getCommonAtkID()    -- 基本攻击的技能 id

        if skill_id == 0 then
            skill_id = 1
        end
        --log.info('ai_attackTarget actor id:'..self.id..' skill id:'..skill_id)

        return self:castSkill(skill_id, target)
    end,

    --[[
    ai_escape = function(self, extends)
        if self:isEscape() then
            return
        end
        
        self:changeState('escape', extends)
        self.is_escape = true
    end,]]

    ai_castUltimateSkill = function(self, target, id)
        -- 目前只有一个技能
        local skill_id = id or self:getSkillID()
        --log.info('ai_castUltimateSkill actor id:'..self.id..' skill id:'..skill_id)
        if self.energy >= 100 then
            local rst, msg = self:changeStateImmediate('skill', {id=skill_id, ta=target, p=target:getPosition(), r=self:getRotation()})
            --log.info("### cast skill success!!")
            return rst
        else
            --log.info("### cast skill failed!!")
            return false
        end
    end,

    ai_castOpeningSkill = function(self, force)
        local effective = battle_helper:getFormationEffective(self.team)
        if effective <= 0 and not force then
            return
        end

        -- 被后方攻击
        if self.team == 1 and level_info.is_attack_from_back and not force then
            return
        end

        local skill_id = self.weapon_cfg.opening_skill
        if self.team == 1 then
            log.info('### ai_castOpeningSkill actor id:'..self.id..' skill id:'..skill_id)
        end
        if skill_id then
            self:changeStateImmediate('skill', {id=skill_id})
        else
            log.error('Failed to find opening_skill in weapon_cfg!')
        end
    end,
}