local log = logger.get("Game", "RTSActor")
local actor_types = require('RTS.enums').actor_types
local bullet_class = require('RTS.rts_bullet')
local penetration_bullet_class = require('RTS.rts_bullet_penetration')

local damage_calculator = require('RTS.prop_sys.damage_calculator')

return {
    
    -- 播放动画
    sk_playAnimation = function(self, anim_name)
        self.curr_state:playAnimation(anim_name)
    end,

    -- 播放特效
    sk_playVfx = function(self, prefab_path, bone, life, force, ignore_if_no_bone)
        if self.curr_state.name == 'preskill' then
            self.cs_inst:PlayUnscaledVFX(prefab_path, bone, life or 3.0);
        else
            --log.info("### PlayVfx:"..prefab_path..", force:"..tostring(force))
            battle_helper:playFXAttached(prefab_path, self, bone, life, force, ignore_if_no_bone)
        end
    end,

    -- 计算伤害,播战报时候忽略  
    calculateDamage = function(self, target_actor, skill_cfg, cast_inst_id)
        local dmg = 0
        local hp = nil
        local is_hit = false
        local is_crit = false

        if not battle_helper.is_replay then
            dmg, is_hit, is_crit = damage_calculator.calculate(damage_calculator, self, target_actor, skill_cfg)
        else
            --[[
            local hit_event = self.skill_state:popHitEvent(cast_inst_id, target_actor.id)
            if hit_event then
                dmg = hit_event.damage
                hp = hit_event.hp
                is_hit = hit_event.is_hit
                is_crit = hit_event.is_crit
            end]]
            dmg = 1
            hp = nil
            is_hit = true
            is_crit = false
        end       

        return dmg, is_hit, is_crit, hp
    end,

    -- 发射一颗子弹
    sk_shootBullet = function(self, target, loop_effect, loop_body, hit_effect, stop_effect, 
        from, to, speed, curve_name, damage_width, penetration, max_dist, 
        curve_height, cam_anim, stop_effect_follow_target) -- 参数的 伤害弃用 
       --[[
        to.y = from.y
        if target then
            to.y = to.y + (target.height - self.height)
        end]]
        --[[
        if self.actor_type == actor_types.SOLDIER then
            log.error("soldier could not shoot bullet, please call shoot_bullet_simple! id:"..self.cfg_id)
            if self.skill_state then
                log.error("skill id:"..self.skill_state.skill_id)
            else
                log.error('No skill state!')
            end
            return
        end]]

        local rot = Quaternion.LookRotation(to-from, Vector3.up)
        local skill_cfg = self.skill_cfg
        local cast_inst_id = self.cast_inst_id
        local hit_events = self.curr_state.hit_events

        local loop_body_path = nil
        if loop_body then loop_body_path = loop_body.effect_path end

        if penetration then
            --ctor = function (self, owner, target, prefab_path, pos, destination, speed, hit_callback)
            local bullet = penetration_bullet_class(self, rot, loop_effect.effect_path, loop_body_path, from, to, speed, max_dist, damage_width,
                function(b, target_actor) -- hit callback
                    -- 目标减血
                    if target_actor then
                        -- 调用伤害计算
                        local dmg, is_hit, is_crit, curr_hp = self:calculateDamage(target_actor, skill_cfg, cast_inst_id, hit_events)

                        if is_hit then
                            target_actor:beAttacked(self, skill_cfg, dmg, is_hit, is_crit, cast_inst_id, game_info.is_replay, curr_hp)
                        end
                        -- 播放击中特效
                        if hit_effect then
                            battle_helper:playFXOnActorPos(hit_effect.effect_path, target_actor, target_actor:getPosition(), hit_effect.effect_time)
                        end

                    end
                end,
                function(b) -- stop callback
                    self.bullets:remove(b)
                    if stop_effect and stop_effect.effect_path then
                        log.info("PlayFXOnPos:"..stop_effect.effect_path..", "..tostring(stop_effect.effect_time))
                        battle_helper:playFXOnPos(stop_effect.effect_path, to, stop_effect.effect_time)
                    end

                    if cam_anim then
                        battle_helper:playCamAnim(cam_anim)
                    end
                end           
                )
            bullet.curve_height = curve_height
            self.bullets:add(bullet)  
        else
            local curve = nil
            if curve_name then
                --log.info(curve_name)
                curve = battle_helper:loadCurve(curve_name)
            end
            --ctor = function (self, owner, target, prefab_path, pos, destination, speed, hit_callback)
            local bullet = bullet_class(self, target, rot, loop_effect.effect_path, loop_body_path, from, to, speed, curve,
                function(b) 
                    --log.info('bullet hit target_actor!') 
                    self.bullets:remove(b)
                    -- 目标减血
                    if b.target then
                        -- 调用伤害计算
                        local dmg, is_hit, is_crit, curr_hp= self:calculateDamage(b.target, skill_cfg, cast_inst_id, hit_events)
                        if is_hit then
                            b.target:beAttacked(self, skill_cfg, dmg, is_hit, is_crit, cast_inst_id, game_info.is_replay, curr_hp)
                        end
                    end
                    -- 播放子弹消失特效
                    if stop_effect then
                        if target then
                            if not stop_effect_follow_target then
                                battle_helper:playFXOnActorPos(stop_effect.effect_path, target, to, stop_effect.effect_time)
                            else
                                battle_helper:playFXOnActor(stop_effect.id, target, 'root')
                            end
                        else
                            battle_helper:playFXOnPos(stop_effect.effect_path, to, stop_effect.effect_time)
                        end

                        if cam_anim then
                            battle_helper:playCamAnim(cam_anim)
                        end

                    end

                    -- 播放击中特效
                    if hit_effect then
                        if to.y ~= target:getPosition().y then -- 打在车体上
                            battle_helper:playFXOnActorPos(hit_effect.effect_path, target, target:getPosition(), hit_effect.effect_time)
                        else -- 打在车辆周围
                            battle_helper:playFXOnActorPos(hit_effect.effect_path, target, to, hit_effect.effect_time)
                        end
                    end

                end)
                bullet.curve_height = curve_height
            self.bullets:add(bullet)              
        end
    end,

    -- 发射一颗子弹，命中后在 to 位置进行半径为 radius 的范围伤害
    sk_shootBulletDamageInArea = function(self, loop_effect, body_effect, hit_effect, stop_effect, target_actor, from, to, speed, curve_name, radius, curve_height, cam_anim, offset)

        local x = to.x + (math.random()-0.5) * radius * (offset or 0.7)
        local z = to.z + (math.random()-0.5) * radius * (offset or 0.7)
        local _dest = Vector3.New(x, 0.0, z)

        local rot = Quaternion.LookRotation(to-from, Vector3.up)
        local skill_cfg = self.skill_cfg
        local cast_inst_id = self.cast_inst_id
        local hit_events = self.curr_state.hit_events

        --ctor = function (self, owner, target, prefab_path, pos, destination, speed, hit_callback)
        local curve = nil
        if curve_name then
            --log.info(curve_name)
            curve = battle_helper:loadCurve(curve_name)
        end

        local body_effect_path = nil
        if body_effect then body_effect_path = body_effect.effect_path end
        local bullet = bullet_class(self, target_actor, rot, loop_effect.effect_path, body_effect_path, from, _dest, speed, curve,
            function(b) 
                --log.info('bullet hit target_actor!') 
                self.bullets:remove(b)
                
                local other_team = 0
                if self.team == 0 then other_team = 1 end
                local actors = battle_helper:getIntersectActorsCircle(other_team, to, radius)

                if actors then 
                    for _, defender in pairs(actors) do
                        local dmg, is_hit, is_crit, curr_hp = self:calculateDamage(defender, skill_cfg, cast_inst_id, hit_events)
                        defender:beAttacked(self, skill_cfg, dmg, is_hit, is_crit, cast_inst_id, game_info.is_replay, curr_hp)

                        --[[
                        if hit_effect then
                            battle_helper:playFXOnPos(hit_effect.effect_path, defender:getPosition(), hit_effect.effect_time)
                        end]]
                    end
                end

                -- 播放击中特效
                if stop_effect and stop_effect.effect_path then
                    if target_actor then
                        battle_helper:playFXOnActorPos(stop_effect.effect_path, target_actor, _dest, stop_effect.effect_time)
                    else
                        battle_helper:playFXOnPos(stop_effect.effect_path, _dest, stop_effect.effect_time)
                    end
                end

                if cam_anim then
                    battle_helper:playCamAnim(cam_anim)
                end

            end)
        bullet.curve_height = curve_height
        self.bullets:add(bullet)        
    end,

    -- 发射激光
    sk_shootLaser = function(self, target, bind_point, loop_effect, hit_effect, from_transform, to_transform)
        if not from_transform or not to_transform then
            return
        end

        log.info("## sk_shootLaser!! point:"..bind_point..", loop_effect:"..loop_effect.effect_path)
        -- ["shoot_effect_effect_libs"]={"effect_id","effect_path","audio","effect_time",},
        -- 循环特效
        local loop_fx = battle_helper:playFXAttached(loop_effect.effect_path, self, bind_point, loop_effect.effect_time, true)

        local elapsed = loop_effect.effect_time

        --log.info("### hit effect:".. hit_effect.effect_path)

        -- 击中特效
        local hit_fx = battle_helper:playFXOnActorPos(hit_effect.effect_path, target, to_transform.position, hit_effect.effect_time)

        local caller = {
                setLaserScale = function(self, node, scale)
                if string.find(node.name, "laser") then
                   -- log.info("set laser scale:"..node.name)
                    node.localScale = scale
                end
                for i=0, node.childCount-1 do
                    self:setLaserScale(node:GetChild(i), scale)
                end
            end
        }

        app:startCoroutine(
            function()
                while elapsed > 0 do
                    local scale = Vector3.Distance(from_transform.position, to_transform.position)
                    caller:setLaserScale(loop_fx.transform, Vector3(scale/10, 1, 1))
                    --log.info("from:"..tostring(from_transform.position)..", to:"..tostring(to_transform.position))
                    loop_fx.transform.rotation = Quaternion.LookRotation(to_transform.position - from_transform.position, Vector3.up)
                    coroutine.yield(nil)
                    elapsed = elapsed - Time.deltaTime
                end

                loop_fx:destroy()
            end
        )
    end,

    sk_shootBulletSimple = function(self, loop_effect_path, stop_effect, from, to, speed, curve_height)
        local fx = battle_helper:loadRes(loop_effect_path, true)
        if not fx then
            log.error("Failed to find bullet prefab:"..tostring(loop_effect_path))
        end

        local simple_bullet = SimpleBullet.Create(fx, from, to, speed, curve_height)
        local total_time = simple_bullet.m_fTotalTime
        battle_helper:destroyObjectAfter(fx, total_time + 0.25)
        if stop_effect then
            -- 75% 几率播放
            if math.random() > 0.24 then
                app:startCoroutine(function()
                    coroutine.yield(WaitForSeconds(total_time))
                    local fx = battle_helper:playFXOnPos(stop_effect.effect_path, to, stop_effect.effect_time)
                    if fx then
                        fx.transform.localRotation = Quaternion.LookRotation(to-from)
                    end
                end)
            end
        end
    end,

    -- 开始冲刺 
    sk_beginRush = function(self, speed, dir, hit_effect, hit_speed)
        
        self:setSpeed(speed)
        self.is_rush = true
        self.rush_radius = radius
        self.rush_hit_effect = hit_effect
        self.rush_hit_speed = hit_speed

        self:clearTargetActor()
        local avoidance_priority = self.cfg.avoid_priority

        if game_info.is_replay then
            self.cs_inst:BeginRush(Vector3.zero, avoidance_priority)
        else
            self.cs_inst:BeginRush(dir, avoidance_priority)
        end
    end,

    -- 结束冲刺
    sk_stopRush = function(self)
       --log.info('########### stop rush!!')
        self:setSpeed(self.cfg.speed)
        self.cs_inst:StopMove()
        self.cs_inst:SetDestination(self:getPosition())
        self.is_rush = false
        self.cs_inst:StopRush()
        self.cs_inst:SetAvoidancePriority(self.cfg.avoid_priority)
        self.cs_inst:SetAvoidanceQuality(4)
        self.curr_state:playAnimation('stop', false);

        self:clearTargetActor()
    end,

    sk_beginRunning = function(self, speed, target_actor, hit_effect, hit_speed)
        --log.info('###sk_beginRunning')
        self:setSpeed(speed)
        --self.cs_inst:SetDestination(target_actor:getPosition())
        self.cs_inst:SetAvoidancePriority(self.cfg.avoid_priority - 20)
        self.is_rush = true
        self.rush_radius = radius
        self.rush_hit_effect = hit_effect
        self.rush_hit_speed = hit_speed

    end,

    -- 结束冲刺
    sk_stopRunning = function(self)
        --log.info('###sk_stopRunning')
        self:setSpeed(self.cfg.speed)
        self.cs_inst:StopMove()
        --self.cs_inst:SetDestination(self:getPosition())
        self.cs_inst:SetAvoidancePriority(self.cfg.avoid_priority)
        self.is_rush = false
        --self.curr_state:playAnimation('stop', false);
    end,

    -- 显示顶部技能释放条
    sk_showCastingBar = function(self, total_time, is_left_to_right, inc_or_dec, in_order, skill_types)
        battle_frame:showTopBar(self.id, total_time, is_left_to_right, inc_or_dec)
        battle_frame:showTopSkillList(in_order, skill_types)
    end,

    -- 隐藏顶部技能释放条
    sk_hideCastingBar = function(self)
        -- battle_frame:hideTopBar()
    end,

    -- 移除一个顶部技能施放条的技能 icon
    sk_removeCastingBarSkillIcon = function(self, type)
        battle_frame:checkTopSkillIcon(type)
    end,

    -- 技能结束
    sk_finished = function(self)
        if self.curr_state.name == 'skill' then
            self.curr_state:finished()
        end
    end,

    sk_breakSkill = function(self)
        self.energy = 0
        self:sk_hideCastingBar()
        self:sk_finished()
    end,

}