local vec3 = require("base.vec3")
local log = logger.get("Game", "bullet")

return class {
    ctor = function (self, owner, target, rot, prefab_path, body_path, pos, destination, speed, curve, hit_callback)

        self.owner = owner
        self.target = target
        self.fx = battle_helper:loadRes(prefab_path, true)    -- 特效
        if not self.fx then
            log.error("Failed to find bullet prefab:"..tostring(prefab_path))
        end

        self.transform = self.fx.transform
        self.transform.rotation = rot
        self.pos = pos   
        self.source = pos
        self.destination = destination
        self.speed = speed
        self.hit_callback = hit_callback
        self.curve = curve
        self.transform:SetPosition(pos[1], pos[2], pos[3])

        if body_path then
            self.body = battle_helper:loadRes(body_path)
            self.body.transform:SetPosition(pos.x, pos.y, pos.z)
            self.body.transform.rotation = rot
            self.body.transform:SetParent(self.transform)
        end

        self.total_len = (destination-pos).magnitude
        self.total_time = self.total_len / self.speed
        self.right = self.transform.right

        self.elapsed = 0.0
        self.last_pos = pos
        self.curve_height = nil
        if curve then
            self:update(0.001)
        end
    end, 

    update = function (self, dt)
        local delta = self.destination - self.pos   
        local distance = delta.magnitude
        self.elapsed = self.elapsed + dt
        local progress = self.elapsed / self.total_time -- 时间进度

        if progress >= 1.0 then -- 移动到目标
            -- hit
            if self.hit_callback then 
                self.hit_callback(self)
            end
            self:destroy()
        else
            -- 进行中
            if self.curve then
                local curve_x = self.curve:Evaluate(progress + 2.0)
                local curve_y = self.curve:Evaluate(progress)
                local curve_z = self.curve:Evaluate(progress + 4.0)

                local p = Vector3.Lerp(self.source, self.destination, curve_x)

                if self.curve_height then
                    p.y = p.y + self.curve_height * curve_y
                else
                    p.y = p.y + self.total_len * curve_y
                end

                p = p + self.right * self.total_len * curve_z
                self.transform.position = p 

                if (p ~= self.last_pos) then
                    local r = Quaternion.New()
                    r:SetLookRotation(p - self.last_pos)
                    self.transform.rotation = r
                end
                self.last_pos = p
            else
                local p = Vector3.Lerp(self.source, self.destination, progress)
                self.transform.position = p
            end

            --[[
            local pos = self.pos
            local normalized = tickspace / distance 
            pos[1] = pos[1] + delta[1] * normalized
            pos[2] = pos[2] + delta[2] * normalized
            local y = pos[2]
            if self.curve then
                local factor = self.elapsed / self.total_time
                local v = self.curve:Evaluate(factor)
                if self.curve_height then
                    y = pos[2] + self.curve_height * v
                else
                    y = pos[2] + self.total_len * v
                end
            end
            pos[3] = pos[3] + delta[3] * normalized

            self.transform:SetPosition(pos[1], y, pos[3])
            if self.curve then
                local current = Vector3.New(pos[1], y, pos[3])
                if self.last_pos then
                    local rot = Quaternion.New()
                    local look = current - self.last_pos
                    if look ~= Vector3.zero then
                        rot:SetLookRotation(current-self.last_pos, Vector3.up)
                        self.transform.rotation = rot
                    end
                end
                self.last_pos = current
            end
            ]]
        end

    end, 

    destroy = function (self)
        app:startCoroutine(
            function()
                coroutine.yield(WaitForSeconds(1.0))
                if self.fx then
                    self.fx:destroy()
                    self.fx = false
                end
            end
        )

        if self.body then
            self.body:destroy()
            self.body = false
        end
        self.hit_callback = nil
    end, 
}