local vec3 = require("base.vec3")
local rect_area = require('RTS.areas.rect')
local log = logger.get("Game", "bullet_penetration")

return class {
    ctor = function (self, owner, rot, prefab_path, body_path, pos, destination, speed, max_distance, width, hit_callback, stop_callback)

        self.owner = owner
        self.fx = battle_helper:loadRes(prefab_path, true)    -- 特效

        self.transform = self.fx.transform
        self.transform.rotation = rot
        self.pos = {pos.x, pos.y, pos.z}          -- vec3
        self.destination = {destination.x, destination.y, destination.z}    -- vec3
        self.speed = speed
        self.damage_callback = damage_callback
        self.hit_callback = hit_callback
        self.stop_callback = stop_callback
        self.transform:SetPosition(pos.x, pos.y, pos.z)

        if body_path then
            self.body = battle_helper:loadRes(body_path)
            self.body.transform:SetPosition(pos.x, pos.y, pos.z)
            self.body.transform.rotation = rot
            self.body.transform:SetParent(self.transform)
        end

        self.elapsed = 0.0
        self:initTargets(owner:getEnemyTeam(), (pos + destination) * 0.5, rot, width * 0.5, max_distance * 0.5)
    end, 

    -- 预先计算能攻击到哪些敌人
    initTargets = function(self, team, center, rot, half_width, half_length)
        local half_extents = Vector3.New(half_width, 1.0, half_length)
        local rect = rect_area(team, center, rot, half_extents)
        local actors = rect:intersectActors(battle_helper.stage.actors)
        self.hit_actors = {}
        if actors then
            --log.info('rect intersect actors:'..#actors)
            for _, actor in pairs(actors) do
                local dist = Vector3.Distance(actor:getPosition(), self.pos)
                local time = dist / self.speed
                table.insert(self.hit_actors, {['time'] = time, ['actor']=actor})
            end
            table.sort(self.hit_actors, function(a,b) return a.time < b.time end)
        end
    end,

    update = function (self, dt)
        local delta = vec3.sub(self.destination, self.pos)
        local distance = vec3.len(delta)
        local tickspace = dt * self.speed

        if tickspace >= distance then 
            -- hit
            self.stop_callback(self)
            self:destroy()
        else
            local pos = self.pos
            local normalized = tickspace / distance 
            pos[1] = pos[1] + delta[1] * normalized
            pos[2] = pos[2] + delta[2] * normalized
            pos[3] = pos[3] + delta[3] * normalized
            self.transform:SetPosition(pos[1], pos[2], pos[3])
        end
        self.elapsed = self.elapsed + dt

        for k,v in pairs(self.hit_actors) do
            if v.time >= self.elapsed then
                self.hit_callback(self, v.actor)
                table.remove(self.hit_actors, k)
                break
            else
                break
            end
        end
    end, 

    destroy = function (self)
        --self.stage.bullets:remove(self)
        app:startCoroutine(
            function()
                coroutine.yield(WaitForSeconds(0.5))
                if self.fx then
                    self.fx:destroy()
                    self.fx = false
                end
            end
        )

        if self.body then
            self.body:destroy()
            self.body = false
        end

        self.hit_callback = nil
    end, 
}