local log = logger.get("Game", "Soldier")
local super = require('RTS.rts_actor_base')
local bullet_class = require('RTS.rts_bullet')
local list = require('base.list')
local actor_types = require('RTS.enums').actor_types
local soldier_props = require('RTS.prop_sys.soldier_props')
local buff_handler = require('RTS.buff.buff_handler')
return class {

    super = super,

    ctor = function(self, cfg_id, cfg, pos, rot, team, id, events, root)
        super.ctor(self, team, id)
        self.cfg_id = cfg_id
        self.cfg = cfg
        self.name = 'cannon_'..team..'_'..id
        self.actor_type = actor_types.CANNON
        self.events = events
        self.cs_inst = nil      -- c# 的对应实例, 也就是 cs 的 soldier
        self.animation = nil    -- animation 组件
        self.level = 1
        self.be_attack_count = 0

        -- cfg
        self.ai_name = self.cfg.ai      -- 单个士兵没有 ai
        self.anim_names =       -- 动画名称配置 
        {
            idle = 'idle',
            move = 'run',
            attack = 'attack',
            die = 'die',
            escape = 'escape',
        }   

        --self.bullet_prefab = consts.assets.soldier_bullet
        self.radius = self.cfg.round
        self.height = self.cfg.scene_height
        
        --props
        --self.hp = cfg.hp
        self.attack_range = cfg.atk_range
        self.attack_range_sq = self.attack_range * self.attack_range
        self.speed = 0
        self.acceleration = cfg.speed   -- 先用速度做为加速度

        --state
        self.res_loaded = false -- 是否资源加载完成
        
        self.frame = 0
        self.active_vfx = {}    -- 播放中的特效 [路径] - 数量
        self.active_vfx_frame = {}  -- 该名称特效是在多少帧播放的

        self.bullets = list()
        self.pool = battle_helper.pool
        self:loadResource(pos, rot, self.name, root)
    end,

    -- 加载资源
    loadResource = function(self, pos, rot, name, root)
        local callback = function(go)
            -- callback
            if go ~= nil then
                self.res_loaded = true
                go.name = name
                self.gameObject = go
                self:init()
                self.transform.position = pos
                self.transform.rotation = rot
                self.transform.localScale = Vector3(4,4,4)               
                self.transform:SetParent(root)
                self.events:dispatch("actor_loaded", self)
            else

            end
        end

        local prefab_path = self.cfg.prefab1
        if self.team == 1 then
            prefab_path = self.cfg.prefab2
        end

        local uobject = self.pool:instantiate(prefab_path)
        if not uobject then
            log.error('Failed to load prefab:'..prefab_path)
            return nil
        end

        if uobject.isLoaded then 
            callback(uobject)
        else 
            uobject.onLoaded = callback
        end
    end,
     
    -- 初始化
    init = function(self)
        self.cs_inst = self.gameObject:GetComponentInChildren('Cannon')
        self.transform =  self.cs_inst.transform

        --self.cs_inst:SetMoveParam(self.speed, self.acceleration, 180, 1.0)
        self.animation = self.cs_inst.m_Animation
        self.shoot_point = self.cs_inst.m_ShootPoint

        self.cs_inst.enabled = false

        -- 初始化属性 
        self:initProps()

        -- 初始化角色状态机
        self.states['idle'] = require('RTS.actor_states.actor_state_idle').new('idle', self)
        self.states['move'] = require('RTS.actor_states.actor_state_move').new('move', self)
        self.states['skill'] = require('RTS.actor_states.actor_state_skill').new('skill', self)
        self.states['die'] = require('RTS.actor_states.actor_state_die').new('die', self)
        self.states['escape'] = require('RTS.actor_states.actor_state_escape').new('escape', self)
        self.states['entrance'] = require('RTS.actor_states.actor_state_entrance').new('entrance', self)

        self:changeState('entrance')
        self.skill_state = self.states['skill']
        self.is_initialized = true
    end,

    initProps = function(self)
        self.props = soldier_props(self.cfg, self.level)
        if not self.init_props then
            self.props:computeStatic()
            self.props:computeFinal()
        else
            for k,v in ipairs(self.init_props) do
                self.props.static_group.props[v.PropId] = v.PropValue
            end
            self.props:computeFinal()

            --[[
            local clientProp = soldier_props(self.cfg, self.level)

            clientProp:computeStatic()
            clientProp:computeFinal()
            log.info('soldier id:'..self.cfg_id..', level:'..self.level..', team:'..self.team)
            self.props.static_group:compair(clientProp.static_group)
            clientProp.base_group:log()
            clientProp.static_group:log()
            ]]
        end

        self.max_hp = self.props:getProp(MAX_HP_IDX)
        self.hp = self.max_hp
    end,

    -- 获取普通攻击技能 id，子类应该重写
    getCommonAtkID = function(self)
        return self.cfg.soldier_commonatk
    end,

    -- 获取技能 id, 子类应当重写
    getSkillID = function(self)
        return self.cfg.soldier_commonatk
    end,

    update = function(self, dt)
        super.update(self, dt)
        self.bullets:update(dt)

    end,

    addHP = function (self, _hp)
    end,

    beAttacked = function(self, attacker, skill_cfg, damage, is_hit, is_crit)
        self.be_attack_count = self.be_attack_count + 1
    end,

}