local log = logger.get("Game", "Soldier")
local super = require('RTS.rts_actor_base')
local bullet_class = require('RTS.rts_bullet')
local list = require('base.list')
local actor_types = require('RTS.enums').actor_types
local soldier_props = require('RTS.prop_sys.soldier_props')
local buff_handler = require('RTS.buff.buff_handler')
return class {

    super = super,

    ctor = function(self, cfg_id, cfg, pos, rot, team, id, group_id, events, root)
        super.ctor(self, team, id)
        self.group_id = group_id
        self.cfg_id = cfg_id
        self.cfg = cfg
        self.name = 'solder_'..team..'_'..id
        self.actor_type = actor_types.SOLDIER
        self.events = events
        self.cs_inst = nil      -- c# 的对应实例, 也就是 cs 的 soldier
        self.animation = nil    -- animation 组件
        self.level = 1
        self.be_attack_count = 0

        -- cfg
        self.ai_name = nil      -- 单个士兵没有 ai
        self.anim_names =       -- 动画名称配置 
        {
            idle = 'idle',
            move = 'run',
            attack = 'attack',
            die = 'die',
            escape = 'escape',
            win = 'win',
        }   

        --self.bullet_prefab = consts.assets.soldier_bullet
        self.radius = self.cfg.round
        self.height = self.cfg.scene_height
        
        --props
        --self.hp = cfg.hp
        self.attack_range = cfg.atk_range
        self.attack_range_sq = self.attack_range * self.attack_range
        self.speed = cfg.speed
        self.acceleration = cfg.speed   -- 先用速度做为加速度

        --state
        self.res_loaded = false -- 是否资源加载完成

        self.bullets = list()
        self.pool = soldier_pool--battle_helper.pool
        self:loadResource(pos, rot, 'soldier_'..team..'_'..id, root)

        self.frame = 0
        self.active_vfx = {}    -- 播放中的特效 [路径] - 数量
        self.active_vfx_frame = {}  -- 该名称特效是在多少帧播放的
    end,

    -- 加载资源
    loadResource = function(self, pos, rot, name, root)
        local callback = function(go)
            -- callback
            if go ~= nil then
                self.res_loaded = true
                go.name = name
                self.gameObject = go
                self:init()
                self.transform.position = pos
                self.transform.rotation = rot
                self.transform.localScale = Vector3.one
                self.transform:SetParent(root)

                self.events:dispatch("actor_loaded", self)
            else

            end
        end

        local prefab_path = self.cfg.prefab1
        if self.team == 1 then
            prefab_path = self.cfg.prefab2
        end

        local uobject = self.pool:instantiate(prefab_path)
        if not uobject then
            log.error('Failed to load prefab:'..prefab_path)
            return nil
        end

        if uobject.isLoaded then 
            callback(uobject)
        else 
            uobject.onLoaded = callback
        end
    end,
     
    -- 初始化
    init = function(self)
        self.cs_inst = self.gameObject:GetComponentInChildren('Soldier')
        self.transform =  self.cs_inst.transform

        --self.cs_inst:SetMoveParam(self.speed, self.acceleration, 180, 1.0)
        self.animation = self.cs_inst.m_Animation
        self.shoot_point = self.cs_inst.m_ShootPoint

        self.cs_inst.enabled = false

        -- 初始化属性 
        self:initProps()

        -- 初始化角色状态机
        self.states['idle'] = require('RTS.actor_states.actor_state_idle').new('idle', self)
        self.states['move'] = require('RTS.actor_states.actor_state_move').new('move', self)
        self.states['skill'] = require('RTS.actor_states.actor_state_skill').new('skill', self)
        self.states['die'] = require('RTS.actor_states.actor_state_die').new('die', self)
        self.states['escape'] = require('RTS.actor_states.actor_state_escape').new('escape', self)
        self.states['entrance'] = require('RTS.actor_states.actor_state_entrance').new('entrance', self)
        self.states['win'] = require('RTS.actor_states.actor_state_win').new('win', self)

        self:changeState('idle')
        self.skill_state = self.states['skill']

        -- buff 处理器
        self.buff_handler = buff_handler(self)
        self.is_initialized = true
    end,

    initProps = function(self)
        self.props = soldier_props(self.cfg, self.level)
        if not self.init_props then
            self.props:computeStatic()
            self.props:computeFinal()
        else
            for k,v in ipairs(self.init_props) do
                self.props.static_group.props[v.PropId] = v.PropValue
            end
            self.props:computeFinal()

            --[[
            local clientProp = soldier_props(self.cfg, self.level)

            clientProp:computeStatic()
            clientProp:computeFinal()
            log.info('soldier id:'..self.cfg_id..', level:'..self.level..', team:'..self.team)
            self.props.static_group:compair(clientProp.static_group)
            clientProp.base_group:log()
            clientProp.static_group:log()
            ]]
        end

        self.max_hp = self.props:getProp(MAX_HP_IDX)
        self.hp = self.max_hp
    end,

    -- 获取动画后缀 sin(22.5) = 0.383, cos(22.5) = 0.924
    getAnimPostfix = function(self, random_postfix)
        local angle = 0

        if random_postfix then
            angle = math.random(1, 360)
        else
            local a = battle_helper:getCameraForward()
            local b = self.transform.forward
            --local dot = Vector3.Dot(a, b)
            angle = Vector3.Angle(a, b)
            local axis = Vector3.Cross(a, b)

            if axis.y < 0 then
                angle = 360 - angle
            end
        end
        --log.info("a:"..tostring(a)..", b:"..tostring(b))
        --log.info(', angle:'..tostring(angle)..", axis:"..tostring(axis))

        local postfix = nil;
        
        if angle < 22.5 or angle > 337.5 then
            postfix = "N_"
        elseif angle < 67.5 then
            postfix = "NE_"
        elseif angle < 112.5 then
            postfix = "E_"
        elseif angle < 157.5 then
            postfix = "SE_"
        elseif angle < 202.5 then
            postfix = "S_"
        elseif angle < 247.5 then
            postfix = "SW_"
        elseif angle < 292.5 then
            postfix = "W_"
        else
            postfix = "NW_"
        end
        return postfix
    end,

    -- 获取普通攻击技能 id，子类应该重写
    getCommonAtkID = function(self)
        return self.cfg.soldier_commonatk
    end,

    -- 获取技能 id, 子类应当重写
    getSkillID = function(self)
        return self.cfg.soldier_commonatk
    end,

    update = function(self, dt)
        super.update(self, dt)
        self.bullets:update(dt)

        --[[
        if self.cs_inst then
            self.cs_inst:Tick(dt)
        end]]
    end,

    addHP = function (self, _hp)
    end,

    beAttacked = function(self, attacker, skill_cfg, damage, is_hit, is_crit)
        self.be_attack_count = self.be_attack_count + 1
        --self.hp = 999
        --self.is_alive = true
        --log.info('soldier beAttack')
        --[[
        if not self:isAlive() then
            return
        end
        
        if is_hit then
            self.hp = self.hp - damage

            --log.info('### soldier got hit:'..damage)
            
            if self.hp < 0 then
                -- 死了
                self.is_alive = false
                self:changeState('die')
                self.events:dispatch('actor_dead', self)

                --self.cs_inst:Dead()
            elseif skill_cfg.hit_add_buff > 0 then
                self.buff_handler:addBuff(attacker, skill_cfg.hit_add_buff)
            end
        end]]
    end,

    kill = function(self)
        super.kill(self)
        self.is_alive = false
    end,

}