local log = logger.get("Game", "SoldierGroup")
local super = require('RTS.rts_actor_base')
local bullet_class = require('RTS.rts_bullet')
local list = require('base.list')
local soldier_class = require('RTS.rts_soldier')
local actor_types = require('RTS.enums').actor_types

return class
{
    super = super,

    ctor = function(self, cfg_id, owner, team, id, events)
        super.ctor(self, team, id)
        self.actor_type = actor_types.SOLDIER_GROUP
        self.name = 'SoldierGroup_'..team..'_'..id
        self.is_alive = true
        self.owner = owner  -- 所属的战车

        self.events = events
        
        self.soldiers = {}
        self.res_loaded = false -- 所有士兵的资源是否加载完成

        --print(config.soldier_soldiers, cfg_id)
        self.cfg_id = cfg_id
        local soldier_cfg = config.tank_basic_ids[cfg_id]
        if not soldier_cfg then
            log.error("Failed to find soldier cfg:"..cfg_id)
            return
        end
        self.cfg = soldier_cfg
        self.ai_name = self.cfg.ai -- 'soldier' 

        self.attack_range = soldier_cfg.atk_range
        self.attack_range_sq = self.attack_range * self.attack_range

    end,

    spawnSoldiers = function(self, cfg_id, orig, rot, offset_tab, num, id_tab, events, root)
        if not self.level or self.level == 0 then
            self.level = 1
        end

        -- 创建 num 个 soldier
        for i=1, num, 1 do 
            local offset = {['posX'] = 0, ['posY'] = 0}
            if offset_tab then
               -- offset = offset_tab[i] -- {"posX","posY",}
            end
            if offset then
                offset = Vector3.New(offset.posX, 0, offset.posY)
                offset = rot * offset
                pos = orig + offset

                --pos = orig + Vector3(math.random(10), 0, math.random(10))
                --[[
                if team == 0 then
                    pos.x = pos.x + offset.posX
                    pos.z = pos.z + offset.posY
                else 
                    pos.x = pos.x - offset.posX
                    pos.z = pos.z - offset.posY
                end]]
                -- (self, cfg, pos, rot, team, id, events, root)
                local soldier = soldier_class(cfg_id, self.cfg, pos, rot, self.team, id_tab[i], events, root)
                soldier.level = self.level
                soldier.init_props = self.init_props
                soldier.offset = offset -- 保存初始偏移
                self.soldiers[i] = soldier
                self.events:dispatch('actor_spawned', soldier)
            else
                log.error("Failed to find offset in offset_tab idx:"..i)
            end
        end

        app:startCoroutine(function() self:check_res_loaded() end)
    end,

    -- 检查是否所有 actor 都资源加载完成的 协程
    check_res_loaded = function(self)
        while true do
            coroutine.yield(WaitForSeconds(0.1))

            if not self.res_loaded then
                local loaded = true
                for k,v in ipairs(self.soldiers) do
                    loaded = loaded and v.res_loaded
                end
                
                if loaded then
                    self.res_loaded = true

                    self:init()
                    self.events:dispatch("actor_loaded", self)

                    break
                end
            else
                log.error('should not be here!')
                break
            end
        end
    end,

    init = function(self)
        -- 初始化状态
        self.states['idle'] = require('RTS.actor_states.group_state_idle').new('idle', self)
        self.states['escape'] = require('RTS.actor_states.group_state_escape').new('escape', self)
        self.states['move_to_and_attack'] = require('RTS.actor_states.group_state_move_to_and_attack').new('move_to_and_attack', self)
        self.states['follow_and_attack'] = require('RTS.actor_states.group_state_follow_and_attack').new('follow_and_attack', self)

        self.is_initialized = true
    end,

    update = function(self, dt)
        super.update(self, dt)
    end,

    isAlive = function(self)
        if self.is_alive then
            local alive = false
            for k,v in ipairs(self.soldiers) do
                if v:isAlive() then
                    alive = true
                    break
                end
            end

            self.is_alive = alive
        end

        return self.is_alive
    end,

    getPosition = function (self)
        if not self.cache_position then
            -- 返回所有士兵的平均位置
            local pos = Vector3.zero
            local count = 0
            for k,v in ipairs(self.soldiers) do
                if v:isAlive() then
                    Vector3.Add(pos, v:getPosition())
                    count = count + 1
                end
            end

            if count > 0 then
                Vector3.Div(pos, count)
            else
                pos = self.soldiers[1]:getPosition()
            end
            self.cache_position = pos
        end

        return self.cache_position
    end,

    -- 选择一个目标
    setTargetActor = function(self, target_actor)
       if self.target_actor ~= target_actor then
            for k,v in ipairs(self.soldiers) do
                if v:isAlive() then
                    v:setTargetActor(target_actor)
                end
            end
            self.target_actor = target_actor
            if target_actor and target_actor:getActorType() == actor_types.SOLDIER_GROUP then
                self.group_target_actor = target_actor
            end
        end
    end,

    beAttacked = function(self, attacker, skill_cfg, damage, is_hit, is_crit)
        log.error('soldier group beAttacked, should not be here!!')
    end,

    clearTargetActor = function(self)
        self.target_actor = nil
        for k,v in pairs(self.soldiers) do
            v:clearTargetActor()
        end
    end,

}