local log = logger.get("Game", "BT do_damage_in_circle")
local super = require('bt.action')
local states = super.states

-- 进行圆形范围的伤害
return class {
    typename = 'action: do_damage_in_circle',
    super = super,
    ctor = function(self, ...)
        super.ctor(self)
        self.offset = select(1, ...)    -- 向前的偏移
        self.radius = select(2, ...)    -- 伤害半径

        --log.info(self.offset..","..self.length..','..self.width)
    end,

    setContext = function (self, ctx)
        self.ctx = ctx
        local skill_cfg = ctx.skill_cfg

        -- 如果传入的是 P1 ... 参数，从配置中读值
        if type(self.offset) == 'string' then
            self.offset = skill_cfg[self.offset]
        end

        if type(self.radius) == 'string' then
            self.radius = skill_cfg[self.radius] or self.radius
        end

        if not self.offset or not self.radius then
            log.error('parameter error!!')
        end
    end, 

    update = function(self, dt)
        local actor = self.ctx
        local skill_state = actor.curr_state
        assert(skill_state.name == 'skill')

        self.damage_rot = actor:getRotation()
        self.damage_pos = skill_state.skill_position or actor:getPosition()

        local direction = Quaternion.Mul(self.damage_rot, Vector3.forward)
        local offset = direction * (self.offset)
        self.damage_center = self.damage_pos + offset

        local other_team = 0
        if actor.team == 0 then other_team = 1 end

        battle_helper:doCircleDamage(actor, other_team, self.damage_center, self.radius, actor.skill_cfg)

        return states.SUCCESS
     end,
}