local log = logger.get("Game", "BT do_damage_in_rect")
local super = require('bt.action')
local states = super.states

-- 进行矩形范围的伤害
return class {
    typename = 'action: do_damage_in_rect',
    super = super,
    ctor = function(self, ...)
        super.ctor(self)
        self.offset = select(1, ...)      -- 向前的偏移
        self.length = select(2, ...)    -- (从偏移值开始的)伤害范围长度
        self.width = select(3, ...)     -- 伤害宽度

        --log.info(self.offset..","..self.length..','..self.width)
    end,

    setContext = function (self, ctx)
        self.ctx = ctx
        local skill_cfg = ctx.skill_cfg

        -- 如果传入的是 P1 ... 参数，从配置中读值
        if type(self.offset) == 'string' then
            self.offset = skill_cfg[self.offset]
        end
        if type(self.length) == 'string' then
            self.length = skill_cfg[self.length]
        end
        if type(self.width) == 'string' then
            self.width = skill_cfg[self.width] 
        end

        if not self.offset or not self.length or not self.width then
            log.error('parameter error!!')
        end
    end, 

    update = function(self, dt)
        local actor = self.ctx
        local skill_state = actor.curr_state
        assert(skill_state.name == 'skill')

        self.damage_rot = skill_state.skill_rotation or actor:getRotation()
        self.damage_pos = skill_state.skill_position or actor:getPosition()

        local direction = Quaternion.Mul(self.damage_rot, Vector3.forward)
        local offset = direction * (self.length*0.5 + self.offset)
        self.damage_center = self.damage_pos + offset

        local other_team = 1
        if actor.team == 1 then other_team = 0 end

        battle_helper:doRectDamage(actor, other_team, self.damage_center, self.damage_rot, self.length, self.width, actor.skill_cfg)

        return states.SUCCESS
     end,
}