local log = logger.get("Game", "BT")
local super = require('bt.action')
local states = super.states
local damage_calculator = require('RTS.prop_sys.damage_calculator')

-- 跑过去打一下
return class {
    typename = 'action: run to and hit',
    super = super,
    ctor = function(self, ...)
        super.ctor(self)
        self.shoot_effect_id = select(1, ...)   -- 
        self.bind_point = select(2, ...)
        self.speed = select(3, ...)     -- 冲刺速度 
        self.radius = select(4, ...)    -- 冲刺时的伤害半径
        self.hit_speed = select(5, ...) -- 受击速度

        local _cfg = config.shoot_effect_shoots[self.shoot_effect_id]
        -- "start","loop","stop"
        if _cfg.start > 0 then
            self.start_effect = config.shoot_effect_effect_libs[_cfg.start]
        end
        if _cfg.loop > 0 then
            self.loop_effect = config.shoot_effect_effect_libs[_cfg.loop]
        end
        -- if _cfg.hit ~= "" then self.hit_effect = config.shoot_effect_effect_libs[_cfg.hit] end
        if _cfg.stop and _cfg.stop > 0 then
            self.stop_effect = config.shoot_effect_effect_libs[_cfg.stop]
        end

        --log.info(self.offset..","..self.length..','..self.width)

        self.inited = false
        self.elapsed = 0.0

        self.check_elapsed = 0.0
        self.hit_enemy = {}

    end,

    setContext = function (self, ctx)
        self.ctx = ctx
        self.skill_cfg = ctx.skill_cfg

        if type(self.speed) == 'string' then
            self.speed = self.skill_cfg[self.speed]
        end

        if type(self.radius) == 'string' then
            self.radius = self.skill_cfg[self.radius]
        end

        if type(self.hit_speed) == 'string' then
            self.hit_speed = self.skill_cfg[self.hit_speed]
        end

    end, 

    reset = function(self)

        self:stopRunning(self.ctx)
    end,

    update = function(self, dt)
        local actor = self.ctx
        local skill_state = actor.curr_state
        assert(skill_state.name == 'skill')

        -- 初始化
        if not self.inited then
            self.inited = true
            local pos = actor:getPosition()
            self.orig_pos = Vector3.New(pos.x, pos.y, pos.z)

            -- 播放开火特效
            if self.start_effect then
                battle_helper:playFXAttached(self.start_effect.effect_path, actor, self.bind_point, self.start_effect.effect_time)
            end
            -- 播放持续特效
            if self.loop_effect then
                self.loop_effect_fx = battle_helper:playFXAttached(self.loop_effect.effect_path, actor, self.bind_point, 999)
            end

           -- log.info('############ sk_beginRunning!!')
            actor:sk_beginRunning(self.speed, actor.target_actor, self.hit_effect, self.hit_speed)
            actor.last_hit_actor = nil

            actor.cs_inst:PlayAnimation("run")
            self.hit_enemy = {}

            self.target_actor = actor.target_actor
        end
        
        self.elapsed = self.elapsed + dt

        if not game_info.is_replay then
            -- 冲刺过程，每0.1秒进行伤害判定
            self.check_elapsed = self.check_elapsed + dt
            if self.check_elapsed > 0.1 then
                self.check_elapsed = 0.0
                --battle_helper:doCircleDamage(actor, actor:getEnemyTeam(), actor:getPosition(), self.radius, self.skill_cfg)

                --log.info('######run_to_and_hit setDestination!!')
                --log.info('target_actor pos:'..tostring(actor.target_actor:getPosition()))
                if actor.target_actor then
                    actor.cs_inst:SetDestination(actor.target_actor:getPosition())
                else
                    actor.last_hit_actor = actor
                    return states.SUCCESS
                end

                local hit_enemy = false
                local enemys = battle_helper:getIntersectActorsCircle(actor:getEnemyTeam(), actor:getPosition(), self.radius)
                if enemys then 
                    for _, defender in pairs(enemys) do
                        if self.hit_enemy[defender.id] == nil and actor.target_actor == defender then
                            self.hit_enemy[defender.id] = 1
                            local dmg, is_hit, is_crit, curr_hp = actor:calculateDamage(defender, self.skill_cfg, actor.cast_inst_id)
                            defender:beAttacked(actor, self.skill_cfg, dmg, is_hit, is_crit, actor.cast_inst_id, game_info.is_replay, curr_hp)
                            
                            if is_hit then 
                                hit_enemy = true 
                                actor.last_hit_actor = defender
                            end
                            if self.hit_effect then
                                battle_helper:playFXOnActorPos(self.hit_effect.effect_path, defender, defender:getPosition(), self.hit_effect.effect_time)
                            end
                        end
                    end
                end

                if hit_enemy then
                    self:stopRunning(actor)
                    return states.SUCCESS
                end
            end
        else
            if actor.target_actor then
                if actor.last_hit_actor == actor.target_actor then
                    self:stopRunning(actor)
                    return states.SUCCESS
                end
            else
                self:stopRunning(actor)
                return states.SUCCESS
            end
        end
        return states.RUNNING
    end,

    -- todo, 播放录像时候这个不会被调用， 要手动删除 effect
    stopRunning = function(self, actor)
        -- 播放结束特效
        if self.stop_effect then
           battle_helper:playFXOnPos(self.stop_effect.effect_path, actor:getPosition(), self.stop_effect.effect_time)
        end

        if self.loop_effect_fx then
            self.loop_effect_fx:destroy()
            self.loop_effect_fx = nil
        end
        self.inited = false
    end,

}