local log = logger.get("Game", "BT")
local super = require('bt.action')
local states = super.states
local damage_calculator = require('RTS.prop_sys.damage_calculator')

-- 冲锋
return class {
    typename = 'action: rush',
    super = super,
    ctor = function(self, ...)
        super.ctor(self)
        self.shoot_effect_id = select(1, ...)   -- 
        self.bind_point = select(2, ...)
        self.speed = select(3, ...)     -- 冲刺速度 
        self.radius = select(4, ...)    -- 冲刺时的伤害半径
        self.dist = select(5, ...)      -- 冲刺距离 
        self.time = select(6, ...)      -- 超时时间
        self.hit_speed = select(7, ...) -- 受击速度

        local _cfg = config.shoot_effect_shoots[self.shoot_effect_id]
        -- "start","loop","stop"
        if _cfg.start and _cfg.start > 0 then self.start_effect = config.shoot_effect_effect_libs[_cfg.start] end 
        if _cfg.loop and _cfg.loop > 0 then self.loop_effect = config.shoot_effect_effect_libs[_cfg.loop] end
        if _cfg.stop and _cfg.stop > 0 then self.stop_effect = config.shoot_effect_effect_libs[_cfg.stop] end

        --log.info(self.offset..","..self.length..','..self.width)

        self.inited = false
        self.elapsed = 0.0

        self.check_elapsed = 0.0
        self.hit_enemy = {}

    end,

    setContext = function (self, ctx)
        self.ctx = ctx
        self.skill_cfg = ctx.skill_cfg

        if type(self.speed) == 'string' then
            self.speed = self.skill_cfg[self.speed]
        end

        if type(self.radius) == 'string' then
            self.radius = self.skill_cfg[self.radius]
        end

        if type(self.dist) == 'string' then
            self.dist = self.skill_cfg[self.dist]
        end
        self.dist_sq = self.dist * self.dist

        if type(self.time) == 'string' then
            self.time = self.skill_cfg[self.time]
        end

        if type(self.hit_speed) == 'string' then
            self.hit_speed = self.skill_cfg[self.hit_speed]
        end

    end, 

    reset = function(self)
        if self.inited then
            self.inited = false
            self:stopRush(self.ctx)
        end

        --log.info('################# rush reset!!')
    end,

    update = function(self, dt)
        local actor = self.ctx
        local skill_state = actor.curr_state
        assert(skill_state.name == 'skill')

        --log.info('################# rush!!')
        -- 初始化
        if not self.inited then
            self.inited = true
            local pos = actor:getPosition()
            self.orig_pos = Vector3.New(pos.x, pos.y, pos.z)

            -- 播放开火特效
            if self.start_effect then
                battle_helper:playFXAttached(self.start_effect.effect_path, actor, self.bind_point, self.start_effect.effect_time)
            end
            -- 播放持续特效
            if self.loop_effect then
                self.loop_vfx = battle_helper:playFXAttached(self.loop_effect.effect_path, actor, self.bind_point, self.time)
            end

            local rot = skill_state.skill_rotation or actor:getRotation()
            local dir = Quaternion.Mul(rot, Vector3.forward)

            actor.last_hit_actor = nil
            actor:sk_beginRush(self.speed, dir, self.hit_effect, self.hit_speed)
            self.hit_enemy = {}
        end
        
        self.elapsed = self.elapsed + dt

        if self.elapsed >= self.time then   -- 冲刺结束
            self:stopRush(actor)
            return states.SUCCESS
        else    
            -- 冲刺过程，每0.1秒进行伤害判定
            self.check_elapsed = self.check_elapsed + dt
            if self.check_elapsed > 0.1 then
            self.check_elapsed = 0.0
                --battle_helper:doCircleDamage(actor, actor:getEnemyTeam(), actor:getPosition(), self.radius, self.skill_cfg)

                local enemys = battle_helper:getIntersectActorsCircle(actor:getEnemyTeam(), actor:getPosition(), self.radius)
                if enemys then 
                    for _, defender in pairs(enemys) do
                    if self.hit_enemy[defender.id] == nil then
                        self.hit_enemy[defender.id] = 1
                        
                        local dmg, is_hit, is_crit, curr_hp = actor:calculateDamage(defender, self.skill_cfg, actor.cast_inst_id)
                            defender:beAttacked(actor, self.skill_cfg, dmg, is_hit, is_crit, actor.cast_inst_id, game_info.is_replay, curr_hp)
                            actor.last_hit_actor = defender
                            if self.hit_effect then
                                battle_helper:playFXOnActorPos(self.hit_effect.effect_path, defender, defender:getPosition(), self.hit_effect.effect_time)
                            end

                            -- 如果对方车重于自己两倍以上，自己进入翻滚
                            -- 石头说服务器有翻滚位移的问题，取消翻滚
                            --[[
                            if defender.cfg.heavy / actor.cfg.heavy >= 2 then
                                actor:changeState("hit_fly")
                            end]]

                        end
                    end
                end

                if Vector3.SqrDistance(self.orig_pos, actor:getPosition()) >= self.dist_sq then
                    --log.info('####### orig:'..tostring(self.orig_pos)..', curr pos:'..tostring(actor:getPosition())..', dist_sq:'..self.dist_sq)
                    self:stopRush(actor)
                    return states.SUCCESS
                end
            end

            return states.RUNNING
        end
    end,

    stopRush = function(self, actor)
        actor:sk_stopRush()
        -- 播放结束特效
        if self.stop_effect then
            battle_helper:playFXOnPos(self.stop_effect.effect_path, actor:getPosition(), self.stop_effect.effect_time)
        end

        if self.loop_vfx then
            self.loop_vfx:destroy()
            self.loop_vfx = nil
        end

        self.inited = false
    end,

}