local log = logger.get("Game", "BT")
local super = require('bt.action')
local states = super.states
local actor_types = require('RTS.enums').actor_types

-- 发射子弹
return class {
    typename = 'action: shoot bullet',
    super = super,
    ctor = function(self, ...) -- bullet_prefab, speed, damage)
        super.ctor(self)
        self.shoot_effect_id = select(1, ...) -- 
        local _cfg = config.shoot_effect_shoots[self.shoot_effect_id]
        if not _cfg then
            log.error("Failed to find shoot_effect_shoots:"..tostring(self.shoot_effect_id))
        end
        self.skin_disappear = _cfg.skin_disappear > 0
        -- "start","loop","stop"
        if _cfg.start ~= 0 then self.start_effect = config.shoot_effect_effect_libs[_cfg.start] end
        if _cfg.loop ~= 0 then self.loop_effect = config.shoot_effect_effect_libs[_cfg.loop] end
        if _cfg.loop_1 ~= 0 then self.loop_body = config.shoot_effect_effect_libs[_cfg.loop_1] end
        --if _cfg.hit ~= 0 then self.hit_effect = config.shoot_effect_effect_libs[_cfg.hit] end
        if _cfg.stop ~= 0 then self.stop_effect = config.shoot_effect_effect_libs[_cfg.stop] end

        self.bind_point = select(2, ...)
        self.speed_param = select(3, ...)
        self.speed = nil
        self.curve = select(4, ...)
        self.damage_radius = select(5, ...) -- 伤害直径
        self.penetration = select(6, ...)    -- 是否穿透
        self.max_dist = select(7, ...) or 0      -- 飞行的最大距离
        self.curve_height = select(8, ...)
        self.cam_anim = select(9, ...)  -- 子弹消失时的摄像机动画名称
        self.stop_effect_follow_target = select(10, ...)    -- stop特效是否跟随目标
    end,

    setContext = function (self, ctx)
        self.ctx = ctx
        self.skill_cfg = ctx.skill_cfg
        if type(self.speed_param) == 'string' then
            self.speed_param = self.skill_cfg[self.speed_param]   -- 从技能表读取真实值
        end

        if type(self.damage_radius) == 'string' then
            self.damage_radius = self.skill_cfg[self.damage_radius] 
        end

        if type(self.max_dist) == 'string' then
            self.max_dist = self.skill_cfg[self.max_dist]
        end

        -- 从 max_dist 中减去坦克中心点到开火点的 距离
        local center_shoot_dist = Vector3.Distance(ctx:getPosition(), ctx.shoot_point.position)
        self.max_dist = self.max_dist - center_shoot_dist
    end, 

    -- 
    update = function(self, dt)
        local actor = self.ctx

        if actor.actor_type == actor_types.SOLDIER then
            --log.error("soldier could not shoot bullet, please call shoot_bullet_simple!")
            --return
        end

        local from = actor.shoot_point.position
        if self.bind_point then
            from = actor.cs_inst:FindChild(self.bind_point).position--actor.shoot_point.position
        end

        local skill_state = actor.curr_state
        local target_actor = skill_state.target_actor

        if type(self.speed_param) == 'function' then
            self.speed = self.speed_param()
        else
            self.speed = self.speed_param
        end

        --assert(skill_state.name == 'skill')
        local to = nil
        if self.penetration then
            -- 穿透型子弹，使用最大距离作为目标点
            if skill_state then
                self.damage_rot = skill_state.skill_rotation or actor:getRotation()
                local direction = Quaternion.Mul(self.damage_rot, Vector3.forward)
                to = from + direction * self.max_dist
            else 
                to = skill_state.target_actor:getPosition()
            end
        else
            -- 普通子弹，用目标敌人的位置
            if not skill_state.target_actor then
                return states.SUCCESS
            end

            to = target_actor:getPosition()
            -- 判断是打到车表面还是车周围
            if math.random() > self.skill_cfg.deflect_percent then
                -- 子弹在表面消失
                if self.skin_disappear then
                    local direction = to - from
                    direction:Normalize()
                    local dist = Vector3.Distance(from, to)
                    dist = dist - target_actor:getRadius()    -- 打到车辆表面，作为子弹停止位置
                    to = from + direction*dist
                end

                -- 高度调整
                to.y = from.y
                if target_actor then
                    to.y = to.y + (target_actor.height - actor.height)
                end

                if self.skill_cfg.hit_effect > 0 then
                    self.hit_effect = config.shoot_effect_effect_libs[self.skill_cfg.hit_effect]
                end
            else
                -- 打到车的周围，选择随机偏移
                local dir = Vector3.New(math.random(-1,1), 0, math.random(-1,1))
                dir:Normalize()
                to = to + dir * (target_actor:getRadius() + self.skill_cfg.deflect_radius)

                if self.skill_cfg.deflect_effect > 0 then
                    self.hit_effect = config.shoot_effect_effect_libs[self.skill_cfg.deflect_effect]
                end
            end
        end

        -- ["shoot_effect_effect_libs"]={"effect_id","effect_path","audio","effect_time",},
        -- 播放开火特效
        if self.start_effect then
            battle_helper:playFXAttached(self.start_effect.effect_path, actor, self.bind_point, self.start_effect.effect_time)
        end

        local prefab_path = self.bullet_prefab or actor.bullet_prefab
        --[[
        log.info("shoot bullet:"..",".. self.loop_effect..
        ",".. self.loop_effect..
        ",".. self.loop_body..
        ",".. self.hit_effect..
        ","..stop_effect..tostring(from)..tostring(to)..self.curve)]]
    
        
        actor:sk_shootBullet(target_actor, self.loop_effect, self.loop_body, self.hit_effect, self.stop_effect, 
            from, to, self.speed or 50, self.curve, self.damage_radius, self.penetration, self.max_dist, 
            self.curve_height, self.cam_anim, self.stop_effect_follow_target)

        return states.SUCCESS
     end,
}