local log = logger.get("Game", "BT")
local super = require('bt.action')
local states = super.states

-- 向前发射穿透子弹
return class {
    typename = 'action: shoot bullet',
    super = super,
    ctor = function(self, ...) -- bullet_prefab, speed, damage)
        super.ctor(self)
        self.shoot_effect_id = select(1, ...) -- 
        local _cfg = config.shoot_effect_shoots[self.shoot_effect_id]
        self.skin_disappear = _cfg.skin_disappear > 1
        -- "start","loop","stop"
        if _cfg.start ~= 0 then self.start_effect = config.shoot_effect_effect_libs[_cfg.start] end
        if _cfg.loop ~= 0 then self.loop_effect = config.shoot_effect_effect_libs[_cfg.loop] end
        if _cfg.loop_1 ~= 0 then self.loop_body = config.shoot_effect_effect_libs[_cfg.loop_1] end
        --if _cfg.hit ~= 0 then self.hit_effect = config.shoot_effect_effect_libs[_cfg.hit] end
        if _cfg.stop ~= 0 then self.stop_effect = config.shoot_effect_effect_libs[_cfg.stop] end

        self.bind_point = select(2, ...)
        self.speed = select(3, ...)
        self.curve = select(4, ...)
        self.max_dist = select(5, ...)      -- 飞行的最大距离
        self.damage_radius = select(6, ...)     -- 伤害宽度
        self.penetration = true
    end,

    setContext = function (self, ctx)
        self.ctx = ctx
        self.skill_cfg = ctx.skill_cfg
        if type(self.speed) == 'string' then
            self.speed = self.skill_cfg[self.speed]   -- 从技能表读取真实值
        end

        if type(self.damage_radius) == 'string' then
            self.damage_radius = self.skill_cfg[self.damage_radius] 
        end

        if type(self.max_dist) == 'string' then
            self.max_dist = self.skill_cfg[self.max_dist]
        end
    end, 

    -- 
    update = function(self, dt)
        local actor = self.ctx
        local from = actor.shoot_point.position

        local skill_state = actor.curr_state
        local target_actor = skill_state.target_actor

        self.damage_rot = actor.init_rot
        local direction = Quaternion.Mul(self.damage_rot, Vector3.forward)
        to = from + direction * self.max_dist

        -- ["shoot_effect_effect_libs"]={"effect_id","effect_path","audio","effect_time",},
        -- 播放开火特效
        if self.start_effect then
            battle_helper:playFXAttached(self.start_effect.effect_path, actor, self.bind_point, self.start_effect.effect_time)
        end

        local prefab_path = self.bullet_prefab or actor.bullet_prefab
        --[[
        log.info("shoot bullet:"..",".. self.loop_effect..
        ",".. self.loop_effect..
        ",".. self.loop_body..
        ",".. self.hit_effect..
        ","..stop_effect..tostring(from)..tostring(to)..self.curve)]]
    
        
        actor:sk_shootBullet(target_actor, self.loop_effect, self.loop_body, self.hit_effect, self.stop_effect, 
            from, to, self.speed or 50, self.curve, self.damage_radius, self.penetration, self.max_dist, self.curve_height, self.cam_anim)

        return states.SUCCESS
     end,
}