local log = logger.get("Game", "BT")
local super = require('bt.action')
local states = super.states

-- 发射简单子弹
return class {
    typename = 'action: shoot bullet simple',
    super = super,
    ctor = function(self, ...) -- bullet_prefab, speed, damage)
        super.ctor(self)
        self.shoot_effect_id = select(1, ...) -- 
        local _cfg = config.shoot_effect_shoots[self.shoot_effect_id]
        if not _cfg then
            log.error("Failed to find shoot_effect_shoots:"..tostring(self.shoot_effect_id))
        end
        self.skin_disappear = _cfg.skin_disappear > 0
        -- "start","loop","stop"
        if _cfg.loop ~= 0 then 
            self.loop_effect = config.shoot_effect_effect_libs[_cfg.loop]
            if not self.loop_effect then
                log.error("Failed to find loop effect, shoot_effect_id:"..tostring(self.shoot_effect_id)..", loop:"..tostring(_cfg.loop))
            end 
        end

        if _cfg.stop ~= 0 then self.stop_effect = config.shoot_effect_effect_libs[_cfg.stop] end

        self.bind_point = select(2, ...)
        self.speed_param = select(3, ...)
        self.speed = nil
        self.curve_height = select(4,...) or 10
    end,

    setContext = function (self, ctx)
        self.ctx = ctx
        self.skill_cfg = ctx.skill_cfg
        if type(self.speed_param) == 'string' then
            self.speed_param = self.skill_cfg[self.speed_param]   -- 从技能表读取真实值
        end
    end, 

    -- 
    update = function(self, dt)
        local actor = self.ctx

        if not actor.shoot_point then
            log.error("Actor have no shoot point:"..tostring(actor.cfg_id))
        end

        local from = actor.shoot_point.position
        

        --[[
        if self.bind_point then
            from = actor.cs_inst:FindChild(self.bind_point).position--actor.shoot_point.position
        end]]

        local skill_state = actor.curr_state
        local target_actor = skill_state.target_actor

        self.speed = self.speed_param

        --assert(skill_state.name == 'skill')
        local to = target_actor:getPosition()
        local dir = Vector3.New(math.random(-1,1), math.random(-1,1), math.random(-1,1)) * 2
        to = to + dir * (target_actor:getRadius())

        -- 普通子弹，用目标敌人的位置
        if not skill_state.target_actor then
            return states.SUCCESS
        end

        -- 高度调整
        to.y = from.y
        if target_actor then
            to.y = to.y + (target_actor.height - actor.height)
        end
        actor:sk_shootBulletSimple(self.loop_effect.effect_path, self.stop_effect, from, to, self.speed, self.curve_height)

        target_actor:beAttacked(self.owner)
        
        return states.SUCCESS
     end,
}