local log = logger.get("Game", "BT")
local super = require('bt.action')
local states = super.states

-- 向地方阵地随机位置发射一颗子弹，落地后在技能选择位置进行圆形范围伤害
return class {
    typename = 'action: shoot bullet',
    super = super,
    ctor = function(self, ...) -- bullet_prefab, speed, damage)
        super.ctor(self)

        -- shoot effect id, 绑定点, 发射速度， 弹道曲线，伤害半径

        local shoot_effect_id = select(1, ...) -- 
        local _cfg = config.shoot_effect_shoots[shoot_effect_id]
        -- "start","loop","stop"
        if _cfg.start ~= "" then self.start_effect = config.shoot_effect_effect_libs[_cfg.start] end
        if _cfg.loop ~= "" then self.loop_effect = config.shoot_effect_effect_libs[_cfg.loop] end
        if _cfg.loop_1 ~= 0 then self.loop_body = config.shoot_effect_effect_libs[_cfg.loop_1] end
        --if _cfg.hit ~= "" then self.hit_effect = config.shoot_effect_effect_libs[_cfg.hit] end
        if _cfg.stop ~= "" then self.stop_effect = config.shoot_effect_effect_libs[_cfg.stop] end

        self.bind_point = select(2, ...)
        self.speed = select(3, ...)
        self.curve = select(4, ...)
        self.damage_radius = select(5, ...)
        self.curve_height = select(6, ...)
        self.cam_anim = select(7, ...)  -- 子弹消失时的摄像机动画名称

    end,

    setContext = function (self, ctx)
        self.ctx = ctx
        self.skill_cfg = ctx.skill_cfg

        if type(self.speed) == 'string' then
            self.speed = self.skill_cfg[self.speed] or self.speed   -- 从技能表读取真实值
        end

        if type(self.damage_radius) == 'string' then
            self.damage_radius = self.skill_cfg[self.damage_radius] or self.damage_radius 
        end
    end, 

    -- 
    update = function(self, dt)
        local actor = self.ctx
        local skill_state = actor.curr_state
        assert(skill_state.name == 'skill')


        -- 选择随机敌方位置
        self.damage_pos = nil
        if skill_state.pos_list then
            if skill_state.pos_idx <= #skill_state.pos_list then
                self.damage_pos = skill_state.pos_list[skill_state.pos_idx]
                skill_state.pos_idx = skill_state.pos_idx + 1
            end
        end

        if not self.damage_pos then self.damage_pos = battle_helper:selectRandomPosOfTeam(actor:getEnemyTeam()) end
        if not self.damage_pos then
            --log.info('!!!no damage pos:'..tostring(self.damage_pos))
            return states.SUCCESS 
        end

        --log.info('damage pos:'..tostring(self.damage_pos))

        local from = actor.shoot_point.position
        -- ["shoot_effect_effect_libs"]={"effect_id","effect_path","audio","effect_time",},
        -- 播放开火特效
        if self.start_effect then 
            battle_helper:playFXAttached(self.start_effect.effect_path, actor, self.bind_point, self.start_effect.effect_time)
        end

        local to = self.damage_pos
         --log.info('damage radius:'..tostring(self.damage_radius))
        actor:sk_shootBulletDamageInArea(self.loop_effect, self.loop_body, self.hit_effect, self.stop_effect, nil, from, to, self.speed, 
        self.curve, self.damage_radius*1.01, self.curve_height, self.cam_anim, 0.0)
        --actor:sk_shootBullet(actor.target_actor, self.loop_effect, self.stop_effect, from, to, self.speed or 50)

        return states.SUCCESS
     end,
}