local log = logger.get("Game", "BT")
local super = require('bt.action')
local states = super.states

-- 发射子弹
return class {
    typename = 'action: shoot laser',
    super = super,
    ctor = function(self, ...) -- bullet_prefab, speed, damage)
        super.ctor(self)
        self.shoot_effect_id = select(1, ...) 
        local _cfg = config.shoot_effect_shoots[self.shoot_effect_id]
        self.skin_disappear = _cfg.skin_disappear > 1
        -- "start","loop","stop"
        if _cfg.start ~= 0 then self.start_effect = config.shoot_effect_effect_libs[_cfg.start] end
        if _cfg.loop ~= 0 then self.loop_effect = config.shoot_effect_effect_libs[_cfg.loop] end
        --if _cfg.loop_1 ~= 0 then self.loop_body = config.shoot_effect_effect_libs[_cfg.loop_1] end
        if _cfg.stop ~= 0 then self.stop_effect = config.shoot_effect_effect_libs[_cfg.stop] end

        self.bind_point = select(2, ...)
        self.no_damage = select(3, ...)
    end,

    setContext = function (self, ctx)
        self.ctx = ctx
        self.skill_cfg = ctx.skill_cfg
    end, 

    -- 
    update = function(self, dt)
        local actor = self.ctx

        local from_transform = actor.shoot_point
        if self.bind_point and self.bind_point ~= "shoot" then
           from_transform =  actor.cs_inst:FindChild(self.bind_point)
           if not from_transform then
            log.error("Failed to find bind_point:"..tostring(self.bind_point))
           end
        end

        local skill_state = actor.curr_state
        local target_actor = skill_state.target_actor
        if not target_actor then
            return states.SUCCESS
        end

        if self.skill_cfg.hit_effect > 0 then
            self.hit_effect = config.shoot_effect_effect_libs[self.skill_cfg.hit_effect]
        end
        -- ["shoot_effect_effect_libs"]={"effect_id","effect_path","audio","effect_time",},
        -- 播放开火特效
        if self.start_effect then
            local force = self.start_effect.effect_important > 0
            battle_helper:playFXAttached(self.start_effect.effect_path, actor, self.bind_point, self.start_effect.effect_time, force)
        end
        to_transform = target_actor.cs_inst:FindChild("hit")
        if not to_transform then
            log.error("Failed to find hit in target:"..target_actor.cfg.id)
            to_transform = target_actor.transform
        end
        --[[
        log.info("shoot bullet:"..",".. self.loop_effect..
        ",".. self.loop_effect..
        ",".. self.loop_body..
        ",".. self.hit_effect..
        ","..stop_effect..tostring(from)..tostring(to)..self.curve)]]
    
        
        actor:sk_shootLaser(target_actor, self.bind_point or "shoot", self.loop_effect, self.hit_effect, from_transform, to_transform)

        -- 计算伤害
        if not self.no_damage then
            local cast_inst_id = actor.cast_inst_id
            local dmg, is_hit, is_crit, curr_hp = actor:calculateDamage(target_actor, self.skill_cfg, cast_inst_id)
            target_actor:beAttacked(actor, self.skill_cfg, dmg, is_hit, is_crit, cast_inst_id, game_info.is_replay, curr_hp)
        end
        return states.SUCCESS
     end,
}