local log = logger.get("Game", "RTSStage")
local super = require('common.state_machine')
rts_helper = require('RTS.rts_helper')
battle_helper = require('RTS.battle_helper')
local list = require('base.list')
cheat = require('RTS.cheat')
local actor_types = require('RTS.enums').actor_types
local battle_types = require('RTS.enums').battle_types
local skills = require('RTS.skill_lib._skill_bt')

require('base.timer')

-- 对一场战斗的封装，包括战斗资源加载，战斗规则，战斗状态/过程控制，战斗结算
-- 
return class {
    inst = nil, -- 唯一实例

    extensions = { "RTS/stage/rts_stage+war_mgr.lua"}, 

    -- ctor = function (self, stage_id, battle_id, replay_data)

    ctor = function (self, eType, args)
        -- 父类的构造
        self.states = {}
        self.curr_state = nil
        self.result = nil

        -- 初始化容器
        self.actors = {}

        -- 初始化状态变量
        self.paused = true
        self.actor_initialized = false

        self.gameObject = GameObject("scene")
        GameObject.DontDestroyOnLoad(self.gameObject);
        self.transform = self.gameObject.transform

        local stage_id = args.stage_id
        self.battle_id = args.battle_id
        local replay_data = args.replay_data
        self.recruit_data = args.recruit_data
        self.eType = eType

        if eType == battle_types.TEST then           -- 测试战斗
            self.stage_cfg = gConfig.stages[stage_id];
            self.stage_cfg = self:cookStageCfg(self.stage_cfg, eType)
        elseif eType == battle_types.LEVEL then      -- 关卡战斗
            self.stage_cfg = gConfig.stages_level[stage_id];
            self.stage_cfg = self:cookStageCfg(self.stage_cfg, eType)
        elseif eType == battle_types.REPLAY then     -- 回放战斗
            self.stage_cfg = replay_data:cookStageCfg()
        elseif eType == battle_types.RECRUIT then    -- 招募战斗
            self.stage_cfg = gConfig.stages[9999];
            self.stage_cfg = self:cookStageCfg(self.stage_cfg, eType)
        end

        if eType == battle_types.REPLAY then
            self.is_replay = true
            self.ai_world = require('RTS.replay.replay_runtime').new(replay_data, self.actors)
        else
            self.is_replay = false
            self.ai_world = require('RTS.ai.world').new()
            RTSHelper.RandomSeed(1, 1)
        end

        game_info.is_replay = self.is_replay

        self.ai_world:setPaused(true)
        self.battle_time = 0.0

        rts_helper:init()
        battle_helper:init(self)
        cheat.stage = self

        self:ext_ctor()

        -- 注册事件处理函数
        self.events = require("base.event").new()

        -- 关卡加载完成回调
        self.events:on('level_loaded', function()
            -- log.info('level loaded!')
            local go = GameObject.Find('LevelManager')
            if go ~= nil then 
                self.level_manager = go:GetComponent('LevelManager')    -- cs 组件，预先缓存关卡中的重要对象
                self.shadow_projector = self.level_manager.shadowProjector
                if self.shadow_projector then
                    self.shadow_projector.gameObject:SetActive(false)
                end
                self.tank_root = self.level_manager.battleTankRoot
                --self.soldier_root = self.level_manager.battleSoldierRoot

                -- self.gui = CreateFrame {
                --     class = "gui.battleui", 
                --     parent = game_director.gameObject,
                --     name = "战斗内界面", 
                --     asset = consts.assets.battleui,
                --     stage = self, 
                -- }

                self.gui = frame_manager:show('battle', self)

                app:startCoroutine(function ()
                    coroutine.yield(frame_manager:waitForFrame(self.gui))
                    --coroutine.yield(nil)
                    --self.gui:setActiveHint()
                    for _,actor in pairs(self.actors) do
                        self.gui:createVehicle(actor)
                    end
                end )

            else
                log.error('无法在关卡中找到 LevelManager 对象')
            end
        end)

        -- 出生 actor 回调
        self.events:on('actor_spawned', function(actor)

            log.info("actor_spawned:"..actor.id..","..actor.name)
            self.actors[actor.id] = actor
            if actor.ai_name then
                self.ai_world:createAI(actor)
            end
        end)

        -- actor 资源加载完成回调
        self.events:on("actor_loaded", function (actor)
        end)

        -- actor 死亡回调
        self.events:on("actor_dead", function (actor)
            if actor.ai_name then
                self.ai_world:removeAI(actor)
            end
        end)

        -- actor 溃逃回调
        self.events:on("actor_escaped", function (actor)
            --log.info('actor_loaded:'..actor.name)
            if actor.transform then
                --self.shadow_projector:RemoveCaster(actor.transform)
                actor:hideGameObject(0.1)
            end

            if actor.ai_name then
                self.ai_world:removeAI(actor)
            end
        end)

    end,

    super = super,

    -- 初始化状态
    init = function(self)
        self.states['loading'] = require('RTS.stage.rts_state_loading').new('loading', self);
        self.states['entrance'] = require('RTS.stage.rts_state_entrance').new('entrance', self);
        self.states['battle'] = require('RTS.stage.rts_state_battle').new('battle', self);
        --self.states['skill'] = require('RTS.stage.rts_state_skill').new('skill', self); 

        --self:changeState('loading')
        -- 等待 battle_helper 初始化完成，切换到 loading
        app:startCoroutine(
            function()
                while true do
                    if battle_helper.inited then
                        self:changeState('loading')
                        break
                    else
                        coroutine.yield(nil)
                    end
                end
            end
        )
    end,

    -- 根据服务器的战斗数据更新 stage 配置
    cookStageCfg = function(self, orig, eType)
        local fmt_cfg_id_t0
        local fmt_cfg_id_t1 
        local formation_t0
        local formation_t1

        if level_info.map_attack_info then
            fmt_cfg_id_t0 = level_info.map_attack_info.playerFormation
            fmt_cfg_id_t1 = level_info.map_attack_info.monsterFormation
            formation_t0 = level_info.formation_t0
            formation_t1 = level_info.formation_t1
        end

        if eType == battle_types.TEST then           -- 测试战斗
        elseif eType == battle_types.LEVEL then      -- 关卡战斗
        elseif eType == battle_types.REPLAY then     -- 回放战斗
        elseif eType == battle_types.RECRUIT then    -- 招募战斗
            fmt_cfg_id_t0 = self.recruit_data.fmt_cfg_id_t0
            fmt_cfg_id_t1 = self.recruit_data.fmt_cfg_id_t1
            formation_t0 = self.recruit_data.formation_t0
            formation_t1 = self.recruit_data.formation_t1
        end

        -- if not level_info.map_attack_info then
        --     log.info("## no map_attack_info!!")
        --     return orig
        -- end

        local origT0 = orig.T0
        local origT1 = orig.T1

        local T0 = {}
        local T1 = {}
        
        local cfg = {
            scene = orig.scene,
            --offset = orig.offset,
            speedT0 = orig.speedT0,
            speedT1 = orig.speedT1,
            vectorT0 = orig.vectorT0,
            vectorT1 = orig.vectorT1,
            T0 = T0,
            T1 = T1,
            S0 = orig.S0,
            S1 = orig.S1,
        }

        --[[
        local function cookHeroTeam(fightHeroPropArray, dest, src)
            for k,v in ipairs(fightHeroPropArray) do 
                local line = src[k]
                if line then
                    dest[k] = {pos = line.pos, yaw = line.yaw, tank_cfg_id = v.tankId, level = v.HeroLevel, props = v.ParPropIdValues, weapon = v.WeaponInfos}
                else
                    log.error('stage cfg 中缺少坦克配置！')
                end
            end
        end]]

        -- 根据 tank 配置找到阵型配置偏移
        local function getFormationOffset(form_pos_cfg, tank_cfg_id, team)

            local tank_cfg = config.tank_basic_ids[tank_cfg_id]
            if not tank_cfg then
                log.error("Failed to find tank cfg:"..tostring(tank_cfg_id))
                return {0,0}
            end

            local str_tmp = nil
            if tank_cfg.type == "type1" then -- heavy
                if team == 0 then
                    str_tmp = form_pos_cfg.heavy_T0_fix
                else
                    str_tmp = form_pos_cfg.heavy_T1_fix
                end
            elseif tank_cfg.type == "type2" then    -- assult
                if team == 0 then
                    str_tmp = form_pos_cfg.assult_T0_fix
                else
                    str_tmp = form_pos_cfg.assult_T1_fix
                end
            elseif tank_cfg.type == "type3" then    -- tank
                if team == 0 then
                    str_tmp = form_pos_cfg.tank_T0_fix
                else
                    str_tmp = form_pos_cfg.tank_T1_fix
                end
            elseif tank_cfg.type == "type4" then    -- artillery
                if team == 0 then
                    str_tmp = form_pos_cfg.artillery_T0_fix
                else
                    str_tmp = form_pos_cfg.artillery_T1_fix
                end
            else
                log.error("Failed to find tank type:"..tank_cfg.type..", id:"..tank_cfg_id)
                str_tmp = "{0,0,}"
            end

            if str_tmp == "{0,0,}" then
                return {0,0}
            else
                local tab = loadstring("return "..str_tmp)()
                if tab == 0 then
                    tab = {0,0}
                end
                return tab
            end
        end

        local function cookHeroTeam(formation, dest, team)
            for i=1,16 do
                local v = formation[i]  -- 上阵的英雄列表
                local form_pos = config.formation_positions[i]
                if v then
                    local offset = getFormationOffset(form_pos, v.tankId, team)

                    if team == 0 then
                        local pos = {form_pos.posX_T0 + offset[1], form_pos.posY_T0 + offset[2]}
                        local yaw = form_pos.yawT0
                        dest[i] = {pos = pos, yaw = yaw, tank_cfg_id = v.tankId, hero_id = v.HeroId, officer_cfg_id = v.officeId, 
                            level = v.HeroLevel, props = v.ParPropIdValues, weapon = v.WeaponInfos, monster_stat_id = v.monsterStatId}
                    else
                        local pos = {form_pos.posX_T1 + offset[1],  form_pos.posY_T1 + offset[2]}
                        local yaw = form_pos.yawT1
                        if level_info.is_attack_from_back then
                            yaw = yaw + 180
                        end
                        dest[i] = {pos = pos, yaw = yaw, tank_cfg_id = v.tankId, hero_id = v.HeroId, officer_cfg_id = v.officeId, 
                            level = v.HeroLevel, props = v.ParPropIdValues, weapon = v.WeaponInfos, monster_stat_id = v.monsterStatId}
                    end
                else
                    if team == 0 then
                        local pos = {form_pos.posX_T0, form_pos.posY_T0 }
                        local yaw = form_pos.yawT0
                        dest[i] = {pos = pos, yaw = yaw}
                    else
                        local pos = {form_pos.posX_T1,  form_pos.posY_T1 }
                        local yaw = form_pos.yawT1
                        dest[i] = {pos = pos, yaw = yaw}
                    end
                end
            end
        end

        cfg.formationT0 = fmt_cfg_id_t0
        cfg.formationT1 = fmt_cfg_id_t1

        cookHeroTeam(formation_t0, T0, 0)
        cookHeroTeam(formation_t1, T1, 1)


        cfg.areaT0 = orig.areaT0
        cfg.areaT1 = orig.areaT1

        --log.info("T0:"..cfg.formationT0..", T1:"..cfg.formationT1)
        return cfg;
    end,
    
    setPaused = function(self, paused)
        self.paused = paused
        self.ai_world:setPaused(paused)
    end,

    -- 是否所有 actor 都初始化完成
    isActorsInitialized = function(self)
        if self.actor_initialized then
            return self.actor_initialized
        end

        local inited = true
        for k,v in pairs(self.actors) do
            if not v.is_initialized then
                inited = false
                break
            end
        end

        self.actor_initialized = inited

        return inited
    end,

    -- 玩家准备施放技能
    tryCastUltimateSkill = function(self, actor_id)
        
        local actor = self.actors[actor_id]

        if not actor then 
            log.error('Failed to find actor to cast skill:'..actor_id)
        else
            self:fastCastUltimateSkill(actor)
        end
        
        --[[
        print("玩家准备施放技能...", self.curr_state.name)   
        local actor = self.actors[actor_id]
        if actor then 
            self:changeState('skill', {actor, actor:getSkillID()})
        else
            log.error('释放技能失败:'.. actor_id)
        end]]
    end,

    -- 快速释放技能
    fastCastUltimateSkill = function(self, actor)
        local caster = actor or self.actor
        local enemy = actor.target_actor
        if not enemy or not enemy:isAlive() then
            enemy = battle_helper:selectNearestVehicle(caster:getPosition(), caster:getEnemyTeam())
        end
        self.selected_actor = enemy
        
        if enemy then
            local skill_id = caster:getSkillID()
            local skill_cfg = config.skill_skills[skill_id]

            if skill_cfg then

                if skill_cfg.skill_aoe > 0 then
                    local rst, msg = caster:changeStateImmediate('skill', {id=skill_id, ta=nil, p=enemy:getPosition(), r=caster:getRotation()})
                    if not rst then
                        log.info('actor:changeState skill, cast untimate skill:'..msg)
                    end
                elseif self.selected_actor then
                    log.info('cast skill, target id:'..self.selected_actor.id)
                    local rst, msg = caster:changeStateImmediate('skill', {id=skill_id, ta=self.selected_actor, p=self.selected_actor:getPosition(), r=self.selected_actor:getRotation()})
                    if not rst then
                        log.info('actor:changeState skill, cast untimate skill:'..msg)
                    end
                end
            end
        end
    end,

    -- 玩家释放技能
    castUltimateSkill = function(self)
        --[[
        if self.curr_state.name == 'skill' then
            self.curr_state:castUltimateSkill()
        else
            -- 非 skill 状态下的技能释放
            log.error('Can not cast skill in stage state:'..self.curr_state.name)
        end
        ]]
    end,

    cancelSkill = function(self)
        if self.curr_state.name == 'skill' then
            self.curr_state.actor:changeState('idle')
        end
        self:changeState('battle')
    end,

    update = function(self, dt)
        for k,v in pairs(self.actors) do
            --if v:isAlive() then 
                v:preUpdate(dt)
            --end
        end

        super.update(self, dt)  -- 状态机 update
        
        if self.curr_state and self.curr_state.name == 'battle' then
            self.battle_time = self.battle_time + dt
        end
        
        if self.ai_world then
            self.ai_world:update(dt)
        end

        battle_helper:update(dt)

        for k,v in pairs(self.actors) do
            v:update(dt)
        end

    end,

    -- 获取当前英雄血量
    getHerosHp = function (self)
        local infos = {}
        for _, actor in pairs(self.actors) do
            if actor.team == 0 and actor:getActorType() == actor_types.VEHICLE then
                if actor:isAlive() and not actor:isEscape() then
                    table.insert(infos, { cfgId = actor.cfg_id, hero_id = actor.hero_id, curHp = actor.hp, camp = 0 })
                else
                    table.insert(infos, { cfgId = actor.cfg_id, hero_id = actor.hero_id, curHp = 0, camp = 0 })
                end
            end
        end
        return infos
    end, 

    -- 战斗是否结束
    isFinished = function(self)
        if self.curr_state and self.curr_state.name == 'battle' then
            if not self._finished then 
                local team_0_over = true
                local team_1_over = true

                -- 检查 Team 0 是否还有活着没逃跑的
                for _,actor in pairs(self.actors) do
                    if actor:getActorType() ~= actor_types.SOLDIER_GROUP then
                        if actor.team == 0 and actor:isAlive() and not actor:isEscape() then
                            team_0_over = false
                            self.result = 3
                            break
                        end
                    end
                end

                -- 检查 Team 1 是否还有或者没逃跑的
                for _,actor in pairs(self.actors) do
                    if actor:getActorType() ~= actor_types.SOLDIER_GROUP then
                        if actor.team == 1 and actor:isAlive() and not actor:isEscape() then
                            team_1_over = false
                            self.result = 0
                            break
                        end
                    end
                end        
                if team_0_over or team_1_over then 
                    self._finished = true
                end
                -- print(team_0_over, team_1_over, self.result)
            end
            return self._finished
        end
        return false
    end,

    destroy = function(self)
        frame_manager:hide('battle')
        for _,actor in pairs(self.actors) do
            actor:destroy()
        end
        self.actors = nil

        for k,state in pairs(self.states) do
            if state.destroy then
                state:destroy()
            end
        end

        battle_helper:destroy()
        Object.DestroyImmediate(self.gameObject)
        
        if self.ai_world then
            self.ai_world:destroy()
        end
    end,
}