local log = logger.get("Game", "RTSStage")
local soldier_group_class = require('RTS.rts_soldier_group_old')
-- 资源加载状态
return class {
    super = require("common/state"),

    onEnter = function(self)
        -- 加载关卡
        log.info("# rts_state_loading LoadSceneAsync:"..self.owner.stage_cfg.scene)
        self.scene_name = self.owner.stage_cfg.scene

        self.is_begin_load_scene = false
        log.info("### Start ReadSceneInMemory:"..self.scene_name)

        if Fenix.App.GetInstance():IsUseAssetDatabase() then
            self.is_begin_load_scene = true
            self.AsyncOp = SceneManager.LoadSceneAsync(self.scene_name)
        else
            app:startCoroutine(function()
                -- 加载 ab 中的场景
                coroutine.yield(
                    Fenix.Assets.Helper.ReadSceneInMemory('scene_'..self.scene_name..".unity3d", function()
                        log.info("### End ReadStreamingAssetSceneInMemory:"..self.scene_name)
                        self.is_begin_load_scene = true
                        self.AsyncOp = SceneManager.LoadSceneAsync(self.scene_name)

                    end)
                )
            end)
        end

        self.stage = self.owner
        self.ai_world = self.owner.ai_world
        self.actor_guid = 0 -- actor 唯一 id
    end,

    onUpdate = function(self, dt)
        if not self.is_begin_load_scene then
            return
        end

        if self.AsyncOp ~= nil then
            -- 场景加载中
            --print('scene loading')
            if self.AsyncOp.isDone then -- 判断场景是否加载完成
                -- 加载完成
                -- print('scene load finished.')
                
                self.AsyncOp = nil

                self:init(self.owner.stage_cfg)
            end
        else
            -- 判断战车，士兵是否加载完成
            if self.stage:isActorsInitialized() then
                --self.stage:changeState('battle') 
                self.stage:changeState('entrance') 
            end
        end
    end,

    -- 初始化关卡
    init = function(self, stage_cfg)
        self.owner.events:dispatch('level_loaded')
        local pos_list_T0, pos_list_T1 = self:spawnVehicles(self.owner.tank_root, stage_cfg)
        self:spawnSoldiers(self.owner.soldier_root, stage_cfg, pos_list_T0, pos_list_T1)
    end,

    genActorGuid = function(self)
        self.actor_guid = self.actor_guid + 1
        return self.actor_guid
    end,

    -- 出生所有战车, 返回两个 team 战车的位置
    spawnVehicles = function(self, root, stage_cfg)

        local buff0 = config.formation_restriction_buffs[stage_cfg.formationT0]["formation"..stage_cfg.formationT1]
        local buff1 = config.formation_restriction_buffs[stage_cfg.formationT1]["formation"..stage_cfg.formationT0]

        local spawnForTeam = function(tab, team)    -- 出生一个 team 的所有战车
            local pos_list = {}
            for slot_pos, v in ipairs(tab) do
                if v.tank_cfg_id then
                    local pos = Vector3(v.pos[1],0,v.pos[2])
                    local rot = Quaternion.AngleAxis(v.yaw, Vector3.up)
                    
                    if v.tank_cfg_id ~= 0 then
                        -- 根据车辆类型偏移位置
                        local tank_cfg = config.tank_basic_ids[v.tank_cfg_id]
                        if not tank_cfg then
                            log.error("Failed to find tank cfg:"..v.tank_cfg_id)
                        end

                        if stage_cfg.offset then
                            local offset = rot * Vector3.New(0,0,1) * stage_cfg.offset

                            -- replay 时用服务器发的位置，不用偏移
                            if not game_info.is_replay then
                                if tank_cfg.type == "type1" or tank_cfg.type == "type2" then
                                    -- 前排
                                    pos = pos + offset * 2
                                elseif tank_cfg.type == "type3" then
                                    -- 中排
                                    pos = pos + offset
                                end
                            end
                        end

                        -- 创建 actor 实例
                        local inst_id = v.inst_id or self:genActorGuid()
                        local actor = require('RTS.rts_actor').new(v.tank_cfg_id, pos, rot, team, inst_id, self.owner.events, root, v.weapon, slot_pos, v.hero_id, v.officer_cfg_id, v.monster_stat_id)
                        actor.level = v.level or 15
                        actor.init_props = v.props
                        if team == 0 and buff0 > 0 then
                            table.insert(actor.init_buffs, buff0)
                        elseif team == 1 and buff1 > 0 then
                            table.insert(actor.init_buffs, buff1)
                        end

                        self.owner.events:dispatch("actor_spawned", actor)
                        pos_list[slot_pos] = {actor, pos, rot}
                    end
                end
            end
            return pos_list
        end

        local pos_list_T0 = spawnForTeam(stage_cfg.T0, 0)
        local pos_list_T1 = spawnForTeam(stage_cfg.T1, 1)
        
        return pos_list_T0, pos_list_T1
    end,

    -- 出生士兵组
    spawnSoldiers = function(self, root, stage_cfg, pos_list_T0, pos_list_T1)
        
        local spawnForTeam = function(tab, pos_list, team) -- 出生一个 team 的所有士兵组
            for k,v in ipairs(tab) do
                local orig = pos_list[k]    -- {pos, rot}

                if orig then
                    local offset_tab = self:randOffsetTab()   -- 所有 soldier 相对 orig 的偏移数组

                    local id = self:genActorGuid() -- solder group 的 id
                    local id_tab = {}   -- soldier_group 包含的所有 soldier 的 id
                    for i=1, v.num do
                        table.insert( id_tab, self:genActorGuid() )
                    end

                    -- 创建 Soldier Group
                    -- ctor = function(self, cfg_id, owner, team, id, events)
                    local ownerTank = orig[1]
                    local pos = orig[2]
                    local rot = orig[3]
                    local soldier_group = soldier_group_class(v.tank_cfg_id, ownerTank, team, id, self.owner.events)
                    soldier_group.level = v.level or 1
                    soldier_group.init_props = v.props
                    --spawnSoldiers = function(self, cfg_id, orig, rot, offset_tab, num, id_tab, events, root)
                    soldier_group:spawnSoldiers(v.tank_cfg_id, pos, rot, offset_tab, v.num, id_tab, self.owner.events, root)
                    self.owner.events:dispatch("actor_spawned", soldier_group)
                end
            end
        end
        

        if stage_cfg.S0 then
            spawnForTeam(stage_cfg.S0, pos_list_T0, 0)
        end
        if stage_cfg.S1 then
            spawnForTeam(stage_cfg.S1, pos_list_T1, 1)
        end
    end,

    -- 随机一个 offset table
    randOffsetTab = function(self)
        local tab = {}
        local r = RTSHelper.RandomUintInRange(1, table.getn(gConfig.formation_randoms))--math.random(1, table.getn(gConfig.formation_randoms))
        local idx_tab = gConfig.formation_randoms[r]    -- 随机取一张索引表
        for i=2, #idx_tab, 1 do -- 遍历索引表，用索引到位置表中取值
            local idx = idx_tab[i]
            tab[i-1] = config.formation_positions[idx]
        end

        log.info("###### tan len:"..#tab)
        return tab
    end,

    onExit = function(self)
        -- log.info('loading onExit!!')
        -- 输出所有在运行过程加载的文件
        RTSHelper.LogAllLoadFiles()
    end,

    -- stage 销毁时调用
    destroy = function(self)
        Fenix.Assets.Helper.UnloadSceneAB('scene_'..self.scene_name..".unity3d")
    end,

}
