local log = logger.get("Game", "RTSStage")
local skill_area_types = require('RTS.enums').skill_area_types

-- 玩家释放技能状态
return class {
    super = require("common/state"),

    -- 子状态
    sub_states = {
        FAST_CAST = 0,      -- 快速施法
        SHOW_INDICATOR = 1, -- 显示技能指示
        CAST_SKILL = 2,     -- 释放技能
    },

    onEnter = function(self, args)
        local actor = args[1]
        self.actor = actor        -- 释放技能的角色
        self.skill_id = args[2]     -- 技能 id
        self.skill_cfg = config.skill_skills[self.skill_id]

        -- 替换描述中的通配符
        local desc = self.skill_cfg.describe
        for i=1,10,1 do
            local replace = '%(p'..i..'%)'
            local k = 'p'..i
            desc = string.gsub(desc, replace, self.skill_cfg[k])

        end
        self.owner.gui:setSkillInfo(self.skill_cfg.skill_name, desc)

        self.camera = Camera.main
        Time.timeScale = 0.0
        self.first_touch = true
        self.first_update = true
        self.overlap_actors = nil   -- 重叠的角色

        -- 根据伤害范围得到技能显示对象
        local v = nil
        self.skill_area_type, self.indicator_go, v = self:getAreaDisplay(self.skill_cfg.area_display)

        -- 如果是矩形范围，旋转的
        if self.skill_area_type == skill_area_types.RECTANGLE then
            self.indicator_offset = v
        else
            self.indicator_max_dist = v
        end

        self.indicator = self.indicator_go:GetComponent('SkillIndicator')

        if not self.indicator then
            log.error('Failed to find SkillIndicator component!')
        end
        self.indicator.m_Team = self.actor.team
        self.indicator_transform = self.indicator_go.transform
        self.indicator_go:SetActive(false)
        self.owner.level_manager:SetTerrainDark(true)
        self.sub_state = self.sub_states.FAST_CAST

        self.owner.gui:setActiveHint(true)

        -- 隐藏队友信息UI
        battle_helper:setVehicleHeadUIEnable(0, false)
        if self.actor.head_info then
            self.actor.head_info.gameObject:SetActive(true)
        end

        for k,v in pairs(self.owner.actors) do
            if v ~= self.actor then
                if v.cs_inst then v.cs_inst:SetSkillOverlaped(false, true, false) end
            end
        end

        -- 目标 actor 切换到 
        --log.info('try change to preskill!!')
        if actor:changeState('preskill', {id = self.skill_id}) then
        else
            log.info("Change to preskill failed!!")
        end
    end,

    -- 显示技能指示
    showSkillIndicator = function(self)
        self.indicator_go:SetActive(true)
        local actor = self.actor
        if self:_isAoeSkill() then

            if self.skill_area_type ==  skill_area_types.RECTANGLE then
                self.indicator_transform.localPosition = Vector3(0,0,self.indicator_offset)
                self.indicator_transform.rotation = actor:getRotation()
                self.indicator_transform.position = actor:getPosition()
            elseif self.skill_area_type == skill_area_types.CIRCLE then
                local _actor = battle_helper:selectRandomVehicleInRange(self.actor:getPosition(), self.indicator_max_dist, self.actor:getEnemyTeam())
                if _actor then
                    self.indicator_transform.position =_actor:getPosition()
                else
                    self.indicator_transform.position = self.actor:getPosition()
                end
            elseif self.skill_area_type == skill_area_types.SINGLE then
                self.indicator_transform.rotation = actor:getRotation()
                self.indicator_transform.position = actor:getPosition()
                
                self.indicator_line = battle_helper:loadRes(consts.assets.skill_indicator_line).transform
                self.indicator_line.position = actor:getPosition()
                self.indicator_line.rotation = actor:getRotation()

                self.indicator_target = battle_helper:loadRes(consts.assets.skill_indicator_target).transform

                --log.info('indicator line:'..tostring(self.indicator_line))
                --log.info('indicator target:'..tostring(self.indicator_target))
            end
        else
            self.indicator_transform.rotation = actor:getRotation()
            self.indicator_transform.position = actor:getPosition()
            
            self.indicator_line_go = battle_helper:loadRes(consts.assets.skill_indicator_line)
            self.indicator_line = self.indicator_line_go.transform
            self.indicator_line.position = actor:getPosition()
            self.indicator_line.rotation = actor:getRotation()

            self.indicator_target_go = battle_helper:loadRes(consts.assets.skill_indicator_target)
            self.indicator_target = self.indicator_target_go.transform

            self.selected_actor = battle_helper:selectRandomVehicleInRange(self.actor:getPosition(), self.indicator_max_dist, self.actor:getEnemyTeam())
            self.indicator_transform.position = self.actor:getPosition()
        end

        -- 刷新显示
        for k,v in pairs(self.owner.actors) do
            if v.cs_inst then v.cs_inst:SetSkillOverlaped(false, true, false) end   -- overlaped, dark, show_skill_selected_indicator
        end

    end,

    -- 根据伤害范围得到技能显示类型 和 技能显示范围对象
    getAreaDisplay = function(self, damage_area)
        _area_display_fun_tab = {
            do_damage_in_rect = function(self, offset, length, width)
                local _offset = offset
                if type(offset) == 'string' then
                    _offset = self.skill_cfg[offset]
                end
                local _len = self.skill_cfg[length] or length
                local _width = self.skill_cfg[width] or width

                go = battle_helper:loadRes(consts.assets.skill_indicator_rect--[['Assets/Prefab/SkillIndicatorRect.prefab']])
                go.transform.localScale = Vector3(_width, 1, _len)
                return skill_area_types.RECTANGLE, go, _offset
            end,

            do_damage_in_circle = function(self, radius, max_dist)
                local _radius = self.skill_cfg[radius] or radius
                local _max_dist = self.skill_cfg[max_dist] or max_dist
                go = battle_helper:loadRes(consts.assets.skill_indicator_circle --[['Assets/Prefab/SkillIndicatorCircle.prefab']])
                go.transform.localScale = Vector3(_radius*2, 1, _radius*2)
                return skill_area_types.CIRCLE, go, _max_dist
            end,

            do_damage_to_single = function(self, max_dist)
                local _max_dist = self.skill_cfg[max_dist] or max_dist
                go = battle_helper:loadRes(consts.assets.skill_indicator_single --[['Assets/Prefab/SkillIndicatorSingle.prefab']])
                go.transform.localScale = Vector3(_max_dist*2, 1, _max_dist*2)
                return skill_area_types.SINGLE, go, _max_dist 
            end,
        }

        _area_display_fun_tab.skill_cfg = self.skill_cfg
        return loadstring('return _area_display_fun_tab:'..damage_area)()

    end,

    onUpdate = function(self, dt)
        --[[
        if self.first_update then
            self.first_update = false
            self:_updateActorDisplay()
            return 
        end]]

        local screen_point = self:getTouchPoint()
        if self.sub_state == self.sub_states.FAST_CAST then
            self:updateFastCast(screen_point, dt)

        elseif self.sub_state == self.sub_states.SHOW_INDICATOR then
            self:updateShowIndicator(screen_point, dt)

        elseif self.sub_state == self.sub_states.CAST_SKILL then

        end

    end,

    updateFastCast = function(self, screen_point, dt)
        if not screen_point then
            -- 手指离开屏幕，快速施法
            self:fastCastUltimateSkill(self.actor)
        else
            -- 是否点击到界面
            if RTSHelper.IsPointerOverUI(screen_point.x, screen_point.y) then
                --log.info('User touch to the UI!!')
                return
            else
                -- 手指离开界面，进入显示技能指示状态
                self:showSkillIndicator()
                self.sub_state = self.sub_states.SHOW_INDICATOR
            end
        end
    end,

    updateShowIndicator = function(self, screen_point, dt)
        if not screen_point then
            -- cast skill
            self:castUltimateSkill(self.actor)
        else
            -- 是否点击到界面
            if RTSHelper.IsPointerOverUI(screen_point.x, screen_point.y) then
                --log.info('User touch to the UI!!')
                return
            end

            local world_point = self:getWorldPoint(screen_point)
            if not world_point then
                return
            end

            local r = Quaternion.LookRotation(world_point-self.actor:getPosition(), Vector3.up)
            self.actor:setRotation(r)

            if self:_isAoeSkill() then -- AOE 技能范围选择
                self:_aoeSkillAreaUpdate(world_point)
            else -- 单体技能目标选择
                self:_singleSkillAreaUpdate(world_point)
            end

            self:_updateActorDisplay()

        end
    end,

    -- 获取 touch 的点
    getTouchPoint = function(self)
        local point = nil
        if (Input.GetMouseButton(0) or Input.touchCount > 0) then
            if Input.GetMouseButton(0) then
                point = Input.mousePosition
            else
                point = Input.GetTouch(0).position
                point.z = 0.0
            end
        end
        return point
    end,

    -- 根据屏幕坐标获取世界坐标
    getWorldPoint = function(self, screen_point)
        local mask = RTSHelper.GetLayerMask(core.layers['Ground'], -1)
        local r = self.camera:ScreenPointToRay(screen_point)
        local rst, hit = UnityEngine.Physics.Raycast(r, hit, 999, mask)
        if rst then
            return hit.point
        else
            return nil
        end
    end,

    -- 是否是 aoe 技能
    _isAoeSkill = function(self)
        return self.skill_area_type == skill_area_types.RECTANGLE or self.skill_area_type == skill_area_types.CIRCLE
    end,

    -- 范围技能目标选择
    _aoeSkillAreaUpdate = function(self, point)

        if self.skill_area_type == skill_area_types.RECTANGLE then
            -- 根据光标位置旋转技能指示范围
            local rot = self.indicator_transform.rotation:Clone()
            local from = self.indicator_transform.position
            local to = point
            to.y = from.y
            rot:SetLookRotation(to - from, Vector3.up)
            self.indicator_transform.rotation = rot
        elseif self.skill_area_type == skill_area_types.CIRCLE then
            -- 根据光标位置定位指示范围
            local dist = Vector3.Distance(point, self.actor:getPosition())
            if  dist <= self.indicator_max_dist then
                self.indicator_transform.position = Vector3(point.x, 0, point.z)
            else
                local dir = point - self.actor:getPosition()
                dir.y = 0
                dir:Normalize()
                local pos = self.actor:getPosition() + dir * self.indicator_max_dist
                pos.y = 0
                self.indicator_transform.position = pos
            end
        end

    end,

    -- 单体技能目标选择
    _singleSkillAreaUpdate = function(self, point)

        local selected_actor = battle_helper:selectNearestVehicle(point, self.actor:getEnemyTeam())
        
        if not selected_actor then
            log.info('no select actor')
            -- 选择最近的一个 tank
            return
        end
        -- 判断距离， 判断team
        if self.actor.team ~= selected_actor.team and
            Vector3.Distance(self.actor:getPosition(), selected_actor:getPosition()) <= self.indicator_max_dist then
            self.selected_actor = selected_actor
            self.overlap_actors = nil
        end

        if self.selected_actor and self.indicator_target then
            self.indicator_target.position = self.selected_actor:getPosition()
            local scale = self.selected_actor.cfg.round * 2
            self.indicator_target.localScale = Vector3.New(scale, 1, scale)
        end

        if self.indicator_line then
            -- 根据光标位置旋转技能指示范围
            local rot = self.indicator_line.rotation:Clone()
            local from = self.indicator_line.position
            local to = point
            to.y = from.y
            rot:SetLookRotation(to - from, Vector3.up)
            self.indicator_line.rotation = rot

            local dist = Vector3.Distance(from, to)
            self.indicator_line.localScale = Vector3.New(1, 1, dist)
        end
    end,

    -- 更新 actor 是否被技能框选中的显示
    _updateActorDisplay = function(self)
        -- 获取技能指示器重叠的角色
        if self:_isAoeSkill() then
            self.overlap_actors = self.indicator:GetOverlapActors(false, true) -- (本队， 敌对)
        end

        -- 刷新显示
        for k,v in pairs(self.owner.actors) do
            if v.cs_inst then v.cs_inst:SetSkillOverlaped(false, true, false) end
        end

        if self:_isAoeSkill() then
            for i=0, self.overlap_actors.Count-1, 1 do 
                local cs_actor = self.overlap_actors:getItem(i)
                cs_actor:SetSkillOverlaped(true, false, true)
            end
        else
            if self.selected_actor and self.selected_actor.cs_inst then
                self.selected_actor.cs_inst:SetSkillOverlaped(true, false, true)
            end
        end

        -- 自己总是亮的
        self.actor.cs_inst:SetSkillOverlaped(true, false, true)
    end,

    fastCastUltimateSkill = function(self, actor)
        local caster = actor or self.actor
        local enemy = actor.target_actor
        if not enemy or not enemy:isAlive() then
            enemy = battle_helper:selectNearestVehicle(caster:getPosition(), caster:getEnemyTeam())
        end
        self.selected_actor = enemy
        
        if enemy then
            local skill_id = caster:getSkillID()
            if self:_isAoeSkill() then
                local rst, msg = caster:changeStateImmediate('skill', {id=skill_id, ta=nil, p=enemy:getPosition(), r=caster:getRotation()})
                if not rst then
                    log.info('actor:changeState skill, cast untimate skill:'..msg)
                end
            elseif self.selected_actor then
                log.info('cast skill, target id:'..self.selected_actor.id)
                local rst, msg = caster:changeStateImmediate('skill', {id=skill_id, ta=self.selected_actor, p=self.selected_actor:getPosition(), r=self.selected_actor:getRotation()})
                if not rst then
                    log.info('actor:changeState skill, cast untimate skill:'..msg)
                end
            end
        end
        
        self.owner:changeState('battle')
    end,

    -- 释放技能，回到战斗状态
    castUltimateSkill = function(self, actor)
        local caster = actor or self.actor

        local skill_id = caster:getSkillID()
        if self:_isAoeSkill() then
            local rst, msg = caster:changeStateImmediate('skill', {id=skill_id, ta=nil, p=self.indicator_transform.position, r=self.indicator_transform.rotation})
            if not rst then
                log.info('actor:changeState skill, cast untimate skill:'..msg)
            end
        elseif self.selected_actor then
            log.info('cast skill, target id:'..self.selected_actor.id)
            local rst, msg = caster:changeStateImmediate('skill', {id=skill_id, ta=self.selected_actor, p=self.selected_actor:getPosition(), r=self.selected_actor:getRotation()})
            if not rst then
                log.info('actor:changeState skill, cast untimate skill:'..msg)
            end
        end

        self.owner:changeState('battle')
    end,

    canChangeTo = function(self, next_state_name, args)
        if next_state_name == 'skill' then
            return false
        else
            return true
        end
    end,

    onExit = function(self)
        if self.indicator_go then
            self.indicator_go:destroy()
            self.indicator_transform = nil
        end
        if self.indicator_line then
            self.indicator_line_go:destroy()
            self.indicator_line = nil
        end
        if self.indicator_target then
            self.indicator_target_go:destroy()
            self.indicator_target = nil
        end

        Time.timeScale = 1.0
        battle_helper:setVehicleHeadUIEnable(0, true)

        -- 刷新显示
        for k,v in pairs(self.owner.actors) do
            if v.cs_inst then v.cs_inst:SetSkillOverlaped(false, false, false) end
        end
        self.owner.level_manager:SetTerrainDark(false)
        self.owner.gui:setActiveHint(false)
    end,

}