local log = logger.get('Game', 'task manager')
local activeness_cls = require('RTS.task_sys.activeness')
local task_cls = require('RTS.task_sys.task')
require ('config.task')
--[[ 
    TASK_STATUS
    {
        TS_NONE,
        TS_GETAWARD,   //可领取
        TS_DOING,      //进行中
        TS_TODAYOVER,  //日常任务已领取奖励
        TS_OVER        //已领取奖励
    }
]]

local task_manager_cls = class{
    ctor = function( self )
        self.tasks = {}

        local types = 3 -- ETASK_TYPE 1-日常 2-成长

        for i=1, types do
            self.tasks[i] = {}
        end

        self.activeness = activeness_cls()

        self.events = require('base.event').new()
    end,

    refresh = function(self, taskList)
        local tab = {}
        for k, v in ipairs(taskList) do
            local id = v.Id
            --log.info("# refresh task:"..id)

            local task_cfg =  self:getTaskCfg(id)
            if not task_cfg then
                log.error("Failed to find task cfg:"..tostring(id))
            else
                local type = task_cfg.type
                if self.tasks[type][id] then
                    self.tasks[type][id]:refresh(v)
                else
                    self.tasks[type][id] = task_cls(v)
                end
                table.insert(tab, self.tasks[type][id])                
            end
        end
        self.events:dispatch("onTaskChanged", tab)
    end,

    getTaskCfg = function(self, cfg_id)
        local cfg = config.task_tasks[cfg_id]
        if not cfg then
            cfg = config.task_growths[cfg_id]
        end
        return cfg
    end,

    getTasks = function(self, type)
        if  not self.tasks[type] then
            log.erro("task type error!!!", type)
            return {}
        end
        return self.tasks[type]
    end,

    -- 根据类型获取任务并排序
    getSortTasks = function(self, type)
        if not self.tasks[type] then
            log.error("task type error !! ", type)
            return {}
        end

        local list = {}
        for _, task in pairs(self.tasks[type]) do
            table.insert( list, task )
        end

        table.sort(list, self.sortFunc)

        return list
    end,

    sortFunc = function(a, b)
        if a.state == b.state then
            return a.id < b.id
        end
        return a.state < b.state
    end,

    -- 刷新活跃值
    refreshActiveness = function(self, msg)
        self.activeness:refresh(msg)
        self.events:dispatch("onActivenessChanged")
    end,

    onGetRewards = function(self, id)
        local cfg = self:getTaskCfg(id)
        if not cfg then
            log.info("task cfg id not exist!!! ", id)
            return
        end

        local type = cfg.type
        if not self.tasks[type] then
            log.info("task type error!!",type) 
            return
        end
        if not self.tasks[type][id] then
            log.info("task not exist !!!",id) 
            return
        end

        if type ~= 1 then
            self.tasks[type][id] = nil
        end
        self.events:dispatch("onTaskFinished", id)
    end,

    onGetGrowthTaskRewards = function(self, task_id)
        local tasks = task_manager:getTasks(common_pb.EGRWOTh_TASK)
        tasks[task_id] = nil    -- 删除该任务
        self.events:dispatch("onTaskFinished", task_id)
    end
}

task_manager = task_manager_cls()

return task_manager