-- 有限状态机

local log = logger.get("Game", "StageMachine")
local state_class = require("common/state") -- state 类

return class 
{
    ctor = function (self)
        self.states = {}        -- 所有状态表 name - State
        self.curr_state = nil   -- 当前状态
        self.next_state_name = nil
        self.args = nil
    end, 
    
    init = function(self)
        --states["state_1"] = state_class.New();
        --states["state_2"] = state_class.New();

    end,    -- 初始化所有状态，子类需要实现
    
    update = function(self, dt)
        -- 判断是否要切换状态
        if self.next_state_name then
            self:doChangeState(self.next_state_name, self.args)
            self.next_state_name = nil
            self.args = nil
        end

        if self.curr_state then
            self.curr_state:onUpdate(dt)
        end
    end,

    changeState = function(self, name, args)
        if self.curr_state then
            -- 判断当前状态能否转换到目标状态
            if not self.curr_state:canChangeTo(name, args) then
                return  false, 'can not change to:'..name..' from:'..self.curr_state.name
            end
        end

        self.next_state_name = name
        self.args = args

        return true
    end,

    changeStateImmediate = function(self, name, args)
        local rst, msg = self:changeState(name, args)
        if rst then
            self:doChangeState(self.next_state_name, self.args)
            self.next_state_name = nil
            self.args = nil
        else
           -- log.error('Failed to change state immediate:'..name)
        end
        return rst, msg
    end,

    doChangeState = function(self, name, args)
    
        local next_state = self.states[name];
        if next_state == nil then
            log.error('Failed to find state:'..tostring(name))
        else
            if self.curr_state ~= nil then
                self.curr_state:onExit(next_state)
            end
            self.curr_state = next_state
            self.curr_state:onEnter(args)
            self.state_name = name
        end

        return self.curr_state
    end,
   
}