local log = logger.get("Game", "Army Queue Info")

local army_queue = class{

    ctor = function(self)
        self.events = require("base.event").new()
        self.all_queue = {}
        self.mass_queue = {}
        self.wait_queue = {}
        self.cur_count = 0
        self.max_count = 0
        self.list = {}

        -- 行军加速道具类型
        self.normal_speed_up = 48
        -- 运输加速道具类型
        self.transport_speed_up = 49
        -- 集结加速道具类型
        self.mass_speed_up = 50
    end,

    onTick = function(self, dt)
        for k, v in pairs(self.all_queue) do
            local army_line = v.march_info

            army_line.elapsedTime = army_line.elapsedTime + dt * 1000
        end

        for k, v in pairs(self.wait_queue) do
            if v.left_time > 0 then
                v.left_time = v.left_time - dt
                v.total_time = v.total_time - dt

                v.left_time = v.left_time > 0 and v.left_time or 0
            end
        end
    end,

    add = function(self, id, data, force)
        self.list[id] = data

        if not force and world_map_battle:isBattleObj(data.march_info.battleId) then
            self.wait_queue[id] = data
            return
        end
        
        self.events:dispatch("ARMYQUEUEAdd", data)
    end,

    remove = function(self, id)
        if self.list[id] then
            self.list[id] = nil
            self.events:dispatch("ARMYQUEUEREMOVED", id)
        end
    end,

    setQueueCount = function(self, cur_cnt, max_cnt)
        self.cur_count = cur_cnt
        self.max_count = max_cnt
        self.events:dispatch("ARMYQUEUECOUNTUPDATE")
    end,

    addQueue = function(self, info, force)
        local data = self:addMarchQueue(info)

        if info.action == common_pb.EMARCH_CREATE_MASS or info.action == common_pb.EMARCH_JOIN_MASS then

            if self.mass_queue[data.mass_id] then
                self.mass_queue[data.mass_id][data.id] = data
            else
                self.mass_queue[data.mass_id] = {}
                self.mass_queue[data.mass_id][data.id] = data
            end

            self:add(data.id, data, force)
        elseif info.action == common_pb.EMARCH_MASS_ATK then
            self:refreshMassMarch(data, force)
        else
            self:add(data.id, data, force)
        end

        self.all_queue[data.id] = data
    end,

    removeQueue = function(self, id, force)
        local data = self.all_queue[id]

        if data then
            if not force and world_map_battle:isBattleObj(data.march_info.battleId) then
                return
            end

            if data.marchAction == common_pb.EMARCH_MASS_ATK then
                local tab = self.mass_queue[data.mass_id]
            
                if tab then
                    for k, v in pairs(tab) do
                        self.mass_queue[data.mass_id][v.id] = nil
                        self:remove(id..v.id)
                    end
                end
            elseif data.marchAction == common_pb.EMARCH_CREATE_MASS or data.marchAction == common_pb.EMARCH_JOIN_MASS then
                if self.mass_queue[data.mass_id] then
                    self.mass_queue[data.mass_id][data.id] = nil
                end

                self:remove(id)
            else
                self:remove(id)
            end

            self.all_queue[id] = nil
        end
    end,

    refreshMassMarch = function(self, data, force)
        local tab = self.mass_queue[data.mass_id]

        if tab then
            for k, v in pairs(tab) do
                local copy_data = self:copyMassData(data, data.id..v.id)
                self:add(data.id..v.id, copy_data, force)
            end
        end
    end,

    copyMassData = function(self, src, id)
        local dest = {}
        dest.id = id
        dest.march_info = src.march_info
        dest.mass_id = src.mass_id
        dest.dest_id = src.dest_id
        dest.back = src.goHome
        dest.type = src.type
        dest.left_time = src.left_time
        dest.total_time = src.total_time
        dest.pos = {}
        dest.pos.x = src.pos.x
        dest.pos.y = src.pos.y
        dest.state = src.state
        dest.show_up = src.show_up
        dest.show_go = src.show_go
        dest.queue_type = src.queue_type
        dest.str = src.str
        dest.marchAction = src.marchAction
        dest.beforeAction = src.beforeAction
        dest.mass_info = src.mass_info
        dest.gather_info = src.gather_info
        dest.build_info = src.build_info
        dest.game_time = Time.time
        dest.icon_path = res_path.res_path_81
        dest.big_icon_path = res_path.big_icon_path
        dest.teams = src.teams
        dest.mass = src.mass
        dest.dstPlace = src.dstPlace
        dest.is_help = src.is_help
        dest.is_spy = src.is_spy
        dest.is_mass = src.is_mass
        dest.is_gather = src.is_gather
        dest.have_team = src.have_team
        dest.is_mass = src.is_mass
        dest.can_back = src.can_back
        dest.back_cb = src.back_cb
        dest.goto_cb = src.goto_cb

        return dest
    end,

    createMarchData = function(self, info)
        local army_line = info.armyline
        local data = {}
        data.id = army_line.instId
        data.march_info = army_line
        data.mass_id = army_line.massid
        data.dest_id = army_line.dstid
        data.type = progress_bar.marchLine
        data.left_time = (army_line.totalTime - army_line.elapsedTime) * 0.001
        data.total_time = army_line.totalTime * 0.001
        data.pos = {}
        data.pos.x = army_line.dstPosX
        data.pos.y = army_line.dstPosY
        data.state = "moving"
        data.str = ""
        data.marchAction = army_line.action
        data.beforeAction = army_line.pre_action
        data.mass_info = army_line.massinfo
        data.gather_info = army_line.gatherInfo
        data.build_info = army_line.buildInfo
        data.game_time = Time.time
        data.icon_path = res_path.res_path_81
        data.big_icon_path = res_path.res_path_81
        data.teams = info.teams
        data.mass = info.mass
        data.dstPlace = info.dstPlace
        data.show_up = true
        data.show_go = false
        data.is_help = false
        data.is_spy = false
        data.is_mass = false
        data.is_gather = false
        data.have_team = true
        data.can_back = true
        data.back_cb = nil
        data.goto_cb = nil

        return data
    end,

    addMarchQueue = function(self, info)
        local data = self:createMarchData(info)

        data.back_cb = function()
            local cb = function()
                network_sender:worldMapGoHome(3, data.march_info.instId)
            end
    
            gui_helper:showCommonTipBox(cb, NativeText.GetLangStr(9000069))
        end

        data.goto_cb = function()
            world_map_helper:junpToArmyLine(data.march_info)
        end

        if data.marchAction == common_pb.EMARCH_GO_HOME then
            if data.beforeAction == common_pb.EMARCH_MASS_ATK then
                data.is_mass = true
                data.show_up = true
                data.str = NativeText.GetLangStr(9000070)
                data.icon_path = res_path.res_path_84
                data.big_icon_path = res_path.res_path_84
            elseif data.beforeAction == common_pb.EMARCH_CREATE_MASS then
                data.is_mass = true
                data.show_up = true
                data.str = NativeText.GetLangStr(9000070)
                data.icon_path = res_path.res_path_84
                data.big_icon_path = res_path.res_path_84
            elseif data.beforeAction == common_pb.EMARCH_JOIN_MASS then
                data.is_mass = true
                data.show_up = true
                data.str = NativeText.GetLangStr(9000070)
                data.icon_path = res_path.res_path_84
                data.big_icon_path = res_path.res_path_84
            elseif data.beforeAction == common_pb.EMARCH_HELP then
                data.is_help = true
                data.show_up = true
                data.str = ""
                data.icon_path = res_path.res_path_84
                data.big_icon_path = res_path.res_path_84
            elseif data.beforeAction == common_pb.EMARCH_ATK then
                data.show_up = true
                data.str = NativeText.GetLangStr(9000071)
                data.icon_path = res_path.res_path_84
                data.big_icon_path = res_path.res_path_84
            elseif data.beforeAction == common_pb.EMARCH_MONSTER then
                data.show_up = true
                data.str = NativeText.GetLangStr(9000071)
                data.icon_path = res_path.res_path_84
                data.big_icon_path = res_path.res_path_84
            elseif data.beforeAction == common_pb.EMARCH_BUILD then
                data.show_up = true
                data.str = NativeText.GetLangStr(9000072)
                data.icon_path = res_path.res_path_84
                data.big_icon_path = res_path.res_path_84
            elseif data.beforeAction == common_pb.EMARCH_GATHER_TITAN then
                data.is_gather = true
                data.show_up = true
                data.str = NativeText.GetLangStr(9000073)
                data.icon_path = res_path.res_path_84
                data.big_icon_path = res_path.res_path_84
            elseif data.beforeAction == common_pb.EMARCH_SPY then
                data.is_spy = true
                data.show_up = true
                data.str = NativeText.GetLangStr(9000074)
                data.icon_path = res_path.res_path_84
                data.big_icon_path = res_path.res_path_84
            end

            data.back_cb = nil
            data.can_back = false
        elseif data.marchAction == common_pb.EMARCH_MASS_ATK then
            data.can_back = false
            data.is_mass = true
            data.show_up = data.left_time > 0
            data.str = NativeText.GetLangStr(9000075)
            data.icon_path = res_path.res_path_84
            data.big_icon_path = res_path.res_path_84

            if data.left_time <= 0 then
                data.str = NativeText.GetLangStr("攻击")
            end

            if local_player.id ~= data.march_info.playerId then
                data.back_cb = nil
            end
        elseif data.marchAction == common_pb.EMARCH_CREATE_MASS then 
            data.is_mass = true
            data.show_up = data.left_time > 0
            data.str = NativeText.GetLangStr(9000075)
            data.icon_path = res_path.res_path_84
            data.big_icon_path = res_path.res_path_84

            if data.mass_info and data.mass_info.leftTime > 0 then
                data.show_up = false
                data.show_go = true
                data.left_time = data.mass_info.leftTime
                data.total_time = data.mass_info.totaltime
                data.str = NativeText.GetLangStr(9000076)

                data.back_cb = function()
                    local callback = function()
                        network_sender:destoryMass(data.mass_id)
                    end
            
                    gui_helper:showCommonTipBox(callback, NativeText.GetLangStr(289002))
                end

                data.goto_cb = function()
                    world_map_helper:jumpToHold(data.dest_id, data.pos.x, data.pos.y)
                end
            end
        elseif data.marchAction == common_pb.EMARCH_JOIN_MASS then 
            data.is_mass = true
            data.show_up = data.left_time > 0
            data.str = NativeText.GetLangStr(9000075)
            data.icon_path = res_path.res_path_84
            data.big_icon_path = res_path.res_path_84

            if data.mass_info and data.mass_info.leftTime > 0 then
                data.show_up = false
                data.show_go = true
                data.left_time = data.mass_info.leftTime
                data.total_time = data.mass_info.totaltime
                data.str = NativeText.GetLangStr(9000076)
                
                data.goto_cb = function()
                    world_map_helper:jumpToHold(data.dest_id, data.pos.x, data.pos.y)
                end
            end
        elseif data.marchAction == common_pb.EMARCH_HELP then
            data.is_help = true
            data.show_up = data.left_time > 0
            data.str = ""
            data.icon_path = res_path.res_path_84
            data.big_icon_path = res_path.res_path_84
        elseif data.marchAction == common_pb.EMARCH_GATHER_TITAN then
            data.is_gather = true
            data.show_up = data.left_time > 0
            data.str = NativeText.GetLangStr(9000077)
            data.icon_path = res_path.res_path_84
            data.big_icon_path = res_path.res_path_84

            if data.gather_info and data.gather_info.remaintime > 0 then
                data.show_up = false
                data.show_go = true
                data.left_time = data.gather_info.remaintime
                data.total_time = data.gather_info.totaltime
                data.str = NativeText.GetLangStr(9000078)
            
                data.goto_cb = function()
                    world_map_helper:jumpToHold(data.dest_id, data.pos.x, data.pos.y)
                end
            end
        elseif data.marchAction == common_pb.EMARCH_ATK then
            data.show_up = data.left_time > 0
            data.str = NativeText.GetLangStr(9000079)
            data.icon_path = res_path.res_path_84
            data.big_icon_path = res_path.res_path_84

            if data.left_time <= 0 then
                data.str = NativeText.GetLangStr("攻击")
            end

        elseif data.marchAction == common_pb.EMARCH_MONSTER then
            data.show_up = data.left_time > 0
            data.str = NativeText.GetLangStr(9000079)
            data.icon_path = res_path.res_path_84
            data.big_icon_path = res_path.res_path_84
        elseif data.marchAction == common_pb.EMARCH_SPY then
            data.is_spy = true
            data.show_up = data.left_time > 0
            data.str = NativeText.GetLangStr(9000080)
            data.icon_path = res_path.res_path_84
            data.big_icon_path = res_path.res_path_84
        elseif data.marchAction == common_pb.EMARCH_BUILD then
            data.show_up = data.left_time > 0
            data.str = NativeText.GetLangStr(9000081)
            data.icon_path = res_path.res_path_84
            data.big_icon_path = res_path.res_path_84

            if data.build_info and string.len(data.build_info.leftTime) > 0 and tonumber(data.build_info.leftTime) > 0 then
                data.show_up = false
                data.show_go = true
                data.left_time = tonumber(data.build_info.leftTime)
                data.total_time = tonumber(data.build_info.totaltime)
                data.str = NativeText.GetLangStr(9000082)
                
                data.goto_cb = function()
                    world_map_helper:jumpToHold(data.dest_id, data.pos.x, data.pos.y)
                end
            end
        end

        return data
    end,

    queueMaxCheck = function(self)
        if self.cur_count >= self.max_count then
            hint:setText(NativeText.GetLangStr(280406))
            return true
        end

        return false
    end,

    armyLineSpeedUp = function(self, army_id, left_time, total_time, action)
        self.speed_up_army = {}
        self.speed_up_army.id = army_id
        self.speed_up_army.lt = left_time
        self.speed_up_army.tt = total_time

        if action == common_pb.EMARCH_MASS_ATK then
            frame_manager:show('resource_exchange', nil, {
                item_id = self.mass_speed_up,
                item_num = 1,
                army_info = {lt = left_time, tt = total_time, id = army_id},
            })
        else
            frame_manager:show('resource_exchange', nil, {
                item_id = self.normal_speed_up,
                item_num = 1,
                army_info = {lt = left_time, tt = total_time, id = army_id},
            })
        end
    end,

    updateSpeedUpArmy = function(self, msg)
        if self.speed_up_army then
            self.speed_up_army.lt = msg.remainTime
            self.speed_up_army.tt = msg.totaltime

            self.events:dispatch("WORLDMAPSPEEDUP", {lt = msg.remainTime, tt = msg.totaltime, id = self.speed_up_army.id})
        end
    end,

    getDataIdArmyId = function(self, id)
        for k, v in pairs(self.list) do
            if id == v.march_info.instId then
                return v
            end
        end
    end,

    addDataByArmyId = function(self, id)
        local data = self:getDataIdArmyId(id)
        self.wait_queue[data.id] = nil
        self.events:dispatch("ARMYQUEUEAdd", data)
    end,

    removeDataByArmyId = function(self, id)
        local data = self:getDataIdArmyId(id)
        self:removeQueue(data.id, true)
    end,
}

army_queue_manager = army_queue()

return army_queue_manager