local log = logger.get("Game", "chat")

chat_cls = class {
    ctor = function(self)
        self.events = require("base.event").new()
        self.content = self.content or {}
        self.content[protocol_pb.PERSONAL] = self:readPersonalCache()
    end,

    setBasicInfo = function(self, msg)
        self.content = self.content or {}
        for __, channel in ipairs(msg.channelList) do
            if protocol_pb.PERSONAL == channel.channelType then
                -- print("聊天基本信息:", channel.chatPlayerId, channel.chatPlayerName)
                local player_id = channel.chatPlayerId
                local player_name = channel.chatPlayerName

                self.content[protocol_pb.PERSONAL] = self:readPersonalCache()
                local personal = self.content[protocol_pb.PERSONAL]
                personal[player_id] = personal[player_id] or {}

                for k, record in ipairs(channel.chatRecord) do

                    local item = {}
                    item.chatId			 = record.chatId			
                    item.playerId        = record.playerId          
                    item.playerName      = record.playerName      
                    item.playerHead      = record.playerHead      
                    item.familyAcronym	 = record.familyAcronym
                    item.content         = record.content   
                    item.sendTime        = record.sendTime     
                    item.familyId        = record.familyId     

                    table.insert(personal[player_id], item)
                end

                self:setPlayerName(player_id, player_name)
                self:writePersonalCache(self.content[protocol_pb.PERSONAL])
            else
                -- print("啊啊啊啊啊啊啊", channel.channelType, channel.chatRecord and #channel.chatRecord)
                self.content[channel.channelType] = channel -- MChatChannelInfo
            end
        end

        self:makeUnread()
        self.events:dispatch('tip_num_change')
    end,

    makeUnread = function(self)
        self.unread_list = self.unread_list or {}
        for channel_type, info in pairs(self.content) do
            -- 个人聊天小红点统计
            if channel_type == protocol_pb.PERSONAL then
                if self.content[channel_type] then
                    local all_index = 0
                    for player_id, record_list in pairs(self.content[channel_type]) do

                        local index = self:getCacheIndex(player_id)
                        local unread = 0
                        for __, record in ipairs(record_list) do
                            if record.sendTime > index then
                                unread = unread + 1
                                all_index = all_index + 1
                            end
                        end
                        self.unread_list[channel_type] = all_index
                        self.unread_list[player_id] = unread
                        
                    end
                end
            else
                local index = self:getCacheIndex(channel_type)
                if index then
                    local unread = 0
                    for __, record in ipairs(info.chatRecord) do
                        if record.sendTime > index then
                            unread = unread + 1
                        end
                    end
                    self.unread_list[channel_type] = unread
                end
            end
        end
    end,

    makeReaded = function(self, channel_type, player_id)
        print("设置已读", channel_type, player_id)
        if channel_type == protocol_pb.PERSONAL then
            local channel = self.content[channel_type]
            if channel and channel[player_id] and #channel[player_id] > 0 then
                local last_record = channel[player_id][#channel[player_id]]
                if last_record then
                    UnityEngine.PlayerPrefs.SetInt(local_player.id .. "_chat_" .. player_id, last_record.sendTime)
                end
            end
        else
            local channel = self.content[channel_type]
            if channel and channel.chatRecord and #channel.chatRecord > 0 then
                local last_record = channel.chatRecord[#channel.chatRecord]
                if last_record then
                    UnityEngine.PlayerPrefs.SetInt(local_player.id .. "_chat_" .. channel_type, last_record.sendTime)
                end
            end
        end
        self:makeUnread()
    end,

    -- id 渠道id or 玩家id
    getCacheIndex = function(self, id)
        local curr_index = UnityEngine.PlayerPrefs.GetInt(local_player.id .. "_chat_" .. id, 0)
        return curr_index
        --[[
        if channel_type == protocol_pb.PERSONAL then
            log.error("chat getCacheIndex error")
        else
            local curr_index = UnityEngine.PlayerPrefs.GetInt(local_player.id .. "_chat_" .. channel_type, 0)
            return curr_index
        end
        ]]
    end,

    -- id 渠道id or 玩家id
    getUnreadNum = function(self, id)
        if self.unread_list and self.unread_list[id] then
            return self.unread_list[id]
        end
        return 0
    end,

    setNtf = function(self, msg)
        print("聊天消息 通知...")
        if not self.content then
            self.content = {}
        end
        if msg.chatChannel == protocol_pb.PERSONAL then
            print("接收方的玩家id", msg.chatPlayerName, msg.chaPlayerId)

            local player_id = msg.chaPlayerId
            local player_name = msg.chatPlayerName

            self.content[protocol_pb.PERSONAL] = self:readPersonalCache()
            local personal = self.content[protocol_pb.PERSONAL]
            personal[player_id] = personal[player_id] or {}

            local item = {}
            item.chatId			 = msg.chatInfo.chatId			
            item.playerId        = msg.chatInfo.playerId          
            item.playerName      = msg.chatInfo.playerName      
            item.playerHead      = msg.chatInfo.playerHead      
            item.familyAcronym	 = msg.chatInfo.familyAcronym
            item.content         = msg.chatInfo.content        
            item.sendTime        = msg.chatInfo.sendTime     

            table.insert(personal[player_id], item)
            
            self:setPlayerName(player_id, player_name)
            self:writePersonalCache(self.content[protocol_pb.PERSONAL])

            local rst = self:readPersonalCache()
        else
            if not self.content[msg.chatChannel] then self.content[msg.chatChannel] = {} end
            if not self.content[msg.chatChannel].chatRecord then self.content[msg.chatChannel].chatRecord = {} end

            if self.content[msg.chatChannel] and self.content[msg.chatChannel].chatRecord then
                table.insert(self.content[msg.chatChannel].chatRecord, msg.chatInfo)
            else
                log.error("not exist channel type", msg.chatChannel)
            end
        end

        self:makeUnread()
        self.events:dispatch('tip_num_change')
        self.events:dispatch('chat_msg_change')
    end,

    setPlayerName = function(self, player_id, player_name)
        UnityEngine.PlayerPrefs.SetString(player_id, player_name)
    end,

    getPlayerName = function(self, player_id)
        return UnityEngine.PlayerPrefs.GetString(player_id, "")
    end,

    writePersonalCache = function(self, personal_content)
        -- print("写入数据")
        local str = require("gui.tool.frame_helper"):serialize(personal_content)
        UnityEngine.PlayerPrefs.SetString(local_player.id .. "personal_chat", str)
    end,

    readPersonalCache = function(self)
        print("读取数据 玩家id:", local_player.id)
        local data = {}
        local str = UnityEngine.PlayerPrefs.GetString(local_player.id .. "personal_chat", "nil")
        if str ~= "nil" then
            data = require("gui.tool.frame_helper"):unserialize(str)
        end
        -- 存在空字符串数据，清理掉
        data[""] = nil
        return data
    end,

    createPersonalChat = function(self, player_id, player_name)
        app:startCoroutine(
            function()
                local vsb = frame_manager:isDisplay('chat')
                if not vsb then
                    frame_manager:show('chat')
                end
                local chat_frame = frame_manager:getFrame('chat')
                while(not chat_frame.loaded) do
                    coroutine.yield(nil)
                end
                if chat_frame then
                    chat_frame:openPersonalTab(player_id, player_name)
                end
            end
        )
    end,

    removePersonalData = function(self, player_id)
        self:makeReaded(protocol_pb.PERSONAL, player_id)
        self.content[protocol_pb.PERSONAL] = self:readPersonalCache()
        local personal = self.content[protocol_pb.PERSONAL]
        personal[player_id] = nil
        self:writePersonalCache(self.content[protocol_pb.PERSONAL])

        self.events:dispatch('tip_num_change')
        self.events:dispatch('chat_msg_change')
    end,

    getContent = function(self, channel_type)
        if not self.content then
            return nil
        end
        return self.content[channel_type]
    end,

    getDefaultChannel = function(self)
        if not self.default_channel then
            self.default_channel = {
                id = tostring(protocol_pb.WORLD),
                name = NativeText.GetLangStr(210011),
                acronym = NativeText.GetLangStr(210002),
            }
        end
        return self.default_channel
    end,

    setDefaultChannel = function(self, channel)
        if self.default_channel.id == channel.id then
            return
        end
        self.default_channel = channel
        self.events:dispatch('default_channel_change')
    end,

    -- 返回 OneChatInfo
    getLatestContent = function(self, channel)
        if channel.id == tostring(protocol_pb.WORLD) or channel.id == tostring(protocol_pb.FAMILYGENERAL) or channel.id == tostring(protocol_pb.FAMILYMANAGER) then
            local rst = self.content[tonumber(channel.id)]
            if not rst then -- syz 空保护
                return nil
            end

            if rst.chatRecord and #rst.chatRecord > 0 then
                local c = rst.chatRecord[#rst.chatRecord]
                return "<color=#FDB300FF>" .. c.playerName .. "</color>" .. ":" .. self:parseContent(c.content) 
            else
                return nil
            end
        else
            local rst = self.content[protocol_pb.PERSONAL]
            local player_id = channel.id
            if rst and rst[player_id] and #rst[player_id] > 0 then
                local c = rst[player_id][#rst[player_id]]
                return "<color=#FDB300FF>" .. c.playerName .. "</color>" .. ":" .. self:parseContent(c.content) 
            end
        end
        return nil
    end,

    parseContent = function(self, content)
        self.helper = require("gui.tool.frame_helper")
        local tb = self.helper:unserialize(content)
        if tb and tb.content_type then
            -- 普通文本消息
            if tb.content_type == 1 then
                return tb.text
            -- 表情消息
            elseif tb.content_type == 2 then
                return NativeText.GetLangStr(210012)
            -- 链接
            elseif tb.content_type == 3 then
                return NativeText.GetLangStr(210013)
            end
        end
        return ""
    end,

    getChannelList = function(self)
        local list = {}

        local data = {
            id = tostring(protocol_pb.WORLD),
            name = NativeText.GetLangStr(210011),
            acronym = NativeText.GetLangStr(210002),
        }
        table.insert(list, data)

        local data = {
            id = tostring(protocol_pb.FAMILYGENERAL),
            name = NativeText.GetLangStr(210009),
            acronym = NativeText.GetLangStr(210003),
        }
        table.insert(list, data)

        local data = {
            id = tostring(protocol_pb.FAMILYMANAGER),
            name = NativeText.GetLangStr(210010),
            acronym = NativeText.GetLangStr(210003),
        }
        local rst, guild_level = local_guild:getGuildLevel()
        if rst and guild_level >= common_pb.MD_R4 then
            table.insert(list, data)
        end

        if self.content[protocol_pb.PERSONAL] then
            local p = self.content[protocol_pb.PERSONAL]
            for player_id, value in pairs(p) do
                local data = {
                    id = player_id,
                    name = self:getPlayerName(player_id),
                    acronym = NativeText.GetLangStr(210004),
                }
                table.insert(list, data)
            end
        end
        return list
    end,

    addToBlackList = function(self, player_id)
        local list = self:getBlackList()
        list[player_id] = "123"
        local str = require("gui.tool.frame_helper"):serialize(list)
        UnityEngine.PlayerPrefs.SetString(local_player.id .. "chat_black_list", str)
    end,

    removeFromBlackList = function(self, player_id)
        local list = self:getBlackList()
        list[player_id] = nil
        local str = require("gui.tool.frame_helper"):serialize(list)
        UnityEngine.PlayerPrefs.SetString(local_player.id .. "chat_black_list", str)
    end,

    getBlackList = function(self)
        local list = {}
        local str = UnityEngine.PlayerPrefs.GetString(local_player.id .. "chat_black_list", "nil")
        if str ~= "nil" then
            list = require("gui.tool.frame_helper"):unserialize(str)
        end
        -- 存在空字符串数据，清理掉
        list[""] = nil
        return list
    end,

    inBlackList = function(self, player_id)
        local list = self:getBlackList()
        if list[player_id] then
            return true
        end
        return false
    end,

}

-- 本地玩家唯一实例
local_chat = chat_cls()
return local_chat
