local super = require('common.state_machine')
local log = logger.get("Game", "Game Director")
require('game.game_info')
require("game.army_queue_manager")
localization_manager = require('game.localization_manager')

network = nil   -- 网络模块 

-- 界面管理器
frame_manager = require('gui.frame_manager')
-- 游戏时钟
game_scheduler = require("game.game_scheduler")

local _game_director_cls = class {
    super = super,

    ctor = function(self)
        super.ctor(self)

        if _inst then
            log.error('Game director already exist!')
        else
            _inst = self
        end
        self.ping_elapsed = 0.0
    end,

    init = function(self)  
        -- 本地化
        localization_manager:init()

        -- 初始化UI管理类
        frame_manager:init()

        game_scheduler:init()

        -- 初始化所有游戏状态
        self:initStates()

        self.gameObject = GameObject("Game")
        GameObject.DontDestroyOnLoad(self.gameObject);
        self.transform = self.gameObject.transform
		self.events = require("base.event").new()

        -- 注册update函数
        --[[
        local bridge = self.gameObject:AddComponent(Fenix.Scripting.LunarBehaviourUpdate)
        bridge:SetBridge(self)]]
        local bridge = self.gameObject:AddComponent(Fenix.UnityBridge)
        bridge:SetUpdate(function(dt)
            self:Update(dt)
        end)
        
        local pool_size = 5
        if app:isSmallMemory() then
            pool_size = 1
        end

        self.pool = require("base.pool") {
            size = pool_size, 
            transform = self.gameObject.transform:CreateChild("global_pool"),
        }

        network = require("network.network")
        require("network/message_handler.lua")

        -- 特效/音效 快表
        vfx_sfx_map = {}
        config.shoot_effect_effect_libs:foreach(
            function(k,v)
                vfx_sfx_map[v.effect_path] = v.audio
            end
        )

        Time.timeScale = 1.0
    end,

    -- 初始化所有状态
    initStates = function(self)
        self.states['sdk_login'] = require('game.game_state_sdk_login').new('sdk_login', self)
        self.states['login'] = require('game.game_state_login').new('login', self)
        self.states['loading'] = require('game.game_state_loading').new('loading', self)
        self.states['create_role'] = require('game.game_state_create_role').new('create_role', self)
        self.states['world_map'] = require('game.game_state_world_map').new('world_map', self)
        --self.states['island'] = require('game.game_state_island').new('island', self)
        self.states['level'] = require('game.game_state_level').new('level', self)
        self.states['battle'] = require('game.game_state_battle').new('battle', self)
        self.states['battle_replay'] = require('game.game_state_battle_replay').new('battle_replay', self)
       -- self.states['select_base'] = require('game.game_state_select_base').new('select_base', self)
        
        self.states['home'] = require('game.game_state_home').new('home', self)
        self.states['empty'] = require('game.game_state_empty').new('empty', self)
        --self.states['war'] = require('game.game_state_war').new('war', self)

        self.states['recruit_battle'] = require('game.game_state_recruit_battle').new('recruit_battle', self)

        self:changeState('empty')            
    end,

    -- 第二步 init, 用于拆分一些初始化，加快 login 界面显示速度
    additionalInit = function(self)
        if not self.is_additional_inited then
            self.is_additional_inited = true


            hint = frame_manager:show('hint')  
            --emp_manager:init()
            dialog_manager:init()

            -- 预加载一些包
            self:constPackage("prefab_curve.unity3d", "Assets/Prefab/Curve/curve.asset")
            consts:preloadCurves()
            --[[
            self:constPackage("arts_ui_atlas_icon.unity3d")
            self:constPackage("arts_ui_atlas_common.unity3d")
            self:constPackage("frames.unity3d")
            self:constPackage("arts_ui_fx.unity3d")]]

            -- 预加载红点 prefab
            app:startCoroutine(
                function()
                    coroutine.yield(WaitForInstantiate("Assets/Prefab/UI/RedPoint.prefab", self.transform, function(go)
                        --go.transform:SetAsLastSibling()
                        self.red_point_prefab = go
                        go.name = "common_red_point"
                        go:SetActive(false)
                    end))
                end)

            -- 预加载 shader 
            self:perloadShaders()
        end
    end,

    addTankPackages = function(self)
        if not self.is_add_tank_packages then
            self.is_add_tank_packages = true

            battle_helper = require("RTS.battle_helper")
            battle_helper:initSoldierPool()
            --self:constPackage("prefab_curve.unity3d")
            --self:loadPackage("prefab_weapon.unity3d")
           -- self:loadPackage("prefab_vehicle.unity3d")
            --self:loadPackage("prefab_wheel.unity3d")
            --self:constPackage("arts_fx.unity3d")
        end
    end,

    -- 预加载  shader
    perloadShaders = function(self)

         app:startCoroutine(
            function()
                local path = "Assets/Shader/ShaderVariants.shadervariants"

                coroutine.yield(Fenix.Assets.Helper.WaitForAsset(path, UnityEngine.ShaderVariantCollection, function(asset) 
                    self.is_shader_warmup = true
                    log.info("### WarmupAllShaders")
                    self.shader_variant = asset:Get()
    
                    if asset.package then
                        asset.package:AddRef()
                    end

                    if self.shader_variant then
                        self.shader_variant:WarmUp() 
                        log.info("###### ShaderVariantCollection.WarmUp finished!")
                    end
                    --Fenix.Assets.Helper.ms_SyncLoad = false
                end))

                local mat_path = 'Assets/Material/SkeletonGraphicDefault.mat'
                coroutine.yield(Fenix.Assets.Helper.WaitForAsset(mat_path, UnityEngine.Material, function(asset) 
                    local go = asset:Get()
                    self.spine_material = Object.Instantiate(go)
                    if asset.package then
                        asset.package:AddRef()
                    end

                    --Fenix.Assets.Helper.ms_SyncLoad = false
                end))
            end
        )

    end,

    FixedUpdate = function(self, dt)

    end,

    Update = function(self, dt)
        if reconnector and network then
            if not reconnector.is_reconnecting then
                -- 非重连中，执行状态机的 update
                super.update(self, dt)

                if self.state_name and self.state_name ~= "login" and self.state_name ~= "loading"  
                    and self.state_name ~= "empty" and self.state_name ~= "sdk_login" then
                    self:_updatePing()
                end
                guide_manager:update(dt)
                network:update(dt)
            else
                -- 重连中，执行重连 update
                reconnector:update()
            end
        end

        frame_manager:update(dt)

        game_scheduler:update(dt)
    end,

    -- ping 值更新，如果 ping 过大发起重连
    _updatePing = function(self)
        if game_info.offline_mode then
            return
        end

        self.ping_elapsed = self.ping_elapsed + Time.unscaledDeltaTime
        if self.ping_elapsed > 1.0 and network:isConnected()  then
            self.ping_elapsed = 0.0

            -- 发送心跳包
            network_sender:heartBeat()
            if self.send_heart_beat_time == nil then
                self.send_heart_beat_time = Time.unscaledTime   -- 发的时候设置上，收到回应置 nil
            end

        elseif self.send_heart_beat_time then
            -- 发出未收到
            local delta = Time.unscaledTime - self.send_heart_beat_time

            if delta > 5.0 then
                -- ping 超过 4 秒， 进行重连
                reconnector:reconnect()
                self.send_heart_beat_time = nil
            end
            
            if delta > 0.1 then
                self.ping = math.floor(delta * 1000)
                if frame_gm then
                    frame_gm:setPing(self.ping)
                end
            end
        end

        -- 如果不在 login 状态并且网络未连接，进行重连
        if not network:isConnected() then
            reconnector:reconnect()
        end
    end,

    -- 收到 ping 回复
    onPing = function(self, time)
        self.ping = math.floor(Time.unscaledTime * 1000 - time)
        if frame_gm then
            frame_gm:setPing(self.ping)
        end

       self.send_heart_beat_time = nil
    end,

    -- 使 package 永久驻留
    constPackage = function(self, pkgName, prefab)
        --[[
        local pkg = self:loadPackage(pkgName)
        if pkg then
            pkg:AddRef()
        end]]
    end,

    -- 预先加载包
    loadPackage = function(self, pkgName)
        if Fenix.App:GetInstance().useAssetDatabase then
            return nil
        else
            local pkgManager = Fenix.App.GetPackageManager()
            return pkgManager:GetPackage(pkgName)
        end
    end,

    enable3DRender = function(self)
        if Camera.main then
            Camera.main.nearClipPlane = 0.5
        end
    end,

    disable3DRender = function(self)
        if Camera.main then
            Camera.main.nearClipPlane = Camera.main.farClipPlane-0.001
        end
    end,

    isWorldMapState = function(self)
        return self.curr_state.name == 'world_map'
    end,

    isHomeState = function(self)
        return self.curr_state.name == 'home'
    end,

}

if not game_director then
    game_director = _game_director_cls()  -- 唯一实例
end
return game_director