super = require('game.game_state_base')
local stage_class = require('RTS.replay_stage.rts_replay_stage')
local ship_battle_stage_class = require('RTS.replay_stage.rts_ship_battle_replay_stage')

local log = logger.get("Game", "Replay")
require('helper.serial_helper')

return class
{
    ctor = function(self, ...)
        super.ctor(self, ...)
        game_state_battle_replay = self
    end,

    super = super,
    
    onEnter = function(self, args)
        battle_helper = require('RTS.battle_helper')
        self.battle_report = args.battle_report
        self:init()
    end,

    -- 初始化
    init = function(self)
        self.stage_id = 0
        self.battle_id = 1
        local battle_types = require('RTS.enums').battle_types

        -- 根据是否是船战，创建不同的 stage
        local the_stage_class = stage_class
        if self.battle_report.is_ship_battle then
            the_stage_class = ship_battle_stage_class
            self.stage_id = -1
        end

        self.stage = the_stage_class(battle_types.REPLAY, {
            owner = self,
            stage_id = self.stage_id, 
            battle_id = self.battle_id,
            battle_report = self.battle_report,
        })       

        network:registerMessageHandler(self)
        
    end,

    canChangeTo = function(self)
        return true
    end,

    onMessage = function(self, cmd, msg)

    end,

    onUpdate = function(self, dt)
        if not self.stage then
            return
        end
        self.stage:update(dt)
    end,

    -- 返回世界地图
    backToWorldMap = function(self)
        log.info("#### game_info.replay_battle_from_state:"..tostring(game_info.replay_battle_from_state))

        if not game_info.replay_battle_from_state or game_info.replay_battle_from_state == 'world_map' then
            game_director:changeState('loading', {next_state = 'world_map', scene_name = 'hex_map', next_state_ui_name = 'worldMap'})
        else
            game_director:changeState('loading', {next_state = 'home', scene_name = 'home', next_state_ui_name = 'home'})
        end
    end,

    onExit = function(self)
        network:unregisterMessageHandler(self)
        if self.stage then
            self.stage:destroy()
            self.stage = nil
        end
        game_director.pool:destroy()
    end,
}
