super = require('game.game_state_base')
local home_map = require('RTS.home_sys.home_map')
local log = logger.get("Game", "Home")
require('RTS.home_sys.home_info')
require('RTS.home_sys.drill_ground_helper')

return class {
    super = super,

    ctor = function(self, ...)
        super.ctor(self, ...)
    end,

    onEnter = function(self, args)
        self.agrs = args
        -- 当前玩家id (如果有浏览模式，玩家id 是别人的)
        self.cur_player_id = game_info.player_id
        -- 显示界面
        self.gui = frame_manager:show('home', self)
        -- 注册消息
        network:registerMessageHandler(self)
        -- 是否点击在UI界面上
        self.is_click_ui = false
        -- 是否是滑动
        self.is_slider = false
        -- 第一次点击的坐标
        self.first_touch_pos = nil

        -- home_info:loadCfg()

        self.home_map = home_map(self)
        self.home_map:init()

        game_state_home = self

        -- 显示队列
        frame_manager:show('summary_queue', self.home_map)

        self:init()

        if level_info.level_over then
            level_info.level_over = false
            if level_info.is_win then
                frame_manager:show('level_select', nil, 
                    {chapter_id = chapter_manager.curr_chapter,complete_level = level_info.map_id})
            end
        end
    end,

    init = function(self)
        -- 预加载坦克的包
        battle_helper = require('RTS.battle_helper')
        game_director:addTankPackages()
        drill_ground_helper:init()

        self.event_ids = {}
        self.event_ids["NOTIFYGETITEM"] = game_director.events:on("NOTIFYGETITEM", functor.make(self.onGetItems, self))

        self.coroutine_id = app:startCoroutine(
            function()
                if self.agrs.jumpToTrain then
                    coroutine.yield(WaitForSeconds(0.5))
                    self:jumpToTrainingBuild()
                end
            end
        ) 

    end,

    canChangeTo = function(self, name, args) 
        return true
    end,

    onUpdate = function(self, dt)
        self:updateUI(dt)
        self.home_map:onUpdate(dt)

        drill_ground_helper:update(dt)
    end,

    -- 网络消息 
    onMessage = function(self, cmd, msg)
        -- 建筑新增
        if cmd == protocol_pb.PERSONBASEAPPEAR then
            app:startCoroutine(function()
                -- 本地缓存
                -- for k, v in ipairs(msg.groups) do
                --     home_info:setGroup(v.groupId, v.bLock)
                -- end
                -- for k, v in ipairs(msg.objs) do
                --     home_info:setBuild(v.gridId, v)
                -- end

                -- -- 广播
                -- home_info:dispatchGroupInfos()
                -- home_info:dispatchGridInfos()

                -- for k, v in ipairs(msg.objs) do
                --     home_info:dispatchBuildInfos(v.instId)
                -- end
                
                -- 检测参数的传递
                -- self:detectArgs(self.agrs)
    
                -- if self.gui and self.gui:isLoaded() then
                --     guide_manager:tryStartGuide()
                -- end
            end) 
        -- 建筑删除
        elseif cmd == protocol_pb.PERSONBASEDISAPPEAR then
            -- for k, id in ipairs(msg.instIds) do
            --     home_info:removeBuild(id)
            -- end
            -- home_info:dispatchGridInfos()
            -- for k, id in ipairs(msg.instIds) do
            --     home_info:dispatchBuildInfos(id)
            -- end
        elseif cmd == protocol_pb.ENTERPERSONBASE then
            if msg.respCode ~= 0 then
            end
        elseif cmd == protocol_pb.PERSONBASEMOVE then
            if msg.respCode ~= 0 then
            end
        elseif cmd == protocol_pb.PERSONBASEADD then
            if msg.respCode ~= 0 then
            end
        elseif cmd == protocol_pb.PERSONBASEDEL then
            if msg.respCode ~= 0 then
            end
        elseif cmd == protocol_pb.PERSONBASEQUERY then
            if msg.respCode == 0 then
                print("查询返回结果", msg.respCode, msg.objType, #msg.train.project)
                if msg.objType == common_pb.EPERSON_BASE_STUDIO then
                    -- local dis, frame = frame_manager:isDisplay('home_entity_research')
                    -- if not dis then
                    --     frame_manager:show('home_entity_research', self, msg.studio)
                    -- else
                    --     frame:addCommandBuff(function()
                    --         frame:refreshUI(msg.studio)
                    --     end)
                    -- end
                elseif msg.objType == common_pb.EPERSON_BASE_TRAIN then
                    local dis, frame = frame_manager:isDisplay('home_entity_train')
                    if not dis then
                        frame_manager:show('home_entity_train', self, msg.train)
                    else
                        frame:addCommandBuff(function()
                            frame:refreshUI(msg.train)
                        end)
                    end
                elseif msg.objType == common_pb.EPERSON_BASE_PARK then
                    local dis, frame = frame_manager:isDisplay('home_entity_garage')
                    if not dis then
                        frame_manager:show('home_entity_garage', self, msg.park)
                    else
                        frame:addCommandBuff(function()
                            frame:refreshUI(msg.park)
                        end)
                    end
                elseif msg.objType >= common_pb.EPERSON_BASE_TRAIN1 and msg.objType <= common_pb.EPERSON_BASE_TRAIN4 then
                end
            end
        elseif cmd == protocol_pb.PERSONBASETECHNOLOGYUPNOTIFY then
            print('科技升级通知')
            -- local dis, frame = frame_manager:isDisplay('home_entity_research')
            -- if dis then
            --     frame:addCommandBuff(function()
            --         frame:refreshUI(msg.studio)
            --     end)
            -- end
            -- guide_manager:tryStartGuide()
        elseif cmd == protocol_pb.TECHNOLYGYSPEEDUP then
            print('个人科技加速')
            -- if msg.respCode == 0 then
            --     local dis, frame = frame_manager:isDisplay('home_entity_research')
            --     if dis then
            --         frame:addCommandBuff(function()
            --             frame:refreshUI(msg.studio)
            --         end)
            --     end
            -- end
        elseif cmd == protocol_pb.PERSONBASESTARTPROJECT then
            if msg.respCode ~= 0 then
                -- log.error("使用道具失败!!!", msg.respCode)
            else
                network_sender:personBaseQuery(msg.train.baseObj.instId)
            end
        elseif cmd == protocol_pb.PERSONBASETECHNOLOGYUP then
            -- if msg.respCode == 0 then
            --     local dis, frame = frame_manager:isDisplay('home_entity_research')
            --     if dis then
            --         frame:addCommandBuff(function()
            --             frame:refreshUI(msg.studio)
            --         end)
            --     end
            -- end
        elseif cmd == protocol_pb.PERSONBASELEVELUP then
            -- if msg.respCode == 0 then
            --     -- prinst("升级成功返回消息! 等级:", msg.obj.level)
            --     home_info:setBuild(msg.obj.gridId, msg.obj)
            --     home_info:dispatchBuildInfos(msg.obj.instId)
            -- end
        elseif cmd == protocol_pb.PERSONBASEADDOFFICE then
            if msg.respCode ~= 0 then
                -- log.error("派遣指挥官失败!!!", msg.respCode)
            else
                network_sender:personBaseQuery(msg.train.baseObj.instId)
            end
        elseif cmd == protocol_pb.PERSONBASEREMOVEOFFICE then
            if msg.respCode ~= 0 then
                -- log.error("移除指挥官失败!!!", msg.respCode)
            else
                network_sender:personBaseQuery(msg.train.baseObj.instId)
            end
        elseif cmd == protocol_pb.PERSONBASELEVELUPNOTIFY then
            -- home_info:setBuild(msg.obj.gridId, msg.obj)
            -- home_info:dispatchBuildInfos(msg.obj.instId)
            -- guide_manager:tryStartGuide()
            if msg.obj.cfgId < 100 then
                hint:setText(string.format(NativeText.GetLangStr(999014), frame_helper:getLangCfg(config.home_config_lists[msg.obj.cfgId].name_id)))
            end
        elseif cmd == protocol_pb.PERSONBASEBUILDINGSPEEDUP then
            -- if msg.respCode == 0 then 
                -- home_info:setBuild(msg.obj.gridId, msg.obj)
                -- home_info:dispatchBuildInfos(msg.obj.instId)
                -- network_sender:buildingExpend(msg.obj.instId, common_pb.EBUILDING_LEVELUP)
            -- end
        elseif cmd == protocol_pb.PERSONBASEADDPARTNER then
            if msg.respCode ~= 0 then 
                -- log.error("添加战车到停车场失败!!!", msg.respCode)
            else
                network_sender:personBaseQuery(msg.park.baseObj.instId)
            end
        elseif cmd == protocol_pb.PERSONBASEREMOVEPARTNER then
            if msg.respCode ~= 0 then 
                -- log.error("从停车场移除战车失败!!!", msg.respCode)
            else
                network_sender:personBaseQuery(msg.park.baseObj.instId)
            end
        elseif cmd == protocol_pb.PERSONBASEFREETIMENOTIFY then
            -- home_info:setBuildFreeTime(msg.buildingFreeTime)
        elseif cmd == protocol_pb.REQHELPFASTBUILD then 
            -- log.info("###### REQHELPFASTBUILD", msg.respCode)
            if msg.respCode == 0 then
                hint:setServerNotify(NativeText.GetLangStr(190246))
                self.home_map.events:dispatch("REQHELPFASTBUILD", msg.buildingId)
            end
        elseif cmd == protocol_pb.GETREPAIRFACTORYINFO then
            
        elseif cmd == protocol_pb.GETNEEDREPAIRHERO then
            
        elseif cmd == protocol_pb.PERSONBASEADDREPAIRHERO then
            
        elseif cmd == protocol_pb.PERSONBASECANCELREPAIRHERO then
            print('取消英雄修理 respCode', msg.respCode)
            if msg.respCode == 0 then
                network_sender:getRepairInfo()
            end
        elseif cmd == protocol_pb.SPEEDUPREPAIRHERO then
            print('加速修理战车 respCode', msg.respCode)
            if msg.respCode == 0 then
                network_sender:getRepairInfo()
            end
        elseif cmd == protocol_pb.GETFIXFACTORYMAKEIFNO then
            
        elseif cmd == protocol_pb.USEREPAIRPACKAGE then
            print('使用修理包 respCode', msg.respCode)
            if msg.respCode == 0 then
                local vsb, frame = frame_manager:isDisplay('repair')
                if vsb then
                    network_sender:getRepairInfo()
                end
            end
        elseif cmd == protocol_pb.MAKEREPAIRPACKAGE then
            
        elseif cmd == protocol_pb.GETREPAIRPACKAGE then
            
        elseif cmd == protocol_pb.GETSHOPLIST then -- 商店类型列表
            
        elseif cmd == protocol_pb.SHOPITEMLIST then -- 商店具体内容
            
        elseif cmd == protocol_pb.BUY_GOODS then
            
        elseif cmd == protocol_pb.GAMESHOPREFRESH then
            
        elseif cmd == protocol_pb.CLICKBUBBLE then
            -- print("game_state_home click bubble:", msg.respCode, msg.bubbleType)
            if msg.respCode == 0 then
                if msg.bubbleType == common_pb.BT_TECHNOLOGY then
                    frame_manager:show('home_tech_complete', nil, msg.techInfo)
                elseif msg.bubbleType == common_pb.BT_REPAIR_HERO then
                    hint:setText(NativeText.GetLangStr(350004))
                elseif msg.bubbleType == common_pb.BT_FIX_PACKAGE then
                    local des = string.format(NativeText.GetLangStr(350005), msg.fixPackageNum)
                    hint:setText(des)
                end
            end
        elseif cmd == protocol_pb.GETCELLINFO then
            -- print("game_state_home cmd GETCELLINFO", msg.respCode, #msg.prisonerInfos, #msg.escapeeList)
            if msg.respCode == 0 then

                print('game_state_home cmd GETCELLINFO:', frame_manager:isDisplay('home_prison'))

                local vsb, frame = frame_manager:isDisplay('home_prison')
                if vsb then
                    frame:addCommandBuff(function()
                        frame:refresh(msg)
                    end)
                else
                    frame_manager:show('home_prison', nil, {
                        msg = msg,
                        recruit_hero = self.recruit_hero,
                    })
                end
                self.recruit_hero = nil
            end
        elseif cmd == protocol_pb.EXILEPRISONER then
            if msg.respCode == 0 then
                network_sender:getCellInfo()
            end
        elseif cmd == protocol_pb.SUMMONPRISONER then
            if msg.respCode == 0 then
                network_sender:getCellInfo()
            end
        elseif cmd == protocol_pb.RELEASEPRISONER then
            if msg.respCode == 0 then
                network_sender:getCellInfo()
            end
        elseif cmd == protocol_pb.STARTRECRUITFIGHT then
            -- print("return msg 开始招募战斗", msg.respCode, msg.instId, #msg.playerForm, #msg.monsterForm)
            game_director:changeState('recruit_battle', { msg = msg })
        elseif cmd == protocol_pb.ENDRECRUITFIGHT then
            network_sender:getCellInfo()
        elseif cmd == protocol_pb.QUERYBUILDEXPEND then
            if msg.respCode == 0 then
                if msg.queryType == common_pb.EBUILDING_LEVELUP then -- 建筑升级
                    local vsb, frame = frame_manager:isDisplay('home_entity_upgrade')
                    if vsb then
                        frame:addCommandBuff(function()
                            frame:refresh(msg)
                        end)
                    else
                        frame_manager:show('home_entity_upgrade', self.home_map, msg)
                    end
                elseif msg.queryType == common_pb.ETECHNOLOGY_LEVELUP then -- 科技升级
                    -- local vsb, frame = frame_manager:isDisplay('home_entity_research')
                    -- if vsb then
                    --     frame:addCommandBuff(function()
                    --         frame:showUpgrade(msg)
                    --     end)
                    -- end
                elseif msg.queryType == common_pb.EPRODUCT_WEAPON then -- 生产武器
                    local vsb, frame = frame_manager:isDisplay("armory_frame")
                    if vsb then
                        frame:addCommandBuff(function()
                            frame:refresh(msg)
                        end)
                    else
                        frame_manager:show("armory_frame", nil, {info=msg})
                    end
                elseif msg.queryType == common_pb.EPRODUCT_ARMOUR then -- 生产装甲
                    local vsb, frame = frame_manager:isDisplay("armory_frame")
                    if vsb then
                        frame:addCommandBuff(function()
                            frame:refresh(msg)
                        end)
                    else
                        frame_manager:show("armory_frame", nil, {info=msg})
                    end
                end
            end
        elseif cmd == protocol_pb.GETSKININFO then
            local vsb, frame = frame_manager:isDisplay("home_skin")
            if vsb then
                frame:addCommandBuff(function()
                    frame:refresh(msg)
                end)
            else
                frame_manager:show("home_skin", nil, msg)
            end

        elseif cmd == protocol_pb.SKININFOCHANGE then
            print("收到服务器通知！！！！！ SKININFOCHANGE")
            local vsb, frame = frame_manager:isDisplay("home_skin")
            if vsb then
                network_sender:getSkinInfo()
            end
        elseif cmd == protocol_pb.USESKIN then
            print("收到服务器通知！！！！！ USESKIN")
            local vsb, frame = frame_manager:isDisplay("home_skin")
            if vsb then
                network_sender:getSkinInfo()
            end
        elseif cmd == protocol_pb.TRAINSOLDIER then
            print("收到服务器训练士兵返回消息!!! TRAINSOLDIER", msg.respCode)
            if msg.respCode == 0 and msg.bPromFinish == 0 then
                frame_manager:hide('home_barrack')
            end
        elseif cmd == protocol_pb.RECEIVESOLDIER then
            print("收到服务器收取士兵返回消息!!! RECEIVESOLDIER", msg.respCode)
            -- local vsb, frame = frame_manager:isDisplay("home_barrack")
            -- if vsb and msg.instId then
            --     if msg.instId == frame:getBuildId() then
            --         network_sender:personBaseQuery(msg.instId)
            --     end
            -- end 
        elseif cmd == protocol_pb.NotifyPbUpdate then
            print("收到服务器士兵训练完成 广播!!! NotifyPbUpdate")
            -- local vsb, frame = frame_manager:isDisplay("home_barrack")
            -- if vsb and msg.instId then
            --     if msg.instId == frame:getBuildId() then
            --         network_sender:personBaseQuery(msg.instId)
            --     end
            -- end 
        elseif cmd == protocol_pb.GETBUILDDETAILS then
            print("GETBUILDDETAILS !!! NotifyPbUpdate")
            frame_manager:show('home_entity_info_detail', nil, {
                is_res = true,
                content = msg,
            })
        elseif cmd == protocol_pb.TreatSoldier or cmd == protocol_pb.REPAIRTRAP then
            if msg.respCode == 0 and msg.bPromFinish == 0 then
                frame_manager:hide('home_hospital')
            end
        elseif cmd == protocol_pb.GETDEPORTINFO then
            if msg.respCode == 0 then
                frame_manager:show('home_vault', nil, msg.vault)
            end

        elseif cmd == protocol_pb.MODIFYROLENAME then
            log.info("修改名字返回消息:", msg.respCode, msg.modifyType)
            if msg.respCode == 0 then
                if msg.modifyType == protocol_pb.EROLE_NAME then
                    local_lord.name = msg.roleName
                    hint:setText(NativeText.GetLangStr(610029))
                else
                    local_lord.nickname = msg.roleName
                    hint:setText(NativeText.GetLangStr(610035))
                end
                local_lord.events:dispatch("LORD_INFO_CHANGE")
            else
                -- hint:setText(NativeText.GetLangStr(610036))
            end

        elseif cmd == protocol_pb.TRAINPROFESSION then
            if msg.respCode == 0 then
                local dis, frame = frame_manager:isDisplay('home_forge_upgrade')
                if dis then
                    frame:addCommandBuff(function()
                        frame:playUpgradeEffect(msg.cfgId, msg.bPromFinish)
                    end)
                end
            end

        elseif cmd == protocol_pb.DRAWCARD then
            -- log.error("酒馆抽卡", msg.getItems and #msg.getItems)
            if msg.respCode == 0 and msg.getItems then
                local length = #msg.getItems
                if length == 1 or length == 10 then
                    frame_manager:show('home_tavern_reward', nil, msg.getItems)
                else
                    -- log.error("酒馆数量不对!", msg.getItems and #msg.getItems)
                end
            end
        elseif cmd == protocol_pb.NOTIFYGETITEM then
            print("收到物品获取消息:", msg.reason, common_pb.FIVE_OPEN)
            if msg.reason == common_pb.FIVE_OPEN or msg.reason == common_pb.PRODUCT_EQUIP then
                frame_manager:show('home_arsenal_reward', nil, frame_helper:shuffle(msg.getItems))
            elseif msg.reason == common_pb.TAVERN_DRAWCARD then
            end    

            if UnityEngine.Application.isEditor and msg.reason == common_pb.FIVE_OPEN then
                sb = sb or {}
                for __, value in ipairs(msg.getItems) do
                    local temp_skill = {}
                    local item_lua = bag_manager:getItemById(value.uId)
                    for __, skill_id in ipairs(item_lua.rand_skill) do
                        table.insert(temp_skill, skill_id)
                    end

                    table.insert(sb, {
                        id = value.uId,
                        item_id = value.itemId,
                        passive_skill = temp_skill
                    })
                end
                if #sb >= 200000 then
                    local str = frame_helper:serialize(sb)
                    local name = "arsenal_" .. os.date("%H_%M_%S",os.time())
                    local path = string.format("c:/%s.txt", name)
                    local cmd = "md " .. path
                    os.execute(cmd)
                    local file = io.open(path, "w")
                    file:write(str)
                    file:close()
                    sb = nil
                end
            end
        end
    end,

    updateUI = function(self, dt)
        local point = Input.mousePosition
        -- 点击在UI上面
        if Input.GetMouseButtonDown(0) then
            self.is_click_ui = RTSHelper.IsPointerOverUI(point.x, point.y)
            self.is_slider = false
            self.first_touch_pos = point
            self.drag_start = true
        elseif Input.GetMouseButton(0) and (not self.is_click_ui) then
            if not self.is_slider then
                self.is_slider = self:isSlide(point)
            else
                if self.drag_start then
                    self.drag_start = not self.drag_start
                    self:dragStart(point)
                end
                self:drag(point)
            end
        elseif Input.GetMouseButtonUp(0) and (not self.is_click_ui) then
            if self.is_slider then
                self:dragEnd(point)
            else
                self:click(point)
            end
        end
    end,

    -- 本次操作是否在屏幕滑动
    isSlide = function(self, point)
        if self.first_touch_pos then
            local d = self.first_touch_pos - point
            return Vector3.SqrMagnitude(d) > 10
        end
    end,

    -- 点击
    click = function(self, point)
        if self.home_map then self.home_map:click(point) end
    end,

    -- 开始拖拽
    dragStart = function(self, point)
        if self.home_map then self.home_map:dragStart(point) end
    end,

    -- 拖拽中...
    drag = function(self, point)
        if self.home_map then self.home_map:drag(point) end
    end,

    -- 拖拽结束
    dragEnd = function(self, point)
        if self.home_map then self.home_map:dragEnd(point) end
    end,

    -- 检测传递参数
    detectArgs = function(self, args)
        if args.recruit_hero_id and args.recruit_rst then
            self.recruit_hero = { id = args.recruit_hero_id }
            network_sender:endRecuritFight(args.recruit_rst, args.recruit_hero_id)
        end
    end,

    -- 检车修理包是否满了
    isRepairBagFull = function(self, msg)
        local max_num = 0
        if msg.makeInfo.maxNumLimit - msg.makeInfo.curNum >= msg.makeInfo.makeMaxNum then
            max_num = msg.makeInfo.makeMaxNum
        else
            max_num = msg.makeInfo.maxNumLimit - msg.makeInfo.curNum
        end
        if msg.makeInfo.state == 1 and max_num <= 0 then
            return true
        end
        return false
    end,

    onGetItems = function(self, msg)
        if msg.reason == common_pb.CHAPTER_TASK_REWARD or msg.reason == common_pb.MAKE_REPAIRBAG then
            frame_manager:show('getItems', self, {items = gui_helper.drop_list[msg.reason], has_Confirm_Btn = true})
        end
    end,

    jumpToTrainingBuild = function(self)
        local build = home_info:getMaxTrainingBuild()

        if build then
            local entity = self.home_map:getHomeBuildById(build.id)

            if entity then
                self.home_map:showOperator(entity)
            else
                log.error("##################################### not find entity id", build.id)
            end
        else
            log.error("########################################### not find training build")
        end
    end,
    
    onExit = function(self, to_state_name)
        --frame_manager:hide('home')
        frame_manager:hideAll()
        network:unregisterMessageHandler(self)
        if self.home_map then self.home_map:onExit() end
        self.home_map = nil
        self.gui = nil
        game_state_home = nil
        game_director.pool:destroy()
        drill_ground_helper:destroy()

        if self.event_ids then
            game_director.events:remove("NOTIFYGETITEM", self.event_ids["NOTIFYGETITEM"])
        end

        if self.coroutine_id then
            app:stopCoroutine(self.coroutine_id)
        end
    end,
}