super = require('game.game_state_base')
local log = logger.get("Game", "game_state_level")
local level = require('level.level')

return class {
    super = super,

    ctor = function(self, ...)
        super.ctor(self, ...)
        game_state_level = self -- 全局表
        self.drop_items = {}    -- 掉落物品
    end,

    onEnter = function(self, args)
        log.info('game state level onEnter, scene_name:'..args.scene_name..', level_id:'..args.level_id)
        self.scene_name = args.scene_name
        self.level_id = args.level_id
        self.level = level(args.scene_name, args.level_id)
        self.level_manager = self.level.level_manager
        self.path_point_manager = self.level.path_point_manager
        self.camera = self.level_manager.mainCamera
        self.level_camera = self.level_manager.levelCamera
        self.level_camera.m_MoveSpeed = 200
        self.level_camera.m_SlideInertia = 5
        self.leave_map = false

        -- 战斗返回
        self.battle_back = args.battle_back

        -- 用户操作
        self.is_break_input = false
        self.is_fps = false

        -- 预加载
        game_director.pool:prepare(DROP_ITEM_PREFAB, 5)

        self.gui = frame_manager:show('level', self, {back = self.battle_back})
        app:startCoroutine(function ()
            -- 等待 level_loading 界面显示完成
            local level_loading_gui = frame_manager:getFrame('level_loading')
            if level_loading_gui then
                level_loading_gui:setProgress(1.0)
                while true do
                    if level_loading_gui.finished then
                        log.info('#### level_loading finished!!')
                        break
                    else
                        coroutine.yield(nil)
                    end
                end
            end
            --coroutine.yield(Fenix.Async.WaitForFrame(self.gui))
            coroutine.yield(frame_manager:waitForFrame(self.gui))

            self.level.gui = self.gui
            
            if self.level.cfg.level_type == 2 then
                -- 默认阵型是散阵, 用关卡默认阵型进行自动布阵
                if not level_info.is_formationed then
                    --log.info("###### Auto formation :"..self.level.cfg.default_formation)
                    self.gui:autoFormation(self.level_id, self.level.cfg.default_formation)
                    level_info.is_formationed = true
                elseif level_info.hero_killed then
                    -- 当前阵型有人阵亡,重新自动上阵
                    self.gui:autoFormation(self.level_id, level_info.formation_id)
                end
            end
            if frame_manager:isDisplay('level_loading') then
                frame_manager:hide('level_loading')
            end
            if frame_manager:isDisplay('loading') then
                frame_manager:hide('loading')
            end
        end )

        self.evt_entityChanged = game_director.events:on("ENTITY_CHANGED", functor.make(self.onEntityChanged, self))

        network:registerMessageHandler(self)

        --emp_manager:closeAimFrame()

        --log.info("self.level.cfg.level_type:"..self.level.cfg.level_type..", formationId:".. game_info.level_attend_heros.formationId)



        --[[
        -- boost信息
        self.evt_SCBoostListInfo = game_director.events:on("BOOSTLISTINFO", functor.make(self.onSCBoostListInfo, self))
        if level_info.boost_info then 
            self:onSCBoostListInfo(level_info.boost_info)
        end

        -- 随机事件 通过 messageHandle 发过来
        self.evt_SCNotifyRandEventInfo = game_director.events:on("NotifyRandEvent", functor.make(self.onSCNotifyRandEventInfo, self))
        ]]
    end,

    -- 关卡所有 entity 加载完成
    onEntitiesLoaded = function(self)
        -- 处理缓存的 entity changed
        if level_info.entityChanged and #level_info.entityChanged > 0 then 
            -- 地图实体变化 (可能在战斗结算时收到，也可能在关卡内收到)
            for k, msg in ipairs(level_info.entityChanged) do 
                self:onEntityChanged(msg)
            end
            level_info.entityChanged = {}
        end
    end,

    onUpdate = function(self, dt)
        self:updateUI(dt)
        self.level:update(dt)
    end,

    -- 用户界面交互
    updateUI = function(self, dt)

        local point = self:getTouchScreenPoint()
        --log.info("touch pos:"..tostring(point))
        if point then
            -- 是否点击到界面
            if RTSHelper.IsPointerOverUI(point.x, point.y) then
               -- log.info("#Point Over UI!!")
                return
            end
            self.last_touch_pos = self.touch_pos
            self.touch_pos = Vector3.New(point.x, point.y, 0)
        else
            -- 用户停止触碰屏幕
            if self.touch_pos and not self.is_fps then
                self:levelInteraction(self.touch_pos)
            end
            self.touch_pos = nil
            self.last_touch_pos = nil
        end

        if self.is_fps then
            self:updateFPS()
        end

    end,

    -- 进入 fps 模式
    setFPSMode = function(self, is_fps, pos, rot, is_raycast, attach_go, shoot_effect_id, fire_callback)
        self.is_fps = is_fps
        self.fps_fire_callback = fire_callback
        self.is_raycast = is_raycast
        self.shoot_effect_id = shoot_effect_id
        if self.is_fps then
            if self.level_manager.FPSCamera then
                self.level_manager.levelCamera:SetActive(false)
                self.level_manager.FPSCamera:SetActive(true)

                if attach_go then
                    self.level_manager.FPSCamera.transform:SetParent(attach_go.transform)
                end

                local trans = self.level_manager.FPSCamera.transform
                if pos then
                    trans.position = pos
                end
                if rot then
                    trans.rotation = rot
                end
                
                frame_manager:show("fps_aim")
               -- frame_manager:hide("level")
                frame_manager:setFloorVisibility(0, false)
            else
                self:setFPSMode(false)
            end
        else
            self.level_manager.levelCamera:SetActive(true)
            self.level_manager.FPSCamera:SetActive(false)

            --self.gui = frame_manager:show('level', self, {back = self.battle_back})
            frame_manager:setFloorVisibility(0, true)
            frame_manager:hide("fps_aim")
        end

        if is_raycast == true then
        end
    end,

    updateShootEffect = function(self, effect_id)
        self.shoot_effect_id = effect_id
    end,

    -- fps 模式更新
    updateFPS = function(self)
        if self.last_touch_pos and self.touch_pos then
            local delta = self.touch_pos - self.last_touch_pos
            local cam = self.level_manager.FPSCamera;
            local t = cam.transform
            local rx = delta.x / UnityEngine.Screen.width * 360
            local ry = delta.y / UnityEngine.Screen.height * 360
            local euler = t.rotation.eulerAngles

            euler.x = euler.x - ry
            euler.y = euler.y + rx

            if euler.x > 180 then
                euler.x = euler.x - 360
            elseif euler.x < -180 then
                euler.x = 360 + euler.x
            end

            if euler.x > 60 then
                euler.x = 60
                self.last_touch_pos = self.touch_pos
            elseif euler.x < -60 then
                euler.x = -60
                self.last_touch_pos = self.touch_pos
            end

            t.rotation = Quaternion.Euler(euler)
        end
    end,

    -- fps 开火
    fpsFire = function(self)
        if self.is_fps then

            --gameobject used to play animation
            local left = level_helper:findNode(self.level_manager.FPSCamera.transform, "Prefab_Weapon_Tank_Aisha")
            local right = level_helper:findNode(self.level_manager.FPSCamera.transform, "Prefab_Weapon_Tank_Aisha (1)")
    
            local left_anim = left:GetComponent(UnityEngine.Animation)
            local right_anim = right:GetComponent(UnityEngine.Animation)
            left_anim:Play("idle")
            right_anim:Play("idle")

            local rst, hit
            if self.is_raycast == true then
                --local mask = core.mask('Default')
                -- local mask = RTSHelper.GetLayerMask(core.layers['Tank'], core.layers['Ground'])
                local left_shoot = self.level_manager.FPSCamera.transform:Find('shoot_left')      --left cannon shoot point
                local right_shoot = self.level_manager.FPSCamera.transform:Find('shoot_right')    --right cannon shoot point
                --local ray_shoot = self.level_manager.FPSCamera.transform:Find('shoot_ray')        --acutal ray point to cast ray

                local fireCB = function()
                    if self.fps_fire_callback then
                        log.info('fps function call back1')
                        self.fps_fire_callback(hit)
                    end
                end
                app:startCoroutine(
                    function()
                        if self.cannon_side then   --left cannon fire
                            local r = UnityEngine.Ray(left_shoot.position, left_shoot.forward)
                            rst, hit = UnityEngine.Physics.Raycast(r, hit, 100000)     --casting ray

                            left_anim:Play("attack")
                            coroutine.yield(WaitForSeconds(0.233))
                            level_script:shootBullet(self.shoot_effect_id, left_shoot, left_shoot.position + left_shoot.forward * hit.distance, 600, fireCB)
                            self.cannon_side = false
                        elseif not self.cannon_side then
                            local r = UnityEngine.Ray(right_shoot.position, right_shoot.forward)
                            rst, hit = UnityEngine.Physics.Raycast(r, hit, 100000)     --casting ray

                            right_anim:Play("attack")
                            coroutine.yield(WaitForSeconds(0.233))
                            level_script:shootBullet(self.shoot_effect_id, right_shoot, right_shoot.position + right_shoot.forward * hit.distance, 600, fireCB)
                            self.cannon_side = true
                        end
                        --log.error('hit distance = '..hit.distance)
                    end
                )

                if hit and hit.distance > 0.1 then
                    log.info('distance = '..hit.distance)
                    
                elseif hit and hit.distance == 0 then
                end
                --debug.drawline(self.level_manager.FPSCamera.transform.position, Vector3.New(-638.3, 16.1, -70.2))
                if rst and hit then
                    log.info('hit sth')
                    --return hit
                    --debug.DrawLine(self.level_manager.FPSCamera.transform.position, hit.point)
                else
                    log.info('hit nothing')
                end
            else
                if self.fps_fire_callback then
                    log.info('fps function call back')
                    self.fps_fire_callback(hit)
                end
            end
        end
    end,

    -- 与关卡交互
    levelInteraction = function(self, point)
        log.info('level interaction!')

        if self.is_break_input --[[or self.gui.is_joystick_moving]] then
            return
        end

        if self.level.player.is_break_input then
            return
        end

        -- 汽油空了,如果携带有汽油道具，提示用户
        if level_info.gasoline <= 0 then
            if level_info:getItem(31005) > 0 then
                --self.gui:onClickItemIcon(nil, nil, 1)
                self.gui:highLightItem(31005)
                hint:setText(NativeText.GetLangStr(170114))
                return
            else
                -- 没汽油了，提示用户开不动了
                hint:setText(NativeText.GetLangStr(170115))
                return
            end
        end

        if level_info.is_gasoline_leak and level_info:getItem(31006) > 0 then
            self.gui:onClickItemIcon(nil, nil, 2)
        end

        local mask = core.mask('PathPoint')
        local r = self.camera:ScreenPointToRay(point)
        local rst, hit = UnityEngine.Physics.Raycast(r, hit, 99999, mask)
        if rst then
            local go = hit.transform.gameObject
            local path_point = go:GetComponent(PathPoint)

            if level_info.gasoline > 0 then
                if level_info.is_gasoline_leak then
                    level_info.gasoline = math.max(level_info.gasoline - 2, 0)
                    self.gui:animLeakIcon()
                else
                    level_info.gasoline = level_info.gasoline - 1
                end
                self.level:updateTurn(path_point.m_ID)
            end
  
        else
            log.info('hit nothing')
        end
    end,

    -- 获取屏幕触点
    getTouchScreenPoint = function(self)
        local point = nil
        if Input.GetMouseButton(0) then
            point = Input.mousePosition
        elseif Input.touchCount > 0 then
            local p = Input.GetTouch(0).position
            point = Vector3.New(p.x, p.y, 0)
        end

        return point
    end,
    
    getPathPoint = function(self, id)
        return self.path_point_manager:FindPathPoint(id)
    end,

    -- 找一个最近的 path point
    getNearestPathPoint = function(self, pos)
        return self.path_point_manager:FindNearestPathPoint(pos)
    end,

    -- 网络消息 
    onMessage = function(self, cmd, msg)
        if cmd == protocol_pb.POSTFOGINFO then
            -- 保存战争迷雾成功 
            if msg.RespCode == 0 then

            else
                log.info('save fog info failed!! RespCode:'..msg.RespCode)
            end
        elseif cmd == protocol_pb.NotityMonEnhance then 
            -- 通知怪物增强
            log.info('NotityMonEnhance', msg.entityIndex)
            self.level:notifyMonsterEnhancement(msg.entityIndex)
        elseif cmd == protocol_pb.POSTPOSINFO then
            -- 保存玩家位置成功 

        elseif cmd == protocol_pb.LEAVEMAP then
            if msg.respCode == 0 then
                self.leave_map = true
                level_info:reset()
                game_director:changeState('loading', {next_state = 'home', scene_name = 'home'})
            else
                log.info('Leave Map failed!! RespCode:'..msg.respCode)
            end
        elseif cmd == protocol_pb.RESETMAPINFO then
            if msg.RespCode == 0 then
                level_info:reset()
                game_director:changeState('loading', {next_state = 'home', scene_name = 'home'})
            else
                log.info('save player pos failed!! RespCode:'..msg.RespCode)
            end
        elseif cmd == protocol_pb.COMMONLEVELINTERATION then 
            if msg.respCode == 0 then
                --msg.rewardItems
                log.info("## COMMONLEVELINTERATION ok")
            else
                log.error("## COMMONLEVELINTERATION fail", msg.respCode)
            end
        elseif cmd == protocol_pb.COPYRESCHANGE then -- 玩家关卡资源改变
            self.gui:setResChange(msg)
        elseif cmd == protocol_pb.ATTACKMONSTER then 
            if msg.RespCode == 0 then
                if self.level.level_script then
                    self.level.level_script:battle()
                end

                --frame_manager:show('loading')
                --local war_types = require('Turn_Based.war_enums').war_types

                -- 浮点型属性要除 10000
                local function convertPropToFloat(props)
                    local str = "\r\n"
                    for _, prop in ipairs(props.ParPropIdValues) do
                        if config.attribute_config_attrs[prop.PropId].isfloat > 0 then
                            prop.PropValue = prop.PropValue * 0.0001
                        end
                        str = str .. ('id:'..prop.PropId..' value:'..prop.PropValue).. "\r\n"
                    end
                    log.info(str)
                end

                level_info.formation_t0 = {}    -- 己方的阵型
                level_info.formation_t1 = {}    -- 敌方的阵型

                -- 己方
                for k, props in ipairs(msg.HerosProps) do
                    log.info("### hero:"..props.HeroId.." tank:"..props.tankId..", pos:"..props.pos)
                    convertPropToFloat(props)
                    level_info.formation_t0[props.pos] = props
                end
                for k, props in ipairs(msg.PetsProps) do
                    convertPropToFloat(props)
                end

                -- 敌方
                for k, props in ipairs(msg.NpcHerosProps) do
                    log.info("### hero:"..props.HeroId.." tank:"..props.tankId..", pos:"..props.pos)
                    convertPropToFloat(props)
                    level_info.formation_t1[props.pos] = props
                end
                for k, props in ipairs(msg.NpcPetsProps) do
                    convertPropToFloat(props)
                end

                level_info.map_attack_info = msg

                -- 备份关卡脚本
                level_info.backup_level_script = self.level.level_script
                self.level.level_script = nil   -- 将关卡的 level_script 置空防止被 destroy

                level_info.is_simple_battle = false
                level_info.is_sneak = (msg.isSneakRaid and msg.isSneakRaid == 1)
                --log.info("###### receive is sneak:"..tostring(level_info.is_sneak))

                level_info.is_attack_from_back = (msg.attackFromBack and msg.attackFromBack == 1)
                --log.info("###### receive is is_attack_from_back:"..tostring(level_info.is_attack_from_back))

                -- 进入战斗
                game_director:changeState('battle', {stage_id=msg.stageId or 1, battle_id=msg.FightUId})
            else
                -- 离开关卡
                self:leaveLevel()

                --[[
                STAGE_ATK_NO_IN_PROCESS             =3004;//不在进行中的关卡
                STAGE_ATK_NO_MONSTER_TYPE           =3005;//目标非怪物
                STAGE_ATK_ENERGY_NO_ENOUGH          =3006;//关卡战斗，行动力不够
                STAGE_ATK_CHAPTER_NO_OPEN           =3007;//章节没开启
                STAGE_ATK_NO_TANK_IN_BAG            =3008;//背包中不存在对应英雄
                STAGE_ATK_FIGHT_ID_ERR              =3009;//战斗id不对
                ]]
                -- 错误处理
                log.error('attack monster failed', msg.RespCode)
            end
        elseif cmd == protocol_pb.COPYEMBATTLE then
            if msg.respCode == 0 then
                local formation_id = msg.formationId
                level_info.formation_id = formation_id

                if level_info.is_req_embattle then
                    level_info.is_req_embattle = false
                    local formation_cfg = config.formation_lists[formation_id]
                    local name = frame_helper:getLangCfg(formation_cfg.name_id)
                    local str = NativeText.GetLangStr(170067)
                    str = string.format(str, name)
                    hint:setText(str)
                end

                -- 尝试更换玩家车辆

            end
        elseif cmd == protocol_pb.LEVELCHANGEHEROPROP then
            level_info.hero_hp_percent = msg.curHpRatio / 10000
            if self.gui and self.gui:isLoaded() then
                self.gui:refreshHealthAndGasoline()
            end

            -- 先直接告诉服务器战斗胜利 syz
--[[
            local fightUId = msg.FightUId
            local heros_hp = {}
            for _,hero in ipairs(msg.HerosProps) do
                table.insert(heros_hp, {camp=0, cfgId=hero.HeroId, curHp=10000})
            end

            network_sender:killMonster(fightUId, 1, heros_hp)]]
        
        elseif cmd == protocol_pb.GETCOPYTASKREWARD then
            if msg.respCode == 0 then
                local quest = level_quest_manager:getQuest(msg.taskId)
                if quest then
                    level_quest_manager:setQuestState(quest, 3)
                else
                    log.error("Failed to find quest:"..msg.taskId)
                end
            else
                log.error("Get copy task reward failed:"..msg.respCode)
            end

        elseif cmd == protocol_pb.CHAPTERATTENDINFO then
            local count = 0
            for _,f in ipairs(msg.formations) do
                if f.location > 0 then
                    count = count + 1
                end
            end
            self.gui:refreshAutoFormationButton(msg.formationId, count)
        end

    end,

    onEntityChanged = function (self, msg)
        logger.log("game_state_level.onEntityChanged", msg.appearEntities and #msg.appearEntities, msg.disappearEntities and #msg.disappearEntities)
        for k, v in ipairs(level_info.entityChanged) do 
            if v == msg then 
                table.remove(level_info.entityChanged, k)
                break
            end
        end
        if msg.appearEntities then 
            for entity_idx, entity_obj in ipairs(msg.appearEntities) do 
                self.level:spawnMapEntity(entity_obj)
            end
        end
        if msg.disappearEntities then 
            for entity_idx, entity_id in ipairs(msg.disappearEntities) do 
                log.info('#### self.level:removeMapEntity:'..entity_id)
                local entity = self.level:getEntity(entity_id)
                if entity then
                    log.info("## entity drop item")
                    -- 执行掉落
                    self:dropItems(entity:getPosition(), entity_id)

                    -- 消失的 entity 是 monster
                    if entity.entity_type == common_pb.MET_MONSTER then
                        if game_info.attack_monster_id == entity_id and game_info.battle_result ~= 0 then
                            -- 之前战斗胜利消失的 entity
                        else
                            -- 其他方式，比如 emp 导致消失的 entity
                            level_script:onBattleWin(entity_id)
                        end
                    end
                else
                    log.info("## failed to find entity")
                end

                --self.level:removeMapEntity(entity_id)
                self.level:entityDisappear(entity_id)
                self:removeMapInfoEntity(entity_id)
            end
        end
    end, 


--[[
//道具获得通知
message SCGetItem
{
  optional ITEM_CHANGE_REASON  reason    = 1;        //原因
  optional int32 reasonId	  			 = 2;		 //怪物实体id	
  repeated MGetItem getItems       		 = 3;        //道具改变列表
}
message MGetItem
{
	required string uId   	= 1;  //唯一Id	
	required uint32 itemId  = 2;  //item Id
	required uint32 getNum  = 3;  //获得数量
}
]]
    -- 物品掉落, 只是表现
    dropItems = function(self, pos, entity_id)
        -- 遍历缓存的 drop item
        for k, drop_item in ipairs(game_info.drop_items) do
            if drop_item then
                log.info("drop_itme reasonId:"..drop_item.reasonId)
                if drop_item.reasonId == entity_id then

                    game_info.drop_items[k] = false
                    for idx,get_item in ipairs(drop_item.getItems) do
                        if get_item.itemId == 31004 then
                            --emp 掉落
                            self:dropEMP(idx*0.2, pos)
                        else
                            -- 其他物品掉落
                            self:_delayCreateDropItem(idx*0.2, pos, get_item.itemId, get_item.getNum)
                        end
                    end
                else
                    log.info("### reasonId:"..tostring(drop_item.reasonId))
                end

            end -- if drop_item
        end
    end,

    _delayCreateDropItem = function(self, delay, pos, id, num)
        app:startCoroutine(
            function()
                coroutine.yield(WaitForSeconds(delay))
                local item = level_helper:dropItem(pos, id, num)
                table.insert(self.drop_items, item)
            end
        )
    end,

    dropEMP = function(self, delay, pos)
        app:startCoroutine(
            function()
                coroutine.yield(WaitForSeconds(delay))
                if level_frame then
                    level_frame:addEmpPower(pos)
                end
            end
        )
    end,

    -- 从 level_info 的 map_info 中删除 entity
    removeMapInfoEntity = function(self, id)
        if level_info.map_info.mapEntities then
            local index = 0
            for k, entity_obj in ipairs(level_info.map_info.mapEntities) do
                if entity_obj.entityId == id then
                    index = k
                end
            end
            if index ~= 0 then
                table.remove( level_info.map_info.mapEntities, index )
            end
        end
    end,

    onSCBoostListInfo = function(self, msg)
        level_info.boost_info = nil
        frame_manager:show('stmt', self, msg)
    end,

    -- 调用gui接口的时候，由于gui没有创建成功，所以在level_frame init时候，主动获取数据
    onSCNotifyRandEventInfo = function(self, msg)

        frameHelper.log("game state level random info type :" .. msg.notifyType)

        if self.gui then
            self.gui:setRandomEvent(level_info.randomEvent)
        else
            print("level frame 尚未加载完成...")
        end
    end,

    -- 中断用户操作
    breakPlayerInput = function(self)
        self.is_break_input = true
        if self.level.player then
            self.level.player:setBreakInput(true)
        end
    end,

    -- 恢复用户操作
    resumePlayerInput = function(self)
        self.is_break_input = false
        if self.level and self.level.player then
            self.level.player:setBreakInput(false)
        end
    end,

    lookTank = function(self)
        self.level_camera:LookTargetWithOffset(self.level.player.transform, self.level_camera.m_InitOffset)
        self.is_slide = false
        -- network_sender:gm("addhero 123")
    end,

    lookMonster = function(self, entityIndex, groupId)
        local mst = self:getRandomEventMonster(entityIndex, groupId)
        if mst then
            -- print("lookMonster", mst.transform.name)
            self.level_camera:SetLookTarget(mst.transform)
            self.is_slide = false
        end
    end,

    lookTarget = function(self, trans)
        self.level_camera:SetLookTarget(trans)
        self.is_slide = false
    end,

    -- 请求离开关卡
    leaveLevel = function(self)
        self:clearCachedLevelInfo()
        log.info("### leaveLevel!!")

        network_sender:leaveMap(level_info.map_id)

        if self.level then
            level_info:reset()
            --[[
            if not self.level:isStoryLevel() then
                self.level:saveLevelInfo()
            else
                level_info:reset()
            end]]
        end
    end,

    -- 放弃关卡
    giveUpLevel = function(self)
        self:clearCachedLevelInfo()
        self.level:resetLevel()
    end,

    -- 保存关卡信息
    cacheLevelInfo = function(self)
        local level = self.level
        if level then
            --level_info.fog_data = level.level_manager:SerializeFog()
            if level.player.path_point_id then
                local point = self.path_point_manager:FindPathPoint(level.player.path_point_id)
                level_info.player_pos = point.position
            else
                level_info.player_pos = level.player:getPosition()
            end

            --log.info("### cacheLevelInfo:"..tostring(level_info.player_pos))
            --self.level:saveLevelInfo()
        else
            log.error("Failed to find level in cacheLevelInfo!!")
        end
    end,

    clearCachedLevelInfo = function(self)
        log.info("### clearCachedLevelInfo.")
        level_info.player_pos = nil
        level_info.fog_data = nil
    end,

    onExit = function(self)
        frame_manager:hideAll()
        -- frame_manager:hide('level')
        -- self.gui:force_destroy()
        game_director.events:off("ENTITY_CHANGED", self.evt_entityChanged)
        game_director.events:off("BOOSTLISTINFO", self.evt_SCBoostListInfo)
        game_director.events:off("NotifyRandEvent", self.evt_SCNotifyRandEventInfo)
        
        network:unregisterMessageHandler(self)

        if self.level then
            self.level:destroy()
            self.level = nil
        end

        game_info.drop_items = {}
        self.gui = nil
    
        if app:isSmallMemory() then
            game_director.pool:destroy()
        else
            --game_director.pool:hideAll()
            game_director.pool:destroy()
        end

        -- 将所有用到的资源输出
        if app:isDebug() then
            local _str = ""
            for asset_path, v in pairs(game_director.pool.pools) do
                _str = _str .. asset_path.."\n"
            end

            print("========================== all assets in pool =========================")
            print(_str)
            print("===============================================================")
        end
  
    end,

    -- 获取随机事件monster的transform
    getRandomEventMonster = function(self, entity_id)
        return self.level:getEntity(entity_id)
    end,


}