local super = require('game.game_state_base')
local log = logger.get("Game", "game_state_world_map")
local world_map = require('world_map.world_map')
require("world_map.search_info")

return class {
    super = super,

    ctor = function(self, ...)
        super.ctor(self, ...)
    end,

    onEnter = function(self, args)
        -- 关闭编辑界面的显示
        local editor = GameObject.Find('Hex Map Editor')
        if editor then
            editor:SetActive(false)
        end

        self.args = args
        -- 清空 world map info
        world_map_info:clear()

        -- 创建 world_map 实例
        self.world_map = world_map(self)
        self.world_map:init()
        
        -- 交互
        self.camera = Camera.main
        self.first_touch = false
        self.is_slide = false
        self.touch_pos = nil
        -- 是否点击在UI界面上
        self.is_click_ui = false
        -- 第一次点击的坐标
        self.first_touch_pos = nil
        
        self.hold_creating_info = {}

        -- 向服务器请求世界地图信息
        -- network_sender:enterWorldMap(self.map_type)
        -- network:registerMessageHandler(self)
    end,

    onUpdate = function(self, dt)
        self:updateUI(dt)
        self.world_map:update(dt)

        -- 每帧创建一个对象
        if #self.hold_creating_info > 0 then
            local info = self.hold_creating_info[1]
            table.remove(self.hold_creating_info, 1)

           -- log.info("### createHold type:"..info.type..", id:"..info.id)
            self.world_map:createHold(info.type, info.value)    
            --if info.type == hold_types.armyLine then
                --game_director.events:dispatch("WORLDMAPARMYLINEAPPEAR", info.value)
            --end
        end
    end,

    -- 添加创建信息
    addCreatingInfo = function(self, id, type, value)
        --log.info("### addCreatingInfo type:"..type..", id:"..id)
        table.insert(self.hold_creating_info, {id=id,type=type, value=value})
    end,

    -- 删除创建信息
    removeCreatingInfo = function(self, id, type)
        local idx = nil
        for k,v in ipairs(self.hold_creating_info) do
            if v.id == id and v.type == type then
                idx = k
                break
            end
        end

        if idx then
            table.remove(self.hold_creating_info, idx)
        end
    end,

    -- 网络消息 
    onMessage = function(self, cmd, msg)
        
        if cmd == protocol_pb.ENTERWORLDMAP then
            -- 进入世界地图
            self.world_map:reload()
            self.world_map:setCameraPos(msg.posX, msg.posY)
            game_info.player_pos.x = msg.posX
            game_info.player_pos.y = msg.posY
            --self.world_map.cs_inst:GenerateMap(msg.posX, msg.posY)
            -- 坐标收藏
            self.collections = {}
            if msg.collectionPos then
                for _, v in ipairs(msg.collectionPos) do
                    self.collections[v.id] = v
                end
            end
            local rst, frame = frame_manager:isDisplay('coord_manager')
            if rst then 
                frame:addCommandBuff(function()
                    frame:refreshCollectCoord() 
                end)
            end

            self:counterJump()
            self:createGuildBuilding()
            world_map_helper:excuteJumpTohold()
            world_map_helper:excuteJumpToArmyLine()
        elseif cmd == protocol_pb.WORLDMAPOBJSAPPEAR then
            -- 世界地图物件出现
            -- 玩家据点信息

            for _,v in ipairs(msg.playerHolds) do
                world_map_info.player_holds[v.playerId] = v
                if v.playerId == local_player.id then
                    world_map_info.self_hold = v    -- 本地玩家自己的据点
                end
                --self.world_map:createHold(hold_types.player, v)   
                self:addCreatingInfo(v.playerId, hold_types.player, v)
            end

            -- 钛矿
            for _,v in ipairs(msg.wildmineholds) do
                world_map_info.titanium_holds[v.objinfo.instid] = v
                --self.world_map:createHold(hold_types.titanMine, v)
                self:addCreatingInfo(v.objinfo.instid, hold_types.titanMine, v)
            end
            
            -- 行军信息
            for _,v in ipairs(msg.armyLine) do
                log.info('############################### 创建行军路线:', v.instId, v.totalTime, v.elapsedTime)
                world_map_info.march_lines[v.instId] = v
                --self.world_map:createHold(hold_types.armyLine, v)
                --game_director.events:dispatch("WORLDMAPARMYLINEAPPEAR", v)
                self:addCreatingInfo(v.instId, hold_types.armyLine, v)
            end

            for _,v in ipairs(msg.monsterHolds) do
               -- log.error("#### create hold:"..v.objinfo.instid)
                world_map_info.monster_holds[v.objinfo.instid] = v
                --self.world_map:createHold(hold_types.monster, v)
                self:addCreatingInfo(v.objinfo.instid, hold_types.monster, v)
            end

            for _,v in ipairs(msg.familybase) do
                 world_map_info.guild_base_holds[v.objinfo.instid] = v
                 self:addCreatingInfo(v.objinfo.instid, hold_types.guild, v)
            end

           -- log.error("#### create monster holds num:"..#msg.monsterHolds)
            -- 怪物信息
            --[[
            app:startCoroutine(
                function()
                    for _,v in ipairs(msg.playerHolds) do
                        coroutine.yield(nil)
                        world_map_info.player_holds[v.playerId] = v
                        if v.playerId == local_player.id then
                            world_map_info.self_hold = v    -- 本地玩家自己的据点
                        end
                        self.world_map:createHold(hold_types.player, v)     
                    end
        
                    -- 钛矿
                    for _,v in ipairs(msg.wildmineholds) do
                        coroutine.yield(nil)
                        world_map_info.titanium_holds[v.objinfo.instid] = v
                        self.world_map:createHold(hold_types.titanMine, v)
                    end
                    
                    -- 行军信息
                    for _,v in ipairs(msg.armyLine) do
                        coroutine.yield(nil)
                        log.info('############################### 创建行军路线:', v.instId, v.totalTime, v.elapsedTime)
                        world_map_info.march_lines[v.instId] = v
                        self.world_map:createHold(hold_types.armyLine, v)
                        game_director.events:dispatch("WORLDMAPARMYLINEAPPEAR", v)
                    end

                    for _,v in ipairs(msg.monsterHolds) do
                        -- log.error("#### create hold:"..v.objinfo.instid)
                        coroutine.yield(nil)
                         world_map_info.monster_holds[v.objinfo.instid] = v
                         self.world_map:createHold(hold_types.monster, v)
                        
                    end
                end
            )]]



        elseif cmd == protocol_pb.WORLDMAPOBJSDISAPPEAR then

            for _,v in ipairs(msg.wmOjbs.playerHolds) do
                world_map_info.player_holds[v] = nil   

                --log.info("### removeHold player:"..v)
                if self.world_map:removeHold(hold_types.player, v) then
                   -- log.info("### removeHold player:"..v)
                else
                    self:removeCreatingInfo(v, hold_types.player)
                end
            end

            -- 钛矿
            for _,v in ipairs(msg.wmOjbs.wildMineHolds) do
                world_map_info.titanium_holds[v] = nil
                if self.world_map:removeHold(hold_types.titanMine, v) then
                else
                    self:removeCreatingInfo(v, hold_types.titanMine)
                end
            end

            --log.error("#### remove monster holds num:"..#msg.monsterHolds)
            for _,v in ipairs(msg.wmOjbs.monsterHolds) do
               -- log.error("#### remove hold:"..v)
                world_map_info.monster_holds[v] = nil
                if self.world_map:removeHold(hold_types.monster, v) then
                      -- log.info("### removeHold monster:"..v)                    
                else
                    self:removeCreatingInfo(v, hold_types.monster)
                end
            end

            -- 行军信息
            for _,v in ipairs(msg.wmOjbs.armyLine) do
               -- log.info('#################### 删除行军路线:', v)
                local march_line = world_map_info.march_lines[v]
                if march_line then
                    if self.world_map:removeHold(hold_types.armyLine, v) then
 
                    else
                        self:removeCreatingInfo(v, hold_types.armyLine)
                    end
                end
                world_map_info.march_lines[v] = nil
            end

            -- 联盟堡垒
            for _,v in ipairs(msg.wmOjbs.familybase) do
                local guild_base = world_map_info.guild_base_holds[v]
                if guild_base then
                    if self.world_map:removeHold(hold_types.guild, v) then
 
                    else
                        self:removeCreatingInfo(v, hold_types.guild)
                    end
                end
                world_map_info.guild_base_holds[v] = nil
            end
        elseif cmd == protocol_pb.WORLDMAPOBJOUT then
            for _,v in ipairs(msg.wmOjbs.playerHolds) do
                if self.world_map:hideHold(hold_types.player, v) then
                else
                    self:removeCreatingInfo(v, hold_types.player)
                end
            end

            -- 钛矿
            for _,v in ipairs(msg.wmOjbs.wildMineHolds) do
                if self.world_map:hideHold(hold_types.titanMine, v) then
                else
                    self:removeCreatingInfo(v, hold_types.titanMine)
                end
            end

           -- log.error("#### out monster holds num:"..#msg.wmOjbs.monsterHolds)
            for _,v in ipairs(msg.wmOjbs.monsterHolds) do
               -- log.error("#### out monster holds:"..v)
                if self.world_map:hideHold(hold_types.monster, v) then
                else
                    self:removeCreatingInfo(v, hold_types.monster)
                end
            end

            -- 行军信息
            for _,v in ipairs(msg.wmOjbs.marchLines) do

            end

            -- 联盟堡垒
            for _,v in ipairs(msg.wmOjbs.familybase) do
                if self.world_map:hideHold(hold_types.guild, v) then

                else
                    self:removeCreatingInfo(v, hold_types.guild)
                end
            end

        elseif cmd == protocol_pb.ENTERMAP then
            -- 进入关卡
            if msg.RespCode == 0 then
                level_info.map_info = msg    -- 保存地图信息
                level_info.map_id = msg.mapId
                local level_cfg = config.game_copy_levels[msg.mapId]
                level_info.level_cfg = level_cfg
                game_director:changeState('loading', {next_state = 'level', scene_name = level_cfg.Map, level_id = msg.mapId})
            else
                log.info('enter failed, RespCode:'..msg.RespCode)
            end
        elseif cmd == protocol_pb.WORLDMAPQUERYOBJ then
            if msg.respCode == 0 then
                print("查询信息:", msg.eObjType, msg.objId)
                self.world_map:queryHoldInfo(msg)
            else
                log.info('query world map Failed!! respCode:'..msg.respCode)
            end
        elseif cmd == protocol_pb.WORLDMAPOBJQUERY then
            if msg.respCode == 0 then
                local info = msg.objQuery
                
                --game_director.events:dispatch("WORLDMAPOBJQUERY")
            end
        elseif cmd == protocol_pb.WORLDMAPCOLLECTIONADD then
            if msg.respCode == 0 then
                self.collections[msg.collectionPos.id] = msg.collectionPos
                self:collectCoordChanged(msg.collectionPos)
                local rst, frame = frame_manager:isDisplay('coord_manager')
                if rst then 
                    frame:addCommandBuff(function()
                        frame:refreshCollectCoord() 
                    end)
                end
            else
                log.info('world map add collection failed:'..msg.respCode)
            end
        elseif cmd == protocol_pb.WORLDMAPCOLLECTIONMOD then
            if msg.respCode == 0 then
                self.collections[msg.collectionPos.id] = msg.collectionPos
                local rst, frame = frame_manager:isDisplay('coord_manager')
                if rst then 
                    frame:addCommandBuff(function()
                        frame:refreshCollectCoord() 
                    end)
                end
                -- print("修改收藏坐标:", msg.collectionPos.eTable)
                self:collectCoordChanged(msg.collectionPos)
            else
                log.info('world map mod collection failed:'..msg.respCode)
            end
        elseif cmd == protocol_pb.WORLDMAPCOLLECTIONDEL then
            if msg.respCode == 0 then
                self.collections[msg.collectionId] = nil
                local rst, frame = frame_manager:isDisplay('coord_manager')
                if rst then 
                    frame:addCommandBuff(function()
                        frame:refreshCollectCoord() 
                    end)
                end
                self:collectCoordChanged(msg.collectionPos)
            else
                log.info('world map del collection failed:'..msg.respCode)
            end
        elseif cmd == protocol_pb.WORLDMAPQUERYTIME then
            if msg.respCode == 0 then
                game_director.events:dispatch("WORLDMAPQUERYTIME", msg)
            end
        elseif cmd == protocol_pb.WORLDMAPMOVERES then
            if msg.respCode == 0 then
                game_director.events:dispatch("WORLDMAPMOVERES", msg)
            end
        elseif cmd == protocol_pb.WORLDMAPGETHELPS then
            if msg.respCode == 0 then
                game_director.events:dispatch("WORLDMAPGETHELPS", msg)
            end
        elseif cmd == protocol_pb.WORLDMAPSEARCH then
            if msg.respCode == 0 then
                search_info:init(msg.machIndexs)
                
                if #msg.monsters > 0 then
                    search_info:addMonsters(msg.monsters)
                end

                --[[ if #msg.mines > 0 then
                    search_info:addRess(msg.mines)
                end ]]

                if #msg.wilds > 0 then
                    search_info:addTitanium(msg.wilds)
                end

                game_director.events:dispatch("WORLDMAPSEARCH", msg.searchType)
            end
        elseif cmd == protocol_pb.WORLDMAPMINIMAP then
            if msg.respCode == 0 then
                game_director.events:dispatch("WORLDMAPMINIMAP", msg.objList)
            end
        elseif cmd == protocol_pb.WORLDMAPCHANGENAME then
            if msg.respCode == 0 then
                self.world_map:holdNameChanged(msg.instId, msg.newName)
            end
        elseif cmd == protocol_pb.WORLDMAPSTARTTRANSITION then
            if msg.respCode == 0 then
                --self.world_map.my_hold:changePos()
            end
        elseif cmd == protocol_pb.WORLDMAPRANDTRANSITION then
            if msg.respCode == 0 then
                --self.world_map.my_hold:changePos()
                --self:lookAtMyHold()
            end
        elseif cmd == protocol_pb.BATSHOWNOTIFY then
            world_map_battle:addBattleInfo(msg.batShow)
        end
    end,

    -- 用户界面交互
    updateUI = function(self, dt)
        --[[
        local point = self:getTouchPos()

        if point then
            -- 是否点击到界面
            if RTSHelper.IsPointerOverUI(point.x, point.y) then
                return
            end

            if self.first_touch then
                self.first_touch_pos = point
                self.first_touch = false
            end

            -- 判断是否要移动摄像机
            if not self.is_slide then
                self.is_slide = self:isSlide(point)
            end

            -- 如果用户滑动屏幕，移动摄像机
            if self.is_slide then

            end

            self.touch_pos = Vector3.New(point.x, point.y, 0)
        else
            -- 用户没有触碰屏幕
            if self.touch_pos then
                if not self.is_slide then
                    self:interaction(self.touch_pos)
                end
            end

            self.first_touch = true
            self.touch_pos = nil
            self.is_slide = false
        end
        ]]

        local point = Input.mousePosition
        -- 点击在UI上面
        if Input.GetMouseButtonDown(0) then
            self.is_click_ui = RTSHelper.IsPointerOverUI(point.x, point.y)
            self.is_slider = false
            self.first_touch_pos = point
            self.drag_start = true
        elseif Input.GetMouseButton(0) and (not self.is_click_ui) then
            if not self.is_slider then
                self.is_slider = self:isSlide(point)
            else
                if self.drag_start then
                    self.drag_start = not self.drag_start
                    self:dragStart(point)
                end
                self:drag(point)
            end
        elseif Input.GetMouseButtonUp(0) and (not self.is_click_ui) then
            if self.is_slider then
                self:dragEnd(point)
            else
                self:click(point)
            end
        end

        -- 更新坐标
        local coord = self.world_map.cs_inst:GetCameraCoord()
        if coord.x >= 0 and coord.y >= 0 and self.gui then
            self.gui:setCameraCoordant(coord.x, coord.y)
        end
    end,

    -- 点击
    click = function(self, point)
        -- print('点击')
        if self.world_map.begain_build then
            return
        end
        self:interaction(Vector3.New(point.x, point.y, 0))
        self.world_map:click(point)
    end,

    -- 开始拖拽
    dragStart = function(self, point)
        -- print('开始拖拽')
        self.world_map:dragStart(point)
    end,

    -- 拖拽中...
    drag = function(self, point)
        -- print('拖拽中...')
        self.world_map:drag(point)
    end,

    -- 拖拽结束
    dragEnd = function(self, point)
        -- print('拖拽结束')
        self.world_map:dragEnd(point)
    end,

    -- 本次操作是否在屏幕滑动
    isSlide = function(self, point)
        if self.first_touch_pos then
            local d = self.first_touch_pos - point
            return Vector3.SqrMagnitude(d) > 10
        end
    end,

    -- 与世界地图交互
    interaction = function(self, point)
        --log.info('level interaction!')
        local mask = core.mask('Tank') -- , 'Ground'
        -- local mask = RTSHelper.GetLayerMask(core.layers['Tank'], core.layers['Ground'])
        local r = self.camera:ScreenPointToRay(point)
        local rst, hit = UnityEngine.Physics.Raycast(r, hit, 999, mask)
        if rst then
            local go = hit.transform.gameObject

            if go.layer == core.layers['Ground'] then
                log.info('touch groudn!!')
            else
                local actor = go:GetComponent(WorldMapActor) or go:GetComponent(MarchLineActor)   -- 
                if actor then
                    log.info('touch world actor:'..actor.id)
                    network_sender:queryWorldMap(actor.id, self.map_type)
                else
                    log.info('touch ???')
                end
            end
        else
            log.info('hit nothing')
        end
    end,

    -- 获取
    getTouchPos = function(self)
        local point = nil
        if Input.GetMouseButton(0) then
            point = Input.mousePosition
        elseif Input.touchCount > 0 then
            local p = Input.GetTouch(0).position
            point = Vector3.New(p.x, p.y, 0)
        end

        return point
    end,

    -- 摄像机回到玩家自己的据点位置
    lookAtMyHold = function(self)
        self.world_map:lookAtMyHold()
    end,

    -- 看任意 transform
    lookAtTarget = function(self, target)
        self.world_map:lookAtTarget(target)
    end,

    lookAtMarchLine= function(self, id)
        self.world_map:lookAtMarchLine(id)
    end,

    setCameraPos = function(self, iX, iZ)
        self.world_map:setCameraPos(iX, iZ)
    end,

    -- 摄像机定位到某个坐标
    lookAtPosition = function(self, x, z)
        self.world_map:lookAtPosition(x, z)
    end,

    getWorldMapBorder = function(self)
        return self.world_map:getWorldMapBorder()
    end,
    
    collectePosCheck = function(self, x, y)
        for k, v in pairs(self.collections) do
            if v.posX == x and v.posY == y then
                return v
            end
        end
        
        return false
    end,

    collectCoordChanged = function(self, collect_pos)
        if self.world_map then
            self.world_map.hold_operator:refresh()
        end
    end,

    counterJump = function(self)
        if self.args.counterJump then
            self.coroutine_id = app:startCoroutine(
                function()
                    coroutine.yield(WaitForSeconds(0.5))
                    local hold = world_map_helper:getHoldByPos(game_info.player_pos.x, game_info.player_pos.y)

                    if hold then
                        network_sender:queryWorldMap(hold.id, protocol_pb.MAP_WORLDMAP)
                    end
                end
            ) 
        end
    end,

    createGuildBuilding = function(self)
        local data = self.args.create_guild_build
        if data then
            self.coroutine_id = app:startCoroutine(
                function()
                    coroutine.yield(WaitForSeconds(0.5))
                    world_map_helper:createTempHold(data.id, data.type)
                end
            ) 
        end
    end,

    onExit = function(self)
        if self.world_map then self.world_map:onExit() end
       -- if self.gui then self.gui:destroy() end
        super.onExit(self)
        frame_manager:hideAll()
        network:unregisterMessageHandler(self)
        world_map_info:clear()
        world_map_battle:clear()
        --self.gui = nil
        self.world_map = nil
        self.camera = nil
        game_director.pool:destroy()

        if self.coroutine_id then
            app:stopCoroutine(self.coroutine_id)
        end
    end,
}