local log = logger.get("Game", "player")

player_cls = class {
    ctor = function(self)
        self.id = 0         -- 玩家 id
        self.name = nil     -- 名称
        self.events = require("base.event").new()
        self.guild_id = "" -- 玩家工会id

        --[[
        //玩家属性类型
        enum PLAYER_PROP
        {
            PROP_LEVEL = 0; //等级
            PROP_CUREXP = 1; //经验
            PROP_GOLD   = 2;   //金币
            PROP_DIAMOND = 3; //钻石
            PROP_COPYENERGY = 4; //关卡行动力
            PROR_STEEL = 5;  //钢材
            PROP_WOOD  = 6;  //木头
            PROP_BRICK = 7;  //砖头
            PROP_NUCLEAR = 8;//核能	
            PROP_HORME_FREETIME = 9; //建造免费加速时间
            PROP_NUM = 9;
        }]]

        self.props = {}     -- 玩家数值类属性，
        local i = 0
        for i=0,protocol_pb.PROP_NUM,1 do 
            self.props[i] = 0
        end
        self.props[""] = 0

        self.globale_props = {}

        self.mail_list = nil
    end,

    -- 玩家属性发生变化
    onPropChange = function(self, prop, value)
        self.props[prop] = value
        if prop == protocol_pb.PROP_HORME_FREETIME then
            home_info:setBuildFreeTime(value)
        end
        if prop == protocol_pb.PROP_FOOD then
            self.food_rmb_time = Time.realtimeSinceStartup
        end
    end,

    dispatchEvents = function(self)
        self.events:dispatch("onPropChange")
    end,

    -- player_config 属性发生变化
    onGlobalPropChange = function(self, prop, value)
        self.globale_props[prop] = value
        if prop == protocol_pb.PROP_FOOD then
            self.food_rmb_time = Time.realtimeSinceStartup
        end
        self.events:dispatch("onGlobalPropChange")
    end,

    getGlobalProp = function(self, id)
        local rst = self.globale_props[id] or 0
        --log.warn("获得属性id:", id , rst)
        return rst
    end,

    getServerId = function(self)
        return self.server_id or 1
    end,

    getLevel = function(self)
        return self.props[protocol_pb.PROP_LEVEL]
    end,

    getExp = function(self)
        return self.props[protocol_pb.PROP_CUREXP]
    end,

    getSteel = function(self)
        return self.props[protocol_pb.PROR_STEEL]
    end,

    getWood = function(self)
        return self.props[protocol_pb.PROP_WOOD]
    end,

    getBrick = function(self)
        return self.props[protocol_pb.PROP_BRICK]
    end,

    getNuclear = function(self)
        return self.props[protocol_pb.PROP_NUCLEAR]
    end,

    getHomeFreeTime = function(self)
        return self.props[protocol_pb.PROP_HORME_FREETIME]
    end,

    getCoin = function(self)
        return self.props[protocol_pb.PROP_GOLD]
    end,

    getGem = function(self)
        return self.props[protocol_pb.PROP_DIAMOND]
    end,

    getBagMaxAmount = function(self)
        return self.props[protocol_pb.PROP_BAG_CAPACITY]
    end,

    getStrength = function(self)
        return self.props[protocol_pb.PROP_PLAYER_STRENGTH]
    end,

    getWeaponMaxLv = function(self)
        return self.props[protocol_pb.PROP_WEAPON_ENHANCE_LEVEL_MAX]
    end,

    getEquipMaxLv = function(self)
        return self.props[protocol_pb.PROP_ARMOR_ENHANCE_LEVEL_MAX]
    end,

    getMaxHeroCnt = function(self)
        return self.props[protocol_pb.PROP_HERO_MAX_NUM]
    end,

    getMaxRepairKit = function(self)
        return self.props[protocol_pb.PROP_FIXPACKAGE_MAX_NUM]
    end,

    getFuel = function(self)
        return self.props[protocol_pb.PROP_FUEL_NUM] or 0
    end,

    getFuelMax = function(self)
        return self.globale_props[common_pb.SYS_FUEL_UPLIMIT] or 0
    end,

    massSoldierMax = function(self)
        local base = self.globale_props[common_pb.SYS_GROUP_CORP_NUM] or 0
        local percent = self.globale_props[common_pb.SYS_GROUP_CORP_NUM_PERCENT] or 0

        local final = base * (percent*0.0001 + 1)

        return math.floor(final)
    end,

    massTeamMax = function(self)
        return self.globale_props[common_pb.SYS_MASS_LIMIT] or 0
    end,

    massPlayerTeamMax = function(self)
        return self.globale_props[common_pb.SYS_RALLY_TEAM_NUM_PERSONAL] or 0
    end,

    getEnergyMax = function(self)
        return self.globale_props[common_pb.SYS_ENERGY_LIMIT]
    end,

    getEnergyRecover = function(self)
        return self.globale_props[common_pb.SYS_ENERGY_RECOVER_RATE]
    end,

    print = function(self)
        local str = ""
        str = str .. ('==== player ===== \n')
        str = str .. ('level:'..tostring(self:getLevel()))..'\n'
        str = str .. ('exp:'..tostring(self:getExp()))..'\n'
        str = str .. ('coin:'..tostring(self:getCoin()))..'\n'
        str = str .. ('gem:'..tostring(self:getGem()))..'\n'
        str = str .. ('steel:'..tostring(self:getSteel()))..'\n'
        str = str .. ('wood:'..tostring(self:getWood()))..'\n'
        str = str .. ('brick:'..tostring(self:getBrick()))..'\n'
        str = str .. ('nuclear:'..tostring(self:getNuclear()))..'\n'
        log.info(str)
    end,

    setGuildId = function(self, guild_id)
        self.guild_id = guild_id
    end,

    getGuildId = function(self, guild_id)
        return self.guild_id
    end,
    
    isInGuild = function(self)
        if self.guild_id == "" then
            return false
        end
        return true
    end,
}

-- 本地玩家唯一实例
local_player = player_cls()
return local_player
