require('game.lord')
require('RTS.home_sys.home_info')

unlock_manager = {

    init = function(self)  
        self.LORD_INFO_CHANGE       = local_lord.events:on("LORD_INFO_CHANGE",              functor.make(self.onLordInfoChange, self))
        self.BUILD_INFO_CHANGE      = home_info.events:on("BUILD_INFO_CHANGE",              functor.make(self.onBuildInfoChange, self))
        self.LOADING_COMPLETED      = frame_manager.events:on("LOADING_COMPLETED",          functor.make(self.onLoadingCompleted, self))
        self.UNINSTALL_COMPLETED    = frame_manager.events:on("UNINSTALL_COMPLETED",        functor.make(self.onUnstallCompleted, self))
    end,

    -- 领主信息变化
    onLordInfoChange = function(self)
        self:excute()
    end,

    -- 建筑信息变化
    onBuildInfoChange = function(self, id)
        local build_data = home_info:getBuildDataById(id)
        if build_data and build_data.cfgId == 1 then
            self:excute()
        end
    end,

    -- 界面加载完成
    onLoadingCompleted = function(self, frame_name)
        self:excute()
    end,

    -- 界面卸载完成
    onUnstallCompleted = function(self, frame_name)
    end,

    excute = function(self)
        local refreshed = false

        config.system_unlock_unlocks:foreach(function(id, cfg)
            local vsb, frame = frame_manager:isDisplay(cfg.frame)
            if vsb and frame.is_loaded then
                local build_data = home_info:getBuildDataByCfgId(1)
                local boat_level = build_data and build_data.level or 0
                local is_lock = true
                if boat_level >= cfg.main_level and cfg.main_level > 0 then
                    is_lock = false
                end
                if local_lord.level >= cfg.lord_level and cfg.lord_level > 0 then
                    is_lock = false
                end

                local btn, image = self:attackTo(frame, cfg.path1)
                btn.onClick:RemoveAllListeners()
                btn.onClick:AddListener(function()
                    if is_lock then
                        if cfg.main_level > 0 then
                            hint:setText(string.format(NativeText.GetLangStr(900017), cfg.main_level))
                        end
                        if cfg.lord_level > 0 then
                            hint:setText(string.format(NativeText.GetLangStr(900018), cfg.lord_level))
                        end
                    end
                end)
                image.raycastTarget = is_lock

                -- 控制红点
                if not refreshed then
                    red_point_manager:refreshRedPoints()
                else
                    refreshed = true
                end
            end
        end)
    end,

    attackTo = function(self, frame, path)
        local btn_trans = frame.transform:Find(path .. "/lock")
        if btn_trans then
            local btn = btn_trans:GetComponent(UnityEngine.UI.Button)
            local image = btn_trans:GetComponent(UnityEngine.UI.Empty4Raycast)
            return btn, image
        else
            local parent_trans = frame.transform:Find(path)
            btn_trans = GameObject()
            btn_trans.name = "lock"
            btn_trans:SetParent(parent_trans)

            local rect = btn_trans:AddComponent(UnityEngine.RectTransform)
            rect.anchorMin = Vector2.zero
            rect.anchorMax = Vector2.one
            rect.localScale = Vector3.one
            rect.offsetMax = Vector2.zero
            rect.offsetMin = Vector2.zero
            -- rect.anchoredPosition = Vector2.zero

            local image = btn_trans:AddComponent(UnityEngine.UI.Empty4Raycast)
            local btn = btn_trans:AddComponent(UnityEngine.UI.Button)
            return btn, image
        end
    end,

    isUnlock = function(self, frame_name, path)
        local is_unlock = false
        local find = false
        config.system_unlock_unlocks:foreach(function(id, cfg)
            if cfg.frame == frame_name and cfg.path1 == path then
                find = true
                local build_data = home_info:getBuildDataByCfgId(1)
                local boat_level = build_data and build_data.level or 0
                if boat_level >= cfg.main_level and cfg.main_level > 0 then
                    is_unlock = true
                end
                if local_lord.level >= cfg.lord_level and cfg.lord_level > 0 then
                    is_unlock = true
                end
            end
        end)
        if not find then
            return true
        else
            return is_unlock
        end
    end,
}