local super = require("ui.frame")
local item_cls = require('RTS.bag_sys.item')

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        self.activity_id = self.args.data

        clickEvents = {
            "content/backBtn",
            "content/infoBtn",
            -- "content/tip/closeTipBtn",
        }

        -- 注册点击事件
        frame_helper:regClickEvents("click", self, clickEvents)

        self.ACTIVITY_INFO_CHANGE = activity_manager.events:on("ACTIVITY_INFO_CHANGE", function()
            self:refreshUI()
        end)

        self.ACTIVITY_TIME_TICK = activity_manager.events:on("ACTIVITY_TIME_TICK", function(id, elpased)
            self:refreshTime(id, elpased)
        end)

        self:init()
        self:refreshUI()
    end,

    onUnload = function (self)
        activity_manager.events:remove("ACTIVITY_INFO_CHANGE", self.ACTIVITY_INFO_CHANGE)
        activity_manager.events:remove("ACTIVITY_TIME_TICK", self.ACTIVITY_TIME_TICK)
    end,

    update = function(self)
        if Input.GetMouseButtonDown(0) then
            self:click_closeTipBtn()
        end
    end,

    showTip = function(self, position, cfg_id)
        local cfg = config.active_point_shows[cfg_id]
        local des = self:findText("content/tip/info/rally_panel_1/container/des")
        des.text = frame_helper:getLangCfg(cfg.PointDes)

        frame_helper.showTrans(self.tip_trans)

        local info_trans = self.transform:Find("content/tip/info")
        info_trans.position = position

        -- app:startCoroutine(function()
        --     coroutine.yield(nil)
        --     frame_helper.showTrans(self.tip_trans)
        --     self.tip_trans.position = position
        --     local height = des.preferredHeight
        --     outer.sizeDelta = Vector2(outer.sizeDelta.x, height)
        -- end)
    end,

    click_closeTipBtn = function(self)
        frame_helper.hideTrans(self.tip_trans)
    end,

    init = function(self)
        self.tip_trans = self.transform:Find("content/tip")
        frame_helper.hideTrans(self.tip_trans)
        local act = activity_manager:getActivityById(self.activity_id)
        local detail_cfg = config.active_active_details[act.detail_cfg_id]
        local list = loadstring("return " .. detail_cfg.PointSource)()
        local temp_tab = {}
        isValid = function(type_id)
            local cfg_id = nil
            config.active_point_shows:foreach(function(id, cfg)
                local list = loadstring("return " .. cfg.Type)()
                for __, ti in ipairs(list) do
                    if ti == type_id then
                        cfg_id = id
                        break
                    end
                end
            end)

            if cfg_id and (not temp_tab[cfg_id]) then
                return true, cfg_id
            else
                return false, cfg_id
            end
        end
        local path = "content/Left Scroll View/Viewport/Content"
        frame_helper:hideAllChild(self, path, true)
        local idx = 0
        for __, type_id in pairs(list) do
            local rst, cfg_id = isValid(type_id)
            if rst and cfg_id then
                temp_tab[cfg_id] = true
                idx = idx + 1
                local child = frame_helper:getChild(self, idx, path)
                child:SetActive(true)

                local temp_cfg = config.active_point_shows[cfg_id]

                local icon = child:GetComponent(UnityEngine.UI.Image)
                icon:SetSprite(temp_cfg.path .. temp_cfg.icon)

                local name = child:Find("Text"):GetComponent(UnityEngine.UI.Text)
                name.text = frame_helper:getLangCfg(temp_cfg.PointName)

                local btn = child:GetComponent(UnityEngine.UI.Button)
                btn.onClick:RemoveAllListeners()
                btn.onClick:AddListener(function()
                    app:startCoroutine(function()
                        coroutine.yield(nil)
                        self:showTip(child.position, cfg_id)
                    end) 

                end)
            end
        end

        local tamp_tab = {
            {
                score = detail_cfg.Section1Point,
                price = detail_cfg.Section1AwardPrice,
                items = loadstring("return " .. detail_cfg.Section1Award)()
            },
            {
                score = detail_cfg.Section2Point,
                price = detail_cfg.Section2AwardPrice,
                items = loadstring("return " .. detail_cfg.Section2Award)()
            },
            {
                score = detail_cfg.Section3Point,
                price = detail_cfg.Section3AwardPrice,
                items = loadstring("return " .. detail_cfg.Section3Award)()
            },
        }

        local path = "content/Right Scroll View/Viewport/Content/"
        frame_helper:hideAllChild(self, path)
        for idx, value in ipairs(tamp_tab) do
            local child = frame_helper:getChild(self, idx, path)
            child:SetActive(true)

            local score = child:Find("info/points"):GetComponent(UnityEngine.UI.Text)
            local price = child:Find("info/price"):GetComponent(UnityEngine.UI.Text)
            score.text = string.format(NativeText.GetLangStr(680008), value.score)
            price.text = string.format(NativeText.GetLangStr(680007), value.price)

            local item_path = path .. child.name .. "/item layout"
            frame_helper:hideAllChild(self, item_path)
            local temp_idx = 1
            for item_id, count in pairs(value.items) do
                local child = frame_helper:getChild(self, temp_idx, item_path)
                child:SetActive(true)
                temp_idx = temp_idx + 1

                local quality = child:GetComponent(UnityEngine.UI.Image)
                local icon = child:Find("icon"):GetComponent(UnityEngine.UI.Image)
                local num = child:Find("num"):GetComponent(UnityEngine.UI.Text)
                num.text = count
                icon:SetSprite(frame_helper:getItemIcon(item_id))
                quality:SetSprite(frame_helper:getItemQualityIcon(item_id))

                local btn = child:Find("icon"):GetComponent(UnityEngine.UI.Button)
                btn.onClick:RemoveAllListeners()
                btn.onClick:AddListener(function()
                    local item_inst = item_cls({cfgId = item_id, count = count})
                    frame_manager:show("itemInfo", nil, {item = item_inst})
                end)
            end
        end
    end,

    refreshUI = function(self)
        local act = activity_manager:getActivityById(self.activity_id)
        if not act then
            frame_manager:hide('activity_detail')
            return
        end

        local detail_cfg = config.active_active_details[act.detail_cfg_id]

        local title = self:findText("content/title")
        local score = self:findText("content/slider/points")
        local fill1 = self:findImage("content/slider/slider1/Building_panel_14_01_Fill")
        local fill2 = self:findImage("content/slider/slider2/Building_panel_14_01_Fill (1)")
        local fill3 = self:findImage("content/slider/slide3/Building_panel_14_01_Fill (2)")


        local step1 = self:findText("content/slider/slider1/value")
        local step2 = self:findText("content/slider/slider2/value")
        local step3 = self:findText("content/slider/slide3/value")

        step1.text = detail_cfg.Section1Point
        step2.text = detail_cfg.Section2Point
        step3.text = detail_cfg.Section3Point

        local des = self:findText("content/slider/nextlevel")
        title.text = frame_helper:getLangCfg(config.active_basics[act.cfg_id].Name)
        score.text = act.score
        
        local temp = {0, detail_cfg.Section1Point, detail_cfg.Section2Point, detail_cfg.Section3Point}
        if act.score >= detail_cfg.Section3Point then
            fill1.fillAmount = 1
            fill2.fillAmount = 1
            fill3.fillAmount = 1
            des.text = string.format(NativeText.GetLangStr(680012))
        else
            for i = 1, #temp - 1 do
                local min_score = temp[i]
                local max_score = temp[i + 1]
                local progress = nil
                local fill_amount = 0
                if i == 1 then
                    progress = fill1
                elseif i == 2 then
                    progress = fill2
                elseif i == 3 then
                    progress = fill3
                end
                if act.score >= min_score then
                    fill_amount = (act.score - min_score) / (max_score - min_score)
                    if fill_amount < 1 then
                        -- des.text = string.format(NativeText.GetLangStr(680011), max_score - act.score)
                        des.text = max_score - act.score
                    end
                end
                progress.fillAmount = fill_amount
            end
        end
    end,

    refreshTime = function(self, id, elpased)
        if self.activity_id == id then
            if not self.time then
                self.time = self:findText("content/slider/time")
            end
            self.time.text = frame_helper:timeFormat(elpased)
        end
    end,

    click_backBtn = function(self)
        frame_manager:hide("activity_detail")
    end,

    click_infoBtn = function(self)
        local act = activity_manager:getActivityById(self.activity_id)
        if act then
            frame_manager:show("activity_detail_info", nil, act.cfg_id)
        end
    end,
}
