local super = require("ui.frame")

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        clickEvents = {
            "content/backBtn",
            "content/infoBtn",
        }

        -- 注册点击事件
        frame_helper:regClickEvents("click", self, clickEvents)

        self.ACTIVITY_INFO_CHANGE = activity_manager.events:on("ACTIVITY_INFO_CHANGE", function()
            self:init()
        end)

        self.ACTIVITY_TIME_TICK = activity_manager.events:on("ACTIVITY_TIME_TICK", function(id, elpased)
            self:refreshTime(id, elpased)
        end)

        self:init()
    end,

    onUnload = function (self)
        activity_manager.events:remove("ACTIVITY_INFO_CHANGE", self.ACTIVITY_INFO_CHANGE)
        activity_manager.events:remove("ACTIVITY_TIME_TICK", self.ACTIVITY_TIME_TICK)
    end,

    click_backBtn = function(self)
        frame_manager:hide('activity_main')
    end,

    click_infoBtn = function(self)
        frame_manager:show('activity_main_info')
    end,

    refreshTime = function(self, id, elpased)
        if self.cache_times and self.cache_times[id] then
            self.cache_times[id].text = frame_helper:timeFormat(elpased)
        end
    end,

    init = function(self)
        self.cache_times = {}
        local activitys = activity_manager:getAllActivitys()
        local path = "content/Scroll View/Viewport/Content"
        frame_helper:hideAllChild(self, path, true)
        local idx = 0
        for id, act in pairs(activitys) do
            idx = idx + 1
            local child = frame_helper:getChild(self, idx, path)
            child:SetActive(true)

            local name = child:Find("info/name"):GetComponent(UnityEngine.UI.Text)
            local des = child:Find("info/des"):GetComponent(UnityEngine.UI.Text)
            local total_price = child:Find("info/total_price"):GetComponent(UnityEngine.UI.Text)
            local time = child:Find("info/time"):GetComponent(UnityEngine.UI.Text)

            self.cache_times[id] = time

            local fill1 = child:Find("info/level/slider1/Buildng_panel_14_02_Bg/fill"):GetComponent(UnityEngine.UI.Image)
            local fill2 = child:Find("info/level/slider2/Buildng_panel_14_02_Bg/fill"):GetComponent(UnityEngine.UI.Image)
            local fill3 = child:Find("info/level/slider3/Buildng_panel_14_02_Bg/fill"):GetComponent(UnityEngine.UI.Image)

            name.text = frame_helper:getLangCfg(config.active_basics[act.cfg_id].Name)
            time.text = frame_helper:timeFormat(act.left_time)

            local detail_cfg = config.active_active_details[act.detail_cfg_id]
            local amount = detail_cfg.Section1AwardPrice + detail_cfg.Section2AwardPrice + detail_cfg.Section3AwardPrice 
            total_price.text = string.format(NativeText.GetLangStr(680007), amount)

            local temp = {0, detail_cfg.Section1Point, detail_cfg.Section2Point, detail_cfg.Section3Point}

            if act.score >= detail_cfg.Section3Point then
                fill1.fillAmount = 1
                fill2.fillAmount = 1
                fill3.fillAmount = 1
                des.text = string.format(NativeText.GetLangStr(680012))
            else
                for i = 1, #temp - 1 do
                    local min_score = temp[i]
                    local max_score = temp[i + 1]
                    local target_fill = nil
                    local fill_amount = 0
                    if i == 1 then
                        target_fill = fill1
                    elseif i == 2 then
                        target_fill = fill2
                    elseif i == 3 then
                        target_fill = fill3
                    end
                    if act.score >= min_score then
                        fill_amount = (act.score - min_score) / (max_score - min_score)
                        if fill_amount < 1 then
                            des.text = string.format(NativeText.GetLangStr(680011), max_score - act.score)
                        end
                    end
                    target_fill.fillAmount = fill_amount
                end
            end

            local ready_trans = child:Find("info/ready")
            local start_trans = child:Find("info/start")

            frame_helper:setActive(ready_trans, act.state == common_pb.EACTIVE_STATE_NONE)
            frame_helper:setActive(start_trans, act.state == common_pb.EACTIVE_STATE_START)

            local btn = child:Find("info/ready/readyBtn"):GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(function()
                local elpased = activity_manager:getActivityElpased(id)
                hint:setText(string.format(NativeText.GetLangStr(680013), frame_helper:timeFormat(elpased)))
            end)
            
            local btn = child:Find("info/start/startBtn"):GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(function()
                frame_manager:show('activity_detail', nil, id)
            end)
        end
    end,
}
