local log = logger.get("Game", "Army Queue Details Frame")
local super = require("ui.frame")

return class{
    super = super,

    onLoaded = function( self )
        super.onLoaded(self)

        self.army_list = army_queue_manager.list

        clickEvents = {
            "Close",
            "DetailPanel/DetailClose",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self.content = self.transform:Find("BG/Scroll View/Viewport/Content")
        self.prefab = self.content:GetChild(0).gameObject

        self.detail_panel = self.transform:Find("DetailPanel")
        self.detail_panel:SetActive(false)

        local team_node = self.detail_panel:Find("BG/Scroll View/Viewport/Content/Team")
        team_node:SetActive(false)
        self.team_ui = {}
        self.team_ui[1] = team_node

        self.lt_fills = {}
        self.ui_list = {}
        self.del_ui_list = {}

        self:init()

        self.army_add_evt = army_queue_manager.events:on("ARMYQUEUEAdd", functor.make(self.onArmyStart, self))
        self.army_remove_evt = army_queue_manager.events:on("ARMYQUEUEREMOVED", functor.make(self.onArmyFinished, self))
    end,

    onUnload = function(self)
        army_queue_manager.events:off("ARMYQUEUEAdd", self.army_add_evt)
        army_queue_manager.events:off("ARMYQUEUEREMOVED", self.army_remove_evt)
    end,

    onArmyStart = function(self, data)
        local elem = self.ui_list[data.id]

        if elem then
            self:setArmyInfo(elem, data)
        else
            elem = self.del_ui_list[1]

            if not elem then
                elem = Object.Instantiate(self.prefab).transform
                elem:SetParent(self.content)
                resetTransformZ(elem)
            else
                table.remove( self.del_ui_list, 1 )
            end

            elem:SetActive(true)
            self:setArmyInfo(elem, data)

            self.ui_list[data.id] = elem
        end
    end,

    onArmyFinished = function(self, id)
        local ui = self.ui_list[id]
        if ui then
            ui:SetActive(false)
            self.ui_list[id] = nil
            table.insert( self.del_ui_list, ui )
        end
    end,

    update = function(self, dt)
        if self.lt_fills then
            for k, v in pairs(self.lt_fills) do
                if v.left_time > 0 then
                    v.arrive_time.text = gui_helper:timeFormat(v.left_time)
                    v.fill.fillAmount = v.left_time / (v.total_time > 0 and v.total_time or 1)

                    v.left_time = v.left_time - dt
                else
                    self.lt_fills[k] = nil
                end
            end
        end
    end,

    init = function(self)
        local cnt = 0

        for k, v in pairs(self.army_list) do
            local elem = nil

            if cnt >= self.content.childCount then
                elem = Object.Instantiate(self.prefab).transform
                elem:SetParent(self.content)
                resetTransformZ(elem)
            else
                elem = self.content:GetChild(cnt)
            end

            elem:SetActive(true)
            elem.name = "Army_"..k

            self:setArmyInfo(elem, v)

            local btn = elem:Find("InfoBtn"):GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(
                function ()
                    self:showDetail(v)
                end
            )

            btn = elem:Find("FillBG/SpeedUp"):GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(
                function ()
                    army_queue_manager:armyLineSpeedUp(v.march_info.instId, v.left_time, v.total_time, v.marchAction)
                end
            )

            self.ui_list[v.id] = elem

            cnt = cnt + 1
        end

        for i=cnt, self.content.childCount -1 do
            local child = self.content:GetChild(i)
            child:SetActive(false)
            table.insert(self.del_ui_list, child)
        end
    end,

    setArmyInfo = function(self, elem, data)
        local name = elem:Find("Name"):GetComponent(UnityEngine.UI.Text)
        local icon = elem:Find("Icon"):GetComponent(UnityEngine.UI.Image)
        local type_bg = elem:Find("NameBG"):GetComponent(UnityEngine.UI.Image)
        local army_type = elem:Find("NameBG/Text"):GetComponent(UnityEngine.UI.Text)
        local right_pos = elem:Find("Target/Pos"):GetComponent(UnderLineText)
        local army_size = elem:Find("ArmySize"):GetComponent(UnityEngine.UI.Text)
        local arrive_text = elem:Find("FillBG/Armying"):GetComponent(UnityEngine.UI.Text)
        local arrive_time = elem:Find("FillBG/FillTime"):GetComponent(UnityEngine.UI.Text)
        local fill_bg = elem:Find("FillBG")
        local fill = elem:Find("FillBG/Fill"):GetComponent(UnityEngine.UI.Image)
        local target_name = elem:Find("Target/Name"):GetComponent(UnityEngine.UI.Text)
        local target_icon = elem:Find("Target"):GetComponent(UnityEngine.UI.Image)
        local speed_up = elem:Find("FillBG/SpeedUp")
        local info_btn = elem:Find("InfoBtn")
        local back_btn = elem:Find("Back")

        cb = function()
            gui_helper:positionJump(data.dstPlace.posX, data.dstPlace.posY)
        end
        right_pos:SetText(gui_helper:positionFormat(data.dstPlace.posX, data.dstPlace.posY), cb)

        if data.dstPlace.objType == common_pb.WORLD_OBJ_PLAYER then
            local cfg = config.worldmap_base_configs[data.dstPlace.cfgId]

            if cfg then
                target_name.text = self.helper:getLangCfg(cfg.name_id)
                target_icon:SetSprite(cfg.iconpath..cfg.iconid)
            else
                log.error("######################################## worldmap_base_configs error ! id ", data.dstInfo.cfgId)
            end
        elseif data.dstPlace.objType == common_pb.WORLD_OBJ_MONSTER then
            local cfg = config.worldmap_monsters[data.dstPlace.cfgId]

            if cfg then
                target_name.text = self.helper:getLangCfg(cfg.monstername)
                target_icon:SetSprite(cfg.iconheadpath..cfg.monsteheadricon)
            else
                log.error("######################################## worldmap_monsters error ! id ", data.dstInfo.cfgId)
            end
        elseif data.dstPlace.objType == common_pb.WORLD_OBJ_FAMILY then
            local cfg = config.union_union_buildings[data.dstPlace.cfgId]

            if cfg then
                target_name.text = self.helper:getLangCfg(cfg.name_id)
                target_icon:SetSprite(cfg.IconPath..cfg.Icon)
            else
                log.error("######################################## union_union_buildings error ! id ", data.dstInfo.cfgId)
            end
        elseif data.dstPlace.objType == common_pb.WORLD_OBJ_WILD_MINE then
            local cfg = config.worldmap_titaniums[data.dstPlace.cfgId]

            if cfg then
                target_name.text = self.helper:getLangCfg(cfg.nameid)
                target_icon:SetSprite(cfg.iconpath..cfg.iconid)
            else
                log.error("######################################## worldmap_titaniums error ! id ", data.dstInfo.cfgId)
            end
        end
        
        icon:SetSprite(data.big_icon_path)
        name.text = data.str
        speed_up:SetActive(data.show_up)
        info_btn:SetActive(data.have_team)
        back_btn:SetActive(data.can_back)
        army_size.transform:SetActive(data.show_up)
        arrive_time.text = gui_helper:timeFormat(data.left_time)

        if data.is_help then
            type_bg:SetSprite(res_path.res_path_113)
            army_type.text = NativeText.GetLangStr(9000048)
        elseif data.is_spy then
            type_bg:SetSprite(res_path.res_path_112)
            army_type.text = NativeText.GetLangStr(9000049)
        elseif data.is_mass then
            type_bg:SetSprite(res_path.res_path_111)
            army_type.text = NativeText.GetLangStr(9000050)
        elseif data.is_gather then
            type_bg:SetSprite(res_path.res_path_113)
            army_type.text = NativeText.GetLangStr(9000051)
        else
            type_bg:SetSprite(res_path.res_path_111)
            army_type.text = NativeText.GetLangStr(9000052)
        end

        self.lt_fills[data.id] = {fill = fill, arrive_time = arrive_time, left_time = data.left_time, total_time = data.total_time}

        local soldier_count =  self:getSoldierCount(data)

        army_size.text = string.format( "%s %d", NativeText.GetLangStr(9000053), soldier_count )

        local btn = elem:Find("GoTo"):GetComponent(UnityEngine.UI.Button)
        btn.onClick:RemoveAllListeners()
        btn.onClick:AddListener(
            function()
                if data.goto_cb then
                    data.goto_cb()
                    self:click_Close()
                end
            end
        )

        btn = elem:Find("Back"):GetComponent(UnityEngine.UI.Button)
        btn.onClick:RemoveAllListeners()
        btn.onClick:AddListener(
            function ()
                if data.back_cb then
                    data.back_cb()
                end
            end
        )
    end,

    showDetail = function(self, data)
        self.detail_panel:SetActive(true)

      --[[   local name = self.detail_panel:Find("BG/Head/Name"):GetComponent(UnityEngine.UI.Text)
        local time_text = self.detail_panel:Find("BG/FillBG/Time"):GetComponent(UnityEngine.UI.Text)
        local fill = self.detail_panel:Find("BG/FillBG/Fill"):GetComponent(UnityEngine.UI.Image)
        local arrive_text = self.detail_panel:Find("BG/FillBG/Text"):GetComponent(UnityEngine.UI.Text)

        name.text = data.baseIfno.playerName

        if string.len(data.baseIfno.familyName) > 2 then
            name.text = string.format( "[%s]%s", data.baseIfno.familyName, data.baseIfno.playerName )
        end

        self.detail_fill = {fill = fill, arrive_time = time_text, left_time = data.leftTime, receive_time = data.receive_time, total_time = data.totalTime} ]]

        local defender_node = self.detail_panel:Find("BG/Scroll View/Viewport/Content/Defender")
        local mass_node = self.detail_panel:Find("BG/Scroll View/Viewport/Content/Mass")
        defender_node:SetActive(false)
        mass_node:SetActive(false)

        if data.marchAction == common_pb.EMARCH_CREATE_MASS or
            data.marchAction == common_pb.EMARCH_JOIN_MASS or
            data.marchAction == common_pb.EMARCH_MASS_ATK then
            self:initMass(mass_node, data)
        else
            self:initDefender(defender_node, data)
        end
    end,

    initDefender = function(self, node, data)
        node:SetActive(true)

        local total_cnt = node:Find("Total/Count"):GetComponent(UnityEngine.UI.Text)

        total_cnt.text = "~"..frame_helper:itemNumFormat(data.teams.soldierInfo.count)

        local list = node:Find("List")
        self:loadSoldiers(list, data.teams.soldierInfo.soldierCounts)

        local hero_node = node:Find("Heros")
        self:loadTeamHeros(hero_node, data.teams.heros)

        app:startCoroutine(
            function()
                coroutine.yield(nil)

                if #data.teams.heros == 0 then
                    hero_node:SetActive(false)
                    node.sizeDelta = Vector2(910, list.rect.height + 140)
                    list.localPosition = Vector3(0, -100, 0)
                else
                    hero_node:SetActive(true)
                    node.sizeDelta = Vector2(910, list.rect.height + 240)
                    list.localPosition = Vector3(0, -200, 0)
                end
            end
        )

        self:loadTeam()
    end,
    
    initMass = function(self, node, data)
        if #data.mass > 0 then
            node:SetActive(true)

            local total_cnt = node:Find("Total/Count"):GetComponent(UnityEngine.UI.Text)

            total_cnt.text = "~"..frame_helper:itemNumFormat(data.mass.massInfo.count)

            local list = node:Find("List")

            self:loadSoldiers(list, data.mass.massInfo.soldierCounts)

            app:startCoroutine(
                function()
                    coroutine.yield(nil)
                    node.sizeDelta = Vector2(910, list.rect.height + 150)
                end
            )

            self:loadTeam(data.mass.teams)
        else
            node:SetActive(false)
        end
    end,

    loadTeam = function(self, teams)
        local content = self.detail_panel:Find("BG/Scroll View/Viewport/Content")
        local prefab = self.team_ui[1].gameObject
        local cnt = 1

        if teams then
            for k, v in ipairs(teams) do
                local elem = nil
                if k > #self.team_ui then
                    elem = Object.Instantiate(prefab).transform
                    elem:SetParent(content)
                    resetTransformZ(elem)

                    local idx = prefab.transform:GetSiblingIndex()
                    elem:SetSiblingIndex(idx + k - 1)

                    table.insert( self.team_ui, elem )
                else
                    elem = self.team_ui[k]
                end

                elem:SetActive(true)

                local name = elem:Find("Top/Name"):GetComponent(UnityEngine.UI.Text)
                name.text = v.playerName

                local hero_node = elem:Find("Heros")
                self:loadTeamHeros(hero_node, v.heros)

                local soldier_node = elem:Find("List")
                self:loadSoldiers(soldier_node, v.soldierInfo.soldierCounts)

                cnt = cnt + 1

                app:startCoroutine(
                    function()
                        coroutine.yield(nil)
                        if #v.heros == 0 then
                            hero_node:SetActive(false)
                            elem.sizeDelta = Vector2(910, soldier_node.rect.height + 78)
                            soldier_node.localPosition = Vector3(0, -75, 0)
                        else
                            hero_node:SetActive(true)
                            elem.sizeDelta = Vector2(910, soldier_node.rect.height + 178)
                            soldier_node.localPosition = Vector3(0, -175, 0)
                        end
                    end
                )
            end
        end

        for i=cnt, #self.team_ui do
            local idx = prefab.transform:GetSiblingIndex()
            idx = idx + i - 1
            local child = content:GetChild(idx)
            child:SetActive(false)
        end
    end,

    loadSoldiers = function(self, list, soldiers)
        local prefab = list:GetChild(0).gameObject

        for k, v in ipairs(soldiers) do
            local elem = nil
            if k > list.childCount then
                elem = Object.Instantiate(prefab).transform
                elem:SetParent(list)
                resetTransformZ(elem)
            else
                elem = list:GetChild(k-1)
            end

            elem:SetActive(true)

            local icon = elem:Find("Icon"):GetComponent(UnityEngine.UI.Image)
            local count = elem:Find("Count"):GetComponent(UnityEngine.UI.Text)
            local name = elem:Find("Name"):GetComponent(UnityEngine.UI.Text)

            local cfg = config.soldier_soldiers[v.cfgId]

            if cfg then
                icon:SetSprite(cfg.iconpath..cfg.soldiericon)
                name.text = frame_helper:getLangCfg(cfg.soldiername)
                count.text = "~"..frame_helper:itemNumFormat(v.count)
            end
        end

        for i=#soldiers, list.childCount - 1 do
            local child = list:GetChild(i)
            child:SetActive(false)
        end
    end,

    loadTeamHeros = function(self, node, heros) 
        local prefab = node:GetChild(0).gameObject

        for k, v in ipairs(heros) do
            local elem = nil
            if k > node.childCount then
                elem = Object.Instantiate(prefab).transform
                elem:SetParent(node)
                resetTransformZ(elem)
            else
                elem = node:GetChild(k-1)
            end

            elem:SetActive(true)

            local icon = elem:Find("Icon"):GetComponent(UnityEngine.UI.Image)
            local grade = elem:Find("Grade"):GetComponent(UnityEngine.UI.Image)
            local level = elem:Find("Image/Level"):GetComponent(UnityEngine.UI.Text)
            local stars = elem:Find("Image/Stars")

            local cfg = config.commander_basics[v.officeCfgId]

            if cfg then
                icon:SetSprite(cfg.path..cfg.icon)
                grade:SetSprite(gui_helper:getNewQualityIcon(v.quality))
                level.text = v.officeLev
                gui_helper:setOfficerStars(v.starLev, stars)
            else
                log.error("##################################### find commander cfg error ! id ", v.officeCfgId)
            end
        end

        for i=#heros, node.childCount - 1 do
            local child = node:GetChild(i)
            child:SetActive(false)
        end
    end,

    getSoldierCount = function(self, data)
        local count = 0

        if data.is_mass then
            count = data.mass.massInfo.count
        else
            count = data.teams.soldierInfo.count
        end

        return count
    end,

    click_Close = function(self)
        frame_manager:hide("army_queue_details")
    end,

    click_DetailClose = function(self)
        self.detail_panel:SetActive(false)
    end,
}