local log = logger.get("Game", "Chest Used Panel")
local gui_helper = require('gui.gui_helper')

-- 使用宝箱 panel
return class {
    ctor = function(self, info_panel, owner)
        self.panel = info_panel
        self.owner = owner
        self.parent = self.panel.parent
        self.helper = require("gui.tool.frame_helper")

        self.grade = info_panel:Find('Top/Grade'):GetComponent(UnityEngine.UI.Image)
        self.icon = info_panel:Find('Top/Icon'):GetComponent(UnityEngine.UI.Image)
        self.name = info_panel:Find('Top/Name'):GetComponent(UnityEngine.UI.Text)
        self.des = info_panel:Find('Top/Des'):GetComponent(UnityEngine.UI.Text)
        self.slot_root = info_panel:Find('Scroll View/Viewport/Content')
        self.total_count = info_panel:Find('Center/CountLayout/Amount'):GetComponent(UnityEngine.UI.Text)
        self.slider = info_panel:Find("Center/CountLayout"):GetComponent(UnityEngine.UI.Slider)

        self.slider.onValueChanged:RemoveAllListeners()
        self.slider.onValueChanged:AddListener(
            function ()
                self:onValueChanged()
            end
        )

        local path_root = "Frame/UsedInfo/ChestUsed/Center/CountLayout/"
        owner:setScript("click", path_root.."Minus", functor.rmake(self.onClick_Minus, self))
        owner:setScript("click", path_root.."Plus", functor.rmake(self.onClick_Plus, self))

        self:hide()
    end,

    onValueChanged = function(self)
        if not self.changed then
            self.count = math.floor(self.slider.value * self.item.count)
        else
            self.changed = false
        end
        self:updateTotalCount()
    end,

    showItem = function(self, item)
        self.helper.showTrans(self.parent)
        self.helper.showTrans(self.panel)      

        self.item = item
        self.count = 1
        
        self.grade:SetSprite(gui_helper:getQualityIconPath(item.cfg.color))
        self.icon:SetSprite(gui_helper:getItemIconPath(item.cfg))
        self.name.text = self.helper:getLangCfg(item.cfg.name_id)
        self.name.color = gui_helper:quality2UnityColor(item.cfg.color)
        self.des.text = self.helper:getLangCfg(item.cfg.des_id)
        --self:updateTotalCount()
        self:setDropItems()
        self:changeCount(0)
    end,

    setDropItems = function(self)
        local items = gui_helper:getDropItems(self.item.cfg.drop_id)

        if not items then
            return
        end
        
        for k, v in ipairs(items) do
            local slot = nil
            if self.slot_root.childCount < k then
                slot = GameObject.Instantiate(self.owner.slot_prefab).transform
                slot:SetParent(self.slot_root)
                makeRectTransformIdentity(slot)
            else
                slot = self.slot_root:GetChild(k-1)
            end

            local item = nil
        
            if slot.childCount == 0 then
                item = GameObject.Instantiate(self.owner.item_prefab).transform
                item:SetParent(slot)
                makeRectTransformIdentity(item) 
            else
                item = slot:GetChild(0)
                item:SetActive(true)
            end

            local icon = item:Find('Icon')
            item:Find('Num'):SetActive(false)
            item:Find('Level'):SetActive(false)

            -- 设置 icon
            local img = icon:GetComponent(UnityEngine.UI.Image)
            img.color = Color.white
            local icon_path = gui_helper:getItemIconPath(v)
            img:SetSprite(icon_path)
        end

        for i = #items, self.slot_root.childCount-1 do
            local slot = self.slot_root:GetChild(i)
            if slot.childCount > 0 then
                local item = slot:GetChild(0)
                item:SetActive(false)
            end
        end
    end,

    updateTotalCount = function(self)
        self.total_count.text = string.format( "%d/%d", self.count, self.item.count )
        gui_helper.used_item_count = self.count
    end,

    changeCount = function(self, val)
        --self.slider.value = val/self.item.count + self.slider.value
        self.count = self.count + val
        self.changed = true
        self.slider.value = self.count/self.item.count
    end,

    onClick_Minus = function(self)
        if self.chest_used_panel.count > 0 then
            self.chest_used_panel:changeCount(-1)
        end
    end,

    onClick_Plus = function(self)
        if self.chest_used_panel.count < self.chest_used_panel.item.count then
            self.chest_used_panel:changeCount(1)
        end
    end,

    hide = function(self)
        self.helper.hideTrans(self.parent)
        self.helper.hideTrans(self.panel)
    end,
}