local gui_helper = require('gui.gui_helper')
local log = logger.get("Game", "Minor Weapon Info Panel")

-- 副武器信息 panel
return class {
    
    ctor = function(self, info_panel)
        self.panel = info_panel
        self.parent = self.panel.parent
        self.helper = require("gui.tool.frame_helper")

        self.grade = info_panel:Find('Top/Grade'):GetComponent(UnityEngine.UI.Image)
        self.icon = info_panel:Find('Top/Icon'):GetComponent(UnityEngine.UI.Image)
        self.name = info_panel:Find('Top/Name'):GetComponent(UnityEngine.UI.Text)
        self.use_level = info_panel:Find('Top/Level'):GetComponent(UnityEngine.UI.Text)
        self.type_text = info_panel:Find("Top/Type"):GetComponent(UnityEngine.UI.Text)

        self.skill_1_icon = info_panel:Find('Bottom/Skill_1'):GetComponent(UnityEngine.UI.Image)
        self.skill_2_icon = info_panel:Find('Bottom/Skill_2'):GetComponent(UnityEngine.UI.Image)
        self.skill_des_1 = info_panel:Find('Bottom/Skill_1/Des'):GetComponent(UnityEngine.UI.Text)
        self.skill_des_2 = info_panel:Find('Bottom/Skill_2/Des'):GetComponent(UnityEngine.UI.Text)
        self.skill_1_icon.transform:SetActive(false)
        self.skill_2_icon.transform:SetActive(false)

        self.skill_trans = {}
        self.skill_trans[1] = {icon = self.skill_1_icon, text = self.skill_des_1}
        self.skill_trans[2] = {icon = self.skill_2_icon, text = self.skill_des_2}

        self.btn_upgrade = info_panel:Find("Bottom/ButtonWeaponUpgrade")
        self.btn_sell = info_panel:Find("Bottom/ButtonSell")

        if self.btn_upgrade and self.btn_sell then
            self.pos = self.btn_upgrade.localPosition
        end

        self:hide()
    end,

    showItem = function(self, item)
        self.helper.showTrans(self.parent)   
        self.helper.showTrans(self.panel)      

        self.icon:SetSprite(gui_helper:getItemIconPath(item.cfg))
        self.grade:SetSprite(gui_helper:getQualityIconPath(item.cfg.color))
        self.name.text = self.helper:getLangCfg(item.cfg.name_id)
        self.name.color = gui_helper:quality2UnityColor(item.cfg.color)
        self.use_level.text = "Lv."..item.level
        self.type_text.text = gui_helper:getWeaponType(item.cfg)

        local prop_panel = self.panel:Find('Center')
        gui_helper:showUpgradeProps(prop_panel, item.props)

        self:loadSkills(item)

        if self.btn_upgrade and self.btn_sell then
            self:setButtonPosition(item)
        end
    end,

    loadSkills = function(self, equipment)
        local skills = equipment.cfg.passive_skill
        local list = loadstring("return "..skills)()

        if type(list) == "table" then
            local idx = 1
            for k, v in pairs(list) do
                local icon = self.skill_trans[idx].icon
                local text = self.skill_trans[idx].text
                icon.transform:SetActive(true)

                local cfg = passive_skill_helper:getSkill(k, v)

                if cfg then
                    text.text = self.helper:getLangCfg(cfg.skill_des)
                    icon:SetSprite(cfg.skill_icon_path..cfg.skill_icon_name)
                else
                    log.error("###### passive skill cfg error ! id "..k.." level "..v)
                end

                idx = idx + 1
            end
        end
    end,

    setButtonPosition = function(self, item)
        if item.cfg.saleprice == 0 then
            self.btn_sell:SetActive(false)
            local pos = self.btn_upgrade.localPosition
            pos.x = 0
            self.btn_upgrade.localPosition = pos
        else
            self.btn_sell:SetActive(true)
            self.btn_upgrade.localPosition = self.pos
        end
    end,

    hide = function(self)
        self.helper.hideTrans(self.parent)
        self.helper.hideTrans(self.panel)
    end,
}