local super = require("ui.frame")
local log = logger.get("Game", "Bag Frame")
local gui_helper = require('gui.gui_helper')
local item_cls = require('RTS.bag_sys.item')
 
local panel_types = {
    weapon = 1,
    minor_weapon = 2,
    chest = 9,
}

return class
{
    super = super,
    
    onLoaded = function (self)
        super.onLoaded(self)

        if self.args.data then
            self.origin_type = self.args.data.bag_type
        end

        -- 路径: root/login
        -- 事件: click_login 
        clickEvents = {
            "Frame/Border/Top/ButtonBack",
            "Frame/Border/Top/ButtonHome",
            -- 排序
            "Frame/Border/Upper/Grop/Quality",
            "Frame/Border/Upper/Grop/Upgrade",
            -- 过滤器
            "Frame/Border/BottomRight/ButtonFilterConsume",
            "Frame/Border/BottomRight/ButtonFilterEquipment",
            "Frame/Border/BottomRight/ButtonFilterMaterial",
            "Frame/Border/BottomRight/ButtonFilterBlueprint",
            "Frame/Border/BottomRight/ButtonFilterPiece",
            -- 详细信息背景
            "Frame/DetailInfo/DetailClose",
            -- 道具详细信息
            "Frame/DetailInfo/WeaponInfo/ButtonSell",
            "Frame/DetailInfo/MaterialInfo/Bottom/ButtonSell",
            "Frame/DetailInfo/ConsumeInfo/Bottom/ButtonSell",
            "Frame/DetailInfo/ConsumeInfo/Bottom/ButtonUsed",
            "Frame/DetailInfo/ChestInfo/Bottom/ButtonSell",
            "Frame/DetailInfo/ChestInfo/Bottom/ButtonUsed",
            -- 出售信息背景
            "Frame/SellInfo/SellClose",
            -- 道具出售信息
            "Frame/SellInfo/ItemSell/Bottom/SellConfirm",
            "Frame/SellInfo/ItemSell/Bottom/ButtonCancel",
            -- 使用道具背景
            "Frame/UsedInfo/UsedClose",
            -- 使用道具信息
            "Frame/UsedInfo/ItemUsed/Bottom/ButtonCancel",
            "Frame/UsedInfo/ItemUsed/Bottom/UsedConfirm",
            "Frame/UsedInfo/ChestUsed/Bottom/ButtonCancel",
            "Frame/UsedInfo/ChestUsed/Bottom/UsedConfirm",
            -- 锁定
            "Frame/Border/Center/Image/ButtonFilterLocked",
            "Frame/Border/Center/Image/ButtonLockedQuit",
            -- 分解
            -- "Frame/Border/BottomRight/ResolveButtons/ButtonFilterResolve",
            -- "Frame/Border/ButtonResolve/ButtonResolveQuit",
            -- "Frame/Border/ButtonResolve/ButtonResolveFast",
            -- "Frame/Border/ButtonResolve/ButtonResolveConfirm",
            -- 快速分解
            -- "Frame/FastSelected/Panel/ButtonLeft",
            -- "Frame/FastSelected/Panel/ButtonRight",
            "Frame/Border/Top/Gem",
            "Frame/Border/Top/Coin",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self.slot_root_path = 'Frame/Border/Center/Scroll View/Viewport/SlotRoot'
        self.slot_root = self.transform:Find(self.slot_root_path)

        -- item prefab
        self.item_prefab = self.transform:Find('Prefabs/Slot/Item').gameObject
        self.slot_prefab = self.transform:Find('Prefabs/Slot').gameObject

        self.scroll_view = self.transform:Find('Frame/Border/Center/Scroll View'):GetComponent(UnityEngine.UI.ScrollRect)

        self.helper.hideTrans(self.transform:Find('Prefabs'))

        self.weapon_bag = 1
        self.material_bag = 2
        self.consume_bag = 3
        self.blueprint_bag = 4
        self.piece_bag = 5

        self.detail_root = self.transform:Find('Frame/DetailInfo')
        local weapon_info = self.transform:Find('Frame/DetailInfo/WeaponInfo')
        self.weapon_info_panel = require('gui.bag.weapon_info_panel')(weapon_info)
        local material_info = self.transform:Find('Frame/DetailInfo/MaterialInfo')
        self.material_info_panel = require('gui.bag.material_info_panel')(material_info)
        local consume_info = self.transform:Find('Frame/DetailInfo/ConsumeInfo')
        self.consume_info_panel = require('gui.bag.consume_info_panel')(consume_info)
        local chest_info = self.transform:Find('Frame/DetailInfo/ChestInfo')
        self.chest_info_panel = require('gui.bag.chest_info_panel')(chest_info, self)

        self.used_info_root = self.transform:Find('Frame/UsedInfo')
        local item_used = self.used_info_root:Find('ItemUsed')
        self.item_used_panel = require('gui.bag.item_used_panel')(item_used, self)
        local chest_used = self.used_info_root:Find('ChestUsed')
        self.chest_used_panel = require('gui.bag.chest_used_panel')(chest_used, self)

        self.sell_info_root = self.transform:Find('Frame/SellInfo')
        local item_sell = self.sell_info_root:Find('ItemSell')
        self.item_sell_panel = require('gui.bag.item_sell_panel')(item_sell, self)

        --local fast_resolve = self.transform:Find('Frame/FastSelected')
        --self.fast_resolve_panel = require('gui.bag.fast_resolve_panel')(fast_resolve)

        self.current_panel = nil

        self.bag_volume = self:findText('Frame/Border/Center/Image/BagVolume/Amount')
        self.gem_text = self:findText('Frame/Border/Top/Gem/Amount')
        self.coin_text = self:findText('Frame/Border/Top/Coin/Amount')

        self.bottom_bar = self.transform:Find("Frame/Border/Center/Image")
        self.lock_btn = self.bottom_bar:Find("ButtonFilterLocked")
        self.lock_quit_btn = self.bottom_bar:Find("ButtonLockedQuit")
        
        -- 排序按钮
        self.sort_root = self.transform:Find("Frame/Border/Upper/Grop")

        self.btns = {}
        self.btns['quality'] = self.transform:Find('Frame/Border/Upper/Grop/Quality')
        self.btns['enhance'] = self.transform:Find('Frame/Border/Upper/Grop/Upgrade')

        -- 缓存所有已注册的 icon 事件，刷新时要清空
        self.icon_events = {}

        -- 锁定的 item
        self.locked_items = {}
        -- 分解选中的 item
        self.selected_items = {}

        self.status = "normal"     -- normal 正常模式  resolve 分解  lock 上锁解锁

        -- 当前排序，背包类型
        self.sort =  nil
        self.sort_invert = false
        self.bag_type = 0
        self.isResolve = false

        -- 背包页签
        self.button_root = self.transform:Find("Frame/Border/BottomRight")

        self.bag_btns = {}
        self.bag_btns[self.consume_bag] = self.button_root:Find("ButtonFilterConsume"):GetComponent(UnityEngine.UI.Button)
        self.bag_btns[self.weapon_bag] = self.button_root:Find("ButtonFilterEquipment"):GetComponent(UnityEngine.UI.Button)
        self.bag_btns[self.material_bag] = self.button_root:Find("ButtonFilterMaterial"):GetComponent(UnityEngine.UI.Button)
        self.bag_btns[self.blueprint_bag] = self.button_root:Find("ButtonFilterBlueprint"):GetComponent(UnityEngine.UI.Button)
        self.bag_btns[self.piece_bag] = self.button_root:Find("ButtonFilterPiece"):GetComponent(UnityEngine.UI.Button)

        self.is_dirty = false

        self.slots = {}
        self.items = {}

        self.event_ids = {}
        -- 注册属性变化事件处理
        --self.event_ids["onPropChange"] = local_player.events:on("onPropChange", functor.make(self.refreshResourceNum, self))
        --self:refreshResourceNum()

        -- 注册道具事件
        self.event_ids["onBagChanged"] = bag_manager.events:on("onBagChanged", functor.make(self.onBagChanged, self))
        self.event_ids["onUseItem"] = bag_manager.events:on("onUseItem", functor.make(self.onUseItem, self))
        self.event_ids["onWeaponResolve"] = bag_manager.events:on("onWeaponResolve", functor.make(self.onWeaponResolve, self))
        self.event_ids["NOTIFYGETITEM"] = game_director.events:on("NOTIFYGETITEM", functor.make(self.onGetItems, self))

        self.sort = 'quality'
        self:refresh(self.sort, self.origin_type or self.consume_bag, false)
        self:setButtonRootActive("normal")
    end,

    onUnload = function (self)
        local_player.events:remove("onPropChange", self.event_ids["onPropChange"] )
        --bag_manager.events:remove("onBagChanged", self.event_ids["onBagChanged"])
        bag_manager.events:remove("onUseItem", self.event_ids["onUseItem"])
        bag_manager.events:remove("onWeaponResolve", self.event_ids["onWeaponResolve"])
        game_director.events:remove("NOTIFYGETITEM", self.event_ids["NOTIFYGETITEM"])
    end,

    onRcvMsg = function(self, cmd, msg)
    end,

    clearIconScripts = function(self)
        for path, evt in pairs(self.icon_events) do
            self:clearScript(evt, path)
        end
        self.icon_events = {}

    end,

    -- 背包发生变化
    onBagChanged = function(self)
        log.info('#### onBagChanged!!')
        self.is_dirty = true
    end,

    -- 使用道具(目前是开宝箱)
    onUseItem = function(self, msg)
        if msg.respCode == 0 then
            
        else
            log.info('used item failed:'..msg.respCode)
        end
    end,

    onGetItems = function(self, msg)
        if msg.reason == common_pb.USE_ITEM or 
            msg.reason == common_pb.REQOLVE_WEAPON then

            --gui_helper:showGetItemsPanel(msg.getItems, true)

            frame_manager:show('getItems', self, {items = gui_helper.drop_list[msg.reason], has_Confirm_Btn = true})
        end
    end,

    onWeaponResolve = function(self)
        self.selected_items = {}
    end,

    update = function(self)
        if self.is_dirty then
            self:refresh(self.sort, self.bag_type, self.sort_invert, self.isResolve, self.isLock)
            self:refreshBagAmount()
            self.is_dirty = false
        end
    end,

    refreshBagAmount = function(self)
        local bag_num = bag_manager:getEquipNum()
        local max = local_player:getBagMaxAmount()
        local str = gui_helper:limitColorFormat(bag_num, max)
        self.bag_volume.text = string.format("%s  %s", NativeText.GetLangStr(320004), str)
    end,

    -- 刷新金币，钻石
    refreshResourceNum = function(self)
        if self.gem_text and self.coin_text then
            self.gem_text.text = tostring(local_player:getGem())
            self.coin_text.text = tostring(local_player:getCoin())
        end
    end,

    -- 当前页签高亮
    updateBagTypeBtn = function(self, disable)
        for k, v in pairs(self.bag_btns) do
            if k == self.bag_type and not disable then
                v.targetGraphic.sprite = v.spriteState.pressedSprite
            else
                v.targetGraphic.sprite = v.spriteState.disabledSprite
            end

            v.transform:GetComponent(UnityEngine.UI.Image):SetNativeSize()

            if disable == true then
                v.enabled = false
            else
                v.enabled = true
            end
        end
    end,

    -- 根据当前的刷新背包
    refresh = function(self, sort, bag_type, sort_invert, isResolve, isLock)
        if (not self.is_dirty) and 
            self.sort == sort and self.bag_type == bag_type and self.isLock == isLock and
             self.sort_invert == sort_invert and self.isResolve == isResolve then

            return
        end

        if bag_type == self.weapon_bag then
            self.sort = sort
        end
        self.bag_type = bag_type
        self.sort_invert = sort_invert
        self.isResolve = isResolve
        self.isLock = isLock

        local bag = bag_manager:getBag(bag_type)
        if bag then

            -- 清空原来的 icon 事件处理
            self:clearIconScripts()

            local items = nil
            if sort == 'quality' then
                items = bag:getItemsSortByQuality()
            elseif sort == 'enhance' then
                items = bag:getItemsSortByEnhanceLevel()
            else 
                items = bag:getItems()
            end

            -- 分解页面  筛选未上锁的道具
            if self.isResolve then
                items = item_helper:getUnlockedItems(items)
            end

            if self.isLock then
                self:addHeroBagItems(items)
            end

            self.slots = {}

            if items then
                --log.info('item num:'..#items)
                if #items > self.slot_root.childCount then
                    for i=self.slot_root.childCount+1, #items do
                        local slot = Object.Instantiate(self.slot_prefab).transform
                        slot.name = "slot ("..i..")"
                        slot:SetParent(self.slot_root)
                        makeRectTransformIdentity(slot) 
                    end
                end
                for idx, item in ipairs(items) do
                    --log.info('bag item ',item.cfg_id)
                    gui_helper.setItemToSlot(self, idx, item)
                end
            
                gui_helper:clearSlotsAfter(self.slot_root, #items)

                -- 滚动置顶
                local pos = self.scroll_view.content.localPosition
                pos.y = 0
                self.scroll_view.content.localPosition = pos

            else
                log.error('No items!!')
            end
            self.items = items or {}

        else
            log.error('Failed to find bag:'..bag_type)
        end

        self.bottom_bar:SetActive(self.bag_type == self.weapon_bag)
        --self.sort_root:SetActive(self.bag_type == self.weapon_bag)
        
        self:updateBagTypeBtn()

        self.is_dirty = false
    end,

    addHeroBagItems = function(self, items)
        local tab = bag_manager:getAllHeroBagItem()
        for k, v in ipairs(tab) do
            table.insert( items, v )
        end

        if self.sort == 'level' then
            item_helper:sortItemByUseLevel(items)
        elseif self.sort == 'quality' then
            item_helper:sortItemByQuality(items)
        elseif self.sort == 'enhance' then
            item_helper:sortItemByEnhanceLevel(items)
        end
    end,

    setButtonRootActive = function(self, st)
        self.status = st
        self:updateBagTypeBtn(st ~= "normal")
        --self.lock_btn:SetActive(st ~= "lock")
        --self.lock_quit_btn:SetActive(st == "lock")
    end,

    updateSortBtns = function(self, filter)
        for k, v in pairs(self.btns) do
            local label = v:GetChild(0):GetComponent(UnityEngine.UI.Text)
            local outline = v:GetChild(0):GetComponent(UnityEngine.UI.Outline)

            local color = Color(1, 1, 1, 1)
            if k ~= filter then
                color = Color(160/255, 160/255, 160/255, 1)
            end

            label.color = color
            outline.enabled = k == filter
        end

        self:refresh(filter, self.bag_type, false, self.isResolve, self.isLock)
    end,

    click_ButtonBack = function(self)
        log.info('click_ButtonBack')
        frame_manager:hide('bag')
    end,

    click_ButtonHome = function(self)
        log.info('click_ButtonHome')
        frame_manager:clickHome()
    end,

    click_Gem = function(self)
        frame_manager:show('resource_convert')
    end,

    click_Coin = function(self)
        frame_manager:show('resource_convert')
    end,

    click_Quality = function(self)
        self:updateSortBtns('quality')
    end,

    click_Upgrade = function(self)
        self:updateSortBtns('enhance')
    end,

    click_ButtonFilterConsume = function(self)
        self:setButtonRootActive("normal")
        self:refresh('quality', self.consume_bag, false, false)
    end,

    click_ButtonFilterEquipment = function(self)
        self:setButtonRootActive("normal")
        self:refresh(self.sort, self.weapon_bag, false, false)
        self:refreshBagAmount()
        self:updateSortBtns("quality")
    end,

    click_ButtonFilterMaterial = function(self)
        self:setButtonRootActive("normal")
        self:refresh('quality', self.material_bag, false, false)
    end,

    click_ButtonFilterBlueprint = function(self)
        self:setButtonRootActive("normal")
        self:refresh('quality', self.blueprint_bag, false, false)
    end,

    click_ButtonFilterPiece = function(self)
        self:setButtonRootActive("normal")
        self:refresh('quality', self.piece_bag, false, false)
    end,

    click_ButtonFilterLocked = function(self)
        self:setButtonRootActive("lock")
        self:refresh(self.sort, self.weapon_bag, false, false, true)
        self:startLockingItem()
    end,

    --[[ click_ButtonFilterResolve = function(self)
        self:setButtonRootActive("resolve")
        self:refresh(self.sort, self.weapon_bag, false, true)
    end, ]]

    click_ButtonLockedQuit = function(self)
        self:setButtonRootActive("normal")
        self:refresh(self.sort, self.weapon_bag, false, false)
        network_sender:lockedEquipments(self.locked_items)
        self.locked_items = {}
    end,

    --[[ click_ButtonResolveQuit = function(self)
        self:setButtonRootActive("normal")
        self:refresh(self.sort, self.weapon_bag, false, false)
        self.selected_items = {}
    end,

    click_ButtonResolveFast = function(self)
        self.fast_resolve_panel:show()
    end,

    click_ButtonResolveConfirm = function(self)
        self:checkHeighGradeItem()
    end, ]]

    click_DetailClose = function(self)
        self.current_panel:hide()
    end,

    click_SellClose = function(self)
        self.current_panel:hide()
    end,

    click_UsedClose = function(self)
        self.current_panel:hide()
    end,

    click_ButtonSell = function(self)
        log.info('click_ButtonSell')
        self.current_panel:hide()
        self:showSellPanel()
    end,

    click_ButtonUsed = function(self)
        log.info('click_ButtonUsed')
        if self:checkHeroMax() then
            return
        end
        self.current_panel:hide()

        local item = gui_helper.current_item
        
        if item.cfg_id == 65017 then
            network_sender:baseRandTransition()
            return
        end
        
        if item.count == 1 then
            network_sender:useItem(gui_helper.current_item.cfg_id, 1)
        else
            self:showUsedPanel()
        end
    end,

    click_SellConfirm = function(self)
        log.info('click_SellConfirm')
        self.current_panel:hide()
        network_sender:sellItem(gui_helper.current_item.inst_id, gui_helper.used_item_count)
    end,

    click_UsedConfirm = function(self)
        log.info('click_UsedConfirm')
        self.current_panel:hide()
        network_sender:useItem(gui_helper.current_item.cfg_id, gui_helper.used_item_count)
    end,

    click_ButtonCancel = function(self)
        log.info('click_ButtonCancel')
        self.current_panel:hide()
    end,

    --[[ click_ButtonLeft = function(self)
        self.fast_resolve_panel:hide()
    end,

    click_ButtonRight = function(self)
        local items = self.fast_resolve_panel:getSelectedItems()
        self.selected_items = {}
        for _, item in ipairs(items) do
            self.selected_items[item.inst_id] = item
        end
        self:updateSelectedSlot()
        self.fast_resolve_panel:hide()
    end, ]]

    updateSelectedSlot = function(self)
        for idx, slot in pairs(self.slots) do
            --log.info("slot idx: ",idx)
            local selected_img = slot:Find("Selected")
            selected_img:SetActive(self.selected_items[idx] ~= nil)
        end
    end,

    -- 点击了一个道具
    onClick_ItemIcon = function(self, handler, data, item)
        log.info('### click item:'..item.inst_id)

        if self.status == "lock" then
            self:lockedItem(item)
        elseif self.status == "resolve" then
            self:selectedItem(item)
        else
            -- 缓存当前选中的道具
            gui_helper.current_item = item

            -- todo: 根据 item 的类型，显示不同详细信息界面
            self:showItemInfo(item)
        end
    end,

    -- 进入锁定操作
    startLockingItem = function(self)
        for k, v in pairs(self.items) do
            local lock_text = self.slots[v.inst_id]:Find('LockText')
            if lock_text then
                lock_text:SetActive(false)
            end
            local lock_img = self.slots[v.inst_id]:Find('Lock')
            if lock_img then
                lock_img:SetActive(v:isLocked())
            end
            local equiped_img = self.slots[v.inst_id]:Find('Equiped')
            if equiped_img then
                equiped_img:SetActive(v.hero_id ~= nil)
            end
        end
    end,

    -- 道具锁定 解锁
    lockedItem = function(self, item)
        local slot  = self.slots[item.inst_id]
        local lock_img = slot:Find("Lock")
        lock_img:SetActive(not lock_img.gameObject.activeSelf)

        -- 道具已锁定
        if item:isLocked() then
            if not lock_img.gameObject.activeSelf then
                self.locked_items[item.inst_id] = item  -- 解锁
            else
                self.locked_items[item.inst_id] = nil
            end
        end

        -- 道具未锁定
        if not item:isLocked() then
            if lock_img.gameObject.activeSelf then
                self.locked_items[item.inst_id] = item  -- 锁定
            else
                self.locked_items[item.inst_id] = nil
            end
        end
    end,

    -- 分解选中道具
    selectedItem = function(self, item)
        local slot  = self.slots[item.inst_id]
        local selected_img = slot:Find("Selected")
        selected_img:SetActive(not selected_img.gameObject.activeSelf)

        if selected_img.gameObject.activeSelf then
            self.selected_items[item.inst_id] = item
        else
            self.selected_items[item.inst_id] = nil
        end
    end,

    -- 检查紫色品质以上道具
    checkHeighGradeItem = function(self)
        for _, item in pairs(self.selected_items) do
            if item.cfg.color > 3 then

                local cb = function()
                    frame_manager:hide('common_box')
                    self:showResolveItems()
                end
                self:showNoticeBox(NativeText.GetLangStr(110045), cb)
                
                return
            end
        end
        self:showResolveItems()
    end,

    showNoticeBox = function(self, msg, cb)
        local tb = {
            title = "",
            content = msg,
            buttons =
            {
                {
                    btn_name = NativeText.GetLangStr(110522),
                    callback = function() 
                        frame_manager:hide('common_box')
                    end
                },
                {
                    btn_name = NativeText.GetLangStr(110521),
                    callback = cb
                },
            },
        }

        frame_manager:show('common_box', nil, tb)
    end,

    -- 显示分解获得的道具
    showResolveItems = function(self)
        local main_total_exp = 0
        local minor_total_exp = 0
        local exp_factor = config.weapon_upgradeconfigs[1].exppercent / 10000

        -- 计算选择的武器的总经验
        for _, item in pairs(self.selected_items) do
            local item_exp = item.cfg.basicexp + item.total_exp * exp_factor
            if item.cfg.family == 1 then
                main_total_exp = main_total_exp + item_exp
            elseif item.cfg.family == 2 then
                minor_total_exp = minor_total_exp + item_exp
            end
        end

        local main_exp_material_cfg = item_helper:getWeaponExpMaterialCfg(24)
        local minor_exp_material_cfg = item_helper:getWeaponExpMaterialCfg(25)

        -- 可以获得的主武器材料
        local get_main_mat = self:calculateItems(main_exp_material_cfg, main_total_exp)
        -- 可以获得的副武器材料
        local get_minor_mat = self:calculateItems(minor_exp_material_cfg, minor_total_exp)

        local item_list = {}
        local insertItem = function(tab)
            local num = 0
            for _, v in ipairs(tab) do
                local item = item_cls(v)
                table.insert(item_list, item)
                num = num + item.count
            end
            return num
        end
        local main_mat_num = insertItem(get_main_mat)
        local minor_mat_num = insertItem(get_minor_mat)

        local cb = function()
            frame_manager:hide('common_box')
            network_sender:resolveEquipments(self.selected_items)
        end
        
        local selected_item_num = 0
        for _, _ in pairs(self.selected_items) do
            selected_item_num = selected_item_num + 1
        end

        local msg = string.format(NativeText.GetLangStr(110046), selected_item_num, main_mat_num, minor_mat_num)
        self:showNoticeBox(msg, cb)
    end,

    -- 计算分解获得的道具
    calculateItems = function(self, cfg, total_exp)
        local mats = {}
        for _, mat in ipairs(cfg) do
            local num = math.floor(total_exp / mat.para1)
            local total_exp = total_exp % mat.para1
            if num > 0 then
                table.insert( mats, {cfgId = mat.id, count = num} )
            end
        end
        return mats
    end,

    -- 显示道具信息
    showItemInfo = function(self, item)
        if self.bag_type == self.weapon_bag then
            self.weapon_info_panel:showItem(item)
            self.current_panel = self.weapon_info_panel
        elseif self.bag_type == self.consume_bag then
            if item.cfg.type ==  panel_types.chest then
                self.chest_info_panel:showItem(item)
                self.current_panel = self.chest_info_panel
            else
                self.consume_info_panel:showItem(item)
                self.current_panel = self.consume_info_panel
            end
        elseif self.bag_type == self.material_bag then
            self.material_info_panel:showItem(item)
            self.current_panel = self.material_info_panel
        elseif self.bag_type == self.piece_bag then
            self.material_info_panel:showItem(item)
            self.current_panel = self.material_info_panel
        elseif self.bag_type == self.blueprint_bag then
            --self.material_info_panel:showItem(item)
        else
            log.error('Failed to find bag: ', self.bag_type)
        end
    end,

    showSellPanel = function(self)
        local item = gui_helper.current_item
        self.item_sell_panel:showItem(item)
        self.current_panel = self.item_sell_panel
    end,

    showUsedPanel = function(self)
        local item = gui_helper.current_item
        if item.cfg.type == panel_types.chest then
            self.chest_used_panel:showItem(item)
            self.current_panel = self.chest_used_panel
        else
            self.item_used_panel:showItem(item)
            self.current_panel = self.item_used_panel
        end
    end,

    checkHeroMax = function(self)
        if gui_helper.current_item.cfg_id == 90003 and hero_manager:heroAmountMax() then
            hint:setText(NativeText.GetLangStr(900006))
            return true
        end

        return false
    end,
}