local super = require("ui.frame")
local stage_events = enums.stage_events
local log = logger.get("Game", "Frame")
local battle_types = require('RTS.enums').battle_types

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        self.war_mgr = self.args.owner  -- 在 rts 下实际是 rts_stage+war_mgr

        clickEvents = {
            "bottomLeft/speedGame",
            "bottomLeft/autoUseSkill",
            "bottomLeft/pauseBtn",
            "pause/layout/continueBtn",
            "pause/layout/restartBtn",
            "pause/layout/exitBtn",
        }
        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)
        
        -- 血量变化
        self.actor_hp_change_event_id = self.war_mgr.events:on('actor_hp_change', function(actor_id, cur_hp, max_hp)
            -- print("actor_hp_change #:", actor_id, cur_hp, max_hp)
            self:onHpChange(actor_id, cur_hp, max_hp)
        end)

        -- 能量变化
        self.actor_energy_change_event_id = self.war_mgr.events:on('actor_energy_change', function(actor_id, cur_energy, max_energy)
            self:onEnergyChange(actor_id, cur_energy, max_energy)
        end)

        self.war_mgr.events:on("actor_spawned", function(actor)
        end)

        self.pause_trans = self.transform:Find("pause")
        self.helper.hideTrans(self.pause_trans)

        self:init()
        battle_frame = self

        self:loadQTE()

        Fenix.Assets.Helper.ms_SyncLoad = false
    end, 

    update = function(self)
        self:updateDamage()
    end,

    onUnload = function (self)
        self.war_mgr.events:off("actor_hp_change", self.actor_hp_change_event_id)
        self.war_mgr.events:off("actor_energy_change", self.actor_energy_change_event_id)
        self.cache_vehicles = nil
        battle_frame = nil
        Object.Destroy(self.qte_trans)
    end, 

    init = function(self)
        -- 战斗速度
        self.speed_list = { 1, 2, 3 }
        self.speed_idx = 1
        self.speed_images = {
            self:findImage("bottomLeft/speedGame/1"), 
            self:findImage("bottomLeft/speedGame/2"), 
            self:findImage("bottomLeft/speedGame/3")
        }
        self:setSpeedImage(self.speed_idx)
        
        local auto_image = self:findImage("bottomLeft/autoUseSkill/1")
        if self.war_mgr.eType ~= battle_types.REPLAY then
            auto_image.enabled = self.war_mgr.auto_skill_self
        else
            auto_image.enabled = true
        end
    end,

    getMonsterName = function(self)
        if level_info and level_info.map_attack_info and level_info.map_attack_info.monsterId then
            local id = level_info.map_attack_info.monsterId
            if config.monster_group_ids[id] then
                local lang_id = config.monster_group_ids[id].Group_Name_ID
                if lang_id then
                    return self.helper:getLangCfg(tostring(lang_id))
                end
            end
        end
        return ""
    end,

    setSpeedImage = function(self, idx)
        for __, image in ipairs(self.speed_images) do
            image.enabled = (__ == idx)
            if __ == idx then
                self.war_mgr:setTimeScale(self.speed_list[__])
            end
        end
    end,

    createVehicle = function(self, actor)
        local actor_types = require('RTS.enums').actor_types

        if actor.team ~= 0 or actor.actor_type ~= actor_types.VEHICLE then return end
        self.vehicle_idx = self.vehicle_idx or 0
        self.vehicle_idx = self.vehicle_idx + 1
        local parent_path = "bottomRight/content/grid"
        if self.vehicle_idx == 1 then
            self.helper:hideAllChild(self, parent_path)
        end

        local child = self.helper:getChild(self, self.vehicle_idx, parent_path)
        child:SetActive(true)
        local icon = child.transform:Find("offset/icon"):GetComponent(UnityEngine.UI.Image)
        local tank_type = child.transform:Find("offset/icon/type"):GetComponent(UnityEngine.UI.Image)
        icon:SetSprite(actor.cfg.battle_icon)
        tank_type:SetSprite("Assets/Arts/ui/atlas/icon/vehicle/"..actor.cfg.type..".png")
        local btn = child.transform:Find("offset"):GetComponent(UnityEngine.UI.Button)
        btn.onClick:RemoveAllListeners()
        btn.onClick:AddListener(
            function()
                self:castSkill(actor)
            end)
        self:createCacheData(actor, child)
        --[[
        local go = self.transform:Find("bottomRight/content").gameObject
        local lt = LeanTween.moveLocalX(go, 163.5 * (8 - self.icon_idx + 1), 0)
        lt.loopType = LeanTweenType.once
        ]]
    end,

    -- 创建缓存数据
    createCacheData = function(self, actor, trans)
        self.cache_vehicles = self.cache_vehicles or {}
        local icon_image = trans:Find("offset/icon"):GetComponent(UnityEngine.UI.Image)
        local icon_type = trans:Find("offset/icon/type"):GetComponent(UnityEngine.UI.Image)
        local offset_go = trans:Find("offset").gameObject
        local energy_image = trans:Find("offset/icon/skillCD"):GetComponent(UnityEngine.UI.Image)
        self.cache_vehicles[actor.id] = 
        {
            actor = actor, 
            trans = trans, 
            energy_image = energy_image,
            icon_image = icon_image, 
            icon_type = icon_type, 
            offset_go = offset_go, 
            alive = true,
            energy_full = nil
        }
    end,

    -- 使用技能
    castSkill = function (self, actor)
        self.war_mgr:castMaiWeaponSkill(actor)
        if self:canCastSkill(actor.id) then
            local cfg = config.weapon_weapons[actor.weapon_id]
            self:checkSkillIcon(cfg.weapon_type, actor.id)
        end
    end,

    -- 是否能使用技能
    canCastSkill = function(self, actor_id)
        local vehicle = self.cache_vehicles and self.cache_vehicles[actor_id] or nil
        if vehicle and vehicle.alive and vehicle.energy_full then
            return true
        end
    end,

    -- 血量变化
    onHpChange = function(self, actor_id, cur_hp, max_hp)
        if cur_hp <= 0 and actor_id == self.top_bar_actor_id then
            -- 顶部技能打断条的 actor 死了
            self:hideTopBar()
        end

        if not self.cache_vehicles then
            return
        end
        local data = self.cache_vehicles[actor_id]
        if data then
            if cur_hp <= 0 then
                if data.alive then
                    data.alive = false
                    self:onEnergyChange(actor_id, 0, 100)
                    self.helper:setImageGrayPlus(data.icon_image, true)
                    self.helper:setImageGrayPlus(data.icon_type, true)
                end
            end
        end
    end,

    -- 能量变化
    onEnergyChange = function(self, actor_id, cur_energy, max_energy)
        --log.info("onEnergyChange id:"..actor_id.." enerty:"..cur_energy..", max:"..max_energy)
        if not self.cache_vehicles then
            return
        end
        local data = self.cache_vehicles[actor_id]
        if data then
            if data.alive then
                data.energy_image.fillAmount = 1 - cur_energy / max_energy
                local temp = cur_energy >= max_energy
                if data.energy_full ~= temp then
                    data.energy_full = temp
                    self:setEnergyState(temp, data)
                end
            else
                data.energy_image.fillAmount = 0
                self:setEnergyState(false, data)
            end
        end
    end,

    -- 设置满能量表现
    setEnergyState = function(self, state, data)
        local go = data.offset_go
        local pos = ( data.energy_full and data.alive ) and Vector3(88,120,0) or Vector3(105,75,0)
        local lt = LeanTween.moveLocal(go, pos, 0.2)
        lt.loopType = LeanTweenType.once
        
        local energy_image = data.trans:Find("canUse/flowLight"):GetComponent(UnityEngine.UI.Image)
        energy_image.enabled = data.energy_full and data.alive
    end,

    -- 显示伤害
    showDamage = function(self, icon_name, value)
        if not self.damage_list then
            self.damage_list = {}
        end
        if icon_name then
            local damage_trans = self.helper:copyTrans(self, "bottomRight/offset/layout/tplt")
            local icon = damage_trans:Find("bg/icon"):GetComponent(UnityEngine.UI.Image)
            local text = damage_trans:Find("bg/num"):GetComponent(UnityEngine.UI.Text)
            text.text = 0
            damage_trans.gameObject:SetActive(true)
            table.insert(self.damage_list, { damage_trans = damage_trans, value = value, text = text})
        end
    end,

    updateDamage = function(self)
        self.damage_interval_time = self.damage_interval_time or 0
        self.damage_interval_time = self.damage_interval_time - Time.unscaledDeltaTime
        if self.damage_interval_time <= 0 and self.damage_list and #self.damage_list > 0 then
            local hori_time = 0.3
            local delay_time = 2
            local vert_time = 0.5

            local damage_trans = self.damage_list[1].damage_trans
            local value = self.damage_list[1].value
            local text = self.damage_list[1].text

            local rect = damage_trans:GetComponent(UnityEngine.RectTransform)
            rect.transform.localPosition = Vector3.zero
            local lt1 = LeanTween.moveX(rect, -424.74, hori_time)
            lt1.loopType = LeanTweenType.once
            lt1.delay = 0
            lt1:setOnComplete(function()
                self.helper:makeNumEffect(text.transform, value)
            end)

            local lt2 = LeanTween.moveY(rect, 300, vert_time)
            lt2.loopType = LeanTweenType.once
            lt2.delay = delay_time
            table.remove(self.damage_list, 1)
            lt2:setOnComplete(function()
                UnityEngine.Object.Destroy(damage_trans.gameObject)
            end)
            self.damage_interval_time = hori_time + delay_time - 0.4
        end
    end,


    click_skipWar = function(self)
    end,

    click_genRecord = function(self)
        if self.war_mgr.states.name ~= 'loading' then
            server_start()
        end
    end,

    -- 点击暂停事件
    click_pauseGame = function(self)
        self.helper.showTrans(self.pause_trans)
        self.war_mgr:setGamePause(true)
    end,

    click_speedGame = function(self)
        self.speed_idx = 1 + self.speed_idx
        if self.speed_idx > #self.speed_list then
            self.speed_idx = 1
        end
        self:setSpeedImage(self.speed_idx)
    end,

    click_autoUseSkill = function(self)
        if self.war_mgr.eType ~= battle_types.REPLAY then
            local is_auto = self.war_mgr:switchAutoSkill()
            local image = self:findImage("bottomLeft/autoUseSkill/1")
            image.enabled = is_auto
        end
    end,

    click_pauseBtn = function(self)
        self.war_mgr:setGamePause(true)
        self.helper.showTrans(self.pause_trans)
    end,

    click_exitBtn = function(self)
        self.war_mgr:setGamePause(false)
        frame_manager:show('loading')
        -- self.war_mgr.level_manager:SetInBattle(false)   
        game_director:changeState('loading', {next_state = 'level', scene_name = level_info.level_cfg.Map, level_id = level_info.map_id, battle_back = true})
        frame_manager:hide('battle')
    end,

    click_continueBtn = function(self)
        self.helper.hideTrans(self.pause_trans)
        self.war_mgr:setGamePause(false)
    end,

    click_restartBtn = function(self)
        print("重新开始...")
    end,

    -- 显示技能图标列表
    showTopSkillList = function(self, in_order, skill_types)
        -- log.error("###: showTopSkillList")
        if self.qte_trans then
            self.qte_trans:showTopSkillList(in_order, skill_types)
        end
    end,

    -- 用户按对了技能 icon
    checkTopSkillIcon = function(self, type)
    end,

    checkSkillIcon = function(self, type, actor_id)
        -- log.error("###: checkSkillIcon")
        if self.qte_trans then
            return self.qte_trans:checkSkillIcon(type, actor_id)
        end
    end,

    -- 显示QTE
    showTopBar = function(self, actor_id, total_time, is_left_to_right, inc_or_dec)
        -- log.error("###: showTopBar")
        if self.qte_trans then
            self.qte_trans:showTopBar(actor_id, total_time, is_left_to_right, inc_or_dec)
        end
    end,

    -- 隐藏顶部技能释放条
    hideTopBar = function(self)
        -- log.error("###: hideTopBar")
        if self.qte_trans then
            self.qte_trans:hideTopBar()
        end
    end,

    showIconBar = function(self)
        -- local go = self.transform:Find("bottomRight/content").gameObject
        -- local pos = 0
        -- local lt = LeanTween.moveLocalY(go, pos, 0.2)
        -- lt.loopType = LeanTweenType.once
    end,

    loadQTE = function(self)
        self.fly_board_trans = self.transform:Find("scaleFB")
        local r2 = 1920 / UnityEngine.Screen.width
        self.fly_board_trans.localScale = Vector3.one * r2
        self.fly_board_trans:SetActive(false)

        if level_info and level_info.map_attack_info then
            local id = level_info.map_attack_info.monsterId
            if config.monster_group_ids[id] then
                if config.monster_group_ids[id] == 0 then
                    return
                end
            else
                return
            end
        end

        local gui = GameObject.Find('gui')
        self.qte_trans = CreateFrame {
            class = "gui.battle_qte_frame",
            parent = gui, 
            name = "battle_qte", 
            asset = "Assets/Frames/BattleQteFrame.prefab",
            owner = self,
        }
    end,
}

