local super = require("ui.frame")
local log = logger.get("Game", "Frame")

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        clickEvents = {
        }
        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self.content_trans = self.transform:Find("root/content/offset")
        self.camera_trans = self.transform:Find("UICamera")

        self.helper.hideTrans(self.content_trans)
        self.transform.localPosition = Vector3(10000, 0, 0)

        self.battle_frame = self.owner
    end, 

    onUnload = function (self)
    end, 

    show = function(self)
        self.is_top_bar_show = true
        self.helper.showTrans(self.content_trans)
        self.battle_frame.fly_board_trans:SetActive(true)
        self.battle_frame.fly_board_trans:GetComponentInChildren(UnityEngine.Animation):CrossFade("idle")
        
        self:playBarAnim("access")
        self.shake_stalls = {1/3, 2/3, 4/5}
    end,

    hide = function(self)
        self.helper.hideTrans(self.content_trans)
    end,

    playBarAnim = function(self, name)
        local anim = self.transform:Find("root/content"):GetComponent(UnityEngine.Animator)
        anim:SetTrigger(name)
    end,

    detectStalls = function(self, value)
        print('stalls', self.shake_stalls and #self.shake_stalls)
        for __, t in ipairs(self.shake_stalls) do
            if value >= t then
                table.remove(self.shake_stalls, __)
                self:playBarAnim("shock")
                break
            end
        end
    end,

    showTopBar = function(self, actor_id, total_time, is_left_to_right, inc_or_dec)
        if self.is_top_bar_show then
            return
        end

        self.top_bar_actor_id = actor_id
        self:show()

        local actor = battle_helper.stage.actors[actor_id]
            if actor and actor.monster_stat_id then
                local monster_stat_cfg = config.monster_stat_ids[actor.monster_stat_id]
                local officer_id = monster_stat_cfg.officer_id
                local officer_res = config.commander_ress[officer_id]
                local boss_icon = self.transform:Find("root/content/offset/BossIcon"):GetComponent(UnityEngine.UI.Image)
                local path = officer_res.bannerhead_path .. officer_res.bannerhead_pic
                log.info("Officer icon path:"..path)
                boss_icon:SetSprite(path)
            else
                log.error("Failed to find actor officer_res, actor id:"..actor_id)
            end

            -- 进度条
            local img_progress = self.transform:Find("root/content/offset/battle_panel_17_1_Fill"):GetComponent(UnityEngine.UI.Image)
            if is_left_to_right then
                img_progress.fillOrigin = 0
            else
                img_progress.fillOrigin = 1
            end

            if inc_or_dec then
                -- 增加方式
                img_progress.fillAmount = 0
                -- text_progress.text = "0.0"
                app:startCoroutine(
                    function()
                        coroutine.yield(WaitForSeconds(0.5))
                        local elapsed = 0.0
                        local start_time = Time.time
                        while self.is_top_bar_show do
                            coroutine.yield(WaitForSeconds(0.1))
                            elapsed = Time.time - start_time
                            -- text_progress.text = string.format("%0.2f", elapsed)
                            img_progress.fillAmount = elapsed / total_time
                            self:detectStalls(elapsed / total_time)

                            if img_progress.fillAmount >= 1.0 then
                                self:hideTopBar()
                            end
                            self:checkQteOver()
                        end
                    end
                )
            else
                -- 减少方式
                img_progress.fillAmount = 1
                -- text_progress.text = string.format("%0.2f", total_time)
                app:startCoroutine(
                    function()
                        coroutine.yield(WaitForSeconds(0.5))
                        local elapsed = 0.0
                        local start_time = Time.time
                        while self.is_top_bar_show do
                            coroutine.yield(WaitForSeconds(0.1))
                            elapsed = Time.time - start_time
                            -- text_progress.text = string.format("%0.2f", total_time - elapsed)
                            img_progress.fillAmount = 1.0 - elapsed / total_time

                            if img_progress.fillAmount <= 0.0 then
                                self:hideTopBar()
                            end
                            self:checkQteOver()
                        end
                    end
                )
            end
    end,

    showTopSkillList = function(self, in_order, skill_types)
        local max_num = 4
        self.top_bar_skill_types = {}
        self.skill_types_in_order = in_order

        local parent_path = "root/content/offset/SkillTypes/"
        for idx = 1, max_num do 
            local child = self.transform:Find(parent_path):GetChild(idx - 1)
            child:SetActive(false)
        end

        for idx, type in ipairs(skill_types) do
            local img = self:findImage("root/content/offset/SkillTypes/SkillIcon"..idx)
            img.gameObject:SetActive(true)
            local check_path = "root/content/offset/SkillTypes/SkillIcon"..idx.."/Check"
            local check_trans = self.transform:Find(check_path)

            local sprite_name = ""
            if type == 1 then
                sprite_name = "Assets/Arts/ui/atlas/icon/vehicle/type1.png"
            elseif type == 2 then
                sprite_name = "Assets/Arts/ui/atlas/icon/vehicle/type2.png"
            elseif type == 3 then
                sprite_name = "Assets/Arts/ui/atlas/icon/vehicle/type3.png"
            elseif type == 4 then
                sprite_name = "Assets/Arts/ui/atlas/icon/vehicle/type4.png"
            end

            img:SetSprite(sprite_name)

            local is_active = false
            if in_order then
                if idx == 1 then
                    is_active = true
                end
            else
                is_active = true
            end

            self.top_bar_skill_types[idx] = {
                index = idx,
                icon = img,
                check_path = check_path,
                check_trans = check_trans,
                type = type,
                is_active = is_active,
            }
        end

        for __, st in pairs(self.top_bar_skill_types) do
            self:syncStateSkillIcon(st, st.is_active)
        end
    end,

    -- 激活顶部的技能图标
    syncStateSkillIcon = function(self, skill_type, is_active)
        if skill_type.is_active and is_active == false then

            skill_type.is_active = is_active
            
            app:startCoroutine(
                    function()
                        local idx = skill_type.index
                        local total_time = 0.5
                        local elapsed_time = 0.5
                        local from_v3 = self.battle_frame.transform:Find("bottomRight/content/grid"):GetChild(idx - 1).position
                        local to_obj = self.battle_frame.transform:Find("layout"):GetChild(idx - 1)
                        local to_v3 = to_obj.position
                        local obj = self.helper:copyTrans(self.battle_frame, "UI_tuowei")
                        obj:SetActive(true)
                        while true do
                            coroutine.yield(nil)
                            elapsed_time = elapsed_time - UnityEngine.Time.deltaTime
                            if elapsed_time <= 0 then elapsed_time = 0 end
                            local rate = elapsed_time / total_time
                            local pos = Vector3.Lerp(from_v3, to_v3, 1 - rate)
                            obj.position = pos
                            if elapsed_time == 0 then
                                to_obj:SetActive(true)
                                self:setImageEnabled(skill_type.check_path, is_active)
                                LeanTween.cancel(skill_type.icon.gameObject)
                                LeanTween.cancel(skill_type.check_trans.gameObject)
                                skill_type.icon.transform.localScale = Vector3.one * 0.9
                                skill_type.check_trans.transform.localScale = Vector3.one * 0.9
                                UnityEngine.Object.Destroy(obj.gameObject)
                                coroutine.yield(WaitForSeconds(1.5))
                                to_obj:SetActive(false)
                                break
                            end
                        end
                end)
            return
        end

        skill_type.is_active = is_active

        self:setImageEnabled(skill_type.check_path, skill_type.is_active)
        LeanTween.cancel(skill_type.icon.gameObject)
        LeanTween.cancel(skill_type.check_trans.gameObject)
        if skill_type.is_active then
            local lt = LeanTween.scale(skill_type.icon.gameObject, Vector3.one * 1.1, 0.25)
            lt.from = Vector3.one
            lt.loopType = LeanTweenType.once
            local lt = LeanTween.scale(skill_type.check_trans.gameObject, Vector3.one * 1.1, 0.25)
            lt.from = Vector3.one
            lt.loopType = LeanTweenType.pingPong
        else
            skill_type.icon.transform.localScale = Vector3.one * 0.9
            skill_type.check_trans.transform.localScale = Vector3.one * 0.9
        end
    end,

    setImageEnabled = function(self, path, enabled)
        local all = self.transform:Find(path):GetComponentsInChildren(UnityEngine.UI.Image).Table
        for __, i in ipairs(all) do
            i.enabled = enabled
        end
    end,

    containQteTank = function(self, actor_id)
        if self.qte_tanks then
            for __, id in ipairs(self.qte_tanks) do
                if id == actor_id then
                    return true
                end
            end
        end
        return false
    end,

    checkSkillIcon = function(self, type, actor_id)
        if self:containQteTank(actor_id) then
            return
        end
        if not self.top_bar_skill_types then
            return
        end
        local getSkillType = function(idx)
            for __, st in pairs(self.top_bar_skill_types) do
                if st.index == idx then
                    return st
                end
            end
        end
        for idx, st in ipairs(self.top_bar_skill_types) do
            if st.is_active and st.type == type then
                -- st.is_active = false
                self:addQteTank(actor_id)
                self:syncStateSkillIcon(st, false)
                local temp_st = getSkillType(idx + 1)
                if temp_st then
                    -- temp_st.is_active = true
                    self:syncStateSkillIcon(temp_st, true)
                end
                break
            else
                if st.is_active and self.skill_types_in_order then
                    log.error("有序序列只能删除首个激活图标！ expect:"..tostring(st.type)..", input:"..tostring(type))
                    break
                end
            end
        end
    end,

    clearQteTank = function(self)
        self.qte_tanks = nil
    end,

    addQteTank = function(self, actor_id)
        self.qte_tanks = self.qte_tanks and self.qte_tanks or {}
        table.insert(self.qte_tanks, actor_id)
    end,

    checkQteOver = function(self)
        if self.is_top_bar_show then
            local over = true
            for __, st in pairs(self.top_bar_skill_types) do
                if st.is_active then
                    over = false
                    break
                end
            end
            if over then
                -- hint:setText(NativeText.GetLangStr(400001))
                self:hideTopBar()
            end
        end
    end,

    hideTopBar = function(self)
        if not self.is_top_bar_show then
            return
        end
        self.is_top_bar_show = false
        self:clearQteTank()

        local ani = self.battle_frame.fly_board_trans:GetComponentInChildren(UnityEngine.Animation)
        app:startCoroutine(
            function()
                coroutine.yield(WaitForSeconds(1.5))
                ani:CrossFade("anim")
                coroutine.yield(WaitForSeconds(1.5))
                self.battle_frame.fly_board_trans:SetActive(false)
                self.helper.hideTrans(self.content_trans)
            end)
    end,
}


