local log = logger.get("Game", "Battle Record Frame")
local item_cls = require('RTS.bag_sys.item')

return class{

    ctor = function(self, panel, owner)
        self.owner = owner
        self.root = panel:Find("Scroll View")
        self.top_node = self.root:Find("Viewport/Content/Top")
        self.result_node = self.root:Find("Viewport/Content/ResultNode")
        self.heros_node = self.root:Find("Viewport/Content/Heros")
        self.reward_node = self.root:Find("Viewport/Content/Reward")
        self.dmg_reward_node = self.root:Find("Viewport/Content/DMGReward")
        self.kill_reward_node = self.root:Find("Viewport/Content/KillReward")
        self.exp_node = self.root:Find("Viewport/Content/ExpNode")
        self.info_node = self.root:Find("Viewport/Content/InfoNode")
        self.monster_info = self.root:Find("Viewport/Content/MonsterInfo")
        self.loss_node = self.root:Find("Viewport/Content/LossNode")
        self.base_loss = self.root:Find("Viewport/Content/BaseLoss")
        self.base_node = self.root:Find("Viewport/Content/BaseNode")
        self.plunder_node = self.root:Find("Viewport/Content/Plunder")
        self.plundered_node = self.root:Find("Viewport/Content/Plundered")
        self.report_btn = self.root:Find("Viewport/Content/Report")

        self.heros_node:SetActive(false)
        self.reward_node:SetActive(false)
        self.dmg_reward_node:SetActive(false)
        self.kill_reward_node:SetActive(false)
        self.exp_node:SetActive(false)
        self.info_node:SetActive(false)
        self.monster_info:SetActive(false)
        self.loss_node:SetActive(false)
        self.base_loss:SetActive(false)
        self.base_node:SetActive(false)
        self.plunder_node:SetActive(false)
        self.plundered_node:SetActive(false)
        self.report_btn:SetActive(true)
    end,

    init = function(self, data)
        self.mail = data
        self.data = data.battleResult

        self.own_info = self.data.atkBaseInfo
        self.enemy_info = self.data.defBaseInfo

        if self.data.selfTeam == 1 then
            self.own_info = self.data.defBaseInfo
            self.enemy_info = self.data.atkBaseInfo
        end

        self:initTop()
        self:initResult()
        self:initHeros()
        self:initReward()
        self:initDmgReward()
        self:initKillReward()
        self:initExp()
        self:initInfo()
        self:initMonsterInfo()
        self:initLoss()
        self:initBaseLoss()
        self:initBase()
        self:initPlunder()
        self:initPlundered()
    end,

    getCfg = function(self, cfg_id)
        local cfg = nil
        if self.enemy_info.objType == common_pb.WORLD_OBJ_PLAYER then
            cfg = config.worldmap_base_configs[cfg_id]
        elseif self.enemy_info.objType == common_pb.WORLD_OBJ_MONSTER or self.enemy_info.objType == common_pb.WORLD_OBJ_RES_SHIP then
            cfg = config.worldmap_monsters[cfg_id]
        end

        if not cfg then
            log.error(string.format("################################## find cfg error ! type %d, id %d", self.enemy_info.objType, cfg_id))
        end

        return cfg
    end,

    setTitle = function(self)
        local sender = self.top_node:Find("Sender"):GetComponent(UnityEngine.UI.Text)
        local title = self.top_node:Find("Title"):GetComponent(UnityEngine.UI.Text)

        local cfg = self:getCfg(self.enemy_info.cfgId)

        if not cfg then
            return
        end

        local pos_str = gui_helper:positionFormat(self.enemy_info.posX, self.enemy_info.posY)

        if self.enemy_info.objType == common_pb.WORLD_OBJ_MONSTER then
            sender.text = string.format("%s %s", pos_str, NativeText.GetLangStr(9000029))
            title.text = string.format( "%sLv.%d %s", NativeText.GetLangStr(9000030), cfg.level, frame_helper:getLangCfg(cfg.monstername) )
            
        elseif self.enemy_info.objType == common_pb.WORLD_OBJ_RES_SHIP then
            sender.text = string.format("%s", NativeText.GetLangStr(9000046))
            title.text = string.format( "%sLv.%d %s", NativeText.GetLangStr(9000030), cfg.level, frame_helper:getLangCfg(cfg.monstername) )
        else
            sender.text = string.format("%s %s", pos_str, NativeText.GetLangStr(9000031))

            local name_str = self.enemy_info.playerName
            if string.len(self.enemy_info.shortName) > 0 then
                name_str = string.format( "[%s]%s", self.enemy_info.shortName, name_str)
            end

            if self.data.selfTeam == 0 then
                title.text = string.format( NativeText.GetLangStr(9000032), name_str, frame_helper:getLangCfg(cfg.nameid or cfg.name_id) )
            else
                title.text = string.format( NativeText.GetLangStr(9000033), name_str, frame_helper:getLangCfg(cfg.nameid or cfg.name_id) )
            end
        end
    end,

    initTop = function(self)
        local time_1 = self.top_node:Find("Time1"):GetComponent(UnityEngine.UI.Text)
        local time_2 = self.top_node:Find("Time2"):GetComponent(UnityEngine.UI.Text)
        local collect_img = self.top_node:Find("CollectBtn/Image"):GetComponent(UnityEngine.UI.Image)

        if self.mail.isCollect == 0 then
            collect_img:SetSprite(res_path.res_path_125)
        else
            collect_img:SetSprite(res_path.res_path_124)
        end

        time_1.text = os.date("%Y/%m/%d", self.mail.Sendtime)
        time_2.text = os.date("%X", self.mail.Sendtime)

        local btn = self.top_node:Find("DelBtn"):GetComponent(UnityEngine.UI.Button)
        btn.onClick:RemoveAllListeners()
        btn.onClick:AddListener(
            function()
                self.owner:onClickDeleteBtn(self.mail)
            end
        )

        btn = self.top_node:Find("CollectBtn"):GetComponent(UnityEngine.UI.Button)
        btn.onClick:RemoveAllListeners()
        btn.onClick:AddListener(
            function()
                self.owner:onClickFavouriteBtn(self.mail)
            end
        )

        self:setTitle()
    end,

    initResult = function(self)
        local victory = self.result_node:Find("Victory")
        local defeat = self.result_node:Find("Defeat")
        local replay_btn = self.result_node:Find("Replay")
        local tip = self.result_node:Find("Text")

        if self.data.winTeam == self.data.selfTeam then
            victory.localScale = Vector3.one
            defeat.localScale = Vector3.zero
        else
            victory.localScale = Vector3.zero
            defeat.localScale = Vector3.one
        end

        local cfg = self:getCfg(self.enemy_info.cfgId)

        if self:isMonster() and cfg and cfg.attack == 4 then
            victory.localScale = Vector3.zero
            defeat.localScale = Vector3.zero
        end
        
        local btn = replay_btn:GetComponent(UnityEngine.UI.Button)
        btn.onClick:RemoveAllListeners()
        btn.onClick:AddListener(
            function()
                self:replayBattle(self.data)
            end
        )
    end,

    initHeros = function(self)

    end,

    initReward = function(self)
        if self.data.comReward and #self.data.comReward > 0 then
            self.reward_node:SetActive(true)

            self:loadItem(self.reward_node, self.data.comReward, "BG/Center/Record/Battle/Scroll View/Viewport/Content/Reward/Panel/")
        else
            self.reward_node:SetActive(false)
        end
    end,

    initDmgReward = function(self)
        if self.data.extReward and #self.data.extReward > 0 then
            self.dmg_reward_node:SetActive(true)

            self:loadItem(self.dmg_reward_node, self.data.extReward, "BG/Center/Record/Battle/Scroll View/Viewport/Content/DMGReward/Panel/")

            local tip = self.dmg_reward_node:Find("Text"):GetComponent(UnityEngine.UI.Text)

            local cfg = config.worldmap_others[1]
            tip.text = string.format( NativeText.GetLangStr(300039), cfg.DamagePercent * 0.01 )
        else
            self.dmg_reward_node:SetActive(false)
        end
    end,

    initKillReward = function(self)
        if self.data.familyReward and #self.data.familyReward > 0 then
            self.kill_reward_node:SetActive(true)

            self:loadItem(self.kill_reward_node, self.data.familyReward, "BG/Center/Record/Battle/Scroll View/Viewport/Content/KillReward/Panel/")
        else
            self.kill_reward_node:SetActive(false)
        end
    end,

    initPlunder = function(self)
        if self.data.selfTeam == 0 and self.data.cityReward and #self.data.cityReward > 0 then
            self.plunder_node:SetActive(true)
            self:loadRes(self.plunder_node, self.data.cityReward)
        else
            self.plunder_node:SetActive(false)
        end
    end,

    initPlundered = function(self)
        if self.data.selfTeam == 1 and self.data.cityReward and #self.data.cityReward > 0 then
            self.plundered_node:SetActive(true)
            self:loadRes(self.plundered_node, self.data.cityReward)
        else
            self.plundered_node:SetActive(false)
        end
    end,

    loadRes = function(self, node, rewards)
        local panel = node:Find("Panel")
        local prefab = panel:GetChild(0).gameObject

        for k, v in ipairs(rewards) do
            local elem = nil
            if k > panel.childCount then
                elem = Object.Instantiate(prefab).transform
                elem:SetParent(panel)
                resetTransformZ(elem)
            else
                elem = panel:GetChild(k-1)
            end

            elem.name = "Item_"..k
            elem.localScale = Vector3.one

            local icon = elem:Find("Image"):GetComponent(UnityEngine.UI.Image)
            local num = elem:Find("Count"):GetComponent(UnityEngine.UI.Text)

            local cfg = item_helper:getItemCfg(v.ItemTemId)
            
            if cfg then
                icon:SetSprite(gui_helper:getItemIconPath(cfg))
                num.text = frame_helper:itemNumFormat(v.Count)
            end
        end

        for i=#rewards, panel.childCount - 1 do
            local child = panel:GetChild(i)
            child.localScale = Vector3.zero
        end
    end,

    loadItem = function(self, node, rewards, path)
        local panel = node:Find("Panel")
        local prefab = panel:GetChild(0).gameObject

        for k, v in ipairs(rewards) do
            local elem = nil
            if k > panel.childCount then
                elem = Object.Instantiate(prefab).transform
                elem:SetParent(panel)
                resetTransformZ(elem)
            else
                elem = panel:GetChild(k-1)
            end

            elem.name = "Item_"..k
            elem.localScale = Vector3.one

            local icon = elem:GetComponent(UnityEngine.UI.Image)
            local grade = elem:Find("Grade"):GetComponent(UnityEngine.UI.Image)
            local num = elem:Find("Count"):GetComponent(UnityEngine.UI.Text)
            local btn = elem:GetComponent(UnityEngine.UI.Button)

            local cfg = item_helper:getItemCfg(v.ItemTemId)
            
            if cfg then
                grade:SetSprite(gui_helper:getNewQualityIcon(cfg.color))
                icon:SetSprite(gui_helper:getItemIconPath(cfg))
                num.text = "x"..frame_helper:itemNumFormat(v.Count)
                
                -- local cb = function()
                --     local item_inst = item_cls({cfgId = v.ItemTemId, count = frame_helper:getItemNum(v.ItemTemId)})
                --     frame_manager:show("itemInfo", nil, {item = item_inst})
                -- end
                
                btn.onClick:RemoveAllListeners()
                btn.onClick:AddListener(
                    function()
                        local item_inst = item_cls({cfgId = v.ItemTemId, count = frame_helper:getItemNum(v.ItemTemId)})
                        frame_manager:show("itemInfo", nil, {item = item_inst})
                    end
                )
                -- path = path..elem.name
                -- self.owner:clearScript("click", path)
                -- self.owner:setScript("click", path, cb)
            end
        end

        for i=#rewards, panel.childCount - 1 do
            local child = panel:GetChild(i)
            child.localScale = Vector3.zero
        end
    end,

    initExp = function(self)

    end,

    initInfo = function(self)
        if not self:isEnergyMst() then
            self.info_node:SetActive(true)

            local left = self.info_node:Find("Left")
            local right = self.info_node:Find("Right")
            local bg_1 = self.info_node:Find("ADBG")
            local bg_2 = self.info_node:Find("ADBG2")
            local atk_text = self.info_node:Find("Attack"):GetComponent(UnityEngine.UI.Text)
            local def_text = self.info_node:Find("Defense"):GetComponent(UnityEngine.UI.Text)

            local setInfo = function(node, info, is_own)
                local icon = node:Find("Icon"):GetComponent(UnityEngine.UI.Image)
                local level = node:Find("Level"):GetComponent(UnityEngine.UI.Text)
                local name = node:Find("Name"):GetComponent(UnityEngine.UI.Text)
                local pos = node:Find("Position"):GetComponent(UnityEngine.UI.Text)
                local vip = node:Find("VIP")
                local vip_value = node:Find("VIP/Text"):GetComponent(UnityEngine.UI.Text)
                
                level.text = "Lv.1"
                name.text = self:getName(info, is_own)
                pos.text = gui_helper:positionFormat(info.posX, info.posY)

                pos.transform:SetActive(info.objType ~= common_pb.WORLD_OBJ_RES_SHIP)

                if is_own then
                    name.color = Color(53/255, 164/255, 210/255, 1)
                else
                    name.color = Color(248/255, 73/255, 73/255, 1)
                end
                
                if not is_own and self:isMonster() then
                    vip:SetActive(false)

                    local cfg = config.worldmap_monsters[info.cfgId]

                    if cfg then
                        icon:SetSprite(cfg.iconheadpath..cfg.monsteheadricon)
                        level.text = "Lv."..cfg.level
                    end
                else
                    vip_value.text = info.vipLev
                    icon:SetSprite(res_path.res_path_139)
                end
            end

            if self.data.selfTeam == 1 then
                bg_1.localRotation = Quaternion.Euler(0, 180, 0)
                bg_2.localRotation = Quaternion.Euler(0, 180, 0)

                atk_text.text = NativeText.GetLangStr(300001)
                def_text.text = string.format("%s[%s]", NativeText.GetLangStr(300004), NativeText.GetLangStr(300002))

                setInfo(left, self.enemy_info, false)
                setInfo(right, self.own_info, true)
            else
                bg_1.localRotation = Quaternion.Euler(0, 0, 0)
                bg_2.localRotation = Quaternion.Euler(0, 0, 0)

                atk_text.text = string.format("%s[%s]", NativeText.GetLangStr(300001), NativeText.GetLangStr(300002))
                def_text.text = NativeText.GetLangStr(300004)

                setInfo(left, self.own_info, true)
                setInfo(right, self.enemy_info, false)
            end

            local mass_tag = left:Find("Image")
            mass_tag:SetActive(self:isMass())
        else
            self.info_node:SetActive(false)
        end
    end,

    initMonsterInfo = function(self)
        if self:isEnergyMst() then
            self.monster_info:SetActive(true)

            local left = self.monster_info:Find("Left")
            local right = self.monster_info:Find("Right")
            local bg_1 = self.monster_info:Find("ADBG")
            local bg_2 = self.monster_info:Find("ADBG2")
            local atk_text = self.monster_info:Find("Attack"):GetComponent(UnityEngine.UI.Text)
            local def_text = self.monster_info:Find("Defense"):GetComponent(UnityEngine.UI.Text)

            local setInfo = function(node, info, is_own)
                local icon = node:Find("Icon"):GetComponent(UnityEngine.UI.Image)
                local level = node:Find("Level"):GetComponent(UnityEngine.UI.Text)
                local name = node:Find("Name"):GetComponent(UnityEngine.UI.Text)
                local pos = node:Find("Position"):GetComponent(UnityEngine.UI.Text)
                local vip = node:Find("VIP")
                local vip_value = node:Find("VIP/Text"):GetComponent(UnityEngine.UI.Text)
                local dmg_1 = node:Find("DMG1"):GetComponent(UnityEngine.UI.Text)
                local dmg_2 = node:Find("DMG2"):GetComponent(UnityEngine.UI.Text)
                
                level.text = "Lv.1"
                name.text = self:getName(info, is_own)
                pos.text = gui_helper:positionFormat(info.posX, info.posY)

                if info.multHurt > 1 then
                    dmg_1.transform:SetActive(true)
                    dmg_2.transform:SetActive(true)

                    dmg_1.text = string.format( NativeText.GetLangStr(300005), info.multHurt )
                    dmg_2.text = string.format( NativeText.GetLangStr(300006), info.multHurt )
                else
                    dmg_1.transform:SetActive(false)
                    dmg_2.transform:SetActive(false)
                end

                if is_own then
                    name.color = Color(53/255, 164/255, 210/255, 1)
                else
                    name.color = Color(248/255, 73/255, 73/255, 1)
                end
                
                if not is_own and self:isMonster() then
                    vip:SetActive(false)
                else
                    vip:SetActive(true)
                    vip_value.text = info.vipLev
                end
            end

            local monsterInfo = function(node, info)
                local icon = node:Find("Icon"):GetComponent(UnityEngine.UI.Image)
                local grade = node:Find("Grade"):GetComponent(UnityEngine.UI.Image)
                local level = node:Find("Level"):GetComponent(UnityEngine.UI.Text)
                local lost = node:Find("Lost"):GetComponent(UnityEngine.UI.Text)
                local fill = node:Find("FillBG/Image"):GetComponent(UnityEngine.UI.Image)
                local percent = node:Find("FillBG/Percent"):GetComponent(UnityEngine.UI.Text)

                local cfg = config.worldmap_monsters[info.cfgId]
                
                if cfg then
                    icon:SetSprite(cfg.iconheadpath..cfg.monsteheadricon)
                    level.text = cfg.level
                    lost.text = string.format( "-%.2f%%", info.lostHp / info.totalHp * 100 )
                    local per = info.curHp / info.totalHp
                    fill.fillAmount = per
                    percent.text = string.format( "%.2f%%", per * 100 )
                end
            end

            if self.data.selfTeam == 1 then
                bg_1.localRotation = Quaternion.Euler(0, 180, 0)
                bg_2.localRotation = Quaternion.Euler(0, 180, 0)

                atk_text.text = NativeText.GetLangStr(300001)
                def_text.text = string.format("%s[%s]", NativeText.GetLangStr(300004), NativeText.GetLangStr(300002))

                setInfo(left, self.own_info, true)
                monsterInfo(right, self.enemy_info)
            else
                bg_1.localRotation = Quaternion.Euler(0, 0, 0)
                bg_2.localRotation = Quaternion.Euler(0, 0, 0)

                atk_text.text = string.format("%s[%s]", NativeText.GetLangStr(300001), NativeText.GetLangStr(300002))
                def_text.text = NativeText.GetLangStr(300004)

                setInfo(left, self.own_info, true)
                monsterInfo(right, self.enemy_info)
            end

            local hunts_text1 = self.monster_info:Find("Hunts/Text1"):GetComponent(UnityEngine.UI.Text)
            local hunts_text2 = self.monster_info:Find("Hunts/Text2"):GetComponent(UnityEngine.UI.Text)
            local hunts_text3 = self.monster_info:Find("Hunts/Text3"):GetComponent(UnityEngine.UI.Text)

            if self.data.continueHurt.atkCount > 0 then
                hunts_text1.transform:SetActive(true)
                hunts_text2.transform:SetActive(true)
                hunts_text3.transform:SetActive(false)

                hunts_text1.text = string.format( NativeText.GetLangStr(300032), self.data.continueHurt.atkCount )
                hunts_text2.text = string.format( NativeText.GetLangStr(300033), self.data.continueHurt.hurtAdd * 0.01 )
            else
                hunts_text1.transform:SetActive(false)
                hunts_text2.transform:SetActive(false)
                hunts_text3.transform:SetActive(true)
            end

        else
            self.monster_info:SetActive(false)
        end
    end,

    initLoss = function(self)
        if not self:isAttackBase() then
            self.loss_node:SetActive(true)

            local left = self.loss_node:Find("Left")
            local right = self.loss_node:Find("Right")
            local bg = self.loss_node:Find("Image (2)/Image")
            local tip = self.loss_node:Find("Tip"):GetComponent(UnityEngine.UI.Text)

            tip.transform:SetActive(self:isMonster())
            
            if self:isMonster() then
                local cfg = config.worldmap_monsters[self.enemy_info.cfgId]

                if cfg then
                    local value_1 = cfg.RevivePercent
                    local value_2 = cfg.DeathReduce
                    local d = value_2 - value_1

                    if d > 0 then
                        tip.text = string.format( NativeText.GetLangStr(9000001), value_1 * 0.01, d * 0.01 )
                    else
                        tip.text = string.format( NativeText.GetLangStr(9000002), value_1 * 0.01 )
                    end
                else
                    log.error("################################ not find monster cfg id ", self.enemy_info.cfgId)
                end
            end

            local setInfo = function(node, info, is_own)
                local total = node:Find("Total"):GetComponent(UnityEngine.UI.Text)
                local tag = node:Find("Tag"):GetComponent(UnityEngine.UI.Text)
                local might = node:Find("Might"):GetComponent(UnityEngine.UI.Text)
                local soldiers = node:Find("Soldiers"):GetComponent(UnityEngine.UI.Text)
                local wounded = node:Find("Wounded"):GetComponent(UnityEngine.UI.Text)
                local dead = node:Find("Dead"):GetComponent(UnityEngine.UI.Text)
                local survived = node:Find("Survived"):GetComponent(UnityEngine.UI.Text)
                
                if is_own then
                    tag.text = NativeText.GetLangStr(300009)
                    tag.color = Color(53/255, 164/255, 210/255, 1)
                else
                    tag.text = NativeText.GetLangStr(300013)
                    tag.color = Color(248/255, 73/255, 73/255, 1)
                end

                total.text = tostring(info.soldierHurt.hurtSoldier + info.soldierHurt.dieSoldier)
                might.text = string.format( "%s: <color=#F84949FF>-%s</color>", NativeText.GetLangStr(300008), info.sword )
                soldiers.text = NativeText.GetLangStr(300011)..": "..info.soldierHurt.totalSoldier
                wounded.text = NativeText.GetLangStr(300041)..": "..info.soldierHurt.hurtSoldier
                dead.text = NativeText.GetLangStr(300025)..": "..info.soldierHurt.dieSoldier
                survived.text = NativeText.GetLangStr(300012)..": "..info.soldierHurt.totalSoldier - info.soldierHurt.hurtSoldier - info.soldierHurt.dieSoldier
            end

            if self.data.selfTeam == 1 then
                bg.localRotation = Quaternion.Euler(0, 180, 0)

                setInfo(left, self.enemy_info, false)
                setInfo(right, self.own_info, true)
            else
                bg.localRotation = Quaternion.Euler(0, 0, 0)

                setInfo(left, self.own_info, true)
                setInfo(right, self.enemy_info, false)
            end
        else
            self.loss_node:SetActive(false)
        end
    end,

    initBaseLoss = function(self)
        if self:isAttackBase() then
            self.base_loss:SetActive(true)

            local left = self.base_loss:Find("Left")
            local right = self.base_loss:Find("Right")
            local bg = self.base_loss:Find("Image (2)/Image")

            local setInfo = function(node, info, is_own, is_def)
                local total = node:Find("Total"):GetComponent(UnityEngine.UI.Text)
                local tag = node:Find("Tag"):GetComponent(UnityEngine.UI.Text)
                local might = node:Find("Might"):GetComponent(UnityEngine.UI.Text)
                local soldiers = node:Find("Soldiers"):GetComponent(UnityEngine.UI.Text)
                local wounded = node:Find("Wounded"):GetComponent(UnityEngine.UI.Text)
                local dead = node:Find("Dead"):GetComponent(UnityEngine.UI.Text)
                local survived = node:Find("Survived"):GetComponent(UnityEngine.UI.Text)
                
                if is_own then
                    tag.text = NativeText.GetLangStr(300009)
                    tag.color = Color(53/255, 164/255, 210/255, 1)
                else
                    tag.text = NativeText.GetLangStr(300013)
                    tag.color = Color(248/255, 73/255, 73/255, 1)
                end

                local total_cnt = info.soldierHurt.hurtSoldier + info.soldierHurt.dieSoldier
                soldiers.text = NativeText.GetLangStr(300011)..": "..info.soldierHurt.totalSoldier
                wounded.text = NativeText.GetLangStr(300041)..": "..info.soldierHurt.hurtSoldier
                dead.text = NativeText.GetLangStr(300025)..": "..info.soldierHurt.dieSoldier
                survived.text = NativeText.GetLangStr(300012)..": "..info.soldierHurt.totalSoldier - info.soldierHurt.hurtSoldier - info.soldierHurt.dieSoldier

                if is_def then
                    local traps_2 = node:Find("Soldiers (1)"):GetComponent(UnityEngine.UI.Text)
                    local wounded_2 = node:Find("Wounded (1)"):GetComponent(UnityEngine.UI.Text)
                    local dead_2 = node:Find("Dead (1)"):GetComponent(UnityEngine.UI.Text)
                    local survived_2 = node:Find("Survived (1)"):GetComponent(UnityEngine.UI.Text)

                    traps_2.text = NativeText.GetLangStr(300010)..": "..self.data.cannonStat.totalSoldier
                    wounded_2.text = NativeText.GetLangStr(300056)..": "..self.data.cannonStat.hurtSoldier
                    dead_2.text = NativeText.GetLangStr(300057)..": "..self.data.cannonStat.dieSoldier
                    survived_2.text = NativeText.GetLangStr(300058)..": "..self.data.cannonStat.totalSoldier - self.data.cannonStat.hurtSoldier - self.data.cannonStat.dieSoldier
                    total_cnt = total_cnt + self.data.cannonStat.hurtSoldier + self.data.cannonStat.dieSoldier
                end
                
                total.text = tostring(total_cnt)
                might.text = string.format( "%s: <color=#F84949FF>-%d</color>", NativeText.GetLangStr(300008), info.sword )
            end

            if self.data.selfTeam == 1 then
                bg.localRotation = Quaternion.Euler(0, 180, 0)

                setInfo(left, self.enemy_info, false, false)
                setInfo(right, self.own_info, true, true)
            else
                bg.localRotation = Quaternion.Euler(0, 0, 0)

                setInfo(left, self.own_info, true, false)
                setInfo(right, self.enemy_info, false, true)
            end
        else
            self.base_loss:SetActive(false)
        end
    end,

    initBase = function(self)
        if self:isAttackBase() then
            self.base_node:SetActive(true)

            local bg_1 = self.base_node:Find("AD")
            local bg_2 = self.base_node:Find("AD (1)")
            local atk_text = self.base_node:Find("Attack"):GetComponent(UnityEngine.UI.Text)
            local def_text = self.base_node:Find("Defense"):GetComponent(UnityEngine.UI.Text)

            if self.data.selfTeam == 1 then
                bg_1.localRotation = Quaternion.Euler(0, 180, 0)
                bg_2.localRotation = Quaternion.Euler(0, 180, 0)

                atk_text.text = NativeText.GetLangStr(300001)
                def_text.text = string.format("%s[%s]", NativeText.GetLangStr(300004), NativeText.GetLangStr(300002))
            else
                bg_1.localRotation = Quaternion.Euler(0, 0, 0)
                bg_2.localRotation = Quaternion.Euler(0, 0, 0)

                atk_text.text = string.format("%s[%s]", NativeText.GetLangStr(300001), NativeText.GetLangStr(300002))
                def_text.text = NativeText.GetLangStr(300004)
            end

            local right = self.base_node:Find("Right")
            local text_1 = right:GetChild(0):GetComponent(UnityEngine.UI.Text)
            local text_2 = right:GetChild(1):GetComponent(UnityEngine.UI.Text)
            local text_3 = right:GetChild(2):GetComponent(UnityEngine.UI.Text)
            local text_4 = right:GetChild(4):GetComponent(UnityEngine.UI.Text)
            local text_5 = right:GetChild(5):GetComponent(UnityEngine.UI.Text)
            local text_6 = right:GetChild(6):GetComponent(UnityEngine.UI.Text)

            text_1.text = string.format("%s: %d", NativeText.GetLangStr(300043), self.data.cityDur.cityTotalDur)
            text_2.text = string.format("%s: %d", NativeText.GetLangStr(300044), self.data.cityDur.cityHurtDur)
            text_3.text = string.format("%s: %d", NativeText.GetLangStr(300045), self.data.cityDur.cityLeftDur)
            text_4.text = string.format("%s: %d", NativeText.GetLangStr(300047), self.data.cityDur.baseTotalDur)
            text_5.text = string.format("%s: %d", NativeText.GetLangStr(300048), self.data.cityDur.baseHurtDur)
            text_6.text = string.format("%s: %d", NativeText.GetLangStr(300049), self.data.cityDur.baseLeftDur)

        else
            self.base_node:SetActive(false)
        end
    end,

    isMass = function(self)
        return #self.data.atkTeams > 1
    end,

    isAttackBase = function(self)
        return self.data.warType == common_pb.EWAR_TARGET_CITY
    end,

    isEnergyMst = function(self)
        return self.data.warType == common_pb.EWAR_TARGET_ACTION_MONSTER
    end,

    isMonster = function(self)
        if self.enemy_info.objType == common_pb.WORLD_OBJ_MONSTER or self.enemy_info.objType == common_pb.WORLD_OBJ_RES_SHIP then
            return true
        end

        if self.data.warType == common_pb.EWAR_TARGET_PLAYER or 
        self.data.warType == common_pb.EWAR_TARGET_CITY then
            return false
        end

        return true
    end,

    getName = function(self, info, is_own)
        local str = info.playerName

        if info.familyName and string.len(info.familyName) > 2 then
            str = string.format( "[%s]%s", info.familyName, str )
        end

        if not is_own and self:isMonster() then
            local cfg = config.worldmap_monsters[info.cfgId]
            if cfg then
                str = frame_helper:getLangCfg(cfg.monstername)
            end
        end

        return str
    end,

    allDead = function(self)
        local soldiers = self.own_info.soldierHurt

        return soldiers.totalSoldier == soldiers.dieSoldier
    end,

    replayBattle = function(self, data)
        local mail_id = self.mail.mailId
        game_info.replay_battle_mail_id = mail_id
        game_info.replay_battle_from_state = game_director.curr_state.name
        log.info("##### mail id:"..tostring(mail_id)..", state:"..game_info.replay_battle_from_state)

        local battle_report = {
            S0 = {
                cfg_id = {0,0,0,0},  -- 4 种类型的 id
                total = {0,0,0,0},  -- 四种士兵参战人数
                mid_left = {0,0,0,0},
                left = {0,0,0,0},
            },
            S1 = {
                cfg_id = {0,0,0,0},
                total = {0,0,0,0},  -- 四种士兵参战人数
                mid_left = {0,0,0,0},
                left = {0,0,0,0},
            },
            T0 = {
                cfg_id = {},
            },
            T1 = {
                cfg_id = {},
            },
            win_team = 0,
            win_degree = 1, -- 胜利的程度
            turns = 30,     -- 回合数量
            is_ship_battle = false, -- 是否是船战
        }

        if data.warType == common_pb.EWAR_TARGET_CITY then
            battle_report.is_ship_battle = true
        else
            battle_report.is_ship_battle = false
        end

        battle_report.win_team = data.winTeam      -- 胜利的队伍 0-进攻方  1-防守方
        battle_report.min_win_team = data.midWinTeam
        battle_report.win_degree = data.batDegree -- 胜利/失败的程度
        battle_report.defence_total = data.defenceTotal
        battle_report.defence_left = data.defenceLeft
        battle_report.name0 = data.nameTeam0
        battle_report.name1 = data.nameTeam1
        battle_report.icon0 = data.icon0
        battle_report.icon1 = data.icon1
        battle_report.timeOut = data.timeOut
        
        battle_report.turns = data.turns           -- 战斗回合数
        if battle_report.turns < 10 then    -- 回合数太少客户端表现不好，设置最小回合数
            battle_report.turns = 10 
        end    

        battle_report.break_wall_turns = data.breakWallTurns
        if battle_report.break_wall_turns and 
            battle_report.break_wall_turns > 0 and 
            battle_report.break_wall_turns < 5 then
                battle_report.break_wall_turns = 5
        end

        battle_report.occupy_turns = data.turns
        if battle_report.occupy_turns < 10 then
            battle_report.occupy_turns = 10 
        end 

        -- 士兵
        for idx, soldier in ipairs(data.atk.batSoldier) do 
            local cfg = config.soldier_soldiers[soldier.cfgId]
            if not cfg then
                log.error("Failed to find soldier cfgId:"..tostring(soldier.cfgId))
            else
                local type = cfg.type   -- 小兵的类型 1~4
                local num = soldier.count -- 数量
                log.info("#### atk soldier type:"..type..", num:"..num)
                local left_num = soldier.leftCount
                local mid_left = soldier.midLeftCount

                log.info("total:"..num..", mid_left:"..mid_left..", left:"..left_num)
                battle_report.S0.cfg_id[type] = cfg.tankid  -- 从小兵 id 转为 tank id
                battle_report.S0.total[type] = num
                battle_report.S0.mid_left[type] = mid_left
                battle_report.S0.left[type] = left_num


            end
        end

        for idx, soldier in ipairs(data.def.batSoldier) do 
            local cfg = config.soldier_soldiers[soldier.cfgId]
            if not cfg then
                log.error("Failed to find soldier cfgId:"..tostring(soldier.cfgId))
            else
                local type = cfg.type   -- 小兵的类型 1~4
                local num = soldier.count -- 数量
                log.info("#### def soldier type:"..type..", num:"..num)
                local left_num = soldier.leftCount
                local mid_left = soldier.midLeftCount
                log.info("total:"..num..", mid_left:"..mid_left..", left:"..left_num)
                battle_report.S1.cfg_id[type] = cfg.tankid  -- 从小兵 id 转为 tank id
                battle_report.S1.total[type] = num
                battle_report.S1.mid_left[type] = mid_left
                battle_report.S1.left[type] = left_num                
            end
        end

        -- 战车
        for idx, id in ipairs(data.atk.tankCfgId) do
            if id ~= 0 then
                table.insert(battle_report.T0.cfg_id, id)
                log.info("#### atk tank type:"..id)
            else
                log.error("############################################## atk tank cfg id is 0")
            end
        end

        for idx, id in ipairs(data.def.tankCfgId) do
            if id ~= 0 then
                table.insert(battle_report.T1.cfg_id, id)
                log.info("#### def tank type:"..id)
            else
                log.error("############################################## def tank cfg id is 0")
            end
        end

        -- 炮台
        if data.def.cannon then
            log.info('### data.def.cannon:'..#data.def.cannon)
            battle_report.Cannon = {
                cfg_id = {},
                total = {},
                left = {},
            }

            for idx = 1,3 do
                local cannon = data.def.cannon[idx]
                if cannon then
                    local cfg = config.soldier_soldiers[cannon.cfgId]
                    if not cfg then
                        log.error('Failed to find cannon soldier cfg:'..cannon.cfgId)
                    else

                    end

                    battle_report.Cannon.cfg_id[idx] = cfg.tankid  -- cfg.tankid 从小兵 id 转为 tank id
                    battle_report.Cannon.total[idx] = cannon.count
                    battle_report.Cannon.left[idx] = cannon.leftCount                    
                else
                    battle_report.Cannon.cfg_id[idx] = 0  -- 从小兵 id 转为 tank id
                    battle_report.Cannon.total[idx] = 0
                    battle_report.Cannon.left[idx] = 0      
                end
            end

        else
            battle_report.Cannon = nil
        end

        if battle_report.is_ship_battle then
            game_director:changeState('loading', {next_state = 'battle_replay', scene_name = 'desert_ship_battle', battle_report = battle_report})
        else
            game_director:changeState('loading', {next_state = 'battle_replay', scene_name = 'desert_1', battle_report = battle_report})
        end
    end,

    show = function(self)
        self.root:SetActive(true)
    end,

    hide = function(self)
        self.root:SetActive(false)
    end,
}