local log = logger.get("Game", "Battle Report Details")

return class{

    ctor = function(self, panel, owner)
        self.owner = owner
        self.root = panel:Find("DetailScroll")
        self.content = self.root:Find("Viewport/Content")
        self.top_node = self.root:Find("Viewport/Content/Top")
        self.info_node = self.root:Find("Viewport/Content/InfoNode")
        self.attack_node = self.root:Find("Viewport/Content/Attacker")
        self.defense_node = self.root:Find("Viewport/Content/Defender")
        self.trap_node = self.root:Find("Viewport/Content/Traps")

        self.trap_node:SetActive(false)

        self.attack_ui = {}
        self.attack_ui[1] = self.attack_node

        self.defend_ui = {}
        self.defend_ui[1] = self.defense_node
    end,

    init = function(self, data)
        self.mail = data
        self.data = data.battleResult

        self.own_info = self.data.atkBaseInfo
        self.enemy_info = self.data.defBaseInfo

        if self.data.selfTeam == 1 then
            self.own_info = self.data.defBaseInfo
            self.enemy_info = self.data.atkBaseInfo
        end

        self:initTop()
        self:initInfo()
        self:initAttacker()
        self:initDefender()
        self:initTraps()
    end,

    getCfg = function(self, cfg_id)
        local cfg = nil
        if self.enemy_info.objType == common_pb.WORLD_OBJ_PLAYER then
            cfg = config.worldmap_base_configs[cfg_id]
        elseif self.enemy_info.objType == common_pb.WORLD_OBJ_MONSTER or self.enemy_info.objType == common_pb.WORLD_OBJ_RES_SHIP then
            cfg = config.worldmap_monsters[cfg_id]
        end

        if not cfg then
            log.error(string.format("################################## find cfg error ! type %d, id %d", self.enemy_info.objType, cfg_id))
        end

        return cfg
    end,
    
    setTitle = function(self)
        local sender = self.top_node:Find("Sender"):GetComponent(UnityEngine.UI.Text)
        local title = self.top_node:Find("Title"):GetComponent(UnityEngine.UI.Text)

        local cfg = self:getCfg(self.enemy_info.cfgId)

        if not cfg then
            return
        end

        local pos_str = gui_helper:positionFormat(self.enemy_info.posX, self.enemy_info.posY)

        if self.enemy_info.objType == common_pb.WORLD_OBJ_MONSTER then
            sender.text = string.format("%s %s", pos_str, NativeText.GetLangStr(9000029))
            title.text = string.format( "%sLv.%d %s", NativeText.GetLangStr(9000030), cfg.level, frame_helper:getLangCfg(cfg.monstername) )
            
        elseif self.enemy_info.objType == common_pb.WORLD_OBJ_RES_SHIP then
            sender.text = string.format("%s", NativeText.GetLangStr(9000046))
            title.text = string.format( "%sLv.%d %s", NativeText.GetLangStr(9000030), cfg.level, frame_helper:getLangCfg(cfg.monstername) )
        else
            sender.text = string.format("%s %s", pos_str, NativeText.GetLangStr(9000031))

            local name_str = self.enemy_info.playerName
            if string.len(self.enemy_info.shortName) > 0 then
                name_str = string.format( "[%s]%s", self.enemy_info.shortName, name_str)
            end

            if self.data.selfTeam == 0 then
                title.text = string.format( NativeText.GetLangStr(9000032), name_str, frame_helper:getLangCfg(cfg.nameid or cfg.name_id) )
            else
                title.text = string.format( NativeText.GetLangStr(9000033), name_str, frame_helper:getLangCfg(cfg.nameid or cfg.name_id) )
            end
        end
    end,

    initTop = function(self)
        local time_1 = self.top_node:Find("Time1"):GetComponent(UnityEngine.UI.Text)
        local time_2 = self.top_node:Find("Time2"):GetComponent(UnityEngine.UI.Text)

        time_1.text = os.date("%Y/%m/%d", self.mail.sendTime)
        time_2.text = os.date("%X", self.mail.sendTime)

        self:setTitle()
    end,

    initInfo = function(self)
        local atk_text = self.info_node:Find("ADBG/Attack"):GetComponent(UnityEngine.UI.Text)
        local def_text = self.info_node:Find("ADBG/Defense"):GetComponent(UnityEngine.UI.Text)

        if self.data.selfTeam == 1 then
            atk_text.text = NativeText.GetLangStr(300001)
            def_text.text = string.format("%s[<color=#1CB0ECFF>%s</color>]", NativeText.GetLangStr(300004), NativeText.GetLangStr(300002))
        else
            atk_text.text = string.format("%s[<color=#1CB0ECFF>%s</color>]", NativeText.GetLangStr(300001), NativeText.GetLangStr(300002))
            def_text.text = NativeText.GetLangStr(300004)
        end

        local panel = self.info_node:Find("ADBG2")
        local prefab = panel:GetChild(1).gameObject

        for k, v in ipairs(self.data.soldierStat) do
            local elem = nil
            if k > panel.childCount - 1 then
                elem = Object.Instantiate(prefab).transform
                elem:SetParent(panel)
                resetTransformZ(elem)
            else
                elem = panel:GetChild(k)
            end

            elem:SetActive(true)

            local cfg = config.soldier_soldiers[v.cfgid]

            if cfg then
                local icon = elem:GetChild(0):GetComponent(UnityEngine.UI.Image)
                local text_1 = elem:GetChild(1):GetComponent(UnityEngine.UI.Text)
                local text_2 = elem:GetChild(2):GetComponent(UnityEngine.UI.Text)
                local text_3 = elem:GetChild(3):GetComponent(UnityEngine.UI.Text)
                local text_4 = elem:GetChild(4):GetComponent(UnityEngine.UI.Text)
                local text_5 = elem:GetChild(5):GetComponent(UnityEngine.UI.Text)

                icon:SetSprite(cfg.iconpath..cfg.soldiericon)
                text_1.text = frame_helper:getLangCfg(cfg.soldiername)
                text_2.text = v.atkTotal
                text_3.text = v.atkHurt
                text_4.text = v.defTotal
                text_5.text = v.defHurt
            end
        end

        for i=#self.data.soldierStat+1, panel.childCount - 1 do
            local child = panel:GetChild(i)
            child:SetActive(false) 
        end

        local bg_1 = self.info_node:Find("BG1")
        local bg_2 = self.info_node:Find("BG2")
        local bg_3 = self.info_node:Find("BG3")

        if self.data.selfTeam == 1 then
            local img = bg_2:GetComponent(UnityEngine.UI.Image)
            img:SetSprite(res_path.res_path_101)
            img = bg_3:GetComponent(UnityEngine.UI.Image)
            img:SetSprite(res_path.res_path_102)
        else
            local img = bg_2:GetComponent(UnityEngine.UI.Image)
            img:SetSprite(res_path.res_path_102)
            img = bg_3:GetComponent(UnityEngine.UI.Image)
            img:SetSprite(res_path.res_path_101)
        end

        app:startCoroutine(
            function()
                coroutine.yield(nil)
                bg_1.sizeDelta = Vector2(454, panel.rect.height + 80)
                bg_2.sizeDelta = Vector2(454, panel.rect.height + 80)
                bg_3.sizeDelta = Vector2(454, panel.rect.height + 80)
                self.info_node.sizeDelta = Vector2(1353, panel.rect.height + 242)
            end
        )
    end,

    initAttacker = function(self)
        local title = self.content:Find("AttackTitle/Text"):GetComponent(UnityEngine.UI.Text)

        if self.data.selfTeam == 0 then
            title.text = string.format("%s[<color=#1CB0ECFF>%s</color>]", NativeText.GetLangStr(300021), NativeText.GetLangStr(300002))
        else
            title.text = NativeText.GetLangStr(300021)
        end

        local prefab = self.attack_node.gameObject
        local cnt = 1

        for k, v in ipairs(self.data.atkTeams) do
            local elem = nil
            if k > #self.attack_ui then
                elem = Object.Instantiate(prefab).transform
                elem:SetParent(self.content)
                resetTransformZ(elem)

                local idx = self.attack_node:GetSiblingIndex()
                elem:SetSiblingIndex(idx + k - 1)

                table.insert( self.attack_ui, elem )
            else
                elem = self.attack_ui[k]
            end

            elem:SetActive(true)

            local name = elem:Find("Top/Name"):GetComponent(UnityEngine.UI.Text)
            local trap_cnt = elem:Find("Top/Traps"):GetComponent(UnityEngine.UI.Text)
            local enemy_cnt = elem:Find("Top/Killed"):GetComponent(UnityEngine.UI.Text)

            if self.data.warType == common_pb.EWAR_TARGET_CITY then
                trap_cnt.transform:SetActive(true)
                trap_cnt.text = string.format("%s: <color=#ffffffff>%d</color>", NativeText.GetLangStr(300053), v.killCannon)
            else
                trap_cnt.transform:SetActive(false)
            end
            enemy_cnt.text = string.format("%s: <color=#ffffffff>%d</color>", NativeText.GetLangStr(300035), v.killSoldiers)

            if v.objType == common_pb.WORLD_OBJ_MONSTER then
                local cfg = config.worldmap_monsters[v.cfgId]

                if cfg then
                    name.text = frame_helper:getLangCfg(cfg.monstername)
                end
            else
                name.text = v.strName
            end
        
            local bg = elem:Find("BG"):GetComponent(UnityEngine.UI.Image)
        
            if self.data.selfTeam == 1 then
                bg:SetSprite(res_path.res_path_101)
            else
                bg:SetSprite(res_path.res_path_102)
            end

            self:loadTeam(elem, v)

            cnt = cnt + 1
        end

        for i=cnt, #self.attack_ui do
            local idx = self.attack_node:GetSiblingIndex()
            idx = idx + i - 1
            local child = self.content:GetChild(idx)
            child:SetActive(false)
        end
    end,

    initDefender = function(self)
        local title = self.content:Find("DefenseTitle/Text"):GetComponent(UnityEngine.UI.Text)

        if self.data.selfTeam == 1 then
            title.text = string.format("%s[<color=#1CB0ECFF>%s</color>]", NativeText.GetLangStr(300027), NativeText.GetLangStr(300002))
        else
            title.text = NativeText.GetLangStr(300027)
        end

        local prefab = self.defense_node.gameObject
        local cnt = 1

        for k, v in ipairs(self.data.defTeams) do
            local elem = nil
            if k > #self.defend_ui then
                elem = Object.Instantiate(prefab).transform
                elem:SetParent(self.content)
                resetTransformZ(elem)

                local idx = self.defense_node:GetSiblingIndex()
                elem:SetSiblingIndex(idx + k - 1)

                table.insert( self.defend_ui, elem )
            else
                elem = self.defend_ui[k]
            end

            elem:SetActive(true)

            local name = elem:Find("Top/Name"):GetComponent(UnityEngine.UI.Text)
            local enemy_cnt = elem:Find("Top/Killed"):GetComponent(UnityEngine.UI.Text)

            enemy_cnt.text = string.format("%s: <color=#ffffffff>%d</color>", NativeText.GetLangStr(300035), v.killSoldiers)

            if v.objType == common_pb.WORLD_OBJ_MONSTER then
                local cfg = config.worldmap_monsters[v.cfgId]

                if cfg then
                    name.text = frame_helper:getLangCfg(cfg.monstername)
                end
            else
                name.text = v.strName
            end
        
            local bg = elem:Find("BG"):GetComponent(UnityEngine.UI.Image)
        
            if self.data.selfTeam == 1 then
                bg:SetSprite(res_path.res_path_102)
            else
                bg:SetSprite(res_path.res_path_101)
            end

            self:loadTeam(elem, v)

            cnt = cnt + 1
        end

        for i=cnt, #self.defend_ui do
            local idx = self.defense_node:GetSiblingIndex()
            idx = idx + i - 1
            local child = self.content:GetChild(idx)
            child:SetActive(false)
        end
    end,

    loadTeam = function(self, elem, info)
        local hero_panel = elem:Find("BG/Heros")
        local soldier_panel = elem:Find("BG/Soldiers")

        if #info.armyHeros > 0 then
            self:loadHeros(hero_panel, info.armyHeros)
        end

        local detail = soldier_panel:Find("Details")
        local bg = elem:Find("BG")

        self:loadSoldiers(detail, info.batSoldier)

        app:startCoroutine(
            function()
                coroutine.yield(nil)
                if #info.armyHeros == 0 then
                    bg.sizeDelta = Vector2(1364, detail.rect.height + 100)
                    hero_panel:SetActive(false)
                    soldier_panel.localPosition = Vector3(0, -70, 0)
                else
                    bg.sizeDelta = Vector2(1364, detail.rect.height + 210)
                    hero_panel:SetActive(true)
                    soldier_panel.localPosition = Vector3(0, -180, 0)
                end

                elem.sizeDelta = Vector2(1353, bg.rect.height + 50)
            end
        )
    end,

    loadHeros = function(self, node, heros) 
        local prefab = node:GetChild(0).gameObject

        for k, v in ipairs(heros) do
            local elem = nil
            if k > node.childCount then
                elem = Object.Instantiate(prefab).transform
                elem:SetParent(node)
                resetTransformZ(elem)
            else
                elem = node:GetChild(k-1)
            end

            elem:SetActive(true)

            local icon = elem:Find("Icon"):GetComponent(UnityEngine.UI.Image)
            local grade = elem:Find("Grade"):GetComponent(UnityEngine.UI.Image)

            local cfg = config.commander_basics[v.officeCfgId]

            if cfg then
                icon:SetSprite(cfg.path..cfg.icon)
                grade:SetSprite(gui_helper:getNewQualityIcon(cfg.color))
            else
                log.error("##################################### find commander res error ! id ", v.officeCfgId)
            end
        end

        for i=#heros, node.childCount - 1 do
            local child = node:GetChild(i)
            child:SetActive(false)
        end
    end,

    loadSoldiers = function(self, node, soldiers)
        local prefab = node:GetChild(0).gameObject

        for k, v in ipairs(soldiers) do
            local elem = nil
            if k > node.childCount then
                elem = Object.Instantiate(prefab).transform
                elem:SetParent(node)
                resetTransformZ(elem)
            else
                elem = node:GetChild(k-1)
            end

            elem:SetActive(true)

            local cfg = config.soldier_soldiers[v.cfgId]

            if cfg then
                local icon = elem:GetChild(0):GetComponent(UnityEngine.UI.Image)
                local text_1 = elem:GetChild(1):GetComponent(UnityEngine.UI.Text)
                local text_2 = elem:GetChild(2):GetComponent(UnityEngine.UI.Text)
                local text_3 = elem:GetChild(3):GetComponent(UnityEngine.UI.Text)
                local text_4 = elem:GetChild(4):GetComponent(UnityEngine.UI.Text)

                icon:SetSprite(cfg.iconpath..cfg.soldiericon)
                text_1.text = frame_helper:getLangCfg(cfg.soldiername)
                text_2.text = v.leftCount
                text_3.text = v.hurtCount
                text_4.text = v.dieCount
            end
        end

        for i=#soldiers, node.childCount - 1 do
            local child = node:GetChild(i)
            child:SetActive(false)
        end
    end,

    initTraps = function(self)
        if self.data.warType == common_pb.EWAR_TARGET_CITY then

            self.trap_node:SetActive(true)

            local enemy_cnt = self.trap_node:Find("Top/Killed"):GetComponent(UnityEngine.UI.Text)

            enemy_cnt.text = string.format("%s: <color=#ffffffff>%d</color>", NativeText.GetLangStr(300035), self.data.cannonTeams.killSoldiers)

            local detail = self.trap_node:Find("BG/Details")
            self:loadSoldiers(detail, self.data.cannonTeams.batSoldier)

            local bg = self.trap_node:Find("BG")

            app:startCoroutine(
                function()
                    coroutine.yield(nil)
                    bg.sizeDelta = Vector2(1364, detail.rect.height + 100)
                    self.trap_node.sizeDelta = Vector2(1353, bg.rect.height + 50)
                end
            )
        else
            self.trap_node:SetActive(false)
        end
    end,

    isMonster = function(self)
        if self.enemy_info.objType == common_pb.WORLD_OBJ_MONSTER or self.enemy_info.objType == common_pb.WORLD_OBJ_RES_SHIP then
            return true
        end
        
        if self.data.warType == common_pb.EWAR_TARGET_PLAYER or 
        self.data.warType == common_pb.EWAR_TARGET_CITY then
            return false
        end

        return true
    end,

    getName = function(self, info, is_own)
        local str = info.playerName

        if info.familyName and string.len(info.familyName) > 2 then
            str = string.format( "[%s]%s", info.familyName, str )
        end

        if not is_own and self:isMonster() then
            local cfg = config.worldmap_monsters[info.cfgId]
            if cfg then
                str = frame_helper:getLangCfg(cfg.monstername)
            end
        end

        return str
    end,

    show = function(self)
        self.root:SetActive(true)
    end,

    hide = function(self)
        self.root:SetActive(false)
    end,
}