local log = logger.get("Game", "Battle Tip Frame")
local super = require("ui.frame")

return class{

    super = super,

    onLoaded = function(self)
        super.onLoaded(self)

        self.battle_data = self.args.data

        clickEvents = 
        {
            "BattleTip",
        }

        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self.battle_tip = self.transform:Find("BattleTip")

        self:init()
    end,

    init = function(self)
        local left = self.battle_tip:Find("Left")
        local right = self.battle_tip:Find("Right")

        local setInfo = function(node, info)
            local name = node:Find("Name"):GetComponent(UnityEngine.UI.Text)
            name.text = info.playerName
        end

        local left_info = self.battle_data.batLeftInfo
        local right_info = self.battle_data.batRightInfo

        if self.battle_data.selfCamp ~= 0 then
            left_info = self.battle_data.batRightInfo
            right_info = self.battle_data.batLeftInfo
        end

        setInfo(left, left_info)

        if right_info.monsterGroupId ~= 0 then
            local cfg = config.monster_group_ids[right_info.monsterGroupId]
            if cfg then
                right_info = {}
                right_info.playerName = self.helper:getLangCfg(cfg.Group_Name_ID)
            else
                log.error("#### battle monster group cfg not find ! id", right_info.monsterGroupId)
            end
        end

        setInfo(right, right_info)

        local status = self.battle_tip:Find("Status"):GetComponent(UnityEngine.UI.Text)
        local result = self.battle_tip:Find("Result"):GetComponent(UnityEngine.UI.Text)

        if self.battle_data.actionType == 0 then
            status.text = NativeText.GetLangStr(300018)
        else
            status.text = NativeText.GetLangStr(300019)
        end

        local cfg = config.formation_result_forecasts[self.battle_data.result]
        if cfg then
            result.text = self.helper:getLangCfg(cfg.result_real)
        end

        app:startCoroutine(
            function()
                coroutine.yield(WaitForSeconds(5))
                local vsb = frame_manager:isDisplay("battle_tip")
                if vsb then
                    frame_manager:hide("battle_tip")
                end
            end
        )
    end,

    onUnloaded = function(self)
    end,

    click_BattleTip = function(self)
        frame_manager:show("battle_record", nil, self.battle_data)
        frame_manager:hide("battle_tip")
    end,
}