local log = logger.get("Game", "Blueprint Drops Frame")
local super = require("ui.frame")
local item_cls = require('RTS.bag_sys.item')

return class{

    super = super,

    onLoaded = function(self)
        super.onLoaded(self)

        self.blueprint_id = self.args.data.id

        clickEvents = {
            "Close",
            "BG/Center/RecoverBtn",
            "BG/Center/TypeBtn",
            "BG/Center/QualityBtn",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self.content = self.transform:Find("BG/Center/Scroll View/Viewport/Content")
        self.prefab = self.transform:Find("BG/Center/Scroll View/Viewport/Content/Item").gameObject

        self.btns = {}
        self.btns["color"] = self.transform:Find('BG/Center/QualityBtn')
        self.btns["type"] = self.transform:Find('BG/Center/TypeBtn')

        self.color_node = self.transform:Find("BG/Center/ToggleColor")
        self.type_node = self.transform:Find("BG/Center/ToggleType")

        self.color_toggles = {}
        self.type_toggles = {}

        self.color_toggles[1] = self.transform:Find("BG/Center/ToggleColor/White"):GetComponent(UnityEngine.UI.Toggle)
        self.color_toggles[2] = self.transform:Find("BG/Center/ToggleColor/Green"):GetComponent(UnityEngine.UI.Toggle)
        self.color_toggles[3] = self.transform:Find("BG/Center/ToggleColor/Blue"):GetComponent(UnityEngine.UI.Toggle)
        self.color_toggles[4] = self.transform:Find("BG/Center/ToggleColor/Purple"):GetComponent(UnityEngine.UI.Toggle)
        self.color_toggles[5] = self.transform:Find("BG/Center/ToggleColor/Orange"):GetComponent(UnityEngine.UI.Toggle)

        for k, v in ipairs(self.color_toggles) do
            v.isOn = true
            v.onValueChanged:AddListener(
                function()
                    if not self.reset then
                        self:itemFilter()
                    end
                end
            )
        end

        self:initTypeToggles()
        self:setDes()

        self.drop_items = gui_helper:getDropItems(self.blueprint_id, 3)
        self.show_items = self.drop_items

        self.filter = ""

        self:updateBtns("color")
    end,

    setDes = function(self)
        local cfg = config.home_config_produces[self.blueprint_id]

        if not cfg then
            log.error("#### BluePrint cfg error! id", id)
            return
        end

        local des = self.transform:Find("BG/Center/Des"):GetComponent(UnityEngine.UI.Text)
        des.text = string.format(NativeText.GetLangStr(320017), self.helper:getLangCfg(cfg.name))
    end,

    initTypeToggles = function(self)
        local node = self.transform:Find("BG/Center/ToggleType")
        local types = {}

        config.weapon_types:foreach(
            function(k, v)
                if v.family == 0 then
                    table.insert( types, v )
                end
            end
        )

        table.sort( types, function(a, b) return a.weapon_type < b.weapon_type end )

        for k, v in ipairs(types) do
            local ui = nil

            if k > node.childCount then
                ui = Object.Instantiate(node:GetChild(0)).transform
                ui:SetParent(node)
                resetTransformZ(ui)
            else
                ui = node:GetChild(k-1)
            end

            if ui then
                ui:SetActive(true)
                local toggle = ui:GetComponent(UnityEngine.UI.Toggle)
                toggle.isOn = true

                local text = ui:Find("Text"):GetComponent(UnityEngine.UI.Text)
                text.text = self.helper:getLangCfg(v.show)

                toggle.onValueChanged:AddListener(
                    function()
                        if not self.reset then
                            self:itemFilter()
                        end
                    end
                )
                self.type_toggles[v.weapon_type] = toggle
            end
        end

        for i=#types, node.childCount - 1 do
            local child = node:GetChild(i)
            child:SetActive(false)
        end
    end,

    resetToggles = function(self)
        for k, v in pairs(self.color_toggles) do
            v.isOn = true
        end

        for k, v in pairs(self.type_toggles) do
            v.isOn = true
        end

        self:itemFilter()
    end,

    updateBtns = function(self, filter)
        for k, v in pairs(self.btns) do
            local label = v:GetChild(0):GetComponent(UnityEngine.UI.Text)
            local outline = v:GetChild(0):GetComponent(UnityEngine.UI.Outline)

            local color = Color(1, 1, 1, 1)
            if k ~= filter then
                color = Color(160/255, 160/255, 160/255, 1)
            end

            label.color = color
            outline.enabled = k == filter
        end

        self:updateToggles(filter)
    end,

    updateToggles = function(self, filter)
        if self.filter == filter then
            return
        end

        if filter == "type" then
            self.type_node:SetActive(true)
            self.color_node:SetActive(false)
        elseif filter == "color" then
            self.type_node:SetActive(false)
            self.color_node:SetActive(true)
        end

        self.filter = filter

        self:itemFilter()
    end,

    itemFilter = function(self)
        self.show_items = {}

        if self.filter == "type" then
            for k, v in pairs(self.type_toggles) do
                if v.isOn then
                    self:getItemByType(k, self.show_items)
                end
            end
        elseif self.filter == "color" then
            for k, v in pairs(self.color_toggles) do
                if v.isOn then
                    self:getItemByColor(k, self.show_items)
                end
            end
        end

        table.sort( self.show_items, function(a, b) return a.color > b.color end )

        self:refresh()
    end,

    getItemByType = function(self, type, items)
        for k, v in ipairs(self.drop_items) do
            if v.weapon_type == type then
                table.insert(items, v)
            end
        end
    end,

    getItemByColor = function(self, color, items)
        for k, v in ipairs(self.drop_items) do
            if v.color == color then
                table.insert(items, v)
            end
        end
    end,

    refresh = function(self)
        for k, v in ipairs(self.show_items) do
            local ui = nil

            if k > self.content.childCount then
                ui = Object.Instantiate(self.prefab).transform
                ui:SetParent(self.content)
                resetTransformZ(ui)
            else
                ui = self.content:GetChild(k-1)
            end

            if ui then
                ui:SetActive(true)
                ui.name = "item_"..k

                local grade = ui:GetComponent(UnityEngine.UI.Image)
                local icon = ui:Find("Icon"):GetComponent(UnityEngine.UI.Image)

                grade:SetSprite(gui_helper:getQualityIconPath(v.color))
                icon:SetSprite(gui_helper:getItemIconPath(v))

                local path = "BG/Center/Scroll View/Viewport/Content/"..ui.name
                local item_inst = item_cls({cfgId = v.id, count = 1})
                self:clearScript("click", path)
                self:setScript("click", path, functor.rmake(self.onClickItem, item_inst))
            end
        end

        for i=#self.show_items, self.content.childCount - 1 do
            local child = self.content:GetChild(i)
            child:SetActive(false)
        end

        self.reset = false
    end,

    onClickItem = function(self, handler, item)
        frame_manager:show("itemInfo", self, {item = item})
    end,

    click_Close = function(self)
        frame_manager:hide("blueprint_drop")
    end,

    click_RecoverBtn = function(self)
        self.reset = true
        self:resetToggles()
    end,

    click_QualityBtn = function(self)
        self:updateBtns("color")
    end,

    click_TypeBtn = function(self)
        self:updateBtns("type")
    end,
}