local log = logger.get("Game", "Blue Print Select Frame")
local super = require("ui.frame")

return class{

    super = super,

    onLoaded = function(self)
        super.onLoaded(self)

        clickEvents = {
            "Close",
            "BG/HelpBtn",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        --self.content = self.transform:Find("BG/Scroll View/Viewport/Content")
        --self.prefab = self.transform:Find("BG/Scroll View/Viewport/Content/Elem").gameObject

        self.armory_lv = 1

        local entity = home_info:getBuildDataByGridId(11)
        if entity then
            self.armory_lv = entity.level
        end

        self.weapons = {}
        self.armours = {}

        config.home_config_produces:foreach(
            function(k, v)
                if v.type == 1 then
                    table.insert( self.weapons, v )
                elseif v.type == 2 then
                    table.insert( self.armours, v )
                end
            end
        )

        table.sort( self.weapons, function(a, b) return a.unlock < b.unlock end )
        table.sort( self.armours, function(a, b) return a.unlock < b.unlock end )

        self:init()

        self.evt = game_director.events:on("GETDRAWINGINFO", functor.make(self.onGetInfo, self))
    end,

    onUnload = function(self)
        game_director.events:remove("GETDRAWINGINFO", self.evt)
    end,

    onGetInfo = function(self, info)
        -- frame_manager:show("armory_frame", self, {info = info})
        if config.home_config_produces[info.drawingId].type == 1 then
            network_sender:buildingExpend(info.drawingId, common_pb.EPRODUCT_WEAPON)
        else
            network_sender:buildingExpend(info.drawingId, common_pb.EPRODUCT_ARMOUR)
        end
    end,

    init = function(self)
        local weapon_bar = self.transform:Find("BG/Weapon")
        local armour_bar = self.transform:Find("BG/Armour")

        local func = function(ui, cfg)
            local img = ui:GetComponent(UnityEngine.UI.Image)
            local btn = ui:GetComponent(UnityEngine.UI.Button)
            local tip = ui:Find("Tip"):GetComponent(UnityEngine.UI.Text)
            local lock_img = ui:Find("LockBG")
            local unlock_text = ui:Find("UnlockText"):GetComponent(UnityEngine.UI.Text)

            lock_img:SetActive(cfg[1].unlock > self.armory_lv)
            btn.interactable = not (cfg[1].unlock > self.armory_lv)

            local is_max  = true
            local cur_cfg = nil

            for k, v in ipairs(cfg) do

                if v.unlock <= self.armory_lv then
                    cur_cfg = v
                end

                if v.unlock > self.armory_lv then
                    --img:SetSprite(v.path..v.icon)
                    tip.text = string.format( NativeText.GetLangStr(320013), v.unlock )
                    unlock_text.text = self.helper:getLangCfg(v.unlockdes)

                    is_max = false

                    break
                end
            end
            
            self:clearScript("click", "BG/"..ui.name)
            self:setScript("click", "BG/"..ui.name, functor.rmake(self.onClickSelect, cur_cfg))

            tip.transform:SetActive(not is_max)
            unlock_text.transform:SetActive(not is_max)
        end

        func(weapon_bar, self.weapons)
        func(armour_bar, self.armours)
    end,

    onClickSelect = function(self, handler, cfg)
        if not cfg then
            log.error("#################### blue print cfg error !")
            return
        end

        network_sender:getDrawingInfo(cfg.type)
    end,

    click_HelpBtn = function(self)
        frame_manager:show('home_entity_info_detail', nil, home_info:getBuildDataByGridId(11).cfgId)
    end,

    click_Close = function(self)
        frame_manager:hide("blueprint_select")
    end,

    hide = function(self)
        frame_manager:hide("blueprint_select")
    end,
}