local super = require("ui.frame")
local log = logger.get("Game", "card speed")

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        -- 道具类型 42 通用
        -- 道具类型 47 车体改造

        self.heroId = self.args.data.heroId

        clickEvents = 
        {
            'close',
            'content/head/cost/completeBtn',
        }

        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self.slider = self.transform:Find('content/head/Slider'):GetComponent(UnityEngine.UI.Slider)
        self.time_text  = self:findText('content/head/time_text')
        self.cost_text  = self:findText('content/head/cost/cost_text')

        network_sender:getCarBodyInfo(self.heroId)
        self.callback = hero_manager.events:on("cardSpeedUp", functor.make(self.init, self))
        -- self.itemCallBack = hero_manager.events:on("cardItemSpeedUp", functor.make(self.init, self))
    end,

    onUnload = function (self)
        hero_manager.events:off("cardSpeedUp", self.callback)
        -- hero_manager.events:off("cardItemSpeedUp", self.itemCallBack)
    end,

    click_close = function(self)
        frame_manager:hide('card_speed_up')
    end,

    -- 立即完成
    click_completeBtn = function(self)
        local temporaryList = {}
        temporaryList.item_id = 0
        temporaryList.count = 0
        network_sender:speedUpRefit(self.heroId, temporaryList, 1)
    end,

    update = function(self, dt)
        if self.count_time and not self.closed then
            if self.count_time > 0 then
                local allTime = self.cfg.time * 60
                self.count_time = self.count_time - dt
                self.slider.value = (allTime - self.count_time) / allTime
                self.cost_text.text = self:getCostDiamond(self.count_time)
                self.time_text.text = string.format("%s%s%s", NativeText.GetLangStr(260004), gui_helper:timeFormat(self.count_time), NativeText.GetLangStr(260014))
            else
                self.count_time = 0
                self.slider.value = 1
                self.closed = true
                frame_manager:hide('card_speed_up')
                frame_manager:hide('use_slider')
            end
        end
    end,

    init = function(self)
        frame_manager:hide('use_slider')

        self.heroInfo = hero_manager.heros[self.heroId]
        
        self.modifyList = {}
        config.tank_basic_modifycosts:foreach(function (k, v)
            if not self.modifyList[tonumber(v.color)] then
                self.modifyList[v.color] = {}
            end
            self.modifyList[v.color][tonumber(v.level)] = v
        end)   

        local tankLevel     = tonumber(self.heroInfo.tankBody_info.carBodyLv)
        self.count_time     = tonumber(self.heroInfo.tankBody_info.levelUpTime)
        local tankQuality   = tonumber(self.heroInfo.tank_cfg.color)
        local temporaryList = self.modifyList[tankQuality]
        self.cfg = temporaryList[tankLevel]

        local title_text = self:findText('content/top/text1_Static[T2]_26')
        local name_text  = self:findText('content/head/name_text')
        
        title_text.text = NativeText.GetLangStr(260001)
        name_text.text  = self.helper:getLangCfg(self.heroInfo.tank_cfg.name)
        
        self:showScrollView()
    end,

    getCostDiamond = function(self, time)
        return gui_helper:getSpeedUpCost(47, time)
    end,

    showScrollView = function(self)
        --local common_type = 42
        local target_type = 47

        local rst = {}
        config.item_resources:foreach(
            function(id, cfg)
                --[[ if cfg.type == common_type then
                    local count = bag_manager:getItemNum(id)
                    if count > 0 then
                        table.insert(rst, cfg)
                    end
                else ]]
                if cfg.type == target_type then
                    table.insert(rst, cfg)
                end
            end
        )

        table.sort(rst, gui_helper.sortItem)

        local parent_path = 'content/Scroll View/Viewport/Content'
        self.helper:hideAllChild(self, parent_path)

        for idx, cfg in ipairs(rst) do
            local child = self.helper:getChild(self, idx, parent_path)
            child:SetActive(true)

            local icon = child:Find('icon'):GetComponent(UnityEngine.UI.Image)
            local name_text = child:Find('name_text'):GetComponent(UnityEngine.UI.Text)
            local des_text  = child:Find('des_text'):GetComponent(UnityEngine.UI.Text)

            local use_trans = child:Find('useTrans')
            local buy_trans = child:Find('buyTrans')

            icon:SetSprite(self.helper:getItemIcon(cfg.id))
            name_text.text = self.helper:getLangCfg(cfg.name_id)
            des_text.text  = self.helper:getLangCfg(cfg.des_id)

            local own_num = self.helper:getItemNum(cfg.id)

            use_trans:SetActive(own_num > 0)
            buy_trans:SetActive(own_num <= 0)
            if own_num > 0 then
                local own_num_text = child:Find('useTrans/own_text'):GetComponent(UnityEngine.UI.Text)
                own_num_text.text  = string.format(NativeText.GetLangStr(260012), own_num) 

                local btn = child:Find('useTrans/useBtn'):GetComponent(UnityEngine.UI.Button)
                btn.onClick:RemoveAllListeners()
                btn.onClick:AddListener(
                    function ()
                        print('使用')
                        local rcmd_cost_num = self:getCostItemNum(cfg.id)  -- 推荐的消耗数量
                        if own_num ~= 1 then
                            frame_manager:show('use_slider', nil, {
                                item_id = cfg.id,
                                item_num = rcmd_cost_num,
                                callback = function(item_id, num)
                                    -- local temporaryList = {}
                                    -- temporaryList.item_id = item_id
                                    -- temporaryList.count = num
                                    --network_sender:speedUpRefit(self.heroId, temporaryList, 0)
                                    self:detect(item_id, num, rcmd_cost_num)
                                end,
                            })
                        else
                            self:detect(cfg.id, own_num, rcmd_cost_num)
                        end
                    end
                )
            else
                local cost_icon = child:Find('buyTrans/cost/Image'):GetComponent(UnityEngine.UI.Image)
                local cost_text = child:Find('buyTrans/cost/cost_text'):GetComponent(UnityEngine.UI.Text)
                
                cost_icon:SetSprite(self.helper:getItemIcon(30001))
                cost_text.text = frame_helper:getDiamondPriceFromStr(cfg.allprice)

                local btn = child:Find('buyTrans/buyBtn'):GetComponent(UnityEngine.UI.Button)
                btn.onClick:RemoveAllListeners()
                btn.onClick:AddListener(
                    function ()
                        print('购买')
                        frame_manager:show('buy_slider', nil, 
                        {
                            item_id = cfg.id
                        })
                    end
                )
            end
        end
    end,

    getCostItemNum = function(self, item_id)
        local left_time = self.count_time
        local free_time = 0
        local minute = (left_time - free_time) / 60
        local item_cost_minute = tonumber(config.item_resources[item_id].para1)
        local count = math.floor(minute / item_cost_minute)
        return count
    end,

    detect = function(self, item_id, actual_cost_num, rcmd_cost_num)
        local sendMsg = function()
            local temporaryList = {}
            temporaryList.item_id = item_id
            temporaryList.count = actual_cost_num
            network_sender:speedUpRefit(self.heroId, temporaryList, 0)
            frame_manager:hide('use_slider')
        end

        if actual_cost_num > rcmd_cost_num then
            local tb = {
                title = "",
                content = NativeText.GetLangStr(260018),
                buttons =
                {
                    {
                        btn_name = NativeText.GetLangStr(130003),
                        callback = function() 
                            frame_manager:hide('common_box')
                        end
                    },
                    {
                        btn_name = NativeText.GetLangStr(130002),
                        callback = function() 
                            frame_manager:hide('common_box')
                            sendMsg()
                        end
                    },
                },
            }
    
            frame_manager:show('common_box', nil, tb)
        else
            sendMsg()
        end
    end,
}
