local super = require("ui.frame")
local log = logger.get("Game", "change equipment Frame")
local gui_helper = require("gui.gui_helper")
local bag_manager = require("RTS.bag_sys.bag_manager")

local filters = {
    level = 1,
    qulity = 2,
    upgrade = 3,
 }

 local slot_family = {
     [common_pb.MAIN_WEAPON_SLOT] = 1,
     [common_pb.MINOR_WEAPON_SLOT_1] = 2,
     [common_pb.MINOR_WEAPON_SLOT_2] = 2,
     [common_pb.MINOR_WEAPON_SLOT_3] = 2,
     [common_pb.MINOR_WEAPON_SLOT_4] = 2,
     [common_pb.DECORATION_SLOT] = 3,
     [common_pb.WHEEL_SLOT] = 4,
     [common_pb.OTHER_SLOT_1] = 5,
     [common_pb.FASHION_SLOT] = 6,
}

return class
{
    super = super,
    
    onLoaded = function (self)
        super.onLoaded(self)

        self.owner = self.args.owner
        self.hero_id = self.args.data.hero_id
        self.slot_id = self.args.data.slot_id

        -- 路径: root/login
        -- 事件: click_login 
        clickEvents = {
            "BG/Top/BackButton",
            "BG/Top/HomeBtn",
            "DetailInfo/Bottom/CancelBtn",
            "DetailInfo/Bottom/ChangeBtn",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self.prefab_root = self.transform:Find('Prefabs')
        self.equip_prefab = self.transform:Find('Prefabs/Equip').gameObject

        self.helper.hideTrans(self.owner.transform)
        self.helper.hideTrans(self.prefab_root)

        self.scroll_content = self.transform:Find('BG/Right/Scroll View/Viewport/Content')
        self.detail_panel = self.transform:Find('DetailInfo')
        self.detail_left = self.transform:Find('DetailInfo/Left')
        self.detail_center = self.transform:Find('DetailInfo/Center')
        self.detail_right = self.transform:Find('DetailInfo/Right')

        --self.btn_layout:SetActive(false)
        self:hideDetailPanel()

        local equipment_info = self.transform:Find('DetailInfo/Center/Equipment')
        self.bag_equipment_info = require('gui.equipment_info_panel')(equipment_info, self)
        equipment_info = self.transform:Find('DetailInfo/Left/Equipment')
        self.hero_equipment_info = require('gui.equipment_info_panel')(equipment_info, self)
        local hero_weapons_panel = self.transform:Find('BG/Left/Weapons')
        self.hero_weapons = require('gui.hero_weapons_panel')(hero_weapons_panel, self)

        local path = "BG/Left/Weapons/"
        self.hero_weapons:show(self.hero_id, path)

        self.slots = self.hero_weapons.slots

        self.equipment = self.slots[self.slot_id].item
        local slot_type = self.slot_id
        self.equipment_family = slot_family[slot_type]
        self.selected_bag_equipment = nil
        self.bag_equipments = {}

        self.equip_evts = {}
        self.selectChanged = true
        self.isBagChanged = false

        self:updateProps()

        self.evts = {}
        self.evts["onBagChanged"] = bag_manager.events:on("onBagChanged", functor.make(self.onBagChanged, self))

    end,

    onUnload = function (self)
        for k, v in pairs(self.evts) do
            bag_manager.events:remove(k, v)
        end
    end,

    onRcvMsg = function(self, cmd, msg)
    end,

    onBagChanged = function(self)
        log.info('###onBagChanged')
        local bag = bag_manager:getHeroBag(self.hero_id)
        if not bag then
            log.error("###### load weapon hero bag error! ", self.hero_id)
            return
        end

        local equip = bag:get(self.slot_id)
        self.equipment = equip

        self.isBagChanged = true
    end,

    update = function(self, dt)
        if self.selectChanged then
            self:refreshHighLight()
            self:equipmentFilter(self.filter)
            self.selectChanged = false
        end

        if self.isBagChanged then
            self:refreshLeftPanel()
            self:equipmentFilter(self.filter)
            self.isBagChanged = false
        end
    end,

    clearEquipEvts = function(self)
        for path, evt in pairs(self.equip_evts) do
            self:clearScript(evt, path)
        end
        self.equip_evts = {}
    end,

    refreshLeftPanel = function(self)
        self.hero_weapons:show(self.hero_id, "BG/Left/Weapons/")
        self:updateProps()
    end,

    refreshHighLight = function(self)
        self.hero_weapons:refreshHighLight(self.slot_id)
    end,

    updateProps = function(self)
        local hero = hero_manager:getHero(self.hero_id)

        if not hero then
            log.error("#### update props error !", self.hero_id)
            return
        end

        local prop_node = self.transform:Find("BG/Left/Props")
        local props = hero:getHeroShowedProps()
        gui_helper:showUpgradeProps(prop_node, props)
    end,

    updateRightPanel = function(self)
        local items = self:getItemsByFamily(self.equipment_family)

        local hero = hero_manager:getHero(self.hero_id)

        if not hero then
            log.error("#### update right panel !", self.hero_id)
            return
        end

        local hero_type = hero.tank_cfg.type

        local equipments = {}
        self.equipments = equipments    -- 缓存当前界面显示的装备
        -- 筛选和战车匹配的武器
        for k, v in ipairs(items) do
            if v.hero_id ~= self.hero_id then 
                if v.cfg.weapon_type == 0 then
                    table.insert( equipments, v )
                else
                    if "type"..v.cfg.weapon_type == hero_type then
                        table.insert( equipments, v )
                    end
                end
            end
        end

        self:clearEquipEvts()

        local icon_root = nil
        local path_root = "BG/Right/Scroll View/Viewport/Content/"
        local index = 0

        if self.equipment ~= nil then
            if self.scroll_content.childCount < 1 then
                local slot = Object.Instantiate(self.equip_prefab)
                slot.transform:SetParent(self.scroll_content)
                resetTransformZ(slot.transform)
            end
            icon_root = self.scroll_content:GetChild(0)
            icon_root:Find('Unload'):SetActive(self.equipment ~= nil)
            icon_root:Find('Grade'):SetActive(self.equipment == nil)
            icon_root.name = "slot_"..index
            local path = path_root..icon_root.name.."/Unload"
            self:setScript("single", path, functor.rmake(self.onClick_unload, self.equipment.slot))
            self.equip_evts[path] = "single"
            icon_root:SetActive(true)
            index = 1
        end

        for k, v in ipairs(equipments) do
            if index >= self.scroll_content.childCount then
                local slot = Object.Instantiate(self.equip_prefab)
                slot.transform:SetParent(self.scroll_content)
                resetTransformZ(slot.transform)
            end
            icon_root = self.scroll_content:GetChild(index)
            icon_root:Find('Unload'):SetActive(false)
            icon_root:Find('Grade'):SetActive(true)
            local icon_path = gui_helper:getItemIconPath(v.cfg)
            local icon = icon_root:Find('Grade/Icon')
            icon:SetSprite(icon_path)
            icon:SetActive(true)
            local grade = icon_root:Find('Grade')
            gui_helper:setItemQualityIcon(grade, v.cfg.color)
            v.ui_item = icon_root   -- 缓存 ui 对象
            icon_root.name = "slot_"..index
            local path = path_root..icon_root.name.."/Grade"
            self:setScript("single", path, functor.rmake(self.onClick_bagEquip, v))
            self.equip_evts[path] = "single"
            icon_root:SetActive(true)

            -- 设置锁定状态
            local lock_text = icon_root:Find('Grade/LockText')
            if lock_text then
                lock_text:SetActive(v:isLocked())
            end

            local equiped = icon_root:Find("Grade/Equiped")
            equiped:SetActive(v.hero_id ~= nil)

            -- 设置强化等级
            local level = icon_root:Find('Grade/Level')
            if v.level and v.level > 0 then
                level:SetActive(true)
                local text = level:GetComponent(UnityEngine.UI.Text)
                text.text = "Lv."..v.level
            else
                level:SetActive(false)
            end

            index = index + 1
        end

        for i = index, self.scroll_content.childCount -1 do
            icon_root = self.scroll_content:GetChild(i)
            --local icon = icon_root:Find('Grade/Icon')
            icon_root:SetActive(false)
        end
    end,

    updateDetailInfo = function(self, equipment)
        if self.equipment ~= nil then
            self.bag_equipment_info.panel:SetParent(self.detail_right)
            self.hero_equipment_info:show(self.equipment, self.slot_id)
        else
            self.bag_equipment_info.panel:SetParent(self.detail_center)
        end
        self.bag_equipment_info.panel:GetComponent(UnityEngine.RectTransform).anchoredPosition = Vector2.zero
        self.helper.showTrans(self.detail_panel)
        self.detail_left:SetActive(self.equipment ~= nil)
        self.detail_right:SetActive(self.equipment ~= nil)
        self.detail_center:SetActive(self.equipment == nil)
        self.bag_equipment_info:show(equipment, self.slot_id, self.equipment)
    end,

    equipmentFilter = function(self, filter)
        self.filter = filter
        local bag = bag_manager:getBag(1)
        self.bag_equipments = bag:getItems(8)
        local hero_equips = bag_manager:getAllHeroBagItem()
        for k, v in ipairs(hero_equips) do
            table.insert( self.bag_equipments, v )
        end

        if filter == filters.level then
            table.sort( self.bag_equipments, self.sortByUseLevel )
        elseif filter == filters.qulity then
            table.sort( self.bag_equipments, self.sortByQuality )
        elseif filter == filters.upgrade then
            table.sort( self.bag_equipments, self.sortByEnhanceLevel )
        end
        self:updateRightPanel()
    end,

    sortByUseLevel = function(a, b)
        if a.cfg and b.cfg and a.cfg.uselevel and b.cfg.uselevel then
            if a.cfg.uselevel == b.cfg.uselevel then
                return false
            else
                return a.cfg.uselevel > b.cfg.uselevel
            end
        else
            return false
        end
    end,

    sortByQuality = function(a, b)
        if a.cfg and b.cfg and a.cfg.color and b.cfg.color then
            if a.cfg.color == b.cfg.color then
                return false
            else
                return a.cfg.color > b.cfg.color
            end
        else
            return false
        end
    end,

    sortByEnhanceLevel = function(a, b)
        if a.level and b.level then
            if a.level == b.level then
                return false
            else
                return a.level > b.level
            end
        else
            return false
        end
    end,

    updateDropInfo = function(self, index)
        local temp = self.drop_down[index]
        self.drop_down[index] = self.drop_down[1]
        self.drop_down[1] = temp
        self.btns[1]:GetComponent(UnityEngine.UI.Text).text = self.drop_down[1].str
        self.btns[index]:GetComponent(UnityEngine.UI.Text).text = self.drop_down[index].str
    end,

    hideDetailPanel = function(self)
        self.helper.hideTrans(self.detail_panel)
        self.detail_left:SetActive(false)
        self.detail_right:SetActive(false)
        self.detail_center:SetActive(false)
    end,

    click_BackButton = function(self)
        frame_manager:hide('changeEquipment')
        self.helper.showTrans(self.owner.transform)
    end,

    click_HomeBtn = function(self)
        frame_manager:clickHome()
    end,

    click_CancelBtn = function(self)
        self:hideDetailPanel()
    end,

    --点击更换按钮
    click_ChangeBtn = function(self)
        local cb = function()
            self:hideDetailPanel()
            network_sender:heroEquip(self.hero_id, self.selected_bag_equipment.inst_id, self.slot_id, protocol_pb.IU_EQUIP)
        end
        local tb = {
            title = "",
            content = NativeText.GetLangStr(110531),
            buttons =
            {
                {
                    btn_name = NativeText.GetLangStr(110522),
                    callback = function() 
                        frame_manager:hide('common_box')
                    end
                },
                {
                    btn_name = NativeText.GetLangStr(110521),
                    callback = function()
                        cb()
                        frame_manager:hide('common_box')
                    end
                },
            },
        }
        if self.selected_bag_equipment.hero_id and self.hero_id ~= self.selected_bag_equipment.hero_id then
            frame_manager:show('common_box', nil, tb)
        else
            cb()
        end
    end,
    
    --点击战车身上的装备
    onClick_weapon = function(self, handler, data, slot)
        self.selectChanged = (self.slot_id ~= slot.id)
        self.slot_id = slot.id
        self.equipment = slot.item
        self.equipment_family = slot_family[slot.id]
    end,

    --点击空的装备槽位
    onClick_slot = function(self, handler, data, slot)
        self.selectChanged = (self.slot_id ~= slot.id)
        self.slot_id = slot.id
        self.equipment = nil
        self.equipment_family = slot_family[slot.id]
    end,

    --点击卸下按钮
    onClick_unload = function(self, handler, data, slot_id)
        network_sender:heroEquip(self.hero_id, self.equipment.inst_id, slot_id, protocol_pb.IU_UNEQUIP)
    end,

    --点击背包里的装备
    onClick_bagEquip = function(self, handler, data, equipment)
        self.selected_bag_equipment = equipment
        self:updateDetailInfo(equipment)
    end,

    --根据武器family获取武器
    getItemsByFamily = function(self, family)
        local items = {}
        for _,item in ipairs(self.bag_equipments) do
            local equip_family = config.weapon_weapons[item.cfg_id].family
            if item.cfg.type == 8 and family == equip_family then
                table.insert(items, item)
            end
        end
        return items
    end,

    getHeroId = function(self)
        return self.hero_id
    end,

}