local log = logger.get("Game", "Chapter Achievement Cutscene Frame")
local super = require("ui.frame")

return class{
    super = super,

    onLoaded = function(self)
        super.onLoaded(self)
        self.helper = require("gui.tool.frame_helper")

        self.text_trans = self.transform:Find("Text")
        self.text = self.text_trans:GetComponent(UnityEngine.UI.Text)

        local chapter_id = self.args.data
        log.info("data:"..tostring(self.args.data))

        self.achievement_cfg = config.chapter_achievement_chapters[chapter_id]

        self.prefab_path = self.achievement_cfg.cinematics_res

        local raw_tab = loadstring('return '.. self.achievement_cfg.cinematics_subtitle)()
        self.subtitles = {}
        for k,v in ipairs(raw_tab) do
            table.insert(self.subtitles, frame_helper:getLangCfg(v) or "none")
        end

        -- 加载对应 prefab
        app:startCoroutine(
            function()
                coroutine.yield(WaitForInstantiate(self.prefab_path, self.transform, function(go)
                    go.transform:SetAsFirstSibling()
                end))
            end)
        
        self.elapsed = 0.0
        self.subtitle_idx = 0

        self:showNextSubtitleOrExit()
    end,

    showNextSubtitleOrExit = function(self)
        self.elapsed = 0.0
        self.subtitle_idx = self.subtitle_idx + 1
        self.text.color = Color(1,1,1,0)

        local subtitle = self.subtitles[self.subtitle_idx]
        if subtitle then
            self.text.text = subtitle
            local lt = LeanTween.textColor(self.text_trans, Color.white, 0.3)
            lt.loopType =  LeanTweenType.once

        else
            frame_manager:hide('chapter_achievement_cutscene')
        end
    end,

    update = function(self, dt)
        if self.elapsed then
            self.elapsed = self.elapsed + dt
            if self.elapsed >= 5.0 then
                self:showNextSubtitleOrExit()
            end
        end
    end,

    onUnload = function(self)
        chapter_achievement_manager:checkChapterStart()
    end,
}