local log = logger.get("Game", "Chapter Achievement Frame")
local super = require("ui.frame")

return class{
    super = super,

    onLoaded = function(self)
        super.onLoaded(self)
        chapter_achievement_frame = self

        clickEvents = {
            "content/bg/top/ButtonBack",
           -- "content/bg/top/ButtonHome",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self.text_title = self.transform:Find("content/text1_Static[T4]_7"):GetComponent(UnityEngine.UI.Text)
        self.text_desc = self.transform:Find("content/TextChapterDesc"):GetComponent(UnityEngine.UI.Text)
        self.achievement_root = self.transform:Find("content/layout")
        self.achievement_prefab = self.achievement_root:GetChild(0).gameObject

        if chapter_achievement_manager.chapter_id >= 0 then
            local id = chapter_achievement_manager.chapter_id
            local achievement_list = chapter_achievement_manager:getChapterAchievementList()
            self:refresh(id, achievement_list)
        end
    end,

    onUnload = function(self)
        log.info("###### chapter achievement frame Unload!!")
        chapter_achievement_frame = nil
    end,

    -- 刷新界面
    refresh = function(self, chapter_id, achievement_list)
        local chapter_cfg = config.chapter_achievement_chapters[chapter_id]
        self.text_title.text = frame_helper:getLangCfg(chapter_cfg.chapter_name)
        self.text_desc.text = frame_helper:getLangCfg(chapter_cfg.chapter_des)

        local parent_path = "content/layout/"
        self.helper:hideAllChild(self, parent_path, true)

        local temp_idx = 1
        for k,v in ipairs(achievement_list) do
            if v.is_final == 0 then
                self:refreshOneAchievement(temp_idx, v)
                temp_idx = temp_idx + 1
            else
                self:selfFinalInfo(v)
                self:setCoolBoxState(v.state)
            end
        end
    end,
    
    selfFinalInfo = function(self, info)
        local up_get_trans = self.transform:Find("content/finalBtn")
        local down_get_trans = self.transform:Find("content/Task_panel_9_2/getFinalRwdBtn")
        local go_trans = self.transform:Find("content/Task_panel_9_2/ButtonGo")
        local rcvd_trans = self.transform:Find("content/Task_panel_9_2/rcvd")

        self.final_box_state = info.state
        if info.state == common_pb.ACHIEVEMENT_NONE then      -- 未解锁
            up_get_trans:SetActive(true)
            down_get_trans:SetActive(false)
            go_trans:SetActive(true)
            rcvd_trans:SetActive(false)

            local btn = go_trans:GetComponent(UnityEngine.UI.Button)
            btn.interactable = false

            local btn = up_get_trans:GetComponent(UnityEngine.UI.Button)
            btn.interactable = true
            btn.onClick:RemoveAllListeners()
                btn.onClick:AddListener(
                    function ()
                        self:showFinalRwd(info)
                    end)

        elseif info.state == common_pb.ACHIEVEMENT_UNLOCK then -- 已解锁
            up_get_trans:SetActive(true)
            down_get_trans:SetActive(false)
            go_trans:SetActive(true)
            rcvd_trans:SetActive(false)

            local btn = go_trans:GetComponent(UnityEngine.UI.Button)
            btn.interactable = true
            btn.onClick:RemoveAllListeners()
                btn.onClick:AddListener(
                    function ()
                        self:onClickAchievementGo(info.id)
                    end)

            local btn = up_get_trans:GetComponent(UnityEngine.UI.Button)
            btn.interactable = true
            btn.onClick:RemoveAllListeners()
                btn.onClick:AddListener(
                    function ()
                        self:showFinalRwd(info)
                    end)

        elseif info.state == common_pb.ACHIEVEMENT_FINISH then -- 完成未领取
            up_get_trans:SetActive(true)
            down_get_trans:SetActive(true)
            go_trans:SetActive(false)
            rcvd_trans:SetActive(false)

            local btn = up_get_trans:GetComponent(UnityEngine.UI.Button)
            btn.interactable = true
            btn.onClick:RemoveAllListeners()
                btn.onClick:AddListener(
                    function ()
                        self:getFinalRwd(info.id)
                    end)


            local btn = down_get_trans:GetComponent(UnityEngine.UI.Button)
            btn.interactable = true
            btn.onClick:RemoveAllListeners()
                btn.onClick:AddListener(
                    function ()
                        self:getFinalRwd(info.id)
                    end)

        elseif info.state == common_pb.ACHIEVEMENT_GETAWARD then -- 已领取
            up_get_trans:SetActive(false)
            down_get_trans:SetActive(false)
            go_trans:SetActive(false)
            rcvd_trans:SetActive(true)
        end

        local text = self:findText("content/Task_panel_9_2/text1_Static[T5]_1")
        local tag = self:findImage("content/Task_panel_9_2/Task_panel_4_2/Icon (1)")
        text.text = frame_helper:getLangCfg(info.desc)
        tag.enabled = info.state == common_pb.ACHIEVEMENT_FINISH or info.state == common_pb.ACHIEVEMENT_GETAWARD
    end,

    showFinalRwd = function(self, info)
        local state = self.final_box_state
        if state == common_pb.ACHIEVEMENT_NONE or state == common_pb.ACHIEVEMENT_UNLOCK then      -- 未解锁
            local item_tab = {}
            for id,num in pairs(info.reward) do
                local item = {
                    cfg_id = id,
                    cfg = item_helper:getItemCfg(id),
                    count = num,
                }
                table.insert(item_tab, item)
            end
            frame_manager:show('getItems', self, {items = item_tab, has_Confirm_Btn = true, title = NativeText.GetLangStr(900004)})
        end
    end,

    -- 刷新一条章节目标, idx - 1 下标
    refreshOneAchievement = function(self, idx, info)

        log.info("### refreshOneAchievement:"..idx)

        -- local child = self.helper:getChild(self, idx, parent_path)
        -- child:SetActive(true)
        local parent_path = "content/layout/"
        local entry_trans = self.helper:getChild(self, idx, parent_path)
        entry_trans:SetActive(true)

        --[[
        if info.is_final > 0 then
            entry_trans = self.transform:Find("content/layout/Quest")
        else
            if self.achievement_root.childCount >= idx then
                entry_trans = self.achievement_root:GetChild(idx-1)
            else
                local go = GameObject.Instantiate(self.achievement_prefab, self.achievement_root)
                go.name = self.achievement_prefab.name ..idx
                entry_trans = go.transform
            end
        end
        ]]
        -- print("refresh:", idx, info.state)

        local state_icon = entry_trans:Find("State/Icon") 
        local text_desc = entry_trans:Find("Desc"):GetComponent(UnityEngine.UI.Text)
        local text_progress = entry_trans:Find("Progress"):GetComponent(UnityEngine.UI.Text)
        local award1 = entry_trans:Find("Award1")
        local award2 = entry_trans:Find("Award2")
        local button_go = entry_trans:Find("ButtonGo")  -- 任务进行链接
        local button_claim = entry_trans:Find("ButtonClaim")  -- 领取奖励
        local text_claimed = entry_trans:Find("TextClaimed")  -- 奖励已领取
        local text_unlock = nil
        if info.is_final then
            text_unlock = entry_trans:Find("TextUnlock")
        end

        text_desc.text = frame_helper:getLangCfg(info.desc)
        text_progress.text = info.current_value.."/"..info.max_value

        -- 奖励
        local reward_idx = 1
        award2.gameObject:SetActive(false)

        -- 对奖励排序
        local sorted_reward = {}
        for k,v in pairs(info.reward) do
            table.insert(sorted_reward, {id = k, num = v})
        end
        table.sort(sorted_reward, function(a, b)
            return a.id > b.id
        end)

        for _,reward in ipairs(sorted_reward) do
            local k = reward.id
            local v = reward.num

            local trans = nil
            if reward_idx == 1 then
                trans = award1
            else
                trans = award2
            end
            trans:SetActive(true)
            -- k-id  v-num
            local img_icon = trans:Find("Icon"):GetComponent(UnityEngine.UI.Image)
            local text_num = trans:Find("Num"):GetComponent(UnityEngine.UI.Text)
            local item_cfg = item_helper:getItemCfg(k)
            if not item_cfg then
                log.error("Failed to find item cfg:"..k)
            end

            img_icon:SetSprite(gui_helper:getItemIconPath(item_cfg))
            text_num.text = frame_helper:itemNumFormat(v)
            reward_idx = reward_idx + 1
        end
        
        if info.state == common_pb.ACHIEVEMENT_NONE then      -- 未解锁
            state_icon:SetActive(false)
            text_progress.gameObject:SetActive(false)
            button_go.gameObject:SetActive(false)
            button_claim.gameObject:SetActive(false)
            text_claimed.gameObject:SetActive(false)
            award1.gameObject:SetActive(false)
            award2.gameObject:SetActive(false)
            if text_unlock then
                text_unlock.gameObject:SetActive(true)
            end

        elseif info.state == common_pb.ACHIEVEMENT_UNLOCK then -- 已解锁
            state_icon:SetActive(false)
            text_progress.gameObject:SetActive(true)
            button_go.gameObject:SetActive(true)
            button_claim.gameObject:SetActive(false)
            text_claimed.gameObject:SetActive(false)
            if text_unlock then
                text_unlock.gameObject:SetActive(false)
            end

            -- 注册按钮事件
            local button_path = "content/layout/"..entry_trans.name.."/ButtonGo"
            self:UnregisterClick(button_path)
            self:RegisterClick(button_path, function()
                self:onClickAchievementGo(info.id)
            end)

            --[[
            local btn = button_go:GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(
                function()
                    self:onClickAchievementGo(info.id)
                end
            )]]

        elseif info.state == common_pb.ACHIEVEMENT_FINISH then -- 完成未领取
            state_icon:SetActive(true)
            text_progress.gameObject:SetActive(true)
            button_go.gameObject:SetActive(false)
            button_claim.gameObject:SetActive(true)
            text_claimed.gameObject:SetActive(false)
            if text_unlock then
                text_unlock.gameObject:SetActive(false)
            end

            -- 注册按钮事件
            local button_path = "content/layout/"..entry_trans.name.."/ButtonClaim"
            self:UnregisterClick(button_path)
            self:RegisterClick(button_path, function()
                self:onClickAchievementClaim(info.id)
            end)
            --[[
            local btn = button_claim:GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(
                function()
                    self:onClickAchievementClaim(info.id)
                end
            )]]
        elseif info.state == common_pb.ACHIEVEMENT_GETAWARD then -- 已领取
            state_icon:SetActive(true)
            text_progress.gameObject:SetActive(true)
            button_go.gameObject:SetActive(false)
            button_claim.gameObject:SetActive(false)
            text_claimed.gameObject:SetActive(true)
            if text_unlock then
                text_unlock.gameObject:SetActive(false)
            end
        end

    end,

    click_ButtonBack = function(self)
        log.info("Back!")
        frame_manager:hide("chapter_achievement")
    end,

    click_ButtonHome = function(self)
        log.info("Home!")
        frame_manager:hide("chapter_achievement")       
    end,

    -- 点了 go
    onClickAchievementGo = function(self, achievement_id)
        local go_callback = chapter_achievement_manager.go_callback[achievement_id]
        if go_callback then
            go_callback()
        else
            log.error("Failed to find go callback:"..achievement_id)
        end
    end,

    -- 点了 claim
    onClickAchievementClaim = function(self, achievement_id)
        -- 发消息请求领奖励
        network_sender:reqChapterAchievementReward(achievement_id)
    end,

    getFinalRwd = function(self, info_id)
        app:startCoroutine(function()
            self:setCoolBoxState()
            coroutine.yield(WaitForSeconds(0.5))
            self:onClickAchievementClaim(info_id)
        end)
    end,

    setCoolBoxState = function(self, state)

        local idle_trans = self.transform:Find("content/bx/idle")
       -- local can_rcv_trans = self.transform:Find("content/bx/get")
        local rcv_trans = self.transform:Find("content/bx/boom")
        local die_trans = self.transform:Find("content/bx/die")
        local skeleton_trans = self.transform:Find("content/bx/SkeletonGraphic (bx)")
        local skeleton = skeleton_trans:GetComponent("Spine.Unity.SkeletonGraphic")

        print("#### state:", state)

        if state == common_pb.ACHIEVEMENT_NONE then      -- 未解锁
            idle_trans:SetActive(true)
            --can_rcv_trans:SetActive(false)
            rcv_trans:SetActive(false)
            die_trans:SetActive(false)
            skeleton_trans:SetActive(true)
            skeleton.AnimationState:SetAnimation(0, "idle", true)
        elseif state == common_pb.ACHIEVEMENT_UNLOCK then -- 已解锁
            idle_trans:SetActive(true)
            --can_rcv_trans:SetActive(false)
            rcv_trans:SetActive(false)
            die_trans:SetActive(false)
            skeleton_trans:SetActive(true)
            skeleton.AnimationState:SetAnimation(0, "idle", true)
        elseif state == common_pb.ACHIEVEMENT_FINISH then -- 完成未领取
            idle_trans:SetActive(true)
            --can_rcv_trans:SetActive(true)
            rcv_trans:SetActive(false)
            die_trans:SetActive(false)
            skeleton_trans:SetActive(true)
            skeleton.AnimationState:SetAnimation(0, "idle2", true)
        elseif state == common_pb.ACHIEVEMENT_GETAWARD then -- 已领取
            idle_trans:SetActive(true)
            --can_rcv_trans:SetActive(false)
            rcv_trans:SetActive(false)
            die_trans:SetActive(false)
            skeleton_trans:SetActive(false)
        else--领取时候的动画表现
            idle_trans:SetActive(true)
           -- can_rcv_trans:SetActive(false)
            rcv_trans:SetActive(true)
            die_trans:SetActive(false)
            skeleton_trans:SetActive(true)
            skeleton.AnimationState:SetAnimation(0, "idle3", false)
        end
        
    end,
}