local super = require("ui.frame")
local log = logger.get("Game", "Chapter Frame")

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        -- 路径: root/login
        -- 事件: click_login 
        clickEvents = {
            "Covers/ButtonBack",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self:levelOver()

        self:init()

        game_director:disable3DRender()
    end,

    onUnload = function (self)
        game_director:enable3DRender()
    end,

    update = function(self)
        -- chapter 更换时刷新 UI
        if self.scrollRect then
            if self.scrollRect.SelectIdx ~= self.scroll_child_idx then

                local getChild = function(idx)
                    return self.content_parent:Find("chapter_"..idx+1)
                end

                self:setCoversVisible(getChild(self.scroll_child_idx), false, self.scroll_child_idx+1)
                self:setCoversVisible(getChild(self.scrollRect.SelectIdx), true, self.scrollRect.SelectIdx+1)

                self.scroll_child_idx = self.scrollRect.SelectIdx
            end
        end
    end,

    -- 设置遮罩和置灰
    setCoversVisible = function(self, trans, visible, idx)
        local green_cover = trans:Find("scale/GreenCover"):GetComponent(UnityEngine.UI.Image)
        local gray_cover = trans:Find("scale/GrayCover"):GetComponent(UnityEngine.UI.Image)
        green_cover:SetActive(visible)
        gray_cover:SetActive(not visible)

        local path = "Center/Scroll View/Viewport/Content/"..trans.name.."/scale/NameTextBG/Image"
        self.helper:setImageGray(self, path, not visible)
        
        path = "Center/Scroll View/Viewport/Content/"..trans.name.."/scale/NameTextBG"
        self.helper:setImageGray(self, path, not visible)

        path = "Center/Scroll View/Viewport/Content/"..trans.name
        self.helper:setImageGray(self, path, not visible)

        local text = trans:Find("scale/NameTextBG/Image/nameText"):GetComponent(UnityEngine.UI.Text)
        local level_text = trans:Find("scale/lvText"):GetComponent(UnityEngine.UI.Text)
        if not visible then
            text.color = Color.gray
            level_text.color = Color.gray
        else
            text.color = Color.white
            local state = self:isUnlock(idx)
            level_text.color = state and Color.yellow or Color.red
        end
    end,

    init = function(self)
        local length = #gConfig.game_copy_chapters

        local tplt = self.transform:Find("tplt")
        self.content_parent = self.transform:Find("Center/Scroll View/Viewport/Content/")
        tplt.gameObject:SetActive(false)

        local startPos = Vector3(261, -421.5)
        local interval = Vector3(222, 0)

        local cur_chapter_id, cur_level_id = self:getCurChapterId()

        if not cur_chapter_id then
            cur_chapter_id = 1
        end

        print("cur_chapter_id:", cur_chapter_id)

        for i = 1, length do
            local cfg = config.game_copy_chapters[i]
            local item = Object.Instantiate(tplt.gameObject);
            item.transform.name = "chapter_" .. i
            -- item.transform.parent = content_parent
            item.transform:SetParent(self.content_parent)
            item.transform.localPosition = startPos + interval * (i - 1)
            item.transform.localScale = Vector3.one
            item:SetActive(true)

            local image = item.transform:GetComponent(UnityEngine.UI.Image)
            image:SetSprite(cfg.path .. cfg.icon)

            local ing = item.transform:Find("scale/IngTextBG")
            ing.gameObject:SetActive(i == cur_chapter_id)

            local text = item.transform:Find("scale/NameTextBG/Image/nameText"):GetComponent(UnityEngine.UI.Text)
            if text then
                text.text = self.helper:getLangCfg(cfg.Chapter_Name_ID)
            end

            text = item.transform:Find("scale/lvText"):GetComponent(UnityEngine.UI.Text)
            if text then
                local state = self:isUnlock(i)
                text.color = state and Color.yellow or Color.red
                local last_id = self:getLastIdInChapter(i)
                local name_id = config.game_copy_levels[last_id].Level_Name_ID
                -- print("last_id:", last_id, name_id)
                if state then
                    text.text = NativeText.GetLangStr(170009) ..  self.helper:getLangCfg(name_id) 
                else
                    text.text = NativeText.GetLangStr(170010)
                end
            end

            self:setCoversVisible(item.transform, i == 1, i)
        end

        local fish = self.transform:Find("Center/Scroll View"):GetComponent(UIFisheye)
        -- 返回的索引是从 0 开始
        fish:InitChilds(function(idx)
            if idx ~= -1 then
                idx = idx + 1
                self:showDetailChapter(idx)
                print("选择的章节索引:", idx)
            end
        end)

        self.scrollRect = fish.scrollrect
        self.scroll_child_idx = self.scrollRect.SelectIdx
    end,

    -- 获得当前进行的章节id (不一定有)
    getCurChapterId = function(self)
        for k, v in ipairs(game_info.chapter.AreaMaps) do
            if v.copyId and v.copyId ~= 0 then
                return config.game_copy_levels[v.copyId].Chapter, v.copyId
            end
        end
        return nil, nil
    end,

    -- 是否解锁
    isUnlock = function(self, chapter_id)
        for k, v in ipairs(game_info.chapter.AreaMaps) do
            if v.CurAreaID == chapter_id then
                return true
            end
        end
        return false
    end,

    -- 获取章节最后一个关卡 ( config.game_copy_levels )
    getLastIdInChapter = function(self, chapter_id)
        local last_id = 0
        for k, v in ipairs(game_info.chapter.AreaMaps) do
            if chapter_id == v.CurAreaID then
                for __, vl in ipairs(v.AreaPosInfos) do
                    last_id = vl.AreaPosID
                end
                break
            end
        end

        if last_id == 0 then
             config.game_copy_levels:foreach(function (k, v)
                if v.Chapter == chapter_id then
                    if k > last_id then
                        last_id = k
                    end
                end
            end)
        end

        return last_id
    end,

    -- 显示具体章节界面
    showDetailChapter = function(self, idx)
        log.info("############## showDetailChapter ", idx)
        if not self:isUnlock(idx) then
            hint:setText(NativeText.GetLangStr(170007))
            return
        end
        local cfg = config.game_copy_chapters[idx]
        if cfg then
            -- frame_manager:hide('chapter')
            frame_manager:show('step', self, { id = idx , cfg = cfg})
        end
    end,

    click_ButtonBack = function(self)
        frame_manager:hide('chapter')
    end,

    levelOver = function(self)
        if level_info.level_over then
            self.helper.hideTrans(self.transform)
            self:showDetailChapter(level_info.cur_chapter)
        end
    end,
}
