local super = require("ui.frame")

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        -- 路径: root/login
        -- 事件: click_login 
        clickEvents = {
            --"BackGround/bg/closeTask"
            "BackGround"
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        -- boost信息
        self.evt_id = game_director.events:on("CHAPTERTASKINFO", functor.make(self.onChapterTask, self))

        self:setData()
    end,

    onUnload = function (self)
        game_director.events:off("CHAPTERTASKINFO", self.evt_id)
    end,

    setData = function(self)
        -- required uint32 taskid              =1;        //任务ID
        -- required TASK_STATUS ts             =2;        //任务状态
        -- required uint32 isUnlock            =3;		   //任务是否开启
        -- repeated MResTaskGold taskGold      =4;        //任务目标类型

        -- enum TASK_STATUS
        -- {
        -- TS_NONE       = 0;  //未开始
        -- TS_GETAWARD   = 1;  //已完成未领取
        -- TS_DOING      = 2;  //未完成
        -- TS_TODAYOVER  = 3;  //已领取(只有日常任务中存在)
        -- TS_OVER       = 4;  //一次性的任务永久完成
        -- }

        -- //研发任务目标
        -- message MResTaskGold
        -- {
        -- required ResTaskEventType eventType   = 1 ; //任务事件类型
        -- required uint32 id   = 2;       //资源或怪物Id
        -- required uint32 num  = 3;       //数量
        -- }

        local sort_list = {}

        for k, v in ipairs(game_info.chapterTask.Reslist) do
            table.insert(sort_list, v)
        end

        table.sort(sort_list, function(a, b)
            return a.ts < b.ts
        end)
        
        if not self.taskTplt then
            self.taskTplt = self.transform:Find("bg/Scroll View/Viewport/Content/itemTemplate")
            self.taskTplt.gameObject:SetActive(false)
        end

        if not self.cacheItems then
            self.cacheItems = {}
        else
            for k, v in ipairs(self.cacheItems) do
                v.obj:SetActive(false)
            end
        end

        for k, v in ipairs(sort_list) do
            local cfg = config.game_copy_chapter_quests[v.taskid]
            local item = nil
            local tar = self:getItem(v.taskid)
            if tar then
                item = tar.obj
                item:SetActive(true)
            else
                -- 创建对象
                item = Object.Instantiate(self.taskTplt.gameObject);
                item:AttachTo(self.taskTplt.transform.parent)
                item:SetActive(true)
                item.transform.name = v.taskid            
                -- 设置icon
                local icon = item.transform:Find('Grade/Icon'):GetComponent(UnityEngine.UI.Image)
                local icon_name = 'Assets/Arts/ui/atlas/icon/item/' .. cfg.icon .. '.png'
                -- local icon_name = self.helper:getItemIcon(cfg.id)
                icon:SetSprite(icon_name)
                -- 设置名字
                local nameText = item.transform:Find('nameText'):GetComponent(UnityEngine.UI.Text)
                nameText.text = self.helper:getLangCfg(cfg.Name_ID)
                -- 设置描述
                local desText = item.transform:Find('desText'):GetComponent(UnityEngine.UI.Text)
                desText.text = self.helper:getLangCfg(cfg.Des_ID)
                -- 设置奖励
                local rwdTemplate = item.transform:Find('rwds/rwdTemplate')
                local rwdParent = item.transform:Find('rwds')
                rwdTemplate.gameObject:SetActive(false)
                local tmpArgs = self.helper:getJsonTab(cfg.Reward)
                for itemId, num in pairs(tmpArgs) do
                    local rwdItem = Object.Instantiate(rwdTemplate.gameObject);
                    rwdItem.gameObject:SetActive(true)
                    rwdItem:AttachTo(rwdParent)

                    local image = rwdItem:GetComponent(UnityEngine.UI.Image)
                    local cfg = self.helper:getItemCfg(itemId)
                    local icon_name = self.helper:getItemIcon(cfg.id)
                    image:SetSprite(icon_name)
                end
                -- 设置点击事件
                item.transform.name = "task"..v.taskid
                if v.ts == 1 then
                    self:setScript("click", "bg/Scroll View/Viewport/Content/" .. item.transform.name, functor.rmake(self.click_getRwd, v.taskid))
                end
                -- 缓存数据
                table.insert(self.cacheItems, { id = v.taskid, cfg = cfg, obj = item })
            end
            -- 设置任务状态
            local completed = item.transform:Find('state/yes')
            local uncompleted = item.transform:Find('state/no')
            local over = item.transform:Find('state/over')
            completed.gameObject:SetActive(v.ts == 1)
            uncompleted.gameObject:SetActive(v.ts == 2)
            over.gameObject:SetActive(v.ts == 3)
            local slider = item.transform:Find('Progress'):GetComponent(UnityEngine.UI.Slider)
            if v.ts == 1 then
                slider.value = 1
            end
            local nowNum = 0;
            local needNum = 0;
            -- 如果 == 0 表示是杀怪
            if cfg.Type == 1 then 
                for kk, vv in pairs(v.taskGold) do
                    if vv.eventType == 1 then
                        nowNum = vv.num + nowNum
                    end
                end
                local tmpArgs = self.helper:getJsonTab(cfg.Para)
                for kk, vv in pairs(tmpArgs) do
                    needNum = vv + needNum
                end
            -- 如果 != 0 表示是收集
            else 
                for kk, vv in ipairs(v.taskGold) do
                    if vv.eventType == 2 then
                        nowNum = vv.num + nowNum
                    end
                end
                local tmpArgs = self.helper:getJsonTab(cfg.Para)
                for kk, vv in pairs(tmpArgs) do
                    needNum = vv + needNum
                end
            end

            local progressText = item.transform:Find('progressText'):GetComponent(UnityEngine.UI.Text)
            progressText.text = nowNum .. '/' .. needNum
            slider.value = nowNum / needNum
        end
    end,

    -- 章节信息变化
    onChapterTask = function(self)
        self:setData()
    end,

    -- 获取奖励
    click_getRwd = function(self, handler, task_id)
        print("获取奖励:", task_id)
        network_sender:getChapterTaskReward(task_id)
    end,

    -- 关闭界面
    click_closeTask = function(self)
        frame_manager:hide('chapterTask')
    end,

    click_BackGround = function(self)
        frame_manager:hide('chapterTask')
    end,

    -- 获取本地缓存数据
    -- id 是 taskid
    getItem = function(self, id)
        if not self.cacheItems then
            return nil
        end
        for k, v in ipairs(self.cacheItems) do
            if v.id == id then
                return v
            end
        end
        return nil
    end,
}
