local super = require("ui.frame")

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        if not local_chat then require('game.chat') end

        clickEvents = 
        {
            "content/bg/top/backBtn",
            "content/right/bottom/sendBtn",
        }

        -- 注册点击事件
        frame_helper:regClickEvents("click", self, clickEvents)

        self.world_trans = self.transform:Find("content/world")
        self.guild_trans = self.transform:Find("content/guild")
        self.input_field = self.transform:Find("content/right/bottom/InputField"):GetComponent(UnityEngine.UI.InputField)

        self.chat_msg_change = local_chat.events:on("chat_msg_change", function()
            print("刷新界面!!!")
            self:showContent()
            self:makeContentBottom(false)
        end)

        self:init()
    end,

    onUnload = function (self)
        local_chat.events:off("chat_msg_change", self.chat_msg_change)
    end,
    
    init = function(self)
        self:initRithgBar()
        self:click_rightBar(1)
        self:showContent()
        self:makeContentBottom(true)
    end,

    -- 初始化右边按钮
    initRithgBar = function(self)
        for i = 1, 2 do 
            local path = "content/left bar/Right (" .. i .. ")/Page"
            self:setScript("click", path, function() self:click_rightBar(i) end)
        end
    end,

    click_rightBar = function(self, idx)
        if self.cur_idx == idx then
            return
        end
        self.cur_idx = idx
        for i = 1, 2 do
            local button = self:findImage("content/left bar/Right (" .. i .. ")/Page")
            button.color = i == idx and Color(1,1,1,1) or Color(1,1,1,0)
        end
        frame_helper:setActive(self.world_trans, idx == 1)
        frame_helper:setActive(self.guild_trans, idx == 2)
    end,

    click_backBtn = function(self)
        frame_manager:hide("chat")
    end,

    showContent = function(self)
        self:showWorldContent()
        self:showGuildContent()
    end,

    makeContentBottom = function(self, force)
        local sr1 = self.transform:Find("content/world/Scroll View/"):GetComponent(UnityEngine.UI.ScrollRect)
        local sr2 = self.transform:Find("content/guild/Scroll View/"):GetComponent(UnityEngine.UI.ScrollRect)
        local temp_table = {sr1, sr2}
        for __, sr in ipairs(temp_table) do
            if force then
                app:startCoroutine(function()
                    coroutine.yield(nil)
                    sr.verticalNormalizedPosition = 0
                end)
            else
                if sr.verticalNormalizedPosition <= 0.01 or sr.verticalNormalizedPosition > 1 then
                    app:startCoroutine(function()
                        coroutine.yield(nil)
                        sr.verticalNormalizedPosition = 0
                    end)
                end
            end
        end
    end,
    
    -- required string   chatId			= 1;//聊天id
    -- required string   playerId        = 2;//玩家ID   
    -- optional string   playerName      = 3;//玩家名字 
    -- optional uint32   playerHead      = 4;//玩家头像 
    -- optional string   familyAcronym	= 5;//工会缩写  
    -- optional string   content         = 6;//内容
    -- optional uint32   sendTime        = 7;//发送时间
    setItemDetailInfo = function(self, child, info, parent_path)
        local tplt_name, content_type, content = self:getTemplateName(info, parent_path)
        local name = child:Find("offset/name"):GetComponent(UnityEngine.UI.Text)
        local time = child:Find("offset/time"):GetComponent(UnityEngine.UI.Text)
        local des = child:Find("offset/des"):GetComponent(UnityEngine.UI.Text)
        local iconBtn = child:Find("offset/icon"):GetComponent(UnityEngine.UI.Button)

        iconBtn.onClick:RemoveAllListeners()
        iconBtn.onClick:AddListener(function()
            network_sender:getPlayerInfo(info.playerId)
        end)

        time.text = self:getFormatTime(info.sendTime)
        if content_type == 1 then -- 普通文本消息
            des.text = content
            if string.len(info.familyAcronym) > 0 then
                name.text = string.format('[%s] %s', info.familyAcronym, info.playerName)
            else
                name.text = info.playerName
            end
        elseif content_type == 2 then -- 表情消息
        elseif content_type == 3 then -- 链接
        end
    end,

    showWorldContent = function(self)
        if not ( local_chat and local_chat:getContent(protocol_pb.WORLD)) then
            return
        end
        local chat_record = local_chat:getContent(protocol_pb.WORLD).chatRecord
        local parent_path = "content/world/Scroll View/Viewport/Content/"
        if #chat_record > 0 then
            frame_helper:hideAllChild(self, parent_path)
        else
            frame_helper:hideAllChild(self, parent_path, true)
        end
        for __, info in ipairs(chat_record) do
            local tplt_path = self:getTemplateName(info, parent_path)
            local child = frame_helper:getChildByTemplatePath(self, __, parent_path, tplt_path)
            child:SetActive(true)
            self:setItemDetailInfo(child, info, parent_path)
        end
    end,

    showGuildContent = function(self)
        if not ( local_chat and local_chat:getContent(protocol_pb.FAMILYGENERAL)) then
            return
        end
        local chat_record = local_chat:getContent(protocol_pb.FAMILYGENERAL).chatRecord
        local parent_path = "content/guild/Scroll View/Viewport/Content/"
        if #chat_record > 0 then
            frame_helper:hideAllChild(self, parent_path)
        else
            frame_helper:hideAllChild(self, parent_path, true)
        end
        for __, info in ipairs(chat_record) do
            local tplt_name = self:getTemplateName(info, parent_path)
            if tplt_name then
                local tplt_trans = self.transform:Find(tplt_name)
                local child = self:getSrollViewChild(__, tplt_trans, parent_path)
                if child then child.gameObject:SetActive(true) end
                self:setItemDetailInfo(child, info, parent_path)
            end
        end
    end,

    getTemplateName = function(self, info, parent_path)
        local tb = frame_helper:unserialize(info.content)
        if tb and tb.content_type then
            local is_me = info.playerId == local_player.id    
            -- 普通文本消息
            if tb.content_type == 1 then
                return parent_path .. ( is_me and "normal_template_me" or "normal_template" ), tb.content_type, tb.text
            -- 表情消息
            elseif tb.content_type == 2 then
                return parent_path .. ( is_me and "emoji_template_me" or "emoji_template" ), tb.content_type, tb.text
            -- 链接
            elseif tb.content_type == 3 then
                return parent_path .. ( is_me and "link_template_me" or "link_template" ), tb.content_type, tb.text
            end
        end
        return nil
    end,

    click_sendBtn = function(self)
        if self.cur_idx == 2 then
            if not local_guild:isSelfInGuild() then
                frame_helper:showCommonBox(NativeText.GetLangStr(210019), function()
                    frame_manager:hide('chat')
                    gui_helper:openGuildListUI()
                end)
                return
            end
        end

        local t0 = string.gsub(self.input_field.text, " ", "")
        if string.len(t0) <= 0 then
            self.input_field.text = ""
            return
        end
        local temp = {}
        temp.content_type = 1
        temp.text = self.input_field.text
        self.input_field.text = ""
        print("发送消息工会普通频道消息", frame_helper:serialize(temp))
        network_sender:chatReq((self.cur_idx == 1) and protocol_pb.WORLD or protocol_pb.FAMILYGENERAL, "", frame_helper:serialize(temp))
    end,

    getSrollViewChild = function(self, idx, child_template_trans, parent_path)
        if not self[child_template_trans] then
            self[child_template_trans] = {}
        end
        local parent_trans = self.transform:Find(parent_path)
        if idx <= #self[child_template_trans] then
            return self[child_template_trans][idx]
        else
            local template = child_template_trans
            for i = 1, idx - #self[child_template_trans] do
                local inst_obj = Object.Instantiate(template.gameObject)
                inst_obj.transform.name = i + #self[child_template_trans]
                inst_obj:AttachTo(parent_trans)
                inst_obj.transform.sizeDelta = child_template_trans.sizeDelta
                table.insert(self[child_template_trans], inst_obj.transform)
            end     
            return self[child_template_trans][idx]
        end
    end,

    --根据时间戳获取时间，排布
    getFormatTime = function(self, time)   
        local format = os.date("*t", time) --获取当时本地时间
        local format_time = string.format("%d/%d/%d %02d:%02d:%02d",format.year,format.month,format.day,format.hour,format.min,format.sec)
        return format_time
    end
}


--[[
local super = require("ui.frame")
local log = logger.get("Game", "Guild Logo")

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        if not local_chat then require('game.chat') end

        clickEvents = 
        {
            "bg/top/ButtonBack",
            "content/1st/sendWorldBtn",
            "content/2st/Scroll View Bar/Viewport/Content/normal/normalBtn",
            "content/2st/Scroll View Bar/Viewport/Content/mgr/mgrBtn",
            "2st/closeViewBtn",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        local active_false_objects = {
            "content/1st/Scroll View/Viewport/Content/normal_template",
            "content/1st/Scroll View/Viewport/Content/normal_template_me",
            "content/1st/Scroll View/Viewport/Content/link_template",
            "content/1st/Scroll View/Viewport/Content/link_template_me",
            "content/1st/Scroll View/Viewport/Content/emoji_template",
            "content/1st/Scroll View/Viewport/Content/emoji_template_me",

            "content/2st/guildNormal/Scroll View/Viewport/Content/normal_template",
            "content/2st/guildNormal/Scroll View/Viewport/Content/normal_template_me",
            "content/2st/guildNormal/Scroll View/Viewport/Content/link_template",
            "content/2st/guildNormal/Scroll View/Viewport/Content/link_template_me",
            "content/2st/guildNormal/Scroll View/Viewport/Content/emoji_template",
            "content/2st/guildNormal/Scroll View/Viewport/Content/emoji_template_me",

            "content/2st/guildMgr/Scroll View/Viewport/Content/normal_template",
            "content/2st/guildMgr/Scroll View/Viewport/Content/normal_template_me",
            "content/2st/guildMgr/Scroll View/Viewport/Content/link_template",
            "content/2st/guildMgr/Scroll View/Viewport/Content/link_template_me",
            "content/2st/guildMgr/Scroll View/Viewport/Content/emoji_template",
            "content/2st/guildMgr/Scroll View/Viewport/Content/emoji_template_me",

            "content/3st/Scroll View Bar/Viewport/Content/template",

            "content/3st/personals/tplt/Scroll View/Viewport/Content/normal_template",
            "content/3st/personals/tplt/Scroll View/Viewport/Content/normal_template_me",
            "content/3st/personals/tplt/Scroll View/Viewport/Content/link_template",
            "content/3st/personals/tplt/Scroll View/Viewport/Content/link_template_me",
            "content/3st/personals/tplt/Scroll View/Viewport/Content/emoji_template",
            "content/3st/personals/tplt/Scroll View/Viewport/Content/emoji_template_me",
            "content/3st/personals/tplt",
        }

        for __, path in ipairs(active_false_objects) do
            self.transform:Find(path):SetActive(false)
        end

        self.world_trans = self.transform:Find("content/1st")
        self.guild_trans = self.transform:Find("content/2st")
        self.person_trans = self.transform:Find("content/3st")

        self.guild_nor_trans = self.transform:Find("content/2st/guildNormal")
        self.guild_mgr_trans = self.transform:Find("content/2st/guildMgr")

        self.nor_btn_trans = self.transform:Find("content/2st/Scroll View Bar/Viewport/Content/normal")
        self.mgr_btn_trans = self.transform:Find("content/2st/Scroll View Bar/Viewport/Content/mgr")

        self.world_red_text = self:findText('content/top_bar/bt (1)/red/Text')
        self.world_red_image = self:findImage('content/top_bar/bt (1)/red')

        self.guild_red_text = self:findText('content/top_bar/bt (2)/red/Text')
        self.guild_red_image = self:findImage('content/top_bar/bt (2)/red')

        self.guild_nor_red_text = self:findText('content/2st/Scroll View Bar/Viewport/Content/normal/unselect/red/Text')
        self.guild_nor_red_image = self:findImage('content/2st/Scroll View Bar/Viewport/Content/normal/unselect/red')

        self.guild_mgr_red_text = self:findText('content/2st/Scroll View Bar/Viewport/Content/mgr/unselect/red/Text')
        self.guild_mgr_red_image = self:findImage('content/2st/Scroll View Bar/Viewport/Content/mgr/unselect/red')

        self.personal_red_text = self:findText('content/top_bar/bt (3)/red/Text')
        self.personal_red_image = self:findImage('content/top_bar/bt (3)/red')

        self.view_trans = self.transform:Find("2st")
        self.helper.hideTrans(self.view_trans)

        self:click_normalBtn()
        self:init()

        self.loaded = true
    end,

    onUnload = function (self)
        local_chat.events:off("chat_msg_change", self.chat_msg_change_id)
        local_chat.events:off("tip_num_change", self.tip_num_change_id)
    end,
    
    init = function(self)
        self:initTopBar()
        self:initGuildBar()
        self:showContent()
        self:setGuildPlus()
        self:showRedTip()

        self.chat_msg_change_id = local_chat.events:on('chat_msg_change', function()
            print("刷新界面...")
            self:showContent()
        end)

        self.tip_num_change_id = local_chat.events:on('tip_num_change', function()
            print("刷新红点")
            self:showRedTip()
        end)
    end,

    showContent = function(self)
        self:showWorldContent()
        self:showGuildNorContent()
        self:showGuildMgrContent()
        self:showPersonalContent()
    end,

    setGuildPlus = function(self)
        local rst, guild_level = local_guild:getGuildLevel()
        self.in_guild = rst
        if not rst then
            self.transform:Find("content/top_bar/bt (2)").gameObject:SetActive(false)
            self.in_mgr_channel = false
        else
            self.in_mgr_channel = guild_level >= common_pb.MD_R4
            if guild_level < common_pb.MD_R4 then
                self.transform:Find("content/2st/Scroll View Bar/Viewport/Content/mgr").gameObject:SetActive(false)
            end
        end
    end,

    initTopBar = function(self)
        for i = 1, 3 do 
            local path = "content/top_bar/bt (" .. i .. ")"
            self:setScript("click", path, function() self:click_top_bar(i) end)
        end
        self.bar_index = 1
        self:click_top_bar(self.bar_index)
    end,

    initGuildBar = function(self)
        local guild_nor_path = "content/2st/Scroll View Bar/Viewport/Content/normal/normalBtn"
        self:setScript("click", guild_nor_path, function() self:click_guild_bar(protocol_pb.FAMILYGENERAL, true) end)
        local guild_mgr_path = "content/2st/Scroll View Bar/Viewport/Content/mgr/mgrBtn"
        self:setScript("click", guild_mgr_path, function() self:click_guild_bar(protocol_pb.FAMILYMANAGER, true) end)

        self:click_guild_bar(protocol_pb.FAMILYGENERAL, false)
    end,

    click_top_bar = function(self, index)
        self.bar_index = index
        for i = 1, 3 do
            local select_trans = self.transform:Find("content/top_bar/bt (" .. i .. ")/select")
            local unselect_trans = self.transform:Find("content/top_bar/bt (" .. i .. ")/unselect")
            local red_trans = self.transform:Find("content/top_bar/bt (" .. i .. ")/red")
            select_trans:SetActive(i == index)
            unselect_trans:SetActive(i ~= index)
            -- red_trans:SetActive(i ~= index)
        end

        if self.bar_index == 1 then
            self.helper.showTrans(self.world_trans)
            self.helper.hideTrans(self.guild_trans)
            self.helper.hideTrans(self.person_trans)

            self.curr_select_tag = protocol_pb.WORLD
            self:showRedTip()

        elseif self.bar_index == 2 then
            self.helper.hideTrans(self.world_trans)
            self.helper.showTrans(self.guild_trans)
            self.helper.hideTrans(self.person_trans)

            if self.guild_select_tag then
                self.curr_select_tag = self.guild_select_tag
                self:showRedTip()
            end

        elseif self.bar_index == 3 then
            self.helper.hideTrans(self.world_trans)
            self.helper.hideTrans(self.guild_trans)
            self.helper.showTrans(self.person_trans)

            if self.cur_select_personal then
                self.curr_select_tag = self.cur_select_personal
                self:showRedTip()
            end
        end
    end,

    click_guild_bar = function(self, guild_channel_type, clear_red)
        local guild_nor_path = "content/2st/Scroll View Bar/Viewport/Content/normal/select"
        local guild_mgr_path = "content/2st/Scroll View Bar/Viewport/Content/mgr/select"

        local nor_sv_trans = self.transform:Find("content/2st/guildNormal")
        local mgr_sv_trans = self.transform:Find("content/2st/guildMgr")

        self.guild_select_tag = guild_channel_type

        if guild_channel_type == protocol_pb.FAMILYGENERAL then
            self.transform:Find(guild_nor_path).gameObject:SetActive(true)
            self.transform:Find(guild_mgr_path).gameObject:SetActive(false)
            self.helper.showTrans(nor_sv_trans)
            self.helper.hideTrans(mgr_sv_trans)
            if clear_red then
                self.curr_select_tag = protocol_pb.FAMILYGENERAL
                self:showRedTip()
            end
        elseif guild_channel_type == protocol_pb.FAMILYMANAGER then
            self.transform:Find(guild_nor_path).gameObject:SetActive(false)
            self.transform:Find(guild_mgr_path).gameObject:SetActive(true)
            self.helper.hideTrans(nor_sv_trans)
            self.helper.showTrans(mgr_sv_trans)
            if clear_red then
                self.curr_select_tag = protocol_pb.FAMILYMANAGER
                self:showRedTip()
            end
        end
    end,

    showRedTip = function(self)
        if self.curr_select_tag == protocol_pb.WORLD then
            local_chat:makeReaded(protocol_pb.WORLD)
        elseif self.curr_select_tag == protocol_pb.FAMILYGENERAL then
            local_chat:makeReaded(protocol_pb.FAMILYGENERAL)
        elseif self.curr_select_tag == protocol_pb.FAMILYMANAGER then
            local_chat:makeReaded(protocol_pb.FAMILYMANAGER)
        else
            local_chat:makeReaded(protocol_pb.PERSONAL, self.curr_select_tag)
        end

        self.world_red_text.enabled = local_chat:getUnreadNum(protocol_pb.WORLD) > 0
        self.world_red_image.enabled = local_chat:getUnreadNum(protocol_pb.WORLD) > 0
        self.world_red_text.text = local_chat:getUnreadNum(protocol_pb.WORLD)

        if self.in_mgr_channel then
            self.guild_red_text.enabled = (local_chat:getUnreadNum(protocol_pb.FAMILYGENERAL) + local_chat:getUnreadNum(protocol_pb.FAMILYMANAGER)) > 0
            self.guild_red_image.enabled = (local_chat:getUnreadNum(protocol_pb.FAMILYGENERAL) + local_chat:getUnreadNum(protocol_pb.FAMILYMANAGER)) > 0
            self.guild_red_text.text = local_chat:getUnreadNum(protocol_pb.FAMILYGENERAL) + local_chat:getUnreadNum(protocol_pb.FAMILYMANAGER)
        else
            self.guild_red_text.enabled = local_chat:getUnreadNum(protocol_pb.FAMILYGENERAL) > 0
            self.guild_red_image.enabled = local_chat:getUnreadNum(protocol_pb.FAMILYGENERAL) > 0
            self.guild_red_text.text = local_chat:getUnreadNum(protocol_pb.FAMILYGENERAL)
        end

        self.guild_nor_red_text.enabled = local_chat:getUnreadNum(protocol_pb.FAMILYGENERAL) > 0
        self.guild_nor_red_image.enabled = local_chat:getUnreadNum(protocol_pb.FAMILYGENERAL) > 0
        self.guild_nor_red_text.text = local_chat:getUnreadNum(protocol_pb.FAMILYGENERAL)

        self.guild_mgr_red_text.enabled = local_chat:getUnreadNum(protocol_pb.FAMILYMANAGER) > 0
        self.guild_mgr_red_image.enabled = local_chat:getUnreadNum(protocol_pb.FAMILYMANAGER) > 0
        self.guild_mgr_red_text.text = local_chat:getUnreadNum(protocol_pb.FAMILYMANAGER)

        self.personal_red_text.enabled = local_chat:getUnreadNum(protocol_pb.PERSONAL) > 0
        self.personal_red_image.enabled = local_chat:getUnreadNum(protocol_pb.PERSONAL) > 0
        self.personal_red_text.text = local_chat:getUnreadNum(protocol_pb.PERSONAL)

        if self.personal_cache then
            for player_id, p in pairs(self.personal_cache) do
                local red_text = p.bar.transform:Find("unselect/red/Text"):GetComponent(UnityEngine.UI.Text)
                local red_image = p.bar.transform:Find("unselect/red"):GetComponent(UnityEngine.UI.Image)

                red_text.enabled = local_chat:getUnreadNum(player_id) > 0
                red_image.enabled = local_chat:getUnreadNum(player_id) > 0
                red_text.text = local_chat:getUnreadNum(player_id)
            end
        end
    end,

    click_normalBtn = function(self)
        self.nor_btn_trans.transform:Find("select").gameObject:SetActive(true)
        self.mgr_btn_trans.transform:Find("select").gameObject:SetActive(false)
        self.helper.showTrans(self.guild_nor_trans)
        self.helper.hideTrans(self.guild_mgr_trans)
    end,
    

    click_mgrBtn = function(self)
        self.nor_btn_trans.transform:Find("select").gameObject:SetActive(false)
        self.mgr_btn_trans.transform:Find("select").gameObject:SetActive(true)
        self.helper.hideTrans(self.guild_nor_trans)
        self.helper.showTrans(self.guild_mgr_trans)
    end,

    click_ButtonBack = function(self)
        frame_manager:hide('chat')
    end,
    
    -- required string   chatId			= 1;//聊天id
    -- required string   playerId        = 2;//玩家ID   
    -- optional string   playerName      = 3;//玩家名字 
    -- optional uint32   playerHead      = 4;//玩家头像 
    -- optional string   familyAcronym	= 5;//工会缩写  
    -- optional string   content         = 6;//内容
    -- optional uint32   sendTime        = 7;//发送时间

    setSingleContent = function(self, child, info, parent_path)
        local tplt_name, content_type, content = self:getWorldTemplateName(info, parent_path)
        local name_text = child:Find('top_bg/name_text'):GetComponent(UnityEngine.UI.Text)
        local time_text = child:Find('top_bg/time_text'):GetComponent(UnityEngine.UI.Text)
        -- print('parent_path', parent_path)
        local button = child:Find('icon'):GetComponent(UnityEngine.UI.Button)
        button.onClick:RemoveAllListeners()
        button.onClick:AddListener(
            function ()
                local is_me = info.playerId == local_player.id
                if not is_me then
                    self:showOperatorView(info)
                end
            end
        )
        -- 普通文本消息
        if content_type == 1 then
            local content_text = child:Find('content/text'):GetComponent(UnityEngine.UI.Text)
            content_text.text = content
            if info.familyAcronym ~= "" then
                name_text.text = string.format('[%s] %s', info.familyAcronym, info.playerName)
            else
                name_text.text = info.playerName
            end
        -- 表情消息
        elseif content_type == 2 then
        -- 链接
        elseif content_type == 3 then
        end
    end,

    refreshScrollRect = function(self, parent_path)
        local state = true
        self.refresh_list = self.refresh_list or {}
        for __, path in ipairs(self.refresh_list) do
            if path == parent_path then
                state = false
                break
            end
        end
        if state then
            table.insert(self.refresh_list, parent_path)
            app:startCoroutine(function()
                self.transform:Find(parent_path):SetActive(false)
                coroutine.yield(nil)
                self.transform:Find(parent_path):SetActive(true)
            end)
        end
    end,

    showWorldContent = function(self)
        if not ( local_chat and local_chat:getContent(protocol_pb.WORLD)) then
            return
        end
        local chatRecord = local_chat:getContent(protocol_pb.WORLD).chatRecord
        local parent_path = "content/1st/Scroll View/Viewport/Content/"
        for __, info in ipairs(chatRecord) do
            local tplt_name, content_type, content = self:getWorldTemplateName(info, parent_path)
            if tplt_name and content_type and content then
                local tplt_trans = self.transform:Find(tplt_name)
                local child = self:getSvChild(__, tplt_trans, parent_path)
                if child then child.gameObject:SetActive(true) end
                self:setSingleContent(child, info, parent_path)
            end
        end
        self:refreshScrollRect(parent_path)
    end,

    showGuildNorContent = function(self)
        if not ( local_chat and local_chat:getContent(protocol_pb.FAMILYGENERAL)) then
            return
        end
        local chatRecord = local_chat:getContent(protocol_pb.FAMILYGENERAL).chatRecord
        local parent_path = "content/2st/guildNormal/Scroll View/Viewport/Content/"
        for __, info in ipairs(chatRecord) do
            local tplt_name = self:getWorldTemplateName(info, parent_path)
            if tplt_name then
                local tplt_trans = self.transform:Find(tplt_name)
                local child = self:getSvChild(__, tplt_trans, parent_path)
                if child then child.gameObject:SetActive(true) end
                self:setSingleContent(child, info, parent_path)
            end
        end

        local input_field = self.transform:Find('content/2st/guildNormal/InputField'):GetComponent(UnityEngine.UI.InputField)
        local button = self.transform:Find('content/2st/guildNormal/sendMsgBtn'):GetComponent(UnityEngine.UI.Button)
        button.onClick:RemoveAllListeners()
        button.onClick:AddListener(
            function ()
                local temp = {}
                temp.content_type = 1
                temp.text = input_field.text
                print("发送消息工会普通频道消息", self.helper:serialize(temp))
                network_sender:chatReq(protocol_pb.FAMILYGENERAL, "", self.helper:serialize(temp))
                input_field.text = ""
            end
        )
        self:refreshScrollRect(parent_path)
    end,
    
    showGuildMgrContent = function(self)
        if not ( local_chat and local_chat:getContent(protocol_pb.FAMILYMANAGER)) then
            return
        end
        local chatRecord = local_chat:getContent(protocol_pb.FAMILYMANAGER).chatRecord
        local parent_path = "content/2st/guildMgr/Scroll View/Viewport/Content/"
        for __, info in ipairs(chatRecord) do
            local tplt_name = self:getWorldTemplateName(info, parent_path)
            if tplt_name then
                local tplt_trans = self.transform:Find(tplt_name)
                local child = self:getSvChild(__, tplt_trans, parent_path)
                if child then child.gameObject:SetActive(true) end
                self:setSingleContent(child, info, parent_path)
            end
        end

        local input_field = self.transform:Find('content/2st/guildMgr/InputField'):GetComponent(UnityEngine.UI.InputField)
        local button = self.transform:Find('content/2st/guildMgr/sendMsgBtn'):GetComponent(UnityEngine.UI.Button)
        button.onClick:RemoveAllListeners()
        button.onClick:AddListener(
            function ()
                local temp = {}
                temp.content_type = 1
                temp.text = input_field.text
                print("发送消息工会管理频道消息", self.helper:serialize(temp))
                network_sender:chatReq(protocol_pb.FAMILYMANAGER, "", self.helper:serialize(temp))
                input_field.text = ""
            end
        )
        self:refreshScrollRect(parent_path)
    end,
    
    showPersonalContent = function(self)
        if not ( local_chat and local_chat:getContent(protocol_pb.PERSONAL) ) then
            return
        end

        if not self.personal_cache then
            self.personal_cache = {}
        else
            for __, trans in pairs(self.personal_cache) do
                trans.bar.gameObject:SetActive(false)
                trans.scv.gameObject:SetActive(false)
            end
        end

        local chatRecord = local_chat:getContent(protocol_pb.PERSONAL)

        local first_player_id = nil

        for player_id, record_list in pairs(chatRecord) do
            if not first_player_id then first_player_id = player_id end
            local index = self:getPersonalCacheLength() + 1
            -- 这里创建标签 创建scrollrect
            if not self.personal_cache[player_id] then
                local bar_tplt_trans = self.transform:Find("content/3st/Scroll View Bar/Viewport/Content/template")
                local bar_parent_path = "content/3st/Scroll View Bar/Viewport/Content/"
                local bar = self:getSvChild(index, bar_tplt_trans, bar_parent_path)
                bar.transform.name = player_id
                bar.transform.gameObject:SetActive(true)
                bar.transform:Find("button/Text"):GetComponent(UnityEngine.UI.Text).text = local_chat:getPlayerName(player_id)
                bar.transform:Find("select/Text"):GetComponent(UnityEngine.UI.Text).text = local_chat:getPlayerName(player_id)

                local button = bar.transform:Find("button"):GetComponent(UnityEngine.UI.Button)
                button.onClick:RemoveAllListeners()
                button.onClick:AddListener(
                    function()
                        self:click_personal_bar(player_id, true)
                    end
                )

                local button = bar.transform:Find("select/delete"):GetComponent(UnityEngine.UI.Button)
                button.onClick:RemoveAllListeners()
                button.onClick:AddListener(
                    function()
                        self:click_remove_chat(player_id)
                    end
                )
                
                local scv_tplt_trans = self.transform:Find("content/3st/personals/tplt")
                local scv_parent_path = "content/3st/personals/"
                local scv = self:getSvChild(index, scv_tplt_trans, scv_parent_path)
                scv.transform.name = player_id
                scv.transform.gameObject:SetActive(true)

                self.personal_cache[player_id] = { bar = bar, scv = scv }

                local input_field = scv.transform:Find('InputField'):GetComponent(UnityEngine.UI.InputField)
    
                local button = scv.transform:Find('sendMsgBtn'):GetComponent(UnityEngine.UI.Button)
                button.onClick:RemoveAllListeners()
                button.onClick:AddListener(
                    function ()
                        local temp = {}
                        temp.content_type = 1
                        temp.text = input_field.text
    
                        print("发送消息 protocol_pb.PERSONAL", player_id, local_player.id, self.helper:serialize(temp))
                        network_sender:chatReq(protocol_pb.PERSONAL, player_id, self.helper:serialize(temp))
                        input_field.text = ""
                    end
                )
            end

            if self.personal_cache[player_id] then

                self.personal_cache[player_id].bar.gameObject:SetActive(true)
                self.personal_cache[player_id].scv.gameObject:SetActive(true)

                local red_text = self.personal_cache[player_id].bar.transform:Find("unselect/red/Text"):GetComponent(UnityEngine.UI.Text)
                local red_image = self.personal_cache[player_id].bar.transform:Find("unselect/red"):GetComponent(UnityEngine.UI.Image)

                red_text.enabled = local_chat:getUnreadNum(player_id) > 0
                red_image.enabled = local_chat:getUnreadNum(player_id) > 0
                red_text.text = local_chat:getUnreadNum(player_id)

                local parent_path = "content/3st/personals/" .. self.personal_cache[player_id].scv.transform.name .. "/Scroll View/Viewport/Content/"
                self:hideAllSvChild(parent_path)
                for __, info in ipairs(record_list) do
                    local tplt_name = self:getWorldTemplateName(info, parent_path)
                    if tplt_name then
                        local tplt_trans = self.transform:Find(tplt_name)
                        local child = self:getSvChild(__, tplt_trans, parent_path)
                        if child then child.gameObject:SetActive(true) end
                        self:setSingleContent(child, info, parent_path)
                    end
                end
                self:refreshScrollRect(parent_path)
            end
        end

        -- self.default_personal_bar_id = first_player_id

        if first_player_id and (not self.personal_inited) then
            self.personal_inited = true
            self:click_personal_bar(first_player_id)
        else
            if self.cur_select_personal then
                self:click_personal_bar(self.cur_select_personal)
            end
        end
    end,
    
    openPersonalTab = function(self, player_id, player_name)
        local_chat:setPlayerName(player_id, player_name)
        self:click_top_bar(3)
        self.personal_cache = self.personal_cache or {}
        if self.personal_cache and self.personal_cache[player_id] then
        else
            local index = self:getPersonalCacheLength() + 1

            local bar_tplt_trans = self.transform:Find("content/3st/Scroll View Bar/Viewport/Content/template")
            local bar_parent_path = "content/3st/Scroll View Bar/Viewport/Content/"
            local bar = self:getSvChild(index, bar_tplt_trans, bar_parent_path)
            bar.transform.name = player_id
            bar.transform.gameObject:SetActive(true)
            bar.transform:Find("button/Text"):GetComponent(UnityEngine.UI.Text).text = local_chat:getPlayerName(player_id)
            bar.transform:Find("select/Text"):GetComponent(UnityEngine.UI.Text).text = local_chat:getPlayerName(player_id)
            local button = bar.transform:Find("button"):GetComponent(UnityEngine.UI.Button)
            button.onClick:RemoveAllListeners()
            button.onClick:AddListener(
                function()
                    self:click_personal_bar(player_id, true)
                end
            )
            local button = bar.transform:Find("select/delete"):GetComponent(UnityEngine.UI.Button)
            button.onClick:RemoveAllListeners()
            button.onClick:AddListener(
                function()
                    self:click_remove_chat(player_id)
                end
            )
            
            local scv_tplt_trans = self.transform:Find("content/3st/personals/tplt")
            local scv_parent_path = "content/3st/personals/"
            local scv = self:getSvChild(index, scv_tplt_trans, scv_parent_path)
            scv.transform.name = player_id
            scv.transform.gameObject:SetActive(true)

            local input_field = scv.transform:Find('InputField'):GetComponent(UnityEngine.UI.InputField)

            local button = scv.transform:Find('sendMsgBtn'):GetComponent(UnityEngine.UI.Button)
            button.onClick:RemoveAllListeners()
            button.onClick:AddListener(
                function ()
                    local temp = {}
                    temp.content_type = 1
                    temp.text = input_field.text

                    print("发送消息 @protocol_pb.PERSONAL", player_id, self.helper:serialize(temp))
                    network_sender:chatReq(protocol_pb.PERSONAL, player_id, self.helper:serialize(temp))
                    input_field.text = ""
                end
            )

            self.personal_cache[player_id] = { bar = bar, scv = scv }
        end
        self:click_personal_bar(player_id, true)
    end,
    
    click_personal_bar = function(self, player_id, rst)
        self.cur_select_personal = player_id
        if rst then
            self.curr_select_tag = player_id
            local_chat:makeReaded(protocol_pb.PERSONAL, player_id)
            self:showRedTip()
        end

        if self.personal_cache[player_id] then
        end
        for id, data in pairs(self.personal_cache) do
            local select = data.bar:Find("select")
            local unselect = data.bar:Find("unselect")
            select.gameObject:SetActive(player_id == id)
            unselect.gameObject:SetActive(player_id ~= id)
            if player_id == id then
                self.helper.showTrans(data.scv.transform)
            else
                self.helper.hideTrans(data.scv.transform)
            end
        end
    end,

    click_remove_chat = function(self, player_id)
        print("移除聊天", player_id, local_chat:getPlayerName(player_id))
        self.personal_inited = false
        self.cur_select_personal = nil
        self.curr_select_tag = nil
        local_chat:removePersonalData(player_id)
    end,

    getPersonalCacheLength = function(self)
        local length = 0
        self.personal_cache = self.personal_cache or {}
        for player_id, record_list in pairs(self.personal_cache) do
            length = length + 1
        end
        return length
    end,

    getWorldTemplateName = function(self, info, parent_path)
        local tb = self.helper:unserialize(info.content)
        if tb and tb.content_type then
            local is_me = info.playerId == local_player.id    
            -- 普通文本消息
            if tb.content_type == 1 then
                return parent_path .. ( is_me and "normal_template_me" or "normal_template" ), tb.content_type, tb.text
            -- 表情消息
            elseif tb.content_type == 2 then
                return parent_path .. ( is_me and "emoji_template_me" or "emoji_template" ), tb.content_type, tb.text
            -- 链接
            elseif tb.content_type == 3 then
                return parent_path .. ( is_me and "link_template_me" or "link_template" ), tb.content_type, tb.text
            end
        end
        return nil
    end,

    click_sendWorldBtn = function(self)
        local trans = self.transform:Find("content/1st/InputField")
        local input_field = trans.gameObject:GetComponent(UnityEngine.UI.InputField)
        local content = input_field.text
        if content == '' then
            return
        end

        local temp = {}
        temp.content_type = 1
        temp.text = content

        local msg = {}
        msg.chatChannel = protocol_pb.WORLD
        msg.chatInfo = {}
        msg.chatInfo.chatId = 1
        msg.chatInfo.playerId = local_player.id
        msg.chatInfo.playerName = local_player.name
        msg.chatInfo.playerHead = 1
        msg.chatInfo.familyAcronym = "WE"
        msg.chatInfo.content = self.helper:serialize(temp)
        msg.chatInfo.sendTime = 100
        
        network_sender:chatReq(protocol_pb.WORLD, nil, self.helper:serialize(temp))
        -- local_chat:setNtf(msg)
        input_field.text = ''
    end,
    
    click_sendGuildNorBtn = function(self)
        print("click_sendGuildNorBtn biubiu 发送")
        local trans = self.transform:Find("content/2st/guildNormal/InputField")
        local input_field = trans.gameObject:GetComponent(UnityEngine.UI.InputField)
        local content = input_field.text
        if content == '' then
            return
        end

        local temp = {}
        temp.content_type = math.random(1,3)
        -- print('temp.content_type:', temp.content_type)
        temp.text = content

        local msg = {}
        msg.chatChannel = protocol_pb.FAMILYGENERAL
        msg.chatInfo = {}
        msg.chatInfo.chatId = 1
        msg.chatInfo.playerId = local_player.id
        msg.chatInfo.playerName = local_player.name
        msg.chatInfo.playerHead = 1
        msg.chatInfo.familyAcronym = "WE"
        msg.chatInfo.content = self.helper:serialize(temp)
        msg.chatInfo.sendTime = 100

        local_chat:setNtf(msg)
        input_field.text = ''
    end,

    click_sendGuildMgrBtn = function(self)
        print("click_sendGuildMgrBtn biubiu 发送")
        local trans = self.transform:Find("content/2st/guildMgr/InputField")
        local input_field = trans.gameObject:GetComponent(UnityEngine.UI.InputField)
        local content = input_field.text
        if content == '' then
            return
        end

        local temp = {}
        temp.content_type = math.random(1,3)
        -- print('temp.content_type:', temp.content_type)
        temp.text = content

        local msg = {}
        msg.chatChannel = protocol_pb.FAMILYMANAGER
        msg.chatInfo = {}
        msg.chatInfo.chatId = 1
        msg.chatInfo.playerId = local_player.id
        msg.chatInfo.playerName = local_player.name
        msg.chatInfo.playerHead = 1
        msg.chatInfo.familyAcronym = "WE"
        msg.chatInfo.content = self.helper:serialize(temp)
        msg.chatInfo.sendTime = 100

        local_chat:setNtf(msg)
        input_field.text = ''
    end,

    hideAllSvChild = function(self, parent_path)
        local parent_trans = self.transform:Find(parent_path)
        for i = 0, parent_trans.childCount - 1 do
            parent_trans:GetChild(i).gameObject:SetActive(false)
        end
    end,

    getSvChild = function(self, idx, child_template_trans, parent_path)
        if not self[child_template_trans] then
            self[child_template_trans] = {}
        end
        local parent_trans = self.transform:Find(parent_path)
        if idx <= #self[child_template_trans] then
            return self[child_template_trans][idx]
        else
            local template = child_template_trans
            for i = 1, idx - #self[child_template_trans] do
                local inst_obj = Object.Instantiate(template.gameObject)
                inst_obj.transform.name = i + #self[child_template_trans]
                inst_obj:AttachTo(parent_trans)
                inst_obj.transform.sizeDelta = child_template_trans.sizeDelta
                table.insert(self[child_template_trans], inst_obj.transform)
            end     
            return self[child_template_trans][idx]
        end
    end,

    click_closeViewBtn = function(self)
        self.helper.hideTrans(self.view_trans)
    end,

    -- required string   chatId			= 1;//聊天id
    -- required string   playerId        = 2;//玩家ID   
    -- optional string   playerName      = 3;//玩家名字 
    -- optional uint32   playerHead      = 4;//玩家头像 
    -- optional string   familyAcronym	= 5;//工会缩写  
    -- optional string   content         = 6;//内容
    -- optional uint32   sendTime        = 7;//发送时间
    showOperatorView = function(self, info)
        local guild_text = self:findText("2st/bg/window/guild_text")
        local lv_text = self:findText("2st/bg/window/lv_text")
        local forbin_text = self:findText("2st/bg/window/layout/forbinBtn/text")

        local rst = local_chat:inBlackList(info.playerId)

        guild_text.text = string.format('[%s]', info.familyAcronym)
        lv_text.text = string.format('Lv.%s  %s', "1", info.playerName)
        forbin_text.text = rst and NativeText.GetLangStr(210018) or NativeText.GetLangStr(210017)

        local button = self.transform:Find('2st/bg/window/layout/playerInfoBtn'):GetComponent(UnityEngine.UI.Button)
        button.onClick:RemoveAllListeners()
        button.onClick:AddListener(
            function ()
                print("玩家信息                ", info.playerId)
            end
        )

        local button = self.transform:Find('2st/bg/window/layout/playerGuildBtn'):GetComponent(UnityEngine.UI.Button)
        button.onClick:RemoveAllListeners()
        button.onClick:AddListener(
            function ()
                print("所在工会                ", info.familyId)
                if info.familyId and info.familyId ~= "" then
                    network_sender:getFamilyInfo(info.familyId)
                end
            end
        )

        local button = self.transform:Find('2st/bg/window/layout/priChatBtn'):GetComponent(UnityEngine.UI.Button)
        button.onClick:RemoveAllListeners()
        button.onClick:AddListener(
            function ()
                print("发起聊天                ", info.playerId, info.playerName)
                local_chat:createPersonalChat(info.playerId, info.playerName)
                self.helper.hideTrans(self.view_trans)
            end
        )

        local button = self.transform:Find('2st/bg/window/layout/forbinBtn'):GetComponent(UnityEngine.UI.Button)
        button.onClick:RemoveAllListeners()
        button.onClick:AddListener(
            function ()
                local rst = local_chat:inBlackList(info.playerId)
                if rst then
                    print("取消屏蔽                ", info.playerId)
                    local_chat:removeFromBlackList(info.playerId)
                else
                    print("屏蔽发言                ", info.playerId)
                    local_chat:addToBlackList(info.playerId)
                end
                local rst = local_chat:inBlackList(info.playerId)
                forbin_text.text = rst and NativeText.GetLangStr(210018) or NativeText.GetLangStr(210017)
            end
        )

        local button = self.transform:Find('2st/bg/window/lookForBtn'):GetComponent(UnityEngine.UI.Button)
        button.onClick:RemoveAllListeners()
        button.onClick:AddListener(
            function ()
                print("查看                ", info.playerId)
            end
        )

        self.helper.showTrans(self.view_trans)
    end,
}
]]