local super = require("ui.frame")
local log = logger.get("Game", "Guild Logo")

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        self.hero_id  = self.args.data.tank_cfg_id
        self.owner = self.args.owner

        clickEvents = 
        {
            "close",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self:init()
    end,

    onUnload = function (self)
        self.rtCamera:clear()
    end,

    init = function(self)
        local hero = hero_manager:getHero(self.hero_id)

        if not hero then
            log.error("#### get officer not find hero ! id", self.hero_id)
            return
        end

        self.cfg = hero.tank_cfg
        
        local rds = self.transform:GetComponentsInChildren(UnityEngine.Renderer).Table
        for __, r in ipairs(rds) do
            r.sortingOrder = 11
        end

        self:findText("test_2/name_text").text = self.helper:getLangCfg(self.cfg.name)
        self.effect_trans = self.transform:Find("CommanderGetFrame_Effect")
        self.effect_trans.gameObject:SetActive(false)

        local icon_trans = self.transform:Find("car/icon")

        self.controllers = self.transform:GetComponentsInChildren(UnityEngine.Animator).Table
        self:setControllersState(false)

        self.rtCamera = self.rtCamera or require("gui.tool.frame_renderTexture").new()
        local idx , camera = self.rtCamera:add(self, icon_trans, self.cfg.show_prefab, 500, 
            function(go)
                go.transform.localScale = Vector3.one
                go.transform.localPosition = Vector3(-0.01, -1.4, 15.72)
                go.transform.localRotation = Quaternion.Euler(17.8,-137.6,17.5) 
                
                self.cs_inst = go:GetComponentInChildren("Tank", true)
                self.cs_inst.enabled = false
                self:setMainWeapon()
            end)

        camera.orthographic = false
    end,

    setMainWeapon = function(self, weapon_id)
        self.weapon_id = weapon_id or self.cfg.initial_weapon_id
        self.weapon_cfg = config.weapon_weapons[self.weapon_id]

        if not self.weapon_cfg then
            log.error('no main weapon cfg!!')
            return
        end
        local attach_point = self.cs_inst:FindChild('z_wq')
        local callback = function(go)
            -- callback
            if go ~= nil then
                --log.info("##### main weapon init !!")
                go.name = "main_weapon_"..self.weapon_id
                go.transform:SetParent(attach_point)
                go.transform.localScale = Vector3.one
                go.transform.localPosition = Vector3.zero
                go.transform.localRotation = Quaternion.identity 

                go:SetLayer(core.layers["Model"])

                if self.main_weapon then
                    --log.info("### destroy old main_weapon")
                    self.main_weapon:destroy()
                    self.main_weapon = nil
                end
                self.main_weapon = go

                -- 主武器初始化完成后设置副武器, 有些副武器插槽在主武器上
                -- for slot, item in pairs(self.attachments_id) do
                --     self:setAttachment(slot, item.cfg_id)
                -- end
                
                self:setWheel()

            else
                log.error('initMainWeapon go is nil')
            end
        end

        local weapon_prefab_path = self.weapon_cfg.model_resource
        self:_LoadPrefab(weapon_prefab_path, callback)
    end,

    setWheel = function(self, wheel_id)
        self.wheel_id = wheel_id or self.cfg.initial_wheel_id
        self.wheel_cfg = config.weapon_weapons[self.wheel_id]

        if not self.wheel_cfg then
            log.error('no wheel cfg!!')
            return
        end

        local attach_point = self.cs_inst:FindChild('lt_1')
        local callback = function(go)
            if go ~= nil then
                go.name = "wheel_"..self.wheel_id
                go.transform:SetParent(attach_point)
                go.transform.localScale = Vector3.one
                go.transform.localPosition = Vector3.zero
                go.transform.localRotation = Quaternion.identity 
                go:SetLayer(core.layers["Model"])

                if self.wheel then
                    self.wheel:destroy()
                    self.wheel = nil
                end
                self.wheel = go

                self.effect_trans.gameObject:SetActive(true)
                self:setControllersState(true)
            else
                log.error('initWheel go is nil')
            end
        end

        local wheel_prefab_path = self.wheel_cfg.model_resource
        self:_LoadPrefab(wheel_prefab_path, callback)    
    end,

    _LoadPrefab = function(self, prefab_path, callback)
        self.pool = self.pool or game_director.pool
        local uobject = self.pool:instantiate(prefab_path)
        if not uobject then
            log.error('Failed to load prefab:'..prefab_path)
            return nil
        end

        if uobject.isLoaded then 
            callback(uobject)
        else 
            uobject.onLoaded = callback
        end

        return uobject
    end,

    setControllersState = function(self, state)
        if self.controllers then
            for __, c in ipairs(self.controllers) do
                c.enabled = state
            end
        end
    end,

    click_close = function(self)
        self.owner:onClick_Hero(self.hero_id)
        frame_manager:hide('commander_get')
    end,

    click_confirmBtn = function(self)
        self.owner:onClick_Hero(self.hero_id)
        frame_manager:hide('commander_get')
    end,
}
