--[[
    local tb = {
            title = "标题",
            content = "升级资源不够",
            buttons =
            {
                {
                    btn_name = "取消",
                    callback = function() print("取消") end
                },
                {
                    btn_name = "打开背包",
                    callback = function() print("打开背包") end
                },
            },
            onClose = function(){

            },
            hero = {
                id = 1,
            },
            cost = {
                cfg_id = 1,
                count = 1,
            },
            addCallback = function(){

            },
        }

    frame_manager:show('common_box', nil, tb)
]]

local super = require("ui.frame")

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        self.owner = self.args.owner
        self.content = self.args.data

        clickEvents = {
            "closeBoxBtn",
            "content/bg/addBtn",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)
        
        self:init(self.content)
    end,

    init = function(self, content)
        local title = NativeText.GetLangStr(270014)
        if content.title and string.len(content.title) > 0 then
            title = content.title
        end

        self:findText("content/bg/title").text = title
        self:findText("content/bg/content").text = content.content or ""


        local layout_trans = self.transform:Find("content/bg/layout")
        for i = 0, layout_trans.childCount - 1 do
            layout_trans:GetChild(i).gameObject:SetActive(false)
        end

        for i, v in ipairs(content.buttons) do
            local btn_trans = layout_trans:GetChild(i - 1)
            btn_trans.gameObject:SetActive(true)
            self:findText("content/bg/layout/" .. btn_trans.name .. "/Text").text = v.btn_name
            if v.callback then self:setScript("click", "content/bg/layout/" .. btn_trans.name, v.callback) end
        end

        local icon = self:findImage("content/bg/icon")
        local quality = self:findImage("content/bg/iconBg")
        local tank_type = self:findImage("content/bg/tank_type")
        local cost_icon = self:findImage("content/bg/costIcon")
        local cost_text = self:findText("content/bg/costText")

        local hero = hero_manager:getHero(content.hero.id)
        local officer = hero.officer
        icon:SetSprite(gui_helper:getOfficerIconFromId(content.hero.id))
        quality:SetSprite(gui_helper:getQualityIconPath(officer.cfg.color))
        tank_type:SetSprite(gui_helper:getHeroTypeIconPath(hero.tank_id))

        cost_icon:SetSprite(self.helper:getItemIcon(content.cost.cfg_id))
        cost_text.text = self.helper:getItemNumFormat(content.cost.cfg_id, content.cost.count)
    end,

    onUnload = function (self)
    end,

    click_closeBoxBtn = function(self)
        frame_manager:hide('common_box_1')

        if self.content.onClose then
            self.content.onClose()
        end
    end,

    click_addBtn = function(self)
        if self.content.addCallback then
            self.content.addCallback()
        end
    end,
}
