--[[
    local tb = {
            title = "标题",
            content = "升级资源不够",
            item_id = cfg_id,
            need_count = 1,
            buttons =
            {
                {
                    btn_name = "取消",
                    callback = function() print("取消") end
                },
                {
                    btn_name = "打开背包",
                    callback = function() print("打开背包") end
                },
            },
            onClose = function(){

            }
        }

    frame_manager:show('common_box', nil, tb)
]]

local log = logger.get("Game", "Common Box 3 Frame")
local super = require("ui.frame")

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        self.owner = self.args.owner
        self.content = self.args.data

        clickEvents = {
            "content/closeBoxBtn",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)
        
        self:init(self.content)
    end,

    init = function(self, content)
        local title = NativeText.GetLangStr(999002)
        if content.title and string.len(content.title) > 0 then
            title = content.title
        end
        self:findText("content/bg/title").text = title
        self:findText("content/bg/bg/content").text = content.content or ""


        local layout_trans = self.transform:Find("content/bg/layout")
        for i = 0, layout_trans.childCount - 1 do
            layout_trans:GetChild(i).gameObject:SetActive(false)
        end

        for i, v in ipairs(content.buttons) do
            local btn_trans = layout_trans:GetChild(i - 1)
            btn_trans.gameObject:SetActive(true)
            self:findText("content/bg/layout/" .. btn_trans.name .. "/Text").text = v.btn_name
            if v.callback then self:setScript("click", "content/bg/layout/" .. btn_trans.name, v.callback) end
        end

        local item_node = self.transform:Find("content/bg/bg/Item")
        local res_node = self.transform:Find("content/bg/bg/Res")

        local is_res = content.item_id > 30000 and content.item_id <= 60000
        item_node:SetActive(not is_res)
        res_node:SetActive(is_res)

        local item_cfg = item_helper:getItemCfg(content.item_id)

        if item_cfg then
            if not is_res then
                local grade = self.transform:Find("content/bg/bg/Item"):GetComponent(UnityEngine.UI.Image)
                local icon = self.transform:Find("content/bg/bg/Item/Icon"):GetComponent(UnityEngine.UI.Image)
                local count = self.transform:Find("content/bg/bg/Item/Count"):GetComponent(UnityEngine.UI.Text)

                grade:SetSprite(gui_helper:getQualityIconPath(item_cfg.color))
                icon:SetSprite(gui_helper:getItemIconPath(item_cfg))

                local str = self.helper:getItemNumFormat(content.item_id, 1)
                count.text = str
            else
                local icon = self.transform:Find("content/bg/bg/Res/Icon"):GetComponent(UnityEngine.UI.Image)
                local count = self.transform:Find("content/bg/bg/Res/Count"):GetComponent(UnityEngine.UI.Text)
                
                icon:SetSprite(gui_helper:getItemIconPath(item_cfg))
                count.text = tostring(content.need_count)

                local own_cnt = self.helper:getItemNum(content.item_id)
                if content.need_count > own_cnt then
                    count.color = Color.red
                else
                    count.color = Color.white
                end
            end
        else
            log.error("#################################### find item cfg error ! id ", content.item_id)
        end
    end,

    onUnload = function (self)
    end,

    click_closeBoxBtn = function(self)
        frame_manager:hide('common_box_3')
        if self.content.onClose then
            self.content.onClose()
        end
    end,
}
