local super = require("ui.frame")
local log = logger.get("Game", "Common Speed Up Frame")

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        self.owner = self.args.owner
        self.data = self.args.data

        --[[ data = {
                id,
                left_time,
                total_time,
                type,
                callback,
        } ]]

        clickEvents = 
        {
            'close',
            'content/head/cost/completeBtn',
        }

        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self.content = self.transform:Find("content/Scroll View/Viewport/Content")
        self.prefab = self.transform:Find("content/Scroll View/Viewport/Content/tplt").gameObject

        --[[ local title = self.transform:Find("content/top/Title"):GetComponent(UnityEngine.UI.Text)
        title.text = self.data.title ]]

        self.onBagChanged_event_id = bag_manager.events:on('onBagChanged', functor.make(self.refreshSV, self))
        self.march_speed_up = game_director.events:on("REFRESHMARCHLINE", functor.make(self.refreshPercent, self))

        self:init()
    end,

    onUnload = function (self)
        self.data = nil
        self.owner = nil
        bag_manager.events:remove("onBagChanged", self.onBagChanged_event_id)
        game_director.events:remove("REFRESHMARCHLINE", self.march_speed_up)
    end,

    update = function(self, dt)
        if self.slider and self.data.left_time > 0 then
            self.cost_text.text = self:getCostDiamond(self.data.left_time)
            self.slider.value = 1 - self.data.left_time / self.data.total_time
            self.data.left_time = self.data.left_time - dt
            self:countDown(self.data.left_time)
        end

        if self.data and self.data.left_time <= 0 then
            self:click_close()
        end
    end,

    refreshPercent = function(self, march)
        if self.data.march then
            if march.id == self.data.march.id then
                self.data.total_time = march:getTotalTime()
                self.data.left_time = march:getLeftTime()
            end
        end
    end,

    click_close = function(self)
        if self.owner.speed_up_frame then
            self.owner.speed_up_frame = nil
        end
        frame_manager:hide('common_speed_up')
    end,

    click_completeBtn = function(self)
        if self.data.callback then
            self.data.callback(0, 0, 1)
        end
    end,

    refresh = function(self, data)
        if data.left_time <= 0 then
            self:click_close()
        else
            self.data = data
            self:init()
        end
    end,

    init = function(self)
        local title = self.transform:Find("content/top/Title"):GetComponent(UnityEngine.UI.Text)
        title.text = speedup_helper:getTitle(self.data.type)

        local name_text = self:findText('content/head/name_text')
        name_text.text = self.data.title
        
        self:countDown(self.data.left_time)

        local cost = self.transform:Find("content/head/cost")
        local cost_visible = speedup_helper:isValueType(self.data.type)
        cost:SetActive(cost_visible)

        self.cost_text = self:findText('content/head/cost/cost_text')
        self.slider = self.transform:Find('content/head/Slider'):GetComponent(UnityEngine.UI.Slider)

        self:refreshSV()
        self.inited = true
    end,

    countDown = function(self, value)
        local text = self.transform:Find('content/head/time_text'):GetComponent(UnityEngine.UI.Text)
        text.text = string.format( "%s%s%s", NativeText.GetLangStr(260004), gui_helper:timeFormat(value), NativeText.GetLangStr(260014) )
    end,

    getCostDiamond = function(self, time)
        local items = speedup_helper.items[self.data.type]

        if not items then
            return 0
        end

        --local tab = loadstring("return ".. items[1].buyprice)() 
        local single_price = frame_helper:getDiamondPriceFromStr(cfg.allprice)

        --[[ for item_id, num in pairs(tab) do
            single_price = num / items[1].para1
            break
        end ]]

        time = time / 60

        local rst = math.floor(single_price * time)

        return rst == 0 and 1 or rst
    end,

    --[[ getCostItemNum = function(self, cfg)
        local cnt = 0
        local lt = cnt * tonumber(cfg.para1) * 60

        while lt < self.data.left_time and cnt < cfg.count do
            cnt = cnt + 1
            lt = cnt * tonumber(cfg.para1) * 60
        end

        return cnt
    end, ]]

    refreshSV = function(self)
        self.items = speedup_helper:getItems(self.data.type)
        
        for k, v in ipairs(self.items) do
            local elem = nil

            if k > self.content.childCount then
                elem = Object.Instantiate(self.prefab).transform
                elem:SetParent(self.content)
                resetTransformZ(elem)
            else
                elem = self.content:GetChild(k-1)
            end

            if elem then
                elem:SetActive(true)
                elem.name = "Child_"..k

                self:setItemInfo(elem, v)
            end
        end

        for i=#self.items, self.content.childCount - 1 do
            local child = self.content:GetChild(i)
            child:SetActive(false)
        end
    end,

    setItemInfo = function(self, ui, cfg)
        local grade = ui:Find("out_bg"):GetComponent(UnityEngine.UI.Image)
        local icon = ui:Find("icon"):GetComponent(UnityEngine.UI.Image)
        local name = ui:Find("name_text"):GetComponent(UnityEngine.UI.Text)
        local des = ui:Find("des_text"):GetComponent(UnityEngine.UI.Text)
        local use_node = ui:Find("useTrans")
        local buy_node = ui:Find("buyTrans")

        name.text = self.helper:getLangCfg(cfg.name_id)
        des.text = self.helper:getLangCfg(cfg.des_id)
        icon:SetSprite(gui_helper:getItemIconPath(cfg))
        grade:SetSprite(gui_helper:getQualityIconPath(cfg.color))

        use_node:SetActive(cfg.count > 0)
        buy_node:SetActive(cfg.count <=0)

        if cfg.count > 0 then
            local count = use_node:Find("own_text"):GetComponent(UnityEngine.UI.Text)
            count.text = string.format(NativeText.GetLangStr(260012), cfg.count)

            local path = "content/Scroll View/Viewport/Content/"..ui.name.."/useTrans/useBtn"
            self:clearScript("click", path)
            self:setScript("click", path, functor.rmake(self.onClickUseBtn, cfg))
        else
            local price = buy_node:Find("cost/cost_text"):GetComponent(UnityEngine.UI.Text)
            local res_icon = buy_node:Find("cost/Image"):GetComponent(UnityEngine.UI.Image)
            local costs = loadstring("return ".. cfg.buyprice)() 

            for item_id, need_num in pairs(costs) do
                res_icon:SetSprite(self.helper:getItemIcon(item_id))
                price.text = need_num
                break
            end

            local path = "content/Scroll View/Viewport/Content/"..ui.name.."/buyTrans/buyBtn"
            self:clearScript("click", path)
            self:setScript("click", path, functor.rmake(self.onClickBuyBtn, cfg.id))
        end
    end,

    onClickBuyBtn = function(self, handler, id)
        frame_manager:show('buy_slider', nil, {item_id = id})
    end,

    onClickUseBtn = function(self, handler, cfg)
        if speedup_helper:isMarchLine(self.data.type) then
            network_sender:speedUp(self.data.id, cfg.id, 0)
        else
            local rcmd_cost_num = self:getCostItemNum(cfg.id)
            if cfg.count > 1 then
                frame_manager:show('use_slider', nil, {
                    item_id = cfg.id,
                    item_num = rcmd_cost_num,
                    callback = 
                    function(item_id, num)
                        self:detect(item_id, num, rcmd_cost_num)
                    end,
                })
            else
                self:detect(cfg.id, cfg.count, rcmd_cost_num)
            end
        end
    end,

    getCostItemNum = function(self, item_id)
        local left_time = self.data.left_time
        local free_time = 0
        local minute = (left_time - free_time) / 60
        local item_cost_minute = tonumber(config.item_resources[item_id].para1)
        local count = math.floor(minute / item_cost_minute)
        return count
    end,

    detect = function(self, item_id, actual_cost_num, rcmd_cost_num)
        local sendMsg = function()
            if self.data.callback then
                self.data.callback(item_id, actual_cost_num, 0)
            end
            frame_manager:hide("use_slider")
        end

        if actual_cost_num > rcmd_cost_num then
            local tb = {
                title = "",
                content = NativeText.GetLangStr(260018),
                buttons =
                {
                    {
                        btn_name = NativeText.GetLangStr(130003),
                        callback = function() 
                            frame_manager:hide('common_box')
                        end
                    },
                    {
                        btn_name = NativeText.GetLangStr(130002),
                        callback = function() 
                            frame_manager:hide('common_box')
                            sendMsg()
                        end
                    },
                },
            }
    
            frame_manager:show('common_box', nil, tb)
        else
            sendMsg()
        end
    end,
}
