local log = logger.get("Game", "Home Frame")
local super = require("ui.frame")
local bar_num = 5

return class {
    super = super, 

    onLoaded = function (self, owner)
        super.onLoaded(self)
        self.owner = self.args.owner

        clickEvents = 
        {
            "closeBtn",
        }
        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self:init()
    end,

    onUnload = function (self)
    end,

    init = function(self)
        for i = 1, bar_num do 
            local path = "content/background/layout/Tag (" .. i .. ")"
            self:setScript("click", path, functor.rmake(self.click_barBtn, i))
        end
        self:click_barBtn(nil, 1)
    end,

    click_closeBtn = function(self)
        frame_manager:hide('coord_manager')
    end,

    click_barBtn = function(self, hander, index)
        for i = 1, bar_num do 
            local path = "content/background/layout/Tag (" .. i .. ")"
            self:findButton(path).interactable = index ~= i
        end
        self:onFillScrollViewContent(index)
    end,

    refreshCollectCoord = function(self)
        self:onFillScrollViewContent(self.iTagIndex)
    end,


    -- //坐标收藏信息
    -- message MCollectionPos
    -- {
        -- required uint32 id = 1;
        -- required ECollectionTable eTable = 2;
        -- required string strDesc = 3;//描述
        -- required EWorldObjType objType = 4;
        -- required uint32 iconId = 5;//玩家对应iconid， 怪物说是资源点对应cfgId
        -- required uint32 posX = 6;
        -- required uint32 posY = 7;
        -- required uint32 marchDist = 8;//行军距离
        -- required uint32 marchTime = 9;//行军时间 
    -- }
    onFillScrollViewContent = function(self, iTagIndex)
        self.iTagIndex = iTagIndex
        
        local count = 0
        local enum = 0
        if iTagIndex == 2 then
            enum = common_pb.ECOLLECTION_RES
        elseif iTagIndex == 3 then
            enum = common_pb.ECOLLECTION_FAVORITE
        elseif iTagIndex == 4 then
            enum = common_pb.ECOLLECTION_ENEMY
        elseif iTagIndex == 5 then
            enum = common_pb.ECOLLECTION_FRIEND
        end

        local path = "content/background/svs/Scroll View/Viewport/Content"
        frame_helper:hideAllChild(self, path, true)

        self.none_trans = self.none_trans or self.transform:Find("content/background/svs/none")

        for id, item in pairs(self.owner.collections) do
            if iTagIndex == 1 then
                count = count + 1
                self:onFillChild(count, path, item)
            else
                if item.eTable == enum then
                    count = count + 1
                    self:onFillChild(count, path, item)
                end
            end
        end

        if count > 0 then
            frame_helper.hideTrans(self.none_trans)
        else
            frame_helper.showTrans(self.none_trans)
        end
    end,

    onFillChild = function(self, count, path, item)
        local child = self.helper:getChild(self, count, path)
        child:SetActive(true)

        local des_text = child:Find("timeText"):GetComponent(UnityEngine.UI.Text)
        des_text.text = item.strDesc

        local coord_text = child:Find("disText"):GetComponent(UnityEngine.UI.Text)
        coord_text.text = gui_helper:coordFormat(item.serverid, item.posX, item.posY) 

        local type_icon = child:Find("worldmap_icon_5_1"):GetComponent(UnityEngine.UI.Image)
        local cfg = config.worldmap_collections[item.eTable]
        type_icon:SetSprite(cfg.iconpath .. cfg.tabicon)

        local edit_btn = child:Find("skipButton"):GetComponent(UnityEngine.UI.Button)
        edit_btn.onClick:RemoveAllListeners()
        edit_btn.onClick:AddListener(function()
            self:click_moveBtn(item)
        end)

        local edit_btn = child:Find("deleteButton"):GetComponent(UnityEngine.UI.Button)
        edit_btn.onClick:RemoveAllListeners()
        edit_btn.onClick:AddListener(function()
            self:click_deleteBtn(item)
        end)

        local edit_btn = child:Find("editBtn"):GetComponent(UnityEngine.UI.Button)
        edit_btn.onClick:RemoveAllListeners()
        edit_btn.onClick:AddListener(function()
            self:click_editBtn(item)
        end)
    end,

    click_editBtn = function(self, mCollectionPos)
        self:showEditPos(mCollectionPos)
    end,

    click_moveBtn = function(self, mCollectionPos)
        self.owner:setCameraPos(mCollectionPos.posX, mCollectionPos.posY)
        frame_manager:hide('coord_manager')
    end,

    click_deleteBtn = function(self, mCollectionPos)
        network_sender:worldMapCollectionDel(mCollectionPos.id)
    end,

    showEditPos = function(self, collect_pos)
        frame_manager:show('collect_coord', self, {
            pos = collect_pos,
            is_edit = true,
        })
    end,
}
