local log = logger.get("Game", "Guild Assemble Frame")
local super = require("ui.frame")

return class{

    super = super,

    onLoaded = function(self)

        super.onLoaded(self)

        self.owner = self.args.owner
        self.to_hold = self.args.data.hold
        self.before_frame = self.args.data.before_frame

        -- 路径: root/login
        -- 事件: click_login 
        clickEvents = {
            "Close",
            "BG/Cancel",
            "BG/Confirm",
        }
 
        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        local position = self.transform:Find("BG/Top/Position"):GetComponent(UnityEngine.UI.Text)
        position.text = string.format( "D:%d X:%d Y:%d", 1, self.to_hold.coordX, self.to_hold.coordY )

        self.idx = 1
        self.toggles = {}

        self:init()
    end,

    onUnloaded = function(self)
        if self.before_frame then
            self.before_frame = nil
        end
    end,

    init = function(self)
        local panel = self.transform:Find("BG/Center")
        local cfg = config.worldmap_others[1]

        if cfg then
            local times = loadstring("return "..cfg.Rally_Time)()
            for i=1, panel.childCount do
                local elem = panel:GetChild(i-1)
                if times[i] then
                    local text = elem:Find("Text"):GetComponent(UnityEngine.UI.Text)
                    if times[i] > 59 then
                        text.text = string.format( "%d %s", math.floor(times[i] / 60), NativeText.GetLangStr(281806) )
                    else
                        text.text = string.format( "%d %s", times[i], NativeText.GetLangStr(281805) )
                    end

                    local toggle = elem:Find("Toggle"):GetComponent(UnityEngine.UI.Toggle)
                    table.insert(self.toggles, toggle)

                    toggle.onValueChanged:RemoveAllListeners()
                    toggle.onValueChanged:AddListener(
                        function ()
                            self:onValueChanged(i)
                        end
                    )

                    elem.transform.localScale = Vector3.one
                else
                    elem.transform.localScale = Vector3.zero
                end
            end

            self.times = times
        else

        end
    end,

    update = function(self, dt)
        if self.toggles then
            for k, v in ipairs(self.toggles) do
                if self.idx ~= k then
                    v.isOn = false
                end
            end
        end
    end,

    onValueChanged = function(self, idx)
        if self.toggles[idx].isOn then
            self.idx = idx
        end
    end,

    toggleCheck = function(self)
        for k, v in ipairs(self.toggles) do
            if v.isOn then
                self.duration = self.times[k]
                return true
            end
        end

        return false
    end,

    click_Close = function(self)
        frame_manager:hide("create_mass")
    end,

    click_Confirm = function(self)
        if not self:toggleCheck() then
            hint:setText(NativeText.GetLangStr(281807))
            return
        end

        local soldier_cnt = home_info:getAvailableSoldiers()

        if #soldier_cnt == 0 then
            hint:setText(NativeText.GetLangStr(900012))
            return
        end

        frame_manager:show("launch_rally", self.owner, {monster = self.to_hold, action = common_pb.EMARCH_CREATE_MASS, duration = self.duration, spy = true})
        self:hide()
    end,

    click_Cancel = function(self)
        self:click_Close()
    end,

    hide = function(self)
        if self.before_frame then
            self.before_frame:hide()
        end
        self:click_Close()
    end,
}