require('game.lord')
local super = require("ui.frame")
local log = logger.get("Game", "crew equip frame")
local max_num = 100
local per_times = 1

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)
        self.mod_id  = self.args.data.mod_id -- 当前是哪个房间,模块 id
        self.room_cfg = config.crew_type_indexs[self.mod_id]
        self.cells = {}   -- 船员对应的界面元素
        self.selected_idx = 0
        self.event_ids = {}       

        clickEvents = 
        {
            'closeBtn',
            'Root/ButtonCreate',
            'Root/Right/buttons/ButtonLevelUp',
            'Root/Right/buttons/ButtonEquip',
            'Root/Right/buttons/ButtonUnequip',
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)
        self.event_ids['onEquipChanged'] = crew_manager.events:on('onEquipChanged', functor.make(self.refresh, self))

        self:init()
    end,

    init = function(self)
        self.crew_content = self.transform:Find('Root/LeftScrollView/Viewport/Content')
        self.crew_prefab = self.transform:Find('Prefab/Crew')
        self.prop_group_prefab = self.transform:Find('Prefab/PropGroup')

        self.icon_events = {}

        self:refresh()



        -- 房间名称
        self:setText("Root/Right/text_cooking_room",self.helper:getLangCfg(self.room_cfg.CrewRoomName))        
     end,

    -- 刷新船员
    refresh = function(self)
        self:clearIconScripts()
        self.crews = crew_manager:getCrewBySlot(self.mod_id, true)  -- 可装备当前船舱的所有船员
        if #self.crews > 0 then
            self.transform:Find("Root/Right"):SetActive(true)
            self.transform:Find('Root/text_NoCrew'):SetActive(false)
        else
            self.transform:Find("Root/Right"):SetActive(false)
            self.transform:Find('Root/text_NoCrew'):SetActive(true)
        end

        for idx, crew in ipairs(self.crews) do
            local cell = self.cells[idx]
            if not cell then
                cell = Object.Instantiate(self.crew_prefab).transform
                self.cells[idx] = cell
                cell:SetParent(self.crew_content)
                cell.localPosition = Vector3.zero
                cell.localScale = Vector3.one
                cell.name = 'cell_'..idx
            end
            
            cell:SetActive(true)

            -- 装备标记
            local equip_icon = cell:Find("QualityFrame/EquipIcon")
            if crew.slot > 0 then
                -- 已装备
                equip_icon:SetActive(true)
            else
                -- 未装备
                equip_icon:SetActive(false)
            end

            -- 品质
            local quality_img = cell:Find("QualityFrame"):GetComponent(UnityEngine.UI.Image)
            quality_img:SetSprite(gui_helper:getQualityIconPath(crew.cfg.color))

            -- 等级
            local level_tex = cell:Find("QualityFrame/LevelBG/LevelText"):GetComponent(UnityEngine.UI.Text)
            level_tex.text = 'Lv.'..tostring(crew.cfg.UserLevel)
            if crew.cfg.UserLevel >= local_lord.level then
                level_tex.color = Color.white
            else
                level_tex.color = Color.red
            end

            -- icon
            local icon_img = cell:Find("QualityFrame/Icon"):GetComponent(UnityEngine.UI.Image)
            local img_path = crew.cfg.path.. crew.cfg.icon
            icon_img:SetSprite(img_path)

            -- star
            for i=1,5 do
                local star_img = cell:Find('QualityFrame/StarBG/star_'..i):GetComponent(UnityEngine.UI.Image)
                if i <= crew.cfg.color then
                    star_img:SetSprite('Assets/Arts/ui/atlas/common/Prison_icon_1_1.png')
                else
                    star_img:SetSprite('Assets/Arts/ui/atlas/common/Prison_icon_1_2.png')
                end
            end

            -- 注册点击事件
            local path = 'Root/LeftScrollView/Viewport/Content/cell_'..idx
            self:setScript('single', path, functor.rmake(self.onClick_ItemIcon, idx))
            self.icon_events[path] = 'single'
        end

        local startIdx = #self.crews+1
        local endIdx = #self.cells
        for i = startIdx, endIdx do
            local cell = self.cells[i]
            if cell then
                cell:SetActive(false)
            end
        end

        -- 默认选择第一个
        if #self.crews > 0 then
            self:select(1)
        end
    end,

    -- 刷新右侧船员信息
    refreshCrewInfo = function(self)
        if self.selected_idx > 0 then
            local crew = self.crews[self.selected_idx]
            if not crew then
                log.error("Failed to find crew:"..self.selected_idx)
                return
            end

            self:setText("Root/Right/text_crew_name", self.helper:getLangCfg(crew.cfg.name_id))

            local level_tex = self.transform:Find('Root/Right/text_captain_level'):GetComponent(UnityEngine.UI.Text)
            level_tex.text = crew.cfg.UserLevel
            if crew.cfg.UserLevel <= local_lord.level then
                level_tex.color = Color(0.05, 1, 05)
            else
                level_tex.color = Color.red
            end

            -- 按照属性类别分类 整理船员属性加成
            local prop_category = {}
            local max_category = 0
            local raw_tab = loadstring('return '..crew.cfg.EffectPara)()
            for id,value in pairs(raw_tab) do
                local prop_name, prop_value = gui_helper:getPropDesc(id, value)
                local category, category_name = self:getPropertyCategory(id)
                if not category then
                    log.error("Failed to find prop category:"..tostring(id))
                else
                    if not prop_category[category] then
                        prop_category[category] = {}
                    end
                    prop_category[category].name = category_name
                    table.insert(prop_category[category], 
                        {name = prop_name, value = prop_value})
                        if category > max_category then
                            max_category = category
                        end                    
                end

            end

            local prop_panel = self.transform:Find('Root/Right/PropPanel')
            -- 遍历所有 category
            local idx = 1
            for i = 1, max_category do
                local category_tab = prop_category[i]
                if category_tab then
                    local group = nil                    
                    if prop_panel.childCount > idx - 1 then
                        group = prop_panel:GetChild(idx - 1)
                    else
                        group = Object.Instantiate(self.prop_group_prefab)
                        group:SetParent(prop_panel)
                        group:SetActive(true)
                        group.localScale = Vector3.one   
                        group.localPosition = Vector3.zero                     
                    end

                    -- 设置 category name
                    local text_category = group:Find('TitleBG/text_category'):GetComponent(UnityEngine.UI.Text)
                    text_category.text = self.helper:getLangCfg(category_tab.name)

                    -- 设置 category 的属性
                    for i=1,4 do
                        local prop = category_tab[i]
                        local prop_ui = group:Find('Prop'..i)                       
                        if prop then
                            prop_ui:SetActive(true)
                            prop_ui:Find('Name'):GetComponent(UnityEngine.UI.Text).text = prop.name
                            prop_ui:Find('Value'):GetComponent(UnityEngine.UI.Text).text = '+' .. prop.value
                        else
                            prop_ui:SetActive(false)
                        end
                    end

                    idx = idx + 1
                end 
            end
        else
            self:clearCrewInfo()
        end
    end,

    -- 获取属性所属类别
    getPropertyCategory = function(self, id)
        local ret = nil
        config.crew_filters:foreach(function(k,v)
            if v.Attribute == id then
                ret = v
            end
        end)

        if ret then
            return ret.Index, ret.IndexName
        else
            return nil
        end
    end,

    -- 清空船员信息
    clearCrewInfo = function(self)
        self:setText("", self.helper:getLangCfg(crew.cfg.name_id))
        self:setText("", crew.cfg.UserLevel)
    end,

    onClick_ItemIcon = function(self, handler, data, idx)
        log.info('### click item:'..idx)
        self:select(idx)
    end,
    
    clearIconScripts = function(self)
        for path, evt in pairs(self.icon_events) do
            self:clearScript(evt, path)
        end
        self.icon_events = {}

    end,

    -- 选择一个船员
    select = function(self, idx)
        -- 取消原来的选中
        if self.selected_idx > 0 then
            local cell = self.cells[self.selected_idx]
            local select_img = cell:Find('SelectIcon')
            select_img:SetActive(false)
        end

        local cell = self.cells[idx]
        local select_img = cell:Find('SelectIcon')
        select_img:SetActive(true)

        -- 判断选中的是否在装备中
        local crew = self.crews[idx]
        if crew then
            if crew.slot > 0 then
                self.transform:Find("Root/Right/buttons/ButtonUnequip"):SetActive(true)
                self.transform:Find("Root/Right/buttons/ButtonEquip"):SetActive(false)
            else
                self.transform:Find("Root/Right/buttons/ButtonUnequip"):SetActive(false)
                self.transform:Find("Root/Right/buttons/ButtonEquip"):SetActive(true)
            end
        else
            log.error("Select no crew!")
        end

        self.cur_crew = crew
        self.selected_idx = idx

        self:refreshCrewInfo()
    end,

    onUnload = function (self)
        crew_manager.events:remove("onEquipChanged", self.event_ids["onEquipChanged"] )

    end,

    click_closeBtn = function(self)
        frame_manager:hide('crewEquip')
    end,

    click_ButtonCreate = function(self)
        log.info("## click_ButtonCreate")
        
        if not self:isBuildCompleted() then
            hint:setText(NativeText.GetLangStr(620051))
            return
        end

        frame_manager:hide('lord_info')
        frame_manager:hide('crewEquip')

        frame_manager:show('home_forge_preview', nil, self.mod_id)
    end,

    click_ButtonEquip = function(self)
        log.info("## click_ButtonEquip")
        local crew = self.crews[self.selected_idx]
        if crew then
            -- 向服务器请求装备船员
            network_sender:crewEquip(crew.inst_id, self.mod_id)
        else
            log.error('No crew selected!!')
        end
    end, 

    click_ButtonUnequip = function(self)
        log.info("## click_ButtonUnequip")
        local crew = self.crews[self.selected_idx]
        if crew then
            -- 向服务器请求装备船员
            network_sender:crewUnequip(crew.inst_id, self.mod_id)
        else
            log.error('No crew selected!!')
        end
    end,

    click_ButtonLevelUp = function(self)
        if not self:isBuildCompleted() then
            hint:setText(NativeText.GetLangStr(620051))
            return
        end

        log.info("## click_ButtonLevelUp")
        frame_manager:hide('lord_info')
        frame_manager:hide('crewEquip')

        if self.cur_crew then
            local target_cfg = self.cur_crew.cfg
            local rst = nil
            config.crew_blocks:foreach(function(id, cfg)
                if cfg.Group == target_cfg.Group and cfg.Slot == target_cfg.Slot and cfg.color == (target_cfg.color + 1) then
                    rst = cfg
                end
            end)
            if rst then
                frame_manager:show('home_forge_upgrade', nil, rst)
            end
        end
    end,

    isBuildCompleted = function(self)
        local build_data = home_info:getBuildDataByCfgId(22)
        if build_data then
            return true
        end
        return false
    end,
}

