local super = require("ui.frame")
local log = logger.get("Game", "Daily Task Frame")
local gui_helper = require('gui.gui_helper')
local item_cls = require('RTS.bag_sys.item')

-- 跳转界面信息
local jump_frames = {
    [1] = {name = "hero_list", params = {}},
    [2] = {name = "bag", params = {bag_type = 1}},
    [3] = {name = "worldmap", params = {}},
    [4] = {name = "repairkit", params = {}},
    [5] = {name = "repairhero", params = {}},
    [6] = {name = "owntech", params = {}},
    [7] = {name = "armory", params = {}},
    [8] = {name = "normalshop", params = {}},
    [9] = {name = "guildshop", params = {}},
    [10] = {name = "guild", params = {}},
    [11] = {name = "home", params = {}},
}

return class
{
    super = super,

    onLoaded = function (self)
        super.onLoaded(self)

        -- 路径: root/login
        -- 事件: click_login 
        clickEvents = {
            "BG/Title/BackBtn",
            "BG/DailyBtn",
            "BG/AchiveBtn",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self.task_normal_prefab = self.transform:Find('Prefabs/NormalTask').gameObject
        --self.task_finished_prefab = self.transform:Find('Prefabs/FinishedTask').gameObject
        self.scroll_content = self.transform:Find('BG/Right/Scroll View/Viewport/Content')
        self.slider = self:findSlider('BG/Left/Slider')
        self.active_value = self:findText('BG/PointCount/Value')
        self.refresh_time = self:findText('BG/Right/RefreshTime')

        local t = string.format("%02d:00", config.task_configs[1].resettime / 60)
        self.refresh_time.text = string.format(NativeText.GetLangStr(150013), t)

        -- 活跃度阶段
        self.active_stage_panel = self.transform:Find('BG/Left')

        self.page_btns = {}
        self.page_btns['daily'] = self.transform:Find("BG/DailyBtn"):GetComponent(UnityEngine.UI.Button)
        self.page_btns['achive'] = self.transform:Find("BG/AchiveBtn"):GetComponent(UnityEngine.UI.Button)

        self.cur_page = "daily"

        --缓存点击事件
        self.activeness_events = {}

        self.task_prefabs = {}

        self.changed_evts = {}
        self.changed_evts["onTaskChanged"] = task_manager.events:on("onTaskChanged", functor.make(self.onTaskChanged, self))
        self.changed_evts["onTaskFinished"] = task_manager.events:on("onTaskFinished", functor.make(self.onTaskFinished, self))
        self.changed_evts["onActivenessChanged"] = task_manager.events:on("onActivenessChanged", functor.make(self.onActivenessChanged, self))
        self.get_items_evt = game_director.events:on("NOTIFYGETITEM", functor.make(self.onGetItems, self))

        self:init()
        self:onActivenessChanged()

        guide_manager:tryStartGuide()
    end,

    onUnload = function (self)
        for k, v in pairs(self.changed_evts) do
            task_manager.events:remove(k, v)
        end
        game_director.events:remove("NOTIFYGETITEM", self.get_items_evt)
    end,

    onRcvMsg = function(self, cmd, msg)
    end,

    update = function(self)
       
    end,

    onTaskChanged = function(self, tasks)
        for _, task in ipairs(tasks) do
            local task_trans = self.task_prefabs[task.id]
            if task_trans then
                if task.state == common_pb.TS_TODAYOVER then
                    self:onTaskFinished(task.id)
                else
                    self:updateNormalTrans(task_trans, task)
                end
            end
        end
    end,

    onTaskFinished = function(self, id)
        local tasks = task_manager:getTasks(1)

        self.task_prefabs[id]:SetAsLastSibling()
        self.task_prefabs[id]:SetActive(false)

        self:createNormalTrans(tasks[id])
    end,

    onActivenessChanged = function(self)
        self:clearActivenessEvents()
        local value = task_manager.activeness.value
        self.active_value.text = tostring(value)
        self.slider.value = task_manager.activeness:getPersent()
        for i=1, 5 do
            self:setActivenessStage(i)
        end
    end,

    onGetItems = function(self, msg)
        if msg.reason == common_pb.TASK_REWARD then
            
        elseif msg.reason == common_pb.ACTIVE_REWARRD then
           frame_manager:show('getItems', self, {items = gui_helper.drop_list[msg.reason], has_Confirm_Btn = true})
        end
    end,

    clearActivenessEvents = function(self)
        for path, evt in pairs(self.activeness_events) do
            self:clearScript(evt, path)
        end
        self.activeness_events = {}
    end,

    init = function(self)
        local tasks = task_manager:getSortTasks(1)
        for _, task in ipairs(tasks) do
            --if task.state ~= common_pb.TS_TODAYOVER then
                self:createNormalTrans(task)
            --else
            --    self:createFinishedTrans(task)
            --end
        end
    end,

    --设置活跃值进度
    setActivenessStage = function(self, index)
        local cfg = config.task_dailyactives[index]
        local need_value = cfg.needactive
        local active_stage = self.active_stage_panel:Find("Stage_"..index)
        local icon = active_stage:Find('State/Icon'):GetComponent(UnityEngine.UI.Image)

        -- 设置道具 icon
        --[[local item_id = task_manager.activeness:getReward().id
        local item_cfg = self.helper:getItemCfg(item_id)
        local icon_path = gui_helper:getItemIconPath(item_cfg)
        icon:SetSprite(icon_path)]]

        -- 设置领取状态
        local state_text = active_stage:Find('State/Text'):GetComponent(UnityEngine.UI.Text)
        local value_text = active_stage:Find('State/Activity'):GetComponent(UnityEngine.UI.Text)
        local state_img = active_stage:Find('State'):GetComponent(UnityEngine.UI.Image)
        local line_img = active_stage:GetComponent(UnityEngine.UI.Image)
        local light = active_stage:Find("Point/Light")
        local dark = active_stage:Find("Point/Dark")
        local icon = active_stage:Find("State/Icon"):GetComponent(UnityEngine.UI.Image)
        local btn = active_stage:Find("State"):GetComponent(UnityEngine.UI.Button)

        btn.onClick:RemoveAllListeners()
        icon:SetSprite(config.task_dailyactives[index].icon)

        light:SetActive(false)
        dark:SetActive(true)

        -- icon 路径
        local path = "BG/Left/"..active_stage.name.."/State"
        local str = NativeText.GetLangStr(150007)
        local color = Color.white
        local state_img_path = res_path.res_path_56
        local line_img_path = res_path.res_path_59

        if task_manager.activeness.value >= need_value then
            if task_manager.activeness.stages[index] then
                str = NativeText.GetLangStr(150005)
                color = Color.gray
                state_img_path = res_path.res_path_58
                line_img_path = res_path.res_path_61
                light:SetActive(true)
                dark:SetActive(false)
                self.helper:setImageGray1(icon, true)
            else
                str = NativeText.GetLangStr(150006)
                color = Color(4/255, 234/255, 249/255, 1)
                state_img_path = res_path.res_path_57
                line_img_path = res_path.res_path_60
                light:SetActive(true)
                dark:SetActive(false)
                self.helper:setImageGray1(icon, false)
                -- 注册点击事件
                --[[ self:setScript("click", path, functor.rmake(self.onClick_activeStage, index))
                self.activeness_events[path] = "click" ]]
                btn.onClick:AddListener(
                    function()
                        self:onClick_activeStage(index)
                    end
                )
            end

        else
            btn.onClick:AddListener(
                function()
                    local list = {}
                    local rewards = task_manager.activeness:getReward(index)

                    if rewards then
                        for id, count in pairs(rewards) do
                            local item_inst = {}
                            item_inst = item_cls({cfgId = id, count = count})

                            table.insert(list, item_inst)
                        end
                    end
                    
                    frame_manager:show('getItems', self, {items = list, has_Confirm_Btn = true, message = string.format(NativeText.GetLangStr(999030), need_value)})
                end
            )
        end
        state_text.color = color
        state_text.text = str
        value_text.text = tostring(need_value)
        state_img:SetSprite(state_img_path)
        line_img:SetSprite(line_img_path)
    end,

    --创建未领取奖励任务panel
    createNormalTrans = function(self, task)
        local task_trans = Object.Instantiate(self.task_normal_prefab).transform
        task_trans:SetParent(self.scroll_content)
        resetTransformZ(task_trans)
        
        if task.state ~= common_pb.TS_TODAYOVER then
            self:updateNormalTrans(task_trans, task)
        else
            self:updateFinishedTrans(task_trans, task)
        end

        --注册点击事件
        task_trans.name = task_trans.name..task.id
        local btn_path = "BG/Right/Scroll View/Viewport/Content/"..task_trans.name.."/Button"
        self:setScript("click", btn_path, functor.rmake(self.onClick_GetReward, task))
        
        self.task_prefabs[task.id] = task_trans
    end,

    updateNormalTrans = function(self, task_trans, task)
        --设置按钮状态
        local btn_image = task_trans:Find('Button'):GetComponent(UnityEngine.UI.Image)
        local btn_text = task_trans:Find('Button/Text'):GetComponent(UnityEngine.UI.Text)
        btn_image.transform:SetActive(true)

        if task.state == common_pb.TS_GETAWARD then
            btn_image:SetSprite(res_path.res_path_42)
            btn_text.text = NativeText.GetLangStr(150009)
        else
            btn_image:SetSprite(res_path.res_path_55)
            btn_text.text = NativeText.GetLangStr(150008)
        end

        local finished = task_trans:Find("Finished")
        finished:SetActive(false)

        self:setTaskInfo(task_trans, task)
        self:setAllChildGray(task_trans, false)
    end,

    --创建已领取奖励任务panel
    updateFinishedTrans = function(self, task_trans, task)
        local btn = task_trans:Find('Button')
        local finished = task_trans:Find("Finished")
        finished:SetActive(true)
        btn:SetActive(false)

        self:setTaskInfo(task_trans, task)
        self:setAllChildGray(task_trans, true)
        task_trans:SetAsLastSibling()
    end,

    setAllChildGray = function(self, trans, gray)
        for i=0, trans.childCount - 1 do
            local child = trans:GetChild(i)

            local img = child:GetComponent(UnityEngine.UI.Image)
            local text = child:GetComponent(UnityEngine.UI.Text)

            if img then
                self.helper:setImageGray1(img, gray)
            end

            if text then
                self.helper:setImageGray1(text, gray)
            end

            if child.childCount > 0 then
                self:setAllChildGray(child, gray)
            end
        end
    end,

    -- 设置任务信息
    setTaskInfo = function(self, task_trans, task)
        local task_icon = task_trans:Find('Icon'):GetComponent(UnityEngine.UI.Image)
        local task_des = task_trans:Find('Des'):GetComponent(UnityEngine.UI.Text)
        local task_title = task_trans:Find('Title'):GetComponent(UnityEngine.UI.Text)

        local icon_path = task.cfg.path..task.cfg.icon
        task_icon:SetSprite(icon_path)
        task_title.text = self.helper:getLangCfg(task.cfg.title)
        task_des.text = self.helper:getLangCfg(task.cfg.des_id)
        local prog_str = string.format( "(%d/%d)", task.progress.prog, task.progress.total )
        task_des.text = string.format("%s %s", task_des.text, prog_str)

        local rewards = loadstring("return "..task.cfg.reward)()
        local active_num = 0
        for k, v in pairs(rewards) do
            if k == 30006 then
                active_num = v
                break
            end
        end

        if active_num > 0 then
            task_title.text = string.format("%s    <color=#40C7FFFF>%s+%d</color>", task_title.text, NativeText.GetLangStr(9000004), active_num)
        end
        --[[ -- 设置任务奖励
        local rewards = task.cfg.reward
        local index = 0
        for k, v in pairs(self.helper:getJsonTab(rewards)) do
            if index >= reward_panel.childCount then
                log.error("task reward too much !!!")
                return
            end
            local reward_trans = reward_panel:GetChild(index)
            reward_trans:SetActive(true)
            local icon = reward_trans:Find('Icon'):GetComponent(UnityEngine.UI.Image)
            local count = reward_trans:Find('text17_Static_16'):GetComponent(UnityEngine.UI.Text)
            icon:SetSprite(gui_helper:getItemIconPath(self.helper:getItemCfg(k)))
            count.text = "x"..v
            index = index + 1
        end

        for i=index, reward_panel.childCount - 1 do
            reward_panel:GetChild(i):SetActive(false)
        end ]]
    end,

    -- 点击任务按钮
    onClick_GetReward = function(self, handler, task)
        log.info('onClick_GetReward '..task.id)
        if task.state == common_pb.TS_GETAWARD then  --可领取状态
            network_sender:getNorTaskRewards(task.id, common_pb.Norm_Task)
        elseif task.state == common_pb.TS_DOING then  --进行中状态
            -- 界面跳转
            self:jump(task.cfg.jump)
        end
    end,

    jump = function(self, index)
        if index == 1 or index == 2 then
            frame_manager:show(jump_frames[index].name, self, jump_frames[index].params)
        elseif index == 3 then
            if not game_director:isWorldMapState() then
                game_director:changeState('loading', {next_state = 'world_map', scene_name = 'hex_map'})
            end
        elseif index == 4 then
            if game_director:isWorldMapState() then
                game_director:changeState('loading', {next_state = 'home', scene_name = 'home'})
            else
                local entity = home_info:getBuildDataByCfgId(9)
                
                if entity then
                    if entity.level > 0 then
                        network_sender:getFactoryMakeInfo()
                    else
                        hint:setText(NativeText.GetLangStr(9000023))
                    end
                else
                    log.error("################################# get home entity error ! cfg id ", 9)
                end
            end
        elseif index == 5 then
            if game_director:isWorldMapState() then
                game_director:changeState('loading', {next_state = 'home', scene_name = 'home'})
            else
                local entity = home_info:getBuildDataByCfgId(9)
                
                if entity then
                    if entity.level > 0 then
                        network_sender:getRepairInfo()
                    else
                        hint:setText(NativeText.GetLangStr(9000023))
                    end
                else
                    log.error("################################# get home entity error ! cfg id ", 9)
                end
            end
        elseif index == 6 then
            if game_director:isWorldMapState() then
                game_director:changeState('loading', {next_state = 'home', scene_name = 'home'})
            else
                local entity = home_info:getBuildDataByCfgId(2)
                
                if entity then
                    if entity.level > 0 then
                        network_sender:personBaseQuery(entity.id)
                    else
                        hint:setText(NativeText.GetLangStr(9000024))
                    end
                else
                    log.error("################################# get home entity error ! cfg id ", 2)
                end
            end
        elseif index == 7 then
            if game_director:isWorldMapState() then
                game_director:changeState('loading', {next_state = 'home', scene_name = 'home'})
            else
                local entity = home_info:getBuildDataByCfgId(11)
                
                if entity then
                    if entity.level > 0 then
                        network_sender:getArmoryInfo()
                    else
                        hint:setText(NativeText.GetLangStr(9000025))
                    end
                else
                    log.error("################################# get home entity error ! cfg id ", 11)
                end
            end
        elseif index == 8 or index == 9 then
            if game_director:isWorldMapState() then
                game_director:changeState('loading', {next_state = 'home', scene_name = 'home'})
            else
                local entity = home_info:getBuildDataByCfgId(12)
                
                if entity then
                    if entity.level > 0 then
                        network_sender:getShopList()
                    else
                        hint:setText(NativeText.GetLangStr(9000026))
                    end
                else
                    log.error("################################# get home entity error ! cfg id ", 12)
                end
            end
        elseif index == 10 then
            if local_guild:inGuild(local_guild:getId()) then
                network_sender:getFamilyInfo(local_player:getGuildId())
            else
                network_sender:getFamilyList(0)
            end
        elseif index == 10 then
            if game_director:isWorldMapState() then
                game_director:changeState('loading', {next_state = 'home', scene_name = 'home'})
            end
        end

        if jump_frames[index] then
            self:click_BackBtn()
        end
    end,

    -- 领取活跃值奖励
    onClick_activeStage = function(self, stage)
        network_sender:getNorTaskRewards(stage, common_pb.ACTIVE_POINT)
    end,

    click_BackBtn = function(self)
        frame_manager:hide("daily_task")
    end,

    click_DailyBtn = function(self)
        
    end,

    click_AchiveBtn = function(self)
        
    end,
}