local log = logger.get("Game", "Dialogue Frame")
local super = require("ui.frame")

return class{
    super = super,

    onLoaded = function(self)
        super:onLoaded(self)

        self.dialogs = self.args.data.scenario
        self.finish_callback = self.args.data.callback

        self.helper = require("gui.tool.frame_helper")
        
        self.left_npc = self.transform:Find("BG/Left")
        self.right_npc = self.transform:Find("BG/Right")
        self.center_npc = self.transform:Find("BG/Center")
        self.bg = self.transform:Find("BG")
        self.dialog_box = self.transform:Find("BG/DialogPanel")
        self.dialog_label = self.transform:Find("BG/DialogPanel/Text"):GetComponent(TypewriterEffect)
        self.left_name = self.transform:Find("BG/DialogPanel/LeftName")
        self.right_name = self.transform:Find("BG/DialogPanel/RightName")
        self.left_name_text = self.left_name:GetChild(0):GetComponent(UnityEngine.UI.Text)
        self.right_name_text = self.right_name:GetChild(0):GetComponent(UnityEngine.UI.Text)

        self.prefab_custom_image = self.transform:Find("Prefab/CustomImage").gameObject
        self.transform:Find("Prefab").gameObject:SetActive(false)

        self.dialog_box.localScale = Vector3(1,0,1)
        self.left_npc:GetComponent(UnityEngine.UI.Image).color = Color(1,1,1,0)
        self.right_npc:GetComponent(UnityEngine.UI.Image).color = Color(1,1,1,0)
        self.center_npc:GetComponent(UnityEngine.UI.Image).color = Color(1,1,1,0)

        self.left_orig = Vector3.Clone(self.left_npc.localPosition)
        self.right_orig = Vector3.Clone(self.right_npc.localPosition)
        self.center_orig = Vector3.Clone(self.center_npc.localPosition)
        --[[
        local pos = self.left_npc.localPosition
        pos.x = -self.left_npc.rect.width * 0.5
        self.left_npc.localPosition = pos

        pos = self.right_npc.localPosition
        pos.x = self.right_npc.rect.width * 0.5
        self.right_npc.localPosition = pos
]]
        pos = self.center_npc.localPosition
        pos.z = 2000
        self.center_npc.localPosition = pos

        self.imgNext = self.transform:Find("BG/DialogPanel/ImgNext")

        self:setScript("click", "BG", functor.make(self.onClickBG, self))

        self.playing = false
        self.dialog_end = false
        self.scenario_end = false

        self.left_npc_display = false
        self.right_npc_display = false
        self.center_npc_display = false

        self.spine_assets = {}

        self:start()

        dialog_frame = self
        self.dialog_script = require("gui.dialog_script")(self)

    end,

    start = function(self)
        self.index = 1
        self.dialog = self.dialogs[1]

        self:dialogPlay()
    end,

    onUnload = function(self)
        
        for k,v in pairs(self.spine_assets) do
            if v then
                v:RemoveRef()
            end
        end

        dialog_frame = nil
        if self.dialog_script then
            self.dialog_script:dtor()
            self.dialog_script = nil
        end
    end,

    -- 播放对话
    dialogPlay = function(self)
        log.info("#### dialogPlay:".. self.dialog.id)
        self.imgNext:SetActive(false)

        self:npcEnter()

        app:startCoroutine(
            function()
                self:setNPCName()
                self:boxDisplay()
                coroutine.yield( WaitForSeconds(0.5) )
                self.playing = true
                self:setContent()

                self:playDialogSound()
                self:playDialogEmotion()
                self:playPerformance()
            end
        )
    end,

    -- 设置NPC名称
    setNPCName = function(self)
        if self.dialog.frame == 1 then
            self.left_name:SetActive(true)
            self.right_name:SetActive(false)
            self.left_name_text.text = self.helper:getLangCfg(self.dialog.charname)
        elseif self.dialog.frame == 2 then
            self.left_name:SetActive(false)
            self.right_name:SetActive(true)
            self.right_name_text.text = self.helper:getLangCfg(self.dialog.charname)
        end
    end,

    -- npc进场
    npcEnter = function(self)
        if self.dialog.leftpic ~= "" and not self.left_npc_display then
            --log.info("### leftNpcDisplay:"..tostring(self.dialog.leftpic))
            self:leftNpcDisplay()
        elseif self.left_npc_display then
            --log.info("### leftNpcDisplay setNpcOpacity")
            self:setNpcOpacity(self.left_npc, 1)
        end

        if self.dialog.centerpic ~= "" and not self.center_npc_display then
            --log.info("### centerNpcDisplay:"..tostring(self.dialog.centerpic))
            self:centerNpcDisplay()
        elseif self.center_npc_display then
            --log.info("### centerNpcDisplay setNpcOpacity")
            self:setNpcOpacity(self.center_npc, 2)
        end

        if self.dialog.rightpic ~= "" and not self.right_npc_display  then
           -- log.info("### rightNpcDisplay:"..tostring(self.dialog.rightpic))
            self:rightNpcDisplay()
        elseif self.right_npc_display then
            --log.info("### rightNpcDisplay setNpcOpacity")
            self:setNpcOpacity(self.right_npc, 3)
        end


    end,

    -- 检测是否切换NPC
    checkNpcChanged = function(self, dialog)
        local changed = false

        if self.dialog.leftpic ~= dialog.leftpic then
            self:leftNpcExeunt()
            changed = true
        end

        if self.dialog.rightpic ~= dialog.rightpic then
            self:rightNpcExeunt()
            changed = true
        end

        if self.dialog.centerpic ~= dialog.centerpic then
            self:centerNpcExeunt()
            changed = true
        end

        self.dialog = dialog

        return changed
    end,

    -- 检测下一段对话
    checkNextDialog = function(self)
        local next_idx = self.dialog.next
        if next_idx == 0 then
            self.index = self.index + 1
            if self:checkNpcChanged(self.dialogs[self.index]) then
                app:startCoroutine(
                    function()
                        coroutine.yield(WaitForSeconds(0.5))
                        self:dialogPlay()
                    end
                )
            else
                self.playing = true
                self:setNPCName()
                self:setContent()

                if self.left_npc_display then
                    self:setNpcOpacity(self.left_npc, 1)
                    self:npcOverturn(self.left_npc, 1)
                end

                if self.center_npc_display then
                    self:setNpcOpacity(self.center_npc, 2)
                    self:npcOverturn(self.center_npc, 2)
                end

                if self.right_npc_display then
                    self:setNpcOpacity(self.right_npc, 3)
                    self:npcOverturn(self.right_npc, 3)
                end

                self:playDialogSound()
                self:playDialogEmotion()
                self:playPerformance()

            end
        elseif next_idx > 0 then
            self:showOptions()
        else
            self.scenario_end = true
            -- log.info("##### dialogue finished !")
        end
    end,

    -- 把 npc 转换为 spine
    changeNpcToSpine = function(self, idx, path, callback)
        local npc = nil
        if idx == 0 then        -- 左
            npc = self.left_npc
        elseif idx == 1 then    -- 中
            npc = self.center_npc
        elseif idx == 2 then    -- 右
            npc = self.right_npc
        else
            log.error("invalid idx:"..idx)
        end

        local image = npc.gameObject:GetComponent(UnityEngine.UI.Image)
        local color = Color.white
        local scale = Vector3.one

        if not image then
            image = npc:GetComponent(Spine.Unity.SkeletonGraphic)
        end

        if image then
            color = image.color
            UnityEngine.Object.Destroy(image)
        end

        if self.spine_assets[idx] then
            self.spine_assets[idx].RemoveRef()
            self.spine_assets[idx] = false
        end
        
        local pos = npc.localPosition
        pos.y = -540
        npc.localPosition = pos

        app:startCoroutine(
            function()
                coroutine.yield(Fenix.Assets.Helper.WaitForAsset(path, Spine.Unity.SkeletonDataAsset, function(asset) 
                    local sda = asset:Get() -- skeleton data asset
                    asset:AddRef()
                    self.spine_assets[idx] = asset

                    if sda then

                        local skeleton_graphic = Spine.Unity.SkeletonGraphic.AddSkeletonGraphicComponent(npc.gameObject, sda)
                        skeleton_graphic.color = color
                        local anim_state = skeleton_graphic.AnimationState
                        if anim_state then
                            anim_state:SetAnimation(0, 'idle', true)
                        else
                            log.error("Failed to find anim state:"..path)
                        end

                    else
                        log.error('failed to load spine:'..path)
                    end

                    if callback then
                        callback(true)
                    end
                end))
            end
        )

    end,

    -- 把 npc 转换为 Image
    changeNpcToImage = function(self, idx, path, callback)
    end,

    -- 左侧NPC入场
    leftNpcDisplay = function(self)

        local callback = function(is_spine)
            self:npcOverturn(self.left_npc, 1)
            --self:setNpcOpacity(self.left_npc, 1)
                 
            local lt_1 = LeanTween.color(self.left_npc, Color(0.4,0.4,0.4,1), 0.5)
            lt_1.loopType = LeanTweenType.once
            lt_1:setOnComplete(function()
                -- log.info("##### left npc color end")
                self:setNpcOpacity(self.left_npc, 1)
            end)

            local pos = Vector3.Clone(self.left_orig)
            pos.x = pos.x - self.left_npc.rect.width * 0.5

            --log.info("## self.left_npc.rect.width:"..self.left_npc.rect.width )

            self.left_npc.localPosition = pos

            local lt_2 = LeanTween.moveLocalX(self.left_npc.gameObject, self.left_orig.x, 0.3)
            lt_2.loopType = LeanTweenType.once
            lt_2.delay = 0.1
            lt_2:setOnComplete(function()
                -- log.info("#### left npc display end")
            end)
        end

        local image = self.left_npc:GetComponent(UnityEngine.UI.Image)
        if image then
            if self.dialog.leftpic:find('.png') then
               -- image.color = Color.white
                image:SetSprite(self.dialog.path..self.dialog.leftpic)
               -- image.color = Color(1,1,1,0))
                callback()
            elseif self.dialog.leftpic:find('.png') then
                self:changeNpcToSpine(0, self.dialog.path..self.dialog.leftpic, callback)
            end
        else
            image = npc:GetComponent(Spine.Unity.SkeletonGraphic)
            if image and self.dialog.leftpic:find('.asset') then
                self:changeNpcToSpine(0, self.dialog.path..self.dialog.leftpic, callback)
            end
        end

        self.left_npc_display = true
    end,

    -- 中间NPC入场
    centerNpcDisplay = function(self)
        local image = self.center_npc:GetComponent(UnityEngine.UI.Image)
        if not image then
            image = npc:GetComponent(Spine.Unity.SkeletonGraphic)
        end

        self:npcOverturn(self.center_npc, 2)
        image:SetSprite(self.dialog.path..self.dialog.centerpic)

        local lt_1 = LeanTween.color(self.center_npc, Color(1,1,1,1), 0.5)
        lt_1.loopType = LeanTweenType.once
        lt_1:setOnComplete(function()
            -- log.info("##### left npc color end")
            self:setNpcOpacity(self.center_npc, 2)
        end)

        local lt_2 = LeanTween.moveLocalZ(self.center_npc.gameObject, 0, 0.3)
        lt_2.loopType = LeanTweenType.once
        lt_2.delay = 0.1
        lt_2:setOnComplete(function()
            -- log.info("#### left npc display end")
        end)

        self.center_npc_display = true
    end,

    -- 右侧NPC入场
    rightNpcDisplay = function(self)
        local image = self.right_npc:GetComponent(UnityEngine.UI.Image)
        if not image then
            image = npc:GetComponent(Spine.Unity.SkeletonGraphic)
        end

        self:npcOverturn(self.right_npc, 3)
        image:SetSprite(self.dialog.path..self.dialog.rightpic)

        local lt_1 = LeanTween.color(self.right_npc, Color(1,1,1,1), 0.5)
        lt_1.loopType = LeanTweenType.once
        lt_1:setOnComplete(function()
            -- log.info("##### right npc color end")
            self:setNpcOpacity(self.right_npc, 3)
        end)

        local pos = Vector3.Clone(self.right_orig)
        pos.x = pos.x + self.right_npc.rect.width*0.5
        self.right_npc.localPosition = pos

        local lt_2 = LeanTween.moveLocalX(self.right_npc.gameObject,self.right_orig.x, 0.3)
        lt_2.loopType = LeanTweenType.once
        lt_2.delay = 0.1
        lt_2:setOnComplete(function()
            -- log.info("#### right npc display end")
        end)

        self.right_npc_display = true
    end,

    -- 左侧NPC退场
    leftNpcExeunt = function(self)
        local lt = LeanTween.moveLocalX(self.left_npc.gameObject, self.left_orig.x-self.left_npc.rect.width*0.5, 0.4)
        lt.loopType = LeanTweenType.once
        lt.delay = 0.1
        lt:setOnComplete(function() self:setNpcEmotion(self.left_npc, "") end) -- 退场时清空表情

        lt = LeanTween.color(self.left_npc:GetComponent(UnityEngine.RectTransform), Color(1,1,1,0), 0.4)
        lt.loopType = LeanTweenType.once

        self.left_npc_display = false
     end,

    -- 中间NPC退场
    centerNpcExeunt = function(self)
        local lt = LeanTween.moveLocalZ(self.center_npc.gameObject, 2000, 0.4)
        lt.loopType = LeanTweenType.once
        lt.delay = 0.1
        lt:setOnComplete(function() self:setNpcEmotion(self.center_npc, "") end) -- 退场时清空表情

        lt = LeanTween.color(self.center_npc:GetComponent(UnityEngine.RectTransform), Color(1,1,1,0), 0.4)
        lt.loopType = LeanTweenType.once

        self.center_npc_display = false

        self:setNpcEmotion(self.center_npc, "")
    end,

    -- 右侧NPC退场
    rightNpcExeunt = function(self)
        local lt = LeanTween.moveLocalX(self.right_npc.gameObject, self.right_orig.x+self.right_npc.rect.width*0.5, 0.4)
        lt.loopType = LeanTweenType.once
        lt.delay = 0.1
        lt:setOnComplete(function() self:setNpcEmotion(self.right_npc, "") end) -- 退场时清空表情

        lt = LeanTween.color(self.right_npc:GetComponent(UnityEngine.RectTransform), Color(1,1,1,0), 0.4)
        lt.loopType = LeanTweenType.once

        self.right_npc_display = false
    end,

    -- 显示对话框
    boxDisplay = function(self)
        local lt = LeanTween.scaleY(self.dialog_box.gameObject, 1, 0.2)
        lt.loopType = LeanTweenType.once
        lt.delay = 0.4
    end,

    -- 隐藏对话框
    boxHide = function(self)
        self.dialog_box.localScale = Vector3(1,0,1)
        self.dialog_label.transform:GetComponent(UnityEngine.UI.Text).text = ""
    end,

    --npc 立绘翻转
    npcOverturn = function(self, npc, idx)
        local tab = loadstring("return "..self.dialog.reverse)()

        if tab[idx] ~= 0 then
           -- log.info("### npcOverturn 180")
            local lt = LeanTween.rotateY(npc.gameObject, 180, 0.1)
            lt.loopType = LeanTweenType.once
        else
           -- log.info("### npcOverturn 0")
            local lt = LeanTween.rotateY(npc.gameObject, 0, 0.1)
            lt.loopType = LeanTweenType.once
        end
    end,

    --设置立绘半透
    setNpcOpacity = function(self, npc, idx)
        local tab = loadstring("return "..self.dialog.opaque)()
        local image = npc:GetComponent(UnityEngine.UI.Image)
        if not image then
            image = npc:GetComponent(Spine.Unity.SkeletonGraphic)
        end

        if tab[idx] ~= 0 then
            image.color = Color(1, 1, 1, 1)
            --local lt_2 = LeanTween.scale(image.transform, Vector3(1.1,1.1,1.1), 0.1)
            --lt_2.loopType = LeanTweenType.once

        else
            image.color = Color(0.4, 0.4, 0.4, 1.0)
            --image.transform.localScale = Vector3(1.0,1.0,1.0)
           -- local lt_2 = LeanTween.scale(image.transform, Vector3(1.0,1.0,1.0), 0.1)
            --lt_2.loopType = LeanTweenType.once
        end
    end,

    -- 设置对话文本
    setContent = function(self, str)
        if not self.playing then
            return
        end
        self.imgNext:SetActive(false)
        local cb = function(finished)
            self.playing = not finished
            self.dialog_end = finished
            if finished then
                self.imgNext:SetActive(true)
            end
        end
        self.dialog_label:SetText(self.helper:getLangCfg(self.dialog.content), cb)
    end,

    -- 显示整段对话
    showAllText = function(self)
        self.dialog_label:showAllText()
        self.imgNext:SetActive(true)
    end,

    -- 显示剧情选项
    showOptions = function(self)
        local cfg = dialog_manager.option_cfg[self.dialog.next]

        if not cfg then
            log.error("option cfg error ! id:", self.dialog.next)
            return
        end

        local tab = {}
        for k, v in ipairs(cfg) do
            table.insert( tab, v.content )
        end

        local cb = function(index)
            local dialog_id = cfg[index].next
            self.dialogs = dialog_manager.scenario_cfg[dialog_id]

            if self:checkNpcChanged(self.dialogs[1]) then
                app:startCoroutine(
                    function()
                        coroutine.yield(WaitForSeconds(0.5))
                        self:start()
                    end
                )
            else
                self:start()
            end
        end

        frame_manager:show("dialog_option", self, {options = tab, callback = cb})
    end,

    -- 播放当前对话的表情
    playDialogEmotion = function(self)
        local tab = loadstring("return "..self.dialog.emotion)()

        -- 播放表情
        if self.left_npc and tab[1] then
            self:setNpcEmotion(self.left_npc, tab[1])
        end
        if self.center_npc and tab[3] then
            self:setNpcEmotion(self.center_npc, tab[2])
        end    
        if self.right_npc and tab[2] then
            self:setNpcEmotion(self.right_npc, tab[3])
        end
   
    end,

    -- 设置表情
    setNpcEmotion = function(self, npc, emotion_name)
        local potrait_emotion = npc:GetComponent(PotraitEmotion)
        potrait_emotion:SetEmotion(emotion_name)
    end,

    -- 播放表演效果
    playPerformance = function(self)
        if self.dialog.performance and self.dialog.performance > 0 then
            self.dialog_script:runPerformance(self.dialog.performance)
        end
    end,

    -- 播放对话的背景音乐和音效
    playDialogSound = function(self)
        if self.dialog.bgm and self.dialog.bgm ~= "" then
            --AudioController.Instance:BGMPlay(self.dialog.bgm, "", 1)
        end

        if self.dialog.sound and self.dialog.sound ~= "" then
            --AudioController.Instance:SoundPlay(self.dialog.sound, "", 1, false, nil)
        end
    end,

    -- 显示自定义的图片
    showCustomImage = function(self, path, pos, fade_in_time, elapsed, fade_out_time)
        local go = GameObject.Instantiate(self.prefab_custom_image, self.bg)
        local img = go:GetComponent(UnityEngine.UI.Image)
        img:SetSprite(path)
        --img:SetNativeSize()
        img.color = Color(1,1,1,0.0)
        local trans = go.transform
        trans.localPosition = pos


        app:startCoroutine(
            function()
                coroutine.yield(nil)
                img:SetNativeSize()
                trans.localScale = Vector3(0.5, 0.5, 0.5)
                img.color = Color(1,1,1,0.5)

                local lt_1 = LeanTween.color(trans, Color(1,1,1,1), fade_in_time)
                lt_1.loopType = LeanTweenType.once

                local lt_2 = LeanTween.scale(trans, Vector3.one, fade_in_time)
                lt_2.loopType = LeanTweenType.once

                coroutine.yield(WaitForSeconds(fade_in_time + elapsed))

                lt_1 = LeanTween.color(trans, Color(1,1,1,0.5), fade_out_time)
                lt_1.loopType = LeanTweenType.once

                lt_2 =  LeanTween.scale(trans, Vector3(0.5,0,5,0.5), fade_out_time)
                lt_2.loopType = LeanTweenType.once

                coroutine.yield(WaitForSeconds(fade_out_time))
                GameObject.Destroy(go)
            end
        )
    end,

    -- 点击事件
    onClickBG = function(self)
        if self.dialog_script and self.dialog_script.is_running then
            return
        end

        if self.scenario_end then
            if self.finish_callback then
                self.finish_callback()
            end
            self:hide()
        elseif self.dialog_end then
            self.dialog_end = false
            self:checkNextDialog()
        elseif self.playing then
            self:showAllText()
        end
    end,

    hide = function(self)
        frame_manager:hide("dialog")
    end,
}