local super = require("ui.frame")
local log = logger.get("Game", "Emp Aim")
local gui_helper = require('gui.gui_helper')
local emp_manager = require('level.emp_manager')

return class
{
    super = super,
    
    onLoaded = function (self)
        super.onLoaded(self)
        self.helper = require("gui.tool.frame_helper")
        self.aim_panel = self.transform:Find("AimPanel")
        self.aim_prefab = self.transform:Find("Prefab/Aim")

        self.prefab_root = self.transform:Find("Prefab")
        self.prefab_root:SetActive(false)

        self.focus = self.transform:Find("RootPanel/Focus")
        self.focus:SetActive(false)

        self.fire_begin = self.transform:Find("RootPanel/FireBegin")
        self.fire_begin:SetActive(false)
        self.fire_button = self.transform:Find("RootPanel/ButtonFire"):GetComponent("Button")
        --modified by Hong Feng
        --modification end
        clickEvents = {
            -- 模块面板
            "RootPanel/RootPanelClose",
            "RootPanel/ButtonFire"
        }

        -- 注册点击事件
        self.helper:regClickEvents("click", self, clickEvents)
        self.aim_pool = {}
        self.entity2Aim = {} -- entity - aim

        app:startCoroutine(
            function()
                coroutine.yield(WaitForSeconds(0.5))
                self.focus:SetActive(true)


                local last_time = Time.time
                while true do
                    if Time.time - last_time > 0.1 then
                        -- 检查攻击范围内的敌人
                        emp_manager:checkInRangeMonsters()    
                        for entity, aim in pairs(self.entity2Aim) do
                            if not entity.is_in_emp_range or entity.is_disappear then
                                self:putAimToPool(aim)  -- entity 不再被选择，将其放回池中
                                self.entity2Aim[entity] = nil
                            end
                        end

                        for _,entity in pairs(emp_manager.in_range_monsters) do
                            if not self.entity2Aim[entity] then
                                -- 创建 aim
                                local aim = self:getAimFromPool()
                                self.entity2Aim[entity] = aim
                            end
                        end

                    end

                    -- 每帧更新 aim 的屏幕位置
                    for entity,aim in pairs(self.entity2Aim) do
                        if aim then
                            local screen_pos = Camera.main:WorldToScreenPoint(entity.transform.position)

                            screen_pos.x = screen_pos.x / UnityEngine.Screen.width * self.aim_panel.rect.width
                            screen_pos.y = screen_pos.y / UnityEngine.Screen.height * self.aim_panel.rect.height
                            screen_pos.z = 0

                            aim.localPosition = screen_pos
                        end

                    end

                    coroutine.yield()
                end
            end
        )
    end,

    putAimToPool = function(self, aim)
        aim:SetActive(false)
        table.insert(self.aim_pool, aim)
        aim:SetParent(self.prefab_root)
    end,

    getAimFromPool = function(self)
        if #self.aim_pool > 0 then
            local idx = #self.aim_pool
            local aim = self.aim_pool[idx]
            table.remove(self.aim_pool, idx)
            aim:SetActive(true)
            aim:SetParent(self.aim_panel)
            return aim
        else
            local go = GameObject.Instantiate(self.aim_prefab.gameObject)
            go:SetActive(true)
            go.transform:SetParent(self.aim_panel)
            go.transform.localScale = Vector3.one
            return go.transform
        end
    end,

    -- 关闭
    click_RootPanelClose = function(self)
        emp_manager:closeAimFrame()
    end,

    -- emp 开火
    click_ButtonFire = function(self)
        -- if #emp_manager.in_range_monsters == 0 then
        --     -- 提示用户要先选择目标
        --     hint:setText(NativeText.GetLangStr(170050))
        --     return
        -- end

        self.fire_button.interactable = false
        self.fire_button.gameObject:SetActive(false)

        local close_button = self.transform:Find("RootPanel/RootPanelClose"):GetComponent("Button")
        close_button.interactable = false
        self.aim_panel:SetActive(false)
        game_state_level.is_break_input = true
        app:startCoroutine(
            function()
                self.fire_begin:SetActive(true)

                local line_1 = self.transform:Find("RootPanel/FireBegin/Emp_Info/Text_Line1"):GetComponent(TypewriterEffect)
                local line_2 = self.transform:Find("RootPanel/FireBegin/Emp_Info/Text_Line2"):GetComponent(TypewriterEffect)
                local line_3 = self.transform:Find("RootPanel/FireBegin/Emp_Info/Text_Line3"):GetComponent(TypewriterEffect)

                line_1:SetText(NativeText.GetLangStr(170047), function(finished) end)
                coroutine.yield(WaitForSeconds(0.667))
                line_2:SetText(NativeText.GetLangStr(170048), function(finished) end)
                coroutine.yield(WaitForSeconds(0.667))
                line_3:SetText(NativeText.GetLangStr(170049), function(finished) end)
                coroutine.yield(WaitForSeconds(0.667))

                self.fire_begin:SetActive(false)

                emp_manager:fireEmp()   --把这段做好就完成了，emp——cannon模型没了！！！！！！ 

                coroutine.yield(WaitForSeconds(4.0))
                --emp_manager:closeAimFrame()
                coroutine.yield(WaitForSeconds(0.5))
                game_state_level.is_break_input = false
            end
        )

        --[[
        local entities = {}
        log.info("## emp fire:".. tostring(emp_manager.emp_target))

        for k,entity in pairs(emp_manager.in_range_monsters) do
            table.insert(entities, entity.id)
            log.info("id:"..entity.id)
        end
        network_sender:useEmp(emp_manager.emp_target, entities)
        ]]
    end,

    searchCarrier = function(self)
        app:startCoroutine(
            function()
                --log.error('searchCarrier')

                self.fire_begin:SetActive(true)
                local line_1 = self.transform:Find("RootPanel/FireBegin/Emp_Info/Text_Line1"):GetComponent(TypewriterEffect)
                local line_2 = self.transform:Find("RootPanel/FireBegin/Emp_Info/Text_Line2"):GetComponent(TypewriterEffect)
                local line_3 = self.transform:Find("RootPanel/FireBegin/Emp_Info/Text_Line3"):GetComponent(TypewriterEffect)
                local line_4 = self.transform:Find("RootPanel/FireBegin/Emp_Info/Text_Line4"):GetComponent(TypewriterEffect)
                SoundEngine.PostEvent("Play_ui_click", self.gameObject)
                line_1:SetText(NativeText.GetLangStr(170123), function(finished) end)
                coroutine.yield(WaitForSeconds(1))
                SoundEngine.PostEvent("Play_ui_click", self.gameObject)
                line_2:SetText(NativeText.GetLangStr(170124), function(finished) end)
                coroutine.yield(WaitForSeconds(1))
                SoundEngine.PostEvent("Play_ui_click", self.gameObject)
                line_3:SetText(NativeText.GetLangStr(170125), function(finished) end)
                coroutine.yield(WaitForSeconds(1))
                SoundEngine.PostEvent("Play_ui_close", self.gameObject)
                line_4:SetText(NativeText.GetLangStr(170126), function(finished) end)
                coroutine.yield(WaitForSeconds(1))
                coroutine.yield(WaitForSeconds(2))  
            end
        )
    end,

    showSearchFrame = function(self)
        --frame_manager:show('emp_aim')
        -- local level_frame = frame_manager:getFrame('level')
        -- if level_frame then
        --     --level_frame.transform:SetActive(false)
        --     frame_helper.hideTrans(level_frame.transform)
        -- end

        -- game_state_level.ignore_tap = true  -- 忽略用户点击
        local level_manager = game_state_level.level.level_manager
        -- level_manager.fogOfWar.gameObject:SetActive(false)  -- 关闭战争迷雾
        local main_cam = level_manager.mainCamera
        self.emp_image_effect = main_cam:GetComponent(EmpImageEffect) or main_cam:AddComponent(EmpImageEffect)
        self.emp_image_effect.enabled = true
    end,

    showFireButton= function(self)
        --emp_manager:showAimFrame()
        --coroutine.yield(WaitForSeconds(0.1))
        self.fire_button:SetActive(true)
    end,

    hideSearchFrame = function(self)
        -- 显示关卡界面
        local level_frame = frame_manager:getFrame('level')
        if level_frame and level_frame.transform then
            --level_frame.transform:SetActive(true)
            frame_helper.showTrans(level_frame.transform)
        end
        -- 关闭瞄准界面
        frame_manager:hide('emp_aim')

        game_state_level.ignore_tap = false -- 恢复用户点击
        local level_manager = game_state_level.level.level_manager
        level_manager.fogOfWar.gameObject:SetActive(true)   -- 开启战斗阴影

        local main_cam = level_manager.mainCamera
        self.emp_image_effect = main_cam:GetComponent(EmpImageEffect)
        self.emp_image_effect.enabled = false
    end,

    onUnload = function(self)

    end,
}