local log = logger.get("Game", "equipment info panel")

return class{
    ctor = function(self, panel, parent)
        self.parent = parent
        self.panel = panel
        self.helper = require("gui.tool.frame_helper")

        self.equip_1 = self.panel:Find('Layout/Equip_1')
        self.equip_2 = self.panel:Find('Layout/Equip_2')

        self.equip_states = {
            show = 1,
            replace = 2,
            equip = 3,
            remove = 4,
        }

        self:hide()
    end,

    show = function(self, equipment, compare_equipment, state)
        self.helper.showTrans(self.panel)

        self.state = state

        self:setEquipInfo(self.equip_1, equipment, 1)

        if compare_equipment then
            self.equip_2:SetActive(true)
            self:setEquipInfo(self.equip_2, compare_equipment, 2)
        else
            self.equip_2:SetActive(false)
        end
    end,

    setEquipInfo = function(self, node, equip, idx)
        local name = node:Find('Top/Name'):GetComponent(UnityEngine.UI.Text)
        local type = node:Find('Top/Type'):GetComponent(UnityEngine.UI.Text)
        local equipped = node:Find('Top/Equipped')
        local props_node = node:Find('Props')
        local grade = node:Find('Grade'):GetComponent(UnityEngine.UI.Image)
        local icon = node:Find('Grade/Icon'):GetComponent(UnityEngine.UI.Image)
        local btn_1 = node:Find("Panel/Remove")
        local btn_2 = node:Find("Panel/ChangeBtn")
        local btn_3 = node:Find("Panel/EquipBtn")

        name.text = self.helper:getLangCfg(equip.cfg.name_id)
        type.text = gui_helper:getWeaponType(equip.cfg)
        name.color = gui_helper:quality2UnityColor(equip.cfg.color)

        local icon_path = gui_helper:getItemIconPath(equip.cfg)
        icon:SetSprite(icon_path)
        grade:SetSprite(gui_helper:getQualityIconPath(equip.cfg.color))

        local cnt = 0
        for k, v in pairs(equip.cfg.basic_attr) do
            local cfg = config.player_config_attrs[k]
            if cfg then
                if cnt < props_node.childCount then
                    local elem = props_node:GetChild(cnt)
                    elem:SetActive(true)

                    local des = elem:GetComponent(UnityEngine.UI.Text)

                    local value = v
                    if cfg.isfloat > 0 then
                        value = value * 0.0001
                    end

                    if cfg.showpercent > 0 then
                        value = value*100 .."%"
                    end

                    des.text = string.format( "%s <color=#0BFF78FF>+%s</color>", frame_helper:getLangCfg(cfg.name_id), tostring(value) )
                end

                cnt = cnt + 1
            end
        end

        for i=cnt, props_node.childCount - 1 do
            local elem = props_node:GetChild(i)
            elem:SetActive(false)
        end

        if self.state == self.equip_states.show then
            equipped:SetActive(false)
            btn_1:SetActive(true)
            btn_2:SetActive(true)
            btn_3:SetActive(false)
        elseif self.state == self.equip_states.replace then
            equipped:SetActive(idx == 1)
            btn_1:SetActive(false)
            btn_2:SetActive(false)
            btn_3:SetActive(idx == 2)
        elseif self.state == self.equip_states.equip then
            equipped:SetActive(false)
            btn_1:SetActive(false)
            btn_2:SetActive(false)
            btn_3:SetActive(true)
        elseif self.state == self.equip_states.remove then
            equipped:SetActive(false)
            btn_1:SetActive(true)
            btn_2:SetActive(false)
            btn_3:SetActive(false)
        end
    end,

    hide = function(self)
        self.helper.hideTrans(self.panel)
    end,
}