--用来控制整个界面的打开和关闭
local log = logger.get("Game", "Frame manager")
-- 帮助类
local frameHelper = require("gui.tool.frame_helper")
res_path = require("gui.tool.res_path")

local sorting_order0 = 0
local sorting_order1 = 10
local sorting_order2 = 20
local sorting_order3 = 30

return{

    --界面的父节点
    parent = GameObject.Find("/gui/root/game"),

    --每个界面的数据
    data = {},

    --已经显示界面
    visibleList = {},

    init = function(self)
        self.events = require("base.event").new()

        self.root =  GameObject.Find("/gui/root")
        self.root_canvas = self.root:GetComponent(Canvas)

        -- 加载配置文件
        require("gui.tool.frame_helper").loadConfig()
        require("gui.tool.dropbag_helper")
        require("gui.tool.speed_up_helper")
        require("gui.tool.passive_skill_helper")
        require("gui.tool.formation_adjust_helper")
        require("gui.tool.player_cfg_helper")

        self.is_loading_show = false

        if(table.getn(self.data) == 0) then

            GameObject.DontDestroyOnLoad(GameObject.Find('gui'));

            local createFloor = function(object_name, sort_order)
                local floor = GameObject()
                floor.name = object_name
                floor:AddComponent(UnityEngine.RectTransform)
                floor:AttachTo(self.parent)
                local c = floor:AddComponent(UnityEngine.Canvas)
                c.overrideSorting = true
                c.sortingOrder = sort_order
                c.sortingLayerName = "UI"
                c.additionalShaderChannels = UnityEngine.AdditionalCanvasShaderChannels.TexCoord1;
                local raycaster = floor:AddComponent(UnityEngine.UI.GraphicRaycaster)
                frameHelper:setChildLayer(floor.transform, 'UI')

                -- add by syz
                local scale = Fenix.Game.GameApp.GetInstance().m_GlobalUIScale;
                floor.transform.localScale = Vector3(scale, scale, scale)
                return floor, raycaster
            end

            self.raycasters = {}
            self.floor0, self.raycasters[0] = createFloor("0st", sorting_order0)
            self.floor1, self.raycasters[1] = createFloor("1st", sorting_order1)
            self.floor2, self.raycasters[2] = createFloor("2st", sorting_order2)
            self.floor3, self.raycasters[3] = createFloor("3st", sorting_order3)
            self.floors = {}
            self.floors[0] = self.floor0
            self.floors[1] = self.floor1
            self.floors[2] = self.floor2
            self.floors[3] = self.floor3

            self.camera = self:getCamera()
            self.full_screen_mask = GameObject.Find("/gui/root/game/FullScreeMask")
            self.full_screen_mask.transform:SetAsLastSibling()

            -- 战斗界面
            table.insert(self.data, 
            {
                k = 'battle',
                v = {
                        class = "gui.battle_frame",
                        name = "battle",
                        path = "Assets/Frames/BattleFrame.prefab",
                        parent = self.floor0,
                    }
            })
            
            -- 战报战斗界面
            table.insert(self.data, 
            {
                k = 'replay_battle',
                v = {
                        class = "gui.replay_battle_frame",
                        name = "replay_battle",
                        path = "Assets/Frames/ReplayBattleFrame.prefab",
                        parent = self.floor0,
                    }
            })

            --登录界面设置
            table.insert(self.data, 
            {
                k = 'login',
                v = {
                        class = "gui.login_frame",
                        name = "login",
                        path = "Assets/Frames/LoginFrame.prefab",
                        parent = self.floor0,
                    }
            })

            table.insert(self.data, 
            {
                k = 'create_role',
                v = {
                        class = "gui.create_role_frame",
                        name = "create_role",
                        path ='Assets/Frames/CreateRoleFrame.prefab',
                        parent = self.floor0,
                        
                    }
            })

            -- 关卡界面
            table.insert(self.data, 
            {
                k = 'level',
                v = {
                        class = "gui.level_frame",
                        name = "level",
                        path = "Assets/Frames/LevelFrame.prefab",
                        parent = self.floor0,

                    }
            })
            
            -- 地图界面
            table.insert(self.data, 
            {
                k = 'worldMap',
                v = {
                        class = "gui.world_map_frame",
                        name = "worldMap",
                        path = "Assets/Frames/WorldMapFrame.prefab",
                        parent = self.floor0,
                    }
            })

            -- GM界面
            table.insert(self.data, 
            {
                k = 'gm',
                v = {
                        class = "gui.frame_gm",
                        name = "gm",
                        path = "Assets/Frames/GMFrame.prefab",
                        parent = self.floor3,
                    }
            })

            -- 购买boost
            table.insert(self.data, 
            {
                k = 'stmt',
                v = {
                        class = "gui.stmt_frame",
                        name = "stmt",
                        path = "Assets/Frames/StmtFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 提示界面
            table.insert(self.data, 
            {
                k = 'hint',
                v = {
                        class = "gui.hint_frame",
                        name = "hint",
                        path = "Assets/Frames/HintFrame.prefab",
                        parent = self.floor3,
                    }
            })

            -- 章节界面
            table.insert(self.data, 
            {
                k = 'chapter',
                v = {
                        class = "gui.chapter_frame",
                        name = "chapter",
                        path = "Assets/Frames/ChapterFrame.prefab",
                        parent = self.floor0,
                    }
            })

             -- 奖励界面
            table.insert(self.data, 
            {
                k = 'reward',
                v = {
                        class = "gui.reward_frame",
                        name = "reward",
                        path = "Assets/Frames/RewardFrame.prefab",
                        parent = self.floor3,
                    }
            })

             -- 关卡选择界面
            table.insert(self.data, 
            {
                k = 'step',
                v = {
                        class = "gui.step_frame",
                        name = "step",
                        path = "Assets/Frames/StepFrame.prefab",
                        parent = self.floor0,
                    }
            })

            -- 背包界面
            table.insert(self.data, 
            {
                k = 'bag',
                v = {
                        class = "gui.bag_frame",
                        name = "bag",
                        path = "Assets/Frames/BagFrame.prefab",
                        parent = self.floor1,
                    }
            })
            
            -- 家园界面
            table.insert(self.data, 
            {
                k = 'home',
                v = {
                        class = "gui.home_frame",
                        name = "home",
                        path = "Assets/Frames/HomeFrame.prefab",
                        parent = self.floor0,
                    }
            })

            -- 武器升级
            table.insert(self.data, 
            {
                k = 'weapon_upgrade',
                v = {
                        class = "gui.weapon_upgrade_frame",
                        name = "weapon_upgrade",
                        path = "Assets/Frames/WeaponUpgradeFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 指挥官养成界面
            table.insert(self.data, 
            {
                k = 'officer',
                v = {
                        class = "gui.officer_frame",
                        name = "commander",
                        path = "Assets/Frames/OfficerFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 指挥官升级界面
            table.insert(self.data, 
            {
                k = 'officer_level_up',
                v = {
                        class = "gui.officer_level_up_frame",
                        name = "officer_level_up",
                        path = "Assets/Frames/OfficerLevelupFrame.prefab",
                        parent = self.floor3,
                    }
            })
            
            -- 建筑信息界面
            table.insert(self.data, 
            {
                k = 'entity_info',
                v = {
                        class = "gui.entity_info_frame",
                        name = "entity_info",
                        path = "Assets/Frames/EntityInfoFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 战车列表界面
            table.insert(self.data, 
            {
                k = 'hero_list',
                v = {
                        class = "gui.hero_list_frame",
                        name = "hero",
                        path = "Assets/Frames/HeroListFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 个人基地建筑信息界面
            table.insert(self.data, 
            {
                k = 'home_entity_info',
                v = {
                        class = "gui.home_entity_info_frame",
                        name = "home_entity_info",
                        path = "Assets/Frames/HomeEntityInfoFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 个人基地建筑详情信息界面
            table.insert(self.data, 
            {
                k = 'home_entity_info_detail',
                v = {
                        class = "gui.home_entity_info_detail_frame",
                        name = "home_entity_info_detail",
                        path = "Assets/Frames/HomeEntityInfoDetailFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 个人基地建筑详情信息界面
            table.insert(self.data, 
            {
                k = 'home_barrack',
                v = {
                        class = "gui.home_barrack_frame",
                        name = "home_barrack",
                        path = "Assets/Frames/HomeBarrackFrame.prefab",
                        parent = self.floor1,
                    }
            })

            table.insert(self.data, 
            {
                k = 'home_delete_soldier',
                v = {
                        class = "gui.home_delete_soldier_frame",
                        name = "home_delete_soldier",
                        path = "Assets/Frames/HomeDeleteSoldierFrame.prefab",
                        parent = self.floor1,
                    }
            })

            table.insert(self.data, 
            {
                k = 'home_defense',
                v = {
                        class = "gui.home_defense_frame",
                        name = "home_defense",
                        path = "Assets/Frames/HomeDefenseFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 个人基地建筑详情信息界面
            table.insert(self.data, 
            {
                k = 'home_hospital',
                v = {
                        class = "gui.home_hospital_frame",
                        name = "home_hospital",
                        path = "Assets/Frames/HomeHospitalFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 个人基地建规则说明界面
            table.insert(self.data, 
            {
                k = 'home_entity_explain',
                v = {
                        class = "gui.home_entity_explain_frame",
                        name = "home_entity_explain",
                        path = "Assets/Frames/HomeEntityExplainFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 个人基地建筑创建界面
            table.insert(self.data, 
            {
                k = 'home_entity_research',
                v = {
                        class = "gui.home_entity_research_frame",
                        name = "home_entity_research",
                        path = "Assets/Frames/HomeEntityResearchFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 科技升级完成提示界面
            table.insert(self.data, 
            {
                k = 'home_tech_complete',
                v = {
                        class = "gui.home_tech_complete_frame",
                        name = "home_tech_complete",
                        path = "Assets/Frames/HomeTechCompleteFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 通用二级框
            table.insert(self.data, 
            {
                k = 'common_box',
                v = {
                        class = "gui.common_box_frame",
                        name = "common_box",
                        path = "Assets/Frames/CommonBoxFrame.prefab",
                        parent = self.floor3,
                    }
            })

            -- 通用二级框
            table.insert(self.data, 
            {
                k = 'common_box_1',
                v = {
                        class = "gui.common_box_1_frame",
                        name = "common_box_1",
                        path = "Assets/Frames/CommonBox1Frame.prefab",
                        parent = self.floor3,
                    }
            })

            -- 无按钮通用二级框
            table.insert(self.data, 
            {
                k = 'common_box_2',
                v = {
                        class = "gui.common_box_2_frame",
                        name = "common_box_2",
                        path = "Assets/Frames/CommonBox2Frame.prefab",
                        parent = self.floor3,
                    }
            })

            

            -- 创建建筑界面
            table.insert(self.data, 
            {
                k = 'home_entity_create',
                v = {
                        class = "gui.home_entity_create_frame",
                        name = "home_entity_create",
                        path = "Assets/Frames/HomeEntityCreateFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 建筑升级界面
            table.insert(self.data, 
            {
                k = 'home_entity_upgrade',
                v = {
                        class = "gui.home_entity_upgrade_frame",
                        name = "home_entity_upgrade",
                        path = "Assets/Frames/HomeEntityUpgradeFrame.prefab",
                        parent = self.floor1,
                    }
            })

            --[[
            -- 建筑仓库界面
            table.insert(self.data, 
            {
                k = 'home_entity_store',
                v = {
                        class = "gui.home_entity_store_frame",
                        name = "home_entity_store",
                        path = "Assets/Frames/HomeEntityStoreFrame.prefab",
                        parent = self.floor1,
                    }
            })
            ]]

            --[[
            -- 建筑仓库界面
            table.insert(self.data, 
            {
                k = 'home_entity_train',
                v = {
                        class = "gui.home_entity_train_frame",
                        name = "home_entity_train",
                        path = "Assets/Frames/HomeEntityTrainFrame.prefab",
                        parent = self.floor1,
                    }
            })
            ]]

            --[[
            -- 建筑车库界面
            table.insert(self.data, 
            {
                k = 'home_entity_garage',
                v = {
                        class = "gui.home_entity_garage_frame",
                        name = "home_entity_garage",
                        path = "Assets/Frames/HomeEntityGarageFrame.prefab",
                        parent = self.floor1,
                    }
            })
            

            -- 建筑车库信息界面
            table.insert(self.data, 
            {
                k = 'home_entity_garage_info',
                v = {
                        class = "gui.home_entity_garage_info_frame",
                        name = "home_entity_garage_info",
                        path = "Assets/Frames/HomeEntityGarageInfoFrame.prefab",
                        parent = self.floor1,
                    }
            })
            ]]

            table.insert(self.data,
            {
                k = 'crewEquip',
                v = {
                        class = "gui.crew_equip_frame",
                        name = "crewEquip",
                        path = "Assets/Frames/crewEquipFrame.prefab",
                        parent = self.floor1,
                    }
            })

            --更换装备界面
            table.insert(self.data,
            {
                k = 'changeEquipment',
                v = {
                        class = "gui.change_equipment_frame",
                        name = "changeEquipment",
                        path = "Assets/Frames/ChangeEquipFrame.prefab",
                        parent = self.floor1,
                    }
            })

            --获得道具界面
            table.insert(self.data,
            {
                k = 'getItems',
                v = {
                        class = "gui.get_item_frame",
                        name = "getItems",
                        path = "Assets/Frames/GetItemFrame.prefab",
                        parent = self.floor2,
                    }
            })

            --显示道具信息界面
            table.insert(self.data,
            {
                k = 'itemInfo',
                v = {
                        class = "gui.item_info_frame",
                        name = "iteminfo",
                        path = "Assets/Frames/ItemInfoFrame.prefab",
                        parent = self.floor2,
                    }
            })

            --日常任务界面
            table.insert(self.data,
            {
                k = 'daily_task',
                v = {
                        class = "gui.daily_task_frame",
                        name = "daily_task",
                        path = "Assets/Frames/DailyTaskFrame.prefab",
                        parent = self.floor0,
                    }
            })

            --主支线任务界面
            table.insert(self.data,
            {
                k = 'main_task',
                v = {
                        class = "gui.main_task_frame",
                        name = "main_task",
                        path = "Assets/Frames/MainTaskFrame.prefab",
                        parent = self.floor0,
                    }
            })

            --[[
            --emp 界面
            table.insert(self.data,
            {
                k = 'emp',
                v = {
                        class = "gui.emp_frame",
                        name = "emp",
                        path = "Assets/Frames/EmpFrame.prefab",
                        parent = self.floor1,
                    }
            })
            ]]
            
            -- emp 瞄准
            table.insert(self.data,
            {
                k = 'emp_aim',
                v = {
                        class = "gui.emp_aim_frame",
                        name = "emp_aim",
                        path = "Assets/Frames/EmpAimFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 第一人称视角瞄准
            table.insert(self.data,
            {
                k = 'fps_aim',
                v = {
                        class = "gui.fps_aim_frame",
                        name = "fps_aim",
                        path = "Assets/Frames/FPSAimFrame.prefab",
                        parent = self.floor1,
                    }
            })

            --关卡信息界面
            table.insert(self.data,
            {
                k = 'level_info',
                v = {
                        class = "gui.level_info_frame",
                        name = "level_info",
                        path = "Assets/Frames/LevelInfoFrame.prefab",
                        parent = self.floor0,
                    }
            })      

            --[[
            -- 神秘商人界面
            table.insert(self.data,
            {
                k = 'gamble',
                v = {
                        class = "gui.gamble_frame",
                        name = "gamble",
                        path = "Assets/Frames/GambleFrame.prefab",
                        parent = self.floor1,
                    }
            })  
            ]]  

            -- 关卡内怪物信息界面
            table.insert(self.data,
            {
                k = 'monster_info',
                v = {
                        class = "gui.monster_info_frame",
                        name = "gamble",
                        path = "Assets/Frames/MonsterInfoFrame.prefab",
                        parent = self.floor1,
                    }
            })    

            -- 重连界面
            table.insert(self.data,
            {
                k = 'reconnect',
                v = {
                        class = "gui.reconnect_frame",
                        name = "reconnect",
                        path = "Assets/Frames/ReconnectFrame.prefab",
                        parent = self.floor3,
                    }
            })

            -- 结算界面
            table.insert(self.data, 
            {
                k = 'battle_settle',
                v = {
                        class = "gui.battle_settlement_frame",
                        name = "battle_settle",
                        path = "Assets/Frames/BattleSettlementFrame.prefab",
                        parent = self.floor1,
                    }
            })   

            -- loading界面
            table.insert(self.data, 
            {
                k = 'loading',
                v = {
                        class = "gui.loading_frame",
                        name = "loading",
                        path = "Assets/Frames/LoadingFrame.prefab",
                        parent = self.floor3,
                    }
            }) 

            -- waiting界面
            table.insert(self.data, 
            {
                k = 'waiting',
                v = {
                        class = "gui.waiting_frame",
                        name = "waiting",
                        path = "Assets/Frames/WaitingFrame.prefab",
                        parent = self.floor3,
                    }
            }) 

            -- 对话界面
            table.insert(self.data, 
            {
                --[[
                k = 'dialog',
                v = {
                        class = "gui.dialog_frame",
                        name = "dialog",
                        path = "Assets/Frames/DialogFrame.prefab",
                        parent = self.floor1,
                    }
                ]]
                k = 'dialog',
                v = {
                        class = "gui.manga_dialog_frame",
                        name = "dialog",
                        path = "Assets/Frames/MangaDialogFrame.prefab",
                        parent = self.floor1,
                    }
            })
            
            -- 对话选项界面
            table.insert(self.data, 
            {
                k = 'dialog_option',
                v = {
                        class = "gui.option_frame",
                        name = "dialog_option",
                        path = "Assets/Frames/OptionFrame.prefab",
                        parent = self.floor1,
                    }
            })


            -- 联盟logo设置界面
            table.insert(self.data, 
            {
                k = 'guild_apply',
                v = {
                        class = "gui.guild_apply_frame",
                        name = "guild_apply",
                        path = "Assets/Frames/GuildApplyFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 联盟信息界面
            table.insert(self.data, 
            {
                k = 'guild_info',
                v = {
                        class = "gui.guild_info_frame",
                        name = "guild_info",
                        path = "Assets/Frames/GuildInfoFrame.prefab",
                        parent = self.floor1,
                    }
            })
            
            -- 岛主界面
            table.insert(self.data, 
            {
                k = 'island',
                v = {
                        class = "gui.island_frame",
                        name = "island",
                        path = "Assets/Frames/IslandFrame.prefab",
                        parent = self.floor0,
                    }
            })

            -- 联盟成员列表界面
            table.insert(self.data, 
            {
                k = 'guild_members',
                v = {
                        class = "gui.guild_members_frame",
                        name = "guild_members",
                        path = "Assets/Frames/GuildMembersFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 联盟邀请界面
            table.insert(self.data, 
            {
                k = 'guild_invite',
                v = {
                        class = "gui.guild_invite_frame",
                        name = "guild_invite",
                        path = "Assets/Frames/GuildInviteFrame.prefab",
                        parent = self.floor3,
                    }
            })

            -- 联盟成员升阶降阶界面
            table.insert(self.data, 
            {
                k = 'guild_member_operate',
                v = {
                        class = "gui.guild_member_operate_frame",
                        name = "guild_member_operate",
                        path = "Assets/Frames/GuildMemberOperateFrame.prefab",
                        parent = self.floor3,
                    }
            })

            -- 联盟留言板界面
            table.insert(self.data, 
            {
                k = 'guild_notes',
                v = {
                        class = "gui.guild_notes_frame",
                        name = "guild_notes",
                        path = "Assets/Frames/GuildNotesFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 联盟日志界面
            table.insert(self.data, 
            {
                k = 'guild_logs',
                v = {
                        class = "gui.guild_logs_frame",
                        name = "guild_logs",
                        path = "Assets/Frames/GuildLogsFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 联盟礼物界面
            table.insert(self.data, 
            {
                k = 'guild_gifts',
                v = {
                        class = "gui.guild_gifts_frame",
                        name = "guild_gifts",
                        path = "Assets/Frames/GuildGiftsFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 联盟设置界面
            table.insert(self.data, 
            {
                k = 'guild_set',
                v = {
                        class = "gui.guild_set_frame",
                        name = "guild_set",
                        path = "Assets/Frames/GuildSetFrame.prefab",
                        parent = self.floor1,
                    }
            })
            
            -- 联盟招募设定界面
            table.insert(self.data, 
            {
                k = 'guild_enlist',
                v = {
                        class = "gui.guild_enlist_frame",
                        name = "guild_enlist",
                        path = "Assets/Frames/GuildEnlistFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 联盟语言设定界面
            table.insert(self.data, 
            {
                k = 'guild_language',
                v = {
                        class = "gui.guild_language_frame",
                        name = "guild_language",
                        path = "Assets/Frames/GuildLanguageFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 联盟logo设定界面
            table.insert(self.data, 
            {
                k = 'guild_logo',
                v = {
                        class = "gui.guild_logo_frame",
                        name = "guild_logo",
                        path = "Assets/Frames/GuildLogoFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 联盟公告设定界面
            table.insert(self.data, 
            {
                k = 'guild_edit_notice',
                v = {
                        class = "gui.guild_edit_notice_frame",
                        name = "guild_edit_notice",
                        path = "Assets/Frames/GuildEditNoticeFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 联盟名字设定界面
            table.insert(self.data, 
            {
                k = 'guild_modify_name',
                v = {
                        class = "gui.guild_modify_name_frame",
                        name = "guild_modify_name",
                        path = "Assets/Frames/GuildModifyNameFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 联盟邮件界面
            table.insert(self.data, 
            {
                k = 'guild_mail',
                v = {
                        class = "gui.guild_mail_frame",
                        name = "guild_mail",
                        path = "Assets/Frames/GuildMailFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 联盟搬迁界面
            table.insert(self.data, 
            {
                k = 'guild_move_base',
                v = {
                        class = "gui.guild_move_base_frame",
                        name = "guild_move_base",
                        path = "Assets/Frames/GuildMoveBaseFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 联盟礼物查看界面
            table.insert(self.data, 
            {
                k = 'guild_gift_info',
                v = {
                        class = "gui.guild_gift_info_frame",
                        name = "guild_gift_info",
                        path = "Assets/Frames/GuildGiftInfoFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 道具使用界面
            table.insert(self.data, 
            {
                k = 'item_cost',
                v = {
                        class = "gui.item_cost_frame",
                        name = "item_cost",
                        path = "Assets/Frames/ItemCostFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 申请列表界面
            table.insert(self.data, 
            {
                k = 'guild_member_apply',
                v = {
                        class = "gui.guild_member_apply_frame",
                        name = "guild_member_apply",
                        path = "Assets/Frames/GuildMemberApplyFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 指挥官获取界面
            table.insert(self.data, 
            {
                k = 'commander_get',
                v = {
                        class = "gui.commander_get_frame",
                        name = "commander_get",
                        path = "Assets/Frames/CommanderGetFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 邮件界面
            table.insert(self.data, 
            {
                k = 'mail',
                v = {
                        class = "gui.mail_frame",
                        name = "mail",
                        path = "Assets/Frames/MailFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 聊天界面
            table.insert(self.data, 
            {
                k = 'chat',
                v = {
                        class = "gui.chat_frame",
                        name = "chat",
                        path = "Assets/Frames/ChatFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 聊天频道
            table.insert(self.data, 
            {
                k = 'chat_channel',
                v = {
                        class = "gui.chat_channel_frame",
                        name = "chat_channel",
                        path = "Assets/Frames/ChatChannelFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 盟主转让界面
            table.insert(self.data, 
            {
                k = 'guild_transfer',
                v = {
                        class = "gui.guild_transfer_frame",
                        name = "guild_transfer",
                        path = "Assets/Frames/GuildTransferFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 加速帮助界面
            table.insert(self.data, 
            {
                k = 'guild_help',
                v = {
                        class = "gui.guild_help_frame",
                        name = "guild_help",
                        path = "Assets/Frames/GuildHelpFrame.prefab",
                        parent = self.floor1,
                    }
            })

            --[[
            -- 联盟建筑升级界面
            table.insert(self.data, 
            {
                k = 'build_lvup',
                v = {
                        class = "gui.guild_build_lvup_frame",
                        name = "build_lvup",
                        path = "Assets/Frames/GuildBuildLvUpFrame.prefab",
                        parent = self.floor1,
                    }
            })
            ]]

            -- 商店界面
            table.insert(self.data, 
            {
                k = 'shop',
                v = {
                        class = "gui.shop_frame",
                        name = "shop",
                        path = "Assets/Frames/ShopFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 矿点信息界面
            table.insert(self.data, 
            {
                k = 'mine_info',
                v = {
                        class = "gui.mine_frame",
                        name = "mine_info",
                        path = "Assets/Frames/MineInfoFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 运输资源界面
            table.insert(self.data, 
            {
                k = 'transport_res',
                v = {
                        class = "gui.transport_mine_frame",
                        name = "transport_res",
                        path = "Assets/Frames/TransportMineFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 支援队伍界面
            table.insert(self.data, 
            {
                k = 'hold_support',
                v = {
                        class = "gui.hold_support_frame",
                        name = "hold_support",
                        path = "Assets/Frames/HoldSupportFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 世界地图战报
            table.insert(self.data, 
            {
                k = 'battle_record',
                v = {
                        class = "gui.battle_record_frame",
                        name = "battle_record",
                        path = "Assets/Frames/BattleRecordFrame.prefab",
                        parent = self.floor1,
                    }
            })

            --[[
            -- 资源转换界面
            table.insert(self.data, 
            {
                k = 'resource_convert',
                v = {
                        class = "gui.resource_convert_frame",
                        name = "resource_convert",
                        path = "Assets/Frames/ResourceConvertFrame.prefab",
                        parent = self.floor1,
                    }
            })
            ]]

            -- 资源兑换界面
            table.insert(self.data, 
            {
                k = 'resource_exchange',
                v = {
                        class = "gui.resource_exchange_frame",
                        name = "resource_exchange",
                        path = "Assets/Frames/ResourceExchangeFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 个人基地buff界面
            table.insert(self.data, 
            {
                k = 'home_buff',
                v = {
                        class = "gui.home_buff_frame",
                        name = "home_buff",
                        path = "Assets/Frames/HomeBuffFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 换装界面
            table.insert(self.data, 
            {
                k = 'home_skin',
                v = {
                        class = "gui.home_skin_frame",
                        name = "home_skin",
                        path = "Assets/Frames/HomeSkinFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 加速界面（行车，修理，建筑）
            table.insert(self.data, 
            {
                k = 'entity_speed_up',
                v = {
                        class = "gui.entity_speed_up_frame",
                        name = "entity_speed_up",
                        path = "Assets/Frames/EntitySpeedUpFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 个人基地通用加速界面
            table.insert(self.data, 
            {
                k = 'home_speed_up',
                v = {
                        class = "gui.home_speed_up_frame",
                        name = "home_speed_up",
                        path = "Assets/Frames/HomeSpeedUpFrame.prefab",
                        parent = self.floor1,
                    }
            })
            
            -- 使用道具滑动条界面
            table.insert(self.data, 
            {
                k = 'use_slider',
                v = {
                        class = "gui.use_slider_frame",
                        name = "use_slider",
                        path = "Assets/Frames/UseSliderFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 购买道具滑动条界面
            table.insert(self.data, 
            {
                k = 'buy_slider',
                v = {
                        class = "gui.buy_slider_frame",
                        name = "buy_slider",
                        path = "Assets/Frames/BuySliderFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 集结设置界面
            table.insert(self.data, 
            {
                k = 'create_mass',
                v = {
                        class = "gui.create_mass_frame",
                        name = "create_mass",
                        path = "Assets/Frames/OpenMassFrame.prefab",
                        parent = self.floor1,
                    }
            })

            table.insert(self.data, 
            {
                k = 'home_prison',
                v = {
                        class = "gui.home_prison_frame",
                        name = "home_prison",
                        path = "Assets/Frames/HomePrisonFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 集结详情界面
            table.insert(self.data, 
            {
                k = 'mass_info',
                v = {
                        class = "gui.mass_info_frame",
                        name = "mass_info",
                        path = "Assets/Frames/MassInfoFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 选择道具补给界面
            table.insert(self.data, 
            {
                k = 'select_level_item_frame',
                v = {
                        class = "gui.select_level_item_frame",
                        name = "repair_bag_box",
                        path = "Assets/Frames/SelectLevelItemFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 兵工厂生产界面
            table.insert(self.data, 
            {
                k = 'armory_frame',
                v = {
                        class = "gui.home_armory_frame",
                        name = "armory_frame",
                        path = "Assets/Frames/HomeArmoryFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 图纸列表界面
            table.insert(self.data, 
            {
                k = 'blueprint_select',
                v = {
                        class = "gui.blueprint_select_frame",
                        name = "blueprint_select",
                        path = "Assets/Frames/BlueprintSelectFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 通用加速界面
            table.insert(self.data, 
            {
                k = 'common_speed_up',
                v = {
                        class = "gui.common_speed_up_frame",
                        name = "common_speed_up",
                        path = "Assets/Frames/SpeedUpFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 战报提示界面
            table.insert(self.data, 
            {
                k = 'battle_tip',
                v = {
                        class = "gui.battle_tip_frame",
                        name = "battle_tip",
                        path = "Assets/Frames/BattleTipFrame.prefab",
                        parent = self.floor3,
                    }
            })

            -- 世界地图怪物信息
            table.insert(self.data, 
            {
                k = 'world_monster_info',
                v = {
                        class = "gui.world_monster_info_frame",
                        name = "world_monster_info",
                        path = "Assets/Frames/WorldMonsterInfoFrame.prefab",
                        parent = self.floor1,
                    }
            })

            table.insert(self.data,
            {
                k = 'chapter_achievement',
                v = {
                        class = "gui.chapter_achievement_frame",
                        name = "chapter_achievement",
                        path = "Assets/Frames/ChapterAchievementFrame.prefab",
                        parent = self.floor1,
                }
            })

            table.insert(self.data,
            {
                k = 'chapter_achievement_cutscene',
                v = {
                        class = "gui.chapter_achievement_cutscene_frame",
                        name = "chapter_achievement_cutscene",
                        path = "Assets/Frames/ChapterAchievementCutscene.prefab",
                        parent = self.floor2,
                }
            })

            table.insert(self.data,
            {
                k = 'chapter_start',
                v = {
                        class = "gui.chapter_start_frame",
                        name = "chapter_start",
                        path = "Assets/Frames/ChapterStartFrame.prefab",
                        parent = self.floor3,
                }
            })

            -- 搜索界面
            table.insert(self.data,
            {
                k = 'world_map_search',
                v = {
                        class = "gui.world_map_search_frame",
                        name = "world_map_search",
                        path = "Assets/Frames/WorldMapSearchFrame.prefab",
                        parent = self.floor1,
                }
            })

            -- 缩略图
            table.insert(self.data,
            {
                k = 'mini_map',
                v = {
                        class = "gui.mini_map_frame",
                        name = "mini_map",
                        path = "Assets/Frames/MiniMapFrame.prefab",
                        parent = self.floor1,
                }
            })

            -- 坐标收藏
            table.insert(self.data,
            {
                k = 'collect_coord',
                v = {
                        class = "gui.collect_coord_frame",
                        name = "collect_coord",
                        path = "Assets/Frames/CollectCoordFrame.prefab",
                        parent = self.floor1,
                }
            })

            -- 坐标管理
            table.insert(self.data,
            {
                k = 'coord_manager',
                v = {
                        class = "gui.coord_manager_frame",
                        name = "coord_manager",
                        path = "Assets/Frames/CoordManagerFrame.prefab",
                        parent = self.floor1,
                }
            })

            -- 收到攻击界面
            table.insert(self.data,
            {
                k = 'rim_red',
                v = {
                        class = "gui.rim_red_frame",
                        name = "rim_red",
                        path = "Assets/Frames/RimRedFrame.prefab",
                        parent = self.floor3,
                }
            })

            -- 出征界面
            table.insert(self.data,
            {
                k = 'launch_rally',
                v = {
                        class = "gui.launch_rally_frame",
                        name = "launch_rally",
                        path = "Assets/Frames/LaunchRallyFrame.prefab",
                        parent = self.floor1,
                }
            })

            -- 城墙修补
            table.insert(self.data,
            {
                k = 'home_wall_repair',
                v = {
                        class = "gui.home_wall_repair_frame",
                        name = "home_wall_repair",
                        path = "Assets/Frames/HomeWallRepairFrame.prefab",
                        parent = self.floor1,
                }
            })
            
            -- 驻守
            table.insert(self.data,
            {
                k = 'home_garrison',
                v = {
                        class = "gui.home_garrison_frame",
                        name = "home_garrison",
                        path = "Assets/Frames/HomeGarrisonFrame.prefab",
                        parent = self.floor1,
                }
            })

            -- 校场
            table.insert(self.data,
            {
                k = 'home_drill_ground',
                v = {
                        class = "gui.home_drill_ground_frame",
                        name = "home_drill_ground",
                        path = "Assets/Frames/HomeDrillGroundFrame.prefab",
                        parent = self.floor1,
                }
            })

            -- 领主信息
            table.insert(self.data,
            {
                k = 'lord_info',
                v = {
                        class = "gui.lord_info_frame",
                        name = "lord_info",
                        path = "Assets/Frames/LordInfoFrame.prefab",
                        parent = self.floor1,
                }
            })

            -- 其他领主信息
            table.insert(self.data,
            {
                k = 'other_lord_info',
                v = {
                        class = "gui.other_lord_info_frame",
                        name = "other_lord_info",
                        path = "Assets/Frames/OtherLordInfoFrame.prefab",
                        parent = self.floor1,
                }
            })

            -- 领主升级
            table.insert(self.data,
            {
                k = 'lord_level_up',
                v = {
                        class = "gui.lord_level_up_frame",
                        name = "lord_level_up",
                        path = "Assets/Frames/LordLevelUpFrame.prefab",
                        parent = self.floor1,
                }
            })

            -- 战力提升效果
            table.insert(self.data,
            {
                k = 'effect_power_up',
                v = {
                        class = "gui.effect_power_up_frame",
                        name = "effect_power_up",
                        path = "Assets/Arts/UIeffcet/Prefab/effect_power_up.prefab",
                        parent = self.floor2,
                }
            })
            
            -- 预警列表
            table.insert(self.data,
            {
                k = 'warn_info',
                v = {
                        class = "gui.warning_info_frame",
                        name = "warn_info",
                        path = "Assets/Frames/WarningInfoFrame.prefab",
                        parent = self.floor1,
                }
            })

            -- 领主天赋
            table.insert(self.data,
            {
                k = 'lord_talent',
                v = {
                        class = "gui.lord_talent_frame",
                        name = "lord_talent",
                        path = "Assets/Frames/LordTalentFrame.prefab",
                        parent = self.floor1,
                }
            })

            -- 通用通知界面
            table.insert(self.data,
            {
                k = 'notice_box',
                v = {
                        class = "gui.notice_box_frame",
                        name = "notice_box",
                        path = "Assets/Frames/NoticeBoxFrame.prefab",
                        parent = self.floor1,
                }
            })

            -- 使用道具
            table.insert(self.data,
            {
                k = 'use_item',
                v = {
                        class = "gui.use_item_frame",
                        name = "use_item",
                        path = "Assets/Frames/UseItemFrame.prefab",
                        parent = self.floor1,
                }
            })

            -- 天赋信息
            table.insert(self.data,
            {
                k = 'lord_talent_info',
                v = {
                        class = "gui.lord_talent_info_frame",
                        name = "lord_talent_info",
                        path = "Assets/Frames/LordTalentInfoFrame.prefab",
                        parent = self.floor1,
                }
            })

            -- 领主属性
            table.insert(self.data,
            {
                k = 'lord_prop',
                v = {
                        class = "gui.lord_prop_frame",
                        name = "lord_prop",
                        path = "Assets/Frames/LordPropFrame.prefab",
                        parent = self.floor1,
                }
            })

            -- 锻造厂主界面
            table.insert(self.data,
            {
                k = 'home_main_forge',
                v = {
                        class = "gui.home_main_forge_frame",
                        name = "home_main_forge",
                        path = "Assets/Frames/HomeMainForgeFrame.prefab",
                        parent = self.floor1,
                }
            })

            -- 锻造预览界面
            table.insert(self.data,
            {
                k = 'home_forge_preview',
                v = {
                        class = "gui.home_forge_preview_frame",
                        name = "home_forge_preview",
                        path = "Assets/Frames/HomeForgePreviewFrame.prefab",
                        parent = self.floor1,
                }
            })

            table.insert(self.data,
            {
                k = 'home_forge_upgrade',
                v = {
                        class = "gui.home_forge_upgrade_frame",
                        name = "home_forge_upgrade",
                        path = "Assets/Frames/HomeForgeUpgradeFrame.prefab",
                        parent = self.floor1,
                }
            })

            table.insert(self.data,
            {
                k = 'home_forge_store',
                v = {
                        class = "gui.home_forge_store_frame",
                        name = "home_forge_store",
                        path = "Assets/Frames/HomeForgeStoreFrame.prefab",
                        parent = self.floor1,
                }
            })

            table.insert(self.data,
            {
                k = 'home_forge_equip',
                v = {
                        class = "gui.home_forge_equip_frame",
                        name = "home_forge_equip",
                        path = "Assets/Frames/HomeForgeEquipFrame.prefab",
                        parent = self.floor1,
                }
            })

            table.insert(self.data,
            {
                k = 'home_forge_dismantle',
                v = {
                        class = "gui.home_forge_dismantle_frame",
                        name = "home_forge_dismantle",
                        path = "Assets/Frames/HomeForgeDismantleFrame.prefab",
                        parent = self.floor1,
                }
            })

            table.insert(self.data,
            {
                k = 'home_forge_obtain',
                v = {
                        class = "gui.home_forge_obtain_frame",
                        name = "home_forge_obtain",
                        path = "Assets/Frames/HomeForgeObtainFrame.prefab",
                        parent = self.floor1,
                }
            })

            table.insert(self.data,
            {
                k = 'home_forge_preview_plus',
                v = {
                        class = "gui.home_forge_preview_plus_frame",
                        name = "home_forge_preview_plus",
                        path = "Assets/Frames/HomeForgePreviewPlusFrame.prefab",
                        parent = self.floor1,
                }
            })

            table.insert(self.data,
            {
                k = 'home_forge_mat_exchange',
                v = {
                        class = "gui.home_forge_mat_exchange_frame",
                        name = "home_forge_mat_exchange",
                        path = "Assets/Frames/HomeForgeMatExchangeFrame.prefab",
                        parent = self.floor1,
                }
            })

            table.insert(self.data,
            {
                k = 'home_arsenal_main',
                v = {
                        class = "gui.home_arsenal_main_frame",
                        name = "home_arsenal_main",
                        path = "Assets/Frames/HomeArsenalMainFrame.prefab",
                        parent = self.floor1,
                }
            })

            table.insert(self.data,
            {
                k = 'home_arsenal_preview',
                v = {
                        class = "gui.home_arsenal_preview_frame",
                        name = "home_arsenal_preview",
                        path = "Assets/Frames/HomeArsenalPreviewFrame.prefab",
                        parent = self.floor1,
                }
            })

            table.insert(self.data,
            {
                k = 'home_arsenal_reward',
                v = {
                        class = "gui.home_arsenal_reward_frame",
                        name = "home_arsenal_reward",
                        path = "Assets/Arts/UIeffcet/Prefab/effect_arsenal_complete.prefab",
                        parent = self.floor1,
                }
            })

            -- 关卡 loading界面
            table.insert(self.data, 
            {
                k = 'level_loading',
                v = {
                        class = "gui.level_loading_frame",
                        name = "level_loading",
                        path = "Assets/Frames/LevelLoadingFrame.prefab",
                        parent = self.floor3,
                    }
            }) 

            -- 关卡选择界面
            table.insert(self.data, 
            {
                k = 'level_select',
                v = {
                        class = "gui.level_select_frame",
                        name = "level_select",
                        path = "Assets/Frames/LevelSelectFrame.prefab",
                        parent = self.floor1,
                    }
            }) 

            -- 成长任务界面
            table.insert(self.data, 
            {
                k = 'growth_task',
                v = {
                        class = "gui.growth_task_frame",
                        name = "growth_task",
                        path = "Assets/Frames/GrowthTaskFrame.prefab",
                        parent = self.floor1,
                    }
            }) 


            -- 联盟成员排行界面
            table.insert(self.data, 
            {
                k = 'guild_member_rank',
                v = {
                        class = "gui.guild_members_rank_frame",
                        name = "guild_member_rank",
                        path = "Assets/Frames/GuildMemberRankFrame.prefab",
                        parent = self.floor1,
                    }
            }) 


            -- 仓库
            table.insert(self.data, 
            {
                k = 'home_vault',
                v = {
                        class = "gui.home_vault_frame",
                        name = "home_vault",
                        path = "Assets/Frames/HomeVaultFrame.prefab",
                        parent = self.floor1,
                    }
            }) 

            -- 队列总览
            table.insert(self.data, 
            {
                k = 'summary_queue',
                v = {
                        class = "gui.summary_queue_frame",
                        name = "summary_queue",
                        path = "Assets/Frames/SummaryQueueFrame.prefab",
                        parent = self.floor0,
                    }
            }) 


            -- 酒馆
            table.insert(self.data, 
            {
                k = 'home_tavern',
                v = {
                        class = "gui.home_tavern_frame",
                        name = "home_tavern",
                        path = "Assets/Frames/HomeTavernFrame.prefab",
                        parent = self.floor1,
                    }
            }) 

            -- 酒馆
            table.insert(self.data, 
            {
                k = 'home_tavern_reward',
                v = {
                        class = "gui.home_tavern_reward_frame",
                        name = "home_tavern_reward",
                        path = "Assets/Arts/UIeffcet/Prefab/effect_tavern_getcommand.prefab",
                        parent = self.floor1,
                    }
            })

            -- 联盟派遣建造界面
            table.insert(self.data, 
            {
                k = 'guild_build_dispatch',
                v = {
                        class = "gui.guild_build_dispatch_frame",
                        name = "guild_build_dispatch",
                        path = "Assets/Frames/GuildBuildDispatchFrame.prefab",
                        parent = self.floor1,
                    }
            })  

            -- 联盟建造出征界面
            table.insert(self.data, 
            {
                k = 'guild_building_launch',
                v = {
                        class = "gui.guild_building_launch_frame",
                        name = "guild_building_launch",
                        path = "Assets/Frames/GuildBuildingLauchFrame.prefab",
                        parent = self.floor1,
                    }
            })   

            -- 联盟建造详情界面
            table.insert(self.data, 
            {
                k = 'guild_building_info',
                v = {
                        class = "gui.guild_building_info_frame",
                        name = "guild_building_info",
                        path = "Assets/Frames/GuildBuildingInfoFrame.prefab",
                        parent = self.floor1,
                    }
            }) 

            -- 联盟堡垒信息界面
            table.insert(self.data, 
            {
                k = 'guild_base_info',
                v = {
                        class = "gui.guild_base_info_frame",
                        name = "guild_base_info",
                        path = "Assets/Frames/GuildBaseInfoFrame.prefab",
                        parent = self.floor1,
                    }
            }) 

            -- 活动
            table.insert(self.data, 
            {
                k = 'activity_main',
                v = {
                        class = "gui.activity_main_frame",
                        name = "activity_main",
                        path = "Assets/Frames/ActivityMainFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 活动信息
            table.insert(self.data, 
            {
                k = 'activity_main_info',
                v = {
                        class = "gui.activity_main_info_frame",
                        name = "activity_main_info",
                        path = "Assets/Frames/ActivityMainInfoFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 活动详情
            table.insert(self.data, 
            {
                k = 'activity_detail',
                v = {
                        class = "gui.activity_detail_frame",
                        name = "activity_detail",
                        path = "Assets/Frames/ActivityDetailFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- 活动详情
            table.insert(self.data, 
            {
                k = 'activity_detail_info',
                v = {
                        class = "gui.activity_detail_info_frame",
                        name = "activity_detail_info",
                        path = "Assets/Frames/ActivityDetailInfoFrame.prefab",
                        parent = self.floor1,
                    }
            })

            -- commonbox3
            table.insert(self.data, 
            {
                k = 'common_box_3',
                v = {
                        class = "gui.common_box_3_frame",
                        name = "common_box_3",
                        path = "Assets/Frames/CommonBox3Frame.prefab",
                        parent = self.floor3,
                    }
            })

            -- 行军队列详情
            table.insert(self.data, 
            {
                k = 'army_queue_details',
                v = {
                        class = "gui.army_queue_details_frame",
                        name = "army_queue_details",
                        path = "Assets/Frames/ArmyQueueDetailsFrame.prefab",
                        parent = self.floor1,
                    }
            })
        end

        local cheat = not Fenix.Game.GameApp.IsSDKLogin() --是否开启GM指令界面
        if cheat then
            self:show('gm')
        end      

        -- 移到 game_director:additionalInit()
        --hint = self:show('hint')  

        self.floor_list = {}
        for __, ui in ipairs(self.data) do
            local temp = nil
            if ui.v.parent == self.floor0 then
                self.floor_list[0] = self.floor_list[0] or {}
                temp = self.floor_list[0]
            elseif ui.v.parent == self.floor1 then
                self.floor_list[1] = self.floor_list[1] or {}
                temp = self.floor_list[1]
            elseif ui.v.parent == self.floor2 then
                self.floor_list[2] = self.floor_list[2] or {}
                temp = self.floor_list[2]
            elseif ui.v.parent == self.floor3 then
                self.floor_list[3] = self.floor_list[3] or {}
                temp = self.floor_list[3]
            end
            table.insert(temp, ui.k)
        end
    end,

    getSortingOrder = function(self, name)
        local rst = 0
        local floor_id = 0
        local ui_data = self:get(name)
        if ui_data.parent == self.floor0 then
            rst = sorting_order0 + 1
            floor_id = 0
        elseif ui_data.parent == self.floor1 then
            rst = sorting_order1 + 1
            floor_id = 1
        elseif ui_data.parent == self.floor2 then
            rst = sorting_order2 + 1
            floor_id = 2
        elseif ui_data.parent == self.floor3 then
            rst = sorting_order3 + 1
            floor_id = 3
        end
        if self.floor_list and self.floor_list[floor_id] then
            rst = rst + #self.floor_list[floor_id]
        end
        return rst
    end,

    --显示界面
    show = function(self, name, owner, ...)
        --先判断是否存在这个界面
        local uidata = self:get(name)
        if(uidata == nil) then
            log.error("Faile to find frame name:"..name)
            return
        end

        if name == 'loading' then
            self.is_loading_show = true
            game_director:disable3DRender()
        end
        local arg = ...
        --判断这个界面是否已经打开
        for k,v in ipairs(self.visibleList) do
            if(v.key == name) then
                if name ~= "loading" then
                    log.info("frame " .. name .. " is Opened !!!")
                end

                if v.value.gameObject then
                    v.value.gameObject:SetActive(true)
                    if name == "loading" then
                        v.value:onLoaded(arg)
                    end
                    --v.value:onLoaded()
                end
                return v.value
            end
        end

        --创建该界面
        --log.error("#### Show frame:"..name)
        local uiframe = CreateFrame {
                            class = uidata.class, 
                            asset = uidata.path,
                            parent = uidata.parent,
                            name = name,
                            owner = owner,
                            data = arg,
                        }
        
        --把数据放到table里面
        table.insert(self.visibleList, { key = name, value = uiframe } )

        -- 加红点
        if red_point_manager then
            red_point_manager:tryAddRedPoint(name, uiframe)
        end
        return uiframe
    end,

    --隐藏界面
    hide = function(self, name)
        local pos = 1
        local gui = nil
        for k,v in ipairs(self.visibleList) do
            if(v.key == name) then
                gui = v.value
                break
            end
            pos = pos + 1
        end

        log.info("## hide frame:"..name)
        --log.info("###"..debug.traceback())

        if not gui then
            log.info("## no gui:"..name)
            return
        end

        if name == 'loading' then   -- loading 界面关闭前，进度条走到 1
            self.is_loading_show = false

            if gui:getProgress() >= 0.99 then
                game_director:enable3DRender()
                if gui.gameObject then
                    gui.gameObject:SetActive(false)
                    gui:onUnload()
                end
            else
                gui:setProgress(1.0, 0.1)
                --log.info("### hide loading, setProgress 1.0, 0.1")
                app:startCoroutine(function()   
                    coroutine.yield(WaitForSeconds(0.15))  
                    coroutine.yield(nil)  
                    --log.info("### delay hide loading frame!!")      
                    if gui.gameObject then
                        gui.gameObject:SetActive(false)
                        gui:onUnload()
                    end
                    game_director:enable3DRender()
                end)   
            end         
        else
            self:destroyFrame(gui)
            table.remove(self.visibleList, pos)
        end
       
    end,

    clickHome = function(self)
        for i=1, 2 do
            self:hideFloor(i)
        end
    end,

    -- 隐藏某层界面 0-4
    hideFloor = function(self, floor_id)
        if self.floor_list[floor_id] then
            local temp = self.floor_list[floor_id]
            for __, name in ipairs(temp) do
                local vsb, frame = self:isDisplay(name)
                if vsb then
                    self:hide(name)
                end
            end
        else
            log.error('不存在ui层', floor_id)
        end
    end,

    -- 隐藏所有界面
    hideAll = function(self)
        local i = 1
        while i <= #self.visibleList do 
            local item = self.visibleList[i]
            if item.key ~= 'gm' and item.key ~= 'hint' and item.key ~= 'loading' then 
                table.remove(self.visibleList, i)
                log.info("## Destroy frame:"..item.key)
                self:destroyFrame(item.value)
            else
                i = i + 1
            end
        end
    end,

    -- 销毁界面
    destroyFrame = function(self, uiframe)
        if uiframe and uiframe.transform then
            local bts = uiframe.transform:GetComponentsInChildren(UnityEngine.UI.Button).Table
            for k, v in ipairs(bts) do
                v.onClick:RemoveAllListeners()
            end
            uiframe:destroy()
        end
    end,

    -- 是否已经显示
    isDisplay = function(self, name)
        for k,v in ipairs(self.visibleList) do
            if(v.key == name) then
                return true, v.value
            end
        end
        return false, nil
    end,

    -- 开启/关闭 某层的操作
    enableFloorTouch = function(self, floor_id)
        local raycaster = self.raycasters[floor_id]
        if raycaster then
            raycaster.enabled = true
        end
    end,

    disableFloorTouch = function(self, floor_id)
        local raycaster = self.raycasters[floor_id]
        if raycaster then
            raycaster.enabled = false
        end
    end,

    enableTouch = function(self)
        for k,v in pairs(self.raycasters) do
            v.enabled = true
        end
    end,

    disableTouch = function(self)
        for k,v in pairs(self.raycasters) do
            v.enabled = false
        end
    end,

    -- 根据名称获取 frame
    getFrame = function(self, name)
        for k,v in ipairs(self.visibleList) do
            if(v.key == name) then
                return v.value
            end
        end
        return nil
    end,

    --判断是否有界面数据
    get = function(self, name)
        if(self.data == nil) then
            return
        else
            for k,v in ipairs(self.data) do
                if(v.k == name) then
                    return v.v
                end
            end
        end
    end,

    -- 逻辑更新
    update = function(self, dt)
        local i = 1
        while i <= #self.visibleList do 
            local item = self.visibleList[i]
            if item.value.update then
                item.value:update(dt)
            end
            i = i + 1
        end  
    end,

    -- 获得镜头
    getCamera = function(self)
        local root = self.root or GameObject.Find("gui/root")
        if root then
            local canvas = root:GetComponent(UnityEngine.Canvas)
            if canvas and canvas.worldCamera then
                return canvas.worldCamera
            end
        end
    end,

    setVisible = function(self, visible)
        if not self.camera then
            return
        end
        if visible then
            self.camera.cullingMask = 2 ^ UnityEngine.LayerMask.NameToLayer("UI")
        else
            self.camera.cullingMask = 0
        end
    end,

    -- 代替原 Fenix.Async.WaitForFrame，需要在協程中調用
    waitForFrame = function(self, frame)
        while not frame:isLoaded() do
            coroutine.yield(nil)
        end
        -- 加載完成
        return
    end,

    -- 隐藏/显示 某层，hide 函数实际上是销毁界面无法恢复。 这个函数是真的隐藏
    setFloorVisibility = function(self, floor_id, is_visible)

        local floor = self.floors[floor_id]
        local canvas = floor:GetComponent(UnityEngine.Canvas)
        canvas.enabled = is_visible
        if is_visible then
            self:enableFloorTouch(floor_id)
        else
            self:disableFloorTouch(floor_id)
        end
    end,
}