local gui_helper = require('gui.gui_helper')
local super = require("ui.frame")
local log = logger.get("Game", "get item Frame")

return class {
    super = super,

    onLoaded = function (self)
        super.onLoaded(self)

        self.owner = self.args.owner
        self.has_Confirm_Btn = self.args.data.has_Confirm_Btn
        self.items = self.args.data.items
        self.title = self.args.data.title
        self.message = self.args.data.message
        self.callback = self.args.data.callback

        clickEvents = {
            "Frame/HasButton/BG/Confirm",
            "Frame/NoButton",
        }

        self.helper = require('gui.tool.frame_helper')
        self.helper:regClickEvents("click", self, clickEvents)

        -- item prefab
        self.item_prefab = self.transform:Find('Prefab/Slot/Item').gameObject
        self.slot_prefab = self.transform:Find('Prefab/Slot').gameObject

        self.helper.hideTrans(self.slot_prefab)

        self.no_button_panel = self.transform:Find('Frame/NoButton')
        self.has_button_panel = self.transform:Find('Frame/HasButton')

        self.no_button_panel:SetActive(not self.has_Confirm_Btn)
        self.has_button_panel:SetActive(self.has_Confirm_Btn)

        local scrollView = self.transform:Find("Frame/NoButton/BG/Center/Items")
        self.slot_root_path = "Frame/NoButton/BG/Center/Items/Viewport/Content"
        if self.has_Confirm_Btn then
            self.slot_root_path = "Frame/HasButton/BG/Center/Items/Viewport/Content"
            scrollView = self.transform:Find("Frame/HasButton/BG/Center/Items")
        end
        self.slot_root = self.transform:Find(self.slot_root_path)
        self.sizeFitter = self.slot_root:GetComponent(UnityEngine.UI.ContentSizeFitter)
        self.scroll_rect = scrollView.rect

        self.text_message =  self.has_button_panel:Find("BG/Center/Message"):GetComponent(UnityEngine.UI.Text)
        if self.message then
            self.text_message.transform:SetActive(true)
            self.text_message.text = self.message
        else
            self.text_message.transform:SetActive(false)
            self.text_message.text = ""
        end

        self.icon_events = {}

        self:setDropItems()

        local title_text = self:findText("Frame/HasButton/BG/Top/Title_Prop")
        title_text.text = self.title and self.title or NativeText.GetLangStr(900003)
    end,

    onUnload = function(self)

    end,

    clearIconScripts = function(self)
        for path, evt in pairs(self.icon_events) do
            self:clearScript(evt, path)
        end
        self.icon_events = {}

    end,

    setDropItems = function(self)
        self:clearIconScripts()
        if not self.items then
            return
        end
        for k, v in ipairs(self.items) do
            local cfg = item_helper:getItemCfg(v.cfg_id)
            if cfg then
                local slot = nil
                if self.slot_root.childCount < k then
                    slot = GameObject.Instantiate(self.slot_prefab).transform
                    slot:SetParent(self.slot_root)
                    resetTransformZ(slot)
                else
                    slot = self.slot_root:GetChild(k-1)
                end

                local item = nil
            
                if slot.childCount == 0 then
                    item = GameObject.Instantiate(self.item_prefab).transform
                    item:SetParent(slot)
                    makeRectTransformIdentity(item) 
                else
                    item = slot:GetChild(0)
                end

                item:SetActive(true)

                local icon = item:Find('Icon')
                local num = item:Find('Num')
                local level = item:Find('Level')
                local piece = item:Find("Piece")

                piece:SetActive(item_helper:isOfficerPiece(v.cfg_id))
                level:SetActive(false)


                local grade = item:GetComponent(UnityEngine.UI.Image)
                grade:SetSprite(gui_helper:getQualityIconPath(cfg.color))

                -- 设置 icon
                local img = icon:GetComponent(UnityEngine.UI.Image)
                img.color = Color.white
                local icon_path = gui_helper:getItemIconPath(cfg)
                img:SetSprite(icon_path)

                -- 设置数量
                if v.count > 1 then
                    num:SetActive(true)
                    local text = num:GetComponent(UnityEngine.UI.Text)
                    -- text.text = tostring(v.count)
                    text.text = frame_helper:itemNumFormat(v.count)
                else
                    num:SetActive(false)
                end
                
                if self.has_Confirm_Btn then
                    slot.name = "slot"..k
                    local path = self.slot_root_path.."/"..slot.name.."/"..item.name.."/Icon"
                    self:setScript("single", path , functor.rmake(self.onClick_ItemIcon, v))
                    self.icon_events[path] = "single"
                end
            end
        end

        for i = #self.items, self.slot_root.childCount-1 do
            local slot = self.slot_root:GetChild(i)
            if slot.childCount > 0 then
                local item = slot:GetChild(0)
                item:SetActive(false)
            end
        end

        app:startCoroutine(
            function()
                coroutine.yield(nil)
                self:setContentPosition()
            end
        )
    end,

    setContentPosition = function(self)
        if self.slot_root.rect.width > self.scroll_rect.width then
            self.sizeFitter.enabled = false
            self.slot_root.pivot = Vector2(0, 0.5)
            local pos = self.slot_root.localPosition
            pos.x = 0
            self.slot_root.localPosition = pos
        end
    end,

    click_NoButton = function(self)
        frame_manager:hide('getItems')
    end,

    click_Confirm = function(self)
        frame_manager:hide('getItems')
        if self.callback then
            self.callback()
        end
    end,

    onClick_ItemIcon = function(self, handler, data, item_inst)
        log.info("onClick_ItemIcon ", cfg_id)
        frame_manager:show('itemInfo', self, {item = item_inst})
    end,

    update = function(self, dt)
        if Input.GetMouseButtonUp(0) then
            self:click_Confirm()
        end
    end,

    hide = function(self)
        self.helper.hideTrans(self.parent)
        self.helper.hideTrans(self.panel)
    end,

}