local log = logger.get("Game", "Growth Task Frame")
local super = require("ui.frame")
require('RTS.task_sys.task_manager')

return class{
    super = super,

    onLoaded = function(self)
        super:onLoaded(self)
        self.helper = require("gui.tool.frame_helper")

        clickEvents = 
        {
            "Root/TOP/buttonBack",
        }
        frame_helper:regClickEvents("click", self, clickEvents)
        self.evts = {}
        self.ui_entities = {} -- task id - ui_entity

        self.prefab_entity = self.transform:Find('Prefab/Entity')
        self.prefab_entity:SetActive(false)

        self.trans_content = self.transform:Find('Root/Outline/Scroll View/Viewport/Content')
        self.trans_recommanded_task_entity = 
            self.transform:Find('Root/Outline/Scroll View/Viewport/Content/RecommandedTask/Entity')

        self:refresh()

        self.changed_evts = {}
        self.changed_evts["onTaskFinished"] = task_manager.events:on("onTaskFinished", functor.make(self.onTaskFinished, self))
        self.changed_evts["onTaskChanged"] = task_manager.events:on("onTaskChanged", functor.make(self.onTaskChanged, self))
     
    end,

    -- 找到推荐任务
    findRecommandedTask = function(self)
        local tmp = {}
        self.tasks = task_manager:getTasks(common_pb.EGRWOTh_TASK)
        for k,v in pairs(self.tasks) do
            table.insert(tmp, v)
        end

        log.info("### num tasks:"..#tmp)
        -- 对任务排序找到推荐任务
        table.sort(tmp, function(a, b)
            if a.cfg.sort ~= b.cfg.sort then
                return a.cfg.sort < b.cfg.sort
            else
                return a.id < b.id
            end
        end)
        
        return tmp
    end,

    refresh = function(self)
        local tmp = self:findRecommandedTask()

        -- 推荐任务
        if #tmp > 0 then
            self.recommanded_task = tmp[1].id
            self:syncTaskToUI(0, tmp[1], self.trans_recommanded_task_entity)
            table.remove(tmp, 1)

            log.info("## add task:"..self.recommanded_task)
        else
            log.info('No growth task!')
            self.trans_recommanded_task_entity:SetActive(false)
        end

        -- 对任务排序
        table.sort(tmp, function(a, b)
            if a.state ~= b.state then
                return a.state < b.state
            else
                if a.cfg.sort ~= b.cfg.sort then
                    return a.cfg.sort < b.cfg.sort
                else
                    return a.id < b.id
                end
            end
            return false
        end)


        local in_progress_count = 0
        for idx, task in ipairs(tmp) do
            --log.info("### add task:"..task.id)

            local ui_entity = nil
            if self.trans_content.childCount > idx then
                ui_entity = self.trans_content:GetChild(idx)
            else
                ui_entity = GameObject.Instantiate(self.prefab_entity.gameObject).transform
                ui_entity.name = 'task_'..idx
                ui_entity:SetParent(self.trans_content)
                ui_entity.localPosition = Vector3.zero
                ui_entity.localScale = Vector3.one
            end

            self:syncTaskToUI(idx, task, ui_entity)

            if task.state == common_pb.TS_DOING then
                in_progress_count = in_progress_count + 1
                if in_progress_count >= config.task_configs[1].max_growth_task_in_progress then
                    break
                end
            end
        end

    end,

    -- 将任务信息同步到界面
    syncTaskToUI = function(self, idx, task, ui_entity)
        ui_entity:SetActive(true)        
        if self.ui_entities[task.id] == ui_entity then
            return
        end

        self.ui_entities[task.id] = ui_entity

        local anim = ui_entity:GetComponent(UnityEngine.Animation)
        anim:Play("get_idle")

        -- 任务名称
        local text_name = ui_entity:Find('text_QuestName'):GetComponent(UnityEngine.UI.Text)
        text_name.text = self.helper:getLangCfg(task.cfg.title)

        -- 任务描述
        local text_name = ui_entity:Find('text_QuestDesc'):GetComponent(UnityEngine.UI.Text)
        text_name.text = self.helper:getLangCfg(task.cfg.des_id)

        -- 取得唯一的 progress
        local progress = task.progress
        if progress then
            -- 进度
            local text_progress = ui_entity:Find('text_Progress'):GetComponent(UnityEngine.UI.Text)
            text_progress.text = progress.prog .. "/".. progress.total

            local img_progress = ui_entity:Find('ProgressBG/ProgressFill'):GetComponent(UnityEngine.UI.Image)
            img_progress.fillAmount = progress.prog / progress.total
        else
        end

        -- Banner
        if string.find(task.cfg.path, "car") then
            ui_entity:Find('Banner'):SetActive(true)    
            ui_entity:Find('BannerOfficer'):SetActive(false)              
            local img_banner = ui_entity:Find('Banner'):GetComponent(UnityEngine.UI.Image)
            img_banner:SetSprite(task.cfg.path..task.cfg.icon)      
        elseif string.find(task.cfg.path, "head") then
            ui_entity:Find('Banner'):SetActive(false)    
            ui_entity:Find('BannerOfficer'):SetActive(true)              
            local img_banner = ui_entity:Find('BannerOfficer'):GetComponent(UnityEngine.UI.Image)
            img_banner:SetSprite(task.cfg.path..task.cfg.icon)  
        end

        local img_banner = ui_entity:Find('Banner'):GetComponent(UnityEngine.UI.Image)
        img_banner:SetSprite(task.cfg.path..task.cfg.icon)

        -- 背景色
        local img_color_bg = ui_entity:Find('ColorBG'):GetComponent(UnityEngine.UI.Image)
        if task.cfg.bg_color == 1 then
            img_color_bg.color = Color(6/255, 241/255, 133/255)
        elseif task.cfg.bg_color == 2 then
            img_color_bg.color = Color(40/255, 208/255, 246/255)
        elseif task.cfg.bg_color == 3 then
            img_color_bg.color = Color(236/255, 7/255, 207/255)
        elseif task.cfg.bg_color == 4 then
            img_color_bg.color = Color(240/255, 165/255, 7/255)
        end

        -- 奖励
        local reward_tab = loadstring('return '..task.cfg.reward)()
        -- 对奖励排序
        local sorted_reward = {}
        for k,v in pairs(reward_tab) do
            table.insert(sorted_reward, {id = k, num = v})
        end
        table.sort(sorted_reward, function(a, b)
            return a.id > b.id
        end)


        local award_idx = 1
        for _,reward in ipairs(sorted_reward) do
            local id = reward.id
            local num = reward.num

            local icon_frame = ui_entity:Find("Rewards/IconFrame"..award_idx)
            if icon_frame == nil then
                log.error("奖励数量太多, task:"..task.id)
            end

            local img_frame = icon_frame:GetComponent(UnityEngine.UI.Image)
            local img_icon = icon_frame:Find("Icon"):GetComponent(UnityEngine.UI.Image)
            local text_reward = icon_frame:Find("NumBG/Num"):GetComponent(UnityEngine.UI.Text)

            img_frame:SetActive(true)

            local reward_cfg = item_helper:getItemCfg(id)
            if reward_cfg then
                 img_frame:SetSprite(gui_helper:getQualityIconPath(reward_cfg.color))               
                 img_icon:SetSprite(reward_cfg.path..reward_cfg.icon)
                 text_reward.text = tostring(num)
            else
            end
            award_idx = award_idx + 1
        end

        for award_idx = award_idx, 5 do
            local icon_frame = ui_entity:Find("Rewards/IconFrame"..award_idx)
            icon_frame:SetActive(false)
        end

        local btn_path = nil
        if idx == 0 then
            btn_path = "Root/Outline/Scroll View/Viewport/Content/RecommandedTask/Entity/buttonClaim"
        else
            btn_path = "Root/Outline/Scroll View/Viewport/Content/task_"..idx .. "/buttonClaim"
        end

        log.info("### path:"..btn_path)

        local trans_btn = self.transform:Find(btn_path)

        self.task_btn_cd = {}

        -- 按钮/回调
        if task.state == common_pb.TS_GETAWARD then
            log.info("## Add button callback!")
            trans_btn:SetActive(true)
            table.insert(self.evts, btn_path)
            self:UnregisterClick(btn_path)
            self:RegisterClick(btn_path, function()
                -- cd
                if not self.task_btn_cd[task.id] then
                    self.task_btn_cd[task.id] = 0.0
                end

                if Time.time - self.task_btn_cd[task.id] > 1.2 then
                    log.info("### req get task reward:"..task.id)                   
                    network_sender:reqGrowthTaskReward(task.id)
                    self.task_btn_cd[task.id] = Time.time
                end
            end)

        elseif task.state == common_pb.TS_DOING then
            trans_btn:SetActive(false)
        else
            log.error("state error:"..task.id..","..task.state)
        end
    end,

    onTaskFinished = function(self, id)
        local finished_entity = self.ui_entities[id]

        app:startCoroutine(
            function()        
                if finished_entity then
                    local anim = finished_entity:GetComponent(UnityEngine.Animation)
                    anim:Play("get_out")
                    coroutine.yield(WaitForSeconds(1.0))
                    finished_entity:SetActive(false)    
                end

                self:refresh()
            end
        )

    end,

    -- 新添加 task
    onTaskChanged = function(self, tasks)
        --log.info("## onTaskChanged!")
    end,

    onUnload = function(self)
        if self.changed_evts then
            for k, v in pairs(self.changed_evts) do
                task_manager.events:remove(k, v)
            end
        end
    end,

    click_buttonBack = function(self)
        frame_manager:hide('growth_task')
    end,

    hide = function(self)
        frame_manager:hide("growth_task")

        red_point_manager:refreshRedPoints()
    end,
}

