local log = logger.get("Game", "gui helper")
local frame_helper = require("gui.tool.frame_helper")
local item_cls = require('RTS.bag_sys.item')
local ui_tank_cls = require('gui.tool.ui_tank')
local long_press = require("gui.tool.long_press")
PlayerPrefs = UnityEngine.PlayerPrefs

makeRectTransformIdentity = function(trans)
    trans.localScale = Vector3.one
    trans.anchorMin = Vector2.zero
    trans.anchorMax = Vector2.one

    trans.offsetMin = Vector2.zero
    trans.offsetMax = Vector2.zero
    trans.localPosition = Vector3.zero
end

resetTransformZ = function(trans)
    local pos = trans.localPosition
    pos.z = 0
    trans.localPosition = pos
    trans.localScale = Vector3.one
end

-- 界面辅助
gui_helper = {
    current_item = nil,     -- 当前界面上选中的道具
    used_item_count = 0,     -- 当前使用的道具数量
    drop_list = {},

    -- 将 item 设置到界面上, 注意，这里的 self 是 bag_frame 或者 其他 frame
    setItemToSlot = function(self, idx, item_inst)
        local slot = self.slot_root:GetChild(idx-1)
        if not slot then
            log.error('Slot is nil!')
        end
        local ui_item = nil
        
        if slot.childCount == 0 then
            ui_item = GameObject.Instantiate(self.item_prefab).transform
            ui_item:SetParent(slot)
            makeRectTransformIdentity(ui_item) 
        else
            ui_item = slot:GetChild(0)
            ui_item:SetActive(true)
        end

        -- 将 ui 对象保存到 item 实例中
        item_inst.ui = ui_item

        gui_helper:refreshOneBagItem(self, item_inst, slot, ui_item, self.slot_root_path, self.onClick_ItemIcon)
        
        if self.slots then
            self.slots[item_inst.inst_id] = ui_item
        end
    end,

    -- 将 item 的信息刷新到 ui_item
    refreshOneBagItem = function(self, frame, item, ui_slot, ui_item, slot_root_path, click_callback)

        local icon = ui_item:Find('Icon')
        local num = ui_item:Find('Num')
        local level = ui_item:Find('Level')

        -- 设置 icon
        local img = icon:GetComponent(UnityEngine.UI.Image)
        img.color = Color.white
        local icon_path = gui_helper:getItemIconPath(item.cfg)
        img:SetSprite(icon_path)

        local path = slot_root_path.."/"..ui_slot.name.."/"..ui_item.name.."/Icon"
        --log.info('#### path:'..path)
        frame:setScript("single", path , functor.rmake(click_callback, item))
        local icon_events = frame.icon_events or frame.bag_icon_events
        icon_events[path] = "single"

        -- 设置数量
        if item.count > 1 then
            num:SetActive(true)
            local text = num:GetComponent(UnityEngine.UI.Text)
            text.text = tostring(item.count)
        else
            num:SetActive(false)
        end
        

        -- 设置强化等级
        if item.level and item.level > 0 then
            level:SetActive(true)
            local text = level:GetComponent(UnityEngine.UI.Text)
            text.text = "Lv."..item.level
        else
            level:SetActive(false)
        end

        -- 设置锁定状态
        local lock_text = ui_item:Find('LockText')
        if lock_text then
            lock_text:SetActive(item:isLocked())
        end
        local lock_img = ui_item:Find('Lock')
        if lock_img then
            lock_img:SetActive(false)
        end
        -- 已装备图标
        local equiped_img = ui_item:Find('Equiped')
        if equiped_img then
            equiped_img:SetActive(item.hero_id ~= nil)
        end

        -- 隐藏选中图标
        local selected_img = ui_item:Find('Selected')
        if selected_img then
            selected_img:SetActive(false)
        end   

        -- 设置品质框
        local quality = item.cfg.color
        gui_helper:setItemQualityIcon(ui_item, quality)    

        -- 新获得标记
        local new_mark = ui_item:Find("New")
        if new_mark then
            new_mark:SetActive(item.is_new or false)
        end

        local piece_tag = ui_item:Find("Piece")
        if piece_tag then
            piece_tag:SetActive(item_helper:isOfficerPiece(item.cfg_id))
        end
    end,

    -- 清除 idx 之后的 slot
    clearSlotsAfter = function(self, slot_root, idx)
        local i = 0
        for i = idx, slot_root.childCount-1, 1 do
            if i < slot_root.childCount then
                local slot = slot_root:GetChild(i)
            
                if slot.childCount > 0 then
                    local item = slot:GetChild(0)
                    item:SetActive(false)
                end
            end
        end
    end,

    -- 显示 props 中的属性 (升级属性)
    showUpgradeProps = function(self, prop_panel, props, changes, showPlusStr)
        local names = prop_panel:Find('PropNames')
        local values = prop_panel:Find('PropValues')

        local idx = 0   -- 0 下标开始的属性索引，统计共多少个有效属性
        for k,v in ipairs(props) do
            if v.PropValue == 0 and (not changes or not changes[v.PropId]) then
                -- 忽略 0 值属性
            else
                if idx < names.childCount then
                    names:GetChild(idx):SetActive(true)
                    values:GetChild(idx):SetActive(true)
                    local prop_cfg = self:getPropCfg(v.PropId)
                    local text_name = names:GetChild(idx):GetComponent(UnityEngine.UI.Text)
                    --log.info('prop cfg ',v.PropId, prop_cfg.name_id)
                    text_name.text = frame_helper:getLangCfg(prop_cfg.name_id)
                    local text_value = values:GetChild(idx):GetComponent(UnityEngine.UI.Text)
                    if prop_cfg.isfloat > 0 then
                        text_value.text = tostring(v.PropValue * 0.0001)
                    else
                        text_value.text = tostring(v.PropValue)
                    end

                    if prop_cfg.showpercent > 0 then
                        text_value.text = tonumber(text_value.text)*100 .."%"
                    end

                    -- 显示 +999
                    if showPlusStr then
                        text_value.text = "+"..text_value.text
                    end

                    -- 属性变化
                    if changes then
                        local trans_arrow = text_value.transform:GetChild(0)
                        if changes[v.PropId] and changes[v.PropId] ~= v.PropValue then
                            trans_arrow:SetActive(true)
                            local to_text = trans_arrow:GetChild(0):GetComponent(UnityEngine.UI.Text)
                            local value = changes[v.PropId]
                            if prop_cfg.isfloat > 0 then
                                value = value * 0.0001
                            end
                            if prop_cfg.showpercent > 0 then
                                to_text.text = value*100 .."%"
                            else
                                to_text.text = tostring(value)
                            end

                            if showPlusStr then
                                to_text.text = "+"..to_text.text
                            end
                        else
                            trans_arrow:SetActive(false)
                        end
                    else
                        if text_value.transform.childCount > 0 then
                            local trans_arrow = text_value.transform:GetChild(0)
                            trans_arrow:SetActive(false)
                        end
                    end

                    idx = idx + 1
                end
            end
        end

        for idx=idx, names.childCount-1, 1 do
            names:GetChild(idx):SetActive(false)
            values:GetChild(idx):SetActive(false)
        end
    end,

    --设置指挥官Icon
    setOfficerIcon = function(self, officer, stars, qualityLv, head_trans)
        local icon = head_trans:Find('Grade/Icon'):GetComponent(UnityEngine.UI.Image)
        local grade = head_trans:Find('Grade'):GetComponent(UnityEngine.UI.Image)
        local quality = head_trans:Find('Grade/Quality'):GetComponent(UnityEngine.UI.Text)
        local star_layout = head_trans:Find('Grade/Stars')

        local icon_path = self:getOfficerIconPath(officer)
        if icon_path then
            icon:SetSprite(icon_path)
        end
        local name_id = config.commander_qualitys[qualityLv].name
        quality.text = frame_helper:getLangCfg(name_id)
        local star_prefab = star_layout:GetChild(0).gameObject
        
        for i=2, stars do
            if i > star_layout.childCount then
                local star = Object.Instantiate(star_prefab)
                star.transform:SetParent(star_layout)
                resetTransformZ(star)
            else
                star_layout:GetChild(i-1):SetActive(true)
            end
        end

        for i = stars, star_layout.childCount - 1 do
            star_layout:GetChild(i):SetActive(false)
        end
    end,

    getHeroTypeIconPath = function(self, tank_id)
        local tank_cfg = config.tank_basic_ids[tank_id]
        if tank_cfg then
            return "Assets/Arts/ui/atlas/icon/vehicle/"..tank_cfg.type..".png"
        else
            log.error("#### get hero type error ! id", tank_id)

            return "Assets/Arts/ui/atlas/icon/vehicle/type1.png"
        end
    end,

    setOfficerHeadIcon = function(self, node, hero)
        local grade = node:GetComponent(UnityEngine.UI.Image)
        local icon = node:Find("Icon"):GetComponent(UnityEngine.UI.Image)
        local type_img = node:Find("Type"):GetComponent(UnityEngine.UI.Image)

        local officer = hero.officer
        grade:SetSprite(self:getQualityIconPath(officer.cfg.color))
        icon:SetSprite(self:getOfficerIconPath(officer))
        type_img:SetSprite(self:getHeroTypeIconPath(hero.tank_id))

        local name = node:Find("Name")
        if name then
            name:GetComponent(UnityEngine.UI.Text).text = self:getOfficerNameFromId(hero.id)
        end

        local hp = node:Find("HpBG/Hp")
        if hp then
            local cur_hp = hero:getHeroPorp(common_pb.PARTNER_PROP_HP)
            local max_hp = hero:getHeroPorp(common_pb.PARTNER_PROP_MAX_HP)

            local img = hp:GetComponent(UnityEngine.UI.Image)
            img.fillAmount = cur_hp.PropValue / (max_hp.PropValue > 0 and max_hp.PropValue or 1)
        end
    end,

    setMWMOfficerHeadIcon = function(self, node, hero)
        local grade = node:GetComponent(UnityEngine.UI.Image)
        local icon = node:Find("Icon"):GetComponent(UnityEngine.UI.Image)
        local name = node:Find("Name"):GetComponent(UnityEngine.UI.Text)
        local type_img = node:Find("Type"):GetComponent(UnityEngine.UI.Image)

        local officer = hero.officer
        local cfg = config.commander_basics[officer.officerId]
        if cfg then
            icon:SetSprite(cfg.path..cfg.icon)
        else
            log.error("##### set head icon res cfg error! id", officer.officerId)
        end
        grade:SetSprite(self:getQualityIconPath(cfg.color))
        if officer.name and string.len(officer.name) >0 then
            name.text = officer.name
        else
            name.text = frame_helper:getLangCfg(cfg.commander_name)
        end
        type_img:SetSprite(self:getHeroTypeIconPath(hero.tankId))

        local hp = node:Find("HpBG/Hp")
        if hp then
            local cur_hp = 0
            local max_hp = 0
            for k, v in ipairs(hero.ParPropIdValues) do
                if v.PropId == common_pb.PARTNER_PROP_HP then
                    cur_hp = v.PropValue
                end

                if v.PropId == common_pb.PARTNER_PROP_MAX_HP then
                    max_hp = v.PropValue
                end
            end

            local img = hp:GetComponent(UnityEngine.UI.Image)
            img.fillAmount = cur_hp / (max_hp > 0 and max_hp or 1)
        end
    end,

    -- 设置指挥官天赋属性
    setOfficerSkillProps = function(self, props_panel, cur_props, next_props, showPlus)
        local props = {}
        local props_change = {}
        for k, v in pairs(cur_props) do
            table.insert( props, {PropId = k, PropValue = v} )

            if next_props then
                props_change[k] = next_props[k]
            else
                props_change = nil
            end
        end
        gui_helper:showUpgradeProps(props_panel, props, props_change, showPlus)
    end,

    -- 根据hero id 拿到officer配置
    convertHeroToOfficer = function(self, hero_id)
        local hero = hero_manager:getHero(hero_id)
        if hero then
            return hero.officer.cfg
        else
            return nil
        end
    end,
    
    -- 根据officer的id 拿到tank配置
    convertOfficerToHero = function(self, officer_id)
        local hero_id = config.commander_basics[officer_id].tank_id
        return config.tank_basic_ids[hero_id]
    end,

    getPropCfg = function(self, prop_id)
        local cfg = nil
        if prop_id < 1000 then
            cfg = config.attribute_config_attrs[prop_id]
        else
            cfg = config.player_config_attrs[prop_id]
        end
        return cfg
    end,

    -- 获取一条属性的文字描述
    getPropDesc = function(self, prop_id, prop_value1, prop_value2)
        local cfg = self:getPropCfg(prop_id)
        if cfg then

            local str_value1 = nil
            local str_value2 = nil

            if prop_value1 then
                local value = prop_value1
                if cfg.isfloat > 0 then
                    value = value * 0.0001
                end
                if cfg.showpercent > 0 then
                    value = value * 100
                    str_value1 = tostring(value).."%"
                else
                    str_value1 = tostring(value)
                end
            end

            if prop_value2 then
                local value = prop_value2
                if cfg.isfloat > 0 then
                    value = value * 0.0001
                end
                if cfg.showpercent > 0 then
                    value = value * 100
                    str_value2 = tostring(value).."%"
                else
                    str_value2 = tostring(value)
                end
            end

            local prop_desc =  frame_helper:getLangCfg(cfg.name_id)
            if not prop_desc then
                log.error('Failed to find prop name in lang_cfg:'..cfg.name_id)
                prop_desc = "[Prop Name]:"..cfg.name_id
            end
            return prop_desc, str_value1, str_value2
        else
            log.error('Faile to find prop cfg:'..prop_id)
        end
    end,

    -- 将属性表字符串{[k]=v, [k]=v} 转 为属性的文字描述 形如 攻击 +10
    propTab2Desc = function(self, tab_str, aline)
        if not tab_str then
            log.error(debug.traceback())
            return 'nil!'
        end

        local str = ""
        local prop_tab = loadstring('return '..tab_str)()
        for prop_id,prop_value in pairs(prop_tab) do
            --log.info('##### id:'..prop_id..' - value:'..prop_value)
            local split = "\n"
            if aline then
                split = "  "
            end
            local desc, value_desc = gui_helper:getPropDesc(prop_id, prop_value)
            str = str..tostring(desc).." + "..tostring(value_desc)..split
        end
        return str
    end,

    -- 品质对应颜色
    quality2Color = function(self, quality)
        if quality == 0 then
            return '#a1a1a1'
        elseif quality == 1 then
            return '#d9d9d9'
        elseif quality == 2 then
            return '#64FF00'
        elseif quality == 3 then
            return '#00D2FF'
        elseif quality == 4 then
            return '#E400FF'
        elseif quality == 5 then
            return '#F39800'
        end
            return '#FFFFFF'
    end,

    -- 品质对应颜色
    quality2ColorPlus = function(self, quality)
        if quality == 0 then
            return UnityEngine.Color(161/255,161/255,161/255, 1)
        elseif quality == 1 then
            return UnityEngine.Color(217/255,217/255,217/255, 1)
        elseif quality == 2 then
            return UnityEngine.Color(100/255,255/255,0/255, 1)
        elseif quality == 3 then
            return UnityEngine.Color(0/255,210/255,255/255, 1)
        elseif quality == 4 then
            return UnityEngine.Color(228/255,0/255,202/255, 1)
        elseif quality == 5 then
            return UnityEngine.Color(243/255,152/255,0/255, 1)
        end
            return UnityEngine.Color(1,1,1,1)
    end,

    quality2UnityColor = function(self, quality)
        if quality == 0 then
            return Color(160/255, 160/255, 160/255)
        elseif quality == 1 then
            return Color(217/255, 217/255, 217/255)
        elseif quality == 2 then
            return Color(100/255, 255/255, 0/255)
        elseif quality == 3 then
            return Color(0/255, 210/255, 255/255)
        elseif quality == 4 then
            return Color(228/255, 0/255, 202/255)
        elseif quality == 5 then
            return Color(243/255, 152/255, 0/255)
        end
            return Color.white    
    end,

    getQualityStr = function(self, quality)
        if quality == 1 then
            return NativeText.GetLangStr(110508)
        elseif quality == 2 then
            return NativeText.GetLangStr(110509)
        elseif quality == 3 then
            return NativeText.GetLangStr(110510)
        elseif quality == 4 then
            return NativeText.GetLangStr(110511)
        elseif quality == 5 then
            return NativeText.GetLangStr(110512)
        end

        return NativeText.GetLangStr(110508)
    end,

    addColorTag = function(self, str, color_value)
        return "<color="..color_value..">"..str.."</color>"
    end,

    -- 获取道具的 icon 路径
    getItemIconPath = function(self, item_cfg)
        if item_cfg.path then 
            return item_cfg.path..item_cfg.icon
        else
            return "Assets/Arts/ui/atlas/icon/item/"..item_cfg.icon
        end
    end,

    -- 获取道具的 icon 路径
    getItemIconPathFromId = function(self, item_id)
        local cfg = item_helper:getItemCfg(item_id)

        if cfg then
            return self:getItemIconPath(cfg)
        else
            log.errro("### get item icon path error ! id", item_id)
            return "Assets/Arts/ui/atlas/icon/item/item_1.png"
        end
    end,

    -- 获取掉落道具
    getDropItems = function(self, drop_id)
        local items = {}
        
        if not dropbag_helper.over_view[drop_id] then
            log.error("##### drop over view error !", drop_id)
            return
        end

        for k1, v1 in pairs(dropbag_helper.over_view[drop_id]) do

            local drop_bag = dropbag_helper.drop_bag[v1.dropbag_id]
            if drop_bag then
                for k2, v2 in pairs(drop_bag) do
                    if v2.isshow == 1 then
                        local item_cfg = item_helper:getItemCfg(v2.drop_item)
                        if not item_cfg then
                            log.error("### drop item not exist ! id",v2.drop_item)
                        else
                            table.insert( items, item_cfg )
                        end
                    end
                end
            else
                log.error("#### drop bag error ! dropbag id ", v1.dropbag_id)
            end
        end

        local sort_func = function(a, b)
            return a.color > b.color
        end
        
        table.sort( items, sort_func )

        return items
    end,

    -- 获得道具时展示道具
    showGetItemsPanel = function(self, msg, has_btn)
        self.drop_list[msg.reason] = {}
        for _, item in ipairs(msg.getItems) do
            local item_inst = {}
            if item.uId and item.uId ~= "0" then
                local cfg = item_helper:getItemCfg(item.itemId)
                local bag = bag_manager:getBag(cfg.bagtype)
                bag.items[item.uId]:copy(item_inst)
                item_inst.count = item.getNum
            else
                item_inst = item_cls({cfgId = item.itemId, count = item.getNum})
            end
            log.info("####### drop item id ",item.itemId, msg.reason)
            table.insert(self.drop_list[msg.reason], item_inst)
        end
    end,

    -- 创建 UI 上展示用的 tank
    createUITank = function(self, hero_id, tank_id)
        local bag = bag_manager:getHeroBag(hero_id)    -- hero bag
        local weapon = bag:get(common_pb.EQUIP_WEAPON_SLOT)
        local wheel = bag:get(common_pb.WHEEL_SLOT)
        local weapon_id = nil
        local wheel_id = nil
        if weapon then weapon_id = weapon.cfg_id end
        if wheel then wheel_id = wheel.cfg_id end

        local minor_weapons = 
        {
            bag:get(common_pb.MINOR_WEAPON_SLOT_1),
            bag:get(common_pb.MINOR_WEAPON_SLOT_2),
            bag:get(common_pb.MINOR_WEAPON_SLOT_3),
            bag:get(common_pb.MINOR_WEAPON_SLOT_4),
        }
        local ui_tank = 
            ui_tank_cls(tank_id , weapon_id, wheel_id, minor_weapons)

        return ui_tank
    end,

    -- 保存快速选择 toggle 状态到本地
    saveToggleState = function(self, toggle_root, save_key)
        local str = "{"
        for i=1, toggle_root.childCount do
            local toggle = toggle_root:GetChild(i-1):GetComponent(UnityEngine.UI.Toggle)
            str = str..tostring(toggle.isOn)..","
        end
        str = str.."}"

        PlayerPrefs.SetString(save_key, str)
    end,

    -- 设置快速选择 toggle 状态
    setToggleState = function(self, toggle_root, save_key)
        if PlayerPrefs.HasKey(save_key) then
            local str = PlayerPrefs.GetString(save_key)
            str = loadstring("return "..str)()
            if type(str) ~= "table" then
                PlayerPrefs.DeleteKey(save_key)
                return
            end

            for i=1, toggle_root.childCount do
                local toggle = toggle_root:GetChild(i-1):GetComponent(UnityEngine.UI.Toggle)
                toggle.isOn = str[i] or false
            end
        end
    end,

    -- 获取指挥官立绘路径
    getOfficerPicturePath = function(self, officer)
        local cfg = officer.cfg

        if cfg then
            return cfg.cmd_path..cfg.commander_pic
        else
            log.error("officer picture cant find !")
            return ""
        end
    end,

    -- 获取指挥官 icon 路径
    getOfficerIconPath = function(self, officer)
        local cfg = officer.cfg

        if cfg then
            return cfg.path..cfg.icon
        else
           -- log.error("officer icon cant find ! officer id:", officer.inst_id, officer.res_id)
           -- log.error(debug.traceback())
            return "Assets/Arts/ui/atlas/icon/head/head_1.png"
        end
    end,

    getOfficerIconFromId = function(self, hero_id)
        local hero = hero_manager:getHero(hero_id)

        if not hero then
            log.error("##### getOfficerIconFromId hero error !", hero_id)
            return
        end

        local officer = hero.officer
        if not officer then
            log.error("##### getOfficerIconFromId officer error !", hero_id)
            return
        end

        local cfg = officer.cfg

        if cfg then
            return cfg.path..cfg.icon
        else
            log.error("getOfficerIconFromId cfg error !", hero.tank_id, officer.cfg_id, officer.res_id)
            log.error(debug.traceback())
            return "Assets/Arts/ui/atlas/icon/head/head_1.png"
        end
    end,

    getOfficerNameFromId = function(self, hero_id)
        local hero = hero_manager:getHero(hero_id)

        if not hero then
            log.error("##### getOfficerIconFromId hero error !", hero_id)
            return
        end

        local officer = hero.officer
        if not officer then
            log.error("##### getOfficerIconFromId officer error !", hero_id)
            return
        end

        return officer.name
    end,

    -- 获取品质 icon 路径
    getQualityIconPath = function(self, quality)
        local icon = res_path.res_path_10   --"growup_icon_8_4.png"
        if quality == 1 then
            icon = res_path.res_path_11   --"growup_icon_8_5.png"
        elseif quality == 2 then
            icon = res_path.res_path_12   --"growup_icon_8_2.png"
        elseif quality == 3 then
            icon = res_path.res_path_13   --"growup_icon_8_1.png"
        elseif quality == 4 then
            icon = res_path.res_path_14   --"growup_icon_8_3.png"
        elseif quality == 5 then
            icon = res_path.res_path_15   --"growup_icon_8_6.png"
        end
        
        return icon     --"Assets/Arts/ui/atlas/icon/color/"..icon
    end,

    getNewQualityIcon = function(self, quality)
        local path = res_path.res_path_95

        if quality == 1 then
            path = res_path.res_path_96
        elseif quality == 2 then
            path = res_path.res_path_97
        elseif quality == 3 then
            path = res_path.res_path_98
        elseif quality == 4 then
            path = res_path.res_path_99
        elseif quality == 5 then
            path = res_path.res_path_100
        end

        return path
    end,

    getHeroNameQualityIcon = function(self, quality)
        local path = res_path.res_path_119

        if quality == 1 then
            path = res_path.res_path_119
        elseif quality == 2 then
            path = res_path.res_path_120
        elseif quality == 3 then
            path = res_path.res_path_121
        elseif quality == 4 then
            path = res_path.res_path_122
        elseif quality == 5 then
            path = res_path.res_path_123
        end

        return path
    end,

    -- 获取技能描述
    getSkillDes = function(self, skill_id)
        local cfg = config.skill_skills[skill_id]

        if cfg then
            local desc = frame_helper:getLangCfg(cfg.describe)
            for i=1,10,1 do
                local replace = '%(p'..i..'%)'
                local k = 'p'..i
                desc = string.gsub(desc, replace, cfg[k])
            end
            return desc
        else
            log.error("skill cfg cant find ! id: ",skill_id)
        end

        return nil
    end,

    -- 获取技能icon
    getSkillIconPath = function(self, skill_id)
        local cfg = config.skill_skills[skill_id]
        local path = res_path.res_path_52

        if cfg then
            path = cfg.skill_iconpath..cfg.skill_icon
        else
            log.error("skill cfg cant find ! id: ",skill_id)
        end

        return path
    end,

    -- 设置道具品质框
    setItemQualityIcon = function(self, trans, quality)
        local icon = self:getQualityIconPath(quality)
        local image = trans:GetComponent(UnityEngine.UI.Image)
        if image then
            image:SetSprite(icon)
        end
    end,

    
    -- 填充界面背包
    -- frame          - 所属的 frame
    -- items          - 有序的道具列表
    -- slot_prefab - ui 界面对象 prefab, 参考 gamble_frame 的 bag_item_prefab
    -- slot_root       - ui 界面对象的跟结点
    -- click_callback - 点击回调 
    refreshBagItems = function(self, frame, items, slot_prefab, slot_root, slot_root_path, click_callback)
        local ui_slot = nil
        local ui_item = nil
        -- 遍历 item
        for idx,item in ipairs(items) do
            if slot_root.childCount >= idx then
                ui_slot = slot_root:GetChild(idx-1)
            else
                ui_slot = GameObject.Instantiate(slot_prefab).transform
                ui_slot.name = "slot_"..idx
                ui_slot:SetParent(slot_root)
                makeRectTransformIdentity(ui_slot)
            end
            ui_item = ui_slot:GetChild(0)
            ui_item:SetActive(true)

            item.ui = ui_item   -- 将 ui 对象保存到 item 中

            self:refreshOneBagItem(frame, item, ui_slot, ui_item, slot_root_path, click_callback)
        end

        self:clearSlotsAfter(slot_root, #items)
    end,

    -- 添加道具到背包的首位
    insertItemToBagHead = function(self, frame, item, slot_prefab, slot_root, slot_root_path, click_callback)
        local ui_slot = nil
        local ui_item = nil

        ui_slot = GameObject.Instantiate(slot_prefab).transform
        ui_slot.name = "slot_"..(slot_root.childCount+1)
        ui_slot:SetParent(slot_root)
        ui_slot:SetAsFirstSibling()
        makeRectTransformIdentity(ui_slot)

        ui_item = ui_slot:GetChild(0)
        ui_item:SetActive(true)
        item.ui = ui_item   -- 将 ui 对象保存到 item 中
        self:refreshOneBagItem(frame, item, ui_slot, ui_item, slot_root_path, click_callback)
    end,

    -- 填充商品信息到界面
    refreshShopItems = function(self, frame, shop_items, item_prefab, item_root, item_root_path, click_callback)
        local ui_item = nil
        for idx, shop_item in ipairs(shop_items) do
            if item_root.childCount >= idx then
                ui_item = item_root:GetChild(idx-1)
            else
                ui_item = GameObject.Instantiate(item_prefab).transform
                ui_item.name = "item_"..idx
                ui_item:SetParent(item_root)
                makeRectTransformIdentity(ui_item)
            end

            local icon = ui_item:Find("Icon")
            local price_icon = icon:Find("PriceIcon")
            local title = icon:Find("Title")
            local price = icon:Find("Price")

             -- 设置 icon
            local img = icon:GetComponent(UnityEngine.UI.Image)
            img.color = Color.white
            local icon_path = gui_helper:getItemIconPath(shop_item.cfg)
            img:SetSprite(icon_path)

            -- 设置道具名称
            local text_title = title:GetComponent(UnityEngine.UI.Text)
            text_title.text = frame_helper:getLangCfg(shop_item.cfg.name_id)

            -- 设置价格
            local text_price = price:GetComponent(UnityEngine.UI.Text)
            text_price.text = shop_item.cost

            -- 价格道具 icon
            local currency_item_cfg = item_helper:getItemCfg(shop_item.cost_item_id)
            img = price_icon:GetComponent(UnityEngine.UI.Image)
            img:SetSprite(gui_helper:getItemIconPath(currency_item_cfg))

            -- 设置商品点击回调
            if shop_item.can_buy then
                local path = item_root_path.."/"..ui_item.name.."/Icon"
                --log.info('#### path:'..path)
                frame:setScript("single", path , functor.rmake(click_callback, shop_item))
                frame.shop_icon_events[path] = "single"
            else
                if frame.shop_icon_events[path] then
                    self:clearScript(frame.shop_icon_events[path], path)
                end
                img.color = Color.gray
            end

            -- 设置货币 icon

        end

        self:clearSlotsAfter(item_root, #shop_items)
    end,

    -- 显示信息框，带一个确认按钮
    showMessageBox = function(self, message, callback)
        message = message:gsub("\\n", "\n")
        local tb = {
            title = "",
            content = message,
            buttons =
            {
                {
                    btn_name = NativeText.GetLangStr(160008),
                    callback = function()
                        if callback then callback() end
                        frame_manager:hide("common_box")
                    end
                },
            },
            onClose = function()
                if callback then callback() end
            end,
        }

        frame_manager:show('common_box', nil, tb)
    end,

    -- 显示 ok, cancel 信息框
    showOkCancelBox = function(self, message, ok_callback, cancel_callback)
        message = message:gsub("\\n", "\n")
        local tb = {
            title = "",
            content = message,
            buttons =
            {
                {
                    btn_name = NativeText.GetLangStr(160008),
                    callback = function()
                        if ok_callback then ok_callback() end
                        frame_manager:hide("common_box")
                    end
                },
                {
                    btn_name = NativeText.GetLangStr(160009),
                    callback = function()
                        if cancel_callback then cancel_callback() end
                        frame_manager:hide("common_box")
                    end
                },
            },
        }

        frame_manager:show('common_box', nil, tb)
    end,

    -- 显示错误提示框
    showErrorBox = function(self, error_code)
        if error_code and error_code ~= 0 then
            local error_cfg = config.announcement_errorcodes[error_code]
            if error_cfg then
                local str = frame_helper:getLangCfg(error_cfg.content)
                hint:setText(str)
            else
                hint:setText("error code:"..tostring(error_code))
            end
        end
    end,

    -- 显示掉落道具
    loadDropItems = function(self, drop_panel, drop_id)
        local drop_items = gui_helper:getDropItems(drop_id)

        if not drop_items then
            log.error("### load drop item failed !")
            return
        end

        local num = math.min(drop_panel.childCount, #drop_items)

        for i=1, num do
            local slot = drop_panel:GetChild(i-1)
            local icon = slot:GetChild(0)
            local path = gui_helper:getItemIconPath(drop_items[i])
            icon:SetSprite(path)
            slot:SetActive(true)
            -- 设置品质框
            local quality = drop_items[i].color
            gui_helper:setItemQualityIcon(slot, quality)

            local effect = slot:Find("UI_LevelInfoFrame_xiyouwupin")
            if effect then
                effect:SetActive(quality > 3)
            end
        end

        for i=num, drop_panel.childCount-1 do
            local slot = drop_panel:GetChild(i)
            slot:SetActive(false)
        end
    end,

    -- 获取武器类型
    getWeaponType = function(self, cfg)
        local type = "???"
        config.weapon_types:foreach(
            function (k, v)
                if v.family == cfg.family and v.weapon_type == cfg.weapon_type then
                    type = frame_helper:getLangCfg(v.show)
                end
            end
        )
        return type
    end,

    getGuildRankIcon = function(self, rank)
        local cfg = config.union_positions[rank]
        if cfg then
            return cfg.path..cfg.icon
        end

        return "Assets/Arts/ui/atlas/guild/guild_icon_2_4.png"
    end,

    -- 返回 24:00:00
    timeFormat = function(self, t)
        --[[
        local h = math.floor(t / 3600)
        local m = math.floor((t%3600) / 60)
        local s = t % 60 

        return string.format( "%02d:%02d:%02d", h, m, s)
        ]]
        return frame_helper:timeFormat(t)
    end,

    -- 返回 (X:123,Y:123)
    positionFormat = function(self, x, y, server_id)
        if not x or not y then
            return
        end
        
        server_id = server_id or 1
        local str = string.format( "D:%d X:%d Y:%d", server_id, x, y )
        
        return str
    end,

    -- 检查资源消耗
    checkCostItems = function(self, cost)
        if not cost then
            log.error("#### check cost item error !")
            return false
        end

        for k, v in pairs(cost) do
            local cur_cnt = frame_helper:getItemNum(k)
            if cur_cnt < v then
                hint:setText(NativeText.GetLangStr(900001))
                return false
            end
        end

        return true
    end,

    setCostItem = function(self, cost_id, cost_num, node)
        local item_cfg = item_helper:getItemCfg(cost_id)
        
        if item_cfg then
            node:SetActive(true)
            local icon = node:Find("Icon"):GetComponent(UnityEngine.UI.Image)
            local count = node:Find("Count"):GetComponent(UnityEngine.UI.Text)

            local path = frame_helper:getItemIcon(cost_id)
            icon:SetSprite(path)
            
            -- 设置数量
            count.text = frame_helper:getItemNumFormat(cost_id, cost_num)
        else
            log.error("##### setCost item cfg error ! id", cost_id)
        end
    end,

    -- 超出上限的字体颜色格式
    limitColorFormat = function(self, curr, max)
        local str = string.format( "%d/%d", curr, max )

        if curr > max then
            str = string.format( "<color=#FF0000FF>%d</color>/%d", curr, max )
        end
        
        return str
    end,

    getFormationBGPath = function(self, hero_type)
        local target = nil
        config.formation_frames:foreach(
            function(k,v)
                if v.slot == hero_type then
                    target = v
                end
            end
        )

        if target then
            return target.framepath .. target.frameicon
        else
            return res_path.res_path_51
        end
    end,

    loadFormationHeros = function(self, node, cfg, heros, isOthers)
        local list = loadstring("return "..cfg.list)()
        if type(list) == "table" then
            local locations = {}
            for k, v in pairs(list) do
                local t = {}
                t.location = k
                t.type = v
                table.insert( locations, t )
            end

            table.sort( locations, function(a, b) return a.location < b.location end )

            local formation = {}
            for i=1, node.childCount do
                if locations[i] then
                    local slot = node:GetChild(i-1)
                    local bg = slot:GetComponent(UnityEngine.UI.Image)

                    local location = locations[i].location
                    local type = locations[i].type
                    bg:SetSprite(self:getFormationBGPath(type))

                    local hero = slot:GetChild(0)
                    hero:SetActive(false)

                    local tab = {}
                    tab.slot = slot
                    tab.type = type
                    formation[location] = tab
                end
            end

            for k, v in ipairs(heros) do
                if formation[v.location] then
                    local slot = formation[v.location].slot
                    local officer = slot:GetChild(0)
                    if isOthers then
                        self:setMWMOfficerHeadIcon(officer, v)
                        officer:SetActive(true)
                    else
                        local hero = hero_manager:getHero(v.heroId)
                        if hero then
                            self:setOfficerHeadIcon(officer, hero)
                            officer:SetActive(true)
                        end
                    end
                end
            end
        end
    end,

    sortItem = function(a, b)
        local a_num = bag_manager:getItemNum(a.id)
        local b_num = bag_manager:getItemNum(b.id)

        if a.type > b.type then
            return true
        elseif a.type == b.type then
            if a_num > b_num then
                return true
            elseif a_num == b_num then
                if a.id <= b.id then
                    return true
                else
                    return false
                end
            else
                return false
            end
        else
            return false
        end
    end,

    heroNeedRepair = function(self, hero)
        local cur_hp = hero:getHeroPorp(common_pb.PARTNER_PROP_HP)
        local max_hp = hero:getHeroPorp(common_pb.PARTNER_PROP_MAX_HP)

        return cur_hp.PropValue < max_hp.PropValue
    end,

    setAllChildGray = function(self, trans, isGray)
        for i=0, trans.childCount - 1 do
            local child = trans:GetChild(i)

            local img = child:GetComponent(UnityEngine.UI.Image)
            local text = child:GetComponent(UnityEngine.UI.Text)

            if img then
                frame_helper:setImageGray1(img, isGray)
            end

            if text then
                text.color = Color.gray
            end

            if child.childCount > 0 then
                self:setAllChildGray(child, isGray)
            end
        end
    end,

    sortCfgById = function(self, cfg)
        local temp = {}
        cfg:foreach(function(k, v) table.insert(temp, v) end)
        local sortFunc = function(a, b)
            return a.id < b.id
        end
        table.sort(temp, sortFunc)
        return temp
    end,

    heroIsWeakness = function(self, hero_id)
        local hero = hero_manager:getHero(hero_id)

        if hero then
            if hero:isWeakness() then
                hint:setText(NativeText.GetLangStr(280401))
                return true
            end
        end

        return false
    end,

    getSpeedUpCost = function(self, item_type, time)
        if item_type == 43 then
            time = time - home_info.building_free_time
            if time < 0 then
                time = 0
            end
        end
        local cost_cfg = nil
        config.item_speed_ups:foreach(
            function(id, cfg)
                if cfg.type == item_type then
                    cost_cfg = cfg
                end
            end)

        if cost_cfg then
            local single_price = cost_cfg.cost
            time = time / cost_cfg.unit
            local total_price = single_price * time
            if total_price <= 0 then
                return 0
            else
                total_price = math.floor(total_price)
                return total_price == 0 and 1 or total_price
            end
        else
            log.error('config : config.item_speed_ups not exist cfg', item_type)
            return 0
        end
    end,

    showCommonTipBox = function(self, cb, content, title)
        content = content:gsub("\\n", "\n")
        local tb = {
            title = title or "",
            content = content,
            buttons =
            {
                {
                    btn_name = NativeText.GetLangStr(110522),
                    callback = function() 
                        frame_manager:hide('common_box')
                    end
                },
                {
                    btn_name = NativeText.GetLangStr(110521),
                    callback = function()
                        cb()
                        frame_manager:hide('common_box')
                    end
                },
            },
        }

        frame_manager:show('common_box', nil, tb)
    end,

    marchLineSpeedUp = function(self, march_line)
        local type = nil

        if marchLine.marchAction == common_pb.EMARCH_HELP then
            type = speedUpTypes.dispatch
        elseif marchLine.marchAction == common_pb.EMARCH_MASS_ATK then
            type = speedUpTypes.mass
        else
            type = speedUpTypes.march
        end
        
        local param = {}
        param.id = march_line.marchLineId
        param.march = march_line
        param.type = type
        param.left_time = march_line.totalTime * 0.001
        param.total_time = (march_line.totalTime - march_line.elapsedTime) * 0.001
        param.title = NativeText.GetLangStr(370005)

        frame_manager:show("common_speed_up", nil, param)
    end,

    checkMonsterAtkCnt = function(self, cfg)
        local cnt_info = world_map_info:getAtkCnt(type)
        if cnt_info then
            if cnt_info.curCount >= cnt_info.maxCount then
                hint:setText(string.format(NativeText.GetLangStr(9000005), frame_helper:getLangCfg(cfg.monstername) ))
                return false
            end
        end

        return true
    end,

    getCorps = function(self, group)
        local soldiers = {}

        if group.is_corp then
            config.monster_stat_corp_ids:foreach(
                function(k, v)
                    if v.Group_ID == group.id then
                        if v.Is_Corp == 1 then
                            soldiers[v.Unit_Id] = {cfg_id = v.Unit_Id, count = v.Corp_Num}
                        end
                    end
                end
            )
        end

        return soldiers
    end,

    coordFormat = function(self, server_id, x, y)
        return string.format("D:%s,X:%s,Y:%s", server_id, x, y)
    end,

    getImageTextItemStr = function(self, item_id)
        local icon_name = frame_helper:getItemIcon(item_id)
        local strs = frame_helper:split(icon_name, "/")
        local rst = string.gsub('[' .. strs[#strs] .. ']', '.png', '')
        return rst
    end,

    registerLongPress = function(self, frame, path, cb, param)
        local long_press = long_press(frame)
        long_press:addClickListener(path, cb, param)
        long_press:addUpDownListener(path, cb, param)
    end,

    setOfficerStars = function(self, level, node)
        for i=1, node.childCount do
            local img = node:GetChild(i-1):GetComponent(UnityEngine.UI.Image)
            if level >= i then
                img:SetSprite(res_path.res_path_68)
            else
                img:SetSprite(res_path.res_path_69)
            end
        end
    end,

    setMassAddition = function(self, node, props)
        local cnt = 0
        local prefab = node:GetChild(0).gameObject
        for k, v in ipairs(props) do
            local id = v.PropId
            local value = v.PropValue

            local cfg = self:getPropCfg(id)
            if cfg then
                local elem = nil
                if cnt < node.childCount then
                    elem = node:GetChild(cnt)
                else
                    elem = Object.Instantiate(prefab).transform
                    elem:SetParent(node)
                    resetTransformZ(elem)
                end

                elem:SetActive(true)

                local prop_name = elem:GetComponent(UnityEngine.UI.Text)
                local prop_value = elem:Find("Value"):GetComponent(UnityEngine.UI.Text)
                local name_str = frame_helper:getLangCfg(cfg.name_id)
                
                prop_name.text = name_str
                prop_value.text = value

                if cfg.isfloat > 0 then
                    prop_value.text = tostring(value * 0.0001)
                else
                    prop_value.text = tostring(value)
                end

                if cfg.showpercent > 0 then
                    prop_value.text = tonumber(prop_value.text)*100 .."%"
                end

                prop_value.text = "+"..prop_value.text
                
                cnt = cnt + 1
            else
                log.error('Faile to find prop cfg ! id '..id)
            end
        end

        for i=cnt, node.childCount - 1 do
            local child = node:GetChild(i)
            child:SetActive(false)
        end
    end,

    positionJump = function(self, x, y)
        if game_director:isWorldMapState() then
            world_map_helper.world_map:setCameraPos(x, y)
        else
            world_map_info.enter_pos = {}
            world_map_info.enter_pos.x = x
            world_map_info.enter_pos.y = y
            game_director:changeState('loading', {next_state = 'world_map', scene_name = 'hex_map'})
        end
    end,

    getSkillTypeIcon = function(self, skill_cfg)
        if skill_cfg.Need_Battle == 1 then
            return res_path.res_path_109
        else
            return res_path.res_path_110
        end
    end,

    getSoldierTypeIcon = function(self, type_id)
        local rst = nil
        config.home_config_soldier_type_icons:foreach(function(id, cfg)
            if cfg.type == type_id then
                rst = cfg
            end
        end)
        return rst
    end,

    openGuildMassUI = function(self)
        self.opend_guild_mass = true
        network_sender:getFamilyInfo(local_guild.id)
    end,

    openGuildListUI = function(self)
        self.opend_guild_list = true
        network_sender:getFamilyList(0)
    end,

    showCommonBox3 = function(self, title, content, item_id, need_cnt, cb)
        content = content:gsub("\\n", "\n")
        local tb = {
            title = title or "",
            content = content,
            item_id = item_id,
            need_count = need_cnt,
            buttons =
            {
                {
                    btn_name = NativeText.GetLangStr(110522),
                    callback = function() 
                        frame_manager:hide('common_box_3')
                    end
                },
                {
                    btn_name = NativeText.GetLangStr(110521),
                    callback = function()
                        cb()
                        frame_manager:hide('common_box_3')
                    end
                },
            },
        }

        frame_manager:show('common_box_3', nil, tb)
    end,

    monsterTypeIcon = function(self, type)
        if type == 2 then
            return res_path.res_path_87
        elseif type == 4 then
            return res_path.res_path_137
        elseif type == 5 then
            return res_path.res_path_138
        elseif type == 7 then
            return res_path.res_path_136
        else
            return res_path.res_path_87
        end
    end,

    getPlayerIcon = function(self, icon_id)
        return res_path.res_path_139
    end,
}

return gui_helper