local super = require("ui.frame")
local log = logger.get("Game", "Guild Logo")

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        self.game_state_island = self.args.owner
        self.msg = self.args.data.msg

        clickEvents = 
        {
            "content/bg/top/ButtonBack",
            "content/bg/top/InfoBtn",
            "Top/confirmIcon",
            "Top/choose",
            "Top/closeLogo",
            "content/create/LogoNode/change",
            "content/create/createBtn",
            "content/create/LangNode/modifyBtn",
            "language/closeLanguage",
            "language/Window/confirmLang",
            "content/guildList/head/Search",
            "content/guildList/head/SearchLang",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)
        
        self.transform:Find("content/guildList/Scroll View/Viewport/Content/template").gameObject:SetActive(false)
        self.transform:Find("content/inviteList/Scroll View/Viewport/Content/template").gameObject:SetActive(false)

        network:registerMessageHandler(self)

        self.guild_list = self.msg.familyList

        self:init()
        --self:sendMsg()
        self:showApplyGuildList(self.msg)

        guide_manager:tryStartGuide()
    end,

    onUnload = function (self)
        network:unregisterMessageHandler(self)
    end,

    -- 网络消息 
    onMessage = function(self, cmd, msg)
        if cmd == protocol_pb.CHECKFAMILYNAMESTR then
            local str = ""
            local valid = false

            if msg.result == 0 then
                str = "Valid"
                valid = true
            elseif msg.result == 8003 then
                str = "Illegal"
            elseif msg.result == 8002 then
                str = "In Use"
            end

            if msg.flag == 0 then
                if valid then
                    self.name_valid.localScale = Vector3.one
                    self.name_used.localScale = Vector3.zero
                else
                    self.name_valid.localScale = Vector3.zero
                    self.name_used.localScale = Vector3.one

                    local text = self.name_used:GetChild(0):GetComponent(UnityEngine.UI.Text)
                    text.text = str
                end
            else
                if valid then
                    self.tag_valid.localScale = Vector3.one
                    self.tag_used.localScale = Vector3.zero
                else
                    self.tag_valid.localScale = Vector3.zero
                    self.tag_used.localScale = Vector3.one

                    local text = self.tag_used:GetChild(0):GetComponent(UnityEngine.UI.Text)
                    text.text = str
                end
            end
        end
    end,

    sendMsg = function(self)
        network_sender:getMyApplyFamily()
    end,

    init = function(self)
        self.create_trans = self.transform:Find("content/create")
        self.logo_trans = self.transform:Find("Top")
        self.language_trans = self.transform:Find("language")
        self.guild_list_trans = self.transform:Find("content/guildList")
        self.guild_invite_trans = self.transform:Find("content/inviteList")

        self.helper.hideTrans(self.create_trans)
        self.helper.hideTrans(self.logo_trans)
        self.helper.hideTrans(self.language_trans)
        self.helper.hideTrans(self.guild_list_trans)
        self.helper.hideTrans(self.guild_invite_trans)

        self.name_des_text = self:findText('content/create/NameNode/name_des_text')
        self.acronym_des_text = self:findText('content/create/TagNode/acronym_des_text')
        self.create_image_text = self:findImage('content/create/createBtn/Image')
        self.create_price_text = self:findText('content/create/createBtn/price')
        self.name_input = self.transform:Find('content/create/NameNode/NameIF'):GetComponent(UnityEngine.UI.InputField)
        self.tag_input = self.transform:Find('content/create/TagNode/AbbreviationIF'):GetComponent(UnityEngine.UI.InputField)
        self.name_valid = self.transform:Find('content/create/NameNode/Valid')
        self.name_used = self.transform:Find('content/create/NameNode/InUse')
        self.tag_valid = self.transform:Find('content/create/TagNode/Valid')
        self.tag_used = self.transform:Find('content/create/TagNode/InUse')
        self.search_name_input = self.transform:Find('content/guildList/head/NameIF'):GetComponent(UnityEngine.UI.InputField)
        self.search_lang_text = self.transform:Find('content/guildList/head/SearchLang/Text'):GetComponent(UnityEngine.UI.Text)
        self.search_pl_text = self.transform:Find('content/guildList/head/SearchLang/Placeholder'):GetComponent(UnityEngine.UI.Text)

        self.name_valid.localScale = Vector3.zero
        self.name_used.localScale = Vector3.zero
        self.tag_valid.localScale = Vector3.zero
        self.tag_used.localScale = Vector3.zero

        self.name_des_text.text = string.format(NativeText.GetLangStr(190015), config.union_others[1].namelen)
        self.acronym_des_text.text = string.format(NativeText.GetLangStr(190016), config.union_others[1].shortnamelen)
        
        self.name_input.characterLimit = config.union_others[1].namelen
        self.tag_input.characterLimit = config.union_others[1].shortnamelen

        self.name_input.onEndEdit:AddListener(
            function()
                if self:strValidCheck(self.name_input.text) then
                    network_sender:checkFamilyNameStr(0, self.name_input.text)
                end
            end
        )

        self.tag_input.onEndEdit:AddListener(
            function()
                if self:strValidCheck(self.tag_input.text) then
                    network_sender:checkFamilyNameStr(1, self.tag_input.text)
                end
            end
        )

        for key, value in pairs(self.helper:getJsonTab(config.union_others[1].createunion)) do
            self.create_image_text:SetSprite(gui_helper:getItemIconPath(self.helper:getItemCfg(key)))
            self.create_price_text.text = value
        end

        local max_num = 6
        for i = 1, max_num do
            local path1 = "Top/layout1 (1)/Button (" .. i .. ")"
            local path2 = "Top/layout1 (2)/Button (" .. i .. ")"
            local child, path3 = self:getScorllRectChild(i)

            self.transform:Find(path1):SetActive(false)
            self.transform:Find(path2):SetActive(false)
            child:SetActive(false)
        end

        self.icon_list1 = self:getConfigByType(2)
        for __, l in ipairs(self.icon_list1) do
            if __ <= max_num then
                local path1 = "Top/layout1 (1)/Button (" .. __ .. ")"
                local trans = self.transform:Find(path1)
                trans:SetActive(true)
                local icon = trans:Find("Image"):GetComponent(UnityEngine.UI.Image)
                icon:SetSprite(l.path .. l.icon)
                self:clearScript("click", path1)
                self:setScript("click", path1, function() self:click_select_bg(__) end)
            end
        end

        self.icon_list2 = self:getConfigByType(3)
        for __, l in ipairs(self.icon_list2) do
            if __ <= max_num then
                 local path2 = "Top/layout1 (2)/Button (" .. __ .. ")"
                local trans = self.transform:Find(path2)
                trans:SetActive(true)
                local icon = trans:GetComponent(UnityEngine.UI.Image)
                icon:SetSprite(l.path .. l.icon)

                self:clearScript("click", path2)
                self:setScript("click", path2, function() self:click_select_color(__) end)
            end
        end

        self.icon_list3 = self:getConfigByType(1)
        for __, l in ipairs(self.icon_list3) do
            local child, path = self:getScorllRectChild(__)
            if child then
                child:SetActive(true)
                local icon = self:findImage(path .. "/icon")
                icon:SetSprite(l.path .. l.icon)

                self:clearScript("click", path)
                self:setScript("click", path, function() self:click_select_icon(__) end)
            end
        end

        self:setIcon(1,1,1)

        for i = 1, 3 do 
            local path = "content/right_bar/Right (" .. i .. ")/Page"
            self:setScript("click", path, function() self:click_right_bar(i) end)
        end
        if gui_helper.opend_guild_list then
            self.right_bar_idx = 1
            gui_helper.opend_guild_list = false
        else
            self.right_bar_idx = 2
        end

        self:click_right_bar(self.right_bar_idx)

        self:initLanguageUI()
    end,

    strValidCheck = function(self, str)
        local len = string.len(str)
        if len > 0 then
            for i=1, len-1 do
                local asc = string.byte(self.name_input.text, i, i)
                if asc < 33 or asc > 126 then
                    return false
                end
            end

            return true
        end

        return false
    end,

    getConfigByType = function(self, type_id)
        local list = {}
        config.union_icons:foreach(function(idx, value)
            if value and value.type == type_id then
                table.insert(list, value)
            end
        end)
        return list
    end,

    click_ButtonBack = function(self)
        frame_manager:hide('guild_apply')
    end,

    click_InfoBtn = function(self)
        frame_manager:hideFloor(1)
    end,

    click_select_bg = function(self, cfg_id)
        log.info("点击背景", cfg_id)
        self:setIcon(cfg_id, self.icon_idx2,self.icon_idx3)
    end,

    click_select_color = function(self, cfg_id)
        log.info("点击颜色", cfg_id)
        self:setIcon(self.icon_idx1, cfg_id,self.icon_idx3)
    end,

    click_select_icon = function(self, cfg_id)
        log.info("点击icon", cfg_id)
        self:setIcon(self.icon_idx1, self.icon_idx2,cfg_id)
    end,

    click_confirmIcon = function(self)
        log.info("确定", self.icon_idx1, self.icon_idx2, self.icon_idx3)
        self.helper.hideTrans(self.logo_trans)
    end,

    click_closeLogo = function(self)
        self.helper.hideTrans(self.logo_trans)
    end,

    click_choose = function(self)
        log.info("随机")
        self:setIcon(math.random(1, #self.icon_list1),math.random(1, #self.icon_list2),math.random(1, #self.icon_list3))
    end,

    click_right_bar = function(self, idx)
        self.right_bar_idx = idx

        for i = 1, 3 do
            local button = self:findImage("content/right_bar/Right (" .. i .. ")/Page")
            if i == idx then
                button.color = Color(1,1,1,1)
            else
                button.color = Color(1,1,1,0)
            end
        end

        if self.right_bar_idx == 1 then
            self.helper.hideTrans(self.create_trans)
            self.helper.showTrans(self.guild_list_trans)
            self.helper.hideTrans(self.guild_invite_trans)
            
            self.search_name_input.text = ""
            self.search_lang_text.text = ""
            self.search_pl_text.text = NativeText.GetLangStr(9000054)

            self:showApplyGuildList({familyList = self.guild_list})
        elseif self.right_bar_idx == 2 then
            self.helper.showTrans(self.create_trans)
            self.helper.hideTrans(self.guild_list_trans)
            self.helper.hideTrans(self.guild_invite_trans)
        elseif self.right_bar_idx == 3 then
            self.helper.hideTrans(self.create_trans)
            self.helper.hideTrans(self.guild_list_trans)
            self.helper.showTrans(self.guild_invite_trans)
            self:sendMsg()
        end
    end,

    click_change = function(self)   
        self.helper.showTrans(self.logo_trans)
    end,

    click_createBtn = function(self)   
        -- print("创建联盟!")
        for key, value in pairs(self.helper:getJsonTab(config.union_others[1].createunion)) do
            
            local curr_num = self.helper:getItemNum(key)
            local need_num = value

            -- print("key:", key)
            -- print('value:', value, curr_num)

            if curr_num < need_num then
                -- 190084
                local tb = {
                    title = "",
                    content = NativeText.GetLangStr(190084),
                    buttons =
                    {
                        {
                            btn_name = NativeText.GetLangStr(130002),
                            callback = function() 
                                frame_manager:hide('common_box')
                            end
                        },
                    },
                }
                frame_manager:show('common_box', nil, tb)
                return
            end
        end

        local guild_name = self.name_input.text
        local nick_name = self.tag_input.text
        local guild_lang = self.cur_lang_cfg.id

        local emble_bg = self.icon_list1[self.icon_idx1].id
        local emble_color = self.icon_list2[self.icon_idx2].id 
        local emble_icon = self.icon_list3[self.icon_idx3].id 

        local default_innotice = self.helper:getLangCfg(config.union_others[1].defaultinsidenotice)
        local default_outnotice = self.helper:getLangCfg(config.union_others[1].defaultoutsidenotice)
        
        network_sender:createFamily(guild_name, nick_name, guild_lang, emble_bg, emble_color, emble_icon, default_innotice, default_outnotice)
    end,

    setIcon = function(self, idx1, idx2, idx3)
        self.icon_idx1 = idx1
        self.icon_idx2 = idx2
        self.icon_idx3 = idx3

        local makeSprite = function(path, list, idx)
            local icon = self:findImage(path)
            local name = list[idx].path .. list[idx].icon
            icon:SetSprite(name)
        end

        makeSprite("Top/lastIcon/icon1", self.icon_list1, self.icon_idx1)
        makeSprite("Top/lastIcon/bg/icon2", self.icon_list2, self.icon_idx2)
        makeSprite("Top/lastIcon/bg/icon3", self.icon_list3, self.icon_idx3)
        makeSprite("content/create/LogoNode/lastIcon/icon1", self.icon_list1, self.icon_idx1)
        makeSprite("content/create/LogoNode/lastIcon/icon2", self.icon_list2, self.icon_idx2)
        makeSprite("content/create/LogoNode/lastIcon/icon3", self.icon_list3, self.icon_idx3)
    end,

    -- 缓存layout,idx从1开始
    getScorllRectChild = function(self, idx)
        local path = "Top/Scroll View/Viewport/Content"
        local layout_trans = self.transform:Find(path)
        if not self.cache_layout then
            self.cache_layout = {}
            for i = 0, layout_trans.childCount - 1 do
                local child_trans = layout_trans:GetChild(i)
                child_trans.transform.name = i + 1
                table.insert(self.cache_layout, child_trans.transform)
            end
        end
        if idx > #self.cache_layout then
            for i = 1, idx - #self.cache_layout do
                local tplt = layout_trans:GetChild(0)
                local child_trans = Object.Instantiate(tplt.gameObject);
                child_trans.transform.name = i + #self.cache_layout
                child_trans:AttachTo(tplt.transform.parent)
                table.insert(self.cache_layout, child_trans.transform)
            end
        end
        self.cache_layout[idx].gameObject:SetActive(true)
        local full_path = path .. "/" .. self.cache_layout[idx].transform.name
        return self.cache_layout[idx], full_path
    end,

    click_closeLanguage = function(self)
        self.helper.hideTrans(self.language_trans)
    end,

    -- 初始化语言选择界面
    initLanguageUI = function(self)
        local sort_list = {}
        config.union_langs:foreach(
            function(__, value)
                table.insert(sort_list, value)
            end)
        table.sort(sort_list, function(a, b)
            return a.id < b.id
        end)

        local parent_path = "language/Window/Scroll View/Viewport/Content/"
        for __, value in ipairs(sort_list) do
            local child = self.helper:getChild(self, __, parent_path)
            local button = child:GetComponent(UnityEngine.UI.Button)
            button.onClick:RemoveAllListeners()
            button.onClick:AddListener(
                function ()
                    self:selectLanguage(value)
                end
            )
            local des_text = child.transform:Find("Text"):GetComponent(UnityEngine.UI.Text)
            des_text.text = self.helper:getLangCfg(value.language)

            local select_bg = child.transform:Find("select_bg"):GetComponent(UnityEngine.UI.Image)
            local select_point = child.transform:Find("select_point"):GetComponent(UnityEngine.UI.Image)
            
            self.cache_language = self.cache_language or {}
            table.insert(self.cache_language, {cfg = value, trans = child, select_bg = select_bg, select_point = select_point})
        end

        if #sort_list >= 1 then
            self.cur_lang_cfg = sort_list[1]
            self.search_lang_cfg = sort_list[1]
            self:selectLanguage(self.cur_lang_cfg)
            local text = self:findText("content/create/LangNode/lang")
            text.text = self.helper:getLangCfg(self.cur_lang_cfg.language)
        end
    end,

    -- 选择语言
    selectLanguage = function(self, cfg)
        self.temp_lang_cfg = cfg
        for __, item in ipairs(self.cache_language) do
            if item.cfg == cfg then
                item.select_bg.enabled = true
                item.select_point.enabled = true
            else
                item.select_bg.enabled = false
                item.select_point.enabled = false
            end
        end
    end,

    click_SearchLang = function(self)
        self.helper.showTrans(self.language_trans)
        self:selectLanguage(self.search_lang_cfg)
    end,

    click_Search = function(self)
        self:searchGuild()
    end,

    searchGuild = function(self)
        local name = string.lower(self.search_name_input.text)
        local lang_id = self.search_lang_cfg.id

        local tab = {}
        for k, v in ipairs(self.guild_list) do
            local short_name = string.lower(v.familycronym)
            local normal_name = string.lower(v.familyName)

            local name_idx = string.find(normal_name, name)
            local short_idx = string.find(short_name, name)

            local find_name = name == "" or name_idx ~= nil or short_idx ~= nil

            if (lang_id == 1 or lang_id == v.familyLanguage) and find_name then
                table.insert( tab, v )
            end
        end

        self:showApplyGuildList({familyList = tab})
    end,

    click_modifyBtn = function(self)
        self.helper.showTrans(self.language_trans)
        self:selectLanguage(self.cur_lang_cfg)
    end,
    
    click_confirmLang = function(self)
        self.helper.hideTrans(self.language_trans)
        if self.temp_lang_cfg then
            if self.right_bar_idx == 1 then
                self.search_lang_cfg = self.temp_lang_cfg
                self.search_lang_text.text = self.helper:getLangCfg(self.temp_lang_cfg.language)
                self.search_pl_text.text = ""
            else
                local text = self:findText("content/create/LangNode/lang")
                text.text = self.helper:getLangCfg(self.temp_lang_cfg.language)
                self.cur_lang_cfg = self.temp_lang_cfg    
            end
        end
    end,

    sortGuildList = function(self, familyList)
        local new_rst = {}
        self:getListByStatus(new_rst, familyList, 1)
        self:getListByStatus(new_rst, familyList, 2)
        self:getListByStatus(new_rst, familyList, 0)
        self:getListByStatus(new_rst, familyList, 3)
        return new_rst
    end,

    getListByStatus = function(self, new_rst, familyList, status_id)
        for __, guild in ipairs(familyList) do
            if guild.status == status_id then
                table.insert(new_rst, guild)
            end
        end
    end,

    showApplyGuildList = function(self, msg)
        self.cache_guild_list = {}

        local list = self:sortGuildList(msg.familyList)

        self.apply_cur_page = msg.curPage
        local parent_path = "content/guildList/Scroll View/Viewport/Content/"

        local parent_trans = self.transform:Find(parent_path)
        for i = 0, parent_trans.childCount - 1 do
            local child = parent_trans:GetChild(i)
            child.gameObject:SetActive(false)
        end

        local none_text = self.guild_list_trans:Find("None")
        none_text:SetActive(list == nil or #list == 0)

        for __, guild in ipairs(list) do
            local child = self.helper:getChild(self, __, parent_path)
            child.gameObject:SetActive(true)
            self.cache_guild_list[guild.familyId] = child
            local lang_content = child:Find("LangImg/Text/lang_content"):GetComponent(UnityEngine.UI.Text)
            --local condition_content = child:Find("condition_content"):GetComponent(UnityEngine.UI.Text)
            local num_content = child:Find("NumImg/Text/num_content"):GetComponent(UnityEngine.UI.Text)
            local gift_content = child:Find("GiftImg/Text/gift_content"):GetComponent(UnityEngine.UI.Text)
            local strength_content = child:Find("strength_text/strength_content"):GetComponent(UnityEngine.UI.Text)
            local name_content = child:Find("name_content"):GetComponent(UnityEngine.UI.Text)
            --local level_content = child:Find("level_content"):GetComponent(UnityEngine.UI.Text)
            --local nickname_content = child:Find("nickname_content"):GetComponent(UnityEngine.UI.Text)
            local button_text = child:Find("applyForBtn/text"):GetComponent(UnityEngine.UI.Text)

            local icon_bg = child:Find("lastIcon/icon1"):GetComponent(UnityEngine.UI.Image)
            local icon_color = child:Find("lastIcon/bg/icon2"):GetComponent(UnityEngine.UI.Image)
            local icon = child:Find("lastIcon/bg/icon3"):GetComponent(UnityEngine.UI.Image)
            -- local bg_plus = child:Find("bg_plus"):GetComponent(UnityEngine.UI.Image)

            icon_bg:SetSprite(config.union_icons[guild.emblem.iconFrame].path .. config.union_icons[guild.emblem.iconFrame].icon) 
            icon_color:SetSprite(config.union_icons[guild.emblem.grounding].path .. config.union_icons[guild.emblem.grounding].icon) 
            icon:SetSprite(config.union_icons[guild.emblem.icon].path .. config.union_icons[guild.emblem.icon].icon) 

            -- bg_plus.enabled = (__ % 2 ~= 0)
            
            if config.union_langs[guild.familyLanguage] then
                lang_content.text = self.helper:getLangCfg(config.union_langs[guild.familyLanguage].language)
            else
                lang_content.text = ""
            end

            --[[ if guild.lvLimit <= 0 and guild.swordLimit <= 0 then
                condition_content.text = NativeText.GetLangStr(190163)
            elseif guild.lvLimit <= 0 then
                condition_content.text = string.format(NativeText.GetLangStr(190165), guild.swordLimit)
            elseif guild.swordLimit <= 0 then
                condition_content.text = string.format(NativeText.GetLangStr(190164), guild.lvLimit)
            else
                condition_content.text = string.format(NativeText.GetLangStr(190027), guild.lvLimit, guild.swordLimit)
            end ]]
            num_content.text = guild.memberNum .. "/" .. guild.maxMemberNum
            --level_content.text = "Lv." .. guild.familyLevel
            strength_content.text = guild.familyStrength
            name_content.text = "[" .. guild.familycronym .. "]"..guild.familyName
            -- nickname_content.text = guild.familycronym
            gift_content.text = "Lv." .. guild.familyLevel

            log.info("############################## status ", guild.status)
            -- 1表示为没申请，2表示已申请，0表示可直接加入
            child:Find("applyForBtn").gameObject:SetActive(guild.status == 1)
            child:Find("cancelApplyBtn").gameObject:SetActive(guild.status == 2)
            child:Find("enterBtn").gameObject:SetActive(guild.status == 0)
            child:Find("notEnoughBtn").gameObject:SetActive(false)

            local button = child:Find("applyForBtn"):GetComponent(UnityEngine.UI.Button)
            button.onClick:RemoveAllListeners()
            button.onClick:AddListener(
                function ()
                    network_sender:familyJoinOrCancel(0, guild.familyId)
                end
            )

            local button = child:Find("cancelApplyBtn"):GetComponent(UnityEngine.UI.Button)
            button.onClick:RemoveAllListeners()
            button.onClick:AddListener(
                function ()
                    network_sender:familyJoinOrCancel(1, guild.familyId)
                end
            )

            local button = child:Find("enterBtn"):GetComponent(UnityEngine.UI.Button)
            button.onClick:RemoveAllListeners()
            button.onClick:AddListener(
                function ()
                    network_sender:familyJoinOrCancel(0, guild.familyId)
                end
            )

            local button = child:Find("Info"):GetComponent(UnityEngine.UI.Button)
            button.onClick:RemoveAllListeners()
            button.onClick:AddListener(
                function ()
                    network_sender:getFamilyInfo(guild.familyId)
                end
            )
        end
    end,

    refreshApplyGuildList = function(self, msg)
        -- print("RespCode", msg.RespCode, msg.status, msg.familyid, msg.lefttime)
        if self.cache_guild_list and self.cache_guild_list[msg.familyid] then
            local child = self.cache_guild_list[msg.familyid]
            child:Find("applyForBtn").gameObject:SetActive(msg.status == 1)
            child:Find("cancelApplyBtn").gameObject:SetActive(msg.status == 2)
            child:Find("enterBtn").gameObject:SetActive(msg.status == 0)

            self:refreshGuildState(msg)
        end
    end,

    refreshGuildState = function(self, msg)
        for k, v in ipairs(self.guild_list) do
            if v.familyId == msg.familyid then
                v.status = msg.status
            end
        end
    end,

    showInviteGuildList = function(self, msg)
        local parent_path = "content/inviteList/Scroll View/Viewport/Content/"
        local parent_trans = self.transform:Find(parent_path)
        for i = 0, parent_trans.childCount - 1 do
            local child = parent_trans:GetChild(i)
            child.gameObject:SetActive(false)
        end

        local none_text = self.guild_invite_trans:Find("None")
        none_text:SetActive(msg.familyList == nil or #msg.familyList == 0)

        for __, guild in ipairs(msg.familyList) do
            local child = self.helper:getChild(self, __, parent_path)
            child.gameObject:SetActive(true)

            local lang_content = child:Find("LangImg/Text/lang_content"):GetComponent(UnityEngine.UI.Text)
            local num_content = child:Find("NumImg/Text/num_content"):GetComponent(UnityEngine.UI.Text)
            local gift_content = child:Find("GiftImg/Text/gift_content"):GetComponent(UnityEngine.UI.Text)
            local strength_content = child:Find("strength_text/strength_content"):GetComponent(UnityEngine.UI.Text)
            local name_content = child:Find("name_content"):GetComponent(UnityEngine.UI.Text)
            --local level_content = child:Find("level_content"):GetComponent(UnityEngine.UI.Text)
            --local nickname_content = child:Find("nickname_content"):GetComponent(UnityEngine.UI.Text)

            local icon_bg = child:Find("lastIcon/icon1"):GetComponent(UnityEngine.UI.Image)
            local icon_color = child:Find("lastIcon/bg/icon2"):GetComponent(UnityEngine.UI.Image)
            local icon = child:Find("lastIcon/bg/icon3"):GetComponent(UnityEngine.UI.Image)
            -- local bg_plus = child:Find("bg_plus"):GetComponent(UnityEngine.UI.Image)

            icon_bg:SetSprite(config.union_icons[guild.emblem.iconFrame].path .. config.union_icons[guild.emblem.iconFrame].icon) 
            icon_color:SetSprite(config.union_icons[guild.emblem.grounding].path .. config.union_icons[guild.emblem.grounding].icon) 
            icon:SetSprite(config.union_icons[guild.emblem.icon].path .. config.union_icons[guild.emblem.icon].icon) 

            -- bg_plus.enabled = (__ % 2 ~= 0)
            
            if config.union_langs[guild.familyLanguage] then
                lang_content.text = self.helper:getLangCfg(config.union_langs[guild.familyLanguage].language)
            else
                lang_content.text = ""
            end

            num_content.text = guild.memberNum .. "/" .. guild.maxMemberNum
            --level_content.text = "Lv." .. guild.familyLevel
            strength_content.text = guild.familyStrength
            name_content.text = "[" .. guild.familycronym .. "]"..guild.familyName
            --nickname_content.text = ""
            gift_content.text = "Lv." .. guild.familyLevel

            local button = child:Find("Withdraw"):GetComponent(UnityEngine.UI.Button)
            button.onClick:RemoveAllListeners()
            button.onClick:AddListener(
                function ()
                    network_sender:familyJoinOrCancel(1, guild.familyId)
                    network_sender:getMyApplyFamily()
                end
            )

            --[[ local button = child:Find("agreeBtn"):GetComponent(UnityEngine.UI.Button)
            button.onClick:AddListener(
                function ()
                    network_sender:disposeInvite(1, guild.familyId)
                    -- network_sender:getInvitedFamilyList()
                    frame_manager:hide('guild_apply')
                end
            )

            local button = child:Find("cancelBtn"):GetComponent(UnityEngine.UI.Button)
            button.onClick:AddListener(
                function ()
                    network_sender:disposeInvite(0, guild.familyId)
                    network_sender:getInvitedFamilyList()
                end
            )

            local button = child:Find("Info"):GetComponent(UnityEngine.UI.Button)
            button.onClick:AddListener(
                function ()
                    network_sender:getFamilyInfo(guild.familyId)
                end
            ) ]]
        end
    end,
}
