local log = logger.get("Game", "Guild Base Info Frame")
local super = require("ui.frame")

return class{
    super = super,

    onLoaded = function( self )
        super.onLoaded(self)

        self.hold = self.args.data.hold

        clickEvents = {
            "Close",
            "Content/Left/Dismantle",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self.time_text = self.transform:Find("Content/Left/Layout/Time/Value"):GetComponent(UnityEngine.UI.Text)
        self.left_time = self.hold.left_time

        self:initLeftUI()
        self:initRightUI()
    end,

    onUnload = function(self)
        
    end,

    update = function(self, dt)
        if self.time_text then
            if self.hold.info.state ~= 2 then
                if self.left_time >= 0 then
                    self:setTime(self.left_time)
                    self.left_time = self.left_time - dt
                end
            end
        end
    end,

    setTime = function(self, lt)
        self.time_text.text = string.format("%s", gui_helper:timeFormat(lt))
    end,

    initLeftUI = function(self)
        local icon = self.transform:Find("Content/Left/Icon"):GetComponent(UnityEngine.UI.Image)
        local hp = self.transform:Find("Content/Left/Layout/HP/Value"):GetComponent(UnityEngine.UI.Text)
        local time_text = self.transform:Find("Content/Left/Layout/Time")
        local count_text = self.transform:Find("Content/Left/Layout/Count/Value"):GetComponent(UnityEngine.UI.Text)
        local bg = self.transform:Find("Content/Left/BG")
        local state_text = self.transform:Find("Content/Left/BG/State"):GetComponent(UnityEngine.UI.Text)

        icon:SetSprite(self.hold.cfg.IconPath..self.hold.cfg.Icon)
        hp.text = self.hold.info.curHp.."/"..self.hold.info.maxHp
        count_text.text = self.hold.info.curSoldierNum.."/"..self.hold.info.maxSoldierNum

        if self.hold.info.state == 0 then
            state_text.text = NativeText.GetLangStr(190228)
        elseif self.hold.info.state == 1 then
            state_text.text = NativeText.GetLangStr(190228)
        elseif self.hold.info.state == 3 then
            state_text.text = NativeText.GetLangStr(190229)
        end

        bg:SetActive(self.hold.info.state ~= 2)
        time_text:SetActive(self.hold.info.state ~= 2)
        count_text.transform.parent:SetActive(self.hold.info.state ~= 2)
    end,

    initRightUI = function(self)
        local des = self.transform:Find("Content/Right/Des"):GetComponent(UnityEngine.UI.Text)

        local content = self.transform:Find("Content/Right/Scroll View/Viewport/Content")
        local prefab = content:GetChild(0).gameObject

        local skills = loadstring("return "..self.hold.cfg.BuffList)()

        local cnt = 0

        for k, v in ipairs(skills) do
            local cfg = config.passive_skill_skills[v]

            if cfg then
                local elem = nil

                if cnt < content.childCount then
                    elem = content:GetChild(cnt)
                else
                    elem = Object.Instantaite(prefab).transform
                    elem:SetParent(content)
                    resetTransformZ(elem)
                end

                elem:SetActive(true)

                local icon = elem:Find("Icon"):GetComponent(UnityEngine.UI.Image)
                local name = elem:Find("Name"):GetComponent(UnityEngine.UI.Text)
                local des = elem:Find("Des"):GetComponent(UnityEngine.UI.Text)

                icon:SetSprite(cfg.skill_icon_path..cfg.skill_icon_name)
                name.text = self.helper:getLangCfg(cfg.lang_id)
                des.text = self.helper:getLangCfg(cfg.skill_des)

                cnt = cnt + 1
            else
                log.error("################################### find passive skill cfg error ! id ", v)
            end
        end

        for i=cnt, content.childCount -1 do
            local child = content:GetChild(i)
            child:SetActive(false)
        end
    end,

    click_Close = function(self)
        frame_manager:hide("guild_base_info")
    end,

    click_Dismantle = function(self)
        if not local_guild:destroyPermission() then
            return
        end

        local cb = function()
            network_sender:delFamilyBuild(self.hold.id)
            self:click_Close()
        end

        gui_helper:showCommonTipBox(cb, NativeText.GetLangStr(190230))
    end,
}