local log = logger.get("Game", "Guild Build Level Up")
local super = require("ui.frame")

return class{

    super = super,

    onLoaded = function(self)
        super.onLoaded(self)

        self.owner = self.args.owner
        self.build_info = self.args.data.info
        self.is_create = self.args.data.is_create

        clickEvents = {
            "common_panel_01",
            "guild_panel_13/OK",
        }

        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self.title = self.transform:Find("guild_panel_13/Title"):GetComponent(UnityEngine.UI.Text)
        self.cost_time = self.transform:Find("guild_panel_13/CostTime"):GetComponent(UnityEngine.UI.Text)
        self.icon_group = self.transform:Find("guild_panel_13/Group")
        self.cost_res_1 = self.transform:Find("guild_panel_13/common_icon_1")
        self.cost_res_2 = self.transform:Find("guild_panel_13/common_icon_2")
        self.tip = self.transform:Find("guild_panel_13/Tip")
        self.btn_text = self.transform:Find("guild_panel_13/OK/Text"):GetComponent(UnityEngine.UI.Text)

        self:init()
    end,

    init = function(self)
        local cfg = local_guild.build_cfg[self.build_info.configId][self.build_info.buildingLv]
        self.cost_time.text = self:countDownFormat(cfg.time)

        if self.is_create then
            self.btn_text.text = NativeText.GetLangStr(190156)
        else
            self.btn_text.text = NativeText.GetLangStr(190157)
        end

        self:setTitle()
        self:setUnlockBuild()
        self:setCostUI()
    end,

    setTitle = function(self)
        local build_cfg = config.camp_buildings[self.build_info.configId]
        local name = self.helper:getLangCfg(build_cfg.name_id)
        local level = self.build_info.buildingLv

        local str = ""
        if self.is_create then
            str = string.format("%sLv.%d  %s", NativeText.GetLangStr(190156), level, name)
        else
            str = string.format("%s%s  Lv.<color=#FFFFFFFF>%d</color> >> Lv.<color=#FFFFFFFF>%d</color>", NativeText.GetLangStr(190157), name, level, level+1)
        end

        self.title.text = str
    end,

    setUnlockBuild = function(self)
        local cfg = local_guild.build_cfg[self.build_info.configId][self.build_info.buildingLv]
        local builds = loadstring("return "..cfg.unlockbuilding)()
        
        local index = 0
        for k, v in pairs(builds) do
            if index < self.icon_group.childCount then
                local ui = self.icon_group:GetChild(index)
                local icon = ui:GetComponent(UnityEngine.UI.Image)
                local name = icon.transform:GetChild(0):GetComponent(UnityEngine.UI.Text)

                local build_cfg = config.camp_buildings[self.build_info.configId]
                local path = build_cfg.iconpath..build_cfg.iconid
                icon:SetSprite(path)
                name.text = string.format("Lv.%d  %s", self.build_info.buildingLv, self.helper:getLangCfg(build_cfg.name_id))

                ui:SetActive(true)
            end
            index = index + 1
        end

        for i=index, self.icon_group.childCount-1 do
            local ui = self.icon_group:GetChild(i)
            ui:SetActive(false)
        end

        self.icon_group:SetActive(index ~= 0)
        self.tip:SetActive(index == 0)
    end,

    setCostUI = function(self)
        local cfg = local_guild.build_cfg[self.build_info.configId][self.build_info.buildingLv]
        local cost = loadstring("return "..cfg.cost)()

        local index = 1
        for k, v in pairs(cost) do
            local path = frame_helper:getItemIcon(k)
            if self["cost_res_"..index] then
                local ui = self["cost_res_"..index]
                local icon = ui:GetComponent(UnityEngine.UI.Image)
                local count = ui:GetChild(0):GetComponent(UnityEngine.UI.Text)
                icon:SetSprite(path)
                count.text = tostring(v)
                ui:SetActive(true)
            end
            index = index + 1
        end

        for i=index, 2 do
            if self["cost_res_"..index] then
                local ui = self["cost_res_"..index]
                ui:SetActive(false)
            end
        end
    end,

    countDownFormat = function(self, seconds)
        local day = math.floor(seconds / 3600 / 24)
        local mod = seconds % (24 * 3600)
        local h = math.floor(mod / 3600)
        mod = seconds % 3600
        local m = math.floor(mod / 60)
        local s = seconds % 60

        if day > 0 then
            return string.format( "%dDay %02d:%02d:%02d",day, h, m, s )
        else
            return string.format( "%02d:%02d:%02d", h, m, s )
        end
    end,

    click_common_panel_01 = function(self)
        frame_manager:hide("build_lvup")
    end,

    click_OK = function(self)
        if self.is_create then
            local state = game_director.curr_state
            if state.name == "world_map" then
                state.world_map:createNewHold(self.build_info.configId)
                frame_manager:hide("guild_info")
            else
                hint:setText(NativeText.GetLangStr(190158))
            end
        else
            network_sender:upgradeFamilyBuilding(self.build_info.buildingId)
        end
        frame_manager:hide("build_lvup")
    end,
}