local log = logger.get("Game", "Guild Build Panel")

local types = {
    BASE = 1,
    RES = 2,
    DEF = 3,
    DEPOT = 4,
}

return class
{
    ctor = function(self, panel, parent)
        self.panel = panel
        self.parent = parent
        self.family_info = parent.msg.familyInfo

        self.content = self.panel:Find("Center/Scroll View/Viewport/Content")
        self.prefab = self.content:GetChild(0).gameObject

        self.is_show = false
        self.pages = {}

        self:initPages()

        self.panel:SetActive(false)
    end,

    initPages = function(self)
        local node = self.panel:Find("Pages")

        local list = {}
        config.union_union_building_types:foreach(
            function(k, v)
                table.insert( list, v )
            end
        )

        table.sort( list, function(a, b) return a.id < b.id end )

        for k, v in ipairs(list) do
            if k <= node.childCount then
                local elem = node:GetChild(k-1)
                local text = elem:Find("Text"):GetComponent(UnityEngine.UI.Text)
                local icon = elem:Find("BG/Icon"):GetComponent(UnityEngine.UI.Image)

                text.text = frame_helper:getLangCfg(v.name)
                icon:SetSprite(v.path..v.icon)

                self.pages[v.id] = elem

                local btn = elem:GetComponent(UnityEngine.UI.Button)
                btn.onClick:RemoveAllListeners()
                btn.onClick:AddListener(
                    function()
                        self:onClickPage(k)
                    end
                )
            end
        end
    end,

    show = function(self, msg)
        self.panel:SetActive(true)

        self.page_idx = 0
        self.msg = msg
        self:updatePages(types.BASE)

        self.is_show = true  

        self.back_base_evt = game_director.events:on("WORLDMAPRETURNFAMILYFORTRESS", functor.make(self.onBackBase, self))
    end,

    onGetItems = function(self, msg)
        if msg.reason == common_pb.HELPFASTBUILDING then
            frame_manager:show('getItems', self, {items = gui_helper.drop_list[msg.reason], has_Confirm_Btn = true})
        end
    end,

    onBackBase = function(self)
        if game_director:isWorldMapState() then
            frame_manager:clickHome()
            world_map_helper.world_map:lookAtMyHold()
        else
            game_director:changeState('loading', {next_state = 'world_map', scene_name = 'hex_map'})
        end
    end,

    updatePages = function(self, idx)
        if self.page_idx == idx then
            return
        end
        
        local text = self.panel:Find("Center/Top/Text"):GetComponent(UnityEngine.UI.Text)
        local cfg = config.union_union_building_types[idx]
        text.text = frame_helper:getLangCfg(cfg.BuildDes)

        for k, elem in pairs(self.pages) do
            local btn = elem:GetComponent(UnityEngine.UI.Button)
            local bg_img = elem:Find("BG"):GetComponent(UnityEngine.UI.Image)
            local icon = elem:Find("BG/Icon"):GetComponent(UnityEngine.UI.Image)
            local text = elem:Find("Text"):GetComponent(UnityEngine.UI.Text)

            if k == idx then
                btn.targetGraphic.sprite = btn.spriteState.pressedSprite
                text.color = Color(1, 1, 1, 1)
                icon.color = Color(1, 1, 1, 1)
                bg_img:SetSprite(res_path.res_path_118)
            else
                btn.targetGraphic.sprite = btn.spriteState.disabledSprite
                text.color = Color(152/255, 152/255, 152/255, 1)
                icon.color = Color(152/255, 152/255, 152/255, 1)
                bg_img:SetSprite(res_path.res_path_117)
            end
        end

        self.page_idx = idx
        self:init(self.msg.buildList)
    end,

    init = function(self, build_list)
        local index = 1

        local createElem = function()
            local elem = nil
            if index > self.content.childCount then
                elem = Object.Instantiate(self.prefab).transform
                elem:SetParent(self.content)
                resetTransformZ(elem)
            else
                elem = self.content:GetChild(index - 1)
            end

            if not elem then
                return
            end

            elem:SetActive(true)
            elem.name = "elem_"..index
            index = index + 1

            return elem
        end

        local contain = function(id, type)
            for k, v in ipairs(build_list) do
                if v.base.cfgId == id and v.base.type == type then
                    return true
                end
            end

            return false
        end

        local list = build_list or {}

        config.union_union_buildings:foreach(
            function(k, v)
                if not contain(v.id, v.type) then
                    local data = common_pb.MFamilyBuild()
                    data.base.cfgId = v.id
                    data.base.type = v.type
                    data.base.state = 0

                    table.insert( list, data )
                end
            end
        )

        table.sort( list, function(a, b) return a.base.cfgId < b.base.cfgId end )

        self.ui_list = {}
        local first_cfg = nil
        local first_info = nil

        for k, v in ipairs(list) do
            local cfg = nil

            if v.base.type == self.page_idx then
                cfg = config.union_union_buildings[v.base.cfgId]
            end

            if cfg then
                local elem = createElem()
                
                self:loadBuild(elem, cfg, v)
                self.ui_list[v.base.cfgId] = elem

                if k == 1 then
                    first_cfg = cfg
                    first_info = v
                end
            end
        end

        for i=index-1, self.content.childCount - 1 do
            local elem = self.content:GetChild(i)
            elem:SetActive(false)
        end

        if first_cfg and first_info then
            self:onClickBuild(first_info, first_cfg)
        end
    end,

    loadBuild = function(self, elem, cfg, info)
        local icon = elem:Find("Icon"):GetComponent(UnityEngine.UI.Image)
        local name = elem:Find("Name"):GetComponent(UnityEngine.UI.Text)
        local pos = elem:Find("Position"):GetComponent(UnityEngine.UI.Text)
        local pos_line = elem:Find("Position"):GetComponent(UnderLineText)
        local lock_text = elem:Find("LockText"):GetComponent(UnityEngine.UI.Text)
        local state = elem:Find("BG/Status"):GetComponent(UnityEngine.UI.Text)
        local bg = elem:Find("BG")
        local lock = elem:Find("Lock")
        local btn = icon:GetComponent(UnityEngine.UI.Button)

        local path = cfg.IconPath..cfg.Icon
        icon:SetSprite(path)
        name.text = string.format("%s", frame_helper:getLangCfg(cfg.name_id))

        local opened, str = self:checkCondition(cfg)

        if opened then
            frame_helper:setImageGray1(icon, false)
            name.color = Color.white
        else
            frame_helper:setImageGray1(icon, true)
            name.color = Color.gray
            lock_text.color = Color.gray
        end

        local cb = function()
            gui_helper:positionJump(info.base.posX, info.base.posY)
        end

        pos.transform:SetActive(info.base.state ~= 0)
        lock_text.transform:SetActive(not opened)
        lock:SetActive(not opened)

        if info.base.state == 0 then
            bg:SetActive(false)
            lock_text.text = NativeText.GetLangStr(190231)
        elseif info.base.state == 1 then
            bg:SetActive(true)
            state.text = NativeText.GetLangStr(190232)
            pos_line:SetText(gui_helper:positionFormat(info.base.posX, info.base.posY), cb)
        elseif info.base.state == 2 then
            bg:SetActive(false)
            pos_line:SetText(gui_helper:positionFormat(info.base.posX, info.base.posY), cb)
        elseif info.base.state == 3 then
            bg:SetActive(true)
            state.text = NativeText.GetLangStr(190233)
            pos_line:SetText(gui_helper:positionFormat(info.base.posX, info.base.posY), cb)
        end

        btn.onClick:RemoveAllListeners()
        btn.onClick:AddListener(
            function()
                self:onClickBuild(info, cfg)
            end
        )
    end,

    setBuildInfo = function(self, info, cfg)
        local name = self.panel:Find("Center/Info/Name"):GetComponent(UnityEngine.UI.Text)
        local locked = self.panel:Find("Center/Info/Locked")
        local point = self.panel:Find("Center/Info/Point")
        local value = self.panel:Find("Center/Info/Point/Value"):GetComponent(UnityEngine.UI.Text)
        local des = self.panel:Find("Center/Info/Des"):GetComponent(UnityEngine.UI.Text)
        local btn_text = self.panel:Find("Center/Info/BuildBtn/Text"):GetComponent(UnityEngine.UI.Text)
        local btn = self.panel:Find("Center/Info/BuildBtn"):GetComponent(UnityEngine.UI.Button)

        name.text = frame_helper:getLangCfg(cfg.name_id)

        local opened, str = self:checkCondition(cfg)

        point:SetActive(opened)
        locked:SetActive(not opened)
        btn.interactable = opened

        if opened then
            name.color = Color(1, 178/255, 30/255, 1)
            des.text = frame_helper:getLangCfg(cfg.des_id)
            value.text = string.format( "%s: %s", NativeText.GetLangStr(190234), frame_helper:itemNumFormat(info.base.curhp))
        else
            name.color = Color.gray
            des.text = str
            btn_text.text = NativeText.GetLangStr(190231)
        end

        if info.base.state == 0 then
            if opened then
                btn_text.text = NativeText.GetLangStr(190235)
            end
        elseif info.base.state == 1 then
            btn_text.text = NativeText.GetLangStr(190236)
        elseif info.base.state == 2 then
            btn_text.text = NativeText.GetLangStr(190237)
        elseif info.base.state == 3 then
            btn_text.text = NativeText.GetLangStr(190237)
        end

        btn.onClick:RemoveAllListeners()
        btn.onClick:AddListener(
            function()
                self:onClickBuildInfoBtn(info, cfg)
            end
        )
    end,

    updateBuildUI = function(self, cfg)
        for id, ui in ipairs(self.ui_list) do
            local icon = ui:Find("Icon")
            local name = ui:Find("Name"):GetComponent(UnityEngine.UI.Text)
            local pos = ui:Find("Position")
            local lock = ui:Find("Lock")
            local lock_text = ui:Find("LockText"):GetComponent(UnityEngine.UI.Text)

            local opened = self:checkCondition(config.union_union_buildings[id])
            
            if id == cfg.id then
                icon.localScale = Vector3(1, 1, 1)
                lock.localScale = Vector3(1.2, 1.2, 1.2)
                name.transform.localScale = Vector3(1.2, 1.2, 1.2)
                pos.localScale = Vector3(1.2, 1.2, 1.2)
                lock_text.transform.localScale = Vector3(1.2, 1.2, 1.2)

                if opened then
                    name.color = Color(1, 178/255, 30/255, 1)
                else
                    name.color = Color.gray
                end
            else
                icon.localScale = Vector3(0.8, 0.8, 0.8)
                lock.localScale = Vector3(1, 1, 1)
                name.transform.localScale = Vector3(1, 1, 1)
                pos.localScale = Vector3(1, 1, 1)
                lock_text.transform.localScale = Vector3(1, 1, 1)

                if opened then
                    name.color = Color(1, 1, 1, 1)
                else
                    name.color = Color.gray
                end
            end
        end
    end,

    checkCondition = function(self, cfg)
        local met = true
        local str = ""

        if cfg.NeedMember > self.family_info.memberNum then
            met = false
            str = string.format( "<color=#EE0000FF>%s ≥ %s</color>", NativeText.GetLangStr(190238), frame_helper:itemNumFormat(cfg.NeedMember) )
        else
            str = string.format( "<color=#FFFFFFFF>%s ≥ %s</color>", NativeText.GetLangStr(190238), frame_helper:itemNumFormat(cfg.NeedMember) )
        end

        if cfg.NeedPower > tonumber(self.family_info.familyStrength) then
            met = false
            str = string.format( "%s\n<color=#EE0000FF>%s ≥ %s</color>", str, NativeText.GetLangStr(190239), frame_helper:itemNumFormat(cfg.NeedPower) )
        else
            str = string.format( "%s\n<color=#FFFFFFFF>%s ≥ %s</color>", str, NativeText.GetLangStr(190239), frame_helper:itemNumFormat(cfg.NeedPower) )
        end

        if cfg.NeedTechNum > self.family_info.technology_num then
            met = false
            str = string.format( "%s\n<color=#EE0000FF>%s ≥ %s</color>", str, NativeText.GetLangStr(190240), frame_helper:itemNumFormat(cfg.NeedTechNum) )
        else
            str = string.format( "%s\n<color=#FFFFFFFF>%s ≥ %s</color>", str, NativeText.GetLangStr(190240), frame_helper:itemNumFormat(cfg.NeedTechNum) )
        end

        return met, str
    end,

    onClickPage = function(self, idx)
        if idx ~= types.BASE then
            hint:setText(NativeText.GetLangStr(900002))
            return
        end

        self:updatePages(idx)
    end,

    onClickBuild = function(self, info, cfg)
        self:setBuildInfo(info, cfg)
        self:updateBuildUI(cfg)
    end,

    onClickBuildInfoBtn = function(self, info, cfg)
        local cb = function()
            network_sender:backToFamilyFortress(info.base.instId)
        end

        local opened = self:checkCondition(cfg)

        if info.base.state == 0 then
            if opened and local_guild:buildPermission() then
                self:createBuilding(cfg.id, types.BASE)
            end
        elseif info.base.state == 1 then
            gui_helper:positionJump(info.base.posX, info.base.posY)
            frame_manager:clickHome()
        elseif info.base.state == 2 then
            gui_helper:showCommonBox3(NativeText.GetLangStr(190241), NativeText.GetLangStr(190242), 65015, 1, cb)
        elseif info.base.state == 3 then
            gui_helper:showCommonBox3(NativeText.GetLangStr(190241), NativeText.GetLangStr(190242), 65015, 1, cb)
        end
    end,

    createBuilding = function(self, id, type)
        if game_director:isWorldMapState() then
            frame_manager:clickHome()
            world_map_helper:createTempHold(id, type)
        else
            game_director:changeState('loading', {next_state = 'world_map', scene_name = 'hex_map', create_guild_build = {id = id, type = type}})
        end
    end,

    hide = function(self)
        self.panel:SetActive(false)
        self.is_show = false
    end,
}