local log = logger.get("Game", "Guild Building Info Frame")
local super = require("ui.frame")

return class{

    super = super,

    onLoaded = function(self)

        super.onLoaded(self)

        self.detail = self.args.data.build

        -- 路径: root/login
        -- 事件: click_login 
        clickEvents = {
            "BG/Title/BackBtn",
            "BG/Content/BuildPanel/Image/Layout/Join",
            --"BG/Content/Commanders/Addition",
            --"BG/AdditionPanel",
        }
 
        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self.content = self.transform:Find("BG/Content/Scroll View/Viewport/Content")
        self.prefab = self.transform:Find("BG/Content/Scroll View/Viewport/Content/Elem").gameObject
        self.build_panel = self.transform:Find("BG/Content/BuildPanel")
        self.addition_panel = self.transform:Find("BG/AdditionPanel")
        self.build_time = self.build_panel:Find("Info/Time/Value"):GetComponent(UnityEngine.UI.Text)

        self.addition_panel.localScale = Vector3.zero

        local name = self.transform:Find("BG/Content/Commanders/Panel/Commander/Text"):GetComponent(UnityEngine.UI.Text)
        name.text = self.detail.info.teams[1].name

        self.team_cnt = 0
        self.soldier_cnt = 0
        self.ui_leftTime = {}

        self:loadTeams()
        self:setTeamCount()
        self:enemyInfo()
        --self:setRallyAddition()

        self.evts = {}
        --self.evts["WORLDMAPMASSNOTIFY"] = game_director.events:on("WORLDMAPMASSNOTIFY", functor.make(self.onTeamChanged, self))
    end,

    onUnload = function(self)
        if self.evts then
            for k, v in pairs(self.evts) do
                game_director.events:remove(k, v)
            end
        end
    end,

    onTeamChanged = function(self, msg)
        
    end,

    update = function(self, dt)
        if not self.ui_leftTime then
            return
        end

        self.arrived = true
        for k, v in pairs(self.ui_leftTime) do
            local lt = v.data.lefttime
            if lt > 0 then
                self.arrived = false
                lt = lt - dt
                v.data.lefttime = lt > 0 and lt or 0
                self:teamCountDown(v.ui, v.data)
            else
                v.ui.localScale = Vector3.zero
                v.arrived.localScale = Vector3.one
            end
        end

        if self.detail.left_time > 0 then
            self:buildTimeCountDown(self.detail.left_time)

            local lt = self.detail.left_time
            lt = lt - dt
            self.detail.left_time = lt > 0 and lt or 0
        end
    end,
    
    setTeamCount = function(self)
        local teams = self.transform:Find("BG/Content/Top/Amount/Value"):GetComponent(UnityEngine.UI.Text)
        local soldiers = self.transform:Find("BG/Content/Top/Cargo/Value"):GetComponent(UnityEngine.UI.Text)

        teams.text = string.format("%d/%d", self.team_cnt, self.detail.info.maxTeamCount)
        soldiers.text = string.format("%s/%s", self.helper:itemNumFormat(self.soldier_cnt), self.helper:itemNumFormat(self.detail.info.maxSoldierNum))
    end,

    buildTimeCountDown = function(self, left_time)
        self.build_time.text = string.format("%s", gui_helper:timeFormat(left_time))
    end,

    enemyInfo = function(self)
        local name = self.transform:Find("BG/Content/Top/BuildName"):GetComponent(UnityEngine.UI.Text)
        local pos = self.transform:Find("BG/Content/Top/BuildName/Position"):GetComponent(UnderLineText)
        local icon = self.build_panel:Find("IconBG/Icon"):GetComponent(UnityEngine.UI.Image)
        local amount = self.build_panel:Find("Info/Amount/Value"):GetComponent(UnityEngine.UI.Text)

        name.text = self.helper:getLangCfg(self.detail.name_id)

        local cb = function()
            gui_helper:positionJump(self.detail.coordX, self.build.coordY)
            self:click_BackBtn()
        end
        pos:SetText(gui_helper:positionFormat(self.detail.coordX, self.detail.coordY), cb)

        icon:SetSprite(self.detail.cfg.IconPath..self.detail.cfg.Icon)
        amount.text = string.format( "%s/%s", self.helper:itemNumFormat(self.detail.info.curHp), self.helper:itemNumFormat(self.detail.info.maxHp) )
    end,

    loadTeams = function(self)
        for k, v in ipairs(self.detail.info.teams) do
            local elem = nil

            if k > self.content.childCount then
                elem = Object.Instantiate(self.prefab).transform
                elem:SetParent(self.content)
                resetTransformZ(elem)
            else
                elem = self.content:GetChild(k-1)
            end

            if elem then
                elem.name = "Team_"..k
                elem:SetActive(true)

                self:setTeamInfo(elem, v)
            end

            self.team_cnt = self.team_cnt + 1
        end

        for i=#self.detail.info.teams, self.content.childCount - 1 do
            local elem = self.content:GetChild(i)
            elem:SetActive(false)
        end
    end,

    setTeamInfo = function(self, elem, data)
        local name = elem:Find("Top/Name"):GetComponent(UnityEngine.UI.Text)
        local count = elem:Find("Bottom/Amount"):GetComponent(UnityEngine.UI.Text)
        local arrived = elem:Find("Top/Arrived")
        local slider = elem:Find("Bottom/Slider")
        local back_btn = elem:Find("Details/Back")
        local details = elem:Find("Details")
        local off_btn = elem:Find("Off")
        local on_btn = elem:Find("On")

        off_btn:SetActive(false)
        on_btn:SetActive(true)

        name.text = data.name
        count.text = self:teamSoldierCount(data.soldiers)

        if data.lefttime > 0 then
            table.insert(self.ui_leftTime, {ui = slider, data = data, arrived = arrived})
            slider.localScale = Vector3.one
            arrived.localScale = Vector3.zero
        else
            slider.localScale = Vector3.zero
            arrived.localScale = Vector3.one
        end

        self:loadSoldiers(details, data.heros, data.soldiers)

        local path = "BG/Content/Scroll View/Viewport/Content/"..elem.name.."/Off"
        self:clearScript("click", path)
        self:setScript("click", path, functor.rmake(self.onClickOff, elem))

        path = "BG/Content/Scroll View/Viewport/Content/"..elem.name.."/On"
        self:clearScript("click", path)
        self:setScript("click", path, functor.rmake(self.onClickOn, elem))
        
        path = "BG/Content/Scroll View/Viewport/Content/"..elem.name.."/Bottom/Slider/SpeedUp"
        self:clearScript("click", path)
        self:setScript("click", path, functor.rmake(self.onClickSpeedUp, data.armylineId))

        path = "BG/Content/Scroll View/Viewport/Content/"..elem.name.."/Details/Back"
        self:clearScript("click", path)

        if local_player.id == data.playerId then
            self:setScript("click", path, functor.rmake(self.onClickBack, data))
            back_btn:SetActive(true)
        else
            back_btn:SetActive(false)
        end
    end,

    loadSoldiers = function(self, elem, heros, soldiers)
        local heros_panel = elem:Find("Panel/heros")
        if #heros > 0 then
            for k, v in ipairs(heros) do
                if k <= heros_panel.childCount then
                    local head = heros_panel:GetChild(k-1)
                    local icon = head:Find("Icon"):GetComponent(UnityEngine.UI.Image)
                    local grade = head:Find("Grade"):GetComponent(UnityEngine.UI.Image)
                    local level = head:Find("Level"):GetComponent(UnityEngine.UI.Text)
                    local stars = head:Find("Stars")

                    local cfg = config.commander_basics[v.officerId]
                    icon:SetSprite(cfg.path..cfg.icon)
                    grade:SetSprite(self:getGradeRes(cfg.color))
                    level.text = v.officerLv

                    gui_helper:setOfficerStars(v.starLv, stars)

                    head:SetActive(true)
                end
            end

            for i=#heros, heros_panel.childCount - 1 do
                local head = heros_panel:GetChild(i)
                head:SetActive(false)
            end

            heros_panel:SetActive(true)
        else
            heros_panel:SetActive(false)
        end

        local content = elem:Find("Panel")
        local perfab = content:GetChild(1).gameObject
        for k, v in ipairs(soldiers) do
            local node = nil
            if k > content.childCount - 1 then
                node = Object.Instantiate(perfab).transform
                node:SetParent(content)
                resetTransformZ(node)
            else
                node = content:GetChild(k)
            end

            node:SetActive(true)
            node.name = "Soldier_"..k

            local icon = node:Find("Image/Icon"):GetComponent(UnityEngine.UI.Image)
            local name = node:Find("Image/Name"):GetComponent(UnityEngine.UI.Text)
            local count = node:Find("Image/Count"):GetComponent(UnityEngine.UI.Text)

            local cfg = config.soldier_soldiers[v.cfgid]

            if cfg then
                icon:SetSprite(cfg.iconpath..cfg.soldiericon)
                name.text = self.helper:getLangCfg(cfg.soldiername)
                count.text = v.count
                self.soldier_cnt = self.soldier_cnt + v.count
            else
                log.error("######################################### soldier cfg not find ! id ", v.cfgid)
            end
        end

        for i=#soldiers+1, content.childCount - 1 do
            local node = content:GetChild(i)
            node:SetActive(false)
        end

        app:startCoroutine(
            function()
                coroutine.yield(nil)
                self:onClickOff(nil, elem.parent)
            end
        )
    end,

    getGradeRes = function(self, color)
        local path = res_path.res_path_95

        if color == 1 then
            path = res_path.res_path_96
        elseif color == 2 then
            path = res_path.res_path_97
        elseif color == 3 then
            path = res_path.res_path_98
        elseif color == 4 then
            path = res_path.res_path_99
        elseif color == 5 then
            path = res_path.res_path_100
        end

        return path
    end,

    teamSoldierCount = function(self, soldiers)
        local count = 0

        for k, v in ipairs(soldiers) do
            count = count + v.count
        end

        return count
    end,

    playerInMass = function(self)
        for k, v in ipairs(self.detail.info.teams) do
            if local_player.id == v.playerId then
                return true
            end
        end

        return false
    end,

    teamCountDown = function(self, ui, data)
        local left_time = ui:Find("LeftTime"):GetComponent(UnityEngine.UI.Text)
        local fill = ui:Find("Fill"):GetComponent(UnityEngine.UI.Image)

        fill.fillAmount = 1 - data.lefttime / data.totaltime
        left_time.text = gui_helper:timeFormat(data.lefttime)
    end,

    onClickBack = function(self, handler, formation)
        local callback = function()
            network_sender:worldMapGoHome(2, formation.armylineId)
            frame_manager:hide('common_box')
        end

        self:tipBox(NativeText.GetLangStr(289003),callback)
    end,

    onClickOff = function(self, handler, ui)
        local off_btn = ui:Find("Off")
        local on_btn = ui:Find("On")
        local details = ui:Find("Details")

        off_btn:SetActive(false)
        on_btn:SetActive(true)
        details:SetActive(false)

        ui.sizeDelta = Vector2(830, 130)
    end,

    onClickOn = function(self, handler, ui)
        local off_btn = ui:Find("Off")
        local on_btn = ui:Find("On")
        local details = ui:Find("Details")
        local panel = details:Find("Panel"):GetComponent(UnityEngine.RectTransform)

        off_btn:SetActive(true)
        on_btn:SetActive(false)
        details:SetActive(true)

        ui.sizeDelta = Vector2(830, panel.rect.height + 130)
    end,

    onClickSpeedUp = function(self, handler, marchLine)
        --gui_helper:marchLineSpeedUp(marchLine)
    end,

    click_BackBtn = function(self)
        frame_manager:hide("guild_building_info")
    end,

    click_Join = function(self)
        frame_manager:show("guild_building_launch", nil, {build = self.detail, action = common_pb.EMARCH_BUILD})
    end,

    tipBox = function(self, content, cb)
        local tb = {
            title = "",
            content = content,
            buttons =
            {
                {
                    btn_name = NativeText.GetLangStr(110522),
                    callback = function() 
                        frame_manager:hide('common_box')
                    end
                },
                {
                    btn_name = NativeText.GetLangStr(110521),
                    callback = cb,
                },
            },
        }

        frame_manager:show('common_box', nil, tb)
    end,
}