local log = logger.get("Game", "Guild Building Launch Frame")
local super = require("ui.frame")

return class{
    super = super,

    onLoaded = function( self )
        super.onLoaded(self)
        
        self.action = self.args.data.action
        self.build = self.args.data.build

        clickEvents = {
            "BG/Title/BackBtn",
            "BG/Content/Commanders/Addition",
            "BG/Content/Buttons/Send",
            "BG/Content/Buttons/Auto",
            "BG/Content/Buttons/Formation",
            "BG/PropUp/PropUpClose",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self.corps_content = self.transform:Find("BG/Content/Scroll View/Viewport/Content")
        self.corps_prefab = self.corps_content:GetChild(0)
        self.officer_content = self.transform:Find("BG/Content/CommanderPanel/CommanderList/Viewport/Content")
        self.officer_prefab = self.officer_content:GetChild(0)

        self.corps_panel = self.transform:Find("BG/Content/Scroll View")
        self.officer_details = self.transform:Find("BG/Content/CommanderPanel/CommanderDetails")
        self.officer_panel = self.transform:Find("BG/Content/CommanderPanel")
        self.addition_info = self.transform:Find("BG/Content/CommanderPanel/AdditionInfo")
        self.build_panel = self.transform:Find("BG/Content/BuildPanel")
        self.prop_up_panel = self.transform:Find("BG/PropUp")

        self.officer_panel.localScale = Vector3.zero
        self.prop_up_panel.localScale = Vector3.zero
        
        self.amount = self.transform:Find("BG/Content/Top/Amount/Value"):GetComponent(UnityEngine.UI.Text)
        self.cargo = self.transform:Find("BG/Content/Top/Cargo/Value"):GetComponent(UnityEngine.UI.Text)
        self.time_text = self.transform:Find("BG/Content/Buttons/Send/TimeIcon/Text"):GetComponent(UnityEngine.UI.Text)

        self.time_text.text = gui_helper:timeFormat(0)

        local panel = self.transform:Find("BG/Content/Commanders/Panel")
        self.officer_heads = {}
        for i=1, panel.childCount do
            local head = panel:GetChild(i-1)
            self:setScript("click", "BG/Content/Commanders/Panel/"..head.name, functor.make(self.clickOfficerHead, self))
            self.officer_heads[i] = {ui = head, id = 0}
        end

        self.speed = 0
        self.total_soldiers = 0
        self.append_officers = {}
        self.append_soldiers = {}
        self.officer_list = {}
        self.soldier_ui = {}
        self.officer_toggles = {}

        config.soldier_soldiers:foreach(
            function(k, v)
                if v.type < 10 then
                    self.append_soldiers[k] = {cfg_id = k, count = 0}
                end
            end
        )

        self.append_max = self:maxSoldiersCount()

        self.soldier_list = {}
        local list = home_info:getAvailableSoldiers()

        for k, v in ipairs(list) do
            self.soldier_list[v.soldier_id] = v.available_num
        end

        self:initSoldierList()
        self:setCargo()
        self:setSoldiersAmount()
        self:setBuildInfo()
    end,

    onUnload = function(self)
    end,

    update = function(self, dt)
        
    end,

    maxSoldiersCount = function(self)
        local count = 0

        --if self.action == common_pb.EMARCH_CREATE_MASS then
        --    count = local_player:massSoldierMax()
        --else
            count = world_map_helper:teamSoldiersAmount({})
        --end

        return count
    end,

    initSoldierList = function(self)
        local cnt = 0

        for k, v in pairs(self.soldier_list) do
            local elem = nil

            if cnt >= self.corps_content.childCount then
                elem = Object.Instantiate(self.corps_prefab).transform
                elem:SetParent(self.corps_content)
                resetTransformZ(elem)
            else
                elem = self.corps_content:GetChild(cnt)
            end

            elem:SetActive(true)
            elem.name = "soldier_"..k

            local icon = elem:Find("Grade/Icon"):GetComponent(UnityEngine.UI.Image)
            local grade = elem:Find("Grade"):GetComponent(UnityEngine.UI.Image)
            local type_icon = elem:Find("Grade/TypeIcon"):GetComponent(UnityEngine.UI.Image)
            local own_count = elem:Find("Grade/Count"):GetComponent(UnityEngine.UI.Text)
            local name = elem:Find("Name"):GetComponent(UnityEngine.UI.Text)
            local append_count = elem:Find("Count"):GetComponent(UnityEngine.UI.Text)
            local slider = elem:Find("Slider"):GetComponent(UnityEngine.UI.Slider)

            own_count.text = v
            append_count.text = 0
            slider.value = 0

            local soldier_cfg = config.soldier_soldiers[k]

            if soldier_cfg then
                name.text = self.helper:getLangCfg(soldier_cfg.soldiername)
                icon:SetSprite(soldier_cfg.headpath..soldier_cfg.soldierhead)
                type_icon:SetSprite(soldier_cfg.iconpath..soldier_cfg.soldiericon)
                grade:SetSprite(gui_helper:getNewQualityIcon(soldier_cfg.color))
            end

            self.soldier_ui[k] = elem

            slider.onValueChanged:RemoveAllListeners()
            slider.onValueChanged:AddListener(
                function ()
                    self:onValueChanged(k)
                end
            )

            local path = "BG/Content/Scroll View/Viewport/Content/"..elem.name.."/Plus"
            gui_helper:registerLongPress(self, path, self.click_Plus, k)

            path = "BG/Content/Scroll View/Viewport/Content/"..elem.name.."/Minus"
            gui_helper:registerLongPress(self, path, self.click_Minus, k)

            self.append_soldiers[k] = {cfg_id = k, count = 0}

            cnt = cnt + 1
        end

        for i=cnt, self.corps_content.childCount - 1 do
            local elem = self.corps_content:GetChild(i)
            elem:SetActive(false)
        end
    end,

    loadOfficers = function(self)
        local heros = hero_manager:getUnDispatchedHeros()
        table.sort( heros, function(a,b) return a.officer.cfg.color > b.officer.cfg.color end )

        for k, v in ipairs(heros) do
            local elem = nil

            if k > self.officer_content.childCount then
                elem = Object.Instantiate(self.officer_prefab).transform
                elem:SetParent(self.officer_content)
                resetTransformZ(elem)
            else
                elem = self.officer_content:GetChild(k-1)
            end

            elem.name = "officer_"..v.id

            local icon = elem:Find("Head"):GetComponent(UnityEngine.UI.Image)
            local grade = elem:Find("Head/Grade"):GetComponent(UnityEngine.UI.Image)
            local level = elem:Find("Head/Level"):GetComponent(UnityEngine.UI.Text)
            local type = elem:Find("Type"):GetComponent(UnityEngine.UI.Image)
            local name = elem:Find("Type/Text"):GetComponent(UnityEngine.UI.Text)
            local cargo_text = elem:Find("Cargo/Text"):GetComponent(UnityEngine.UI.Text)
            local toggle = elem:Find("Toggle/Image")
            local stars = elem:Find("Head/Stars")

            local cfg = v.officer.cfg
            icon:SetSprite(gui_helper:getOfficerIconPath(v.officer))
            grade:SetSprite(self:getGradeRes(cfg.color))
            type:SetSprite(cfg.icon_path..cfg.soldiertypeicon)
            name.text = self.helper:getLangCfg(cfg.soldier_typename)
            level.text = v.officer.level
            cargo_text.text = self.helper:itemNumFormat(v:getGlobalProp(1171))
            
            if self:officerIsAppend(v.id) then
                toggle.localScale = Vector3.one
            else
                toggle.localScale = Vector3.zero
            end

            gui_helper:setOfficerStars(v.officer.star_level, stars)

            self:clearScript("click", "BG/Content/CommanderPanel/CommanderList/Viewport/Content/"..elem.name)
            self:clearScript("click", "BG/Content/CommanderPanel/CommanderList/Viewport/Content/"..elem.name.."/Head/Grade")
            self:setScript("click", "BG/Content/CommanderPanel/CommanderList/Viewport/Content/"..elem.name, functor.rmake(self.click_officer, v.id))
            self:setScript("click", "BG/Content/CommanderPanel/CommanderList/Viewport/Content/"..elem.name.."/Head/Grade", functor.rmake(self.click_Head, v.id))

            self.officer_toggles[v.id] = toggle
        end

        for i=#heros, self.officer_content.childCount - 1 do
            local elem = self.officer_content:GetChild(i)
            elem.localScale = Vector3.zero
        end

        self:clearScript("click", "BG/Content/CommanderPanel/Confirm")
        self:setScript("click", "BG/Content/CommanderPanel/Confirm", functor.make(self.closeOfficerPanel, self))
    end,

    getGradeRes = function(self, color)
        local path = res_path.res_path_95

        if color == 1 then
            path = res_path.res_path_96
        elseif color == 2 then
            path = res_path.res_path_97
        elseif color == 3 then
            path = res_path.res_path_98
        elseif color == 4 then
            path = res_path.res_path_99
        elseif color == 5 then
            path = res_path.res_path_100
        end

        return path
    end,

    officerIsAppend = function(self, id)
        for k, v in ipairs(self.append_officers) do
            if v == id then
                return true, k
            end
        end

        return false
    end,

    appendOfficer = function(self, id)
        local idx = #self.append_officers
        local head_data = self.officer_heads[idx]

        if head_data then
            local hero = hero_manager:getHero(id)

            if hero then
                head_data.id = id
                local ui = head_data.ui
                local head = ui:Find("Head")
                local icon = ui:Find("Head/Icon"):GetComponent(UnityEngine.UI.Image)
                local grade = ui:Find("Head/Grade"):GetComponent(UnityEngine.UI.Image)
                local level = ui:Find("Head/Level"):GetComponent(UnityEngine.UI.Text)
                local stars = ui:Find("Head/Stars")

                head.transform.localScale = Vector3.one

                icon:SetSprite(gui_helper:getOfficerIconPath(hero.officer))
                grade:SetSprite(self:getGradeRes(hero.officer.cfg.color))
                level.text = hero.officer.level
                gui_helper:setOfficerStars(hero.officer.star_level, stars)
                
                if self.action == common_pb.EMARCH_GATHER_TITAN then
                    self:setGatherSpeed()
                end

                self:setCargo()
                self:setSoldiersAmount()
                self:setMaxCount()
            else
                log.error("####################################### hero id error ! id ", id)
            end
        end
    end,

    disappendOfficer = function(self, id)
        for k, v in ipairs(self.officer_heads) do
            if v.id == id then
                local head = v.ui:Find("Head")
                head.localScale = Vector3.zero

                --local grade = v.ui:Find("Grade"):GetComponent(UnityEngine.UI.Image)
                --grade:SetSprite(self:getGradeRes(0))

                v.id = 0
                v.ui:SetAsLastSibling()
            end
        end
        
        local sortFunc = function(a, b)
            local sibling_a = a.ui:GetSiblingIndex()
            local sibling_b = b.ui:GetSiblingIndex()

            return sibling_a < sibling_b
        end

        table.sort( self.officer_heads, sortFunc )

        if self.action == common_pb.EMARCH_GATHER_TITAN then
            self:setGatherSpeed()
        end

        self:setCargo()
        self:setSoldiersAmount()
    end,

    soldierCntCheck = function(self)
        local max = world_map_helper:teamSoldiersAmount(self.append_officers)
        if max < self.total_soldiers then
            for k, v in pairs(self.append_soldiers) do
                if v.count > 0 then
                    self.changed = true
                    self:changeSoldierCount(v.cfg_id, v.count*-1)
                    v.count = 0
                end
            end
        end
    end,

    setOfficerDetails = function(self, id)
        local hero = hero_manager:getHero(id)

        if hero then
            local icon = self.officer_details:Find("Head"):GetComponent(UnityEngine.UI.Image)
            local grade = self.officer_details:Find("Head/Grade"):GetComponent(UnityEngine.UI.Image)
            local type_icon = self.officer_details:Find("Head/Type"):GetComponent(UnityEngine.UI.Image)
            local type_name = self.officer_details:Find("Head/Type/Text"):GetComponent(UnityEngine.UI.Text)
            local level = self.officer_details:Find("Head/Level"):GetComponent(UnityEngine.UI.Text)
            local stars = self.officer_details:Find("Head/Stars")
            local name = self.officer_details:Find("Head/Name"):GetComponent(UnityEngine.UI.Text)
            local cargo_text = self.officer_details:Find("Head/Cargo/Text"):GetComponent(UnityEngine.UI.Text)

            local cfg = hero.officer.cfg
            icon:SetSprite(gui_helper:getOfficerIconPath(hero.officer))
            grade:SetSprite(self:getGradeRes(hero.officer.cfg.color))
            type_icon:SetSprite(cfg.icon_path..cfg.soldiertypeicon)
            type_name.text = self.helper:getLangCfg(cfg.soldier_typename)
            level.text = hero.officer.level
            name.text = hero.officer.name
            gui_helper:setOfficerStars(hero.officer.star_level, stars)
            cargo_text.text = self.helper:itemNumFormat(hero:getGlobalProp(1171))

            local skill_panel = self.officer_details:Find("Panel")
            local skills = hero.officer.skills

            for i=1, skill_panel.childCount do
                local elem = skill_panel:GetChild(i-1)
                local data = skills[i]

                if data then
                    local skill_lv = data.skillLv > 0 and data.skillLv or 1
                    local cfg = passive_skill_helper:getSkill(data.skillId, skill_lv)

                    if not cfg then
                        log.error("###### officer talent cfg error ! id "..data.skillId.." level "..skill_lv)
                    else
                        local icon = elem:GetComponent(UnityEngine.UI.Image)
                        local name = elem:Find("Name"):GetComponent(UnityEngine.UI.Text)
                        local des = elem:Find("Des"):GetComponent(UnityEngine.UI.Text)
                        local lock_img = elem:Find("Locked")

                        icon:SetSprite(cfg.skill_icon_path..cfg.skill_icon_name)
                        name.text = self.helper:getLangCfg(cfg.lang_id)
                        des.text = self.helper:getLangCfg(cfg.skill_des)

                        --local unlock_star = hero.officer:getSkillUnlockStar(i)
                        --local star_lv = hero.officer.star_level

                        --if star_lv < unlock_star then
                        --    lock_img.localScale = Vector3.one
                        --else
                            lock_img.localScale = Vector3.zero
                        --end

                        elem.localScale = Vector3.one
                    end
                else
                    elem.localScale = Vector3.zero
                end
            end
        end
    end,

    setOfficerProps = function(self)
        local props = {}

        for k, v in ipairs(self.append_officers) do
            local hero = hero_manager:getHero(v)

            if hero then
                for _, prop in ipairs(hero.global_props) do
                    if props[prop.PropId] then
                        props[prop.PropId] = props[prop.PropId] + prop.PropValue
                    else
                        props[prop.PropId] = prop.PropValue
                    end
                end
            end
        end

        local cargo = self.addition_info:Find("Amount/Value"):GetComponent(UnityEngine.UI.Text)
        local amount = props[1171] or 0
        cargo.text = "+"..self.helper:itemNumFormat(amount)

        local node = self.addition_info:Find("Scroll View/Viewport/Content")
        local prefab = node:GetChild(0).gameObject
        local cnt = 0

        for k, v in pairs(props) do
            if v > 0 then
                local cfg = config.player_config_attrs[k]

                if cfg then
                    local elem = nil

                    if cnt >= node.childCount then
                        elem = Object.Instantiate(prefab).transform
                        elem:SetParent(node)
                        resetTransformZ(elem)
                    else
                        elem = node:GetChild(cnt)
                    end

                    elem:SetActive(true)

                    local name = elem:GetComponent(UnityEngine.UI.Text)
                    local value = elem:GetChild(0):GetComponent(UnityEngine.UI.Text)

                    name.text = self.helper:getLangCfg(cfg.name_id)

                    local val = v
                    if cfg.isfloat > 0 then
                        val = val * 0.0001
                    end

                    if cfg.showpercent > 0 then
                        val = val*100 .."%"
                    end

                    value.text = string.format( "+%s", tostring(val) )

                    cnt = cnt + 1
                else
                    log.info("########################################### player config attrs error ! id", k)
                end
            end
        end

        for i=cnt, node.childCount - 1 do
            local elem = node:GetChild(i)
            elem:SetActive(false)
        end
    end,

    onValueChanged = function(self, idx)
        if not self.changed then
            local elem = self.soldier_ui[idx]
            local slider = elem:Find("Slider"):GetComponent(UnityEngine.UI.Slider)
            local own_count = elem:Find("Grade/Count"):GetComponent(UnityEngine.UI.Text)
            local append_count = elem:Find("Count"):GetComponent(UnityEngine.UI.Text)
            local count = math.ceil(slider.value * self.soldier_list[idx])

            local append_soldier = self.append_soldiers[idx]
            local value = count - append_soldier.count

            if self.total_soldiers + value > self.append_max then
                value = self.append_max - self.total_soldiers
            end

            self.total_soldiers = self.total_soldiers + value
            append_soldier.count = append_soldier.count + value
    
            own_count.text = self.soldier_list[idx] - append_soldier.count
            append_count.text = append_soldier.count

            if self.total_soldiers >= self.append_max then
                slider.value = append_soldier.count / self.soldier_list[idx]
            end

            self:setSpeed()
            self:setCargo()
            self:setSoldiersAmount()
        end

        self.changed = false
    end,

    changeSoldierCount = function(self, id, value)
        if self.total_soldiers + value > self.append_max then
            value = self.append_max - self.total_soldiers
        end

        self.total_soldiers = self.total_soldiers + value

        local elem = self.soldier_ui[id]
        local slider = elem:Find("Slider"):GetComponent(UnityEngine.UI.Slider)
        local own_count = elem:Find("Grade/Count"):GetComponent(UnityEngine.UI.Text)
        local append_count = elem:Find("Count"):GetComponent(UnityEngine.UI.Text)

        local append_soldier = self.append_soldiers[id]
        append_soldier.count = append_soldier.count + value

        own_count.text = self.soldier_list[id] - append_soldier.count
        append_count.text = append_soldier.count
        local percent = append_soldier.count / self.soldier_list[id]

        if slider.value == percent then
            self.changed = false
        else
            slider.value = percent
        end

        self:setSpeed()
        self:setCargo()
        self:setSoldiersAmount()
    end,

    setSpeed = function(self)
        local speed = 9999
        for k, v in pairs(self.append_soldiers) do
            if v.count > 0 then
                local cfg = config.soldier_soldiers[v.cfg_id]

                if cfg then
                    speed = speed < cfg.speed and speed or cfg.speed
                end
            end
        end

        if speed ~= self.speed and speed ~= 9999 then
            self.speed = speed
            self:setTime()
        end
    end,

    setPower = function(self)
        if self.action ~= common_pb.EMARCH_CREATE_MASS and self.action ~= common_pb.EMARCH_GATHER_TITAN then
            self.team_power = world_map_helper:calculateTeamPower(self.append_officers, self.append_soldiers)

            local power = self.team_power
            if power > 0 then
                power = power * (1 + self.subdue_factor)
                power = math.floor(power) > 0 and math.floor(power) or 1
            end
            self.power.text = tostring(power)
        end
    end,

    setCargo = function(self)
        local value = player_cfg_helper:teamCargo(self.append_officers, self.append_soldiers)
        self.cargo.text = self.helper:itemNumFormat(value)
    end,

    setSoldiersAmount = function(self)
        local value = world_map_helper:teamSoldiersAmount(self.append_officers)
        self.amount.text = string.format( "%s/%s", self.helper:itemNumFormat(self.total_soldiers), self.helper:itemNumFormat(value) ) 
    end,

    setBuildInfo = function(self)
        local name = self.transform:Find("BG/Content/Top/BuildName"):GetComponent(UnityEngine.UI.Text)
        local pos = self.transform:Find("BG/Content/Top/BuildName/Position"):GetComponent(UnderLineText)
        local icon = self.build_panel:Find("IconBG/Icon"):GetComponent(UnityEngine.UI.Image)
        local amount = self.build_panel:Find("Info/Amount/Value"):GetComponent(UnityEngine.UI.Text)

        name.text = self.helper:getLangCfg(self.build.name_id)

        local cb = function()
            gui_helper:positionJump(self.build.coordX, self.build.coordY)
            self:click_BackBtn()
        end
        pos:SetText(gui_helper:positionFormat(self.build.coordX, self.build.coordY), cb)

        icon:SetSprite(self.build.cfg.IconPath..self.build.cfg.Icon)
        amount.text = string.format( "%s/%s", self.helper:itemNumFormat(self.build.info.curHp), self.helper:itemNumFormat(self.build.info.maxHp) )

        self:setBuildTime()
    end,

    setBuildTime = function(self)
        --local time_text = self.build_panel:Find("Info/Time/Value"):GetComponent(UnityEngine.UI.Text)

        local time_text = self.build_panel:Find("Info/Time")
        time_text:SetActive(false)
    end,

    setTime = function(self)
        local dis = 0
        if self.action == common_pb.EMARCH_JOIN_MASS then
            local pos_1 = {}
            pos_1.x = self.mass_info.start_pos.posX
            pos_1.z = self.mass_info.start_pos.posY

            local pos_2 = {}
            pos_2.x = self.mass_info.mass_pos.posX
            pos_2.z = self.mass_info.mass_pos.posY

            dis = world_map_helper:getCoordDistance(pos_1, pos_2)
        else
            dis = world_map_helper:getDistToBase({x = self.build.coordX, z = self.build.coordY})
        end

        local time = world_map_helper:marchTime(dis, self.speed)

        log.info("######################################################## time ", dis, self.speed, time)
        self.time_text.text = gui_helper:timeFormat(time)
    end,

    closeOfficerPanel = function(self)
        self.officer_panel.localScale = Vector3.zero
        self.corps_panel.localScale = Vector3.one
        self.build_panel.localScale = Vector3.one
        
        self:soldierCntCheck()
        self:setMaxCount()
    end,

    clickOfficerHead = function(self)
        self.officer_panel.localScale = Vector3.one
        self.addition_info.localScale = Vector3.one
        self.officer_details.localScale = Vector3.zero
        self.corps_panel.localScale = Vector3.zero
        self.build_panel.localScale = Vector3.zero

        self:loadOfficers()
    end,

    click_officer = function(self, handler, id)
        self.addition_info.localScale = Vector3.one
        self.officer_details.localScale = Vector3.zero

        local toggle = self.officer_toggles[id]

        if toggle then
            local append, idx = self:officerIsAppend(id)
            if append then
                toggle.localScale = Vector3.zero
                table.remove( self.append_officers, idx )
                self:disappendOfficer(id)
            elseif #self.append_officers < 5 then
                toggle.localScale = Vector3.one
                table.insert( self.append_officers, id )
                self:appendOfficer(id)
            end
        end

        self:setOfficerProps()
    end,

    click_Head = function(self, handler, id)
        self.addition_info.localScale = Vector3.zero
        self.officer_details.localScale = Vector3.one
        self:setOfficerDetails(id)
    end,

    click_Plus = function(self, id, value)
        local append_soldier = self.append_soldiers[id]
        value = value or 1

        if append_soldier.count < self.soldier_list[id] then
            self.changed = true
            if append_soldier.count + value > self.soldier_list[id] then
                value = self.soldier_list[id] - append_soldier.count
            end
            self:changeSoldierCount(id, value)
        end
    end,

    click_Minus = function(self, id, value)
        local append_soldier = self.append_soldiers[id]
        value = value or 1

        if append_soldier.count > 0 then
            self.changed = true
            if append_soldier.count - value < 0 then
                value = append_soldier.count
            end
            self:changeSoldierCount(id, -1*value)
        end
    end,

    click_BackBtn = function(self)
        frame_manager:hide("guild_building_launch")
    end,

    click_Addition = function(self)
        self:showAddition()
    end,

    click_Send = function(self)
        if self.total_soldiers <= 0 then
            hint:setText(NativeText.GetLangStr(280420))
            return
        end

        network_sender:startArmyLine(self.action, self.append_officers, self.append_soldiers, self.build.id)

        self:click_BackBtn()
    end,

    click_Auto = function(self)
        self:clearAppend()
        self:autoAppendHero()
        self:autoAppendSoldier()
    end,

    click_Formation = function(self)

    end,
    
    click_PropUpClose = function(self)
        self.prop_up_panel.localScale = Vector3.zero
    end,

    clearAppend = function(self)
        for k, v in ipairs(self.append_officers) do
            self:disappendOfficer(v)
        end

        self.append_officers = {}

        for k, v in pairs(self.append_soldiers) do
            if v.count > 0 then
                self.changed = true
                self:changeSoldierCount(v.cfg_id, v.count*-1)
                v.count = 0
            end
        end
    end,

    autoAppendHero = function(self)
        local heros = hero_manager:getUnDispatchedHeros()
        table.sort( heros, function(a,b) return a.officer.cfg.color > b.officer.cfg.color end )

        for k, v in ipairs(heros) do
            if #self.append_officers >= 5 then
                break
            end

            if not self:officerIsAppend(v.id) then
                table.insert( self.append_officers, v.id )
                self:appendOfficer(v.id)
            end
        end
    end,

    autoAppendSoldier = function(self)
        local all_count = 0

        for k, v in pairs(self.soldier_list) do
            all_count = all_count + v
        end

        if all_count <= self.append_max or not self.enemy_soldiers then
            self:appendAllSoldiers()
        else
            local list = {}
            for k, v in pairs(self.append_soldiers) do
                table.insert( list, v )
            end

            local a1, a2, a3, a4 = world_map_helper:herosAddition(self.append_officers)
            local sort_func = function(a, b)
                local p_a = world_map_helper:soldierPower(a.cfg_id, a1, a2, a3, a4)
                local p_b = world_map_helper:soldierPower(b.cfg_id, a1, a2, a3, a4)

                if p_a == p_b then
                    return a.cfg_id < b.cfg_id
                end

                return p_a > p_b
            end

            table.sort(list, sort_func)

            -- 按战力选择士兵
            local total = 0
            local team_1 = {}
            for k, v in ipairs(list) do
                local max = self.append_max - total
                local amount = self.soldier_list[v.cfg_id]
                if amount then
                    local value = amount > max and max or amount
                    table.insert(team_1, {cfg_id = v.cfg_id, count = value})
                    total = total + value
                end
            end

            -- 选择最优兵种
            total = 0
            local team_2 = {}
            local type = world_map_helper:bestCorps(self.enemy_soldiers)
            log.info("#################################################### best type ", type)

            for k, v in ipairs(list) do
                local cfg = config.soldier_soldiers[v.cfg_id]
                if cfg then
                    if cfg.type == type then
                        local max = self.append_max - total
                        local amount = self.soldier_list[v.cfg_id]
                        if amount then
                            local value = amount > max and max or amount
                            table.insert(team_2, {cfg_id = v.cfg_id, count = value})
                            total = total + value
                        end
                    end
                end
            end

            local team_power = function(team)
                factor = world_map_helper:corpsSubdued(team, self.enemy_soldiers)
                local power = world_map_helper:calculateTeamPower(self.append_officers, team)
                power = power * (1 + factor)

                return power
            end
            
            local power_1 = team_power(team_1)
            local power_2 = team_power(team_2)

            if power_1 < power_2 then
                team_1 = team_2
            end

            for k, v in ipairs(team_1) do
                self.changed = true
                self:changeSoldierCount(v.cfg_id, v.count)
            end
        end
    end,

    appendAllSoldiers = function(self)
        for k, v in pairs(self.soldier_list) do
            self.changed = true
            self:changeSoldierCount(k, v)
        end
    end,

    setMaxCount = function(self)
        self.append_max = world_map_helper:teamSoldiersAmount(self.append_officers)
    end,

    showAddition = function(self)
        local list = self:officerAdditions()
        local cnt = 0

        local node = self.prop_up_panel:Find("BG")

        for type, props in pairs(list) do
            if cnt < node.childCount then
                local elem = node:GetChild(cnt)
                elem:SetActive(true)
                
                local content = elem:Find("Panel")
                local pb = content:GetChild(0).gameObject
                local size = 0

                for id, value in pairs(props) do
                    local cfg = config.player_config_attrs[id]

                    if cfg then
                        local child = nil

                        if size < content.childCount then
                            child = content:GetChild(size)
                        else
                            child = Object.Instantiate(pb).transform
                            child:SetParent(content)
                            resetTransformZ(child)
                        end

                        child:SetActive(true)

                        local name = child:GetComponent(UnityEngine.UI.Text)
                        local val = child:GetChild(0):GetComponent(UnityEngine.UI.Text)

                        name.text = self.helper:getLangCfg(cfg.name_id)
                        
                        if cfg.isfloat > 0 then
                            value = value * 0.0001
                        end

                        if cfg.showpercent > 0 then
                            value = value*100 .."%"
                        end

                        val.text = string.format( "+%s", tostring(value) )

                        size = size + 1
                    else
                        log.error("########################################### find player config attr error ! id", id)
                    end

                    app:startCoroutine(
                        function()
                            coroutine.yield(nil)
                            local height = content.rect.height + 50
                            height = height > 183 and height or 183
                            elem.sizeDelta = Vector2(590, height)
                        end
                    )                 
                end

                for i=size, content.childCount - 1 do
                    local child = content:GetChild(i)
                    child:SetActive(false)
                end

                cnt = cnt + 1
            end
        end

        for i=cnt, node.childCount - 1 do
            local child = node:GetChild(i)
            child:SetActive(false)
        end

        if cnt > 0 then
            self.prop_up_panel.localScale = Vector3.one
        end
    end,

    officerAdditions = function(self)
        local corp_props = {}
        local props = {}

        config.commander_soldier_additionals:foreach(
            function(k, v)
                corp_props[v.SoldierType] = {}

                local value = 0

                for _, id in ipairs(self.append_officers) do
                    local hero = hero_manager:getHero(id)
                    
                    local list = loadstring("return "..v.AttributeCount)()

                    for _, prop_id in ipairs(list) do
                        value = value + hero:getGlobalProp(prop_id)
                    end
                end

                props[v.ShowAttribute] = value
            end
        )

        config.commander_soldier_additionals:foreach(
            function(k, v)
                corp_props[v.SoldierType][v.ShowAttribute] = props[v.ShowAttribute]
            end
        )

        for idx, list in pairs(corp_props) do
            local cnt = 0
            for id, value in pairs(list) do
                if value > 0 then
                    cnt = cnt + 1
                else
                    corp_props[idx][id] = nil
                end
            end

            if cnt == 0 then
                corp_props[idx] = nil
            end
        end

        return corp_props
    end,
}