local super = require("ui.frame")
local log = logger.get("Game", "Guild Logo")

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        self.msg = self.args.data.msg

        clickEvents = 
        {
            "Close",
            "BG/Top/InfoBtn",
            "BG/Info/DelAllBtn",
            "BG/Tips",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self.tips = self.transform:Find("BG/Tips")
        self.up_progress_slider = self.transform:Find("BG/Info/FillBG/Fill"):GetComponent(UnityEngine.UI.Image)
        self.up_progress_text = self.transform:Find("BG/Info/Exp"):GetComponent(UnityEngine.UI.Text)
        self.up_level_text = self.transform:Find("BG/Info/Level"):GetComponent(UnityEngine.UI.Text)

        self.tips.localScale = Vector3.zero

        self.right_progress_slider = self.transform:Find("BG/Left/FillBG/Fill"):GetComponent(UnityEngine.UI.Image)
        self.right_progress_text = self.transform:Find("BG/Left/FillBG/Count"):GetComponent(UnityEngine.UI.Text)
        self.none_text = self.transform:Find("BG/Right/None")

        self.get_award_evt = local_guild.events:on("GETFAMILYGIFTREWARD", functor.make(self.onGetAward, self))
        self.get_item_evt = game_director.events:on("NOTIFYGETITEM", functor.make(self.onGetItems, self))
        self.del_gift_evt = local_guild.events:on("DELFAMILYGIFT", functor.make(self.onDelGift, self))
        self.gift_exp_evt = local_guild.events:on("NOTIFYFAMILYGIFTEXP", functor.make(self.onExpChanged, self))
        self.gift_notify_evt = local_guild.events:on("NOTIFYNEWFAMILYGIFT", functor.make(self.onGiftChanged, self))

        self:refresh(self.msg)
    end,

    onUnload = function (self)
        local_guild.events:remove("GETFAMILYGIFTREWARD", self.get_award_evt)
        local_guild.events:remove("DELFAMILYGIFT", self.del_gift_evt)
        local_guild.events:remove("NOTIFYFAMILYGIFTEXP", self.gift_exp_evt)
        local_guild.events:remove("NOTIFYNEWFAMILYGIFT", self.gift_notify_evt)
        game_director.events:remove("NOTIFYGETITEM", self.get_item_evt)
    end,

    onGetItems = function(self, msg)
        if msg.reason == common_pb.FAMILYGIFT then
            frame_manager:show('getItems', self, {items = gui_helper.drop_list[msg.reason], has_Confirm_Btn = true})
        end
    end,

    onGetAward = function(self, msg)
        log.info("############################## inst id ", msg.instId)
        local info = self.gift_info[msg.instId]

        if info then
            info.data.status = 1
            self:setGiftInfo(info.data, info.ui)
        end
    end,

    onDelGift = function(self)
        network_sender:getFamilyGiftInfo()
    end,

    onExpChanged = function(self, msg)
        log.info("################################### big gift exp ", msg.bigGiftId, msg.bigGiftExp)
        self:setRightProgress(msg.bigGiftId, msg.bigGiftExp)
        self:setUpProgress(msg)
    end,

    onGiftChanged = function(self, msg)
        for k, v in ipairs(msg.gifts) do
            local info = self.gift_info[v.giftId]
            if info then
                self:setGiftInfo(v, info.ui)
            else
                self:addGift(v)
            end
        end
    end,

    update = function(self, dt)
        if self.gift_info then
            for k, v in pairs(self.gift_info) do
                if v.data.status == 0 then
                    local lt = v.data.time
                    if lt > 0 then
                        local tiem_text = v.ui:Find("Open/Time"):GetComponent(UnityEngine.UI.Text)
                        lt = lt - dt
                        tiem_text.text = gui_helper:timeFormat(lt)
                        v.data.time = lt > 0 and lt or 0
                    else
                        v.data.status = 2
                        self:setGiftInfo(v.data, v.ui)
                    end
                end
            end
        end
    end,

    refresh = function(self, msg)
        self.gift_info = {}
        self.gift_list = {}

        for k, v in ipairs(msg.giftList) do
            table.insert( self.gift_list, v )
        end

        local sort_fun = function(a, b)
            if a.status == b.status then
                return a.time > b.time
            end

            return a.status < b.status
        end

        table.sort( self.gift_list, sort_fun )
        
        self:initUI(msg)
        self:loadGiftElem()
    end,

    initUI = function(self, msg)
        local gift_icon = self.transform:Find("BG/Left/GiftBG/Gift"):GetComponent(UnityEngine.UI.Image)
        local gift_name = self.transform:Find("BG/Left/GiftBG/GiftName"):GetComponent(UnityEngine.UI.Text)

        local big_gift_cfg = config.union_gift_biggifts[msg.bigGiftId]

        if big_gift_cfg then
            local item_cfg = item_helper:getItemCfg(big_gift_cfg.rewardid)

            if item_cfg then
                gift_icon:SetSprite(gui_helper:getItemIconPath(item_cfg))
                gift_name.text = self.helper:getLangCfg(item_cfg.name_id)
            else
                log.error("#################################### find item cfg error ! id ", cfg.rewardid)
            end
        else
            log.error("############################## find union biggift cfg error ! id", msg.bigGiftId)
        end

        self:setGiftCount()
        self:setUpProgress(msg)
        self:setRightProgress(msg.bigGiftId, msg.bigGiftExp)
    end,

    setGiftCount = function(self)
        local gift_count = self.transform:Find("BG/Right/Count"):GetComponent(UnityEngine.UI.Text)
        local cur_cnt = #self.gift_list
        local max_cnt = config.union_others[1].GiftLimit

        gift_count.text = string.format( "%s:  %d/%d", NativeText.GetLangStr(190243), cur_cnt, max_cnt )
    end,

    setUpProgress = function(self, msg)
        local cfg = config.union_gift_giftexps[msg.giftLevel]
        self.up_progress_slider.fillAmount = msg.giftExp / cfg.exp
        self.up_level_text.text = "Lv."..msg.giftLevel
        self.up_progress_text.text = string.format( "%d<color=#3CC7FFFF>/%d</color>", msg.giftExp, cfg.exp )
    end,

    setRightProgress = function(self, id, exp)
        local cfg = config.union_gift_biggifts[id]
        if not cfg then
            return
        end
        self.right_progress_slider.fillAmount = exp / cfg.NeedKeys
        self.right_progress_text.text = string.format( "%d<color=#3CC7FFFF>/%d</color>", exp, cfg.NeedKeys )
    end,

    loadGiftElem = function(self)
        local content = self.transform:Find("BG/Right/Scroll View/Viewport/Content")
        local trans = content:GetChild(0)
        for k, v in ipairs(self.gift_list) do
            local elem = nil
            if k > content.childCount then
                elem = Object.Instantiate(trans.gameObject).transform
                elem:SetParent(content)
                resetTransformZ(elem)
            else
                elem = content:GetChild(k-1)
            end

            if not elem then
                return
            end

            elem:SetActive(true)
            elem.name = "elem_"..v.giftId

            self:setGiftInfo(v, elem)

            self.gift_info[v.giftId] = {ui = elem, data = v}
        end

        for i=#self.gift_list, content.childCount-1 do
            content:GetChild(i):SetActive(false)
        end

        self.none_text:SetActive(#self.gift_list == 0)
    end,

    addGift = function(self, info)
        local content = self.transform:Find("BG/Right/Scroll View/Viewport/Content")
        local trans = content:GetChild(0)

        local elem = Object.Instantiate(trans.gameObject).transform
        elem:SetParent(content)
        resetTransformZ(elem)


        elem:SetActive(true)
        elem.name = "elem_"..info.giftId

        elem:SetAsFirstSibling()

        self:setGiftInfo(info, elem)

        self.gift_info[info.giftId] = {ui = elem, data = info}
        table.insert( self.gift_list, 1, info )

        self.none_text:SetActive(#self.gift_list == 0)
    end,

    setGiftInfo = function(self, info, elem)
        local grade = elem:Find("Grade"):GetComponent(UnityEngine.UI.Image)
        local icon = grade.transform:GetChild(0):GetComponent(UnityEngine.UI.Image)
        local name = elem:Find("Name"):GetComponent(UnityEngine.UI.Text)
        local des = elem:Find("Des"):GetComponent(UnityEngine.UI.Text)
        local left_time = elem:Find("Open/Time"):GetComponent(UnityEngine.UI.Text)
        local open_btn = elem:Find("Open"):GetComponent(UnityEngine.UI.Button)
        local del_btn = elem:Find("Delete"):GetComponent(UnityEngine.UI.Button)
        local collected = elem:Find("Collected")
        local expired = elem:Find("Expired")

        local item_id = nil
        local cfg = config.union_gift_gifts[info.cfgId]

        if cfg then
            if info.status == 0 then
                item_id = cfg.commongift
            else
                item_id = info.rewards.ItemTemId
            end

            collected:SetActive(info.status == 1)
            expired:SetActive(info.status == 2)
            open_btn.transform:SetActive(info.status == 0)
            del_btn.transform:SetActive(info.status ~= 0)

            local item_cfg = item_helper:getItemCfg(item_id)

            if item_cfg then
                grade:SetSprite(gui_helper:getQualityIconPath(item_cfg.color))
                icon:SetSprite(gui_helper:getItemIconPath(item_cfg))
                name.text = string.format("[%s]%s", frame_helper:getLangCfg(cfg.quality), frame_helper:getLangCfg(item_cfg.name_id))

                des.transform:SetActive(info.status ~= 0)

                if info.status ~= 0 then
                    des.text = string.format( "%s x%d, %s x%d", self.helper:getLangCfg(item_cfg.name_id), info.rewards.Count, NativeText.GetLangStr(190244), cfg.AddKeys )
                end
            else
                log.info("######################################### find item cfg error ! id", item_id)
            end

            open_btn.onClick:RemoveAllListeners()
            open_btn.onClick:AddListener(
                function()
                    self:onClickGetAward(info.giftId)
                end
            )

            del_btn.onClick:RemoveAllListeners()
            del_btn.onClick:AddListener(
                function()
                    self:onClickDel(info.giftId)
                end
            )

            local item_btn = grade:GetComponent(UnityEngine.UI.Button)
            item_btn.onClick:RemoveAllListeners()

            if info.status == 0 then
                item_btn.onClick:AddListener(
                    function()
                        self:showTips(item_cfg, cfg.quality)
                    end
                )
            end
        else
            log.error("########################################### find gift cfg error !", info.cfgId)
        end
    end,

    showTips = function(self, item_cfg, quality_id)
        self.tips.localScale = Vector3.one

        local name = self.tips:Find("BG/Name"):GetComponent(UnityEngine.UI.Text)
        local des = self.tips:Find("BG/Des"):GetComponent(UnityEngine.UI.Text)
        local grade = self.tips:Find("BG/Grade"):GetComponent(UnityEngine.UI.Image)
        local icon = self.tips:Find("BG/Grade/Icon"):GetComponent(UnityEngine.UI.Image)

        grade:SetSprite(gui_helper:getQualityIconPath(item_cfg.color))
        icon:SetSprite(gui_helper:getItemIconPath(item_cfg))
        name.text = string.format("[%s]%s", frame_helper:getLangCfg(quality_id), frame_helper:getLangCfg(item_cfg.name_id))
        des.text = self.helper:getLangCfg(item_cfg.des_id)
    end,

    onClickDel = function(self, id)
        network_sender:delFamilyGift(id, false)
    end,

    onClickGetAward = function(self, id)
        network_sender:getFamilyGiftAward(id)
    end,

    click_Close = function(self)
        frame_manager:hide('guild_gifts')
    end,

    click_Tips = function(self)
        self.tips.localScale = Vector3.zero
    end,

    click_InfoBtn = function(self)
        
    end,

    click_DelAllBtn = function(self)
        network_sender:delFamilyGift("", true)
    end,
}
