local log = logger.get("Game", "Guild Help Frame")

return class {
    ctor = function (self, panel, parent)
       self.panel = panel
       self.transform = parent.transform

        self.content = self.transform:Find("content/HelpList/Scroll View/Viewport/Content")
        self.prefab = self.content:GetChild(0).gameObject

        self:init()
    end,

    init = function(self)
        self.progress_text = self.transform:Find("content/HelpList/Bottom/Contribution/Amount"):GetComponent(UnityEngine.UI.Text)
        self.none_trans = self.transform:Find("content/HelpList/none")
        self.list_cnt = 0
        self.list = {}
        self.own_list = {}
        self.del_list = {}

        local btn = self.transform:Find("content/HelpList/Bottom/HelpAll"):GetComponent(UnityEngine.UI.Button)
        btn.onClick:RemoveAllListeners()
        btn.onClick:AddListener(
            function()
                self:click_helpAllBtn()
            end
        )
    end,

    show = function(self, msg)
        self.panel:SetActive(true)
        self.msg = msg
        self:loadHelpList()
        self.is_show = true
    end,

    refresh = function(self, msg)
        if not self.is_show then
            return
        end

        self:addHelp(msg.helpList)
        self:removeHelp(msg.delhelps)
    end,

    onHelpSuccess = function(self, msg)
        if not self.is_show then
            return
        end

        self:updateProgress(msg.getFCoinNum, msg.getMaxFCoinNum)
        
        if msg.helptype == common_pb.HT_ALL then
            for k, v in pairs(self.list) do
                v:SetActive(false)
                self.list_cnt = self.list_cnt - 1
                table.insert( self.del_list, v )
            end
        else
            local elem = self.list[msg.instId]

            if elem then
                elem:SetActive(false)
                self.list_cnt = self.list_cnt - 1
                table.insert( self.del_list, elem )
            end
        end

        self.list_cnt = math.max(self.list_cnt, 0)

        if self.list_cnt <= 0 then
            frame_helper.showTrans(self.none_trans)
        end
    end,

    removeHelp = function(self, list)
        for k, v in ipairs(list) do
            local elem = self.list[v] or self.own_list[v]

            if elem then
                elem:SetActive(false)
                self.list_cnt = self.list_cnt - 1
                table.insert( self.del_list, elem )
            end
        end

        self.list_cnt = math.max(self.list_cnt, 0)

        if self.list_cnt <= 0 then
            frame_helper.showTrans(self.none_trans)
        end
    end,

    updateProgress = function(self, cur, max)
        self.progress_text.text = cur .. "/" .. max
    end,

    loadHelpList = function(self)
        if self.msg.helpList and #self.msg.helpList > 0 then
            frame_helper.hideTrans(self.none_trans)
        else
            frame_helper.showTrans(self.none_trans)
        end

        self:updateProgress(self.msg.getFCoinNum, self.msg.getMaxFCoinNum)

        for idx, data in ipairs(self.msg.helpList) do
            local child = nil

            if idx > self.content.childCount then
                child = Object.Instantiate(self.prefab).transform
                child:SetParent(self.content)
                resetTransformZ(child)
            else
                child = self.content:GetChild(idx - 1)
            end

            child:SetActive(true)

            self:setInfo(child, data)

            if data.playerId ~= local_player.id then
                self.list[data.instid] = child
            else
                self.own_list[data.instid] = child
            end

            self.list_cnt = self.list_cnt + 1
        end

        for i=#self.msg.helpList, self.content.childCount - 1 do
            local child = self.content:GetChild(i)
            child:SetActive(false)
            
            table.insert( self.del_list, child )
        end
    end,

    addHelp = function(self, list)
        if not list then
            return
        end
        
        for k, v in ipairs(list) do
            local elem = self.del_list[1]

            if elem == nil then
                elem = Object.Instantiate(self.prefab).transform
                elem:SetParent(self.content)
                resetTransformZ(elem)
            else
                table.remove( self.del_list, 1 )
            end

            elem:SetActive(true)
            self:setInfo(elem, v)

            if v.playerId ~= local_player.id then
                self.list[v.instid] = elem
            else
                self.own_list[v.inst_id] = elem
            end

            self.list_cnt = self.list_cnt + 1
        end

        frame_helper.hideTrans(self.none_trans)
    end,

    setInfo = function(self, child, data)
        local icon = child:Find('Grade/Icon'):GetComponent(UnityEngine.UI.Image)
        local quality = child:Find('Grade'):GetComponent(UnityEngine.UI.Image)
        local guild_rank = child:Find('Level'):GetComponent(UnityEngine.UI.Image)

        local name_text = child:Find('Name'):GetComponent(UnityEngine.UI.Text)
        local des_text = child:Find('Des'):GetComponent(UnityEngine.UI.Text)
        local progress_text = child:Find('FillBG/Count'):GetComponent(UnityEngine.UI.Text)
        local fill = child:Find('FillBG/Fill'):GetComponent(UnityEngine.UI.Image)
        local btn = child:Find('HelpBtn'):GetComponent(UnityEngine.UI.Button)

        name_text.text = data.playerName
        progress_text.text = data.curHelpNum.."/"..data.maxHeleNum
        fill.fillAmount = data.curHelpNum / (data.maxHeleNum > 0 and data.maxHeleNum or 1)

        local cfg = nil

        if data.helpType == common_pb.HT_BUILD then
            cfg = config.home_config_lists[data.cfgId]
        elseif data.helpType == common_pb.HT_RES then
            cfg = config.home_config_research_lists[data.cfgId]
        end

        if cfg then
            des_text.text = string.format(NativeText.GetLangStr(190161), frame_helper:getLangCfg(cfg.name_id) .. " Lv." .. data.buildLevel)
        else
            log.error("################################### find cfg error ! id "..data.cfgId.." type "..data.helpType)
        end

        btn.transform:SetActive(true)
        btn.onClick:RemoveAllListeners()
        if data.playerId == local_player.id then
            btn.transform:SetActive(false)
        else
            btn.onClick:AddListener(
                function ()
                    self:click_single(data.instid, data.helpType)
                end
            )
        end
    end,

    click_single = function(self, inst_id, type)
        network_sender:helpFastBuild(inst_id, type)
    end,

    click_helpAllBtn = function(self)
        network_sender:helpFastBuild("", common_pb.HT_ALL)
    end,

    hide = function(self)
        self.panel:SetActive(false)
        self.is_show = false
    end,
}