local super = require("ui.frame")
local log = logger.get("Game", "Guild Logo")

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        self.game_state_island = self.args.owner
        self.msg = self.args.data.msg
        self.familyId = self.msg.familyInfo.familyId

        clickEvents = 
        {
            "content/bg/top/ButtonBack",
            "content/bg/top/InfoBtn",
            "content/info/layout/gift",
            "content/info/layout/application",
            "content/info/layout/diplomacy",
            "content/info/layout/member",
            "content/info/Bottom/Ranking",
            "content/info/Bottom/Message",
            "content/info/Bottom/Manage",
            "content/info/Bottom/Shop",
            "content/info/Bottom/ContactBtn",
            "content/info/Bottom/EnterBtn",
            "content/info/Bottom/ApplyBtn",
            "content/info/down/Top/ModifyBtn",
            "tech_detail/closeTechDetailBtn",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self.is_member = local_player:getGuildId() == self.familyId

        self.info_panel = self.transform:Find("content/info")
        self.build_panel = self.transform:Find("content/Build")
        self.mass_panel = self.transform:Find("content/MassList")
        self.tech_panel = self.transform:Find("content/tech")
        self.help_panel = self.transform:Find("content/HelpList")

        self.build_ui = require("gui.guild_build_panel")(self.build_panel, self)
        self.mass_ui = require("gui.guild_mass_panel")(self.mass_panel, self)
        self.help_ui = require("gui.guild_help_frame")(self.help_panel, self)

        self.panels = {}
        self.panels[1] = {ui = self.info_panel, name = "info"}
        self.panels[2] = {ui = self.mass_panel, frame = self.mass_ui, name = "build", func = function() self:onClick_Mass() end}
        self.panels[3] = {ui = self.build_panel, frame = self.build_ui, name = "build", func = function() self:onClick_Building() end}
        self.panels[4] = {ui = self.tech_panel, name = "tech"}
        self.panels[5] = {ui = self.help_panel, frame = self.help_ui, name = "help", func = function() self:onClick_Help() end}

        self:init(self.msg)
        self:refreshInfo(self.msg)

        self.member_evts = local_guild.events:on("GETMEMBERLIST", functor.make(self.onGetMemberInfo, self))

        self.guild_info_change_event_id = local_guild.events:on("guild_info_change", functor.make(self.onGuildInfoChange, self))
        self.item_evt = game_director.events:on("NOTIFYGETITEM", functor.make(self.onGetItems, self))
        self.mass_evts = game_director.events:on("WORLDMAPMASSDEL", functor.make(self.onMassChanged, self))
        self.mass_notify_id = game_director.events:on("WORLDMAPMASSNOTIFY", functor.make(self.onMassChanged, self))

        guide_manager:tryStartGuide()
    end,

    onUnload = function (self)
        self.game_state_island.guild_info = nil
        local_guild.events:remove("GETMEMBERLIST", self.member_evts)
        local_guild.events:remove("guild_info_change", self.guild_info_change_event_id)
        game_director.events:remove("NOTIFYGETITEM", self.item_evt)
        game_director.events:remove("WORLDMAPMASSDEL", self.mass_evts)
        game_director.events:remove("WORLDMAPMASSNOTIFY", self.mass_notify_id)
    end,

    onGetMemberInfo = function(self, msg)
        if self.showMembers then
            self.showMembers = false
            frame_manager:show('guild_members', self, {msg = msg})
        end

        if self.showRank then
            self.showRank = false
            frame_manager:show("guild_member_rank", self, {msg = msg, info = self.msg.familyInfo})
        end
    end,

    onGetItems = function(self, msg)
        if self.build_ui then
            self.build_ui:onGetItems(msg)
        end
    end,

    onMassChanged = function(self)
        if self.mass_ui and self.mass_ui.is_show then
            self:onClick_Mass()
        end
    end,

    onGuildInfoChange = function(self)
        local info = local_guild:getInfo()
        local num_content = self.transform:Find("content/info/up/num_text"):GetComponent(UnityEngine.UI.Text)
        num_content.text = info.memberNum .. "/" .. info.maxMemberNum
    end,

    onGetFamilyBuild = function(self, msg)
        self.build_ui:show(msg)
    end,

    onGetMassList = function(self, msg)
        self.mass_ui:show(msg.massInfos)
    end,

    onGetHelpList = function(self, msg)
        self.help_ui:show(msg)
    end,

    refreshBuild = function(self, msg)
        self.build_ui:refresh(msg)
    end,

    refreshHelpList = function(self, msg)
        self.help_ui:refresh(msg)
    end,

    onHelpSuccess = function(self, msg)
        self.help_ui:onHelpSuccess(msg)
    end,

    click_ButtonBack = function(self)
        frame_manager:hide('guild_info')
        local_guild.events:dispatch("GUILDFRAMEHIDE")
    end,

    click_InfoBtn = function(self)
        
    end,


    init = function(self, msg)
        --self:initFunctions(msg)
        self:initRithgBar(msg)
        self:initBottomBtn()
        self:initTech()
    end,

    -- 初始化功能列表
    initFunctions = function(self, msg)
        msg = msg.familyInfo

        local btn_list = {}
        local apply_btn =               self.transform:Find("content/info/layout/apply")
        local undo_apply_btn =          self.transform:Find("content/info/layout/undo_apply")
        local enter_btn =               self.transform:Find("content/info/layout/enter")
        local notEnough_btn =           self.transform:Find("content/info/layout/notEnough")
        local msg_btn =                 self.transform:Find("content/info/layout/msg")
        local member_btn =              self.transform:Find("content/info/layout/member")
        local member_accelerated_btn =  self.transform:Find("content/info/layout/member_accelerated")
        local fun_btn =                 self.transform:Find("content/info/layout/fun")
        local move_btn =                self.transform:Find("content/info/layout/move")
        local shop_btn =                self.transform:Find("content/info/layout/shop")
        local btn_all = { apply_btn, undo_apply_btn, enter_btn, notEnough_btn, msg_btn, member_btn, member_accelerated_btn, fun_btn, move_btn, shop_btn}

        -- 没有工会
        if not local_player:isInGuild() then
            if msg.status == 0 then
                btn_list = { apply_btn, msg_btn, member_btn }
            elseif msg.status == 1 then
                btn_list = { undo_apply_btn, msg_btn, member_btn }
            elseif msg.status == 2 then
                btn_list = { enter_btn, msg_btn, member_btn }
            elseif msg.status == 3 then
                btn_list = { notEnough_btn, msg_btn, member_btn }
            end
        else
            -- 是自己工会
            if local_player:getGuildId() == msg.familyId then
                if msg.isRemoval == 0 then 
                    btn_list = { msg_btn, member_btn, member_accelerated_btn, fun_btn, move_btn }
                else
                    btn_list = { msg_btn, member_btn, member_accelerated_btn, fun_btn, shop_btn }
                end
            else 
                btn_list = { msg_btn, member_btn }
            end
        end

        for __, b in ipairs(btn_all) do
            local visible = false
            for ___, btn in ipairs(btn_list) do
                if btn == b then
                    visible = true
                    break
                end
            end
            b.gameObject:SetActive(visible)
        end
    end,

    update = function(self, dt)
        if self.mass_ui then
            self.mass_ui:update(dt)
        end
    end,

    click_member = function(self)
        print("click member", self.familyId)
        self.showMembers = true
        network_sender:getFamilyMemberList(self.familyId)
    end,

    click_application = function(self)
        local _, degree = local_guild:getGuildLevel()
        if not self.is_member or degree > 2 then
            hint:setText(NativeText.GetLangStr(190092))
            return
        end
        network_sender:getFamilyApplyList()
    end,

    click_diplomacy = function(self)
        print("click msg ", self.msg.familyInfo.familyId)
        network_sender:getFamilyLeaveWord(self.msg.familyInfo.familyId)
    end,

    click_member_accelerated = function(self)
        print("click member_accelerated ")
        network_sender:getFamilyHelpList()
    end,

    click_Ranking = function(self)
        print("click Ranking ")
        self.showRank = true
        network_sender:getFamilyMemberList(self.familyId)
    end,

    click_Message = function(self)
        print("click Message ")
        hint:setText(NativeText.GetLangStr(900002))
        --[[ local _, degree = local_guild:getGuildLevel()
        if not self.is_member or degree > 2 then
            hint:setText(NativeText.GetLangStr(190092))
            return
        end
        frame_manager:show('guild_mail') ]]
    end,

    click_Manage = function(self)
        print("click Manage ")
        if not self.is_member then
            hint:setText(NativeText.GetLangStr(190092))
            return
        end
        self.guild_set = frame_manager:show('guild_set', self.game_state_island, {msg = self.msg})
    end,

    click_Shop = function(self)
        network_sender:getShopList()
    end,

    click_ContactBtn = function(self)
        hint:setText(NativeText.GetLangStr(900002))
    end,

    click_EnterBtn = function(self)
        network_sender:familyJoinOrCancel(0, self.msg.familyInfo.familyId)
    end,

    click_ApplyBtn = function(self)
        network_sender:familyJoinOrCancel(0, self.msg.familyInfo.familyId)
    end,

    click_gift = function(self)
        print("click gift")
        if not self.is_member then
            hint:setText(NativeText.GetLangStr(190092))
            return
        end
        network_sender:getFamilyGiftInfo()
    end,

    click_ModifyBtn = function(self)
        local _, degree = local_guild:getGuildLevel()
        if not self.is_member or degree > 2 then
            hint:setText(NativeText.GetLangStr(190092))
            return
        end
        frame_manager:show('guild_edit_notice', nil, {msg=self.msg, is_in_notice = true})
    end,

    -- 初始化右边按钮
    initRithgBar = function(self, msg)
        for i = 1, 5 do 
            local path = "content/right_bar/Right (" .. i .. ")/Page"
            self:setScript("click", path, function() self:click_right_bar(i) end)
        end

        if gui_helper.opend_guild_mass then
            self.right_bar_idx = 2
            gui_helper.opend_guild_mass = false
        else
            self.right_bar_idx = 1
        end
        
        self:click_right_bar(self.right_bar_idx)

        if not self.is_member then
            for i = 2, 5 do 
                local path = "content/right_bar/Right (" .. i .. ")"
                self.transform:Find(path).gameObject:SetActive(false)
            end
        end
    end,

    initBottomBtn = function(self)
        local btn_1 = self.transform:Find("content/info/Bottom/Manage")
        local btn_2 = self.transform:Find("content/info/Bottom/Shop")
        local btn_3 = self.transform:Find("content/info/Bottom/EnterBtn")
        local btn_4 = self.transform:Find("content/info/Bottom/ApplyBtn")
        local btn_5 = self.transform:Find("content/info/Bottom/ContactBtn")
        local btn_6 = self.transform:Find("content/info/down/Top/ModifyBtn")
        local btn_7 = self.transform:Find("content/info/Bottom/Message")

        btn_1:SetActive(self.is_member)
        btn_2:SetActive(self.is_member)
        btn_6:SetActive(self.is_member)
        btn_7:SetActive(self.is_member)
        btn_3:SetActive(not self.is_member and self.msg.familyInfo.status == 0)
        btn_4:SetActive(not self.is_member and self.msg.familyInfo.status == 1)
        btn_5:SetActive(not self.is_member)
    end,

    click_right_bar = function(self, idx)
        if idx == 4 then
            hint:setText(NativeText.GetLangStr(900002))
            return
        end

        self.right_bar_idx = idx

        for i = 1, 5 do
            local button = self:findImage("content/right_bar/Right (" .. i .. ")/Page")
            if i == idx then
                button.color = Color(1,1,1,1)
            else
                button.color = Color(1,1,1,0)
            end
            
            self:showPanel(i, idx)
        end
    end,

    showPanel = function(self, idx, cur_idx)
        local panel = self.panels[idx]
        if panel then
            if cur_idx == idx then
                if panel.func then
                    panel.func()
                else
                    self.helper.showTrans(panel.ui)
                end
            else
                if panel.frame and panel.frame.hide then
                    panel.frame:hide()
                else
                    self.helper.hideTrans(panel.ui)
                end
            end
        end
    end,

    onClick_Building = function(self)
        log.info("##### onClick_Building")
        network_sender:getFamilyBuildList()
        --self.build_ui:show({})
    end,

    onClick_Mass = function(self)
        network_sender:getMassList(local_guild:getId())
    end,

    onClick_Help = function(self)
        network_sender:getFamilyHelpList()
    end,

    -- 选择语言
    selectLanguage = function(self, cfg)
        self.cur_lang_cfg = cfg
        for __, item in ipairs(self.cache_language) do
            if item.cfg == cfg then
                item.select_bg.enabled = true
                item.select_point.enabled = true
            else
                item.select_bg.enabled = false
                item.select_point.enabled = false
            end
        end
        local text = self:findText("create/lang")
        text.text = self.helper:getLangCfg(cfg.language)
    end,

    refreshInfo = function(self, msg)
        if self.msg and msg.familyInfo.familyId ~= self.msg.familyInfo.familyId then
            return
        end
        self.msg = msg
        -- 设置界面刷新数据
        if self.guild_set then
            self.guild_set:refreshMsg(msg)
        end

        local guild = msg.familyInfo
        local lang_content =        self.transform:Find("content/info/up/lang_text"):GetComponent(UnityEngine.UI.Text)
        local num_content =         self.transform:Find("content/info/up/num_text"):GetComponent(UnityEngine.UI.Text)
        local gift_content =        self.transform:Find("content/info/layout/gift/Text"):GetComponent(UnityEngine.UI.Text)
        local strength_content =    self.transform:Find("content/info/up/strength_text"):GetComponent(UnityEngine.UI.Text)
        local name_content =        self.transform:Find("content/info/up/name_text"):GetComponent(UnityEngine.UI.Text)
        --local level_content =       self.transform:Find("content/info/up/level_text"):GetComponent(UnityEngine.UI.Text)
        --local nickname_content =    self.transform:Find("content/info/up/nickname_text"):GetComponent(UnityEngine.UI.Text)
        --local coord_content =       self.transform:Find("content/info/up/coord_text"):GetComponent("UnderLineText")
        local host_content =        self.transform:Find("content/info/up/host_text"):GetComponent(UnityEngine.UI.Text)
        local msg_content =         self.transform:Find("content/info/down/msg_text"):GetComponent(UnityEngine.UI.Text)
        local msg_title =         self.transform:Find("content/info/down/Top/Title"):GetComponent(UnityEngine.UI.Text)

        local icon_bg =             self.transform:Find("content/info/up/lastIcon/icon1"):GetComponent(UnityEngine.UI.Image)
        local icon_color =          self.transform:Find("content/info/up/lastIcon/bg/icon2"):GetComponent(UnityEngine.UI.Image)
        local icon =                self.transform:Find("content/info/up/lastIcon/bg/icon3"):GetComponent(UnityEngine.UI.Image)

        icon_bg:SetSprite(config.union_icons[guild.settingInfo.emblem.iconFrame].path .. config.union_icons[guild.settingInfo.emblem.iconFrame].icon) 
        icon_color:SetSprite(config.union_icons[guild.settingInfo.emblem.grounding].path .. config.union_icons[guild.settingInfo.emblem.grounding].icon) 
        icon:SetSprite(config.union_icons[guild.settingInfo.emblem.icon].path .. config.union_icons[guild.settingInfo.emblem.icon].icon) 

        if config.union_langs[guild.settingInfo.familylanguage] then
            lang_content.text = self.helper:getLangCfg(config.union_langs[guild.settingInfo.familylanguage].language)
        else
            lang_content.text = ""
        end

        num_content.text = guild.memberNum .. "/" .. guild.maxMemberNum
        --level_content.text = "Lv." .. guild.familyLevel
        strength_content.text = self.helper:itemNumFormat(guild.familyStrength)
        name_content.text = string.format("[%s]%s", guild.familycronym, guild.familyName)
        --nickname_content.text = ""
        gift_content.text = string.format( "%s Lv.%d", NativeText.GetLangStr(190227), guild.giftLevel )
        --coord_content:SetText(guild.familyBasePosx .. "," .. guild.familyBasePosy, nil)
        host_content.text = guild.leaderName
        msg_content.text = self.is_member and guild.settingInfo.innotice or guild.settingInfo.outnotice
        msg_title.text = self.is_member and NativeText.GetLangStr(190205) or NativeText.GetLangStr(190245)

        --[[ local button = self.transform:Find("content/info/up/detail/jumpToCoordBtn"):GetComponent(UnityEngine.UI.Button)
        button.onClick:AddListener(
            function ()
                -- print("跳转坐标:", guild.familyBasePosx .. "," .. guild.familyBasePosy)

                local ix = guild.familyBasePosx
                local iy = guild.familyBasePosy
                
                --self.owner:setCameraPos(ix, iy)
                if game_director:isWorldMapState() then
                    game_director.curr_state.world_map:setCameraPos(ix, iy)
                else
                    world_map_info.enter_pos = {}
                    world_map_info.enter_pos.x = ix
                    world_map_info.enter_pos.y = iy
                    game_director:changeState('loading', {next_state = 'world_map', scene_name = 'hex_map'})
                end

                frame_manager:hide('guild_info')
            end
        ) ]]
    end,

    -- 初始化联盟科技
    initTech = function(self)
        if local_guild:getId() == self.familyId then
            network_sender:getFamilyTech(self.familyId)
        end

        self.detail_trans = self.transform:Find("tech_detail")
        self.normal_trans = self.transform:Find("tech_detail/normal_trans")
        self.limit_trans =  self.transform:Find("tech_detail/limit_trans")
        self.max_trans =    self.transform:Find("tech_detail/max_trans")

        self.helper.hideTrans(self.detail_trans)
        self.helper.hideTrans(self.normal_trans)
        self.helper.hideTrans(self.limit_trans )
        self.helper.hideTrans(self.max_trans )
    end,

    -- msg = SCGetFamilyTech
    refreshTech = function(self, msg)
        self.tech_msg = msg

        local path = "content/tech/content/Scroll View/Viewport/Content/icons"
        --local line_path = "content/tech/content/Scroll View/Viewport/Content/lines"

        self.helper:hideAllChild(self, path)
        --self.helper:hideAllChild(self, line_path)

        local line_idx = 1

        local list = self:getTechCfgList()
        for __, cfg in ipairs(list) do
            local child = self.helper:getChild(self, __, path)
            child:SetActive(true)
            local icon = child:Find("icon"):GetComponent(UnityEngine.UI.Image)
            local n_bg = child:Find("n_bg"):GetComponent(UnityEngine.UI.Image)
            local s_bg = child:Find("s_bg"):GetComponent(UnityEngine.UI.Image)
            local ing_bg = child:Find("ing_bg"):GetComponent(UnityEngine.UI.Image)

            local level_text = child:Find("levelText"):GetComponent(UnityEngine.UI.Text)
            local name_text = child:Find("nameText"):GetComponent(UnityEngine.UI.Text)
            local can_upgrade = self:isTechCanUpgrade(cfg.id)

            icon:SetSprite(cfg.path .. cfg.icon, function()
                icon.color = can_upgrade and Color.white or Color.grey
            end)

            n_bg.enabled = cfg.quality == 1
            s_bg.enabled = cfg.quality == 2
            ing_bg.enabled = false
            icon.color = can_upgrade and Color.white or Color.grey
            n_bg.color = can_upgrade and Color.white or Color.grey  
            s_bg.color = can_upgrade and Color.white or Color.grey  

            name_text.text = self.helper:getLangCfg(cfg.name_id)

            local rect = child.transform:GetComponent(UnityEngine.RectTransform)
            rect.anchoredPosition = Vector2(cfg.x, cfg.y)
            level_text.text = "Lv." .. self:getTechLevel(cfg.id) .. "/" .. self:getTechMaxLevel(cfg.id)

            local btn = child:Find('icon'):GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(
                function ()
                    self:click_tech(cfg)
                end)
        end
        
        local getCenter = function(list1, list2)
            if #list1 == 1 and #list2 == 1 then
                local from = list1[1]
                local to = list2[1]
                return Vector2( (from.x + to.x) * 0.5, from.y)
            end
            local a = nil
            local b = nil
            if #list1 == 1 then
                a = list1
                b = list2
            else
                a = list2
                b = list1
            end
            if #b % 2 > 0 then
                local from = a[1]
                local temp_distance = frame_helper.max_number
                local rst = nil
                for __, to in ipairs(b) do
                    local d = Vector2.Distance(from, to)
                    if d < temp_distance then
                        temp_distance = d
                        rst = to
                    end
                end
                return Vector2( (from.x + rst.x) * 0.5, (from.y + rst.y) * 0.5 )
            else
                local x = (b[1].x + a[1].x) * 0.5
                local y = (b[1].y + b[#b].y) * 0.5
                return Vector2( x, y )
            end
        end

        local sortFunc = function(a, b)
            return a.y > b.y
        end

        self:initLine()
        self:clearLine()

        local cook_line = {}
        for __, cfg in ipairs(list) do
            local start_list = { Vector2(cfg.x, cfg.y) }
            local end_list = {}
            local precondition = loadstring('return ' .. cfg.fronttech)()
            for cfg_id, level in pairs(precondition) do
                local temp = config.union_research_lists[cfg_id]
                table.insert(end_list, Vector2(temp.x, temp.y))
            end
            table.sort(end_list, sortFunc)

            if #start_list > 0 and #end_list > 0 then
                local center = getCenter(start_list, end_list)
                local can_upgrade, need_cfg_id, need_level = self:isTechCanUpgrade(cfg.id)
                if can_upgrade then
                    local cur_level = self:getTechLevel(cfg.id)
                    if cur_level <= 0 then
                        self:addUnConnectLine(start_list, end_list, center, Color.white)
                    else
                        self:addConnectedLine(start_list, end_list, center, Color.white)
                    end
                else
                    self:addUnConnectLine(start_list, end_list, center, Color.white)
                end
            end
        end

        self:makeLine()

        local cfg = config.union_others[1]
        local content_trans = self.transform:Find("content/tech/content/Scroll View/Viewport/Content"):GetComponent(UnityEngine.RectTransform)
        content_trans.sizeDelta = Vector2(cfg.length, content_trans.sizeDelta.y)

        self.tech_tags = {}
        local layout_path = "tech_detail/normal_trans/cost"
        self.helper:hideAllChild(self, layout_path)
        for idx, cost in ipairs(self.tech_msg.donate_counts) do
            local child = self.helper:getChild(self, idx, layout_path)
            child:SetActive(true)
            local cost_icon = child:Find('icon'):GetComponent(UnityEngine.UI.Image)
            local cost_text = child:Find('text'):GetComponent(UnityEngine.UI.Text)
            cost_icon:SetSprite(self.helper:getItemIcon(cost.id))
            if cost.state == 0 then
                cost_text.text = self.helper:getItemNumFormat(cost.id, cost.count)
            else
                cost_text.text = NativeText.GetLangStr(400007)
                cost_text.color = Color.white
            end
            local tag = child:Find('tag'):GetComponent(UnityEngine.UI.Image)
            self.tech_tags[idx] = tag

            local btn = child:Find('checkBtn'):GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(
                function ()
                    self:click_selectCost(idx, cost)
                end)
        end

        self:makeDefaultCost(self.tech_msg.donate_counts)

        if self.tech_cfg then
            self:click_tech(self.tech_cfg)
        end

        self:recommendInfo(self.tech_msg.recommend_techs)
    end,

    recommendInfo = function(self, list)
        local layout = self.transform:Find("content/tech/content/Recommend/Layout")
        local prefab = layout:GetChild(0).gameObject
        
        self.recommend_techs = {}

        local cnt = 0

        for k, v in ipairs(list) do
            local cfg = config.union_research_lists[v]

            if cfg then
                local elem = nil

                if cnt < layout.childCount then
                    elem = layout:GetChild(cnt)
                else
                    elem = Object.Instantiate(prefab).transform
                    elem:SetParent(layout)
                    resetTransformZ(elem)
                end

                elem:SetActive(true)

                local icon = elem:Find("icon"):GetComponent(UnityEngine.UI.Image)
                local n_bg = elem:Find("n_bg"):GetComponent(UnityEngine.UI.Image)
                local s_bg = elem:Find("s_bg"):GetComponent(UnityEngine.UI.Image)

                local level_text = elem:Find("levelText"):GetComponent(UnityEngine.UI.Text)
                local name_text = elem:Find("nameText"):GetComponent(UnityEngine.UI.Text)
                local can_upgrade = self:isTechCanUpgrade(cfg.id)

                icon:SetSprite(cfg.path .. cfg.icon, function()
                    icon.color = can_upgrade and Color.white or Color.grey
                end)

                n_bg.enabled = cfg.quality == 1
                s_bg.enabled = cfg.quality == 2
                icon.color = can_upgrade and Color.white or Color.grey
                n_bg.color = can_upgrade and Color.white or Color.grey  
                s_bg.color = can_upgrade and Color.white or Color.grey  

                name_text.text = self.helper:getLangCfg(cfg.name_id)
                level_text.text = "Lv." .. self:getTechLevel(cfg.id)

                cnt = cnt + 1

                self.recommend_techs[v] = true
            else
                log.error("###################################### find union tech cfg error ! id", v)
            end
        end

        for i=cnt, layout.childCount - 1 do
            local elem = layout:GetChild(i)
            elem:SetActive(false)
        end
    end,

    initLine = function(self)
        self.connected_mono = self.transform:Find("content/tech/content/Scroll View/Viewport/Content/connected"):GetComponent(TreeNodeArrows)
        self.un_connect_mono = self.transform:Find("content/tech/content/Scroll View/Viewport/Content/un_connect"):GetComponent(TreeNodeArrows)
    end,

    clearLine = function(self)
        self.connected_mono:ClearArrow()
        self.un_connect_mono:ClearArrow()
    end,

    addConnectedLine = function(self, start_list, end_list, center, color)
        self.connected_mono:AddNodeData(start_list, end_list, center, color)
    end,

    addUnConnectLine = function(self, start_list, end_list, center, color)
        self.un_connect_mono:AddNodeData(start_list, end_list, center, color)
    end,

    makeLine = function(self)
        self.connected_mono:Excute()
        self.un_connect_mono:Excute()
    end,

    makeDefaultCost = function(self, list)
        if list and #list > 0 then
            if self.select_index then
                local cost = list[self.select_index]
                if cost.state == 0 then
                    self:click_selectCost(self.select_index, cost)
                    return
                end
            end

            for idx, cost in ipairs(list) do
                if cost.state == 0 then
                    self:click_selectCost(idx, cost)
                    return
                end
            end
            self:click_selectCost(-1, nil)
        else
            self:click_selectCost(-1, nil)
        end
    end,

    click_selectCost = function(self, index, cost)
        for idx, img in ipairs(self.tech_tags) do
            img.enabled = index == idx
        end
        self.select_index = index
        self.select_cost = cost
    end,

    -- 联盟研发配置表
    getTechCfgList = function(self)
        local list = {}
        config.union_research_lists:foreach(function(k, v)
            table.insert(list, v)
        end)
        local sortTab = function(a, b)
            return a.id < b.id
        end
        table.sort(list, sortTab)
        return list
    end,

    -- 获取研发等级
    getTechLevel = function(self, cfg_id)
        local level = 0
        for __, t in ipairs(self.tech_msg.techs) do
            if t.cfgId == cfg_id then
                return t.level
            end
        end
        return level
    end,

    -- 获取研发最大等级
    getTechMaxLevel = function(self, cfg_id)
        local max_level = 0
        config.union_research_details:foreach(function(k, v)
            if v.list_id == cfg_id then
                max_level = max_level + 1
            end
        end)
        return max_level
    end,

    -- 获取升级配置
    getTechDetailCfg = function(self, cfg_id, level)
        local rst = nil
        config.union_research_details:foreach(
            function(k ,v)
                if v.list_id == cfg_id and level == v.level then
                    rst = v
                end
            end)
        return rst
    end,

     -- 获取研发是否可点击升级
    -- fronttech 必须满足是1级
    isTechCanUpgrade = function(self, cfg_id)
        local cur_level = self:getTechLevel(cfg_id)
        local max_level = self:getTechMaxLevel(cfg_id)
        -- 满级
        if cur_level >= max_level then
            return false
        end
        -- 前置科技
        local cfg = config.union_research_lists[cfg_id]
        local cdt = loadstring('return ' .. cfg.fronttech)()
        for cfg_id, need_level in pairs(cdt) do
            local level = self:getTechLevel(cfg_id)
            if level < need_level then
                return false, cfg_id, need_level
            end
        end
        return true
    end,

    -- 获取单个科技数据
    getTechExp = function(self, cfg_id)
        for __, vl in ipairs(self.tech_msg.techs) do
            if vl.cfgId == cfg_id then
                return vl.curExp, vl.nextLevelExp
            end
        end
        local level = 1
        local cfg = self:getTechDetailCfg(cfg_id, level)
        return 0, cfg.exp
    end,

    click_tech = function(self, cfg)
        self.tech_cfg = cfg
        self.helper.showTrans(self.detail_trans)
        self.helper.hideTrans(self.normal_trans)
        self.helper.hideTrans(self.limit_trans )
        self.helper.hideTrans(self.max_trans )

        local cur_level = self:getTechLevel(cfg.id)
        local max_level = self:getTechMaxLevel(cfg.id)
        -- 满级
        local is_max = cur_level >= max_level
        if is_max then
            self.helper.showTrans(self.max_trans)
        end
        -- 条件限制
        local can_upgrade, need_cfg_id, need_level = self:isTechCanUpgrade(cfg.id)
        if not can_upgrade then
            self.helper.showTrans(self.limit_trans)
            local text = self:findText("tech_detail/limit_trans/text")
            text.text = string.format(NativeText.GetLangStr(400004), self.helper:getLangCfg(config.union_research_lists[need_cfg_id].name_id) .. " Lv." .. need_level)
        else
            -- 正常情况
            self.helper.showTrans(self.normal_trans)
        end

        local name_text =   self:findText("tech_detail/name_text")
        local exp_text =    self:findText("tech_detail/exp_text")
        local level_text =  self:findText("tech_detail/level_text")
        local curLv_text =  self:findText("tech_detail/curLv_text")
        local nextLv_text = self:findText("tech_detail/nextLv_text")
        local slider =      self:findImage("tech_detail/slider")
        local icon =        self:findImage("tech_detail/info/icon"):GetComponent(UnityEngine.UI.Image)
        local n_bg =        self:findImage("tech_detail/info/n_bg"):GetComponent(UnityEngine.UI.Image)
        local s_bg =        self:findImage("tech_detail/info/s_bg"):GetComponent(UnityEngine.UI.Image)


        local cur_exp, max_exp = self:getTechExp(cfg.id)
        if is_max then
            exp_text.text = string.format(NativeText.GetLangStr(400002), "<color=#ffffffff>MAX</color>")
            nextLv_text.text = "MAX"
        else
            exp_text.text = string.format(NativeText.GetLangStr(400002), cur_exp .. "/" .. max_exp) 
            nextLv_text.text = cur_level + 1
        end

        icon:SetSprite(cfg.path .. cfg.icon)
        n_bg.enabled = cfg.quality == 1
        s_bg.enabled = cfg.quality == 2
        curLv_text.text = cur_level
        slider.fillAmount = cur_exp / max_exp
        name_text.text = self.helper:getLangCfg(cfg.name_id)

        local getPropertyById = function(cfg, id)
            if cfg then
                local args = self.helper:getPropertyDes(cfg.effect)    
                for __, map in pairs(args) do
                    if map.id == id then
                        return map
                    end
                end
            end
        end

        local curr_cfg = self:getTechDetailCfg(cfg.id, cur_level)
        local next_cfg = self:getTechDetailCfg(cfg.id, cur_level + 1)
        local temp_cfg = next_cfg and next_cfg or curr_cfg
        local temp = temp_cfg.union_effect
        local temp_tb = loadstring("return " .. temp)()
        if temp_tb and #temp_tb == 0 then
            temp = temp_cfg.effect
        end
        local args = self.helper:getPropertyDes(temp)
        local layout_path = "tech_detail/effect"
        self.helper:hideAllChild(self, layout_path)
        local idx = 1
        for __, map in pairs(args) do
            local child = self.helper:getChild(self, idx, layout_path)
            child:SetActive(true)
            local name_text = child.transform:Find("name_text"):GetComponent(UnityEngine.UI.Text)
            local cur_text = child.transform:Find("cur_text"):GetComponent(UnityEngine.UI.Text)
            local next_text = child.transform:Find("next_text"):GetComponent(UnityEngine.UI.Text)
            -- name_text.text = map.name_str
            name_text.text = self.helper:getLangCfg(cfg.des_id)
            local cur_p = getPropertyById(curr_cfg, map.id)
            cur_text.text = cur_p and cur_p.num_str or "0"
            next_text.text = map.num_str
            idx = idx + 1
        end

        local btn = self:findButton("tech_detail/donateBtn")
        btn.interactable = self:isTechCanUpgrade(cfg.id)
        if btn.interactable then
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(
                function ()
                    if self.select_cost then
                        network_sender:familyTechDonate(self.select_cost.id, cfg.id)
                        self.select_index = nil
                        self.select_cost = nil
                    end
                end)
        end

        btn = self:findButton("tech_detail/info/icon")
        btn.onClick:RemoveAllListeners()
        btn.onClick:AddListener(
            function ()
                if self.recommend_techs[cfg.id] then
                    network_sender:setRecommendTech(0, cfg.id)
                else
                    network_sender:setRecommendTech(cfg.id, 0)
                end
            end
        )
    end,

    click_closeTechDetailBtn = function(self)
        self.tech_cfg = nil
        self.helper.hideTrans(self.detail_trans)
    end,
}
