local super = require("ui.frame")
local log = logger.get("Game", "Guild Logo")

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        self.msg = self.args.data.msg -- SCGetFamilyInfo

        clickEvents = 
        {
            "content/closeLanguage",
            "content/Window/confirmLang",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self.language_id = self.msg.familyInfo.settingInfo.familylanguage
        -- print("工会语言", self.language_id)
        self:initLanguageUI(self.language_id)
    end,

    onUnload = function (self)
    end,

    click_closeLanguage = function(self)
        frame_manager:hide('guild_language')
    end,

    click_confirmLang = function(self)
        print("发送语言id", self.cur_lang_cfg.id)
        network_sender:modifyFamilyInfoLanguage(self.cur_lang_cfg.id)
        frame_manager:hide('guild_language')
    end,

    -- 初始化语言选择界面
    initLanguageUI = function(self, language_id)
        local sort_list = {}
        config.union_langs:foreach(
            function(__, value)
                table.insert(sort_list, value)
            end)
        table.sort(sort_list, function(a, b)
            return a.id < b.id
        end)

        local parent_path = "content/Window/Scroll View/Viewport/Content/"
        for __, value in ipairs(sort_list) do
            local child = self.helper:getChild(self, __, parent_path)
            local button = child:GetComponent(UnityEngine.UI.Button)
            button.onClick:AddListener(
                function ()
                    self:selectLanguage(value)
                end
            )
            local des_text = child.transform:Find("Text"):GetComponent(UnityEngine.UI.Text)
            des_text.text = self.helper:getLangCfg(value.language)

            local select_bg = child.transform:Find("select_bg"):GetComponent(UnityEngine.UI.Image)
            local select_point = child.transform:Find("select_point"):GetComponent(UnityEngine.UI.Image)
            
            self.cache_language = self.cache_language or {}
            table.insert(self.cache_language, {cfg = value, trans = child, select_bg = select_bg, select_point = select_point})
        end

        self:selectLanguage(config.union_langs[language_id])
    end,

    -- 选择语言
    selectLanguage = function(self, cfg)
        self.cur_lang_cfg = cfg
        for __, item in ipairs(self.cache_language) do
            if item.cfg == cfg then
                item.select_bg.enabled = true
                item.select_point.enabled = true
            else
                item.select_bg.enabled = false
                item.select_point.enabled = false
            end
        end
        -- local text = self:findText("create/lang")
        -- text.text = self.helper:getLangCfg(cfg.language)
    end,
}
