local super = require("ui.frame")
local log = logger.get("Game", "Guild Logo")

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        -- self.game_state_island = self.args.owner
        self.msg = self.args.data.msg -- SCGetFamilyInfo

        clickEvents = 
        {
            "Top/confirmIcon",
            "Top/choose",
            "Top/closeLogo",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self:init()
    end,

    onUnload = function (self)
    end,

    init = function(self)
        local max_num = 6
        for i = 1, max_num do
            local path1 = "Top/layout1 (1)/Button (" .. i .. ")"
            local path2 = "Top/layout1 (2)/Button (" .. i .. ")"
            local child, path3 = self:getScorllRectChild(i)

            self.transform:Find(path1):SetActive(false)
            self.transform:Find(path2):SetActive(false)
            child:SetActive(false)
        end

        self.icon_list1 = self:getConfigByType(2)
        for __, l in ipairs(self.icon_list1) do
            if __ <= max_num then
                local path1 = "Top/layout1 (1)/Button (" .. __ .. ")"
                local trans = self.transform:Find(path1)
                trans:SetActive(true)
                local icon = trans:Find("Image"):GetComponent(UnityEngine.UI.Image)
                icon:SetSprite(l.path .. l.icon)
                self:clearScript("click", path1)
                self:setScript("click", path1, function() self:click_select_bg(__) end)
            end
        end

        self.icon_list2 = self:getConfigByType(3)
        for __, l in ipairs(self.icon_list2) do
            if __ <= max_num then
                 local path2 = "Top/layout1 (2)/Button (" .. __ .. ")"
                local trans = self.transform:Find(path2)
                trans:SetActive(true)
                local icon = trans:GetComponent(UnityEngine.UI.Image)
                icon:SetSprite(l.path .. l.icon)

                self:clearScript("click", path2)
                self:setScript("click", path2, function() self:click_select_color(__) end)
            end
        end

        self.icon_list3 = self:getConfigByType(1)
        for __, l in ipairs(self.icon_list3) do
            local child, path = self:getScorllRectChild(__)
            if child then
                child:SetActive(true)
                local icon = self:findImage(path .. "/icon")
                icon:SetSprite(l.path .. l.icon)

                self:clearScript("click", path)
                self:setScript("click", path, function() self:click_select_icon(__) end)
            end
        end

        local getIndex = function(list, id)
            for __, data in ipairs(list) do
                if data.id == id then
                    return __
                end
            end
        end

        local emblem = self.msg.familyInfo.settingInfo.emblem
        local iconFrame = getIndex(self.icon_list1, emblem.iconFrame)
        local grounding = getIndex(self.icon_list2, emblem.grounding)
        local icon = getIndex(self.icon_list3, emblem.icon)

        if iconFrame and grounding and icon then
            self:setIcon(iconFrame, grounding, icon)
        else
            log.error("有问题")
        end
    end,

    getConfigByType = function(self, type_id)
        local list = {}
        config.union_icons:foreach(function(idx, value)
            if value and value.type == type_id then
                table.insert(list, value)
            end
        end)
        return list
    end,

    click_select_bg = function(self, cfg_id)
        log.info("点击背景", cfg_id)
        self:setIcon(cfg_id, self.icon_idx2,self.icon_idx3)
    end,

    click_select_color = function(self, cfg_id)
        log.info("点击颜色", cfg_id)
        self:setIcon(self.icon_idx1, cfg_id, self.icon_idx3)
    end,

    click_select_icon = function(self, cfg_id)
        log.info("点击icon", cfg_id)
        self:setIcon(self.icon_idx1, self.icon_idx2,cfg_id)
    end,

    click_confirmIcon = function(self)
        log.info("确定", self.icon_idx1, self.icon_idx2, self.icon_idx3)
        frame_manager:hide('guild_logo')
        local emble_bg = self.icon_list1[self.icon_idx1].id
        local emble_color = self.icon_list2[self.icon_idx2].id 
        local emble_icon = self.icon_list3[self.icon_idx3].id 

        network_sender:modifyFamilyInfoLogo(emble_bg, emble_color, emble_icon)
    end,

    click_closeLogo = function(self)
        frame_manager:hide('guild_logo')
    end,

    click_choose = function(self)
        log.info("随机")
        self:setIcon(math.random(1, #self.icon_list1),math.random(1, #self.icon_list2),math.random(1, #self.icon_list3))
    end,

    setIcon = function(self, idx1, idx2, idx3)
        self.icon_idx1 = idx1
        self.icon_idx2 = idx2
        self.icon_idx3 = idx3

        local makeSprite = function(path, list, idx)
            local icon = self:findImage(path)
            local name = list[idx].path .. list[idx].icon
            icon:SetSprite(name)
        end

        makeSprite("Top/lastIcon/icon1", self.icon_list1, self.icon_idx1)
        makeSprite("Top/lastIcon/bg/icon2", self.icon_list2, self.icon_idx2)
        makeSprite("Top/lastIcon/bg/icon3", self.icon_list3, self.icon_idx3)
    end,

    -- 缓存layout,idx从1开始
    getScorllRectChild = function(self, idx)
        local path = "Top/Scroll View/Viewport/Content"
        local layout_trans = self.transform:Find(path)
        if not self.cache_layout then
            self.cache_layout = {}
            for i = 0, layout_trans.childCount - 1 do
                local child_trans = layout_trans:GetChild(i)
                child_trans.transform.name = i + 1
                table.insert(self.cache_layout, child_trans.transform)
            end
        end
        if idx > #self.cache_layout then
            for i = 1, idx - #self.cache_layout do
                local tplt = layout_trans:GetChild(0)
                local child_trans = Object.Instantiate(tplt.gameObject);
                child_trans.transform.name = i + #self.cache_layout
                child_trans:AttachTo(tplt.transform.parent)
                table.insert(self.cache_layout, child_trans.transform)
            end
        end
        self.cache_layout[idx].gameObject:SetActive(true)
        local full_path = path .. "/" .. self.cache_layout[idx].transform.name
        return self.cache_layout[idx], full_path
    end,
}
