local super = require("ui.frame")
local log = logger.get("Game", "Guild Logo")

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        self.game_state_island = self.args.owner
        self.msg = self.args.data.msg

        clickEvents = 
        {
            "closeBtn",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self.template = self.transform:Find("content/Scroll View/Viewport/Content/template")
        self.template:SetActive(false)

        self:initUI(self.msg)
    end,

    onUnload = function (self)
    end,

    initUI = function(self, msg)
        local cook_msg = self:cookMsg(msg)

        -- local idx = 1
        local parent_path = "content/Scroll View/Viewport/Content/"
        for idx, data in ipairs(cook_msg) do
            local child = self.helper:getChild(self, idx, parent_path)
            child.gameObject:SetActive(true)
            local child_path = parent_path .. child.transform.name .. "/"
            local day_text = self:findText(child_path .. "day_text")
            day_text.text = data.key
            
            local layout_path = child_path .. "layout"
            for ___, value in ipairs(data.value) do
                local cfg = config.union_logs[value.logId]
                local layout_child = self.helper:getChild(self, ___, layout_path)
                local combine_text = ""

                for ___, p in ipairs(value.args) do
                    if ___ ~= #value.args then
                        combine_text = combine_text  .. p .. ','
                    else
                        combine_text = combine_text  .. p
                    end
                end

                local time_text = layout_child.transform:Find("time_text"):GetComponent(UnityEngine.UI.Text)
                time_text.text = os.date("%H:%M", value.logTime) 
                local content_text = layout_child.transform:Find("content_text"):GetComponent(UnityEngine.UI.Text)
                content_text.text = string.format(self.helper:getLangCfg(cfg.lang_id), combine_text)
            end
            idx = 1 + idx
        end
    end,

    cookMsg = function(self, msg)
        local cook_msg = {}
        local sortTb = function(a, b)
            return a.logTime > b.logTime
        end
        table.sort(msg.logList, sortTb)

        contain = function(tab, key)
            for __, v in ipairs(tab) do
                if v.key == key then
                    return true
                end
            end
            return false
        end

        getValue = function(tab, key)
            for __, v in ipairs(tab) do
                if v.key == key then
                    return v.value
                end
            end
            return nil
        end

        for __, data in ipairs(msg.logList) do
            -- local key = os.date("%Y.%m.%d", data.logTime)
            -- if not cook_msg[key] then
            --     cook_msg[key] = {}
            -- end
            -- table.insert(cook_msg[key], data)

            local key = os.date("%Y.%m.%d", data.logTime)
            if not contain(cook_msg, key) then
                table.insert(cook_msg, { key = key, value = {}})
            end

            local v = getValue(cook_msg, key)
            table.insert(v, data)
        end
        return cook_msg
    end,

    click_closeBtn = function(self)
        frame_manager:hide('guild_logs')
    end,
}
