local log = logger.get("Game", "Guild Mass Panel")

return class
{
    ctor = function(self, panel, parent)
        self.panel = panel
        self.parent = parent

        self.prefab = self.panel:Find("Scroll View/Viewport/Content/Elem").gameObject
        self.content = self.panel:Find("Scroll View/Viewport/Content")
        
        self.mass_btn = self.panel:Find("Right/MassBtn"):GetComponent(UnityEngine.UI.Button)
        self.defend_btn = self.panel:Find("Right/DefendBtn"):GetComponent(UnityEngine.UI.Button)

        --self.parent:clearScript("click", "content/MassList/Right/MassBtn")
        --self.parent:setScript("click", "content/MassList/Right/MassBtn", functor.rmake(self.onClickMassBtn, self))

        self.is_show = false

        self.panel:SetActive(false)
    end,

    update = function(self, dt)
        if self.is_show then
            for k, v in pairs(self.mass_list) do
                if v.data.leftTime <= 0 then
                    --v.ui:SetActive(false)
                else
                    local lt = v.data.leftTime - dt
                    v.data.leftTime = lt > 0 and lt or 0
                    self:setCountDown(v.ui, v.data.leftTime, v.data.totaltime)
                end
            end
        end
    end,

    show = function(self, msg)
        self.panel:SetActive(true)

        self.mass_list = {}

        self:init(msg)

        self.is_show = true
    end,

    refresh = function(self)
        self.mass_list = {}

        self:init(msg)
    end,

    init = function(self, msg)
        self.msg = msg
        if not self.none_trans then
            self.none_trans = self.parent.transform:Find("content/MassList/none")
        end
        if self.msg and #self.msg > 0 then
            frame_helper.hideTrans(self.none_trans)
        else
            frame_helper.showTrans(self.none_trans)
        end
        
        for k, v in ipairs(msg) do
            local elem = nil

            if k > self.content.childCount then
                elem = Object.Instantiate(self.prefab).transform
                elem:SetParent(self.content)
                resetTransformZ(elem)
            else
                elem = self.content:GetChild(k-1)
            end

            if elem then
                elem:SetActive(true)
                elem.name = "child_"..k

                local place_node = elem:Find("Place_2")
                self:setPlace(v.dstPlace, place_node)

                local name = elem:Find("Place_1/Name"):GetComponent(UnityEngine.UI.Text)
                local icon = elem:Find("Place_1/Icon"):GetComponent(UnityEngine.UI.Image)
                local amount = elem:Find("Place_1/Amount/Value"):GetComponent(UnityEngine.UI.Text)
                local level = elem:Find("Place_2/Level"):GetComponent(UnityEngine.UI.Text)

                name.text = v.playerName
                local cfg = config.worldmap_base_configs[1]
                icon:SetSprite(cfg.iconpath..cfg.iconid)
                amount.text = string.format( "%d/%d",  v.soldierNum, v.maxSoldierCount)
                
                local is_monster = v.dstPlace.objType == common_pb.WORLD_OBJ_MONSTER

                if is_monster then
                    local cfg = config.worldmap_monsters[v.dstPlace.cfgId]
                    if cfg then
                        level.text = "Lv."..tostring(cfg.level)
                    else
                        log.error("######################## find monster cfg error ! id ", v.dstPlace.cfgId)
                    end
                else
                    level.text = "Lv."..tostring(v.dstPlace.level or 1)
                end

                --level.transform:SetActive(is_monster)

                local time_text = elem:Find("Center/Time"):GetComponent(UnityEngine.UI.Text)
                local des = elem:Find("Center/Text"):GetComponent(UnityEngine.UI.Text)
                local arrow = elem:Find("Center/Arrow")

                if v.status == 0 then
                    des.text = NativeText.GetLangStr(9000055)
                    time_text.color = Color(1, 80/255, 80/255, 1)
                else
                    des.text = NativeText.GetLangStr(9000056)
                    time_text.color = Color(11/255, 1, 120/255, 1)
                end

                arrow:SetActive(v.status ~= 0)

                local btn = elem:GetComponent(UnityEngine.UI.Button)
                btn.onClick:RemoveAllListeners()
                btn.onClick:AddListener(
                    function()
                        self:onClickMassView(v.massInstid)
                    end
                )

                self.mass_list[v.massInstid] = {data = v, ui = elem}
            end
        end

        for i=#msg, self.content.childCount - 1 do
            local elem = self.content:GetChild(i)
            elem:SetActive(false)
        end
    end,

    setPlace = function(self, info, ui)
        local name = ui:Find("Name"):GetComponent(UnityEngine.UI.Text)
        local icon = ui:Find("Icon"):GetComponent(UnityEngine.UI.Image)
        --local pos = ui:Find("Position"):GetComponent(UnityEngine.UI.Text)

        local cfg_info = self:getPlaceInfo(info)
        name.text = cfg_info.name
        icon:SetSprite(cfg_info.icon)
        --pos.text = gui_helper:positionFormat(info.posX, info.posY)
    end,

    getPlaceInfo = function(self, info)
        local tab = {}
        if info.objType == common_pb.WORLD_OBJ_PLAYER then
            local cfg = config.worldmap_base_configs[info.cfgId]
            tab.name = info.name
            if info.shortName and string.len(info.shortName) > 1 then
                tab.name = string.format( "[%s]%s", info.shortName, info.name )
            end
            tab.icon = cfg.iconpath..cfg.iconid
        elseif info.objType == common_pb.WORLD_OBJ_FAMILY then
            local cfg = config.camp_buildings[info.cfgId]
            tab.name = frame_helper:getLangCfg(cfg.name_id)
            tab.icon = cfg.iconpath..cfg.iconid
        elseif info.objType == common_pb.WORLD_OBJ_MONSTER then
            local cfg = config.worldmap_monsters[info.cfgId]
            tab.name = frame_helper:getLangCfg(cfg.monstername)
            tab.icon = cfg.iconheadpath..cfg.monsteheadricon
        end

        return tab
    end,

    setCountDown = function(self, elem, left_time, total_time)
        local slider = elem:Find("Center/FillBG/Fill"):GetComponent(UnityEngine.UI.Image)
        local time_text = elem:Find("Center/Time"):GetComponent(UnityEngine.UI.Text)

        slider.fillAmount = 1 - left_time / (total_time > 0 and total_time or 1)
        time_text.text = gui_helper:timeFormat(left_time)
    end,

    onClickMassView = function(self, id)
        network_sender:getMassInfo(id)
    end,

    getSoldierCount = function(self, info)
        local count = 0

        for _, team in ipairs(info.formationDetails) do
            for _, soldier in ipairs(team.soldiers) do
                count = count + soldier.count
            end
        end

        return count
    end,

    hide = function(self)
        self.panel:SetActive(false)
        self.is_show = false
    end,
}