local super = require("ui.frame")
local log = logger.get("Game", "Guild Members Frame")

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        self.msg = self.args.data.msg

        clickEvents = 
        {
            "Close",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self.prefab_root = self.transform:Find("Prefabs")
        self.prefab = self.prefab_root:GetChild(0).gameObject
        self.ranks = {}
        self.ui_members = {}
        self.info = nil
        self.rank_idx = 0
        self.guild_id = self.msg.familyId

        self:initRankLayout()
        self:addMembers()

        self.evts = {}
        self.evts["FAMILYMEMBERPROMOTE"] = game_director.events:on("FAMILYMEMBERPROMOTE", functor.make(self.onMemberPromote, self))
        self.evts["EXPELFAMILYMEMBER"] = game_director.events:on("EXPELFAMILYMEMBER", functor.make(self.onExpelMember, self))
    end,

    onUnload = function (self)
        for k, v in pairs(self.evts) do
            game_director.events:remove(k, v)
        end
    end,

    onMemberPromote = function(self, msg)
        if not self.ui_members[msg.memberData.playerid] then
            log.error("promote member ui not exist !", msg.memberData.playerid)
            return
        end
        local elem = self.ui_members[msg.memberData.playerid].ui
        local idx = self.ui_members[msg.memberData.playerid].index
        self.ranks[idx].childCount = self.ranks[idx].childCount - 1
        self:setRankUISize(idx)
        idx = self:getIndex(msg.memberData.degree)
        elem:SetParent(self.ranks[idx].members)
        self.ranks[idx].childCount = self.ranks[idx].childCount + 1
        self.ui_members[msg.memberData.playerid].index = idx
        self:setRankUISize(idx)
        local path = "Bg/Scroll View/Viewport/Content/"..self.ranks[idx].root.name.."/"..self.ranks[idx].members.name.."/"..elem.name
        self:setMemberInfo(elem, path, msg.memberData)
    end,

    onExpelMember = function(self, msg)
        if not self.ui_members[msg.playerId] then
            log.error("expel member ui not exist !", msg.playerId)
            return
        end
        self.ui_members[msg.playerId].ui:SetActive(false)
        local idx = self.ui_members[msg.playerId].index
        self.ranks[idx].childCount = self.ranks[idx].childCount - 1
        self:setRankUISize(idx)
    end,

    initRankLayout = function(self)
        local content = self.transform:Find("Bg/Scroll View/Viewport/Content")
        self.original_height = content:GetChild(0).sizeDelta.y
        for i=1, content.childCount do
            local elem = content:GetChild(i-1)
            local members = elem:Find("Members")
            local btn = elem:Find("Line/Arrow"):GetComponent(UnityEngine.UI.Button)
            members.localScale = Vector3.zero
            self:setScript("click", "Bg/Scroll View/Viewport/Content/"..elem.name.."/Line/Arrow", functor.rmake(self.onClickArrow, i))
            self.ranks[i] = {root = elem, members = members, arrow = btn, childCount = 0, display = false}
        end
    end,

    getIndex = function(self, degree)
        local index = 0
        if degree == common_pb.MD_LEADER then
            index = 1
        elseif degree == common_pb.MD_R4 then
            index = 2
        elseif degree == common_pb.MD_R3 then
            index = 3
        elseif degree == common_pb.MD_R2 then
            index = 4
        elseif degree == common_pb.MD_R1 then
            index = 5
        end
        return index
    end,

    addMembers = function(self)
        if self.msg.memberList and #self.msg.memberList > 0 then
            for i, member in ipairs(self.msg.memberList) do
                local elem = nil
                if self.prefab_root.childCount > 0 then
                    elem = self.prefab_root:GetChild(0)
                else
                    elem = Object.Instantiate(self.prefab).transform
                end
                if not elem then
                    return
                end
                local index = self:getIndex(member.degree)

                if self.ranks[index] then
                    elem.name = "member_"..i
                    elem:SetParent(self.ranks[index].members)
                    resetTransformZ(elem)
                    self.ranks[index].childCount = self.ranks[index].childCount + 1
                    local path = "Bg/Scroll View/Viewport/Content/"..self.ranks[index].root.name.."/"..self.ranks[index].members.name.."/"..elem.name
                    self:setMemberInfo(elem, path, member)
                    self.ui_members[member.playerid] = {}
                    self.ui_members[member.playerid].ui = elem
                    self.ui_members[member.playerid].index = index
                end

                if member.playerid == local_player.id then
                    self.info = member
                end
            end
            for i=1, 5 do
                self:onClickArrow(nil, i)
            end
        else
            log.error("##### guild member list error !")
        end
    end,

    setMemberInfo = function(self, elem, path, member)
        local icon = elem:Find("Icon"):GetComponent(UnityEngine.UI.Image)
        local name = elem:Find("Name"):GetComponent(UnityEngine.UI.Text)
        local fight = elem:Find("FightTile/Fight"):GetComponent(UnityEngine.UI.Text)
        local contribute = elem:Find("Text/Contribute"):GetComponent(UnityEngine.UI.Text)
        local online = elem:Find("OnLine"):GetComponent(UnityEngine.UI.Text)
        local level = elem:Find("Level"):GetComponent(UnityEngine.UI.Text)
        local operate_btn = elem:Find("OperateBtn")
        local btn_text = operate_btn:GetChild(0):GetComponent(UnityEngine.UI.Text)
        local info_btn = elem:Find("Info")
        local nick_name = elem:Find("Name/NickName"):GetComponent(UnityEngine.UI.Text)
        local player_icon = elem:Find("IconBG/PlayerIcon"):GetComponent(UnityEngine.UI.Image)

        player_icon:SetSprite(gui_helper:getPlayerIcon())
        icon:SetSprite(gui_helper:getGuildRankIcon(member.degree))
        name.text = member.playerName or ""
        nick_name.text = member.nickname or ""
        fight.text = tostring(member.playerSword)
        contribute.text = member.contribution
        local str = self:offlineTime(member.onlinetime)
        online.text = str
        if str == NativeText.GetLangStr(190091) then
            online.color = Color(6/255,249/255,155/255,255/255)
        else
            online.color = Color(1,1,1,1)
        end
        level.text = "Lv."..member.playerLevel
        self:clearScript("click", path.."/Info")
        self:clearScript("click", path.."/OperateBtn")
        self:setScript("click", path.."/Info", functor.rmake(self.onClickInfo, member.playerid))
        self:setScript("click", path.."/OperateBtn", functor.rmake(self.onClickOperate, member))

        operate_btn:SetActive(member.playerid ~= local_player.id and local_guild:isOwnGuild(self.guild_id))
        if local_guild:isOwnGuild(self.guild_id) then
            btn_text.text = NativeText.GetLangStr(190133)
        else
            btn_text.text = NativeText.GetLangStr(190134)
        end
    end,

    onClickArrow = function(self, handle, index)
        log.info("##### on click info ", index)
        self.rank_idx = index
        local rank = self.ranks[index]
        if rank.show then
            rank.show = false
            rank.members.localScale = Vector3.zero
            rank.arrow.targetGraphic.sprite = rank.arrow.spriteState.disabledSprite
        else
            rank.show = true
            rank.members.localScale = Vector3.one
            rank.arrow.targetGraphic.sprite = rank.arrow.spriteState.pressedSprite
        end

        self:setRankUISize(index)
    end,

    setRankUISize = function(self, index)
        local delta_height = self.original_height
        local rank = self.ranks[index]

        if not rank then
            log.error("set rank ui size error !")
            return
        end

        if rank.show then
            delta_height = delta_height + 230*rank.childCount
        end
        rank.root.sizeDelta = Vector2(rank.root.sizeDelta.x, delta_height)

        -- 刷新 content Size Fitter
        app:startCoroutine(
            function()
                rank.members:SetActive(false)
                coroutine.yield( nil )
                rank.members:SetActive(true)
            end
        )
    end,

    offlineTime = function(self, seconds)
        local t = math.floor(seconds / 3600 / 24 / 30)
        if t >= 1 then
            return t..NativeText.GetLangStr(190115)
        end

        local t = math.floor(seconds / 3600 / 24)
        if t >= 1 then
            return t..NativeText.GetLangStr(190088)
        end

        t = math.floor(seconds / 3600)
        if t >= 1 then
            return t..NativeText.GetLangStr(190089)
        end

        t = math.ceil(seconds / 60)
        if t >= 1 then
            return t..NativeText.GetLangStr(190090)
        end

        return NativeText.GetLangStr(190091)
    end,

    onClickOperate = function(self, handle, member)
        if not local_guild:isOwnGuild(self.guild_id) then
            -- 私聊
            print("#member.playerid:", member.playerid, member.playerName)
            local_chat:createPersonalChat(member.playerid, member.playerName)
            return
        end

        frame_manager:show("guild_member_operate", self, {info = member})
    end,

    onClickInfo = function(self, handle, id)
        log.info("##### on click info ", id)
        network_sender:getPlayerInfo(id)
    end,

    click_Close = function(self)
        frame_manager:hide('guild_members')
    end,

   
}
