local super = require("ui.frame")
local log = logger.get("Game", "Guild Members Frame")

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        self.msg = self.args.data.msg
        self.family_info = self.args.data.info

        clickEvents = 
        {
            "Close",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self.content = self.transform:Find("BG/Scroll View/Viewport/Content")
        self.prefab = self.content:GetChild(0).gameObject
        
        self.member_list = {}

        for k, v in ipairs(self.msg.memberList) do
            table.insert( self.member_list, v )
        end

        table.sort( self.member_list, function(a, b) return a.playerSword > b.playerSword end )

        self:loadMembers()
    end,

    onUnload = function (self)
        
    end,

    loadMembers = function(self)
        local own_rank = 0

        for i, member in ipairs(self.member_list) do
            local elem = nil
            if i <= self.content.childCount then
                elem = self.content:GetChild(i-1)
            else
                elem = Object.Instantiate(self.prefab).transform
                elem:SetParent(self.content)
                resetTransformZ(elem)
            end
            
            elem:SetActive(true)
            elem.name = "member_"..i
            
            local name = elem:Find("Name"):GetComponent(UnityEngine.UI.Text)
            local rank = elem:Find("Rank"):GetComponent(UnityEngine.UI.Text)
            local might = elem:Find("Might"):GetComponent(UnityEngine.UI.Text)
            local btn = elem:Find("Info"):GetComponent(UnityEngine.UI.Button)
            local img = elem:GetComponent(UnityEngine.UI.Image)

            name.text = member.playerName
            rank.text = tostring(i)
            might.text = self.helper:itemNumFormat(member.playerSword)

            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(
                function()
                    self:onClickInfo()
                end
            )

            img.enabled = member.playerid == local_player.id 

            if member.playerid == local_player.id then
                self.own_info = member
                own_rank = i
            end
        end

        for i=#self.member_list, self.content.childCount - 1 do
            local elem = self.content:GetChild(i)
            elem:SetActive(false)
        end

        local guild_name = self.transform:Find("BG/GuildName"):GetComponent(UnityEngine.UI.Text)
        local own_node = self.transform:Find("BG/OwnInfo")

        if self.own_info then
            local rank = own_node:Find("YourRank"):GetComponent(UnityEngine.UI.Text)
            local might = own_node:Find("YourMight"):GetComponent(UnityEngine.UI.Text)

            rank.text = string.format( "%s <color=#FFC30CFF>%d</color>", NativeText.GetLangStr(9000066), own_rank )
            might.text = string.format( "%s <color=#FFC30CFF>%s</color>", NativeText.GetLangStr(9000067), self.helper:itemNumFormat(self.own_info.playerSword) )
        else
            guild_name.text = string.format("[%s]%s", self.family_info.familycronym, self.family_info.familyName)
        end

        guild_name.transform:SetActive(self.own_info == nil)
        own_node:SetActive(self.own_info ~= nil)
    end,

    onClickInfo = function(self)
        
    end,

    click_Close = function(self)
        frame_manager:hide('guild_member_rank')
    end,

   
}
