local super = require("ui.frame")
local log = logger.get("Game", "Guild Logo")

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        self.game_state_island = self.args.owner
        -- MFamilyLeaveWord
        self.msg = self.args.data.msg

        clickEvents = 
        {
            "closeBtn",
            "content/sendNoteBtn",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self.input_field = self.transform:Find("content/InputField"):GetComponent(UnityEngine.UI.InputField)
        self.input_field.characterLimit = config.union_others[1].boardchatlen

        self.content = self.transform:Find("content/Scroll View/Viewport/Content")
        self.prefab = self.content:GetChild(0).gameObject

        self.word_list = self.msg.leaveWordList

        self:refreshUI(self.word_list)

        self.familyId = self.msg.familyId
    end,

    onUnload = function (self)
        network_sender:closeFamilyLeaveWord(self.familyId)
    end,

    click_closeBtn = function(self)
        frame_manager:hide('guild_notes')
    end,

    click_sendNoteBtn = function(self)
        if self.input_field.text == "" then
            return
        end
        local content = self.input_field.text
        network_sender:familyLeaveWord(self.familyId, content)
        self.input_field.text = ""
    end,

    addWordList = function(self, data)
        if self.word_list then
            table.insert(self.word_list, data)
            self:refreshUI(self.word_list)
        end
    end,

    refreshUI = function(self, word_list)
        for k, data in ipairs(word_list) do
            local child = nil

            if k > self.content.childCount then
                child = Object.Instantiate(self.prefab).transform
                child:SetParent(self.content)
                resetTransformZ(child)
            else
                child = self.content:GetChild(k-1)
            end

            child:SetActive(true)

            local name_text = child:Find("name_text"):GetComponent(UnityEngine.UI.Text)
            if data.familyAcronym and data.familyAcronym ~= "" then
                name_text.text = string.format('[%s]%s  %s', data.familyAcronym, data.familyName, data.playerName)
            else
                name_text.text = data.playerName
            end
            
            local content_text = child:Find("Text"):GetComponent(UnityEngine.UI.Text)
            content_text.text = data.leaveWord
            local time_text = child:Find("time_text"):GetComponent(UnityEngine.UI.Text)
            time_text.text = os.date("%Y-%m-%d %H:%M:%S", data.leaveWordTime)

            local function haveGuild(emblem)
                if not emblem then
                    return false
                end
                if emblem.iconFrame == 0 and emblem.iconFrame == 0 and emblem.iconFrame == 0 then
                    return false
                else
                    return true
                end
            end

            local rst = haveGuild(data.emblem)

            local icon_bg = child:Find('personIcon')
            local person_icon = child:Find('personIcon/Icon'):GetComponent(UnityEngine.UI.Image)
            local icon_bg = child:Find("guildIcon/icon1"):GetComponent(UnityEngine.UI.Image)
            local icon_color = child:Find("guildIcon/bg/icon2"):GetComponent(UnityEngine.UI.Image)
            local icon = child:Find("guildIcon/bg/icon3"):GetComponent(UnityEngine.UI.Image)
            local bg1 = child:Find("guildIcon/"):GetComponent(UnityEngine.UI.Image)
            local bg2 = child:Find("guildIcon/bg"):GetComponent(UnityEngine.UI.Image)

            icon_bg:SetActive(not rst)
            icon_bg.enabled = rst
            icon_color.enabled = rst
            icon.enabled = rst
            bg1.enabled = rst
            bg2.enabled = rst
            
            if rst then
                print("data.emblem.iconFrame", data.emblem.iconFrame)
                print("data.emblem.iconFrame", data.emblem.grounding)
                print("data.emblem.iconFrame", data.emblem.icon)
                icon_bg:SetSprite(config.union_icons[data.emblem.iconFrame].path .. config.union_icons[data.emblem.iconFrame].icon) 
                icon_color:SetSprite(config.union_icons[data.emblem.grounding].path .. config.union_icons[data.emblem.grounding].icon) 
                icon:SetSprite(config.union_icons[data.emblem.icon].path .. config.union_icons[data.emblem.icon].icon)
            else
                -- 设置个人头像
                person_icon:SetSprite(gui_helper:getPlayerIcon())
            end

            local btn = child:Find("guildIcon"):GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(
                function()
                    self:showInfo(data.playerId)
                end
            )

            btn = child:Find("personIcon"):GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(
                function()
                    self:showInfo(data.playerId)
                end
            )

            app:startCoroutine(
                function()
                    coroutine.yield(nil)
                    local height = content_text.transform.rect.height + 70
                    height = math.max(175, height)

                    child.sizeDelta = Vector2(1270, height)
                end
            )
        end

        for i=#word_list, self.content.childCount - 1 do
            local child = self.content:GetChild(i)
            child:SetActive(false)
        end

        app:startCoroutine(
            function()
                coroutine.yield(nil)
                local y = math.max(self.content.rect.height - 640, 0)
                self.content.localPosition = Vector3(self.content.localPosition.x, y, 0)
            end
        )
    end,

    showInfo = function(self, id)
        if id == local_player.id then
            frame_manager:show('lord_info')
        else
            network_sender:getPlayerInfo(id)
        end
    end,
}
