local super = require("ui.frame")
local log = logger.get("Game", "Guild Members Frame")

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        self.msg = self.args.data.msg

        clickEvents = 
        {
            "closeBtn",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self:init(self.msg.memberList)
    end,

    onUnload = function (self)
    end,

    init = function(self, members)
        local rst = self:cookMsg(members)
        local path = "bg/Scroll View/Viewport/Content/"
        self.helper:hideAllChild(self, path)
        for k, v in ipairs(rst) do
            local child = self.helper:getChild(self, k, path)
            child:SetActive(true)
            local title_text = child:Find("title"):GetComponent(UnityEngine.UI.Text)
            title_text.text = self:getRankDes(v.rank)
            local child_path = path .. child.name .. "/players"
            self.helper:hideAllChild(self, child_path)
            for index, member in ipairs(v.list) do
                local child = self.helper:getChild(self, index, child_path)
                child:SetActive(true)
                local icon =        child:Find("rankIcon"):GetComponent(UnityEngine.UI.Image)
                local name =        child:Find("Name"):GetComponent(UnityEngine.UI.Text)
                local fight =       child:Find("powerDes/power_text"):GetComponent(UnityEngine.UI.Text)
                local contri_text =       child:Find("contriDes/contri_text"):GetComponent(UnityEngine.UI.Text)
                local online =      child:Find("onlineState"):GetComponent(UnityEngine.UI.Text)
                local level =       child:Find("Level"):GetComponent(UnityEngine.UI.Text)
                local operate_btn = child:Find("transferBtn")
                local info_btn =    child:Find("infoBtn")
                local player_icon = child:Find("IconBG/Icon"):GetComponent(UnityEngine.UI.Image)
                player_icon:SetSprite(gui_helper:getPlayerIcon())
                icon:SetSprite(gui_helper:getGuildRankIcon(member.degree))
                name.text = member.playerName or ""
                fight.text = tostring(member.playerSword)
                contri_text.text = member.contribution
                local str = self:offlineTime(member.onlinetime)
                online.text = str
                online.color = str == NativeText.GetLangStr(190091) and Color(0,1,0,1) or Color(1,1,1,1)
                level.text = "Lv."..member.playerLevel
                self:clearScript("click",   child_path .. "/" .. child.name .."/infoBtn")
                self:clearScript("click",   child_path .. "/" .. child.name .."/transferBtn")
                self:setScript("click",     child_path .. "/" .. child.name .."/infoBtn", functor.rmake(self.click_infoBtn, member.playerid))
                self:setScript("click",     child_path .. "/" .. child.name .."/transferBtn", functor.rmake(self.click_transferBtn, member))
            end
        end
    end,

    getRankDes = function(self, degree)
        if degree == common_pb.MD_R4 then
            return "Rank 4"
        elseif degree == common_pb.MD_R3 then
            return "Rank 3"
        elseif degree == common_pb.MD_R2 then
            return "Rank 2"
        elseif degree == common_pb.MD_R1 then
            return "Rank 1"
        end
        return ""
    end,

    cookMsg = function(self, members)
        local rst = {}
        local getRankMbs = function(rank)
            for k, v in ipairs(rst) do
                if v.rank == rank then
                    return v.list
                end
            end
            local temp = {
                rank = rank,
                list = {}
            }
            table.insert(rst, temp)
            return temp.list
        end
        for __, mb in ipairs(members) do
            if mb.degree ~= common_pb.MD_LEADER then
                local list = getRankMbs(mb.degree)
                table.insert(list, mb)
            end
        end
        sortFunc = function(a, b)        
            return a.rank < b.rank
        end
        table.sort(rst, sortFunc)
        return rst
    end,

    offlineTime = function(self, seconds)
        local t = math.floor(seconds / 3600 / 24 / 30)
        if t >= 1 then
            return t..NativeText.GetLangStr(190115)
        end
        local t = math.floor(seconds / 3600 / 24)
        if t >= 1 then
            return t..NativeText.GetLangStr(190088)
        end
        t = math.floor(seconds / 3600)
        if t >= 1 then
            return t..NativeText.GetLangStr(190089)
        end
        t = math.ceil(seconds / 60)
        if t >= 1 then
            return t..NativeText.GetLangStr(190090)
        end
        return NativeText.GetLangStr(190091)
    end,

    checkPower = function(self, member)
        if self.info.degree < member.degree and self.info.degree < common_pb.MD_R4 then
            return true
        end
        return false
    end,

    click_transferBtn = function(self, handle, member)
        -- if not self:checkPower(member) then
        --     hint:setText(NativeText.GetLangStr(190092))
        --     return
        -- end
        -- self.selectMember = member
        -- self:showConfirmBox()

        local tb = {
            title = NativeText.GetLangStr(190170),
            content = string.format(NativeText.GetLangStr(190171), member.playerName),
            buttons =
            {
                {
                    btn_name = NativeText.GetLangStr(130003),
                    callback = function() 
                        frame_manager:hide('common_box')
                    end
                },
                {
                    btn_name = NativeText.GetLangStr(130002),
                    callback = function() 
                        frame_manager:hide('common_box')
                        frame_manager:hide('guild_transfer')
                        frame_manager:hide('guild_set')
                        network_sender:transferFamilyLeader(member.playerid)
                    end
                },
            },
        }

        frame_manager:show('common_box', nil, tb)
    end,

    click_infoBtn = function(self, handle, id)
        log.info("##### on click info ", id)
    end,

    click_closeBtn = function(self)
        frame_manager:hide('guild_transfer')
    end,
}
