local log = logger.get("Game", "equipment info panel")

return class{
    tabs = {
        prop = 1,
        talent = 2,
    },

    ctor = function(self, panel)
        self.panel = panel
        self.helper = require("gui.tool.frame_helper")

        self.prop_panel = self.panel:Find('BG/PropPanel')
        self.talent_panel = self.panel:Find('BG/TalentPanel')
        self.prop_btn = self.panel:Find('BG/Top/PropBtn'):GetComponent(UnityEngine.UI.Image)
        self.talent_btn = self.panel:Find('BG/Top/TalentBtn'):GetComponent(UnityEngine.UI.Image)
        self.fight_text = self.panel:Find('BG/Top/Fight/FightText'):GetComponent(UnityEngine.UI.Text)

        self.tab_index = self.tabs.prop

        self:hide()
    end,

    show = function(self, hero_inst)
        self.helper.showTrans(self.panel)
        self.hero = hero_inst
        self:setFightData()
        self:changeTab(self.tab_index)
    end,

    changeTab = function(self, idx)
        if idx == self.tabs.prop then
            log.info('change tab ',idx)
            self.helper.showTrans(self.prop_panel)
            self.helper.hideTrans(self.talent_panel)
            self:setPropData()
        elseif idx == self.tabs.talent then
            log.info('change tab ',idx)
            self.helper.showTrans(self.talent_panel)
            self.helper.hideTrans(self.prop_panel)
            self:setTalentData()
        end
        self.tab_index = idx
        self:setButtonState()
    end,

    setButtonState = function(self)
        local normal_path = 'Assets/Arts/ui/atlas/common/button_03'..'.png'
        local light_path = 'Assets/Arts/ui/atlas/common/button_03_01'..'.png'
        
        if self.tab_index == self.tabs.prop then
            self.prop_btn:SetSprite(light_path)
            self.talent_btn:SetSprite(normal_path)
        elseif self.tab_index == self.tabs.talent then
            self.prop_btn:SetSprite(normal_path)
            self.talent_btn:SetSprite(light_path)
        end
    end,

    setFightData = function(self)
        self.fight_text.text = self.hero.strength
    end,

    setPropData = function(self)
        local node = self.prop_panel:Find('Top/Props')
        local props = self.hero:getHeroShowedProps()
        gui_helper:showUpgradeProps(node, props)
    end,

    setTalentData = function(self)
        local officer = self.hero.officer
        local cur_talent = self.talent_panel:Find("Top/Props")
        local skill_prop = officer:getAwakeCfg(officer.star_level, officer.awake_level).skill
        gui_helper:setOfficerSkillProps(cur_talent, skill_prop, nil, true)

        local next_talent_panel = self.talent_panel:Find('Bottom')
        next_talent_panel:SetActive(officer.star_level ~= 6)

        if officer.star_level ~= 6 then
            local next_talent = self.talent_panel:Find("Bottom/Props")
            local skill_prop = officer:getAwakeCfg(officer.star_level+1, officer.awake_level).skill
            gui_helper:setOfficerSkillProps(next_talent, skill_prop, nil, true)
        end
    end,

    hide = function(self)
        self.helper.hideTrans(self.panel)
    end,

}