local log = logger.get("Game", "hero info panel")

local slot_types = {
    main_weapon = 1,
    minor_weapon = 2,
    decoration = 3,
    wheel = 4,
    fashion = 5,
}

local pages = {
    weapon  =1, 
    prop = 2,
}

return class{
    ctor = function(self, panel, owner)
        self.owner = owner
        self.panel = panel
        self.helper = require("gui.tool.frame_helper")
        
        self.weapon_node = self.panel:Find("Info")
        self.prop_node = self.panel:Find("Props")

        self.tank_name = self.weapon_node:Find("Name"):GetComponent(UnityEngine.UI.Text)
        self.prop_tank_name = self.prop_node:Find("Viewport/Content/TypeBG/Type/Name"):GetComponent(UnityEngine.UI.Text)
        self.prop_type = self.prop_node:Find("Viewport/Content/TypeBG/Type"):GetComponent(UnityEngine.UI.Image)

        self.skill_1 = self.prop_node:Find("Viewport/Content/Skill_1")
        self.skill_2 = self.prop_node:Find("Viewport/Content/Skill_2")

        self.nodes = {}
        self.nodes[pages.weapon] = self.weapon_node
        self.nodes[pages.prop] = self.prop_node

        self.btns = {}
        self.btns[pages.weapon] = self.panel:Find("WeaponBtn"):GetComponent(UnityEngine.UI.Button)
        self.btns[pages.prop] = self.panel:Find("PropBtn"):GetComponent(UnityEngine.UI.Button)

        local hero_weapons_panel = self.panel:Find('Info/Weapons')
        self.hero_weapons = require('gui.hero_weapons_panel')(hero_weapons_panel, self.owner)

        self.page_idx = pages.weapon
    end,

    show = function(self, hero_id)
        self.hero = hero_manager:getHero(hero_id)
        
        self.helper.showTrans(self.panel)

        if not self.hero then
            log.error("##### show develop panel hero not find ! hero_id", hero_id)
            return
        end

        self:changeTab(self.page_idx)
    end,

    clickPropBtn = function(self)
        self:changeTab(pages.prop)
    end,

    clickWeaponBtn = function(self)
        self:changeTab(pages.weapon)
    end,

    changeTab = function(self, idx)
        if idx == pages.weapon then
            self:refreshWeaponNode()
        elseif idx == pages.prop then
            self:refreshPropNode()
        end

        for k, v in pairs(self.nodes) do
            v:SetActive(k == idx)
        end

        self.page_idx = idx
        self:setButtonState()
    end,

    setButtonState = function(self)
        for k, v in pairs(self.btns) do
            if k ==  self.page_idx then
                v.targetGraphic.sprite = v.spriteState.pressedSprite
            else
                v.targetGraphic.sprite = v.spriteState.disabledSprite
            end
            v.transform:GetComponent(UnityEngine.UI.Image):SetNativeSize()
        end
    end,

    refreshWeaponNode = function(self)
        self.tank_name.text = self.helper:getLangCfg(self.hero.tank_cfg.name)

        local type_icon = self.weapon_node:Find("TypeIcon"):GetComponent(UnityEngine.UI.Image)
        type_icon:SetSprite("Assets/Arts/ui/atlas/icon/vehicle/"..self.hero.tank_cfg.type..".png")

        local path = "BG/Right/TankInfo/Info/Weapons/"
        self.hero_weapons:show(self.hero.id, path)
    end,

    refreshPropNode = function(self)
        self:setFightProps()
        self:setBaseProps()
        self:setSkill()
    end,

    setProps = function(self, props, node)
        local prefab = self.prop_node:Find("Prop").gameObject

        local childs = {}
        local idx = 1

        for i=0, node.childCount - 1 do
            local child = node:GetChild(i)
            if child.name == "Prop_"..idx then
                child:SetActive(false)
                table.insert( childs, child )
                idx = idx + 1
            end
        end

        local prop_list = {}
        local num = 0
        idx = 0

        for k, v in ipairs(props) do
            if v.PropId ~= common_pb.PARTNER_PROP_HP and v.PropId ~= common_pb.PARTNER_PROP_MAX_HP then
                if idx % 2 == 0 then
                    num = num + 1
                    prop_list[num] = {}
                    table.insert( prop_list[num], v )
                else
                    table.insert( prop_list[num], v )
                end
                idx = idx + 1
            end
        end

        for k, v in ipairs(prop_list) do
            local child = nil

            if k <= #childs then
                child = childs[k]
            else
                child = Object.Instantiate(prefab).transform
                child:SetParent(node)
                resetTransformZ(child)
            end

            if child then
                child.name = "Prop_"..k
                child:SetActive(true)

                for i=0, child.childCount-1 do
                    child:GetChild(i):SetActive(false)
                end

                local setProp = function(index, prop)
                    local prop_ui = child:GetChild(index-1)
                    local name = prop_ui:GetComponent(UnityEngine.UI.Text)
                    local value = prop_ui:Find("Value"):GetComponent(UnityEngine.UI.Text)

                    prop_ui:SetActive(true)

                    local prop_cfg = gui_helper:getPropCfg(prop.PropId)
                    name.text = frame_helper:getLangCfg(prop_cfg.name_id)

                    if prop_cfg.isfloat > 0 then
                        value.text = tostring(prop.PropValue * 0.0001)
                    else
                        value.text = tostring(prop.PropValue)
                    end

                    if prop_cfg.showpercent > 0 then
                        value.text = tonumber(value.text)*100 .."%"
                    end
                end
                
                for k1, v1 in ipairs(v) do
                    setProp(k1, v1)
                end
            end
        end
    end,

    setFightProps = function(self)
        local node = self.prop_node:Find("Viewport/Content/FightProps")
        local hp_name = node:Find("PropHP/Name"):GetComponent(UnityEngine.UI.Text)
        local hp_value = node:Find("PropHP/Name/Value"):GetComponent(UnityEngine.UI.Text)
        local repair_btn = node:Find("PropHP/Name/Repair"):GetComponent(UnityEngine.UI.Button)

        local prop_cfg = gui_helper:getPropCfg(common_pb.PARTNER_PROP_HP)
        local hp = self.hero:getHeroPorp(common_pb.PARTNER_PROP_HP)
        local max_hp = self.hero:getHeroPorp(common_pb.PARTNER_PROP_MAX_HP)

        hp_name.text = frame_helper:getLangCfg(prop_cfg.name_id)
        hp_value.text = string.format("%d  (%.1f%%)", hp.PropValue, hp.PropValue/max_hp.PropValue*100)

        local props = self:getProps(1)

        self:setProps(props, node)

        local type_icon = self.prop_node:Find("Viewport/Content/TypeBG/Type"):GetComponent(UnityEngine.UI.Image)
        local name = type_icon.transform:Find("Name"):GetComponent(UnityEngine.UI.Text)
        type_icon:SetSprite("Assets/Arts/ui/atlas/icon/vehicle/"..self.hero.tank_cfg.type..".png")
        name.text = self.helper:getLangCfg(self.hero.tank_cfg.name)

        repair_btn.onClick:RemoveAllListeners()
        repair_btn.onClick:AddListener(
            function()
                self:onClickReapirBtn()
            end
        )
    end,

    onClickReapirBtn = function(self)
        if gui_helper:heroIsWeakness(self.hero.id) then
            return
        end

        local cnt = bag_manager:getItemNum(40005)
        if cnt == 0 then
            hint:setText(NativeText.GetLangStr(360002))
            return
        end
        network_sender:getHeroNeedFixInfo({self.hero.id})
    end,

    setBaseProps = function(self)
        local node = self.prop_node:Find("Viewport/Content/OtherProps")
        local props = self:getProps(0)

        self:setProps(props, node)
    end,

    setSkill = function(self)
        local weapon = self.hero:getEquipItem(common_pb.MAIN_WEAPON_SLOT)

        if weapon then
            local sillInfo = function(id, ui)
                local cfg = config.skill_skills[id]

                if cfg then
                    ui:SetActive(true)
                    local name = ui:Find("Name"):GetComponent(UnityEngine.UI.Text)
                    local level = ui:Find("Level"):GetComponent(UnityEngine.UI.Text)
                    local des = ui:Find("Des"):GetComponent(UnityEngine.UI.Text)
                    local max_tip = ui:Find("Max")

                    name.text = self.helper:getLangCfg(cfg.skill_name)
                    des.text = gui_helper:getSkillDes(id)

                    max_tip:SetActive(false)
                    level.transform:SetActive(false)
                else
                    ui:SetActive(false)
                end
            end

            sillInfo(weapon.cfg.skill_id, self.skill_1)
            sillInfo(weapon.cfg.opening_skill, self.skill_2)
        else
            self.skill_1:SetActive(false)
            self.skill_2:SetActive(false)
        end
    end,

    getProps = function(self, isbattle)
        local props = {}
        local show_props = self.hero:getHeroShowedProps()

        for k, v in ipairs(show_props) do
            local cfg = config.attribute_config_attrs[v.PropId]

            if cfg then
                if cfg.isbattle == isbattle then
                    table.insert( props, v )
                end
            end
        end

        return props
    end,

    hide = function(self)
        self.helper.hideTrans(self.panel)
    end,
}