local super = require("ui.frame")
local log = logger.get("Game", "Hero List Frame")

return class
{
    super = super,

    filters = {
        quality = 1,
        level = 2,
        stars = 3,
    },

    onLoaded = function (self)
        super.onLoaded(self)

        -- 路径: root/login
        -- 事件: click_login 
        clickEvents = {
            "BG/Title/BackBtn",
            "BG/Title/SortBG",
            "BG/Title/SortBG/List/Page_1",
            "BG/Title/SortBG/List/Page_2",
            "BG/Title/SortNode/SelectBtn",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self.scroll_content = self.transform:Find('BG/Scroll View/Viewport/Content')
        self.hero_prefab = self.scroll_content:GetChild(0).gameObject

        self.sort_bg = self.transform:Find("BG/Title/SortBG")
        self.sort_bg.localScale = Vector3.zero

        self.drop_text = self.transform:Find("BG/Title/SortNode/Name"):GetComponent(UnityEngine.UI.Text)

        local drop_1 = self.transform:Find('BG/Title/SortBG/List/Page_1')
        local drop_2 = self.transform:Find('BG/Title/SortBG/List/Page_2')

        self.drop_down = {}
        self.drop_down[1] = {idx = self.filters.level, ui = drop_1}
        self.drop_down[2] = {idx = self.filters.stars, ui = drop_2}

        self.strs = {}
        self.strs[self.filters.quality] = "Quality"
        self.strs[self.filters.level] = "Level"
        self.strs[self.filters.stars] = "Star"


        --缓存点击事件
        self.hero_events = {}
        self.active_count = 0
        self.total_count = 0
        self.heros = {}
        self.filter = 0
        self:heroFilter(self.filters.quality)
        self:setCurrDropText(self.filters.quality)

        network:registerMessageHandler(self)

        --self.prop_changed_id = hero_manager.events:on("PARTNERPROPCHANGE", functor.make(self.onPropChanged, self))
        self.refresh_id = hero_manager.events:on("HEROREFRESH", functor.make(self.onHeroRefresh, self))
    end,

    onUnload = function (self)
        network:unregisterMessageHandler(self)
        --hero_manager.events:remove("PARTNERPROPCHANGE", self.prop_changed_id)
        hero_manager.events:remove("HEROREFRESH", self.refresh_id)
    end,

    onHeroRefresh = function(self)
        self:heroFilter(self.filter, true)
    end,

    onPropChanged = function(self, msg)
        --self:heroFilter(self.filter, true)
        local getHero = function(id)
            for k, v in ipairs(self.heros) do
                if v.id == id then
                    return v
                end
            end
        end
        local hero = getHero(msg.heroId)
        if hero then
            if hero.hero_elem then
                self:setGotHeroElem(hero.hero_elem.transform, hero)
            end
        end
    end,

    -- 网络消息 
    onMessage = function(self, cmd, msg)
        if cmd == protocol_pb.RECRUITHERO then
            if msg.respCode == 0 then
                -- self:heroFilter(self.filter, true)
                -- frame_manager:show('commander_get', self, { tank_cfg_id = msg.heroId })
            end
        end
    end,

    onRcvMsg = function(self, cmd, msg)
    end,

    update = function(self)
       
    end,

    clearHeroEvents = function(self)
        for path, evt in pairs(self.hero_events) do
            self:clearScript(evt, path)
        end
        self.hero_events = {}
    end,

    --战车排序过滤
    heroFilter = function(self, filter, force)
        if self.filter == filter and not force then
            return
        end

        self.heros = self.helper:deepCopy(hero_manager.heros) 
        for __, hero in pairs(self.heros) do
            local command_cfg_id = self:getCommandCfgId(hero)
            local need_piece = config.commander_basics[command_cfg_id].recruitneed
            local curr_piece = bag_manager:getItemNum(config.commander_basics[command_cfg_id].piece_id)

            if not hero.is_actived then
                hero.piece_full = curr_piece >= need_piece
            else
                hero.piece_full = false
            end
        end

        if filter == self.filters.quality then
            self.heros = self:getHerosSort(self.heros, self.compHeroByOfficerColor)
        elseif filter == self.filters.stars then
            self.heros = self:getHerosSort(self.heros, self.compHeroByOfficerStar)
        elseif filter == self.filters.level then
            self.heros = self:getHerosSort(self.heros, self.compHeroByOfficerLevel)
        end

        --[[ local remove_idxs = {}
        for k, v in ipairs(self.heros) do
            if v.officer.cfg.isshow == 0 then
                table.insert( remove_idxs, k )
            end
        end ]]

        for i=#self.heros, 1, -1 do
            local hero = self.heros[i]
            if hero.officer.cfg.isshow == 0 and not hero.is_actived then
                table.remove( self.heros, i )
            end
        end

        self.active_count = 0
        self.total_count = 0
        self.filter = filter
        self:refreshList()
    end,

    getHerosSort = function(self, heros, comp)
        local tab = {}
        for _,v in pairs(heros) do
            table.insert(tab, v)
        end

        table.sort(tab, comp)

        return tab
    end,

    -- 根据等级比较两个 hero
    compHeroByOfficerLevel = function(a, b)
        if a.piece_full and b.piece_full then
            return a.officer.cfg.color > b.officer.cfg.color
        end
        if a.piece_full and ( not b.piece_full ) then
            return true
        end
        if ( not a.piece_full ) and b.piece_full then
            return false
        end

        if ( not a.piece_full ) and ( not b.piece_full ) then
            if not a.is_actived and not b.is_actived then
                return a.officer.cfg.color > b.officer.cfg.color
            end

            if a.is_actived and ( not b.is_actived ) then
                return true
            end

            if ( not a.is_actived ) and b.is_actived then
                return false
            end

            if a.officer.level == b.officer.level then
                if a.officer.cfg.color == b.officer.cfg.color then
                    return a.officer.star_level > b.officer.star_level
                end

                return a.officer.cfg.color > b.officer.cfg.color
            end

            return a.officer.level > b.officer.level  
        end
    end,

    -- 根据 star 比较两个 hero 
    compHeroByOfficerStar = function(a, b)
        if a.piece_full and b.piece_full then
            return a.officer.cfg.color > b.officer.cfg.color
        end
        if a.piece_full and ( not b.piece_full ) then
            return true
        end
        if ( not a.piece_full ) and b.piece_full then
            return false
        end

        if ( not a.piece_full ) and ( not b.piece_full ) then
            if not a.is_actived and not b.is_actived then
                return a.officer.cfg.color > b.officer.cfg.color
            end

            if a.is_actived and ( not b.is_actived ) then
                return true
            end

            if ( not a.is_actived ) and b.is_actived then
                return false
            end

            if a.officer.star_level == b.officer.star_level then
                if a.officer.cfg.color == b.officer.cfg.color then
                    return a.officer.level > b.officer.level
                end

                return a.officer.cfg.color > b.officer.cfg.color
            end

            return a.officer.star_level > b.officer.star_level  
        end
    end,

    -- 根据战斗力比较两个 hero
    compHeroByBattleForce = function(a, b)
        if a.piece_full and b.piece_full then
            return a.officer.cfg.color > b.officer.cfg.color
        end
        if a.piece_full and ( not b.piece_full ) then
            return true
        end
        if ( not a.piece_full ) and b.piece_full then
            return false
        end

        if ( not a.piece_full ) and ( not b.piece_full ) then
            if not a.is_actived and not b.is_actived then
                return a.officer.cfg.color > b.officer.cfg.color
            end

            if a.is_actived and ( not b.is_actived ) then
                return true
            end

            if ( not a.is_actived ) and b.is_actived then
                return false
            end

            if a.strength == b.strength then
                if a.officer.cfg.color == b.officer.cfg.color then
                    if a.officer.star_level == b.officer.star_level then
                        return a.officer.level > b.officer.level
                    end

                    return a.officer.star_level > b.officer.star_level
                end

                return a.officer.cfg.color > b.officer.cfg.color
            end

            return a.strength > b.strength  
        end
    end,

    -- 根据指挥官品质比较两个 hero
    compHeroByOfficerColor = function(a, b)
        if a.piece_full and b.piece_full then
            return a.officer.cfg.color > b.officer.cfg.color
        end
        if a.piece_full and ( not b.piece_full ) then
            return true
        end
        if ( not a.piece_full ) and b.piece_full then
            return false
        end

        if ( not a.piece_full ) and ( not b.piece_full ) then
            if not a.is_actived and not b.is_actived then
                return a.officer.cfg.color > b.officer.cfg.color
            end

            if a.is_actived and ( not b.is_actived ) then
                return true
            end

            if ( not a.is_actived ) and b.is_actived then
                return false
            end

            if a.officer.cfg.color == b.officer.cfg.color then
                if a.officer.star_level == b.officer.star_level then
                    return a.officer.level > b.officer.level
                end

                return a.officer.star_level > b.officer.star_level
            end

            return a.officer.cfg.color > b.officer.cfg.color
        end
    end,

    refreshList = function(self)
        self:clearHeroEvents()

        for k, v in ipairs(self.heros) do
            local hero_elem = nil
            if k > self.scroll_content.childCount then
                hero_elem = Object.Instantiate(self.hero_prefab)
                hero_elem.transform:SetParent(self.scroll_content)
                resetTransformZ(hero_elem.transform)
            else
                hero_elem = self.scroll_content:GetChild(k-1)
            end
            
            if not hero_elem then
                return
            end

            hero_elem.transform:SetActive(true)

            if v.is_actived then
                self:setGotHeroElem(hero_elem.transform, v)
            else
                self:setUnGetHeroElem(hero_elem.transform, v)
            end

            v.hero_elem = hero_elem -- 将 界面元素缓存到 hero 中
        end

        for i=#self.heros, self.scroll_content.childCount-1 do
            self.scroll_content:GetChild(i):SetActive(false)
        end

        self:setHeroAmount()
    end,

    --设置战车数量
    setHeroAmount = function(self)
        local count = self.transform:Find("BG/Vehicles/Amount"):GetComponent(UnityEngine.UI.Text)
        --local max = local_player:getMaxHeroCnt()
        count.text = string.format("%s <color=#0BFF78FF>%d</color>/%d", NativeText.GetLangStr(9000068), self.active_count, self.total_count)
    end,

    getOfficerColorBG = function(self, color)
        local path = res_path.res_path_104
        local path_1 = res_path.res_path_107

        if color == 4 then
            path = res_path.res_path_103
            path_1 = res_path.res_path_106
        elseif color == 5 then
            path = res_path.res_path_105
            path_1 = res_path.res_path_108
        end

        return path, path_1
    end,

    --设置已获得的战车
    setGotHeroElem = function(self, elem, hero)
        local bg = elem:GetComponent(UnityEngine.UI.Image)
        local bg_2 = elem:Find("ColorImg"):GetComponent(UnityEngine.UI.Image)
        local tank_icon = elem:Find('TankImg'):GetComponent(UnityEngine.UI.Image)
        local officer_icon = elem:Find('OfficerImg'):GetComponent(UnityEngine.UI.Image)
        local officer_name = elem:Find('NameNode/OfficerName'):GetComponent(UnityEngine.UI.Text)
        local tank_name = elem:Find('NameNode/TankName'):GetComponent(UnityEngine.UI.Text)
        local hero_star = elem:Find('LevelBG/Stars')
        local hero_level = elem:Find("LevelBG/Level"):GetComponent(UnityEngine.UI.Text)
        local piece_cnt = elem:Find("PieceNode/Count"):GetComponent(UnityEngine.UI.Text)
        local lock_img = elem:Find("Locked")
        local red_point = elem:Find("RedPoint")

        lock_img.localScale = Vector3.zero

        local hero_inst = hero_manager:getHero(hero.id)

        --设置战车名字
        officer_name.text = hero_inst.officer.name
        --officer_name.color = gui_helper:quality2UnityColor(hero_inst.officer.cfg.color)
        tank_name.text = self.helper:getLangCfg(hero_inst.tank_cfg.name)

        hero_level.text = hero_inst.officer.level

        gui_helper:setOfficerStars(hero_inst.officer.star_level, hero_star)

        hero_level.transform.localScale = Vector3.one
        hero_star.localScale = Vector3.one

        tank_icon:SetSprite(hero_inst.tank_cfg.bannercar_path..hero_inst.tank_cfg.bannercar_pic)
        
        officer_icon:SetSprite(hero_inst.officer.cfg.bannerhead_path..hero_inst.officer.cfg.bannerhead_pic)

        local path, path_1 = self:getOfficerColorBG(hero_inst.officer.cfg.color)
        bg:SetSprite(path_1)
        bg_2:SetSprite(path)
        
        self.helper:setImageGray1(tank_icon, false)
        self.helper:setImageGray1(officer_icon, false)

        log.info("(已获得英雄)碎片id:", hero_inst.officer.cfg.piece_id)

        local curr_piece = self.helper:getItemNum(hero_inst.officer.cfg.piece_id)
        local need_piece = hero_inst.officer:getStarUpPiece()

        piece_cnt.text = curr_piece.."/"..need_piece

        if curr_piece >= need_piece then
            red_point.localScale = Vector3.one
        else
            red_point.localScale = Vector3.zero
        end

        --注册点击事件
        elem.name = "Hero_"..hero_inst.id
        
        local cb = function(id)
            self:onClick_Hero(id)
        end

        local btn = elem:GetComponent(UnityEngine.UI.Button)
        btn.onClick:RemoveAllListeners()
        btn.onClick:AddListener(functor.rmake(cb, hero_inst.id))

        self.active_count = self.active_count + 1
        self.total_count = self.total_count + 1
    end,

    --设置未获得的战车
    setUnGetHeroElem = function(self, elem, hero)
        local bg = elem:GetComponent(UnityEngine.UI.Image)
        local bg_2 = elem:Find("ColorImg"):GetComponent(UnityEngine.UI.Image)
        local tank_icon = elem:Find('TankImg'):GetComponent(UnityEngine.UI.Image)
        local officer_icon = elem:Find('OfficerImg'):GetComponent(UnityEngine.UI.Image)
        local officer_name = elem:Find('NameNode/OfficerName'):GetComponent(UnityEngine.UI.Text)
        local tank_name = elem:Find('NameNode/TankName'):GetComponent(UnityEngine.UI.Text)
        local hero_star = elem:Find('LevelBG/Stars')
        local hero_level = elem:Find("LevelBG/Level"):GetComponent(UnityEngine.UI.Text)
        local piece_cnt = elem:Find("PieceNode/Count"):GetComponent(UnityEngine.UI.Text)
        local lock_img = elem:Find("Locked")
        local red_point = elem:Find("RedPoint")

        lock_img.localScale = Vector3.one

        local hero_inst = hero_manager:getHero(hero.id)

        --设置战车名字
        officer_name.text = hero_inst.officer.name
        --officer_name.color = gui_helper:quality2UnityColor(hero_inst.officer.cfg.color)
        tank_name.text = self.helper:getLangCfg(hero_inst.tank_cfg.name)

        --hero_level.text = hero_inst.officer.level

        --gui_helper:setOfficerStars(hero_inst.officer.star_level, hero_star)

        hero_level.transform.localScale = Vector3.zero
        hero_star.localScale = Vector3.zero

        tank_icon:SetSprite(hero_inst.tank_cfg.bannercar_path..hero_inst.tank_cfg.bannercar_pic)
        
        officer_icon:SetSprite(hero_inst.officer.cfg.bannerhead_path..hero_inst.officer.cfg.bannerhead_pic)

        local path, path_1 = self:getOfficerColorBG(hero_inst.officer.cfg.color)
        bg:SetSprite(path_1)
        bg_2:SetSprite(path)
        
        self.helper:setImageGray1(tank_icon, true)
        self.helper:setImageGray1(officer_icon, true)

        log.info("(没有获得英雄)碎片id:", hero_inst.officer.cfg.piece_id)

        local curr_piece = self.helper:getItemNum(hero_inst.officer.cfg.piece_id)
        local need_piece = hero_inst.officer.cfg.recruitneed

        piece_cnt.text = curr_piece.."/"..need_piece

        --注册点击事件
        elem.name = "Hero_"..hero_inst.id

        local btn = elem:GetComponent(UnityEngine.UI.Button)
        btn.onClick:RemoveAllListeners()

        if curr_piece >= need_piece then
            local cb = function(id)
                self:click_getHero(id)
            end
            btn.onClick:AddListener(functor.rmake(cb, hero_inst.id))

            red_point.localScale = Vector3.one
        else
            red_point.localScale = Vector3.zero
        end

        self.total_count = self.total_count + 1
    end,

    -- 这里 tank_cfg_id 要改成 hero_id
    getCommandCfgId = function(self, hero)
        if hero.officer then
            return hero.officer.cfg_id
        end

        return nil
    end,

    setCurrDropText = function(self, idx)
        self.drop_text.text = self.strs[idx]
    end,

    updateFilterBtns = function(self, idx)
        local filter = self.drop_down[idx].idx
        self:setCurrDropText(filter)
        self:heroFilter(filter)

        if filter == self.filters.quality then
            self.drop_down[1].idx = self.filters.level
            self.drop_down[2].idx = self.filters.stars
        elseif filter == self.filters.level then
            self.drop_down[1].idx = self.filters.stars
            self.drop_down[2].idx = self.filters.quality
        elseif filter == self.filters.stars then
            self.drop_down[1].idx = self.filters.quality
            self.drop_down[2].idx = self.filters.level
        end

        self:setDropDownText()
    end,

    setDropDownText = function(self)
        for k, v in ipairs(self.drop_down) do
            v.ui:GetComponent(UnityEngine.UI.Text).text = self.strs[v.idx]
        end
    end,

    click_BackBtn = function(self)
        log.info('click_BackButton')
        frame_manager:hide('hero_list')
    end,

    click_SelectBtn = function(self)
        if self.sort_bg.localScale.magnitude == 0 then
            self.sort_bg.localScale = Vector3.one
            self:setDropDownText()
        else
            self.sort_bg.localScale = Vector3.zero
        end
    end,

    click_SortBG = function(self)
        self.sort_bg.localScale = Vector3.zero
    end,

    click_Page_1 = function(self)
        self:updateFilterBtns(1)
        self.sort_bg.localScale = Vector3.zero
    end,

    click_Page_2 = function(self)
        self:updateFilterBtns(2)
        self.sort_bg.localScale = Vector3.zero
    end,
    
    -- 点击战车
    onClick_Hero = function(self, hero_id)
       log.info('click hero id '..hero_id)
       frame_manager:show('officer', self, {id = hero_id, heros = self.heros})
    end,

    click_getHero = function(self, hero_id)
        log.info("获取英雄", hero_id)
        network_sender:recruitHero(hero_id)
    end,
}