local log = logger.get("Game", "Hero Weapons Panel")

local slot_types = {
    [1] = common_pb.MAIN_WEAPON_SLOT,
    [2] = common_pb.MINOR_WEAPON_SLOT_1,
    [3] = common_pb.FASHION_SLOT,
    [4] = common_pb.OTHER_SLOT_1,
    [5] = common_pb.WHEEL_SLOT,
    [6] = common_pb.DECORATION_SLOT,
}

return class{

    ctor = function(self, panel, owner)
        self.panel = panel
        self.owner = owner

        self.slots = {}
        self.scriptEvts = {}
    end,

    clearEvts = function(self)
        for path, evt in pairs(self.scriptEvts) do
            self.owner:clearScript(evt, path)
        end
        self.scriptEvts = {}
    end,

    show = function(self, hero_id, path)
        self.path = path
        self.hero_id = hero_id
        self:loadWeapons()
    end,

    loadWeapons = function(self)
        self:clearEvts()
        local bag = bag_manager:getHeroBag(self.hero_id)
        if not bag then
            log.error("###### load weapon hero bag error! ", self.hero_id)
            return
        end
        
        local idx = 0
        for k, v in ipairs(slot_types) do
            local weapon = bag:get(v)
            self:setWeaponInfo(idx, v, weapon)
            idx = idx + 1
        end
    end,

    setWeaponInfo = function(self, index, id, item_inst)
        local slot = self.panel:GetChild(index)
        local icon_root = slot:Find('Grade')
        local icon = slot:Find('Grade/Icon')
        local add  = slot:Find('Grade/Add')
        local level = slot:Find('Grade/Level')
        local path = self.path..slot.name.."/Grade"
        if item_inst then
            icon:SetActive(true)
            add:SetActive(false)
            level:SetActive(true)
            local img = icon:GetComponent(UnityEngine.UI.Image)
            local icon_path = gui_helper:getItemIconPath(item_inst.cfg)
            local level_text = level:GetComponent(UnityEngine.UI.Text)
            img:SetSprite(icon_path)
            icon_root:SetSprite(gui_helper:getQualityIconPath(item_inst.cfg.color))
            level_text.text = "Lv."..item_inst.level

            level_text.transform:SetActive(item_inst.level ~= 0)
            
            self.owner:setScript("single", path, functor.rmake(self.owner.onClick_weapon, {id = id, item = item_inst}))
        else
            icon:SetActive(false)
            add:SetActive(true)
            level:SetActive(false)
            icon_root:SetSprite(gui_helper:getQualityIconPath(0))
            self.owner:setScript("single", path, functor.rmake(self.owner.onClick_slot, {id = id, item = item_inst}))
        end

        self.slots[id] = {ui = slot, item = item_inst}
        self.scriptEvts[path] = "single"
    end,

    refreshHighLight = function(self, slot_id)
        for k, slot in pairs(self.slots) do
            local highLight_img = slot.ui:Find('Grade/HighLight')
            highLight_img:SetActive(slot_id == k)
        end
    end,
}