local super = require("ui.frame")
local log = logger.get("Game", "Hint Frame")

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        -- 路径: root/login
        -- 事件: click_login 
        clickEvents = {
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self.static_notice = self.transform:Find("StaticNotice")
        self.scroll_notice = self.transform:Find("ScrollNotice")
        self.server_notice = self.transform:Find("Notify")
        self.static_notice.localScale = Vector3(1, 0, 1)
        self.server_notice.localScale = Vector3(1, 0, 1)
        self.scroll_notice:SetActive(false)
        self.start = false

        self.tips = self.transform:Find("Tips")
        self.tips:SetActive(false)

        self.annoounce_evt = announce_manager.event:on("announceNotify", functor.make(self.announceNotify, self))

        self.text_list = {}
    end,

    onUnload = function (self)
        announce_manager.event:remove("announceNotify", self.annoounce_evt)
    end,

    announceNotify = function(self)
        if self.start then
            return
        end

        self:playAnnounce()
    end,

    setServerNotify = function(self, content)
        if not self.transform then
            return
        end

        local text = self:findText('Notify/Text')
        text.text = content

        LeanTween.cancel(self.server_notice.gameObject)

        local duration = 0.2
        local lt = LeanTween.scaleY(self.server_notice.gameObject, 1, duration)
        lt:setFrom(0)
        lt.loopType = LeanTweenType.once

        lt:setOnComplete(function()
            local tw = LeanTween.scaleY(self.server_notice.gameObject, 0, duration)
            tw:setFrom(1)
            tw.delay = 2
            tw.loopType = LeanTweenType.once
        end)
    end,

    setText = function(self, content)
        if not self.transform then
            return
        end

        self.transform:SetAsLastSibling()
        local text = self:findText('StaticNotice/Text')
        text.text = content

        LeanTween.cancel(self.static_notice.gameObject)

        local duration = 0.2
        local lt = LeanTween.scaleY(self.static_notice.gameObject, 1, duration)
        lt:setFrom(0)
        lt.loopType = LeanTweenType.once

        lt:setOnComplete(function()
            local tw = LeanTween.scaleY(self.static_notice.gameObject, 0, duration)
            tw:setFrom(1)
            tw.delay = 2
            tw.loopType = LeanTweenType.once
        end)
    end,

    setScrollNotice = function(self, content)
        if not self.transform then
            return 
        end

        self.transform:SetAsLastSibling()
        self.scroll_notice:SetActive(true)
        local scroll_content = self.scroll_notice:Find("Viewport/Content")
        local text = scroll_content:GetChild(0):GetComponent(UnityEngine.UI.Text)
        text.text = content

        local to = 0
        coroutine.yield(WaitForSeconds(UnityEngine.Time.deltaTime))
        scroll_content.sizeDelta = Vector2(text.rectTransform.rect.width+self.scroll_notice.rect.width*2, text.rectTransform.rect.height);
        to = scroll_content.localPosition.x;
        to = to - (scroll_content.rect.width - self.scroll_notice.rect.width);
        
        LeanTween.cancel(scroll_content.gameObject)
        local duration = 10.0
        local lt = LeanTween.moveLocalX(scroll_content.gameObject, to, duration)
        lt:setFrom(0)

        lt:setOnComplete(function()
            text.text = ""
            local pos = scroll_content.localPosition
            pos.x = 0
            scroll_content.localPosition = pos
        end)
        
        coroutine.yield(WaitForSeconds(duration+1.0))
    end,

    playAnnounce = function(self)
        app:startCoroutine(function()
            self.start = true
            while announce_manager:canPlay() do
                local str = announce_manager:getAnnounce()
                if str then
                    coroutine.yield(self:setScrollNotice(str))
                end
            end
            self.start = false
            self.scroll_notice:SetActive(false)
        end)
    end,

    fingerTipsShow = function(self, text, trans)
        self.transform:SetAsLastSibling()
        self.tips:SetActive(true)
        local label = self:findText("Tips/Text")
        label.text = text
        local screen_pos = self.helper:worldToScreenPoint(trans.position)
        self.tips:SetParent(trans)
        if screen_pos.x >= UnityEngine.Screen.width * 0.5 then
            self.tips.localPosition = Vector3(-(trans.rect.width+self.tips.rect.width+30)*0.5, 0, 0)
        else
            self.tips.localPosition = Vector3((trans.rect.width+self.tips.rect.width+30)*0.5, 0, 0)
        end

        app:startCoroutine(
            function()
                coroutine.yield(nil)
                self.tips:SetActive(false)
                coroutine.yield(nil)
                self.tips:SetActive(true)
            end
        )
    end,

    fingerTipsHide = function(self)
        self.tips:SetActive(false)
    end,
}
