local log = logger.get("Game", "Hold Support Frame")
local super = require("ui.frame")

return class{
    super = super,

    onLoaded = function( self )
        super.onLoaded(self)

        self.owner = self.args.owner
        self.hold = self.args.data.hold

        clickEvents = {
            "Close",
            "BG/Top/SupportBtn",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self.content = self.transform:Find("BG/Scroll View/Viewport/Content")
        self.team_prefab = self.transform:Find("BG/Scroll View/Viewport/Content/Team").gameObject
        self.team_prefab.transform:SetActive(false)

        self.btn_img = self.transform:Find("BG/Top/SupportBtn"):GetComponent(UnityEngine.UI.Image)

        if self.hold.info.playerId == local_player.id then
            self.btn_img:SetSprite(res_path.res_path_41)
        else
            self.btn_img:SetSprite(res_path.res_path_42)
        end

        self.team_count = 0
        self.team_max = 0

        self.evts = {}
        self.evts["WORLDMAPGETHELPS"] = game_director.events:on("WORLDMAPGETHELPS", functor.make(self.onGetTeams, self))
        self.evts["WORLDMAPMARCHLINE"] = game_director.events:on("WORLDMAPMARCHLINE", functor.make(self.onMarchLine, self))

        network_sender:supportTeams(self.hold.id)
    end,

    onUnload = function(self)
        if self.evts then
            for k, v in pairs(self.evts) do
                game_director.events:remove(k, v)
            end
        end
        -- self:StopAllCoroutines()
    end,

    onMarchLine = function(self, march_line)
        if march_line.marchAction == common_pb.EMARCH_GO_HOME then
            network_sender:supportTeams(self.hold.id)
            --self:click_Close()
        end
    end,

    onGetTeams = function(self, msg)
        -- self:StopAllCoroutines()
        self:teamCount(msg)
        self:loadTeam(msg)
    end,

    teamCount = function(self, msg)
        self.team_count = #msg.teams or self.team_count
        self.team_max = msg.teamMax or self.team_max

        local count = self.transform:Find("BG/Top/Count"):GetComponent(UnityEngine.UI.Text)
        count.text = string.format("%d/%d", self.team_count, self.team_max)
    end,

    loadTeam = function(self, msg)
        for k, v in ipairs(msg.teams) do
            if k > self.content.childCount then
                team = Object.Instantiate(self.team_prefab).transform
                team:SetParent(self.content)
                resetTransformZ(team)
            else
                team = self.content:GetChild(k-1)
            end

            if not team then
                return
            end

            team:SetActive(true)
            team.name = "team_"..k

            self:setTeamInfo(team, v)
        end

        for i=#msg.teams, self.content.childCount - 1 do
            local child = self.content:GetChild(i)
            child:SetActive(false)
        end
    end,

    setTeamInfo = function(self, elem, data)
        local player_name = elem:Find("Name"):GetComponent(UnityEngine.UI.Text)
        local fight = elem:Find("Fight"):GetComponent(UnityEngine.UI.Text)
        local formation = elem:Find("Formation_name"):GetComponent(UnityEngine.UI.Text)
        local btn = elem:Find("BackBtn")
        local team = elem:Find("Heros")

        player_name.text = data.playerName
        fight.text = string.format("%s <color=#FFFFFFFF>%d</color>", NativeText.GetLangStr(280028), data.formaton.sword)
        
        local cfg = config.formation_lists[data.formaton.formationId]
        formation.text = self.helper:getLangCfg(cfg.name_id)

        if data.playerId == local_player.id then
            local path = "BG/Scroll View/Viewport/Content/"..elem.name.."/BackBtn"
            self:clearScript("click", path)
            self:setScript("click", path, functor.rmake(self.onClickBack, data.formaton.teamId))
        end

        btn:SetActive(data.playerId == local_player.id)

        self:loadHeros(team, data.formaton)

        if data.helpTime > 0 then
            self:loadSupportTime(elem, data.helpTime, data.totalTime)
        end
    end,

    loadSupportTime = function(self, elem, left_time, total_time)
        app:startCoroutine(
            function()
                local time_slider = elem:Find("Slider"):GetComponent(UnityEngine.UI.Slider)
                local time_text = time_slider.transform:Find("Text"):GetComponent(UnityEngine.UI.Text)
                while left_time > 0 do
                    local vsb = frame_manager:isDisplay('hold_support')
                    if not vsb then
                        return
                    end
                    left_time = left_time - 1
                    time_text.text = gui_helper:timeFormat(left_time)
                    time_slider.value = left_time / total_time
                    coroutine.yield(WaitForSeconds(1))
                end

                elem:SetActive(false)
            end
        )
    end,

    loadHeros = function(self, team, data)
        local cfg = config.formation_lists[data.formationId]

        if cfg then
            gui_helper:loadFormationHeros(team, cfg, data.formationHero, true)
        else
            log.error("#### load hero formation cfg error ! id", data.formationId)
        end
    end,

    onClickBack = function(self, handle, id)
        network_sender:worldMapGoHome(2, id)
    end,

    click_Close = function(self)
        frame_manager:hide("hold_support")
    end,

    click_SupportBtn = function(self)
        if army_queue_manager:queueMaxCheck() then
            return
        end
        
        if self.hold.info.playerId == local_player.id then
            hint:setText(NativeText.GetLangStr(280030))
            return
        end

        if self.team_count >= self.team_max then
            hint:setText(NativeText.GetLangStr(280009))
            return
        end
        frame_manager:show('dispatch_team', self.owner, {hold = self.hold, count = 1, action = common_pb.EMARCH_HELP})
    end,

}