local log = logger.get("Game", "Home Armory Frame")
local super = require("ui.frame")

return class{

    super = super,

    onLoaded = function(self)
        super.onLoaded(self)

        self.info = self.args.data.info
        self.owner = self.args.owner

        clickEvents = {
            "Close",
            "BG/Top/ChangeBtn",
            "BG/Top/ViewBtn",
            "BG/FinishBtn",
            "BG/CreateBtn",
            --"BG/Production/EndBtn",
            "BG/Production/Slider/SpeedUpBtn",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self.start_panel = self.transform:Find("BG/Start")
        self.production_panel = self.transform:Find("BG/Production")

        self.prog = self.production_panel:Find("Slider"):GetComponent(UnityEngine.UI.Slider)
        self.lt_text = self.production_panel:Find("Slider/LeftTime"):GetComponent(UnityEngine.UI.Text)
        self.change_btn = self.transform:Find("BG/Top/ChangeBtn"):GetComponent(UnityEngine.UI.Button)
        self.speed_up_btn = self.transform:Find("BG/Production/Slider/SpeedUpBtn"):GetComponent(UnityEngine.UI.Button)

        self.start_panel:SetActive(false)
        self.production_panel:SetActive(false)

        self.click_cd = 1
        self.click_time = 0

        self:refresh(self.info)

        self.evt = game_director.events:on("REFRESHARMORYINFO", functor.make(self.transer, self))
        self.item_evt = game_director.events:on("NOTIFYGETITEM", functor.make(self.onGetItems, self))
        self.bag_evt = bag_manager.events:on("onBagChanged", functor.make(self.setBagAmount, self))
    end,

    onUnload = function(self)
        game_director.events:remove("REFRESHARMORYINFO", self.evt)
        game_director.events:remove("NOTIFYGETITEM", self.item_evt)
        bag_manager.events:remove("onBagChanged", self.bag_evt)
    end,

    transer = function(self)
        network_sender:buildingExpend(self.info.id, self.info.queryType)
    end,

    refresh = function(self, info)
        self.info = info
        self.blueprint_id = info.id
        self.left_time = info.leftTime
        self.state = self.info.state

        local cfg = config.home_config_produces[self.blueprint_id]

        if not cfg then
            log.error("#### refresh BluePrint cfg error! id", self.blueprint_id)
            return
        end
        
        self.cfg = cfg

        --log.info("##### changeState ", tostring(info.state), info.drawingId, info.makeTime, info.leftTime)
        self:changeState(self.state)
        self:setBagAmount()
        self:setBluePrint()

        if self.speed_up_frame then
            local data = {}
            data.id = self.blueprint_id
            data.type = 45
            data.title = NativeText.GetLangStr(320001)
            data.left_time = self.left_time
            data.total_time = self.info.time
            data.callback = self.sendMsg
            self.speed_up_frame:refresh(data)
        end
    end,

    onGetItems = function(self, msg)
        if msg.reason == common_pb.PRODUCT_EQUIP then
            -- frame_manager:show('getItems', self, {items = gui_helper.drop_list[msg.reason], has_Confirm_Btn = true})
        end
    end,

    changeState = function(self, state)
        if state == common_pb.MMAKE_NONE then
            self:onBluePrintChanged(self.blueprint_id)
        elseif state == common_pb.MMAKE_MAKE_DONE then
            self.left_time = 0
            self:setProductProg()
        end

        self.change_btn.interactable = state == common_pb.MMAKE_NONE
        self.start_panel:SetActive(state == common_pb.MMAKE_NONE)
        self.production_panel:SetActive(state ~= common_pb.MMAKE_NONE)
        self.speed_up_btn.interactable = state == common_pb.MMAKE_MAKING

        self:refreshCreateBtn(state)
        self:refreshFinishBtn(state)

        self.state = state
    end,

    onBluePrintChanged = function(self, id)
        local cfg = config.home_config_produces[id]

        if not cfg then
            log.error("#### BluePrint cfg error! id", id)
            return
        end

        if self.blueprint_id ~= id then
            self.info.time = cfg.time*60
        end
        
        self.cfg = cfg
        self.blueprint_id = id

        self:setCost()
        self:setTotalTime()
        self:setFinishCost(self.info.time)
    end,

    update = function(self, dt)
        if self.state == common_pb.MMAKE_MAKING then
            if self.left_time > 0 then
                self:setFinishCost(self.left_time)
                self.left_time = self.left_time - dt
                if self.left_time <= 0 then
                    self.left_time = 0
                    self:changeState(common_pb.MMAKE_MAKE_DONE)
                end
                self:setProductProg()
            end
        end
    end,

    setBagAmount = function(self)
        local amount = self.transform:Find("BG/CountBG/Count"):GetComponent(UnityEngine.UI.Text)

        local bag_num = bag_manager:getEquipNum()
        local max = local_player:getBagMaxAmount()
        local str = gui_helper:limitColorFormat(bag_num, max)
        amount.text = string.format("%s  %s", NativeText.GetLangStr(320004), str)
    end,

    setBluePrint = function(self)
        --local img = self.transform:Find("BG/Top"):GetComponent(UnityEngine.UI.Image)
        --local name = self.transform:Find("BG/Top/Name"):GetComponent(UnityEngine.UI.Text)
        local tip = self.transform:Find("BG/Top/Text"):GetComponent(UnityEngine.UI.Text)
        --img:SetSprite(self.cfg.path..self.cfg.icon)
        --name.text = self.helper:getLangCfg(self.cfg.name)
        local str = NativeText.GetLangStr(320020)
        if self.cfg.type == 2 then
            str = NativeText.GetLangStr(320021)
        end
        tip.text = string.format( str, gui_helper:quality2Color(self.cfg.color), gui_helper:getQualityStr(self.cfg.color) )
    end,

    setCost = function(self)
        -- local cost = loadstring("return "..self.cfg.cost)()
        local cost = self.info.levelUpCost

        if type(cost) == "table" then
            local node = self.start_panel:Find("Cost")
            local idx = 0
            for k, v in ipairs(cost) do
                local child = nil
                if idx < node.childCount then
                    child = node:GetChild(idx)
                else
                    child = Object.Instantiate(node:GetChild(0).gameObject).transform
                    child:SetParent(node)
                    resetTransformZ(child)
                end
                
                if child then
                    child:SetActive(true)
                    gui_helper:setCostItem(v.ItemTemId, v.Count, child)
                    idx = idx + 1
                end
            end

            for i=idx, node.childCount - 1 do
                local child = node:GetChild(i)
                child:SetActive(false)
            end
        end
    end,

    setTotalTime = function(self)
        local total_time = self.transform:Find("BG/CreateBtn/CreateTime"):GetComponent(UnityEngine.UI.Text)
        total_time.text = gui_helper:timeFormat(self.info.time)
    end,

    setFinishCost = function(self, time)
        local count = self.transform:Find("BG/FinishBtn/Icon/Amount"):GetComponent(UnityEngine.UI.Text)
        local icon = self.transform:Find("BG/FinishBtn/Icon"):GetComponent(UnityEngine.UI.Image)

        local cost_amount, item_id = speedup_helper:getCostDiamond(time, speedUpTypes.produce)
        count.text = tostring(cost_amount)

        icon:SetSprite(gui_helper:getItemIconPathFromId(item_id))
    end,

    setProductProg = function(self)
        local percent = self.left_time / self.info.time
        self.prog.value = 1 - percent

        self.lt_text.text = gui_helper:timeFormat(math.floor(self.left_time))
    end,

    refreshFinishBtn = function(self, state)
        local btn = self.transform:Find("BG/FinishBtn"):GetComponent(UnityEngine.UI.Button)
        local icon = self.transform:Find("BG/FinishBtn/Icon")

       if state == common_pb.MMAKE_MAKE_DONE then
            icon:SetActive(false)
            btn.interactable = false
            btn.targetGraphic.sprite = btn.spriteState.disabledSprite
        else
            icon:SetActive(true)
            btn.interactable = true
            btn.targetGraphic.sprite = btn.spriteState.highlightedSprite
        end
    end,


    refreshCreateBtn = function(self, state)
        local btn = self.transform:Find("BG/CreateBtn"):GetComponent(UnityEngine.UI.Button)
        local text = self.transform:Find("BG/CreateBtn/Text"):GetComponent(UnityEngine.UI.Text)
        local time_text = self.transform:Find("BG/CreateBtn/CreateTime"):GetComponent(UnityEngine.UI.Text)

        if state == common_pb.MMAKE_MAKING then
            time_text:SetActive(false)
            btn.interactable = false
            text.text = NativeText.GetLangStr(320008)
        elseif state == common_pb.MMAKE_MAKE_DONE then
            time_text:SetActive(false)
            btn.interactable = true
            text.text = NativeText.GetLangStr(320009)
        else
            time_text:SetActive(true)
            btn.interactable = true
            text.text = NativeText.GetLangStr(320010)
        end
    end,

    click_Close = function(self)
        frame_manager:hide("armory_frame")
    end,

    click_ChangeBtn = function(self)
        local cb = function(id)
            self:onBluePrintChanged(id)
            self:setBluePrint()
        end
        frame_manager:show("blueprint_select", self, {callback = cb})
    end,

    click_ViewBtn = function(self)
        frame_manager:show("blueprint_drop", self, {id = self.blueprint_id})
    end,

    click_FinishBtn = function(self)
        if Time.time - self.click_time > self.click_cd then
            self.click_time = Time.time
            if self.state == common_pb.MMAKE_NONE then
                network_sender:armoryProductEquip(self.cfg.type, 1)
            elseif self.state == common_pb.MMAKE_MAKING then
                network_sender:speedUpProductEquip(0, 0, 1)
            end
        end
    end,

    click_CreateBtn = function(self)
        if self.state == common_pb.MMAKE_NONE then
            -- local cost = loadstring("return "..self.cfg.cost)()
            -- local cost = self.info.levelUpCost
            -- if gui_helper:checkCostItems(cost) then
                network_sender:armoryProductEquip(self.cfg.type, 0)
            -- end
        elseif self.state == common_pb.MMAKE_MAKE_DONE then
            network_sender:getArmoryEquip()
        end

        if self.owner then
            self.owner:hide()
        end
    end,

    --[[ click_EndBtn = function(self)
        network_sender:getArmoryEquip()
    end, ]]

    sendMsg = function(id, count, finish)
        network_sender:speedUpProductEquip(id, count, finish)
    end,

    click_SpeedUpBtn = function(self)
        --[[ local cb = function(id, count, finish)
            network_sender:speedUpProductEquip(id, count, finish)
        end ]]
        self.speed_up_frame = frame_manager:show("common_speed_up", self, {id = self.blueprint_id, type = 45, left_time = self.left_time, total_time = self.info.time, title = NativeText.GetLangStr(320001), callback = self.sendMsg})
    end,
}