local super = require("ui.frame")

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        -- 路径: root/login
        -- 事件: click_login 
        clickEvents = {
            "content/bg/making/time/speedUpBtn",
            "content/bg/make done/time/rcvBtn",
            "content/bg/top/backBtn",
        }

        -- 注册点击事件
        frame_helper:regClickEvents("click", self, clickEvents)

        self.BUILD_INFO_CHANGE = home_info.events:on("BUILD_INFO_CHANGE", function(build_id)
            self:init()
        end)

        self.ARSENAL_CREATE_TICK = home_info.events:on("ARSENAL_CREATE_TICK", function(build_data, elapsed, total_time)
            if build_data.cfgId == 11 then
                self:onArsenalCreateTick(elapsed, total_time)
            end
        end)

        self:init()
    end,

    onUnload = function (self)
        home_info.events:off("BUILD_INFO_CHANGE", self.BUILD_INFO_CHANGE)
        home_info.events:off("ARSENAL_CREATE_TICK", self.ARSENAL_CREATE_TICK)
    end,

    init = function(self)
        self.cache_btns = {}

        local cfg_list = {}
        config.home_config_arsenals:foreach(function(id, cfg)
            table.insert(cfg_list, cfg)
        end)
        local sortFunc = function(a, b)
            return a.weapon_type < b.weapon_type
        end
        table.sort(cfg_list, sortFunc)

        local none = self.transform:Find("content/bg/none")
        local making = self.transform:Find("content/bg/making")
        local make_done = self.transform:Find("content/bg/make done")

        frame_helper.hideTrans(none)
        frame_helper.hideTrans(making)
        frame_helper.hideTrans(make_done)
        
        local cur_state = self:getCurState()
        if cur_state == common_pb.MMAKE_NONE then
            frame_helper.showTrans(none)
        elseif cur_state == common_pb.MMAKE_MAKING then
            frame_helper.showTrans(making)
            local e = home_info:getBuildDataByCfgId(11)
            local elapsed = e.armory.leftTime - Time.realtimeSinceStartup + e.rmbTime
            self:onArsenalCreateTick(elapsed, e.armory.makeTime)
            local item_cfg = frame_helper:getItemCfg(e.armory.itemId)
            config.home_config_arsenals:foreach(function(id, cfg)
                if cfg.weapon_type == item_cfg.family then
                    local image1 = self:findImage("content/bg/making/icon")
                    local image2 = self:findImage("content/bg/make done/icon")
                    image1:SetSprite(cfg.building_icon)
                    image2:SetSprite(cfg.building_icon)
                end
            end)

        elseif cur_state == common_pb.MMAKE_MAKE_DONE then
            frame_helper.showTrans(make_done)
        end

        local path = "content/bg/layout"
        frame_helper:hideAllChild(self, path, true)
        for __, cfg in ipairs(cfg_list) do
            local child = frame_helper:getChild(self, __, path)
            child:SetActive(true)

            local name = child:Find("name"):GetComponent(UnityEngine.UI.Text)
            local type_icon = child:Find("typ_icon"):GetComponent(UnityEngine.UI.Image)
            local des = child:Find("des"):GetComponent(UnityEngine.UI.Text)
            local icon = child:Find("icon"):GetComponent(UnityEngine.UI.Image)

            icon:SetSprite(cfg.pic)
            type_icon:SetSprite(cfg.icon)
            name.text = frame_helper:getLangCfg(cfg.name)
            des.text = frame_helper:getLangCfg(cfg.des)

            local temp_time = home_info:getArsenalMakeTime(cfg)
            local config_time = child:Find("makeBtn/unreal"):GetComponent(UnityEngine.UI.Text)
            config_time.text = frame_helper:timeFormat(cfg.init_time)
            local actual_time = child:Find("makeBtn/real"):GetComponent(UnityEngine.UI.Text)
            actual_time.text = frame_helper:timeFormat(temp_time)

            local speed_up_type = home_speed_up_type.equip_create
            local time_cost_diamond = gui_helper:getSpeedUpCost(speed_up_type, temp_time)
            local DIAMOND    = 30001 -- 钻石
            local item_name = gui_helper:getImageTextItemStr(DIAMOND)
            local image_text = child:Find("finishBtn/image_text"):GetComponent(UnityEngine.UI.Text)
            image_text:SetImageText(string.format("%s %s", item_name, time_cost_diamond))

            local toggle_btn = child:Find("btn"):GetComponent(UnityEngine.UI.Button)
            toggle_btn.onClick:RemoveAllListeners()
            toggle_btn.onClick:AddListener(function()
                self:click_selectType(cfg)
            end)

            local btn = child:Find("infoBtn"):GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(function()
                self:click_showMakeList(cfg)
            end)

            local make_btn = child:Find("makeBtn"):GetComponent(UnityEngine.UI.Button)
            make_btn.onClick:RemoveAllListeners()
            make_btn.onClick:AddListener(function()
                self:click_makeBtn(cfg)
            end)

            local finish_btn = child:Find("finishBtn"):GetComponent(UnityEngine.UI.Button)
            finish_btn.onClick:RemoveAllListeners()
            finish_btn.onClick:AddListener(function()
                self:click_finishBtn(cfg, time_cost_diamond * 5)
            end)

            table.insert(self.cache_btns, {
                cfg = cfg,
                make_btn = make_btn,
                finish_btn = finish_btn,
                toggle_btn = toggle_btn,
            })
        end

        self:click_selectType(self.cur_cfg or cfg_list[1])
    end,

    getCurState = function(self)
        local cur_state = common_pb.MMAKE_NONE
        local build_data = home_info:getBuildDataByCfgId(11)
        if build_data.armory then
            cur_state = build_data.armory.state
        end
        return cur_state
    end,

    click_backBtn = function(self)
        frame_manager:hide('home_arsenal_main')
    end,

    click_selectType = function(self, cfg)
        self.cur_cfg = cfg

        local cur_state = self:getCurState()
        for __, value in ipairs(self.cache_btns) do
            if cur_state == common_pb.MMAKE_NONE and value.cfg == cfg then
                frame_helper.showTrans(value.make_btn.transform)
                frame_helper.showTrans(value.finish_btn.transform)
                value.toggle_btn.interactable = false
            else
                frame_helper.hideTrans(value.make_btn.transform)
                frame_helper.hideTrans(value.finish_btn.transform)
                value.toggle_btn.interactable = true
            end
        end
    end,

    click_showMakeList = function(self, cfg)
        frame_manager:show('home_arsenal_preview', nil, cfg)
    end,

    click_makeBtn = function(self, cfg)
        network_sender:armoryProductEquip(cfg.weapon_type, 0)
    end,

    click_finishBtn = function(self, cfg, cost_diamond)
        if frame_helper:isEnoughDiamond(cost_diamond) then
            network_sender:armoryProductEquip(cfg.weapon_type, 1)    
        end
    end,

    onArsenalCreateTick = function(self, elapsed, total_time)
        local time = self:findText("content/bg/making/time/time")
        local slider = self:findSlider("content/bg/making/time/Slider")
        slider.value = 1 - elapsed / total_time
        local e = home_info:getBuildDataByCfgId(11)
        local item_cfg = frame_helper:getItemCfg(e.armory.itemId)
        config.home_config_arsenals:foreach(function(id, cfg)
            if cfg.weapon_type == item_cfg.family then
                time.text = string.format(NativeText.GetLangStr(630013), frame_helper:getLangCfg(cfg.name), frame_helper:timeFormat(elapsed))
            end
        end)
    end,

    click_speedUpBtn = function(self)
        local e = home_info:getBuildDataByCfgId(11)
        if e.cfgId == 11 and e.armory.state == common_pb.MMAKE_MAKING then
            local elapsed = e.armory.leftTime - Time.realtimeSinceStartup + e.rmbTime
            if elapsed > 0 then
                local item_cfg = frame_helper:getItemCfg(e.armory.itemId)
                config.home_config_arsenals:foreach(function(id, cfg)
                    if cfg.weapon_type == item_cfg.family then
                        frame_manager:show("home_speed_up", nil, {
                            speed_up_type = home_speed_up_type.equip_create,
                            id = e.id,
                            arsenal_cfg = cfg,
                        })
                    end
                end)
            end
        end
    end,

    click_rcvBtn = function(self)
        network_sender:getArmoryEquip()
    end,
}
