local super = require("ui.frame")

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        self.default_cfg = self.args.data        

        -- 路径: root/login
        -- 事件: click_login 
        clickEvents = {
            "closeBtn",
        }

        -- 注册点击事件
        frame_helper:regClickEvents("click", self, clickEvents)

        self:init()
    end,

    onUnload = function (self)
    end,

    init = function(self)

        self.cache_btns = {}

        local cfg_list = {}
        config.home_config_arsenals:foreach(function(id, cfg)
            table.insert(cfg_list, cfg)
        end)
        local sortFunc = function(a, b)
            return a.weapon_type < b.weapon_type
        end
        table.sort(cfg_list, sortFunc)

        local path = "content/bg/bar layout"
        frame_helper:hideAllChild(self, path, true)
        for __, cfg in ipairs(cfg_list) do
            local child = frame_helper:getChild(self, __, path)
            child:SetActive(true)

            local name = child:Find("text1_Static[T2]_3"):GetComponent(UnityEngine.UI.Text)
            local type_icon = child:Find("building_icon_13_2"):GetComponent(UnityEngine.UI.Image)

            type_icon:SetSprite(cfg.icon)
            name.text = frame_helper:getLangCfg(cfg.name)


            local btn = child:GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(function()
                self:click_toggle(cfg.weapon_type)
            end)

            table.insert(self.cache_btns, {
                weapon_type = cfg.weapon_type,
                btn = btn,
            })
        end

        if self.default_cfg then
            self:click_toggle(self.default_cfg.weapon_type)
        else
            self:click_toggle(config.home_config_arsenals[1].weapon_type)
        end
    end,

    click_toggle = function(self, weapon_type)
        for __, value in ipairs(self.cache_btns) do
            value.btn.interactable = value.weapon_type ~= weapon_type
        end
        self:refreshContent(weapon_type)
    end,

    refreshContent = function(self, weapon_type)
        local level = 0
        local build_data = home_info:getBuildDataByCfgId(11)
        if build_data then
            level = build_data.level
        end

        local cook_cfgs = self:getCookCfgs(weapon_type)

        local path = "content/bg/Scroll View/Viewport/Content/"
        frame_helper:hideAllChild(self, path, true)
        for __, value in ipairs(cook_cfgs) do
            local child = frame_helper:getChild(self, __, path)
            child:SetActive(true)

            local ok = child:Find("info/title_ok")
            local no = child:Find("info/title_no")

            local green = child:GetComponent(UnityEngine.UI.Image)
            local red = child:Find("red"):GetComponent(UnityEngine.UI.Image)

            frame_helper:setActive(ok, level >= value.arsenal_level)
            frame_helper:setActive(no, level < value.arsenal_level)

            green.enabled = level >= value.arsenal_level
            red.enabled = level < value.arsenal_level

            local ok_title = ok:GetComponent(UnityEngine.UI.Text)
            local no_title = no:GetComponent(UnityEngine.UI.Text)

            ok_title.text = NativeText.GetLangStr(630006)
            no_title.text = string.format(NativeText.GetLangStr(630007), value.arsenal_level)

            local item_path = path .. child.name .. "/layout"
            frame_helper:hideAllChild(self, item_path, true)
            for idx, cfg in ipairs(value.list) do
                local item = frame_helper:getChild(self, idx, item_path)
                item:SetActive(true)
                
                local icon = item:Find("icon"):GetComponent(UnityEngine.UI.Image)
                local quality = item:Find("quality"):GetComponent(UnityEngine.UI.Image)

                local weapon_cfg = config.weapon_weapons[cfg.equipment_id]
                icon:SetSprite(weapon_cfg.path .. weapon_cfg.icon)
                quality:SetSprite(gui_helper:getQualityIconPath(cfg.quality))
            end

            -- local btn = child:Find(""):GetComponent(UnityEngine.UI.Button)
            -- btn.onClick:RemoveAllListeners()
            -- btn.onClick:AddListener(function()
            -- end)
        end
    end,

    getCookCfgs = function(self, weapon_type)
        local cook_cfgs = {}

        local getList = function(arsenal_level)
            for __, value in ipairs(cook_cfgs) do
                if value.arsenal_level == arsenal_level then
                    return value.list
                end
            end
            local new_list = {
                arsenal_level = arsenal_level,
                list = {}
            }
            table.insert(cook_cfgs, new_list)
            return new_list.list
        end

        config.home_config_arsenal_libs:foreach(function(id, cfg)

            if weapon_type == cfg.type then
                local list = getList(cfg.arsenal_level)
                table.insert(list, cfg)
            end
        end)

        local sortId = function(a, b)
            return a.id < b.id
        end

        for __, value in ipairs(cook_cfgs) do
            table.sort(value.list, sortId)    
        end

        local sortLevel = function(a, b)
            return a.arsenal_level < b.arsenal_level
        end

        table.sort(cook_cfgs, sortLevel)

        return cook_cfgs
    end,


    click_closeBtn = function(self)
        frame_manager:hide('home_arsenal_preview')
    end,
}
