local super = require("ui.frame")

local state = {
    display = 1,
    hide = 1,
    reset = 1,
}

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        self.list = self.args.data

        clickEvents = {
            "stepBtn",
        }

        frame_helper:regClickEvents("click", self, clickEvents)

        self:init()
    end,

    onUnload = function (self)
    end,

    init = function(self)
        self.all_cards = {
            self:findImage("kapian_4"),
            self:findImage("kapian_3"),
            self:findImage("kapian_2"),
            self:findImage("kapian_1"),
        }

        if #self.list <= 1 then
            for __, image in ipairs(self.all_cards) do
                image.enabled = false
            end
        end

        local animator = self.transform:GetComponent(UnityEngine.Animator)
        self.play_queue = {}

        for __, item in ipairs(self.list) do
            local cfg = frame_helper:getItemCfg(item.itemId)
            local color_str = self:getColorStr(cfg.color)
            -- 卡牌出现效果
            local start_name = string.format("effect_arsenal_complete_%s_2001", color_str)
            local stop_name = string.format("effect_arsenal_complete_%s_2002", color_str)
            table.insert(self.play_queue, {
                start_name = start_name,
                stop_name = stop_name,
                setInfo = function()
                    self:setItemInfo(item)
                end,
                skip = function()
                    local state = animator:GetCurrentAnimatorStateInfo(0)
                    local rst = state:IsName(start_name)
                    if rst then
                        animator:Play(stop_name)
                    else
                        self.idx = self.idx + 1
                        self:click_stepBtn()
                    end
                end,
            })
            -- 卡牌消失效果
            -- 最后一次抽卡的消失效果
            if __ < #self.list then
                local start_name = string.format("effect_arsenal_complete_%s_3001", color_str)
                local stop_name = "effect_arsenal_complete_1002"
                table.insert(self.play_queue, {
                    start_name = start_name,
                    stop_name = stop_name,
                    setInfo = function()
                        self:refreshBackCard()
                    end,
                    skip = function()
                        local state = animator:GetCurrentAnimatorStateInfo(0)
                        local rst = state:IsName(start_name)
                        if rst then
                            animator:Play(stop_name)
                        else
                            self.idx = self.idx + 1
                            self:click_stepBtn()
                        end
                    end,
                })
            end
        end
    end,

    click_stepBtn = function(self)
        self.idx = self.idx or 1
        if self.idx <= #self.play_queue then
            local data = self.play_queue[self.idx]
            if data.inited then
                if data.skip then
                    data.skip()
                end
            else
                data.inited = true
                if data.setInfo then
                    data.setInfo()
                end
                local animator = self.transform:GetComponent(UnityEngine.Animator)
                animator:Play(data.start_name)
            end
        else
            frame_manager:hide('home_arsenal_reward')
        end
    end,

    getColorStr = function(self, color)
        if color == 1 then
            return "huise"
        elseif color == 2 then
            return "lvse"
        elseif color == 3 then
            return "lanse"
        elseif color == 4 then
            return "zise"
        elseif color == 5 then
            return "huangse"
        end
        return ""
    end,

    refreshBackCard = function(self)
        self.card_idx = self.card_idx and self.card_idx + 1 or 1
        if self.card_idx <= #self.all_cards then
            local value = self.all_cards[self.card_idx]
            -- if type(value) ~= "number" then
                value.enabled = false
            -- end
        end
    end,

    setItemInfo = function(self, item)
        local cfg = frame_helper:getItemCfg(item.itemId)
        local item_cfg_id = item.itemId
        local id = item.uId
        local item_cfg = frame_helper:getItemCfg(item_cfg_id)
        local color_str = self:getColorStr(item_cfg.color)

        local name =        self:findText(string.format("kaika_%s/kapai/canvas group/name", color_str))
        local type_name =   self:findText(string.format("kaika_%s/kapai/canvas group/type_name", color_str))
        local type_icon =   self:findImage(string.format("kaika_%s/kapai/canvas group/type_icon", color_str))
        local icon =        self:findImage(string.format("kaika_%s/kapai/canvas group/icon", color_str))
        local bg =          self:findImage(string.format("kaika_%s/kapai/canvas group/kapian_zhengmian (1)", color_str))


        name.text = frame_helper:getItemName(item_cfg_id)
        name.color = gui_helper:quality2ColorPlus(cfg.color)

        print("item_id:", item_cfg_id, frame_helper:getItemName(item_cfg_id))

        icon:SetSprite(frame_helper:getItemIcon(item_cfg_id))
        config.home_config_arsenals:foreach(function(id, cfg)
            if cfg.weapon_type == item_cfg.family then
                type_icon:SetSprite(cfg.icon)
                type_name.text = frame_helper:getLangCfg(cfg.name)
            end
        end)
        config.home_config_arsenal_card_qualitys:foreach(function(id, cfg)
            if cfg.quality == item_cfg.color then
                bg:SetSprite(cfg.card)
            end
        end)
        local own_skill = self.transform:Find(string.format("kaika_%s/kapai/canvas group/own_skill", color_str))
        local none_skill = self.transform:Find(string.format("kaika_%s/kapai/canvas group/none_skill", color_str))
        local item_lua = bag_manager:getItemById(id)
        frame_helper:setActive(own_skill, #item_lua.rand_skill > 0)
        frame_helper:setActive(none_skill, #item_lua.rand_skill <= 0)

        if #item_lua.rand_skill > 0 then
            local cfg_id = item_lua.rand_skill[1]
            local cfg = config.passive_skill_skills[cfg_id]
            local passive_icon = self:findImage(string.format("kaika_%s/kapai/canvas group/own_skill/passive_icon", color_str))
            local passive_des = self:findText(string.format("kaika_%s/kapai/canvas group/own_skill/passive_des", color_str))
            passive_icon:SetSprite(cfg.skill_icon_path .. cfg.skill_icon_name)
            passive_des.text = frame_helper:getLangCfg(cfg.skill_des)
            local star_path = string.format("kaika_%s/kapai/canvas group/own_skill/light stars", color_str) 
            frame_helper:hideAllChild(self, star_path, true)
            for i = 1, cfg.skill_level do
                local star = frame_helper:getChild(self, i, star_path)
                star:SetActive(true)
            end
        end
    end,
}
