local super = require("ui.frame")
local log = logger.get("Game", "home_info")

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        self.build_id = self.args.data.build_id
        self.soldier_type = self.args.data.soldier_type
        self.home_map = self.args.owner
        self.build_data = home_info:getBuildDataById(self.build_id)

        clickEvents = 
        {
            "content/backBtn",
            "content/deleteBtn",
            "content/infoBtn",
            "content/un_upgrade/finishBtn",
            "content/un_upgrade/trainBtn",
            "content/un_upgrade/reduceBtn",
            "content/un_upgrade/addBtn",
            "content/upgrading/un_receive/finishBtn1",
            "content/upgrading/un_receive/speedUpBtn",
            "content/upgrading/receive/receiveBtn",
            "content/lock/goBtn",
        }

        -- 注册点击事件
        frame_helper:regClickEvents("click", self, clickEvents)

        self.BUILD_INFO_CHANGE = home_info.events:on("BUILD_INFO_CHANGE", function(build_id)
            local build_data = home_info:getBuildDataById(build_id)
            -- 监听兵营数据变化，同时还有校场的数据变化
            if build_id == self.build_id then
                self:refreshUI(build_id)
            end

            if build_data.cfgId == 10 then
                self:refreshUI(build_id)
            end
        end)

        self.onBagChanged = bag_manager.events:on('onBagChanged', function() 
            self:refreshUI(self.build_id) 
        end)

        if self:isTrapBuild() then
            self.TRAP_TRAIN_TICK = home_info.events:on("TRAP_TRAIN_TICK", function(build_data, left_time, total_time)
                local can_rcv = build_data.guardsystem.build.canRecv 
                self:onTickTrainTime(build_data, left_time, total_time, can_rcv)
            end)
        else
            self.BARRACK_TRAIN_TICK = home_info.events:on("BARRACK_TRAIN_TICK", function(build_data, left_time, total_time)
                if build_data.id == self.build_id then
                    local can_rcv = build_data.barrack.canRecv
                    self:onTickTrainTime(build_data, left_time, total_time, can_rcv)
                end
            end)
        end

        self.onPropChange = local_player.events:on("onPropChange", function() 
            self:refreshUI(self.build_id) 
        end)

        self:init()

        guide_manager:tryStartGuide()
    end,

    onUnload = function (self)
        self:unloadSpine()

        home_info.events:off("BUILD_INFO_CHANGE", self.BUILD_INFO_CHANGE)
        bag_manager.events:off("onBagChanged", self.onBagChanged)
        home_info.events:off("BARRACK_TRAIN_TICK", self.BARRACK_TRAIN_TICK)
        local_player.events:off("onPropChange", self.onPropChange)
        if self:isTrapBuild() then
            home_info.events:off("TRAP_TRAIN_TICK", self.TRAP_TRAIN_TICK)
        else
            home_info.events:off("BARRACK_TRAIN_TICK", self.BARRACK_TRAIN_TICK)
        end
    end,

    init = function(self)
        local cfgs = self:getCfgs()
        self.cur_index = self:getDefaultSoldierIndex(cfgs)
        self.cur_barrack_cfg = cfgs[self.cur_index]

        self:refreshUI(self.build_id)
        self:onLocate(self.cur_index)
    end,

    refreshUI = function(self, build_id)
        self.build_data = home_info:getBuildDataById(build_id)
        if self.build_data then
            if self:isTrapBuild() then
                self.training_soldiers = self.build_data.guardsystem.build    
            else
                self.training_soldiers = self.build_data.barrack
            end   
        end

        self:refreshBar()
        self:click_soldier(self.cur_index, self.cur_barrack_cfg)
    end,

    onLocate = function(self, index)
        local content_trans = self.transform:Find("content/left_bar/Scroll View/Viewport/Content")
        for __, value in ipairs(self.cache_toggles) do
            if index == value.index then
                frame_helper:onScrollViewLocate(content_trans, value.child.transform, false)
                break
            end
        end
    end,

    getBuildId = function(self)
        return self.build_id
    end,

    getSoldierNum = function(self, soldier_id)
        return home_info:getAvailableSoldierNum(soldier_id)
    end,

    refreshBar = function(self)
        local build_data = home_info:getBuildDataById(self.build_id)
        local cfgs = self:getCfgs()

        self.cache_toggles = {}
        local path = "content/left_bar/Scroll View/Viewport/Content"
        frame_helper:hideAllChild(self, path)
        for __, cfg in ipairs(cfgs) do
            local child = frame_helper:getChild(self, __, path)
            child:SetActive(true)

            local can_train = true
            if cfg.unlock > 0 then
                local tech_cfg = config.home_config_research_details[cfg.unlock]
                local tech_cfg_id = tech_cfg.list_id
                local tech_level = tech_cfg.level
                local cur_level, is_unlock = home_info:getTechLevel(tech_cfg_id)
                can_train = cur_level >= tech_level
            end

            local normal_trans = child.transform:Find("nomal")
            local lock_trans = child.transform:Find("lock")

            frame_helper:setActive(normal_trans, can_train)
            frame_helper:setActive(lock_trans, not can_train)

            local barrack_cfg = cfg
            local soldier_cfg = config.soldier_soldiers[cfg.soldier_id] 

            local toggle = nil
            
            if can_train then
                local name = normal_trans:Find("name"):GetComponent(UnityEngine.UI.Text)
                local icon = normal_trans:Find("icon"):GetComponent(UnityEngine.UI.Image)
                local quality = normal_trans:Find("quality"):GetComponent(UnityEngine.UI.Image)
                local num = normal_trans:Find("num"):GetComponent(UnityEngine.UI.Text)
                name.text = frame_helper:getLangCfg(soldier_cfg.soldiername)

                num.text = string.format(NativeText.GetLangStr(550008), self:getSoldierNum(cfg.soldier_id))
                icon:SetSprite(soldier_cfg.headpath .. soldier_cfg.soldierhead)
                quality:SetSprite(gui_helper:getQualityIconPath(soldier_cfg.color))
                toggle = normal_trans:Find("toggle"):GetComponent(UnityEngine.UI.Image)
                icon.enabled = true
                frame_helper:setImageGray1(icon, false)
            else
                local name = lock_trans:Find("name"):GetComponent(UnityEngine.UI.Text)
                local icon = lock_trans:Find("icon"):GetComponent(UnityEngine.UI.Image)
                local quality = lock_trans:Find("quality"):GetComponent(UnityEngine.UI.Image)
                name.text = frame_helper:getLangCfg(soldier_cfg.soldiername)
                icon:SetSprite(soldier_cfg.headpath .. soldier_cfg.soldierhead)
                quality:SetSprite(gui_helper:getQualityIconPath(soldier_cfg.color))
                toggle = lock_trans:Find("toggle"):GetComponent(UnityEngine.UI.Image)
                icon.enabled = false
                frame_helper:setImageGray1(icon, true)
                frame_helper:setImageGray1(quality, true)
            end

            table.insert(self.cache_toggles, {
                index = __,
                toggle = toggle,
                barrack_cfg = cfg,
                can_train = can_train,
                child = child,
            })

            local btn = child:GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(function()
                self:click_soldier(__, cfg)
            end)
        end
    end,

    isSoldierCanTrain = function(self, soldier_id)
        for __, value in ipairs(self.cache_toggles) do
            if value.barrack_cfg.soldier_id == soldier_id then
                return value.can_train
            end
        end
    end,

    isSoldierIsTraining = function(self, soldier_id)
        if self.training_soldiers then
            return self.training_soldiers.bTran and self.training_soldiers.cfgId == soldier_id
        end
        return false
    end,

    getCanTrainSoldierNum = function(self, barrack_cfg, max_num)
        local cost_table = loadstring("return " .. barrack_cfg.cost)()
        local num = frame_helper.max_number
        for item_id, need_num in pairs(cost_table) do
            local own_num = frame_helper:getItemNum(item_id)
            local single_price = need_num / (1 + self:getPercentRes())

            local floor_num = math.floor(own_num / single_price)
            local ceil_num = math.ceil(own_num / single_price)

            local real_need = self:getRealRes(ceil_num * need_num)
            local rst
            if own_num >= real_need then
                rst = ceil_num
            else
                rst = floor_num
            end
            if rst < num then
                num = rst
            end
        end
        local can_train = num > 0
        if num <= 0 then
            num = 1
        elseif num > max_num then
            num = max_num
        end
        return num, can_train
    end,

    click_soldier = function(self, index, cfg)
        log.info("click_soldier事件：", index, cfg.soldier_id)

        for __, value in ipairs(self.cache_toggles) do
            value.toggle.enabled = index == value.index
        end

        local barrack_cfg = cfg
        local soldier_cfg = config.soldier_soldiers[cfg.soldier_id]

        self.cur_index = index
        self.cur_barrack_cfg = barrack_cfg
        self.cur_soldier_cfg = soldier_cfg

        local name = self:findText("content/name")
        -- local icon = self:findImage("")
        name.text = frame_helper:getLangCfg(soldier_cfg.soldiername)
        -- icon:SetSprite(soldier_cfg.soldier_pic_path .. soldier_cfg.soldier_pic)

        local icon_path = soldier_cfg.soldier_pic_path .. soldier_cfg.soldier_pic
        local start_pos, end_pos = string.find(icon_path, ".asset", 1) 
        local is_spine = start_pos and true or false
        
        local image_trans = self.transform:Find("content/icon/Image")
        frame_helper.hideTrans(image_trans)

        self.cache_spine = self.cache_spine or {}
        if not self.cache_spine[cfg.soldier_id] then
            if is_spine then
                local spine_root = self.transform:Find("content/icon")
                local spine_child = GameObject()
                spine_child.name = "soldier id:" .. cfg.soldier_id
                spine_child:SetParent(spine_root)
                frame_helper:resetTransform(spine_child)
                self.cache_spine[cfg.soldier_id] = spine_child
                self:loadSpine(icon_path, spine_child)
            else
                self.cache_spine[cfg.soldier_id] = image_trans.gameObject
            end
        end

        for soldier_id, spine_child in pairs(self.cache_spine) do
            if soldier_id == cfg.soldier_id then
                if is_spine then
                    spine_child.transform.localScale = Vector3.one * soldier_cfg.SpineZoom / 10000
                    local coord = loadstring("return " .. soldier_cfg.position)()
                    spine_child.transform.localPosition = Vector3(coord[1], coord[2], 0)
                else
                    local icon = self:findImage("content/icon/Image")
                    icon:SetSprite(icon_path)
                end
            else
                spine_child.transform.localScale = Vector3.zero
            end
        end

        if not is_spine then
            frame_helper.showTrans(image_trans)
        end

        local un_upgrade_trans = self.transform:Find("content/un_upgrade")
        local upgrading_trans = self.transform:Find("content/upgrading")
        local lock_trans = self.transform:Find("content/lock")
        local delete_trans = self.transform:Find("content/deleteBtn")
        local max_trans = self.transform:Find("content/max")

        frame_helper:setActive(delete_trans, home_info:getAvailableSoldierNum(barrack_cfg.soldier_id) > 0)
        frame_helper:setActive(un_upgrade_trans, false)
        frame_helper:setActive(upgrading_trans , false)
        frame_helper:setActive(lock_trans, false)
        frame_helper:setActive(max_trans, false)

        self.cur_cost_diamond = nil
        local is_training = self:isSoldierIsTraining(cfg.soldier_id)
        if is_training then
            frame_helper.showTrans(upgrading_trans)
            self:setUpgradingInfo()
        else
            local can_train = self:isSoldierCanTrain(cfg.soldier_id)
            if not can_train then
                local detail_cfg = config.home_config_research_details[cfg.unlock]
                local tech_cfg = config.home_config_research_lists[detail_cfg.list_id]
                local des = self:findText("content/lock/text1_Static[T4]_4")
                des.text = string.format(NativeText.GetLangStr(550009), frame_helper:getLangCfg(tech_cfg.name_id))
                frame_helper.showTrans(lock_trans)
            else
                local max_num = self:getMaxNum()
                -- local max_num = 1
                if max_num <= 0 then
                    frame_helper.showTrans(max_trans)
                else
                    frame_helper.showTrans(un_upgrade_trans)
                    self.soldier_slider = self:findSlider("content/un_upgrade/slider")
                    self.soldier_cur_num = self:getCanTrainSoldierNum(cfg, max_num)
                    self:removeSliderListener()
                    self.soldier_max_num = self:getMaxNum()
                    self.soldier_slider.value = self.soldier_cur_num / self.soldier_max_num
                    self:setSoldierInfo(barrack_cfg, self.soldier_cur_num, self.soldier_max_num)
                    self:addSliderListener()
                end
            end
        end

        -- 刷新右上标识
        local getSoldierTypeCfg = function(type_id)
            local rst = nil
            config.home_config_soldier_type_icons:foreach(function(id, cfg)
                if cfg.type == type_id then
                    rst = cfg
                end
            end)
            return rst
        end

        local type_image = self:findImage("content/right_bar/self_type")
        type_image:SetSprite(getSoldierTypeCfg(self.cur_barrack_cfg.type).icon)
        
        local path = "content/right_bar/counter"
        frame_helper:hideAllChild(self, path)
        for __, type_id in ipairs(loadstring("return " .. self.cur_barrack_cfg.counter)()) do
            local child = frame_helper:getChild(self, __, path)
            child:SetActive(true)
            local image = child:GetComponent(UnityEngine.UI.Image)
            image:SetSprite(getSoldierTypeCfg(type_id).icon)
        end

        local path = "content/right_bar/countered"
        frame_helper:hideAllChild(self, path)
        for __, type_id in ipairs(loadstring("return " .. self.cur_barrack_cfg.countered)()) do
            local child = frame_helper:getChild(self, __, path)
            child:SetActive(true)
            local image = child:GetComponent(UnityEngine.UI.Image)
            image:SetSprite(getSoldierTypeCfg(type_id).icon)
        end

        local cur_tab = {
            self.cur_barrack_cfg.display_hp,
            self.cur_barrack_cfg.display_def,
            self.cur_barrack_cfg.display_atk,
            self.cur_barrack_cfg.display_spd,
            self.cur_barrack_cfg.display_cargo,
            self.cur_barrack_cfg.display_upkeep,
        }

        local max_tab = {
            config.home_config_configs[1].display_max_hp,
            config.home_config_configs[1].display_max_def,
            config.home_config_configs[1].display_max_atk,
            config.home_config_configs[1].display_max_spd,
            config.home_config_configs[1].display_max_cargo,
            config.home_config_configs[1].display_max_upkeep,
        }

        local rst = {}

        for i = 1, 6 do 
            local des = self:findText(string.format("content/UIPolygon/prop/text (%s)", i))
            local content = string.format(NativeText.GetLangStr(550015 + i - 1), cur_tab[i])
            content = content:gsub("\\n", "\n")
            des.text = content

            rst[i] = cur_tab[i] / max_tab[i]
        end

        local param = {
            rst[3],
            rst[4],
            rst[5],
            rst[6],
            rst[1],
            rst[2],
            rst[3],
        }

        local polygon = self.transform:Find("content/UIPolygon/pl"):GetComponent("UnityEngine.UI.Extensions.UIPolygon")
        polygon:DrawPolygon(6, param)
    end,

    onSliderValueChange = function(self)
        local temp = self.soldier_slider.value * self.soldier_max_num
        if temp <= 0 then 
            temp = 1 
        else
            temp = math.ceil(temp)
        end
        if temp ~= self.soldier_cur_num then
            self.soldier_cur_num = temp
            self:setSoldierInfo(self.cur_barrack_cfg, self.soldier_cur_num, self.soldier_max_num)
        end
    end,

    addSliderListener = function(self)
        self.soldier_slider.onValueChanged:RemoveAllListeners()
        self.soldier_slider.onValueChanged:AddListener(function()
            self:onSliderValueChange()
        end)
    end,

    removeSliderListener = function(self)
        self.soldier_slider.onValueChanged:RemoveAllListeners()
    end,

    click_reduceBtn = function(self)
        self:removeSliderListener()
        if self.soldier_cur_num > 1 then
            self.soldier_cur_num = self.soldier_cur_num - 1
            local rate = self.soldier_cur_num / self.soldier_max_num
            self.soldier_slider.value = rate 
            self:setSoldierInfo(self.cur_barrack_cfg, self.soldier_cur_num, self.soldier_max_num)
        end
        self:addSliderListener()
    end,

    click_addBtn = function(self)
        self:removeSliderListener()
        if self.soldier_cur_num < self.soldier_max_num then
            self.soldier_cur_num = self.soldier_cur_num + 1
            local rate = self.soldier_cur_num / self.soldier_max_num
            self.soldier_slider.value = rate 
            self:setSoldierInfo(self.cur_barrack_cfg, self.soldier_cur_num, self.soldier_max_num)
        end
        self:addSliderListener()
    end,

    setSoldierInfo = function(self, barrack_cfg, cur_num, max_num)
        local num = self:findText("content/un_upgrade/num")
        num.text = cur_num .. "/" .. max_num

        self.lack_tab = {}

        local path = "content/un_upgrade/Right Scroll View/Viewport/Content"
        local idx = 1
        local res_cost_diamond = 0
        local cost_table = loadstring("return " .. barrack_cfg.cost)()
        for item_id, need_num in pairs(cost_table) do
            local child = frame_helper:getChild(self, idx, path)
            child:SetActive(true)

            local own_num = frame_helper:getItemNum(item_id)
            local total_need_num = self:getRealRes(need_num * cur_num)

            print("需要的资源id:", item_id, "本来数量：", need_num * cur_num, "实际数量：", total_need_num)

            local str ,full_condition = frame_helper:getItemNumFormat(item_id, total_need_num)
            local lack_num = total_need_num - own_num
            local temp = frame_helper:getResToDiamondNum(item_id, lack_num)
            res_cost_diamond = res_cost_diamond + temp

            if not full_condition then
                self.lack_tab[item_id] = item_num
            end
            
            local bg = child.transform:Find("bg"):GetComponent(UnityEngine.UI.Image)
            local icon = child.transform:Find("icon"):GetComponent(UnityEngine.UI.Image)
            local num = child.transform:Find("num"):GetComponent(UnityEngine.UI.Text)
            local right_icon = child.transform:Find("rightIcon"):GetComponent(UnityEngine.UI.Image)
            local wrongIcon = child.transform:Find("wrongIcon"):GetComponent(UnityEngine.UI.Image)

            bg.enabled = not full_condition
            right_icon.enabled = full_condition
            wrongIcon.enabled = not full_condition
            icon:SetSprite(frame_helper:getItemIcon(item_id))
            num.text = str

            local btn_trans = child.transform:Find("addBtn")
            frame_helper:setActive(btn_trans, not full_condition)
            local btn = btn_trans:GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(function()
                if not full_condition then
                    frame_manager:show('resource_exchange', nil, {
                        item_id = item_id,
                        item_num = total_need_num,
                    })
                end
            end)
            idx = idx + 1
        end

        local speed_up_type = home_speed_up_type.soldier_create
        local total_time = self:getRealTime(barrack_cfg.time * cur_num)
        local time_cost_diamond = gui_helper:getSpeedUpCost(speed_up_type, total_time)
        local total_diamond = res_cost_diamond + time_cost_diamond

        print("资源兑换成钻石总量：", res_cost_diamond)
        print("时间兑换成钻石总量：", time_cost_diamond)

        local DIAMOND    = 30001 -- 钻石
        local item_name = gui_helper:getImageTextItemStr(DIAMOND)
        local image_text = self:findText("content/un_upgrade/finishBtn/image_text")
        image_text:SetImageText(string.format("%s %s", item_name, total_diamond))
        self.cur_cost_diamond = total_diamond

        local unreal_upgrade_time = self:findText("content/un_upgrade/trainBtn/unreal")
        local real_upgrade_time = self:findText("content/un_upgrade/trainBtn/real")
        unreal_upgrade_time.text = string.format(NativeText.GetLangStr(290017), gui_helper:timeFormat(barrack_cfg.time * cur_num)) 
        real_upgrade_time.text = gui_helper:timeFormat(total_time)
    end,

    getMaxNum = function(self)
        local prop = local_player.globale_props
        local type_id = self.cur_barrack_cfg.type

        if type_id == 1 or type_id == 2 or type_id == 3 or type_id == 4 then
            -- 基本训练量
            local base_num = local_player:getGlobalProp(common_pb.SYS_ALL_CORP_TRAINING_NUM)
            local base_rate = local_player:getGlobalProp(common_pb.SYS_ALL_CORP_TRAINING_NUM_PERCENT) / 10000
            -- 具体兵种训练量
            local soldier_num = local_player:getGlobalProp(common_pb["SYS_CORP" .. type_id .. "_TRAINING_NUM"])
            local soldier_rate = local_player:getGlobalProp(common_pb["SYS_CORP" .. type_id .. "_TRAINING_NUM_PERCENT"]) / 10000

            local rst = ( base_num + soldier_num) * ( 1 + base_rate + soldier_rate)
            -- 总训练量
            return math.floor(rst)
        elseif type_id == 11 or type_id == 12 or type_id == 13 then
            -- 上限
            local max_num = local_player:getGlobalProp(common_pb.SYS_MAX_DEFENSE_WEAPON_NUM) * (1 + local_player:getGlobalProp(common_pb.SYS_MAX_DEFENSE_WEAPON_NUM_PERCENT) / 10000)
            max_num = math.floor(max_num)
            -- （防御武器单次制造数量+火神炮单次制造数量）×（10000+防御武器单次制造数量加成百分比）/10000
             -- 训练量
            local base_num = local_player:getGlobalProp(common_pb.SYS_DEFENSE_WEAPON_PRODUCE_NUM)
            local soldier_num = local_player:getGlobalProp(common_pb["SYS_CORP" .. type_id .. "_PRODUCE_NUM"])
            local rst = ( base_num + soldier_num ) * ( 10000 + local_player:getGlobalProp(common_pb.SYS_DEFENSE_WEAPON_PRODUCE_NUM_ADD_PERCENT)) / 10000
            rst = math.floor(rst)
            -- 当前的拥有的数量
            -- local cfg_id_list = {}
            -- config.soldier_soldiers:foreach(function(id, cfg)
            --     if cfg.type == type_id then
            --         table.insert(cfg_id_list, id)
            --     end
            -- end)
            -- local cur_num = 0
            -- local build_data = home_info:getBuildDataByCfgId(24)
            -- for k, v in ipairs(build_data.guardsystem.traps) do
            --     local temp = false
            --     for _, cfg_id in ipairs(cfg_id_list) do
            --         if v.cfgId == cfg_id then
            --             temp = true
            --             break
            --         end
            --     end
            --     if temp then
            --         cur_num = cur_num + v.totalNum
            --     end
            -- end
            local cur_num = 0
            local build_data = home_info:getBuildDataByCfgId(24)
            for k, v in ipairs(build_data.guardsystem.traps) do
                cur_num = cur_num + v.totalNum
            end
            -- 剩余空间
            local left_num = max_num - cur_num
            if rst <=  left_num then
                return rst
            else
                return left_num
            end
        end
    end,

    getPercentTime = function(self)
        local prop = local_player.globale_props
        local percent = nil
        if self.cur_barrack_cfg.type == 1 then
            percent = ( local_player:getGlobalProp(common_pb.SYS_CORP1_TRAINING_SPEED_PERCENT) + local_player:getGlobalProp(common_pb.SYS_SOLDIER_PRODUCE_SPEED_PERCENT) ) / 10000
        elseif self.cur_barrack_cfg.type == 2 then
            percent = ( local_player:getGlobalProp(common_pb.SYS_CORP2_TRAINING_SPEED_PERCENT) + local_player:getGlobalProp(common_pb.SYS_SOLDIER_PRODUCE_SPEED_PERCENT) ) / 10000
        elseif self.cur_barrack_cfg.type == 3 then
            percent = ( local_player:getGlobalProp(common_pb.SYS_CORP3_TRAINING_SPEED_PERCENT) + local_player:getGlobalProp(common_pb.SYS_SOLDIER_PRODUCE_SPEED_PERCENT) ) / 10000
        elseif self.cur_barrack_cfg.type == 4 then
            percent = ( local_player:getGlobalProp(common_pb.SYS_CORP4_TRAINING_SPEED_PERCENT) + local_player:getGlobalProp(common_pb.SYS_SOLDIER_PRODUCE_SPEED_PERCENT) ) / 10000
        elseif self.cur_barrack_cfg.type == 11 then
            percent = local_player:getGlobalProp(common_pb.SYS_DEFENSE_WEAPON_PRODUCE_SPEED_PERCENT) / 10000
        elseif self.cur_barrack_cfg.type == 12 then
            percent = local_player:getGlobalProp(common_pb.SYS_DEFENSE_WEAPON_PRODUCE_SPEED_PERCENT) / 10000
        elseif self.cur_barrack_cfg.type == 13 then
            percent = local_player:getGlobalProp(common_pb.SYS_DEFENSE_WEAPON_PRODUCE_SPEED_PERCENT) / 10000
        end
        return percent
    end,

    getPercentRes = function(self)
        local prop = local_player.globale_props
        local percent = nil
        if self.cur_barrack_cfg.type == 1 then
            percent = ( local_player:getGlobalProp(common_pb.SYS_CORP1_TRAINING_COST_REDUCE_PERCENT) + local_player:getGlobalProp(common_pb.SYS_SOLDIER_PRODUCE_COST_PERCENT) ) / 10000
        elseif self.cur_barrack_cfg.type == 2 then
            percent = ( local_player:getGlobalProp(common_pb.SYS_CORP2_TRAINING_COST_REDUCE_PERCENT) + local_player:getGlobalProp(common_pb.SYS_SOLDIER_PRODUCE_COST_PERCENT) ) / 10000
        elseif self.cur_barrack_cfg.type == 3 then
            percent = ( local_player:getGlobalProp(common_pb.SYS_CORP3_TRAINING_COST_REDUCE_PERCENT) + local_player:getGlobalProp(common_pb.SYS_SOLDIER_PRODUCE_COST_PERCENT) ) / 10000
        elseif self.cur_barrack_cfg.type == 4 then
            percent = ( local_player:getGlobalProp(common_pb.SYS_CORP4_TRAINING_COST_REDUCE_PERCENT) + local_player:getGlobalProp(common_pb.SYS_SOLDIER_PRODUCE_COST_PERCENT) ) / 10000
        elseif self.cur_barrack_cfg.type == 11 then
            percent = local_player:getGlobalProp(common_pb.SYS_DEFENSE_WEAPON_PRODUCE_COST_PERCENT) / 10000
        elseif self.cur_barrack_cfg.type == 12 then
            percent = local_player:getGlobalProp(common_pb.SYS_DEFENSE_WEAPON_PRODUCE_COST_PERCENT) / 10000
        elseif self.cur_barrack_cfg.type == 13 then
            percent = local_player:getGlobalProp(common_pb.SYS_DEFENSE_WEAPON_PRODUCE_COST_PERCENT) / 10000
        end
        return percent
    end,

    -- 拿到实际时间
    getRealTime = function(self, expect_time)
        local real_time = expect_time / (1 + self:getPercentTime())
        real_time = tostring(real_time)
        real_time = tonumber(real_time)
        if real_time <= 0 then
            return 0
        else
            real_time = math.floor(real_time)
            return real_time == 0 and 1 or real_time
        end
    end,

    -- 拿到实际资源
    getRealRes = function(self, expect_res)
        local real_res = expect_res / (1 + self:getPercentRes())
        real_res = tostring(real_res)
        real_res = tonumber(real_res)
        if real_res <= 0 then
            return 0
        else
            real_res = math.floor(real_res)
            return real_res == 0 and 1 or real_res
        end
    end,

    getDefaultSoldierIndex = function(self, cfgs)
        local rst = 1
        for index, cfg in ipairs(cfgs) do
            local can_train = true
            if cfg.unlock > 0 then
                local tech_cfg = config.home_config_research_details[cfg.unlock]
                local tech_cfg_id = tech_cfg.list_id
                local tech_level = tech_cfg.level
                can_train = home_info:getTechLevel(tech_cfg_id) >= tech_level
            end
            if can_train then
                if index > rst then
                    rst = index
                end
            end
        end
        return rst
    end,

    getCfgs = function(self)
        if self.cook_cfgs then
            return self.cook_cfgs
        else
            local build_data = home_info:getBuildDataById(self.build_id)
            if not build_data then
                return
            end
            local is_trap = self:isTrapBuild()
            local build_cfg_id = build_data.cfgId
            local rst = {}
            config.home_config_barracks:foreach(function(id, cfg)
                if is_trap then
                    if cfg.building == build_cfg_id and cfg.type == self.soldier_type then
                        table.insert(rst, cfg)
                    end
                else
                    if cfg.building == build_cfg_id then
                        table.insert(rst, cfg)
                    end
                end
            end)
            local sortFunc = function(a, b)
                return a.level < b.level
            end
            table.sort(rst, sortFunc)
            self.cook_cfgs = rst
            return self.cook_cfgs
        end
    end,

    getPassTime = function(self)
        return Time.realtimeSinceStartup - self.build_data.rmbTime
    end,

    click_finishBtn1 = function(self)
        local build_data = home_info:getBuildDataById(self.build_id)
        if frame_helper:isEnoughDiamond(self.cur_cost_diamond) then
            network_sender:perSonBaseSpeedUp(self.build_id, common_pb.EPB_PRO_SOLDIER, 1, 0, {})
        end
    end,

    click_speedUpBtn = function(self)
        if self:isTrapBuild() then
            frame_manager:show("home_speed_up", nil, {
                speed_up_type = home_speed_up_type.trap_create,
                id = self.build_id
            })
        else
            frame_manager:show("home_speed_up", nil, {
                speed_up_type = home_speed_up_type.soldier_create,
                id = self.build_id
            })
        end
    end,

    setUpgradingInfo = function(self)
        local temp_time = math.floor(self.training_soldiers.reamintime - self:getPassTime())
        if self:isTrapBuild() then
            local can_rcv = self.build_data.guardsystem.build.canRecv 
            self:onTickTrainTime(self.build_data, temp_time, self.training_soldiers.totaltime, can_rcv)
        else
            local can_rcv = self.build_data.barrack.canRecv
            self:onTickTrainTime(self.build_data, temp_time, self.training_soldiers.totaltime, can_rcv)
        end
    end,

    onTickTrainTime = function(self, build_data, left_time, total_time, can_rcv)
        local un_receive_trans = self.transform:Find("content/upgrading/un_receive")
        local receive_trans = self.transform:Find("content/upgrading/receive")
        local slider = self:findSlider("content/upgrading/slider")
        local image_text = self:findText("content/upgrading/un_receive/finishBtn1/image_text")
        local time = self:findText("content/upgrading/time")

        frame_helper:setActive(un_receive_trans, not can_rcv)
        frame_helper:setActive(receive_trans, can_rcv)

        if left_time <= 0 then
            slider.value = 1
            time.text = NativeText.GetLangStr(550014)
        else
            local speed_up_type = home_speed_up_type.soldier_create
            local DIAMOND    = 30001 -- 钻石
            local item_name = gui_helper:getImageTextItemStr(DIAMOND)
            self.cur_cost_diamond = gui_helper:getSpeedUpCost(speed_up_type, left_time)
            image_text:SetImageText(string.format("%s %s", item_name, self.cur_cost_diamond))
            time.text = NativeText.GetLangStr(550011) .. frame_helper:timeFormat(left_time)
            slider.value = 1 - left_time / total_time
        end
    end,

    click_goBtn = function(self)
        frame_manager:hide('home_barrack')
        frame_manager:hide('home_defense')
        local cfg = self.cur_barrack_cfg
        if cfg.unlock > 0 then
            local tech_cfg = config.home_config_research_details[cfg.unlock]
            local tech_cfg_id = tech_cfg.list_id
            local tech_level = tech_cfg.level
            local cur_level, has_build = home_info:getTechLevel(tech_cfg_id)
            local can_train = cur_level >= tech_level

            print("科技id:", tech_cfg.list_id, "等级:",tech_cfg.level, home_info:getTechLevel(tech_cfg_id))
            if not has_build then
                hint:setText(NativeText.GetLangStr(550027))
            elseif not can_train then
                local build_data = home_info:getBuildDataByCfgId(2)
                if build_data then

                    self:click_backBtn()
                    local model_trans = self.home_map:getHomeBuildById(build_data.id).model.transform
                    self.home_map.home_camera:LookAt(model_trans, Vector3(UnityEngine.Screen.width / 2, UnityEngine.Screen.height / 2, 0), config.home_config_lists[build_data.cfgId].upgrade_fov, function()
                        local frame = frame_manager:show('home_entity_research', self.home_map, build_data.id)
                        frame:addCommandBuff(function()
                            local target_cfg = nil
                            config.home_config_research_lists:foreach(function(id, cfg)
                                if id == tech_cfg_id then
                                    target_cfg = cfg
                                end
                            end)
                            if target_cfg then
                                frame:click_category(target_cfg.type)
                                frame:click_tech(target_cfg)
                            end
                        end)
                    end)
                    -- app:startCoroutine(function ()
                    --     self:click_backBtn()
                    --     local model_trans = self.home_map:getHomeBuildById(build_data.id).model.transform
                    --     self.home_map.home_camera:LookAt(model_trans, Vector3(UnityEngine.Screen.width / 2, UnityEngine.Screen.height / 2, 0), config.home_config_lists[build_data.cfgId].upgrade_fov)
                    --     coroutine.yield(WaitForSeconds(0.8))
                    --     local frame = frame_manager:show('home_entity_research', self.home_map, build_data.id)
                    --     frame:addCommandBuff(function()
                    --         local target_cfg = nil
                    --         config.home_config_research_lists:foreach(function(id, cfg)
                    --             if id == tech_cfg_id then
                    --                 target_cfg = cfg
                    --             end
                    --         end)
                    --         if target_cfg then
                    --             frame:click_category(target_cfg.type)
                    --             frame:click_tech(target_cfg)
                    --         end
                    --     end)
                    -- end)
                end
            end
        end
    end,

    isTrapBuild = function(self)
        local trap_cfg_id = 24
        return self.build_data.cfgId == trap_cfg_id
    end,

    click_receiveBtn = function(self)
        local build_data = home_info:getBuildDataById(self.build_id)
        network_sender:receiveSoldier(self.build_id)
    end,

    click_backBtn = function(self)
        frame_manager:hide('home_barrack')
    end,

    click_deleteBtn = function(self)
        local num = self:getSoldierNum(self.cur_barrack_cfg.soldier_id)
        frame_manager:show('home_delete_soldier', nil, {
            soldier_id = self.cur_barrack_cfg.soldier_id,
            soldier_max_num = num,
            state = 0,
            be_hurted = false,
        })
    end,
    
    click_infoBtn = function(self)
    end,

    click_finishBtn = function(self)
        if self.training_soldiers.bTran and self.training_soldiers.cfgId ~= self.cur_barrack_cfg.soldier_id then
            hint:setText(NativeText.GetLangStr(550026))
            return
        end
        if self.soldier_cur_num <= 0 then
            hint:setText(NativeText.GetLangStr(550026))
        elseif frame_helper:isEnoughDiamond(self.cur_cost_diamond) then
            network_sender:trainSoldier(self.build_id, self.cur_barrack_cfg.soldier_id, self.soldier_cur_num, 1)
        end
    end,

    click_trainBtn = function(self)
        -- 别的兵种处于训练中
        if self.training_soldiers.bTran and self.training_soldiers.cfgId ~= self.cur_barrack_cfg.soldier_id then
            if self:isTrapBuild() then
                hint:setText(NativeText.GetLangStr(550031))
            else
                hint:setText(NativeText.GetLangStr(550026))
            end
            return
        end

        -- 面板缺少的资源，不是实际资源
        if self.lack_tab then
            for item_id, need_num in pairs(self.lack_tab) do
                frame_helper:showCommonBox(NativeText.GetLangStr(290022), function()
                    frame_manager:show('resource_exchange', nil, {
                        item_id = item_id,
                        item_num = need_num,
                    })
                end)
                return  
            end
        end

        print("发送消息：",self.build_id, self.cur_barrack_cfg.soldier_id, self.soldier_cur_num, 0)
        network_sender:trainSoldier(self.build_id, self.cur_barrack_cfg.soldier_id, self.soldier_cur_num, 0)
    end,


    loadSpine = function(self, path, parent_go)
        app:startCoroutine(
                function()
                    coroutine.yield(Fenix.Assets.Helper.WaitForAsset(path, Spine.Unity.SkeletonDataAsset, function(asset)
                        -- 释放原来的资源
                        if asset then
                            self.spine_assets = self.spine_assets or {}
                            table.insert(self.spine_assets, asset)
                            asset:AddRef()                             
                            local skeleton_data = asset:Get() -- skeleton data asset
                            local skeleton_graphic = Spine.Unity.SkeletonGraphic.AddSkeletonGraphicComponent(parent_go, skeleton_data)
                            skeleton_graphic.material = game_director.spine_material
                            local anim_state = skeleton_graphic.AnimationState
                            if anim_state then
                                anim_state:SetAnimation(0, 'idle', true)
                            else
                                log.error("Failed to find anim state:"..path)
                            end   
                        else
                            log.error("Failed to load spine asset:"..path)
                        end
                    end))
                end
            )
    end,

    unloadSpine = function(self, slot)
        if self.spine_assets then
            for path, asset in ipairs(self.spine_assets) do
                asset:RemoveRef()
            end
            self.spine_assets = nil
        end
    end,
}

