local super = require("ui.frame")

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        self.msg = self.args.data

        clickEvents = 
        {
            "content/backBtn",
        }

        -- 注册点击事件
        frame_helper:regClickEvents("click", self, clickEvents)

        self.cache_type = {}
        self.cache_group = {}

        self.event_id1 = buff_info.events:on("BUFF_INFO_CHANGE", function()
            self:refresh()
        end)
        self.event_id2 = bag_manager.events:on('onBagChanged', function()
            self:refresh()
        end)
        self:refresh()
    end,

    onUnload = function (self)
        buff_info.events:off("BUFF_INFO_CHANGE", self.event_id1)
        bag_manager.events:off("onBagChanged", self.event_id2)

        if self.cache_effect then
            for __, value in pairs(self.cache_effect) do
                if value.model then
                    value.model:destroy()
                end
            end
        end
    end,

    refresh = function(self)
        self:init()
        self:calculateHeight()
    end,

    getBuffInfo = function(self, group_id)
        local cfgs = self:getCfgsByGroupId(group_id)
        for __, c in ipairs(cfgs) do
            local buff_data = buff_info:getBuffByBuffId(c.id)
            if buff_data then
                return buff_data
            end
        end
        return nil
    end,

    init = function(self)
        local getCacheTypeData = function(type_id)
            for __, value in ipairs(self.cache_type) do
                if value.type_id == type_id then
                    return value
                end
            end
            return nil
        end

        local getCacheGroupData = function(group_id)
            for __, value in ipairs(self.cache_group) do
                if value.group_id == group_id then
                    return value
                end
            end
            return nil
        end

        
        local mask_trans = self.transform:Find("content/Scroll View/Viewport")
        ParticleMask.SetClip(mask_trans, mask_trans)

        self.aaaaa = {}

        local big_path = "content/Scroll View/Viewport/Content/"
        frame_helper:hideAllChild(self, big_path)
        local cook_cfgs = self:cookCfgs()
        for __, value in ipairs(cook_cfgs) do
            local child = frame_helper:getChild(self, __, big_path)
            child:SetActive(true)
            local title = child.transform:Find("title/title"):GetComponent(UnityEngine.UI.Text)
            title.text = frame_helper:getLangCfg(value.name_id)

            local data = getCacheTypeData(value.type_id)
            if not data then
                local init_height = 78 
                local per_small_type_height = 186
                local no_expend_height = init_height + per_small_type_height * #value.list
                local rect = child:GetComponent(UnityEngine.RectTransform)
                table.insert(self.cache_type, {
                    type_id = value.type_id,
                    rect = rect,
                    no_expend_height = no_expend_height,
                    group_ids = value.list
                })
            end

            local small_path = big_path .. child.name .. "/layout/"
            frame_helper:hideAllChild(self, small_path)
            for idx, group_id in ipairs(value.list) do
                local child = frame_helper:getChild(self, idx, small_path)
                child:SetActive(true)
                local icon = child.transform:Find("bg/icon"):GetComponent(UnityEngine.UI.Image)
                local btn = child.transform:Find("bg/expendBtn"):GetComponent(UnityEngine.UI.Button)
                local btn_image = child.transform:Find("bg/expendBtn/icon"):GetComponent(UnityEngine.UI.Image)
                local rect = child:GetComponent(UnityEngine.RectTransform)
                local un_use_trans = child.transform:Find("bg/un_use")
                local using_trans = child.transform:Find("bg/using")

                local group_cfg = config.home_config_buff_groups[group_id]

                print("group_id:", group_id)

                icon:SetSprite(group_cfg.group_icon)
                btn_image.enabled = group_cfg.can_expand == 1

                local data = getCacheGroupData(group_id)
                if not data then
                    table.insert(self.cache_group, {
                        group_id = group_id,
                        rect = rect,
                        no_expend_height = 166,
                        expend_child_count = 0,
                        expend_state = false,
                    })
                end

                local buff_data = self:getBuffInfo(group_id)
                if buff_data then
                    frame_helper.showTrans(using_trans)
                    frame_helper.hideTrans(un_use_trans)
                    local name = child.transform:Find("bg/using/name_text"):GetComponent(UnityEngine.UI.Text)
                    local progress = child.transform:Find("bg/using/progress_text"):GetComponent(UnityEngine.UI.Text)
                    local slider = child.transform:Find("bg/using/slider"):GetComponent(UnityEngine.UI.Slider)
                    name.text = frame_helper:getLangCfg(config.buff_buffs[buff_data.buff_id].buff_name)
                    local left_time = buff_info:getLeftTimeByBuffId(buff_data.buff_id)
                    progress.text = NativeText.GetLangStr(530006) .. frame_helper:timeFormat(left_time)
                    slider.value = left_time / buff_data.total_time
                    local clk_trans =  child.transform:Find("bg/using/progress_text")
                    local clk = clk_trans:GetComponent(Clock)
                    if not clk then clk = clk_trans:AddComponent(Clock) end
                    clk.prefix = NativeText.GetLangStr(530006)
                    clk:Set2(left_time, function(time)
                        slider.value = time / buff_data.total_time
                    end)
                    
                else
                    frame_helper.hideTrans(using_trans)
                    frame_helper.showTrans(un_use_trans)
                    local name = child.transform:Find("bg/un_use/name_text"):GetComponent(UnityEngine.UI.Text)
                    local des = child.transform:Find("bg/un_use/des_text"):GetComponent(UnityEngine.UI.Text)
                    name.text = frame_helper:getLangCfg(group_cfg.group_name)
                    des.text = frame_helper:getLangCfg(group_cfg.group_des)
                end

                
                -- log.error("##################################", group_id, idx, buff_data)
                -- self.cache_effect = self.cache_effect or {}

                -- local temp_idx = group_id

                -- if buff_data then
                --     if self.cache_effect[temp_idx] then
                --         if self.cache_effect[temp_idx].model_trans then
                --             frame_helper.showTrans(self.cache_effect[temp_idx].model_trans)
                --             log.error("模型显示:", temp_idx)
                --         end
                --     else
                --         log.error("地方了看见第三方丽江的是流放:", temp_idx)
                --         self.cache_effect[temp_idx] = {}

                --         local callback = function (model_go)
                --             if model_go then
                --                 model_go.transform.parent = icon.transform
                --                 frame_helper:resetTransform(model_go.transform)
        
                --                 self.cache_effect[temp_idx].model_trans = model_go.transform
        
                --                 log.error("模型加载完成:", temp_idx)
        
                                
                --                 local mask_trans = self.transform:Find("content/Scroll View/Viewport")
                --                 ParticleMask.SetClip(mask_trans, mask_trans)
                --             else
                --                 log.error("加载模型失败:", model_path)
                --             end
                --         end

                --         local new_uobject = game_director.pool:instantiate("Assets/Arts/UIeffcet/Prefab/effect_buff_activate.prefab")
                --         if new_uobject.isLoaded then 
                --             callback(new_uobject)
                --         else 
                --             new_uobject.onLoaded = callback
                --         end
                --     end
                -- else
                --     if self.cache_effect[temp_idx] then
                --         if self.cache_effect[temp_idx].model_trans then
                --             frame_helper.hideTrans(self.cache_effect[temp_idx].model_trans)
                --             log.error("模型隐藏:", temp_idx)
                --         end
                --     end
                -- end


                local effect_buff_activate = child.transform:Find("bg/icon/effect_buff_activate")
                effect_buff_activate:SetActive(buff_data and true or false)
                effect_buff_activate.localPosition = Vector3.one

                if buff_data then
                    table.insert(self.aaaaa, effect_buff_activate)
                end

                print("group_cfg.can_expend:", group_cfg.can_expand)

                local refreshGroupDetailItems = function(expend_state, cfgs)
                    local expend_path = small_path .. child.name .. "/items_items"
                    frame_helper:hideAllChild(self, expend_path)
                    if expend_state then
                        local count = 0
                        for k, v in ipairs(cfgs) do
                            local buff_cfg = config.buff_buffs[v.buff_id]
                            local item_id = buff_cfg.item
                            local own_num = frame_helper:getItemNum(item_id)
                            if (not (v.is_always_display == 0 and own_num == 0)) or v.is_always_display == 1 then
                                count = count + 1
                                local child = frame_helper:getChild(self, count, expend_path)
                                child:SetActive(true)

                                local icon = child.transform:Find("icon"):GetComponent(UnityEngine.UI.Image)
                                local quality = child.transform:Find("out_bg"):GetComponent(UnityEngine.UI.Image)
                                local name = child.transform:Find("name_text"):GetComponent(UnityEngine.UI.Text)
                                local des = child.transform:Find("des_text"):GetComponent(UnityEngine.UI.Text)
                                local own = child.transform:Find("own_text"):GetComponent(UnityEngine.UI.Text)
                                local light = child.transform:Find("GameObject"):GetComponent(UnityEngine.UI.Image)
                                
                                local buy_trans = child.transform:Find("buyTrans")
                                local use_trans = child.transform:Find("useTrans")
                                
                                local item_cfg = item_helper:getItemCfg(item_id)
                                icon:SetSprite(frame_helper:getItemIcon(item_id))
                                quality:SetSprite(frame_helper:getItemQualityIcon(item_id))
                                name.text = frame_helper:getLangCfg(item_cfg.name_id)
                                des.text = frame_helper:getLangCfg(item_cfg.des_id)
                                own.text = string.format(NativeText.GetLangStr(own_num > 0 and 250008 or 250010), own_num) 
                                light.enabled = k == 1

                                if own_num > 0 then
                                    frame_helper.showTrans(use_trans)
                                    frame_helper.hideTrans(buy_trans)
                                else
                                    frame_helper.showTrans(buy_trans)
                                    frame_helper.hideTrans(use_trans)

                                    local image_text = buy_trans.transform:Find("image_text")
                                    local DIAMOND    = 30001 -- 钻石
                                    local image_text = buy_trans.transform:Find("image_text"):GetComponent(UnityEngine.UI.Text)
                                    local item_name = gui_helper:getImageTextItemStr(DIAMOND)
                                    local single_price = frame_helper:getItemBuyPrice(item_id)
                                    image_text:SetImageText(string.format("%s %s", item_name, single_price))
                                end 

                                local btn = child:Find("useTrans/useBtn"):GetComponent(UnityEngine.UI.Button)
                                btn.onClick:RemoveAllListeners()
                                btn.onClick:AddListener(function ()
                                    print("使用单个")
                                    self:useItems(group_id, v.buff_id, item_id, 1)
                                end)

                                local btn = child:Find("buyTrans/buyBtn"):GetComponent(UnityEngine.UI.Button)
                                btn.onClick:RemoveAllListeners()
                                btn.onClick:AddListener(function ()
                                    print("购买使用")
                                    self:buyItems(group_id, v.buff_id, item_id, 1)
                                end)
                            end
                        end
                        return count
                    else
                        return 0
                    end
                end

                if group_cfg.can_expand == 1 then
                    local cfgs = self:getCfgsByGroupId(group_id)
                    if cfgs and #cfgs > 0 then
                        local data = getCacheGroupData(group_id)
                        data.expend_child_count = refreshGroupDetailItems(data.expend_state, cfgs)
                        btn_image.transform.localRotation = data.expend_state and Quaternion.Euler(0, 0, -90) or Quaternion.Euler(0, 0, 0)
                    end
                    btn.onClick:RemoveAllListeners()
                    btn.onClick:AddListener(function()
                        local cfgs = self:getCfgsByGroupId(group_id)
                        if cfgs and #cfgs > 0 then
                            local data = getCacheGroupData(group_id)
                            data.expend_state = not data.expend_state
                            data.expend_child_count = refreshGroupDetailItems(data.expend_state, cfgs)
                            btn_image.transform.localRotation = data.expend_state and Quaternion.Euler(0, 0, -90) or Quaternion.Euler(0, 0, 0)
                            self:calculateHeight()
                        end
                        -- local cfgs = self:getCfgsByGroupId(group_id)
                        -- if cfgs and #cfgs > 0 then
                        --     local data = getCacheGroupData(group_id)
                        --     data.expend_state = not data.expend_state

                        --     local expend_path = small_path .. child.name .. "/items_items"
                        --     frame_helper:hideAllChild(self, expend_path)
                        --     if data.expend_state then
                        --         local count = 0
                        --         for k, v in ipairs(cfgs) do
                        --             local buff_cfg = config.buff_buffs[v.buff_id]
                        --             local item_id = buff_cfg.item
                        --             local own_num = frame_helper:getItemNum(item_id)
                        --             if (not (v.is_always_display == 0 and own_num == 0)) or v.is_always_display == 1 then
                        --                 count = count + 1
                        --                 local child = frame_helper:getChild(self, count, expend_path)
                        --                 child:SetActive(true)

                        --                 local icon = child.transform:Find("icon"):GetComponent(UnityEngine.UI.Image)
                        --                 local quality = child.transform:Find("out_bg"):GetComponent(UnityEngine.UI.Image)
                        --                 local name = child.transform:Find("name_text"):GetComponent(UnityEngine.UI.Text)
                        --                 local des = child.transform:Find("des_text"):GetComponent(UnityEngine.UI.Text)
                        --                 local own = child.transform:Find("own_text"):GetComponent(UnityEngine.UI.Text)

                        --                 local buy_trans = child.transform:Find("buyTrans")
                        --                 local use_trans = child.transform:Find("useTrans")
                                        
                        --                 local item_cfg = item_helper:getItemCfg(item_id)
                        --                 icon:SetSprite(frame_helper:getItemIcon(item_id))
                        --                 quality:SetSprite(frame_helper:getItemQualityIcon(item_id))
                        --                 name.text = frame_helper:getLangCfg(item_cfg.name_id)
                        --                 des.text = frame_helper:getLangCfg(item_cfg.des_id)
                        --                 own.text = string.format(NativeText.GetLangStr(own_num > 0 and 250008 or 250010), own_num) 

                        --                 if own_num > 0 then
                        --                     frame_helper.showTrans(use_trans)
                        --                     frame_helper.hideTrans(buy_trans)
                        --                 else
                        --                     frame_helper.showTrans(buy_trans)
                        --                     frame_helper.hideTrans(use_trans)

                        --                     local image_text = buy_trans.transform:Find("image_text")
                        --                     local DIAMOND    = 30001 -- 钻石
                        --                     local image_text = buy_trans.transform:Find("image_text"):GetComponent(UnityEngine.UI.Text)
                        --                     local item_name = gui_helper:getImageTextItemStr(DIAMOND)
                        --                     local single_price = frame_helper:getItemBuyPrice(item_id)
                        --                     image_text:SetImageText(string.format("%s %s", item_name, single_price))
                        --                 end 

                        --                 local btn = child:Find("useTrans/useBtn"):GetComponent(UnityEngine.UI.Button)
                        --                 btn.onClick:RemoveAllListeners()
                        --                 btn.onClick:AddListener(function ()
                        --                     print("使用单个")
                        --                     self:useItems(group_id, v.buff_id, item_id, 1)
                        --                 end)

                        --                 local btn = child:Find("buyTrans/buyBtn"):GetComponent(UnityEngine.UI.Button)
                        --                 btn.onClick:RemoveAllListeners()
                        --                 btn.onClick:AddListener(function ()
                        --                     print("购买使用")
                        --                     self:buyItems(group_id, v.buff_id, item_id, 1)
                        --                 end)
                        --             end
                        --         end
                        --         data.expend_child_count = count
                        --     else
                        --         data.expend_child_count = 0
                        --     end
                        -- end
                        
                    end)
                end
            end
        end


        app:startCoroutine(function()
            coroutine.yield(nil)
            for __, effect in ipairs(self.aaaaa) do
                effect.transform.localPosition = Vector3.zero
            end
        end)
    end,

    calculateHeight = function(self)
        local getCacheGroupData = function(group_id)
            for __, value in ipairs(self.cache_group) do
                if value.group_id == group_id then
                    return value
                end
            end
            return nil
        end

        local getTypeExpendHeight = function(group_ids)
            local height = 0
            for __, group_id in ipairs(group_ids) do
                local data = getCacheGroupData(group_id)
                if data.expend_child_count > 0 then
                    height = height + data.expend_child_count * 166 - 10
                end
            end
            return height
        end

        for __, value in ipairs(self.cache_type) do
            -- print("calculateHeight type_id:", value.type_id, value.no_expend_height)
            local expend_height = getTypeExpendHeight(value.group_ids)
            value.rect.sizeDelta = Vector2(value.rect.sizeDelta.x, value.no_expend_height + expend_height)
        end

        for __, value in ipairs(self.cache_group) do
            if value.expend_child_count > 0 then
                local expend_height = value.expend_child_count * 166 - 10
                value.rect.sizeDelta = Vector2(value.rect.sizeDelta.x, expend_height + value.no_expend_height)
            else
                value.rect.sizeDelta = Vector2(value.rect.sizeDelta.x, value.no_expend_height)
            end
        end
    end,

    -- 返回的数据类型
    -- {
    --     {
    --         type_id = 1,
    --         name_id = 1,
    --         list = { group_id_1, group_id_2 },
    --     },
    --     {
    --         type_id = 1,
    --         name_id = 1,
    --         list = { group_id_1, group_id_2 },
    --     },
    -- },
    cookCfgs = function(self)
        local rst = {}
        insertTable = function(type_id, group_id)
            for __, value in ipairs(rst) do
                if value.type_id == type_id then
                    for k, v in ipairs(value.list) do
                        if v == group_id then
                            return true
                        end
                    end
                    table.insert(value.list, group_id)
                    return true
                end
            end
            return false
        end

        config.home_config_buffs:foreach(function(id, cfg)
            -- if cfg.type <= 2 then
                local tab = insertTable(cfg.type, cfg.group)
                if not tab then
                    local temp = {
                        type_id = cfg.type,
                        name_id = cfg.type_name,
                        list = { cfg.group }
                    }
                    table.insert(rst, temp)
                end 
            -- end
        end)

        local sortTypeId = function(a, b)
            return a.type_id < b.type_id
        end

        local sortList = function(a, b)
            return a < b
        end

        table.sort(rst, sortTypeId)

        for __, value in ipairs(rst) do
            table.sort(value.list, sortList)
        end

        return rst
    end,

    getCfgsByGroupId = function(self, group_id)
        self.group_cfgs = self.group_cfgs and self.group_cfgs or self:cookGroupCfgs()
        for __, value in ipairs(self.group_cfgs) do
            if value.group_id == group_id then
                return value.list
            end
        end
        return nil
    end,

    -- {
    --     {
    --         group_id = 1,
    --         list = {
    --             cfg,
    --             cfg,
    --         }
    --     },
    --     {
    --         group_id = 1,
    --         list = {
    --             cfg,
    --             cfg,
    --         }
    --     },
    -- }

    cookGroupCfgs = function(self)
        local rst = {}
        insertTable = function(group_id, cfg)
            for __, value in ipairs(rst) do
                if value.group_id == group_id then
                    table.insert(value.list, cfg)
                    return true
                end
            end
            return false
        end
        config.home_config_buffs:foreach(function(id, cfg)
            local bool = insertTable(cfg.group, cfg)
            if not bool then
                local temp = {
                    group_id = cfg.group,
                    list = { cfg }
                }
                table.insert(rst, temp)
            end 
        end)

        local sortGroupId = function(a, b)
            return a.group_id < b.group_id
        end

        local sortList = function(a, b)
            return a.id < b.id
        end

        table.sort(rst, sortGroupId)

        for __, value in ipairs(rst) do
            table.sort(value.list, sortList)
        end

        return rst
    end,

    click_backBtn = function(self)
        frame_manager:hide('home_buff')
    end,

    getBuffInfoPlus = function(self, group_id)
        if buff_info.buffs then
            for __, value in ipairs(buff_info.buffs) do
                local buff_cfg = config.buff_buffs[value.buff_id]
                if buff_cfg.group == group_id then
                    return value
                end
            end
        end
        return nil
    end,
    
    useItems = function(self, group_id, target_buff_id, item_id, item_num)
        local send = function()
            network_sender:useItem(item_id, item_num)
        end

        local tar_buff_cfg = config.buff_buffs[target_buff_id]
        local buff_data = self:getBuffInfoPlus(tar_buff_cfg.group)
        if buff_data then
            local cur_buff_cfg = config.buff_buffs[buff_data.buff_id]
            if tar_buff_cfg.level >= cur_buff_cfg.level then
                local cur_des = string.format("%s Lv.%s", frame_helper:getLangCfg(cur_buff_cfg.buff_name), cur_buff_cfg.level)
                local tar_des = string.format("%s Lv.%s", frame_helper:getLangCfg(tar_buff_cfg.buff_name), tar_buff_cfg.level)
                frame_helper:showCommonBox(string.format(NativeText.GetLangStr(530003), cur_des, tar_des),function()
                    send()
                end)
            else
                local cur_des = string.format("%s Lv.%s", frame_helper:getLangCfg(cur_buff_cfg.buff_name), cur_buff_cfg.level)
                frame_helper:showConfirmBox(string.format(NativeText.GetLangStr(530002), cur_des),function()
                end)
            end
        else
            send()
            hint:setText(NativeText.GetLangStr(530005))
        end
    end,

    buyItems = function(self, group_id, target_buff_id, item_id, item_num)
        local send = function()
            network_sender:buyItem(
                        {
                            {
                                item_id = item_id,
                                count = item_num,
                            }
                        },
                        1
                    )
        end

        print("目标 Buff Id:", target_buff_id, "当前组:", group_id)

        local tar_buff_cfg = config.buff_buffs[target_buff_id]
        local buff_data = self:getBuffInfoPlus(tar_buff_cfg.group)
        if buff_data then
            local cur_buff_cfg = config.buff_buffs[buff_data.buff_id]
            if tar_buff_cfg.level >= cur_buff_cfg.level then
                local cur_des = string.format("%s Lv.%s", frame_helper:getLangCfg(cur_buff_cfg.buff_name), cur_buff_cfg.level)
                local tar_des = string.format("%s Lv.%s", frame_helper:getLangCfg(tar_buff_cfg.buff_name), tar_buff_cfg.level)
                frame_helper:showCommonBox(string.format(NativeText.GetLangStr(530003), cur_des, tar_des),function()
                    send()
                end)
            else
                local cur_des = string.format("%s Lv.%s", frame_helper:getLangCfg(cur_buff_cfg.buff_name), cur_buff_cfg.level)
                frame_helper:showConfirmBox(string.format(NativeText.GetLangStr(530002), cur_des),function()
                end)
            end
        else
            local name = string.format("%s Lv.%s", frame_helper:getLangCfg(tar_buff_cfg.buff_name), tar_buff_cfg.level)
            frame_helper:showCommonBox(string.format(NativeText.GetLangStr(530004), name),function()
                send()
                hint:setText(NativeText.GetLangStr(530005))
            end)
        end
    end,
}

