local super = require("ui.frame")
local log = logger.get("Game", "home_info")

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        self.home_map = self.args.owner

        clickEvents = 
        {
            "content/common_bg/backBtn"
        }

        -- 注册点击事件
        frame_helper:regClickEvents("click", self, clickEvents)

        self.build_data = home_info:getBuildDataByCfgId(24)

        self.TRAP_TRAIN_TICK = home_info.events:on("TRAP_TRAIN_TICK", function(build_data, left_time, total_time)
            self:onTickTrainTime(build_data, left_time, total_time)
        end)

        self.BUILD_INFO_CHANGE = home_info.events:on("BUILD_INFO_CHANGE", function(build_id)
            local build_data = home_info:getBuildDataByCfgId(24)
            if build_id == build_data.id then
                self:init()
            end
        end)

        self:init()
    end,

    onUnload = function (self)
        home_info.events:off("BUILD_INFO_CHANGE", self.BUILD_INFO_CHANGE)
        home_info.events:off("TRAP_TRAIN_TICK", self.TRAP_TRAIN_TICK)
    end,

    init = function(self)
        self.tick_trans = nil
        local list = {}
        config.home_config_walls:foreach(function(id, cfg)
            table.insert(list, cfg)
        end)
        local sortFunc = function(a, b)
            return a.id < b.id
        end
        table.sort(list, sortFunc)

        local cur_num = 0
        local path = "content/layout"
        frame_helper:hideAllChild(self, path)
        for __, value in ipairs(list) do
            local child = frame_helper:getChild(self, __, path)
            child:SetActive(true)

            local icon = child.transform:Find("icon"):GetComponent(UnityEngine.UI.Image)
            local des = child.transform:Find("des"):GetComponent(UnityEngine.UI.Text)
            local total = child.transform:Find("total"):GetComponent(UnityEngine.UI.Text)

            icon:SetSprite(value.icon)
            des.text = frame_helper:getLangCfg(value.des)
            local temp_num = 0
            local is_repair = (__ == #list)
            if is_repair then
                total.text = ""
            else
                temp_num = self:getSoldierTotalNum(value.soldier_type)
                total.text = string.format(NativeText.GetLangStr(570002), temp_num)
            end

            cur_num = cur_num + temp_num

            local upgrading_trans = child.transform:Find("upgrading")
            local receive_trans = child.transform:Find("receiveBtn")

            frame_helper.hideTrans(upgrading_trans)
            frame_helper.hideTrans(receive_trans)

            local unlock = true
            if value.unlock > 0 then
                local tech_cfg = config.home_config_research_details[value.unlock]
                local tech_cfg_id = tech_cfg.list_id
                local tech_level = tech_cfg.level
                unlock = home_info:getTechLevel(tech_cfg_id) >= tech_level

                frame_helper:setImageGray1(icon, not unlock)
            end

            local btn = child.transform:Find("btn"):GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(function()
                if unlock then
                    self:click_soldier(is_repair, value)
                else
                    local detail_cfg = config.home_config_research_details[value.unlock]
                    local tech_cfg = config.home_config_research_lists[detail_cfg.list_id]
                    frame_helper:showCommonBox(string.format(NativeText.GetLangStr(570005), frame_helper:getLangCfg(tech_cfg.name_id)), function()
                        frame_manager:hide('home_defense')
                        self:goToResearch(value.unlock)
                    end)
                end
            end)

            local build_data = home_info:getBuildDataByCfgId(24)
            local data = build_data.guardsystem.build
            if data then
                local soldier_cfg = config.soldier_soldiers[data.cfgId]
                if soldier_cfg then
                    if soldier_cfg.type == value.soldier_type then
                        -- log.error("城墙界面刷新 Init:", data.canRecv, value.soldier_type, data.bTran)
                        if data.canRecv then
                            frame_helper.showTrans(receive_trans)
                            local btn = child.transform:Find("receiveBtn"):GetComponent(UnityEngine.UI.Button)
                            btn.onClick:RemoveAllListeners()
                            btn.onClick:AddListener(function()
                                network_sender:receiveSoldier(build_data.id)
                            end)
                        elseif data.bTran then
                            frame_helper.showTrans(upgrading_trans)
                            self.tick_trans = upgrading_trans
                            self:setUpgradingInfo()
                            local btn = child.transform:Find("upgrading/speedUpBtn"):GetComponent(UnityEngine.UI.Button)
                            btn.onClick:RemoveAllListeners()
                            btn.onClick:AddListener(function()
                                frame_manager:show("home_speed_up", nil, {
                                    speed_up_type = home_speed_up_type.trap_create,
                                    id = home_info:getBuildDataByCfgId(24).id,
                                    soldier_id = data.cfgId,
                                })
                            end)
                        end
                    end
                end
            end
        end

        local capacity = self:findText("content/count")
        capacity.text = NativeText.GetLangStr(570004) .. cur_num .. "/" .. home_info:getDefenseCapacity()
    end,

    isUpgrading = function(self, soldier_type)
        local build_data = home_info:getBuildDataByCfgId(24)
        local data = build_data.guardsystem.build
        if data and data.bTran then
            return config.soldier_soldiers[data.cfgId].type == soldier_type
        end
        return false
    end,

    click_toggle = function(self, cfg)
        print("点击")
    end,

    click_backBtn = function(self)
        frame_manager:hide('home_defense')
    end,

    click_soldier = function(self, is_repair, cfg)
        if is_repair then
            frame_manager:show('home_hospital', nil, {
                is_defense = true,
            })
        else
            frame_manager:show('home_barrack', self.home_map, {
                build_id = self.build_data.id,
                soldier_type = cfg.soldier_type
            })
        end
    end,

    getSoldierTotalNum = function(self, soldier_type)
        local list = {}
        config.soldier_soldiers:foreach(function(id, cfg)
            if soldier_type == cfg.type then
                table.insert(list, cfg)
            end
        end)
        local num = 0
        for __, cfg in ipairs(list) do
            num = num + self:getSoldierNumByCfgId(cfg.id)
        end
        return num
    end,

    getSoldierNumByCfgId = function(self, soldier_id)
        for __, soldier in ipairs(self.build_data.guardsystem.traps) do
            if soldier_id == soldier.cfgId then
                return soldier.totalNum
            end
        end
        return 0
    end,

    setUpgradingInfo = function(self)
        if self.tick_trans then
            local e = home_info:getBuildDataByCfgId(24)
            local elapsed = e.guardsystem.build.reamintime - Time.realtimeSinceStartup + e.rmbTime
            self:onTickTrainTime(e, elapsed, e.guardsystem.build.totaltime)  
        end
    end,

    onTickTrainTime = function(self, build_data, left_time, total_time)
        if self.tick_trans then
            local slider = self.tick_trans:Find("slider"):GetComponent(UnityEngine.UI.Slider)
            local time = self.tick_trans:Find("count_time"):GetComponent(UnityEngine.UI.Text)
            if left_time <= 0 then
                slider.value = 1
            else
                time.text = frame_helper:timeFormat(left_time)
                slider.value = 1 - left_time / total_time
            end
        end
    end,

    goToResearch = function(self, cfg_id)
        local tech_cfg = config.home_config_research_details[cfg_id]
        local tech_cfg_id = tech_cfg.list_id
        local tech_level = tech_cfg.level
        local cur_level, has_build = home_info:getTechLevel(tech_cfg_id)
        local can_train = cur_level >= tech_level

        if not has_build then
            hint:setText(NativeText.GetLangStr(550027))
        elseif not can_train then
            local build_data = home_info:getBuildDataByCfgId(2)
            if build_data then
                    local model_trans = self.home_map:getHomeBuildById(build_data.id).model.transform
                    self.home_map.home_camera:LookAt(model_trans, Vector3(UnityEngine.Screen.width / 2, UnityEngine.Screen.height / 2, 0), config.home_config_lists[build_data.cfgId].upgrade_fov, function()
                        local frame = frame_manager:show('home_entity_research', self.home_map, build_data.id)
                        frame:addCommandBuff(function()
                            local target_cfg = nil
                            config.home_config_research_lists:foreach(function(id, cfg)
                                if id == tech_cfg_id then
                                    target_cfg = cfg
                                end
                            end)
                            if target_cfg then
                                frame:click_category(target_cfg.type)
                                frame:click_tech(target_cfg)
                            end
                        end)
                    end)

                    
                -- app:startCoroutine(function ()
                --     local model_trans = self.home_map:getHomeBuildById(build_data.id).model.transform
                --     self.home_map.home_camera:LookAt(model_trans, Vector3(UnityEngine.Screen.width / 2, UnityEngine.Screen.height / 2, 0), config.home_config_lists[build_data.cfgId].upgrade_fov)
                --     coroutine.yield(WaitForSeconds(0.5))
                --     local frame = frame_manager:show('home_entity_research', self.home_map, build_data.id)
                --     frame:addCommandBuff(function()
                --         local target_cfg = nil
                --         config.home_config_research_lists:foreach(function(id, cfg)
                --             if id == tech_cfg_id then
                --                 target_cfg = cfg
                --             end
                --         end)
                --         if target_cfg then
                --             frame:click_category(target_cfg.type)
                --             frame:click_tech(target_cfg)
                --         end
                --     end)
                -- end)
            end
        end
    end,
}
