local super = require("ui.frame")

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        self.soldier_id = self.args.data.soldier_id
        self.soldier_max_num = self.args.data.soldier_max_num
        self.state = self.args.data.state
        self.be_hurted = self.args.data.be_hurted

        clickEvents = 
        {
            "content/addBtn",
            "content/reduceBtn",
            "content/leftBtn",
            "content/rightBtn",
        }

        -- 注册点击事件
        frame_helper:regClickEvents("click", self, clickEvents)

        self:init()
    end,

    onUnload = function (self)
    end,

    init = function(self)
        -- local name = self:findText("content/name")
        local max_num = self:findText("content/max_num")
        local icon = self:findImage("content/icon")
        local quality = self:findImage("content/quality")

        local soldier_cfg = config.soldier_soldiers[self.soldier_id]
        icon:SetSprite(soldier_cfg.headpath .. soldier_cfg.soldierhead)
        quality:SetSprite(gui_helper:getQualityIconPath(soldier_cfg.color))
        max_num.text = "/" .. self.soldier_max_num
        
        self.input_field = self:findInputField("content/InputField")
        self.slider = self:findSlider("content/slider")
        
        -- self.select_num = self.soldier_max_num
        self.select_num = 1
        self:refreshItemInfo()
    end,

    click_addBtn = function(self)
        if self.select_num < self.soldier_max_num then
            self.select_num = self.select_num + 1
            self:refreshItemInfo()
        end
    end,

    click_reduceBtn = function(self)
        if self.select_num > 1 then
            self.select_num = self.select_num - 1
            self:refreshItemInfo()
        end
    end,

    refreshItemInfo = function(self)
        self:removeAllListener()
        self.input_field.text = self.select_num
        self.slider.value = self.select_num / self.soldier_max_num
        self:addAllListener()
    end,

    addAllListener = function(self)
        self.slider.onValueChanged:RemoveAllListeners()
        self.slider.onValueChanged:AddListener(function()
            self:onSliderValueChange()
        end)
        self.input_field.onValueChanged:RemoveAllListeners()
        self.input_field.onValueChanged:AddListener(function()
            self:onInputFieldValueChange()
        end)
    end,

    removeAllListener = function(self)
        self.slider.onValueChanged:RemoveAllListeners()
        self.input_field.onValueChanged:RemoveAllListeners()
    end,

    onSliderValueChange = function(self)
        self.select_num = math.ceil(self.slider.value * self.soldier_max_num)
        self.select_num = self.select_num <= 0 and 1 or self.select_num
        self:refreshItemInfo()
    end,

    onInputFieldValueChange = function(self)
        self.select_num = tonumber(self.input_field.text)
        self.select_num = self.select_num <= 0 and 1 or ( self.select_num > self.soldier_max_num and self.soldier_max_num or self.select_num )
        self:refreshItemInfo()
    end,

    click_leftBtn = function(self)
        frame_manager:hide('home_delete_soldier')
    end,

    click_rightBtn = function(self)
        local soldier_cfg = config.soldier_soldiers[self.soldier_id]
        local soldier_name = frame_helper:getLangCfg(soldier_cfg.soldiername)
        local power = self.select_num * soldier_cfg.power

        if self.be_hurted then
            frame_helper:showCommonBox(string.format(NativeText.GetLangStr(560018), self.select_num, soldier_name ),function()
                frame_manager:hide('home_delete_soldier')
                        network_sender:deleteSoldier({
                    {
                        cfg_id = self.soldier_id,
                        count = self.select_num, 
                        state = self.state,
                    }
                })
            end)
        else
            frame_helper:showCommonBox(string.format(NativeText.GetLangStr(560016),power, self.select_num, soldier_name ),function()
                frame_manager:hide('home_delete_soldier')
                        network_sender:deleteSoldier({
                    {
                        cfg_id = self.soldier_id,
                        count = self.select_num, 
                        state = self.state,
                    }
                })
            end)
        end
    end,
}