local super = require("ui.frame")
local log = logger.get("Game", "home_info")

local drop_down_state = {
    base = 1,
    all = 2,
}

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        self.home_map = self.args.owner

        clickEvents = 
        {
            "content/backBtn",
        }

        -- 注册点击事件
        frame_helper:regClickEvents("click", self, clickEvents)

        self.BUILD_INFO_CHANGE = home_info.events:on("BUILD_INFO_CHANGE", function(build_id)
            local build_data = home_info:getBuildDataById(build_id)
            if build_data.cfgId == 10 then
                self:refresh()
            end
        end)

        self:init()
    end,

    onUnload = function (self)
        home_info.events:off("BUILD_INFO_CHANGE", self.BUILD_INFO_CHANGE)
    end,

    init = function(self)

        local dd_trans = self.transform:Find("content/home_panel_10/DropDownList")
        local ddh = dd_trans:GetComponent("DropDownHelper")
        if not ddh then ddh:AddComponent("DropDownHelper") end
        ddh:Clear()
        ddh:Add(NativeText.GetLangStr(600010))
        ddh:Add(NativeText.GetLangStr(600011))
        local dd = self:findDropdown("content/home_panel_10/DropDownList")
        dd.onValueChanged:RemoveAllListeners()
        dd.onValueChanged:AddListener(function(value)
            self:click_dpd(value + 1)
        end)
        self:click_dpd(1)
    end,

    click_dpd = function(self, value)
        print("value:", value)
        self.cur_state = value
        self:refresh()
    end,

    refresh = function(self)
        local build_data = home_info:getBuildDataByCfgId(10)
        if not build_data then
            return
        end

        local path = "content/Scroll View/Viewport/Content"
        frame_helper:hideAllChild(self, path, true)

        local total_amount = 0
        local total_hurt = 0

        for idx, s in ipairs(build_data.parade.soldiers) do
            local child = frame_helper:getChild(self, idx, path)
            child:SetActive(true)

            local name =    child.transform:Find("layout/name"):GetComponent(UnityEngine.UI.Text)
            local power =   child.transform:Find("layout/power"):GetComponent(UnityEngine.UI.Text)
            local amount =  child.transform:Find("layout/amout"):GetComponent(UnityEngine.UI.Text)
            local hurt =    child.transform:Find("layout/hurt"):GetComponent(UnityEngine.UI.Text)
            local protect = child.transform:Find("layout/protect"):GetComponent(UnityEngine.UI.Text)
            local icon =    child.transform:Find("icon"):GetComponent(UnityEngine.UI.Image)
            local quality = child.transform:Find("quality"):GetComponent(UnityEngine.UI.Image)

            local soldier_cfg = config.soldier_soldiers[s.cfgId] 
            icon:SetSprite(soldier_cfg.headpath .. soldier_cfg.soldierhead)
            quality:SetSprite(gui_helper:getQualityIconPath(soldier_cfg.color))
            name.text = frame_helper:getLangCfg(soldier_cfg.soldiername)

            local protect_num = self:getProtectSoldierNum(s.cfgId)
            if self.cur_state == drop_down_state.base then
                local total_num = protect_num + s.totalNum - s.bInArmy
                power.text = soldier_cfg.power * total_num
                amount.text = total_num
                total_amount = total_amount + total_num
            elseif self.cur_state == drop_down_state.all then
                local total_num = protect_num + s.totalNum
                power.text = soldier_cfg.power * total_num
                amount.text = total_num
                total_amount = total_amount + total_num
            end
            hurt.text = s.bHurt + s.bTreat
            total_hurt = total_hurt + s.bHurt + s.bTreat
            protect.text = protect_num

            local btn = child.transform:Find("deleteBtn"):GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(function()
                frame_manager:show('home_delete_soldier', nil, {
                    soldier_id = s.cfgId,
                    soldier_max_num = s.totalNum - s.bInArmy - s.bTreat - s.bHurt,
                    state = 0,
                    be_hurted = false,
                })           
            end)
        end

        upkeep = self:findText("content/home_panel_10/upkeep")
        total = self:findText("content/home_panel_10/hurt")
        hurt = self:findText("content/home_panel_10/total")
        upkeep.text = string.format(NativeText.GetLangStr(600007), self:numFormat(local_player.props[protocol_pb.PROP_REDUCEFOOD] * 3600) .. "/H")
        total.text = string.format(NativeText.GetLangStr(600008), total_amount)
        hurt.text = string.format(NativeText.GetLangStr(600009), total_hurt)
    end,

    numFormat = function(self, num)
        if num <= 9999999 then
            num = math.floor(num / 10^2)
            return string.format("%.1f", num / 10^1) .. 'K'
        else
            num = math.floor(num / 10^4)
            return string.format("%.2f", num / 10^2) .. 'M'
        end
    end,


    getProtectSoldierNum = function(self, soldier_id)
        return 0
    end,

    click_backBtn = function(self)
        frame_manager:hide('home_drill_ground')
    end,
}
