local super = require("ui.frame")
local max_officers = 10
local att = 1010

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        self.owner = self.args.owner    -- game_state_home.lua
        self.msg = self.args.data       -- MPersonPark

        clickEvents = 
        {
            "closeBtn",
            "centerLeft/clearBtn",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self:initRtCamera()
        self:initUI(self.msg)
    end,

    onUnload = function (self)
         self.rtCamera:clear()
    end,

    refreshUI = function(self, msg)
        self.msg = msg
        self:initUI(self.msg)
    end,

    initRtCamera = function(self)
        self.rtCamera = self.rtCamera or require("gui.tool.frame_renderTexture").new()
    end,

    -- msg = MPersonPark
    initUI = function(self, msg)
        local heros = hero_manager.heros
        local idx = 1
        for __, h in pairs(heros) do
            local child = self:getRightChild(idx)
            local mono = child:Find("icon"):GetComponent(DragImage)
            mono:SetCallback(
                function(eventData)
                    -- print("拖拽开始", eventData.position)
		            local tmp = GameObject("icon")
		            tmp.transform:SetParent(self.transform)
                    tmp.transform.localScale = Vector3.one
		            tmp.transform:SetAsLastSibling()

                    local ri = tmp.transform:AddComponent(UnityEngine.UI.RawImage)
                    ri.color = Color(1,1,1,1)
                    ri.raycastTarget = false

                    local rt = tmp:GetComponent(UnityEngine.RectTransform)
                    rt.anchorMax = Vector2.zero
                    rt.anchorMin = Vector2.zero
                    rt.sizeDelta = Vector2.one * 300

                    self.drag_tank_trans = tmp.transform
                    self.drag_tank_rt = rt
                    -- 拖拽tank的唯一id
                    self.drag_tank_idx = h.cfg_id
                    
                    local cfg = config.tank_basic_ids[h.cfg_id]
                    self.rt_idx = self.rtCamera:add(self, self.drag_tank_trans, cfg.prefab1, 300, 
                        -- 模型的游戏对象
                        function(model_obj)
                            self:rtTmpCompleted(model_obj, h.cfg_id)
                        end
                    )

                    self:setDragImagePos(self.drag_tank_trans, eventData)
                end,
                function(eventData)
                    -- print("拖拽中...", eventData.position)
                    self:setDragImagePos(self.drag_tank_trans, eventData)
                end,
                function(eventData)
                    -- 由于每次销毁GC大，暂定不销毁了
                    -- self.rtCamera:remove(self.rt_idx)
                    Object.Destroy(self.drag_tank_trans.gameObject)
                    self.drag_tank_trans = nil
                    self.drag_tank_rt = nil
                    self.rt_idx = nil
                    self.drag_tank_idx = nil
                    -- print("拖拽结束")
                end
            )
            idx = idx + 1
        end
        self:hideRightChildsAtPos(idx)

        local parking_num = config.home_config_configs[1].parking_num
        for i = 1, parking_num do
            local child = self:getLeftChild(i)
            local tank = self:getTankAtPos(msg, i)
            local icon = child:Find("icon"):GetComponent(UnityEngine.UI.RawImage)
            icon.color = Color(1, 1, 1, tank ~= nil and 1 or 0)
            local icon_trans = child:Find("icon")
            if tank then
                local cfg = config.tank_basic_ids[tank.partnerId]
                self.rtCamera:add(self, icon_trans, cfg.prefab1, 300, 
                    -- functor.make(self.rtCallback, self)
                    function(model_obj)
                        self:rtLeftCompleted(model_obj, i, icon)
                    end
                )
            end

            local mono = child:Find("icon"):GetComponent(DropImage)
            mono:SetCallback(
                 function(eventData)
                    -- print("放下")
                    if self.drag_tank_idx then
                        network_sender:pbAddPartner(i - 1, self.drag_tank_idx)
                        self.drag_tank_idx = nil
                    end
                 end,
                 function(eventData)
                    -- print("进入")
                 end,
                 function(eventData)
                    -- print("移出")
                 end
            )
        end
    end,

    setDragImagePos = function(self, image_trans, eventData)
        if self.drag_tank_rt then
            self.drag_tank_rt.anchoredPosition3D = Vector3(eventData.position.x, eventData.position.y, 0)
        end
    end,

    -- rt 创建完成回调
    rtLeftCompleted = function(self, model, idx)
        -- print("游戏对象创建完成:", model.name, idx)
        model.transform.localPosition = Vector3(-0.56, -2.64, 30.8)
        model.transform.localRotation = Quaternion.Euler(0, -123.39, 0)
        self.rt_list = self.rt_list or {}
        table.insert(self.rt_list, {idx = idx, gameObject = gameObject})
    end,

    rtTmpCompleted = function(self, model, idx)
        -- print("游戏对象创建完成:", model.name, idx)
        model.transform.localPosition = Vector3(-0.56, -2.64, 30.8)
        model.transform.localRotation = Quaternion.Euler(0, -123.39, 0)
    end,

    -- 判断该位置是否有车
    getTankAtPos = function(self, msg, pos)
        -- pi = MParkInfo
        for __, pi in ipairs(msg.parkInfos) do
            -- print("服务器数据", __, pi.posIndex, pi.partnerId)
            if pi.posIndex == (pos - 1) then
                return pi
            end
        end
        return nil
    end,

    -- 缓存layout,idx从1开始
    getRightChild = function(self, idx)
        local path = "centerRight/upContent/Scroll View/Viewport/Content"
        local layout_trans = self.transform:Find(path)
        if not self.cache_layout_right then
            self.cache_layout_right = {}
            for i = 0, layout_trans.childCount - 1 do
                local child_trans = layout_trans:GetChild(i)
                child_trans.transform.name = i + 1
                table.insert(self.cache_layout_right, child_trans.transform)
            end
        end
        if idx > #self.cache_layout_right then
            for i = 1, idx - #self.cache_layout_right do
                local tplt = layout_trans:GetChild(0)
                local child_trans = Object.Instantiate(tplt.gameObject);
                child_trans.transform.name = i + #self.cache_layout_right
                child_trans:AttachTo(tplt.transform.parent)
                table.insert(self.cache_layout_right, child_trans.transform)
            end
        end
        self.cache_layout_right[idx].gameObject:SetActive(true)
        local full_path = path .. "/" .. self.cache_layout_right[idx].transform.name
        return self.cache_layout_right[idx], full_path
    end,

    -- 隐藏从索引后面的对象
    hideRightChildsAtPos = function(self, start_pos)
        if not self.cache_layout_right then
            return
        end
        for i = start_pos, #self.cache_layout_right do
            local child_trans = self.cache_layout_right[i]
            if child_trans then
                child_trans.gameObject:SetActive(false)
            end
        end
    end,

        -- 缓存layout,idx从1开始
    getLeftChild = function(self, idx)
        local path = "centerLeft/Scroll View/Viewport/Content"
        local layout_trans = self.transform:Find(path)
        if not self.cache_layout_left then
            self.cache_layout_left = {}
            for i = 0, layout_trans.childCount - 1 do
                local child_trans = layout_trans:GetChild(i)
                child_trans.transform.name = i + 1
                table.insert(self.cache_layout_left, child_trans.transform)
            end
        end
        if idx > #self.cache_layout_left then
            for i = 1, idx - #self.cache_layout_left do
                local tplt = layout_trans:GetChild(0)
                local child_trans = Object.Instantiate(tplt.gameObject);
                child_trans.transform.name = i + #self.cache_layout_left
                child_trans:AttachTo(tplt.transform.parent)
                table.insert(self.cache_layout_left, child_trans.transform)
            end
        end
        self.cache_layout_left[idx].gameObject:SetActive(true)
        local full_path = path .. "/" .. self.cache_layout_left[idx].transform.name
        return self.cache_layout_left[idx], full_path
    end,

    -- 隐藏从索引后面的对象
    hideLeftChildsAtPos = function(self, start_pos)
        if not self.cache_layout_left then
            return
        end
        for i = start_pos, #self.cache_layout_left do
            local child_trans = self.cache_layout_left[i]
            if child_trans then
                child_trans.gameObject:SetActive(false)
            end
        end
    end,

    -- 关闭界面
    click_closeBtn = function(self)
        frame_manager:hide('home_entity_garage')
    end,

    -- 清空车库事件
    click_clearBtn = function(self)
        local tb = {
            title = NativeText.GetLangStr(130001),
            content = NativeText.GetLangStr(130004),
            buttons =
            {
                {
                    btn_name = NativeText.GetLangStr(130003),
                    callback = function() 
                        frame_manager:hide('common_box')
                    end
                },
                {
                    btn_name = NativeText.GetLangStr(130002),
                    callback = function() 
                        frame_manager:hide('common_box')
                        network_sender:pbRemovePartner(1)
                    end
                },
            },
        }

        frame_manager:show('common_box', nil, tb)
    end,
}
