local super = require("ui.frame")

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)
        
        self.home_entity = self.args.data -- home_entity
        self.home_map = self.args.owner
        self.id = self.home_entity.id

        print("id", self.home_entity.id)

        clickEvents = {
            "closeBtn",
            "content/explainBtn",
            "content/detailInfoBtn",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self:init(self.id)
    end,

    onUnload = function(self)
    end,

    click_closeBtn = function(self)
        frame_manager:hide('home_entity_info')
    end,

    click_explainBtn = function(self)
        local dt = home_info:getBuildDataByGridId(self.id)
        if dt then
            local cfg = config.home_config_lists[dt.cfgId]
            if cfg then
                frame_manager:show('home_entity_explain', nil, cfg)
            end
        end
    end,

    click_detailInfoBtn = function(self)
        frame_manager:show('home_entity_info_detail', nil, home_info:getBuildDataByGridId(self.id).cfgId)
    end,

    init = function(self, id)
        local icon = self:findImage('content/icon')
        local name = self:findText('content/name')
        local level = self:findText('content/level')
        local des = self:findText('content/des')
        local explain_btn = self.transform:Find("content/explainBtn")

        local layout_path = 'content/top/Scroll View/Viewport/Content'
        self.helper:hideAllChild(self, layout_path)
        
        local dt = home_info:getBuildDataByGridId(id)
        if not dt then
            return
        end

        local cfg = config.home_config_lists[dt.cfgId]
        if cfg then
            name.text = self.helper:getLangCfg(cfg.name_id)
            level.text = "Lv." .. dt.level
            des.text = self.helper:getLangCfg(cfg.info_id)
            icon:SetSprite(cfg.path .. cfg.icon)
            explain_btn:SetActive(cfg.rule ~= "")
        end

        local upgrade_cfg = self:getUpgradeCfg(dt.cfgId, dt.level)
        if upgrade_cfg then        
            local args = self.helper:getPropertyDes(upgrade_cfg.effect)
            local idx = 1
            for __, map in pairs(args) do
                local child = self.helper:getChild(self, idx, layout_path)
                child:SetActive(true)

                local bg = child.transform:Find("home_panel_7"):GetComponent(UnityEngine.UI.Image)
                bg.enabled = idx % 2 ~= 0

                local name = child.transform:Find("name"):GetComponent(UnityEngine.UI.Text)
                name.text = map.name_str

                local cur_level = child.transform:Find("cur_level"):GetComponent(UnityEngine.UI.Text)
                cur_level.text = map.num_str

                idx = idx + 1
            end
        end
    end,

    getUpgradeCfg = function(self, cfg_id, level)
        local rst = nil
        local max_level = 1
        config.home_config_upgrades:foreach(
            function(k, v)
                if v.building_id == cfg_id then
                    if v.level == level then rst = v end
                    if v.level > max_level then max_level = v.level end
                end
            end
        )
        return rst, max_level
    end,
}
