local super = require("ui.frame")
local log = logger.get("Game", "UI")

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        self.home_map = self.args.owner       
        self.build_id = self.args.data       
        self.MPersonStudio = home_info:getBuildDataById(self.build_id).studio

        clickEvents = {
            "content/base_trans/bg/top/ButtonBack",
            "content/base_trans/bg/top/homeBtn",
            "content/detail_trans/bg/top/toMain",
            "content/detail_trans/bg/top/homeBtn",
            "upgrade_trans/closeUpgradeBtn",
            "upgrade_trans/content/1/bg/upgrading/speedUpBtn",
            "upgrade_trans/content/1/bg/upgrading/clearCountBtn",
        }

        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self.base_trans = self.transform:Find("content/base_trans")
        self.detail_trans = self.transform:Find("content/detail_trans")

        self.slider_trans = self.transform:Find("content/base_trans/bottom/upgrading")
        self.recommend_trans = self.transform:Find("content/base_trans/bottom/recommend")
        self.completed_trans = self.transform:Find("content/base_trans/bottom/completed")
        self.upgrading_detail_trans = self.transform:Find("content/detail_trans/bottom")
        self.upgrade_trans = self.transform:Find("upgrade_trans")
        self.upgrading_trans = self.transform:Find("upgrading_trans")
        self.max_level_trans = self.transform:Find("max_level_trans")

        self.helper.showTrans(self.base_trans)
        self.helper.hideTrans(self.detail_trans)
        self.helper.hideTrans(self.slider_trans)
        self.helper.hideTrans(self.recommend_trans)
        self.helper.hideTrans(self.completed_trans)
        self.helper.hideTrans(self.upgrade_trans)
        self.helper.hideTrans(self.upgrading_trans)
        self.helper.hideTrans(self.max_level_trans)
        self.helper.hideTrans(self.upgrading_detail_trans)

        self:init()
        self:initBaseUI(self.MPersonStudio)

        self.onPropChange = local_player.events:on("onPropChange", function() 
            print("资源变化刷新")
            if self.select_tech_cfg then
                self:click_tech(self.select_tech_cfg)
            end
        end)

        self.BUILD_INFO_CHANGE = home_info.events:on("BUILD_INFO_CHANGE", function(build_id)
            if build_id == self.build_id then
                self:refreshUI(home_info:getBuildDataById(self.build_id).studio)
                if self.select_tech_cfg then
                    self:click_tech(self.select_tech_cfg)
                end
            end
        end)

        self.TECH_UPGRADE_TICK = home_info.events:on("TECH_UPGRADE_TICK", function(e, elapsed, total_time)
            if self.select_tech_cfg and self.select_tech_cfg.id == e.studio.cfgId then
                self:showUpgrading(e, elapsed, total_time)
            end
            self:setDetailProgress(elapsed, total_time)
            self:onTechUpgradeTick(e, elapsed, total_time)
        end)

        self.event_id1 = home_info.events:on("BUILD_UPGRADE_TICK", functor.make(self.onBuildUpgradeTick, self))

        self:isTechCompleted()
        
        frame_helper.hideTrans(self.transform:Find("upgrade_trans/content/2"))
    end,

    onUnload = function (self)
        local_player.events:off("onPropChange", self.onPropChange)
        home_info.events:off("BUILD_INFO_CHANGE", self.BUILD_INFO_CHANGE)
        home_info.events:off("TECH_UPGRADE_TICK", self.TECH_UPGRADE_TICK)
        home_info.events:off("BUILD_UPGRADE_TICK", self.BUILD_UPGRADE_TICK)
    end,

    click_cancelBtn = function(self)
    end,

    click_speedBtn = function(self)
    end,

    click_ButtonBack = function(self)
        frame_manager:hide('home_entity_research')
    end,

    click_homeBtn = function(self)
        frame_manager:hide('home_entity_research')
    end,

    click_toMain = function(self)
        self.helper.showTrans(self.base_trans)
        self.helper.hideTrans(self.detail_trans)
    end,

    init = function(self)
    end,

    -- 收到服务器消息刷新界面
    refreshUI = function(self, msg)
        self.MPersonStudio = msg
        self:initBaseUI(self.MPersonStudio)
        if self.cur_category then
            self:setDetailUIInfo(self.cur_category)
        end
    end,

    -- 初始化主界面
    -- msg = MPersonStudio
    initBaseUI = function(self, msg)
        self.helper.hideTrans(self.slider_trans)
        self.helper.hideTrans(self.recommend_trans)
        self.helper.hideTrans(self.completed_trans)
        self.helper.hideTrans(self.upgrading_detail_trans)

        if msg.tLevelUp > 0 then
            self.helper.showTrans(self.slider_trans)
            self.helper.showTrans(self.upgrading_detail_trans)
            self:setSliderInfo(msg)
        else
            local full_level = self:isAllResearchFullLevel(msg)        
            if full_level then
                self.helper.showTrans(self.completed_trans)
            else
                self.helper.showTrans(self.recommend_trans)
                self:setRecommendInfo(msg)
            end
        end

        self:setMainInfo()
    end,

    setSliderInfo = function(self, msg)
        local icon = self:findImage("content/base_trans/bottom/upgrading/icon")
        local n_bg = self:findImage("content/base_trans/bottom/upgrading/n_bg")
        local s_bg = self:findImage("content/base_trans/bottom/upgrading/s_bg")
        local name_text = self:findText("content/base_trans/bottom/upgrading/nameText")
        local level_text = self:findText("content/base_trans/bottom/upgrading/levelText")
        local slider = self.transform:Find('content/base_trans/bottom/upgrading/Slider'):GetComponent(UnityEngine.UI.Slider)
        local btn = self.transform:Find('content/base_trans/bottom/upgrading/speedBtn'):GetComponent(UnityEngine.UI.Button)
        local clk_trans = self.transform:Find('content/base_trans/bottom/upgrading/text1_Static[T4]_3')

        local cfg = config.home_config_research_lists[msg.cfgId]
        local next_detail_cfg = self:getTechDetailCfg(msg.cfgId, self:getTechLevel(msg.cfgId) + 1)
        icon:SetSprite(cfg.path .. cfg.icon)
        n_bg.enabled = cfg.quality == 1
        s_bg.enabled = cfg.quality == 2
        name_text.text = self.helper:getLangCfg(cfg.name_id)
        level_text.text = "Lv." .. self:getTechLevel(msg.cfgId) + 1

        local clk = clk_trans:GetComponent(Clock)
        if not clk then clk = clk_trans:AddComponent(Clock) end
        clk:Set2(self:getLevelUpTime(), function(time)
            slider.value = 1 - time / next_detail_cfg.time
        end)

        btn.onClick:RemoveAllListeners()
        btn.onClick:AddListener(function()
            self:showSpeedUp()
        end)
    end,

    setRecommendInfo = function(self, msg)
        local list = self:getRecommendList(msg)
        local max_num = 3
        local layout_path = "content/base_trans/bottom/recommend/layout"
        self.helper:hideAllChild(self, layout_path)
        for k, v in ipairs(list) do
            if k <= max_num then
                local child = self.helper:getChild(self, k, layout_path)
                child:SetActive(true)
                local icon = child:Find("icon"):GetComponent(UnityEngine.UI.Image)
                local n_bg = child:Find("n_bg"):GetComponent(UnityEngine.UI.Image)
                local s_bg = child:Find("s_bg"):GetComponent(UnityEngine.UI.Image)
                local level_text = child:Find("levelText"):GetComponent(UnityEngine.UI.Text)
                local name_text = child:Find("nameText"):GetComponent(UnityEngine.UI.Text)
                local des_text = child:Find("desText"):GetComponent(UnityEngine.UI.Text)
                local btn = child:Find("Tech_panel_7"):GetComponent(UnityEngine.UI.Button)
                local cfg = config.home_config_research_lists[v.cfg_id]
                icon:SetSprite(cfg.path .. cfg.icon)
                n_bg.enabled = cfg.quality == 1
                s_bg.enabled = cfg.quality == 2
                level_text.text = "Lv." .. v.next_level
                name_text.text = self.helper:getLangCfg(cfg.name_id)
                local next_detail_cfg = self:getTechDetailCfg(cfg.id, v.next_level)
                local args = self.helper:getPropertyDes(next_detail_cfg.effect)
                for __, map in pairs(args) do
                    des_text.text = map.name_str .. " " .. map.num_str
                end

                btn.onClick:RemoveAllListeners()
                btn.onClick:AddListener(function()
                    self:click_tech(cfg)
                end)
            end
        end
    end,

    isAllResearchFullLevel = function(self, msg)
        local total_cur = 0
        local total_max = 0
        local length = #gConfig.home_config_research_trees
        for i = 1, length do
            local cur_num, max_num = self:getCategoryProgress(i)
            total_cur = total_cur + cur_num
            total_max = total_max + max_num
        end
        return total_cur >= total_max
    end,

    -- 获取推荐升级列表
    getRecommendList = function(self, msg)
        local list = {}
        config.home_config_research_lists:foreach(function(cfg_id, v)
            if self:isTechCanUpgrade(cfg_id) then
                local next_level = self:getTechLevel(cfg_id) + 1
                local detail_cfg = self:getTechDetailCfg(cfg_id, next_level)
                table.insert(list, { cfg_id = cfg_id, next_level = next_level, priority = detail_cfg.priority})
            end
        end)
        local sortTab = function(a, b)
            return a.priority < b.priority
        end
        table.sort(list, sortTab)
        return list
    end,

    -- 获取研发等级
    getTechLevel = function(self, cfg_id)
        local level = 0
        for __, t in ipairs(self.MPersonStudio.technologys) do
            if t.cfgId == cfg_id then
                return t.level
            end
        end
        return level
    end,

    isTechUnlock = function(self, cfg_id)
        local cfg = config.home_config_research_lists[cfg_id]
        local cdt = loadstring('return ' .. cfg.fronttech)()
        for __, c in ipairs(cdt) do
            local lv = self:getTechLevel(c)
            if lv <= 0 then
                return false
            end
        end
        return true
        -- return self:isTechCanUpgrade(cfg_id)
    end,

    -- 获取研发是否可点击升级
    -- fronttech 必须满足是1级
    -- lab_level 研究所等级
    -- andcondition 条件等级
    -- orcondition 条件等级
    isTechCanUpgrade = function(self, cfg_id)
        local cur_level = self:getTechLevel(cfg_id)
        local next_level = cur_level + 1
        local max_level = self:getTechMaxLevel(cfg_id)
        -- 满级
        if cur_level >= max_level then
            return false
        end
        -- 研究所等级
        local detail_cfg = self:getTechDetailCfg(cfg_id, next_level)
        local research_level = self:getResearchLevel()
        if research_level < detail_cfg.lab_level then
            return false
        end
        -- and 条件
        local cdt = loadstring('return ' .. detail_cfg.andcondition)()
        for __, level in pairs(cdt) do
            local lv = self:getTechLevel(__)
            if lv < level then
                return false
            end
        end
        return true
        -- 前置科技
        --[[
        local cfg = config.home_config_research_lists[cfg_id]
        local cdt = loadstring('return ' .. cfg.fronttech)()
        for __, c in ipairs(cdt) do
            local lv = self:getTechLevel(c)
            if lv <= 0 then
                return false
            end
        end
        ]]
        -- or 条件
        --[[
        local cdt = loadstring('return ' .. detail_cfg.orcondition)()
        local temp = false
        for __, level in pairs(cdt) do
            temp = true
            local lv = self:getTechLevel(__)
            if lv >= level then
                return true
            end
        end
        if temp then
            return false
        else
            return true
        end
        return temp
        ]]
    end,

    -- 获取研发最大等级
    getTechMaxLevel = function(self, cfg_id)
        local max_level = 0
        config.home_config_research_details:foreach(function(k, v)
            if v.list_id == cfg_id then
                max_level = max_level + 1
            end
        end)
        return max_level
    end,

    -- 研究所等级
    getResearchLevel = function(self)
        return home_info:getBuildDataById(self.build_id).level
    end,

    getTechDetailCfg = function(self, cfg_id, level)
        local cfg = nil
        config.home_config_research_details:foreach(function(k, v)
            if v.list_id == cfg_id and v.level == level then
                cfg = v
            end
        end)
        return cfg
    end,

    getResearchDetail = function(self, list_id, level)
        local rst = nil
        config.home_config_research_details:foreach(
            function(k, v)
                if v.list_id == list_id and v.level == level then
                    rst = v
                end
            end)
        return rst
    end,

    getCategoryCfg = function(self, category)
        local list = {}
        config.home_config_research_lists:foreach(function(k, v)
            if v.type == category then
                table.insert(list, v)
            end
        end)
        local sortTab = function(a, b)
            return a.id < b.id
        end
        table.sort(list, sortTab)
        return list
    end,

    -- 初始化二级界面
    showDetailUI = function(self, category)
        self.helper.showTrans(self.detail_trans)
        self.helper.hideTrans(self.base_trans)
        self.cur_category = category
        self:setDetailUIInfo(category)
    end,

    setDetailProgress = function(self, elapsed, total_time)
        local slider = self:findSlider("content/detail_trans/Scroll View/Viewport/Content/progress/bg/slider")
        local time = self:findText("content/detail_trans/Scroll View/Viewport/Content/progress/bg/time")
        slider.value = 1 - elapsed / total_time
        time.text = frame_helper:timeFormat(elapsed)
    end,

    getUpgradingCfgId = function(self)
        local build_data = home_info:getBuildDataByCfgId(2)
        if build_data.studio.tLevelUp > 0 then
            return build_data.studio.cfgId
        end
        return 0
    end,

    setDetailUIInfo = function(self, category)
        local progress_trans = self.transform:Find("content/detail_trans/Scroll View/Viewport/Content/progress")
        frame_helper.hideTrans(progress_trans)
        local upgrading_id = self:getUpgradingCfgId()

        local path = "content/detail_trans/Scroll View/Viewport/Content/icons"
        self.helper:hideAllChild(self, path)
        local line_idx = 1
        local list = self:getCategoryCfg(category)
        for __, cfg in ipairs(list) do
            local child = self.helper:getChild(self, __, path)
            child:SetActive(true)
            local icon = child:Find("icon"):GetComponent(UnityEngine.UI.Image)
            local n_bg = child:Find("n_bg"):GetComponent(UnityEngine.UI.Image)
            local s_bg = child:Find("s_bg"):GetComponent(UnityEngine.UI.Image)
            local ing_bg = child:Find("ing_bg"):GetComponent(UnityEngine.UI.Image)

            local level_text = child:Find("levelText"):GetComponent(UnityEngine.UI.Text)
            local name_text = child:Find("nameText"):GetComponent(UnityEngine.UI.Text)
            local can_upgrade = self:isTechUnlock(cfg.id)
            
            local tempF = function()
                if self.MPersonStudio.cfgId == cfg.id and self.MPersonStudio.tLevelUp > 0 then    
                    icon.color = Color.white
                else
                    icon.color = can_upgrade and Color.white or Color.grey
                end
            end

            tempF()
            icon:SetSprite(cfg.path .. cfg.icon, function() tempF() end)

            if self.MPersonStudio.cfgId == cfg.id and self.MPersonStudio.tLevelUp > 0 then
                n_bg.enabled = false
                s_bg.enabled = false
                ing_bg.enabled = true
            else
                n_bg.enabled = cfg.quality == 1
                s_bg.enabled = cfg.quality == 2
                ing_bg.enabled = false
                n_bg.color = can_upgrade and Color.white or Color.grey  
                s_bg.color = can_upgrade and Color.white or Color.grey  
            end

            name_text.text = self.helper:getLangCfg(cfg.name_id)

            local rect = child.transform:GetComponent(UnityEngine.RectTransform)
            rect.anchoredPosition = Vector2(cfg.x, cfg.y)
            level_text.text = "Lv." .. self:getTechLevel(cfg.id) .. "/" .. self:getTechMaxLevel(cfg.id)

            local btn = child:Find('icon'):GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(function()
                print("click_tech", cfg.id)
                self:click_tech(cfg)
            end)

            if upgrading_id == cfg.id then
                frame_helper.showTrans(progress_trans)
                local rect = self.transform:Find("content/detail_trans/Scroll View/Viewport/Content/progress/bg"):GetComponent(UnityEngine.RectTransform)
                rect.anchoredPosition = Vector2(cfg.x + 200, cfg.y)
                local e = home_info:getBuildDataByCfgId(2)
                local elapsed = e.studio.tLevelUp - Time.realtimeSinceStartup + e.rmbTime
                local total_time = e.studio.totalTime
                self:setDetailProgress(elapsed, total_time)
            end
        end

        local cfg = config.home_config_research_trees[category]
        local content_trans = self.transform:Find("content/detail_trans/Scroll View/Viewport/Content"):GetComponent(UnityEngine.RectTransform)
        content_trans.sizeDelta = Vector2(cfg.length, content_trans.sizeDelta.y)

        local getCenter = function(list1, list2)
            if #list1 == 1 and #list2 == 1 then
                local from = list1[1]
                local to = list2[1]
                return Vector2( (from.x + to.x) * 0.5, from.y)
            end
            local a = nil
            local b = nil
            if #list1 == 1 then
                a = list1
                b = list2
            else
                a = list2
                b = list1
            end
            if #b % 2 > 0 then
                local from = a[1]
                local temp_distance = frame_helper.max_number
                local rst = nil
                for __, to in ipairs(b) do
                    local d = Vector2.Distance(from, to)
                    if d < temp_distance then
                        temp_distance = d
                        rst = to
                    end
                end
                return Vector2( (from.x + rst.x) * 0.5, (from.y + rst.y) * 0.5 )
            else
                local x = (b[1].x + a[1].x) * 0.5
                local y = (b[1].y + b[#b].y) * 0.5
                return Vector2( x, y )
            end
        end

        local sortFunc = function(a, b)
            return a.y > b.y
        end

        self:initLine()
        self:clearLine()

        -- local offset = -115
        local offset = 0

        local cook_line = {}
        for __, cfg in ipairs(list) do

            local start_list = { Vector2(cfg.x + offset, cfg.y) }


            local end_list = {}


            local precondition = loadstring("return " .. cfg.fronttech)()

            for __, cfg_id in pairs(precondition) do

                local temp_cfg = config.home_config_research_lists[cfg_id]

                table.insert(end_list, Vector2(temp_cfg.x - offset, temp_cfg.y))

            end

            table.sort(end_list, sortFunc)

            if #start_list > 0 and #end_list > 0 then
                local center = getCenter(start_list, end_list)
                local unlock = self:isTechUnlock(cfg.id)
                if unlock then
                    local cur_level = self:getTechLevel(cfg.id)
                    if cur_level <= 0 then
                        self:addUnConnectLine(start_list, end_list, center, Color.white)
                    else
                        self:addConnectedLine(start_list, end_list, center, Color.white)
                    end
                else
                    self:addUnConnectLine(start_list, end_list, center, Color.white)
                end
            end
        end

        self:makeLine()
    end,

    initLine = function(self)
        self.connected_mono = self.transform:Find("content/detail_trans/Scroll View/Viewport/Content/connected"):GetComponent(TreeNodeArrows)
        self.un_connect_mono = self.transform:Find("content/detail_trans/Scroll View/Viewport/Content/un_connect"):GetComponent(TreeNodeArrows)
    end,

    clearLine = function(self)
        self.connected_mono:ClearArrow()
        self.un_connect_mono:ClearArrow()
    end,

    addConnectedLine = function(self, start_list, end_list, center, color)
        self.connected_mono:AddNodeData(start_list, end_list, center, color)
    end,

    addUnConnectLine = function(self, start_list, end_list, center, color)
        self.un_connect_mono:AddNodeData(start_list, end_list, center, color)
    end,

    makeLine = function(self)
        self.connected_mono:Excute()
        self.un_connect_mono:Excute()
    end,

    click_closeUpgradeBtn = function(self)
        self.helper.hideTrans(self.upgrade_trans)
        self.select_tech_cfg = nil
        self:hideTechInfo()
    end,

    -- msg = SCBuildingExpend
    showUpgrade = function(self, cfg)
        -- if guide_manager:isGuideRunning() and self.helper.isTransShow(self.upgrade_trans) then
        --     return
        -- end

        local cur_level = self:getTechLevel(cfg.id)
        local max_level = self:getTechMaxLevel(cfg.id)
        local cur_detail_cfg = self:getTechDetailCfg(cfg.id, cur_level)
        local next_detail_cfg = self:getTechDetailCfg(cfg.id, cur_level + 1)

        self:setTechBasicInfo(cfg)

        local setItemDetail = function(child, name, state1, state2, state3, state4, state5, state6)
            local des = child.transform:Find("text1_Static[T5]_9"):GetComponent(UnityEngine.UI.Text)
            local right_icon = child.transform:Find("rightIcon"):GetComponent(UnityEngine.UI.Image)
            local wrongIcon = child.transform:Find("wrongIcon"):GetComponent(UnityEngine.UI.Image)
            local go_btn = child.transform:Find("goBtn"):GetComponent(UnityEngine.UI.Button)
            local add_btn = child.transform:Find("addBtn"):GetComponent(UnityEngine.UI.Button)
            local speedup_btn = child.transform:Find("speedUpBtn"):GetComponent(UnityEngine.UI.Button)
            local free_btn = child.transform:Find("freenBtn"):GetComponent(UnityEngine.UI.Button)
            des.color = Color.white

            des.text = name
            right_icon.enabled = state1
            wrongIcon.enabled = state2
            go_btn.transform.localScale = state3 and Vector3.one or Vector3.zero
            add_btn.transform.localScale = state4 and Vector3.one or Vector3.zero
            speedup_btn.transform.localScale = state5 and Vector3.one or Vector3.zero
            free_btn.transform.localScale = state6 and Vector3.one or Vector3.zero
        end

        local path = "upgrade_trans/content/1/bg/un_upgrade/Right Scroll View/Viewport/Content"
        local build_data = home_info:getBuildDataByCfgId(2)

        -- 显示升级要求
        self.helper:hideAllChild(self, path, true)
        local idx = 1

        -- 1.正在升级的科技(非当前显示的科技)
        self.cache_upgrading_tech = nil
        if build_data.studio.tLevelUp > 0 and build_data.studio.cfgId ~= cfg.id then
            local child = self.helper:getChild(self, idx, path)
            child:SetActive(true)
            setItemDetail(
                child,
                self.helper:getLangCfg(config.home_config_research_lists[build_data.studio.cfgId].name_id),
                false,
                true,
                false,
                false,
                true,
                false)

            local speed_up_btn = child.transform:Find("speedUpBtn"):GetComponent(UnityEngine.UI.Button)
            speed_up_btn.onClick:RemoveAllListeners()
            speed_up_btn.onClick:AddListener(function()
                self:showSpeedUp()
            end)
            self.cache_upgrading_tech = child.transform:Find("text1_Static[T5]_9"):GetComponent(UnityEngine.UI.Text)

            local e = build_data
            local elapsed = e.studio.tLevelUp - Time.realtimeSinceStartup + e.rmbTime
            self:onTechUpgradeTick(e, elapsed, e.studio.totalTime)

            idx = idx + 1
        end

        -- 2.实验室等级
        local child = self.helper:getChild(self, idx, path)
        child:SetActive(true)
        
        local full_condition = build_data.level >= next_detail_cfg.lab_level
        setItemDetail(
                child,
                self.helper:getLangCfg(config.home_config_lists[2].name_id) .. " Lv." .. next_detail_cfg.lab_level,
                full_condition,
                not full_condition,
                not full_condition,
                false,
                false,
                false)

        -- local des = child.transform:Find("text1_Static[T5]_9"):GetComponent(UnityEngine.UI.Text)
        -- des.color = full_condition and Color.white or Color(1, 80/255, 80/255, 1)

        local bg = child.transform:Find("bg"):GetComponent(UnityEngine.UI.Image)
        bg.enabled = not full_condition
        local go_btn = child.transform:Find("goBtn"):GetComponent(UnityEngine.UI.Button)
        go_btn.onClick:RemoveAllListeners()
        go_btn.onClick:AddListener(function()
            print("前往研究所升级", next_detail_cfg.lab_level)
            frame_manager:hide('home_entity_research')
            local build_data = home_info:getBuildDataByCfgId(2)
            if build_data then
                network_sender:buildingExpend(build_data.id, common_pb.EBUILDING_LEVELUP)
            end
        end)
        idx = idx + 1

        -- 3.前置科技
        local temp = loadstring('return ' .. next_detail_cfg.andcondition)()
        for cfg_id, need_level in pairs(temp) do
            local child = self.helper:getChild(self, idx, path)
            child:SetActive(true)
            local cur_level = self:getTechLevel(cfg_id)
            local full_condition = cur_level >= need_level
            setItemDetail(
                child,
                frame_helper:getLangCfg(config.home_config_research_lists[cfg_id].name_id) .. " Lv." .. need_level,
                full_condition,
                not full_condition,
                not full_condition,
                false,
                false,
                false)

            local bg = child.transform:Find("bg"):GetComponent(UnityEngine.UI.Image)
            bg.enabled = not full_condition
            local go_btn = child.transform:Find("goBtn"):GetComponent(UnityEngine.UI.Button)
            go_btn.onClick:RemoveAllListeners()
            go_btn.onClick:AddListener(function()
                self:click_tech(config.home_config_research_lists[cfg_id])
            end)
            idx = idx + 1
        end

        -- 4.升级资源
        local need_res_diamond = 0
        local cost_table = home_info:getTechUpgradeCost(next_detail_cfg)
        for __, value in ipairs(cost_table) do
            local item_id = value.item_id
            local need_num = value.need_num

            local own_num = frame_helper:getItemNum(item_id)
            if need_num > own_num then
                need_res_diamond = need_res_diamond + frame_helper:getResToDiamondNum(item_id, need_num - own_num)
            end

            local child = self.helper:getChild(self, idx, path)
            child:SetActive(true)
            local str ,full_condition = self.helper:getItemNumFormat(item_id, need_num)
            setItemDetail(
                child,
                str,
                full_condition,
                not full_condition,
                false,
                not full_condition,
                false,
                false)
            
            local bg = child.transform:Find("bg"):GetComponent(UnityEngine.UI.Image)
            bg.enabled = not full_condition

            local icon = child.transform:Find("icon"):GetComponent(UnityEngine.UI.Image)
            icon:SetSprite(frame_helper:getItemIcon(item_id))

            if not full_condition then
                local btn = child.transform:Find("addBtn"):GetComponent(UnityEngine.UI.Button)
                btn.onClick:RemoveAllListeners()
                btn.onClick:AddListener(function()
                    print("添加资源!!!!")
                    frame_manager:show('resource_exchange', nil, {
                        item_id = item_id,
                        item_num = need_num,
                    })
                end)
            end
            idx = idx + 1
        end

        -- 科技详细信息
        local btn = self.transform:Find('upgrade_trans/content/1/detailInfoBtn'):GetComponent(UnityEngine.UI.Button)
        btn.onClick:RemoveAllListeners()
        btn.onClick:AddListener(function ()
            self:click_detailInfoBtn(cfg)
        end)
        
        -- 升级
        local btn = self.transform:Find('upgrade_trans/content/1/bg/un_upgrade/upgradeBtn'):GetComponent(UnityEngine.UI.Button)
        btn.onClick:RemoveAllListeners()
        btn.onClick:AddListener(function()
            self:click_upgradeBtn(cfg)
        end)

        local real_time = home_info:getTechUpgradeTime(next_detail_cfg)
        local unreal_upgrade_time = self:findText("upgrade_trans/content/1/bg/un_upgrade/upgradeBtn/unreal")
        local real_upgrade_time = self:findText("upgrade_trans/content/1/bg/un_upgrade/upgradeBtn/real")
        local image_text = self:findText("upgrade_trans/content/1/bg/un_upgrade/finishBtn/image_text")
        unreal_upgrade_time.text = string.format(NativeText.GetLangStr(290017), gui_helper:timeFormat(next_detail_cfg.time)) 
        real_upgrade_time.text = gui_helper:timeFormat(real_time)
        local cost_diamond = self:getCostDiamond(real_time) + need_res_diamond
        local DIAMOND    = 30001 -- 钻石
        local item_name = gui_helper:getImageTextItemStr(DIAMOND)
        image_text:SetImageText(string.format("%s %s", item_name, cost_diamond))

        local btn = self.transform:Find('upgrade_trans/content/1/bg/un_upgrade/finishBtn'):GetComponent(UnityEngine.UI.Button)
        btn.onClick:RemoveAllListeners()
        btn.onClick:AddListener(function()
            print("立即完成...")
            if frame_helper:isEnoughDiamond(cost_diamond) then
                network_sender:technologyUp(cfg.id, 1)
            end
        end)
    end,

    showUpgrading = function(self, build_data, elapsed, total_time)
        local slider = self:findSlider("upgrade_trans/content/1/bg/upgrading/slider")
        local time = self:findText("upgrade_trans/content/1/bg/upgrading/time")
        local image_text = self:findText("upgrade_trans/content/1/bg/upgrading/clearCountBtn/image_text")
        slider.value = 1 - elapsed / total_time
        time.text = string.format(NativeText.GetLangStr(340021), frame_helper:timeFormat(elapsed))
        local DIAMOND    = 30001 -- 钻石
        local item_name = gui_helper:getImageTextItemStr(DIAMOND)
        image_text:SetImageText(string.format("%s %s", item_name, self:getCostDiamond(elapsed)))
    end,

    click_tech = function(self, cfg)
        self:setTechDetailInfo(cfg)

        self.select_tech_cfg = cfg
        frame_helper.showTrans(self.upgrade_trans)
        local max_trans = self.transform:Find("upgrade_trans/content/1/bg/max_level")
        local un_upgrade_trans = self.transform:Find("upgrade_trans/content/1/bg/un_upgrade")
        local upgrading_trans = self.transform:Find("upgrade_trans/content/1/bg/upgrading")
        frame_helper.hideTrans(max_trans)
        frame_helper.hideTrans(un_upgrade_trans)
        frame_helper.hideTrans(upgrading_trans)

        local cur_level = self:getTechLevel(cfg.id)
        local max_level = self:getTechMaxLevel(cfg.id)
        -- 满级
        if cur_level >= max_level then
            self:setTechBasicInfo(cfg)
            frame_helper.showTrans(max_trans)
            return
        end
        -- 正在升级
        local build_data = home_info:getBuildDataByCfgId(2)
        if build_data.studio.tLevelUp > 0 and build_data.studio.cfgId == cfg.id then
            frame_helper.showTrans(upgrading_trans)
            self:setTechBasicInfo(cfg)
            local e = build_data
            local elapsed = e.studio.tLevelUp - Time.realtimeSinceStartup + e.rmbTime
            local total_time = e.studio.totalTime
            self:showUpgrading(e, elapsed, total_time)
        else
            frame_helper.showTrans(un_upgrade_trans)
            self:showUpgrade(cfg)
        end
    end,

    showTechInfo = function(self)
        local a = self.transform:Find("upgrade_trans/content/1")
        local b = self.transform:Find("upgrade_trans/content/2")
        frame_helper.showTrans(b)
        LeanTween.cancel(a.gameObject)
        LeanTween.cancel(b.gameObject)
        LeanTween.moveLocalX(a.gameObject, -346, 0.25)
        LeanTween.moveLocalX(b.gameObject, 346, 0.25)
    end,

    hideTechInfo = function(self)
        local a = self.transform:Find("upgrade_trans/content/1")
        local b = self.transform:Find("upgrade_trans/content/2")
        a.transform.localPosition = Vector3.zero
        b.transform.localPosition = Vector3.zero
        frame_helper.hideTrans(b)
    end,

    setTechDetailInfo = function(self, cfg)
        local cur_level = self:getTechLevel(cfg.id)
        local cfg = config.home_config_research_lists[cfg.id]

        local icon = self:findImage("upgrade_trans/content/2/rally_panel_2/Top/image/icon")
        local n_bg = self:findImage("upgrade_trans/content/2/rally_panel_2/Top/image/n_bg")
        local s_bg = self:findImage("upgrade_trans/content/2/rally_panel_2/Top/image/s_bg")
        local name = self:findText("upgrade_trans/content/2/rally_panel_2/Top/name")
        local prop_name =   self:findText("upgrade_trans/content/2/rally_panel_2/Top/prop_name")

        name.text = self.helper:getLangCfg(cfg.name_id)
        icon:SetSprite(cfg.path .. cfg.icon)
        n_bg.enabled = cfg.quality == 1
        s_bg.enabled = cfg.quality == 2

        local cfg_list = {}
        config.home_config_research_details:foreach(function(k, v)
            if v.list_id == cfg.id then
                table.insert(cfg_list, v)
            end
        end)
        local sortFunc = function(a, b)
            return a.id < b.id
        end
        table.sort(cfg_list, sortFunc)
        local layout_path = "upgrade_trans/content/2/rally_panel_2/Top/Scroll View/Viewport/Content"
        self.helper:hideAllChild(self, layout_path, true)
        for __, value in ipairs(cfg_list) do
            local child = self.helper:getChild(self, __, layout_path)
            child:SetActive(true)
            local level = child:Find("level"):GetComponent(UnityEngine.UI.Text)
            local power = child:Find("power"):GetComponent(UnityEngine.UI.Text)
            local des = child:Find("des"):GetComponent(UnityEngine.UI.Text)
            local select = child:Find("select"):GetComponent(UnityEngine.UI.Image)
            level.text = __
            power.text = value.power
            
            if string.len(value.EffectLang) <= 0 then
                local cur_args = self.helper:getPropertyDes(value.effect)
                prop_name.text = cur_args[1].name_str
                des.text = cur_args[1].num_str
            else
                prop_name.text = frame_helper:getLangCfg(value.EffectLang)
                des.text = NativeText.GetLangStr(340024)
            end
            
            select.enabled = cur_level == __
        end
    end,

    -- 设置科技的基本信息
    setTechBasicInfo = function(self, cfg)
        local cur_level = self:getTechLevel(cfg.id)
        local max_level = self:getTechMaxLevel(cfg.id)
        local cfg = config.home_config_research_lists[cfg.id]

        local icon = self:findImage("upgrade_trans/content/1/image/icon")
        local n_bg = self:findImage("upgrade_trans/content/1/image/n_bg")
        local s_bg = self:findImage("upgrade_trans/content/1/image/s_bg")
        local name = self:findText("upgrade_trans/content/1/name")


        local prop_arrow = self:findImage("upgrade_trans/content/1/is_prop/arrow1")
        prop_arrow.enabled = true

        local is_prop_trans = self.transform:Find("upgrade_trans/content/1/is_prop")
        local prop_name =   self:findText("upgrade_trans/content/1/prop_name1")
        local cur_prop =    self:findText("upgrade_trans/content/1/is_prop/cur1")
        local next_prop =   self:findText("upgrade_trans/content/1/is_prop/next1")

        local power_un_max_trans = self.transform:Find("upgrade_trans/content/1/power_un_max")
        
        local power_name =   self:findText("upgrade_trans/content/1/prop_name2")
        local cur_power =    self:findText("upgrade_trans/content/1/cur2")
        local next_power =   self:findText("upgrade_trans/content/1/power_un_max/next2")

        name.text = string.format("%s(%s/%s)", self.helper:getLangCfg(cfg.name_id), cur_level, max_level)
        icon:SetSprite(cfg.path .. cfg.icon)
        n_bg.enabled = cfg.quality == 1
        s_bg.enabled = cfg.quality == 2

        if cur_level <= 0 then
            local next_detail_cfg = self:getTechDetailCfg(cfg.id, cur_level + 1)
            local is_prop = string.len(next_detail_cfg.EffectLang) <= 0
            frame_helper:setActive(is_prop_trans, is_prop)
            frame_helper:setActive(power_un_max_trans, true)

            -- 是属性
            if is_prop then
                local next_args = self.helper:getPropertyDes(next_detail_cfg.effect)
                prop_name.text = next_args[1].name_str
                cur_prop.text = next_args[1].is_percent and "0%" or "0"
                next_prop.text = next_args[1].num_str
            else
                prop_name.text = frame_helper:getLangCfg(next_detail_cfg.EffectLang)
                cur_prop.text = ""
                next_prop.text = ""
            end

            cur_power.text = "0"
            next_power.text = next_detail_cfg.power
        elseif cur_level >= max_level then
            local cur_detail_cfg = self:getTechDetailCfg(cfg.id, cur_level)

            local is_prop = string.len(cur_detail_cfg.EffectLang) <= 0
            frame_helper:setActive(is_prop_trans, is_prop)
            frame_helper:setActive(power_un_max_trans, false)

            prop_arrow.enabled = false

            if is_prop then
                local cur_args = self.helper:getPropertyDes(cur_detail_cfg.effect)
                prop_name.text = cur_args[1].name_str
                cur_prop.text = cur_args[1].num_str
                next_prop.text = ""
                
            else
                prop_name.text = frame_helper:getLangCfg(cur_detail_cfg.EffectLang)
                cur_prop.text = ""
                next_prop.text = ""
            end

            cur_power.text = cur_detail_cfg.power
            next_power.text = ""
        else
            local cur_detail_cfg = self:getTechDetailCfg(cfg.id, cur_level)
            local next_detail_cfg = self:getTechDetailCfg(cfg.id, cur_level + 1)

            local is_prop = string.len(cur_detail_cfg.EffectLang) <= 0
            frame_helper:setActive(is_prop_trans, is_prop)
            frame_helper:setActive(power_un_max_trans, true)

            if is_prop then
                local cur_args = self.helper:getPropertyDes(cur_detail_cfg.effect)
                prop_name.text = cur_args[1].name_str
                cur_prop.text = cur_args[1].num_str
                local next_args = self.helper:getPropertyDes(next_detail_cfg.effect)
                next_prop.text = next_args[1].num_str
            else
                prop_name.text = frame_helper:getLangCfg(cur_detail_cfg.EffectLang)
                cur_prop.text = ""
                next_prop.text = ""
            end

            cur_power.text = cur_detail_cfg.power
            next_power.text = next_detail_cfg.power
        end
    end,

    -- cfg 是 home_config_research_lists
    click_select_rsc = function(self, cfg_id, cfg, cur_lv, max_lv)
        local cur_lv_cfg = self:getResearchDetailCfg(cfg_id, cur_lv)

        self.curr_skill = {id = cfg_id, cfg = cfg, cur_lv = cur_lv, max_lv = max_lv}

        local icon = self:findImage("detail_trans/leftContent/top_icon")
        icon:SetSprite(cfg.path .. cfg.icon)

        local btn_path = "detail_trans/leftContent/upgradeBtn"
        local cost_icon = self:findImage("detail_trans/leftContent/down_icon")
        local cost_text = self:findText("detail_trans/leftContent/down_icon/Text")
        local upgrade_btn = self.transform:Find(btn_path)
        local name_text = self:findText("detail_trans/leftContent/top_icon/Text")
        local time_text = self:findText("detail_trans/leftContent/Image/Text")
        local top_name_text = self:findText("detail_trans/leftContent/nameText")
        local prop_text1 = self:findText("detail_trans/leftContent/textLine/text")
        local prop_text2 = self:findText("detail_trans/leftContent/textLine (1)/text")
        local speed_up_btn = self.transform:Find("detail_trans/leftContent/speedUpBtn")
        
        if cur_lv_cfg then
            local t = self.helper:getPropertyDes(cur_lv_cfg.effect)
            prop_text1.text = ""
            for k, v in ipairs(t) do
                prop_text1.text = prop_text1.text .. v.name_str .. " " .. v.num_str .. ""
            end
        else
            prop_text1.text = ""
        end

        cost_icon.gameObject:SetActive(cur_lv < max_lv)
        --upgrade_btn.gameObject:SetActive(cur_lv < max_lv)

        name_text.text = self.helper:getLangCfg(cfg.name_id)
        top_name_text.text = self.helper:getLangCfg(cfg.name_id) .. "<color=#07F9C9FF>" .. " lv" .. cur_lv .. "</color>"

        local is_enough_res = true

        local next_lv_cfg = self:getResearchDetailCfg(cfg_id, cur_lv + 1)

        if cur_lv < max_lv then
            if next_lv_cfg then
                print("########### 属性:", cfg_id, (cur_lv + 1), next_lv_cfg.effect)
                local t = self.helper:getPropertyDes(next_lv_cfg.effect)
                prop_text2.text = ""
                for k, v in ipairs(t) do
                    prop_text2.text = prop_text2.text .. v.name_str .. " " .. v.num_str .. ""
                end

                local cost_args = self.helper:getJsonTab(next_lv_cfg.cost)
                for item_id, need_num in pairs(cost_args) do
                    local owner_num = self.helper:getItemNum(tonumber(item_id))
                    cost_text.text = owner_num .. "/" .. need_num
                    cost_text.color = tonumber(need_num) > tonumber(owner_num) and Color.red or Color.green
                    local icon_name = self.helper:getItemIcon(item_id)
                    cost_icon:SetSprite(icon_name)
                    time_text.text = self.helper:timeFormat(next_lv_cfg.time) 

                    if tonumber(need_num) > tonumber(owner_num) then
                        is_enough_res = false
                        print(string.format("升级需求道具%s", item_id))
                    end
                end
                self:speedUpBtnState(cfg_id)
            end
        else
            print("满级")
            upgrade_btn.gameObject:SetActive(false)
            speed_up_btn:SetActive(false)
        end

        self:clearScript("click", btn_path)
        self:setScript("click", btn_path, function()
            print("升级:", cfg_id, self.MPersonStudio.tLevelUp)
            -- 当前研究所等级
            local cur_rsc_lv = home_info:getResearchEntityLv()

            if self.MPersonStudio.tLevelUp > 0 then
                local tb = {
                    title = NativeText.GetLangStr(130001),
                    content = NativeText.GetLangStr(130005),
                    buttons =
                    {
                        {
                            btn_name = NativeText.GetLangStr(130002),
                            callback = function() frame_manager:hide('common_box') end
                        }
                    },
                }
                frame_manager:show('common_box', nil, tb)
            elseif not is_enough_res then
                local tb = {
                    title = NativeText.GetLangStr(130001),
                    content = NativeText.GetLangStr(130006),
                    buttons =
                    {
                        {
                            btn_name = NativeText.GetLangStr(130002),
                            callback = function() frame_manager:hide('common_box') end
                        }
                    },
                }
                frame_manager:show('common_box', nil, tb)
            elseif cur_rsc_lv < next_lv_cfg.lab_level then
                local tb = {
                    title = NativeText.GetLangStr(130001),
                    content = NativeText.GetLangStr(130007),
                    buttons =
                    {
                        {
                            btn_name = NativeText.GetLangStr(130002),
                            callback = function() frame_manager:hide('common_box') end
                        }
                    },
                }
                frame_manager:show('common_box', nil, tb)
            else
                network_sender:technologyUp(cfg_id)
            end
        end)
    end,

    getDetailCfg = function(self, category)
        local sort_list = {}

        config.home_config_research_lists:foreach(
            function(k, v)
                if v.type == category then
                    table.insert(sort_list, v)
                end
            end
        )

        table.sort(sort_list, function(a, b)
            if a.quality == b.quality then
                return a.id < b.id
            else
                return a.quality < b.quality
            end
        end)

        return sort_list
    end,

    -- home_config_research_lists
    getDetailLevel = function(self, cfg_id)
        local cur_lv = 0
        for k, v in ipairs(self.MPersonStudio.technologys) do
            if v.cfgId == cfg_id then
                cur_lv = v.level
                break
            end
        end

        local max_level = 0

        config.home_config_research_details:foreach(
            function(k, v)
                if v.list_id == cfg_id and v.level > max_level then
                    max_level = v.level
                end
            end
        )

        return cur_lv, max_level
    end,

    -- home_config_research_details
    getResearchDetailCfg = function(self, cfg_id, next_lv)
        local cfg = nil
        config.home_config_research_details:foreach(
            function(k, v)
                if v.list_id == cfg_id and v.level == next_lv then
                    cfg = v
                end
            end
        )
        return cfg
    end,

    -- home_config_research_lists
    isUpgrade = function(self, cfg_id)
        if self.MPersonStudio.cfgId == cfg_id and self.MPersonStudio.tLevelUp > 0 then
            return true
        end
        return false
    end,

    setMainInfo = function(self)
        local length = #gConfig.home_config_research_trees
        for i = 1, length do
            local path = "content/base_trans/layout/tplt (" .. i .. ")"
            local trans = self.transform:Find(path)
            if not trans then break end
            local cfg = config.home_config_research_trees[i]
            local title_text = self:findText(path .. "/nameText")
            title_text.text = self.helper:getLangCfg(cfg.name_id)
            local big_icon = self:findImage(path .. "/big_icon")
            big_icon:SetSprite(cfg.big_path .. cfg.big_icon)
            local big_icon = self:findImage(path .. "/big_icon")
            big_icon:SetSprite(cfg.big_path .. cfg.big_icon)
            local small_icon = self:findImage(path .. "/small_icon")
            small_icon:SetSprite(cfg.small_path .. cfg.small_icon)

            local btn = self.transform:Find(path):GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(
                function ()
                   self:click_category(i)
                end)
            local num_text = self:findText(path .. "/progressText")
            local cur_num, max_num = self:getCategoryProgress(i)
            num_text.text = cur_num .. "/" .. max_num
        end
    end,

    getCategoryProgress = function(self, category)
        local tb = {}
        config.home_config_research_lists:foreach(
            function(k, v)
                if v.type == category then
                    table.insert(tb, k)
                end
            end
        )

        local max_num = 0
        config.home_config_research_details:foreach(
            function(k, v)
                for i, j in ipairs(tb) do
                    if v.list_id == j then
                        max_num = max_num + 1
                        break
                    end
                end
            end
        )

        local num = 0
        for k, v in ipairs(self.MPersonStudio.technologys) do
            local cfg = config.home_config_research_lists[v.cfgId]
            if cfg.type == category then
                num = num + v.level
            end
        end
        return num, max_num
    end,

    -- 选择主界面上的类别
    click_category = function (self, index)
        self:showDetailUI(index)
    end,

    click_closeDetailBtn = function(self)
        self.helper.showTrans(self.base_trans)
        self.helper.hideTrans(self.detail_trans)
    end,

     -- 获取配置文件
    getUpgradeCfg = function(self, cfg_id, level)
        local rst = nil
        local max_level = 1
        config.home_config_upgrades:foreach(
            function(k, v)
                if v.building_id == cfg_id then
                    if v.level == level then rst = v end
                    if v.level > max_level then max_level = v.level end
                end
            end
        )
        return rst, max_level
    end,

    speedUpBtnState = function(self, id)
        local speed_up_btn = self.transform:Find("detail_trans/leftContent/speedUpBtn"):GetComponent(UnityEngine.UI.Button)
        local upgrade_btn = self.transform:Find("detail_trans/leftContent/upgradeBtn"):GetComponent(UnityEngine.UI.Button)

        local showSpeedUp = local_guild:isSelfInGuild() and self:isUpgrade(id)

        speed_up_btn.transform:SetActive(showSpeedUp)
        upgrade_btn.transform:SetActive(not showSpeedUp)

        if showSpeedUp then
            if self.MPersonStudio.canHelp then
                speed_up_btn.targetGraphic.sprite = speed_up_btn.spriteState.highlightedSprite
                local path = "detail_trans/leftContent/speedUpBtn"
                self:clearScript("click", path)
                self:setScript("click", path, functor.rmake(self.onClickSpeedUp, id))
            else
                speed_up_btn.targetGraphic.sprite = speed_up_btn.spriteState.disabledSprite
            end

            speed_up_btn.interactable = self.MPersonStudio.canHelp
        end
    end,

    onClickSpeedUp = function(self, handler, id)
        network_sender:reqHelpFastBuild(id, common_pb.HT_RES)
    end,

    getCostDiamond = function(self, time)
        return gui_helper:getSpeedUpCost(44, time)
    end,

    showSpeedUp = function(self)
        --[[
        local cb = function(id, count, finish)
            network_sender:technologySpeedUp(self.MPersonStudio.cfgId, id, count, finish)
            frame_manager:hide("common_speed_up")
        end
        self.speed_up_frame = frame_manager:show("common_speed_up", self, 
            {
                id = self.MPersonStudio.cfgId,
                type = 44,
                left_time = self:getLevelUpTime(), 
                total_time = self:getTotalTime(), 
                title = NativeText.GetLangStr(340017), 
                callback = cb})
        ]]

        local e = home_info:getBuildDataByCfgId(2)
        if e.studio.tLevelUp > 0 then
            local elapsed = e.studio.tLevelUp - Time.realtimeSinceStartup + e.rmbTime
            local total_time = e.studio.totalTime
            frame_manager:show("home_speed_up", nil, {
                speed_up_type = home_speed_up_type.tech_upgrade,
                id = e.id,
                tech_cfg_id = e.studio.cfgId,
            })
        end
    end,

    getLevelUpTime = function(self)
        local temp = self.MPersonStudio.tLevelUp -  Time.realtimeSinceStartup + home_info:getBuildDataById(self.build_id).rmbTime
        temp = temp >= 0 and temp or 0
        return temp
    end,

    getTotalTime = function(self)
        local cur_level = self:getTechLevel(self.MPersonStudio.cfgId)
        local next_detail_cfg = self:getTechDetailCfg(self.MPersonStudio.cfgId, cur_level + 1)
        return next_detail_cfg.time
    end,

    -- 判断是否科技已经完成(显示气泡)
    isTechCompleted = function(self)
        local research_cfg_id = 2
        local entity_data = home_info:getBuildDataByCfgId(research_cfg_id)
        if entity_data and entity_data.studio then
            for __, t in ipairs(entity_data.studio) do
                if t.isBubble then  -- 气泡
                    network_sender:clickBubblie(common_pb.BT_TECHNOLOGY, t.cfgId)
                end
            end
        end
    end,

    click_speedUpBtn = function(self)
        self:showSpeedUp()
    end,

    click_clearCountBtn = function(self)
        local build_data = home_info:getBuildDataByCfgId(2)
        network_sender:perSonBaseSpeedUp(build_data.id, common_pb.EPB_TECH_RESEARCH, 1, 0, {}, build_data.studio.cfgId)
    end,

    onBuildUpgradeTick = function(self, build_data, elapsed, total_time)
    end,

    onTechUpgradeTick = function(self, build_data, elapsed, total_time)
        if self.cache_upgrading_tech then
            local tech_name = frame_helper:getLangCfg(config.home_config_research_lists[build_data.studio.cfgId].name_id)
            -- local tech_level = home_info:getTechLevel(build_data.studio.cfgId)
            -- self.cache_upgrading_tech.text = string.format("%s %s %s", tech_name, "Lv." .. tech_level, frame_helper:timeFormat(elapsed))
            self.cache_upgrading_tech.text = string.format("%s %s", tech_name, frame_helper:timeFormat(elapsed))
        end
    end,

    click_detailInfoBtn = function(self, cfg)
        self:showTechInfo()
    end,

    click_upgradeBtn = function(self, cfg)
        if self.MPersonStudio.tLevelUp > 0  then
            hint:setText(NativeText.GetLangStr(340013))
        else
            network_sender:technologyUp(cfg.id, 0)
        end
    end,
}
