local super = require("ui.frame")
local att = 1010

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        self.owner = self.args.owner    -- game_state_home.lua
        self.msg = self.args.data       -- MPersonTrain

        -- self.home_entity = self.args.data

        clickEvents = 
        {
            "closeBtn",
            "offices/closeOfficesBtn",
            "offices/bg/sendBtn",
            "centerLeft/addItemBtn",
            "items/closeItemsBtn",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self.officers_trans = self.transform:Find("offices")
        self.helper.hideTrans(self.officers_trans)
        
        self.items_trans = self.transform:Find("items")
        self.helper.hideTrans(self.items_trans)

        self:initUI(self.msg)
    end,

    onUnload = function (self)
    end,

    refreshUI = function(self, msg)
        self.msg = msg
        -- print("刷新界面")
        self:initUI(self.msg)
    end,

    -- msg = MPersonTrain
    initUI = function(self, msg)
        local max_slot = 5

        local upgrade_cfg = self:getUpgradeCfg(msg.baseObj.cfgId, msg.baseObj.level)
        if not upgrade_cfg then
            log.error("init error !!!", msg.baseObj.cfgId, msg.baseObj.level)
            return
        end

        local tmp = self.helper:getJsonTab(upgrade_cfg.unlock)
        local unlock_num = 0
        for k, v in pairs(tmp) do
            if k == att then
                unlock_num = tonumber(v)
                break
            end
        end

        if msg.project[1].trainId ~= 0 then
            local item_cfg = self.helper:getItemCfg(msg.project[1].trainId)
            self:findText("centerLeft/addItemBtn/line/line (1)/text").text = self.helper:getLangCfg(item_cfg.name_id)
            local train_cfg = config.home_config_trainings[msg.project[1].trainId]
            self:findImage("centerLeft/addItemBtn/icon"):SetSprite(train_cfg.path .. train_cfg.icon)

            local layout_trans = self.transform:Find("centerLeft/addItemBtn/layout")
            for i = 0, layout_trans.childCount - 1 do
                layout_trans:GetChild(i).gameObject:SetActive(false)
            end

            local args = self.helper:getJsonTab(train_cfg.client_item_reward)
            local pos = 0
            for k, v in pairs(args) do 
                local child = layout_trans:GetChild(pos)
                child.gameObject:SetActive(true)
                local icon = child.transform:Find("icon"):GetComponent(UnityEngine.UI.Image)
                icon:SetSprite(self.helper:getItemIcon(k)) 
                local text = child.transform:Find("icon/Text"):GetComponent(UnityEngine.UI.Text)
                text.text = v
                pos = pos + 1
            end
        end

        for i = 1, max_slot do
            local path = "centerRight/up_layout/item (" .. i .. ")/"
            
            local icon = self:findImage(path .. "icon")
            local add = self:findImage(path .. "add")
            local lock = self:findImage(path .. "lock")
            local slider = self:findImage(path .. "time/slider")
            local time_trans = self.transform:Find(path .. "time")
            local text = self:findText(path .. "time/Text")
            local removeBtn = self.transform:Find(path .. "removeBtn")

            self.helper:setImageGray(self, path .. "btn", i > unlock_num)

            self:clearScript("click", path .. "btn")
            self:setScript("click", path .. "btn", function()
                self:click_officerSlot(i, i <= unlock_num and i > #msg.trainOffices)
            end)

            self:clearScript("click", path .. "removeBtn")

            if i <= unlock_num then
                lock.enabled = false
                if msg.trainOffices[i] then
                    time_trans.gameObject:SetActive(true)
                    icon.enabled = true
                    add.enabled = false
                    local officer = self:getOfficerById(msg.trainOffices[i].officeId)
                    if officer then
                        text.text = "Lv." .. officer.level
                        slider.transform.localScale = Vector3(officer.exp / officer.level_exp.exp, 1, 1)
                    end
                    
                    removeBtn.gameObject:SetActive(true)
                    self:setScript("click", path .. "removeBtn", function()
                        self:click_remove_office(msg.trainOffices[i].officeId)
                    end)
                else
                    time_trans.gameObject:SetActive(false)
                    icon.enabled = false
                    add.enabled = true
                    removeBtn.gameObject:SetActive(false)
                end
            else
                time_trans.gameObject:SetActive(false)
                icon.enabled = false
                add.enabled = false
                lock.enabled = true
                removeBtn.gameObject:SetActive(false)
            end
        end
    end,

    -- 根据指挥官id，返回officer.lua实例
    getOfficerById = function(self, id)
        local ofs = hero_manager:getOwnOfficers()
        for __, officer in ipairs(ofs) do
            if officer.cfg_id == id then
                return officer
            end
        end
        log.error("getOfficerById ！！！", id)
        return nil
    end,

    -- 显示指挥官界面
    showOfficers = function(self)
        self.officers = hero_manager:getOwnOfficers()
        local pos = 1
        for idx, officer in ipairs(self.officers) do
            local except = false
            for __, data in ipairs(self.msg.trainOffices) do
                if data.officeId == officer.inst_id then
                    except = true
                    break
                end
            end
            if not except then
                local trans, full_path = self:getLayoutChild(idx)
                if trans then
                    local slider_trans = trans:Find("time/slider")
                    local level_text = trans:Find("time/Text"):GetComponent(UnityEngine.UI.Text)
                    level_text.text = "Lv." .. officer.level
                    slider_trans.localScale = Vector3(officer.exp / officer.level_exp.exp, 1, 1)
                    self:clearScript("click", full_path)
                    self:setScript("click", full_path, function() self:click_select_officer(idx, officer) end)
                end
                pos = pos + 1
            end
        end
        self:hideChildsAtPos(pos)
        self.select_officer = nil
        self:setSelect(nil)
        self.helper.showTrans(self.officers_trans)
    end,

    -- 缓存layout,idx从1开始
    getLayoutChild = function(self, idx)
        local path = "offices/bg/layout"
        local layout_trans = self.transform:Find(path)
        if not self.cache_layout then
            self.cache_layout = {}
            for i = 0, layout_trans.childCount - 1 do
                local child_trans = layout_trans:GetChild(i)
                child_trans.transform.name = i + 1
                table.insert(self.cache_layout, child_trans.transform)
            end
        end
        if idx > #self.cache_layout then
            for i = 1, idx - #self.cache_layout do
                local tplt = layout_trans:GetChild(0)
                local child_trans = Object.Instantiate(tplt.gameObject);
                child_trans.transform.name = i + #self.cache_layout
                child_trans:AttachTo(tplt.transform.parent)
                table.insert(self.cache_layout, child_trans.transform)
            end
        end
        self.cache_layout[idx].gameObject:SetActive(true)
        local full_path = path .. "/" .. self.cache_layout[idx].transform.name
        return self.cache_layout[idx], full_path
    end,

    -- 隐藏从索引后面的对象
    hideChildsAtPos = function(self, start_pos)
        if not self.cache_layout then
            return
        end
        for i = start_pos, #self.cache_layout do
            local child_trans = self.cache_layout[i]
            if child_trans then
                child_trans.gameObject:SetActive(false)
            end
        end
    end,

    -- 选中框
    setSelect = function(self, idx)
        for __, child in ipairs(self.cache_layout) do
            local select_trans = child:Find("select")
            select_trans.gameObject:SetActive(idx == __)
        end
    end,

    -- 指挥官列表界面，选择指挥官事件
    click_select_officer = function (self, idx, officer)
        self.select_officer = officer
        self:setSelect(idx)
    end,

    -- 点击指挥官槽位事件
    click_officerSlot = function(self, slot_idx, can_add)
        if can_add then
            self:showOfficers()
        else
            -- hint:setText("未开启!!!")
        end
    end,

    -- 移除槽位指挥官事件
    click_remove_office = function(self, slot_idx)
        -- print("移除指挥官", slot_idx)

        local tb = {
            title = NativeText.GetLangStr(130001),
            content = NativeText.GetLangStr(130008),
            buttons =
            {
                {
                    btn_name = NativeText.GetLangStr(130003),
                    callback = function() 
                        frame_manager:hide('common_box')
                    end
                },
                {
                    btn_name = NativeText.GetLangStr(130002),
                    callback = function() 
                        frame_manager:hide('common_box')
                        self.helper.hideTrans(self.officer_trans)
                        network_sender:pbRemoveOffice(slot_idx)
                    end
                },
            },
        }

        frame_manager:show('common_box', nil, tb)
    end,

    -- 指挥官界面派遣指挥官
    click_sendBtn = function(self)
        if self.select_officer ~= nil then
            -- print("派遣", self.select_officer.cfg_id)
            network_sender:pbAddOffice(self.select_officer.cfg_id)
        end
        self.helper.hideTrans(self.officers_trans)
    end,

    -- 主界面添加道具
    click_addItemBtn = function(self)
        -- print("添加道具")
        self.helper.showTrans(self.items_trans)
        local items = {}
        local bag = bag_manager:getBag(2)
        bag:getItemsByType(22, items)

        for i = 1, 10 do
            local path = "items/bg/layout/tplt (".. i .. ")"
            local trans = self.transform:Find(path)
            trans.gameObject:SetActive(i < #items)
            if i < #items then
                local icon_name = self.helper:getItemIcon(items[i].cfg_id)           
                local icon = trans.transform:Find("icon"):GetComponent(UnityEngine.UI.Image)
                icon:SetSprite(icon_name)
                local text = trans.transform:Find("num_text"):GetComponent(UnityEngine.UI.Text)
                text.text = items[i].count

                self:clearScript("click", path)
                self:setScript("click", path, function () self:click_useItemBtn(items[i]) end )
            end
        end
    end,

    -- item = item.lua
    click_useItemBtn = function(self, item)
        self.helper.hideTrans(self.items_trans)
        print(item.cfg_id)
        network_sender:pbStartProject(item.cfg_id)
    end,

    click_closeItemsBtn = function(self)
        self.helper.hideTrans(self.items_trans)
    end,

    getUpgradeCfg = function(self, cfg_id, level)
        local rst = nil
        local max_level = 1
        config.home_config_upgrades:foreach(
            function(k, v)
                if v.building_id == cfg_id then
                    if v.level == level then rst = v end
                    if v.level > max_level then max_level = v.level end
                end
            end
        )
        return rst, max_level
    end,

    click_closeBtn = function(self)
        frame_manager:hide('home_entity_train')
    end,

    click_closeOfficesBtn = function(self)
        self.helper.hideTrans(self.officers_trans)
    end,
}
