local super = require("ui.frame")

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)
        
        self.msg = self.args.data -- SCBuildingExpend
        self.home_map = self.args.owner
        self.id = self.msg.id

        -- 第一次进来的建筑id
        self.init_id = self.id

        clickEvents = {
            "content/backBtn",
            "content/finishBtn",
            "content/freeBtn",
            "content/un_upgrading/upgradeBtn",
            "content/upgrading/speedUpBtn",
            "content/deleteBtn",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self.BUILD_UPGRADE_TICK = home_info.events:on("BUILD_UPGRADE_TICK", functor.make(self.onUpgradingEntity, self))
        self.BUILD_INFO_CHANGE = home_info.events:on("BUILD_INFO_CHANGE", functor.make(self.onEntityInfoChange, self))
        self.onPropChange = local_player.events:on("onPropChange", functor.make(self.onResourceChange, self))
        self.onBagChanged = bag_manager.events:on('onBagChanged', functor.make(self.onResourceChange, self))

        self:init()
    end,

    onUnload = function(self)
        home_info.events:off("BUILD_UPGRADE_TICK", self.BUILD_UPGRADE_TICK)
        home_info.events:off("BUILD_INFO_CHANGE", self.BUILD_INFO_CHANGE)
        local_player.events:off("onPropChange", self.onPropChange)
        bag_manager.events:off("onBagChanged", self.onBagChanged)
        self:setHomeFrameData(true)
    end,

    init = function(self)
        if self:isMaxLevel() then
            self:click_backBtn()
            return
        end

        self:setBasicInfo(self.id)
        self:moveCameraToBuildPos()
        self:setHomeFrameData(false)
    end,

    setHomeFrameData = function(self, rst)
        local vsb, frame = frame_manager:isDisplay("home")
        if vsb then
            frame:addCommandBuff(function()
                frame:setTopLeftVsb(rst)
            end)
        end
    end,

    -- refresh = function(self, msg)
    --     if self:isSkipFrame(msg) then
    --         local id = self:removeExBuildId()
    --         network_sender:buildingExpend(id, common_pb.EBUILDING_LEVELUP)
    --     else
    --         self:addExBuildId(self.msg.id)
    --         self.msg = msg
    --         self.id = self.msg.id
    --         self:init()
    --     end
    -- end,

    -- 是否跳转界面
    isSkipFrame = function(self, msg)
        if self.init_id == msg.id then
            return false
        else
            local target_data = home_info:getBuildDataById(msg.id)
            local build_data = home_info:getBuildDataById(self.id)
            local next_cfg, max_level = self:getUpgradeCfg(build_data.cfgId, build_data.level + 1)
            local temp = loadstring("return " .. next_cfg.upgrade_limit)()
            local target_level = temp[target_data.cfgId]
            return target_data.level >= target_level
        end
    end,

    addExBuildId = function(self, id)
        self.ex_ids = self.ex_ids or {}
        local count = #self.ex_ids
        if count > 0 and self.ex_ids[count] == id then
            return
        end
        table.insert(self.ex_ids, id)
    end,

    removeExBuildId = function(self)
        self.ex_ids = self.ex_ids or {}
        local count = #self.ex_ids
        local id = self.ex_ids[count]
        table.remove(self.ex_ids, count)
        return id
    end,

    click_backBtn = function(self)
        frame_manager:hide('home_entity_upgrade')
        if self.init_id == self.id then
            local model_trans = self.home_map:getHomeBuildById(self.id).transform
            local build_data = home_info:getBuildDataById(self.id)
            local cfg_id = build_data.cfgId
            self.home_map.home_camera:LookAt(model_trans.transform, Vector3(UnityEngine.Screen.width / 2, UnityEngine.Screen.height / 2, 0), config.home_config_lists[cfg_id].exit_upgrade_fov, nil)
        end
    end,

    click_finishBtn = function(self)
        local is_enough = frame_helper:isEnoughDiamond(self.msg.finishCost.Count)
        if not is_enough then
            return
        end
        local build_data = home_info:getBuildDataById(self.id)
        local cur_cfg, max_level = self:getUpgradeCfg(build_data.cfgId, build_data.level)
        local next_cfg, max_level = self:getUpgradeCfg(build_data.cfgId, build_data.level + 1)
        -- 2.前置建筑等级
        local temp = loadstring("return " .. next_cfg.upgrade_limit)()
        for cfg_id, level in pairs(temp) do
            local entity = home_info:getBuildDataByCfgId(cfg_id)
            local full_condition = ( entity and entity.level or 0 ) >= level
            if not full_condition then
                local build_data = home_info:getBuildDataByCfgId(cfg_id)
                if build_data then
                    frame_helper:showCommonBox(NativeText.GetLangStr(290023), function()
                        network_sender:buildingExpend(build_data.id, common_pb.EBUILDING_LEVELUP)
                    end)
                else
                    hint:setText(string.format(NativeText.GetLangStr(290018), frame_helper:getLangCfg(config.home_config_lists[cfg_id].name_id)))
                end
                return
            end
        end
        network_sender:personBaseLevelUp(self.id, 1)
    end,

    click_freeBtn = function(self)
        print("免费时间:", frame_helper:timeFormat(home_info.building_free_time))

        local build_data = home_info:getBuildDataById(self.id)
        if build_data.tLevelUp > 0 then
            network_sender:perSonBaseSpeedUp(self.id, common_pb.EPB_BUILD, 1, 0, {})
        else
            local cur_cfg, max_level = self:getUpgradeCfg(build_data.cfgId, build_data.level)
            local next_cfg, max_level = self:getUpgradeCfg(build_data.cfgId, build_data.level + 1)
            -- 2.前置建筑等级
            local temp = loadstring("return " .. next_cfg.upgrade_limit)()
            for cfg_id, level in pairs(temp) do
                local entity = home_info:getBuildDataByCfgId(cfg_id)
                local full_condition = ( entity and entity.level or 0 ) >= level
                if not full_condition then
                    local build_data = home_info:getBuildDataByCfgId(cfg_id)
                    if build_data then
                        frame_helper:showCommonBox(NativeText.GetLangStr(290023), function()
                            network_sender:buildingExpend(build_data.id, common_pb.EBUILDING_LEVELUP)
                        end)
                    else
                        hint:setText(string.format(NativeText.GetLangStr(290018), frame_helper:getLangCfg(config.home_config_lists[cfg_id].name_id)))
                    end
                    return
                end
            end
            -- 3.升级资源
            for __, cost in ipairs(self.msg.levelUpCost) do
                local str ,full_condition = self.helper:getItemNumFormat(cost.ItemTemId, cost.Count)
                if not full_condition then
                    frame_helper:showCommonBox(NativeText.GetLangStr(290022), function()
                        frame_manager:show('resource_exchange', nil, {
                            item_id = cost.ItemTemId,
                            item_num = cost.Count,
                        })
                    end)
                    return
                end
            end

            network_sender:personBaseLevelUp(self.id, 1)
        end
    end,

    click_upgradeBtn = function(self)
        local build_data = home_info:getBuildDataById(self.id)
        local cur_cfg, max_level = self:getUpgradeCfg(build_data.cfgId, build_data.level)
        local next_cfg, max_level = self:getUpgradeCfg(build_data.cfgId, build_data.level + 1)
        -- 1.建筑队列
        local upgrading_entitys = home_info:getAllUpgradingEntity()
        local builder_num = home_info:getBuilderNum()
        if #upgrading_entitys >= builder_num then
            for __, build_data in ipairs(upgrading_entitys) do
                frame_helper:showCommonBox(NativeText.GetLangStr(290021), function()
                    frame_manager:show("home_speed_up", nil, {
                        speed_up_type = home_speed_up_type.build_upgrade,
                        id = build_data.id
                    })
                end)
                return
            end
        end
        -- 2.前置建筑等级
        local temp = loadstring("return " .. next_cfg.upgrade_limit)()
        for cfg_id, level in pairs(temp) do
            local entity = home_info:getBuildDataByCfgId(cfg_id)
            local full_condition = ( entity and entity.level or 0 ) >= level
            if not full_condition then
                local build_data = home_info:getBuildDataByCfgId(cfg_id)
                if build_data then
                    frame_helper:showCommonBox(NativeText.GetLangStr(290023), function()
                        network_sender:buildingExpend(build_data.id, common_pb.EBUILDING_LEVELUP)
                    end)
                else
                    hint:setText(string.format(NativeText.GetLangStr(290018), frame_helper:getLangCfg(config.home_config_lists[cfg_id].name_id)))
                end
                return
            end
        end
        -- 3.升级资源
        for __, cost in ipairs(self.msg.levelUpCost) do
            local str ,full_condition = self.helper:getItemNumFormat(cost.ItemTemId, cost.Count)
            if not full_condition then
                frame_helper:showCommonBox(NativeText.GetLangStr(290022), function()
                    frame_manager:show('resource_exchange', nil, {
                        item_id = cost.ItemTemId,
                        item_num = cost.Count,
                    })
                end)
                return
            end
        end
        network_sender:personBaseLevelUp(self.id, 0)
        self:click_backBtn()
    end,

    click_deleteBtn = function(self)
        local tb = {
            title = "",
            content = NativeText.GetLangStr(260031),
            buttons =
            {
                {
                    btn_name = NativeText.GetLangStr(260032),
                    callback = function() 
                        frame_manager:hide('common_box')
                        frame_manager:hide('home_entity_upgrade')
                        network_sender:personBaseDel(self.id)
                    end
                },
                {
                    btn_name = NativeText.GetLangStr(260033),
                    callback = function()
                        frame_manager:hide('common_box')
                    end
                },
            },
        }
        frame_manager:show('common_box', nil, tb)
    end,

    getUpgradeCfg = function(self, cfg_id, level)
        local rst = nil
        local max_level = 1
        config.home_config_upgrades:foreach(
            function(k, v)
                if v.building_id == cfg_id then
                    if v.level == level then rst = v end
                    if v.level > max_level then max_level = v.level end
                end
            end
        )
        return rst, max_level
    end,

    setBasicInfo = function(self, id)
        local build_data = home_info:getBuildDataById(self.id)
        self.cur_cfg, self.max_level = self:getUpgradeCfg(build_data.cfgId, build_data.level)
        self.next_cfg, self.max_level = self:getUpgradeCfg(build_data.cfgId, build_data.level + 1)

        local icon =    self:findImage('content/icon')
        local name =    self:findText('content/name_text')
        local delete_trans = self.transform:Find("content/deleteBtn")

        local can_destroy = config.home_config_lists[build_data.cfgId].can_destory == 1
        if can_destroy then
            frame_helper.showTrans(delete_trans)
        else
            frame_helper.hideTrans(delete_trans)
        end
        
        local cur_cfg, max_level = self:getUpgradeCfg(build_data.cfgId, build_data.level)
        local next_cfg, max_level = self:getUpgradeCfg(build_data.cfgId, build_data.level + 1)

        local cfg = config.home_config_lists[build_data.cfgId]
        if cfg then
            name.text = string.format("Lv.%s <color=#00D2FFFF>%s</color>", build_data.level, self.helper:getLangCfg(cfg.name_id))
            icon:SetSprite(cfg.path .. cfg.icon)
        end
        
        -- 下一级预览
        local getPropertyById = function(cfg, id)
            if cfg then
                local args = self.helper:getPropertyDes(cfg.effect)    
                for __, map in pairs(args) do
                    if map.id == id then
                        return map
                    end
                end
            end
        end
        local path = "content/Left Scroll View/Viewport/Content"
        self.helper:hideAllChild(self, path)
        local idx = 1

        for i = 1, 1 do
            local child = self.helper:getChild(self, idx, path)
            child:SetActive(true)
            local name = child.transform:Find("name_text"):GetComponent(UnityEngine.UI.Text)
            local cur_level = child.transform:Find("cur_text"):GetComponent(UnityEngine.UI.Text)
            local next_level = child.transform:Find("next_text"):GetComponent(UnityEngine.UI.Text)
            name.text = NativeText.GetLangStr(330002)
            cur_level.text = "Lv." .. build_data.level
            next_level.text = "Lv." .. build_data.level + 1
            idx = idx + 1
        end

        local args = self.helper:getPropertyDes(next_cfg and next_cfg.effect or cur_cfg.effect)
        for __, map in pairs(args) do
            local child = self.helper:getChild(self, idx, path)
            child:SetActive(true)
            local name = child.transform:Find("name_text"):GetComponent(UnityEngine.UI.Text)
            local cur_level = child.transform:Find("cur_text"):GetComponent(UnityEngine.UI.Text)
            local next_level = child.transform:Find("next_text"):GetComponent(UnityEngine.UI.Text)
            name.text = map.name_str
            next_level.text = map.num_str
            local cur_p = getPropertyById(cur_cfg, map.id)
            cur_level.text = cur_p and cur_p.num_str or ( map.is_percent and "0%" or "0" ) 
            idx = idx + 1
        end

        -- 下一个解锁预览
        local unlock_trans = self.transform:Find("content/unlock")
        local unlock_cfg = self:getNextUnlock(build_data.cfgId, build_data.level)
        if unlock_cfg then
            frame_helper.showTrans(unlock_trans)
            local name = self:findText("content/unlock/unlock_condition_text")
            local effect = self:findText("content/unlock/unlock_des_text")
            local icon = self:findImage("content/unlock/icon")

            local effect_table = frame_helper:getPropertyDes(unlock_cfg.unlock)
            local effect_str = ""
            for __, e in ipairs(effect_table) do
                effect_str = effect_str .. e.name_str .. " <color=#00FF00FF>+" .. e.num_str .. "</color>"
                effect_str = effect_str .. (__ == #effect_table and "" or ", ")
            end

            name.text = string.format("Lv.%s %s", unlock_cfg.level, self.helper:getLangCfg(config.home_config_lists[build_data.cfgId].name_id))
            effect.text = effect_str
            icon:SetSprite(unlock_cfg.unlock_des_path .. unlock_cfg.unlock_des_icon)
        else
            frame_helper.hideTrans(unlock_trans)
        end

        -- 升级要求
        self:setRequirements()
    end,

    onResourceChange = function(self)
        -- self:setRequirements()
        network_sender:buildingExpend(self.id, common_pb.EBUILDING_LEVELUP)
    end,

    setRequirements = function(self)
        local build_data = home_info:getBuildDataById(self.id)

        local un_upgrading_trans = self.transform:Find("content/un_upgrading")
        local upgrading_trans = self.transform:Find("content/upgrading")
        

        local is_upgrading = build_data.tLevelUp > 0
        if is_upgrading then
            frame_helper.showTrans(upgrading_trans)
            frame_helper.hideTrans(un_upgrading_trans)

            self:onUpgradingEntity(build_data, home_info:getBuildUpgradeLeftTime(self.id), build_data.totaTime)
        else
            frame_helper.showTrans(un_upgrading_trans)
            frame_helper.hideTrans(upgrading_trans)

            self:initBtns()

            local unreal_upgrade_time = self:findText("content/un_upgrading/upgradeBtn/unreal")
            local real_upgrade_time = self:findText("content/un_upgrading/upgradeBtn/real")

            unreal_upgrade_time.text = string.format(NativeText.GetLangStr(290017), gui_helper:timeFormat(self.next_cfg.time)) 
            real_upgrade_time.text = gui_helper:timeFormat(self.msg.time)

            local cur_cfg, max_level = self:getUpgradeCfg(build_data.cfgId, build_data.level)
            local next_cfg, max_level = self:getUpgradeCfg(build_data.cfgId, build_data.level + 1)

            local setItemDetail = function(child, name, state1, state2, state3, state4, state5, state6)
                local des = child.transform:Find("text1_Static[T5]_9"):GetComponent(UnityEngine.UI.Text)
                local right_icon = child.transform:Find("rightIcon"):GetComponent(UnityEngine.UI.Image)
                local wrongIcon = child.transform:Find("wrongIcon"):GetComponent(UnityEngine.UI.Image)
                local go_btn = child.transform:Find("goBtn"):GetComponent(UnityEngine.UI.Button)
                local add_btn = child.transform:Find("addBtn"):GetComponent(UnityEngine.UI.Button)
                local speedup_btn = child.transform:Find("speedUpBtn"):GetComponent(UnityEngine.UI.Button)
                local free_btn = child.transform:Find("freenBtn"):GetComponent(UnityEngine.UI.Button)

                des.text = name
                des.color = Color.white
                right_icon.enabled = state1
                wrongIcon.enabled = state2
                go_btn.transform.localScale = state3 and Vector3.one or Vector3.zero
                add_btn.transform.localScale = state4 and Vector3.one or Vector3.zero
                speedup_btn.transform.localScale = state5 and Vector3.one or Vector3.zero
                speedup_btn.transform.localScale = state5 and Vector3.one or Vector3.zero
                free_btn.transform.localScale = state6 and Vector3.one or Vector3.zero
            end

            -- 显示升级要求
            local path = "content/un_upgrading/Right Scroll View/Viewport/Content"
            self.helper:hideAllChild(self, path)
            self.upgrading_ctx = {}
            local idx = 1
            -- 1.建筑队列
            local upgrading_entitys = home_info:getAllUpgradingEntity()
            local builder_num = home_info:getBuilderNum()
            if #upgrading_entitys >= builder_num then
                for __, build_data in ipairs(upgrading_entitys) do
                    local child = self.helper:getChild(self, idx, path)
                    child:SetActive(true)

                    local id = build_data.id
                    local is_free = home_info:isBuildInFreeTime(id)

                    setItemDetail(
                        child,
                        string.format(NativeText.GetLangStr(290016), build_data.level + 1, frame_helper:getLangCfg(config.home_config_lists[build_data.cfgId].name_id), build_data.tLevelUp),
                        false,
                        false,
                        false,
                        false,
                        not is_free,
                        is_free)
                    
                    local full_condition = false
                    local bg = child.transform:Find("bg"):GetComponent(UnityEngine.UI.Image)
                    bg.enabled = not full_condition

                    local icon = child.transform:Find("icon"):GetComponent(UnityEngine.UI.Image)
                    icon:SetSprite(config.home_config_ui_icons[1].building_queue)

                    local free_btn = child.transform:Find("freenBtn"):GetComponent(UnityEngine.UI.Button)
                    free_btn.onClick:RemoveAllListeners()
                    free_btn.onClick:AddListener(function()
                        network_sender:perSonBaseSpeedUp(id, common_pb.EPB_BUILD, 1, 0, {})
                    end)

                    local speed_btn = child.transform:Find("speedUpBtn"):GetComponent(UnityEngine.UI.Button)
                    speed_btn.onClick:RemoveAllListeners()
                    speed_btn.onClick:AddListener(function()
                        frame_manager:show("home_speed_up", nil, {
                            speed_up_type = home_speed_up_type.build_upgrade,
                            id = build_data.id
                        })
                    end)

                    self.upgrading_ctx[id] = {
                        text = child.transform:Find("text1_Static[T5]_9"):GetComponent(UnityEngine.UI.Text),
                        free_btn = free_btn,
                        speed_btn = speed_btn,
                    }

                    idx = idx + 1
                end
            end
            -- 2.前置建筑等级
            local temp = loadstring("return " .. next_cfg.upgrade_limit)()
            for cfg_id, level in pairs(temp) do
                local child = self.helper:getChild(self, idx, path)
                child:SetActive(true)
                local entity = home_info:getBuildDataByCfgId(cfg_id)
                local full_condition = ( entity and entity.level or 0 ) >= level
                setItemDetail(
                    child,
                    string.format('%s Lv.%s', self.helper:getLangCfg(config.home_config_lists[cfg_id].name_id), level),
                    full_condition,
                    not full_condition,
                    not full_condition,
                    false,
                    false,
                    false)

                -- local des = child.transform:Find("text1_Static[T5]_9"):GetComponent(UnityEngine.UI.Text)
                -- des.color = full_condition and Color.white or Color(1, 80/255, 80/255, 1)

                local bg = child.transform:Find("bg"):GetComponent(UnityEngine.UI.Image)
                bg.enabled = not full_condition

                local icon = child.transform:Find("icon"):GetComponent(UnityEngine.UI.Image)
                icon:SetSprite(config.home_config_ui_icons[1].pre_building)

                if not full_condition then
                    local btn = child.transform:Find("goBtn"):GetComponent(UnityEngine.UI.Button)
                    btn.onClick:RemoveAllListeners()
                    btn.onClick:AddListener(function()
                        local build_data = home_info:getBuildDataByCfgId(cfg_id)
                        if build_data then
                            print("前往升级!!!!")
                            network_sender:buildingExpend(build_data.id, common_pb.EBUILDING_LEVELUP)
                            self:cacheCondition(build_data.id, level)
                        else
                            -- hint:setText(string.format(NativeText.GetLangStr(290018), frame_helper:getLangCfg(config.home_config_lists[cfg_id].name_id)))
                            local grid_id = home_info:getGridIdByBuildCfgId(cfg_id)
                            if not grid_id then
                                return
                            end

                            local is_lock = home_info:isGridLock(grid_id)
                            if is_lock then
                                hint:setText(NativeText.GetLangStr(999015))
                                return
                            end

                            frame_manager:hide('home_entity_upgrade')
                            self.home_map:clickEntityByGridId(grid_id, 1)
                        end
                    end)
                end
                idx = idx + 1
            end

            -- 3.升级资源
            for __, cost in ipairs(self.msg.levelUpCost) do
                local child = self.helper:getChild(self, idx, path)
                child:SetActive(true)
                local str ,full_condition = self.helper:getItemNumFormat(cost.ItemTemId, cost.Count)
                setItemDetail(
                    child,
                    str,
                    full_condition,
                    not full_condition,
                    false,
                    not full_condition,
                    false,
                    false)
                
                local bg = child.transform:Find("bg"):GetComponent(UnityEngine.UI.Image)
                bg.enabled = not full_condition

                local icon = child.transform:Find("icon"):GetComponent(UnityEngine.UI.Image)
                icon:SetSprite(frame_helper:getItemIcon(cost.ItemTemId))

                if not full_condition then
                    local btn = child.transform:Find("addBtn"):GetComponent(UnityEngine.UI.Button)
                    btn.onClick:RemoveAllListeners()
                    btn.onClick:AddListener(function()
                        print("添加资源!!!!", cost.ItemTemId, cost.Count)
                        frame_manager:show('resource_exchange', nil, {
                            item_id = cost.ItemTemId,
                            item_num = cost.Count,
                        })
                    end)
                end

                idx = idx + 1
            end
        end
    end,

    getEffectStr = function(self, effect)
        local tab = frame_helper:getPropertyDes(effect)
        local str = {}
        for k, v in ipairs(tab) do
        end
    end,

    getNextUnlock = function(self, cfg_id, level)
        local cfg = nil
        config.home_config_upgrades:foreach(
            function(k, v)
                if v.building_id == cfg_id and v.level > level and v.unlock then
                    if #frame_helper:getPropertyDes(v.unlock) > 0 then
                        if cfg then
                            if v.level < cfg.level then
                                cfg = v
                            end
                        else
                            cfg = v
                        end
                    end
                end
            end)
        return cfg
    end,

    click_speedUpBtn = function(self)
        frame_manager:show("home_speed_up", nil, {
            speed_up_type = home_speed_up_type.build_upgrade,
            id = self.id
        })
    end,

    -- 判断条件是否满足
    -- isFullCondition = function(self)
        --[[
        local build_data = home_info:getBuildDataById(self.id)
        if not build_data then
            return false
        end
        local cur_cfg, max_level = self:getUpgradeCfg(build_data.cfgId, build_data.level)
        local next_cfg, max_level = self:getUpgradeCfg(build_data.cfgId, build_data.level + 1)
        -- 判断是否有建筑在升级
        local upgrading_entity = home_info:getUpgradingEntity()
        if upgrading_entity then
            if upgrading_entity.id == self.id then
                return true
            end

            local tb = {
                title = "",
                content = NativeText.GetLangStr(290014),
                buttons =
                {
                    {
                        btn_name = NativeText.GetLangStr(110522),
                        callback = function() 
                            frame_manager:hide('common_box')
                        end
                    },
                    {
                        btn_name = NativeText.GetLangStr(110521),
                        callback = function()
                            frame_manager:hide('common_box')
                            self:click_backBtn()
                            frame_manager:show('entity_speed_up', nil, {
                                build_data = upgrading_entity,
                                callback = function()
                                    self:click_backBtn()
                                end,
                            })

                        end
                    },
                },
            }
            frame_manager:show('common_box', nil, tb)
            return false
        end
        -- 其他建筑等级限制
        local limit_table = loadstring("return " .. next_cfg.upgrade_limit)()
        for cfg_id, level in pairs(limit_table) do
            local build_data = home_info:getBuildDataByCfgId(cfg_id)
            local self_cfg = config.home_config_lists[build_data.cfgId]
            local other_cfg = config.home_config_lists[build_data.cfgId]
            local name1 = self.helper:getLangCfg(self_cfg.name_id)
            local name2 = self.helper:getLangCfg(other_cfg.name_id)

            if build_data and build_data.level < level then
                local tb = {
                    title = "",
                    content = string.format(NativeText.GetLangStr(290013), name1, name2),
                    buttons =
                    {
                        {
                            btn_name = NativeText.GetLangStr(110522),
                            callback = function() 
                                frame_manager:hide('common_box')
                            end
                        },
                        {
                            btn_name = NativeText.GetLangStr(110521),
                            callback = function()
                                frame_manager:hide('common_box')
                                self:click_backBtn()
                                self.home_map:moveToEntityOpenUI(cfg_id)
                            end
                        },
                    },
                }
                frame_manager:show('common_box', nil, tb)
                return false
            end
        end
        -- 消耗判断
        local rst = true
        local need_table = {}
        -- local cost_table = loadstring("return " .. next_cfg.cost)()
        local cost_table = self.msg.levelUpCost
        for k, v in ipairs(cost_table) do
            local item_id = v.ItemTemId 
            local need_num = v.Count
            local own_num = self.helper:getItemNum(item_id)
            if need_num > own_num then
                rst = false
                need_table[item_id] = need_num
            end
        end
        if not rst then
            frame_manager:show('resource_convert', nil, 
                {
                    bar_idx = 1,
                    need_res = need_table,
                })
            return rst
        end
        return true
        ]]
    -- end,

    -- 升级中的建筑
    onUpgradingEntity = function(self, build_data, elapsed, total_time)
        if self.upgrading_ctx and self.upgrading_ctx[build_data.id] then
            local ctx = self.upgrading_ctx[build_data.id]
            local entity_name = frame_helper:getLangCfg(config.home_config_lists[build_data.cfgId].name_id)
            ctx.text.text = string.format(NativeText.GetLangStr(290016), build_data.level + 1, entity_name, self.helper:timeFormat(elapsed))
            local is_free = (elapsed <= home_info.building_free_time)
            if is_free then
                frame_helper.showTrans(ctx.free_btn.transform)
                frame_helper.hideTrans(ctx.speed_btn.transform)
            else
                frame_helper.showTrans(ctx.speed_btn.transform)
                frame_helper.hideTrans(ctx.free_btn.transform)
            end
        end

        if build_data.id == self.id then
            local slider = self:findSlider("content/upgrading/slider")
            local time = self:findText("content/upgrading/time")

            local is_free = home_info:isBuildInFreeTime(self.id)
            slider.value = 1 - elapsed / total_time
            time.text = self.helper:timeFormat(elapsed)

            self:refreshBtns(elapsed)
        end
    end,

    initBtns = function(self, cost)
        local finish_trans = self.transform:Find("content/finishBtn")
        local free_trans = self.transform:Find("content/freeBtn")
        local left_time, is_upgrading = home_info:getBuildUpgradeLeftTime(self.id)
        local is_free = left_time <= home_info.building_free_time

        if is_upgrading and left_time <= home_info.building_free_time then
            frame_helper.showTrans(free_trans)
            frame_helper.hideTrans(finish_trans)
        else
            frame_helper.showTrans(finish_trans)
            frame_helper.hideTrans(free_trans)

            local DIAMOND    = 30001 -- 钻石
            local item_name = gui_helper:getImageTextItemStr(DIAMOND)
            local image_text = self:findText("content/finishBtn/image_text")
            local str_cost = cost and cost or self.msg.finishCost.Count
            image_text:SetImageText(string.format("%s %s", item_name, str_cost))
        end
    end,

    refreshBtns = function(self, left_time)
        local str_cost = gui_helper:getSpeedUpCost(43, left_time)
        self:initBtns(str_cost)
    end,

    -- 信息变化的时候
    onEntityInfoChange = function(self, id)
        if id == self.id then
            if self:isMaxLevel() then
                self:click_backBtn()
            else
                network_sender:buildingExpend(id, common_pb.EBUILDING_LEVELUP)
            end
        else
            self:setRequirements()
        end
    end,

    isMaxLevel = function(self)
        local build_data = home_info:getBuildDataById(self.id)
        self.cur_cfg, self.max_level = self:getUpgradeCfg(build_data.cfgId, build_data.level)
        return build_data.level >= self.max_level
    end,

    moveCameraToBuildPos = function(self)
        local build_data = home_info:getBuildDataById(self.id)
        local cfg_id = build_data.cfgId
        local model_trans = self.home_map:getHomeBuildById(self.id).model.transform
        local end_point = frame_manager.camera:WorldToScreenPoint(self.transform:Find("content/icon").position)
        self.home_map.home_camera:LookAt(model_trans, Vector3(end_point.x, end_point.y, 0), config.home_config_lists[cfg_id].upgrade_fov, nil)
    end,

    

    refresh = function(self, msg)
        local rst, ex_build_id = self:isFullCondition(self.msg.id)
        if rst then
            network_sender:buildingExpend(ex_build_id, common_pb.EBUILDING_LEVELUP)
        else
            self.msg = msg
            self.id = self.msg.id
            self:init()
        end
    end,

    isFullCondition = function(self, build_id)
        if not self.cache_cdt then
            return false
        end

        local skip = false
        local build_data = home_info:getBuildDataById(build_id)
        for __, value in ipairs(self.cache_cdt) do
            if value.build_id == build_id and build_data.level >= value.level then
                table.remove(self.cache_cdt, __)
                -- log.error("缓存移除：", build_id, value.level)
                skip = true
                break
            end
        end

        if skip then
            local length = #self.cache_cdt
            if length > 0 then
                local ex_build_id = self.cache_cdt[length].build_id
                -- log.error("返回结果:", skip, ex_build_id)
                return skip, ex_build_id
            else
                -- log.error("返回结果:", skip, self.init_id)
                return skip, self.init_id
            end
        else
            -- log.error("返回结果:", false, self.init_id)
            return false, self.init_id
        end
    end,

    cacheCondition = function(self, build_id, level)
        self.cache_cdt = self.cache_cdt or {}
        if #self.cache_cdt > 0 then
            local data = self.cache_cdt[#self.cache_cdt]
            if data.build_id == build_id then
                return
            end
        end

        -- log.error("缓存插入：", build_id, level)

        table.insert(self.cache_cdt, {
            build_id = build_id,
            level = level,
        })
    end,
}
