local super = require("ui.frame")

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        self.item_cls = self.args.data

        -- 路径: root/login
        -- 事件: click_login 
        clickEvents = {
            "closeBtn",
            "content/dismantleBtn",
            "content/upgradeBtn",
            "content/equipBtn",
            "content/unEquipBtn"
        }

        -- 注册点击事件
        frame_helper:regClickEvents("click", self, clickEvents)

        self:init()
    end,

    onUnload = function (self)
    end,

    init = function(self)
        local icon = self:findImage("content/item/icon")
        local quality = self:findImage("content/item/quality")
        local equip_tag = self:findImage("content/item/equip_tag")

        local equip_btn = self.transform:Find("content/equipBtn")
        local un_equip_btn = self.transform:Find("content/unEquipBtn")

        local cfg = self.item_cls.cfg
        icon:SetSprite(cfg.path .. cfg.icon)
        quality:SetSprite(gui_helper:getQualityIconPath(cfg.color))
        local star_path = "content/item/light stars"
        frame_helper:hideAllChild(self, star_path, true)
        for i = 1, cfg.color do
            local star = frame_helper:getChild(self, i, star_path)
            star:SetActive(true)
        end

        local is_equipped = crew_manager:isItemEquipped(self.item_cls.inst_id)
        equip_tag.enabled = is_equipped

        frame_helper:setActive(equip_btn, not is_equipped)
        frame_helper:setActive(un_equip_btn, is_equipped)


        local prop_list = frame_helper:getPropertyDes(cfg.EffectPara)
        local prop_path = "content/left prop layout"
        frame_helper:hideAllChild(self, prop_path, true)
        for __, v in ipairs(prop_list) do
            local item = frame_helper:getChild(self, __, prop_path)
            item:SetActive(true)
            local name = item:Find("name"):GetComponent(UnityEngine.UI.Text)
            local value = item:Find("value"):GetComponent(UnityEngine.UI.Text)
            name.text = v.name_str
            value.text = v.num_str
        end

        local need_level = self:findText("content/need_level")
        if local_lord.level >= cfg.UserLevel then
            need_level.text = string.format(NativeText.GetLangStr(620011), cfg.UserLevel)
        else
            need_level.text = string.format(NativeText.GetLangStr(620015), cfg.UserLevel)
        end
        
        local prop_path = "content/right prop layout"
        frame_helper:hideAllChild(self, prop_path, true)

        local equipped_item = crew_manager:getEquippedCrewBySlot(cfg.Slot)


        if is_equipped then
            local prop_list = frame_helper:getPropertyDes(cfg.EffectPara)
            for __, v in ipairs(prop_list) do
                local item = frame_helper:getChild(self, __, prop_path)
                item:SetActive(true)
                local name = item:Find("name"):GetComponent(UnityEngine.UI.Text)
                local value = item:Find("value"):GetComponent(UnityEngine.UI.Text)
                name.text = v.name_str
                value.text = "-" .. v.num_str
                value.color = Color.red
            end
        else
            if equipped_item then
                local select_item = self.item_cls
    
                local un_equip_prop = frame_helper:getPropertyDes(cfg.EffectPara)
                local equipped_prop = frame_helper:getPropertyDes(equipped_item.cfg.EffectPara)
    
                local getPropValue = function(list, id)
                    for __, value in ipairs(list) do
                        if value.id == id then
                            return value.value
                        end
                    end
                    return nil
                end
    
                for idx, v in ipairs(un_equip_prop) do
                    local item = frame_helper:getChild(self, idx, prop_path)
                    item:SetActive(true)
                    local name = item:Find("name"):GetComponent(UnityEngine.UI.Text)
                    local value = item:Find("value"):GetComponent(UnityEngine.UI.Text)
                    name.text = v.name_str
                    local rst = v.value - ( getPropValue(equipped_prop, v.id) or 0 )
                    if rst >= 0 then
                        value.color = Color.green
                        value.text = "+" .. (v.is_percent and ( rst * 0.01 .. "%" ) or rst)
                    else
                        value.color = Color.red
                        value.text = (v.is_percent and ( rst * 0.01 .. "%" ) or rst)
                    end
                end
    
                local temp_idx = 1 + #un_equip_prop
                for idx, v in ipairs(equipped_prop) do
                    local has_prop = getPropValue(un_equip_prop, v.id) and true or false
                    if not has_prop then
                        local item = frame_helper:getChild(self, temp_idx, prop_path)
                        item:SetActive(true)
                        local name = item:Find("name"):GetComponent(UnityEngine.UI.Text)
                        local value = item:Find("value"):GetComponent(UnityEngine.UI.Text)
                        name.text = v.name_str
                        value.color = Color.green
                        value.text = "-" .. v.num_str
                        temp_idx = temp_idx + 1
                    end
                end
            else
                local prop_list = frame_helper:getPropertyDes(cfg.EffectPara)
                for __, v in ipairs(prop_list) do
                    local item = frame_helper:getChild(self, __, prop_path)
                    item:SetActive(true)
                    local name = item:Find("name"):GetComponent(UnityEngine.UI.Text)
                    local value = item:Find("value"):GetComponent(UnityEngine.UI.Text)
                    name.text = v.name_str
                    value.text = "+" .. v.num_str
                end
            end
        end
    end,

    click_closeBtn = function(self)
        frame_manager:hide('home_forge_equip')
    end,

    click_dismantleBtn = function(self)
        frame_manager:hide('home_forge_equip')
        frame_manager:show('home_forge_dismantle', nil, self.item_cls)
    end,

    click_upgradeBtn = function(self)
        frame_manager:show('home_forge_upgrade', nil, self.item_cls.cfg)
        frame_manager:hide('home_forge_equip')
    end,

    click_equipBtn = function(self)
        frame_manager:hide('home_forge_equip')
        network_sender:crewEquip(self.item_cls.inst_id, self.item_cls.cfg.Slot)
    end,

    click_unEquipBtn = function(self)
        frame_manager:hide('home_forge_equip')
        network_sender:crewUnequip(self.item_cls.inst_id, self.item_cls.cfg.Slot)
    end,
}
