local super = require("ui.frame")

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        self.item_cls = self.args.data

        -- 路径: root/login
        -- 事件: click_login 
        clickEvents = {
            "closeBtn",
            "content/fuse_content/fuseBtn",
            "content/fuse_content/fuseAllBtn",
            "content/divide_content/devideBtn",
            "content/divide_content/devideAllBtn",
        }

        -- 注册点击事件
        frame_helper:regClickEvents("click", self, clickEvents)

        self.event_ids = {}
        self.event_ids['onBagChanged'] = bag_manager.events:on('onBagChanged', functor.make(self.onBagChanged, self))

        self:init()
    end,

    onUnload = function (self)
        bag_manager.events:remove("onBagChanged", self.event_ids["onBagChanged"] )
    end,

    init = function(self)
        self.cache_layout = {}
        
        local path = "content/layout"
        frame_helper:hideAllChild(self, path, true)
        local cfg_list = self:getCfgList(self.item_cls.cfg)
        for idx, cfg in ipairs(cfg_list) do
            local child = frame_helper:getChild(self, idx, path)
            child:SetActive(true)
            local icon = child:Find("icon"):GetComponent(UnityEngine.UI.Image)
            local quality = child:Find("quality"):GetComponent(UnityEngine.UI.Image)
            local num = child:Find("num"):GetComponent(UnityEngine.UI.Text)
            icon:SetSprite(cfg.path .. cfg.icon)
            quality:SetSprite(gui_helper:getQualityIconPath(cfg.color))
            local btn = child:GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(function()
                self:click_selectButtom(cfg)
            end)

            local fuse_select = child:Find("fuse_select")
            local divide_select = child:Find("divide_select")

            table.insert(self.cache_layout, {
                fuse_select = fuse_select,
                divide_select = divide_select,
                cfg = cfg,
                num = num
            })
        end

        self.cache_toggles = {}
        table.insert(self.cache_toggles, self:findButton("content/fuseToggleBtn"))
        table.insert(self.cache_toggles, self:findButton("content/divideToggleBtn"))
        self.fuse_content = self.transform:Find("content/fuse_content")
        self.divide_content = self.transform:Find("content/divide_content")
        for idx, btn in ipairs(self.cache_toggles) do
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(function()
                self:click_toggle(idx)
            end)
        end

        self.cur_toggle_idx = 1
        local cfg = self.item_cls.cfg
        self:click_selectButtom(cfg)

        -- if self.cur_toggle_idx == 1 then
        --     local next_cfg = self:getNextCfg(cfg)
        --     if next_cfg then
        --         self:click_selectButtom(next_cfg)
        --     else
        --         self:click_selectButtom(cfg)
        --     end
        -- else
        --     local prvious_cfg = self:getPreviousCfg(cfg)
        --     if prvious_cfg then
        --         self:click_selectButtom(prvious_cfg)
        --     else
        --         self:click_selectButtom(cfg)
        --     end
        -- end
    end,

    click_toggle = function(self, idx)
        self.cur_toggle_idx = idx
        self:refreshButtomLayout()
    end,

    click_selectButtom = function(self, cfg)
        self.cur_layout_cfg = cfg
        self:refreshButtomLayout()
    end,

    onBagChanged = function(self)
        self:refreshButtomLayout()
    end,

    refreshButtomLayout = function(self)

        local icon = self:findImage("content/item/icon")
        local quality = self:findImage("content/item/quality")
        local name = self:findText("content/item/name")
        local des = self:findText("content/item/des")
        local num = self:findText("content/item/num")

        local cfg = self.cur_layout_cfg
        icon:SetSprite(cfg.path .. cfg.icon)
        quality:SetSprite(gui_helper:getQualityIconPath(cfg.color))
        name.color = gui_helper:quality2UnityColor(cfg.color)
        name.text = frame_helper:getLangCfg(cfg.name_id)
        des.text = frame_helper:getItemDes(cfg.id)
        num.text = bag_manager:getItemNum(cfg.id)

        frame_helper:setActive(self.fuse_content, self.cur_toggle_idx == 1)
        frame_helper:setActive(self.divide_content, self.cur_toggle_idx == 2)
        for idx, btn in ipairs(self.cache_toggles) do
            btn.interactable = self.cur_toggle_idx ~= idx
        end
        local buttom_des = self:findText("content/buttom_des")
        buttom_des.text = NativeText.GetLangStr(self.cur_toggle_idx == 1 and 620041 or 620042)

        local cfg_list = self:getCfgList(self.item_cls.cfg)
        local idx = nil
        for __, cfg in ipairs(cfg_list) do
            if cfg == self.cur_layout_cfg then
                idx = __
                break
            end
        end

        if self.cur_toggle_idx == 1 then
            if idx >= #cfg_list then
                self.cur_layout_cfg = self:getPreviousCfg(self.cur_layout_cfg)
                self:refreshButtomLayout()
                return
            else
                local left_cfg = self.cur_layout_cfg
                local right_cfg = self:getNextCfg(self.cur_layout_cfg)
                local left_idx = idx
                local right_idx = idx + 1

                for __, content in ipairs(self.cache_layout) do
                    frame_helper.hideTrans(content.divide_select)
                    if __ == idx then
                        frame_helper.showTrans(content.fuse_select)

                        local arrow_effect = content.fuse_select:Find("arrow_effect")
                        local frame_effect = content.fuse_select:Find("frame_effect")

                        local lt = LeanTween.color(arrow_effect, Color(0, 198 / 255,1 ,0.5) , 0.5)
                        lt.loopType = LeanTweenType.pingPong

                        local lt = LeanTween.scale(arrow_effect.gameObject, Vector3.one * 1.2 , 1)
                        lt.from = Vector3.one
                        lt.loopType = LeanTweenType.pingPong

                        -- local lt = LeanTween.color(arrow_effect, Color(0, 198 / 255,1 ,0.5) , 0.83)
                        -- lt.loopType = LeanTweenType.pingPong

                    else
                        frame_helper.hideTrans(content.fuse_select)
                    end

                    if left_idx == __ then
                        content.num.text = frame_helper:getItemNumFormat(content.cfg.id, content.cfg.MaterialNum)
                    else
                        content.num.text = bag_manager:getItemNum(content.cfg.id)
                    end
                end
            end
        else
            if idx <= 1 then
                self.cur_layout_cfg = self:getNextCfg(self.cur_layout_cfg)
                self:refreshButtomLayout()
                return
             else
                local left_cfg = self:getPreviousCfg(self.cur_layout_cfg)
                local right_cfg = self.cur_layout_cfg
                local left_idx = idx - 1 
                local right_idx = idx

                -- local temp_num = math.floor(frame_helper:getItemNum(right_cfg.id) / right_cfg.DismantleNum)
                -- local temp_color = frame_helper:getItemNum(right_cfg.id) > 0 and Color.white or Color.red

                 for __, content in ipairs(self.cache_layout) do
                    frame_helper.hideTrans(content.fuse_select)
                    if __ == idx then
                        frame_helper.showTrans(content.divide_select)

                        local arrow_effect = content.divide_select:Find("arrow_effect")
                        local frame_effect = content.divide_select:Find("frame_effect")

                        local lt = LeanTween.color(arrow_effect, Color(0, 198 / 255,1 ,0.5) , 0.83)
                        lt.loopType = LeanTweenType.pingPong

                        local lt = LeanTween.scale(arrow_effect.gameObject, Vector3.one * 1.38 , 1.66)
                        lt.from = Vector3.one
                        lt.loopType = LeanTweenType.pingPong

                        -- local lt = LeanTween.color(arrow_effect, Color(0, 198 / 255,1 ,0.5) , 0.83)
                        -- lt.loopType = LeanTweenType.pingPong

                    else
                        frame_helper.hideTrans(content.divide_select)
                    end

                     if right_idx == __ then
                        content.num.text = frame_helper:getItemNumFormat(content.cfg.id, 1)
                    else
                        content.num.text = bag_manager:getItemNum(content.cfg.id)
                    end
                 end
             end
        end

        print("选择的id：", self.cur_layout_cfg.id)
    end,

    getNextCfg = function(self, mat_cfg)
        local rst = nil
        config.crew_materials:foreach(function(id, cfg)
            if cfg.Group == mat_cfg.Group and cfg.color == (mat_cfg.color + 1) then
                rst = cfg
            end
        end)
        return rst
    end,

    getPreviousCfg = function(self, mat_cfg)
        local rst = nil
        config.crew_materials:foreach(function(id, cfg)
            if cfg.Group == mat_cfg.Group and cfg.color == (mat_cfg.color - 1) then
                rst = cfg
            end
        end)
        return rst
    end,

    getCfgList = function(self, mat_cfg)
        if self.cfg_list then
            return self.cfg_list
        end
        local cfg_list = {}
        config.crew_materials:foreach(function(id, cfg)
            if cfg.Group == mat_cfg.Group then
                table.insert(cfg_list, cfg)
            end
        end)
        local sortFunc = function(a, b)
            return a.id < b.id
        end
        table.sort(cfg_list, sortFunc)
        self.cfg_list = cfg_list
        return cfg_list
    end,

    click_closeBtn = function(self)
        frame_manager:hide('home_forge_mat_exchange')
    end,

    click_fuseBtn = function(self)
        if self.cur_layout_cfg then
            local cfg = self:getNextCfg(self.cur_layout_cfg)
            network_sender:composeMaterial(cfg.id, false)
        end
    end,

    click_fuseAllBtn = function(self)
        if self.cur_layout_cfg then
            local cfg = self:getNextCfg(self.cur_layout_cfg)
            network_sender:composeMaterial(cfg.id, true)
        end
    end,

    click_devideBtn = function(self)
        if self.cur_layout_cfg then
            local cfg = self.cur_layout_cfg
            network_sender:dismantleMaterial(cfg.id, false)
        end
    end,

    click_devideAllBtn = function(self)
        if self.cur_layout_cfg then
            local cfg = self.cur_layout_cfg
            network_sender:dismantleMaterial(cfg.id, true)
        end
    end,
}
