local super = require("ui.frame")

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        self.item_cls = self.args.data

        -- 路径: root/login
        -- 事件: click_login 
        clickEvents = {
            "closeBtn",
        }

        -- 注册点击事件
        frame_helper:regClickEvents("click", self, clickEvents)

        self.onBagChanged = bag_manager.events:on('onBagChanged', function()
            self:init()
        end)

        self:init()
    end,

    onUnload = function (self)
        bag_manager.events:off("onBagChanged", self.onBagChanged)
    end,

    init = function(self)
        local cfg_list = {}
        config.item_useables:foreach(function(id, cfg)
            if cfg.type == 85 then
                table.insert(cfg_list, cfg)
            end
        end)
        local sortFunc = function(a, b)
            return a.id < b.id
        end
        table.sort(cfg_list, sortFunc)
        self:refresh(cfg_list)
    end,


    refresh = function(self, cfg_list)
        local parent_path = "Scroll View (2)/Viewport/Content"
        frame_helper:hideAllChild(self, parent_path, true)
        for idx, cfg in ipairs(cfg_list) do
            local child = frame_helper:getChild(self, idx, parent_path)
            child:SetActive(true)

            local icon = child:Find("icon"):GetComponent(UnityEngine.UI.Image)
            local quality = child:Find("quality"):GetComponent(UnityEngine.UI.Image)
            local name_text = child:Find('name'):GetComponent(UnityEngine.UI.Text)
            local des_text = child:Find('des'):GetComponent(UnityEngine.UI.Text)
            local own_text = child:Find('own'):GetComponent(UnityEngine.UI.Text)

            local use_trans = child:Find('useTrans')
            local buy_trans = child:Find('buyTrans')

            local own_num = frame_helper:getItemNum(cfg.id)

            icon:SetSprite(frame_helper:getItemIcon(cfg.id))
            quality:SetSprite(frame_helper:getItemQualityIcon(cfg.id))
            name_text.text = frame_helper:getLangCfg(cfg.name_id)
            des_text.text = frame_helper:getLangCfg(cfg.des_id)
            own_text.text = string.format(NativeText.GetLangStr(own_num > 0 and 250008 or 250010), own_num) 

            use_trans:SetActive(own_num > 0)
            buy_trans:SetActive(own_num <= 0)

            if own_num > 0 then
                local btn = child:Find('useTrans/useBtn'):GetComponent(UnityEngine.UI.Button)
                btn.onClick:RemoveAllListeners()
                btn.onClick:AddListener(
                    function ()
                        if own_num <= 0 then
                            network_sender:useItem(cfg.id, 1)
                        else
                            frame_manager:show('use_slider', nil, {
                                item_id = cfg.id,
                                select_num = 1,
                                max_num = own_num,
                                callback = function(item_id, item_num)
                                    network_sender:useItem(item_id, item_num)
                                end,
                            })
                        end
                    end
                )
            else
                local buy_cost = frame_helper:getDiamondPriceFromStr(cfg.allprice)

                local DIAMOND    = 30001 -- 钻石
                local image_text = child:Find('buyTrans/image_text'):GetComponent(UnityEngine.UI.Text)
                local item_name = gui_helper:getImageTextItemStr(DIAMOND)
                image_text:SetImageText(string.format("%s %s", item_name, buy_cost))

                local btn = child:Find('buyTrans/buyBtn'):GetComponent(UnityEngine.UI.Button)
                btn.onClick:RemoveAllListeners()
                btn.onClick:AddListener(
                    function ()
                        local max_num = 10
                        frame_manager:show('buy_slider', nil, {
                            item_id = cfg.id,
                            select_num = 1,
                            max_num = max_num,
                            callback = function(item_id, item_num)
                                print("使用物品:", item_id,item_num)
                                network_sender:buyItem(
                                {
                                    {
                                        item_id = item_id,
                                        count = item_num,
                                    }
                                }, 1)
                            end,
                        })
                    end
                )
            end

        end
    end,

    --[[
    init = function(self)
        local icon = self:findImage("content/item/icon")
        local quality = self:findImage("content/item/quality")
        local equip_tag = self:findImage("content/item/equip_tag")
        local name = self:findText("content/name")
        equip_tag.enabled = crew_manager:isItemEquipped(self.item_cls.inst_id)
        local cfg = self.item_cls.cfg
        icon:SetSprite(cfg.path .. cfg.icon)
        name.text = frame_helper:getLangCfg(cfg.name_id)
        quality:SetSprite(gui_helper:getQualityIconPath(cfg.color))
        local star_path = "content/item/light stars"
        frame_helper:hideAllChild(self, star_path, true)
        for i = 1, cfg.color do
            local star = frame_helper:getChild(self, i, star_path)
            star:SetActive(true)
        end

        local path = "content/layout"
        frame_helper:hideAllChild(self, path, true)
        local idx = 1
        local tab = frame_helper:split(self.item_cls.cfg.Dismantle, "$")
        for __, s in ipairs(tab) do
            local temp = frame_helper:split(s, "_")
            local item_id = tonumber(temp[1])
            local cfg = item_helper:getItemCfg(item_id)
            if cfg then
                local child = frame_helper:getChild(self, idx, path)
                child:SetActive(true)
                idx = idx + 1
                local icon = child:Find("icon"):GetComponent(UnityEngine.UI.Image)
                local quality = child:Find("quality"):GetComponent(UnityEngine.UI.Image)
                icon:SetSprite(cfg.path .. cfg.icon)
                quality:SetSprite(gui_helper:getQualityIconPath(cfg.color))
            end
        end
    end,
    ]]

    click_closeBtn = function(self)
        frame_manager:hide('home_forge_obtain')
    end,
}
