local super = require("ui.frame")

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        self.default_type = self.args.data

        -- 路径: root/login
        -- 事件: click_login 
        clickEvents = {
            "content/bg/backBtn",
            "content/bg/filterBtn",
            "filter/closeFilterBtn",
            "filter/confirmBtn",
        }

        -- 注册点击事件
        frame_helper:regClickEvents("click", self, clickEvents)

        self.FORGE_UPGRADE_TICK = home_info.events:on("FORGE_UPGRADE_TICK", function(e, elapsed, total_time)
            self:onTick(e, elapsed, total_time)
        end)

        self.BUILD_INFO_CHANGE = home_info.events:on("BUILD_INFO_CHANGE", function(build_id)
            self:onBuildInfoChange(build_id)
        end)

        self:init()
    end,

    onUnload = function (self)
        home_info.events:off("FORGE_UPGRADE_TICK", self.FORGE_UPGRADE_TICK)
        home_info.events:off("BUILD_INFO_CHANGE", self.BUILD_INFO_CHANGE)
    end,

    init = function(self)
        self.cur_type = self.default_type
        self.cur_quality = nil
        self.cur_filter_prop = nil

        self.cache_type = {}
        self.cache_quality = {}

        self:initFilter()

        local cfg_list = {}
        config.crew_type_indexs:foreach(function(id, cfg)
            table.insert(cfg_list, cfg)
        end)
        local sortFunc = function(a, b)
            return a.id < b.id
        end
        table.sort(cfg_list, sortFunc)

        local path = "content/bg/type layout"
        for idx, cfg in ipairs(cfg_list) do
            local child = frame_helper:getChild(self, idx, path)
            child:SetActive(true)
            local name = child:Find("name"):GetComponent(UnityEngine.UI.Text)
            local icon = child:Find("icon"):GetComponent(UnityEngine.UI.Image)
            local select = child:Find("effect_forge_select_1")
            name.text = frame_helper:getLangCfg(cfg.CrewName)
            icon:SetSprite(cfg.CrewPath .. cfg.CrewIcon)
            local btn = child:GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(function()
                self:click_forgeType(cfg)
            end)
            table.insert(self.cache_type, {
                cfg = cfg,
                btn = btn,
                select = select,
            })

            if not self.cur_type then
                self.cur_type = cfg.id
            end
            btn.interactable = self.cur_type ~= cfg.id
            select.gameObject:SetActive(self.cur_type == cfg.id)
        end

        for i = 1, 5 do
            local path = "content/bg/quality layout/qulity " .. i
            local btn = self.transform:Find(path):GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(function()
                self:click_forgeQuality(i)
            end)

            local select = self.transform:Find(path .. "/effect_forge_select_2")

            table.insert(self.cache_quality, {
                idx = i,
                btn = btn,
                select = select,
            })

            if not self.cur_quality then
                self.cur_quality = i
            end
            btn.interactable = self.cur_quality ~= i
        end
        self:refreshRight()
    end,

    click_forgeType = function(self, cfg)
        for __, c in ipairs(self.cache_type) do
            c.btn.interactable = c.cfg ~= cfg
            c.select.gameObject:SetActive(c.cfg == cfg)
        end
        self.cur_type = cfg.id
        self:refreshRight()
    end,

    click_forgeQuality = function(self, quality)
        for __, c in ipairs(self.cache_quality) do
            c.btn.interactable = c.idx ~= quality
            c.select.gameObject:SetActive(c.idx == quality)
        end
        self.cur_quality = quality
        self:refreshRight()
    end,

    refreshRight = function(self)
        local type_cfg = config.crew_type_indexs[self.cur_type]
        local build_data = home_info:getBuildDataByCfgId(22)
        self.cache_upgrading = nil

        local path = "content/bg/Scroll View/Viewport/Content"
        frame_helper:hideAllChild(self, path, true)
        local list = self:getCfgList(self.cur_type, self.cur_quality, self.cur_filter_prop)
        for idx, cfg in ipairs(list) do
            local child = frame_helper:getChild(self, idx, path)
            child:SetActive(true)
            local name = child:Find("info/name"):GetComponent(UnityEngine.UI.Text)
            local level = child:Find("info/level"):GetComponent(UnityEngine.UI.Text)
            local type_icon = child:Find("info/un_upgrade/type_icon"):GetComponent(UnityEngine.UI.Image)
            local icon = child:Find("info/icon"):GetComponent(UnityEngine.UI.Image)
            local quality = child:Find("info/quality"):GetComponent(UnityEngine.UI.Image)

            local type_name = frame_helper:getLangCfg(type_cfg.CrewName)
            name.text = frame_helper:getLangCfg(cfg.name_id)
            name.color = gui_helper:quality2UnityColor(cfg.color)
            level.text = (local_lord.level >= cfg.UserLevel) and string.format(NativeText.GetLangStr(620007), cfg.UserLevel, type_name) or string.format(NativeText.GetLangStr(620008), cfg.UserLevel, type_name)
            type_icon:SetSprite(type_cfg.CrewPath .. type_cfg.CrewIcon)
            icon:SetSprite(cfg.path .. cfg.icon)
            quality:SetSprite(gui_helper:getQualityIconPath(cfg.color))

            local prop_list = frame_helper:getPropertyDes(cfg.EffectPara)
            local prop_path = path .. "/" .. child.name .. "/des layout"
            frame_helper:hideAllChild(self, prop_path, true)
            for __, v in ipairs(prop_list) do
                local item = frame_helper:getChild(self, __, prop_path)
                item:SetActive(true)
                local name = item:Find("name"):GetComponent(UnityEngine.UI.Text)
                local value = item:Find("value"):GetComponent(UnityEngine.UI.Text)
                name.text = v.name_str
                value.text = v.num_str
            end

            local full_condition = true
            local progress_path = path .. "/" .. child.name .. "/info/un_upgrade/progress layout"
            local tplt_path = path .. "/" .. child.name .. "/info/un_upgrade/progress layout/tplt"
            local right_path = path .. "/" .. child.name .. "/info/un_upgrade/progress layout/right"
            frame_helper:hideAllChild(self, progress_path, true)

            local temp_table = loadstring("return " .. cfg.UpgradeMaterial)()
            local temp_idx = 1
            for item_id, need_num in pairs(temp_table) do
                local item = frame_helper:getChildByTemplatePath(self, temp_idx, progress_path, tplt_path)
                item:SetActive(true)
                local own_num = bag_manager:getItemNum(item_id)
                local value = item:Find("Building_panel_14_02_Fill"):GetComponent(UnityEngine.UI.Image)
                value.enabled = own_num >= need_num
                temp_idx = temp_idx + 1
                if own_num < need_num then
                    full_condition = false
                end
            end

            type_icon.enabled = false
            if cfg.FrontCrew > 0 then
                type_icon.enabled = true
                local own_num = bag_manager:getItemNum(cfg.FrontCrew)
                type_icon.color = own_num > 0 and Color.green or Color.gray
                if own_num <= 0 then
                    full_condition = false
                end
            end

            local right_trans = self.transform:Find(right_path)
            right_trans.transform:SetSiblingIndex(right_trans.parent.childCount)
            right_trans:SetActive(full_condition)

            local btn = child:Find("info/un_upgrade/forgeBtn"):GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(function()
                self:showCreateUI(cfg)
            end)

            local btn = child:Find("info/upgrading/speedUpBtn"):GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(function()
                frame_manager:show("home_speed_up", nil, {
                    speed_up_type = home_speed_up_type.quantum_create,
                    id = home_info:getBuildDataByCfgId(22).id,
                    forge_block_id = cfg.id,
                })
            end)

            local un_upgrade = child:Find("info/un_upgrade")
            local upgrading = child:Find("info/upgrading")

            if cfg.id == build_data.trainCenter.cfgId and build_data.trainCenter.bWork then
                frame_helper.showTrans(upgrading)
                frame_helper.hideTrans(un_upgrade)
                local slider = child:Find("info/upgrading/slider"):GetComponent(UnityEngine.UI.Slider)
                local time = child:Find("info/upgrading/slider/time"):GetComponent(UnityEngine.UI.Text)
                self.cache_upgrading = {
                    slider = slider,
                    time = time,
                }

                local e = build_data
                local elapsed = e.trainCenter.remainTime - Time.realtimeSinceStartup + e.rmbTime
                self:onTick(e, elapsed, e.trainCenter.totaltime)
            else
                frame_helper.showTrans(un_upgrade)
                frame_helper.hideTrans(upgrading)
            end
        end
    end,

    showCreateUI = function(self, cfg)
        frame_manager:show('home_forge_upgrade', nil, cfg)
    end,

    getCfgList = function(self, slot, quality, filter_prop)
        local rst = {}
        config.crew_blocks:foreach(function(id, cfg)
            if cfg.Slot == slot and cfg.color == quality and cfg.IsSet == 0 then
                if not filter_prop then
                    table.insert(rst, cfg)
                else
                    local prop_tab = frame_helper:getPropertyDes(cfg.EffectPara)
                    for __, value in ipairs(prop_tab) do
                        if value.id == filter_prop then
                            table.insert(rst, cfg)
                            break
                        end
                    end
                end
            end
        end)
        local sortFunc = function(a, b)
            return a.id < b.id
        end
        table.sort(rst, sortFunc)
        return rst
    end,

    click_backBtn = function(self)
        frame_manager:hide('home_forge_preview')
    end,

    click_filterBtn = function(self)
        frame_helper.showTrans(self.filter_trans)
    end,

    onTick = function(self, build_data, elapsed, total_time)
        if self.cache_upgrading then
            self.cache_upgrading.time.text = string.format(NativeText.GetLangStr(620016), frame_helper:timeFormat(elapsed))
            self.cache_upgrading.slider.value = 1 - elapsed / total_time
        end
    end,

    onBuildInfoChange = function(self, build_id)
        local build_data = home_info:getBuildDataById(build_id)
        if build_data.cfgId == 22 then
            self:click_forgeQuality(self.cur_quality)
        end
    end,

    initFilter = function(self)
        self.filter_trans = self.transform:Find("filter")
        frame_helper.hideTrans(self.filter_trans)

        self.cache_filter = {}
        local cook_cfgs = {}
        local getList = function(index)
            for __, value in ipairs(cook_cfgs) do
                if value.index == index then
                    return value.list
                end
            end
            local new_list = {
                index = index,
                list = {}
            }
            table.insert(cook_cfgs, new_list)
            return new_list.list
        end

        config.crew_filters:foreach(function(id, cfg)
            local list = getList(cfg.Index)
            table.insert(list, cfg)
        end)

        local path = "filter/Scroll View/Viewport/Content/"
        frame_helper:hideAllChild(self, path, true)

        for __, value in ipairs(cook_cfgs) do
            local child = frame_helper:getChild(self, __, path)
            child:SetActive(true)

            local title = child:Find("top/text1_Static[T4]_2"):GetComponent(UnityEngine.UI.Text)
            title.text = frame_helper:getLangCfg(value.list[1].IndexName)

            local item_path = path .. child.name .. "/layout"
            frame_helper:hideAllChild(self, item_path, true)

            for idx, cfg in ipairs(value.list) do
                local item = frame_helper:getChild(self, idx, item_path)
                item:SetActive(true)

                local name = item:Find("text1_Static[T5]_3"):GetComponent(UnityEngine.UI.Text)
                name.text = frame_helper:getProptyName(cfg.Attribute)

                local select_image = item:Find("select"):GetComponent(UnityEngine.UI.Image)
                select_image.enabled = false

                local btn = item:GetComponent(UnityEngine.UI.Button)
                btn.onClick:RemoveAllListeners()
                btn.onClick:AddListener(function()
                    self:select_filter(cfg)
                end)

                table.insert(self.cache_filter, {
                    cfg = cfg,
                    select_state = false,
                    select_image = select_image,
                })
            end
        end
        self:setFilterName()
    end,

    setFilterName = function(self)
        local filter_name = self:findText("content/bg/filterBtn/text1_Static[T2]_7")
        if self.cur_filter_prop then
            filter_name.text = frame_helper:getProptyName(self.cur_filter_prop)
        else
            filter_name.text = NativeText.GetLangStr(620006)
        end
    end,

    select_filter = function(self, cfg)
        self.cur_filter_prop = nil
        for __, value in ipairs(self.cache_filter) do
            if value.cfg == cfg then
                value.select_state = not value.select_state
            else
                value.select_state = false
            end
            value.select_image.enabled = value.select_state

            if value.select_state then
                self.cur_filter_prop = value.cfg.Attribute
            end
        end
        self:setFilterName()
        self:refreshRight()
    end,

    click_closeFilterBtn = function(self)
        frame_helper.hideTrans(self.filter_trans)
    end,

    click_confirmBtn = function(self)
        frame_helper.hideTrans(self.filter_trans)
    end,
}
