local super = require("ui.frame")

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        -- 路径: root/login
        -- 事件: click_login 
        clickEvents = {
            "content/bg/backBtn",
        }

        -- 注册点击事件
        frame_helper:regClickEvents("click", self, clickEvents)

        self.FORGE_UPGRADE_TICK = home_info.events:on("FORGE_UPGRADE_TICK", function(e, elapsed, total_time)
            self:onTick(e, elapsed, total_time)
        end)

        self.BUILD_INFO_CHANGE = home_info.events:on("BUILD_INFO_CHANGE", function(build_id)
            self:onBuildInfoChange(build_id)
        end)

        self:init()
    end,

    onUnload = function (self)
        home_info.events:off("FORGE_UPGRADE_TICK", self.FORGE_UPGRADE_TICK)
        home_info.events:off("BUILD_INFO_CHANGE", self.BUILD_INFO_CHANGE)
    end,

    init = function(self)
        self.cur_type = nil
        self.cur_quality = nil

        self.cache_type = {}
        self.cache_quality = {}

        local cfg_list = {}
        config.crew_setconfigs:foreach(function(id, cfg)
            table.insert(cfg_list, cfg)
        end)
        local sortFunc = function(a, b)
            return a.id < b.id
        end
        table.sort(cfg_list, sortFunc)

        local path = "content/bg/Left Scroll View/Viewport/Content"
        frame_helper:hideAllChild(self, path, true)
        for idx, cfg in ipairs(cfg_list) do
            local child = frame_helper:getChild(self, idx, path)
            child:SetActive(true)
            local name = child:Find("name"):GetComponent(UnityEngine.UI.Text)
            local icon = child:GetComponent(UnityEngine.UI.Image)

            name.text = frame_helper:getLangCfg(cfg.NameId)
            icon:SetSprite(cfg.Path .. cfg.Banner)

            local select_trans = child:Find("select")

            local btn = child:GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(function()
                self:click_forgeType(cfg)
            end)
            table.insert(self.cache_type, {
                cfg = cfg,
                select_trans = select_trans,
            })

            if not self.cur_type then
                self.cur_type = cfg.id
            end

            frame_helper:setActive(select_trans, self.cur_type == cfg.id)
        end

        for i = 1, 5 do
            local path = "content/bg/quality layout/qulity " .. i
            local btn = self.transform:Find(path):GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(function()
                self:click_forgeQuality(i)
            end)

            local select = self.transform:Find(path .. "/effect_forge_select_2")

            table.insert(self.cache_quality, {
                idx = i,
                btn = btn,
                select = select,
            })

            if not self.cur_quality then
                self.cur_quality = i
            end
            btn.interactable = self.cur_quality ~= i
        end
        self:refreshRight()
    end,

    click_forgeType = function(self, cfg)
        for __, c in ipairs(self.cache_type) do
            frame_helper:setActive(c.select_trans, c.cfg == cfg)
        end
        self.cur_type = cfg.id
        self:refreshRight()
    end,

    click_forgeQuality = function(self, quality)
        for __, c in ipairs(self.cache_quality) do
            c.btn.interactable = c.idx ~= quality
            c.select.gameObject:SetActive(c.idx == quality)
        end
        self.cur_quality = quality
        self:refreshRight()
    end,

    refreshRight = function(self)
        -- local type_cfg = config.crew_setconfigs[self.cur_type]
        -- crew_type_indexs


        local build_data = home_info:getBuildDataByCfgId(22)
        self.cache_upgrading = nil

        local path = "content/bg/Scroll View/Viewport/Content"
        frame_helper:hideAllChild(self, path, true)
        local list = self:getCfgList(self.cur_type, self.cur_quality)
        for idx, cfg in ipairs(list) do
            local child = frame_helper:getChild(self, idx, path)
            child:SetActive(true)
            local name = child:Find("info/name"):GetComponent(UnityEngine.UI.Text)
            local level = child:Find("info/level"):GetComponent(UnityEngine.UI.Text)
            local type_icon = child:Find("info/un_upgrade/type_icon"):GetComponent(UnityEngine.UI.Image)
            local icon = child:Find("info/icon"):GetComponent(UnityEngine.UI.Image)
            local quality = child:Find("info/quality"):GetComponent(UnityEngine.UI.Image)

            local type_cfg = config.crew_type_indexs[cfg.Slot]

            local type_name = frame_helper:getLangCfg(type_cfg.CrewName)
            name.text = frame_helper:getLangCfg(cfg.name_id)
            name.color = gui_helper:quality2UnityColor(cfg.color)
            level.text = (local_lord.level >= cfg.UserLevel) and string.format(NativeText.GetLangStr(620007), cfg.UserLevel, type_name) or string.format(NativeText.GetLangStr(620008), cfg.UserLevel, type_name)
            type_icon:SetSprite(type_cfg.CrewPath .. type_cfg.CrewIcon)
            icon:SetSprite(cfg.path .. cfg.icon)
            quality:SetSprite(gui_helper:getQualityIconPath(cfg.color))

            local prop_list = frame_helper:getPropertyDes(cfg.EffectPara)
            local prop_path = path .. "/" .. child.name .. "/des layout"
            frame_helper:hideAllChild(self, prop_path, true)
            for __, v in ipairs(prop_list) do
                local item = frame_helper:getChild(self, __, prop_path)
                item:SetActive(true)
                local name = item:Find("name"):GetComponent(UnityEngine.UI.Text)
                local value = item:Find("value"):GetComponent(UnityEngine.UI.Text)
                name.text = v.name_str .. string.format("  <color=#0BFF78FF>%s</color>", v.num_str)
                value.text = ""
            end

            local full_condition = true
            local progress_path = path .. "/" .. child.name .. "/info/un_upgrade/progress layout"
            local tplt_path = path .. "/" .. child.name .. "/info/un_upgrade/progress layout/tplt"
            local right_path = path .. "/" .. child.name .. "/info/un_upgrade/progress layout/right"
            frame_helper:hideAllChild(self, progress_path, true)

            local temp_table = loadstring("return " .. cfg.UpgradeMaterial)()
            local temp_idx = 1
            for item_id, need_num in pairs(temp_table) do
                local item = frame_helper:getChildByTemplatePath(self, temp_idx, progress_path, tplt_path)
                item:SetActive(true)
                local own_num = bag_manager:getItemNum(item_id)
                local value = item:Find("Building_panel_14_02_Fill"):GetComponent(UnityEngine.UI.Image)
                value.enabled = own_num >= need_num
                temp_idx = temp_idx + 1
                if own_num < need_num then
                    full_condition = false
                end
            end

            type_icon.enabled = false
            if cfg.FrontCrew > 0 then
                type_icon.enabled = true
                local own_num = bag_manager:getItemNum(cfg.FrontCrew)
                type_icon.color = own_num > 0 and Color.green or Color.gray
                if own_num <= 0 then
                    full_condition = false
                end
            end

            local right_trans = self.transform:Find(right_path)
            right_trans.transform:SetSiblingIndex(right_trans.parent.childCount)
            right_trans:SetActive(full_condition)

            local btn = child:Find("info/un_upgrade/forgeBtn"):GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(function()
                self:showCreateUI(cfg)
            end)

            local btn = child:Find("info/upgrading/speedUpBtn"):GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(function()
                frame_manager:show("home_speed_up", nil, {
                    speed_up_type = home_speed_up_type.quantum_create,
                    id = home_info:getBuildDataByCfgId(22).id,
                    forge_block_id = cfg.id,
                })
            end)

            local un_upgrade = child:Find("info/un_upgrade")
            local upgrading = child:Find("info/upgrading")

            if cfg.id == build_data.trainCenter.cfgId and build_data.trainCenter.bWork then
                frame_helper.showTrans(upgrading)
                frame_helper.hideTrans(un_upgrade)
                local slider = child:Find("info/upgrading/slider"):GetComponent(UnityEngine.UI.Slider)
                local time = child:Find("info/upgrading/slider/time"):GetComponent(UnityEngine.UI.Text)
                self.cache_upgrading = {
                    slider = slider,
                    time = time,
                }

                local e = build_data
                local elapsed = e.trainCenter.remainTime - Time.realtimeSinceStartup + e.rmbTime
                self:onTick(e, elapsed, e.trainCenter.totaltime)
            else
                frame_helper.showTrans(un_upgrade)
                frame_helper.hideTrans(upgrading)
            end
        end
    end,

    showCreateUI = function(self, cfg)
        frame_manager:show('home_forge_upgrade', nil, cfg)
    end,

    getCfgList = function(self, id, quality)
        local group_list = loadstring("return " .. config.crew_setconfigs[id].CrewGroupId)()

        local existGroupId = function(id)
            for __, group_id in ipairs(group_list) do
                if id == group_id then
                    return true
                end
            end
            return false
        end

        local rst = {}
        config.crew_blocks:foreach(function(id, cfg)
            if cfg.color == quality and cfg.IsSet == 1 and existGroupId(cfg.Group) then
                table.insert(rst, cfg)
            end
        end)

        local sortFunc = function(a, b)
            return a.id < b.id
        end
        table.sort(rst, sortFunc)
        return rst
    end,

    click_backBtn = function(self)
        frame_manager:hide('home_forge_preview_plus')
    end,

    onTick = function(self, build_data, elapsed, total_time)
        if self.cache_upgrading then
            self.cache_upgrading.time.text = string.format(NativeText.GetLangStr(620016), frame_helper:timeFormat(elapsed))
            self.cache_upgrading.slider.value = 1 - elapsed / total_time
        end
    end,

    onBuildInfoChange = function(self, build_id)
        local build_data = home_info:getBuildDataById(build_id)
        if build_data.cfgId == 22 then
            self:click_forgeQuality(self.cur_quality)
        end
    end,
}
