local super = require("ui.frame")

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        self.cfg = self.args.data

        -- 路径: root/login
        -- 事件: click_login 
        clickEvents = {
            "content/bg/backBtn",
            "content/bg/material_content/obtainBtn",
            "content/bg/equipBtn",
            "content/bg/materialBtn",
            "content/bg/equip_content/filterBtn",
            "filter/closeFilterBtn",
            "filter/confirmBtn",
        }

        -- 注册点击事件
        frame_helper:regClickEvents("click", self, clickEvents)

        -- self.FORGE_UPGRADE_TICK = home_info.events:on("FORGE_UPGRADE_TICK", function(e, elapsed, total_time)
        --     self:onTick(e, elapsed, total_time)
        -- end)

        -- self.BUILD_INFO_CHANGE = home_info.events:on("BUILD_INFO_CHANGE", function(build_id)
        --     self:onBuildInfoChange(build_id)
        -- end)

        self.event_ids = {}    
        self.event_ids['onEquipChanged'] = crew_manager.events:on('onEquipChanged', functor.make(self.onEquipChanged, self))
        self.event_ids['onBagChanged'] = bag_manager.events:on('onBagChanged', functor.make(self.onBagChanged, self))

        self:init()
    end,

    onUnload = function (self)
        crew_manager.events:remove("onEquipChanged", self.event_ids["onEquipChanged"] )
        bag_manager.events:remove("onBagChanged", self.event_ids["onBagChanged"] )
        -- home_info.events:off("FORGE_UPGRADE_TICK", self.FORGE_UPGRADE_TICK)
        -- home_info.events:off("BUILD_INFO_CHANGE", self.BUILD_INFO_CHANGE)
    end,

    init = function(self)
        self.cur_type = nil
        self.cur_filter_prop = nil

        self:select_toggle(1)
        self:initTypeLayout()
        self:initFilter()

        -- 刷新装备
        self:refreshEquip()
        -- 刷新材料
        self:refreshMaterials()
    end,

    onEquipChanged = function(self)
        self:refreshEquip()
        self:refreshMaterials()
    end,

    onBagChanged = function(self)
        self:refreshEquip()
        self:refreshMaterials()
    end,

    click_obtainBtn = function(self)
        frame_manager:show('home_forge_obtain')
    end,

    click_equipBtn = function(self)
        self:select_toggle(1)
    end,

    click_materialBtn = function(self)
        self:select_toggle(2)
    end,

    click_filterBtn = function(self)
        frame_helper.showTrans(self.filter_trans)
    end,

    click_closeFilterBtn = function(self)
        frame_helper.hideTrans(self.filter_trans)
    end,

    click_backBtn = function(self)
        frame_manager:hide('home_forge_store')
    end,

    select_toggle = function(self, idx)
        if not self.cache_toggle then
            self.cache_toggle = {
                self:findButton("content/bg/equipBtn"),
                self:findButton("content/bg/materialBtn"),
            }
        end

        for __, value in ipairs(self.cache_toggle) do
            value.interactable = __ ~= idx
        end

        local equip_content = self.transform:Find("content/bg/equip_content")
        local material_content = self.transform:Find("content/bg/material_content")
        frame_helper:setActive(equip_content, idx == 1)
        frame_helper:setActive(material_content, idx == 2)
    end,

    initTypeLayout = function(self)
        local cfg_list = {}
        config.crew_type_indexs:foreach(function(id, cfg)
            table.insert(cfg_list, cfg)
        end)
        local sortFunc = function(a, b)
            return a.id < b.id
        end
        table.sort(cfg_list, sortFunc)

        self.cache_type = {}
        local path = "content/bg/equip_content/type layout"
        for idx, cfg in ipairs(cfg_list) do
            local child = frame_helper:getChild(self, idx, path)
            child:SetActive(true)
            local icon = child:Find("tpye_icon"):GetComponent(UnityEngine.UI.Image)
            local select_image = child:Find("select"):GetComponent(UnityEngine.UI.Image)
            icon:SetSprite(cfg.CrewPath .. cfg.CrewIcon)
            local btn = child:GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(function()
                self:click_forgeType(cfg)
            end)
            table.insert(self.cache_type, {
                cfg = cfg,
                select_image = select_image,
            })
            if not self.cur_type then
                self.cur_type = cfg.id
            end
            select_image.enabled = self.cur_type == cfg.id
        end
    end,

    click_forgeType = function(self, cfg)
        for __, c in ipairs(self.cache_type) do
            c.select_image.enabled = c.cfg == cfg
        end

        if self.cur_type == cfg.id then
            return            
        end
        self.cur_type = cfg.id
        self:refreshEquip()
    end,

    refreshEquip = function(self)
        local path = "content/bg/equip_content/Scroll View/Viewport/Content/"
        frame_helper:hideAllChild(self, path, true)
        local idx = 1

        local setChildInfo = function(item_cls, child)
            local icon = child:Find("icon"):GetComponent(UnityEngine.UI.Image)
            local quality = child:Find("quality"):GetComponent(UnityEngine.UI.Image)
            local is_equip = child:Find("is_equip"):GetComponent(UnityEngine.UI.Image)
            local num = child:Find("num"):GetComponent(UnityEngine.UI.Text)
            local cfg = item_cls.cfg
            icon:SetSprite(cfg.path .. cfg.icon)
            quality:SetSprite(gui_helper:getQualityIconPath(cfg.color))
            num.text = item_cls.count
            is_equip.enabled = crew_manager:isItemEquipped(item_cls.inst_id)

            local star_path = path .. child.name .. "/light stars"
            frame_helper:hideAllChild(self, star_path, true)
            for i = 1, cfg.color do
                local star = frame_helper:getChild(self, i, star_path)
                star:SetActive(true)
            end

            local btn = child:GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(function()
                self:click_equip(item_cls)
            end)
        end

        local equipped_item = crew_manager:getEquippedCrewBySlot(self.cur_type)
        if equipped_item then
            local child = frame_helper:getChild(self, idx, path)
            child:SetActive(true)
            setChildInfo(equipped_item, child)
            idx = idx + 1
        end
        
        local all_list = self:getCfgList()
        for __, value in ipairs(all_list) do
            if value.slot == self.cur_type then
                for _, cfg in ipairs(value.list) do
                    local vsb = false
                    if not self.cur_filter_prop then
                        vsb = true
                    else
                        local prop_tab = frame_helper:getPropertyDes(cfg.EffectPara)
                        for k, v in ipairs(prop_tab) do
                            if v.id == self.cur_filter_prop then
                                vsb = true
                                break
                            end
                        end
                    end
                    if vsb then
                        local item_list = bag_manager:getItemsByCfgId(cfg.id)
                        for __, item_cls in ipairs(item_list) do
                            local child = frame_helper:getChild(self, idx, path)
                            child:SetActive(true)
                            setChildInfo(item_cls, child)
                            idx = idx + 1
                        end
                    end
                end
            end 
        end

        local none_trans = self.transform:Find("content/bg/equip_content/none")
        frame_helper:setActive(none_trans, idx == 1)
    end,

    click_equip = function(self, item_cls)
        frame_manager:show('home_forge_equip', nil, item_cls)
    end,

    getCfgList = function(self)
        if self.cook_cfgs then
            return self.cook_cfgs
        end
        local cook_cfgs = {}
        local getList = function(slot)
            for __, value in ipairs(cook_cfgs) do
                if value.slot == slot then
                    return value.list
                end
            end
            local new_list = {
                slot = slot,
                list = {}
            }
            table.insert(cook_cfgs, new_list)
            return new_list.list
        end

        config.crew_blocks:foreach(function(id, cfg)
            local list = getList(cfg.Slot)
            table.insert(list, cfg)
        end)

        local sortFunc = function(a, b)
            return a.id < b.id
        end

        for __, value in ipairs(cook_cfgs) do
            table.sort(value.list, sortFunc)    
        end

        self.cook_cfgs = cook_cfgs
        return cook_cfgs
    end,

    initFilter = function(self)
        self.filter_trans = self.transform:Find("filter")
        frame_helper.hideTrans(self.filter_trans)

        self.cache_filter = {}
        local cook_cfgs = {}
        local getList = function(index)
            for __, value in ipairs(cook_cfgs) do
                if value.index == index then
                    return value.list
                end
            end
            local new_list = {
                index = index,
                list = {}
            }
            table.insert(cook_cfgs, new_list)
            return new_list.list
        end

        config.crew_filters:foreach(function(id, cfg)
            local list = getList(cfg.Index)
            table.insert(list, cfg)
        end)

        local path = "filter/Scroll View/Viewport/Content/"
        frame_helper:hideAllChild(self, path, true)

        for __, value in ipairs(cook_cfgs) do
            local child = frame_helper:getChild(self, __, path)
            child:SetActive(true)

            local title = child:Find("top/text1_Static[T4]_2"):GetComponent(UnityEngine.UI.Text)
            title.text = frame_helper:getLangCfg(value.list[1].IndexName)

            local item_path = path .. child.name .. "/layout"
            frame_helper:hideAllChild(self, item_path, true)

            for idx, cfg in ipairs(value.list) do
                local item = frame_helper:getChild(self, idx, item_path)
                item:SetActive(true)

                local name = item:Find("text1_Static[T5]_3"):GetComponent(UnityEngine.UI.Text)
                name.text = frame_helper:getProptyName(cfg.Attribute)

                local select_image = item:Find("select"):GetComponent(UnityEngine.UI.Image)
                select_image.enabled = false

                local btn = item:GetComponent(UnityEngine.UI.Button)
                btn.onClick:RemoveAllListeners()
                btn.onClick:AddListener(function()
                    self:select_filter(cfg)
                end)

                table.insert(self.cache_filter, {
                    cfg = cfg,
                    select_state = false,
                    select_image = select_image,
                })
            end
        end
        self:setFilterName()
    end,

    setFilterName = function(self)
        local filter_name = self:findText("content/bg/equip_content/filterBtn/text1_Static[T2]_7")
        if self.cur_filter_prop then
            filter_name.text = frame_helper:getProptyName(self.cur_filter_prop)
        else
            filter_name.text = NativeText.GetLangStr(620006)
        end
    end,

    select_filter = function(self, cfg)
        self.cur_filter_prop = nil
        for __, value in ipairs(self.cache_filter) do
            if value.cfg == cfg then
                value.select_state = not value.select_state
            else
                value.select_state = false
            end
            value.select_image.enabled = value.select_state

            if value.select_state then
                self.cur_filter_prop = value.cfg.Attribute
            end
        end
        self:setFilterName()
        self:refreshEquip()
    end,

    refreshMaterials = function(self)
        local cfg_list = {}
        config.crew_materials:foreach(function(id, cfg)
            table.insert(cfg_list, cfg)
        end)
        local sortFunc = function(a, b)
            return a.id < b.id
        end
        table.sort(cfg_list, sortFunc)

        local path = "content/bg/material_content/Scroll View/Viewport/Content"
        frame_helper:hideAllChild(self, path, true)
        local idx = 1
        for __, cfg in ipairs(cfg_list) do
            local item_list = bag_manager:getItemsByCfgId(cfg.id)
            -- print("item_list:", item_list and #item_list)
            for __, item_cls in ipairs(item_list) do
                local child = frame_helper:getChild(self, idx, path)
                child:SetActive(true)
                local icon = child:Find("icon"):GetComponent(UnityEngine.UI.Image)
                local quality = child:Find("quality"):GetComponent(UnityEngine.UI.Image)
                local num = child:Find("num"):GetComponent(UnityEngine.UI.Text)
                icon:SetSprite(frame_helper:getItemIcon(cfg.id))
                quality:SetSprite(frame_helper:getItemQualityIcon(cfg.id))
                num.text = item_cls.count
                idx = idx + 1

                local btn = child:GetComponent(UnityEngine.UI.Button)
                btn.onClick:RemoveAllListeners()
                btn.onClick:AddListener(function()
                    self:click_showMatExchange(item_cls)
                end)

            end
        end

        local none_trans = self.transform:Find("content/bg/material_content/none")
        frame_helper:setActive(none_trans, idx == 1)
    end,

    click_confirmBtn = function(self)
        frame_manager.hideTrans(self.filter_trans)
    end,

    click_showMatExchange = function(self, item_cls)
        frame_manager:show('home_forge_mat_exchange', nil, item_cls)
    end,
}
