local super = require("ui.frame")
local item_cls = require('RTS.bag_sys.item')

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        self.cfg = self.args.data

        -- 路径: root/login
        -- 事件: click_login 
        clickEvents = {
            "closeBtn",
        }

        -- 注册点击事件
        frame_helper:regClickEvents("click", self, clickEvents)

        self.FORGE_UPGRADE_TICK = home_info.events:on("FORGE_UPGRADE_TICK", function(e, elapsed, total_time)
            self:onTick(e, elapsed, total_time)
        end)

        self.BUILD_INFO_CHANGE = home_info.events:on("BUILD_INFO_CHANGE", function(build_id)
            self:onBuildInfoChange(build_id)
        end)

        self:init(self.cfg)
    end,

    onUnload = function (self)
        home_info.events:off("FORGE_UPGRADE_TICK", self.FORGE_UPGRADE_TICK)
        home_info.events:off("BUILD_INFO_CHANGE", self.BUILD_INFO_CHANGE)
    end,

    click_closeBtn = function(self)
        frame_manager:hide('home_forge_upgrade')
    end,

    init = function(self, init_cfg)
        self.cache_top = {}

        local cfg_list = self:getCfgList(init_cfg.Group, init_cfg.Slot)
        local path = "Left/group/"
        frame_helper:hideAllChild(self, path, true)
        for idx, cfg in ipairs(cfg_list) do
            local child = frame_helper:getChild(self, idx, path)
            child:SetActive(true)
            local icon = child:Find("Image"):GetComponent(UnityEngine.UI.Image)
            local quality = child:Find("quality"):GetComponent(UnityEngine.UI.Image)
            local arrow = child:Find("arrow"):GetComponent(UnityEngine.UI.Image)
            icon:SetSprite(cfg.path .. cfg.icon)
            quality:SetSprite(gui_helper:getQualityIconPath(cfg.color))
            local star_path = path .. child.name .. "/stars"
            frame_helper:hideAllChild(self, star_path, true)
            for i = 1, cfg.color do
                local star = frame_helper:getChild(self, i, star_path)
                star:SetActive(true)
            end
            arrow.enabled = idx ~= 1

            local btn = child:Find("Image"):GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(function()
                self:click_forge(cfg)
            end)

            local select_image = child:Find("select"):GetComponent(UnityEngine.UI.Image)

            table.insert(self.cache_top, {
                select_image = select_image,
                cfg = cfg,
            })
        end
        self:click_forge(init_cfg)
    end,

    click_forge = function(self, cfg)

        self.effect_list = {}

        self.cur_cfg = cfg
        for __, value in ipairs(self.cache_top) do
            value.select_image.enabled = value.cfg == cfg
        end

        local effect = self.transform:Find("Left/target/effect_forge_forge_3")
        effect:SetActive(false)
        table.insert(self.effect_list, effect)
        local icon = self:findImage("Left/target/bg/icon")
        local quality = self:findImage("Left/target/bg/quality")
        local star_path = "Left/target/bg/stars"
        icon:SetSprite(cfg.path .. cfg.icon)
        quality:SetSprite(gui_helper:getQualityIconPath(cfg.color))
        frame_helper:hideAllChild(self, star_path, true)
        for i = 1, cfg.color do
            local star = frame_helper:getChild(self, i, star_path)
            star:SetActive(true)
        end


        local front_trans = self.transform:Find("Left/front")
        local effect = self.transform:Find("Left/front/bg/effect_forge_forge_1")
        effect:SetActive(false)

        local arrow_effect = self.transform:Find("Left/front/bg/arrow/effect_forge_forge_2 (1)")
        arrow_effect:SetActive(false)
        front_trans:SetActive(cfg.FrontCrew > 0)

        if cfg.FrontCrew > 0 then
            local front_cfg = config.crew_blocks[cfg.FrontCrew]
            local icon = self:findImage("Left/front/bg/icon")
            local quality = self:findImage("Left/front/bg/quality")
            local num = self:findText("Left/front/bg/text1_Static[T4]_5")
            local star_path = "Left/front/bg/stars"
            icon:SetSprite(front_cfg.path .. front_cfg.icon)
            quality:SetSprite(gui_helper:getQualityIconPath(front_cfg.color))
            num.text = frame_helper:getItemNumFormat(cfg.FrontCrew, 1)
            frame_helper:hideAllChild(self, star_path, true)
            for i = 1, front_cfg.color do
                local star = frame_helper:getChild(self, i, star_path)
                star:SetActive(true)
            end
            table.insert(self.effect_list, effect)
            table.insert(self.effect_list, arrow_effect)
        end

        local path = "Left/need"
        local temp_table = loadstring("return " .. cfg.UpgradeMaterial)()
        local temp_idx = 1
        for item_id, need_num in pairs(temp_table) do
            local item = frame_helper:getChild(self, temp_idx, path)
            item:SetActive(true)
            local num = item:Find("text1_Static[T4]_5"):GetComponent(UnityEngine.UI.Text)
            num.text = frame_helper:getItemNumFormat(item_id, need_num)
            local left_arrow = item:Find("left_arrow")
            local right_arrow = item:Find("right_arrow")


            left_arrow:SetActive(temp_idx % 2 == 0)
            right_arrow:SetActive(temp_idx % 2 > 0)

            local left_effect = item:Find("left_arrow/effect_forge_forge_2")
            local right_effect = item:Find("right_arrow/effect_forge_forge_2")
            local effect = item:Find("effect_forge_forge_1")

            left_effect:SetActive(false)
            right_effect:SetActive(false)
            effect:SetActive(false)

            if temp_idx % 2 == 0 then
                table.insert(self.effect_list, left_effect)
            else
                table.insert(self.effect_list, right_effect)
            end
            table.insert(self.effect_list, effect)


            local item_cfg = frame_helper:getItemCfg(item_id)
            local icon = item:Find("icon"):GetComponent(UnityEngine.UI.Image)
            local quality = item:Find("quality"):GetComponent(UnityEngine.UI.Image)
            icon:SetSprite(frame_helper:getItemIcon(item_id))
            quality:SetSprite(frame_helper:getItemQualityIcon(item_id))
            temp_idx = temp_idx + 1

            local btn = item:Find("icon"):GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(function()
                local item_inst = item_cls({cfgId = item_id, count = frame_helper:getItemNum(item_id)})
                frame_manager:show("itemInfo", nil, {item = item_inst})
            end)

            print("升级材料：", item_id, need_num)
        end

        for i = 1, 5 do
            local type_icon = self:findImage(string.format("Right/Building_button_54_normal/quality_icon (%s)", i))
            type_icon.enabled = cfg.color == i
        end
        local des = self:findText("Right/Building_button_54_normal/des")
        local name = self:findText("Right/item_name")

        name.text = frame_helper:getLangCfg(cfg.name_id)
        if local_lord.level >= cfg.UserLevel then
            des.text = string.format(NativeText.GetLangStr(620011), cfg.UserLevel)
        else
            des.text = string.format(NativeText.GetLangStr(620015), cfg.UserLevel)
        end
        
        local cost_icon = self:findImage("Left/cost_icon")
        local cost_num = self:findText("Left/cost_num")
        local cost_table = loadstring("return " .. cfg.UpgradeMoney)()
        local res_cost_diamond = 0
        for item_id, need_num in pairs(cost_table) do
            cost_icon:SetSprite(frame_helper:getItemIcon(item_id))
            cost_num.text = frame_helper:getItemNumFormat(item_id, need_num)
            local own_num = frame_helper:getItemNum(item_id)
            local lack_num = need_num - own_num
            if lack_num > 0 then
                local temp = frame_helper:getResToDiamondNum(item_id, lack_num)
                res_cost_diamond = res_cost_diamond + temp
            end
        end

        local prop_list = frame_helper:getPropertyDes(cfg.EffectPara)
        local prop_path = "Right/prop layout"
        frame_helper:hideAllChild(self, prop_path, true)
        for __, v in ipairs(prop_list) do
            local item = frame_helper:getChild(self, __, prop_path)
            item:SetActive(true)
            local name = item:Find("name"):GetComponent(UnityEngine.UI.Text)
            local value = item:Find("value"):GetComponent(UnityEngine.UI.Text)
            name.text = v.name_str
            value.text = v.num_str
        end

        local unreal = self:findText("Right/un_upgrade/upgradeBtn/unreal")
        local real = self:findText("Right/un_upgrade/upgradeBtn/real")
        unreal.text = frame_helper:timeFormat(cfg.TrainTime)
        real.text = frame_helper:timeFormat(home_info:getForgeUpgradeTime(cfg))

        local speed_up_type = home_speed_up_type.quantum_create
        local time_cost_diamond = gui_helper:getSpeedUpCost(speed_up_type, home_info:getForgeUpgradeTime(cfg))
        local total_diamond = res_cost_diamond + time_cost_diamond

        local DIAMOND    = 30001 -- 钻石
        local item_name = gui_helper:getImageTextItemStr(DIAMOND)
        local image_text = self:findText("Right/un_upgrade/finishBtn/image_text")
        image_text:SetImageText(string.format("%s %s", item_name, total_diamond))

        local upgrade_btn = self:findButton("Right/un_upgrade/upgradeBtn")
        upgrade_btn.onClick:RemoveAllListeners()
        upgrade_btn.onClick:AddListener(function()
            if self:isUpgardeEquiped(cfg) then
                frame_helper:showCommonBox(NativeText.GetLangStr(900016), function()
                    network_sender:trainProfession(cfg.id, false)
                end)
            else
                network_sender:trainProfession(cfg.id, false)
            end
        end)

        local build_data = home_info:getBuildDataByCfgId(22)
        local finish_btn = self:findButton("Right/un_upgrade/finishBtn")
        finish_btn.onClick:RemoveAllListeners()
        finish_btn.onClick:AddListener(function()
            if frame_helper:isEnoughDiamond(total_diamond) then
                if self:isUpgardeEquiped(cfg) then
                    frame_helper:showCommonBox(NativeText.GetLangStr(900016), function()
                        network_sender:trainProfession(cfg.id, true)
                    end)
                else
                    network_sender:trainProfession(cfg.id, true)
                end
            end
        end)

        local speed_up_btn = self:findButton("Right/upgrading/speedUpBtn")
        speed_up_btn.onClick:RemoveAllListeners()
        speed_up_btn.onClick:AddListener(function()
            frame_manager:show("home_speed_up", nil, {
                speed_up_type = home_speed_up_type.quantum_create,
                id = build_data.id,
                forge_block_id = build_data.trainCenter.cfgId,
            })
        end)

        local clear_time_btn = self:findButton("Right/upgrading/clearTimeBtn")
        clear_time_btn.onClick:RemoveAllListeners()
        clear_time_btn.onClick:AddListener(function()
            network_sender:perSonBaseSpeedUp(build_data.id, common_pb.EPB_PRO_QUA_EQUIP, 1, 0, {}, build_data.trainCenter.cfgId)
        end)

        local un_upgrade = self.transform:Find("Right/un_upgrade")
        local upgrading = self.transform:Find("Right/upgrading")

        frame_helper:setActive(un_upgrade, not build_data.trainCenter.bWork)
        frame_helper:setActive(upgrading, build_data.trainCenter.bWork)

        if build_data.trainCenter.bWork then
            local e = build_data
            local elapsed = e.trainCenter.remainTime - Time.realtimeSinceStartup + e.rmbTime
            self:onTick(e, elapsed, e.trainCenter.totaltime)
        end
    end,

    isUpgardeEquiped = function(self, cfg)
        if cfg.FrontCrew > 0 then
            local num_in_bag = bag_manager:getItemNum(cfg.FrontCrew)
            local num_in_crew = crew_manager:getItemNumByCfgId(cfg.FrontCrew)

            -- log.error("num_in_bag:", num_in_bag)
            -- log.error("num_in_crew:", num_in_crew)

            if num_in_bag <= 0 and num_in_crew > 0 then
                return true
            else
                return false
            end
        else
            return false
        end
    end,

    getCfgList = function(self, group_id, slot_id)
        local rst = {}
        config.crew_blocks:foreach(function(id, cfg)
            if cfg.Group == group_id and cfg.Slot == slot_id then
                table.insert(rst, cfg)
            end
        end)
        local sortFunc = function(a, b)
            return a.id < b.id
        end
        table.sort(rst, sortFunc)
        return rst
    end,
    
    onTick = function(self, build_data, elapsed, total_time)
        local time = self:findText("Right/upgrading/slider/time")
        local slider = self:findSlider("Right/upgrading/slider")
        time.text = string.format(NativeText.GetLangStr(620016), frame_helper:timeFormat(elapsed))
        slider.value = 1 - elapsed / total_time
    end,

    onBuildInfoChange = function(self, build_id)
        local build_data = home_info:getBuildDataById(build_id)
        if build_data.cfgId == 22 then
            self:click_forge(self.cur_cfg)
        end
    end,

    playUpgradeEffect = function(self, cfg_id, bPromFinish)
        if bPromFinish then
            for __, e in ipairs(self.effect_list) do 
                e:SetActive(true)
            end
        end
    end,
}
