local log = logger.get("Game", "Home Frame")
local super = require("ui.frame")
local enums = require("RTS.home_sys.home_enums")

return class {
    super = super, 

    onLoaded = function (self, owner)
        super.onLoaded(self)
        self.owner = self.args.owner

        clickEvents = 
        {
            "new/buttomLeft/bg/homeBtn",
            "new/buttomLeft/bg/layout/emailBtn",
            "new/buttomLeft/bg/layout/carBtn",
            "new/buttomLeft/bg/layout/storeHouseBtn",
            "new/buttomLeft/bg/layout/dsfdsf/unionBtn",
            "new/buttomLeft/bg/layout/setBtn",

            "new/buttomLeft/bg/layout/dsfdsf/GameObject (4)/taskBtn",
            "new/buttomLeft/bg/layout/dsfdsf/GameObject (4)/dailyBtn",
            "new/buttomLeft/bg/layout/dsfdsf/GameObject (4)/achieveBtn",

            "new/buttomLeft/bg/chatBtn",
            -- "new/buttomLeft/bg/chatChannelBtn",
            "new/buttomLeft/bg/layout/robinBtn",

            "new/topRight/res layout/Asset Gem (1)/addAssetBtn1",
            "new/topRight/res layout/Asset Gem (2)/addAssetBtn2",
            "new/topRight/res layout/Asset Gem (3)/addAssetBtn3",
            "new/topRight/res layout/Asset Gem (4)/addAssetBtn4",
            "new/topRight/res layout/Asset Gem (5)/addAssetBtn5",
            "new/topRight/Asset Gem/addGemBtn",

            "new/leftCenter/ButtonChapterAchievement",
            "new/leftCenter/ButtonChapterAchievementGo",
            -- 伸缩
            "new/buttomRight/showBtn",
            "new/buttomRight/hideBtn",

            "new/buffBtn",
            "new/WarningBtn",
            "new/WarningNode/Reinforce",
            "new/WarningNode/EnemyWarn",
            "new/topLeft/bg/icon",

            'new/buttomRight/triggerResShipBtn',

            -- hong feng , temp level enterance
            "1001",
            "1002",
            "2001",
            "2002",
            "2003",
            "new/topLeft/activity/activityBtn",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self.steel_text =   self:findText('new/topRight/res layout/Asset Gem (1)/Text')
        self.wood_text =    self:findText('new/topRight/res layout/Asset Gem (2)/Text')
        self.brick_text =   self:findText('new/topRight/res layout/Asset Gem (3)/Text')
        self.uranium_text = self:findText('new/topRight/res layout/Asset Gem (4)/Text')
        self.food_text = self:findText('new/topRight/res layout/Asset Gem (5)/Text')


        self.diamond_text = self:findText('new/topRight/Asset Gem/Text')
        self.enemy_warn_text = self:findText("new/WarningNode/EnemyWarn/Time")
        self.reinforce_text = self:findText("new/WarningNode/Reinforce/Time")
        self.warn_cnt_text = self:findText("new/WarningBtn/RedPoint/Count")

        self.enemy_warn_panel = self.transform:Find("new/WarningNode/EnemyWarn")
        self.reinforce_panel = self.transform:Find("new/WarningNode/Reinforce")
        self.warn_btn = self.transform:Find("new/WarningBtn")

        self.left_form_tplt = self.transform:Find("new/m_line/offset/marchs/line_tplt").gameObject
        self.left_form = self.transform:Find('new/m_line/offset/marchs')
        self.left_form_tplt:SetActive(false)
        
        self.chapter_achievement_shortcut = self.transform:Find("new/leftCenter")

        self:refreshResourceNum()
        self.prop_change_event_id = local_player.events:on("onPropChange", functor.make(self.refreshResourceNum, self))

        self:refreshChapterAchievementShortcut()

        self:init()

        level_info.level_over = false


        local upgrading_entity = home_info:getUpgradingEntity()
        if upgrading_entity then
            self:addLeftProgress(upgrading_entity.id)
        end

        self.BUILD_INFO_CHANGE = home_info.events:on("BUILD_INFO_CHANGE", functor.make(self.addLeftProgress, self))
        self.BUFF_INFO_CHANGE = buff_info.events:on("BUFF_INFO_CHANGE", functor.make(self.onBuffInfoChange, self))
        self.RES_FOOD_CHANGE = game_scheduler.events:on("RES_FOOD_CHANGE", functor.make(self.onResFoodChange, self))
        self.ACTIVITY_TIME_TICK = activity_manager.events:on("ACTIVITY_TIME_TICK", functor.make(self.onActivityTimeTick, self))
        self.LORD_INFO_CHANGE = local_lord.events:on("LORD_INFO_CHANGE", functor.make(self.onLordInfoChange, self))

        self.warning_evt = game_director.events:on("WARNNOTIFY", functor.make(self.onWarnNotify, self))
        self.warn_changed = game_director.events:on("WARNSTATECHANGED", functor.make(self.onWarnNotify, self))

        home_frame = self

        guide_manager:tryStartGuide()
        
        -- 缓存事件id
        self.event_ids = self.event_ids or {}

        if game_info.replay_battle_mail_id then
            frame_manager:show("mail", nil, {mails = local_player.mail_list, mail_id = game_info.replay_battle_mail_id})
            game_info.replay_battle_mail_id = nil
        end

        self:onWarnNotify()

        chapter_manager:initChapterResShip()

        -- 关闭 loading
        frame_manager:hide('loading')
        -- 终止同步加载
        Fenix.Assets.Helper.ms_SyncLoad = false
    end,

    onUnload = function (self)

        home_frame = nil
        local_player.events:off("onPropChange", self.prop_change_event_id)
        local_chat.events:off("default_channel_change", self.default_channel_change_id)
        local_chat.events:off("chat_msg_change", self.chat_msg_change_id)
        home_info.events:off("BUILD_INFO_CHANGE", self.BUILD_INFO_CHANGE)
        buff_info.events:off("BUFF_INFO_CHANGE", self.BUFF_INFO_CHANGE)
        game_director.events:off("WARNNOTIFY", self.warning_evt)
        game_director.events:off("WARNSTATECHANGED", self.warn_changed)
        game_scheduler.events:off("RES_FOOD_CHANGE", self.RES_FOOD_CHANGE)
        local_lord.events:off("LORD_INFO_CHANGE", self.LORD_INFO_CHANGE)
        activity_manager.events:off("ACTIVITY_TIME_TICK", self.ACTIVITY_TIME_TICK)

        if self.event_ids then
            for __, id in pairs(self.event_ids) do
                home_info.events:off("BUILD_UPGRADE_TICK", id)
            end
        end

        if self.speed_btn then
            self.speed_btn.onClick:RemoveAllListeners()
        end
    end,

    init = function(self)
        self:setChatInfo()
        self.default_channel_change_id = local_chat.events:on('default_channel_change', function()
            self:setChatInfo()
        end)
        self.chat_msg_change_id = local_chat.events:on('chat_msg_change', function()
            self:setChatInfo()
        end)
        -- self:click_hideBtn()
        self:onBuffInfoChange()

        self:onLordInfoChange()
    end,

    onLordInfoChange = function(self)
        local force = self:findText("new/topLeft/res_bg/Text")
        local vip = self:findText("new/topLeft/res_bg/Text (1)")
        local level = self:findText("new/topLeft/bg/lvbg/Text")

        force.text = local_lord.fight_force
        vip.text = local_lord.vip_level
        level.text = "Lv." .. local_lord.level
    end,

    click_openEntityBtn = function(self)
        frame_manager:show('home_entity_create', self.owner)
    end,

    click_addAssetBtn1 = function(self)
        frame_manager:show('resource_exchange', nil, { 
            item_id = 32001,
            item_num = 0,
        })
    end,

    click_addAssetBtn2 = function(self)
        frame_manager:show('resource_exchange', nil, { 
            item_id = 30002,
            item_num = 0,
        })
    end,

    click_addAssetBtn3 = function(self)
        frame_manager:show('resource_exchange', nil, { 
            item_id = 32002,
            item_num = 0,
        })
    end,

    click_addAssetBtn4 = function(self)
        frame_manager:show('resource_exchange', nil, { 
            item_id = 32003,
            item_num = 0,
        })
    end,

    click_addAssetBtn5 = function(self)
        frame_manager:show('resource_exchange', nil, { 
            item_id = 32004,
            item_num = 0,
        })
    end,

    -- 基地
    click_homeBtn = function(self)
        print("基地")
        if local_guild:canEnterWorldMap() then
            Fenix.Assets.Helper.ms_SyncLoad = true
            game_director:changeState('loading', {next_state = 'world_map', scene_name = 'hex_map'})
        else
            game_director:changeState('loading', {next_state = 'island', scene_name = 'island'})
        end
    end,

    click_taskBtn = function(self)
        frame_manager:show('growth_task')
        self:click_unionBtn()
    end,

    click_dailyBtn = function(self)
        frame_manager:show('daily_task', self)
        self:click_unionBtn()
    end,

    click_achieveBtn = function(self)
        frame_manager:show('chapter_achievement', self)
        self:click_unionBtn()
    end,
    
    click_unionBtn = function(self)
        local task_trans = self.transform:Find("new/buttomLeft/bg/layout/dsfdsf/GameObject (4)")
        if not self.task_vsb then
            self.task_vsb = true
        else
            self.task_vsb = false
        end
        frame_helper:setActive(task_trans, self.task_vsb)
    end,

    click_setBtn = function(self)
        app:callJava("sdkSwitchAccount")
    end,

    -- 仓库
    click_storeHouseBtn = function(self)
        print("工会")
        if local_guild:inGuild(local_guild:getId()) then
            network_sender:getFamilyInfo(local_player:getGuildId())
        else
            network_sender:getFamilyList(0)
        end
    end,

    -- 车库
    click_carBtn = function(self)
        print("车库")
        frame_manager:show('hero_list', self)
    end,

    -- 进入关卡
    click_enterLevelBtn = function(self)
        frame_manager:show('chapter')
    end,

    -- 邮件
    click_emailBtn = function(self)
        print("邮件")
        network_sender:getMailListInfo()
    end,

    -- 地图
    click_mapBtn = function(self)
        print("地图")
    end,

    -- 战报
    click_recordBtn = function(self)
        print("战报")
    end,

    -- 罗宾
    click_robinBtn = function(self)
        print("仓库")
        frame_manager:show('bag', self)
        -- frame_manager:show('quantum', self)
        -- hint:setText(NativeText.GetLangStr(900002))
    end,

    -- 触发状态为 ECHAPTER_NO_TRIGGER 的资源船
    click_triggerResShipBtn = function(self)
        local rst = chapter_manager:buttonTriggerResShip()
        if not rst then
            log.error('Trigger ResShip Failed!!!')
        end
    end,

    refreshResourceNum = function(self)
        self.helper:makeNumEffect(self.steel_text.transform,    local_player.props[protocol_pb.PROR_STEEL])
        self.helper:makeNumEffect(self.wood_text.transform,     local_player.props[protocol_pb.PROP_GOLD])
        self.helper:makeNumEffect(self.brick_text.transform,    local_player.props[protocol_pb.PROP_WOOD])
        self.helper:makeNumEffect(self.uranium_text.transform,  local_player.props[protocol_pb.PROP_BRICK])
        self.helper:makeNumEffect(self.food_text.transform,  frame_helper:getItemNum(32004))
        self.helper:makeNumEffect(self.diamond_text.transform,  local_player.props[protocol_pb.PROP_DIAMOND])
    end,

    onResFoodChange = function(self, num)
        self.food_text.text = frame_helper:itemNumFormat(num)
    end,

    onActivityTimeTick = function(self, id, elapsed)
        if self.cur_activity_id and self.cur_activity_id == id then
            self.activity_time = self.activity_time or self:findText("new/topLeft/activity/activityBtn/lvbg/timebg/time")
            self.activity_time.text = frame_helper:timeFormat(elapsed)
        end
    end,

    onActivityTimeUpdate = function(self, dt)
        if not self.is_loaded then
            return
        end

        self.activity_trans = self.activity_trans or self.transform:Find("new/topLeft/activity")
        local actives = activity_manager:getAllActivitys()
        local count = activity_manager:getCount()
        if actives and count > 0 then
            frame_helper.showTrans(self.activity_trans)
            self.activity_idx = self.activity_idx or 1
            self.activity_accumulate = self.activity_accumulate and self.activity_accumulate + dt or 10
            if self.activity_accumulate > 10 then
                self.activity_accumulate = 0
                self.activity_idx = self.activity_idx + 1
                if self.activity_idx > count then
                    self.activity_idx = 1
                end
                local act = activity_manager:getActivityByIdx(self.activity_idx)
                local cfg = config.active_basics[act.cfg_id]
                self.activity_name = self.activity_name or self:findText("new/topLeft/activity/activityBtn/lvbg/name")
                self.activity_icon = self.activity_icon or self:findImage("new/topLeft/activity/activityBtn")
                self.activity_name.text = frame_helper:getLangCfg(cfg.Name)
                self.activity_icon:SetSprite(cfg.path .. cfg.icon)
                self.cur_activity_id = act.id
            end
        else
            frame_helper.hideTrans(self.activity_trans)
        end
    end,

    click_showBtn = function(self)
        self:setExtendState(true)
    end,

    click_hideBtn = function(self)
        self:setExtendState(false)
    end,

    setExtendState = function(self, extend)
        local duration = 0.25

        local buttom = self.transform:Find('new/buttomLeft/bg').gameObject
        LeanTween.cancel(buttom)
        local lt = LeanTween.moveLocalY(buttom, not extend and 0 or -180, duration)

        local fun_layout = self.transform:Find('new/fun layout'):GetComponent(UnityEngine.RectTransform)
        LeanTween.cancel(fun_layout)
        local lt = LeanTween.moveX(fun_layout, not extend and 0 or 130, duration)

        local m_line = self.transform:Find('new/m_line'):GetComponent(UnityEngine.RectTransform)
        LeanTween.cancel(m_line)
        local lt = LeanTween.moveX(m_line, not extend and 0 or -360, duration)

        local top_left = self.transform:Find('new/topLeft'):GetComponent(UnityEngine.RectTransform)
        LeanTween.cancel(top_left)
        local lt = LeanTween.moveY(top_left, not extend and 0 or 230 , duration)

        local top_right = self.transform:Find('new/topRight'):GetComponent(UnityEngine.RectTransform)
        LeanTween.cancel(top_right)
        local lt = LeanTween.moveY(top_right, not extend and 0 or 230 , duration)

        local bg = self.transform:Find('new/bg (1)'):GetComponent(UnityEngine.RectTransform)
        LeanTween.cancel(bg)
        local lt = LeanTween.moveY(bg, not extend and 0 or 125 , duration)

        local bg = self.transform:Find('new/bg (2)'):GetComponent(UnityEngine.RectTransform)
        LeanTween.cancel(bg)
        local lt = LeanTween.moveY(bg, not extend and 0 or -125 , duration)

        local bg = self.chapter_achievement_shortcut
        LeanTween.cancel(bg)
        local lt = LeanTween.moveX(bg, not extend and 0 or -850 , duration)

        local show_trans = self.transform:Find('new/buttomRight/showBtn')
        local hide_trans = self.transform:Find('new/buttomRight/hideBtn')

        show_trans.gameObject:SetActive(not extend)
        hide_trans.gameObject:SetActive(extend)
    end,
    
    click_chatBtn = function(self)
        frame_manager:show('chat')
    end,
    
    click_chatChannelBtn = function(self)
        frame_manager:show('chat_channel')
    end,

    -- 点击章节目标
    click_ButtonChapterAchievement = function(self)
        frame_manager:show('chapter_achievement')
    end,

    -- 点击章节目标引导/领取奖励
    click_ButtonChapterAchievementGo = function(self)
        if not self.achievement_go_time then
            self.achievement_go_time = 0.0
        end
        if Time.time - self.achievement_go_time < 0.5 then
            return
        end
        self.achievement_go_time = Time.time

        local achievement = chapter_achievement_manager:getPrimaryAchievement()
        if achievement then
            if achievement.state == common_pb.ACHIEVEMENT_FINISH then
                -- 发消息请求领奖励
                network_sender:reqChapterAchievementReward(achievement.id)

            elseif achievement.state == common_pb.ACHIEVEMENT_UNLOCK then
                -- 任务链接
                local go_callback = chapter_achievement_manager.go_callback[achievement.id]
                if go_callback then
                    go_callback()
                else
                    log.error("Failed to find go callback:"..achievement.id)
                end
            end
        else
        end

    end,

    -- 刷新章节目标快捷按钮
    refreshChapterAchievementShortcut = function(self)
        local achievement = chapter_achievement_manager:getPrimaryAchievement()
        if achievement then
            self.chapter_achievement_shortcut:SetActive(true)
            local t = self.chapter_achievement_shortcut:Find("ButtonChapterAchievementGo/TextDesc")
            t:GetComponent(UnityEngine.UI.Text).text = frame_helper:getLangCfg(achievement.desc)

           -- t = self.chapter_achievement_shortcut:Find("ButtonChapterAchievementGo")
            local can_get = self.chapter_achievement_shortcut:Find("canGet")
            local img = self.transform:Find("new/leftCenter/bg/Image"):GetComponent(UnityEngine.UI.Image)

            local icon_img = self.transform:Find("new/leftCenter/ButtonChapterAchievement"):GetComponent(UnityEngine.UI.Image)
            local icon_img_add = self.transform:Find("new/leftCenter/ButtonChapterAchievement_add"):GetComponent(UnityEngine.UI.Image)
            local anim = self.transform:Find("new/leftCenter"):GetComponent(UnityEngine.Animation)

            if achievement.state == common_pb.ACHIEVEMENT_FINISH then
                img:SetSprite("Assets/Arts/ui/atlas/Main/main_panel_15_2.png")
                can_get:SetActive(true)
                icon_img:SetSprite("Assets/Arts/ui/atlas/Main/main_button_16_normal.png")
                icon_img_add:SetSprite("Assets/Arts/ui/atlas/Main/main_button_16_normal.png")
                anim:Play("cashing_prize")
            elseif achievement.state == common_pb.ACHIEVEMENT_UNLOCK then
                img:SetSprite("Assets/Arts/ui/atlas/Main/main_panel_15.png")
                can_get:SetActive(false)
                icon_img:SetSprite("Assets/Arts/ui/atlas/Main/main_button_16_1_normal.png")
                icon_img_add:SetSprite("Assets/Arts/ui/atlas/Main/main_button_16_1_normal.png")
                anim:Play("idle")
            end

        else
            self.chapter_achievement_shortcut:SetActive(false)
        end
    end,

    setChatInfo = function(self)
        if not local_chat then
            print('local_chat is nil !!!')
            return 
        end

        local text = self:findText('new/buttomLeft/bg/chatChannelBtn/Text')
        text.text = local_chat:getDefaultChannel().acronym
        local text = self:findText('new/buttomLeft/bg/chatBtn/Text')
        local des = local_chat:getLatestContent(local_chat:getDefaultChannel())
        text.text = des and des or ""
    end,

    addBuildingProgress = function(self, entity_data, leftTime)
        self.cache_progresses = self.cache_progresses or {}
        local id = entity_data.id
        if leftTime > 0 then
            local go = nil
            if not self.cache_progresses[id] then
                go = Object.Instantiate(self.left_form_tplt);
                go.transform:SetParent(self.left_form)
                go.transform.localPosition = Vector3.zero
                go.transform.name = id
                go.transform.localScale = Vector3.one
                go:SetActive(true)
                self.cache_progresses[id] = go
            else
                go = self.cache_progresses[id]
            end
            local cfg = config.home_config_lists[entity_data.cfgId]

            local des_text = go.transform:Find('desText').gameObject:GetComponent(UnityEngine.UI.Text)
            des_text.text = self.helper:getLangCfg(cfg.name_id)

            self.speed_btn = go.transform:Find('speedBtn'):GetComponent(UnityEngine.UI.Button)
            self.speed_btn.onClick:RemoveAllListeners()
            self.speed_btn.onClick:AddListener(
                function ()
                    frame_manager:show("home_speed_up", nil, {
                        speed_up_type = home_speed_up_type.build_upgrade,
                        id = entity_data.id
                    })
                end)
            
            local time_text = go.transform:Find('timeText').gameObject:GetComponent(UnityEngine.UI.Text)
            time_text.text = self.helper:timeFormat(leftTime) 

            self.event_ids = self.event_ids or {}
            if self.event_ids[id] then
                home_info.events:off("BUILD_UPGRADE_TICK", self.event_ids[id])
                self.event_ids[id] = nil
            end
            local evt_id =  home_info.events:on("BUILD_UPGRADE_TICK", 
                function(ed, elapsed, total_time)
                    if ed.id == entity_data.id then
                        time_text.text = self.helper:timeFormat(elapsed) 
                    end
                end)
            self.event_ids[id] = evt_id
        else
            if self.cache_progresses[id] then
                if self.event_ids[id] then
                    home_info.events:off("BUILD_UPGRADE_TICK", self.event_ids[id])
                    self.event_ids[id] = nil
                end
                GameObject.Destroy(self.cache_progresses[id])
                self.cache_progresses[id] = nil
            end
            self:tweenBP()
        end

        self:tweenBP()
    end,

    tweenBP = function(self)
        if self.cache_progresses_visible == nil then
            self.cache_progresses_visible = false
        end
        local visible = false
        if self.cache_progresses then
            for __, value in pairs(self.cache_progresses) do
                if value then
                    visible = true
                    break
                end
            end
        end
        if self.cache_progresses_visible == visible then
            return
        else
            self.cache_progresses_visible = visible
            local offset = self.transform:Find('new/m_line/offset').gameObject
            LeanTween.cancel(offset)
            local lt = LeanTween.moveLocalX(offset, visible and 0 or -360, 0.25)
        end
    end,

    addLeftProgress = function(self, id)
        local entity_data = home_info:getBuildDataByGridId(id)
        if entity_data then
            self:addBuildingProgress(entity_data, entity_data.tLevelUp)
        end
    end,

    click_buffBtn = function(self)
        frame_manager:show('home_buff')
    end,

    onBuffInfoChange = function(self)
        local num = buff_info:getBuffNum()
        frame_helper:setActive(self.transform:Find("new/buffBtn"), num > 0)
        local text = self:findText("new/buffBtn/Image/Text")
        text.text = num

        if num > 0 then
            local icon = self:findImage("new/buffBtn")
            local rst = frame_helper.max_number
            for __, b in ipairs(buff_info.buffs) do
                if b.buff_id < rst then
                    rst = b.buff_id
                end
            end
            local buff_cfg = config.buff_buffs[rst]
            local group_cfg = config.home_config_buff_groups[buff_cfg.group]
            icon:SetSprite(group_cfg.group_icon)
        end
    end,

    click_WarningBtn = function(self)
        frame_manager:show("warn_info")
    end,

    click_Reinforce = function(self)
        frame_manager:show("warn_info")
    end,

    click_EnemyWarn = function(self)
        frame_manager:show("warn_info")
    end,

    onWarnNotify = function(self)
        self.warn_count = 0
        self.curr_enemy_warn = nil
        self.curr_help_warn = nil

        for k, v in pairs(game_info.warning_info) do
            if not v.is_off then
                local dt = os.time() - v.receive_time
    
                if v.type == common_pb.EMARCH_HELP then
                    if self.curr_help_warn then
                        if self.curr_help_warn.leftTime > v.leftTime - dt then
                            self.curr_help_warn = v
                        end
                    else
                        self.curr_help_warn = v
                    end
                else
                    if self.curr_enemy_warn then
                        if self.curr_enemy_warn.leftTime > v.leftTime - dt then
                            self.curr_enemy_warn = v
                        end
                    else
                        self.curr_enemy_warn = v
                    end
                end
            end

            self.warn_count = self.warn_count + 1
        end

        self.enemy_warn_panel:SetActive(self.curr_enemy_warn ~= nil)
        self.reinforce_panel:SetActive(self.curr_help_warn ~= nil)
        self.warn_btn:SetActive(self.warn_count > 0)
        self.warn_cnt_text.text = tostring(self.warn_count)

        if self.curr_enemy_warn then
            frame_manager:show('rim_red')
        else
            local is_display = frame_manager:isDisplay('rim_red')
            
            if is_display then
                frame_manager:hide('rim_red')
            end
        end
    end,

    update = function(self, dt)
        if self.curr_enemy_warn and self.enemy_warn_text then
            local detal = os.time() - self.curr_enemy_warn.receive_time

            if self.curr_enemy_warn.leftTime - detal > 0 then
                --self.curr_enemy_warn.leftTime = self.curr_enemy_warn.leftTime - dt
                self.enemy_warn_text.text = gui_helper:timeFormat(self.curr_enemy_warn.leftTime - detal)
            else
                self:onWarnNotify()
            end
        end

        if self.curr_help_warn and self.reinforce_text then
            local detal = os.time() - self.curr_help_warn.receive_time

            if self.curr_help_warn.leftTime - detal > 0 then
                --self.curr_help_warn.leftTime = self.curr_help_warn.leftTime - dt
                self.reinforce_text.text = gui_helper:timeFormat(self.curr_help_warn.leftTime - detal)
            else
                self:onWarnNotify()
            end
        end

        self:onActivityTimeUpdate(dt)
        
        if Input.GetMouseButtonDown(0) then
            local point = Input.mousePosition
            local click_on_ui = RTSHelper.IsPointerOverUI(point.x, point.y)
            if not click_on_ui then
                if self.task_vsb then
                    self.task_vsb = not self.task_vsb
                    local task_trans = self.transform:Find("new/buttomLeft/bg/layout/dsfdsf/GameObject (4)")
                    frame_helper:setActive(task_trans, self.task_vsb)
                end
            end
        end
    end,

    setTopLeftVsb = function(self, visible)
        local trans = self.transform:Find("new/topLeft")
        frame_helper:setActive(trans, visible)
    end,

    click_icon = function(self)
        frame_manager:show('lord_info')
    end,

    --hong feng, temp level entrance
    click_1001 = function(self)
        network_sender:enterMap(1001, 0)
    end,

    click_1002 = function(self)
        network_sender:enterMap(1002, 0)
    end,

    click_2001 = function(self)
        network_sender:enterMap(2001, 0)
    end,

    click_2002 = function(self)
        network_sender:enterMap(2002, 0)
    end,

    click_2003 = function(self)
        network_sender:enterMap(2003, 0)
    end,

    click_activityBtn = function(self)
        frame_manager:show('activity_main')
    end,

    click_addGemBtn = function(self)
        hint:setText(NativeText.GetLangStr(999034))
    end,
}
