local super = require("ui.frame")
local log = logger.get("Game", "home_info")

local button_state = {
    has_hero = 1,
    emtpy = 2,
    lock = 3,
}

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        self.home_map = self.args.owner

        clickEvents = 
        {
            "content/backBtn",
            "content/hero/hideHeroBtn",
            "content/hero/clearHeroBtn",
            "content/hero/autoHeroBtn",
        }

        -- 注册点击事件
        frame_helper:regClickEvents("click", self, clickEvents)

        -- self.build_data = home_info:getBuildDataByCfgId(24)
        -- self.WALL_CURE_TICK = home_info.events:on("WALL_CURE_TICK", function(build_data, elapsed, total_time)
        --     self:refresh()
        -- end)
        -- self.BUILD_INFO_CHANGE = home_info.events:on("BUILD_INFO_CHANGE", function(build_id)
        --     local build_data = home_info:getBuildDataById(build_id)
        --     if build_data.cfgId == 24 then
        --         self:refresh()
        --     end
        -- end)

        self:init()
    end,

    onUnload = function (self)
        -- home_info.events:off("BUILD_INFO_CHANGE", self.BUILD_INFO_CHANGE)
        -- home_info.events:off("WALL_CURE_TICK", self.WALL_CURE_TICK)
    end,

    init = function(self)
        self.heros_trans = self.transform:Find("content/hero")
        self.soldiers_trans = self.transform:Find("content/soldier")
        frame_helper:setActive(self.heros_trans, false)
        frame_helper:setActive(self.soldiers_trans, true)

        self:setOfficers()
        self:setSoldiers()
        self:refresh()
    end,

    getOfficers = function(self)
        local rst = {}
        local officers =  hero_manager:getOwnOfficers()
        for id, officer in pairs(officers) do
            table.insert(rst, officer)
        end
        local sortFunc = function(a, b)
            return hero_manager:getHero(a.hero_id).strength > hero_manager:getHero(b.hero_id).strength
        end
        table.sort(rst, sortFunc)
        return rst
    end,

    setOfficers = function(self)
        local officers =  self:getOfficers()
        self.cache_officers = {}
        for idx, officer in ipairs(officers) do
            local child = frame_helper:getChild(self, idx, "content/hero/Scroll View/Viewport/Content")
            local level = child:Find('Grade/Level'):GetComponent(UnityEngine.UI.Text)
            local icon = child:Find('Grade/Icon'):GetComponent(UnityEngine.UI.Image)
            local quality = child:Find('Grade/Quality'):GetComponent(UnityEngine.UI.Image)
            local stars = child:Find('Grade/Stars')

            local cfg = officer.cfg
            icon:SetSprite(gui_helper:getOfficerIconPath(officer))
            quality:SetSprite(gui_helper:getNewQualityIcon(cfg.color))
            level.text = officer.level
            gui_helper:setOfficerStars(officer.star_level, stars)

            local temp_selcet = false
            local build_data = home_info:getBuildDataByCfgId(24)
            
            for __, data in ipairs(build_data.guardsystem.heros) do
                if data.heroId == officer.hero_id then
                    temp_selcet = true
                    break
                end
            end

            table.insert(self.cache_officers, {
                idx = idx,
                child = child,
                officer = officer,
                select = temp_selcet,
            })

            local temp_idx = idx
            local btn = child:GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(function()
                self:click_buttom_officer(temp_idx)
            end)
        end

        local num_text = self:findText("content/hero/info/num")
        num_text.text = #self.cache_officers
    end,

    setSoldiers = function(self)
        self.troops_power = 0
        local is_empty = true
        local path = "content/soldier/Scroll View/Viewport/Content"
        frame_helper:hideAllChild(self, path, true)
        local build_data = home_info:getBuildDataByCfgId(10)
        if build_data then
            local idx = 0
            for __, s in ipairs(build_data.parade.soldiers) do
                local idle_num = s.totalNum - s.bInArmy - s.bHurt - s.bTreat
                if idle_num > 0 then
                    is_empty = false
                    idx = idx + 1
                    local child = frame_helper:getChild(self, idx, path)
                    child:SetActive(true)
                    local name = child:Find("name"):GetComponent(UnityEngine.UI.Text)
                    local num = child:Find("num"):GetComponent(UnityEngine.UI.Text)
                    local icon = child:Find("icon"):GetComponent(UnityEngine.UI.Image)
                    local quality = child:Find("quality"):GetComponent(UnityEngine.UI.Image)
                    local soldier_cfg = config.soldier_soldiers[s.cfgId]
                    name.text = frame_helper:getLangCfg(soldier_cfg.soldiername)
                    num.text = string.format(NativeText.GetLangStr(550008), idle_num)
                    icon:SetSprite(soldier_cfg.headpath .. soldier_cfg.soldierhead)
                    quality:SetSprite(gui_helper:getQualityIconPath(soldier_cfg.color))

                    self.troops_power = self.troops_power + idle_num * soldier_cfg.power
                end
            end
            local num_text = self:findText("content/soldier/info/num")
            num_text.text = idx

            local power = self:findText("content/soldier/info/power")
            power.text = self.troops_power
        end
        self:findText("content/soldier/Text").enabled = is_empty
    end,

    refresh = function(self)
        local defense_heros = {}
        local hero_length = 0
        for __, value in ipairs(self.cache_officers) do
            local select_trans = value.child.transform:Find("Grade/select")
            frame_helper:setActive(select_trans, value.select)
            if value.select then
                hero_length = hero_length + 1
                table.insert(defense_heros, value.officer)
            end
        end

        local build_data = home_info:getBuildDataByCfgId(24)
        local strength = 0
        -- 刷新驻守英雄
        local max_num = 5
        local unlock_num = self:getUnlockNum()
        for idx = 1, max_num do
            local child = frame_helper:getChild(self, idx, "content/top_hero")
            local hero_trans = child.transform:Find("hero")
            local empty_trans = child.transform:Find("empty")
            local lock_trans = child.transform:Find("lock")
            frame_helper.hideTrans(hero_trans)
            frame_helper.hideTrans(empty_trans)
            frame_helper.hideTrans(lock_trans)
            local state = nil
            if idx <= hero_length then
                frame_helper.showTrans(hero_trans)
                state = button_state.has_hero

                local level = child:Find('hero/Level'):GetComponent(UnityEngine.UI.Text)
                local icon = child:Find('hero/icon'):GetComponent(UnityEngine.UI.Image)
                local quality = child:Find('hero/Grade'):GetComponent(UnityEngine.UI.Image)
                local stars = child:Find('hero/Stars')
                local officer = defense_heros[idx]
                local cfg = officer.cfg
                icon:SetSprite(gui_helper:getOfficerIconPath(officer))
                quality:SetSprite(gui_helper:getNewQualityIcon(cfg.color))
                level.text = officer.level
                gui_helper:setOfficerStars(officer.star_level, stars)
                strength = strength + hero_manager:getHero(officer.hero_id).strength
            elseif idx <= unlock_num then
                frame_helper.showTrans(empty_trans)
                state = button_state.emtpy
            else
                frame_helper.showTrans(lock_trans)
                state = button_state.lock
            end

            local btn = child:GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(function()
                self:click_top_officer(idx, state)
            end)
        end

        local btn1 = self.transform:Find("content/hero/clearHeroBtn")
        local btn2 = self.transform:Find("content/hero/autoHeroBtn")
        frame_helper:setActive(btn1, hero_length ~= 0)
        frame_helper:setActive(btn2, hero_length == 0)

        local num_text = self:findText("content/top/num")
        num_text.text = hero_length .. "/" .. unlock_num

        local power_text = self:findText("content/top/power")
        power_text.text = strength

        local power_text = self:findText("content/hero/info/power")
        power_text.text = strength

        self:refreshProp()
    end,

    click_top_officer = function(self, idx, state)
        print('idx:', idx)
        if state == button_state.lock then
            hint:setText(NativeText.GetLangStr(590012))
        else
            frame_helper:setActive(self.heros_trans, true)
            frame_helper:setActive(self.soldiers_trans, false)
        end
    end,

    click_hideHeroBtn = function(self)
        frame_helper:setActive(self.heros_trans, false)
        frame_helper:setActive(self.soldiers_trans, true)

        local heros = {}
        for __, value in ipairs(self.cache_officers) do
            if value.select then
                local hero_id = value.officer.hero_id
                table.insert(heros, hero_id)
            end
        end
        network_sender:changeGarrison(heros)
    end,

    click_clearHeroBtn = function(self)
        for __, value in ipairs(self.cache_officers) do
            value.select = false
        end
        self:refresh()
    end,

    click_autoHeroBtn = function(self)
        local unlock_num = self:getUnlockNum()
        for __, value in ipairs(self.cache_officers) do
            if __ <= unlock_num then
                value.select = true
            end
        end
        self:refresh()
    end,

    click_buttom_officer = function(self, idx)
        local unlock_num = self:getUnlockNum()
        local data = self.cache_officers[idx]
        data.select = not data.select
        local hero_length = 0
        for __, value in ipairs(self.cache_officers) do
            hero_length = hero_length + (value.select and 1 or 0)
        end
        if hero_length > unlock_num then
            data.select = not data.select
            hint:setText(NativeText.GetLangStr(590011))
            return
        end
        self:refresh()
    end,

    getUnlockNum = function(self)
        local build_data = home_info:getBuildDataByCfgId(24)
        if build_data then
            local unlock_num = 0
            config.home_config_upgrades:foreach(function(id, cfg)
                if cfg.level <= build_data.level then
                    local prop = frame_helper:getPropertyDes(cfg.unlock)
                    for __, value in ipairs(prop) do
                        if tonumber(value.id) == 1289 then
                            local temp = tonumber(value.num_str)
                            if temp > unlock_num then
                                unlock_num = temp
                            end
                            break
                        end
                    end
                end
            end)
            return unlock_num
        end
        return 0
    end,

    getCfgByUnlockNum = function(self, unlock_num)
        local rst = nil
        config.home_config_upgrades:foreach(function(id, cfg)
            local prop = frame_helper:getPropertyDes(cfg.unlock)
            for __, value in ipairs(prop) do
                if tonumber(value.id) == 1289 then
                    local temp = tonumber(value.num_str)
                    if temp == unlock_num then
                        rst = cfg
                    end
                    break
                end
            end
        end)
        return rst
    end,

    refreshProp = function(self)
        local has_hero = false
        local heros = {}
        for __, value in ipairs(self.cache_officers) do
            if value.select then
                local hero_id = value.officer.hero_id
                heros[hero_id] = hero_id
                has_hero = true
            end
        end

        local defense_buff = {}
        local troops_buff = {}

        if has_hero then
            addToTable = function(value, tab, lang_id)
                if value > 0 then
                    local str = value * 0.01 .. "%"
                    table.insert(tab, string.format(NativeText.GetLangStr(lang_id), str))
                end
            end

            addToTable(player_cfg_helper:getDefenseATKPercent(heros), defense_buff, 590016)
            addToTable(player_cfg_helper:getDefenseDEFPercent(heros), defense_buff, 590017)

            addToTable(player_cfg_helper:getCorp1ATKPercent(heros), troops_buff, 590018)
            addToTable(player_cfg_helper:getCorp2ATKPercent(heros), troops_buff, 590019)
            addToTable(player_cfg_helper:getCorp3ATKPercent(heros), troops_buff, 590020)
            addToTable(player_cfg_helper:getCorp4ATKPercent(heros), troops_buff, 590021)

            addToTable(player_cfg_helper:getCorp1DEFPercent(heros), troops_buff, 590022)
            addToTable(player_cfg_helper:getCorp2DEFPercent(heros), troops_buff, 590023)
            addToTable(player_cfg_helper:getCorp3DEFPercent(heros), troops_buff, 590024)
            addToTable(player_cfg_helper:getCorp4DEFPercent(heros), troops_buff, 590025)

            -- local d1 = player_cfg_helper:getDefenseATKPercent(heros) * 0.01 .. "%"
            -- local d2 = player_cfg_helper:getDefenseDEFPercent(heros) * 0.01 .. "%"

            -- defense_buff = {
            --     string.format(NativeText.GetLangStr(590016), d1),
            --     string.format(NativeText.GetLangStr(590017), d2),
            -- }

            -- local t1 = player_cfg_helper:getCorp1ATKPercent(heros, false, true) * 0.01 .."%"
            -- local t2 = player_cfg_helper:getCorp2ATKPercent(heros, false, true) * 0.01 .."%"
            -- local t3 = player_cfg_helper:getCorp3ATKPercent(heros, false, true) * 0.01 .."%"
            -- local t4 = player_cfg_helper:getCorp4ATKPercent(heros, false, true) * 0.01 .."%"
            -- local t5 = player_cfg_helper:getCorp1DEFPercent(heros, false, true) * 0.01 .."%"
            -- local t6 = player_cfg_helper:getCorp2DEFPercent(heros, false, true) * 0.01 .."%"
            -- local t7 = player_cfg_helper:getCorp3DEFPercent(heros, false, true) * 0.01 .."%"
            -- local t8 = player_cfg_helper:getCorp4DEFPercent(heros, false, true) * 0.01 .."%"

            -- troops_buff = {
            --     -- string.format(NativeText.GetLangStr(590026), self.troops_power),
            --     string.format(NativeText.GetLangStr(590018), t1),
            --     string.format(NativeText.GetLangStr(590019), t2),
            --     string.format(NativeText.GetLangStr(590020), t3),
            --     string.format(NativeText.GetLangStr(590021), t4),
            --     string.format(NativeText.GetLangStr(590022), t5),
            --     string.format(NativeText.GetLangStr(590023), t6),
            --     string.format(NativeText.GetLangStr(590024), t7),
            --     string.format(NativeText.GetLangStr(590025), t8),
            -- }
        end

        local rst = #defense_buff + #troops_buff > 0
        local prop_buff = self.transform:Find("content/prop_buff")
        local prop_empty = self.transform:Find("content/prop_empty")
        frame_helper:setActive(prop_buff, rst)
        frame_helper:setActive(prop_empty, not rst)



        if rst then
            local idx = 0
            local path = "content/prop_buff/Scroll View/Viewport/Content/"
            frame_helper:hideAllChild(self, path, true)
            idx = idx + 1
            local child = frame_helper:getChild(self, idx, path)
            child:SetActive(true)
            local title = child:Find("bg/text1_Static[T3]_2"):GetComponent(UnityEngine.UI.Text)
            title.text = NativeText.GetLangStr(idx == 1 and 590004 or 590005)

            local none_trans = child:Find("none")
            none_trans:SetActive(#defense_buff <= 0)
            local path = path .. child.name .. "/layout"
            frame_helper:hideAllChild(self, path, true)
            for idx, value in ipairs(defense_buff) do
                local child = frame_helper:getChild(self, idx, path)
                child:SetActive(true)
                local text = child:GetComponent(UnityEngine.UI.Text)
                text.text = value
            end

            local path = "content/prop_buff/Scroll View/Viewport/Content/"
            idx = idx + 1
            local child = frame_helper:getChild(self, idx, path)
            child:SetActive(true)
            local title = child:Find("bg/text1_Static[T3]_2"):GetComponent(UnityEngine.UI.Text)
            title.text = NativeText.GetLangStr(idx == 1 and 590004 or 590005)
            local none_trans = child:Find("none")

            none_trans:SetActive(#troops_buff <= 0)
            local path = path .. child.name .. "/layout"
            frame_helper:hideAllChild(self, path, true)
            for idx, value in ipairs(troops_buff) do
                local child = frame_helper:getChild(self, idx, path)
                child:SetActive(true)
                local text = child:GetComponent(UnityEngine.UI.Text)
                text.text = value
            end

            



            -- local defense_path = "content/prop_buff/Scroll View/Viewport/Content/defense/layout"
            -- local troops_path = "content/prop_buff/Scroll View/Viewport/Content/troops/layout"

            -- local defense_trans = self.transform:Find("content/prop_buff/Scroll View/Viewport/Content/defense")
            -- local troops_trans = self.transform:Find("content/prop_buff/Scroll View/Viewport/Content/troops")

            -- frame_helper:hideAllChild(self, defense_path, true)
            -- frame_helper:hideAllChild(self, troops_path, true)

            -- for idx, value in ipairs(defense_buff) do
            --     local child = frame_helper:getChild(self, idx, defense_path)
            --     child:SetActive(true)
            --     local text = child:GetComponent(UnityEngine.UI.Text)
            --     text.text = value
            -- end

            -- frame_helper:setActive(defense_trans, #defense_buff > 0)
            -- local rect = defense_trans:GetComponent(UnityEngine.RectTransform)
            -- if #defense_buff > 0 then
            --     local offset = 122
            --     local layout_height = math.ceil((#defense_buff) / 2) * 68
            --     local total_height = layout_height +  offset
            --     rect.sizeDelta = Vector2(rect.sizeDelta.x, total_height)    
            -- else
            --     rect.sizeDelta = Vector2(rect.sizeDelta.x, 0)    
            -- end

            -- for idx, value in ipairs(troops_buff) do
            --     local child = frame_helper:getChild(self, idx, troops_path)
            --     child:SetActive(true)
            --     local text = child:GetComponent(UnityEngine.UI.Text)
            --     text.text = value
            -- end

            -- frame_helper:setActive(troops_trans, #troops_buff > 0)
            -- local rect = troops_trans:GetComponent(UnityEngine.RectTransform)
            -- if #troops_buff > 0 then
            --     local offset = 122
            --     local layout_height = math.ceil((#troops_buff) / 2) * 68
            --     local total_height = layout_height +  offset
            --     rect.sizeDelta = Vector2(rect.sizeDelta.x, total_height)    
            -- else
            --     rect.sizeDelta = Vector2(rect.sizeDelta.x, 0)    
            -- end

            -- local group = self.transform:Find("content/prop_buff/Scroll View/Viewport/Content"):GetComponent(UnityEngine.UI.VerticalLayoutGroup)
            -- group.enabled = false
            -- group.enabled = true
        end
    end,

    click_backBtn = function(self)
        frame_manager:hide('home_garrison')
    end,
}
