local super = require("ui.frame")

local select_enum = {
    min = 1,
    max = 2,
    all = 3,
}

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        self.is_defense = self.args.data.is_defense
        if self.is_defense then
            self.build_data = home_info:getBuildDataByCfgId(24)
        else
            self.build_data = home_info:getBuildDataByCfgId(7)
        end

        clickEvents = {
            "content/buttom/layout/maxBtn",
            "content/buttom/layout/allBtn",
            "content/sss/cureBtn",
            "content/sss/finishBtn",
            "content/backBtn",
        }

        frame_helper:regClickEvents("click", self, clickEvents)

        self:init()

        self.BUILD_INFO_CHANGE = home_info.events:on("BUILD_INFO_CHANGE", function(id) self:onBuildInfoChange(id) end)
        self.onPropChange = local_player.events:on("onPropChange", function() self:refreshCost() end)

        if self.is_defense then
            self.TRAP_CURE_TICK = home_info.events:on("TRAP_CURE_TICK", function(build_data, elapsed, total_time) self:onHospitalCureTick(build_data, elapsed, total_time) end)
        else
            self.HOSPITAL_CURE_TICK = home_info.events:on("HOSPITAL_CURE_TICK", function(build_data, elapsed, total_time) self:onHospitalCureTick(build_data, elapsed, total_time) end)
        end
    end,

    onUnload = function(self)
        if self.is_defense then
            home_info.events:off("TRAP_CURE_TICK", self.TRAP_CURE_TICK)
        else
            home_info.events:off("HOSPITAL_CURE_TICK", self.HOSPITAL_CURE_TICK)
        end
        home_info.events:off("BUILD_INFO_CHANGE", self.BUILD_INFO_CHANGE)
        local_player.events:off("onPropChange", self.onPropChange)
    end,

    init = function(self)
        self:refreshUI()
        self.select_enum = select_enum.max
        self:refreshToggle()

        local t1 = self:findText("content/common_bg/title")
        local t2 = self:findText("content/frame_bg/text1_Static[T3]_4")
        local t3 = self:findText("content/emtpy/Text")

        if self.is_defense then
            t1.text = NativeText.GetLangStr(999018)
            t2.text = NativeText.GetLangStr(999019)
            t3.text = NativeText.GetLangStr(999020)
        else
            t1.text = NativeText.GetLangStr(560001)
            t2.text = NativeText.GetLangStr(560002)
            t3.text = NativeText.GetLangStr(560013)
        end
    end,

    refreshUI = function(self)
        self.cure_soldiers = {}
        self.hurted_soldiers = {}
        self.cache_cure = {}
        self.cache_hurted = {}

        -- 医院
        if self.is_defense then
            local build_data = home_info:getBuildDataByCfgId(24)
            if build_data and build_data.hospital then
                self.cure_soldiers = build_data.guardsystem.repair
            end
        else
            local build_data = home_info:getBuildDataByCfgId(7)
            if build_data and build_data.hospital then
                self.cure_soldiers = build_data.hospital.soldiers
            end
        end

        if self.is_defense then
            -- 防御系统
            local build_data = home_info:getBuildDataByCfgId(24)
            if build_data and build_data.guardsystem then
                for __, s in ipairs(build_data.guardsystem.traps) do
                    if s.bHurt > 0 then
                        table.insert(self.hurted_soldiers, s)
                    end
                end
            end
        else
            -- 校场
            local build_data = home_info:getBuildDataByCfgId(10)
            if build_data and build_data.parade then
                for __, s in ipairs(build_data.parade.soldiers) do
                    if s.bHurt > 0 then
                        table.insert(self.hurted_soldiers, s)
                    end
                end
            end
        end

        local sortFunc = function(a, b)
            local level_a = config.soldier_soldiers[a.cfgId].level
            local level_b = config.soldier_soldiers[b.cfgId].level

            if level_a == level_b then
                return a.cfgId < b.cfgId
            else
                return level_a > level_b
            end
        end

        table.sort(self.cure_soldiers, sortFunc)
        table.sort(self.hurted_soldiers, sortFunc)

        local has_data = ( #self.cure_soldiers + #self.hurted_soldiers ) > 0
        local empty_trans = self.transform:Find("content/emtpy")
        local sv_trans = self.transform:Find("content/sss")
        frame_helper:setActive(empty_trans, not has_data)
        frame_helper:setActive(sv_trans, has_data)
        if has_data then
            self:refreshSrollView()
        end

        self:setCapacityInfo()
    end,

    onBuildInfoChange = function(self, build_id)
        local build_data = home_info:getBuildDataByCfgId(10)
        if self.build_data.id == build_id or build_data.id == build_id then
            self:refreshUI()
            self:refreshToggle()
        end
    end,

    onHospitalCureTick = function(self, build_data, elapsed, total_time)
        local pass_time = Time.realtimeSinceStartup - build_data.rmbTime
        if self.cache_cure then
            for __, c in ipairs(self.cache_cure) do
                local rate = 1
                local elapsed = 0
                if c.value.totaltime > 0 then
                    elapsed = c.value.reamintime - pass_time
                    elapsed = elapsed >= 0 and math.floor(elapsed) or 0
                    rate = 1 - elapsed / c.value.totaltime
                end
                c.slider.value = rate
                c.num_text.text = string.format(NativeText.GetLangStr(560009), math.floor(rate * c.value.count), c.value.count)
                c.time_text.text = frame_helper:timeFormat(elapsed)
                c.receive_btn.interactable = c.value.canRecv
                frame_helper:setActive(c.speed_up_btn.transform, elapsed > 0)
            end
        end
    end,

    click_speedUpBtn = function(self, index)
        local c = self.cache_cure[index]
    end,

    click_receiveBtn = function(self, index)
        local c = self.cache_cure[index]
    end,

    refreshSrollView = function(self)
        local path = "content/sss/Left Scroll View/Viewport/Content/"
        frame_helper:hideAllChild(self, path, true)
        for index, value in ipairs(self.cure_soldiers) do
            local child = frame_helper:getChild(self, index, path)
            child:SetActive(true)
            local cure_trans = child.transform:Find("cure")
            local nomal_trans = child.transform:Find("normal")
            frame_helper:setActive(cure_trans, true)
            frame_helper:setActive(nomal_trans, false)

            local name = child.transform:Find("name"):GetComponent(UnityEngine.UI.Text)
            local num = child.transform:Find("cure/num"):GetComponent(UnityEngine.UI.Text)
            local quality = child.transform:Find("quality"):GetComponent(UnityEngine.UI.Image)
            local icon = child.transform:Find("icon"):GetComponent(UnityEngine.UI.Image)

            local soldier_id = value.cfgId
            local soldier_cfg = config.soldier_soldiers[soldier_id]
            name.text = frame_helper:getLangCfg(soldier_cfg.soldiername)
            icon:SetSprite(soldier_cfg.headpath .. soldier_cfg.soldierhead)
            quality:SetSprite(gui_helper:getQualityIconPath(soldier_cfg.color))

            local slider = child.transform:Find("cure/slider"):GetComponent(UnityEngine.UI.Slider)
            local time = child.transform:Find("cure/time"):GetComponent(UnityEngine.UI.Text)

            local speed_up_btn = child.transform:Find("cure/speedUpBtn"):GetComponent(UnityEngine.UI.Button)
            speed_up_btn.onClick:RemoveAllListeners()
            speed_up_btn.onClick:AddListener(function ()
                print("speedUpBtn")
                self:click_speedUpBtn(index)
                self:showSpeedUpFrame(value.cfgId)
            end)

            local receive_btn = child.transform:Find("cure/receiveBtn"):GetComponent(UnityEngine.UI.Button)
            receive_btn.onClick:RemoveAllListeners()
            receive_btn.onClick:AddListener(function ()
                print("receiveBtn")
                self:click_receiveBtn(index)
                if self.is_defense then
                    network_sender:recvRepairedTrap(value.cfgId)
                else
                    network_sender:recvTreatedSoldier(value.cfgId)
                end
            end)

            table.insert(self.cache_cure, {
                value = value,
                slider = slider,
                num_text = num,
                time_text = time,
                speed_up_btn = speed_up_btn,
                receive_btn = receive_btn,
            })
        end

        -- 立即刷新
        local e = self.build_data
        local pass_time = Time.realtimeSinceStartup - e.rmbTime
        self:onHospitalCureTick(e, pass_time)

        for index, value in ipairs(self.hurted_soldiers) do
            local child = frame_helper:getChild(self, index + #self.cure_soldiers, path)
            child:SetActive(true)
            local cure_trans = child.transform:Find("cure")
            local nomal_trans = child.transform:Find("normal")
            frame_helper:setActive(cure_trans, false)
            frame_helper:setActive(nomal_trans, true)

            local name = child.transform:Find("name"):GetComponent(UnityEngine.UI.Text)
            local num = child.transform:Find("normal/num"):GetComponent(UnityEngine.UI.Text)
            local quality = child.transform:Find("quality"):GetComponent(UnityEngine.UI.Image)
            local icon = child.transform:Find("icon"):GetComponent(UnityEngine.UI.Image)
            local soldier_id = value.cfgId
            local soldier_cfg = config.soldier_soldiers[soldier_id]
            name.text = frame_helper:getLangCfg(soldier_cfg.soldiername)
            icon:SetSprite(soldier_cfg.headpath .. soldier_cfg.soldierhead)
            quality:SetSprite(gui_helper:getQualityIconPath(soldier_cfg.color))

            local time = child.transform:Find("normal/time"):GetComponent(UnityEngine.UI.Text)

            local btn = child.transform:Find("normal/addBtn"):GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(function ()
                self:addHurtedSoldier(index)
                print("addBtn")
            end)

            local btn = child.transform:Find("normal/reduceBtn"):GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(function ()
                self:reduceHurtedSoldier(index)
                print("reduceBtn")
            end)

            local btn = child.transform:Find("normal/deleteBtn"):GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(function ()
                print("deleteBtn")
                -- network_sender:deleteSoldier({
                --     {
                --         cfg_id = value.cfgId,
                --         count = value.bHurt, 
                --         state = 1,
                --     }
                -- })
                frame_manager:show('home_delete_soldier', nil, {
                    soldier_id = value.cfgId,
                    soldier_max_num = value.bHurt, 
                    state = 1,
                    be_hurted = true,
                })
            end)

            local slider = child.transform:Find("normal/slider"):GetComponent(UnityEngine.UI.Slider)
            slider.onValueChanged:RemoveAllListeners()
            slider.onValueChanged:AddListener(function()
                -- print('slider:', slider.value)
                -- self:onSliderValueChange(index)
            end)

            table.insert(self.cache_hurted, {
                index = index,
                value = value,
                slider = slider,
                time_text = time,
                num_text = num,
                select_num = 0,
                barrack_cfg = self:getBarrackCfg(value.cfgId),
            })
        end
    end,

    addSliderListener = function(self, slider, index)
        slider.onValueChanged:RemoveAllListeners()
        slider.onValueChanged:AddListener(function()
            self:onSliderValueChange(index)
        end)
    end,

    removeSliderListener = function(self, slider)
        slider.onValueChanged:RemoveAllListeners()
    end,

    addHurtedSoldier = function(self, index)
        local c = self.cache_hurted[index]
        if c.select_num < c.value.bHurt then
            c.select_num = c.select_num + 1
            self:refreshItemInfo(c, true)
        end
    end,

    reduceHurtedSoldier = function(self, index)
        local c = self.cache_hurted[index]
        if c.select_num > 1 then
            c.select_num = c.select_num - 1
            self:refreshItemInfo(c, true)
        end
    end,

    refreshItemInfo = function(self, c, want_refresh_cost)
        self:removeSliderListener(c.slider)
        local rate = c.select_num / c.value.bHurt
        local total_time = home_info:getSoldierCureTime(c.barrack_cfg, c.select_num)
        c.num_text.text = string.format("%s/%s", c.select_num, c.value.bHurt)
        c.time_text.text = frame_helper:timeFormat(total_time)
        c.slider.value = rate
        self:addSliderListener(c.slider, c.index)

        if want_refresh_cost then self:refreshCost() end
    end,

    onSliderValueChange = function(self, index)
        print("onSliderValueChange:", index)
        local c = self.cache_hurted[index]
        local temp = math.ceil(c.slider.value * c.value.bHurt)
        if temp ~= c.select_num then
            c.select_num = temp
            self:refreshItemInfo(c, true)
        end
    end,

    getBarrackCfg = function(self, soldier_id)
        local rst = nil
        config.home_config_barracks:foreach(function(id, cfg)
            if cfg.soldier_id == soldier_id then
                rst = cfg
            end
        end)
        return rst
    end,

    makeSelectNumMax = function(self)
        -- 收集用到的资源id
        local used_res = {}
        for __, c in ipairs(self.cache_hurted) do
            local cost_table = loadstring("return " .. c.barrack_cfg.heal_cost)()
            for item_id, need_num in pairs(cost_table) do
                used_res[item_id] = frame_helper:getItemNum(item_id)
            end
        end
        -- 合理分配资源
        for __, c in ipairs(self.cache_hurted) do
            print("####:", c.barrack_cfg, used_res, c.value.bHurt)
            c.select_num = home_info:getSoldierCureNumInLimitRes(c.barrack_cfg, used_res, c.value.bHurt)
            self:removeSliderListener(c.slider)
            local rate = c.select_num / c.value.bHurt
            c.slider.value = rate
            c.num_text.text = string.format("%s/%s", c.select_num, c.value.bHurt)
            local total_time = home_info:getSoldierCureTime(c.barrack_cfg, c.select_num)
            c.time_text.text = frame_helper:timeFormat(total_time)
            self:addSliderListener(c.slider, index)
        end
    end,

    refreshCost = function(self)
        self.lack_tab = {}

        local used_res = {}
        local real_total_time = 0
        local expect_total_time = 0
        for __, c in ipairs(self.cache_hurted) do
            for item_id, item_num in pairs(home_info:getSoldierCureCost(c.barrack_cfg, c.select_num)) do
                used_res[item_id] = used_res[item_id] or 0
                used_res[item_id] = used_res[item_id] + item_num
            end
            -- 时间兑换成钻石
            local cure_total_time = home_info:getSoldierCureTime(c.barrack_cfg, c.select_num)
            if cure_total_time > real_total_time then
                real_total_time = cure_total_time
                expect_total_time = c.barrack_cfg.heal_time * c.select_num
            end
        end

        local res_cost_diamond = 0
        local path = "content/sss/Right Scroll View/Viewport/Content/"
        frame_helper:hideAllChild(self, path, true)
        local index = 1
        for item_id, item_num in pairs(used_res) do
            local child = frame_helper:getChild(self, index, path)
            child:SetActive(true)
            local icon = child.transform:Find("icon"):GetComponent(UnityEngine.UI.Image)
            local bg = child.transform:Find("bg"):GetComponent(UnityEngine.UI.Image)
            local des = child.transform:Find("text1_Static[T5]_9"):GetComponent(UnityEngine.UI.Text)
            local right_icon = child.transform:Find("rightIcon"):GetComponent(UnityEngine.UI.Image)
            local wrongIcon = child.transform:Find("wrongIcon"):GetComponent(UnityEngine.UI.Image)
            local add_btn = child.transform:Find("addBtn"):GetComponent(UnityEngine.UI.Button)

            local str ,full_condition = frame_helper:getItemNumFormat(item_id, item_num)
            icon:SetSprite(frame_helper:getItemIcon(item_id))
            bg.enabled = not full_condition
            des.text = str
            right_icon.enabled = full_condition
            wrongIcon.enabled = not full_condition
            add_btn.transform.localScale = full_condition and Vector3.zero or Vector3.one

            -- 缺少的资源兑换成钻石
            local own_num = frame_helper:getItemNum(item_id)
            local lack_num = item_num - own_num
            if lack_num > 0 then
                res_cost_diamond = res_cost_diamond + frame_helper:getResToDiamondNum(item_id, lack_num)
            end

            if not full_condition then
                self.lack_tab[item_id] = item_num
            end

            local btn = child.transform:Find("addBtn"):GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(function()
                if not full_condition then
                    frame_manager:show('resource_exchange', nil, {
                        item_id = item_id,
                        item_num = item_num,
                    })
                end
            end)
            index = index + 1
        end

        local speed_up_type = home_speed_up_type.soldier_repair
        local time_cost_diamond = gui_helper:getSpeedUpCost(speed_up_type, real_total_time)
        local total_diamond = res_cost_diamond + time_cost_diamond

        print("资源兑换成钻石总量：", res_cost_diamond)
        print("时间兑换成钻石总量：", time_cost_diamond)

        local DIAMOND    = 30001 -- 钻石
        local item_name = gui_helper:getImageTextItemStr(DIAMOND)
        local image_text = self:findText("content/sss/finishBtn/image_text")
        image_text:SetImageText(string.format("%s %s", item_name, total_diamond))
        self.cur_cost_diamond = total_diamond

        local unreal_upgrade_time = self:findText("content/sss/cureBtn/unreal")
        local real_upgrade_time = self:findText("content/sss/cureBtn/real")
        unreal_upgrade_time.text = string.format(NativeText.GetLangStr(290017), gui_helper:timeFormat(expect_total_time)) 
        real_upgrade_time.text = gui_helper:timeFormat(real_total_time)
    end,

    refreshToggle = function(self)
        local max_image = self:findImage("content/buttom/layout/maxBtn/ok")
        local all_image = self:findImage("content/buttom/layout/allBtn/ok")

        if self.select_enum == select_enum.min then
            max_image.enabled = false
            all_image.enabled = false
            for __, c in ipairs(self.cache_hurted) do
                c.select_num = 0
            end
        elseif self.select_enum == select_enum.max then
            max_image.enabled = true
            all_image.enabled = false
            self:makeSelectNumMax()
        elseif self.select_enum == select_enum.all then
            max_image.enabled = false
            all_image.enabled = true
            for __, c in ipairs(self.cache_hurted) do
                c.select_num = c.value.bHurt
            end
        end

        for __, c in ipairs(self.cache_hurted) do
            self:refreshItemInfo(c)
        end
        self:refreshCost()
    end,

    setCapacityInfo = function(self)
        local cur_num = 0
        for __, value in ipairs(self.cure_soldiers) do
            cur_num = cur_num + value.count
        end
        for __, value in ipairs(self.hurted_soldiers) do
            cur_num = cur_num + value.bHurt
        end

        local capacity = self:findText("content/buttom/num")
        capacity.text = string.format(NativeText.GetLangStr(560014), frame_helper:itemNumFormat(cur_num) .. "/" .. frame_helper:itemNumFormat(home_info:getHospitalCapacity()))
    end,

    click_maxBtn = function(self)
        if self.select_enum == select_enum.max then
            self.select_enum = select_enum.min
        else
            self.select_enum = select_enum.max
        end
        self:refreshToggle()
    end,

    click_allBtn = function(self)
        if self.select_enum == select_enum.all then
            self.select_enum = select_enum.min
        else
            self.select_enum = select_enum.all
        end
        self:refreshToggle()
    end,

    click_cureBtn = function(self)
        -- 面板缺少的资源，不是实际资源
        if self.lack_tab then
            for item_id, need_num in pairs(self.lack_tab) do
                frame_helper:showCommonBox(NativeText.GetLangStr(290022), function()
                    frame_manager:show('resource_exchange', nil, {
                        item_id = item_id,
                        item_num = need_num,
                    })
                end)
                return  
            end
        end

        local list = {}
        for __, c in ipairs(self.cache_hurted) do
            if c.select_num > 0 then
                table.insert(list, {
                    soldierId = c.value.cfgId,
                    soldierCount = c.select_num,
                })
            end
        end
        if #list > 0 then
            if self.is_defense then
                network_sender:repairTrap(list, 0)
            else
                network_sender:treatSoldier(list, 0)
            end
        else
            hint:setText(NativeText.GetLangStr(560015))
        end
    end,

    click_finishBtn = function(self)
        if frame_helper:isEnoughDiamond(self.cur_cost_diamond) then
            print("发送服务器消息 治疗士兵")
            local list = {}
            for __, c in ipairs(self.cache_hurted) do
                if c.select_num > 0 then
                    table.insert(list, {
                        soldierId = c.value.cfgId,
                        soldierCount = c.select_num,
                    })
                end
            end
            if #list > 0 then
                if self.is_defense then
                    network_sender:repairTrap(list, 1)
                else
                    network_sender:treatSoldier(list, 1)
                end
            else
                hint:setText(NativeText.GetLangStr(560015))
            end
        end
    end,

    click_backBtn = function(self)
        frame_manager:hide('home_hospital')
    end,

    showSpeedUpFrame = function(self, soldier_id)
        if self.is_defense then
            frame_manager:show("home_speed_up", nil, {
                speed_up_type = home_speed_up_type.trap_repair,
                id = self.build_data.id,
                soldier_id = soldier_id,
            })
        else
            frame_manager:show("home_speed_up", nil, {
                speed_up_type = home_speed_up_type.soldier_repair,
                id = self.build_data.id,
                soldier_id = soldier_id,
            })
        end
    end,
}
