local super = require("ui.frame")
local log = logger.get("Game", "UI")
local entity_cfg_id = 16
local notice_type = {
    prison_break = 1,
    release = 2,
    exile = 3,
}

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        -- SCGetCellInfo
        self.msg = self.args.data.msg
        self:checkCommandName(self.msg)

        clickEvents = 
        {
            "content/main/bg/top/ButtonBack",
            "content/main/bg/top/homeBtn",
            "content/main/ExileBtn",
            "content/main/ReleaseBtn",
            "content/main/RecruitBtn",
            "content/notice/closeNoticeBtn",
            "content/condition/closeContitionBtn",
            "content/recruitment/closeRecruitmentBtn",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self:init()
        self:detectArgs(self.args)
    end,

    checkCommandName = function(self, msg)
        for __, p in ipairs(msg.prisonerInfos) do
            if p.officerName and string.len(p.officerName) > 0 then
            else
                local cfg = config.commander_basics[p.officerId]
                p.officerName = frame_helper:getLangCfg(cfg.commander_name)
            end
        end
        for __, p in ipairs(msg.escapeeList) do
            if p.officerName and string.len(p.officerName) > 0 then
            else
                local cfg = config.commander_basics[p.officerId]
                p.officerName = frame_helper:getLangCfg(cfg.commander_name)
            end
        end
    end,

    detectArgs = function(self, args)
        local recruit_hero = args.data.recruit_hero
        if recruit_hero then
            local recruit_hero_id = recruit_hero.id
            local data = nil
            for __, p in ipairs(self.msg.prisonerInfos) do
                if p.instId == recruit_hero_id then
                    data = p
                    break
                end
            end
            if data then
                local child = nil
                for __, c in ipairs(self.all_child) do
                    if c.info == data then
                        child = c.tsf
                        break
                    end
                end
                if child then
                    self:click_selectHero(data, child)
                    self:click_RecruitBtn()
                end
            end
        end
    end,

    onUnload = function (self)
    end,

    init = function(self)
        self.base_trans = self.transform:Find("content/main/base_trans")
        self.barrack_trans = self.transform:Find("content/main/barrack_trans")
        self.notice_trans = self.transform:Find("content/notice")
        self.recruitment_trans = self.transform:Find("content/recruitment")
        self.condition_trans = self.transform:Find("content/condition")
        self.helper.hideTrans(self.base_trans)
        self.helper.hideTrans(self.barrack_trans)
        self.helper.hideTrans(self.notice_trans)
        self.helper.hideTrans(self.recruitment_trans)
        self.helper.hideTrans(self.condition_trans)

        self.exile_btn = self.transform:Find("content/main/ExileBtn"):GetComponent(UnityEngine.UI.Button)
        self.release_btn = self.transform:Find("content/main/ReleaseBtn"):GetComponent(UnityEngine.UI.Button)
        self.recruit_btn = self.transform:Find("content/main/RecruitBtn"):GetComponent(UnityEngine.UI.Button)
        
        self:initBar()
        self:refresh(self.msg)
    end,

    -- repeated MPrisonerInfo prisonerInfos = 2;
	-- repeated MPrisonerInfo escapeeList = 3;
    refresh = function(self, msg)
        self.msg = msg
        self:cookMsg(msg)
        self:syncUI(msg)
    end,

    cookMsg = function(self, msg)
        sortFunc = function(a, b)
            return config.commander_basics[a.officerId].color > config.commander_basics[b.officerId].color
        end

        table.sort(msg.prisonerInfos, sortFunc)
        table.sort(msg.escapeeList, sortFunc)

        self.base_list = {}
        for __, p in ipairs(msg.prisonerInfos) do
            if p.prisonerType == common_pb.EPrisoner_Monster then
                -- print("common_pb.EPrisoner_Monster")
                table.insert(self.base_list, p)
            end
        end

        self.barrack_list = {}
        for __, p in ipairs(msg.prisonerInfos) do
            if p.prisonerType == common_pb.EPrisoner_Player then
                -- print("common_pb.EPrisoner_Player")
                if not self.barrack_list[p.campName] then
                    self.barrack_list[p.campName] = {}
                end
                table.insert(self.barrack_list[p.campName], p)
            end
        end
    end,

    syncUI = function(self, msg)
        self:clearChild()
        self:syncText(msg)
        self:showNoticeContent(msg.escapeeList, notice_type.prison_break)
        self:syncBaseContent(self.base_list)
        self:syncBarrackContent(self.barrack_list)
        self:clearSelectHero()
    end,

    syncText = function(self, msg)
        local production_text = self:findText("content/main/productionText")
        local prisoners_text = self:findText("content/main/prisonersText")
        local executed_text = self:findText("content/main/executedText")

        local rate = 0.0
        -- 浮点型属性要除 10000
        local function convertPropToFloat(props)
            local str = "\r\n"
            for _, prop in ipairs(props) do
                if config.player_config_attrs[prop.PropId] then
                    if config.player_config_attrs[prop.PropId].isfloat > 0 then
                        prop.PropValue = prop.PropValue * 0.0001
                    end
                    rate = prop.PropValue
                    str = str .. ('prop:id'..prop.PropId..' value:'..prop.PropValue).."\r\n"
                else
                    log.error("attribute_config_attrs 不存在属性id!", prop.PropId)     
                end
            end
            log.info(str)
        end

        convertPropToFloat(msg.produce_rate_add)

        rate = rate * 100 .. "%"
        production_text.text = string.format(NativeText.GetLangStr(380003), rate)
        local str_num = ""
        if #msg.prisonerInfos < msg.max_prison_num then
            str_num = "<color=#ffffffff>" .. #msg.prisonerInfos .. "/" .. msg.max_prison_num .. "</color>"
        else
            str_num = "<color=#ff0000ff>" .. #msg.prisonerInfos .. "/" .. msg.max_prison_num .. "</color>"
        end
        prisoners_text.text = string.format(NativeText.GetLangStr(380002), str_num)

        local prison_cfg = self:getPrisonCfg()
        local exile_unlock_lv = self:getExileUnlockLevel()
        if prison_cfg.level >= exile_unlock_lv then
            executed_text.text = string.format(NativeText.GetLangStr(380005), msg.remain_exile_chance .. "/" .. msg.max_exile_chance)
        else
            executed_text.text = string.format(NativeText.GetLangStr(380025), exile_unlock_lv)
        end
    end,

    syncBaseContent = function(self, list)
        local path = "content/main/base_trans/Scroll View/Viewport/Content"
        self.helper:hideAllChild(self, path)
        local has_prisoner = false
        for __, p in ipairs(list) do
            has_prisoner = true
            -- print('icon:', p.iconId, 'qulity:', p.quality, 'starlv', p.starlv, "playerName:", p.playerName, "remainTime", p.remainTime, "campName", p.campName
            --         ,"escapeCount", p.escapeCount, "officerName", p.officerName, "officerId", p.officerId, "needbattle", p.needbattle, "tankid", p.tankid)
            local child = self.helper:getChild(self, __, path)
            child:SetActive(true)
            local icon = child:Find("top_bg/icon"):GetComponent(UnityEngine.UI.Image)
            local quality = child:Find("top_bg/quality"):GetComponent(UnityEngine.UI.Image)
            local tank_type = child:Find("top_bg/tank_type"):GetComponent(UnityEngine.UI.Image)
            local name_text = child:Find("buttom_bg/name"):GetComponent(UnityEngine.UI.Text)
            local owner_text = child:Find("buttom_bg/owner"):GetComponent(UnityEngine.UI.Text)
            local icon_cfg = config.commander_ress[p.iconId]
            icon:SetSprite(icon_cfg.path .. icon_cfg.icon)
            quality:SetSprite(gui_helper:getQualityIconPath(config.commander_basics[p.officerId].color))
            tank_type:SetSprite(gui_helper:getHeroTypeIconPath(p.tankid))
            name_text.text = p.officerName
            owner_text.text = p.needbattle == 1 and NativeText.GetLangStr(380023) or NativeText.GetLangStr(380010)
            local star_path = "top_bg/star layout"
            local star_layout_trans = child.transform:Find(star_path)
            for i=0, star_layout_trans.childCount - 1 do
                star_layout_trans:GetChild(i):SetActive(i < p.starlv)
            end
            local btn = child:Find("top_bg"):GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(
                function ()
                    self:click_selectHero(p, child)
                end)

            self:addChild(p, child)
        end
        if has_prisoner then
            self.helper.hideTrans(self.transform:Find("content/main/base_trans/none"))
        else
            self.helper.showTrans(self.transform:Find("content/main/base_trans/none"))
        end
    end,

    syncBarrackContent = function(self, list)
        local path = "content/main/barrack_trans/Scroll View/Viewport/Content/"
        self.helper:hideAllChild(self, path)
        local index = 1
        local has_prisoner = false
        for camp_name, ps in pairs(list) do
            local tplt = self.helper:getChild(self, index, path)
            tplt:SetActive(true)
            local camp_text = tplt:Find("day_text"):GetComponent(UnityEngine.UI.Text)
            camp_text.text = string.format(NativeText.GetLangStr(380028), camp_name)
            index = index + 1
            local layout_path = path .. tplt.transform.name .. "/layout"
            self.helper:hideAllChild(self, layout_path)
            for __, p in ipairs(ps) do
                has_prisoner = true
                local child = self.helper:getChild(self, __, layout_path)
                child:SetActive(true)
                local icon = child:Find("top_bg/icon"):GetComponent(UnityEngine.UI.Image)
                local quality = child:Find("top_bg/quality"):GetComponent(UnityEngine.UI.Image)
                local tank_type = child:Find("top_bg/tank_type"):GetComponent(UnityEngine.UI.Image)
                local name_text = child:Find("buttom_bg/name"):GetComponent(UnityEngine.UI.Text)
                local owner_text = child:Find("buttom_bg/owner"):GetComponent(UnityEngine.UI.Text)
                local icon_cfg = config.commander_ress[p.iconId]
                icon:SetSprite(icon_cfg.path .. icon_cfg.icon)
                quality:SetSprite(gui_helper:getQualityIconPath(config.commander_basics[p.officerId].color))
                tank_type:SetSprite(gui_helper:getHeroTypeIconPath(p.tankid))
                name_text.text = p.officerName
                owner_text.text = p.playerName
                local star_path = "top_bg/star layout"
                local star_layout_trans = child.transform:Find(star_path)
                for i=0, star_layout_trans.childCount - 1 do
                    star_layout_trans:GetChild(i):SetActive(i < p.starlv)
                end
                local btn = child:Find("top_bg"):GetComponent(UnityEngine.UI.Button)
                btn.onClick:RemoveAllListeners()
                btn.onClick:AddListener(
                    function ()
                        self:click_selectHero(p, child)
                    end)
                self:addChild(p, child)
            end
        end
        if has_prisoner then
            self.helper.hideTrans(self.transform:Find("content/main/barrack_trans/none"))
        else
            self.helper.showTrans(self.transform:Find("content/main/barrack_trans/none"))
        end
    end,

    showNoticeContent = function(self, list, nt, confirmCallback)
        if #list == 0 then return end

        local cost_trans = self.transform:Find("content/notice/content/cost")
        local cancel_trans = self.transform:Find("content/notice/content/cancelNoticeBtn")
        cost_trans:SetActive(nt == notice_type.exile)
        cancel_trans:SetActive(nt ~= notice_type.prison_break)

        local des_text = self:findText("content/notice/content/desText")

        if nt == notice_type.prison_break then -- 越狱
            des_text.text = NativeText.GetLangStr(380012)
        elseif nt == notice_type.release then -- 释放
            des_text.text = NativeText.GetLangStr(380015)
        elseif nt == notice_type.exile then -- 流放
            des_text.text = NativeText.GetLangStr(380016)
        end

        local cost_cfg_id = nil
        local cost_total_num = nil

        self.helper.showTrans(self.notice_trans)
        local path = "content/notice/content/Scroll View/Viewport/Content"
        self.helper:hideAllChild(self, path)
        for __, p in ipairs(list) do
            local child = self.helper:getChild(self, __, path)
            child:SetActive(true)
            local icon = child:Find("top_bg/icon"):GetComponent(UnityEngine.UI.Image)
            local quality = child:Find("top_bg/quality"):GetComponent(UnityEngine.UI.Image)
            local tank_type = child:Find("top_bg/tank_type"):GetComponent(UnityEngine.UI.Image)
            local name_text = child:Find("buttom_bg/name"):GetComponent(UnityEngine.UI.Text)
            local owner_text = child:Find("buttom_bg/owner"):GetComponent(UnityEngine.UI.Text)
            local icon_cfg = config.commander_ress[p.iconId]
            icon:SetSprite(icon_cfg.path .. icon_cfg.icon)
            quality:SetSprite(gui_helper:getQualityIconPath(config.commander_basics[p.officerId].color))
            tank_type:SetSprite(gui_helper:getHeroTypeIconPath(p.tankid))
            name_text.text = p.officerName
            if p.prisonerType == common_pb.EPrisoner_Monster then
                owner_text.text = p.needbattle == 1 and NativeText.GetLangStr(380023) or NativeText.GetLangStr(380010)
            elseif p.prisonerType == common_pb.EPrisoner_Player then
                owner_text.text = p.playerName
            end
            local star_path = "top_bg/star layout"
            local star_layout_trans = child.transform:Find(star_path)
            for i=0, star_layout_trans.childCount - 1 do
                star_layout_trans:GetChild(i):SetActive(i < p.starlv)
            end

            if nt == notice_type.exile then -- 流放
                for k,v in ipairs(p.exile_items) do
                    cost_cfg_id = v.ItemTemId
                    cost_total_num = cost_total_num or 0
                    cost_total_num = cost_total_num + v.Count
                end
            end
        end

        if nt == notice_type.exile then
            if cost_cfg_id and cost_total_num then
                local cost_icon = self:findImage("content/notice/content/cost/costIcon")
                local cost_text = self:findText("content/notice/content/cost/costText")
                local num_text = self:findText("content/notice/content/cost/numText")
                local btn = self:findButton("content/notice/content/cost/addBtn")
                cost_icon:SetSprite(self.helper:getItemIcon(cost_cfg_id))
                cost_text.text = self.helper:getItemNumFormat(cost_cfg_id, cost_total_num)
                num_text.text = string.format(NativeText.GetLangStr(380005), self.msg.remain_exile_chance)
                btn.onClick:RemoveAllListeners()
                btn.onClick:AddListener(
                    function ()
                        print("add item event!")
                    end)
            else
                cost_trans:SetActive(false)
            end
        end

        local confirm_btn = self.transform:Find("content/notice/content/confirmNoticeBtn"):GetComponent(UnityEngine.UI.Button)
        confirm_btn.onClick:RemoveAllListeners()
        confirm_btn.onClick:AddListener(
            function ()
                self.helper.hideTrans(self.notice_trans)   
                if confirmCallback then confirmCallback() end             
            end)

        local cancel_btn = self.transform:Find("content/notice/content/cancelNoticeBtn"):GetComponent(UnityEngine.UI.Button)
        cancel_btn.onClick:RemoveAllListeners()
        cancel_btn.onClick:AddListener(
            function ()
                self.helper.hideTrans(self.notice_trans)   
            end)
    end,

    showConditionContent = function(self, list)
        self.helper.showTrans(self.condition_trans)
        local p = list[1]
        local big_icon = self.transform:Find("content/condition/content/big_icon"):GetComponent(UnityEngine.UI.Image)
        local icon_cfg = config.commander_ress[p.iconId]
        big_icon:SetSprite(icon_cfg.cmd_path .. icon_cfg.commander_pic)

        local confirm_btn = self.transform:Find("content/condition/content/acceptConditionBtn"):GetComponent(UnityEngine.UI.Button)
        confirm_btn.onClick:RemoveAllListeners()
        confirm_btn.onClick:AddListener(
            function ()
                -- self.helper.hideTrans(self.condition_trans)   
                -- frame_manager:show('recruit_formation', nil, p.instId)
                self:showTeamFromation(p.instId)
            end)

        local cancel_btn = self.transform:Find("content/condition/content/cancelConditionBtn"):GetComponent(UnityEngine.UI.Button)
        cancel_btn.onClick:RemoveAllListeners()
        cancel_btn.onClick:AddListener(
            function ()
                self.helper.hideTrans(self.condition_trans)   
            end)
    end,

    showRecruitmentContent = function(self, list)
        self.helper.showTrans(self.recruitment_trans)
        local p = list[1]
        local icon = self.transform:Find("content/recruitment/content/hero/top_bg/icon"):GetComponent(UnityEngine.UI.Image)
        local quality = self.transform:Find("content/recruitment/content/hero/top_bg/quality"):GetComponent(UnityEngine.UI.Image)
        local tank_type = self.transform:Find("content/recruitment/content/hero/top_bg/tank_type"):GetComponent(UnityEngine.UI.Image)
        local name_text = self.transform:Find("content/recruitment/content/hero/buttom_bg/name"):GetComponent(UnityEngine.UI.Text)
        local des_text = self.transform:Find("content/recruitment/content/desText"):GetComponent(UnityEngine.UI.Text)
        local cost_icon = self:findImage("content/recruitment/content/costIcon")
        local cost_text = self:findText("content/recruitment/content/costText")
        local btn = self:findButton("content/recruitment/content/addBtn")
        local icon_cfg = config.commander_ress[p.iconId]
        icon:SetSprite(icon_cfg.path .. icon_cfg.icon)
        quality:SetSprite(gui_helper:getQualityIconPath(config.commander_basics[p.officerId].color))
        tank_type:SetSprite(gui_helper:getHeroTypeIconPath(p.tankid))
        name_text.text = p.officerName
        des_text.text = NativeText.GetLangStr(380017)
        local cost_cfg_id = nil
        local cost_total_num = nil
        for k,v in ipairs(p.summon_items) do
            cost_cfg_id = v.ItemTemId
            cost_total_num = cost_total_num or 0
            cost_total_num = cost_total_num + v.Count
        end
        if cost_cfg_id and cost_total_num then
            cost_icon:SetSprite(self.helper:getItemIcon(cost_cfg_id))
            cost_text.text = self.helper:getItemNumFormat(cost_cfg_id, cost_total_num)
        end

        local star_path = "content/recruitment/content/hero/top_bg/star layout"
        local star_layout_trans = self.transform:Find(star_path)
        for i=0, star_layout_trans.childCount - 1 do
            star_layout_trans:GetChild(i):SetActive(i < p.starlv)
        end

        btn.onClick:RemoveAllListeners()
        btn.onClick:AddListener(
            function ()
                print(" showRecruitmentContent add item event!")
            end)

        local confirm_btn = self.transform:Find("content/recruitment/content/confirmBtn"):GetComponent(UnityEngine.UI.Button)
        confirm_btn.onClick:RemoveAllListeners()
        confirm_btn.onClick:AddListener(
            function ()
                network_sender:summonPrisoner({p.instId})
                self.helper.hideTrans(self.recruitment_trans)   
            end)

        local cancel_btn = self.transform:Find("content/recruitment/content/cancelBtn"):GetComponent(UnityEngine.UI.Button)
        cancel_btn.onClick:RemoveAllListeners()
        cancel_btn.onClick:AddListener(
            function ()
                self.helper.hideTrans(self.recruitment_trans)   
            end)
    end,

    clearChild = function(self)
        self.all_child = nil
    end,

    addChild = function(self, MPrisonerInfo, child)
        self.all_child = self.all_child or {}
        table.insert(self.all_child, { info = MPrisonerInfo, tsf = child})
    end,

    clearSelectHero = function(self)
        if self.all_child then
            for __, c in ipairs(self.all_child) do
                c.tsf:Find("top_bg/Formation_panel_03"):GetComponent(UnityEngine.UI.Image).enabled =false
                c.tsf:Find("top_bg/Formation_panel_03_1"):GetComponent(UnityEngine.UI.Image).enabled =false
            end
        end
        self.select_info = nil
    end,

    click_selectHero = function(self, MPrisonerInfo, child)
        self.select_info = self.select_info or {}
        local remove = false
        for pos, h in ipairs(self.select_info) do
            if h == MPrisonerInfo then
                remove = true
                table.remove(self.select_info, pos)
                break
            end
        end
        if not remove then
            table.insert(self.select_info, MPrisonerInfo)
        end

        for __, c in ipairs(self.all_child) do
            local vsb = false
            for pos, h in ipairs(self.select_info) do
                if h == c.info then
                    vsb = true
                    break
                end
            end
            local child = c.tsf
            local bg = child:Find("top_bg/Formation_panel_03"):GetComponent(UnityEngine.UI.Image)
            local toggle = child:Find("top_bg/Formation_panel_03_1"):GetComponent(UnityEngine.UI.Image)
            bg.enabled = #self.select_info > 0
            toggle.enabled = vsb
        end
    end,

    initBar = function(self)
        for i = 1, 2 do 
            local path = "content/main/bar/Right (" .. i .. ")"
            self:setScript("click", path, function() self:click_barBtn(i) end)
        end
        self.bar_index = 1
        self:click_barBtn(self.bar_index)
    end,

    click_homeBtn = function(self)
        frame_manager:hide('home_prison')
    end,

    click_ButtonBack = function(self)
        frame_manager:hide('home_prison')
    end,

    click_barBtn = function(self, idx)
        local prison_cfg = self:getPrisonCfg()
        -- 军营未解锁
        if idx == 2 then
            local barrack_unlock_lv = self:getBarrackUnlockLevel()
            if prison_cfg.level < barrack_unlock_lv then
                hint:setText(string.format(NativeText.GetLangStr(380024), barrack_unlock_lv))
                return
            end
        end

        self.bar_index = idx
        for i = 1, 2 do
            local button = self:findButton("content/main/bar/Right (" .. i .. ")")
            button.interactable = i ~= idx
            local text = self:findText("content/main/bar/Right (" .. i .. ")/Text")
            text.color = i == idx and Color.black or Color.white
        end

        if self.bar_index == 1 then
            self.helper.showTrans(self.base_trans)
            self.helper.hideTrans(self.barrack_trans)
            
        elseif self.bar_index == 2 then
            self.helper.hideTrans(self.base_trans)
            self.helper.showTrans(self.barrack_trans)
        end

        self.exile_btn.interactable = self.bar_index == 2 and prison_cfg.level >= self:getExileUnlockLevel()
        self.recruit_btn.interactable = self.bar_index == 1

        self:clearSelectHero()
    end,

    -- 军营监狱解锁等级
    getBarrackUnlockLevel = function(self)
        local unlock_level = nil
        local cfg_list = gui_helper:sortCfgById(config.home_config_prison_upgardes)
        for __, v in ipairs(cfg_list) do
            if v.prison_time > 0 then
                unlock_level = v.level
                break
            end
        end
        return unlock_level
    end,

    -- 流放解锁等级
    getExileUnlockLevel = function(self)
        local unlock_level = nil
        local cfg_list = gui_helper:sortCfgById(config.home_config_prison_upgardes)
        for __, v in ipairs(cfg_list) do
            if v.exile_num > 0 then
                unlock_level = v.level
                break
            end
        end
        return unlock_level
    end,

    -- 当前监狱配置
    getPrisonCfg = function(self)
        local entity_data = home_info:getBuildDataByCfgId(entity_cfg_id)
        local level = entity_data.level
        local rst = nil
        config.home_config_prison_upgardes:foreach(
            function(k, v)
                if v.level == level then
                    rst = v
                end
            end)
        return rst
    end,

    click_confirmNoticeBtn = function(self)
        self.helper.hideTrans(self.notice_trans)
    end,

    click_cancelNoticeBtn = function(self)
        self.helper.hideTrans(self.notice_trans)
    end,

    click_ExileBtn = function(self)
        local excute = function()
            local list = {}
            for __, p in ipairs(self.select_info) do
                table.insert(list, p.instId)
            end
            network_sender:exilePrisoner(list, common_pb.EPrisoner_Player)
        end
        if self.select_info and #self.select_info > 0 then
            for __, p in ipairs(self.select_info) do
                if p.prisonerType ~= common_pb.EPrisoner_Player then
                    return
                end
            end
            self:showNoticeContent(self.select_info, notice_type.exile, excute)
        else
            hint:setText(NativeText.GetLangStr(380027))
        end
    end,

    click_ReleaseBtn = function(self)
        local excute = function()
            local list = {}
            for __, p in ipairs(self.select_info) do
                table.insert(list, p.instId)
            end
            network_sender:releasePrisoner(list, common_pb.EPrisoner_Monster)
        end
        if self.select_info and #self.select_info > 0 then
            self:showNoticeContent(self.select_info, notice_type.release, excute)
        else
            hint:setText(NativeText.GetLangStr(380027))
        end
    end,

    click_RecruitBtn = function(self)
        if hero_manager:heroAmountMax() then
            hint:setText(NativeText.GetLangStr(900006))
            return
        end

        if self.select_info and #self.select_info == 1 and self.select_info[1].prisonerType == common_pb.EPrisoner_Monster then
            if self.select_info[1].needbattle == 1 then
                self:showConditionContent(self.select_info)
            else
                self:showRecruitmentContent(self.select_info)
            end
        else
            if self.select_info and #self.select_info == 0 then
                hint:setText(NativeText.GetLangStr(380027))
            else
                hint:setText(NativeText.GetLangStr(380026))
            end
        end
    end,

    click_closeNoticeBtn = function(self)
        self.helper.hideTrans(self.notice_trans)
    end,

    click_closeRecruitmentBtn = function(self)
        self.helper.hideTrans(self.recruitment_trans)
    end,

    click_closeContitionBtn = function(self)
        self.helper.hideTrans(self.condition_trans)
    end,

    showTeamFromation = function(self, recurit_id)
        local startBattle = function(team)
            network_sender:startRecuritFight(recurit_id, team.formationId, team.formationHero)
        end

        frame_manager:show('world_formation_adjust', self, {
            curr_team = nil, 
            hero_list = hero_manager:getUnDispatchedHeros(), 
            callback = startBattle,
        })
    end,
}
