local super = require("ui.frame")
local right_bar_length = 2

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        self.msg = self.args.data

        clickEvents = 
        {
            "content/backBtn",
            "content/un_own/enough_item/useBtn",
            "content/un_own/less_item/buyUseBtn",
            "content/own_un_use/replaceBtn",
        }

        -- 注册点击事件
        frame_helper:regClickEvents("click", self, clickEvents)

        for k, v in ipairs(self.msg.skinList) do
            print("皮肤Id:", v.skinCfgId, v.leftTime, v.isUse)
        end

        -- self.cur_select_cfg 当前选择的配置
        -- self.cur_custom_cfg 当前选择的自定义配置
        -- self.cur_use_item    当前选择的道具id 

        self.rmb_time = Time.realtimeSinceStartup -- 记下收到消息的时间点

        self:init()
    end,

    onUnload = function (self)
    end,

    getPassTime = function(self)
        return Time.realtimeSinceStartup - self.rmb_time
    end,

    init = function(self)
        self.cook_cfgs = self.cook_cfgs or self:cookCfgs()
        self.name = self:findText("content/name")
        self.icon = self:findImage("content/icon")
        self:initLeftBar()
        self:initRightBar()

        self:click_leftBar(1, #self.cook_cfgs, self.cook_cfgs[1])
    end,

    refresh = function(self, msg)
        print("刷新数据")
        self.rmb_time = Time.realtimeSinceStartup -- 记下收到消息的时间点
        self.msg = msg
        self:refreshRightInfo()
        -- for k, v in ipairs(self.msg.skinList) do
        --     print("皮肤Id:", v.skinCfgId, v.leftTime, v.isUse)
        -- end
    end,

    initRightBar = function(self)
        local path = "content/right_bar/"
        for i = 1, right_bar_length do 
            local child = self.transform:Find(path..i)
            local btn = child:GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(function()
                self:click_rightBar(i)
            end)
        end
    end,

    initLeftBar = function(self)
        local path = "content/left_bar"
        frame_helper:hideAllChild(self, path)
        for __, value in ipairs(self.cook_cfgs) do
            local child = frame_helper:getChild(self, __, path)
            child:SetActive(true)

            local btn = child:GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(function()
                self:click_leftBar(__, #self.cook_cfgs, value)
            end)
        end
    end,

    click_leftBar = function(self, index, length, custom_cfg)
        for i = 1, length do 
            local path = "content/left_bar/" .. i
            self:findButton(path).interactable = index ~= i
        end

        self.cur_custom_cfg = custom_cfg
        self:click_rightBar(1)

        print("左边索引:", index, "类型:", custom_cfg.type_id)
    end,

    click_rightBar = function(self, index)
        for i = 1, right_bar_length do 
            local path = "content/right_bar/" .. i
            self:findButton(path).interactable = index ~= i
        end
        print("右边索引:", index)

        if index == 1 then
            self.scroll_view_cfgs = self.cur_custom_cfg.list
        else
            self.scroll_view_cfgs = {}
            for k, cfg in ipairs(self.cur_custom_cfg.list) do
                for __, sk in ipairs(self.msg.skinList) do
                    if cfg.id == sk.skinCfgId then
                        table.insert(self.scroll_view_cfgs, cfg)
                        break
                    end
                end
            end
        end
        self:refreshScrollView(self.scroll_view_cfgs)
    end,

    refreshScrollView = function(self, cfgs)
        self:resetScrollRectIndex()
        local startPos = Vector3(261, -421.5)
        local interval = Vector3(222, 0)
        local path = "content/Center/Scroll View/Viewport/Content/"
        frame_helper:hideAllChild(self, path)
        for i = 1, #cfgs do
            local item = frame_helper:getChild(self, i, path)
            item:SetActive(true)
            item.transform.localPosition = startPos + interval * (i - 1)
            item.transform.localScale = Vector3.one
            local cfg = cfgs[i]
            local icon = item.transform:Find("icon"):GetComponent(UnityEngine.UI.Image)
            icon:SetSprite(cfg.prefab_preview)
        end
        local fish = self.transform:Find("content/Center/Scroll View"):GetComponent(UIFisheye)
        fish:InitChilds(function(idx)
            -- 返回的索引是从 0 开始
            if idx ~= -1 then
                idx = idx + 1
                print("选择索引:", idx)
            end
        end)

        self.scroll_rect = fish.scrollrect
    end,

    resetScrollRectIndex = function(self)
        self.scroll_rect_index = nil
    end,

    setScrollRectIndex = function(self, index)
        if index == self.scroll_rect_index then
            return
        end
        self.scroll_rect_index = index 
        print("scroll rect 索引:", index)

        self:refreshRightInfo()
    end,

    getOwnState = function(self, cfg_id)
        for __, sk in ipairs(self.msg.skinList) do
            if sk.skinCfgId == cfg_id then
                return true, sk.leftTime
            end
        end
        return false, 0
    end,

    isUsing = function(self, cfg_id)
        for __, sk in ipairs(self.msg.skinList) do
            if sk.isUse == 1 and cfg_id == sk.skinCfgId then
                return true
            end
        end
        return false
    end,

    -- 1.拥有 使用中
    -- 2.拥有 未使用
    -- 3.未拥有 拥有道具
    -- 4.未拥有 没有道具
    refreshRightInfo = function(self)
        local cfg = self.scroll_view_cfgs[self.scroll_rect_index]
        if not cfg then
            return
        end
        self.cur_select_cfg = cfg

        local name = self:findText("content/name")
        local icon = self:findImage("content/icon")

        name.text = frame_helper:getLangCfg(cfg.name)
        icon:SetSprite(cfg.prefab_preview)

        local path = "content/right_prop"
        frame_helper:hideAllChild(self, path)
        local args = frame_helper:getPropertyDes(cfg.attribute)
        local idx = 1
        for __, map in pairs(args) do
            local child = frame_helper:getChild(self, idx, path)
            child:SetActive(true)
            local name = child.transform:Find("text1_Static[T5]_9"):GetComponent(UnityEngine.UI.Text)
            name.text = map.name_str .. map.num_str
            idx = idx + 1
        end

        local own_using = self.transform:Find("content/own_using")
        local own_un_use = self.transform:Find("content/own_un_use")
        local un_own = self.transform:Find("content/un_own")

        frame_helper.hideTrans(own_using)
        frame_helper.hideTrans(own_un_use)
        frame_helper.hideTrans(un_own)

        local is_using = self:isUsing(cfg.id)
        local own_state, left_time = self:getOwnState(cfg.id)
        if own_state then
            if is_using then
                frame_helper.showTrans(own_using)
            else
                frame_helper.showTrans(own_un_use)
            end

            local time_path = "content/" .. (is_using and "own_using/" or "own_un_use/") .. "time"
            local time = self:findText(time_path)
            local clk_trans =  self.transform:Find(time_path)
            local clk = clk_trans:GetComponent(Clock)
            if not clk then clk = clk_trans:AddComponent(Clock) end
            clk.enabled = false

            -- 如果剩余时间是0的话，认为是永久的
            if left_time == 0 then
                clk:Stop()
                local temp_str = NativeText.GetLangStr(540015)
                temp_str = temp_str:gsub("\\n", "\n")
                time.text = temp_str
            else
                local temp_str = NativeText.GetLangStr(540016) .. frame_helper:timeFormat(left_time)
                temp_str = temp_str:gsub("\\n", "\n")
                time.text = temp_str
                clk.prefix = NativeText.GetLangStr(540016)
                local temp_time = math.floor(left_time - self:getPassTime())
                if temp_time <= 0 then
                    network_sender:getSkinInfo()
                else
                    clk:Set2(temp_time, nil)
                    -- clk:Set2(temp_time, function(time)
                    --     if time <= 0 then
                    --         network_sender:getSkinInfo()
                    --     end
                    -- end)
                end
            end
        else
            frame_helper.showTrans(un_own)

            local dd_trans = self.transform:Find("content/un_own/DropDownList")
            local ddh = dd_trans:GetComponent("DropDownHelper")
            local use_items = loadstring("return " .. cfg.item)()
            if not ddh then
                ddh:AddComponent("DropDownHelper")
            end
            ddh:Clear()

            local temp = {}
            for item_id, duration in pairs(use_items) do
                ddh:Add(frame_helper:getItemName(item_id))
                table.insert(temp, item_id)
            end
    
            local dd = self:findDropdown("content/un_own/DropDownList")
            dd.onValueChanged:RemoveAllListeners()
            dd.onValueChanged:AddListener(function(value)
                self:refreshItemInfo(value + 1, temp)
            end)
            dd.value = 0
            self:refreshItemInfo(1, temp)
        end
    end,

    refreshItemInfo = function(self, index, use_items)
        local item_id = use_items[index]

        local less_item = self.transform:Find("content/un_own/less_item")
        local enough_item = self.transform:Find("content/un_own/enough_item")

        local num = frame_helper:getItemNum(item_id)
        frame_helper:setActive(enough_item, num > 0)
        frame_helper:setActive(less_item, num <= 0)
        if num <= 0 then
            local DIAMOND    = 30001 -- 钻石
            local image_text = self:findText("content/un_own/less_item/buyUseBtn/image_text")
            local item_name = gui_helper:getImageTextItemStr(DIAMOND)
            local single_price = frame_helper:getItemBuyPrice(item_id)
            image_text:SetImageText(string.format("%s %s", item_name, single_price))
        end

        self.cur_use_item = item_id
        print("当前物品id:",self.cur_use_item)
    end,

    update = function(self)
        -- chapter 更换时刷新 UI
        if self.scroll_rect then
            self:setScrollRectIndex(self.scroll_rect.SelectIdx + 1)
        end
    end,

    -- 返回的数据类型
    -- {
    --     {
    --         type_id = 1,
    --         list = { cfg, cfg },
    --     },
    --     {
    --         type_id = 1,
    --         list = { cfg, cfg },
    --     },
    -- },
    cookCfgs = function(self)
        local rst = {}
        getList = function(type_id)
            for __, value in ipairs(rst) do
                if value.type_id == type_id then
                    return value.list
                end
            end
            local temp = {
                type_id = type_id,
                list = {}
            }
            table.insert(rst, temp)
            return temp.list
        end
        config.home_config_skins:foreach(function(id, cfg)
            local list = getList(cfg.type)
            table.insert(list, cfg)
        end)
        local sortTypeId = function(a, b)
            return a.type_id < b.type_id
        end
        local sortList = function(a, b)
            return a.id < b.id
        end
        table.sort(rst, sortTypeId)
        for __, value in ipairs(rst) do
            table.sort(value.list, sortList)
        end
        return rst
    end,

    click_backBtn = function(self)
        frame_manager:hide('home_skin')
    end,
    
    click_useBtn = function(self)
        print("使用道具:", self.cur_use_item)
        network_sender:useItem(self.cur_use_item, 1)
    end,

    click_buyUseBtn = function(self)
        print("购买使用道具:", self.cur_use_item)
        local des = NativeText.GetLangStr(540010)
        frame_helper:showCommonBox(des, function()
            network_sender:buyItem(
            {
                {
                    item_id = self.cur_use_item,
                    count = 1,
                }
            },
            1
        )
        end)
    end,

    click_replaceBtn = function(self)
        print(" 更换皮肤", self.cur_select_cfg.id)
        network_sender:useSkin(self.cur_select_cfg.id)
    end,
}
