local super = require("ui.frame")
local log = logger.get("Game", "Guild Logo")
local bar_num = 4
local diamond_id = 30001
local key = local_player.id .. os.date("%Y%m%d", os.time())

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        -- 加速类型
        self.speed_up_type = self.args.data.speed_up_type
        -- 类型是建筑 id：建筑唯一id，
        self.id = self.args.data.id
        self.build_id = self.args.data.id
        self.soldier_id = self.args.data.soldier_id
        self.tech_cfg_id = self.args.data.tech_cfg_id
        self.forge_block_id = self.args.data.forge_block_id
        self.arsenal_cfg = self.args.data.arsenal_cfg

        clickEvents = 
        {
            "content/backBtn",
            "content/top/finishBtn",
            "content/top/freeBtn",
            "content/Scroll View/Viewport/Content/fixed/detailBtn",
            "content/Scroll View/Viewport/Content/fixed/quickUseBtn",
            "view/closeViewBtn",
            "view/toggleBtn",
            "view/confirmBtn",
            "view/cancelBtn",
        }

        -- 注册点击事件
        frame_helper:regClickEvents("click", self, clickEvents)

        self.view_trans = self.transform:Find("view")

        self.toggle_state = self:getToggleState()
        self:onToggle()
        frame_helper.hideTrans(self.view_trans)

        self.onBagChanged = bag_manager.events:on('onBagChanged', function() self:showScorllView() end)

        self.BUILD_INFO_CHANGE = home_info.events:on("BUILD_INFO_CHANGE", function(build_id)
            self:refreshTime(build_id, true)
        end)

        self:init()

        guide_manager:tryStartGuide()
        -- log.error("加速 onEnter ...", self)
    end,

    onUnload = function (self)
        -- log.error("加速 onExit ...", self)
        bag_manager.events:off("onBagChanged", self.onBagChanged)
        home_info.events:off("BUILD_INFO_CHANGE", self.BUILD_INFO_CHANGE)
        home_info.events:off("BUILD_UPGRADE_TICK", self.event_id)
        home_info.events:off("BARRACK_TRAIN_TICK", self.event_id)
        home_info.events:off("HOSPITAL_CURE_TICK", self.event_id)
        home_info.events:off("TRAP_TRAIN_TICK", self.event_id)
        home_info.events:off("TRAP_CURE_TICK", self.event_id)
        home_info.events:off("WALL_CURE_TICK", self.event_id)
        home_info.events:off("TECH_UPGRADE_TICK", self.event_id)
        home_info.events:off("FORGE_UPGRADE_TICK", self.event_id)
        home_info.events:off("ARSENAL_CREATE_TICK", self.event_id)
    end,

    click_backBtn = function(self)
        frame_manager:hide('home_speed_up')
    end,

    click_closeViewBtn = function(self)
        frame_helper.hideTrans(self.view_trans)
    end,

    click_confirmBtn = function(self)
        print("确认")
        if self.quick_items and #self.quick_items > 0 then
            local temp = {}
            for index, data in ipairs(self.quick_items) do
                local cfg = item_helper:getItemCfg(data.item_id)
                -- num.text = data.item_num
                table.insert(temp,{
                    item_id = data.item_id,
                    item_num = data.item_num
                })
            end
            self:useDiffItems(temp)
        end

        frame_helper.hideTrans(self.view_trans)
    end,

    click_cancelBtn = function(self)
        frame_helper.hideTrans(self.view_trans)
        print("取消")
    end,

    click_toggleBtn = function(self)
        self.toggle_state = not self.toggle_state
        self:onToggle()
        self:setToggleState(self.toggle_state)
    end,

    onToggle = function(self)
        local image = self:findImage("view/toggleBtn/Image")
        image.enabled = self.toggle_state
    end,

    init = function(self)
        self.effect_trans = self.transform:Find("content/effect")
        self.effect_trans:SetActive(false)

        local root_trans = self.transform:Find("content/Scroll View/Viewport/Content/fixed/icon")
        local mask_trans = self.transform:Find("content/Scroll View/Viewport")
        ParticleMask.SetClip(root_trans, mask_trans)

        app:startCoroutine(function()
            coroutine.yield(nil)
            root_trans.localPosition = root_trans.localPosition + Vector3.one
            coroutine.yield(nil)
            root_trans.localPosition = root_trans.localPosition - Vector3.one
        end)

        self:initTop()
        self:showScorllView()
    end,

    initTop = function(self)
        self:refreshTime(self:getBuildId(), false)
        self:registerEvent()
    end,

    update = function(self, dt)
        if not self.transform then
            return
        end

        if self.target_value then
            if not self.progress_slider then
                self.progress_slider = self.transform:Find("content/top/slider"):GetComponent(UnityEngine.UI.Slider)
                self.progress_slider.value = self.target_value
                self.speed = 0.5
            end

            local temp_value = self.progress_slider.value
            if self.progress_slider.value < self.target_value then
                temp_value = temp_value + self.speed * dt
                if temp_value > self.target_value then
                    temp_value = self.target_value
                end
                self.progress_slider.value = temp_value
            end

            if self.effect_time then
                self.effect_time = self.effect_time - dt
                if self.effect_time < 0 then
                    self.effect_time = nil  
                else
                    self.effect_trans:SetActive(true)
                    local fill_rect = self.transform:Find("content/effect/fill"):GetComponent(UnityEngine.RectTransform)
                    fill_rect.sizeDelta = Vector2(self.progress_slider.value * 1389, fill_rect.sizeDelta.y)
                end
            else
                self.effect_trans:SetActive(false)
            end
        end
    end,

    setTimeInfo = function(self, elapsed, total_time, want_play_effect)
        if want_play_effect and not self.effect_time then
            self.effect_time = ((1- elapsed / total_time) - self.progress_slider.value) / self.speed
        end

        self.target_value = 1- elapsed / total_time

        local name = self:findText("content/top/name")
        local des = self:findText("content/top/des")
        local time = self:findText("content/top/time")
        local helper_trans = self.transform:Find("content/top/helper")
        local helper_text = self:findSlider("content/top/helper/text")
        local finishBtn = self:findButton("content/top/finishBtn")
        local freeBtn = self:findButton("content/top/freeBtn")
        local image_text = self:findText("content/top/finishBtn/image_text")

        local free_time = self:getFreeTime()

        name.text = self:getName()
        des.text = self:getDes(free_time, elapsed)

        local image_text_str = self:getSpeedUpCostStr(elapsed)
        time.text = string.format(NativeText.GetLangStr(260022), frame_helper:timeFormat(elapsed)) 
        frame_helper.hideTrans(helper_trans)
        image_text:SetImageText(image_text_str)

        if free_time > elapsed then
            frame_helper.showTrans(freeBtn.transform)
            frame_helper.hideTrans(finishBtn.transform)
        else
            frame_helper.showTrans(finishBtn.transform)
            frame_helper.hideTrans(freeBtn.transform)
        end

        local child = self.transform:Find("content/Scroll View/Viewport/Content/fixed")
        local name = child.transform:Find("name_text"):GetComponent(UnityEngine.UI.Text)
        local detail_text = child.transform:Find("detailBtn/text"):GetComponent(UnderLineText)
        detail_text:SetText(NativeText.GetLangStr(260024), nil)

        local time = elapsed - free_time;
        
        self.quick_items = {}
        local ignore = 300
        local des_time = 0
        for __, item_type in ipairs(self:getMultipleItemType()) do
            local temp1, temp2 = self:getQuickItems(item_type, time, ignore, self.quick_items)
            time = temp1
            if temp2 then
                des_time = des_time + temp2
            end
            -- print("%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%:", __, #self.quick_items)
        end
        name.text = string.format(NativeText.GetLangStr(260023), frame_helper:timeFormat(des_time))  
        self.quick_time = des_time
    end,

    click_finishBtn = function(self)
        print("立即完成")
        network_sender:perSonBaseSpeedUp(self:getBuildId(), self:getProtcolEnum(), 1, 0, {}, self:getSubId())
    end,

    click_freeBtn = function(self)
        self:click_finishBtn()
    end,

    click_quickUseBtn = function(self)
        if self.quick_items and #self.quick_items > 0 and self:getToggleState() then
            local temp = {}
            for index, data in ipairs(self.quick_items) do
                table.insert(temp,{
                    item_id = data.item_id,
                    item_num = data.item_num
                })
            end
            self:useDiffItems(temp)
            print("发送消息")
        else
            self:click_detailBtn()
        end
    end,

    click_detailBtn = function(self)
        if self.quick_items and #self.quick_items > 0 then
            frame_helper.showTrans(self.view_trans)
            local parent_path = "view/Scroll View/Viewport/Content/"
            frame_helper:hideAllChild(self, parent_path)
            for index, data in ipairs(self.quick_items) do
                local child = frame_helper:getChild(self, index, parent_path)
                child.gameObject:SetActive(true)

                local icon = child:Find("icon"):GetComponent(UnityEngine.UI.Image)
                local quality = child:Find("out_bg"):GetComponent(UnityEngine.UI.Image)
                local name_text = child:Find('name'):GetComponent(UnityEngine.UI.Text)
                local num = child:Find('num'):GetComponent(UnityEngine.UI.Text)

                local cfg = item_helper:getItemCfg(data.item_id)

                icon:SetSprite(frame_helper:getItemIcon(cfg.id))
                quality:SetSprite(frame_helper:getItemQualityIcon(cfg.id))
                name_text.text = frame_helper:getLangCfg(cfg.name_id)
                num.text = data.item_num
            end

            local top_des = self:findText("view/des")
            top_des.text = string.format(NativeText.GetLangStr(260029), frame_helper:timeFormat(self.quick_time))  
            self:onToggle()
        else
            hint:setText(NativeText.GetLangStr(260030))
        end
    end,

    getSpeedUpCostStr = function(self, time)
        local DIAMOND = 30001
        local item_name = gui_helper:getImageTextItemStr(DIAMOND)
        local item_type = self:getItemType()
        local cost = gui_helper:getSpeedUpCost(item_type, time)
        return string.format("%s %s", item_name, cost)
    end,

    -- 扩展修改这里
    getItemType = function(self)
        return self.speed_up_type
    end,

    -- 扩展修改这里
    getMultipleItemType = function(self)
        return { self:getItemType(), 42}
        -- if self.speed_up_type == home_speed_up_type.build_upgrade then
        -- end
    end,

    getAllItemCfgs = function(self)
        local sortFunc = function(a, b)

            local a_num = bag_manager:getItemNum(a.id)
            local b_num = bag_manager:getItemNum(b.id)

            if (a_num > 0 and b_num > 0) or (a_num == 0 and b_num == 0) then 
                if a.type > b.type then
                    return true
                elseif a.type == b.type then
                    return a.id < b.id
                else
                    return false
                end
            else
                return a_num > b_num
            end
        end

        if not self.item_cfgs then
            self.item_cfgs = {}
            for __, item_type in ipairs(self:getMultipleItemType()) do
                local temp = item_helper:getItemCfgByType(item_type)
                for __, cfg in ipairs(temp) do
                    table.insert(self.item_cfgs, cfg)
                end
            end
            table.sort(self.item_cfgs, sortFunc)
        end
        return self.item_cfgs
    end,

    getQuickItems = function(self, item_type, time, ignore, use_items)
        if time <= 0 then
            return 0
        end
        local cfgs = item_helper:getItemCfgByType(item_type)
        local sortFunc = function(a, b)
            return tonumber(a.para1) > tonumber(b.para1)
        end
        table.sort(cfgs, sortFunc)
        local data = {}
        for __, c in ipairs(cfgs) do
            local temp = {}
            temp.cfg = c
            temp.own_num = frame_helper:getItemNum(c.id)
            temp.item_id = c.id
            temp.speed_up_time = c.para1
            table.insert(data, temp)
        end

        local total_use_time = 0;
        for __, d in ipairs(data) do
            if time <= 0 then
                break
            end

            local item_time = d.speed_up_time * 60
            local item_num = d.own_num

            local ceil_num = math.ceil(time / item_time)
            local floor_num = math.floor(time / item_time)

            local real_use = nil

            
            local ceil_time = ceil_num * item_time
            local floor_time = floor_num * item_time

            local temp_num = 0
            if ceil_time - time < ignore then
                temp_num = ceil_num
            elseif floor_time - time < ignore then
                temp_num = floor_num
            end

            if temp_num > 0 then
                if temp_num <= item_num then
                    real_use = temp_num
                else
                    real_use = item_num
                end

                time = time - real_use * item_time


                if real_use and real_use > 0 then
                    -- table.insert(console_content, string.format("使用道具{%s秒}, 使用:%s，拥有:%s", item_time, real_use, item_num))
                    table.insert(use_items, {
                        item_id = d.item_id,
                        item_num = real_use,
                    })

                    total_use_time = real_use * item_time
                end
            end
        end

        -- for __, content in ipairs(console_content) do
        --     print(content)
        -- end

        return time, total_use_time
    end,

    getToggleState = function(self)
        return UnityEngine.PlayerPrefs.GetInt(key, 0) == 1
    end,

    setToggleState = function(self, value)
        if value then
            UnityEngine.PlayerPrefs.SetInt(key, 1)
        else
            UnityEngine.PlayerPrefs.SetInt(key, 0)
        end
    end,

    showScorllView = function(self)
        local cfgs = self:getAllItemCfgs()
        local parent_path = "content/Scroll View/Viewport/Content"
        local template_path =  "content/Scroll View/Viewport/Content/tplt"
        frame_helper:hideAllChild(self, parent_path)

        for idx, cfg in ipairs(cfgs) do
            local child = frame_helper:getChildByTemplatePath(self, idx, parent_path, template_path)
            child:SetActive(true)

            local icon = child:Find("icon"):GetComponent(UnityEngine.UI.Image)
            local quality = child:Find("out_bg"):GetComponent(UnityEngine.UI.Image)
            local name_text = child:Find('name_text'):GetComponent(UnityEngine.UI.Text)
            local des_text = child:Find('des_text'):GetComponent(UnityEngine.UI.Text)
            local own_text = child:Find('own_text'):GetComponent(UnityEngine.UI.Text)

            local use_trans = child:Find('useTrans')
            local buy_trans = child:Find('buyTrans')

            local own_num = frame_helper:getItemNum(cfg.id)

            print("物品id:", cfg.id)

            icon:SetSprite(frame_helper:getItemIcon(cfg.id))
            quality:SetSprite(frame_helper:getItemQualityIcon(cfg.id))
            name_text.text = frame_helper:getLangCfg(cfg.name_id)
            des_text.text = frame_helper:getLangCfg(cfg.des_id)
            own_text.text = string.format(NativeText.GetLangStr(own_num > 0 and 250008 or 250010), own_num) 

            use_trans:SetActive(own_num > 0)
            buy_trans:SetActive(own_num <= 0)

            if own_num > 0 then
                local btn = child:Find('useTrans/useBtn'):GetComponent(UnityEngine.UI.Button)
                btn.onClick:RemoveAllListeners()
                btn.onClick:AddListener(
                    function ()
                        print("使用单个")
                        -- network_sender:useItem(cfg.id, 1)
                        self:useItems(cfg.id, 1)
                    end
                )

                local btn = child:Find('useTrans/batBtn'):GetComponent(UnityEngine.UI.Button)
                btn.onClick:RemoveAllListeners()
                btn.onClick:AddListener(
                    function ()
                        local select_num, max_num = self:getUseSliderData(cfg.id)
                        frame_manager:show('use_slider', nil, {
                            item_id = cfg.id,
                            select_num = select_num,
                            max_num = max_num,
                            callback = function(item_id, item_num)
                                self:useItems(item_id, item_num)
                            end,
                        })
                    end
                )
            else
                local buy_cost = frame_helper:getDiamondPriceFromStr(cfg.allprice)
                
                local DIAMOND    = 30001 -- 钻石
                local image_text = child:Find('buyTrans/image_text'):GetComponent(UnityEngine.UI.Text)
                local item_name = gui_helper:getImageTextItemStr(DIAMOND)
                image_text:SetImageText(string.format("%s %s", item_name, buy_cost))

                local btn = child:Find('buyTrans/buyBtn'):GetComponent(UnityEngine.UI.Button)
                btn.onClick:RemoveAllListeners()
                btn.onClick:AddListener(
                    function ()
                        local select_num, max_num = self:getBuySliderData(cfg.id)
                        print("选择数量，最大数量，", select_num, max_num)
                        if select_num and max_num then
                            frame_manager:show('buy_slider', nil, {
                                item_id = cfg.id,
                                select_num = select_num,
                                max_num = max_num,
                                callback = function(item_id, item_num)
                                    self:buyItems(item_id, item_num)
                                end,
                            })
                        end
                    end
                )
            end
        end
    end,

    getUseSliderData = function(self, item_id)
        local select_num, max_num = self:getExpectSliderData(item_id)
        local own_num = frame_helper:getItemNum(item_id)
        if own_num < max_num then
            max_num = own_num
        end
        return select_num, max_num
    end,

    getBuySliderData = function(self, item_id)
        local expect_select_num, expect_max_num = self:getExpectSliderData(item_id)
        local single_price = frame_helper:getItemBuyPrice(item_id)
        local diamond_num = frame_helper:getDiamondNum()
        local can_buy_num = math.floor(diamond_num / single_price)
        if expect_select_num <= can_buy_num then
            return expect_select_num, expect_max_num
        else
            return can_buy_num > 0 and can_buy_num or 1, expect_max_num
        end
    end,

    getExpectSliderData = function(self, item_id)
        local elapsed = self:getLeftTime()
        if elapsed > 0 then
            local one_cost = tonumber(item_helper:getItemCfg(item_id).para1 * 60)
            local rst = elapsed / one_cost
            if rst < 1 then
                return 1, 1
            else
                rst = math.floor(rst)
            end
            return rst, rst + 1
        else
            return 1, 1
        end
    end,

    -- //个人基地加速类型
    -- enum EPB_SPEEDUP
    -- {
    -- EPB_NONE    		= 0;
    -- EPB_BUILD			= 1;//建造建筑
    -- EPB_TECH_RESEARCH	= 2;//科技研发
    -- EPB_PRO_EQUIP		= 3;//装备生产
    -- EPB_REPAIR_WALL	= 4;//城墙修复
    -- EPB_PRO_SOLDIER	= 5;//生产士兵
    -- EPB_PRO_TRAP 		= 6;//生产陷阱
    -- EPB_REPAIR_TRAP	= 7;//陷阱修复
    -- EPB_CURE_SOLDIER	= 8;//治疗士兵
    -- EPB_PRO_QUA_EQUIP	= 9;//生产量子装备
    -- }

    getProtcolEnum = function(self)
        if self.speed_up_type == home_speed_up_type.build_upgrade then
            return common_pb.EPB_BUILD
        elseif self.speed_up_type == home_speed_up_type.tech_upgrade then
            return common_pb.EPB_TECH_RESEARCH
        elseif self.speed_up_type == home_speed_up_type.equip_create then
            return common_pb.EPB_PRO_EQUIP
        elseif self.speed_up_type == home_speed_up_type.wall_reqair then
            return common_pb.EPB_REPAIR_WALL
        elseif self.speed_up_type == home_speed_up_type.soldier_create then
            return common_pb.EPB_PRO_SOLDIER
        elseif self.speed_up_type == home_speed_up_type.trap_create then
            return common_pb.EPB_PRO_TRAP
        elseif self.speed_up_type == home_speed_up_type.trap_repair then
            return common_pb.EPB_REPAIR_TRAP
        elseif self.speed_up_type == home_speed_up_type.soldier_repair then
            return common_pb.EPB_CURE_SOLDIER
        elseif self.speed_up_type == home_speed_up_type.quantum_create then
            return common_pb.EPB_PRO_QUA_EQUIP
        end
    end,

    useDiffItems = function(self, use_items)
        network_sender:perSonBaseSpeedUp(
            self:getBuildId(),
            self:getProtcolEnum(),
            0,
            0,
            use_items,
            self:getSubId()
        )
    end,

    useItems = function(self, item_id, item_num)
        network_sender:perSonBaseSpeedUp(
            self:getBuildId(),
            self:getProtcolEnum(),
            0,
            0,
            {
                {
                    item_id = item_id,
                    item_num = item_num,
                }
            },
            self:getSubId()
        )
    end,

    buyItems = function(self, item_id, item_num)
        network_sender:perSonBaseSpeedUp(
            self:getBuildId(),
            self:getProtcolEnum(),
            0,
            1,
            {
                {
                    item_id = item_id,
                    item_num = item_num,
                }
            },
            self:getSubId()
        )
    end,



    -- ---------------------------------------------------------- 添加类型 修改 start ---------------------------------------------------------------- 

    getBuildId = function(self)
        return self.build_id
    end,

    getSubId = function(self)
        if self.speed_up_type == home_speed_up_type.soldier_repair then
            return self.soldier_id
        elseif self.speed_up_type == home_speed_up_type.trap_repair then
            return self.soldier_id
        elseif self.speed_up_type == home_speed_up_type.tech_upgrade then
            return self.tech_detail_id
        elseif self.speed_up_type == home_speed_up_type.quantum_create then
            return self.forge_block_id
        end
        return 0
    end,

    getName = function(self)
        if self.speed_up_type == home_speed_up_type.build_upgrade then
            local build_data = home_info:getBuildDataById(self.build_id)
            local build_cfg = config.home_config_lists[build_data.cfgId]
            return string.format(NativeText.GetLangStr(260020), frame_helper:getLangCfg(build_cfg.name_id), "Lv." .. build_data.level)
        elseif self.speed_up_type == home_speed_up_type.soldier_create then
            local build_data = home_info:getBuildDataById(self.build_id)
            local build_cfg = config.home_config_lists[build_data.cfgId]
            return string.format(NativeText.GetLangStr(260020), frame_helper:getLangCfg(build_cfg.name_id), "Lv." .. build_data.level)
        elseif self.speed_up_type == home_speed_up_type.soldier_repair then
            local soldier_cfg = config.soldier_soldiers[self.soldier_id]
            local soldier_name = frame_helper:getLangCfg(soldier_cfg.soldiername)
            return string.format(NativeText.GetLangStr(560017), soldier_name)
        elseif self.speed_up_type == home_speed_up_type.trap_create then
            local build_data = home_info:getBuildDataById(self.build_id)
            local soldier_cfg = config.soldier_soldiers[build_data.guardsystem.build.cfgId]
            local soldier_name = frame_helper:getLangCfg(soldier_cfg.soldiername)
            return string.format(NativeText.GetLangStr(560017), soldier_name)
        elseif self.speed_up_type == home_speed_up_type.trap_repair then
            local soldier_cfg = config.soldier_soldiers[self.soldier_id]
            local soldier_name = frame_helper:getLangCfg(soldier_cfg.soldiername)
            return string.format(NativeText.GetLangStr(560017), soldier_name)
        elseif self.speed_up_type == home_speed_up_type.wall_reqair then
            return NativeText.GetLangStr(570006)
        elseif self.speed_up_type == home_speed_up_type.tech_upgrade then
            local cfg = config.home_config_research_lists[self.tech_cfg_id]
            local cur_level =  home_info:getTechLevel(self.tech_cfg_id)
            return string.format(NativeText.GetLangStr(560017), frame_helper:getLangCfg(cfg.name_id) .. " Lv." .. (cur_level + 1) )
        elseif self.speed_up_type == home_speed_up_type.quantum_create then
            local cfg = config.crew_blocks[self.forge_block_id]
            return string.format(NativeText.GetLangStr(620018), frame_helper:getLangCfg(cfg.name_id))
        elseif self.speed_up_type == home_speed_up_type.equip_create then
            return string.format(NativeText.GetLangStr(630011), frame_helper:getLangCfg(self.arsenal_cfg.name))
        end
        log.error("加速类型：", self.speed_up_type, "没有写if语句！！！")
        return ""
    end,

    getDes = function(self, free_time, elapsed)
        if free_time <= 0 then
            return ""
        else
            if elapsed > free_time then
                return string.format(NativeText.GetLangStr(260021), frame_helper:timeFormat(elapsed-free_time), frame_helper:timeFormat(free_time))   
            else
                return ""
            end
        end
    end,

    getFreeTime = function(self)
        if self.speed_up_type == home_speed_up_type.build_upgrade then
            return home_info.building_free_time
        end
        return 0
    end,

    registerEvent = function(self)
        if self.speed_up_type == home_speed_up_type.build_upgrade then
            self.event_id = home_info.events:on("BUILD_UPGRADE_TICK", function(build_data, elapsed, total_time)
                if build_data.id == self.build_id then
                    self:setTimeInfo(elapsed, total_time)
                end
            end)
        elseif self.speed_up_type == home_speed_up_type.soldier_create then
            self.event_id = home_info.events:on("BARRACK_TRAIN_TICK", function(build_data, elapsed, total_time)
                if build_data.id == self.build_id then
                    self:setTimeInfo(elapsed, total_time)
                end
            end)
        elseif self.speed_up_type == home_speed_up_type.soldier_repair then
            self.event_id = home_info.events:on("HOSPITAL_CURE_TICK", function(build_data, elapsed, total_time)
                local elapsed, total_time = self:getLeftTime()
                self:setTimeInfo(elapsed, total_time)
            end)
        elseif self.speed_up_type == home_speed_up_type.trap_create then
            self.event_id = home_info.events:on("TRAP_TRAIN_TICK", function(build_data, elapsed, total_time)
                local elapsed, total_time = self:getLeftTime()
                self:setTimeInfo(elapsed, total_time)
            end)
        elseif self.speed_up_type == home_speed_up_type.trap_repair then
            self.event_id = home_info.events:on("TRAP_CURE_TICK", function(build_data, elapsed, total_time)
                local elapsed, total_time = self:getLeftTime()
                self:setTimeInfo(elapsed, total_time)
            end)
        elseif self.speed_up_type == home_speed_up_type.wall_reqair then
            self.event_id = home_info.events:on("WALL_CURE_TICK", function(build_data, elapsed, total_time)
                local elapsed, total_time = self:getLeftTime()
                self:setTimeInfo(elapsed, total_time)
            end)
        elseif self.speed_up_type == home_speed_up_type.tech_upgrade then
            self.event_id = home_info.events:on("TECH_UPGRADE_TICK", function(build_data, elapsed, total_time)
                local elapsed, total_time = self:getLeftTime()
                self:setTimeInfo(elapsed, total_time)
            end)
        elseif self.speed_up_type == home_speed_up_type.quantum_create then
            self.event_id = home_info.events:on("FORGE_UPGRADE_TICK", function(build_data, elapsed, total_time)
                local elapsed, total_time = self:getLeftTime()
                self:setTimeInfo(elapsed, total_time)
            end)
        elseif self.speed_up_type == home_speed_up_type.equip_create then
            self.event_id = home_info.events:on("ARSENAL_CREATE_TICK", function(build_data, elapsed, total_time)
                local elapsed, total_time = self:getLeftTime()
                self:setTimeInfo(elapsed, total_time)
            end)
        end
    end,

    getLeftTime = function(self)
        if self.speed_up_type == home_speed_up_type.build_upgrade then
            local build_data = home_info:getBuildDataById(self.build_id)
            return home_info:getBuildUpgradeLeftTime(self.build_id), build_data.totalTime
        elseif self.speed_up_type == home_speed_up_type.soldier_create then
            local build_data = home_info:getBuildDataById(self.build_id)
            local elapsed = build_data.barrack.reamintime - ( Time.realtimeSinceStartup - build_data.rmbTime )
            elapsed = elapsed >= 0 and math.floor(elapsed) or 0
            return elapsed, build_data.barrack.totaltime
        elseif self.speed_up_type == home_speed_up_type.soldier_repair then
            local build_data = home_info:getBuildDataByCfgId(7)
            local pass_time = Time.realtimeSinceStartup - build_data.rmbTime
            local data = nil
            for __, value in ipairs(build_data.hospital.soldiers) do
                if value.cfgId == self.soldier_id then
                    data = value
                    break
                end
            end
            if data then
                local elapsed = data.reamintime - pass_time
                elapsed = elapsed >= 0 and math.floor(elapsed) or 0
                return elapsed, data.totaltime
            else
                return 0, 0
            end
        elseif self.speed_up_type == home_speed_up_type.trap_create then
            local build_data = home_info:getBuildDataByCfgId(24)
            local pass_time = Time.realtimeSinceStartup - build_data.rmbTime
            local data = build_data.guardsystem.build
            if data and data.bTran then
                local elapsed = data.reamintime - pass_time
                elapsed = elapsed >= 0 and math.floor(elapsed) or 0
                return elapsed, data.totaltime
            else
                return 0, 0
            end
        elseif self.speed_up_type == home_speed_up_type.trap_repair then
            local build_data = home_info:getBuildDataByCfgId(24)
            local pass_time = Time.realtimeSinceStartup - build_data.rmbTime
            local list = build_data.guardsystem.repair
            local data = nil
            for __, value in ipairs(list) do
                if value.cfgId == self.soldier_id then
                    data = value
                    break
                end
            end
            if data then
                local elapsed = data.reamintime - pass_time
                elapsed = elapsed >= 0 and math.floor(elapsed) or 0
                return elapsed, data.totaltime
            else
                return 0, 0
            end
        elseif self.speed_up_type == home_speed_up_type.wall_reqair then
            local e = home_info:getBuildDataByCfgId(24)
            if e.guardsystem and e.guardsystem.repairWallRemainTime > 0 then
                local total_time = math.floor(e.guardsystem.repairWallRemainTime / ( 1 - e.guardsystem.WallHp / e.guardsystem.WallMaxHp ))
                local elapsed = e.guardsystem.repairWallRemainTime - Time.realtimeSinceStartup + e.rmbTime
                if e.guardsystem.bFire then
                    elapsed = e.guardsystem.repairWallRemainTime
                end
                elapsed = elapsed >= 0 and math.floor(elapsed) or 0
                return elapsed, total_time
            else
                return 0, 0
            end
        elseif self.speed_up_type == home_speed_up_type.tech_upgrade then
            local e = home_info:getBuildDataByCfgId(2)
            if e.studio.tLevelUp > 0 then
                local elapsed = e.studio.tLevelUp - Time.realtimeSinceStartup + e.rmbTime
                local total_time = e.studio.totalTime
                return elapsed, total_time
            else
                return 0, 0
            end
        elseif self.speed_up_type == home_speed_up_type.quantum_create then
            local e = home_info:getBuildDataByCfgId(22)
            if e.trainCenter.remainTime > 0 and e.trainCenter.bWork then
                local elapsed = e.trainCenter.remainTime - Time.realtimeSinceStartup + e.rmbTime
                local total_time = e.trainCenter.totaltime
                return elapsed, total_time
            else
                return 0, 0
            end
        elseif self.speed_up_type == home_speed_up_type.equip_create then
            local e = home_info:getBuildDataByCfgId(11)
            if e.armory.state == common_pb.MMAKE_MAKING then
                local elapsed = e.armory.leftTime - Time.realtimeSinceStartup + e.rmbTime
                local total_time = e.armory.makeTime
                return elapsed, total_time
            else
                return 0, 0
            end
        end
    end,

    refreshTime = function(self, build_id, want_play_effect)
        if self.speed_up_type == home_speed_up_type.build_upgrade then
            if build_id == self.build_id then
                local elapsed, total_time = self:getLeftTime()
                if elapsed <= 0 then
                    frame_manager:hide('home_speed_up')
                else
                    self:setTimeInfo(elapsed, total_time, want_play_effect)
                end
            end
        elseif self.speed_up_type == home_speed_up_type.soldier_create then
            if build_id == self.build_id then
                local elapsed, total_time = self:getLeftTime()
                if elapsed <= 0 then
                    frame_manager:hide('home_speed_up')
                else
                    self:setTimeInfo(elapsed, total_time, want_play_effect)
                end
            end
        elseif self.speed_up_type == home_speed_up_type.soldier_repair then
            local build_data = home_info:getBuildDataByCfgId(7)
            if build_data.id == build_id then
                local elapsed, total_time = self:getLeftTime()
                if elapsed <= 0 then
                    frame_manager:hide('home_speed_up')
                else
                    self:setTimeInfo(elapsed, total_time, want_play_effect)
                end
            end
        elseif self.speed_up_type == home_speed_up_type.trap_create then
            local elapsed, total_time = self:getLeftTime()
            if elapsed <= 0 then
                frame_manager:hide('home_speed_up')
            else
                self:setTimeInfo(elapsed, total_time, want_play_effect)
            end
        elseif self.speed_up_type == home_speed_up_type.trap_repair then
            local elapsed, total_time = self:getLeftTime()
            if elapsed <= 0 then
                frame_manager:hide('home_speed_up')
            else
                self:setTimeInfo(elapsed, total_time, want_play_effect)
            end
        elseif self.speed_up_type == home_speed_up_type.wall_reqair then
            local elapsed, total_time = self:getLeftTime()
            if elapsed <= 0 then
                frame_manager:hide('home_speed_up')
            else
                self:setTimeInfo(elapsed, total_time, want_play_effect)
            end
        elseif self.speed_up_type == home_speed_up_type.tech_upgrade then
            local elapsed, total_time = self:getLeftTime()
            if elapsed <= 0 then
                frame_manager:hide('home_speed_up')
            else
                self:setTimeInfo(elapsed, total_time, want_play_effect)
            end
        elseif self.speed_up_type == home_speed_up_type.quantum_create then
            local elapsed, total_time = self:getLeftTime()
            if elapsed <= 0 then
                frame_manager:hide('home_speed_up')
            else
                self:setTimeInfo(elapsed, total_time, want_play_effect)
            end
        elseif self.speed_up_type == home_speed_up_type.equip_create then
            local elapsed, total_time = self:getLeftTime()
            if elapsed <= 0 then
                frame_manager:hide('home_speed_up')
            else
                self:setTimeInfo(elapsed, total_time, want_play_effect)
            end
        end
    end,

    -- ---------------------------------------------------------- 添加类型 修改 end ----------------------------------------------------------------
}
