local super = require("ui.frame")

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        self.details = self.args.data

        -- 路径: root/login
        -- 事件: click_login 
        clickEvents = {
            "content/info/backBtn",
            "content/info/leftOnceBtn",
            "content/info/leftMultBtn",
            "content/info/rightOnceBtn",
            "content/info/rightMultBtn",
        }

        -- 注册点击事件
        frame_helper:regClickEvents("click", self, clickEvents)

        self.BUILD_INFO_CHANGE = home_info.events:on("BUILD_INFO_CHANGE", function(build_id)
            local build_data = home_info:getBuildDataById(build_id)
            if build_data.cfgId == 25 then
                self:init()
            end
        end)

        self.TAVERN_COOL_DOWN_TICK = home_info.events:on("TAVERN_COOL_DOWN_TICK", function(build_data, elapsed, total_time)
            self:onTimeTick(build_data, elapsed, total_time)
        end)

        self.onBagChanged = bag_manager.events:on('onBagChanged', function(build_id)
            self:init()
        end)

        self:init()
    end,

    onUnload = function (self)
        home_info.events:off("BUILD_INFO_CHANGE", self.BUILD_INFO_CHANGE)
        home_info.events:off("TAVERN_COOL_DOWN_TICK", self.TAVERN_COOL_DOWN_TICK)
        bag_manager.events:off("onBagChanged", self.onBagChanged)
    end,

    onTimeTick = function(self, build_data, elapsed, total_time)
        if self.time_tick then
            self.time_tick.text = string.format(NativeText.GetLangStr(670007), frame_helper:timeFormat(elapsed))
        end
    end,

    init = function(self)

        local e_rate = 1080 / 1920
        local r_rate = UnityEngine.Screen.height / UnityEngine.Screen.width

        local bg = self.transform:Find("content/bg/Image")
        if r_rate > e_rate then
            local scale = r_rate / e_rate
            bg.localScale = Vector3.one * scale
        elseif r_rate < e_rate then
            local scale = e_rate / r_rate
            bg.localScale = Vector3.one * scale
        else
            bg.localScale = Vector3.one
        end

        self.time_tick = nil

        local build_data = home_info:getBuildDataByCfgId(25)
        if build_data then
            for __, value in ipairs(build_data.tavern.drawCardInfo) do
                if value.drawType == common_pb.LOW1DRAW then
                    local top_des = self:findText("content/info/des")

                    local des = self:findText("content/info/leftdes")
                    local elapsed = value.updateTime - Time.realtimeSinceStartup + build_data.rmbTime

                    if value.leftFreeTime > 0 then
                        if elapsed <= 0 then
                            des.text = NativeText.GetLangStr(670003)
                            top_des.text = string.format(NativeText.GetLangStr(670002), value.leftFreeTime .. "/".. value.totalFreeTime)
                        else
                            des.text = "X1"
                            self.time_tick = top_des
                            top_des.text = string.format(NativeText.GetLangStr(670007), frame_helper:timeFormat(elapsed))
                            local item_id = config.tavern_taverns[value.drawType].NeedItem
                            local item_num = frame_helper:getItemNum(item_id)
                            des.color = item_num > 0 and Color.white or Color.red
                        end
                    else
                        des.text = "X1"
                        top_des.text = string.format(NativeText.GetLangStr(670002), value.leftFreeTime .. "/".. value.totalFreeTime)
                        local item_id = config.tavern_taverns[value.drawType].NeedItem
                        local item_num = frame_helper:getItemNum(item_id)
                        des.color = item_num > 0 and Color.white or Color.red
                    end
                elseif value.drawType == common_pb.HIGHT1DRAW then
                    local des = self:findText("content/info/rightdes")
                    local elapsed = value.updateTime - Time.realtimeSinceStartup + build_data.rmbTime

                    if value.leftFreeTime > 0 then
                        if elapsed <= 0 then
                            des.text = NativeText.GetLangStr(670003)
                        else
                            des.text = "X1"
                            local item_id = config.tavern_taverns[value.drawType].NeedItem
                            local item_num = frame_helper:getItemNum(item_id)
                            des.color = item_num > 0 and Color.white or Color.red
                        end
                    else
                        des.text = "X1"
                        local item_id = config.tavern_taverns[value.drawType].NeedItem
                        local item_num = frame_helper:getItemNum(item_id)
                        des.color = item_num > 0 and Color.white or Color.red
                    end

                end
            end
        end

        local left_des = self:findText("content/bg/text1_Static[T4]_7")
        local item_id = config.tavern_taverns[common_pb.LOW1DRAW].NeedItem
        local item_num = frame_helper:getItemNum(item_id)
        left_des.color = item_num >= 10 and Color.white or Color.red

        print("左边:", item_id)

        local right_des = self:findText("content/bg/text1_Static[T4]_11")
        local item_id = config.tavern_taverns[common_pb.HIGHT1DRAW].NeedItem
        local item_num = frame_helper:getItemNum(item_id)
        right_des.color = item_num >= 10 and Color.white or Color.red

        print("右边:", item_id)
    end,

    click_leftOnceBtn = function(self)
        network_sender:drawCard(common_pb.LOW1DRAW)
    end,

    click_leftMultBtn = function(self)
        network_sender:drawCard(common_pb.LOW10DRAW)
    end,

    click_rightOnceBtn = function(self)
        network_sender:drawCard(common_pb.HIGHT1DRAW)
    end,

    click_rightMultBtn = function(self)
        network_sender:drawCard(common_pb.HIGHT10DRAW)
    end,

    click_backBtn = function(self)
        frame_manager:hide('home_tavern')
    end,
}
