local super = require("ui.frame")

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        self.list = self.args.data

        -- 路径: root/login
        -- 事件: click_login 
        clickEvents = {
            "closeBtn",
        }

        -- 注册点击事件
        frame_helper:regClickEvents("click", self, clickEvents)

        self:init()
    end,

    onUnload = function (self)
        self:unloadSpine()
    end,

    init = function(self)
        self.close_btn = self.transform:Find("closeBtn")
        frame_helper.hideTrans(self.close_btn)

        self.normalized_time = {
            0.25612187385559 - 0.005,
            0.32809478044510 - 0.005,
            0.39941537380219 - 0.005,
            0.47209584712982 - 0.005,
            0.54283928871155 - 0.005,
            0.61568337678909 - 0.005,
            0.68649864196777 - 0.005,
            0.75834929943085 - 0.005,
            0.83135098218918 - 0.005,
            0.90266889333725 - 0.005,
        }

        for i = 1, #self.list do
            local icon = self:findImage(string.format("shilianchou/%s/icon (%s)", i, i))
            local num = self:findText(string.format("shilianchou/%s/icon (%s)/Text (%s)", i, i, i))
            local quality = self:findImage(string.format("shilianchou/%s/quality (%s)", i, i))
            local effect_trans = self.transform:Find(string.format("shilianchou/%s/jinse (%s)", i, i))
            local piece = self:findImage(string.format("shilianchou/%s/piece%s", i, i))

            
            local item_num = self.list[i].count
            local hero_id = self.list[i].heroId
            local item_id = self.list[i].itemId

            local item_cfg = hero_id > 0 and config.commander_basics[hero_id] or frame_helper:getItemCfg(item_id)
            local icon_path = item_cfg.path .. item_cfg.icon

            print("英雄id:", hero_id, "icon:", icon_path)

            icon:SetSprite(icon_path)
            quality:SetSprite(frame_helper:getItemQualityIcon(item_id))
            num.text = frame_helper:itemNumFormat(item_num) 
            piece.enabled = item_id >= 120001 and item_id <= 130000

            effect_trans:SetActive(item_cfg.color >= 5)
            -- if item_cfg.type == 31 then
            --     effect_trans:SetActive(hero_id > 0)
            -- else
            --     effect_trans:SetActive(false)
            -- end
        end

        -- if #self.list == 1 then
        --     self.list = {
        --         true
        --     }
        -- else
        --     self.list = {
        --         true,
        --         true,
        --         false,
        --         true,
        --         false,
        --         true,
        --         false,
        --         false,
        --         true,
        --         false,
        --     }
        -- end

        self.event_times = 0
        self.animator_idx = 1
        self.cur_normalized_time = 0

        self.animator = self.transform:GetComponent(UnityEngine.Animator)
        local event_receiver = self.transform:GetComponent(TavernEventReceiver)
        event_receiver:SetTriggerCallback(function(event_times)
            if event_times <= 10 then
                self.event_times = event_times
                    
                local state_info = self.animator:GetCurrentAnimatorStateInfo(0)
                if state_info.normalizedTime > self.cur_normalized_time then
                    self.cur_normalized_time = state_info.normalizedTime
                end

                if self.event_times == self.animator_idx then
                    self.animator_idx = self.animator_idx + 1
                    self:diplayAnimator()
                end

                -- log.error("SetTriggerCallback:", event_times, self.animator_idx, self.cur_normalized_time)
                
                
            elseif event_times == 100 then
                app:startCoroutine(function()
                    coroutine.yield(WaitForSeconds(2))
                    if self.event_times < self.animator_idx then
                        self:diplayAnimator(true)
                    end
                end) 
            elseif event_times == 1000 then
                app:startCoroutine(function()
                    coroutine.yield(WaitForSeconds(1))
                    
                    frame_helper.showTrans(self.close_btn)

                    local lt = LeanTween.alphaText(self.transform:Find("closeBtn/Text"):GetComponent(UnityEngine.RectTransform), 0.4 , 0.83)
                    lt.loopType = LeanTweenType.pingPong
                    
                    -- frame_manager:hide('home_tavern_reward')
                    -- if self.event_times < self.animator_idx then
                    --     self:diplayAnimator(true)
                    -- end
                end)
            end
        end)

        self:diplayAnimator()
    end,

    diplayAnimator = function(self, force)
        local commander_ani_name = "chouka_2"
        local state_name = ""
        if #self.list <= 1 then
            state_name = "chouka_0"
        else
            state_name = "chouka_1"
        end

        if force then
            self.animator:CrossFade(state_name, 0, 0, self.cur_normalized_time)
        else
            if self.animator_idx <= #self.list then
                local item = self.list[self.animator_idx]
                local is_commander = item.heroId > 0
                if is_commander then
                    self.animator.enabled = false
                    app:startCoroutine(function()
                        local icon_path = "Assets/Arts/spines/15_qin/15_qin_SkeletonData.asset"
                        local cmd_cfg = config.commander_basics[item.heroId]
                        local icon_path = cmd_cfg.cmd_path .. cmd_cfg.commander_pic
                        local start_pos, end_pos = string.find(icon_path, ".asset", 1) 
                        local is_spine = start_pos and true or false
                        local icon = self:findImage("zhongjiang/lihui/xiaochou")
                        icon.enabled = not is_spine
                        local name = self:findText("zhongjiang/mingzi/Text")
                        name.text = frame_helper:getLangCfg(cmd_cfg.commander_name)
                        -- name.color = gui_helper:quality2ColorPlus(cmd_cfg.color)

                        local bg1 = self:findImage("zhongjiang/fangsheguang003")
                        local bg2 = self:findImage("zhongjiang/zhongjianpangzi")

                        if cmd_cfg.color == 3 then
                            bg1:SetSprite("Assets/Arts/ui/atlas/tavern_effect/fangsheguang003_lan.png")
                            bg2:SetSprite("Assets/Arts/ui/atlas/tavern_effect/growup_panel_27_2.png")
                        elseif cmd_cfg.color == 4 then
                            bg1:SetSprite("Assets/Arts/ui/atlas/tavern_effect/fangsheguang003.png")
                            bg2:SetSprite("Assets/Arts/ui/atlas/tavern_effect/zhongjianpangzi.png")
                        elseif cmd_cfg.color == 5 then
                            bg1:SetSprite("Assets/Arts/ui/atlas/tavern_effect/fangsheguang003_cheng.png")
                            bg2:SetSprite("Assets/Arts/ui/atlas/tavern_effect/growup_panel_27_4.png")
                        end

                        self.spine_assets = self.spine_assets or {}
                        self:hideSpine()

                        if is_spine then
                            local parent_go = self.spine_assets[item.heroId]
                            if parent_go then
                                frame_helper.showTrans(parent_go.transform)
                            else
                                coroutine.yield(Fenix.Assets.Helper.WaitForAsset(icon_path, Spine.Unity.SkeletonDataAsset, function(asset)
                                    if asset then
                                        local spine_root = self.transform:Find("zhongjiang/lihui/spine_root")
                                        local parent_go = GameObject()
                                        parent_go.name = "soldier id:" .. #self.spine_assets
                                        parent_go:SetParent(spine_root)
                                        frame_helper:resetTransform(parent_go)
                                        self.spine_assets[item.heroId] = parent_go
                                        
                                        parent_go.transform.localScale = Vector3.one * cmd_cfg.BarSpineZoom / 10000
                                        local coord = loadstring("return " .. cmd_cfg.BarSpinePosition)()
                                        local rect = parent_go:GetComponent(UnityEngine.RectTransform)
                                        if not rect then
                                            rect = parent_go:AddComponent(UnityEngine.RectTransform)
                                        end
                                        rect.sizeDelta = Vector2(1115,1533)
                                        rect.pivot = Vector2(0.5, 0)
                                        rect.anchoredPosition = Vector2(coord[1], coord[2])

                                        asset:AddRef()                             
                                        local skeleton_data = asset:Get() -- skeleton data asset
                                        local skeleton_graphic = Spine.Unity.SkeletonGraphic.AddSkeletonGraphicComponent(parent_go, skeleton_data)
                                        skeleton_graphic.material = game_director.spine_material
                                        local anim_state = skeleton_graphic.AnimationState
                                        if anim_state then
                                            anim_state:SetAnimation(0, 'idle', true)
                                        else
                                            log.error("Failed to find anim state:"..icon_path)
                                        end  
                                        
                                        self.cache_models = self.cache_models or {}
                                        table.insert(self.cache_models, asset)
                                    else
                                        log.error("Failed to load spine asset:"..icon_path)
                                    end
                                end))
                            end

                            local parent_go = self.spine_assets[item.heroId]
                            parent_go.transform.localScale = Vector3.one * cmd_cfg.BarSpineZoom / 10000
                            local coord = loadstring("return " .. cmd_cfg.BarSpinePosition)()
                            local rect = parent_go:GetComponent(UnityEngine.RectTransform)
                            rect.sizeDelta = Vector2(1115,1533)
                            rect.pivot = Vector2(0.5, 0)
                            rect.anchoredPosition = Vector2(coord[1], coord[2])

                            coroutine.yield(WaitForSeconds(1))
                        else
                            icon:SetSprite(icon_path)
                            coroutine.yield(WaitForSeconds(0.5))
                        end
                        
                        self.animator.enabled = true
                        self.animator:CrossFade(commander_ani_name, 0, 0)
                        self.cur_normalized_time = self.normalized_time[self.animator_idx]
                    end) 
                    -- log.error("索引:", self.animator_idx, "次数:", self.event_times, "单位化:", self.cur_normalized_time, commander_ani_name)
                else
                    self.animator:CrossFade(state_name, 0, 0, self.cur_normalized_time)
                    -- log.error("索引:", self.animator_idx, "次数:", self.event_times, "单位化:", self.cur_normalized_time, state_name)
                end
            else
                app:startCoroutine(function()
                    coroutine.yield(WaitForSeconds(1))
                    frame_helper.showTrans(self.close_btn)

                    local lt = LeanTween.alphaText(self.transform:Find("closeBtn/Text"):GetComponent(UnityEngine.RectTransform), 0.4 , 0.83)
                    lt.loopType = LeanTweenType.pingPong
                end) 
            end
        end
    end,

    isCommander = function(self, item_id)
        local item_cfg = frame_helper:getItemCfg(item_id)
        return 
    end,

    click_closeBtn = function(self)
        frame_manager:hide('home_tavern_reward')
    end,

    unloadSpine = function(self, slot)
        if self.cache_models then
            for idx, asset in ipairs(self.cache_models) do
                asset:RemoveRef()
            end
        end
        self.cache_models = nil
    end,

    hideSpine = function(self)
        if self.spine_assets then
            for hero_id, go in pairs(self.spine_assets) do
                frame_helper.hideTrans(go.transform)
            end
        end
    end,
}
