local super = require("ui.frame")
local log = logger.get("Game", "home_info")

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        self.home_map = self.args.owner

        clickEvents = 
        {
            "content/common_bg/backBtn"
        }

        -- 注册点击事件
        frame_helper:regClickEvents("click", self, clickEvents)

        self.build_data = home_info:getBuildDataByCfgId(24)
        self.WALL_CURE_TICK = home_info.events:on("WALL_CURE_TICK", function(build_data, elapsed, total_time)
            self:refresh()
        end)
        self.BUILD_INFO_CHANGE = home_info.events:on("BUILD_INFO_CHANGE", function(build_id)
            local build_data = home_info:getBuildDataById(build_id)
            if build_data.cfgId == 24 then
                self:refresh()
            end
        end)
        self:refresh()
    end,

    onUnload = function (self)
        home_info.events:off("BUILD_INFO_CHANGE", self.BUILD_INFO_CHANGE)
        home_info.events:off("WALL_CURE_TICK", self.WALL_CURE_TICK)
    end,

    refresh = function(self)
        self:refreshWallCure()
        self:refreshDetail()
    end,

    refreshDetail = function(self)
        local data = self.build_data.guardsystem
        local core_hp = self:findText("content/layout/right/name")
        local core_slider = self:findSlider("content/layout/right/slider")

        core_hp.text = string.format(NativeText.GetLangStr(580003), data.CoreHp .. "/" .. data.CoreMaxHp)
        core_slider.value = data.CoreHp / data.CoreMaxHp

        local useItemBtn = self.transform:Find("content/left_layout/useItemBtn")
        local speedUpBtn = self.transform:Find("content/left_layout/speedUpBtn")
        local useDiamondBtn = self.transform:Find("content/left_layout/useDiamondBtn")

        useItemBtn:SetActive(false)
        speedUpBtn:SetActive(false)
        useDiamondBtn:SetActive(false)

        local item_num = frame_helper:getItemNum(50002)
        if data.bFire then
            local fire_cool_down_path = ""
            if item_num > 0 then
                useItemBtn:SetActive(true)
                fire_cool_down_path = "content/left_layout/useItemBtn/des"
                local btn = self:findButton("content/left_layout/useItemBtn")
                btn.onClick:RemoveAllListeners()
                btn.onClick:AddListener(function()
                    print("使用道具")
                    network_sender:fireFight()
                end)

                local item_icon = self:findImage("content/left_layout/useItemBtn/icon")
                local item_num = self:findText("content/left_layout/useItemBtn/num")
                item_icon:SetSprite(frame_helper:getItemIcon(50002))
                item_num.text = item_num
            else
                useDiamondBtn:SetActive(true)
                fire_cool_down_path = "content/left_layout/useDiamondBtn/des"
                local cfg = item_helper:getItemCfg(50002)
                local buy_cost = frame_helper:getDiamondPriceFromStr(cfg.allprice)
                local DIAMOND    = 30001 -- 钻石
                local image_text = self.transform:Find('content/left_layout/useDiamondBtn/image_text'):GetComponent(UnityEngine.UI.Text)
                local item_name = gui_helper:getImageTextItemStr(DIAMOND)
                image_text:SetImageText(string.format("%s %s", item_name, buy_cost))

                local btn = self:findButton("content/left_layout/useDiamondBtn")
                btn.onClick:RemoveAllListeners()
                btn.onClick:AddListener(function()
                    if frame_helper:isEnoughDiamond(buy_cost) then
                        network_sender:fireFight()
                        print("使用钻石")
                    end
                end)
            end

            local elapsed = self.build_data.guardsystem.fire_remaintime - Time.realtimeSinceStartup + self.build_data.rmbTime
            elapsed = math.floor(elapsed)
            local temp_text = self:findText(fire_cool_down_path)
            temp_text.text = frame_helper:timeFormat(elapsed)
        else
            speedUpBtn:SetActive(true)
            local btn = self:findButton("content/left_layout/speedUpBtn")
            btn.interactable = data.WallHp < data.WallMaxHp
            if btn.interactable then
                btn.onClick:RemoveAllListeners()
                btn.onClick:AddListener(function()
                    frame_manager:show("home_speed_up", nil, {
                        speed_up_type = home_speed_up_type.wall_reqair,
                        id = home_info:getBuildDataByCfgId(24).id,
                    })
                    print("加速")
                end)
            end
        end

        local btn = self:findButton("content/right_layout/repairBtn")
        btn.interactable = data.repairCoreCd <= 0 and (data.CoreHp < data.CoreMaxHp)
        if data.repairCoreCd > 0 then
            self:addClock(data.repairCoreCd, "content/right_layout/repairBtn/des")
        else
            self:findText("content/right_layout/repairBtn/des").text = ""
        end
        if btn.interactable then
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(function()
                network_sender:repairCore()
                print("修复核心")
            end)
        end
    end,

    refreshWallCure = function(self)
        local wall_hp = self:findText("content/layout/left/name")
        local wall_slider = self:findSlider("content/layout/left/slider")
        local slider_num = self:findText("content/layout/left/slider_num")
        local e = home_info:getBuildDataByCfgId(24)
        local elapsed = 0
        local total_time = 0
        if e.guardsystem and e.guardsystem.repairWallRemainTime > 0 then
            total_time = math.floor(e.guardsystem.repairWallRemainTime / ( 1 - e.guardsystem.WallHp / e.guardsystem.WallMaxHp ))
            elapsed = e.guardsystem.repairWallRemainTime - Time.realtimeSinceStartup + e.rmbTime
            if e.guardsystem.bFire then
                elapsed = e.guardsystem.repairWallRemainTime
            end
            elapsed = elapsed >= 0 and math.floor(elapsed) or 0
            local rate = 1- elapsed / total_time
            wall_slider.value = rate
            wall_hp.text = string.format(NativeText.GetLangStr(580002), math.floor(e.guardsystem.WallMaxHp * rate) .. "/" .. e.guardsystem.WallMaxHp)
            slider_num.text = frame_helper:timeFormat(elapsed)
        else
            wall_slider.value = 1
            wall_hp.text = string.format(NativeText.GetLangStr(580002), e.guardsystem.WallHp .. "/" .. e.guardsystem.WallMaxHp)
            slider_num.text = ""
        end
    end,

    addClock = function(self, time, path)
        local clk_trans =  self.transform:Find(path)
        local clk = clk_trans:GetComponent(Clock)
        if not clk then clk = clk_trans:AddComponent(Clock) end
        clk:Set2(time, function() end)
    end,

    click_backBtn = function(self)
        frame_manager:hide('home_wall_repair')
    end,
}
