local super = require("ui.frame")
local log = logger.get("Game", "item info Frame")
local gui_helper = require('gui.gui_helper')
 
local panel_types = {
    weapon = 1,
    minor_weapon = 2,
    chest = 9,
}

return class
{
    super = super,
    
    onLoaded = function (self)
        super.onLoaded(self)

        self.owner = self.args.owner
        self.item = self.args.data.item
        -- 路径: root/login
        -- 事件: click_login 
        clickEvents = {
            "Close",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self.item_prefab = self.transform:Find('Prefabs/Slot/Item').gameObject
        self.slot_prefab = self.transform:Find('Prefabs/Slot').gameObject

        self.detail_root = self.transform:Find('Root')
        local weapon_info = self.detail_root:Find('WeaponInfo')
        self.weapon_info_panel = require('gui/bag/weapon_info_panel')(weapon_info)
        local material_info = self.detail_root:Find('MaterialInfo')
        self.material_info_panel = require('gui/bag/material_info_panel')(material_info)
        local consume_info = self.detail_root:Find('ConsumeInfo')
        self.consume_info_panel = require('gui/bag/consume_info_panel')(consume_info)
        local chest_info = self.detail_root:Find('ChestInfo')
        self.chest_info_panel = require('gui/bag/chest_info_panel')(chest_info, self)

        self.resource_bag = 0
        self.weapon_bag = 1
        self.material_bag = 2
        self.consume_bag = 3
        self.blueprint_bag = 4
        self.piece_bag = 5
        self.crew_bag = 12

        self.current_panel = nil

        self:showItemInfo(self.item)
    end,

    onUnload = function (self)
    end,

    onRcvMsg = function(self, cmd, msg)
    end,

    initWeaponProps = function(self, weapon)
        local basic_attr = weapon.cfg.basic_attr
        local props = {}
        for k, v in pairs(basic_attr) do
            table.insert( props, {PropId = k, PropValue = v} )
        end

        --[[ local sortFunc = function(a, b)
            local order_a = config.attribute_config_attrs[a.PropId].show_order
            local order_b = config.attribute_config_attrs[b.PropId].show_order

            return order_a < order_b
        end

        table.sort( props, sortFunc ) ]]

        weapon.props = props
        weapon.level = 1
    end,

    -- 显示武器信息
    showItemInfo = function(self, item)
        if item.cfg.bagtype == self.weapon_bag then

            self.current_panel = self.weapon_info_panel     -- 主武器
            
            if not item.props then
                self:initWeaponProps(item)
            end
        elseif item.cfg.bagtype == self.consume_bag then

            if item.cfg.type ==  panel_types.chest then

                self.current_panel = self.chest_info_panel     -- 宝箱
            else
                self.current_panel = self.consume_info_panel    -- 消耗品
            end
        elseif item.cfg.bagtype == self.material_bag then

            self.current_panel = self.material_info_panel    -- 材料

        elseif item.cfg.bagtype == self.piece_bag then

            self.current_panel = self.material_info_panel    -- 碎片

        elseif item.cfg.bagtype == self.blueprint_bag then     -- 图纸

        elseif item.cfg.bagtype == self.resource_bag then

            self.current_panel = self.material_info_panel    -- 金币钻石等资源

        elseif item.cfg.bagtype == self.crew_bag then

            self.current_panel = self.material_info_panel    -- 材料
        else
            self.current_panel = nil
            log.error('Failed to find bag: ', item.cfg.bagtype)
        end
        if self.current_panel then
            self.current_panel:showItem(item)
        end
    end,

    click_Close = function(self)
        frame_manager:hide('itemInfo')
    end,

}