local log = logger.get("Game", "Launch Rally Frame")
local super = require("ui.frame")

return class{
    super = super,

    onLoaded = function( self )
        super.onLoaded(self)

        self.monster = self.args.data.monster
        self.action = self.args.data.action
        self.duration = self.args.data.duration
        self.mass_info = self.args.data.mass_info
        self.atk_cnt = self.args.data.atk_cnt
        self.can_spy = self.args.data.spy

        -- 资源船战斗会有 chapter_id
        self.chapter_id = self.args.data.chapter_id
        launch_rally = self

        clickEvents = {
            "BG/Title/BackBtn",
            "BG/Content/Commanders/Addition",
            "BG/Content/Buttons/Send",
            "BG/Content/Buttons/Auto",
            "BG/Content/Buttons/StartRally",
            "BG/Content/Buttons/Formation",
            "BG/Content/EnemyPanel/Tips/Detect",
            "BG/Content/EnemyPanel/PlayerDetect/Detect",
            "BG/Content/EnemyPanel/PlayerDetect/Report",
            "BG/Content/EnemyPanel/MassDetect/Image/Detect",
            "BG/Content/EnemyPanel/MassDetect/Image/Report",
            "BG/Content/EnemyPanel/Subdue",
            "BG/PropUp/PropUpClose",
            "DetailProps/PropsBG",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self.corps_content = self.transform:Find("BG/Content/Scroll View/Viewport/Content")
        self.corps_prefab = self.corps_content:GetChild(0)
        self.officer_content = self.transform:Find("BG/Content/CommanderPanel/CommanderList/Viewport/Content")
        self.officer_prefab = self.officer_content:GetChild(0)

        self.corps_panel = self.transform:Find("BG/Content/Scroll View")
        self.officer_details = self.transform:Find("BG/Content/CommanderPanel/CommanderDetails")
        self.officer_panel = self.transform:Find("BG/Content/CommanderPanel")
        self.addition_info = self.transform:Find("BG/Content/CommanderPanel/AdditionInfo")
        self.enemy_panel = self.transform:Find("BG/Content/EnemyPanel")
        self.gather_panel = self.transform:Find("BG/Content/GatherPanel")
        self.subdue_info = self.enemy_panel:Find("Info")
        self.soldier_percent = self.enemy_panel:Find("SoldierPercent")
        self.tip_panel = self.enemy_panel:Find("Tips")
        self.player_detect = self.enemy_panel:Find("PlayerDetect")
        self.mass_detect = self.enemy_panel:Find("MassDetect")
        self.detecting_panel = self.enemy_panel:Find("Detecting")
        self.prop_up_panel = self.transform:Find("BG/PropUp")
        self.all_props_panel = self.transform:Find("DetailProps")

        self.all_props_panel:SetActive(false)

        self.tip_panel.localScale = Vector3.zero
        self.player_detect.localScale = Vector3.zero
        self.mass_detect.localScale = Vector3.zero
        self.detecting_panel.localScale = Vector3.zero
        self.soldier_percent.localScale = Vector3.zero
        self.subdue_info.localScale = Vector3.zero
        self.officer_panel.localScale = Vector3.zero
        self.prop_up_panel.localScale = Vector3.zero
        
        self.amount = self.transform:Find("BG/Content/Top/Amount/Value"):GetComponent(UnityEngine.UI.Text)
        self.cargo = self.transform:Find("BG/Content/Top/Cargo/Value"):GetComponent(UnityEngine.UI.Text)
        self.power = self.transform:Find("BG/Content/Top/Power/Value"):GetComponent(UnityEngine.UI.Text)
        self.subdue_percent = self.enemy_panel:Find("Subdue/Text"):GetComponent(UnityEngine.UI.Text)
        self.time_text = self.transform:Find("BG/Content/Buttons/Send/TimeIcon/Text"):GetComponent(UnityEngine.UI.Text)

        self.time_text.text = gui_helper:timeFormat(0)

        self:actionDiff()

        local panel = self.transform:Find("BG/Content/Commanders/Panel")
        self.officer_heads = {}
        for i=1, panel.childCount do
            local head = panel:GetChild(i-1)
            self:setScript("click", "BG/Content/Commanders/Panel/"..head.name, functor.make(self.clickOfficerHead, self))
            self.officer_heads[i] = {ui = head, id = 0}
        end

        self.subdue_factor = 0
        self.speed = 0
        self.total_soldiers = 0
        self.append_officers = {}
        self.append_soldiers = {}
        self.officer_list = {}
        self.soldier_ui = {}
        self.officer_toggles = {}

        if self.monster and self.monster.type == hold_types.player then
            self.origin_x, self.origin_y = self.monster:getCurrPos()
        end
        
        if not self.monster.cfg and self.monster.type == hold_types.monster then
            self.monster.cfg = config.worldmap_monsters[self.monster.cfg_id]
        end

        config.soldier_soldiers:foreach(
            function(k, v)
                if v.type < 10 then
                    self.append_soldiers[k] = {cfg_id = k, count = 0}
                end
            end
        )

        self.append_max = world_map_helper:teamSoldiersAmount({})

        self.soldier_list = {}
        local list = home_info:getAvailableSoldiers()

        for k, v in ipairs(list) do
            self.soldier_list[v.soldier_id] = v.available_num
        end

        self.evts = {}
        self.evts["WORLDMAPBATMSGNOTIFY"] = game_director.events:on("WORLDMAPBATMSGNOTIFY", functor.make(self.onGetReport, self))
        self.army_evt_id = army_queue_manager.events:on("ARMYQUEUEAdd", functor.make(self.onArmyLineAppear, self))

        self:setEnemyInfo()
        self:initSoldierList()
        self:setPower()
        self:setCargo()
        self:setSoldiersAmount()


        if self.chapter_id then
        else
            self:finalResult()            
            self:getReport()
        end
    end,

    onUnload = function(self)
        launch_rally = nil

        for k, v in pairs(self.evts) do
            game_director.events:remove(k, v)
        end

        army_queue_manager.events:remove("ARMYQUEUEAdd", self.army_evt_id)

        if self.mass_coroutine then
            app:stopCoroutine(self.mass_coroutine)
        end
    end,

    update = function(self, dt)
        if self.total_time and self.left_time then
            if self.left_time > 0 then
                self.left_time = self.left_time - dt
                self:detectCountDown()
            end
        end
    end,

    onGetReport = function(self, msg)
        local panel = nil
        if self.action == common_pb.EMARCH_CREATE_MASS or self.action == common_pb.EMARCH_JOIN_MASS then
            panel = self.mass_detect
        else
            panel = self.player_detect
        end

        if msg.hasReprot == 1 then
            self.has_report = true

            panel.localScale = Vector3.one
            self.tip_panel.localScale = Vector3.zero
            self.subdue_info.localScale = Vector3.one

            if self.action == common_pb.EMARCH_CREATE_MASS or self.action == common_pb.EMARCH_JOIN_MASS then
                --local btn = panel:Find("Image/Report")
                --btn:SetActive(true)
            end

            local soldiers = msg.spyReport.soldier.soldierCounts
            local list = {}
            for k, v in ipairs(soldiers) do
                list[v.cfgId] = {cfg_id = v.cfgId, count = v.count}
            end

            self.enemy_soldiers = list

            local power = world_map_helper:calculateTeamPower({}, self.enemy_soldiers, true)

            self:setEnemyPower(power)
            self:setEnemySoldiers(self.enemy_soldiers)
            self:setSubduePercent()
        else
            if self.action == common_pb.EMARCH_CREATE_MASS or self.action == common_pb.EMARCH_JOIN_MASS then
                panel.localScale = Vector3.one
                self.tip_panel.localScale = Vector3.zero

                --local btn = panel:Find("Image/Report")
                --btn:SetActive(false)
            else
                panel.localScale = Vector3.zero
                self.tip_panel.localScale = Vector3.one
            end
        end

        self.detecting_panel.localScale = Vector3.zero
    end,

    onArmyLineAppear = function(self, data)
        local army_line = data.march_info

        if army_line.action == common_pb.EMARCH_SPY and army_line.dstid == self.monster.id then
            self.tip_panel.localScale = Vector3.zero
            self.mass_detect.localScale = Vector3.zero
            self.player_detect.localScale = Vector3.zero
            self.detecting_panel.localScale = Vector3.one

            self.total_time = army_line.totalTime * 0.001
            self.left_time = (army_line.totalTime - army_line.elapsedTime) * 0.001
        end
    end,

    detectCountDown = function(self)
        local fill = self.detecting_panel:Find("FillBG/Fill"):GetComponent(UnityEngine.UI.Image)
        local time_text = self.detecting_panel:Find("FillBG/LeftTime"):GetComponent(UnityEngine.UI.Text)

        fill.fillAmount = 1 - self.left_time / self.total_time
        time_text.text = gui_helper:timeFormat(self.left_time)
    end,

    getEnemyName = function(self)
        local name = ""
        if self.monster.type == hold_types.monster then
            local cfg = config.worldmap_monsters[self.monster.cfg_id]
            if cfg then
                name = self.helper:getLangCfg(cfg.monstername)
            end
        elseif self.monster.type == hold_types.player then
            name = self.monster.player_name
            
            if string.len(self.monster.short_name) > 2 then
                name = string.format( "[%s]%s", self.monster.short_name, name )
            end
        elseif self.monster.type == hold_types.titanMine then
            if string.len(self.monster.info.playerId) == 0 then
                name = frame_helper:getLangCfg(self.monster.name_id)
            else
                name = self.monster.info.playerName

                if string.len(self.monster.info.familyShortName) > 2 then
                    name = string.format( "[%s]%s", self.monster.info.familyShortName, name )
                end
            end
        end

        return name
    end,

    getMonsterSoldiers = function(self, group_id)
    
        local soldiers = nil

        local group_cfg = config.monster_group_ids[group_id]

        if group_cfg then
            local list = gui_helper:getCorps(group_cfg)

            if soldiers then
                for k1, v1 in pairs(soldiers) do
                    if list[k1] then
                        list[k1].count = list[k1].count + v1.count
                    else
                        list[k1] = v1
                    end
                end
            end

            soldiers = list
        end

        return soldiers or {}
    end,

    getReport = function(self)
        if self.action == common_pb.EMARCH_CREATE_MASS then
            network_sender:getBattleReport(local_player.id, self.monster.id, common_pb.MT_SPYREPORT)
        elseif self.action == common_pb.EMARCH_JOIN_MASS then
            network_sender:getBattleReport(self.mass_info.player_id, self.monster.id, common_pb.MT_SPYREPORT)
        elseif self.action == common_pb.EMARCH_ATK then
            network_sender:getBattleReport(local_player.id, self.monster.id, common_pb.MT_SPYREPORT)
        end
    end,

    actionDiff = function(self, has_report)
        local cargo_text = self.transform:Find("BG/Content/Top/Cargo/Text"):GetComponent(UnityEngine.UI.Text)
        local power_text = self.transform:Find("BG/Content/Top/Power/Text"):GetComponent(UnityEngine.UI.Text)
        local send_btn = self.transform:Find("BG/Content/Buttons/Send")
        local rally_btn = self.transform:Find("BG/Content/Buttons/StartRally")
        local mass_time = self.mass_detect:Find("FinishTime/Text"):GetComponent(UnityEngine.UI.Text)
        local power_node = self.transform:Find("BG/Content/Top/Power")
        local enemy_node = self.transform:Find("BG/Content/Top/Enemy")
        local res_node = self.transform:Find("BG/Content/Top/ResName")
        local top_bg = self.transform:Find("BG/Content/Top"):GetComponent(UnityEngine.UI.Image)
        local power_icon = self.transform:Find("BG/Content/Top/Power/Icon_1")
        local rally_icon = self.transform:Find("BG/Content/Top/Power/Icon_2")

        power_icon:SetActive(self.action ~= common_pb.EMARCH_CREATE_MASS)
        rally_icon:SetActive(self.action == common_pb.EMARCH_CREATE_MASS)

        if self.action == common_pb.EMARCH_CREATE_MASS then
            cargo_text.text = NativeText.GetLangStr(280418)
            power_text.text = NativeText.GetLangStr(280419)
            self.power.text = self.helper:itemNumFormat(local_player:massSoldierMax())
            self.cargo.text = local_player:massTeamMax()

            send_btn.localScale = Vector3.zero
            rally_btn.localScale = Vector3.one
            res_node.localScale = Vector3.zero

            self.player_detect.localScale = Vector3.zero
            self.mass_detect.localScale = Vector3.one
            self.gather_panel.localScale = Vector3.zero

            mass_time.text = gui_helper:timeFormat(self.duration * 60).." "..NativeText.GetLangStr(280401)
        elseif self.action == common_pb.EMARCH_JOIN_MASS then
            cargo_text.text = NativeText.GetLangStr(280406)
            power_text.text = NativeText.GetLangStr(280405)
            self.power.text = 0
            self.cargo.text = 0

            send_btn.localScale = Vector3.one
            rally_btn.localScale = Vector3.zero
            res_node.localScale = Vector3.zero

            self.player_detect.localScale = Vector3.zero
            self.mass_detect.localScale = Vector3.one
            self.gather_panel.localScale = Vector3.zero

            if not self.can_spy then
                local btn = self.mass_detect:Find("Image/Detect")
                btn:SetActive(false)
            end

            self.mass_coroutine = app:startCoroutine(
                function()
                    while self.mass_info.mass_time > 0 do
                        self.mass_info.mass_time = self.mass_info.mass_time - 1
                        mass_time.text = gui_helper:timeFormat(self.mass_info.mass_time).." "..NativeText.GetLangStr(280401)
                        coroutine.yield(WaitForSeconds(1.0))
                    end
                end
            )
        elseif self.action == common_pb.EMARCH_ATK then
            cargo_text.text = NativeText.GetLangStr(280406)
            power_text.text = NativeText.GetLangStr(280405)
            self.power.text = 0
            self.cargo.text = 0

            send_btn.localScale = Vector3.one
            rally_btn.localScale = Vector3.zero
            res_node.localScale = Vector3.zero

            self.player_detect.localScale = Vector3.one
            self.mass_detect.localScale = Vector3.zero
            self.gather_panel.localScale = Vector3.zero

            local victory = self.player_detect:Find("Victory")
            local defeat = self.player_detect:Find("Defeat")
            victory.localScale = Vector3.zero
            defeat.localScale = Vector3.zero

        elseif self.action == common_pb.EMARCH_MONSTER then
            cargo_text.text = NativeText.GetLangStr(280406)
            power_text.text = NativeText.GetLangStr(280405)
            self.power.text = 0
            self.cargo.text = 0

            send_btn.localScale = Vector3.one
            rally_btn.localScale = Vector3.zero
            res_node.localScale = Vector3.zero

            self.player_detect.localScale = Vector3.one
            self.mass_detect.localScale = Vector3.zero
            self.gather_panel.localScale = Vector3.zero

            local btn = self.player_detect:Find("Detect")
            btn.localScale = Vector3.zero

            btn = self.player_detect:Find("Report")
            btn.localScale = Vector3.zero

            if self.monster.cfg.attack == 4 then
                local victory = self.player_detect:Find("Victory")
                local defeat = self.player_detect:Find("Defeat")
                victory.localScale = Vector3.zero
                defeat.localScale = Vector3.zero
            end
            --self.subdue_info.localScale = Vector3.one
        elseif self.action == common_pb.EMARCH_GATHER_TITAN then
            cargo_text.text = NativeText.GetLangStr(280406)
            power_text.text = NativeText.GetLangStr(280405)
            self.power.text = 0
            self.cargo.text = 0

            send_btn.localScale = Vector3.one
            rally_btn.localScale = Vector3.zero
            power_node.localScale = Vector3.zero
            enemy_node.localScale = Vector3.zero

            self.enemy_panel.localScale = Vector3.zero
            
            top_bg:SetSprite(res_path.res_path_116)
        elseif self.action == common_pb.EMARCH_ROB_TITAN then
            cargo_text.text = NativeText.GetLangStr(280406)
            power_text.text = NativeText.GetLangStr(280405)
            self.power.text = 0
            self.cargo.text = 0

            send_btn.localScale = Vector3.one
            rally_btn.localScale = Vector3.zero
            res_node.localScale = Vector3.zero

            self.player_detect.localScale = Vector3.zero
            self.mass_detect.localScale = Vector3.zero
            self.gather_panel.localScale = Vector3.zero
        end
    end,

    initSoldierList = function(self)
        local cnt = 0

        for k, v in pairs(self.soldier_list) do
            local elem = nil

            if cnt >= self.corps_content.childCount then
                elem = Object.Instantiate(self.corps_prefab).transform
                elem:SetParent(self.corps_content)
                resetTransformZ(elem)
            else
                elem = self.corps_content:GetChild(cnt)
            end

            elem:SetActive(true)
            elem.name = "soldier_"..k

            local icon = elem:Find("Grade/Icon"):GetComponent(UnityEngine.UI.Image)
            local grade = elem:Find("Grade"):GetComponent(UnityEngine.UI.Image)
            local type_icon = elem:Find("Grade/TypeIcon"):GetComponent(UnityEngine.UI.Image)
            local own_count = elem:Find("Grade/Count"):GetComponent(UnityEngine.UI.Text)
            local name = elem:Find("Name"):GetComponent(UnityEngine.UI.Text)
            local append_count = elem:Find("Count"):GetComponent(UnityEngine.UI.Text)
            local slider = elem:Find("Slider"):GetComponent(UnityEngine.UI.Slider)

            own_count.text = v
            append_count.text = 0
            slider.value = 0

            local soldier_cfg = config.soldier_soldiers[k]

            if soldier_cfg then
                name.text = self.helper:getLangCfg(soldier_cfg.soldiername)
                icon:SetSprite(soldier_cfg.headpath..soldier_cfg.soldierhead)
                type_icon:SetSprite(soldier_cfg.iconpath..soldier_cfg.soldiericon)
                grade:SetSprite(gui_helper:getNewQualityIcon(soldier_cfg.color))
            end

            self.soldier_ui[k] = elem

            slider.onValueChanged:RemoveAllListeners()
            slider.onValueChanged:AddListener(
                function ()
                    self:onValueChanged(k)
                end
            )

            local path = "BG/Content/Scroll View/Viewport/Content/"..elem.name.."/Plus"
            gui_helper:registerLongPress(self, path, self.click_Plus, k)

            path = "BG/Content/Scroll View/Viewport/Content/"..elem.name.."/Minus"
            gui_helper:registerLongPress(self, path, self.click_Minus, k)

            self.append_soldiers[k] = {cfg_id = k, count = 0}

            cnt = cnt + 1
        end

        for i=cnt, self.corps_content.childCount - 1 do
            local elem = self.corps_content:GetChild(i)
            elem:SetActive(false)
        end
    end,

    setSubduePercent = function(self)
        if not self.enemy_soldiers or not self.append_soldiers then
            self.subdue_percent.text = "???"
        else
            self.subdue_factor = world_map_helper:corpsSubdued(self.append_soldiers, self.enemy_soldiers)

            if self.subdue_factor >= 0 then
                self.subdue_percent.text = string.format("+%.2f%%", self.subdue_factor*100)
                self.subdue_percent.color = Color(6/255, 243/255, 153/255, 1)
            else
                self.subdue_percent.text = string.format("-%.2f%%", math.abs(self.subdue_factor*100))
                self.subdue_percent.color = Color(1, 0, 0, 1)
            end
        end
    end,

    setEnemyInfo = function(self)
        if self.monster.type == hold_types.monster then
            if self.action == common_pb.EMARCH_MONSTER then

                if self.monster.cfg.attack ~= 4 then
                    self.enemy_soldiers = self:getMonsterSoldiers(self.monster.info.groupid)
                    local power = world_map_helper:calculateTeamPower({}, self.enemy_soldiers, true)
                    self:setEnemyPower(power)
                    self:setEnemySoldiers(self.enemy_soldiers)
                    self.subdue_info.localScale = Vector3.one
                else
                    self:setEnemyPower()
                end
            else
                self:setEnemyPower()
            end

            self:setEnemyName()
            self:setEnemyIcon()
            self:setEnemyPic()
        elseif self.monster.type == hold_types.player then
            self:setEnemyPower()
            self:setEnemyName()
            self:setEnemyPic()
        elseif self.monster.type == hold_types.titanMine then
            if self.action == common_pb.EMARCH_GATHER_TITAN then
                self:setResInfo()
            elseif self.action == common_pb.EMARCH_ROB_TITAN then
                self:setEnemyPower()
                self:setEnemyName()
            end
        end
    end,

    setEnemyIcon = function(self)
        local monster_icon = self.transform:Find("BG/Content/Top/Enemy/Icon"):GetComponent(UnityEngine.UI.Image)
        monster_icon:SetSprite(self.monster.cfg.iconheadpath..self.monster.cfg.monsteheadricon)
    end,

    setEnemyPic = function(self)
        local img = self.enemy_panel:Find("BG/Icon"):GetComponent(UnityEngine.UI.Image)

        if self.monster.type == hold_types.player then
            local cfg = config.worldmap_base_configs[self.monster.cfg_id]

            if cfg then
                img:SetSprite(cfg.iconpath..cfg.iconid)
            end
        elseif self.monster.type == hold_types.monster then
            local cfg = config.worldmap_monsters[self.monster.cfg_id]

            if cfg then
                img:SetSprite(cfg.iconpath..cfg.monstericon)
            end
        end
    end,

    setEnemyName = function(self)
        local monster_name = self.transform:Find("BG/Content/Top/Enemy/Name"):GetComponent(UnityEngine.UI.Text)
        monster_name.text = self:getEnemyName()
    end,

    setEnemyPower = function(self, value)
        local power = self.transform:Find("BG/Content/Top/Enemy/Power"):GetComponent(UnityEngine.UI.Text)
        power.text = value or "???"
        self.enemy_power = value
    end,

    setEnemySoldiers = function(self, soldiers)
        local corps_info = self.enemy_panel:Find("Info")

        local cnt = 0
        for k, v in pairs(soldiers) do
            if cnt < corps_info.childCount then
                local elem = corps_info:GetChild(cnt)
                local name = elem:GetComponent(UnityEngine.UI.Text)
                local count = elem:Find("Value"):GetComponent(UnityEngine.UI.Text)

                local corps_cfg = config.soldier_soldiers[v.cfg_id]

                if corps_cfg then
                    name.text = self.helper:getLangCfg(corps_cfg.soldiername)
                    local times = self:monsterSoldierCntTimes()
                    count.text = self.helper:itemNumFormat(v.count * times)
                    log.info("################################# count ", v.count, times)
                end

                elem.localScale = Vector3.one

                cnt = cnt + 1
            end
        end

        for i=cnt, corps_info.childCount - 1 do
            local elem = corps_info:GetChild(i)
            elem.localScale = Vector3.zero
        end

        self:setSoldierPercent()
        self:finalResult()
    end,

    monsterSoldierCntTimes = function(self)
        if self.monster.type == hold_types.monster and self.monster.cfg.attack == 4 then
            local cur_hp = self.monster.info.curHp
            local count = 0

            for k, v in pairs(self.enemy_soldiers) do
                count = count + v.count
            end

            local m = math.ceil(cur_hp / (count > 0 and count or 1))

            return m
        end

        return 1
    end,

    setSoldierPercent = function(self)
        if not self.enemy_soldiers or not self.append_soldiers then
            return
        end

        local own_soldiers = {}
        local enemy_soldiers = {}
        local own_amount = 0
        local enemy_amount = 0

        local typeSoldiers = function(soldiers, table)
            for k, v in pairs(soldiers) do
                local cfg = config.soldier_soldiers[v.cfg_id]

                if cfg then
                    if table[cfg.type] then
                        table[cfg.type].count = table[cfg.type].count + v.count
                    else
                        table[cfg.type] = {}
                        table[cfg.type].count = v.count
                        table[cfg.type].type_name = cfg.TypeName
                    end
                end
            end
        end

        typeSoldiers(self.append_soldiers, own_soldiers)
        typeSoldiers(self.enemy_soldiers, enemy_soldiers)

        local list = {}
        
        for k, v in pairs(own_soldiers) do
            if list[k] then
                list[k].own = list[k].own + v.count
            else
                list[k] = {own = v.count, enemy = 0, type_name = v.type_name}
            end

            own_amount = own_amount + v.count
        end

        for k, v in pairs(enemy_soldiers) do
            if list[k] then
                list[k].enemy = list[k].enemy + v.count
            else
                list[k] = {own = 0, enemy = v.count, type_name = v.type_name}
            end

            enemy_amount = enemy_amount + v.count
        end

        local panel = self.soldier_percent:Find("Soldiers")
        local counter = self.soldier_percent:Find("Counter"):GetComponent(UnityEngine.UI.Text)
        local up_img = self.soldier_percent:Find("Counter/Up")
        local down_img = self.soldier_percent:Find("Counter/Down")

        local percent = self.subdue_factor
        
        if percent > 0 then
            counter.text = string.format("+%.2f%%", percent*100)
            counter.color = Color(6/255, 243/255, 153/255, 1)
            up_img.localScale = Vector3.one
            down_img.localScale = Vector3.zero
        elseif percent == 0 then
            counter.text = string.format("+%.2f%%", percent*100)
            counter.color = Color(6/255, 243/255, 153/255, 1)
            up_img.localScale = Vector3.zero
            down_img.localScale = Vector3.zero
        else
            counter.text = string.format("-%.2f%%", math.abs(percent*100))
            counter.color = Color(1, 0, 0, 1)
            up_img.localScale = Vector3.zero
            down_img.localScale = Vector3.one
        end

        local cnt = 0
        for k, v in pairs(list) do
            local own_percent = v.own / (own_amount > 0 and own_amount or 1)
            local enemy_percent = v.enemy / (enemy_amount > 0 and enemy_amount or 1)

            if cnt < panel.childCount then
                local elem = panel:GetChild(cnt)
                elem.localScale = Vector3.one
                local name = elem:GetComponent(UnityEngine.UI.Text)
                local value_1 = elem:Find("Value1"):GetComponent(UnityEngine.UI.Text)
                local value_2 = elem:Find("Value2"):GetComponent(UnityEngine.UI.Text)
                local fill_img = elem:Find("Percent/Image"):GetComponent(UnityEngine.UI.Image)

                name.text = self.helper:getLangCfg(v.type_name)
                value_1.text = string.format( "%.0f%%", own_percent*100 )
                value_2.text = string.format( "%.0f%%", enemy_percent*100 )

                if own_percent == enemy_percent then
                    fill_img.fillAmount = 0.5
                else
                    local sum = own_percent + enemy_percent
                    fill_img.fillAmount = own_percent / (sum > 0 and sum or 1)
                end

                cnt = cnt + 1
            end
        end

        for i=cnt, panel.childCount - 1 do
            local elem = panel:GetChild(i)
            elem.localScale = Vector3.zero
        end
    end,

    loadOfficers = function(self)
        local heros = hero_manager:getUnDispatchedHeros()
        table.sort( heros, function(a,b) return a.officer.cfg.color > b.officer.cfg.color end )

        for k, v in ipairs(heros) do
            local elem = nil

            if k > self.officer_content.childCount then
                elem = Object.Instantiate(self.officer_prefab).transform
                elem:SetParent(self.officer_content)
                resetTransformZ(elem)
            else
                elem = self.officer_content:GetChild(k-1)
            end

            elem.name = "officer_"..v.id

            local icon = elem:Find("Head"):GetComponent(UnityEngine.UI.Image)
            local grade = elem:Find("Head/Grade"):GetComponent(UnityEngine.UI.Image)
            local level = elem:Find("Head/Level"):GetComponent(UnityEngine.UI.Text)
            local type = elem:Find("Type"):GetComponent(UnityEngine.UI.Image)
            local name = elem:Find("Type/Text"):GetComponent(UnityEngine.UI.Text)
            local cargo_text = elem:Find("Cargo/Text"):GetComponent(UnityEngine.UI.Text)
            local toggle = elem:Find("Toggle/Image")
            local stars = elem:Find("Head/Stars")

            local cfg = v.officer.cfg
            icon:SetSprite(gui_helper:getOfficerIconPath(v.officer))
            grade:SetSprite(self:getGradeRes(cfg.color))
            type:SetSprite(cfg.icon_path..cfg.soldiertypeicon)
            name.text = self.helper:getLangCfg(cfg.soldier_typename)
            level.text = v.officer.level
            cargo_text.text = self.helper:itemNumFormat(v:getGlobalProp(1171))
            
            if self:officerIsAppend(v.id) then
                toggle.localScale = Vector3.one
            else
                toggle.localScale = Vector3.zero
            end

            gui_helper:setOfficerStars(v.officer.star_level, stars)

            self:clearScript("click", "BG/Content/CommanderPanel/CommanderList/Viewport/Content/"..elem.name)
            self:clearScript("click", "BG/Content/CommanderPanel/CommanderList/Viewport/Content/"..elem.name.."/Head/Grade")
            self:setScript("click", "BG/Content/CommanderPanel/CommanderList/Viewport/Content/"..elem.name, functor.rmake(self.click_officer, v.id))
            self:setScript("click", "BG/Content/CommanderPanel/CommanderList/Viewport/Content/"..elem.name.."/Head/Grade", functor.rmake(self.click_Head, v.id))

            self.officer_toggles[v.id] = toggle
        end

        for i=#heros, self.officer_content.childCount - 1 do
            local elem = self.officer_content:GetChild(i)
            elem.localScale = Vector3.zero
        end

        self:clearScript("click", "BG/Content/CommanderPanel/Confirm")
        self:setScript("click", "BG/Content/CommanderPanel/Confirm", functor.make(self.closeOfficerPanel, self))
    end,

    getGradeRes = function(self, color)
        local path = res_path.res_path_95

        if color == 1 then
            path = res_path.res_path_96
        elseif color == 2 then
            path = res_path.res_path_97
        elseif color == 3 then
            path = res_path.res_path_98
        elseif color == 4 then
            path = res_path.res_path_99
        elseif color == 5 then
            path = res_path.res_path_100
        end

        return path
    end,

    officerIsAppend = function(self, id)
        for k, v in ipairs(self.append_officers) do
            if v == id then
                return true, k
            end
        end

        return false
    end,

    appendOfficer = function(self, id)
        local idx = #self.append_officers
        local head_data = self.officer_heads[idx]

        if head_data then
            local hero = hero_manager:getHero(id)

            if hero then
                head_data.id = id
                local ui = head_data.ui
                local head = ui:Find("Head")
                local icon = ui:Find("Head/Icon"):GetComponent(UnityEngine.UI.Image)
                local grade = ui:Find("Head/Grade"):GetComponent(UnityEngine.UI.Image)
                local level = ui:Find("Head/Level"):GetComponent(UnityEngine.UI.Text)
                local stars = ui:Find("Head/Stars")

                head.transform.localScale = Vector3.one

                icon:SetSprite(gui_helper:getOfficerIconPath(hero.officer))
                grade:SetSprite(self:getGradeRes(hero.officer.cfg.color))
                level.text = hero.officer.level
                gui_helper:setOfficerStars(hero.officer.star_level, stars)
                
                if self.action == common_pb.EMARCH_GATHER_TITAN then
                    self:setGatherSpeed()
                end

                self:setPower()
                self:setCargo()
                self:setSoldiersAmount()
                self:finalResult()
                self:setMaxCount()
            else
                log.error("####################################### hero id error ! id ", id)
            end
        end
    end,

    disappendOfficer = function(self, id)
        for k, v in ipairs(self.officer_heads) do
            if v.id == id then
                local head = v.ui:Find("Head")
                head.localScale = Vector3.zero

                --local grade = v.ui:Find("Grade"):GetComponent(UnityEngine.UI.Image)
                --grade:SetSprite(self:getGradeRes(0))

                v.id = 0
                v.ui:SetAsLastSibling()
            end
        end
        
        local sortFunc = function(a, b)
            local sibling_a = a.ui:GetSiblingIndex()
            local sibling_b = b.ui:GetSiblingIndex()

            return sibling_a < sibling_b
        end

        table.sort( self.officer_heads, sortFunc )

        if self.action == common_pb.EMARCH_GATHER_TITAN then
            self:setGatherSpeed()
        end

        self:setPower()
        self:setCargo()
        self:setSoldiersAmount()
        self:finalResult()
    end,

    soldierCntCheck = function(self)
        local max = world_map_helper:teamSoldiersAmount(self.append_officers)
        if max < self.total_soldiers then
            for k, v in pairs(self.append_soldiers) do
                if v.count > 0 then
                    self.changed = true
                    self:changeSoldierCount(v.cfg_id, v.count*-1)
                    v.count = 0
                end
            end
        end
    end,

    setOfficerDetails = function(self, id)
        local hero = hero_manager:getHero(id)

        if hero then
            local icon = self.officer_details:Find("Head"):GetComponent(UnityEngine.UI.Image)
            local grade = self.officer_details:Find("Head/Grade"):GetComponent(UnityEngine.UI.Image)
            local type_icon = self.officer_details:Find("Head/Type"):GetComponent(UnityEngine.UI.Image)
            local type_name = self.officer_details:Find("Head/Type/Text"):GetComponent(UnityEngine.UI.Text)
            local level = self.officer_details:Find("Head/Level"):GetComponent(UnityEngine.UI.Text)
            local stars = self.officer_details:Find("Head/Stars")
            local name = self.officer_details:Find("Head/Name"):GetComponent(UnityEngine.UI.Text)
            local cargo_text = self.officer_details:Find("Head/Cargo/Text"):GetComponent(UnityEngine.UI.Text)

            local cfg = hero.officer.cfg
            icon:SetSprite(gui_helper:getOfficerIconPath(hero.officer))
            grade:SetSprite(self:getGradeRes(hero.officer.cfg.color))
            type_icon:SetSprite(cfg.icon_path..cfg.soldiertypeicon)
            type_name.text = self.helper:getLangCfg(cfg.soldier_typename)
            level.text = hero.officer.level
            name.text = hero.officer.name
            gui_helper:setOfficerStars(hero.officer.star_level, stars)
            cargo_text.text = self.helper:itemNumFormat(hero:getGlobalProp(1171))

            local skill_panel = self.officer_details:Find("Panel")
            local skills = hero.officer.skills

            for i=1, skill_panel.childCount do
                local elem = skill_panel:GetChild(i-1)
                local data = skills[i]

                if data then
                    local skill_lv = data.skillLv > 0 and data.skillLv or 1
                    local cfg = passive_skill_helper:getSkill(data.skillId, skill_lv)

                    if not cfg then
                        log.error("###### officer talent cfg error ! id "..data.skillId.." level "..skill_lv)
                    else
                        local icon = elem:GetComponent(UnityEngine.UI.Image)
                        local name = elem:Find("Name"):GetComponent(UnityEngine.UI.Text)
                        local des = elem:Find("Des"):GetComponent(UnityEngine.UI.Text)
                        local lock_img = elem:Find("Locked")

                        icon:SetSprite(cfg.skill_icon_path..cfg.skill_icon_name)
                        name.text = self.helper:getLangCfg(cfg.lang_id)
                        des.text = self.helper:getLangCfg(cfg.skill_des)

                        --local unlock_star = hero.officer:getSkillUnlockStar(i)
                        --local star_lv = hero.officer.star_level

                        --if star_lv < unlock_star then
                        --    lock_img.localScale = Vector3.one
                        --else
                            lock_img.localScale = Vector3.zero
                        --end

                        elem.localScale = Vector3.one
                    end
                else
                    elem.localScale = Vector3.zero
                end
            end
        end
    end,

    setOfficerProps = function(self)
        local props = {}

        for k, v in ipairs(self.append_officers) do
            local hero = hero_manager:getHero(v)

            if hero then
                for _, prop in ipairs(hero.global_props) do
                    if props[prop.PropId] then
                        props[prop.PropId] = props[prop.PropId] + prop.PropValue
                    else
                        props[prop.PropId] = prop.PropValue
                    end
                end
            end
        end

        local cargo = self.addition_info:Find("Amount/Value"):GetComponent(UnityEngine.UI.Text)
        local amount = props[1171] or 0
        cargo.text = "+"..self.helper:itemNumFormat(amount)

        local node = self.addition_info:Find("Scroll View/Viewport/Content")
        local prefab = node:GetChild(0).gameObject
        local cnt = 0

        for k, v in pairs(props) do
            if v > 0 then
                local cfg = config.player_config_attrs[k]

                if cfg then
                    local elem = nil

                    if cnt >= node.childCount then
                        elem = Object.Instantiate(prefab).transform
                        elem:SetParent(node)
                        resetTransformZ(elem)
                    else
                        elem = node:GetChild(cnt)
                    end

                    elem:SetActive(true)

                    local name = elem:GetComponent(UnityEngine.UI.Text)
                    local value = elem:GetChild(0):GetComponent(UnityEngine.UI.Text)

                    name.text = self.helper:getLangCfg(cfg.name_id)

                    local val = v
                    if cfg.isfloat > 0 then
                        val = val * 0.0001
                    end

                    if cfg.showpercent > 0 then
                        val = val*100 .."%"
                    end

                    value.text = string.format( "+%s", tostring(val) )

                    cnt = cnt + 1
                else
                    log.info("########################################### player config attrs error ! id", k)
                end
            end
        end

        for i=cnt, node.childCount - 1 do
            local elem = node:GetChild(i)
            elem:SetActive(false)
        end
    end,

    onValueChanged = function(self, idx)
        if not self.changed then
            local elem = self.soldier_ui[idx]
            local slider = elem:Find("Slider"):GetComponent(UnityEngine.UI.Slider)
            local own_count = elem:Find("Grade/Count"):GetComponent(UnityEngine.UI.Text)
            local append_count = elem:Find("Count"):GetComponent(UnityEngine.UI.Text)
            local count = math.ceil(slider.value * self.soldier_list[idx])

            local append_soldier = self.append_soldiers[idx]
            local value = count - append_soldier.count

            if self.total_soldiers + value > self.append_max then
                value = self.append_max - self.total_soldiers
            end

            self.total_soldiers = self.total_soldiers + value
            append_soldier.count = append_soldier.count + value
    
            own_count.text = self.soldier_list[idx] - append_soldier.count
            append_count.text = append_soldier.count

            if self.total_soldiers >= self.append_max then
                slider.value = append_soldier.count / self.soldier_list[idx]
            end

            self:setSpeed()
            self:setSubduePercent()
            self:setPower()
            self:setCargo()
            self:setSoldiersAmount()
            self:setSoldierPercent()
            self:finalResult()

            if self.action == common_pb.EMARCH_GATHER_TITAN then
                self:setEsimatedGatherCount()
                self:setGatherTime()
            end
        end

        self.changed = false
    end,

    changeSoldierCount = function(self, id, value)
        if self.total_soldiers + value > self.append_max then
            value = self.append_max - self.total_soldiers
        end

        self.total_soldiers = self.total_soldiers + value

        local elem = self.soldier_ui[id]
        local slider = elem:Find("Slider"):GetComponent(UnityEngine.UI.Slider)
        local own_count = elem:Find("Grade/Count"):GetComponent(UnityEngine.UI.Text)
        local append_count = elem:Find("Count"):GetComponent(UnityEngine.UI.Text)

        local append_soldier = self.append_soldiers[id]
        append_soldier.count = append_soldier.count + value

        own_count.text = self.soldier_list[id] - append_soldier.count
        append_count.text = append_soldier.count
        local percent = append_soldier.count / self.soldier_list[id]

        if slider.value == percent then
            self.changed = false
        else
            slider.value = percent
        end

        self:setSpeed()
        self:setSubduePercent()
        self:setPower()
        self:setCargo()
        self:setSoldiersAmount()
        self:setSoldierPercent()
        self:finalResult()

        if self.action == common_pb.EMARCH_GATHER_TITAN then
            self:setEsimatedGatherCount()
            self:setGatherTime()
        end
    end,

    finalResult = function(self)
        if not self.enemy_power or not self.team_power then
            return
        end

        local victory = self.player_detect:Find("Victory")
        local defeat = self.player_detect:Find("Defeat")

        local final_power = self.team_power * (1 + self.subdue_factor)

        if final_power > self.enemy_power then
            victory:SetActive(true)
            defeat:SetActive(false)
        else
            victory:SetActive(false)  
            defeat:SetActive(true)
        end
    end,

    setSpeed = function(self)
        local speed = 9999
        for k, v in pairs(self.append_soldiers) do
            if v.count > 0 then
                local cfg = config.soldier_soldiers[v.cfg_id]

                if cfg then
                    speed = speed < cfg.speed and speed or cfg.speed
                end
            end
        end

        if speed ~= self.speed and speed ~= 9999 then
            self.speed = speed
            self:setTime()
        end
    end,

    setPower = function(self)
        if self.action ~= common_pb.EMARCH_CREATE_MASS and self.action ~= common_pb.EMARCH_GATHER_TITAN then
            self.team_power = world_map_helper:calculateTeamPower(self.append_officers, self.append_soldiers)

            local power = self.team_power
            if power > 0 then
                power = power * (1 + self.subdue_factor)
                power = math.floor(power) > 0 and math.floor(power) or 1
            end
            self.power.text = tostring(power)
        end
    end,

    setCargo = function(self)
        if self.action ~= common_pb.EMARCH_CREATE_MASS then
            local value = player_cfg_helper:teamCargo(self.append_officers, self.append_soldiers)
            self.cargo.text = self.helper:itemNumFormat(value)
        end
    end,

    setSoldiersAmount = function(self)
        local value = world_map_helper:teamSoldiersAmount(self.append_officers)
        self.amount.text = string.format( "%s/%s", self.helper:itemNumFormat(self.total_soldiers), self.helper:itemNumFormat(value) ) 
    end,

    setResInfo = function(self)
        local name = self.transform:Find("BG/Content/Top/ResName"):GetComponent(UnityEngine.UI.Text)
        local pos = self.transform:Find("BG/Content/Top/ResName/Position"):GetComponent(UnderLineText)
        local icon = self.gather_panel:Find("IconBG/Icon"):GetComponent(UnityEngine.UI.Image)
        local reserves_text = self.gather_panel:Find("Info/Reserves/Value"):GetComponent(UnityEngine.UI.Text)

        name.text = self:getEnemyName()

        local cb = function()
            gui_helper:positionJump(self.monster.coordX, self.monster.coordY)
            self:click_BackBtn()
        end
        pos:SetText(gui_helper:positionFormat(self.monster.coordX, self.monster.coordY), cb)

        icon:SetSprite(self.monster.cfg.iconpath..self.monster.cfg.iconid)
        reserves_text.text = string.format( "%s", self.helper:itemNumFormat(self.monster.info.resAmmount) )

        self:setEsimatedGatherCount()
        self:setGatherSpeed()
        self:setGatherTime()
    end,

    setGatherSpeed = function(self)
        local speed_text = self.gather_panel:Find("Info/Speed/Value"):GetComponent(UnityEngine.UI.Text)

        local base_speed = self.monster.cfg.gatherspeed * 3600
        local add_percent = player_cfg_helper:getResGatherSpeedPercent(self.monster.cfg.type, self.append_officers)
        local add_value = math.floor(base_speed * add_percent * 0.0001)

        self.gather_speed = base_speed + add_value

        if add_value > 0 then
            speed_text.text = string.format( "%s/h<color=#0BFF78FF>+%s/h</color>", self.helper:itemNumFormat(base_speed), self.helper:itemNumFormat(add_value) )
        else
            speed_text.text = string.format( "%s/h", self.helper:itemNumFormat(base_speed) )
        end
    end,

    setEsimatedGatherCount = function(self)
        local estimated_text = self.gather_panel:Find("Info/Estimated/Value"):GetComponent(UnityEngine.UI.Text)

        local cargo = player_cfg_helper:teamCargo(self.append_officers, self.append_soldiers)
        local value = math.min(self.monster.info.resAmmount, cargo)

        self.gather_amount = value

        estimated_text.text = string.format( "%s", self.helper:itemNumFormat(value) )
    end,

    setGatherTime = function(self)
        local time_text = self.gather_panel:Find("Info/Time/Value"):GetComponent(UnityEngine.UI.Text)

        local speed = self.gather_speed / 3600
        local value = self.gather_amount / (speed > 0 and speed or 1)

        time_text.text = gui_helper:timeFormat(value)
    end,

    setTime = function(self)
        if self.chapter_id then
            return
        end

        local dis = 0
        if self.action == common_pb.EMARCH_JOIN_MASS then
            local pos_1 = {}
            pos_1.x = self.mass_info.start_pos.posX
            pos_1.z = self.mass_info.start_pos.posY

            local pos_2 = {}
            pos_2.x = self.mass_info.mass_pos.posX
            pos_2.z = self.mass_info.mass_pos.posY

            dis = world_map_helper:getCoordDistance(pos_1, pos_2)
        else
            dis = world_map_helper:getDistToBase({x = self.monster.coordX, z = self.monster.coordY})
        end

        local time = world_map_helper:marchTime(dis, self.speed)

        log.info("######################################################## time ", dis, self.speed, time)
        self.time_text.text = gui_helper:timeFormat(time)
    end,

    closeOfficerPanel = function(self)
        self.officer_panel.localScale = Vector3.zero
        self.corps_panel.localScale = Vector3.one

        if self.action == common_pb.EMARCH_GATHER_TITAN then
            self.gather_panel.localScale = Vector3.one
        else
            self.enemy_panel.localScale = Vector3.one
        end
        
        self:soldierCntCheck()
        self:setMaxCount()
    end,

    clickOfficerHead = function(self)
        self.officer_panel.localScale = Vector3.one
        self.addition_info.localScale = Vector3.one
        self.officer_details.localScale = Vector3.zero
        self.corps_panel.localScale = Vector3.zero
        self.enemy_panel.localScale = Vector3.zero
        self.gather_panel.localScale = Vector3.zero

        self:loadOfficers()
    end,

    click_officer = function(self, handler, id)
        self.addition_info.localScale = Vector3.one
        self.officer_details.localScale = Vector3.zero

        local toggle = self.officer_toggles[id]

        if toggle then
            local append, idx = self:officerIsAppend(id)
            if append then
                toggle.localScale = Vector3.zero
                table.remove( self.append_officers, idx )
                self:disappendOfficer(id)
            elseif #self.append_officers < 5 then
                toggle.localScale = Vector3.one
                table.insert( self.append_officers, id )
                self:appendOfficer(id)
            end
        end

        self:setOfficerProps()
    end,

    click_Head = function(self, handler, id)
        self.addition_info.localScale = Vector3.zero
        self.officer_details.localScale = Vector3.one
        self:setOfficerDetails(id)
    end,

    click_Plus = function(self, id, value)
        local append_soldier = self.append_soldiers[id]
        value = value or 1

        if append_soldier.count < self.soldier_list[id] then
            self.changed = true
            if append_soldier.count + value > self.soldier_list[id] then
                value = self.soldier_list[id] - append_soldier.count
            end
            self:changeSoldierCount(id, value)
        end
    end,

    click_Minus = function(self, id, value)
        local append_soldier = self.append_soldiers[id]
        value = value or 1

        if append_soldier.count > 0 then
            self.changed = true
            if append_soldier.count - value < 0 then
                value = append_soldier.count
            end
            self:changeSoldierCount(id, -1*value)
        end
    end,

    click_BackBtn = function(self)
        frame_manager:hide("launch_rally")
    end,

    click_Addition = function(self)
        self:showAddition()
    end,

    click_Send = function(self)
        if self.total_soldiers <= 0 then
            hint:setText(NativeText.GetLangStr(280420))
            return
        end

        if self.action == common_pb.EMARCH_MONSTER then
            network_sender:startArmyLine(self.action, self.append_officers, self.append_soldiers, self.monster.id, nil, self.atk_cnt or 1)
        elseif self.action == common_pb.EMARCH_JOIN_MASS then
            network_sender:startArmyLine(self.action, self.append_officers, self.append_soldiers, self.mass_info.hold_id, self.mass_info.id)
        elseif self.action == common_pb.EMARCH_ATK then
            if not self.chapter_id then
                network_sender:startArmyLine(self.action, self.append_officers, self.append_soldiers, self.monster.id, nil, nil, self.origin_x, self.origin_y)
            else
                network_sender:startChapterFight(self.chapter_id, self.append_officers, self.append_soldiers, false)
            end
        elseif self.action == common_pb.EMARCH_GATHER_TITAN then
            network_sender:startArmyLine(self.action, self.append_officers, self.append_soldiers, self.monster.id)
        elseif self.action == common_pb.EMARCH_ROB_TITAN then
            network_sender:startArmyLine(common_pb.EMARCH_GATHER_TITAN, self.append_officers, self.append_soldiers, self.monster.id)
        end

        self:click_BackBtn()
    end,

    click_StartRally = function(self)
        if self.total_soldiers <= 0 then
            hint:setText(NativeText.GetLangStr(280420))
            return
        end

        if self.monster.type ~= hold_types.player then
            self.origin_x = self.monster.coordX
            self.origin_y = self.monster.coordY
        end

        network_sender:startMass(self.monster.id, self.duration, self.append_officers, self.append_soldiers, self.origin_x, self.origin_y)
        self:click_BackBtn()
    end,

    click_Auto = function(self)
        self:clearAppend()

        if self.monster.type == hold_types.titanMine then
            self:autoAppendToGather()
        else
            self:autoAppendHero()
            self:autoAppendSoldier()
        end
    end,

    click_Formation = function(self)

    end,

    click_Detect = function(self)
        network_sender:startArmyLine(common_pb.EMARCH_SPY, nil, nil, self.monster.id, nil, nil, self.origin_x, self.origin_y)
    end,
    
    click_Report = function(self)

    end,
    
    click_PropUpClose = function(self)
        self.prop_up_panel.localScale = Vector3.zero
    end,

    click_Subdue = function(self)
        if not self.enemy_soldiers then
            return
        end

        local vec = self.subdue_info.localScale
        if vec.magnitude == 0 then
            self.subdue_info.localScale = Vector3.one
            self.soldier_percent.localScale = Vector3.zero
        else
            self.subdue_info.localScale = Vector3.zero
            self.soldier_percent.localScale = Vector3.one
        end
    end,

    clearAppend = function(self)
        for k, v in ipairs(self.append_officers) do
            self:disappendOfficer(v)
        end

        self.append_officers = {}

        for k, v in pairs(self.append_soldiers) do
            if v.count > 0 then
                self.changed = true
                self:changeSoldierCount(v.cfg_id, v.count*-1)
                v.count = 0
            end
        end
    end,

    autoAppendToGather = function(self)
        self:autoAppendGatherHero()
        self:autoAppendGatherSoldier()
    end,

    autoAppendGatherHero = function(self)
        local cfg = self.monster.cfg

        local sort_func = function(a, b)
            return true
        end

        if cfg.type == 1 then
            sort_func = function(a, b)
                local a_val = player_cfg_helper:getIronGatherSpeedPercent({a.id})
                local b_val = player_cfg_helper:getIronGatherSpeedPercent({b.id})

                if a_val == b_val then
                    return a.officer.cfg.color > b.officer.cfg.color
                end

                return a_val > b_val
            end
        elseif cfg.type == 2 then
            sort_func = function(a, b)
                local a_val = player_cfg_helper:getGearGatherSpeedPercent({a.id})
                local b_val = player_cfg_helper:getGearGatherSpeedPercent({b.id})

                if a_val == b_val then
                    return a.officer.cfg.color > b.officer.cfg.color
                end

                return a_val > b_val
            end
        elseif cfg.type == 3 then
            sort_func = function(a, b)
                local a_val = player_cfg_helper:getECTGatherSpeedPercent({a.id})
                local b_val = player_cfg_helper:getECTGatherSpeedPercent({b.id})

                if a_val == b_val then
                    return a.officer.cfg.color > b.officer.cfg.color
                end

                return a_val > b_val
            end
        elseif cfg.type == 4 then
            sort_func = function(a, b)
                local a_val = player_cfg_helper:getFoodGatherSpeedPercent({a.id})
                local b_val = player_cfg_helper:getFoodGatherSpeedPercent({b.id})

                if a_val == b_val then
                    return a.officer.cfg.color > b.officer.cfg.color
                end

                return a_val > b_val
            end
        elseif cfg.type == 5 then
            sort_func = function(a, b)
                local a_val = player_cfg_helper:getTITANGatherSpeedPercent({a.id})
                local b_val = player_cfg_helper:getTITANGatherSpeedPercent({b.id})

                if a_val == b_val then
                    return a.officer.cfg.color > b.officer.cfg.color
                end

                return a_val > b_val
            end
        end

        local heros = hero_manager:getUnDispatchedHeros()
        table.sort( heros, sort_func )

        for k, v in ipairs(heros) do
            if #self.append_officers >= 5 then
                break
            end

            if not self:officerIsAppend(v.id) then
                table.insert( self.append_officers, v.id )
                self:appendOfficer(v.id)
            end
        end
    end,

    autoAppendGatherSoldier = function(self)
        local list = {}
        for k, v in pairs(self.append_soldiers) do
            table.insert( list, v )
        end

        local sort_func = function(a, b)
            local a_cfg = config.soldier_soldiers[a.cfg_id]
            local b_cfg = config.soldier_soldiers[b.cfg_id]

            if a_cfg and b_cfg then
                return a_cfg.Capacity > b_cfg.Capacity
            end

            return true
        end

        table.sort(list, sort_func)

        local total = 0
        local reserves = self.monster.info.resAmmount
        local team_1 = {}

        for k, v in ipairs(list) do
            local cfg = config.soldier_soldiers[v.cfg_id]
            local max = self.append_max - total
            local cnt = math.ceil(reserves / cfg.Capacity)
            max = math.min( max, cnt )

            local amount = self.soldier_list[v.cfg_id]
            if amount then
                local value = amount > max and max or amount
                table.insert(team_1, {cfg_id = v.cfg_id, count = value})
                total = total + value
                reserves = reserves - cfg.Capacity * value

                if reserves <= 0 then
                    break
                end
            end
        end

        for k, v in ipairs(team_1) do
            self.changed = true
            self:changeSoldierCount(v.cfg_id, v.count)
        end
    end,

    autoAppendHero = function(self)
        local heros = hero_manager:getUnDispatchedHeros()
        table.sort( heros, function(a,b) return a.officer.cfg.color > b.officer.cfg.color end )

        for k, v in ipairs(heros) do
            if #self.append_officers >= 5 then
                break
            end

            if not self:officerIsAppend(v.id) then
                table.insert( self.append_officers, v.id )
                self:appendOfficer(v.id)
            end
        end
    end,

    autoAppendSoldier = function(self)
        local all_count = 0

        for k, v in pairs(self.soldier_list) do
            all_count = all_count + v
        end

        local list = {}
        for k, v in pairs(self.append_soldiers) do
            table.insert( list, v )
        end

        local a1, a2, a3, a4 = world_map_helper:herosAddition(self.append_officers)
        local sort_func = function(a, b)
            local p_a = world_map_helper:soldierPower(a.cfg_id, a1, a2, a3, a4)
            local p_b = world_map_helper:soldierPower(b.cfg_id, a1, a2, a3, a4)

            if p_a == p_b then
                return a.cfg_id < b.cfg_id
            end

            return p_a > p_b
        end

        table.sort(list, sort_func)

        if all_count <= self.append_max or not self.enemy_soldiers then
            self:appendAllSoldiers(list)
        else
            -- 按战力选择士兵
            local total = 0
            local team_1 = {}
            for k, v in ipairs(list) do
                local max = self.append_max - total
                local amount = self.soldier_list[v.cfg_id]
                if amount then
                    local value = amount > max and max or amount
                    table.insert(team_1, {cfg_id = v.cfg_id, count = value})
                    total = total + value
                end
            end

            -- 选择最优兵种
            total = 0
            local team_2 = {}
            local type = world_map_helper:bestCorps(self.enemy_soldiers)
            log.info("#################################################### best type ", type)

            for k, v in ipairs(list) do
                local cfg = config.soldier_soldiers[v.cfg_id]
                if cfg then
                    if cfg.type == type then
                        local max = self.append_max - total
                        local amount = self.soldier_list[v.cfg_id]
                        if amount then
                            local value = amount > max and max or amount
                            table.insert(team_2, {cfg_id = v.cfg_id, count = value})
                            total = total + value
                        end
                    end
                end
            end

            local team_power = function(team)
                factor = world_map_helper:corpsSubdued(team, self.enemy_soldiers)
                local power = world_map_helper:calculateTeamPower(self.append_officers, team)
                power = power * (1 + factor)

                return power
            end
            
            local power_1 = team_power(team_1)
            local power_2 = team_power(team_2)

            if power_1 < power_2 then
                team_1 = team_2
            end

            for k, v in ipairs(team_1) do
                self.changed = true
                self:changeSoldierCount(v.cfg_id, v.count)
            end
        end
    end,

    appendAllSoldiers = function(self, list)
        for k, v in pairs(list) do
            self.changed = true
            local count = self.soldier_list[v.cfg_id]

            if count and count > 0 then
                self:changeSoldierCount(v.cfg_id, count)
            end
        end
    end,

    setMaxCount = function(self)
        self.append_max = world_map_helper:teamSoldiersAmount(self.append_officers)
    end,

    showAddition = function(self)
        local list = self:officerAdditions()
        local cnt = 0

        local node = self.prop_up_panel:Find("BG")

        for type, props in pairs(list) do
            if cnt < node.childCount then
                local elem = node:GetChild(cnt)
                elem:SetActive(true)
                
                local content = elem:Find("Panel")
                local pb = content:GetChild(0).gameObject
                local size = 0

                for id, value in pairs(props) do
                    local cfg = config.player_config_attrs[id]

                    if cfg then
                        local child = nil

                        if size < content.childCount then
                            child = content:GetChild(size)
                        else
                            child = Object.Instantiate(pb).transform
                            child:SetParent(content)
                            resetTransformZ(child)
                        end

                        child:SetActive(true)

                        local name = child:GetComponent(UnityEngine.UI.Text)
                        local val = child:GetChild(0):GetComponent(UnityEngine.UI.Text)

                        name.text = self.helper:getLangCfg(cfg.name_id)
                        
                        if cfg.isfloat > 0 then
                            value = value * 0.0001
                        end

                        if cfg.showpercent > 0 then
                            value = value*100 .."%"
                        end

                        val.text = string.format( "+%s", tostring(value) )

                        size = size + 1
                    else
                        log.error("########################################### find player config attr error ! id", id)
                    end

                    app:startCoroutine(
                        function()
                            coroutine.yield(nil)
                            local height = content.rect.height + 50
                            height = height > 183 and height or 183
                            elem.sizeDelta = Vector2(590, height)
                        end
                    )                 
                end

                for i=size, content.childCount - 1 do
                    local child = content:GetChild(i)
                    child:SetActive(false)
                end

                cnt = cnt + 1
            end
        end

        for i=cnt, node.childCount - 1 do
            local child = node:GetChild(i)
            child:SetActive(false)
        end

        if cnt > 0 then
            self.prop_up_panel.localScale = Vector3.one
        end
    end,

    officerAdditions = function(self)
        local corp_props = {}
        local props = {}

        config.commander_soldier_additionals:foreach(
            function(k, v)
                corp_props[v.SoldierType] = {}

                local value = 0

                for _, id in ipairs(self.append_officers) do
                    local hero = hero_manager:getHero(id)
                    
                    local list = loadstring("return "..v.AttributeCount)()

                    for _, prop_id in ipairs(list) do
                        value = value + hero:getGlobalProp(prop_id)
                    end
                end

                props[v.ShowAttribute] = value
            end
        )

        config.commander_soldier_additionals:foreach(
            function(k, v)
                corp_props[v.SoldierType][v.ShowAttribute] = props[v.ShowAttribute]
            end
        )

        for idx, list in pairs(corp_props) do
            local cnt = 0
            for id, value in pairs(list) do
                if value > 0 then
                    cnt = cnt + 1
                else
                    corp_props[idx][id] = nil
                end
            end

            if cnt == 0 then
                corp_props[idx] = nil
            end
        end

        return corp_props
    end,

    showAllProps = function(self)
        self.all_props_panel:SetActive(true)

        local props = {}

        for k, v in ipairs(self.append_officers) do
            local hero = hero_manager:getHero(v)

            if hero then
                for _, prop in ipairs(hero.global_props) do
                    if prop.PropValue > 0 then
                        if props[prop.PropId] then
                            props[prop.PropId] = props[prop.PropId] + prop.PropValue
                        else
                            props[prop.PropId] = prop.PropValue
                        end
                    end
                end
            end
        end

        local name_node = self.all_props_panel:Find("Frame/Center/Props/PropNames")
        local value_node = self.all_props_panel:Find("Frame/Center/Props/PropValues")
        local cnt = 0

        for id, value in pairs(props) do
            if cnt < name_node.childCount then
                local name_text = name_node:GetChild(cnt):GetComponent(UnityEngine.UI.Text)
                local value_text = value_node:GetChild(cnt):GetComponent(UnityEngine.UI.Text)

                name_text.transform:SetActive(true)
                value_text.transform:SetActive(true)

                name_text.text = tostring(id)
                value_text.text = tostring(value)

                cnt = cnt + 1
            end
        end

        for i=cnt, name_node.childCount - 1 do
            local child = name_node:GetChild(i)
            child:SetActive(false)
        end

        for i=cnt, value_node.childCount - 1 do
            local child = value_node:GetChild(i)
            child:SetActive(false)
        end
    end,

    click_PropsBG = function(self)
        self.all_props_panel:SetActive(false)
    end,
}