local log = logger.get("Game", "Game Copy Info Frame")
local super = require("ui.frame")

return class{
    super = super,

    onLoaded = function(self)
        super:onLoaded(self)

        self.owner = self.args.owner
        self.level_id = self.args.data.level_id
        self.chapter_id = self.args.data.chapter_id
        self.level_cfg = config.game_copy_levels[self.level_id]

        level_info_frame = self   -- global

        -- 路径: root/login
        -- 事件: click_login 
        clickEvents = {
            "Cover/ButtonBack",
            "Bottom/Attack",
            "Bottom/Reset",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self.helper.hideTrans(self.owner.transform)

        self.bag_slot = self.transform:Find("Prefabs/BagSlot").gameObject
        self.attend_slot = self.transform:Find("Prefabs/AttendSlot").gameObject

        self.attend_view = self.transform:Find("Right/TeamPanel/AttendView/Viewport/Content")
        self.bag_view = self.transform:Find("Right/BagView/Viewport/Content")

        self.battle_force_text = self:findText("Right/TeamPanel/gamecopy_panel_06_7/text1_Static_9")

        self.attack_btn = self.transform:Find("Bottom/Attack")

        self.attend_slots = {}
        self.attend_heros = {}
        self.max_attends = local_player.props[protocol_pb.PROP_ATTEND_NUM]

        self:init()

        self.reset_evt = game_director.events:on("onResetMapInfo", functor.make(self.onResetMapInfo, self))
        self.attend_heros_evt = game_info.events:on('attend_heros_msg', function() self:refresh()  end)

    end,

    onUnload = function(self)
        game_director.events:remove("onResetMapInfo", self.reset_evt)
        game_info.events:remove("attend_heros_msg", self.attend_heros_evt)

        level_info_frame = nil
    end,

    onResetMapInfo = function(self, msg)
        self:setLevelInfo()
    end,

    init = function(self)
        if self:isStoryLevel() then
            network_sender:reqChapterAttendHeros(self.chapter_id, self.level_id)
        end

        self:setLevelInfo()
        self:loadOfficers()
        self:loadAttendSlot()
        self:refresh()
        self:setBattleForce()
        self:setDropItems()
    end,

    -- 是否是剧情关卡，剧情关卡上阵英雄是固定的，不可替换
    isStoryLevel = function(self)
        return self.level_cfg.level_type == 2 
    end,

    setLevelInfo = function(self)
        local level_pic = self:findImage("Left/GameCopy/gamecopy_picture_01_2")
        local level_name = self:findText("Left/GameCopy/TextTemp/text13_Static_3")

        local cfg = config.game_copy_levels[self.level_id]

        level_pic:SetSprite(cfg.path_icon .. cfg.icon)
        level_name.text = self.helper:getLangCfg(cfg.Level_Name_ID)

        local cur_chapter_id, cur_level_id = self.owner:getCurChapterId()
        self.transform:Find("Bottom/Reset").gameObject:SetActive(cur_level_id == self.level_id)

        local pos = self.attack_btn.localPosition
        if cur_level_id ~= self.level_id then
            pos.x = 0
        end
        self.attack_btn.localPosition = pos
    end,

    setDropItems = function(self)
        local drop_panel = self.transform:Find("Left/Item")
        local cfg = config.game_copy_levels[self.level_id]
        gui_helper:loadDropItems(drop_panel, cfg.boss_id)
    end,

    setBattleForce = function(self)
        local force = 0
        for k, v in ipairs(self.attend_heros) do
            force = force + hero_manager.heros[v].strength
        end
        self.battle_force_text.text = tostring(force)
    end,

    -- 加载可以选择出战的指挥官
    loadOfficers = function(self)
        -- 剧情关卡不显示出战指挥官
        if self:isStoryLevel() then
            return
        end

        local officers =  hero_manager:getOwnOfficers()

        local cnt = 1
        for id, officer in pairs(officers) do
            local elem = nil
            if cnt > self.bag_view.childCount then
                elem = GameObject.Instantiate(self.bag_slot).transform
                elem:SetParent(self.bag_view)
                makeRectTransformIdentity(elem)
            else
                elem = self.bag_view:GetChild(cnt - 1)
            end
            cnt = cnt + 1

            if elem  == nil then
                return
            end

            --local image = elem:Find("Image"):GetComponent(UnityEngine.UI.Image)
            --image:SetSprite()
            gui_helper:setOfficerIcon(officer, officer.star_level, officer.quality, elem)
            local select_img = elem:Find("Selected")
            select_img:SetActive(false)

            elem.name = elem.name..officer.inst_id
            local path = "Right/BagView/Viewport/Content/"..elem.name.."/Grade/Icon"
            self:setScript("single", path, functor.rmake(self.onClick_officer, officer))

            officer.ui = elem
            officer.isAttend = false
        end
    end,

    -- 初始化出战 slot
    loadAttendSlot = function(self)
        for i=1, self.max_attends do
            self:setAttendSlot(i, nil)
        end
    end,

    refresh = function(self)
        -- 如果是剧情关卡，出站前要把获得英雄信息
        if self:isStoryLevel() then
            -- 剧情关卡，上阵英雄存储在 level_attend_heros 中
            self.heros = game_info.level_attend_heros[self.level_id]
            if self.heros then

                for k, v in ipairs(self.heros.formations) do

                    -- 创建剧情英雄
                    story_hero_manager:createStoryHero(v.heroId, v.tankId, v.officerId)
                    local officer_inst = {
                        cfg_id = v.officerId, 
                        star_level = 1, 
                        quality = 1,
                        res_id = v.officerId
                    }
                    self:setAttendSlot(k, officer_inst)
                end
            end
        else
            self.heros = game_info.chapter_attend_heros[self.chapter_id]
            if self.heros then
                for k, v in ipairs(self.heros.formations) do
                    local hero = hero_manager.heros[v.heroId]
                    if not hero then
                        log.erro("attend hero not exist ", v)
                        return
                    end
    
                    if hero.officer then
                        self:updateOfficerSlot(hero.officer)
                        self:setAttendSlot(k, hero.officer)
                    end
                end
            end
        end
    end,

    -- 更新指挥官出战张状态
    updateOfficerSlot = function(self, officer)
        officer.isAttend = not officer.isAttend

        if officer.isAttend then
            table.insert( self.attend_heros, officer.hero_id )
        else
            for k, v in ipairs(self.attend_heros) do
                if v == officer.hero_id then
                    table.remove( self.attend_heros, k )
                    break
                end
            end
        end

        local select_img = officer.ui:Find("Selected")
        select_img:SetActive(officer.isAttend)

        self:setBattleForce()
    end,

    -- 设置出战 slot
    setAttendSlot = function(self, index, officer_inst)
        local elem = nil
        if index > self.attend_view.childCount then
            elem = GameObject.Instantiate(self.attend_slot).transform
            elem:SetParent(self.attend_view)
            makeRectTransformIdentity(elem)
        else
            elem = self.attend_view:GetChild(index - 1)
        end
        
        if elem  == nil then
            return
        end

        local grade = elem:Find("Grade")
        grade:SetActive(officer_inst ~= nil)

        if officer_inst then
            gui_helper:setOfficerIcon(officer_inst, officer_inst.star_level, officer_inst.quality, elem)
        end

        self.attend_slots[index] = {officer = officer_inst, ui = elem}

        elem.name = elem.name..index
        local path = "Right/TeamPanel/AttendView/Viewport/Content/"..elem.name.."/Grade/Icon"
        self:setScript("single", path, functor.rmake(self.onClick_attendSlot, self.attend_slots[index]))
    end,

    -- 更新出战 slot
    updateAttendSlot = function(self, officer)
        for k, v in ipairs(self.attend_slots) do
            if officer.isAttend then
                -- 出战  找到空 slot
                if not v.officer then
                    v.officer = officer
                    local icon = v.ui:GetChild(0)
                    local image = icon:GetChild(0):GetComponent(UnityEngine.UI.Image)
                    gui_helper:setOfficerIcon(officer, officer.star_level, officer.quality, v.ui)
                    icon:SetActive(true)
                    local effect = icon:Find("UI_LevelInfoFrame_yingxiongxiangqian")
                    effect:SetActive(true)
                    break
                end
            else
                -- 取消出战
                if v.officer and v.officer.inst_id == officer.inst_id then
                    local icon = v.ui:GetChild(0)
                    icon:SetActive(false)
                    v.officer = nil
                    break
                end
            end
        end
    end,

    checkInProgress = function(self)
        local _, cur_level_id = self.owner:getCurChapterId()
        
        return cur_level_id == self.level_id
    end,

    -- 点击出战 slot
    onClick_attendSlot = function(self, handler, data, slot)
        if self.isStoryLevel then
            return
        end
        
        local officer = slot.officer
        if not officer then
            return
        end

        if self:checkInProgress() then
            hint:setText(NativeText.GetLangStr(170013))
            return
        end

        self:updateOfficerSlot(officer)
        self:updateAttendSlot(officer)
    end,

    -- 点击可选择的指挥官
    onClick_officer = function(self, handler, data, officer)
        if self:checkInProgress() then
            hint:setText(NativeText.GetLangStr(170013))
            return
        end

        if #self.attend_heros == self.max_attends and not officer.isAttend then
            return
        end

        self:updateOfficerSlot(officer)
        self:updateAttendSlot(officer)
    end,

    click_ButtonBack = function(self)
        if #self.attend_heros > 0 then
            network_sender:selectChapterAttendHeros(self.chapter_id, self.level_id, self.attend_heros)
        end
        frame_manager:hide("level_info")
        self.helper.showTrans(self.owner.transform)
    end,

    click_Attack = function(self)
        
        if not self:checkInProgress() then
            network_sender:selectChapterAttendHeros(self.chapter_id, self.level_id, self.attend_heros)
        end

        level_info.level_items = loadstring('return '..self.level_cfg.default_supply)()
        network_sender:enterMap(self.level_id, 0)

        --frame_manager:show("select_level_item_frame", nil, self.level_id)
        --network_sender:enterMap(self.level_id)
        
        --[[
        if #self.attend_heros < self.max_attends and not self:checkInProgress() then
            local tb = {
                title = "",
                content = NativeText.GetLangStr(170005),
                buttons =
                {
                    {
                        btn_name = NativeText.GetLangStr(110522),
                        callback = function() 
                            frame_manager:hide('common_box')
                        end
                    },
                    {
                        btn_name = NativeText.GetLangStr(110521),
                        callback = function()
                            network_sender:enterMap(self.level_id)
                            frame_manager:hide('common_box')
                        end
                    },
                },
            }

            frame_manager:show('common_box', nil, tb)
        else
            --network_sender:enterMap(self.level_id)
            frame_manager:show("select_level_item_frame", nil, self.level_id)

        end]]
    end,

    click_Reset = function(self)
        network_sender:resetMap(self.level_id)
    end,
}