local super = require("ui.frame")
local log = logger.get("Game", "Level Loading Frame")

return class {
    super = super,

    onLoaded = function(self, data)
        log.info('LevelLoadingFrame onLoaded!!')
        super.onLoaded(self)

        if data then
            self.args.data = data
        end
        local param = self.args.data

        self.max_progress = 0.95
        self.inc_speed = 0.5
        self.start_progress = 0

        self.progress = self.start_progress
        
        self.value = 0.0
        self:setProgress(self.max_progress, 1.0 / self.inc_speed)

        self.text_root = self.transform:Find('BG/Layout')
        self.text_components = {}
        self.typewriter = {}
        self.text_string = {}
        for i=0,self.text_root.childCount-1 do
            local text_go =  self.text_root:GetChild(i)
            local text_comp = 
               text_go:GetComponent(UnityEngine.UI.Text)
            self.text_components[text_comp] = text_comp
            self.text_string[i+1] = text_comp.text
            text_comp.text = ''

            local tw = text_go:GetComponent(TypewriterEffect)
            self.typewriter[i+1] = tw
        end
        self.current_line = 0   -- 当前码到哪一行
        self.is_typing = false  -- 是否正在码字
        self.finished = false

        self.transform:SetAsLastSibling()
    end,

    -- 设置进度 value [0,1]
    setProgress = function(self, value)
        log.info("### progress:"..value)
        self.value = value
    end,

    -- 获取进度
    getProgress = function(self)
        return  value
    end,

    update = function(self, dt)

        if not self.is_typing and self.typewriter then
            local line = math.ceil(self.value * #self.typewriter)
            if line > self.current_line then
                self:doTyping(self.current_line + 1)
            end
        end
    end,

    doTyping = function(self, line)
        if line > #self.typewriter then
            self.finished = true
            return
        end

        self.current_line = line
        self.is_typing = true

        self.typewriter[line]:SetText(self.text_string[line], function(finished)
            if finished then
                app:startCoroutine(
                    function()
                        coroutine.yield(WaitForSeconds(0.2))
                        log.info("##### line:"..line.." finished!!")
                        self.is_typing = false
                        if line >= #self.typewriter then
                            self.finished = true
                        end                        
                    end
                )
            end
        end)
    end,
    
    onUnload = function(self)

    end,

}