local super = require("ui.frame")
local log = logger.get("Game", "Level Select Frame")

return class {
    super = super,

    onLoaded = function(self)
        log.info('LevelSelectFrame onLoaded!!')
        super.onLoaded(self)
        level_select_frame = self

        self.chapter_id = self.args.data.chapter_id
        self.complete_level = self.args.data.complete_level

        self.chapter_info = chapter_manager:getChapterInfo(self.chapter_id)
        if not self.chapter_info then
            log.error("## No chapter info:"..self.chapter_id)
            self.curr_level = 0
        else
            self.curr_level = self.chapter_info.curr_level  
        end

        log.info('chapter_id:'..tostring(self.chapter_id)..', curr_level:'..self.curr_level)
        -- 找到该章的所有关卡
        self.levels = {}
        config.game_copy_levels:foreach(
            function(k,v)
                if v.Chapter == self.chapter_id then
                    self.levels[v.level_sequence] = v
                end
            end
        )

        clickEvents = 
        {
            "Root/Formation_button_01_normal",
        }
        frame_helper:regClickEvents("click", self, clickEvents)

        self.evts = {}
        self:init()

        -- 如果第一个关卡都没解锁，请求解锁关卡
        if self.curr_level == 0 then
            local level_cfg = self.levels[1]
            
            local panel =  self.transform:Find('Root/LevelPanel'..1)     
            network_sender:unlockLevel(self.chapter_id, level_cfg.id)
        else
            -- 通关百分比的初始值             
            local idx = config.game_copy_levels[self.curr_level].level_sequence
            local num_levels = #self.levels 
            local percent = math.floor((idx-1) / num_levels * 100)

            local number_effect = self.transform:Find('Root/building_panel_64/percent'):GetComponent(NumberEffect)
            number_effect:makeFromTo(0, percent)  
        end

        -- 播放关卡完成，新关卡开启动画
        if self.complete_level then

            local complete_idx = config.game_copy_levels[self.complete_level].level_sequence
            local panel =  self.transform:Find('Root/LevelPanel'..complete_idx)
            
            self:onLevelComplete(panel, complete_idx, function()
                -- 关卡完成后，向服务器请求解锁
                local unlock_idx = complete_idx + 1
                local level_cfg = self.levels[unlock_idx]
                if level_cfg then
                    network_sender:unlockLevel(self.chapter_id, level_cfg.id)
                else
                    self:onChapterUnlock()
                end
            end)

         end

         game_director:disable3DRender()
    end,

    -- 解锁关卡
    onLevelUnlock = function(self, chapter_id, level_id)
        log.info("##### onLevelUnlock:"..level_id)
        -- 验证 chapter_id
        if chapter_id == self.chapter_id then

            -- 通过 level_id 找到关卡顺序号
            local level_cfg = config.game_copy_levels[level_id]
            local idx = level_cfg.level_sequence
            local path = 'Root/LevelPanel'..idx
            local panel =  self.transform:Find(path) 
            local anim = panel:GetComponent(UnityEngine.Animation)
            log.info(debug.traceback())

            app:startCoroutine(
                function()
                    coroutine.yield(WaitForSeconds(0.3))

                    log.info("level id:"..level_id)
                    anim:Play("LevelPanel_unlock")
                    log.info("## anim:Play(LevelPanel_unlock)")
                    coroutine.yield(WaitForSeconds(0.5))

                    --anim:Play("LevelPanel_idle")
                    if self.evts[idx] then
                        self.evts[idx]:ClearClickListener()
                    end

                    self.curr_level = level_id
                    self.evts[idx] = self:RegisterSingleClick(path, functor.rmake(self.onClick_EnterLevel, level_id))
                end
            )


        else
            log.error('### onUnlockLevel, chapter id not equal, self:'..self.chapter_id..", in:"..chapter_id)
        end

    end,

    -- 关卡完成
    onLevelComplete = function(self, panel, level_idx, callback)
        log.info("#### onLevelComplete: "..level_idx)
        -- 需要移除点击事件
        local path = 'Root/LevelPanel'..level_idx
        self:UnregisterSingleClick(path)
        local panel = self.transform:Find(path) 
        local anim = panel:GetComponent(UnityEngine.Animation)        
        log.info(debug.traceback())

        app:startCoroutine(
            function()
                coroutine.yield(WaitForSeconds(0.3))

                anim:Play('LevelPanel_complete')
                coroutine.yield(WaitForSeconds(0.5))

                -- 百分比跳字
                local number_effect = self.transform:Find('Root/building_panel_64/percent'):GetComponent(NumberEffect)
                local num_levels = #self.levels 
                local from = math.ceil((level_idx-1) / num_levels * 100)
                local to = math.ceil((level_idx) / num_levels * 100)

                number_effect:makeFromTo(from, to)  
                coroutine.yield(WaitForSeconds(1.0))

                if callback then
                    callback()
                end
            end
        )


    end,

    -- 本章解锁（所有关卡通关，可以种田了）
    onChapterUnlock = function(self)
        network_sender:setChapterState(self.chapter_id, common_pb.ECHAPTER_OVER)
        frame_manager:hide('level_select')

        -- 摄像机移到船上
        local screen_center = Vector3(Screen.width / 2, Screen.height / 2, 0)
        local ship_root = GameObject.Find('ResShipRoot/ResShipRoot'..self.chapter_id.."/AnimRoot")
        game_state_home.home_map.home_camera:LookAt(ship_root.transform, screen_center, 80, nil)
    end,

    init = function(self)
        log.info('## curr level:'..self.curr_level)
        -- 遍历这章的所有关卡
        for i=1,4 do
            log.info('# level idx:'..i)
            local panel = self.transform:Find('Root/LevelPanel'..i)
            local level_cfg = self.levels[i]
            if level_cfg then

                -- 位置
                local pos_tab = loadstring('return '..level_cfg.frame_pos)()
                panel.anchoredPosition = Vector2(pos_tab[1], pos_tab[2])
                panel.localRotation = Quaternion.AngleAxis(level_cfg.frame_angle, Vector3.up)
                
                local text_name = panel:Find('text_LevelName'):GetComponent(UnityEngine.UI.Text)
                text_name.text = frame_helper:getLangCfg(level_cfg.Level_Name_ID)
                local img_complete = panel:Find('building_panel_68')

                local img_portrait = panel:Find('portrait'):GetComponent(UnityEngine.UI.Image)
                img_portrait:SetSprite(level_cfg.path_icon.. level_cfg.icon)
                img_portrait = panel:Find('portrait_add'):GetComponent(UnityEngine.UI.Image)
                img_portrait:SetSprite(level_cfg.path_icon.. level_cfg.icon)

                local img_lock = panel:Find('Lock')
                local anim = panel:GetComponent(UnityEngine.Animation)                
                if level_cfg.id < self.curr_level then
                    log.info('# level complete!')
                    -- 已完成
                    --img_lock:SetActive(false)
                    log.info("## anim:Play(LevelPanel_complete_idle)")
                    anim:Play("LevelPanel_complete_idle")

                    local path = 'Root/LevelPanel'..i
                    self.evts[i] = self:RegisterSingleClick(path, functor.rmake(self.onClick_CompleteLevel, i))

                elseif level_cfg.id == self.curr_level then
                    -- 进行中
                    --img_complete:SetActive(false)
                    --img_lock:SetActive(false)

                    log.info("## anim:Play(LevelPanel_idle)")
                    local anim = panel:GetComponent(UnityEngine.Animation)
                    anim:Play("LevelPanel_idle")

                    -- 进关卡回调
                    local path = 'Root/LevelPanel'..i
                    self.evts[i] = self:RegisterSingleClick(path, functor.rmake(self.onClick_EnterLevel, self.curr_level))

                else
                    -- 未开启
                    --img_complete:SetActive(false)
                    log.info("## anim:Play(LevelPanel_lock_idle)")
                    anim:Play("LevelPanel_lock_idle")

                    local path = 'Root/LevelPanel'..i
                    self.evts[i] = self:RegisterSingleClick(path, functor.rmake(self.onClick_LockedLevel, i))
                end
            else
                panel:SetActive(false)
            end
        end
    end,

    onClick_CompleteLevel = function(self, handler, data, level_idx)
        log.info("### onClick_CompleteLevel")
        if self.is_onClick_CompleteLevel then
            return
        end

        self.is_onClick_CompleteLevel = true

        local path = 'Root/LevelPanel'..level_idx.."/Bubble"
        local trans = self.transform:Find(path)
        local anim = trans:GetComponent(UnityEngine.Animation)

        local text = trans:Find('Text'):GetComponent(UnityEngine.UI.Text)
        text.text = NativeText.GetLangStr(170128)

        app:startCoroutine(
            function()
                trans:SetActive(true)
                anim:Play("appear")  
                
                coroutine.yield(WaitForSeconds(2.0))
                if anim then
                    anim:Play("vanish")
                    self.is_onClick_CompleteLevel = false
                end
            end
        )

    end,

    onClick_EnterLevel = function(self, handler, data, level_id)
        network_sender:enterMap(level_id, 0)
    end,

    onClick_LockedLevel = function(self, handler, data, level_idx)
        log.info("### onClick_LockedLevel")
        if self.is_onClick_LockedLevel then
            return
        end

        self.is_onClick_CompleteLevel = true

        local path = 'Root/LevelPanel'..level_idx.."/Bubble"
        local trans = self.transform:Find(path)
        local anim = trans:GetComponent(UnityEngine.Animation)

        local text = trans:Find('Text'):GetComponent(UnityEngine.UI.Text)
        text.text = NativeText.GetLangStr(170129)

        app:startCoroutine(
            function()
                trans:SetActive(true)
                anim:Play("appear")  
                
                coroutine.yield(WaitForSeconds(2.0))
                if anim then
                    anim:Play("vanish")
                    self.is_onClick_LockedLevel = false                       
                end
             
            end
        )

    end,

    -- 返回
    click_Formation_button_01_normal = function(self)
        frame_manager:hide('level_select')
    end,

    update = function(self, dt)

    end,
    
    onUnload = function(self)
        level_select_frame = nil
        game_director:enable3DRender()
    end,

}