local super = require("ui.frame")
local log = logger.get("Game", "Login Frame")
local PlayerPrefs = UnityEngine.PlayerPrefs

return class {
    super = super,

    onLoaded = function(self)
        log.info('LoginFrame onLoaded!!')

        super.onLoaded(self)
        self:setScript("click", "ButtonLogin", self.onClickLogin)
        self:setScript("click", "ButtonBattle", self.onClickBattle)
        self:setScript("click", "ButtonWorldMap", self.onClickWorldMap)
        self:setScript("click", "ButtonSound", self.onClickSound)
        self:setScript("click", "TbButtonBattle", self.onClickTbBattle)

        local trans_account = self.transform:Find("InputFieldAccount")
        self.input_account = trans_account.gameObject:GetComponent(UnityEngine.UI.InputField)

        if PlayerPrefs.HasKey('account') then
            self.input_account.text = PlayerPrefs.GetString('account')
        end

         -- 路径: root/login
        -- 事件: click_login 
        --[[
        clickEvents = {
            "preViewBtn",
            "homeBtn",
            "genBattleRecord",
            "replayRecord",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)]]

        -- local trans = self.transform:Find("Image")
        -- local lt = LeanTween.color(trans, Color(0, 198 / 255,1 ,0.5) , 0.83)
        -- lt.loopType = LeanTweenType.pingPong

        -- local lt = LeanTween.scale(trans.gameObject, Vector3.one * 1.38 , 1.66)
        -- lt.from = Vector3.one
        -- lt.loopType = LeanTweenType.linear

        -- local trans = self.transform:Find("Image1")
        -- local lt = LeanTween.color(trans, Color(0, 198 / 255,1 ,0.5) , 0.83)
        -- lt.loopType = LeanTweenType.pingPong
    end,

    onClickLogin = function(self)
        log.info('onClickLogin')

        -- 播放声音
        --SoundEngine.PostEvent("Play_ui_click")

        game_info.account = self.input_account.text
        Fenix.Game.GameApp.account = game_info.account

        local trans_password = self.transform:Find("InputFieldPassword")
        local input_password = trans_password.gameObject:GetComponent(UnityEngine.UI.InputField)
        game_info.password = input_password.text

        if game_info.account ~= "" then
            PlayerPrefs.SetString('account', game_info.account)
            if self.owner:connect() then
                frame_manager:show('loading')
            end
            -- 用户点了连接，启动附加初始化
            game_director:additionalInit()
        else
            log.error("please input account!!")
        end
        
        -- 执行登陆过程
        --game_director:changeState('loading', {scene_name = 'world_map', next_state = 'world_map'})
    end,

    onClickBattle = function(self)
        level_info.is_simple_battle = true
        game_director:changeState('battle', {stage_id=1, battle_id=1})
    end,

    onClickTbBattle = function(self)
        game_info.offline_mode = true
        -- 战斗类型
        local war_types = require('Turn_Based.war_enums').war_types
        frame_manager:show('loading')
        game_director:changeState('war', {war_type = war_types.TEST, stage_id=1, battle_id=1})
        -- game_director:changeState('loading', {next_state = 'level', scene_name = level_cfg.level_cfg.Map, level_id = game_info.level_info.mapId})
    end,

    click_genBattleRecord = function(self)
        local test = require('Turn_Based.war_test')
        test:generateRecord()
    end,

    click_replayRecord = function(self)
        game_info.offline_mode = true
        local war_types = require('Turn_Based.war_enums').war_types
        frame_manager:show('loading')
        game_director:changeState('war', {war_type = war_types.RECORD})
    end,

    onClickWorldMap = function(self)
        -- 离线方式
        game_info.role_info = {playerid = 1, rolename = 'offline role', level = 1,}
        game_director:changeState('loading', {scene_name = 'hex_map', next_state = 'world_map'})
    end,

    click_preViewBtn = function(self)
        frame_manager:hide('login')
        frame_manager:show('preview')
    end,

    click_homeBtn = function(self)
        game_director:changeState('loading', {scene_name = 'home', next_state = 'home'})
    end,

    onClickSound = function(self)
        -- SoundEngine.PostEvent("Play_BGM_heroic", self.gameObject)
        frame_manager:show('home_tavern_reward')
    end,

    onUnload = function(self)
        self:clearScript("click", "ButtonLogin")
        self:clearScript("click", "ButtonBattle")
        self:clearScript("click", "ButtonWorldMap")
        self:clearScript("click", "ButtonSound")
        self:clearScript("click", "TbButtonBattle")


    end,
}