local super = require("ui.frame")
local log = logger.get("Game", "lord info frame")
require('config.crew')

local drop_down_state = {
    base = 1,
    all = 2,
}

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)
        self.helper = require("gui.tool.frame_helper")
        self.home_map = self.args.owner
        self.event_ids = {}    

        clickEvents = 
        {
            "closeBtn",
            "content/layout/tag (1)/statisticsBtn",
            "content/layout/tag (2)/talentBtn",
            "content/layout/tag (3)/propBtn",
            "content/layout/tag (4)/guildBtn",
            "content/exp/addEnergyBtn",
            "content/exp/addLordExpBtn",
            "content/name/nickName/modifyBtn",
            "view/bg/sendNameBtn",
            "view/closeViewBtn",
            "select/modifyPlayerNameBtn",
            "select/modifyRoleNameBtn",
            "select/closeSelectBtn",
        }

        frame_helper:regClickEvents("click", self, clickEvents)
        self.event_ids['onEquipChanged'] = crew_manager.events:on('onEquipChanged', functor.make(self.refreshRooms, self))
        self.event_ids['LORD_INFO_CHANGE'] = local_lord.events:on('LORD_INFO_CHANGE', functor.make(self.refreshLordInfo, self))

        self:init()
    end,

    onUnload = function (self)
        crew_manager.events:remove("onEquipChanged", self.event_ids["onEquipChanged"] )
        local_lord.events:remove("LORD_INFO_CHANGE", self.event_ids["LORD_INFO_CHANGE"] )

        for i = 1,6 do 
            self:unloadSpine(i)
        end

    end,

    init = function(self)
        self.tip_trans = self.transform:Find("content/tip")
        frame_helper.hideTrans(self.tip_trans)

        self.view_trans = self.transform:Find("view")
        frame_helper.hideTrans(self.view_trans)

        self.select_trans = self.transform:Find("select")
        frame_helper.hideTrans(self.select_trans)

        self.input_field = self.transform:Find("view/bg/InputField"):GetComponent(UnityEngine.UI.InputField)

        self:refreshLordInfo()
        
        self.crew_spines = {}           -- spine 组件 skeleton graphic
        self.spine_assets = {}          -- 每个槽位的 spine 资源
        self.crews = {}                 -- 所有船员
        self:refreshRooms()

        self:refreshGuildInfo()
    end,

    -- 刷新工会信息
    refreshGuildInfo = function(self)
        local icon = self:findImage("content/info/guild_icon_2_2")
        local in_guild, degree = local_guild:getGuildLevel()
        icon.enabled = in_guild
        if in_guild then
            icon:SetSprite(gui_helper:getGuildRankIcon(degree))
        end
    end,

    -- 刷新领主信息
    refreshLordInfo = function(self)
        self:setText('content/vip', local_lord.vip_level)
        self:setText('content/name', local_lord.name)
        self:setText('content/name/nickName', local_lord.nickname)

        self:setText('content/power', local_lord.fight_force)
        self:setText('content/kills', local_lord.kill_num)

        -- 行动力
        self:setText('content/exp/text1_Static[T5]_4', local_lord.energy..'/'..local_lord.max_energy)
        local percent = local_lord.energy / local_lord.max_energy
        self.transform:Find('content/exp/Building_panel_14_05_Fill'):GetComponent(UnityEngine.UI.Image).fillAmount = percent
        local slider = self:findImage("content/exp/Building_panel_14_05_Fill")
        slider.fillAmount = percent

        -- 经验
        self:setText('content/exp/text1_Static[T5]_3', local_lord.exp..'/'..local_lord.max_exp)
        percent = local_lord.exp / local_lord.max_exp
        self.transform:Find('content/exp/Building_panel_14_01_Fill'):GetComponent(UnityEngine.UI.Image).fillAmount = percent
        local slider = self:findImage("content/exp/Building_panel_14_01_Fill")
        slider.fillAmount = percent

        -- 等级
        self:setText('content/exp/text1_Static[T5]_2', local_lord.level)
    end,

    refreshRooms = function(self)
        local setRoomStars = function(room, stars)
            local stars_root = room:Find("Stars")
            for i=1,5 do
                local star = stars_root:GetChild(i-1)
                if i <= stars then
                    star:SetActive(true)
                else
                    star:SetActive(false)
                end
            end
        end

        -- 已装备的船员
        local crews = crew_manager:getEquipedCrew()
        -- 6 个房间
        self.slots = {}
        for i=1,6 do
            local slot = self.transform:Find("content/equip/Slot_"..i)
            self.slots[i] = slot

            -- 房间名
            local room_cfg = config.crew_type_indexs[i]
            slot:Find('Title/Name'):GetComponent(UnityEngine.UI.Text).text = 
                                    self.helper:getLangCfg(room_cfg.CrewRoomName)

            local crew = crews[i]
            if crew then
                -- 装备了船员
                -- 设置品质
                setRoomStars(slot, crew.cfg.color)

                -- 隐藏加号
                slot:Find('img_add'):SetActive(false)

                local spine_path = "Assets/Arts/spines/npc/npc_new_SkeletonData.asset"
               -- local spine_path = "Assets/Arts/spines/npc/skeleton_SkeletonData.asset"
                -- 加载船员的 spine
                local callback = function(skeleton_graphic)
                    local anim_state = skeleton_graphic.AnimationState
                    if anim_state then
                        anim_state:SetAnimation(0, 'idle', true)
                    else
                        log.error("Failed to find anim state:"..spine_path)
                    end   
                    skeleton_graphic:SetSkin(crew.cfg.SpineSkin)
                    --initialSkinName.initialSkinName = crew.cfg.SpineSkin
                    self.crews[i] = {
                        trans = skeleton_graphic.transform, 
                        anim_state = anim_state, 
                        anim = 'idle', 
                        facing = 1,
                        elapsed = 0.0,
                        anim_time = -1.0,
                    }
                end

                local crew_go = slot:Find('Room/crew').gameObject
                self:loadSpine(spine_path, crew_go, i, callback)
            else
                -- 清除品质
                setRoomStars(slot, 0)

                -- 显示加号
                if crew_manager:canEquipCrew(i) then
                    slot:Find('img_add'):SetActive(true)                    
                else
                    slot:Find('img_add'):SetActive(false)
                end

                -- 卸载船员 spine
                self:unloadSpine(i)
            end
            -- 点击事件                        
            self:RegisterSingleClick("content/equip/Slot_"..i.."/Room", functor.rmake(self.onClick_room, i))
        end        
    end,

    -- 加载 spine
    loadSpine = function(self, path, crew_go, slot_idx, callback)
        if self.crew_spines[slot_idx] then
            callback(self.crew_spines[slot_idx])
        else
            app:startCoroutine(
                function()
                    coroutine.yield(Fenix.Assets.Helper.WaitForAsset(path, Spine.Unity.SkeletonDataAsset, function(asset)
                        -- 释放原来的资源
                        if asset then
                            self.spine_assets[slot_idx] = asset  
                            asset:AddRef()                             
                            local skeleton_data = asset:Get() -- skeleton data asset
                            local skeleton_graphic = Spine.Unity.SkeletonGraphic.AddSkeletonGraphicComponent(crew_go, skeleton_data)
                            skeleton_graphic.material = game_director.spine_material
                            self.crew_spines[slot_idx] = skeleton_graphic

                            callback(skeleton_graphic)
                        else
                            log.error("Failed to load spine asset:"..path)
                        end
                    end))
                end
            )
        end
    end,

    -- 释放 spine
    unloadSpine = function(self, slot)
        if self.crew_spines[slot] then
            Object.DestroyImmediate(self.crew_spines[slot])
            self.crew_spines[slot] = nil
        end
        if self.spine_assets[slot] then
            self.spine_assets[slot]:RemoveRef()
            self.spine_assets[slot] = nil
        end
    end,

    --[[
    self.crews[i] = {
        trans = skeleton_graphic.transform, 
        transform = s
        anim_state = anim_state, 
        anim = 'idle', 
        facing = 1,
        elapsed = 0.0,
    }]]
    -- 逻辑更新
    update = function(self, dt)
        if not self.crews then
            return
        end

        local randomBehaviour = function(crew)
            local r = math.random()
            local anim_time = 0
            local anim = 'idle'
            if r < 0.3 then
                anim = 'idle'
                anim_time = math.random()*2 + 1
            elseif r < 0.667 then
                anim = 'run'
                anim_time = 4.0 + math.random() * 2
            else
                anim = 'attack'
                anim_time = 1.0 + math.random()
            end

            if crew.anim ~= anim then
                crew.anim_state:SetAnimation(0, anim, true)
                crew.anim = anim
            end
            crew.elapsed = 0.0                
            crew.anim_time = anim_time
        end

        local move = function(crew, dt)

            if crew.anim == 'run' then
                local pos = crew.trans.localPosition
                pos = pos + Vector3(30, 0, 0) * crew.facing  * dt
                crew.trans.localPosition = pos

                if pos.x > 100 then
                    crew.facing = -1
                    crew.trans.localRotation = Quaternion.AngleAxis(180, Vector3.up)
                elseif pos.x < -100 then
                    crew.facing = 1
                    crew.trans.localRotation = Quaternion.identity
                end
            end
        end

        for k,crew in pairs(self.crews) do
            crew.elapsed = crew.elapsed + dt

            if crew.elapsed > crew.anim_time then
                randomBehaviour(crew)
            end
            move(crew, dt)
        end
    end,

    -- 点击某个房间
    onClick_room = function(self, handler, data, slot)
        log.info("## click room:"..slot)
        frame_manager:show('crewEquip', nil, {mod_id = slot})
    end,

    click_closeBtn = function(self)
        frame_manager:hide('lord_info')
    end,

    click_statisticsBtn = function(self)
        network_sender:getPlayerPvpData(local_player.id)
    end,

    click_talentBtn = function(self)    
        network_sender:getCaptainGift()
    end,

    click_propBtn = function(self)   
        network_sender:getGlobalPropInfo()
    end,

    click_guildBtn = function(self)    
        if local_guild:inGuild(local_guild:getId()) then
            network_sender:getFamilyInfo(local_player:getGuildId())
        else
            network_sender:getFamilyList(0)
        end
    end,

    click_addEnergyBtn = function(self)
        frame_manager:show('resource_exchange', nil, { 
            item_id = local_lord.EXP_ID,
            item_num = local_lord.max_exp,
        })
    end,

    click_addLordExpBtn = function(self)
        frame_manager:show('resource_exchange', nil, { 
            item_id = 50001,
            item_num = local_lord.max_enregy,
        })
    end,

    click_closeViewBtn = function(self)
        frame_helper.hideTrans(self.view_trans)
    end,

    click_modifyBtn = function(self)
        frame_helper.showTrans(self.select_trans)
    end,

    click_modifyPlayerNameBtn = function(self)
        frame_helper.hideTrans(self.select_trans)
        frame_helper.showTrans(self.view_trans)
        self:setViewInfo(42029, NativeText.GetLangStr(610032), true)
    end,

    click_modifyRoleNameBtn = function(self)
        frame_helper.hideTrans(self.select_trans)
        frame_helper.showTrans(self.view_trans)
        self:setViewInfo(42030, NativeText.GetLangStr(610033), false)
    end,

    setViewInfo = function(self, item_id, title_name, temp)
        self.modify_player_name = temp

        local title = self:findText("view/bg/text1_Static[T1]_1")
        title.text = title_name
        self.input_field.text = ""
        local own = self.transform:Find("view/bg/sendNameBtn/own")
        local none = self.transform:Find("view/bg/sendNameBtn/none")
        local item_num = frame_helper:getItemNum(item_id)
        frame_helper:setActive(own, item_num > 0)
        frame_helper:setActive(none, item_num <= 0)
        if item_num > 0 then
            local icon_name = frame_helper:getItemIcon(item_id)
            local icon = self:findImage("view/bg/sendNameBtn/own/icon")
            icon:SetSprite(icon_name)
        else
            local DIAMOND    = 30001 -- 钻石
            local item_name = gui_helper:getImageTextItemStr(DIAMOND)
            local image_text = self:findText("view/bg/sendNameBtn/none/image_text")
            image_text:SetImageText(string.format("%s %s", item_name, frame_helper:getItemBuyPrice(item_id)))
        end
    end,

    click_sendNameBtn = function(self)
        local name = self.input_field.text
        self.input_field.text = ""
        if string.len(name) <= 0 then
            return
        end
        if self.modify_player_name then
            network_sender:modifyRoleName(protocol_pb.EROLE_NAME, name)
        else
            network_sender:modifyRoleName(protocol_pb.ENICK_NAME, name)
        end
        frame_helper.hideTrans(self.view_trans)
    end,

    click_closeSelectBtn = function(self)
        frame_helper.hideTrans(self.select_trans)
    end,
}
