local super = require("ui.frame")
local log = logger.get("Game", "home_info")

local drop_down_state = {
    base = 1,
    all = 2,
}

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        clickEvents = 
        {
            "content/detail_trans/bg/top/toMain",
            "upgrade/closeUpgradeBtn",
            "content/resetBtn",
            "upgrade/infoBtn",
        }

        frame_helper:regClickEvents("click", self, clickEvents)

        self:init()

        local msg = self.args.data
        self:refresh(msg)
    end,

    onUnload = function (self)
    end,

    init = function(self)
        self.upgrade_box = self.transform:Find("upgrade")
        frame_helper.hideTrans(self.upgrade_box)   
        self.cur_cfg = nil     
    end,

    click_closeUpgradeBtn = function(self)
        frame_helper.hideTrans(self.upgrade_box)  
        self.cur_cfg = nil
    end,

    getCfgList = function(self)
        local rst = {}
        config.captain_skill_lists:foreach(function(id, cfg)
            table.insert(rst, cfg)
        end)
        local sortFunc = function(a, b)
            return a.id < b.id
        end
        table.sort(rst, sortFunc)
        return rst
    end,

    refresh = function(self, msg)
        self.SCGetCaptainGift = msg

        local list = self:getCfgList()

        local path = "content/detail_trans/Scroll View/Viewport/Content/icons"
        frame_helper:hideAllChild(self, path)

        for __, cfg in ipairs(list) do
            local child = frame_helper:getChild(self, __, path)
            child:SetActive(true)
            local icon = child:Find("icon"):GetComponent(UnityEngine.UI.Image)
            local n_bg = child:Find("n_bg"):GetComponent(UnityEngine.UI.Image)
            local s_bg = child:Find("s_bg"):GetComponent(UnityEngine.UI.Image)
            local ing_bg = child:Find("ing_bg"):GetComponent(UnityEngine.UI.Image)
            local level_text = child:Find("levelText"):GetComponent(UnityEngine.UI.Text)
            local name_text = child:Find("nameText"):GetComponent(UnityEngine.UI.Text)

            local unlock = self:isTalentUnlock(cfg.id)

            
            n_bg.color = unlock and Color.white or Color.grey
            s_bg.color = unlock and Color.white or Color.grey

            name_text.color = unlock and Color.white or Color.gray
            level_text.color = unlock and Color.white or Color.gray

            n_bg.enabled = cfg.quality == 1
            s_bg.enabled = cfg.quality == 2
            ing_bg.enabled = false

            local tempF = function()
                icon.color = unlock and Color.white or Color.grey
            end
            tempF()
            icon:SetSprite(cfg.path .. cfg.icon, function() tempF() end)
            name_text.text = frame_helper:getLangCfg(cfg.name_id)
            level_text.text = "Lv." .. self:getTalentLevel(cfg.id) .. "/" .. self:getTalentMaxLevel(cfg.id)

            local rect = child.transform:GetComponent(UnityEngine.RectTransform)
            rect.anchoredPosition = Vector2(cfg.x, cfg.y)
            
            local btn = child:Find('icon'):GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(function ()
                self:click_talent(cfg)
            end)
        end
        local cfg = config.captain_others[1]
        local content_trans = self.transform:Find("content/detail_trans/Scroll View/Viewport/Content"):GetComponent(UnityEngine.RectTransform)
        content_trans.sizeDelta = Vector2(cfg.SkillLength, content_trans.sizeDelta.y)


        -- 画自定义的线
        local getCenter = function(list1, list2)
            if #list1 == 1 and #list2 == 1 then
                local from = list1[1]
                local to = list2[1]
                return Vector2( (from.x + to.x) * 0.5, from.y)
            end
            local a = nil
            local b = nil
            if #list1 == 1 then
                a = list1
                b = list2
            else
                a = list2
                b = list1
            end
            if #b % 2 > 0 then
                local from = a[1]
                local temp_distance = frame_helper.max_number
                local rst = nil
                for __, to in ipairs(b) do
                    local d = Vector2.Distance(from, to)
                    if d < temp_distance then
                        temp_distance = d
                        rst = to
                    end
                end
                return Vector2( (from.x + rst.x) * 0.5, (from.y + rst.y) * 0.5 )
            else
                local x = (b[1].x + a[1].x) * 0.5
                local y = (b[1].y + b[#b].y) * 0.5
                return Vector2( x, y )
            end
        end

        local sortFunc = function(a, b)
            return a.y > b.y
        end

        -- start

        self:initLine()
        self:clearLine()

        local cook_line = {}
        for __, cfg in ipairs(list) do
            local start_list = { Vector2(cfg.x, cfg.y) }
            local end_list = {}
            local precondition = loadstring('return ' .. cfg.fronttech)()
            for cfg_id, level in pairs(precondition) do
                local temp = config.captain_skill_lists[cfg_id]
                table.insert(end_list, Vector2(temp.x, temp.y))
            end
            table.sort(end_list, sortFunc)

            if #start_list > 0 and #end_list > 0 then
                local center = getCenter(start_list, end_list)
                local unlock = self:isTalentUnlock(cfg.id)
                if unlock then
                    local cur_level = self:getTalentLevel(cfg.id)
                    if cur_level <= 0 then
                        self:addLevel0Line(start_list, end_list, center, Color.white)
                    else
                        self:addConnectedLine(start_list, end_list, center, Color.white)
                    end
                else
                    self:addUnConnectLine(start_list, end_list, center, Color.white)
                end
            end
        end
        self:makeLine()
        -- end

        if self.cur_cfg then
            self:click_talent(self.cur_cfg)
        end
    end,

    initLine = function(self)
        self.connected_mono = self.transform:Find("content/detail_trans/Scroll View/Viewport/Content/connected"):GetComponent(TreeNodeArrows)
        self.un_connect_mono = self.transform:Find("content/detail_trans/Scroll View/Viewport/Content/un_connect"):GetComponent(TreeNodeArrows)
        self.level0_mono = self.transform:Find("content/detail_trans/Scroll View/Viewport/Content/level0"):GetComponent(TreeNodeArrows)
    end,

    clearLine = function(self)
        self.connected_mono:ClearArrow()
        self.un_connect_mono:ClearArrow()
        self.level0_mono:ClearArrow()
    end,

    addConnectedLine = function(self, start_list, end_list, center, color)
        self.connected_mono:AddNodeData(start_list, end_list, center, color)
    end,

    addUnConnectLine = function(self, start_list, end_list, center, color)
        -- self.un_connect_mono:AddNodeData(start_list, end_list, center, color)
        self:addLevel0Line(start_list, end_list, center, color)
    end,

    addLevel0Line = function(self, start_list, end_list, center, color)
        self.level0_mono:AddNodeData(start_list, end_list, center, color)
    end,

    makeLine = function(self)
        self.connected_mono:Excute()
        self.un_connect_mono:Excute()
        self.level0_mono:Excute()
    end,

    getTalentLevel = function(self, cfg_id)
        local level = 0
        for __, t in ipairs(self.SCGetCaptainGift.gifts) do
            if t.listId == cfg_id then
                return t.level
            end
        end
        return level
    end,

    getTalentMaxLevel = function(self, cfg_id)
        local max_level = 0
        config.captain_skill_details:foreach(function(k, v)
            if v.list_id == cfg_id then
                max_level = max_level + 1
            end
        end)
        return max_level
    end,

    isTalentUnlock = function(self, cfg_id)
        local cfg = config.captain_skill_lists[cfg_id]
        local cdt = loadstring('return ' .. cfg.fronttech)()
        for cfg_id, need_level in pairs(cdt) do
            local cur_level = self:getTalentLevel(cfg_id)
            if cur_level < need_level then
                return false
            end
        end
        return true
    end,

    click_toMain = function(self)
        frame_manager:hide('lord_talent')
    end,

    click_talent = function(self, cfg)
        self.cur_cfg = cfg

        frame_helper.showTrans(self.upgrade_box)
        local child = self.transform:Find("upgrade/info")
        child:SetActive(true)
        local cur_level = self:getTalentLevel(cfg.id)
        local max_level = self:getTalentMaxLevel(cfg.id)
        local is_max = cur_level >= max_level
        local cur_upgrade_cfg = self:getUpgradeCfg(cfg.id, cur_level)
        local next_upgrade_cfg = self:getUpgradeCfg(cfg.id, cur_level + 1)

        local icon = child:Find("icon"):GetComponent(UnityEngine.UI.Image)
        local n_bg = child:Find("n_bg"):GetComponent(UnityEngine.UI.Image)
        local s_bg = child:Find("s_bg"):GetComponent(UnityEngine.UI.Image)
        local name_text = child:Find("name"):GetComponent(UnityEngine.UI.Text)
        local des_text = child:Find("des"):GetComponent(UnityEngine.UI.Text)
        local level = child:Find("level"):GetComponent(UnityEngine.UI.Text)
        local cur_num = child:Find("cur_num"):GetComponent(UnityEngine.UI.Text)

        icon:SetSprite(cfg.path .. cfg.icon)
        name_text.text = frame_helper:getLangCfg(cfg.name_id)
        des_text.text = frame_helper:getLangCfg(cfg.des_id)
        level.text = "Lv." .. cur_level
        n_bg.enabled = cfg.quality == 1
        s_bg.enabled = cfg.quality == 2

        local un_upgrade_trans = child:Find("un_upgrade")
        local can_upgrade_trans = child:Find("can_upgrade")
        local max_trans = child:Find("max")

        frame_helper.hideTrans(un_upgrade_trans)
        frame_helper.hideTrans(can_upgrade_trans)
        frame_helper.hideTrans(max_trans)

        local full_condition = false
        if is_max then
            frame_helper.showTrans(max_trans)
        else
            local unlock = self:isTalentUnlock(cfg.id)
            if unlock then
                frame_helper.showTrans(can_upgrade_trans)
                local cost = self:findText("upgrade/info/can_upgrade/num")
                cost.text = next_upgrade_cfg.need_captain_sp .. "/" .. self.SCGetCaptainGift.remainSp
                full_condition = self.SCGetCaptainGift.remainSp >= next_upgrade_cfg.need_captain_sp
            else
                frame_helper.showTrans(un_upgrade_trans)

                local path = "upgrade/info/un_upgrade/layout"
                frame_helper:hideAllChild(self, path)
                local idx = 0
                local cdt = loadstring('return ' .. cfg.fronttech)()
                for cfg_id, need_level in pairs(cdt) do
                    idx = idx + 1
                    local tplt = frame_helper:getChild(self, idx, path)
                    tplt:SetActive(true)
                    local des = tplt:GetComponent(UnityEngine.UI.Text)
                    des.text = "Lv." .. need_level .. " " .. frame_helper:getLangCfg(config.captain_skill_lists[cfg_id].name_id)
                    local cur_level = self:getTalentLevel(cfg_id)
                    des.color = cur_level >= need_level and Color.white or Color.red
                end
            end

            local next_num = child:Find((unlock and "can_upgrade" or "un_upgrade") .. "/next_num"):GetComponent(UnityEngine.UI.Text)
            local next_effect = frame_helper:getPropertyDes(next_upgrade_cfg.effect)
            next_num.text = next_effect[1].num_str
        end

        if cur_upgrade_cfg then
            local cur_effect = frame_helper:getPropertyDes(cur_upgrade_cfg.effect)
            cur_num.text = cur_effect[1].num_str
        else
            local next_effect = frame_helper:getPropertyDes(next_upgrade_cfg.effect)
            cur_num.text = next_effect[1].is_percent and "0%" or "0"
        end

        local btn = child:Find("can_upgrade/upgradeBtn"):GetComponent(UnityEngine.UI.Button)
        btn.interactable = full_condition
        btn.onClick:RemoveAllListeners()
        btn.onClick:AddListener(function ()
            network_sender:naturalGiftLevelup(cfg.id)
        end)
    end,

    getUpgradeCfg = function(self, cfg_id, level)
        local rst = nil
        local max_level = 1
        config.captain_skill_details:foreach(
            function(k, v)
                if v.list_id == cfg_id then
                    if v.level == level then rst = v end
                    if v.level > max_level then max_level = v.level end
                end
            end
        )
        return rst, max_level
    end,

    click_resetBtn = function(self)
        frame_manager:show('use_item', nil, {
            item_id = 65701,
            callback = function(is_use)
                network_sender:naturalGiftReset(not is_use)
            end
        })

    end,

    click_infoBtn = function(self)
        local cur_level = self:getTalentLevel(self.cur_cfg.id)
        frame_manager:show('lord_talent_info', nil, {
            cfg = self.cur_cfg,
            level = cur_level,
        })
    end,
}
