local log = logger.get("Game", "Mail Frame")
local super = require("ui.frame")
local item_cls = require('RTS.bag_sys.item')

local SYS = 1
local BAT = 2
local PER = 3
local FAV = 4

local page_types = {
    [SYS] = {[common_pb.MT_SYSTEM] = true},
    [BAT] = {[common_pb.MT_FIGHTREOPRT] = true, [common_pb.MT_SPYREPORT] = true, [common_pb.MT_BE_SPY] = true, [common_pb.MT_GATHER] = true},
    [PER] = {[common_pb.MT_PERSONAL] = true},
}

return class{

    super = super,

    onLoaded = function(self)

        super.onLoaded(self)

        self.mail_id = self.args.data.mail_id

        mail_frame = self

        clickEvents = {
            "BG/Top/BackBtn",
            "BG/Center/Left/SelectAllBtn",
            "BG/Center/Right/ActivityTemp/MoreBtn",
            "BG/Center/Right/Battle/Scroll View/Viewport/Content/Report",
            "BG/Center/Right/Battle/DetailScroll/Viewport/Content/Top/Back",
            "BG/Center/Right/EnemySpy/Scroll View/Viewport/Content/Top/DelBtn",
            "BG/Center/Right/Gather/Scroll View/Viewport/Content/Top/DelBtn",
            "BG/Center/Buttons/SystemBtn",
            "BG/Center/Buttons/BattleBtn",
            "BG/Center/Buttons/ServiceBtn",
            "BG/Center/Buttons/Favourite",
            "BG/Center/Operate/Delete",
            "BG/Center/Operate/Read",
            "BG/Center/Operate/Cancel",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self.tag_btns = {}
        self.tag_btns[SYS] = self.transform:Find("BG/Center/Buttons/SystemBtn"):GetComponent(UnityEngine.UI.Button)
        self.tag_btns[BAT] = self.transform:Find("BG/Center/Buttons/BattleBtn"):GetComponent(UnityEngine.UI.Button)
        self.tag_btns[PER] = self.transform:Find("BG/Center/Buttons/ServiceBtn"):GetComponent(UnityEngine.UI.Button)
        self.tag_btns[FAV] = self.transform:Find("BG/Center/Buttons/Favourite"):GetComponent(UnityEngine.UI.Button)

        --self.record_node = self.transform:Find("BG/Center/Record")
        self.left_panel = self.transform:Find("BG/Center/Left")
        self.right_panel = self.transform:Find("BG/Center/Right")
        self.operate_panel = self.transform:Find("BG/Center/Operate")

        self.operate_panel:SetActive(false)

        self.normal_content = self.transform:Find("BG/Center/Right/TextTemp")
        self.append_content = self.transform:Find("BG/Center/Right/GetItemTemp")
        self.activity_content = self.transform:Find("BG/Center/Right/ActivityTemp")
        self.crew_content = self.transform:Find("BG/Center/Right/CrewTemp")
        self.prison_content = self.transform:Find("BG/Center/Right/Officer")
        self.gather_content = self.transform:Find("BG/Center/Right/Gather")
        self.no_mail = self.transform:Find("BG/Center/Right/Text")

        self.normal_content:SetActive(false)
        self.append_content:SetActive(false)
        self.activity_content:SetActive(false)
        self.crew_content:SetActive(false)
        self.prison_content:SetActive(false)
        self.gather_content:SetActive(false)
        self.no_mail:SetActive(false)

        self.left_scroll_content = self.transform:Find("BG/Center/Left/Scroll View/Viewport/Content")
        self.mail_prefab = self.transform:Find("BG/Center/Left/Scroll View/Viewport/Content/Elem").gameObject

        self.battle_node = self.right_panel:Find("Battle")
        self.battle_record = require("gui.battle_record_frame")(self.battle_node, self)
        self.battle_details = require("gui.battle_report_details")(self.battle_node, self)
        
        local panel = self.right_panel:Find("Spy")
        self.spy_report = require("gui.spy_report_panel")(panel, self)

        self.enemy_spy = self.right_panel:Find("EnemySpy")
        self.enemy_spy:SetActive(false)

        self.battle_node:SetActive(false)
        self.spy_report:hide()

        self.mail_list = {}

        self.curr_mail = nil
        self.select_mails = {}
        self.mail_type = SYS
        self.operating = false
        self.toggle_list = {}
        self.page_mail_cnt = 0

        self.select_text = self.transform:Find("BG/Center/Left/SelectAllBtn/text1_Static[T4]_20"):GetComponent(UnityEngine.UI.Text)
        self.select_text.text = NativeText.GetLangStr(9000027)

        self.evts = {}
        self.evts["READMAIL"] = game_director.events:on("READMAIL", functor.make(self.onReadMail, self))
        self.evts["DELMAIL"] = game_director.events:on("DELMAIL", functor.make(self.onDeleteMail, self))
        self.evts["ALLREASMAIL"] = game_director.events:on("ALLREASMAIL", functor.make(self.onReadMoreMail, self))
        self.evts["COLLECTMAIL"] = game_director.events:on("COLLECTMAIL", functor.make(self.onCollectMail, self))

        self:init(self.args.data.mails)
    end,

    onUnload = function(self)
        if self.evts then
            for k, v in pairs(self.evts) do
                game_director.events:remove(k, v)
            end
        end
        mail_frame = nil
    end,

    onCollectMail = function(self, mail_id)
        if self.curr_mail then
            local is_collected = self.curr_mail.isCollect
            if is_collected == 1 then
                self.curr_mail.isCollect = 0
            else
                self.curr_mail.isCollect = 1
            end

            local _mail = self.mail_list[mail_id]

            if _mail then
                _mail.mail.isCollect = self.curr_mail.isCollect
            end

            self:readFirstMail(self.mail_type)
        end
    end,

    onReadMoreMail = function(self, mail_list)
        local refreshState = function(id)
            for k, v in ipairs(self.mails) do
                if v.mailId == id then
                    v.mailHaveRead = 1
                end
            end
        end
        
        for k, v in ipairs(mail_list) do
            refreshState(v.mailId)
        end

        self:init(self.mails)
    end,

    onReadMail = function(self, mail)
        local _mail = self.mail_list[mail.MailId]
        if _mail then
            log.info("##################################### mail id ", mail.MailId, _mail.mail.mailType)
            _mail.mail.mailHaveRead = 1
            self.curr_mail = mail

            if _mail.mail.mailType == common_pb.MT_SYSTEM then
                self:readSystemMail(mail)
            elseif _mail.mail.mailType == common_pb.MT_FIGHTREOPRT then
                self:readBattleMail(mail)
            elseif _mail.mail.mailType == common_pb.MT_SPYREPORT then
                self:readSpyMail(mail)
            elseif _mail.mail.mailType == common_pb.MT_BE_SPY then
                self:readBeSpyMail(mail)
            elseif _mail.mail.mailType == common_pb.MT_GATHER then
                self:readGatherMail(mail)
            elseif _mail.mail.mailType == common_pb.MT_PERSONAL then
                self:readPersonalMail(mail)
            end

            self:updateMailCount()
            self:refreshMailList()
        else
            log.info("mail id error ! ", mail.MailId)
        end
    end,

    onDeleteMail = function(self)
        local isRead = false
        for k, v in ipairs(self.select_mails) do
            local _mail = self.mail_list[v]
            if _mail then
                isRead = isRead or (self.curr_mail and self.curr_mail.MailId == _mail.mail.mailId)
                self:deleteMailinfo(_mail.mail)
                _mail.ui:SetActive(false)
                _mail.mail = nil
                self.mail_list[v] = nil
            end
        end

        self:loadMailList(self.mail_type)

        if isRead then
            self.curr_mail = nil
            self:readFirstMail(self.mail_type)
        else
            self:refreshMailList()
        end
        
        self:updateMailCount()

        self:exitOperating()

        hint:setText(NativeText.GetLangStr(9000028))
    end,

    deleteMailinfo = function(self, mail)
        for k, v in ipairs(self.mails) do
            if mail.mailId == v.mailId then
                table.remove( self.mails, k )
                break
            end
        end
    end,

    readFirstMail = function(self, type)
        if self.curr_mail then
            network_sender:readMail(self.curr_mail.MailId)
            return
        end

        if type == FAV then
            for k, v in ipairs(self.mails) do
                if v.isCollect == 1 then
                    network_sender:readMail(v.mailId)
                    break
                end
            end
        else
            for k, v in ipairs(self.mails) do
                if page_types[type][v.mailType] then
                    network_sender:readMail(v.mailId)
                    break
                end
            end
        end
    end,

    getMailSheet = function(self, mail)
        local cfg = config.mail_mails[mail.templet_id]

        if cfg then
            return cfg.sheet_type
        end

        return 0
    end,

    exitOperating = function(self)
        for k, v in pairs(self.toggle_list) do
            v.isOn = false
        end

        self.select_text.text = NativeText.GetLangStr(9000027)
    end,

    enabledPanel = function(self, type, temp_type)
        self.normal_content:SetActive(type == common_pb.MT_SYSTEM and temp_type == 1)
        self.append_content:SetActive(type == common_pb.MT_SYSTEM and temp_type == 2)
        self.activity_content:SetActive(type == common_pb.MT_SYSTEM and temp_type == 6)
        self.crew_content:SetActive(type == common_pb.MT_SYSTEM and temp_type == 5)
        self.prison_content:SetActive(type == common_pb.MT_SYSTEM and temp_type == 3)
        self.enemy_spy:SetActive(type == common_pb.MT_BE_SPY)
        self.gather_content:SetActive(type == common_pb.MT_GATHER)
        self.battle_node:SetActive(type == common_pb.MT_FIGHTREOPRT)

        if type == common_pb.MT_SPYREPORT then
            self.spy_report:show()
        else
            self.spy_report:hide()
        end

        self.no_mail:SetActive(type == nil)
    end,

    init = function(self, mails)
        self.mails = mails

        self:loadMailList(self.mail_type)
        self:readFirstMail(self.mail_type)
        self:updateBtnState(self.mail_type)
        self:updateMailCount()
    end,

    loadMailList = function(self, type)
        local index = 1
        table.sort(self.mails, function(a, b) return a.sendTime > b.sendTime end)

        self.mail_list = {}
        self.toggle_list = {}
        self.select_mails = {}
        self.operating = false
        self.curr_mail = nil

        if type == FAV then
            index = self:loadFavouriteMails()
        else
            for k, v in ipairs(self.mails) do
                if page_types[type][v.mailType] then
                    if v.mailType == common_pb.MT_SYSTEM then
                        local cfg = config.mail_mails[v.templet_id]

                        if cfg then
                            local elem = self:addElem(v, index)
                            self:setTempMailInfo(elem, v, cfg)
                            index = index + 1
                        else
                            log.error("########################################## not find mail cfg ! id ", v.templet_id)    
                        end
                    elseif v.mailType == common_pb.MT_FIGHTREOPRT then
                        local elem = self:addElem(v, index)
                        self:setBattleMailInfo(elem, v)
                        index = index + 1
                    elseif v.mailType == common_pb.MT_PERSONAL then
                        local elem = self:addElem(v, index)
                        self:setPersonalMailInfo(elem, v)
                        index = index + 1
                    elseif v.mailType == common_pb.MT_SPYREPORT then
                        local elem = self:addElem(v, index)
                        self:setBattleMailInfo(elem, v)
                        index = index + 1
                    elseif v.mailType == common_pb.MT_BE_SPY then
                        local elem = self:addElem(v, index)
                        self:setBattleMailInfo(elem, v)
                        index = index + 1
                    elseif v.mailType == common_pb.MT_GATHER then
                        local elem = self:addElem(v, index)
                        self:setBattleMailInfo(elem, v)
                        index = index + 1
                    end
                end
            end
        end

        for i=index-1, self.left_scroll_content.childCount - 1 do
            self.left_scroll_content:GetChild(i):SetActive(false)
        end

        if index == 1 then
            self:enabledPanel()
        end

        self.page_mail_cnt = index - 1
    end,

    loadFavouriteMails = function(self)
        local index = 1
        for k, v in ipairs(self.mails) do
            if v.isCollect == 1 then
                if v.mailType == common_pb.MT_SYSTEM then
                    local cfg = config.mail_mails[v.templet_id]

                    if cfg then
                        local elem = self:addElem(v, index)
                        self:setTempMailInfo(elem, v, cfg)
                        index = index + 1
                    else
                        log.error("########################################## not find mail cfg ! id ", v.templet_id)    
                    end
                elseif v.mailType == common_pb.MT_FIGHTREOPRT then
                    local elem = self:addElem(v, index)
                    self:setBattleMailInfo(elem, v)
                    index = index + 1
                elseif v.mailType == common_pb.MT_PERSONAL then
                    local elem = self:addElem(v, index)
                    self:setPersonalMailInfo(elem, v)
                    index = index + 1
                elseif v.mailType == common_pb.MT_SPYREPORT then
                    local elem = self:addElem(v, index)
                    self:setBattleMailInfo(elem, v)
                    index = index + 1
                elseif v.mailType == common_pb.MT_BE_SPY then
                    local elem = self:addElem(v, index)
                    self:setBattleMailInfo(elem, v)
                    index = index + 1
                elseif v.mailType == common_pb.MT_GATHER then
                    local elem = self:addElem(v, index)
                    self:setBattleMailInfo(elem, v)
                    index = index + 1
                end
            end
        end

        return index
    end,

    addElem = function(self, data, index)
        local elem = nil
        if index > self.left_scroll_content.childCount then
            elem = Object.Instantiate(self.mail_prefab).transform
            elem:SetParent(self.left_scroll_content)
            resetTransformZ(elem)
        else
            elem = self.left_scroll_content:GetChild(index-1)
        end

        elem:SetActive(true)
        elem.name = "mail_"..index

        local path = "BG/Center/Left/Scroll View/Viewport/Content/"..elem.name
        self:clearScript("click", path)
        self:setScript("click", path, functor.rmake(self.onClickMail, data.mailId))

        self.mail_list[data.mailId] = {ui = elem, mail = data}

        return elem
    end,

    getCfg = function(self, type, cfg_id)
        if type == common_pb.WORLD_OBJ_MONSTER or type == common_pb.WORLD_OBJ_RES_SHIP then
            return config.worldmap_monsters[cfg_id]
        elseif type == common_pb.WORLD_OBJ_PLAYER then
            return config.worldmap_base_configs[cfg_id]
        elseif type == common_pb.WORLD_OBJ_WILD_MINE then
            return config.worldmap_titaniums[cfg_id]
        end

        return nil
    end,

    toggleChanged = function(self, mail_id, toggle)
        if toggle.isOn then
            table.insert( self.select_mails, mail_id )
        else
            for k, v in ipairs(self.select_mails) do
                if mail_id == v then
                    table.remove( self.select_mails, k )
                    break
                end
            end
        end

        self.operating = #self.select_mails > 0
        self.operate_panel:SetActive(self.operating)
    end,

    setBattleMailInfo = function(self, ui, mail)
        local title = ui:Find("Title"):GetComponent(UnityEngine.UI.Text)
        local time = ui:Find("Time"):GetComponent(UnityEngine.UI.Text)
        local name = ui:Find("Name"):GetComponent(UnityEngine.UI.Text)
        local simple = ui:Find("Simple"):GetComponent(UnityEngine.UI.Text)
        local icon = ui:Find("Icon"):GetComponent(UnityEngine.UI.Image)
        local toggle = ui:Find("Toggle"):GetComponent(UnityEngine.UI.Toggle)

        toggle.isOn = false

        toggle.onValueChanged:RemoveAllListeners()
        toggle.onValueChanged:AddListener(
            function()
                self:toggleChanged(mail.mailId, toggle)
            end
        )

        self.toggle_list[mail.mailId] = toggle

        time.text = os.date("%Y/%m/%d", mail.sendTime)
        time.text = time.text.." "..os.date("%X", mail.sendTime)

        local cfg = self:getCfg(mail.brBatReoprt.objType, mail.brBatReoprt.cfgId)

        if not cfg then
            log.error("############################################ find cfg error ! type  id", mail.brBatReoprt.objType, mail.brBatReoprt.cfgId)
            return
        end

        local pos_str = gui_helper:positionFormat(mail.brBatReoprt.posX, mail.brBatReoprt.posY)
        local show_simple = true

        if mail.mailType == common_pb.MT_FIGHTREOPRT then
            if mail.brBatReoprt.objType == common_pb.WORLD_OBJ_MONSTER then
                name.text = string.format("%s %s", pos_str, NativeText.GetLangStr(9000029))
                title.text = string.format( "%sLv.%d %s", NativeText.GetLangStr(9000030), cfg.level, self.helper:getLangCfg(cfg.monstername) )
                
                icon:SetSprite(res_path.res_path_129)
                show_simple = cfg.attack ~= 4
            elseif mail.brBatReoprt.objType == common_pb.WORLD_OBJ_RES_SHIP then
                name.text = string.format("%s", NativeText.GetLangStr(9000046))
                title.text = string.format( "%sLv.%d %s", NativeText.GetLangStr(9000030), cfg.level, self.helper:getLangCfg(cfg.monstername) )
                
                icon:SetSprite(res_path.res_path_129)
            else
                name.text = string.format("%s %s", pos_str, NativeText.GetLangStr(9000031))

                local name_str = mail.brBatReoprt.playerName
                if string.len(mail.brBatReoprt.shortName) > 0 then
                    name_str = string.format( "[%s]%s", mail.brBatReoprt.shortName, name_str)
                end
                if mail.brBatReoprt.isAttack == 0 then
                    title.text = string.format( NativeText.GetLangStr(9000032), name_str, self.helper:getLangCfg(cfg.nameid or cfg.name_id) )
                else
                    title.text = string.format( NativeText.GetLangStr(9000033), name_str, self.helper:getLangCfg(cfg.nameid or cfg.name_id) )
                end

                icon:SetSprite(res_path.res_path_128)
            end

            simple.transform:SetActive(show_simple)

            if mail.brBatReoprt.isWin == 0 then
                simple.text = NativeText.GetLangStr(9000034)
            else
                simple.text = NativeText.GetLangStr(9000035)
            end
        elseif mail.mailType == common_pb.MT_SPYREPORT then
            name.text = NativeText.GetLangStr(9000036)
            if mail.brBatReoprt.objType == common_pb.WORLD_OBJ_MONSTER or mail.brBatReoprt.objType == common_pb.WORLD_OBJ_RES_SHIP then
                title.text = string.format( "%s%sLv.%d", NativeText.GetLangStr(9000037), self.helper:getLangCfg(cfg.monstername), cfg.level )
            elseif mail.brBatReoprt.objType == common_pb.WORLD_OBJ_PLAYER then
                title.text = string.format( "%s%s", NativeText.GetLangStr(9000037), self.helper:getLangCfg(cfg.name_id) )
            end

            if mail.brBatReoprt.objType == common_pb.WORLD_OBJ_RES_SHIP then
                simple.text = ""
            else
                simple.text = pos_str
            end

            icon:SetSprite(res_path.res_path_126)
        elseif mail.mailType == common_pb.MT_BE_SPY then
            name.text = NativeText.GetLangStr(9000038)
            title.text = NativeText.GetLangStr(9000039)
            simple.text = ""
            icon:SetSprite(res_path.res_path_127)
        elseif mail.mailType == common_pb.MT_GATHER then
            name.text = NativeText.GetLangStr(9000040)
            title.text = string.format( NativeText.GetLangStr(9000041), self.helper:getLangCfg(cfg.nameid) )
            simple.text = ""
            icon:SetSprite(res_path.res_path_131)
        end
    end,

    setPersonalMailInfo = function(self, ui, mail)
        local title = ui:Find("Title"):GetComponent(UnityEngine.UI.Text)
        local time = ui:Find("Time"):GetComponent(UnityEngine.UI.Text)
        local name = ui:Find("Name"):GetComponent(UnityEngine.UI.Text)
        local simple = ui:Find("Simple"):GetComponent(UnityEngine.UI.Text)
        local icon = ui:Find("Icon"):GetComponent(UnityEngine.UI.Image)
        local toggle = ui:Find("Toggle"):GetComponent(UnityEngine.UI.Toggle)

        toggle.isOn = false

        toggle.onValueChanged:RemoveAllListeners()
        toggle.onValueChanged:AddListener(
            function()
                self:toggleChanged(mail.mailId, toggle)
            end
        )

        self.toggle_list[mail.mailId] = toggle

        time.text = os.date("%Y/%m/%d", mail.sendTime)
        time.text = time.text.." "..os.date("%X", mail.sendTime)

        title.text = mail.title
        name.text = mail.sender
        simple.text = ""
        icon:SetSprite(res_path.res_path_132)
    end,

    setTempMailInfo = function(self, ui, mail, cfg)
        local title = ui:Find("Title"):GetComponent(UnityEngine.UI.Text)
        local time = ui:Find("Time"):GetComponent(UnityEngine.UI.Text)
        local name = ui:Find("Name"):GetComponent(UnityEngine.UI.Text)
        local simple = ui:Find("Simple"):GetComponent(UnityEngine.UI.Text)
        local icon = ui:Find("Icon"):GetComponent(UnityEngine.UI.Image)
        local toggle = ui:Find("Toggle"):GetComponent(UnityEngine.UI.Toggle)

        toggle.isOn = false

        toggle.onValueChanged:RemoveAllListeners()
        toggle.onValueChanged:AddListener(
            function()
                self:toggleChanged(mail.mailId, toggle)
            end
        )

        self.toggle_list[mail.mailId] = toggle

        time.text = os.date("%Y/%m/%d", mail.sendTime)
        time.text = time.text.." "..os.date("%X", mail.sendTime)

        title.text = self.helper:getLangCfg(cfg.title_id)
        name.text = self.helper:getLangCfg(cfg.sender_id)
        simple.text = self.helper:getLangCfg(cfg.short_id)
        icon:SetSprite(res_path.res_path_130)
    end,

    checkMailUnread = function(self, mail)
        if mail.mailHaveRead == 0 then
            return true
        end

        return false
    end,

    refreshMailList = function(self)
        for k, v in pairs(self.mail_list) do
            local icon = v.ui:Find("Icon"):GetComponent(UnityEngine.UI.Image)
            local top_img = v.ui:Find("TopImg"):GetComponent(UnityEngine.UI.Image)
            local left_img = v.ui:Find("LeftImg")
            local bottom_img = v.ui:Find("Image_1")

            left_img:SetActive(false)
            bottom_img:SetActive(false)
            self.helper:setImageGray1(icon, v.mail.mailHaveRead ~= 0)

            if self.curr_mail.MailId == v.mail.mailId then
                left_img:SetActive(true)
                bottom_img:SetActive(true)
                top_img:SetSprite(res_path.res_path_133)
            else
                if self:checkMailUnread(v.mail) then
                    top_img:SetSprite(res_path.res_path_135)
                else
                    top_img:SetSprite(res_path.res_path_134)
                end
            end
        end
    end,

    readSystemMail = function(self, mail)
        local cfg = config.mail_mails[mail.templet_id]

        if cfg then
            if cfg.mail_type == 1 then
                self:normalMailContent(mail, cfg)
            elseif cfg.mail_type == 2 then
                self:haveItemMailContent(mail, cfg)
            elseif cfg.mail_type == 6 then
                self:activityMailcontent(mail, cfg)
            elseif cfg.mail_type == 5 then
                self:crewMailContent(mail, cfg)
            end
        else
            log.error("#################################### find mail cfg error ! id ", mail.templet_id)
        end

        self:enabledPanel(mail.mailType, cfg.mail_type)
    end,

    readBattleMail = function(self, mail)
        self.battle_mail = mail
        self.battle_record:init(mail)
        self:click_Back()
        self:enabledPanel(mail.mailType)
    end,

    readSpyMail = function(self, mail)
        self.spy_report:init(mail)
        self:enabledPanel(mail.mailType)
    end,

    readBeSpyMail = function(self, mail)
        self:loadEnemySpyMail(mail)
        self:enabledPanel(mail.mailType)
    end,

    readGatherMail = function(self, mail)
        self:loadGatherMail(mail)
        self:enabledPanel(mail.mailType)
    end,

    setTopInfo = function(self, mail, panel, cfg)
        local time_1 = panel:Find("Top/Time1"):GetComponent(UnityEngine.UI.Text)
        local time_2 = panel:Find("Top/Time2"):GetComponent(UnityEngine.UI.Text)
        local collect_img = panel:Find("Top/FavouriteBtn/Image"):GetComponent(UnityEngine.UI.Image)
        local title = panel:Find("Top/Title"):GetComponent(UnityEngine.UI.Text)
        local sender = panel:Find("Top/Sender"):GetComponent(UnityEngine.UI.Text)

        time_1.text = os.date("%Y/%m/%d", mail.Sendtime)
        time_2.text = os.date("%X", mail.Sendtime)

        title.text = self.helper:getLangCfg(cfg.title_id)
        sender.text = self.helper:getLangCfg(cfg.sender_id)

        if mail.isCollect == 0 then
            collect_img:SetSprite(res_path.res_path_125)
        else
            collect_img:SetSprite(res_path.res_path_124)
        end

        local btn = panel:Find("Top/DelBtn"):GetComponent(UnityEngine.UI.Button)
        btn.onClick:RemoveAllListeners()
        btn.onClick:AddListener(
            function()
                self:onClickDeleteBtn(mail)
            end
        )

        btn = panel:Find("Top/FavouriteBtn"):GetComponent(UnityEngine.UI.Button)
        btn.onClick:RemoveAllListeners()
        btn.onClick:AddListener(
            function()
                self:onClickFavouriteBtn(mail)
            end
        )
    end,

    normalMailContent = function(self, mail, cfg)
        local text = self.normal_content:Find("Context"):GetComponent(UnityEngine.UI.Text)
        text.text = self.helper:getLangCfg(cfg.text_id)

        for k, v in ipairs(mail.values) do
            text.text = string.gsub(text.text, "/s", v, 1)
        end

        self:setTopInfo(mail, self.normal_content, cfg)
    end,

    haveItemMailContent = function(self, mail, cfg)
        local text = self.append_content:Find("BG/Context"):GetComponent(UnityEngine.UI.Text)
        text.text = self.helper:getLangCfg(cfg.text_id)

        for k, v in ipairs(mail.values) do
            text.text = string.gsub(text.text, "/s", v, 1)
        end

        local layout = self.append_content:Find("Reward/Layout")
        self:loadItem(layout, mail.attachments)

        self:setTopInfo(mail, self.append_content, cfg)
    end,

    activityMailcontent = function(self, mail, cfg)
        local text = self.activity_content:Find("BG/Context"):GetComponent(UnityEngine.UI.Text)
        text.text = self.helper:getLangCfg(cfg.text_id)

        for k, v in ipairs(mail.values) do
            label.text = string.gsub(label.text, "/s", v, 1)
        end

        local title = self.activity_content:Find("Reward/Title/Text"):GetComponent(UnityEngine.UI.Text)
        title.text = string.format( NativeText.GetLangStr(9000042), mail.activeStage )

        local layout = self.activity_content:Find("Reward/Layout")
        self:loadItem(layout, mail.attachments)

        self:setTopInfo(mail, self.activity_content, cfg)
    end,

    crewMailContent = function(self, mail, cfg)
        local cfg_id = tonumber(mail.values[1])
        local crew_cfg = config.crew_blocks[cfg_id]

        if crew_cfg then
            local grade = self.crew_content:Find("BG/Grade"):GetComponent(UnityEngine.UI.Image)
            local icon = self.crew_content:Find("BG/Grade/Icon"):GetComponent(UnityEngine.UI.Image)
            local stars = self.crew_content:Find("BG/Grade/Stars")
            local need_lv = self.crew_content:Find("NeedLevel/Value"):GetComponent(UnityEngine.UI.Text)
            local text = self.crew_content:Find("BG/Context"):GetComponent(UnityEngine.UI.Text)

            text.text = self.helper:getLangCfg(cfg.text_id)
            text.text = string.gsub(text.text, "/s", self.helper:getLangCfg(crew_cfg.name_id), 1)

            grade:SetSprite(gui_helper:getQualityIconPath(crew_cfg.color))
            icon:SetSprite(crew_cfg.path..crew_cfg.icon)

            for i=1, stars.childCount do
                local child = stars:GetChild(i-1)
                child:SetActive(i <= crew_cfg.color)
            end

            need_lv.text = string.format("%s Lv.%d", NativeText.GetLangStr(9000043), crew_cfg.UserLevel)

            local prop_list = loadstring("return "..crew_cfg.EffectPara)()
            local content = self.crew_content:Find("Panel")
            local prefab = content:GetChild(0).gameObject

            local cnt = 0
            for id, value in pairs(prop_list) do
                local elem = nil

                if cnt < content.childCount then
                    elem = content:GetChild(cnt)
                else
                    elem = Object.Instantiate(prefab).transform
                    elem:SetParent(content)
                    resetTransformZ(elem)
                end

                elem:SetActive(true)

                local name = elem:Find("Name"):GetComponent(UnityEngine.UI.Text)
                local val = elem:Find("Value"):GetComponent(UnityEngine.UI.Text)

                local prop_cfg = gui_helper:getPropCfg(id)

                if prop_cfg then
                    name.text = self.helper:getLangCfg(prop_cfg.name_id)

                    if prop_cfg.isfloat > 0 then
                        value = value * 0.0001
                    end

                    if prop_cfg.showpercent > 0 then
                        value = value * 100
                        val.text = tostring(value).."%"
                    else
                        val.text = tostring(value)
                    end
                else
                    log.error("########################################### find crew prop cfg error ! id", id)
                end

                cnt = cnt + 1
            end

            for i=cnt, content.childCount - 1 do
                local child = content:GetChild(i)
                child:SetActive(false)
            end
        else
            log.error("############################################### find crew cfg error ! id", cfg_id)
        end

        self:setTopInfo(mail, self.crew_content, cfg)
    end,

    loadItem = function(self, node, rewards)
        local panel = node:Find("Items/Scroll View/Viewport/Content")
        local prefab = panel:GetChild(0).gameObject
        
        local diamond_node = node:Find("Diamond")
        diamond_node:SetActive(false)
        
        local list = {}

        for k, v in ipairs(rewards) do
            if v.ItemTemId == 30001 then
                diamond_node:SetActive(true)
                local count = diamond_node:Find("Icon/Count"):GetComponent(UnityEngine.UI.Text)
                count.text = "x"..v.Count
            else
                table.insert( list, v )
            end
        end
        
        local items = node:Find("Items")
        items:SetActive(#list > 0)

        for k, v in ipairs(list) do
            local elem = nil
            if k > panel.childCount then
                elem = Object.Instantiate(prefab).transform
                elem:SetParent(panel)
                resetTransformZ(elem)
            else
                elem = panel:GetChild(k-1)
            end

            elem.name = "Item_"..k
            elem.localScale = Vector3.one

            local grade = elem:GetComponent(UnityEngine.UI.Image)
            local icon = elem:Find("Icon"):GetComponent(UnityEngine.UI.Image)
            local num = elem:Find("Count"):GetComponent(UnityEngine.UI.Text)
            local btn = elem:GetComponent(UnityEngine.UI.Button)

            local cfg = item_helper:getItemCfg(v.ItemTemId)
            
            if cfg then
                grade:SetSprite(gui_helper:getQualityIconPath(cfg.color))
                icon:SetSprite(gui_helper:getItemIconPath(cfg))
                num.text = "x"..v.Count
                
                btn.onClick:RemoveAllListeners()
                btn.onClick:AddListener(
                    function()
                        local item_inst = item_cls({cfgId = v.ItemTemId, count = frame_helper:getItemNum(v.ItemTemId)})
                        frame_manager:show("itemInfo", nil, {item = item_inst})
                    end
                )
            end
        end

        for i=#list, panel.childCount - 1 do
            local child = panel:GetChild(i)
            child.localScale = Vector3.zero
        end
    end,

    prisonMailContent = function(self, mail, cfg)
        self.prison_content:SetActive(true)

        local label = self.prison_content:Find("Viewport/Content/Context"):GetComponent(UnityEngine.UI.Text)
        label.text = self.helper:getLangCfg(cfg.text_id)

        for k, v in ipairs(mail.values) do
            label.text = string.gsub(label.text, "/s", v, 1)
        end

        local heros = self.prison_content:Find("Viewport/Content/Heros")
        local prefab = heros:GetChild(0).gameObject

        for k, v in ipairs(mail.prisonermail) do
            local elem = nil

            if k > heros.childCount then
                elem = Object.Instantiate(prefab).transform
                elem:SetParent(heros)
                resetTransformZ(elem)
            else
                elem = heros:GetChild(k-1)
            end

            elem:SetActive(true)
            elem.name = "Hero_"..k

            self:setOfficerHead(elem, v)
        end

        for i=#mail.prisonermail, heros.childCount - 1 do
            local elem = heros:GetChild(i)
            elem:SetActive(false)
        end
    end,

    setOfficerHead = function(self, node, hero)
        local icon = node:Find("Icon"):GetComponent(UnityEngine.UI.Image)
        local name = node:Find("Name"):GetComponent(UnityEngine.UI.Text)
        local type_img = node:Find("Type"):GetComponent(UnityEngine.UI.Image)
        local grade = node:GetComponent(UnityEngine.UI.Image)

        local res_id = 0

        local cfg = config.commander_basics[hero.officerId]
        if cfg then
            grade:SetSprite(gui_helper:getQualityIconPath(cfg.color))
            icon:SetSprite(cfg.path..cfg.icon)
        else
            log.error("#### officer cfg not find ! id", hero.officerId)
        end

        if string.len(hero.officerName) == 0 then
            name.text = self.helper:getLangCfg(cfg.commander_name)
            res_id = hero.officerId
        else
            name.text = hero.officerName
            res_id = hero.iconId
        end

        type_img:SetSprite(gui_helper:getHeroTypeIconPath(hero.tankid))
    end,

    updateBtnState = function(self, type)
        for k, btn in pairs(self.tag_btns) do
            if k == type then
                btn.targetGraphic.sprite = btn.spriteState.pressedSprite
            else
                btn.targetGraphic.sprite = btn.spriteState.disabledSprite
            end

            local img = btn.transform:GetComponent(UnityEngine.UI.Image)
            img:SetNativeSize()
        end

        self.mail_type = type
    end,

    updateMailCount = function(self)
        for k, btn in pairs(self.tag_btns) do
            local red_point = btn.transform:Find("RedPoint")
            local count = red_point:GetChild(0):GetComponent(UnityEngine.UI.Text)

            local cnt = self:unreadMailCount(k)
            red_point:SetActive(cnt > 0)
            count.text = tostring(cnt)
        end
    end,

    unreadMailCount = function(self, type)
        local cnt = 0
        for k, v in ipairs(self.mails) do
            if type == FAV then
                if v.isCollect == 1 and v.mailHaveRead == 0 then
                    cnt = cnt + 1
                end
            else
                if page_types[type][v.mailType] and v.mailHaveRead == 0 then
                    cnt = cnt + 1
                end
            end
        end

        return cnt
    end,

    tipBox = function(self, context, cb)
        local tb = {
            title = "",
            content = context,
            buttons =
            {
                {
                    btn_name = NativeText.GetLangStr(110522),
                    callback = function() 
                        frame_manager:hide('common_box')
                    end
                },
                {
                    btn_name = NativeText.GetLangStr(110521),
                    callback = function()
                        if cb then cb() end
                        frame_manager:hide('common_box')
                    end,
                },
            },
        }

        frame_manager:show('common_box', nil, tb)
    end,

    loadEnemySpyMail = function(self, mail)
        local time_1 = self.enemy_spy:Find("Scroll View/Viewport/Content/Top/Time1"):GetComponent(UnityEngine.UI.Text)
        local time_2 = self.enemy_spy:Find("Scroll View/Viewport/Content/Top/Time2"):GetComponent(UnityEngine.UI.Text)

        time_1.text = os.date("%Y/%m/%d", mail.Sendtime)
        time_2.text = os.date("%X", mail.Sendtime)

        local title = self.enemy_spy:Find("Scroll View/Viewport/Content/Top/Title"):GetComponent(UnityEngine.UI.Text)
        local sender = self.enemy_spy:Find("Scroll View/Viewport/Content/Top/Sender"):GetComponent(UnityEngine.UI.Text)
        sender.text = NativeText.GetLangStr(9000038)
        title.text = NativeText.GetLangStr(9000039)

        local content = self.enemy_spy:Find("Scroll View/Viewport/Content")
        local prefab = content:GetChild(1).gameObject

        for k, v in ipairs(mail.beSpyReport) do
            local elem = nil

            if k > content.childCount - 1 then
                elem = Object.Instantiate(prefab).transform
                elem:SetParent(content)
                resetTransformZ(elem)
            else
                elem = content:GetChild(k)
            end

            elem:SetActive(true)

            local build_name = elem:Find("Top/Name"):GetComponent(UnityEngine.UI.Text)
            local pos = elem:Find("Top/Position"):GetComponent(UnityEngine.UI.Text)
            local time_text = elem:Find("Top/Time"):GetComponent(UnityEngine.UI.Text)
            local player_name = elem:Find("Panel/Name"):GetComponent(UnityEngine.UI.Text)
            local scout_name = elem:Find("Panel/Text1"):GetComponent(UnityEngine.UI.Text)
            local player_icon = elem:Find("Panel/Head/Icon"):GetComponent(UnityEngine.UI.Image)

            player_icon:SetSprite(gui_helper:getPlayerIcon())
            
            if v.objType == common_pb.WORLD_OBJ_PLAYER then
                local cfg = config.worldmap_base_configs[v.cfgId]
                build_name.text = self.helper:getLangCfg(cfg.name_id)
                scout_name.text = self.helper:getLangCfg(cfg.name_id).." "..NativeText.GetLangStr(9000044)
            elseif v.objType == common_pb.WORLD_OBJ_WILD_MINE then
                local cfg = config.worldmap_titaniums[v.cfgId]
                build_name.text = self.helper:getLangCfg(cfg.nameid)
                scout_name.text = self.helper:getLangCfg(cfg.nameid).." "..NativeText.GetLangStr(9000044)
            end

            pos.text = gui_helper:positionFormat(v.posX, v.posY, v.svrId)
            
            if string.len(v.shortName) > 2 then
                player_name.text = string.format( "[%s]%s", v.shortName, v.playerName )
            else
                player_name.text = v.playerName
            end

            log.info("##################################### player name ", v.playerName)

            time_text.text = string.format( "%s %s", os.date("%Y/%m/%d", v.spyTime), os.date("%X", v.spyTime) )
        end

        for i=#mail.beSpyReport + 1, content.childCount - 1 do
            local child = content:GetChild(i)
            child:SetActive(false)
        end
    end,

    loadGatherMail = function(self, mail)
        local time_1 = self.gather_content:Find("Scroll View/Viewport/Content/Top/Time1"):GetComponent(UnityEngine.UI.Text)
        local time_2 = self.gather_content:Find("Scroll View/Viewport/Content/Top/Time2"):GetComponent(UnityEngine.UI.Text)

        time_1.text = os.date("%Y/%m/%d", mail.Sendtime)
        time_2.text = os.date("%X", mail.Sendtime)

        local title = self.gather_content:Find("Scroll View/Viewport/Content/Top/Title"):GetComponent(UnityEngine.UI.Text)
        local sender = self.gather_content:Find("Scroll View/Viewport/Content/Top/Sender"):GetComponent(UnityEngine.UI.Text)
        sender.text = NativeText.GetLangStr(9000040)

        local cfg = self:getCfg(mail.gatherReport[1].objType, mail.gatherReport[1].cfgId)

        if cfg then
            title.text = string.format( NativeText.GetLangStr(9000041), self.helper:getLangCfg(cfg.nameid) )
        else
            log.error("###################################### find mine cfg error ! id", mail.gatherReport[1].cfgId)
        end

        local content = self.gather_content:Find("Scroll View/Viewport/Content")
        local prefab = content:GetChild(1).gameObject

        for k, v in ipairs(mail.gatherReport) do
            local elem = nil

            if k > content.childCount - 1 then
                elem = Object.Instantiate(prefab).transform
                elem:SetParent(content)
                resetTransformZ(elem)
            else
                elem = content:GetChild(k)
            end

            elem:SetActive(true)

            local build_name = elem:Find("Top/Name"):GetComponent(UnityEngine.UI.Text)
            local pos = elem:Find("Top/Position"):GetComponent(UnityEngine.UI.Text)
            local time_text = elem:Find("Top/Time"):GetComponent(UnityEngine.UI.Text)

            local cfg = config.worldmap_titaniums[v.cfgId]

            if cfg then
                build_name.text = self.helper:getLangCfg(cfg.nameid)
            else
                log.error("########################################## find res cfg error ! id", v.cfgId)
            end

            pos.text = gui_helper:positionFormat(v.posX, v.posY)
            time_text.text = string.format( "%s %s", os.date("%Y/%m/%d", v.spyTime), os.date("%X", v.gatherTime) )

            local panel = elem:Find("Panel")
            local prefab = panel:GetChild(0).gameObject

            for idx, item in ipairs(v.item) do
                local res = nil

                if idx > panel.childCount then
                    res = Object.Instantiate(prefab).transform
                    res:SetParent(panel)
                    resetTransformZ(res)
                else
                    res = panel:GetChild(idx-1)
                end

                res:SetActive(true)

                local icon = res:Find("Icon"):GetComponent(UnityEngine.UI.Image)
                local count = res:Find("Icon/Text"):GetComponent(UnityEngine.UI.Text)

                local item_cfg = item_helper:getItemCfg(item.ItemTemId)

                if item_cfg then
                    icon:SetSprite(gui_helper:getItemIconPath(item_cfg))
                    count.text = self.helper:itemNumFormat(item.Count)
                else
                    log.error("########################################## find item cfg error ! id", item.ItemTemId)
                end
            end

            for i=#v.item, panel.childCount - 1 do
                local child = panel:GetChild(i)
                child:SetActive(false)
            end
        end

        for i=#mail.gatherReport + 1, content.childCount - 1 do
            local child = content:GetChild(i)
            child:SetActive(false)
        end
    end,

    onClickMail = function(self, handle, mail_id)
        log.info("#### click mail id ", mail_id)
        if self.operating then
            if self.toggle_list[mail_id] then
                self.toggle_list[mail_id].isOn = not self.toggle_list[mail_id].isOn
            end
        else
            network_sender:readMail(mail_id)
        end
    end,

    onClickDeleteBtn = function(self, mail)
        self.select_mails = {}
        table.insert(self.select_mails, mail.MailId)
        network_sender:deleteMail(self.select_mails)
    end,

    onClickFavouriteBtn = function(self, mail)
        network_sender:collectMail(mail.MailId)
    end,

    click_BackBtn = function(self)
        frame_manager:hide("mail")
    end,

    click_SelectAllBtn = function(self)
        if self.page_mail_cnt == 0 then
            return
        end

        if self.operating then
            self:exitOperating()
        else
            for k, v in pairs(self.toggle_list) do
                v.isOn = true
            end
            self.select_text.text = NativeText.GetLangStr(9000045)
        end
    end,

    click_SystemBtn = function(self)
        self:onClickTag(SYS)
    end,

    click_BattleBtn = function(self)
        self:onClickTag(BAT)
    end,

    click_ServiceBtn = function(self)
        self:onClickTag(PER)
    end,

    click_Favourite = function(self)
        self:onClickTag(FAV)
    end,

    onClickTag = function(self, type)
        self:exitOperating()
        self:updateBtnState(type)
        self:loadMailList(type)
        self:readFirstMail(type)
    end,

    click_Report = function(self)
        self.battle_record:hide()
        self.battle_details:show()
        self.battle_details:init(self.battle_mail)
    end,

    click_Back = function(self)
        self.battle_record:show()
        self.battle_details:hide()
    end,

    click_DelBtn = function(self)
        self.select_mails = {}
        table.insert(self.select_mails, self.curr_mail.MailId)
        network_sender:deleteMail(self.select_mails)
    end,

    click_MoreBtn = function(self)
        self:click_BackBtn()
        frame_manager:show('activity_main')
    end,

    click_Delete = function(self)
        network_sender:deleteMail(self.select_mails)
    end,

    click_Read = function(self)
        network_sender:readMoreMails(self.select_mails)
    end,

    click_Cancel = function(self)
        self:exitOperating()
    end,
}