local super = require("ui.frame")
local log = logger.get("Game", "Main Task Frame")
local gui_helper = require('gui.gui_helper')

-- 跳转界面信息
local jump_frames = {
    [1] = {name = "hero_list", params = {}},
    [2] = {name = "bag", params = {}},
}

return class
{
    super = super,

    onLoaded = function (self)
        super.onLoaded(self)

        self.owner = self.args.owner
        self.hide_branch = self.args.data
        self.transform:SetParent(self.owner)
        makeRectTransformIdentity(self.transform)
        self.transform:SetAsFirstSibling()

        -- 路径: root/login
        -- 事件: click_login 
        clickEvents = {
            "HideButton",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self.task_prefab = self.transform:Find('TaskPrefab').gameObject
        self.helper.hideTrans(self.task_prefab.transform)

        self.task_root = self.transform:Find('TaskRoot')
        self.scroll_content = self.transform:Find('TaskRoot/BranchTask/Scroll View/Viewport/Content')
        self.decoration_bar = self:findImage('DecorationBar')
        self.branch_task = self.transform:Find('TaskRoot/BranchTask')
        self.main_task = self.transform:Find('TaskRoot/MainTask')
        self.hide_btn = self.transform:Find('HideButton')

        self.task_evts = {}
        self.task_changed = false
        self.isVisible = true

        -- 注册事件回调
        self.changed_evts = {}
        self.changed_evts["onTaskChanged"] = task_manager.events:on("onTaskChanged", functor.make(self.onTaskChanged, self))
        self.changed_evts["onTaskFinished"] = task_manager.events:on("onTaskFinished", functor.make(self.onTaskFinished, self))
        self.get_items_evt = game_director.events:on("NOTIFYGETITEM", functor.make(self.onGetItems, self))

        self:refreshMainTask()
        self:refreshBranchTask()
    end,

    onUnload = function (self)
        if self.changed_evts then
            for k, v in pairs(self.changed_evts) do
                task_manager.events:remove(k, v)
            end
        end
        game_director.events:remove("NOTIFYGETITEM", self.get_items_evt)
    end,

    onRcvMsg = function(self, cmd, msg)
    end,

    update = function(self)
        if self.task_changed then
            self:clearTaskEvts()
            self:refreshMainTask()
            self:refreshBranchTask()
            self.task_changed = false
        end
    end,

    clearTaskEvts = function(self)
        for path, evt in pairs(self.task_evts) do
            self:clearScript(evt, path)
        end
        self.task_evts = {}
    end,

    onTaskChanged = function(self, tasks)
        self.task_changed = true
    end,

    onTaskFinished = function(self, id)
        self.task_changed = true
    end,

    onGetItems = function(self, msg)
        if msg.reason == common_pb.TASK_REWARD then
           --gui_helper:showGetItemsPanel(msg.getItems, true)
           frame_manager:show('getItems', self, {items = gui_helper.drop_list[msg.reason], has_Confirm_Btn = true})
        end
    end,

    -- 刷新主线任务
    refreshMainTask = function(self)
        local tasks = task_manager:getSortTasks(2)
        local detail = self.transform:Find('TaskRoot/MainTask/DetailPanel/Panel')
        local none_task = self.transform:Find('TaskRoot/MainTask/DetailPanel/None')

        for _, task in ipairs(tasks) do
            local path = 'TaskRoot/MainTask/DetailPanel'
            self:setTaskInfo(detail, task, path)
            has_task = true
        end

        detail:SetActive(#tasks ~= 0)
        none_task:SetActive(#tasks == 0)
    end,

    -- 刷新支线任务
    refreshBranchTask = function(self)
        if self.hide_branch then
            self.branch_task:SetActive(false)
            return 
        end

        local tasks = task_manager:getSortTasks(3)
        local childCount = self.scroll_content.childCount
        for idx, task in ipairs(tasks) do
            if idx > childCount then
                -- 创建新的 prefab
                local task_trans = Object.Instantiate(self.task_prefab).transform
                task_trans:SetParent(self.scroll_content)
                resetTransformZ(task_trans)
            end

            local trans = self.scroll_content:GetChild(idx-1)
            trans:SetActive(true)

            local path = 'TaskRoot/BranchTask/Scroll View/Viewport/Content/'..trans.name..'_'..task.id
            self:setTaskInfo(trans, task, path)
        end

        for i=#tasks, self.scroll_content.childCount-1 do
            self.scroll_content:GetChild(idx):SetActive(false)
        end

        self.branch_task:SetActive(#tasks ~= 0)
    end,

    -- 设置任务信息
    setTaskInfo = function(self, task_trans, task, path)
        local task_title = task_trans:Find('Title'):GetComponent(UnityEngine.UI.Text)
        local task_des = task_trans:Find('Detail'):GetComponent(UnityEngine.UI.Text)
        local task_progress = task_trans:Find('Progress'):GetComponent(UnityEngine.UI.Text)
        local task_status = task_trans:Find('Status')
        
        task_title.text = self.helper:getLangCfg(task.cfg.title)
        task_des.text = self.helper:getLangCfg(task.cfg.des_id)
        -- 设置任务进度
        for k, v in pairs(self.helper:getJsonTab(task.cfg.p1)) do
            task_progress.text = task.progress[k].prog.."/"..task.progress[k].total
        end
        -- 隐藏/显示可领取状态
        task_status:SetActive(task.state == common_pb.TS_GETAWARD)

        -- 注册点击事件
        self:setScript("single", path, functor.rmake(self.onClick_GetReward, task))
        self.task_evts[path] = "single"
    end,

    -- 点击任务按钮
    onClick_GetReward = function(self, handler, data, task)
        log.info('onClick_GetReward '..task.id)
        if task.state == common_pb.TS_GETAWARD then  --可领取状态
            network_sender:getNorTaskRewards(task.id, common_pb.Norm_Task)
        elseif task.state == common_pb.TS_DOING then  --进行中状态
            -- 界面跳转
            if jump_frames[task.cfg.jump] then
                frame_manager:show(jump_frames[task.cfg.jump].name, self, jump_frames[task.cfg.jump].params)
            end
        end
    end,

    -- 点击显示/隐藏按钮
    click_HideButton = function(self)
        if self.isVisible then
            self.decoration_bar.fillAmount = 0.12
            self.hide_btn.localRotation = Vector3(0,0,180)
        else
            self.decoration_bar.fillAmount = 1.0
            self.hide_btn.localRotation = Vector3.zero
        end
        self.isVisible = not self.isVisible
        self.task_root:SetActive(self.isVisible)
    end,
}