local log = logger.get("Game", "March List Panel")

return class{
    ctor = function(self, panel, owner)
        self.panel = panel
        self.owner = owner
        self.helper = require("gui.tool.frame_helper")
        
        self.content = self.panel:Find("offset/marchs/Viewport/Content")
        self.prefab = self.panel:Find("marchLineUI").gameObject
        self.battle_cnt_text = self.panel:Find("offset/bg/Battle/Text"):GetComponent(UnityEngine.UI.Text)
        self.gather_cnt_text = self.panel:Find("offset/bg/Gather/Text"):GetComponent(UnityEngine.UI.Text)

        self.elem_num = 0
        self.all_info = {}
        self.hold_queue = {}
        self.march_queue = {}
        self.mass_queue = {}
        self.support_queue = {}
        self.mass_atk = {}

        self:setQueueCnt()
    end,

    setQueueCnt = function(self)
        local cur_cnt, max_cnt = world_map_helper:getQueueCount(common_pb.EQUEUE_BATTLE)
        self.battle_cnt_text.text = string.format( "%d/%d", cur_cnt, max_cnt)

        cur_cnt, max_cnt = world_map_helper:getQueueCount(common_pb.EQUEUE_TITAN)
        self.gather_cnt_text.text = string.format( "%d/%d", cur_cnt, max_cnt)
    end,

    update = function(self, dt)
        local tab = {}
        local countDown = function(v)
            local fill = v.ui:Find("Fill"):GetComponent(UnityEngine.UI.Image)

            if v.data.left_time > 0 then
                v.data.left_time = v.data.left_time - dt
        
                fill.fillAmount = 1 - v.data.left_time / (v.data.total_time > 0 and v.data.total_time or 1)
            else
                if v.data.type == progress_bar.hold then
                    table.insert(tab, v.data.id)
                end
                v.data.left_time = 0
                fill.fillAmount = 0
            end
        end

        for k, v in pairs(self.hold_queue) do
            countDown(v)
        end

        for k, v in pairs(self.march_queue) do
            countDown(v)
        end

        for k, v in pairs(self.mass_queue) do
            countDown(v)
        end

        for k, v in pairs(self.support_queue) do
            countDown(v)
        end

        for k, v in pairs(tab) do
            self:removeQueueInfo(progress_bar.hold, v)
        end
    end,

    storQueueInfo = function(self, data, elem)
        if data.type == progress_bar.hold then
            self.hold_queue[data.id] = {ui = elem, data = data}

        elseif data.type == progress_bar.marchLine then
            self.march_queue[data.id] = {ui = elem, data = data}

        elseif data.type == progress_bar.mass then
            self.mass_queue[data.id] = {ui = elem, data = data}

        elseif data.type == progress_bar.support then
            self.support_queue[data.id] = {ui = elem, data = data}
        end

        self.all_info[data.id] = data

        --self:setQueueCnt()
    end,

    removeQueueInfo = function(self, type, id)
        if type == progress_bar.hold then

            if self.hold_queue[id] then
                self:removeElem(self.hold_queue[id].ui)
                self.hold_queue[id] = nil
            end

        elseif type == progress_bar.marchLine then
            for k, v in pairs(self.all_info) do
                if v.id == id then
                    if v.type == progress_bar.marchLine then

                        if self.march_queue[id] then
                            self:removeElem(self.march_queue[id].ui)
                            self.march_queue[id] = nil
                        end

                    elseif v.type == progress_bar.mass then
                        
                        if self.mass_queue[id] then
                            self:removeElem(self.mass_queue[id].ui)
                            self.mass_queue[id] = nil
                        end

                        for k, v in pairs(self.mass_atk) do
                            if v.id == id then
                                self.mass_atk[v.mass_id] = nil
                                break
                            end
                        end

                    elseif v.type == progress_bar.support then
                        if self.support_queue[id] then
                            self:removeElem(self.support_queue[id].ui)
                            self.support_queue[id] = nil
                        end
                    end
                end
            end
        end

        self.all_info[id] = nil

        --self:setQueueCnt()
    end,

    removeElem = function(self, elem)
        elem:SetAsLastSibling()
        elem:SetActive(false)
        self.elem_num = self.elem_num - 1
        self:tweenMarchLine()
    end,

    addElem = function(self, info, prog_type)
        if prog_type == progress_bar.marchLine then
            self:addMarchQueue(info)
        elseif prog_type == progress_bar.hold then
            self:addHoldQueue(info)
        end
    end,

    addHoldQueue = function(self, info)
        if info.playerId ~= local_player.id then
            return
        end

        if info.buildState ~= common_pb.EBUILD_STATE_BUILD then
            return
        end

        local data = {}
        data.type = progress_bar.hold
        data.left_time = info.leftTime
        data.total_time = info.totalTime
        data.id = info.instId
        data.pos = {}
        data.pos.x = info.posX
        data.pos.y = info.posY
        data.icon_path = res_path.res_path_83

        local elem = self:createElem(data)
        self:setHoldInfo(elem, data)
    end,

    setHoldInfo = function(self, elem, data)
        local clk_trans = elem:Find('timeText')
        clk_trans:SetActive(true)

        local clk = clk_trans:GetComponent(Clock)
        if not clk then
            clk = clk_trans:AddComponent(Clock)
        end
        clk:Set(math.floor(data.left_time))

        local des = elem:Find("desText"):GetComponent(UnityEngine.UI.Text)
        des.text = data.str or ""

        local fill = elem:Find("Fill"):GetComponent(UnityEngine.UI.Image)
        fill.type = UnityEngine.UI.Image.Type.Filled
        fill:SetSprite(res_path.res_path_62)

        local speed_btn = elem:Find("speedBtn")
        local info_btn = elem:Find("infoBtn")
        local cancel_btn = elem:Find("cancelBtn")
        local back_btn = elem:Find("backBtn")
        speed_btn:SetActive(false)
        info_btn:SetActive(false)
        cancel_btn:SetActive(true)
        back_btn:SetActive(false)
        
        local icon = elem:Find("Icon"):GetComponent(UnityEngine.UI.Image)
        icon:SetSprite(data.icon_path)

        self:storQueueInfo(data, elem)

        local path = "new/m_line/offset/marchs/Viewport/Content/"..elem.name
        self.owner:clearScript("click", path)
        self.owner:setScript("click", path, functor.rmake(self.onClickHoldElem, data))

        path = "new/m_line/offset/marchs/Viewport/Content/"..elem.name.."/cancelBtn"
        self.owner:clearScript("click", path)
        self.owner:setScript("click", path, functor.rmake(self.onClickCancel, data))

        self:tweenMarchLine()
    end,

    copyMassData = function(self, src, id)
        local dest = {}
        dest.id = id
        dest.march_info = src.march_info
        dest.mass_id = src.mass_id
        dest.dest_id = src.dest_id
        dest.back = src.goHome
        dest.type = src.type
        dest.left_time = src.left_time
        dest.total_time = src.total_time
        dest.pos = {}
        dest.pos.x = src.pos.x
        dest.pos.y = src.pos.y
        dest.state = src.state
        dest.show_up = src.show_up
        dest.queue_type = src.queue_type
        dest.str = src.str
        dest.marchAction = src.marchAction
        dest.beforeAction = src.beforeAction
        dest.icon_path = res_path.res_path_81

        return dest
    end,

    createMarchData = function(self, info)
        local data = {}
        data.id = info.marchLineId
        data.march_info = info
        data.mass_id = info.massid
        data.dest_id = info.dstInstid
        data.back = info.goHome
        data.type = progress_bar.marchLine
        data.left_time = (info.totalTime - info.elapsedTime) * 0.001
        data.total_time = info.totalTime * 0.001
        data.pos = {}
        data.pos.x = info.dstPosX
        data.pos.y = info.dstPosY
        data.show_up = true
        data.state = "moving"
        data.queue_type = common_pb.EQUEUE_TRANS
        data.str = ""
        data.marchAction = info.marchAction
        data.beforeAction = info.beforeAction
        data.icon_path = res_path.res_path_81

        return data
    end,

    addMarchQueue = function(self, info)
        if not world_map_helper:isOwnMarchLine(info) then
            return
        end

        local data = self:createMarchData(info)

        if info.marchAction == common_pb.EMARCH_GO_HOME then
            if info.beforeAction == common_pb.EMARCH_MASS_ATK or 
                info.beforeAction == common_pb.EMARCH_CREATE_MASS or 
                info.beforeAction == common_pb.EMARCH_JOIN_MASS then

                data.type = progress_bar.mass
                data.queue_type = common_pb.EQUEUE_BATTLE

            elseif info.beforeAction == common_pb.EMARCH_HELP then
                data.type = progress_bar.support
                data.queue_type = common_pb.EQUEUE_BATTLE
            elseif info.beforeAction == common_pb.EMARCH_ATK then
                data.queue_type = common_pb.EQUEUE_BATTLE
            elseif info.beforeAction == common_pb.EMARCH_BUILD then
                data.queue_type = common_pb.EQUEUE_BATTLE
            elseif info.beforeAction == common_pb.EMARCH_GATHER_TITAN then
                data.queue_type = common_pb.EQUEUE_BATTLE
                data.icon_path = res_path.res_path_84
            elseif info.beforeAction == common_pb.EMARCH_ROB_TITAN then
                data.queue_type = common_pb.EQUEUE_BATTLE
            end
        elseif info.marchAction == common_pb.EMARCH_MASS_ATK then
            data.type = progress_bar.mass
            data.queue_type = common_pb.EQUEUE_BATTLE
            if data.left_time <= 0  then
                data.state = "fighting"
                data.str = NativeText.GetLangStr(370001)
                data.icon_path = res_path.res_path_86
            end
        elseif info.marchAction == common_pb.EMARCH_HELP then
            data.type = progress_bar.support
            data.queue_type = common_pb.EQUEUE_BATTLE
            if data.left_time <= 0  then
                data.state = "supporting"
                data.str = NativeText.GetLangStr(370002)
            end
        elseif info.marchAction == common_pb.EMARCH_GATHER_TITAN then
            data.type = progress_bar.marchLine
            data.queue_type = common_pb.EQUEUE_TITAN
            if data.left_time <= 0  then
                data.state = "gathering"
                data.str = "采集中"
            end
            data.icon_path = res_path.res_path_84
        elseif info.marchAction == common_pb.EMARCH_CREATE_MASS or 
            info.marchAction == common_pb.EMARCH_JOIN_MASS then

            data.type = progress_bar.mass
            data.queue_type = common_pb.EQUEUE_BATTLE
            if data.left_time <= 0  then
                data.state = "waiting"
                data.str = NativeText.GetLangStr(370003)
            end
        elseif info.marchAction == common_pb.EMARCH_ATK then
            data.queue_type = common_pb.EQUEUE_BATTLE
            if data.left_time <= 0  then
                data.state = "fighting"
                data.str = NativeText.GetLangStr(370001)
                data.icon_path = res_path.res_path_86
            end
        elseif info.marchAction == common_pb.EMARCH_BUILD then
            data.queue_type = common_pb.EQUEUE_BATTLE
            if data.left_time <= 0  then
                data.state = "fighting"
                data.str = NativeText.GetLangStr(370001)
                data.icon_path = res_path.res_path_86
            end
        elseif info.marchAction == common_pb.EMARCH_ROB_TITAN then
            data.queue_type = common_pb.EQUEUE_BATTLE
            if data.left_time <= 0  then
                data.state = "fighting"
                data.str = NativeText.GetLangStr(370001)
                data.icon_path = res_path.res_path_86
            end
        elseif info.marchAction == common_pb.EMARCH_PRE_TEAM then
            data.queue_type = common_pb.EQUEUE_TRANS
            data.icon_path = res_path.res_path_85
        end

        if data.type == progress_bar.mass then
            if self:isMassAttack(data.marchAction, data.beforeAction) then
                self.mass_atk[data.mass_id] = data
                self:refreshMassMarch(data)
                return
            else
                local atk_data = self.mass_atk[data.mass_id]
                if atk_data then
                    local id = data.id
                    data = self:copyMassData(atk_data, id)
                end
            end
        end

        local elem = self:createElem(data)
        self:setMarchInfo(elem, data)
    end,

    setMarchInfo = function(self, elem, data)
        local battle_cnt = world_map_helper:getQueueCount(common_pb.EQUEUE_BATTLE)
        local trans_cnt = world_map_helper:getQueueCount(common_pb.EQUEUE_TRANS)
        local idx = battle_cnt + trans_cnt
        idx = idx > 0 and idx-1 or 0

        if data.queue_type == common_pb.EQUEUE_BATTLE then
            idx = battle_cnt > 0 and battle_cnt-1 or 0
        end
        
        elem:SetSiblingIndex(idx)

        local clk_trans = elem:Find('timeText')
        clk_trans:SetActive(data.state == "moving")

        if data.state == "moving" then
            local clk = clk_trans:GetComponent(Clock)
            if not clk then
                clk = clk_trans:AddComponent(Clock)
            end
            clk:Set(math.floor(data.left_time))
        end

        local des = elem:Find("desText"):GetComponent(UnityEngine.UI.Text)
        des.text = data.str or ""

        local fill = elem:Find("Fill"):GetComponent(UnityEngine.UI.Image)
        fill.type = UnityEngine.UI.Image.Type.Filled
        if data.back then
            fill:SetSprite(res_path.res_path_62)
        else
            fill:SetSprite(res_path.res_path_63)
        end

        local speed_btn = elem:Find("speedBtn")
        local info_btn = elem:Find("infoBtn")
        local cancel_btn = elem:Find("cancelBtn")
        local back_btn = elem:Find("backBtn")
        speed_btn:SetActive(data.state == "moving")
        info_btn:SetActive(data.state ~= "moving")
        cancel_btn:SetActive(false)
        back_btn:SetActive(data.state == "gathering")

        local icon = elem:Find("Icon"):GetComponent(UnityEngine.UI.Image)
        icon:SetSprite(data.icon_path)

        self:storQueueInfo(data, elem)

        local path = "new/m_line/offset/marchs/Viewport/Content/"..elem.name
        self.owner:clearScript("click", path)
        self.owner:setScript("click", path, functor.rmake(self.onClickMarchElem, data))

        path = "new/m_line/offset/marchs/Viewport/Content/"..elem.name.."/speedBtn"
        self.owner:clearScript("click", path)
        self.owner:setScript("click", path, functor.rmake(self.onClickSpeedUp, data))

        path = "new/m_line/offset/marchs/Viewport/Content/"..elem.name.."/infoBtn"
        self.owner:clearScript("click", path)
        self.owner:setScript("click", path, functor.rmake(self.onClickInfo, data))

        path = "new/m_line/offset/marchs/Viewport/Content/"..elem.name.."/backBtn"
        self.owner:clearScript("click", path)
        self.owner:setScript("click", path, functor.rmake(self.onClickBack, data))

        self:tweenMarchLine()
    end,

    refreshMassMarch = function(self, data)
        for k, v in pairs(self.mass_queue) do
            if v.data.mass_id == data.mass_id then
                local id = v.data.id
                local copy_data = self:copyMassData(data, id)
                self:setMarchInfo(v.ui, copy_data)
            end
        end
    end,

    getElem = function(self, data)
        if data.type == progress_bar.hold then

            if self.hold_queue[data.id] then
                return self.hold_queue[data.id].ui
            end

        elseif data.type == progress_bar.marchLine then
            
            if self.march_queue[data.id] then
                return self.march_queue[data.id].ui
            end

        elseif data.type == progress_bar.mass then

            if self.mass_queue[data.id] then
                return self.mass_queue[data.id].ui
            end

        elseif data.type == progress_bar.support then
            
            if self.support_queue[data.id] then
                return self.support_queue[data.id].ui
            end
        end
    end,

    createElem = function(self, data)
        local elem = self:getElem(data)

        if not elem then
            elem = self:addPrefab(data.id)
        end

        return elem
    end,

    addPrefab = function(self, id)
        local ui = nil

        if self.elem_num < self.content.childCount then
            ui = self.content:GetChild(self.elem_num)
        else
            ui = Object.Instantiate(self.prefab).transform
            ui:SetParent(self.content)
            resetTransformZ(ui)
        end

        if not ui then
            return nil
        end

        ui:SetActive(true)
        ui.name = "line_"..id

        self.elem_num = self.elem_num + 1

        return ui
    end,

    isMassAttack = function(self, action, before)
        if (action == common_pb.EMARCH_GO_HOME and
            before == common_pb.EMARCH_MASS_ATK) or 
            action == common_pb.EMARCH_MASS_ATK then

            return true
        end

        return false
    end,

    onClickCancel = function(self, handler, data)
        cb = function()
            network_sender:destroyHold(data.id)
            self:hide()
        end
        
        local tb = {
            title = "",
            content = NativeText.GetLangStr(370004),
            buttons =
            {
                {
                    btn_name = NativeText.GetLangStr(110522),
                    callback = function() 
                        frame_manager:hide('common_box')
                    end
                },
                {
                    btn_name = NativeText.GetLangStr(110521),
                    callback = function()
                        cb()
                        frame_manager:hide('common_box')
                    end
                },
            },
        }

        frame_manager:show('common_box', nil, tb)
    end,

    onClickHoldElem = function(self, handler, data)
        self.owner:lookAtPosition(data.pos.x, data.pos.y)
    end,

    onClickMarchElem = function(self, handler, data)
        if data.state == "moving" then
            network_sender:queryWorldMap(data.march_info.marchLineId, protocol_pb.MAP_WORLDMAP)
        else
            self.owner:lookAtPosition(data.pos.x, data.pos.y)
        end
    end,

    onClickInfo = function(self, handler, data)
        if data.type == progress_bar.mass then
            network_sender:getMassInfo(data.mass_id)
        elseif data.type == progress_bar.support then
            self.evt_id = game_director.events:on("WORLDMAPOBJQUERY", functor.rmake(self.march_list.onQueryObj, {owner = self, id = data.dest_id}))
            network_sender:queryWorldObj(data.dest_id)
        end
    end,

    onQueryObj = function(args)
        local world_map = args.owner.owner.world_map
        local hold_manager = world_map.hold_manager

        if hold_manager then
            local hold = hold_manager:getHoldFromId(args.id)

            if hold then
                frame_manager:show("hold_support", world_map, {hold = hold})
            else
                log.error("########################## hold not find !", args.id)
            end
        end

        game_director.events:remove("WORLDMAPOBJQUERY", args.owner.evt_id)
    end,

    onClickSpeedUp = function(self, handler, data)
        local type = nil

        if data.march_info.marchAction == common_pb.EMARCH_HELP then
            type = speedUpTypes.dispatch
        elseif data.march_info.marchAction == common_pb.EMARCH_MASS_ATK then
            type = speedUpTypes.mass
        else
            type = speedUpTypes.march
        end

        local world_map = self.owner.world_map
        local march_line = world_map.hold_manager:getHoldFromId(data.march_info.marchLineId)
        
        local param = {}
        param.id = data.march_info.marchLineId
        param.march = march_line
        param.type = type
        param.left_time = data.left_time
        param.total_time = data.total_time
        param.title = NativeText.GetLangStr(370005)

        frame_manager:show("common_speed_up", self, param)
    end,

    onClickBack = function(self, handler, data)
        local cb = function()
            network_sender:worldMapGoHome(3, data.id)
        end

        gui_helper:showCommonTipBox(cb, "是否令采集部队撤退？")
    end,

    tweenMarchLine = function(self)
        if self.is_visible == nil then
            self.is_visible = false
        end

        local visible = self.elem_num > 0
        
        if self.is_visible == visible then
            return
        else
            self.is_visible = visible
            local offset = self.panel:Find('offset').gameObject
            LeanTween.cancel(offset)
            local lt = LeanTween.moveLocalX(offset, visible and 0 or -360, 0.25)
        end
    end,
}