local log = logger.get("Game", "Mass Info Frame")
local super = require("ui.frame")

return class{

    super = super,

    onLoaded = function(self)

        super.onLoaded(self)

        self.detail = self.args.data.info

        -- 路径: root/login
        -- 事件: click_login 
        clickEvents = {
            "BG/Title/BackBtn",
            "BG/Content/EnemyPanel/Image/Layout/Start",
            "BG/Content/EnemyPanel/Image/Layout/Dissolve",
            "BG/Content/EnemyPanel/Image/Layout/Join",
            "BG/Content/Commanders/Addition",
            "BG/Content/EnemyPanel/Subdue",
            "BG/AdditionPanel",
            "BG/Content/EnemyPanel/Detect",
            "BG/Content/EnemyPanel/Report",
        }
 
        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self.content = self.transform:Find("BG/Content/Scroll View/Viewport/Content")
        self.prefab = self.transform:Find("BG/Content/Scroll View/Viewport/Content/Elem").gameObject
        self.mass_time = self.transform:Find("BG/Content/EnemyPanel/FinishTime/Text"):GetComponent(UnityEngine.UI.Text)
        self.addition_panel = self.transform:Find("BG/AdditionPanel")
        self.detecting_panel = self.transform:Find("BG/Content/EnemyPanel/Detecting")
        self.detect_btn = self.transform:Find("BG/Content/EnemyPanel/Detect")
        self.report_btn = self.transform:Find("BG/Content/EnemyPanel/Report")
        self.subdue_info = self.transform:Find("BG/Content/EnemyPanel/Info")
        self.soldier_percent = self.transform:Find("BG/Content/EnemyPanel/SoldierPercent")
        self.subdue_percent = self.transform:Find("BG/Content/EnemyPanel/Subdue/Text"):GetComponent(UnityEngine.UI.Text)

        self.addition_panel.localScale = Vector3.zero
        self.detecting_panel.localScale = Vector3.zero
        self.subdue_info.localScale = Vector3.zero
        self.soldier_percent.localScale = Vector3.zero

        self.detect_btn:SetActive(self:playerIsCreator(local_player.id))
        self.report_btn:SetActive(false)

        local start_btn = self.transform:Find("BG/Content/EnemyPanel/Image/Layout/Start")
        local dissolve_btn = self.transform:Find("BG/Content/EnemyPanel/Image/Layout/Dissolve")
        local join_btn = self.transform:Find("BG/Content/EnemyPanel/Image/Layout/Join")

        start_btn:SetActive(self.detail.status == 0)
        dissolve_btn:SetActive(self.detail.status == 0)
        join_btn:SetActive(self.detail.status == 0)

        if self.detail.status == 0 then
            if local_player.id == self.detail.playerId then
                start_btn:SetActive(self.detail.dstPlace.objType == common_pb.WORLD_OBJ_MONSTER)
                dissolve_btn:SetActive(true)
                join_btn:SetActive(true)

            elseif self:playerInMass() then
                start_btn:SetActive(false)
                dissolve_btn:SetActive(false)
                join_btn:SetActive(false)
            else
                start_btn:SetActive(false)
                dissolve_btn:SetActive(false)
                join_btn:SetActive(true)
            end
        end

        local name = self.transform:Find("BG/Content/Commanders/Panel/Commander/Text"):GetComponent(UnityEngine.UI.Text)
        name.text = self.detail.playerName

        self.team_cnt = 0
        self.soldier_cnt = 0
        self.ui_leftTime = {}
        self.has_report = false
        self.append_soldiers = {}

        self:loadTeams()
        self:setTeamCount()
        self:enemyInfo()
        self:setRallyAddition()

        self.evts = {}
        self.evts["WORLDMAPMASSNOTIFY"] = game_director.events:on("WORLDMAPMASSNOTIFY", functor.make(self.onTeamChanged, self))
        self.evts["WORLDMAPMARCHLINE"] = game_director.events:on("WORLDMAPMARCHLINE", functor.make(self.onStartMarch, self))
        self.evts["WORLDMAPMASSDEL"] = game_director.events:on("WORLDMAPMASSDEL", functor.make(self.onMassDelete, self))
        self.evts["WORLDMAPBATMSGNOTIFY"] = game_director.events:on("WORLDMAPBATMSGNOTIFY", functor.make(self.onGetReport, self))
        self.army_evt_id = army_queue_manager.events:on("ARMYQUEUEAdd", functor.make(self.onArmyLineAppear, self))

        network_sender:getBattleReport(self.detail.playerId, self.detail.dstPlace.instId, common_pb.MT_SPYREPORT)
    end,

    onUnload = function(self)
        if self.evts then
            for k, v in pairs(self.evts) do
                game_director.events:remove(k, v)
            end
        end

        army_queue_manager.events:remove("ARMYQUEUEAdd", self.army_evt_id)
    end,

    onMassDelete = function(self)
        self:click_BackBtn()
    end,

    onGetReport = function(self, msg)
        if msg.hasReprot == 1 then
            self.has_report = true

            self.subdue_info.localScale = Vector3.one

            local soldiers = msg.spyReport.soldier.soldierCounts
            local list = {}
            for k, v in ipairs(soldiers) do
                list[v.cfgId] = {cfg_id = v.cfgId, count = v.count}
            end

            self.enemy_soldiers = list

            local power = world_map_helper:calculateTeamPower({}, self.enemy_soldiers, true)

            self:setEnemyPower(power)
            self:setEnemySoldiers(self.enemy_soldiers)
            self:setSubduePercent()
            self:setSoldierPercent()
        else
            self:setEnemyPower()
        end

        self.detecting_panel.localScale = Vector3.zero
        self.detect_btn:SetActive(self:playerIsCreator(local_player.id))
    end,

    onArmyLineAppear = function(self, data)
        local army_line = data.march_info
        if army_line.action == common_pb.EMARCH_SPY and army_line.dstid == self.detail.dstPlace.instId then
            self.detecting_panel.localScale = Vector3.one
            self.detect_btn.localScale = Vector3.zero
            self.report_btn.localScale = Vector3.zero

            self.total_time = army_line.totalTime * 0.001
            self.left_time = (army_line.totalTime - army_line.elapsedTime) * 0.001
        end

        self:onStartMarch(army_line)
    end,

    onStartMarch = function(self, march_line)
        if march_line.action == common_pb.EMARCH_MASS_ATK then
            if self.detail.playerId == local_player.id then
                if world_map_info.ui then
                    frame_manager:clickHome()
                    local x = self.detail.srcPlace.posX
                    local y = self.detail.srcPlace.posY
                    game_director.curr_state.world_map:setCameraPos(x, y)
                else
                    self:click_BackBtn()
                end
            end
        end
    end,

    onTeamChanged = function(self, msg)
        if msg.massInfo.massInstid == self.detail.massInstid then
            self.detail = msg.massInfo
            self.team_cnt = 0
            self.soldier_cnt = 0
            self.ui_leftTime = {}

            self:loadTeams()
            self:setTeamCount()
        end
    end,

    update = function(self, dt)
        if not self.ui_leftTime then
            return
        end

        self.arrived = true
        for k, v in pairs(self.ui_leftTime) do
            local lt = v.data.lefttime
            if lt > 0 then
                self.arrived = false
                lt = lt - dt
                v.data.lefttime = lt > 0 and lt or 0
                self:teamCountDown(v.ui, v.data)
            else
                v.ui.localScale = Vector3.zero
                v.arrived.localScale = Vector3.one
            end
        end

        if self.detail.leftTime > 0 then
            self:massCountDown(self.detail.leftTime)

            local lt = self.detail.leftTime
            lt = lt - dt
            self.detail.leftTime = lt > 0 and lt or 0
        end

        if self.total_time and self.left_time then
            if self.left_time > 0 then
                self.left_time = self.left_time - dt
                self:detectCountDown()
            end
        end
    end,

    detectCountDown = function(self)
        local fill = self.detecting_panel:Find("FillBG/Fill"):GetComponent(UnityEngine.UI.Image)
        local time_text = self.detecting_panel:Find("FillBG/LeftTime"):GetComponent(UnityEngine.UI.Text)

        fill.fillAmount = 1 - self.left_time / self.total_time
        time_text.text = gui_helper:timeFormat(self.left_time)
    end,

    setRallyAddition = function(self)
        local node = self.addition_panel:Find("Image/PropUp")

        local none_text = self.addition_panel:Find("Image/None")
        none_text:SetActive(#self.detail.massValues == 0)

        gui_helper:setMassAddition(node, self.detail.massValues)
    end,
    
    setTeamCount = function(self)
        local teams = self.transform:Find("BG/Content/Top/Amount/Value"):GetComponent(UnityEngine.UI.Text)
        local soldiers = self.transform:Find("BG/Content/Top/Cargo/Value"):GetComponent(UnityEngine.UI.Text)

        teams.text = string.format("%d/%d", self.team_cnt, self.detail.maxTeamCount)
        soldiers.text = string.format("%s/%s", self.helper:itemNumFormat(self.soldier_cnt), self.helper:itemNumFormat(self.detail.maxSoldierCount))
    end,

    massCountDown = function(self, left_time)
        local str = NativeText.GetLangStr(281902)

        if self.detail.status == 1 then
            str = NativeText.GetLangStr(281917)
        end
        
        self.mass_time.text = string.format("%s %s", gui_helper:timeFormat(left_time), str)
    end,

    setEnemyPower = function(self, value)
        local power = self.transform:Find("BG/Content/Top/Enemy/Power"):GetComponent(UnityEngine.UI.Text)

        if value then
            power.text = self.helper:itemNumFormat(value)
        else
            power.text = "???"
        end
    end,

    setEnemySoldiers = function(self, soldiers)
        local cnt = 0
        for k, v in pairs(soldiers) do
            if cnt < self.subdue_info.childCount then
                local elem = self.subdue_info:GetChild(cnt)
                local name = elem:GetComponent(UnityEngine.UI.Text)
                local count = elem:Find("Value"):GetComponent(UnityEngine.UI.Text)

                local corps_cfg = config.soldier_soldiers[v.cfg_id]

                if corps_cfg then
                    name.text = self.helper:getLangCfg(corps_cfg.soldiername)
                    count.text = self.helper:itemNumFormat(v.count)
                end

                elem.localScale = Vector3.one

                cnt = cnt + 1
            end
        end

        for i=cnt, self.subdue_info.childCount - 1 do
            local elem = self.subdue_info:GetChild(i)
            elem.localScale = Vector3.zero
        end
    end,

    setSubduePercent = function(self)
        if not self.enemy_soldiers or not self.append_soldiers then
            self.subdue_percent.text = "???"
        else
            self.subdue_factor = world_map_helper:corpsSubdued(self.append_soldiers, self.enemy_soldiers)

            if self.subdue_factor >= 0 then
                self.subdue_percent.text = string.format("+%.2f%%", self.subdue_factor*100)
                self.subdue_percent.color = Color(6/255, 243/255, 153/255, 1)
            else
                self.subdue_percent.text = string.format("-%.2f%%", math.abs(self.subdue_factor*100))
                self.subdue_percent.color = Color(1, 0, 0, 1)
            end
        end
    end,

    setSoldierPercent = function(self)
        if not self.enemy_soldiers or not self.append_soldiers then
            return
        end

        local own_soldiers = {}
        local enemy_soldiers = {}
        local own_amount = 0
        local enemy_amount = 0

        local typeSoldiers = function(soldiers, table)
            for k, v in pairs(soldiers) do
                local cfg = config.soldier_soldiers[v.cfg_id]

                if cfg then
                    if table[cfg.type] then
                        table[cfg.type].count = table[cfg.type].count + v.count
                    else
                        table[cfg.type] = {}
                        table[cfg.type].count = v.count
                        table[cfg.type].type_name = cfg.TypeName
                    end
                end
            end
        end

        typeSoldiers(self.append_soldiers, own_soldiers)
        typeSoldiers(self.enemy_soldiers, enemy_soldiers)

        local list = {}
        
        for k, v in pairs(own_soldiers) do
            if list[k] then
                list[k].own = list[k].own + v.count
            else
                list[k] = {own = v.count, enemy = 0, type_name = v.type_name}
            end

            own_amount = own_amount + v.count
        end

        for k, v in pairs(enemy_soldiers) do
            if list[k] then
                list[k].enemy = list[k].enemy + v.count
            else
                list[k] = {own = 0, enemy = v.count, type_name = v.type_name}
            end

            enemy_amount = enemy_amount + v.count
        end

        local panel = self.soldier_percent:Find("Soldiers")
        local counter = self.soldier_percent:Find("Counter"):GetComponent(UnityEngine.UI.Text)
        local up_img = self.soldier_percent:Find("Counter/Up")
        local down_img = self.soldier_percent:Find("Counter/Down")

        local percent = self.subdue_factor
        
        if percent > 0 then
            counter.text = string.format("+%.2f%%", percent*100)
            counter.color = Color(6/255, 243/255, 153/255, 1)
            up_img.localScale = Vector3.one
            down_img.localScale = Vector3.zero
        elseif percent == 0 then
            counter.text = string.format("+%.2f%%", percent*100)
            counter.color = Color(6/255, 243/255, 153/255, 1)
            up_img.localScale = Vector3.zero
            down_img.localScale = Vector3.zero
        else
            counter.text = string.format("-%.2f%%", math.abs(percent*100))
            counter.color = Color(1, 0, 0, 1)
            up_img.localScale = Vector3.zero
            down_img.localScale = Vector3.one
        end

        local cnt = 0
        for k, v in pairs(list) do
            local own_percent = v.own / (own_amount > 0 and own_amount or 1)
            local enemy_percent = v.enemy / (enemy_amount > 0 and enemy_amount or 1)

            if cnt < panel.childCount then
                local elem = panel:GetChild(cnt)
                elem.localScale = Vector3.one
                local name = elem:GetComponent(UnityEngine.UI.Text)
                local value_1 = elem:Find("Value1"):GetComponent(UnityEngine.UI.Text)
                local value_2 = elem:Find("Value2"):GetComponent(UnityEngine.UI.Text)
                local fill_img = elem:Find("Percent/Image"):GetComponent(UnityEngine.UI.Image)

                name.text = self.helper:getLangCfg(v.type_name)
                value_1.text = string.format( "%.0f%%", own_percent*100 )
                value_2.text = string.format( "%.0f%%", enemy_percent*100 )

                if own_percent == enemy_percent then
                    fill_img.fillAmount = 0.5
                else
                    local sum = own_percent + enemy_percent
                    fill_img.fillAmount = own_percent / (sum > 0 and sum or 1)
                end

                cnt = cnt + 1
            end
        end

        for i=cnt, panel.childCount - 1 do
            local elem = panel:GetChild(i)
            elem.localScale = Vector3.zero
        end
    end,

    enemyInfo = function(self)
        local name = self.transform:Find("BG/Content/Top/Enemy/Name"):GetComponent(UnityEngine.UI.Text)
        local img = self.transform:Find("BG/Content/EnemyPanel/BG/Icon"):GetComponent(UnityEngine.UI.Image)

        if self.detail.dstPlace.objType == common_pb.WORLD_OBJ_PLAYER then
            local cfg = config.worldmap_base_configs[self.detail.dstPlace.cfgId]

            if cfg then
                img:SetSprite(cfg.iconpath..cfg.iconid)
            end
        elseif self.detail.dstPlace.objType == common_pb.WORLD_OBJ_MONSTER then
            local cfg = config.worldmap_monsters[self.detail.dstPlace.cfgId]

            if cfg then
                img:SetSprite(cfg.iconpath..cfg.monstericon)
            end
        end

        local player_name = self.detail.dstPlace.name
        if self.detail.dstPlace.objType == common_pb.WORLD_OBJ_MONSTER then
            local cfg = config.worldmap_monsters[self.detail.dstPlace.cfgId]
            if cfg then
                player_name = self.helper:getLangCfg(cfg.monstername)
            end
        end

        local short_name = self.detail.dstPlace.shortName
        if string.len(short_name) > 2 then
            player_name = string.format( "[%s]%s", short_name, player_name )
        end

        name.text = player_name
    end,

    loadTeams = function(self)
        for k, v in ipairs(self.detail.formationDetails) do
            local elem = nil

            if k > self.content.childCount then
                elem = Object.Instantiate(self.prefab).transform
                elem:SetParent(self.content)
                resetTransformZ(elem)
            else
                elem = self.content:GetChild(k-1)
            end

            if elem then
                elem.name = "Team_"..k
                elem:SetActive(true)

                self:setTeamInfo(elem, v)
            end

            self.team_cnt = self.team_cnt + 1
        end

        for i=#self.detail.formationDetails, self.content.childCount - 1 do
            local elem = self.content:GetChild(i)
            elem:SetActive(false)
        end
    end,

    setTeamInfo = function(self, elem, data)
        local name = elem:Find("Top/Name"):GetComponent(UnityEngine.UI.Text)
        local count = elem:Find("Bottom/Amount"):GetComponent(UnityEngine.UI.Text)
        local arrived = elem:Find("Top/Arrived")
        local slider = elem:Find("Bottom/Slider")
        local back_btn = elem:Find("Details/Back")
        local details = elem:Find("Details")
        local off_btn = elem:Find("Off")
        local on_btn = elem:Find("On")

        off_btn:SetActive(false)
        on_btn:SetActive(true)

        name.text = data.name
        count.text = self:teamSoldierCount(data.soldiers)

        if data.lefttime > 0 then
            table.insert(self.ui_leftTime, {ui = slider, data = data, arrived = arrived})
            slider.localScale = Vector3.one
            arrived.localScale = Vector3.zero
        else
            slider.localScale = Vector3.zero
            arrived.localScale = Vector3.one
        end

        self:loadSoldiers(details, data.heros, data.soldiers)

        local path = "BG/Content/Scroll View/Viewport/Content/"..elem.name.."/Off"
        self:clearScript("click", path)
        self:setScript("click", path, functor.rmake(self.onClickOff, elem))

        path = "BG/Content/Scroll View/Viewport/Content/"..elem.name.."/On"
        self:clearScript("click", path)
        self:setScript("click", path, functor.rmake(self.onClickOn, elem))
        
        --[[ path = "BG/Content/Scroll View/Viewport/Content/"..elem.name.."/Bottom/Slider/SpeedUp"
        self:clearScript("click", path)
        self:setScript("click", path, functor.rmake(self.onClickSpeedUp, data)) ]]

        path = "BG/Content/Scroll View/Viewport/Content/"..elem.name.."/Details/Back"
        self:clearScript("click", path)

        local btn = elem:Find("Bottom/Slider/SpeedUp"):GetComponent(UnityEngine.UI.Button)
        btn.onClick:RemoveAllListeners()
        btn.onClick:AddListener(
            function()
                self:onClickSpeedUp(data)
            end
        )

        if local_player.id == self.detail.playerId and local_player.id ~= data.playerId then
            self:setScript("click", path, functor.rmake(self.onClickRemove, data))
            back_btn:SetActive(true)
        elseif local_player.id == data.playerId then
            self:setScript("click", path, functor.rmake(self.onClickBack, data))
            back_btn:SetActive(true)
        else
            back_btn:SetActive(false)
        end

        if self.detail.status == 1 then
            back_btn:SetActive(false)
        end
    end,

    loadSoldiers = function(self, elem, heros, soldiers)
        local heros_panel = elem:Find("Panel/heros")
        if #heros > 0 then
            for k, v in ipairs(heros) do
                if k <= heros_panel.childCount then
                    local head = heros_panel:GetChild(k-1)
                    local icon = head:Find("Icon"):GetComponent(UnityEngine.UI.Image)
                    local grade = head:Find("Grade"):GetComponent(UnityEngine.UI.Image)
                    local level = head:Find("Level"):GetComponent(UnityEngine.UI.Text)
                    local stars = head:Find("Stars")

                    local cfg = config.commander_basics[v.officerId]
                    icon:SetSprite(cfg.path..cfg.icon)
                    grade:SetSprite(self:getGradeRes(cfg.color))
                    level.text = v.officerLv

                    gui_helper:setOfficerStars(v.starLv, stars)

                    head:SetActive(true)
                end
            end

            for i=#heros, heros_panel.childCount - 1 do
                local head = heros_panel:GetChild(i)
                head:SetActive(false)
            end

            heros_panel:SetActive(true)
        else
            heros_panel:SetActive(false)
        end

        local content = elem:Find("Panel")
        local perfab = content:GetChild(1).gameObject
        for k, v in ipairs(soldiers) do
            local node = nil
            if k > content.childCount - 1 then
                node = Object.Instantiate(perfab).transform
                node:SetParent(content)
                resetTransformZ(node)
            else
                node = content:GetChild(k)
            end

            node:SetActive(true)
            node.name = "Soldier_"..k

            local icon = node:Find("Image/Icon"):GetComponent(UnityEngine.UI.Image)
            local name = node:Find("Image/Name"):GetComponent(UnityEngine.UI.Text)
            local count = node:Find("Image/Count"):GetComponent(UnityEngine.UI.Text)

            local cfg = config.soldier_soldiers[v.cfgid]

            if cfg then
                icon:SetSprite(cfg.iconpath..cfg.soldiericon)
                name.text = self.helper:getLangCfg(cfg.soldiername)
                count.text = v.count
                self.soldier_cnt = self.soldier_cnt + v.count

                if not self.append_soldiers[v.cfg_id] then
                    self.append_soldiers[v.cfgid] = {cfg_id = v.cfgid, count = v.count}
                else
                    self.append_soldiers[v.cfgid].count = self.append_soldiers[v.cfgid].count + v.count
                end
            else
                log.error("######################################### soldier cfg not find ! id ", v.cfgid)
            end
        end

        for i=#soldiers+1, content.childCount - 1 do
            local node = content:GetChild(i)
            node:SetActive(false)
        end

        app:startCoroutine(
            function()
                coroutine.yield(nil)
                self:onClickOff(nil, elem.parent)
            end
        )
    end,

    getGradeRes = function(self, color)
        local path = res_path.res_path_95

        if color == 1 then
            path = res_path.res_path_96
        elseif color == 2 then
            path = res_path.res_path_97
        elseif color == 3 then
            path = res_path.res_path_98
        elseif color == 4 then
            path = res_path.res_path_99
        elseif color == 5 then
            path = res_path.res_path_100
        end

        return path
    end,

    teamSoldierCount = function(self, soldiers)
        local count = 0

        for k, v in ipairs(soldiers) do
            count = count + v.count
        end

        return count
    end,

    playerInMass = function(self)
        for k, v in ipairs(self.detail.formationDetails) do
            if local_player.id == v.playerId then
                return true
            end
        end

        return false
    end,

    playerIsCreator = function(self, id)
        return self:playerInMass(id) and self.detail.playerId == id or false
    end,

    playerTeamCnt = function(self)
        local cnt = 0

        for k, v in ipairs(self.detail.formationDetails) do
            if local_player.id == v.playerId then
                cnt = cnt + 1
            end
        end

        return cnt
    end,

    lastTeamCheck = function(self)
        if self.detail.playerId == local_player.id then
            local cnt = 0
            for k, v in ipairs(self.detail.formationDetails) do
                if local_player.id == v.playerId then
                    cnt = cnt + 1
                end
            end

            return cnt <= 1
        else
            return false
        end
    end,

    teamCountDown = function(self, ui, data)
        local left_time = ui:Find("LeftTime"):GetComponent(UnityEngine.UI.Text)
        local fill = ui:Find("Fill"):GetComponent(UnityEngine.UI.Image)

        fill.fillAmount = 1 - data.lefttime / data.totaltime
        left_time.text = gui_helper:timeFormat(data.lefttime)
    end,

    onClickRemove = function(self, handler, data)
        local callback = function()
            network_sender:removeMassTeam(self.detail.massInstid, data.playerId, data.armylineId)
            frame_manager:hide('common_box')
        end

        self:tipBox(NativeText.GetLangStr(289004),callback)
    end,

    onClickBack = function(self, handler, formation)

        if self:lastTeamCheck() then
            local callback = function()
                network_sender:worldMapGoHome(2, formation.armylineId)
                self:click_BackBtn()
                frame_manager:hide('common_box')
            end
    
            self:tipBox(NativeText.GetLangStr(281916),callback)
        else
            local callback = function()
                network_sender:worldMapGoHome(2, formation.armylineId)
                frame_manager:hide('common_box')
            end
    
            self:tipBox(NativeText.GetLangStr(289003),callback)
        end
    end,

    onClickOff = function(self, handler, ui)
        local off_btn = ui:Find("Off")
        local on_btn = ui:Find("On")
        local details = ui:Find("Details")

        off_btn:SetActive(false)
        on_btn:SetActive(true)
        details:SetActive(false)

        ui.sizeDelta = Vector2(830, 130)
    end,

    onClickOn = function(self, handler, ui)
        local off_btn = ui:Find("Off")
        local on_btn = ui:Find("On")
        local details = ui:Find("Details")
        local panel = details:Find("Panel"):GetComponent(UnityEngine.RectTransform)

        off_btn:SetActive(true)
        on_btn:SetActive(false)
        details:SetActive(true)

        ui.sizeDelta = Vector2(830, panel.rect.height + 130)
    end,

    onClickSpeedUp = function(self, data)
        frame_manager:show('resource_exchange', nil, {
            item_id = army_queue_manager.normal_speed_up,
            item_num = 1,
            army_info = {lt = data.lefttime, tt = data.totaltime, id = data.armylineId},
        })
    end,

    click_BackBtn = function(self)
        frame_manager:hide("mass_info")
    end,

    click_Dissolve = function(self)
        local callback = function()
            network_sender:destoryMass(self.detail.massInstid)
            self:click_BackBtn()
            frame_manager:hide('common_box')
        end

        self:tipBox(NativeText.GetLangStr(289002),callback)
    end,

    click_Start = function(self)
        if not self.arrived then
            hint:setText(NativeText.GetLangStr(289005))
        else
            network_sender:massAttack(self.detail.massInstid)
            self:click_BackBtn()
        end
    end,

    click_Join = function(self)
        local cnt = self:playerTeamCnt()
        local max = local_player:massPlayerTeamMax()
        if cnt >= max then
            hint:setText(string.format( NativeText.GetLangStr(289006), max ))
            return
        end

        if self.soldier_cnt >= self.detail.maxSoldierCount then
            hint:setText(NativeText.GetLangStr(289007))
            return
        end

        local soldier_cnt = home_info:getAvailableSoldiers()

        if #soldier_cnt == 0 then
            hint:setText(NativeText.GetLangStr(900012))
            return
        end

        local info = {}
        info.id = self.detail.massInstid
        info.hold_id = self.detail.playerHoldId
        info.start_pos = self.detail.OwnerPos
        info.mass_pos = self.detail.InitiatorPos
        info.player_id = self.detail.playerId
        info.mass_time = self.detail.leftTime

        local hold = {}
        hold.id = self.detail.dstPlace.instId
        hold.type = self.detail.dstPlace.objType
        hold.cfg_id = self.detail.dstPlace.cfgId
        hold.player_name = self.detail.dstPlace.name
        hold.short_name = self.detail.dstPlace.shortName
        hold.coordX = self.detail.dstPlace.posX
        hold.coordY = self.detail.dstPlace.posY

        local can_spy = local_player.id == self.detail.playerId
        frame_manager:show("launch_rally", self.owner, {monster = hold, action = common_pb.EMARCH_JOIN_MASS, mass_info = info, spy = can_spy})
    end,

    click_Addition = function(self)
        self.addition_panel.localScale = Vector3.one
    end,

    click_AdditionPanel = function(self)
        self.addition_panel.localScale = Vector3.zero
    end,

    click_Subdue = function(self)
        if not self.enemy_soldiers then
            return
        end

        local vec = self.subdue_info.localScale
        if vec.magnitude == 0 then
            self.subdue_info.localScale = Vector3.one
            self.soldier_percent.localScale = Vector3.zero
        else
            self.subdue_info.localScale = Vector3.zero
            self.soldier_percent.localScale = Vector3.one
        end
    end,

    click_Detect = function(self)
        network_sender:startArmyLine(common_pb.EMARCH_SPY, nil, nil, self.detail.dstPlace.instId, nil, nil, self.detail.dstPlace.posX, self.detail.dstPlace.posY)
    end,

    click_Report = function(self)
        
    end,

    tipBox = function(self, content, cb)
        local tb = {
            title = "",
            content = content,
            buttons =
            {
                {
                    btn_name = NativeText.GetLangStr(110522),
                    callback = function() 
                        frame_manager:hide('common_box')
                    end
                },
                {
                    btn_name = NativeText.GetLangStr(110521),
                    callback = cb,
                },
            },
        }

        frame_manager:show('common_box', nil, tb)
    end,

    getAppendOfficers = function(self)

    end,
}