local log = logger.get("Game", "Mine Frame")
local super = require("ui.frame")

return class{
    super = super,

    onLoaded = function( self )
        super.onLoaded(self)

        self.owner = self.args.owner
        self.mine = self.args.data.hold

        clickEvents = {
            "Close",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self.content = self.transform:Find("BG/Right/Scroll View/Viewport/Content")
        self.own_prefab = self.transform:Find("Prefabs/OwnElem").gameObject
        self.other_prefab = self.transform:Find("Prefabs/OtherElem").gameObject
        self.other_car_cnt = self.transform:Find("BG/Right/CarCount"):GetComponent(UnityEngine.UI.Text)

        self.cars = {}
        self.cnt = 0

        --[[ self:setLeftUI()
        
        local info = self.mine.queryInfo
        self:loadMineCar(info.myCars, self.content, self.own_prefab, true)
        self:loadMineCar(info.gatherCars, self.content, self.other_prefab, false)

        local count = #info.gatherCars
        self.other_car_cnt.text = string.format( "%s <color=#ffffffff>(%d/%d)</color>", "Mining List", count, self.mine.cfg.gatherlimit or 0 ) ]]

        --self:init()

        self.evts = {}
        self.evts["WORLDMAPGATHERCARNOTIFY"] = game_director.events:on("WORLDMAPGATHERCARNOTIFY", functor.make(self.onCarNotify, self))
        self.evts["WORLDMAPQUERYOBJ"] = game_director.events:on("WORLDMAPQUERYOBJ", functor.make(self.init, self))

        --network_sender:queryWorldMap(self.mine.id, protocol_pb.MAP_WORLDMAP)
    end,

    init = function(self)
        self:setLeftUI()
        
        local info = self.mine.queryInfo
        self:loadMineCar(info.myCars, self.content, self.own_prefab, true)
        self:loadMineCar(info.gatherCars, self.content, self.other_prefab, false)

        local count = #info.gatherCars
        self.other_car_cnt.text = string.format( "%s <color=#ffffffff>(%d/%d)</color>", NativeText.GetLangStr(280709), count, self.mine.cfg.gatherlimit or 0 )
    end,

    onUnload = function(self)
        if self.evts then
            for k, v in pairs(self.evts) do
                game_director.events:remove(k, v)
            end
        end
    end,

    onCarNotify = function(self, info)
        self:refresh(info)
    end,

    refresh = function(self, info)
        local elem = self.cars[info.carId]
        if elem then
            self:setOwnCarInfo(elem, info)
        end
    end,

    setLeftUI = function(self)
        local name = self.transform:Find("BG/Left/Name"):GetComponent(UnityEngine.UI.Text)
        local icon = self.transform:Find("BG/Left/IconBG/Icon"):GetComponent(UnityEngine.UI.Image)
        local position = self.transform:Find("BG/Left/Position"):GetComponent(UnityEngine.UI.Text)
        local main_res = self.transform:Find("BG/Left/NameNode/Name"):GetComponent(UnityEngine.UI.Text)
        local left_count = self.transform:Find("BG/Left/AmountNode/Amount"):GetComponent(UnityEngine.UI.Text)
        local speed = self.transform:Find("BG/Left/SpeedNode/Speed"):GetComponent(UnityEngine.UI.Text)
        
        local cfg = self.mine.cfg

        position.text = gui_helper:positionFormat(self.mine.coordX, self.mine.coordY)
        name.text = string.format( "<color=#ffffffff>Lv.%d</color> %s", cfg.level, self.helper:getLangCfg(cfg.nameid) )
        speed.text = string.format( "%d/%s", self.mine.queryInfo.gatherSpeed, "min" )
        
        local item_cfg = item_helper:getItemCfg(cfg.item)
        if item_cfg then
            main_res.text = self.helper:getLangCfg(item_cfg.name_id)
        else
            log.error("##### mine res error ! id", cfg.item)
        end

        left_count.text = self.mine.queryInfo.resAmount
    end,

    loadMineCar = function(self, cars, content, prefab, own_car)
        log.info("########################################## car count ", #cars)
        for k, v in ipairs(cars) do

            local createPrefab = function()
                local car = Object.Instantiate(prefab).transform
                car:SetParent(content)
                resetTransformZ(car)
                car:SetActive(true)
                car.name = "car_"..k

                return car
            end

            if own_car then
                local car = createPrefab()
                self:setOwnCarInfo(car, v)
            else
                if v.playerId ~= local_player.id then
                    local car = createPrefab()
                    self:setOtherCarInfo(car, v)
                end
            end
        end

        local img = self.content:GetChild(0)
        local tip = img:Find("Image")
        tip:SetActive(self.cnt == 0)

        if self.cnt == 0 then
            img.sizeDelta = Vector2(917, 250)
        else
            img.sizeDelta = Vector2(917, 50)
        end
    end,

    setOwnCarInfo = function(self, elem, data)
        local icon = elem:Find("Icon"):GetComponent(UnityEngine.UI.Image)
        local state = elem:Find("Status"):GetComponent(UnityEngine.UI.Text)
        local name = elem:Find("Name"):GetComponent(UnityEngine.UI.Text)
        local position = elem:Find("Position"):GetComponent(UnityEngine.UI.Text)
        local back_btn = elem:Find("Recall")
        local dispatch_btn = elem:Find("Depatrue")

        if data.status == 0 then
            state.text = NativeText.GetLangStr(280045)
            state.transform:SetActive(true)
            position.transform:SetActive(false)
            name.transform:SetActive(false)
        elseif data.status == 1 then
            state.text = NativeText.GetLangStr(280041)
            state.transform:SetActive(false)
            position.transform:SetActive(true)
            name.transform:SetActive(true)
            name.text = self.helper:getLangCfg(data.resName)
            position.text = gui_helper:positionFormat(data.posX, data.posY)

            if data.posX == self.mine.coordX and data.posY == self.mine.coordY then
                name.text = string.format( "%s  <color=#00FF00FF>(%s)</color>", name.text, NativeText.GetLangStr(280115) )
            end

        elseif data.status == 2 then
            state.text = NativeText.GetLangStr(280710)
            state.transform:SetActive(true)
            position.transform:SetActive(true)
            name.transform:SetActive(false)

            local info = world_map_info:getGatherCarMarch(data.carId)
            if info then
                local clk_trans = elem:Find("Position")
                local clk = clk_trans:GetComponent(Clock)
                if not clk then
                    clk = clk_trans:AddComponent(Clock)
                end
                local left_time = (info.totalTime - info.elapsedTime) * 0.000001
                clk:Set(math.floor(left_time))
            end
        end

        back_btn:SetActive(data.status == 1)
        dispatch_btn:SetActive(data.status == 0)

        local path = "BG/Right/Scroll View/Viewport/Content/"..elem.name.."/Recall"
        self:clearScript("click", path)
        self:setScript("click", path, functor.rmake(self.onClickBack, data.carId))

        path = "BG/Right/Scroll View/Viewport/Content/"..elem.name.."/Depatrue"
        self:clearScript("click", path)
        self:setScript("click", path, functor.rmake(self.onClickDispatch, data.carId))

        self.cnt = self.cnt + 1
        self.cars[data.carId] = elem
        elem:SetSiblingIndex(1)
    end,

    setOtherCarInfo = function(self, elem, data)
        local icon = elem:Find("Icon"):GetComponent(UnityEngine.UI.Image)
        local family_name = elem:Find("FamilyName"):GetComponent(UnityEngine.UI.Text)
        local name = elem:Find("Name"):GetComponent(UnityEngine.UI.Text)

        family_name.text = string.len( data.familyName ) > 0 and data.familyName or NativeText.GetLangStr(280101)
        name.text = data.playerName

        elem:SetSiblingIndex(self.content.childCount - 1)
    end,

    onClickBack = function(self, handle, id)
        network_sender:worldMapGoHome(1, id)
    end,

    onClickDispatch = function(self, handle, id)
        self.car_id = id
        self.owner:gatherMine(self.mine, id)
    end,

    click_Close = function(self)
        frame_manager:hide("mine_info")
    end,

    click_DispatchBtn = function(self)
        network_sender:getGatherCar(self.mine.id)
    end,
}