local log = logger.get("Game", "Mini Map Frame")
local super = require("ui.frame")

local icon_type = {
    own_camp = 1,
    friend_camp = 2,
    own_factory = 3,
    friend_factory = 4,
    sub_base = 5,
    mining = 6,
    fortress = 7,
    own_point = 8,
    own_base = 9,
}

return class{

    super = super,

    onLoaded = function(self)
        super.onLoaded(self)

        self.owner = self.args.owner

        -- 路径: root/login
        -- 事件: click_login 
        clickEvents = {
            "BG/Close",
            "BG/Building",
            "BG/Resources",
        }
 
        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self:setScript("single", "BG", self.onClickBG)

        self.bg = self.transform:Find("BG")
        self.build_panel = self.transform:Find("BG/BuildPanel")
        self.res_panel = self.transform:Find("BG/ResPanel")
        self.build_effect = self.transform:Find("BG/Building/leida")
        self.res_effect = self.transform:Find("BG/Resources/leida")
        self.pos_effect = self.transform:Find("List/UI_xuanze")

        self.build_panel:SetActive(true)
        self.res_panel:SetActive(false)
        self.build_effect:SetActive(true)
        self.res_effect:SetActive(false)

        self.own_factory_displayed = false
        self.friend_factory_displayed = false

        self.icons = {}
        self.icons[icon_type.own_camp] = self.transform:Find("List/Icon_1").gameObject
        self.icons[icon_type.friend_camp] = self.transform:Find("List/Icon_2").gameObject
        self.icons[icon_type.sub_base] = self.transform:Find("List/Icon_3").gameObject
        self.icons[icon_type.own_factory] = self.transform:Find("List/Icon_4").gameObject
        self.icons[icon_type.friend_factory] = self.transform:Find("List/Icon_5").gameObject
        self.icons[icon_type.fortress] = self.transform:Find("List/Icon_6").gameObject
        self.icons[icon_type.mining] = self.transform:Find("List/Icon_7").gameObject
        self.icons[icon_type.own_base] = self.transform:Find("List/Icon_8").gameObject
        self.icons[icon_type.own_point] = self.transform:Find("List/Icon_9").gameObject

        local map = self.transform:Find("BG/Map")
        self.ui_center = map.localPosition

        self:initData()
        self:drawBase()
        self:drawMining()
        self:ownPosition()

        self.evts = {}
        self.evts["WORLDMAPMINIMAP"] = game_director.events:on("WORLDMAPMINIMAP", functor.make(self.refresh, self))

        network_sender:requestMiniMap()
    end,

    onUnload = function(self)
        if self.evts then
            for k, v in pairs(self.evts) do
                game_director.events:remove(k, v)
            end
        end
        -- self:StopAllCoroutines()
    end,

    initData = function(self)
        self.max_x, self.max_z = world_map_helper:getWorldMapBorder()
        local point = world_map_helper:getCellPosition(self.max_x, self.max_z )

        self.len_x = point.x
        self.len_y = point.z

        self.center_point =  world_map_helper:getCellPosition(math.floor(self.max_x/2), math.floor(self.max_z/2))
    end,

    drawBase = function(self)
        local base = world_map_helper.world_map.my_hold

        if not base then
            return
        end

        local prefab = self.icons[icon_type.own_base]

        if prefab then
            icon = Object.Instantiate(prefab).transform
            icon:SetParent(self.bg)
            resetTransformZ(icon)

            local pos = world_map_helper:getCellPosition(base.coordX, base.coordY)
            if pos then
                pos = self:worldPosToUI(pos)
                icon.localPosition = pos
            end
        end
    end,

    drawMining = function(self)
        local minings = world_map_helper:getMinings()

        if minings then
            for k, v in pairs(minings) do
                local x = v.outerbottomleftx + (v.outeruprightx - v.outerbottomleftx)*0.5
                local y = v.outerbottomlefty + (v.outeruprighty - v.outerbottomlefty)*0.5
                local icon = self:addIcon(x, y, icon_type.mining, self.res_panel, k)
                v.mini_icon = icon
            end
        end
    end,

    ownPosition = function(self)
        local prefab = self.icons[icon_type.own_point]

        if prefab then
            self.own_icon = Object.Instantiate(prefab).transform
            self.own_icon:SetParent(self.bg)
            resetTransformZ(self.own_icon)

            local pos = self:getScreenCenterPoint()
            if pos then
                pos = self:worldPosToUI(pos)
                self.own_icon.localPosition = pos
            end

            self.pos_effect:SetParent(self.own_icon)
            resetTransformZ(self.pos_effect)
            self.pos_effect.localPosition = Vector3.zero
        end
    end,

    refresh = function(self, objs)
        for k, v in ipairs(objs) do
            if v.objType == common_pb.WORLD_OBJ_CAMPE then
                if local_player.id == v.playerId then
                    self:addIcon(v.posX, v.posY, icon_type.own_camp, self.build_panel, k)
                else
                    self:addIcon(v.posX, v.posY, icon_type.friend_camp, self.build_panel, k)
                end
            elseif v.objType == common_pb.WORLD_OBJ_STONE_FACTORY then
                self:addFactoryIcon(v)
            elseif v.objType == common_pb.WORLD_OBJ_FAMILY_SUB_HOLD then
                self:addIcon(v.posX, v.posY, icon_type.sub_base, self.build_panel, k)
            end
        end
    end,

    addIcon = function(self, x, y, type, panel, idx)
        local prefab = self.icons[type]

        if prefab then
            local icon = Object.Instantiate(prefab).transform
            icon:SetParent(panel)
            resetTransformZ(icon)

            local pos = world_map_helper:getCellPosition(x, y)
            pos = self:worldPosToUI(pos)
            icon.localPosition = pos

            --[[ icon.name = "icon_"..idx
            local path = "BG/"..panel.name.."/"..icon.name
            self:setScript("click", path, functor.rmake(self.onClickIcon, {x = x, y = y})) ]]

            return icon
        end

        return nil
    end,

    addFactoryIcon = function(self, info)
        local in_mining, mining = world_map_helper:posInMining(info.posX, info.posY)

        if in_mining then
            local add = function(type, param, pos)
                local prefab = self.icons[type]
                
                if prefab then
                    local icon = Object.Instantiate(prefab).transform
                    icon:SetParent(param.mini_icon)
                    resetTransformZ(icon)

                    icon.localPosition = pos
                    param.mini_icon:SetAsLastSibling()
                end
            end

            if local_player.id == info.playerId then
                if not self.own_factory_displayed then
                    add(icon_type.own_factory, mining, Vector3(10, 15, 0))
                    self.own_factory_displayed = true
                end
            else
                if not self.friend_factory_displayed then
                    add(icon_type.friend_factory, mining, Vector3(-10, 15, 0))
                    self.friend_factory_displayed = true
                end
            end
        end
    end,

    onClickBG = function(self, handler, data)
        local point = data.position
        local pos = self:touchPosToUI(point)

        if math.abs( pos.x ) > UnityEngine.Screen.height / 2 then
            return
        end

        if self.own_icon then
            self.own_icon.localPosition = Vector3(pos.x, pos.y, 0)
        end

        pos = self:uiPosToWorld(pos)

        if pos ~= Vector3.zero then
            world_map_helper:setCameraPos(pos)
            app:startCoroutine(
                function()
                    coroutine.yield(WaitForSeconds(1))
                    local vsb = frame_manager:isDisplay("mini_map")
                    if vsb then
                        self:click_Close()
                    end
                end
            )
        end
    end,

    onClickIcon = function(self, handler, pos)
        local pos = world_map_helper:getCellPosition(pos.x, pos.y)
        world_map_helper:setCameraPos(pos)
        self:click_Close()
    end,

    click_Close = function(self)
        --self.pos_effect:SetActive(false)
        frame_manager:hide("mini_map")
    end,

    click_Building = function(self)
        local active = self.build_panel.gameObject.activeSelf
        self.build_panel:SetActive(not active)
        self.build_effect:SetActive(not active)
    end,

    click_Resources = function(self)
        local active = self.res_panel.gameObject.activeSelf
        self.res_panel:SetActive(not active)
        self.res_effect:SetActive(not active)
    end,

    touchPosToUI = function(self, touch_pos)
        local pos = Vector2.zero
        local canvas = UnityEngine.GameObject.Find('gui/root'):GetComponent(UnityEngine.Canvas)
        local rect_trans = canvas.transform:GetComponent(UnityEngine.RectTransform)
        local p1, p2 = UnityEngine.RectTransformUtility.ScreenPointToLocalPointInRectangle(rect_trans, touch_pos, canvas.worldCamera, pos)

        return p2
    end,

    uiPosToWorld = function(self, pos)
        local factor_x = self.len_x / UnityEngine.Screen.height
        local factor_y = self.len_y / UnityEngine.Screen.height

        local x = (pos.x - self.ui_center.x) * factor_x + self.center_point.x
        local z = (pos.y - self.ui_center.y) * factor_y + self.center_point.z

        x = x > self.len_x and self.len_x or x
        z = z > self.len_y and self.len_y or z

        return Vector3(x, 0, z)
    end,

    worldPosToUI = function(self, pos)

        local factor_x = UnityEngine.Screen.height / self.len_x
        local factor_y = UnityEngine.Screen.height / self.len_y

        pos = pos - self.center_point

        local x = pos.x * factor_x + self.ui_center.x
        local y = pos.z * factor_y + self.ui_center.y

        return Vector2(x, y)
    end,

    getScreenCenterPoint = function(self)
        local screen_pos = Vector2(UnityEngine.Screen.width/2, UnityEngine.Screen.height / 2)
        local mask = core.mask('Ground')
        local r = self.owner.camera:ScreenPointToRay(screen_pos)
        local rst, hit = UnityEngine.Physics.Raycast(r, hit, 999, mask)
        if rst and hit then
            return hit.point
        end

        return nil
    end,
}