local log = logger.get("Game", "officer develop panel")
local item_cls = require('RTS.bag_sys.item')

return class{
    tabs = {
        promote = 1,
        awake = 2,
        talent = 3,
    },

    ctor = function(self, panel, owner)
        self.owner = owner
        self.panel = panel
        self.helper = require("gui.tool.frame_helper")

        self.awake_trans = self.panel:Find('AwakePanel')
        self.talent_trans = self.panel:Find("TalentPanel")
        self.promote_trans = self.panel:Find("PromotePanel")
        self.promote_btn = self.panel:Find('ButtonPromote'):GetComponent(UnityEngine.UI.Button)
        self.awake_btn = self.panel:Find('ButtonAwake'):GetComponent(UnityEngine.UI.Button)
        self.talent_btn = self.panel:Find('ButtonTalent'):GetComponent(UnityEngine.UI.Button)

        self.awake_normal = self.awake_trans:Find("Normal")
        self.awake_max = self.awake_trans:Find("Max")

        self.awake_stars = self.awake_trans:Find("Stars")
        self.progress = self.awake_trans:Find("Progress")
        self.mat_name = self.awake_trans:Find("Normal/Material/Name"):GetComponent(UnityEngine.UI.Text)
        self.mat_cnt = self.awake_trans:Find("Normal/Material/Count"):GetComponent(UnityEngine.UI.Text)
        self.prog_img = self.awake_trans:Find("Normal/Material/Progress/Image"):GetComponent(UnityEngine.UI.Image)
        self.mat_grade = self.awake_trans:Find("Normal/Material/Grade"):GetComponent(UnityEngine.UI.Image)
        self.mat_icon = self.awake_trans:Find("Normal/Material/Grade/Icon"):GetComponent(UnityEngine.UI.Image)

        self.awake_lvup_btn = self.awake_trans:Find("AwakeBtn"):GetComponent(UnityEngine.UI.Button)

        self.prom_normal = self.promote_trans:Find("Normal")
        self.prom_max = self.promote_trans:Find("Max")

        self.rank_icon_1 = self.prom_normal:Find("Rank/Icon_1"):GetComponent(UnityEngine.UI.Image)
        self.rank_icon_2 = self.prom_normal:Find("Rank/Icon_2"):GetComponent(UnityEngine.UI.Image)
        self.prom_tip = self.prom_normal:Find("Tip")
        self.prom_tip:SetActive(false)

        self.prom_max_icon = self.prom_max:Find("Image"):GetComponent(UnityEngine.UI.Image)
        self.rank_name = self.prom_max:Find("Text"):GetComponent(UnityEngine.UI.Text)

        self.prom_lvup_btn = self.promote_trans:Find("PromoteBtn"):GetComponent(UnityEngine.UI.Button)

        self.prom_mats = {}
        for i=1, 4 do
            local ui = self.prom_normal:Find("Material/Grade_"..i)
            table.insert( self.prom_mats, ui )
        end

        self.talent_content = self.talent_trans:Find("Viewport/Content")
        self.talent_prefab = self.talent_content:GetChild(0).gameObject

        self.nodes = {}
        self.nodes[self.tabs.promote] = self.promote_trans
        self.nodes[self.tabs.awake] = self.awake_trans
        self.nodes[self.tabs.talent] = self.talent_trans

        self.btns = {}
        self.btns[self.tabs.promote] = self.promote_btn
        self.btns[self.tabs.awake] = self.awake_btn
        self.btns[self.tabs.talent] = self.talent_btn

        self.tab_index = self.tabs.promote

        self:hide()
    end,

    show = function(self, hero_id)
        self.helper.showTrans(self.panel)
        self.hero = hero_manager:getHero(hero_id)
        local officer = self.hero.officer
        self.awake_levels = officer:getAwakeNum(officer.star_level)

        self.can_awake = true
        self.can_promote = true

        if not self.hero then
            log.error("##### show develop panel hero not find ! hero_id", hero_id)
            return
        end

        self:changeTab(self.tab_index)
    end,

    clickPromoteBtn = function(self)
        self:changeTab(self.tabs.promote)
    end,

    clickAwakeBtn = function(self)
        self:changeTab(self.tabs.awake)
    end,

    clickTalentBtn = function(self)
        self:changeTab(self.tabs.talent)
    end,

    changeTab = function(self, idx)
        if idx == self.tabs.promote then
            self:refreshPromote()
        elseif idx == self.tabs.awake then
            self:refreshAwake()
        elseif idx == self.tabs.talent then
            self:refreshTalent()
        end

        for k, v in pairs(self.nodes) do
            v:SetActive(k == idx)
        end

        self.tab_index = idx
        self:setButtonState()
    end,

    setButtonState = function(self)
        for k, v in pairs(self.btns) do
            if k ==  self.tab_index then
                v.targetGraphic.sprite = v.spriteState.pressedSprite
            else
                v.targetGraphic.sprite = v.spriteState.disabledSprite
            end
            v.transform:GetComponent(UnityEngine.UI.Image):SetNativeSize()
        end
    end,

    --设置晋升UI
    refreshPromote = function(self)
        local officer = self.hero.officer
        local rank = officer.quality
        local max_rank = officer:getMaxPromote()

        if rank >= max_rank then
            self.prom_normal:SetActive(false)
            self.prom_max:SetActive(true)
            self:promoteMax()
        else
            self.prom_normal:SetActive(true)
            self.prom_max:SetActive(false)
            self:setRankIcon(rank)
            self:setPromoteProps()
            self:setPromoteMaterial()
        end

        self.prom_lvup_btn.interactable = rank < max_rank
    end,

    -- 军衔Icon
    setRankIcon = function(self, rank)
        local setIcon = function(ui, r)
            local cfg = config.commander_qualitys[r]

            if cfg then
                ui:SetSprite(cfg.icon_path..cfg.icon)
            else
                log.error("##### officer rank icon not find ! ", tostring(r))
            end
        end

        setIcon(self.rank_icon_1, rank)
        setIcon(self.rank_icon_2, rank+1)
    end,

    -- 晋升属性成长
    setPromoteProps = function(self)
        local officer = self.hero.officer
        local cfg = officer:getPromotionCfg(officer.quality+1)

        if cfg then
            local props, changed = self:getOfficerProps(cfg.attr)
            gui_helper:showUpgradeProps(self.prom_normal, props, changed)
        else
            log.error("##### set promote props error !", officer.quality)
        end
    end,

    --晋升需要的材料
    setPromoteMaterial = function(self)
        local officer = self.hero.officer
        local cfg = officer:getPromotionCfg(officer.quality)
        local idx = 1

        for k, v in pairs(cfg.stuff) do
            local item_cfg = item_helper:getItemCfg(k)
            local item_num = self.helper:getItemNum(k)
            -- 设置材料 icon
            local mat = self.prom_mats[idx]
            if not mat then
                log.error('promotion material more than 4 !', idx)
                return
            end
            mat:SetActive(true)
            local grade = mat:GetComponent(UnityEngine.UI.Image)
            local icon = mat:Find('Icon')
            local img = icon:GetComponent(UnityEngine.UI.Image)
            grade:SetSprite(gui_helper:getQualityIconPath(item_cfg.color))
            local icon_path = gui_helper:getItemIconPath(item_cfg)
            img:SetSprite(icon_path)

            -- 设置数量
            local num_text = mat:Find('Count'):GetComponent(UnityEngine.UI.Text)
            local color = "#FFFFFFFF"
            if item_num < v then
                color = "#FF0000FF"
                self.can_promote = false
            end
            num_text.text = string.format("<color=%s>%d</color>/%d", color, item_num, v)

            idx = idx + 1

            local path = "BG/Right/OfficerNode/PromotePanel/Normal/Material/"..mat.name
            local item_inst = item_cls({cfgId = k, count = item_num})
            self.owner:clearScript("click", path)
            self.owner:setScript("click", path, functor.rmake(self.onClickItem, item_inst))
        end

        for i=idx, #self.prom_mats do
            self.prom_mats[i]:SetActive(false)
        end
    end,

    onClickItem = function(self, handler, item)
        frame_manager:show("itemInfo", self, {item = item})
    end,

    -- 晋升max
    promoteMax = function(self)
        local officer = self.hero.officer
        local rank = officer.quality

        local cfg = config.commander_qualitys[rank]
        self.rank_name.text = self.helper:getLangCfg(cfg.name)
        self.prom_max_icon:SetSprite(cfg.icon_path..cfg.icon)

        cfg = officer:getPromotionCfg(rank)
        local props = self:getOfficerProps(cfg.attr)
        gui_helper:showUpgradeProps(self.prom_max, props)
    end,

    ----- 觉醒部分

    refreshAwake = function(self)
        self:setAwakeStars()
        self:setAwakeProgress()

        local officer = self.hero.officer
        local star_level = officer.star_level
        local awake_level = officer.awake_level
        local max_star = officer:getMaxStar()

        if star_level >= max_star and awake_level >= self.awake_levels then
            self.awake_normal:SetActive(false)
            self.awake_max:SetActive(true)
            self.awake_lvup_btn.interactable = false
            self:setAwakeMax()
        else
            self.awake_normal:SetActive(true)
            self.awake_max:SetActive(false)
            self:setAwakeMat()
            self:setAwakeProps()
            self.awake_lvup_btn.interactable = true
        end
    end,

    setAwakeStars = function(self)
        local star_lv = self.hero.officer.star_level
        for i=1, star_lv do
            local star = nil
            
            if i > self.awake_stars.childCount then
                star = Object.Instantiate(self.awake_stars:GetChild(0)).transform
                star:SetParent(self.awake_stars)
                resetTransformZ(star)
            else
                star = self.awake_stars:GetChild(i-1)
            end

            if not star then
               return 
            end

            star:SetActive(true)
        end

        for i=star_lv, self.awake_stars.childCount - 1 do
            local child = self.awake_stars:GetChild(i)
            child:SetActive(false)
        end
    end,

    setAwakeProgress = function(self)
        local officer = self.hero.officer
        local awake_level = officer.awake_level

        for idx=1, self.awake_levels do
            local elem = nil
            
            if idx > self.progress.childCount then
                elem = Object.Instantiate(self.progress:GetChild(0)).transform
                elem:SetParent(self.progress)
                resetTransformZ(elem)
            else
                elem = self.progress:GetChild(idx-1)
            end

            if not elem then
               return 
            end

            elem:SetActive(true)

            local ligh = elem:Find("Ligh")
            local dark = elem:Find("Dark")

            ligh:SetActive(idx <= awake_level)
            dark:SetActive(idx > awake_level)
        end

        for i=self.awake_levels, self.progress.childCount - 1 do
            local child = self.progress:GetChild(i)
            child:SetActive(false)
        end
    end,

    setAwakeProps = function(self)
        local officer = self.hero.officer
        local star_lv = officer.star_level
        local awake_lv = officer.awake_level

        if awake_lv >= self.awake_levels then
            star_lv = star_lv + 1
            awake_lv = 0
        else
            awake_lv = awake_lv + 1
        end

        local cfg = officer:getAwakeCfg(star_lv, awake_lv)

        if cfg then
            local props, changed = self:getOfficerProps(cfg.attr)
            gui_helper:showUpgradeProps(self.awake_normal, props, changed)
        else
            log.error(string.format("##### set awake props error ! growup:%d, star lv:%d, awake lv:%d", officer.cfg.growup, officer.star_level, officer.awake_level))
        end
    end,

    setAwakeMat = function(self)
        local officer = self.hero.officer
        local cfg = officer:getAwakeCfg(officer.star_level, officer.awake_level)

        if not cfg then
            log.error(string.format("##### set awake mat error ! growup:%d, star lv:%d, awake lv:%d", officer.cfg.growup, officer.star_level, officer.awake_level))
            return
        end

        local item_cfg = item_helper:getItemCfg(officer.cfg.piece_id)
        self.mat_name.text = self.helper:getLangCfg(item_cfg.name_id)
        self.mat_grade:SetSprite(gui_helper:getQualityIconPath(item_cfg.color))
        self.mat_icon:SetSprite(gui_helper:getItemIconPath(item_cfg))
        
        local had_amount = self.helper:getItemNum(item_cfg.id)
        local need_amount = cfg.piece_num
        
        local color = "#FFFFFFFF"
        if had_amount < need_amount then
            color = "#FF0000FF"
            self.can_awake = false
        end

        self.mat_cnt.text = string.format("<color=%s>%d</color>/%d", color, had_amount, need_amount)
        self.prog_img.fillAmount = had_amount / need_amount
    end,

    setAwakeMax = function(self)
        local officer = self.hero.officer
        local talents = officer.skills

        local content = self.awake_max:Find("Viewport/Content")
        local prefab = content:GetChild(0).gameObject

        for k, v in ipairs(talents) do
            local elem = nil
            if k > content.childCount then
                elem = Object.Instantiate(prefab).transform
                elem:SetParent(content)
                resetTransformZ(elem)
            else
                elem = content:GetChild(k-1)
            end

            if elem then
                elem:SetActive(true)
                elem.name = "Skill_"..k

                local icon = elem:GetComponent(UnityEngine.UI.Image)
                local name = elem:GetChild(0):GetComponent(UnityEngine.UI.Text)

                local cfg = config.passive_skill_skills[v.skill]
                if cfg then
                    icon:SetSprite(cfg.skill_icon_path..cfg.skill_icon_name)
                    name.text = string.format( "%s Lv.%d", self.helper:getLangCfg(cfg.lang_id), cfg.skill_level )
                end
            end
        end

        for i=#talents, content.childCount - 1 do
            local elem = content:GetChild(i)
            elem:SetActive(false)
        end
    end,

    refreshTalent = function(self)
        local officer = self.hero.officer
        local talents = officer.skills

        for k, v in ipairs(talents) do
            local elem = nil
            if k > self.talent_content.childCount then
                elem = Object.Instantiate(self.talent_prefab).transform
                elem:SetParent(self.talent_content)
                resetTransformZ(elem)
            else
                elem = self.talent_content:GetChild(k-1)
            end

            if elem then
                elem:SetActive(true)
                elem.name = "Skill_"..k

                self:setTalentInfo(elem, v, k)
            end
        end

        for i=#talents, self.talent_content.childCount - 1 do
            local elem = self.talent_content:GetChild(i)
            elem:SetActive(false)
        end
    end,

    setTalentInfo = function(self, elem, data, idx)
        local cfg = passive_skill_helper:getSkill(data.skillId, data.skillLv)

        if not cfg then
            log.error("###### officer talent cfg error ! id "..data.skillId.." level "..data.skillLv)
            return
        end

        local icon = elem:Find("Icon"):GetComponent(UnityEngine.UI.Image)
        local name = elem:Find("Name"):GetComponent(UnityEngine.UI.Text)
        local des = elem:Find("Des"):GetComponent(UnityEngine.UI.Text)
        local level = elem:Find("Level"):GetComponent(UnityEngine.UI.Text)
        local unlock_tip = elem:Find("UnlockTip"):GetComponent(UnityEngine.UI.Text)
        local max_tip = elem:Find("MaxText"):GetComponent(UnityEngine.UI.Text)
        local lock_img = elem:Find("Lock")

        icon:SetSprite(cfg.skill_icon_path..cfg.skill_icon_name)
        name.text = self.helper:getLangCfg(cfg.lang_id)
        level.text = "Lv."..cfg.skill_level
        des.text = self.helper:getLangCfg(cfg.skill_des)

        local unlock_star = self.hero.officer:getSkillUnlockStar(idx)
        local star_lv = self.hero.officer.star_level
        unlock_tip.text = string.format( NativeText.GetLangStr(110559), unlock_star )

        unlock_tip.transform:SetActive(star_lv < unlock_star)
        lock_img:SetActive(star_lv < unlock_star)
        max_tip.transform:SetActive(false)
    end,

    getOfficerProps = function(self, attrs)
        local props = {}
        local props_change = {}

        local contains = function(id)
            for k, v in ipairs(self.hero.hero_props) do
                if v.PropId == id then
                    return v
                end
            end
            return false
        end

        for k, v in pairs(attrs) do
            local param = contains(k)

            if not param then
                param = {PropId = k, PropValue = 0}
            end

            table.insert( props, param )
            props_change[k] = v + param.PropValue
        end

        return props, props_change
    end,

    onClickAwake = function(self)
        if gui_helper:heroIsWeakness(self.hero.id) then
            return
        end

        local officer = self.hero.officer
        
        if self.can_awake then
            network_sender:officerAwake(officer.inst_id)
        else
            hint:setText(NativeText.GetLangStr(110077))
        end
    end,

    onClickPromote = function(self)
        if gui_helper:heroIsWeakness(self.hero.id) then
            return
        end
        local officer = self.hero.officer

        if self.can_promote then
            network_sender:officerPromote(officer.inst_id)
        else
            hint:setText(NativeText.GetLangStr(110078))
        end
    end,

    hide = function(self)
        self.helper.hideTrans(self.panel)
    end,

}