local log = logger.get("Game", "commander promote result panel")

return class{
    ctor = function(self, info_panel)
        self.panel = info_panel
        self.helper = require("gui.tool.frame_helper")

        self.cur_icon = self.panel:Find('Info/Icons/Icon1')
        self.next_icon = self.panel:Find('Info/Icons/Icon2')
        self.fight_text = self.panel:Find('Info/Fight/FightText'):GetComponent(UnityEngine.UI.Text)
        self.title = self.panel:Find('Info/Text'):GetComponent(UnityEngine.UI.Text)
        self.promote_node = self.panel:Find('Info/PromoteProps')
        self.awake_node = self.panel:Find('Info/AwakeProps')

        self:hide()
    end,

    showPromoteResult = function(self, officer_inst, isPromote)
        self.helper.showTrans(self.panel)

        if isPromote then
            self.title.text = NativeText.GetLangStr("110006")
            self:setProps(officer_inst)
        else
            self.title.text = NativeText.GetLangStr("110031")
            local cur_skill = officer_inst:getAwakeCfg(officer_inst.star_level-1, officer_inst.awake_level).skill
            local next_skill = officer_inst:getAwakeCfg(officer_inst.star_level, officer_inst.awake_level).skill
            self:setSkill(self.awake_node, cur_skill, next_skill, true)
        end
        self.promote_node:SetActive(isPromote)
        self.awake_node:SetActive(not isPromote)
        
        local hero_id = officer_inst:getTankId()
        self:setFight(hero_manager.heros[hero_id].strength)
        self:setIcon(officer_inst, isPromote)
    end,

    setProps = function(self, officer)
        local props = officer:getProps(officer.level, officer.quality-1, officer.awake_level, officer.star_level)
        local next_props = officer:getProps(officer.level, officer.quality, officer.awake_level, officer.star_level)
        local props_change = {}
        for k, v in pairs(next_props) do
            props_change[v.PropId] = v.PropValue
        end
        gui_helper:showUpgradeProps(self.promote_node, props, props_change, showPlus)
    end,

    setSkill = function(self, props_panel, cur_props, next_props, showPlus)
        local props = {}
        local props_change = {}
        for k, v in pairs(cur_props) do
            table.insert( props, {PropId = k, PropValue = v} )
            props_change[k] = next_props[k]
        end 
        gui_helper:showUpgradeProps(props_panel, props, props_change, showPlus)
    end,

    setFight = function(self, value)
        self.fight_text.text = value
    end,

    setIcon = function(self, officer, isPromote)
        if isPromote then
            gui_helper:setOfficerIcon(officer, officer.star_level, officer.quality-1, self.cur_icon)
            gui_helper:setOfficerIcon(officer, officer.star_level, officer.quality, self.next_icon)
        else
            gui_helper:setOfficerIcon(officer, officer.star_level-1, officer.quality, self.cur_icon)
            gui_helper:setOfficerIcon(officer, officer.star_level, officer.quality, self.next_icon)
        end
    end,

    hide = function(self)
        self.helper.hideTrans(self.panel)
    end,


}