local super = require("ui.frame")
local log = logger.get("Game", "Officer Frame")
local gui_helper = require("gui.gui_helper")

local trans_tab = {
    officer = 1,
    hero = 2,
}

local panel_indexs = {
    dev = 1,
    info = 2,
    detail = 3,
}

return class
{
    super = super,

    onLoaded = function (self)
        super.onLoaded(self)

        self.owner = self.args.owner
        self.hero_id = self.args.data.id
        self.heros = self.args.data.heros

        -- 路径: root/login
        -- 事件: click_login 
        clickEvents = {
           "BG/Title/BackBtn",
           "BG/Content/LeftBtn",
           "BG/Content/RightBtn",
           "BG/Content/ViewBtn",
           "BG/Content/Right/Top/FillBG/Plus",
           "BG/Content/Right/Center/FillBG/StarupBG/StarupBtn",
           "BG/Content/Right/Skills/InfoBtn",
           "BG/Content/Weapons/MakeBtn",
           "BG/Content/Weapons/Back",
           "BG/EquipInfo/CloseEquipInfo",
           "BG/EquipInfo/Layout/Equip_1/Panel/Remove",
           "BG/EquipInfo/Layout/Equip_1/Panel/ChangeBtn",
           "BG/EquipInfo/Layout/Equip_1/Panel/EquipBtn",
           "BG/EquipInfo/Layout/Equip_2/Panel/Remove",
           "BG/EquipInfo/Layout/Equip_2/Panel/ChangeBtn",
           "BG/EquipInfo/Layout/Equip_2/Panel/EquipBtn",
           "BG/SkillDetails/CloseSkillDetails",
           "BG/UseExp/CloseUseExp",
           "BG/GetWay/CloseGetWay",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self.officer_name = self.transform:Find("BG/Content/Left/NameNode/OfficerName"):GetComponent(UnityEngine.UI.Text)
        self.tank_name = self.transform:Find("BG/Content/Left/NameNode/TankName"):GetComponent(UnityEngine.UI.Text)
        self.officer_card = self.transform:Find("BG/Content/OfficerCard"):GetComponent(UnityEngine.UI.Image)
        self.officer_spine = self.transform:Find("BG/Content/Canvas/Mask/OfficerSpine")
        self.view_btn = self.transform:Find("BG/Content/ViewBtn")

        self.equip_node = self.transform:Find("BG/Content/Left/EquipNode")
        self.right_node = self.transform:Find("BG/Content/Right")
        self.weapons_node = self.transform:Find("BG/Content/Weapons")
        self.name_node = self.transform:Find("BG/Content/Left/NameNode")

        self.weapons_node.localScale = Vector3.zero

        local panel = self.transform:Find("BG/EquipInfo")
        self.equipment_detail = require("gui.equipment_info_panel")(panel, self)

        self.skill_panel = self.transform:Find("BG/SkillDetails")
        self.use_exp_panel = self.transform:Find("BG/UseExp")
        self.getway_panel = self.transform:Find("BG/GetWay")

        self.skill_panel:SetActive(false)
        self.use_exp_panel:SetActive(false)
        self.getway_panel:SetActive(false)

        --self.scroll_content = self.transform:Find('BG/Left/Scroll View/Viewport/Content')
        --self.scroll_snap = self.transform:Find('BG/Left/Scroll View'):GetComponent(UnityEngine.UI.Extensions.InfiniteScrollSnap)
        self.tank_root = self.transform:Find("TankView")
        --self.mask_image = self.transform:Find("BG/Left/Scroll View/Viewport"):GetComponent(UnityEngine.UI.Image)
        self.rawImage = self.transform:Find("BG/TankImage")
        --self.view_rawImage = self.transform:Find("BG/ViewNode/TankImage")
        self.tank_scroll_snap = self.transform:Find('TankView/TankScrollView'):GetComponent(UnityEngine.UI.Extensions.InfiniteScrollSnap)
        --self.view_node = self.transform:Find("BG/ViewNode")
        self.origin_camera = self.transform:Find("TankView/GameObject/Camera")
        self.view_camera = self.transform:Find("TankView/GameObject/Camera_1")

        self.effect_panel = self.transform:Find("BG/EffectPanel")
        self.star_up_effect = self.right_node:Find("effect_command_starup")
        self.effect_animator = self.star_up_effect:GetComponent(UnityEngine.Animator)
        self.effect_panel:SetActive(false)
        self.star_up_effect:SetActive(false)

        self.origin_camera_pos = self.origin_camera.localPosition
        self.origin_camera_rot = self.origin_camera.localRotation
        self.view_camera_pos = self.view_camera.localPosition
        self.view_camera_rot = self.view_camera.localRotation

        self.spine_assets = {}
        self.hero_mods = {}
        self.active_heros = {}
        self.hero_trans = {}
        self.base_plate = self.transform:Find("TankView/GameObject/garage/yuanpan")

        self.hero_idx = 1
        
        local idx = 1
        for k, hero in ipairs(self.heros) do
            if hero.is_actived then
                local hero_inst = hero_manager:getHero(hero.id)
                table.insert( self.active_heros, hero_inst )

                if hero.id == self.hero_id then
                    self.hero_idx = idx
                end
                idx = idx + 1
            end
        end

        self.isBagChanged = false
        self.officer_changed = false
        self.is_view = false
        self.spine_zoom = 1

        self.draging = false
        self.drag_began_pos = 0
        self.drag_end_pos = 0

        self.curr_state = trans_tab.officer

        self.effect_playing = false

        self.load_mod_queue = {}
        self.load_spine_queue = {}

        self:loadOfficerPic()
        self:loadTankNode()
        self:loadTankMods()
        --self:setOfficerInfo()
        --self:loadOfficerSkills()

        self:onOfficerChanged()
        self:weaponSelected(0)

        self:setDragCallback()

        self.refresh_id = hero_manager.events:on("HEROREFRESH", functor.make(self.onHeroRefresh, self))
        self.equip_changed_id = bag_manager.events:on("onEquipChanged", functor.make(self.onEquipChanged, self))
        self.bag_change_id = bag_manager.events:on("onBagChanged", functor.make(self.onBagChanged, self))
        self.star_up_id = hero_manager.events:on("OFFICERSTARUP", functor.make(self.onStarUp, self))

        self.helper.hideTrans(self.owner.transform)
        game_director:disable3DRender()
    end,

    onUnload = function (self)
        hero_manager.events:off("HEROREFRESH", self.refresh_id)
        hero_manager.events:off("OFFICERSTARUP", self.star_up_id)
        bag_manager.events:off("onEquipChanged", self.equip_changed_id)
        bag_manager.events:off("onBagChanged", self.bag_change_id)

        game_director:enable3DRender()

        for k,v in pairs(self.spine_assets) do
            if v then
                v:RemoveRef()
            end
        end

        if self.starup_coroutine then
            app:stopCoroutine(self.starup_coroutine)
        end
    end,

    update = function(self, dt)
        if self.officer_changed and not self.tank_scroll_snap:GetClickCD() then

            self:onOfficerChanged()
            self:loadTankMods()
            self.officer_changed = false
        end

        if self.isBagChanged then
            self:onOfficerChanged()
            self.isBagChanged = false
        end

        if self.draging then
            local id = self.active_heros[self.hero_idx].id
            local mod = self.hero_mods[id].transform
            local angle = self.drag_end_pos - self.drag_began_pos
            local to_qua = Quaternion.Euler(0, self.form_qua.y - angle*0.1, 0)
            mod.rotation = to_qua
            self.base_plate.rotation = Quaternion.Euler(-90, self.base_plate_from_qua.y - angle*0.1, 0)
        end

        if self.load_mod_queue and #self.load_mod_queue > 0 then
            self:addMod(self.load_mod_queue[1])
            table.remove( self.load_mod_queue, 1 )
        end

        if self.load_spine_queue and #self.load_spine_queue > 0 then
            self:addSpineAsset(self.load_spine_queue[1])
            table.remove( self.load_spine_queue, 1 )
        end
    end,

    setDragCallback = function(self)
            local setCB = function(mono)
                mono:SetCallback(
                    function(eventData)
                        log.info("拖拽开始", eventData.position)
                        self.draging = true
                        local id = self.active_heros[self.hero_idx].id
                        local mod = self.hero_mods[id].transform
                        self.form_qua = mod.rotation.eulerAngles
                        self.base_plate_from_qua = self.base_plate.rotation.eulerAngles

                        self.drag_began_pos = eventData.position.x
                    end,
                    function(eventData)
                        self.drag_end_pos = eventData.position.x
                    end,
                    function(eventData)
                        self.draging = false
                        self.drag_began_pos = 0
                        self.drag_end_pos = 0
                    end
                )
            end

            local drag_image = self.rawImage:GetComponent(DragImage)
            setCB(drag_image)
            --drag_image = self.view_rawImage:GetComponent(DragImage)
            --setCB(drag_image)
    end,

    onStarUp = function(self)
        self.starup_coroutine = app:startCoroutine(
            function()
                self.effect_playing = true
                self.effect_panel:SetActive(true)
                self.star_up_effect:SetActive(true)
                self.effect_animator:SetTrigger("Play")
                coroutine.yield(WaitForSeconds(4.8))
                self.effect_panel:SetActive(false)
                self.effect_playing = false

                local officer = self:getCurrOfficer()
                self.star_up_effect:SetActive(officer:canStarUp())
            end
        )
    end,

    onHeroRefresh = function(self)
        self.isBagChanged = true
    end,

    onBagChanged = function(self)
        if self.change_weapon then
            self:loadBagWeapons(self.slot_id)
        end

        if self.use_exp_item then
            self:showExpItems()

            if self.use_exp_sldier then
                self.use_exp_sldier:refreshMaxCnt()
            end
        end
    end,

    onEquipChanged = function(self, hero_id)
        local hero = hero_manager.heros[hero_id]
        if not hero then
            log.error("Failed to find hero:"..hero_id)
        end
        
        local tank_mod = self.hero_mods[hero_id]
        if tank_mod then
            local tank_node = tank_mod.transform:GetParent()

            local zoom = hero.tank_cfg.zoom/10000
            self.hero_mods[hero_id] = gui_helper:createUITank(hero_id, hero.tank_id)
            self.hero_mods[hero_id].transform:SetParent(tank_node)
            self.hero_mods[hero_id].transform.localScale = Vector3.New(zoom, zoom, zoom)
            self.hero_mods[hero_id].transform.localPosition = Vector3.zero
            self.hero_mods[hero_id].transform.localRotation = Quaternion.Euler(0, -145, 0)
            tank_mod:destroy()
        end
    end,

    onPropChanged = function(self)
        --self:setFight()
    end,
    
    onOfficerChanged = function(self)
        self:setOfficerInfo()
        self:loadOfficerSkills()
        self:loadOfficerWeapon()

        if self.change_weapon then
            self:loadBagWeapons(self.slot_id)
        end
    end,

    getCurrOfficer = function(self)
        local hero_id = self.active_heros[self.hero_idx].id
        local hero = hero_manager:getHero(hero_id)
        local officer = hero.officer

        return officer
    end,

    updateExp = function(self)
        local hero_id = self.active_heros[self.hero_idx].id
        local hero = hero_manager:getHero(hero_id)
        local officer = hero.officer

        local officer_lv = officer.level
        local next_lv = officer_lv + 1
        local lv_max = config.commander_level_ups[next_lv] == nil

        local exp_fill = self.right_node:Find("Top/FillBG/Fill"):GetComponent(UnityEngine.UI.Image)
        local exp_value = self.right_node:Find("Top/FillBG/Exp"):GetComponent(UnityEngine.UI.Text)
        local plus_btn = self.right_node:Find("Top/FillBG/Plus")

        if lv_max then
            exp_fill.fillAmount = 1
            exp_value.text = "Max"
        else
            local need_exp = officer:getLevelUpExp(officer_lv)
            exp_fill.fillAmount = officer.exp / (need_exp > 0 and need_exp or 1)
            exp_value.text = string.format( "%d/%d", officer.exp, need_exp )
        end

        plus_btn:SetActive(not lv_max)
    end,

    weaponSelected = function(self, slot_id)
        local light_1 = self.equip_node:GetChild(0):Find("Light")
        local light_2 = self.equip_node:GetChild(1):Find("Light")
        local light_3 = self.equip_node:GetChild(2):Find("Light")

        light_1:SetActive(slot_id == common_pb.EQUIP_WEAPON_SLOT)
        light_2:SetActive(slot_id == common_pb.EQUIP_ARMOUR_SLOT)
        light_3:SetActive(slot_id == common_pb.EQUIP_DECORATION_SLOT)
    end,

    loadOfficerWeapon = function(self)
        local hero_id = self.active_heros[self.hero_idx].id
        local hero = hero_manager:getHero(hero_id)

        local hero_bag = hero.hero_bag

        setEquipment = function(item, ui, slot_id)
            local icon = ui:Find("Grade/Icon"):GetComponent(UnityEngine.UI.Image)
            local grade = ui:Find("Grade"):GetComponent(UnityEngine.UI.Image)
            local none_node = ui:Find("None")
            local type_icon = ui:Find("None/Icon")
            local add = ui:Find("None/Add")

            if item then
                grade.transform.localScale = Vector3.one
                none_node.localScale = Vector3.zero

                local icon_path = gui_helper:getItemIconPath(item.cfg)
                icon:SetSprite(icon_path)
                grade:SetSprite(gui_helper:getQualityIconPath(item.cfg.color))
            else
                grade.transform.localScale = Vector3.zero
                none_node.localScale = Vector3.one
        
                local items = self:getItemsByFamily(slot_id)

                if #items > 0 then
                    add.localScale = Vector3.one
                    type_icon.localScale = Vector3.zero
                else
                    add.localScale = Vector3.zero
                    type_icon.localScale = Vector3.one
                end
            end

            if self.slot_id and self.slot_id == slot_id then
                self.select_equip = item
            end

            local path = "BG/Content/Left/EquipNode/"
            self:clearScript("click", path..ui.name)
            self:setScript("click", path..ui.name, functor.rmake(self.onClick_weapon, slot_id))
        end

        local slot = self.equip_node:GetChild(0)
        local equipment = hero_bag:get(common_pb.EQUIP_WEAPON_SLOT)
        setEquipment(equipment, slot, common_pb.EQUIP_WEAPON_SLOT)

        slot = self.equip_node:GetChild(1)
        equipment = hero_bag:get(common_pb.EQUIP_ARMOUR_SLOT)
        setEquipment(equipment, slot, common_pb.EQUIP_ARMOUR_SLOT)

        slot = self.equip_node:GetChild(2)
        equipment = hero_bag:get(common_pb.EQUIP_DECORATION_SLOT)
        setEquipment(equipment, slot, common_pb.EQUIP_DECORATION_SLOT)
    end,

    setOfficerInfo = function(self)
        local hero_id = self.active_heros[self.hero_idx].id
        local hero = hero_manager:getHero(hero_id)
        local officer = hero.officer

        local officer_lv = officer.level
        local next_lv = officer_lv + 1
        local lv_max = config.commander_level_ups[next_lv] == nil
        --self.rawImage:SetActive(false)

        self.officer_name.text = hero.officer.name
        local name_bg = self.transform:Find("BG/Content/Left/NameNode/Image"):GetComponent(UnityEngine.UI.Image)
        name_bg:SetSprite(gui_helper:getHeroNameQualityIcon(officer.cfg.color))
        self.tank_name.text = self.helper:getLangCfg(hero.tank_cfg.name)
        self.tank_name.color = gui_helper:quality2UnityColor(officer.cfg.color)
        --self.officer_card:SetSprite(gui_helper:getOfficerPicturePath(officer))
        --local img = self.officer_card:GetComponent(UnityEngine.UI.Image)
        --img:SetSprite(gui_helper:getOfficerPicturePath(officer))
        self:setOfficerCard()

        local level = self.right_node:Find("Top/LevelBG/Level"):GetComponent(UnityEngine.UI.Text)
        local type_icon = self.right_node:Find("Top/TypeBG/Icon"):GetComponent(UnityEngine.UI.Image)

        level.text = "Lv."..officer_lv
        type_icon:SetSprite(officer.cfg.icon_path..officer.cfg.soldiertypeicon)

        local power_1 = self.right_node:Find("Top/Power/Value1"):GetComponent(UnityEngine.UI.Text)
        local power_2 = self.right_node:Find("Top/Power/Value2"):GetComponent(UnityEngine.UI.Text)
        local power_arrow = self.right_node:Find("Top/Power/Arrow")

        power_1.text = hero.strength
        
        local curr_lv_strength = officer:getLevelStrength(officer_lv)
        local next_lv_stringth = officer:getLevelStrength(next_lv)

        power_2.text = hero.strength + next_lv_stringth - curr_lv_strength

        power_2.transform:SetActive(not lv_max)
        power_arrow:SetActive(not lv_max)

        local amount_1 = self.right_node:Find("Top/Amount/Value1"):GetComponent(UnityEngine.UI.Text)
        local amount_2 = self.right_node:Find("Top/Amount/Value2"):GetComponent(UnityEngine.UI.Text)
        local amount_arrow = self.right_node:Find("Top/Amount/Arrow")

        amount_1.text = hero:getGlobalProp(1171)
        amount_2.text = officer.cfg.corp_num_up + hero:getGlobalProp(1171)

        amount_2.transform:SetActive(not lv_max)
        amount_arrow:SetActive(not lv_max)

        local exp_fill = self.right_node:Find("Top/FillBG/Fill"):GetComponent(UnityEngine.UI.Image)
        local exp_value = self.right_node:Find("Top/FillBG/Exp"):GetComponent(UnityEngine.UI.Text)
        local plus_btn = self.right_node:Find("Top/FillBG/Plus")

        if lv_max then
            exp_fill.fillAmount = 1
            exp_value.text = "Max"
        else
            local need_exp = officer:getLevelUpExp(officer_lv)
            exp_fill.fillAmount = officer.exp / (need_exp > 0 and need_exp or 1)
            exp_value.text = string.format( "%d/%d", officer.exp, need_exp )
        end

        plus_btn:SetActive(not lv_max)

        local officer_icon = self.right_node:Find("Center/Head/Icon"):GetComponent(UnityEngine.UI.Image)
        local officer_grade = self.right_node:Find("Center/Head/Grade"):GetComponent(UnityEngine.UI.Image)
        local piece_fill = self.right_node:Find("Center/FillBG/Fill"):GetComponent(UnityEngine.UI.Image)
        local piece_count = self.right_node:Find("Center/FillBG/Count"):GetComponent(UnityEngine.UI.Text)
        local stars_1 = self.right_node:Find("Center/FillBG/Stars1")
        local stars_2 = self.right_node:Find("Center/FillBG/Stars2")
        local star_arrow = self.right_node:Find("Center/FillBG/Arrow")
        local star_up = self.right_node:Find("Center/FillBG/StarupBG")
        local tip = self.right_node:Find("Center/Tip")

        officer_icon:SetSprite(gui_helper:getOfficerIconPath(officer))
        officer_grade:SetSprite(gui_helper:getNewQualityIcon(officer.cfg.color))

        local curr_piece = self.helper:getItemNum(officer.cfg.piece_id)
        local need_piece = officer:getStarUpPieceCnt(officer.star_level)
        local star_max = config.commander_star_ups[officer.star_level + 1] == nil
        
        tip:SetActive(not star_max)
        
        if star_max then
            piece_fill.fillAmount = 1
            piece_count.text = tostring(curr_piece)
        else
            piece_fill.fillAmount = curr_piece / (need_piece > 0 and need_piece or 1)
            piece_count.text = string.format( "%d/%d", curr_piece, need_piece )
            self:setOfficerStars(stars_2, officer.star_level+1)
        end

        self:setOfficerStars(stars_1, officer.star_level)

        stars_2:SetActive(not star_max)
        star_arrow:SetActive(not star_max)
        star_up:SetActive(not star_max and curr_piece >= need_piece)

        self.star_up_effect:SetActive(officer:canStarUp() or self.effect_playing)
    end,

    loadOfficerSkills = function(self)
        local skill_ui = {}
        skill_ui[1] = self.right_node:Find("Skills/Skill1")
        skill_ui[2] = self.right_node:Find("Skills/Skill2")
        skill_ui[3] = self.right_node:Find("Skills/Skill3")

        local hero_id = self.active_heros[self.hero_idx].id
        local hero = hero_manager:getHero(hero_id)
        local officer = hero.officer

        local skills = officer.skills

        for k, v in ipairs(skills) do
            local ui = skill_ui[k]

            if ui then
                local name = ui:Find("Name"):GetComponent(UnityEngine.UI.Text)
                local icon = ui:Find("Icon"):GetComponent(UnityEngine.UI.Image)
                local des = ui:Find("Des"):GetComponent(UnityEngine.UI.Text)
                local level = ui:Find("Level"):GetComponent(UnityEngine.UI.Text)
                local type_icon = ui:Find("Type"):GetComponent(UnityEngine.UI.Image)

                local skill_cfg = passive_skill_helper:getSkill(v.skillId, v.skillLv)
                if skill_cfg then
                    name.text = self.helper:getLangCfg(skill_cfg.lang_id)
                    icon:SetSprite(skill_cfg.skill_icon_path..skill_cfg.skill_icon_name)
                    level.text = "Lv."..v.skillLv
                    des.text = self.helper:getLangCfg(skill_cfg.skill_des)
                    type_icon:SetSprite(gui_helper:getSkillTypeIcon(skill_cfg))

                    local btn = ui:Find("Selected"):GetComponent(UnityEngine.UI.Button)
                    btn.onClick:RemoveAllListeners()
                    btn.onClick:AddListener(
                        function()
                            self:showSkillDetails(v.skillId, v.skillLv)
                        end
                    )
                end

                ui:SetActive(true)
            end
        end

        for i=#skills+1, #skill_ui do
            skill_ui[i]:SetActive(false)
        end
    end,

    setOfficerStars = function(self, node, level)
        local prefab = node:GetChild(0).gameObject

        for i=1, level do
            local elem = nil
            if i > node.childCount then
                elem = Object.Instantiate(prefab).transform
                elem:SetParent(node)
                resetTransformZ(elem)
            else
                elem = node:GetChild(i - 1)
            end

            elem:SetActive(true)
        end

        for i=level, node.childCount - 1 do
            local elem = node:GetChild(i)
            elem:SetActive(false)
        end
    end,

    loadBagWeapons = function(self, slot_id)

        self.change_weapon = true

        local equipments = self:getItemsByFamily(slot_id)

        local icon_root = nil
        local path_root = "BG/Content/Weapons/Scroll View/Viewport/Content/"
        local index = 0
        local content = self.weapons_node:Find("Scroll View/Viewport/Content")
        local prefab = content:GetChild(0).gameObject

        for k, v in ipairs(equipments) do
            local elem = nil
            if k > content.childCount then
                elem = Object.Instantiate(prefab).transform
                elem:SetParent(content)
                resetTransformZ(elem)
            else
                elem = content:GetChild(k-1)
            end

            elem:SetActive(true)
            elem.name = "Weapon_"..k

            local grade = elem:Find("Grade"):GetComponent(UnityEngine.UI.Image)
            local icon = elem:Find("Grade/Icon"):GetComponent(UnityEngine.UI.Image)

            local icon_path = gui_helper:getItemIconPath(v.cfg)
            icon:SetSprite(icon_path)
            grade:SetSprite(gui_helper:getQualityIconPath(v.cfg.color))

            local btn = elem:GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(
                function()
                    self:onClickBagWeapon(v, elem)
                end
            )
        end

        for i = #equipments, content.childCount -1 do
            local child = content:GetChild(i)
            child:SetActive(false)
        end

        local none_text = self.weapons_node:Find("NoneText")
        none_text:SetActive(#equipments == 0)
    end,

    --设置指挥官立绘
    loadOfficerPic = function(self)
        --local officer_pic = self.transform:Find('OfficerPic').gameObject
        local pictures = self:getOfficerPictures()
        for k, v in ipairs(pictures) do
            --[[ local pic = nil
            if k > self.scroll_content.childCount then
                pic = Object.Instantiate(officer_pic).transform
            else
                pic = self.scroll_content:GetChild(k-1)
            end
            pic:SetParent(self.scroll_content)
            -- pic.localScale = Vector3.one
            -- local pos = pic.localPosition
            -- pos.z = 0
            -- pic.localPosition = pos
            resetTransformZ(pic)
            pic:SetActive(true)
            if v.path then
                pic:GetChild(0):GetComponent(UnityEngine.UI.Image):SetSprite(v.path)
            end

            local tab = {}
            tab.pic_node = pic ]]
            self.hero_trans[v.id] = {}
        end
    end,

    loadTankNode = function(self)
        local view_content = self.transform:Find('TankView/TankScrollView/Viewport/Content')
        local tank_node = self.transform:Find('TankNode').gameObject
        local pictures = self:getOfficerPictures()
        for k, v in ipairs(pictures) do
            local node = nil
            if k > view_content.childCount then
                node = Object.Instantiate(tank_node)
            else
                node = view_content:GetChild(k-1)
            end
            node.transform:SetParent(view_content)
            
            resetTransformZ(node.transform)

            if self.hero_trans[v.id] then
                self.hero_trans[v.id].tank_node = node.transform
            end
        end
    end,

    loadTankMods = function(self)
        for i=-1, 1, 1 do
            local idx = self.hero_idx + i
            if idx <= 0 then
                idx = #self.active_heros
            elseif idx > #self.active_heros then
                idx = 1
            end
            --self:addMod(idx)
            self:addLoadModQueue(idx)
            self:addLoadSpineQueue(idx)
        end
    end,

    addLoadModQueue = function(self, idx)
        for k, v in ipairs(self.load_mod_queue) do
            if v == idx then
                return
            end
        end

        table.insert( self.load_mod_queue, idx )
    end,

    removeLoadModQueue = function(self, idx)
        local cnt = nil
        for k, v in ipairs(self.load_mod_queue) do
            if idx == v then
                cnt = k
                break
            end
        end

        if cnt then
            table.remove(self.load_mod_queue, cnt)
        end
    end,

    addMod = function(self, idx)  
        local hero = self.active_heros[idx]
        local hero_trans = self.hero_trans[hero.id]

        if hero_trans.mod then
            return
        end

        local zoom = hero.tank_cfg.zoom/10000
        local tank = gui_helper:createUITank(hero.id, hero.tank_id)
        if tank and tank.transform then
            tank.transform:SetParent(self.tank_root)
            tank.transform.localScale = Vector3.New(zoom, zoom, zoom)
            tank.transform.localPosition = Vector3.New(1000, 0, 0)
            tank.transform.localRotation = Quaternion.Euler(0, -145, 0)

            tank.transform:SetParent(hero_trans.tank_node)
            tank.transform.localPosition = Vector3.zero
            tank.transform.localRotation = Quaternion.Euler(0, -145, 0)

            hero_trans.mod = tank
            self.hero_mods[hero.id] = tank
        end
    end,

    addLoadSpineQueue = function(self, idx)
        for k, v in ipairs(self.load_spine_queue) do
            if v == idx then
                return
            end
        end

        table.insert( self.load_spine_queue, idx )
    end,

    removeLoadSpineQueue = function(self, idx)
        local cnt = nil
        for k, v in ipairs(self.load_spine_queue) do
            if idx == v then
                cnt = k
                break
            end
        end

        if cnt then
            table.remove(self.load_spine_queue, cnt)
        end
    end,

    addSpineAsset = function(self, idx)
        local hero = self.active_heros[idx]

        if not hero then
            return
        end

        local officer = hero.officer

        if officer.cfg.commander_pic:find('.asset') then
            local path = officer.cfg.cmd_path..officer.cfg.commander_pic

            app:startCoroutine(
                function()
                    if not self.spine_assets[idx] then
                        coroutine.yield(Fenix.Assets.Helper.WaitForAsset(path, Spine.Unity.SkeletonDataAsset, 
                            function(asset) 
                                local sda = asset:Get() -- skeleton data asset
                                asset:AddRef()
                                self.spine_assets[idx] = asset
                            end
                        ))
                    end
                end
            )
        end
    end,

    getOfficerPictures = function(self)
        local pictures = {}
        for i=self.hero_idx, #self.active_heros do
            local path = gui_helper:getOfficerPicturePath(self.active_heros[i].officer)
            table.insert(pictures,{path = path, id = self.active_heros[i].id})
        end
        for i=1, self.hero_idx-1 do
            local path = gui_helper:getOfficerPicturePath(self.active_heros[i].officer)
            table.insert(pictures,{path = path, id = self.active_heros[i].id})
        end
        return pictures
    end,

    click_BackBtn = function(self)
        log.info('click_ButtonBack')
        for k, v in pairs(self.hero_mods) do
            v:destroy()
        end
        
        self.helper.showTrans(self.owner.transform)
        frame_manager:hide('officer')
    end,

    click_ViewBtn = function(self)
        local top = self.transform:Find("BG/Title")
        local bg_img = self.transform:Find("BG/Content"):GetComponent(UnityEngine.UI.Image)

        top:SetActive(self.is_view)
        bg_img.enabled = self.is_view

        if self.is_view then
            self.view_btn.localPosition = Vector3(131.5, 432.5, 0)
            self.equip_node.localScale = Vector3.one
            self.name_node.localScale = Vector3.one
            self.officer_card.transform:SetActive(true)
            self.officer_spine:SetActive(true)
            self.officer_spine.localScale = Vector3.one * self.spine_zoom

            if self.spine_assets[self.hero_idx] then
                self.officer_card.transform:SetActive(false)
                self.officer_spine:SetActive(true)
                self.officer_spine.localScale = Vector3.one * self.spine_zoom
            else
                self.officer_card.transform:SetActive(true)
                self.officer_spine:SetActive(false)
            end

            if self.change_weapon then
                self.weapons_node.localScale = Vector3.one
            else
                self.right_node.localScale = Vector3.one
            end
        else
            self.view_btn.localPosition = Vector3(866.5, 432.5, 0)
            self.equip_node.localScale = Vector3.zero
            self.name_node.localScale = Vector3.zero
            self.officer_card.transform:SetActive(false)
            self.officer_spine:SetActive(false)

            if self.change_weapon then
                self.weapons_node.localScale = Vector3.zero
            else
                self.right_node.localScale = Vector3.zero
            end
        end

        self.is_view = not self.is_view
    end,

    click_LeftBtn = function(self) 
        log.info('click_ButtonLeft')
        if not self.officer_changed and self.spine_finished then
            self:removeLoadModQueue(self.hero_idx + 1)
            self.tank_scroll_snap:PreviousScreen()
            self.hero_idx = self.hero_idx - 1
            if self.hero_idx < 1 then
                self.hero_idx = #self.active_heros
            end
            self.officer_changed = true
        end
    end,
   
    click_RightBtn = function(self) 
        log.info('click_ButtonRight')
        if not self.officer_changed and self.spine_finished then
            self:removeLoadModQueue(self.hero_idx - 1)
            self.tank_scroll_snap:NextScreen()
            self.hero_idx = self.hero_idx + 1
            if self.hero_idx > #self.active_heros then
                self.hero_idx = 1
            end
            self.officer_changed = true
        end
    end,

    click_Plus = function(self)
        local items = self:getExpItems()

        if #items == 0 then
            hint:setText(NativeText.GetLangStr(280321))
            return
        end

        self:showExpItems()
    end,

    click_InfoBtn = function(self)
        local content = NativeText.GetLangStr(280320)
        content = content:gsub("\\n", "\n")
        
        local tb = {
            title = NativeText.GetLangStr(280319),
            content = content,
        }

        frame_manager:show('common_box_2', nil, tb)
    end,
    
    click_StarupBtn = function(self)
        local hero_id = self.active_heros[self.hero_idx].id
        local hero = hero_manager:getHero(hero_id)
        local officer = hero.officer

        if officer:canStarUp() then
            network_sender:officerStarUp(officer.inst_id)
        else
            hint:setText(NativeText.GetLangStr(280322))
        end
    end,

    click_MakeBtn = function(self)
        local build_data = home_info:getBuildDataByCfgId(11)

        if build_data then
            frame_manager:show('home_arsenal_main')
        else
            hint:setText(NativeText.GetLangStr(9000008))
        end
    end,

    click_Back = function(self)
        self.right_node.localScale = Vector3.one
        self.weapons_node.localScale = Vector3.zero
        self.change_weapon = false
        self:weaponSelected(0)
    end,

    click_CloseEquipInfo = function(self)
        self.equipment_detail:hide()
    end,

    click_Remove = function(self)
        self.equipment_detail:hide()
        local hero_id = self.active_heros[self.hero_idx].id
        network_sender:heroEquip(hero_id, self.select_equip.inst_id, self.slot_id, protocol_pb.IU_UNEQUIP)
        if self.change_weapon then
            self:weaponSelected(self.slot_id)
        end
    end,

    click_ChangeBtn = function(self)
        self.equipment_detail:hide()
        self.right_node.localScale = Vector3.zero
        self.weapons_node.localScale = Vector3.one

        self:loadBagWeapons(self.slot_id)
        self:weaponSelected(self.slot_id)
    end,

    click_EquipBtn = function(self)
        self.equipment_detail:hide()
        local hero_id = self.active_heros[self.hero_idx].id
        network_sender:heroEquip(hero_id, self.bag_equip.inst_id, self.slot_id, protocol_pb.IU_EQUIP)
        self:weaponSelected(self.slot_id)
    end,

    onClick_weapon = function(self, handler, slot_id)
        local hero_id = self.active_heros[self.hero_idx].id
        local hero = hero_manager:getHero(hero_id)

        local hero_bag = hero.hero_bag
        local equipment = hero_bag:get(slot_id)
        
        self.slot_id = slot_id

        if equipment then
            if self.change_weapon and self.select_equip and self.select_equip.inst_id == equipment.inst_id  then
                self.equipment_detail:show(equipment, nil, self.equipment_detail.equip_states.remove)
            else
                self.select_equip = equipment
                self.equipment_detail:show(equipment, nil, self.equipment_detail.equip_states.show)
            end
        else
            self.select_equip = nil
            self.right_node.localScale = Vector3.zero
            self.weapons_node.localScale = Vector3.one

            self:loadBagWeapons(slot_id)
            self:weaponSelected(slot_id)
        end
    end,

    onClickBagWeapon = function(self, weapon, ui)
        self.bag_equip = weapon

        if self.select_equip then
            self.equipment_detail:show(self.select_equip, weapon, self.equipment_detail.equip_states.replace)
        else
            self.equipment_detail:show(weapon, nil, self.equipment_detail.equip_states.equip)
        end
    end,

    useOriginCamera = function(self)
        self.origin_camera.localPosition = self.origin_camera_pos
        self.origin_camera.localRotation = self.origin_camera_rot
    end,

    useViewCamera = function(self)
        self.origin_camera.localPosition = self.view_camera_pos
        self.origin_camera.localRotation = self.view_camera_rot
    end,
    
    --根据武器family获取武器
    getItemsByFamily = function(self, family)
        local items = {}
        local bag = bag_manager:getBag(1)
        local bag_equipments = bag:getItems(8)

        for _,item in ipairs(bag_equipments) do
            local equip_family = config.weapon_weapons[item.cfg_id].family
            if family == equip_family then
                table.insert(items, item)
            end
        end

        local equipments = {}
        local hero_id = self.active_heros[self.hero_idx].id
        local hero = hero_manager:getHero(hero_id)
        local hero_type = hero.tank_cfg.type

        for k, v in ipairs(items) do
            if v.hero_id ~= hero_id then 
                if v.cfg.weapon_type == 0 then
                    table.insert( equipments, v )
                else
                    if "type"..v.cfg.weapon_type == hero_type then
                        table.insert( equipments, v )
                    end
                end
            end
        end

        return equipments
    end,

    showSkillDetails = function(self, skill_id, level)
        self.skill_panel:SetActive(true)
        
        local curr_skill = passive_skill_helper:getSkill(skill_id, level)

        local icon = self.skill_panel:Find("BG/Icon"):GetComponent(UnityEngine.UI.Image)
        local name = self.skill_panel:Find("BG/Name"):GetComponent(UnityEngine.UI.Text)
        local des = self.skill_panel:Find("BG/Des"):GetComponent(UnityEngine.UI.Text)
        local tip = self.skill_panel:Find("BG/Tip"):GetComponent(UnityEngine.UI.Text)

        name.text = self.helper:getLangCfg(curr_skill.lang_id).."  Lv."..level
        icon:SetSprite(curr_skill.skill_icon_path..curr_skill.skill_icon_name)
        des.text = self.helper:getLangCfg(curr_skill.skill_des)

        if curr_skill.Need_Battle == 1 then
            tip.text = string.format( "(%s)", NativeText.GetLangStr(280323) )
            name.text = string.format( "%s [%s]",name.text, NativeText.GetLangStr(280324) )
        else
            tip.text = string.format( "(%s)", NativeText.GetLangStr(280317) )
            name.text = string.format( "%s [%s]",name.text, NativeText.GetLangStr(280325) )
        end

        local props = self.skill_panel:Find("BG/Props")
        local skills = passive_skill_helper.skill_lvs[skill_id]

        for k, v in ipairs(skills) do
            if k <= props.childCount then
                local elem = props:GetChild(k-1)
                elem:SetActive(true)

                local des_text = elem:Find("Des"):GetComponent(UnityEngine.UI.Text)
                des_text.text = self.helper:getLangCfg(v.skill_des)

                local img = elem:Find("Image")
                img:SetActive(level == v.skill_level)
            end
        end

        for i=#skills, props.childCount - 1 do
            local elem = props:GetChild(i)
            elem:SetActive(false)
        end
    end,

    showExpItems = function(self)
        self.use_exp_panel:SetActive(true)
        self.use_exp_item = true

        local hero_id = self.active_heros[self.hero_idx].id
        local hero = hero_manager:getHero(hero_id)
        local officer = hero.officer

        local use_node = self.use_exp_panel:Find("BG/SliderBG")
        local items_node = self.use_exp_panel:Find("BG/Scroll View")
        local name = self.use_exp_panel:Find("BG/Name"):GetComponent(UnityEngine.UI.Text)
        local icon = self.use_exp_panel:Find("BG/Head/Icon"):GetComponent(UnityEngine.UI.Image)
        local grade = self.use_exp_panel:Find("BG/Head/Grade"):GetComponent(UnityEngine.UI.Image)
        local exp_fill = self.use_exp_panel:Find("BG/FillBG/Fill"):GetComponent(UnityEngine.UI.Image)
        local exp_value = self.use_exp_panel:Find("BG/FillBG/Exp"):GetComponent(UnityEngine.UI.Text)

        name.text = string.format("Lv.%d <color=%s>%s</color>", officer.level, gui_helper:quality2Color(officer.cfg.color), officer.name)
        icon:SetSprite(gui_helper:getOfficerIconPath(officer))
        grade:SetSprite(gui_helper:getNewQualityIcon(officer.cfg.color))

        local officer_lv = officer.level
        local next_lv = officer_lv + 1
        local lv_max = config.commander_level_ups[next_lv] == nil

        if lv_max then
            exp_fill.fillAmount = 1
            exp_value.text = "Max"
        else
            local need_exp = officer:getLevelUpExp(officer_lv)
            exp_fill.fillAmount = officer.exp / (need_exp > 0 and need_exp or 1)
            exp_value.text = string.format( "%d/%d", officer.exp, need_exp )
        end

        local content = self.use_exp_panel:Find("BG/Scroll View/Viewport/Content")
        local prefab = content:GetChild(0).gameObject

        local items = self:getExpItems()
        for k, v in ipairs(items) do
            local elem = nil
            if k > content.childCount then
                elem = Object.Instantiate(prefab).transform
                elem:SetParent(content)
                resetTransformZ(elem)
            else
                elem = content:GetChild(k - 1)
            end

            elem:SetActive(true)

            local grade = elem:Find("Grade"):GetComponent(UnityEngine.UI.Image)
            local icon = elem:Find("Grade/Icon"):GetComponent(UnityEngine.UI.Image)
            local des = elem:Find("Des"):GetComponent(UnityEngine.UI.Text)
            local count = elem:Find("Count"):GetComponent(UnityEngine.UI.Text)
            local use_btn = elem:Find("Buttons/Use"):GetComponent(UnityEngine.UI.Button)
            local mutle_btn = elem:Find("Buttons/MultiUse"):GetComponent(UnityEngine.UI.Button)

            grade:SetSprite(gui_helper:getQualityIconPath(v.cfg.color))
            icon:SetSprite(gui_helper:getItemIconPath(v.cfg))
            des.text = self.helper:getLangCfg(v.cfg.des_id)
            count.text = NativeText.GetLangStr(280301)..": "..v.count

            use_btn.onClick:RemoveAllListeners()
            use_btn.onClick:AddListener(
                function()
                    network_sender:useExpItem(v.inst_id, 1, hero_id)
                end
            )

            mutle_btn.onClick:RemoveAllListeners()
            mutle_btn.onClick:AddListener(
                function()
                    use_node:SetActive(true)
                    items_node:SetActive(false)
                    self:updateUseExpInfo(use_node, v, hero_id, items_node)
                end
            )
        end

        for i=#items, content.childCount - 1 do
            local elem = content:GetChild(i)
            elem:SetActive(false)
        end

        if self.use_exp_sldier and self.use_exp_sldier.is_show then
            items_node:SetActive(false)
            use_node:SetActive(true)
        else
            items_node:SetActive(true)
            use_node:SetActive(false)
        end

        local none_text = self.use_exp_panel:Find("BG/Scroll View/None")
        none_text:SetActive(#items == 0)
    end,

    updateUseExpInfo = function(self, node, item, hero_id, items_node)
        if not self.use_exp_sldier then
            self.use_exp_sldier = require("gui.officer_use_exp_panel")(node, item, hero_id, items_node)
        end

        self.use_exp_sldier:init()
    end,

    getExpItems = function(self)
        local items = bag_manager:getItemsByType(79)

        table.sort( items, function(a, b) return a.cfg_id < b.cfg_id end )

        return items
    end,

    click_CloseSkillDetails = function(self)
        self.skill_panel:SetActive(false)
    end,

    click_CloseUseExp = function(self)
        self.use_exp_panel:SetActive(false)
        self.use_exp_item = false

        if self.use_exp_sldier then
            self.use_exp_sldier:hide()
        end
    end,

    click_CloseGetWay = function(self)
        self.getway_panel:SetActive(false)
    end,

    setOfficerCard = function(self)
        local hero_id = self.active_heros[self.hero_idx].id
        local hero = hero_manager:getHero(hero_id)
        local officer = hero.officer

        self.spine_zoom = officer.cfg.SpineZoom*0.0001
        local pos = loadstring("return "..officer.cfg.SpinePosition)()

        if officer.cfg.commander_pic:find('.png') then
            if not self.is_view then
                self.officer_card.transform:SetActive(true)
                self.officer_spine:SetActive(false)
            end

            self.officer_card.transform.localPosition = Vector3(pos[1], pos[2], 0)
            self.officer_card:SetSprite(gui_helper:getOfficerPicturePath(officer))
        elseif officer.cfg.commander_pic:find('.asset') then
            if not self.is_view then
                self.officer_card.transform:SetActive(false)
                self.officer_spine:SetActive(true)
            end

            local path = officer.cfg.cmd_path..officer.cfg.commander_pic

            self.spine_finished = false

            if self.spine_assets[self.hero_idx] then
                app:startCoroutine(
                    function()
                        local sg = self.officer_spine:GetComponent(Spine.Unity.SkeletonGraphic)
                        coroutine.yield(UnityEngine.Object.Destroy(sg))

                        local asset = self.spine_assets[self.hero_idx]
                        local sda = asset:Get() -- skeleton data asset

                        if sda then
                            local go = self.officer_spine.gameObject
                            local skeleton_graphic = Spine.Unity.SkeletonGraphic.AddSkeletonGraphicComponent(go, sda)
                            skeleton_graphic.material = game_director.spine_material
                            local anim_state = skeleton_graphic.AnimationState

                            if anim_state then
                                anim_state:SetAnimation(0, 'idle', true)
                            else
                                log.error("Failed to find anim state:"..path)
                            end

                            
                            self.officer_spine.localScale = Vector3.one * self.spine_zoom
                            self.officer_spine.localPosition = Vector3(pos[1], pos[2], 0)
                            self.spine_finished = true
                        end
                    end
                )
            else
                self:removeLoadSpineQueue(self.hero_idx)

                app:startCoroutine(
                    function()
                        local sg = self.officer_spine:GetComponent(Spine.Unity.SkeletonGraphic)
                        coroutine.yield(UnityEngine.Object.Destroy(sg))
                        
                        coroutine.yield(Fenix.Assets.Helper.WaitForAsset(path, Spine.Unity.SkeletonDataAsset, function(asset) 
                            local sda = asset:Get() -- skeleton data asset
                            asset:AddRef()
                            self.spine_assets[self.hero_idx] = asset

                            if sda then

                                local go = self.officer_spine.gameObject
                                local skeleton_graphic = Spine.Unity.SkeletonGraphic.AddSkeletonGraphicComponent(go, sda)
                                skeleton_graphic.material = game_director.spine_material
                                local anim_state = skeleton_graphic.AnimationState

                                if anim_state then
                                    anim_state:SetAnimation(0, 'idle', true)
                                else
                                    log.error("Failed to find anim state:"..path)
                                end

                            else
                                log.error('failed to load spine:'..path)
                            end
                            
                            self.officer_spine.localScale = Vector3.one * self.spine_zoom
                            self.officer_spine.localPosition = Vector3(pos[1], pos[2], 0)
                            self.spine_finished = true
                        end))
                    end
                )
            end
        end
    end,
}