local log = logger.get("Game", "Officer Use Exp Panel")
local max_num = 100
local per_times = 1

return class {

    ctor = function (self, panel, item, hero_id, items_node)
        
        self.root = panel
        self.item = item
        self.item_id  = item.inst_id
        self.hero_id = hero_id
        self.items_node = items_node
        --self.callback = self.args.data.callback

        self.helper = require("gui.tool.frame_helper")

        self.get_num_text = self.root:Find('get_num_text'):GetComponent(UnityEngine.UI.Text)
        self.slider = self.root:Find('Slider'):GetComponent(UnityEngine.UI.Slider)
        
        local des = self.root:Find("ItemDes"):GetComponent(UnityEngine.UI.Text)
        des.text = self.helper:getLangCfg(self.item.cfg.des_id)
        
        local btn = self.root:Find("ConfirmBtn"):GetComponent(UnityEngine.UI.Button)
        btn.onClick:RemoveAllListeners()
        btn.onClick:AddListener(
            function()
                self:click_confirmBtn()
            end
        )

        btn = self.root:Find("CancelBtn"):GetComponent(UnityEngine.UI.Button)
        btn.onClick:RemoveAllListeners()
        btn.onClick:AddListener(
            function()
                self:click_cancelBtn()
            end
        )

        btn = self.root:Find("addBtn"):GetComponent(UnityEngine.UI.Button)
        btn.onClick:RemoveAllListeners()
        btn.onClick:AddListener(
            function()
                self:click_addBtn()
            end
        )

        btn = self.root:Find("delBtn"):GetComponent(UnityEngine.UI.Button)
        btn.onClick:RemoveAllListeners()
        btn.onClick:AddListener(
            function()
                self:click_delBtn()
            end
        )
    end,

    onUnload = function (self)
    end,

    click_confirmBtn = function(self)
        network_sender:useExpItem(self.item_id, self.select_num, self.hero_id)
    end,

    click_cancelBtn = function(self)
        self:hide()
    end,

    click_addBtn = function(self)
        self:removeListener()
        self.select_num = self.select_num + per_times
        self.select_num = self.select_num > max_num and max_num or self.select_num
        self.slider.value = self.select_num / max_num
        self.get_num_text.text = string.format(NativeText.GetLangStr(260007), self.select_num, max_num)
        self:addListener()
    end,

    click_delBtn = function(self)
        self:removeListener()
        self.select_num = self.select_num - per_times
        self.select_num = self.select_num < 1 and 1 or self.select_num
        self.slider.value = self.select_num / max_num
        self.get_num_text.text = string.format(NativeText.GetLangStr(260007), self.select_num, max_num)
        self:addListener()
    end,

    init = function(self)
        local cfg = self.item.cfg
        max_num = self.helper:getItemNum(self.item.cfg_id)

        self.select_num = 1
        self.slider.value = self.select_num / max_num
        self.get_num_text.text = string.format(NativeText.GetLangStr(260007), self.select_num, max_num)

        self:removeListener()
        self:addListener()

        self.is_show = true
    end,

    refreshMaxCnt = function(self)
        local cfg = self.item.cfg
        max_num = self.helper:getItemNum(self.item.cfg_id)

        if max_num == 0 then
            self:hide()
        end
        
        self:removeListener()

        if self.select_num > max_num then
            self.select_num = max_num
        end

        self.slider.value = self.select_num / max_num
        self.get_num_text.text = string.format(NativeText.GetLangStr(260007), self.select_num, max_num)
        
        self:addListener()
    end,

    addListener = function(self)
        self.slider.onValueChanged:AddListener(
            function()
                self:refreshSlider()
            end)
    end,

    removeListener = function(self)
        self.slider.onValueChanged:RemoveAllListeners()
    end,

    refreshSlider = function(self)
        local own_num = self.helper:getItemNum(self.item.cfg_id)
        self.select_num = math.ceil(self.slider.value * own_num)
        self.select_num = self.select_num < 1 and 1 or self.select_num
        self.get_num_text.text = string.format(NativeText.GetLangStr(260007), self.select_num, own_num)
    end,

    hide = function(self)
        self.root:SetActive(false)
        self.items_node:SetActive(true)
        self.is_show = false
    end,
}

