local super = require("ui.frame")
local log = logger.get("Game", "lord info frame")
require('config.crew')

local drop_down_state = {
    base = 1,
    all = 2,
}

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        self.SCGetPlayerInfo = self.args.data

        clickEvents = 
        {
            "closeBtn",
            "content/layout/tag (1)/statisticsBtn",
            "content/layout/tag (4)/guildBtn",
        }

        frame_helper:regClickEvents("click", self, clickEvents)

        self:init()
    end,

    onUnload = function (self)
        for i = 1,6 do 
            self:unloadSpine(i)
        end
    end,

    init = function(self)
        self:refreshLordInfo()
        self.crew_spines = {}           -- spine 组件 skeleton graphic
        self.spine_assets = {}          -- 每个槽位的 spine 资源
        self.crews = {}                 -- 所有船员
        self:refreshRooms()
        self:refreshGuildInfo()
    end,

    -- 刷新工会信息
    refreshGuildInfo = function(self)
        local in_guild = tonumber(self.SCGetPlayerInfo.familyId) > 0
        local icon = self:findImage("content/info/guild_icon_2_2")
        icon.enabled = in_guild
        local degree = self.SCGetPlayerInfo.familyPost
        if in_guild then
            icon:SetSprite(gui_helper:getGuildRankIcon(degree))
        end
    end,

    -- 刷新领主信息
    refreshLordInfo = function(self)
        self:setText('content/vip',     self.SCGetPlayerInfo.vipLv)
        self:setText('content/name',    self.SCGetPlayerInfo.playerName)
        self:setText('content/power',   self.SCGetPlayerInfo.strength)
        self:setText('content/kills',   self.SCGetPlayerInfo.killNum)
        -- 等级
        self:setText('content/exp/text1_Static[T5]_2', self.SCGetPlayerInfo.lordLv)
    end,

    refreshRooms = function(self)
        local setRoomStars = function(room, stars)
            local stars_root = room:Find("Stars")
            for i=1,5 do
                local star = stars_root:GetChild(i-1)
                if i <= stars then
                    star:SetActive(true)
                else
                    star:SetActive(false)
                end
            end
        end

        -- 6 个房间
        self.slots = {}
        -- for i=1,6 do
        for i, cfg_id in ipairs(self.SCGetPlayerInfo.professionId) do
            local slot = self.transform:Find("content/equip/Slot_"..i)
            self.slots[i] = slot

            -- 房间名
            local room_cfg = config.crew_type_indexs[i]
            slot:Find('Title/Name'):GetComponent(UnityEngine.UI.Text).text = 
                                    frame_helper:getLangCfg(room_cfg.CrewRoomName)


            local cfg = config.crew_blocks[cfg_id]
            if cfg then
                -- 装备了船员
                -- 设置品质
                setRoomStars(slot, cfg.color)

                -- 隐藏加号
                slot:Find('img_add'):SetActive(false)

                local spine_path = "Assets/Arts/spines/npc/npc_new_SkeletonData.asset"
               -- local spine_path = "Assets/Arts/spines/npc/skeleton_SkeletonData.asset"
                -- 加载船员的 spine
                local callback = function(skeleton_graphic)
                    local anim_state = skeleton_graphic.AnimationState
                    if anim_state then
                        anim_state:SetAnimation(0, 'idle', true)
                    else
                        log.error("Failed to find anim state:"..spine_path)
                    end   
                    skeleton_graphic:SetSkin(cfg.SpineSkin)
                    --initialSkinName.initialSkinName = crew.cfg.SpineSkin
                    self.crews[i] = {
                        trans = skeleton_graphic.transform, 
                        anim_state = anim_state, 
                        anim = 'idle', 
                        facing = 1,
                        elapsed = 0.0,
                        anim_time = -1.0,
                    }
                end

                local crew_go = slot:Find('Room/crew').gameObject
                self:loadSpine(spine_path, crew_go, i, callback)
            else
                -- 清除品质
                setRoomStars(slot, 0)

                slot:Find('img_add'):SetActive(false)

                -- 显示加号
                -- if crew_manager:canEquipCrew(i) then
                --     slot:Find('img_add'):SetActive(true)                    
                -- else
                --     slot:Find('img_add'):SetActive(false)
                -- end

                -- 卸载船员 spine
                self:unloadSpine(i)
            end
        end        
    end,

    -- 加载 spine
    loadSpine = function(self, path, crew_go, slot_idx, callback)
        if self.crew_spines[slot_idx] then
            callback(self.crew_spines[slot_idx])
        else
            app:startCoroutine(
                function()
                    coroutine.yield(Fenix.Assets.Helper.WaitForAsset(path, Spine.Unity.SkeletonDataAsset, function(asset)
                        -- 释放原来的资源
                        if asset then
                            self.spine_assets[slot_idx] = asset  
                            asset:AddRef()                             
                            local skeleton_data = asset:Get() -- skeleton data asset
                            local skeleton_graphic = Spine.Unity.SkeletonGraphic.AddSkeletonGraphicComponent(crew_go, skeleton_data)
                            skeleton_graphic.material = game_director.spine_material
                            self.crew_spines[slot_idx] = skeleton_graphic

                            callback(skeleton_graphic)
                        else
                            log.error("Failed to load spine asset:"..path)
                        end
                    end))
                end
            )
        end
    end,

    -- 释放 spine
    unloadSpine = function(self, slot)
        if self.crew_spines[slot] then
            Object.DestroyImmediate(self.crew_spines[slot])
            self.crew_spines[slot] = nil
        end
        if self.spine_assets[slot] then
            self.spine_assets[slot]:RemoveRef()
            self.spine_assets[slot] = nil
        end
    end,

    --[[
    self.crews[i] = {
        trans = skeleton_graphic.transform, 
        transform = s
        anim_state = anim_state, 
        anim = 'idle', 
        facing = 1,
        elapsed = 0.0,
    }]]
    -- 逻辑更新
    update = function(self, dt)
        if not self.crews then
            return
        end

        local randomBehaviour = function(crew)
            local r = math.random()
            local anim_time = 0
            local anim = 'idle'
            if r < 0.3 then
                anim = 'idle'
                anim_time = math.random()*2 + 1
            elseif r < 0.667 then
                anim = 'run'
                anim_time = 4.0 + math.random() * 2
            else
                anim = 'attack'
                anim_time = 1.0 + math.random()
            end

            if crew.anim ~= anim then
                crew.anim_state:SetAnimation(0, anim, true)
                crew.anim = anim
            end
            crew.elapsed = 0.0                
            crew.anim_time = anim_time
        end

        local move = function(crew, dt)

            if crew.anim == 'run' then
                local pos = crew.trans.localPosition
                pos = pos + Vector3(30, 0, 0) * crew.facing  * dt
                crew.trans.localPosition = pos

                if pos.x > 100 then
                    crew.facing = -1
                    crew.trans.localRotation = Quaternion.AngleAxis(180, Vector3.up)
                elseif pos.x < -100 then
                    crew.facing = 1
                    crew.trans.localRotation = Quaternion.identity
                end
            end
        end

        for k,crew in pairs(self.crews) do
            crew.elapsed = crew.elapsed + dt

            if crew.elapsed > crew.anim_time then
                randomBehaviour(crew)
            end
            move(crew, dt)
        end
    end,

    click_closeBtn = function(self)
        frame_manager:hide('other_lord_info')
    end,

    -- 统计
    click_statisticsBtn = function(self)
        network_sender:getPlayerPvpData(self.SCGetPlayerInfo.playerId)
    end,

    -- 
    click_guildBtn = function(self)    
        -- network_sender:getFamilyList(self.SCGetPlayerInfo.familyId)
        network_sender:getFamilyInfo(self.SCGetPlayerInfo.familyId)
    end,
}
