local super = require("ui.frame")
local min_pos = 1
local max_pos = 16

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        self.recurit_id = self.args.data

        clickEvents = 
        {
            "closeBtn",
            "content/btns/emptyBtn",
            "content/btns/quickBtn",
            "content/btns/editBtn",
            "content/btns/confirmBtn",
            "content/bg/formation_icon",
        }

        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self.formation_name_text = self:findText("content/bg/formation_name_text")
        self.formation_power_text = self:findText("content/bg/power_text")
        self.formation_icon = self:findImage("content/bg/formation_icon")

        self:init()
    end,

    onUnload = function (self)
        level_formation.events:off('preset_formation_change', self.preset_formation_change_id)
    end,

    click_quickBtn = function(self)
        local hero_list = self:getHeroList()
        frame_manager:show('formation_quick', nil, 
            { 
                team_max = nil, 
                hero_list = hero_list, 
                callback = function(MFormation_list) 
                    self:quickCallback(MFormation_list) 
                end })
    end,

    click_confirmBtn = function(self)
        local hero_list = self.select_hero_list
        local success = false
        for __, hero in ipairs(hero_list) do
            if hero.location > 0 then
                success = true
            end
        end
        if success then
            network_sender:startRecuritFight(self.recurit_id, self.select_formation_id, self.select_hero_list)
            frame_manager:hide('recruit_formation')

        else
            hint:setText(NativeText.GetLangStr(240020))
        end
    end,

    click_closeBtn = function(self)
        frame_manager:hide('recruit_formation')
    end,

    click_formation_icon = function(self)
        frame_manager:show('formation_select', self, 
            { 
                callback = function(formation_id) 
                    self:click_selectFormation(formation_id) 
                end })
    end,

    click_selectFormation = function(self, formation_id)
        print('选择阵型清空英雄列表:', formation_id)
        local MFormation = level_formation:spawnEmptyServerData(formation_id, self:getHeroList())
        self:showContent(MFormation.formationId, MFormation.formationHero)
    end,

    click_emptyBtn = function(self)
        if self.select_formation_id and self.select_hero_list then
            local MFormation = level_formation:completeFormation(self.select_formation_id, self.select_hero_list)
            self:showContent(MFormation.formationId, MFormation.formationHero)
        end
    end,

    click_editBtn = function(self, param)
        local own_formations = {}
        -- local own_ids = {}
        local preset_formations = level_formation:getPresetFormations()
        for id, MFormation in pairs(preset_formations) do
            MFormation.teamId = id
            table.insert(own_formations, MFormation)
            -- table.insert(own_ids, id)
        end

        if param ~= true then
            param = false
        end

        local spare_formation = nil
        if self.select_formation_id and self.select_hero_list and (not param) then
            local mf = common_pb.MFormation()
            mf.formationId = self.select_formation_id
            for __, hero in ipairs(self.select_hero_list) do
                local item = mf.formationHero:add()
                item.heroId = hero.heroId
                item.location = hero.location
            end
            spare_formation = mf
        end

        local data = {
            -- own_ids = own_ids,
            own_formations = own_formations,
            spare_formation = spare_formation,
            callback = function(MFormation)
                self:switchCurrFormation(MFormation)
            end
        }

        local is_display, frame = frame_manager:isDisplay('formation_preset')
        if is_display then
            frame:refreshData(data)
        else
            frame_manager:show('formation_preset', nil, data)    
        end
    end,

    switchCurrFormation = function(self, MFormation)
        self:showContent(MFormation.formationId, MFormation.formationHero)
    end,

    init = function(self)
        local parent_path = "content/grid_heros"
        self.cache_heros = {}
        for i = min_pos, max_pos do
            local child = self.transform:Find(parent_path .. "/Image (" .. i .. ")")
            self.cache_heros[i] = child
        end
        
        self:refreshHeros()

        self.preset_formation_change_id = level_formation.events:on('preset_formation_change',
            function()
                self:click_editBtn()
            end)
    end,

    quickCallback = function(self, MFormation_list)
        frame_manager:hide('formation_quick')
        local MFormation = MFormation_list[1]
        if MFormation then
            self:showContent(MFormation.formationId, MFormation.formationHero)
        end
    end,

    getHeroList = function(self, slot_type)
        local hero_list = {}
        local active_heros = self:getActiveHeros()
        if ( not slot_type ) or slot_type == 0 then
            for __, hero in pairs(active_heros) do
                table.insert(hero_list, hero_manager:getHero(hero.id))
            end
            return hero_list
        else
            for __, hero in pairs(active_heros) do
                local cfg = hero.tank_cfg
                if cfg.type == ( "type" .. slot_type ) then
                    table.insert(hero_list, hero_manager:getHero(hero.id))
                end
            end
            return hero_list
        end
    end,

    getActiveHeros = function(self)
        return hero_manager:getUnDispatchedHeros()
        --[[
        local rst = {}
        for __, hero_lua in pairs(hero_manager.heros) do
            if hero_lua.is_actived then
                table.insert(rst, hero_lua)
            end
        end
        return rst
        ]]
    end,

    refreshHeros = function(self)
        local msg = protocol_pb.CSLevelEmbattle()
        msg.formationId = 1
        local active_heros = self:getActiveHeros()
        for __, hero_lua in pairs(active_heros) do
            if hero_lua.is_actived then
                local formation = msg.formations:add()
                formation.location = 0
                formation.heroId = hero_lua.id
                formation.officerId = hero_lua.officer.cfg_id
            end
        end
        local fmt_cfg_id = msg.formationId
        local hero_list = msg.formations
        self:showContent(fmt_cfg_id, hero_list)
    end,

    -- hero_list = repeated MHeroFormation
    showContent = function(self, formation_id, hero_list)
        print('recruit show content:', formation_id, #hero_list)

        self.select_formation_id = formation_id
        self.select_hero_list = hero_list

        local findHero = function(list, location)
            local target = nil
            for __, tank in ipairs(list) do
                if tank.location == location then
                    target = tank
                    break
                end
            end
            return target
        end

        local getBgPath = function(id)
            local target = nil
            config.formation_frames:foreach(
                function(k,v)
                    if v.slot == id then
                        target = v
                    end
                end)
            if target then
                return target.framepath .. target.frameicon
            else
                return nil
            end
        end

        local cfg = level_formation:getFormationCfg(formation_id)
        for slot_pos, child in pairs(self.cache_heros) do
            local slot_type = cfg[slot_pos]
            child.gameObject:SetActive(cfg[slot_pos] and true or false)
            if cfg[slot_pos] then
                local hero = findHero(hero_list, slot_pos)
                local out_bg_trans = child:Find('out_bg')
                out_bg_trans.gameObject:SetActive(hero and true or false)
                if hero then
                    local icon = child.transform:Find('out_bg/icon'):GetComponent(UnityEngine.UI.Image)
                    local quality = child.transform:Find('out_bg'):GetComponent(UnityEngine.UI.Image)
                    local color = hero_manager:getHero(hero.heroId).officer.cfg.color
                    icon:SetSprite(gui_helper:getOfficerIconFromId(hero.heroId))
                    quality:SetSprite(gui_helper:getQualityIconPath(color))
                end
                local bg = child:GetComponent(UnityEngine.UI.Image)
                local bg_sprite_name = getBgPath(cfg[slot_pos])
                if bg_sprite_name then
                    bg:SetSprite(bg_sprite_name)
                end

                local button = child:GetComponent(UnityEngine.UI.Button)
                button.onClick:RemoveAllListeners()
                button.onClick:AddListener(
                    function ()
                        print("点击槽位", slot_pos, slot_type, self:getHeroList(slot_type) and #self:getHeroList(slot_type))

                        frame_manager:show('formation_hero_list', nil, { 
                            slot_hero_id = hero and hero.heroId or nil,
                            hero_list = { self:getHeroList(slot_type) }, 
                            hide_team = true,
                            callback = function(hero) 
                                self:selectHeroCallback(hero, slot_pos) 
                            end })
                    end
                )
            end
        end

        local cfg = config.formation_lists[formation_id]
        self.formation_name_text.text = self.helper:getLangCfg(cfg.shortname_id)
        self.formation_icon:SetSprite(cfg.iconpath .. cfg.icon)

        local mf = common_pb.MFormation()
        mf.formationId = formation_id, hero_list
        for __, hero in ipairs(hero_list) do
            local item = mf.formationHero:add()
            item.heroId = hero.heroId
            item.location = hero.location
        end

        level_formation:getForce(mf, nil, 
            function(SCQueryTeamSword)
                self.formation_power_text.text = SCQueryTeamSword.baseSword + SCQueryTeamSword.extraSword
            end)
    end,

    selectHeroCallback = function(self, hero_inst, slot_pos)
        local list = self.select_hero_list

        function getHeroByLocation(list, location)
            for __, hero in ipairs(list) do
                if hero.location == location then
                    return hero
                end
            end
        end

        function getHeroById(list, id)
            for __, hero in ipairs(list) do
                if hero.heroId == id then
                    return hero
                end
            end
        end

        local curr_hero = getHeroByLocation(list, slot_pos)
        local will_hero = getHeroById(list, hero_inst.id)
        
        if not curr_hero and not will_hero then
            local mf_hero = common_pb.MHeroFormation()
            mf_hero.location = slot_pos
            mf_hero.heroId = hero_inst.id
            table.insert( list, mf_hero )
        elseif curr_hero and will_hero then -- 当前槽位有英雄
            if curr_hero == will_hero then  -- 卸下英雄
                will_hero.location = 0
            else
                local temp = will_hero.location
                will_hero.location = curr_hero.location
                curr_hero.location = temp
            end
        elseif not curr_hero and will_hero then -- 当前槽位没有英雄
            will_hero.location = slot_pos
        elseif curr_hero and not will_hero then
            local mf_hero = common_pb.MHeroFormation()
            mf_hero.location = slot_pos
            mf_hero.heroId = hero_inst.id
            table.insert( list, mf_hero )
            curr_hero.location = 0
        end

        local mf = common_pb.MFormation()
        mf.formationId = self.select_formation_id
        for __, h in ipairs(list) do
            local item = mf.formationHero:add()
            item.heroId = h.heroId
            item.location = h.location
        end

        local MFormation = mf
        self:showContent(MFormation.formationId, MFormation.formationHero)
        self.curr_mf = MFormation

        formation_tool:validMFormation(MFormation)
    end,
}
