local super = require("ui.frame")
local log = logger.get("Game", "Report Battle Frame")
require("RTS.replay_stage.battle_replay_helper")

return class {
    super = super,
 
    onLoaded = function(self)
        log.info('Report Battle Frame onLoaded!!')
        super.onLoaded(self)
        -- 关闭 loading
        frame_manager:hide('loading')
        self.helper = require("gui.tool.frame_helper")

        self:init()
    end,

    init = function(self)
        self.img_turn_progress = self.transform:Find("Top/BG/TurnProgress"):GetComponent(SmoothFillAmount)--UnityEngine.UI.Image
        self.text_count_down = self.transform:Find("Top/BG/TimeCountDown"):GetComponent(UnityEngine.UI.Text)
        self.is_count_down_display = false

        self.text_name_left = self.transform:Find("Top/BG/NameLeft"):GetComponent(UnityEngine.UI.Text)
        self.img_soldier_left = self.transform:Find("Top/BG/SoldierProgressLeft"):GetComponent(SmoothFillAmount)
        self.text_soldier_left = self.transform:Find("Top/BG/SoldierProgressLeft/SoldierAmount/Num"):GetComponent(UnityEngine.UI.Text)
        self.img_icon_left = self.transform:Find("Top/BG/IconLeft/Icon"):GetComponent(UnityEngine.UI.Image)

        self.text_name_right = self.transform:Find("Top/BG/NameRight"):GetComponent(UnityEngine.UI.Text)
        self.img_soldier_right = self.transform:Find("Top/BG/SoldierProgressRight"):GetComponent(SmoothFillAmount)
        self.text_soldier_right = self.transform:Find("Top/BG/SoldierProgressRight/SoldierAmount/Num"):GetComponent(UnityEngine.UI.Text)
        self.img_icon_right = self.transform:Find("Top/BG/IconRight/Icon"):GetComponent(UnityEngine.UI.Image)

        self.is_left_alarm = false
        self.is_right_alarm = false

        -- 结算界面
        self.settlement_frame = self.transform:Find("SettlementFrame")
        self.settlement_frame:SetActive(false)


        clickEvents = {
            "Top Left/ButtonPause",
            "SettlementFrame",
        }

        -- 注册点击事件
        self.helper:regClickEvents("click", self, clickEvents)

        ----填充界面----
        self.battle_report = battle_helper.stage.battle_report
        -- 双方名称
        self.text_name_left.text = self.battle_report.name0
        self.text_name_right.text = self.battle_report.name1
        -- 士兵数量
        if self.owner:isShipBattle() then
            local num0 = self.owner:getSoldierTotalNums()
            local num1 = self.owner:getDefenceTotalNums()
            self.text_soldier_left.text = tostring(num0)
            self.text_soldier_right.text = tostring(num1)
        else
            local num0, num1 = self.owner:getSoldierTotalNums()
            self.text_soldier_left.text = tostring(num0)
            self.text_soldier_right.text = tostring(num1)
        end

        -- icon
        self.img_icon_left:SetSprite("Assets/Arts/ui/atlas/icon/battle_icon/battle_icon_15.png")
        self.img_icon_right:SetSprite("Assets/Arts/ui/atlas/icon/battle_icon/battle_icon_21.png")

        -- 回合更新
        self.owner.events:on("update_turn", function(curr_turn, total_turn, soldier_num0, soldier_num1, percent0, percent1, turn_elapsed, count_down)
            self.img_turn_progress:make((total_turn - curr_turn) / total_turn, turn_elapsed)
            self.text_soldier_left.text = tostring(math.round(soldier_num0))
            self.text_soldier_right.text = tostring(math.round(soldier_num1))

            self.img_soldier_left:make(percent0, turn_elapsed)
            self.img_soldier_right:make(percent1, turn_elapsed)

            if percent0 < 0.4 and not self.is_left_alarm then
                self.is_left_alarm = true
                self.img_soldier_left:BeginAlarm()
            elseif percent0 >=0.4 and self.is_left_alarm then
                self.is_left_alarm = false
                self.img_soldier_left:EndAlarm()
            end

            if percent1 < 0.4 and not self.is_right_alarm then
                self.is_right_alarm = true
                self.img_soldier_right:BeginAlarm()
            elseif percent1 >=0.4 and self.is_right_alarm then
                self.is_right_alarm = false
                self.img_soldier_right:EndAlarm()
            end

            if count_down then
                if not self.is_count_down_display then
                    self.text_count_down.gameObject:SetActive(true)
                    self.is_count_down_display = true
                end

                if self.text_count_down.text ~= tostring(count_down) then
                    self.text_count_down.text = tostring(count_down)    -- 中间的倒计时
                    LeanTween.cancel(self.text_count_down.gameObject)    
                    self.text_count_down.transform.localScale = Vector3.one
                    self.text_count_down.color = Color(0.9,0,0,1)

                    if curr_turn < total_turn-1 then
                        log.info("## curr_turn:"..curr_turn..", total_turn:"..total_turn)
                        local lt = LeanTween.scale(self.text_count_down.gameObject, Vector3.one * 1.5, 0.3)
                        lt.from = Vector3.one
                        lt.loopType = LeanTweenType.once
                        lt.delay = 0.7

                        lt = LeanTween.textColor(self.text_count_down.transform, Color(0.9,0,0,0.1), 0.3)
                        lt.from = Color(0.9,0,0,1)
                        lt.loopType = LeanTweenType.once
                        lt.delay = 0.7
                    end
                end

            end
        end)

        -- 结算
        self.owner.events:on("battle_finished", function(win_or_lose, lost_soldier_0, lost_soldier_1)
            app:startCoroutine(
                function()
                    -- 播放 timeout
                    if self.battle_report.timeOut then
                        local trans_overtime = self.transform:Find('overtime')
                        trans_overtime:SetActive(true)
                        trans_overtime:GetComponent(UnityEngine.Animation):Play('overtime_in')
                        coroutine.yield(WaitForSeconds(1.2))
                        trans_overtime:SetActive(false)
                    end

                    self.settlement_frame:SetActive(true)

                    local icon_left = self.transform:Find("SettlementFrame/IconLeft/Icon"):GetComponent(UnityEngine.UI.Image)
                    local icon_right = self.transform:Find("SettlementFrame/IconRight/Icon"):GetComponent(UnityEngine.UI.Image)

                    local name_left = self.transform:Find("SettlementFrame/NameLeft"):GetComponent(UnityEngine.UI.Text)
                    local name_right = self.transform:Find("SettlementFrame/NameRight"):GetComponent(UnityEngine.UI.Text)

                    local lost_left = self.transform:Find("SettlementFrame/LostNumLeft"):GetComponent(NumberEffect)
                    local lost_right = self.transform:Find("SettlementFrame/LostNumRight"):GetComponent(NumberEffect)

                    local win = self.transform:Find("SettlementFrame/IconLeft/Win")
                    local lose = self.transform:Find("SettlementFrame/IconLeft/Lose")

                    app:startCoroutine(
                        function()

                            coroutine.yield(WaitForSeconds(1.0))
                            lost_left:make(math.round(lost_soldier_0))
                            lost_right:make(math.round(lost_soldier_1))
                        end
                    )


                    if win_or_lose then
                        self.settlement_frame:GetComponent(UnityEngine.Animation):Play('win')
                        --win:SetActive(true)
                        --lose:SetActive(false)
                    else
                        self.settlement_frame:GetComponent(UnityEngine.Animation):Play('lose')
                    -- win:SetActive(false)
                        --lose:SetActive(true)
                    end

                    -- icon
                    icon_left:SetSprite("Assets/Arts/ui/atlas/icon/battle_icon/battle_icon_15.png")
                    icon_right:SetSprite("Assets/Arts/ui/atlas/icon/battle_icon/battle_icon_21.png")

                    -- name
                    name_left.text = self.battle_report.name0
                    name_right.text = self.battle_report.name1

                    coroutine.yield(WaitForSeconds(2.0))
                    self.all_finished = true
                end
            )

        end)
    end,

    click_ButtonPause = function(self)
        game_state_battle_replay:backToWorldMap()
        --game_director:changeState('loading', {next_state = 'home', scene_name = 'home'})
    end,

    click_SettlementFrame = function(self)
        if self.all_finished then
            game_state_battle_replay:backToWorldMap()
        end
    end,
}

