local super = require("ui.frame")
local log = logger.get("Game", "Guild Logo")
local bar_num = 4
local diamond_id = 30001

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)
        
        self.need_item_id = self.args.data.item_id
        self.need_item_num = self.args.data.item_num or 0
        self.army_info = self.args.data.army_info

        clickEvents = 
        {
            "content/backBtn",
        }

        -- 注册点击事件
        frame_helper:regClickEvents("click", self, clickEvents)

        self.mapping_type = {
            -- 资源
            [32001] = 38, 
            [30002] = 39, 
            [32002] = 40, 
            [32003] = 41, 
            [32004] = 76,

            [32008] = 92,

            -- 领主经验
            [local_lord.EXP_ID] = 83,

            -- 领主体力
            [50001] = 72,

            -- 行军加速
            [army_queue_manager.normal_speed_up] = 48,
            -- 运输加速
            [army_queue_manager.transport_speed_up] = 49,
            -- 集结加速
            [army_queue_manager.mass_speed_up] = 50,
        }

        self.onPropChange = local_player.events:on("onPropChange", functor.make(self.initTop, self))
        self.onBagChanged = bag_manager.events:on('onBagChanged', 
            function()
                self:initTop()
                self:showScorllView()
            end)

        self.LORD_INFO_CHANGE = local_lord.events:on('LORD_INFO_CHANGE', 
            function()
                self:initTop()
                self:showScorllView()
            end)

        self.speed_up_evt = army_queue_manager.events:on('WORLDMAPSPEEDUP', 
            function(info)
                self.army_info = info
                self:initTop()
                self:showScorllView()
            end)

        self.mass_notify = game_director.events:on('WORLDMAPMASSNOTIFY', 
            function(msg)
                if self.army_info then
                    for k, v in ipairs(msg.massInfo.formationDetails) do
                        if v.armylineId == self.army_info.id then
                            self.army_info = {lt = v.lefttime, tt = v.totaltime, id = v.armylineId}
                            self:initTop()
                            self:showScorllView()
                        end
                    end
                end
            end)

        self:init()
    end,

    onUnload = function (self)
        local_player.events:off("onPropChange", self.onPropChange)
        bag_manager.events:off("onBagChanged", self.onBagChanged)
        local_lord.events:off("LORD_INFO_CHANGE", self.LORD_INFO_CHANGE)
        army_queue_manager.events:off("WORLDMAPSPEEDUP", self.speed_up_evt)
        game_director.events:off("WORLDMAPMASSNOTIFY", self.mass_notify)
    end,

    update = function(self, dt)
        if self.transform and self:isSpeedUp() then
            --local lt = army_queue_manager.speed_up_army.lt
            local lt = self.army_info.lt
            local tt = self.army_info.tt

            if lt > 0 then
                local name = self:findText("content/speedUpTop/name")
                local slider = self:findSlider("content/speedUpTop/slider")

                name.text = string.format("%s %s", NativeText.GetLangStr(9000009), gui_helper:timeFormat(lt))
                slider.value = 1 - lt / tt

                lt = lt - dt
                self.army_info.lt = lt
            else
                --army_queue_manager.speed_up_army = nil
                self:click_backBtn()
            end
        end
    end,

    click_backBtn = function(self)
        frame_manager:hide('resource_exchange')
    end,

    init = function(self)
        self:initTop()
        self:showScorllView()
    end,

    initTop = function(self)
        local resource_top_trans = self.transform:Find("content/resource_top")
        local exp_top_trans = self.transform:Find("content/exp top")
        local energy_top_trans = self.transform:Find("content/energy top")
        local speedup_top_trans = self.transform:Find("content/speedUpTop")
        frame_helper.hideTrans(resource_top_trans)
        frame_helper.hideTrans(exp_top_trans)
        frame_helper.hideTrans(energy_top_trans)
        frame_helper.hideTrans(speedup_top_trans)

        local title = self:findText("content/title")

        if  self.need_item_id == 32001 or 
            self.need_item_id == 30002 or 
            self.need_item_id == 32002 or 
            self.need_item_id == 32003 or 
            self.need_item_id == 32004 or
            self.need_item_id == 32008 then
                
            title.text = NativeText.GetLangStr(250001)

            if self.need_item_num > 0 then
                frame_helper.showTrans(resource_top_trans)
                local icon1 =  self:findImage("content/resource_top/icon1")
                local icon2 =  self:findImage("content/resource_top/icon2")
                local own = self:findText("content/resource_top/own_text")
                local need = self:findText("content/resource_top/need_text")
                local icon_name = frame_helper:getItemIcon(self.need_item_id)
                local own_num = frame_helper:getItemNum(self.need_item_id)
                icon1:SetSprite(icon_name)
                icon2:SetSprite(icon_name)
                if own_num >= self.need_item_num then
                    own.text = string.format(NativeText.GetLangStr(250010), "<color=#ffffffff>" .. frame_helper:itemNumFormat(own_num) .. "</color>")
                else
                    own.text = string.format(NativeText.GetLangStr(250010), "<color=#ff0000ff>" .. frame_helper:itemNumFormat(own_num) .. "</color>")
                end
                need.text = string.format(NativeText.GetLangStr(250011), frame_helper:itemNumFormat(self.need_item_num))
            end
        elseif self.need_item_id == local_lord.EXP_ID then
            frame_helper.showTrans(exp_top_trans)
            title.text = NativeText.GetLangStr(250012)

            local name = self:findText("content/exp top/name")
            local exp  = self:findText("content/exp top/exp")
            local slider = self:findSlider("content/exp top/slider")

            name.text = string.format(NativeText.GetLangStr(250013), local_lord.level)
            exp.text = string.format(NativeText.GetLangStr(250014), local_lord.exp .. "/" .. local_lord.max_exp)
            slider.value = local_lord.exp / local_lord.max_exp

        elseif self.need_item_id == 50001 then
            frame_helper.showTrans(energy_top_trans)
            title.text = NativeText.GetLangStr(250015)
            local value = self:findText("content/energy top/value")
            value.text = local_lord.energy .. "/" .. local_lord.max_energy
        elseif self:isSpeedUp() then
            frame_helper.showTrans(speedup_top_trans)
            title.text = NativeText.GetLangStr(9000010)

            local name = self:findText("content/speedUpTop/name")
            local slider = self:findSlider("content/speedUpTop/slider")

            name.text = string.format("%s %s", NativeText.GetLangStr(9000009), gui_helper:timeFormat(self.army_info.lt))
            slider.value = self.army_info.lt / self.army_info.tt
        end
    end,

    showResScorllView = function(self)
        local height = 755.3
        if self.need_item_num <= 0 then
            height = 923
        end
        local have_trans = self.transform:Find("content/own/Scroll View")
        have_trans.sizeDelta = Vector2(have_trans.sizeDelta.x, height)

        local none_trans = self.transform:Find("content/none")
        frame_helper.hideTrans(none_trans)

        local type_id = self.mapping_type[self.need_item_id]
        local cfgs = item_helper:getItemCfgByType(type_id)
        table.sort(cfgs, frame_helper.sortItem)
        local parent_path = "content/own/Scroll View/Viewport/Content"
        frame_helper:hideAllChild(self, parent_path, true)

        for idx, cfg in ipairs(cfgs) do
            local child = frame_helper:getChild(self, idx, parent_path)
            child:SetActive(true)

            local icon = child:Find("icon"):GetComponent(UnityEngine.UI.Image)
            local quality = child:Find("out_bg"):GetComponent(UnityEngine.UI.Image)
            local name_text = child:Find('name_text'):GetComponent(UnityEngine.UI.Text)
            local des_text = child:Find('des_text'):GetComponent(UnityEngine.UI.Text)
            local own_text = child:Find('own_text'):GetComponent(UnityEngine.UI.Text)

            local use_trans = child:Find('useTrans')
            local buy_trans = child:Find('buyTrans')

            local own_num = frame_helper:getItemNum(cfg.id)

            icon:SetSprite(frame_helper:getItemIcon(cfg.id))
            quality:SetSprite(frame_helper:getItemQualityIcon(cfg.id))
            name_text.text = frame_helper:getLangCfg(cfg.name_id)
            des_text.text = frame_helper:getLangCfg(cfg.des_id)
            own_text.text = string.format(NativeText.GetLangStr(250010), "<color=#ffffffff>" .. own_num .. "</color>") 

            use_trans:SetActive(own_num > 0)
            buy_trans:SetActive(own_num <= 0)

            if own_num > 0 then
                local btn = child:Find('useTrans/useBtn'):GetComponent(UnityEngine.UI.Button)
                btn.onClick:RemoveAllListeners()
                btn.onClick:AddListener(
                    function ()
                        network_sender:useItem(cfg.id, 1)
                    end
                )

                local btn = child:Find('useTrans/batBtn'):GetComponent(UnityEngine.UI.Button)
                btn.onClick:RemoveAllListeners()
                btn.onClick:AddListener(
                    function ()
                        local select_num, max_num = self:getUseSliderData(cfg.id)
                        frame_manager:show('use_slider', nil, {
                            item_id = cfg.id,
                            select_num = select_num,
                            max_num = max_num,
                            callback = function(item_id, item_num)
                                network_sender:useItem(item_id, item_num)
                            end,
                        })
                    end
                )
            else
                local buy_cost = frame_helper:getDiamondPriceFromStr(cfg.allprice)

                local DIAMOND    = 30001 -- 钻石
                local image_text = child:Find('buyTrans/image_text'):GetComponent(UnityEngine.UI.Text)
                local item_name = gui_helper:getImageTextItemStr(DIAMOND)
                image_text:SetImageText(string.format("%s %s", item_name, buy_cost))

                local btn = child:Find('buyTrans/buyBtn'):GetComponent(UnityEngine.UI.Button)
                btn.onClick:RemoveAllListeners()
                btn.onClick:AddListener(
                    function ()
                        local select_num, max_num = self:getBuySliderData(cfg.id)
                        frame_manager:show('buy_slider', nil, {
                            item_id = cfg.id,
                            select_num = select_num,
                            max_num = max_num,
                            callback = function(item_id, item_num)
                                network_sender:buyItem(
                                {
                                    {
                                        item_id = item_id,
                                        count = item_num,
                                    }
                                }, 1)
                            end,
                        })
                    end
                )
            end
        end
    end,

    showExpScorllView = function(self)
        local type_id = self.mapping_type[self.need_item_id]
        local cfgs = item_helper:getItemCfgByType(type_id)
        table.sort(cfgs, frame_helper.sortItem)
        local parent_path = "content/own/Scroll View/Viewport/Content"
        frame_helper:hideAllChild(self, parent_path, true)

        local temp_idx = 0
        for idx, cfg in ipairs(cfgs) do
            print("经验:", cfg.id)
            local own_num = frame_helper:getItemNum(cfg.id)
            if own_num > 0 then
                temp_idx = temp_idx + 1
                local child = frame_helper:getChild(self, temp_idx, parent_path)
                child:SetActive(true)

                local icon = child:Find("icon"):GetComponent(UnityEngine.UI.Image)
                local quality = child:Find("out_bg"):GetComponent(UnityEngine.UI.Image)
                local name_text = child:Find('name_text'):GetComponent(UnityEngine.UI.Text)
                local des_text = child:Find('des_text'):GetComponent(UnityEngine.UI.Text)
                local own_text = child:Find('own_text'):GetComponent(UnityEngine.UI.Text)

                local use_trans = child:Find('useTrans')
                local buy_trans = child:Find('buyTrans')

                icon:SetSprite(frame_helper:getItemIcon(cfg.id))
                quality:SetSprite(frame_helper:getItemQualityIcon(cfg.id))
                name_text.text = frame_helper:getLangCfg(cfg.name_id)
                des_text.text = frame_helper:getLangCfg(cfg.des_id)
                own_text.text = string.format(NativeText.GetLangStr(250010), "<color=#ffffffff>" .. own_num .. "</color>") 

                use_trans:SetActive(own_num > 0)
                buy_trans:SetActive(own_num <= 0)

                if own_num > 0 then
                    local btn = child:Find('useTrans/useBtn'):GetComponent(UnityEngine.UI.Button)
                    btn.onClick:RemoveAllListeners()
                    btn.onClick:AddListener(
                        function ()
                            network_sender:useItem(cfg.id, 1)
                        end
                    )

                    local btn = child:Find('useTrans/batBtn'):GetComponent(UnityEngine.UI.Button)
                    btn.onClick:RemoveAllListeners()
                    btn.onClick:AddListener(
                        function ()
                            local select_num, max_num = self:getUseSliderData(cfg.id)
                            frame_manager:show('use_slider', nil, {
                                item_id = cfg.id,
                                select_num = select_num,
                                max_num = max_num,
                                callback = function(item_id, item_num)
                                    network_sender:useItem(item_id, item_num)
                                end,
                            })
                        end
                    )
                end
            end
        end

        local none_trans = self.transform:Find("content/none")
        local have_trans = self.transform:Find("content/own")

        frame_helper:setActive(none_trans, temp_idx <= 0)
        frame_helper:setActive(have_trans, temp_idx > 0)
    end,

    showArmyLineScorllView = function(self)
        local none_trans = self.transform:Find("content/none")
        local have_trans = self.transform:Find("content/Scroll View")
        frame_helper.hideTrans(none_trans)

        local type_id = self.mapping_type[self.need_item_id]
        local cfgs = item_helper:getItemCfgByType(type_id)
        table.sort(cfgs, frame_helper.sortItem)
        local parent_path = "content/own/Scroll View/Viewport/Content"
        frame_helper:hideAllChild(self, parent_path, true)

        for idx, cfg in ipairs(cfgs) do
            print("加速道具:", cfg.id)
            local child = frame_helper:getChild(self, idx, parent_path)
            child:SetActive(true)

            local icon = child:Find("icon"):GetComponent(UnityEngine.UI.Image)
            local quality = child:Find("out_bg"):GetComponent(UnityEngine.UI.Image)
            local name_text = child:Find('name_text'):GetComponent(UnityEngine.UI.Text)
            local des_text = child:Find('des_text'):GetComponent(UnityEngine.UI.Text)
            local own_text = child:Find('own_text'):GetComponent(UnityEngine.UI.Text)

            local use_trans = child:Find('useTrans')
            local buy_trans = child:Find('buyTrans')

            local own_num = frame_helper:getItemNum(cfg.id)

            icon:SetSprite(frame_helper:getItemIcon(cfg.id))
            quality:SetSprite(frame_helper:getItemQualityIcon(cfg.id))
            name_text.text = frame_helper:getLangCfg(cfg.name_id)
            des_text.text = frame_helper:getLangCfg(cfg.des_id)
            own_text.text = string.format(NativeText.GetLangStr(250010), "<color=#ffffffff>" .. own_num .. "</color>") 

            use_trans:SetActive(own_num > 0)
            buy_trans:SetActive(own_num <= 0)

            local bat_btn = child:Find("useTrans/batBtn")
            bat_btn:SetActive(false)

            if own_num > 0 then
                local btn = child:Find('useTrans/useBtn'):GetComponent(UnityEngine.UI.Button)
                btn.onClick:RemoveAllListeners()
                btn.onClick:AddListener(
                    function ()
                        network_sender:speedUp(self.army_info.id, cfg.id)
                    end
                )
            else
                local buy_cost = frame_helper:getDiamondPriceFromStr(cfg.allprice)

                local DIAMOND    = 30001 -- 钻石
                local image_text = child:Find('buyTrans/image_text'):GetComponent(UnityEngine.UI.Text)
                local item_name = gui_helper:getImageTextItemStr(DIAMOND)
                image_text:SetImageText(string.format("%s %s", item_name, buy_cost))

                local btn = child:Find('buyTrans/buyBtn'):GetComponent(UnityEngine.UI.Button)
                btn.onClick:RemoveAllListeners()
                btn.onClick:AddListener(
                    function ()
                        frame_manager:show('buy_slider', nil, {
                            item_id = cfg.id,
                            select_num = 1,
                            max_num = 1,
                            callback = function(item_id, item_num)
                                network_sender:buyItem(
                                {
                                    {
                                        item_id = item_id,
                                        count = item_num,
                                    }
                                }, 0)
                            end,
                        })
                    end
                )
            end
        end
    end,

    showEnergyScorllView = function(self)
        local type_id = self.mapping_type[self.need_item_id]
        local cfgs = item_helper:getItemCfgByType(type_id)
        table.sort(cfgs, frame_helper.sortItem)
        local parent_path = "content/own/Scroll View/Viewport/Content"
        frame_helper:hideAllChild(self, parent_path, true)

        local temp_idx = 0
        for idx, cfg in ipairs(cfgs) do
            print("体力:", cfg.id)
            local own_num = frame_helper:getItemNum(cfg.id)
            if own_num > 0 then
                temp_idx = temp_idx + 1
                local child = frame_helper:getChild(self, temp_idx, parent_path)
                child:SetActive(true)

                local icon = child:Find("icon"):GetComponent(UnityEngine.UI.Image)
                local quality = child:Find("out_bg"):GetComponent(UnityEngine.UI.Image)
                local name_text = child:Find('name_text'):GetComponent(UnityEngine.UI.Text)
                local des_text = child:Find('des_text'):GetComponent(UnityEngine.UI.Text)
                local own_text = child:Find('own_text'):GetComponent(UnityEngine.UI.Text)

                local use_trans = child:Find('useTrans')
                local buy_trans = child:Find('buyTrans')

                icon:SetSprite(frame_helper:getItemIcon(cfg.id))
                quality:SetSprite(frame_helper:getItemQualityIcon(cfg.id))
                name_text.text = frame_helper:getLangCfg(cfg.name_id)
                des_text.text = frame_helper:getLangCfg(cfg.des_id)
                own_text.text = string.format(NativeText.GetLangStr(250010), "<color=#ffffffff>" .. own_num .. "</color>") 

                use_trans:SetActive(own_num > 0)
                buy_trans:SetActive(own_num <= 0)

                if own_num > 0 then
                    local btn = child:Find('useTrans/useBtn'):GetComponent(UnityEngine.UI.Button)
                    btn.onClick:RemoveAllListeners()
                    btn.onClick:AddListener(
                        function ()
                            network_sender:useItem(cfg.id, 1)
                        end
                    )

                    local btn = child:Find('useTrans/batBtn'):GetComponent(UnityEngine.UI.Button)
                    btn.onClick:RemoveAllListeners()
                    btn.onClick:AddListener(
                        function ()
                            local select_num, max_num = self:getUseSliderData(cfg.id)
                            frame_manager:show('use_slider', nil, {
                                item_id = cfg.id,
                                select_num = select_num,
                                max_num = max_num,
                                callback = function(item_id, item_num)
                                    network_sender:useItem(item_id, item_num)
                                end,
                            })
                        end
                    )
                end
            end
        end

        local none_trans = self.transform:Find("content/none")
        local have_trans = self.transform:Find("content/own")

        frame_helper:setActive(none_trans, temp_idx <= 0)
        frame_helper:setActive(have_trans, temp_idx > 0)
    end,

    showScorllView = function(self)
        if  self.need_item_id == 32001 or 
            self.need_item_id == 30002 or 
            self.need_item_id == 32002 or 
            self.need_item_id == 32003 or 
            self.need_item_id == 32004 or
            self.need_item_id == 32008 then
            self:showResScorllView()
        elseif self.need_item_id == local_lord.EXP_ID then
            self:showExpScorllView()
        elseif self.need_item_id == 50001 then
            self:showEnergyScorllView()
        elseif self:isSpeedUp() then
            self:showArmyLineScorllView()
        end
    end,

    getUseSliderData = function(self, item_id)
        local select_num, max_num = self:getExpectSliderData(item_id)
        local own_num = frame_helper:getItemNum(item_id)
        if own_num < max_num then
            max_num = own_num
        end
        return select_num, max_num
    end,

    getBuySliderData = function(self, item_id)
        local expect_select_num, expect_max_num = self:getExpectSliderData(item_id)
        local single_price = frame_helper:getItemBuyPrice(item_id)
        local diamond_num = frame_helper:getDiamondNum()
        local can_buy_num = math.floor(diamond_num / single_price)
        if expect_select_num <= can_buy_num then
            return expect_select_num, expect_max_num
        else
            return can_buy_num > 0 and can_buy_num or 1, expect_max_num
        end
    end,

    getExpectSliderData = function(self, item_id)
        local cfg = item_helper:getItemCfg(item_id)
        local per_reward = self:getCfgRewardNum(cfg)
        local own_num = frame_helper:getItemNum(self.need_item_id)
        local need_num = self.need_item_num
        local lack = need_num - own_num
        if lack > 0 then
            local rst = math.ceil(lack / per_reward)
            return rst, rst
        else
            return 1, 1
        end
    end,

    getCfgRewardNum = function(self, cfg)
        local tab = loadstring("return ".. cfg.reward_item)() 
        for item_id, num in pairs(tab) do
            return num
        end
    end,

    isSpeedUp = function(self)
        if self.need_item_id == army_queue_manager.normal_speed_up or
            self.need_item_id == army_queue_manager.transport_speed_up or
            self.need_item_id == army_queue_manager.mass_speed_up then

                return true
        end

        return false
    end,
}
