local log = logger.get("Game", "SelectItemFrame")
local super = require("ui.frame")
require("level.level_quest_manager")

return class{
    super = super,

    onLoaded = function( self )
        super.onLoaded(self)

        self.owner = self.args.owner
        self.level_id = self.args.data

        clickEvents = {
            "ButtonBack",
            "PanelItem/PanelBottom/ButtonAuto",
            "PanelItem/PanelBottom/ButtonFight",
            "PanelItem/PanelUp/Layout/Item1/Icon1",
            "PanelItem/PanelUp/Layout/Item2/Icon2",
            "PanelItem/PanelUp/Layout/Item3/Icon3",
            "PanelItem/PanelUp/Layout/Item4/Icon4",
            "PanelItem/PanelUp/Layout/Item5/Icon5",
            "PanelItem/PanelUp/Layout/Item6/Icon6",
            "PanelItem/PanelUp/Layout/Item7/Icon7",
            "PanelItem/PanelUp/Layout/Item8/Icon8",

            "PanelItem/PanelBottom/Layout/Slot1/SlotIcon1",
            "PanelItem/PanelBottom/Layout/Slot2/SlotIcon2",
            "PanelItem/PanelBottom/Layout/Slot3/SlotIcon3",
            "PanelItem/PanelBottom/Layout/Slot4/SlotIcon4",            
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self:setScript("hover", "PanelItem/PanelUp/Layout/Item1/Icon1", functor.rmake(self.onHoverIcon, 1))
        self:setScript("hover", "PanelItem/PanelUp/Layout/Item2/Icon2", functor.rmake(self.onHoverIcon, 2))
        self:setScript("hover", "PanelItem/PanelUp/Layout/Item3/Icon3", functor.rmake(self.onHoverIcon, 3))
        self:setScript("hover", "PanelItem/PanelUp/Layout/Item4/Icon4", functor.rmake(self.onHoverIcon, 4))

        local price_go = self.transform:Find("PanelItem/PanelUp/Price/Text").gameObject
        self.price_text = price_go:GetComponent(UnityEngine.UI.Text)

        self.tips_panel = self.transform:Find("TipsPanel")
        self.tips_panel:SetActive(false)
        ----self.content = self.transform:Find("content/Scroll View/Viewport/Content")
        -- self.prefab = self.transform:Find("content/Scroll View/Viewport/Content/tplt").gameObject

        self.selected_items = selected_level_items or {}    -- 已经携带的道具  slot_id - {id=xx,num=xx}
        self.carry_item_num = 4     -- 可以带几组道具进关卡
        self:refreshAllSlotUI()
        --self.evts = game_director.events:on("WORLDMAPGETOWNERBUILD", functor.make(self.loadHold, self))
    end,

    onUnload = function(self)
        --game_director.events:remove("WORLDMAPGETOWNERBUILD", self.evts)
    end,

    click_ButtonBack = function(self)
        selected_level_items = self.selected_items
        frame_manager:hide("select_level_item_frame")
    end,

    click_ButtonAuto = function(self)
        self.selected_items = {
            {id=31005, num = item_helper:getItemCfg(31005).stack},
            {id=31006, num = item_helper:getItemCfg(31006).stack},
            {id=31007, num = item_helper:getItemCfg(31007).stack},
            {id=31008, num = item_helper:getItemCfg(31008).stack},
        }
        self:refreshAllSlotUI()
    end,

    click_ButtonFight = function(self)
        local do_fight = function()
            selected_level_items = nil
            level_info.level_items = self.selected_items
            network_sender:enterMap(self.level_id, self.item_price or 0)

            self:setButtonCD("PanelItem/PanelBottom/ButtonFight", 1.0)
        end

        local selected = false
        for _,slot in pairs(self.selected_items) do
            if slot.num > 0 then
                selected = true
                break
            end
        end

        if selected then
            do_fight()
        else
            local tb = {
                title = "",
                content = NativeText.GetLangStr(170100),
                buttons =
                {
                    {
                        btn_name = NativeText.GetLangStr(170101),
                        callback = function() 
                            frame_manager:hide('common_box')
                        end
                    },
                    {
                        btn_name = NativeText.GetLangStr(170102),
                        callback = function()
                            do_fight()
                            frame_manager:hide('common_box')
                        end
                    },
                },
            }

            frame_manager:show('common_box', nil, tb)  
        end
    
    end,

    -- 添加携带的道具
    addCarryItem = function(self, item_id)
        local item_cfg = item_helper:getItemCfg(item_id)
        if not item_cfg then
            log.error("Failed to find item cfg:"..item_id)
            return
        end

        -- 找到要添加道具的 slot idx
        local idx = nil         -- 要把道具放在这个 idx
        local empty_idx = nil   -- 第一个空格的 idx
        for i=1, self.carry_item_num do
            local slot = self.selected_items[i]
            if not slot then    -- 这格还是空的
                if not empty_idx then
                    empty_idx = i
                end
            else                -- 已经放了道具
                if slot.id == item_id and slot.num < item_cfg.stack then
                    idx = i
                    break
                end
            end
        end

        -- 原来没带，放到一个空格子上
        if not idx and empty_idx then
            idx = empty_idx
        end

        if idx == nil then
            return -- 没有格子放了 
        end

        -- 把 item 放到 idx slot 上
        local slot = self.selected_items[idx]
        if not slot then
            slot = {id=item_id, num = 0}
            self.selected_items[idx] = slot
        end
        slot.num = slot.num + 1

        self:refreshSlotUI(idx) -- 刷新 slot 界面
        self:refreshPrice()
    end,

    -- 移除携带的道具
    removeCarrayItem = function(self, idx)
        local slot = self.selected_items[idx]
        if slot and slot.num > 0 then
            slot.num = slot.num - 1
            if slot.num <= 0 then
                self.selected_items[idx] = nil
            end
            self:refreshSlotUI(idx)
            self:refreshPrice()
        end
    end,

    refreshAllSlotUI = function(self)
        for i=1, self.carry_item_num do
            self:refreshSlotUI(i)
        end
        self:refreshPrice()
    end,

    -- 更新某个槽
    refreshSlotUI = function(self, idx)
        local ui_slot = self.transform:Find("PanelItem/PanelBottom/Layout/Slot"..idx)
        if not ui_slot then
            log.error("Failed to find ui slot idx:"..idx)
            return
        end
        local ui_icon = ui_slot:Find("SlotIcon"..idx).gameObject
        local ui_num = ui_slot:Find("Num").gameObject

        local slot = self.selected_items[idx]
        if not slot or slot.num == 0 then
            -- 空的格子
            local img = ui_icon:GetComponent(UnityEngine.UI.Image)
            img.color = UnityEngine.Color(1,1,1,0)
            ui_num:SetActive(false)
        else
            -- 有道具的
            local item_cfg = item_helper:getItemCfg(slot.id)
            local img = ui_icon:GetComponent(UnityEngine.UI.Image)
            img.color = UnityEngine.Color(1,1,1,1)
            img:SetSprite(gui_helper:getItemIconPath(item_cfg))
            --img:SetNativeSize()

            ui_num:SetActive(true)
            local text = ui_num:GetComponent(UnityEngine.UI.Text)
            text.text = tostring(slot.num)
        end
    end,

    -- 更新价格
    refreshPrice = function(self)
        self.item_price = 0
        for _,slot in pairs(self.selected_items) do
            if slot and slot.num > 0 then
                local cfg = item_helper:getItemCfg(slot.id)
                self.item_price = self.item_price + item_helper:getItemBuyPrice(cfg, 30002) * slot.num
            end
        end
        self.price_text.text = tostring(self.item_price)
    end,

    click_Icon1 = function(self)
        self:addCarryItem(31005)
    end,

    click_Icon2 = function(self)
        self:addCarryItem(31006)
    end,

    click_Icon3 = function(self)
        self:addCarryItem(31007)
    end,

    click_Icon4 = function(self)
        self:addCarryItem(31008)
    end,

    click_Icon5 = function(self)
        self:addCarryItem(31009)
    end,

    click_Icon6 = function(self)
        self:addCarryItem(31010)
    end,

    click_Icon7 = function(self)
        self:addCarryItem(31011)
    end,

    click_Icon8 = function(self)
        self:addCarryItem(31012)
    end,

    click_SlotIcon1 = function(self)
        self:removeCarrayItem(1)
    end,

    click_SlotIcon2 = function(self)
        self:removeCarrayItem(2)
    end,

    click_SlotIcon3 = function(self)
        self:removeCarrayItem(3)
    end,

    click_SlotIcon4 = function(self)
        self:removeCarrayItem(4)
    end,

    showItemTip = function(self, icon_idx)
        log.info("Show icon_idx:"..icon_idx)
        local item_cfg = nil
        if icon_idx == 1 then
            item_cfg = item_helper:getItemCfg(31005)
        elseif icon_idx == 2 then
            item_cfg = item_helper:getItemCfg(31006)
        elseif icon_idx == 3 then
            item_cfg = item_helper:getItemCfg(31007)
        elseif icon_idx == 4 then
            item_cfg = item_helper:getItemCfg(31008)
        end

        if not item_cfg then
            log.error("Failed to show tip, idx:"..icon_idx)
            return
        end

        local go = self.tips_panel.gameObject
        go:SetActive(true)
        local pos = self:getIconPos(icon_idx)
        pos.x = pos.x - 5
        go.transform.position = pos
        --log.info("pos:"..tostring(pos))

        local text_name = self.tips_panel:Find("TextName"):GetComponent(UnityEngine.UI.Text)
        text_name.text = frame_helper:getLangCfg(item_cfg.name_id)

        local text_price = self.tips_panel:Find("Price/Text"):GetComponent(UnityEngine.UI.Text)
        text_price.text = tostring(item_helper:getItemBuyPrice(item_cfg))

        local text_desc = self.tips_panel:Find("TextDesc"):GetComponent(UnityEngine.UI.Text)
        text_desc.text = frame_helper:getLangCfg(item_cfg.des_id)

        local img = self.tips_panel:Find("Slot/Icon"):GetComponent(UnityEngine.UI.Image)
        img:SetSprite(gui_helper:getItemIconPath(item_cfg))
    end,

    -- 获得 icon 在最上层 panel 中的位置
    getIconPos = function(self, icon_idx)
        return self.transform:Find("PanelItem/PanelUp/Layout/Item"..icon_idx).position
    end,

    hideItemTip = function(self, icon_idx)
        --log.info("Hide icon_idx:"..icon_idx)
        local go = self.tips_panel.gameObject

        go:SetActive(false)
    end,

    onHoverIcon = function(self, handler, eventData, evt_type, icon_idx)
        --log.info("Hover:"..evt_type..","..tostring(icon_idx))
        if evt_type == "enter" then
            self.hover_idx = icon_idx
            local start_time = Time.time

            app:startCoroutine(
                function()
                    while self.hover_idx do
                        coroutine.yield(nil)
                        if not is_show and Time.time - start_time > 0.3 then
                            self:showItemTip(icon_idx)
                            break
                        end
                    end
                end
            )
        elseif evt_type == "exit" then
            self.hover_idx = nil
            self:hideItemTip(icon_idx)
        end
    end,
}