local super = require("ui.frame")

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        -- 路径: root/login
        -- 事件: click_login 
        clickEvents = {
            "content/bg/top/ButtonBack",
            "content/bg/top/homeBtn",
            "2st/view/closeViewBtn",
            "2st/view/Window/cancelBtn",
            "2st/view/Window/confirmBtn",
            "refreshView/view/Window/closeRefreshBtn",
            "refreshView/view/closeRefreshBtn",
            "refreshView/view/Window/refreshBtn",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self.second_view = self.transform:Find('2st')
        self.helper.hideTrans(self.second_view)
        self.refresh_view = self.transform:Find('refreshView')
        self.helper.hideTrans(self.refresh_view)

        self:init()

        self.shop_item_change_event_id = local_shop.events:on("shop_item_change", functor.make(self.onShopItemsChange, self))
    end,

    onUnload = function (self)
        local_shop.events:remove("shop_item_change", self.shop_item_change_event_id)
        local_player.events:off("onPropChange", self.prop_change_event_id)
    end,

    init = function(self)
        if not local_shop then
            return
        end
        local first_select = nil
        local idx = 1
        local parent_path = "content/bar/"
        for shop_type, items in pairs(local_shop.shop) do
            if not first_select then first_select = shop_type end
            local child = self.helper:getChild(self, idx, parent_path)
            local button = child:GetComponent(UnityEngine.UI.Button)
            local temp = idx
            button.onClick:AddListener(
                function ()
                    self:selectShopType(shop_type, temp)
                end
            )
            local cfg = config.store_lists[shop_type]
            if cfg then
                local des_text = child.transform:Find("Text"):GetComponent(UnityEngine.UI.Text)
                des_text.text = self.helper:getLangCfg(cfg.name)
            end
            idx = idx + 1
        end

        if first_select then
            self:selectShopType(first_select, 1)
        end

        local refresh_text = self:findText('content/refresh/text')
        -- refresh_text.text = string.format(NativeText.GetLangStr(230006), " 12:00")
        local clk = refresh_text.transform:GetComponent(Clock)
        if not clk then
            clk = refresh_text.transform:AddComponent(Clock)
        end
        clk.prefix = NativeText.GetLangStr(230006)
        clk:Set(local_shop.count_time)

        self.prop_change_event_id = local_player.events:on("onPropChange", 
        function()
            self:refreshResourceNum(self.cur_shop_type)
        end)
    end,

    selectShopType = function(self, shop_type, idx)
        self.curr_shop_type = shop_type
        local bar_trans = self.transform:Find('content/bar/')
        for i = 1, bar_trans.childCount do
            local child = bar_trans:GetChild(i - 1)
            local button = child:GetComponent(UnityEngine.UI.Button)
            button.interactable = i ~= idx
            local text = child:Find('Text'):GetComponent(UnityEngine.UI.Text)
            text.color = i == idx and Color.black or Color.white
        end

        self:refreshResourceNum(shop_type)
        self:refreshItems(shop_type)
        self:registerRefresh(shop_type)
    end,

    registerRefresh = function(self, shop_type)
        local btn = self.transform:Find("content/refresh/openRefreshBtn"):GetComponent(UnityEngine.UI.Button)
        btn.onClick:RemoveAllListeners()
        btn.onClick:AddListener(
            function ()
                self:click_openRefreshBtn(shop_type)
            end)
    end,

    refreshResourceNum = function(self, shop_type)
        self.cur_shop_type = shop_type
        local cfg = config.store_lists[shop_type]
        if cfg and cfg.showcoin then
            local tmpArgs = self.helper:getJsonTab(cfg.showcoin)
            local parent_path = "content/res layout/"
            self.helper:hideAllChild(self, parent_path)
            for __, item_cfg_id in ipairs(tmpArgs) do
                item_cfg_id = tonumber(item_cfg_id)
                local item_cfg = self.helper:getItemCfg(item_cfg_id)
                local item_icon_name = self.helper:getItemIcon(item_cfg.id)
                local child = self.helper:getChild(self, __, parent_path)
                child:SetActive(true)
                local image = child:Find('addAssetBtn'):GetComponent(UnityEngine.UI.Image)
                local text = child:Find('Text'):GetComponent(UnityEngine.UI.Text)
                image:SetSprite(item_icon_name)
                self.helper:makeNumEffect(text.transform, tonumber(self.helper:getItemNum(item_cfg_id)))
                local btn = child:Find('addAssetBtn'):GetComponent(UnityEngine.UI.Button)
                btn.onClick:RemoveAllListeners()
                btn.onClick:AddListener(
                    function ()
                        frame_manager:show('resource_convert')
                    end)
            end
        end
    end,

    onShopItemsChange = function(self)
        if self.curr_shop_type then
            self:refreshItems(self.curr_shop_type)
        end
    end,

    refreshItems = function(self, shop_type)
        local parent_path = "content/Scroll View/Viewport/Content"
        self.helper:hideAllChild(self, parent_path)
        local items = local_shop.shop[shop_type]
        if items then
            for idx, i in ipairs(items) do
                local child = self.helper:getChild(self, idx, parent_path)
                child.gameObject:SetActive(true)

                local green_bg = child:Find('shop_panel_4'):GetComponent(UnityEngine.UI.Image)
                local red_bg = child:Find('shop_panel_5'):GetComponent(UnityEngine.UI.Image)
                local cost_image = child:Find('buyBtn/costIcon'):GetComponent(UnityEngine.UI.Image)
                local cost_text = child:Find('buyBtn/Text'):GetComponent(UnityEngine.UI.Text)
                local des_text = child:Find('name'):GetComponent(UnityEngine.UI.Text)
                local icon_image = child:Find('item'):GetComponent(UnityEngine.UI.Image)
                local quality_image = child:Find('itemBg'):GetComponent(UnityEngine.UI.Image)
                local rate_text = child:Find('rate'):GetComponent(UnityEngine.UI.Text)
                local sellOut_text = child:Find('sellOut'):GetComponent(UnityEngine.UI.Text)
                local num_text = child:Find('count'):GetComponent(UnityEngine.UI.Text)

                local item_cfg = self.helper:getItemCfg(i.itemId)
                local item_name = self.helper:getLangCfg(item_cfg.name_id)
                local item_icon_name = self.helper:getItemIcon(item_cfg.id)
                local item_des = self.helper:getItemDes(item_cfg.id)
                local item_quality_icon = self.helper:getItemQualityIcon(item_cfg.id)
                local cost_cfg = self.helper:getItemCfg(i.cost.ItemTemId)
                local cost_item_name = self.helper:getItemIcon(cost_cfg.id)

                green_bg.enabled = i.isCanBuy == 1
                rate_text.enabled = i.isCanBuy == 1

                red_bg.enabled = i.isCanBuy ~= 1
                sellOut_text.enabled = i.isCanBuy ~= 1

                self.helper:setImageGray1(cost_image,       i.isCanBuy ~= 1)
                self.helper:setImageGray1(icon_image,       i.isCanBuy ~= 1)
                self.helper:setImageGray1(quality_image,    i.isCanBuy ~= 1)
                
                cost_image:SetSprite(cost_item_name)
                cost_text.text = math.floor(i.cost.Count * i.discount / 10000)
                des_text.text = item_name
                icon_image:SetSprite(item_icon_name)
                quality_image:SetSprite(item_quality_icon)
                num_text.text = i.count

                if green_bg.enabled and rate_text.enabled then
                    green_bg.enabled = i.discount < 10000
                    rate_text.enabled = i.discount < 10000
                    rate_text.text = i.discount / 100 .. "%"
                end

                -- local buy_btn = child:Find('buyBtn'):GetComponent(UnityEngine.UI.Button)
                local buy_btn = child:GetComponent(UnityEngine.UI.Button)
                buy_btn.interactable = i.isCanBuy == 1
                buy_btn.onClick:RemoveAllListeners()
                buy_btn.onClick:AddListener(
                    function ()
                        self:buyItem(shop_type, i)
                    end)
            end
        end
    end, 

    click_ButtonBack = function(self)
        frame_manager:hide('shop')
    end,

    click_homeBtn = function(self)
        frame_manager:clickHome()
    end,

    buyItem = function(self, shop_type, MShopItemInfo)
        self.curr_MShopItemInfo = MShopItemInfo

        local item_cfg = self.helper:getItemCfg(MShopItemInfo.itemId)
        local item_icon_name = self.helper:getItemIcon(item_cfg.id)
        local item_des = self.helper:getItemDes(item_cfg.id)
        local item_quality_icon = self.helper:getItemQualityIcon(item_cfg.id)
        local item_own_num = self.helper:getItemNum(MShopItemInfo.itemId)
        local item_name = self.helper:getLangCfg(item_cfg.name_id)

        local cost_cfg = self.helper:getItemCfg(MShopItemInfo.cost.ItemTemId)
        local cost_item_name = self.helper:getItemIcon(cost_cfg.id)
        
        local quality_image = self:findImage("2st/view/Window/quality_bg")
        local icon_image = self:findImage("2st/view/Window/icon")
        local num_text = self:findText("2st/view/Window/num_text")
        local name_text = self:findText("2st/view/Window/name_text")
        local des_text = self:findText("2st/view/Window/des_text")
        local own_text = self:findText("2st/view/Window/own_text")
        local explain_text = self:findText("2st/view/Window/explain_text")

        quality_image:SetSprite(item_quality_icon)
        icon_image:SetSprite(item_icon_name)
        num_text.text = MShopItemInfo.count
        name_text.text = item_name
        des_text.text = item_des
        own_text.text = string.format(NativeText.GetLangStr(230010), item_own_num)  

        local strs = self.helper:split(cost_item_name, "/")
        if #strs > 0 then
            cost_item_name = '[' .. strs[#strs] .. ']'
            cost_item_name = string.gsub(cost_item_name, '.png', '') .. " " .. math.floor(MShopItemInfo.cost.Count * MShopItemInfo.discount / 10000)
            local temp = string.gsub(NativeText.GetLangStr(230011), '\\n', '\n')
            -- explain_text.text = string.format(temp, cost_item_name)  
            explain_text:SetImageText(string.format(temp, cost_item_name))
        end
        self.helper.showTrans(self.second_view)
    end,

    click_closeViewBtn = function(self)
        self.helper.hideTrans(self.second_view)
    end,

    click_cancelBtn = function(self)
        self.helper.hideTrans(self.second_view)
    end,

    click_confirmBtn = function(self)
        self.helper.hideTrans(self.second_view)
        if self.curr_shop_type and self.curr_MShopItemInfo and self.curr_MShopItemInfo.isCanBuy == 1 then
            network_sender:buyGoods(self.curr_shop_type, self.curr_MShopItemInfo.index)
        end
    end,

    click_openRefreshBtn = function(self, shop_type)
        local info = local_shop:getRefreshInfo(shop_type)
        -- if info.useTime >= info.refreshMaxTime then
        --     network_sender:gameShopRefresh(shop_type)
        -- else
            -- self.helper.showTrans(self.refresh_view)
            -- local explain_text = self:findText("refreshView/view/Window/explain_text")

            -- local emoji_text = ""
            -- for __, value in ipairs(info.refreshCost) do
            --     local icon_name = self.helper:getItemIcon(value.ItemTemId)
            --     local strs = self.helper:split(icon_name, "/")
            --     if #strs > 0 then
            --         local temp = string.gsub('[' .. strs[#strs] .. ']', '.png', '')
            --         temp = temp .. value.Count
            --         emoji_text = emoji_text .. " " .. temp
            --     end
            -- end
            -- local shop_name = self.helper:getLangCfg(config.store_lists[shop_type].name)
            -- -- explain_text.text = string.format(NativeText.GetLangStr(230013), shop_name, emoji_text)
            -- explain_text:SetImageText(string.format(NativeText.GetLangStr(230013), shop_name, emoji_text))
        -- end


        self.helper.showTrans(self.refresh_view)
        local image_text = self:findText("refreshView/view/Window/explain_text")
        local shop_name = self.helper:getLangCfg(config.store_lists[shop_type].name)
        for __, value in ipairs(info.refreshCost) do
            local emoji_text = gui_helper:getImageTextItemStr(value.ItemTemId) .. " " .. value.Count
            image_text:SetImageText(string.format(NativeText.GetLangStr(230013), shop_name, emoji_text))
            break
        end
    end,

    click_refreshBtn = function(self)
        if self.curr_shop_type then
            network_sender:gameShopRefresh(self.curr_shop_type)
        end
        self.helper.hideTrans(self.refresh_view)
    end,

    click_closeRefreshBtn = function(self)
        self.helper.hideTrans(self.refresh_view)
    end,
}
