local super = require("ui.frame")
local stage_events = enums.stage_events
local log = logger.get("Game", "Frame")

return class {
    super = super, 

    onLoaded = function (self)
        log.info('battleui onLoaded!!!')

        super.onLoaded(self)
        -- print(self._error)
        self.stage = self.args.owner
        self.hints = {}
        self.activeHint = 0
        self.dragging_icon = false
        self.active_slots = 0
        self.slots = {}
        self.actors = {}

        self.castBar = self:find("bottomLeft/cast_bar")
        self:setScript("click", "ButtonCancel", self.onClickCancel)
        self:setScript("hover", "ButtonCancel", self.onHoverCancel)

        self:setScript("drag", "dragger", self.onDrag_fulldrag)
        self:setScript("drop", "dragger", self.onDrop_fulldrag)
        self:setScript("single", "dragger", self.onSingleClick_fulldrag)
       
        self.castBar:SetActive(false)

        self.button_cancel = self:find("ButtonCancel")
        self.button_cancel:SetActive(false)

        if self.stage.state == enums.stage_states.RUN then 
            -- 角色已经创建
            self.stage.groups[1]:foreach(functor.make(self.onSpawn, self))
            self:postStageLoad()
        else
            -- 角色尚未创建
            self.stage.events:on(stage_events.SPAWN, functor.make(self.onSpawn, self))
            self.stage.events:on(stage_events.STATE, functor.make(self.onStageState, self))
        end
        -- print("战斗界面加载完成", self.args.stage)

        clickEvents = {
        }
        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self:createTankItem()

        -- 注册血量变化的消息
        self.stage.events:on("actor_hp_change", functor.make(self.onTankHpChange, self))
        -- 注册能量变化的消息
        self.stage.events:on("actor_energy_change", functor.make(self.onTankEnergyChange, self))

        self:init()
    end, 

    init = function(self)
        local grid_tsf = self.transform:Find("bottomLeft/grid")
        grid_tsf.localPosition = Vector3(0, -120, 0)
        local lt = LeanTween.moveLocalY(grid_tsf.gameObject, 0, 0.25)
        lt.delay = 0.5
    end,

    createTankItem = function(self)
        if not self.tanks_icon_tsf then
            local grid_tsf = self.transform:Find("bottomLeft/grid")
            for i = 1, grid_tsf.childCount do
                local tsf = self.transform:Find("bottomLeft/grid/tank (" .. i .. ")")
                tsf.gameObject:SetActive(false)
            end
        end

        if not self.tank_data then
            self.tank_data = {}
        end

        local i = 1
        for k, actor in pairs(self.stage.actors) do
            -- team == 0 表示是自己
            -- unit_type 1 战车 2 小兵
            if actor.team == 0 and actor.cfg.unit_type == 1 then
                local path = "bottomLeft/grid/tank (" .. i .. ")"
                local tsf = self.transform:Find(path)
                tsf.gameObject:SetActive(true)
                local icon = tsf:Find("icon"):GetComponent(UnityEngine.UI.Image)
                
                icon:SetSprite(actor.cfg.battle_icon, function()
                    icon:SetNativeSize()
                end)

                self:setScript("click", path, functor.rmake(self.onClick_select, actor.id))
                self:setScript("updown", path, functor.rmake(self.onPress_select, actor.id))

                local element = {}
                element.tsf = tsf
                element.id = actor.id
                table.insert(self.tank_data, element)

                i = i + 1
            end
        end
    end,

    -- id 是actor分配的id
    getTankData = function(self, id)
        if not self.tank_data then
            return nil
        end

        for __, element in pairs(self.tank_data) do
            if element.id == id then
                return element
            end
        end

        return nil
    end,

    -- id 是actor分配的id
    onTankHpChange = function(self, id, is_alive, hp, max_hp)
        -- print("血量变化:", id)
        local element = self:getTankData(id)
        if element and element.tsf then
            local hp_image = element.tsf:Find("hp"):GetComponent(UnityEngine.UI.Image)

            if is_alive then
                hp_image.fillAmount = hp / max_hp
            else
                hp_image.fillAmount = 0
            end
        end
    end,

    -- id 是actor分配的id
    onTankEnergyChange = function(self, id, is_alive, energy, max_energy)
        -- print("skill_frame onTankEnergyChange:", id, is_alive, energy, max_energy)
        if not self.energy_data then
            self.energy_data = {}
        end

        local target = nil

        for __, eg in pairs(self.energy_data) do
            if eg.id == id then
                target = eg
                eg.canbe = energy == max_energy
            end
        end

        if target == nil then
            local eg = {}
            eg.id = id
            eg.canbe = energy == max_energy
            table.insert(self.energy_data, eg)
        end

        local element = self:getTankData(id)
        if element and element.tsf then
            local energy_image = element.tsf:Find("energy"):GetComponent(UnityEngine.UI.Image)
            local bg_image = element.tsf:Find("bg"):GetComponent(UnityEngine.UI.Image)
            local bie = element.tsf:GetComponent(BattleIconEffect)
            if is_alive then
                bie:make(energy == max_energy)
                bg_image.enabled = energy == max_energy
                -- print("energy:", energy, max_energy)
                energy_image.fillAmount = energy / max_energy

                -- BattleIconEffect
            else
                bie:make(false)
                bg_image.enabled = false
                energy_image.fillAmount = 0
            end
        end
    end,

    canbeUseSkill = function(self, id)
        if not self.energy_data then
            return false
        end

         for __, eg in pairs(self.energy_data) do
            if eg.id == id then
                return eg.canbe
            end
        end

        return false
    end,

    -- 界面初始化之后做一些额外处理, 比如禁用多余的控件
    postStageLoad = function (self)
        for i = self.active_slots + 1, 8 do 
            self.slots[i]:SetActive(false)
        end

        self.orbitViewer = self.stage.transform:FindComponent("target", Fenix.Game.Unity.OrbitViewer) 
        if self.orbitViewer then 
            self.orbitViewer.useMouseInput = false
        end
    end, 

    onStageState = function (self, state)
        if state == enums.stage_states.RUN then 
            self:postStageLoad()
        end
    end, 

    onSpawn = function (self, index, actor)
        if actor.force == 1 then 
            local path = "bottomLeft/mygroup/actor (" .. index .. ")"
            self.active_slots = index
            table.insert(self.actors, actor)
            -- print("spawn", index, actor:tostring())
            actor.properties:on("hp", function (oldValue, newValue)
                self:findImage(path .. "/hp").fillAmount = math.div(newValue, actor.properties.hp_max)
            end)
            actor.properties:on("morale", function (oldValue, newValue)
                self:findImage(path .. "/sp").fillAmount = math.div(newValue, actor.properties.morale_max)
            end)

            self:findImage(path .. "/hp").fillAmount = math.div(actor.properties.hp, actor.properties.hp_max)
            self:findImage(path .. "/sp").fillAmount = math.div(actor.properties.morale, actor.properties.morale_max)
        end
    end, 

    onClickCast = function (self)
        battle_helper:castUltimateSkill()
        -- print("you clicked cast button", self.activeHint)
        --[[
        local actor = self.actors[self.activeHint]
        actor.properties.morale = 0
        self:setActiveHint(0)
        ]]
    end, 

    onClick_select = function (self, handler, id)
        -- self.castBar.transform:Find("button"):SetActive(actor.properties.morale >= actor.properties.morale_max)
        -- self:setActiveHint(index)
        -- print("you clicked actor", index, self.actors[index]:tostring())
        -- print("onClick_select:", id)
    end, 

    onPress_select = function (self, handler, evt, str, id)
        -- print("onPress_select id:", id)
        -- print("onPress_select handler:", handler)
        -- print("onPress_select evt:", evt)
        -- print("onPress_select str:", str)

        local rst = self:canbeUseSkill(id)
        if not rst then
            return
        end

        if str == "down" then
            battle_helper:tryCastUltimateSkill(id)
            self.cancel_skill = false
        else
            if self.cancel_skill then
                self.stage:cancelSkill()
            end
        end
    end, 

    onHoverCancel = function(self, handler, eventData, str)
        -- print('悬停', str)
        if str == "enter" then
            self.cancel_skill = true
        else
            self.cancel_skill = false
        end
    end,

    onDrop_fulldrag = function (self, handler, data)
        -- print("drop")
    end, 

    onDrag_fulldrag = function (self, handler, data, phase)
        -- print("onDrag_fulldrag", phase, data.delta)
        if not self.dragging_icon and phase == "drag" then 
            if self.orbitViewer then 
                self.orbitViewer:Transition(data.delta.x * 0.5, data.delta.y * -0.3)
            end
        end
    end, 

    -- 取消技能释放
    onClickCancel = function(self)
        print('calcel skill')
        self.stage:cancelSkill()
    end,

    onSingleClick_fulldrag = function (self, handler, data)
    --[[
        if not data.dragging then 
            self:setActiveHint(0)
        end]]
    end, 

    setSkillInfo = function(self, name, desc)
        local text_name = self:findText("bottomLeft/cast_bar/skill_name")
        local text_desc = self:findText("bottomLeft/cast_bar/skill_desc")
        text_name.text = name
        text_desc.text = desc
    end,

    setActiveHint = function (self, is_active)
        -- self.castBar:SetActive(is_active)
        self.button_cancel:SetActive(is_active)

        -- 显示或隐藏伤害数字
        battle_helper:setDamageTextVsb(is_active)
    end, 
}
