local super = require("ui.frame")
local stage_events = enums.stage_events
local log = logger.get("Game", "Frame")

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        self.war_mgr = self.args.owner

        clickEvents = {
            "topLeft/pauseGame",
            "bottomLeft/speedGame",
            "bottomLeft/autoUseSkill",
            "pause/exitButton",
            "pause/continueButton",
            "bottomLeft/skipWar",
            "bottomLeft/genRecord",
        }
        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self.commander_count_trans = self.transform:Find("topRight/commanderCount")
        self.commander_count_vsb = false
        -- self.commander_count_trans.gameObject:SetActive(self.commander_count_vsb)

        self.game_start_trans = self.transform:Find("center/startGame")
        self.game_start_trans:SetActive(false)

        self.commander_trans_t0 = self.transform:Find("commander_t0")
        self.commander_trans_t0:SetActive(false)

        self.commander_trans_t1 = self.transform:Find("commander_t1")
        self.commander_trans_t1:SetActive(false)

        self.small_fire_trans = self.transform:Find("top/Image/small")
        self.small_fire_trans:SetActive(false)

        self.big_fire_trans = self.transform:Find("top/Image/big")
        self.big_fire_trans:SetActive(false)
        
        -- 血量变化
        self.actor_hp_change_event_id = self.war_mgr.events:on('actor_hp_change', function(actor_id, cur_hp, max_hp)
            self:onHpChange(actor_id, cur_hp, max_hp)
        end)

        -- 能量变化
        self.actor_energy_change_event_id = self.war_mgr.events:on('actor_energy_change', function(actor_id, cur_energy, max_energy)
            self:onEnergyChange(actor_id, cur_energy, max_energy)
        end)


        if self.war_mgr.vehicle_actors then
            self:createVehicles(self.war_mgr.vehicle_actors)
        else
             -- 注册坦克生成回调
            self.war_mgr.events:on("actor_spawned", function(vehicle_actors)
                self:createVehicles(vehicle_actors)
            end)
        end

        -- 指挥官倒计时
        self.war_mgr.events:on("sum_time", function(total_time, stage_id)
            self:setCommanderCountDown(total_time)
            self:showGameCountDown(total_time)
            self:showTopBar(total_time, stage_id)
        end)

        self.fire_vfx = self.transform:Find("UI_zhandou_jiemian_chixuhuo")
        self.fire_vfx.gameObject:SetActive(false)

        self.clock_vfx = {
            self.transform:Find("bottomRight/content/UI_zhandou_jiemian_qiyabiaozhizhen (1)/qiyabiao_low").gameObject,
            self.transform:Find("bottomRight/content/UI_zhandou_jiemian_qiyabiaozhizhen (1)/qiyabiao_mid").gameObject,
            self.transform:Find("bottomRight/content/UI_zhandou_jiemian_qiyabiaozhizhen (1)/qiyabiao_high").gameObject,
            self.transform:Find("bottomRight/content/UI_zhandou_jiemian_qiyabiaozhizhen (1)/qiyabiao_high").gameObject
        }

        for __, vfx in ipairs(self.clock_vfx) do
            vfx:SetActive(false)
        end

        self.clock_boom_vfx = self.transform:Find("bottomRight/content/UI_zhandou_jiemian_qiyabiaoposui (1)").gameObject
        self.top_flash_vfx = self.transform:Find("top/Image/UI_zhandou_jiemian_duijuezhuanhuan").gameObject

        self.fly_board_vfx = self.transform:Find("fly_board_vfx").gameObject
        
        local rds = self.transform:Find("fly_board_vfx"):GetComponentsInChildren(UnityEngine.Renderer).Table
        for __, r in ipairs(rds) do
            r.sortingOrder = 4
        end
        self.fly_board_vfx:SetActive(false)

        app:startCoroutine(function()
            while true do
                self:updateDamage()

                coroutine.yield(nil)
                coroutine.yield(nil)
            end
        end)

        -- self:showDamage(1, 100)
        -- self:showDamage(1, 100)
        -- self:showDamage(1, 100)
        -- self:showDamage(1, 100)

        local debug = UnityEngine.Application.isEditor
        self.transform:Find("bottomLeft/skipWar").gameObject:SetActive(debug)
        self.transform:Find("bottomLeft/genRecord").gameObject:SetActive(debug)

        self.pause_trans = self.transform:Find("pause")
        self.helper.hideTrans(self.pause_trans)

        self.debuff_layout = self.transform:Find("topRight/DebuffLayout")
        self:showDebuff()

        self:init()
    end, 

    onUnload = function (self)
        self.war_mgr.events:off("actor_hp_change", self.actor_hp_change_event_id)
        self.war_mgr.events:off("actor_energy_change", self.actor_energy_change_event_id)
        self.cache_vehicles = nil
    end, 

    init = function(self)
        self.speed_list = {1,2,3}
        self.speed_idx = 1
        self.speed_images = {self:findImage("bottomLeft/speedGame/1"), self:findImage("bottomLeft/speedGame/2"), self:findImage("bottomLeft/speedGame/3")}
        self:setSpeedImage(self.speed_idx)

        local text_l = self:findText("fly_board_vfx/ZhandouFrame_Feiban/jueseming_L")
        local text_r = self:findText("fly_board_vfx/ZhandouFrame_Feiban/jueseming_R")
        local commander_name_t0 = self:findText("commander_t0/name_bg/commander_name")
        local commander_name_t1 = self:findText("commander_t1/name_bg/commander_name")

        text_l.text = local_player.name or ""
        commander_name_t0.text = local_player.name or ""

        if level_info.map_attack_info then
            text_r.text = self:getMonsterName()
            commander_name_t1.text = text_r.text
        else
            text_r.text = ""
            commander_name_t1.text = ""
        end

        self:findText("bottomLeft/autoUseSkill/Text").enabled = self.war_mgr.auto_skill_self

        local vfx_parnet = self.transform:Find("center/vfx")

        for i = 0, vfx_parnet.childCount - 1 do
            local child = vfx_parnet:GetChild(i)
            if child then
                child.gameObject:SetActive(false)
            end
        end
    end,

    getMonsterName = function(self)
        if level_info and level_info.map_attack_info and level_info.map_attack_info.monsterId then
            local id = level_info.map_attack_info.monsterId
            if config.monster_group_ids[id] then
                local lang_id = config.monster_group_ids[id].Group_Name_ID
                if lang_id then
                    return self.helper:getLangCfg(tostring(lang_id))
                end
            end
        end

        return ""
    end,

    setSpeedImage = function(self, idx)
        for __, image in ipairs(self.speed_images) do
            image.enabled = (__ == idx)
            if __ == idx then
                self.war_mgr:setTimeScale(self.speed_list[__])
            end
        end
    end,

    -- 创建vehicle
    createVehicles = function(self, vehicle_actors)
        local pos = 1
        self.vehicle_count = 0
        for __, actor in pairs(vehicle_actors) do
            if actor then
                self.vehicle_count = self.vehicle_count + 1
            end
        end
        for __, actor in pairs(vehicle_actors) do
            if actor then
                local path = "bottomRight/content/grid/tank (" .. pos .. ")"
                local item_trans = self.transform:Find(path)
                local icon = self:findImage(path .. "/offset/icon")
                icon:SetSprite(actor.cfg.battle_icon, 
                    function()
                        self:setSpriteCallback()
                    end)
                self:setScript("click", path, functor.rmake(self.castSkill, actor))
                pos = pos + 1
                self:createCacheData(actor, item_trans)
            end
        end

        local go = self.transform:Find("bottomRight/content").gameObject
        local lt = LeanTween.moveLocalX(go, 163.5 * (6 - pos + 1), 0)
        lt.loopType = LeanTweenType.once

        for i = pos, 6 do
            local path = "bottomRight/content/grid/tank (" .. i .. ")"
            local item_trans = self.transform:Find(path)
        end
    end,

    showIconBar = function(self)
        local go = self.transform:Find("bottomRight/content").gameObject
        local pos = 0
        local lt = LeanTween.moveLocalY(go, pos, 0.2)
        lt.loopType = LeanTweenType.once
    end,

    setSpriteCallback = function(self)
        --[[
        if not self.vehicle_cur then
            self.vehicle_cur = 1
        else
            self.vehicle_cur = 1 + self.vehicle_cur
        end
        if self.vehicle_cur >= self.vehicle_count then
            local go = self.transform:Find("bottomRight/content").gameObject
            local pos = 0
            local lt = LeanTween.moveLocalY(go, pos, 0.2)
            lt.loopType = LeanTweenType.once
        end
        ]]
    end,

    -- 创建缓存数据
    createCacheData = function(self, actor, trans)
        if not self.cache_vehicles then
            self.cache_vehicles = {}
        end

        local energy_image = trans:Find("energy"):GetComponent(UnityEngine.UI.Image)
        local hp_image = trans:Find("hp"):GetComponent(UnityEngine.UI.Image)
        local can_use_image = trans:Find("canUse"):GetComponent(UnityEngine.UI.Image)
        local no_use_image = trans:Find("noUse"):GetComponent(UnityEngine.UI.Image)
        local icon_image = trans:Find("offset/icon"):GetComponent(UnityEngine.UI.Image)
        local offset_go = trans:Find("offset").gameObject

        self.cache_vehicles[actor.id] = {actor = actor, trans = trans, 
            energy_image = energy_image, hp_image = hp_image,
            can_use_image = can_use_image, no_use_image = no_use_image,
            icon_image = icon_image, offset_go = offset_go, alive = true,
            energy_full = nil}
    end,

    -- 使用技能
    castSkill = function (self, handler, actor)
        self.war_mgr:castMaiWeaponSkill(actor)
    end, 

    -- 显示开始战斗
    setGameStartVsb = function(self, visible)
        if self.game_start_trans and self.game_start_trans.gameObject.activeSelf ~= visible then
            self.game_start_trans:SetActive(visible)
        end
    end,

    setCommanderVsb = function(self, visible, team, skill_id)
        if (self.commander_trans_t0 and self.commander_trans_t0.gameObject.activeSelf ~= visible) or
           (self.commander_trans_t1 and self.commander_trans_t1.gameObject.activeSelf ~= visible) then
            if team == 0 then
                self.commander_trans_t0:SetActive(visible)
            else
                self.commander_trans_t1:SetActive(visible)
            end

            if visible then
                local skill_cfg = config.skill_skills[skill_id]
                local params = {}
                local skill_icon = nil
                local skill_name = nil
                local commander_name = nil

                if team == 0 then
                    table.insert(params, { rect_path = "commander_t0/icon", from = -677, to = 80, duration = 0.25, loop_type = LeanTweenType.once, delay = 0.2 })
                    table.insert(params, { rect_path = "commander_t0/name_bg", from = -422, to = 0, duration = 0.25, loop_type = LeanTweenType.once, delay = 0.2 })
                    table.insert(params, { rect_path = "commander_t0/bg", from = 1122, to = 0, duration = 0.25, loop_type = LeanTweenType.once, delay = 0.2 })
                    skill_icon = self:findImage("commander_t0/bg/skill_icon")
                    skill_name = self:findText("commander_t0/bg/skill_name")
                    commander_name = self:findText("commander_t0/name_bg/commander_name")
                else
                    table.insert(params, { rect_path = "commander_t1/icon", from = -677, to = 80, duration = 0.25, loop_type = LeanTweenType.once, delay = 0.2 })
                    table.insert(params, { rect_path = "commander_t1/name_bg", from = -422, to = 0, duration = 0.25, loop_type = LeanTweenType.once, delay = 0.2 })
                    table.insert(params, { rect_path = "commander_t1/bg", from = 1122, to = 0, duration = 0.25, loop_type = LeanTweenType.once, delay = 0.2 })
                    skill_icon = self:findImage("commander_t1/bg/skill_icon")
                    skill_name = self:findText("commander_t1/bg/skill_name")
                    commander_name = self:findText("commander_t1/name_bg/commander_name")
                end
                -- print("#指挥官技能skill_icon", skill_cfg.skill_icon, skill_id)
                skill_icon:SetSprite(skill_cfg.skill_icon)
                skill_name.text = self.helper:getLangCfg(skill_cfg.skill_name)

                for __, p in ipairs(params) do
                    local go = self.transform:Find(p.rect_path):GetComponent(UnityEngine.RectTransform)
                    local lt = LeanTween.moveX(go, p.to, p.duration)
                    lt.loopType = p.loop_type
                    lt:setFrom(p.from)
                    lt.delay = p.delay
                end
            end
        end
    end,

    -- 血量变化
    onHpChange = function(self, actor_id, cur_hp, max_hp)
        if not self.cache_vehicles then
            return
        end
        local data = self.cache_vehicles[actor_id]
        if data and data.hp_image then
            data.hp_image.fillAmount = cur_hp / max_hp
            if cur_hp <= 0 then
                if data.alive then
                    data.alive = false
                    self.helper:setImageGrayPlus(data.icon_image, true)
                end
            end
        end
    end,

    -- 能量变化
    onEnergyChange = function(self, actor_id, cur_energy, max_energy)
        if not self.cache_vehicles then
            return
        end
        local data = self.cache_vehicles[actor_id]
        if data and data.alive and data.energy_image then
            data.energy_image.fillAmount = cur_energy / max_energy
            local temp = cur_energy >= max_energy
            if data.energy_full == nil then
                data.energy_full = temp
                self:setEnergyState(temp, data)
            elseif data.energy_full ~= temp then
                data.energy_full = temp
                self:setEnergyState(temp, data)
            end
        end
    end,

    -- 设置满能量表现
    setEnergyState = function(self, state, data)
        data.can_use_image.enabled = data.energy_full
        data.no_use_image.enabled = not data.energy_full
        local go = data.icon_image.gameObject
        local pos = data.energy_full and 70 or 44
        local lt = LeanTween.moveLocalY(go, pos, 0.2)
        lt.loopType = LeanTweenType.once

        local energy_slider_vfx = data.trans:Find("energy/UI_zhandou_jiemian_jinengtiaomancao/").gameObject
        local energy_icon_vfx = data.trans:Find("offset/icon/UI_zhandou_jiemian_jinengkuangchixu").gameObject
        local energy_flash_vfx = data.trans:Find("canUse/UI_zhandou_jiemian_jinengkuangchuxian").gameObject

        energy_slider_vfx:SetActive(data.energy_full)
        energy_icon_vfx:SetActive(data.energy_full)
        energy_flash_vfx:SetActive(data.energy_full)
    end,

    setPhaseClockVfx = function(self, phase_id)
        if not self.clock_vfx then
            return
        end
        for __, vfx in ipairs(self.clock_vfx) do
            vfx:SetActive(phase_id == __)
        end
        self.clock_boom_vfx:SetActive(phase_id == 4)
        self.fire_vfx:SetActive(phase_id == 4)

        self.top_flash_vfx:SetActive(false)
        self.top_flash_vfx:SetActive(true)

        local top_text = self:findText("top/Image/text")
        if phase_id == 1 then
            top_text.text = NativeText.GetLangStr(180001)
        elseif phase_id == 2 then
            top_text.text = NativeText.GetLangStr(180002)
        elseif phase_id == 3 then
            top_text.text = NativeText.GetLangStr(180003)
        elseif phase_id == 4 then
            top_text.text = NativeText.GetLangStr(180004)
        end
    end,

    -- 设置飞板
    setFlyBoardVisible = function(self, vsb)
        if self.fly_board_vfx and self.fly_board_vfx.activeSelf ~= vsb then
            self.fly_board_vfx:SetActive(vsb)
            if vsb then

                local ani = self.fly_board_vfx.transform:Find("ZhandouFrame_Feiban"):GetComponent(UnityEngine.Animation)
                if ani then
                    ani:Play("ZhandouFrame_Feiban")
                end
            end
        end
    end,

    -- 显示伤害
    showDamage = function(self, icon_name, value)
        if not self.damage_list then
            self.damage_list = {}
        end
        if icon_name then
            local damage_trans = self.helper:copyTrans(self, "bottomRight/offset/layout/tplt")
            local icon = damage_trans:Find("bg/icon"):GetComponent(UnityEngine.UI.Image)
            local text = damage_trans:Find("bg/num"):GetComponent(UnityEngine.UI.Text)
            text.text = 0
            damage_trans.gameObject:SetActive(true)
            icon:SetSprite(icon_name, function()
                table.insert(self.damage_list, { damage_trans = damage_trans, value = value, text = text})
            end)
        end
    end,

    updateDamage = function(self)
        if not self.damage_interval_time then
            self.damage_interval_time = 0.5
        end
        self.damage_interval_time = self.damage_interval_time - Time.unscaledDeltaTime
        if self.damage_interval_time < 0 and self.damage_list and #self.damage_list > 0 then
            local damage_trans = self.damage_list[1].damage_trans
            local value = self.damage_list[1].value
            local text = self.damage_list[1].text

            local rect = damage_trans:GetComponent(UnityEngine.RectTransform)
            rect.transform.localPosition = Vector3.zero
            local lt1 = LeanTween.moveX(rect, -424.74, 0.3)
            lt1.loopType = LeanTweenType.once
            lt1.delay = 0
            lt1:setOnComplete(function()
                self.helper:makeNumEffect(text.transform, value)
            end)

            local lt2 = LeanTween.moveY(rect, 200, 0.3)
            lt2.loopType = LeanTweenType.once
            lt2.delay = 2.3
            table.remove(self.damage_list, 1)
            lt2:setOnComplete(function()
                UnityEngine.Object.Destroy(damage_trans.gameObject)
            end)
            self.damage_interval_time = 0.5
        end
    end,

    click_skipWar = function(self)
        local dt = 0.1
        self.war_mgr:readEvents(Tick(dt, true), dt)
        --[[
        if self.war_mgr.states.name ~= 'loading' then
            server_start()
            hint:setText("根目录：log_battle.log")
        else
            hint:setText("处于加载中")
        end
        ]]
        --[[
        local name = os.date("%H_%M_%S",os.time())
        local path = string.format("c:/%s.txt", name)
        local cmd = "md " .. path
        os.execute(cmd)
        local file = io.open(path, "w")
        file:write("Hello world.")
        file:close()
        ]]
    end,

    click_genRecord = function(self)
        if self.war_mgr.states.name ~= 'loading' then
            server_start()
        end
    end,

    -- 点击暂停事件
    click_pauseGame = function(self)
        self.helper.showTrans(self.pause_trans)
        self.war_mgr:setGamePause(true)
    end,

    click_speedGame = function(self)
        self.speed_idx = 1 + self.speed_idx
        if self.speed_idx > #self.speed_list then
            self.speed_idx = 1
        end
        self:setSpeedImage(self.speed_idx)
    end,

    click_autoUseSkill = function(self)
        self.war_mgr.auto_skill_self = not self.war_mgr.auto_skill_self
        self:findText("bottomLeft/autoUseSkill/Text").enabled = self.war_mgr.auto_skill_self
        SetBattleMode(0, self.war_mgr.auto_skill_self)
    end,

    click_exitButton = function(self)
        self.war_mgr:setGamePause(false)
        frame_manager:show('loading')
        self.war_mgr.level_manager:SetInBattle(false)   
        game_director:changeState('level', {scene_name = level_info.level_cfg.Map, level_id = level_info.map_id})
        frame_manager:hide('skill')
    end,

    click_continueButton = function(self)
        self.helper.hideTrans(self.pause_trans)
        self.war_mgr:setGamePause(false)
    end,

    -- 显示or隐藏 指挥官倒计时
    setCommanderCountDown = function(self, total_time)
        local trigger_start = config.battle_config_commons[1].p3_start - 10
        local trigger_end = config.battle_config_commons[1].p3_start

        if total_time >= trigger_start and total_time <= trigger_end then
            if not self.commander_count_vsb then
                self.commander_count_vsb = true
                local go = self.commander_count_trans.gameObject
                local lt = LeanTween.moveLocalX(go, 8.97, 0.25)
                lt.loopType = LeanTweenType.once

                self.commander_count_text = self:findText("topRight/commanderCount/bg/Text")
            end

            local text = string.format(NativeText.GetLangStr(180005), math.ceil(trigger_end - total_time))
            self.commander_count_text.text = text
        else
            if self.commander_count_vsb then
                self.commander_count_vsb = false
                local go = self.commander_count_trans.gameObject
                local lt = LeanTween.moveLocalX(go, 300, 0.25)
                lt.loopType = LeanTweenType.once
            end
        end
    end,

    showGameCountDown = function(self, total_time)
        local trigger_start = config.battle_config_commons[1].p4_start
        local trigger_end = config.battle_config_commons[1].total_time

        if total_time >= trigger_start and total_time <= trigger_end then
            if not self.final_count then
                self.final_count = self.transform:Find("topRight/count")
                local lt = LeanTween.moveLocalX(self.final_count.gameObject, 0, 0.25)
                lt.loopType = LeanTweenType.once
            end
            local left_time = math.ceil(trigger_end-total_time)
            local text_label = self:findText("topRight/count/Image/Text")
            if left_time % 2 == 0 then
                text_label.color = Color.white
            else
                text_label.color = Color.red
            end
            left_time = tostring(left_time)
            if string.len(left_time) > 1 then
                left_time = string.format("00:%s", left_time)
            else
                left_time = string.format("00:0%s", left_time)
            end
            text_label.text = left_time
        end
    end,

    showTopBar = function(self, total_time, stage_id)
        if stage_id == 2 then
            if not self.top_bar_vsb2 then
                self.top_bar_vsb2 = true
                local lt = LeanTween.moveLocalY(self.transform:Find("top/Image").gameObject, 0, 0.5)
                lt.loopType = LeanTweenType.once
                self.small_fire_trans:SetActive(false)
                self.big_fire_trans:SetActive(false)
                lt:setOnComplete(function()
                    self.small_fire_trans:SetActive(true)
                end)
            end
        elseif stage_id == 3 then
            if not self.top_bar_vsb3 then
                self.top_bar_vsb3 = true
                local lt = LeanTween.moveLocalY(self.transform:Find("top/Image").gameObject, 300, 0.5)
                lt.loopType = LeanTweenType.once
            end
        elseif stage_id == 4 then
            if not self.top_bar_vsb4 then
                self.top_bar_vsb4 = true
                local lt = LeanTween.moveLocalY(self.transform:Find("top/Image").gameObject, 0, 0.5)
                lt.loopType = LeanTweenType.once
                self.small_fire_trans:SetActive(false)
                self.big_fire_trans:SetActive(false)
                lt:setOnComplete(function()
                    self.big_fire_trans:SetActive(true)
                end)
            end
        end
    end,

    showSplashUI = function(self, path)
        if type(path) ~= 'string' then
            return
        end

        if not self.transform then
            return
        end

        local target = self.transform:Find(path)
        if not target then
            return
        end
        app:startCoroutine(function()
            target.gameObject:SetActive(true)
            coroutine.yield(WaitForSeconds(1))
            target.gameObject:SetActive(false)
        end)
    end,

    showDebuff = function(self)
        if not level_info.map_attack_info then
            return
        end

        local debuffs = level_info.map_attack_info.monsterBuff
        log.info("###### debuff info ", #debuffs)
        if debuffs and #debuffs > 0 then
            local lt = LeanTween.moveLocalX(self.debuff_layout.gameObject, -100, 0.25)
            lt.loopType = LeanTweenType.once
            local debuff_prefab = self.debuff_layout:GetChild(0).gameObject
            for k, v in ipairs(debuffs) do
                local elem = nil
                if k > self.debuff_layout.childCount then
                    elem = Object.Instantiate(debuff_prefab).transform
                    resetTransformZ(elem)
                else
                    elem = self.debuff_layout:GetChild(k-1)
                end
                if not elem then
                    return
                end
                elem.name = "Debuff_"..k
                if v.buffType == common_pb.BT_EMP then
                    local cfg = config.game_copy_emp_configs[v.buffId]
                    elem:GetComponent(UnityEngine.UI.Image):SetSprite(cfg.icon)
                    self:setScript("updown", "topRight/DebuffLayout/"..elem.name, functor.rmake(self.onClickDebuff, {ui = elem, lang_id = cfg.des}))
                end
            end
        end
    end,

    onClickDebuff = function(self, handler, data, type, param)
        if type == "down" then
            hint:fingerTipsShow(self.helper:getLangCfg(param.lang_id), param.ui)
        elseif type == "up" then
            hint:fingerTipsHide()
        end
    end,
}

