local super = require("ui.frame")
local log = logger.get("Game", "Guild Logo")
local bar_num = 4

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        -- 道具类型 42 通用
        -- 道具类型 43 建造

        -- speed_up_type =
        -- {
        --     build = 1, -- 建筑
        --     march = 2, -- 行军
        --     repair =3, -- 修理
        -- }
        self.speed_up_type = self.args.data.speed_up_type
        -- build:   element = home_entity.lua 
        -- march: 
        -- repair: 
        self.element = self.args.data.element
        -- 倒计时
        self.count_time = self.args.data.count_time

        clickEvents = 
        {
            'close',
            'content/head/completeBtn',
        }

        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        -- print('count_time:', self.helper:timeFormat(self.count_time) )

        self.onBagChanged_event_id = bag_manager.events:on('onBagChanged', 
            function()
                self:showScrollView()
            end)

        self:init()
    end,

    onUnload = function (self)
        bag_manager.events:off("onBagChanged", self.onBagChanged_event_id)
    end,

    click_close = function(self)
        frame_manager:hide('speed_up')
    end,

    click_completeBtn = function(self)
    end,

    init = function(self)
        local title_text =  self:findText('content/top/text1_Static[T2]_26')
        local name_text =   self:findText('content/head/name_text')
        local time_text =   self:findText('content/head/time_text')
        local slider = self.transform:Find('content/head/Slider'):GetComponent(UnityEngine.UI.Slider)
        local total_time = 0
        local cost_text = self:findText('content/head/cost/cost_text')
        -- local icon = self:findImage('')

        if self.speed_up_type == speed_up_type.build then
            local cfg = config.home_config_lists[self.element.cfg_id]
            title_text.text = NativeText.GetLangStr(260001)
            name_text.text = self.helper:getLangCfg(cfg.name_id)
            time_text.text = self.helper:timeFormat(self.count_time)
            total_time = self.element.next_detail_cfg.time
        elseif self.speed_up_type == speed_up_type.march then
            title_text.text = NativeText.GetLangStr(260002)
        elseif self.speed_up_type == speed_up_type.repair then
            title_text.text = NativeText.GetLangStr(260003)
        end

        local clk_trans = self.transform:Find('content/head/time_text')
        local clk = clk_trans:GetComponent(Clock)
        if not clk then
            clk = clk_trans:AddComponent(Clock)
        end
        clk.prefix = NativeText.GetLangStr(260004)
        clk.suffix = NativeText.GetLangStr(260014)
        clk:Set(self.count_time)

        app:startCoroutine(function()
            local interval = 1
            while true do
                self.count_time = self.count_time - 1
                slider.value = 1 - self.count_time / (total_time ~= 0 and total_time or frame_helper.max_number)
                cost_text.text = math.ceil(self.count_time / 60)
                if self.count_time < 0 then
                    self.count_time = 0
                    slider.value = 1
                    cost_text.text = math.ceil(self.count_time / 60)
                    break
                end

                coroutine.yield(WaitForSeconds(interval))
            end
        end)

        self:showScrollView()
    end,

    showScrollView = function(self)
        local common_type = 42
        local target_type = nil

        if self.speed_up_type == speed_up_type.build then
            target_type = 43
        elseif self.speed_up_type == speed_up_type.march then
        elseif self.speed_up_type == speed_up_type.repair then
        end

        local rst = {}
        config.item_resources:foreach(
            function(id, cfg)
                if cfg.type == common_type then
                    local count = bag_manager:getItemNum(id)
                    if count > 0 then
                        table.insert(rst, cfg)
                    end
                elseif cfg.type == target_type then
                    table.insert(rst, cfg)
                end
            end
        )

        table.sort(rst, gui_helper.sortItem)

        local parent_path = 'content/Scroll View/Viewport/Content'
        self.helper:hideAllChild(self, parent_path)

        for idx, cfg in ipairs(rst) do
            local child = self.helper:getChild(self, idx, parent_path)
            child:SetActive(true)

            local icon = child:Find('icon'):GetComponent(UnityEngine.UI.Image)
            local name_text = child:Find('name_text'):GetComponent(UnityEngine.UI.Text)
            local des_text = child:Find('des_text'):GetComponent(UnityEngine.UI.Text)

            local use_trans = child:Find('useTrans')
            local buy_trans = child:Find('buyTrans')

            icon:SetSprite(self.helper:getItemIcon(cfg.id))
            name_text.text = self.helper:getLangCfg(cfg.name_id)
            des_text.text = self.helper:getLangCfg(cfg.des_id)

            local own_num = self.helper:getItemNum(cfg.id)

            use_trans:SetActive(own_num > 0)
            buy_trans:SetActive(own_num <= 0)
            if own_num > 0 then
                local own_num_text = child:Find('useTrans/own_text'):GetComponent(UnityEngine.UI.Text)
                own_num_text.text = string.format(NativeText.GetLangStr(260012), own_num) 

                local btn = child:Find('useTrans/useBtn'):GetComponent(UnityEngine.UI.Button)
                btn.onClick:RemoveAllListeners()
                btn.onClick:AddListener(
                    function ()
                        print('使用')
                        frame_manager:show('use_slider', nil, {
                            item_id = cfg.id
                        })
                    end
                )
            else
                local cost_icon = child:Find('buyTrans/cost/Image'):GetComponent(UnityEngine.UI.Image)
                local cost_text = child:Find('buyTrans/cost/cost_text'):GetComponent(UnityEngine.UI.Text)
                local cost_tab = loadstring("return ".. cfg.buyprice)() 

                for item_id, need_num in pairs(cost_tab) do
                    cost_icon:SetSprite(self.helper:getItemIcon(item_id))
                    cost_text.text = need_num
                    break
                end

                local btn = child:Find('buyTrans/buyBtn'):GetComponent(UnityEngine.UI.Button)
                btn.onClick:RemoveAllListeners()
                btn.onClick:AddListener(
                    function ()
                        print('购买')
                        frame_manager:show('buy_slider', nil, {
                            item_id = cfg.id
                        })
                    end
                )
            end
        end
    end,
}
