local log = logger.get("Game", "Spy Report Panel")
local item_cls = require('RTS.bag_sys.item')

return class{

    ctor = function(self, panel, owner)
        self.owner = owner
        self.root = panel
        self.content = self.root:Find("Right/Viewport/Content")
        self.top_node = self.root:Find("Top")
        self.left_node = self.root:Find("Left")
        self.res_node = self.root:Find("Right/Viewport/Content/Resource")
        self.trap_node = self.root:Find("Right/Viewport/Content/Traps")
        self.defense_node = self.root:Find("Right/Viewport/Content/Defender")
        self.mass_node = self.root:Find("Right/Viewport/Content/Mass")
        self.team_node = self.root:Find("Right/Viewport/Content/Team")
        self.base_node = self.root:Find("Right/Viewport/Content/BaseInfo")
        self.build_node = self.root:Find("Right/Viewport/Content/Build")
        self.reward_node = self.root:Find("Right/Viewport/Content/Reward")


        self.res_node:SetActive(false)
        self.trap_node:SetActive(false)
        self.defense_node:SetActive(false)
        self.mass_node:SetActive(false)
        self.team_node:SetActive(false)
        self.base_node:SetActive(false)
        self.build_node:SetActive(false)
        self.reward_node:SetActive(false)

        self.team_ui = {}
        self.team_ui[1] = self.team_node
    end,

    init = function(self, data)
        self.mail = data
        self.data = data.spyReport

        self:initTop()
        self:initLeft()
        self:initRes()
        self:initTraps()
        self:initDefender()
        self:initMass()
        self:initBaseInfo()
        self:initBuild()
        self:initReward()
    end,

    getCfg = function(self, cfg_id)
        local cfg = nil
        if self.data.baseIfno.objType == common_pb.WORLD_OBJ_PLAYER then
            cfg = config.worldmap_base_configs[cfg_id]
        elseif self.data.baseIfno.objType == common_pb.WORLD_OBJ_MONSTER or self.data.baseIfno.objType == common_pb.WORLD_OBJ_RES_SHIP then
            cfg = config.worldmap_monsters[cfg_id]
        end

        if not cfg then
            log.error(string.format("################################## find cfg error ! type %d, id %d", self.data.baseIfno.objType, cfg_id))
        end

        return cfg
    end,

    initTop = function(self)
        local sender = self.top_node:Find("Sender"):GetComponent(UnityEngine.UI.Text)
        local title = self.top_node:Find("Title"):GetComponent(UnityEngine.UI.Text)
        local time_1 = self.top_node:Find("Time1"):GetComponent(UnityEngine.UI.Text)
        local time_2 = self.top_node:Find("Time2"):GetComponent(UnityEngine.UI.Text)

        sender.text = NativeText.GetLangStr(9000036)

        local cfg = self:getCfg(self.data.baseIfno.cfgId)

        if cfg then
            if self.data.objType == common_pb.WORLD_OBJ_MONSTER or self.data.objType == common_pb.WORLD_OBJ_RES_SHIP then
                title.text = string.format( "%s%sLv.%d", NativeText.GetLangStr(9000037), frame_helper:getLangCfg(cfg.monstername), cfg.level )
            elseif self.data.objType == common_pb.WORLD_OBJ_PLAYER then
                title.text = string.format( "%s%s", NativeText.GetLangStr(9000037), frame_helper:getLangCfg(cfg.name_id) )
            end
        end

        time_1.text = os.date("%Y/%m/%d", self.mail.Sendtime)
        time_2.text = os.date("%X", self.mail.Sendtime)

        local btn = self.top_node:Find("DelBtn"):GetComponent(UnityEngine.UI.Button)
        btn.onClick:RemoveAllListeners()
        btn.onClick:AddListener(
            function()
                self.owner:onClickDeleteBtn(self.mail)
            end
        )
    end,

    initLeft = function(self)
        local name = self.left_node:Find("NameBG/Name"):GetComponent(UnityEngine.UI.Text)
        local icon = self.left_node:Find("Node/Icon"):GetComponent(UnityEngine.UI.Image)

        name.text = self:getName(self.data.baseIfno)

        local cfg = nil
        if self.data.baseIfno.objType == common_pb.WORLD_OBJ_PLAYER then
            local cfg = config.worldmap_base_configs[self.data.baseIfno.cfgId]
            icon:SetSprite(cfg.iconpath..cfg.iconid)
        elseif self.data.baseIfno.objType == common_pb.WORLD_OBJ_MONSTER or self.data.baseIfno.objType == common_pb.WORLD_OBJ_RES_SHIP then
            local cfg = config.worldmap_monsters[self.data.baseIfno.cfgId]
            icon:SetSprite(cfg.iconpath..cfg.monstericon)
        end
    end,

    initRes = function(self)
        if #self.data.resItem > 0 then
            self.res_node:SetActive(true)
            local node = self.res_node:Find("List")

            local prefab = node:GetChild(0).gameObject
            local cnt = 1

            for k, v in ipairs(self.data.resItem) do
                local elem = nil
                if k > node.childCount then
                    elem = Object.Instantiate(prefab).transform
                    elem:SetParent(node)
                    resetTransformZ(elem)
                else
                    elem = node:GetChild(k-1)
                end

                elem:SetActive(true)

                local icon = elem:Find("Image"):GetComponent(UnityEngine.UI.Image)
                local count = elem:Find("Count"):GetComponent(UnityEngine.UI.Text)

                local item_cfg = item_helper:getItemCfg(v.ItemTemId)

                if item_cfg then
                    icon:SetSprite(gui_helper:getItemIconPath(item_cfg))
                    count.text = frame_helper:itemNumFormat(v.Count)
                end
            end

            for i=#self.data.resItem, node.childCount - 1 do
                local child = node:GetChild(i)
                child:SetActive(false)
            end
        else
            self.res_node:SetActive(false)
        end
    end,

    initReward = function(self)
        if self.data.dropItems and #self.data.dropItems > 0 then
            self.reward_node:SetActive(true)

            local node = self.reward_node:Find("List")
            local prefab = node:GetChild(0).gameObject

            for k, v in ipairs(self.data.dropItems) do
                local elem = nil
                if k > node.childCount then
                    elem = Object.Instantiate(prefab).transform
                    elem:SetParent(node)
                    resetTransformZ(elem)
                else
                    elem = node:GetChild(k-1)
                end

                elem:SetActive(true)

                local icon = elem:GetComponent(UnityEngine.UI.Image)
                local grade = elem:GetChild(0):GetComponent(UnityEngine.UI.Image)
                local btn = elem:GetComponent(UnityEngine.UI.Button)

                local item_cfg = item_helper:getItemCfg(v.ItemTemId)

                if item_cfg then
                    icon:SetSprite(gui_helper:getItemIconPath(item_cfg))
                    grade:SetSprite(gui_helper:getNewQualityIcon(item_cfg.color))

                    btn.onClick:RemoveAllListeners()
                    btn.onClick:AddListener(
                        function()
                            local item_inst = item_cls({cfgId = v.ItemTemId, count = frame_helper:getItemNum(v.ItemTemId)})
                            frame_manager:show("itemInfo", nil, {item = item_inst})
                        end
                    )
                end
            end

            for i=#self.data.dropItems, node.childCount - 1 do
                local child = node:GetChild(i)
                child:SetActive(false)
            end
        else
            self.reward_node:SetActive(false)
        end
    end,

    initTraps = function(self)
        if self.data.cannon and self.data.cannon.count > 0 then
            self.trap_node:SetActive(true)

            local total_cnt = self.trap_node:Find("Total/Count"):GetComponent(UnityEngine.UI.Text)

            total_cnt.text = "~"..frame_helper:itemNumFormat(self.data.cannon.count)

            local list = self.trap_node:Find("List")

            self:loadSoldiers(list, self.data.cannon.soldierCounts)

            app:startCoroutine(
                function()
                    coroutine.yield(nil)
                    self.trap_node.sizeDelta = Vector2(950, list.rect.height + 100)
                end
            )
        else
            self.trap_node:SetActive(false)
        end
    end,

    initDefender = function(self)
        self.defense_node:SetActive(true)

        local total_cnt = self.defense_node:Find("Total/Count"):GetComponent(UnityEngine.UI.Text)

        total_cnt.text = "~"..frame_helper:itemNumFormat(self.data.soldier.count)

        local list = self.defense_node:Find("List")
        self:loadSoldiers(list, self.data.soldier.soldierCounts)

        local hero_node = self.defense_node:Find("Heros")
        self:loadTeamHeros(hero_node, self.data.baseHeros)

        app:startCoroutine(
            function()
                coroutine.yield(nil)

                if #self.data.baseHeros == 0 then
                    hero_node:SetActive(false)
                    self.defense_node.sizeDelta = Vector2(950, list.rect.height + 140)
                    list.localPosition = Vector3(0, -100, 0)
                else
                    hero_node:SetActive(true)
                    self.defense_node.sizeDelta = Vector2(950, list.rect.height + 240)
                    list.localPosition = Vector3(0, -200, 0)
                end
            end
        )
    end,

    initMass = function(self)
        if #self.data.mass > 0 then
            self.mass_node:SetActive(true)

            local total_cnt = self.mass_node:Find("Total/Count"):GetComponent(UnityEngine.UI.Text)

            total_cnt.text = "~"..frame_helper:itemNumFormat(self.data.mass[1].massInfo.count)

            local list = self.mass_node:Find("List")

            self:loadSoldiers(list, self.data.mass[1].massInfo.soldierCounts)

            app:startCoroutine(
                function()
                    coroutine.yield(nil)
                    self.mass_node.sizeDelta = Vector2(950, list.rect.height + 150)
                end
            )

            self:loadTeam()
        else
            self.mass_node:SetActive(false)
        end
    end,

    loadTeam = function(self)
        local prefab = self.team_node.gameObject
        local cnt = 1

        for k, v in ipairs(self.data.mass[1].teams) do
            local elem = nil
            if k > #self.team_ui then
                elem = Object.Instantiate(prefab).transform
                elem:SetParent(self.content)
                resetTransformZ(elem)

                local idx = self.team_node:GetSiblingIndex()
                elem:SetSiblingIndex(idx + k - 1)

                table.insert( self.team_ui, elem )
            else
                elem = self.team_ui[k]
            end

            elem:SetActive(true)

            local name = elem:Find("Top/Name"):GetComponent(UnityEngine.UI.Text)
            name.text = v.playerName

            local hero_node = elem:Find("Heros")
            self:loadTeamHeros(hero_node, v.heros)

            local soldier_node = elem:Find("List")
            self:loadSoldiers(soldier_node, v.soldierInfo.soldierCounts)

            cnt = cnt + 1

            app:startCoroutine(
                function()
                    coroutine.yield(nil)
                    if #v.heros == 0 then
                        hero_node:SetActive(false)
                        elem.sizeDelta = Vector2(950, soldier_node.rect.height + 78)
                        soldier_node.localPosition = Vector3(0, -75, 0)
                    else
                        hero_node:SetActive(true)
                        elem.sizeDelta = Vector2(950, soldier_node.rect.height + 178)
                        soldier_node.localPosition = Vector3(0, -175, 0)
                    end
                end
            )
        end

        for i=cnt, #self.team_ui do
            local idx = self.defense_node:GetSiblingIndex()
            idx = idx + i - 1
            local child = self.content:GetChild(idx)
            child:SetActive(false)
        end
    end,

    loadSoldiers = function(self, list, soldiers)
        local prefab = list:GetChild(0).gameObject

        for k, v in ipairs(soldiers) do
            local elem = nil
            if k > list.childCount then
                elem = Object.Instantiate(prefab).transform
                elem:SetParent(list)
                resetTransformZ(elem)
            else
                elem = list:GetChild(k-1)
            end

            elem:SetActive(true)

            local icon = elem:Find("Icon"):GetComponent(UnityEngine.UI.Image)
            local count = elem:Find("Count"):GetComponent(UnityEngine.UI.Text)
            local name = elem:Find("Name"):GetComponent(UnityEngine.UI.Text)

            local cfg = config.soldier_soldiers[v.cfgId]

            if cfg then
                icon:SetSprite(cfg.iconpath..cfg.soldiericon)
                name.text = frame_helper:getLangCfg(cfg.soldiername)
                count.text = "~"..frame_helper:itemNumFormat(v.count)
            end
        end

        for i=#soldiers, list.childCount - 1 do
            local child = list:GetChild(i)
            child:SetActive(false)
        end
    end,

    loadTeamHeros = function(self, node, heros) 
        local prefab = node:GetChild(0).gameObject

        for k, v in ipairs(heros) do
            local elem = nil
            if k > node.childCount then
                elem = Object.Instantiate(prefab).transform
                elem:SetParent(node)
                resetTransformZ(elem)
            else
                elem = node:GetChild(k-1)
            end

            elem:SetActive(true)

            local icon = elem:Find("Icon"):GetComponent(UnityEngine.UI.Image)
            local grade = elem:Find("Grade"):GetComponent(UnityEngine.UI.Image)
            local level = elem:Find("Image/Level"):GetComponent(UnityEngine.UI.Text)
            local stars = elem:Find("Image/Stars")

            local cfg = config.commander_basics[v.officeCfgId]

            if cfg then
                icon:SetSprite(cfg.path..cfg.icon)
                grade:SetSprite(gui_helper:getNewQualityIcon(v.quality))
                level.text = v.officeLev
                gui_helper:setOfficerStars(v.starLev, stars)
            else
                log.error("##################################### find commander cfg error ! id ", v.officeCfgId)
            end
        end

        for i=#heros, node.childCount - 1 do
            local child = node:GetChild(i)
            child:SetActive(false)
        end
    end,

    initBaseInfo = function(self)
        if self.data.objType == common_pb.WORLD_OBJ_PLAYER then
            self.base_node:SetActive(true)

            local list = self.base_node:Find("List")

            local cnt = 0

            local setInfo = function(str, value)
                local elem = list:GetChild(cnt)
                local name = elem:Find("Name"):GetComponent(UnityEngine.UI.Text)
                local count = elem:Find("Count"):GetComponent(UnityEngine.UI.Text)

                name.text = str
                count.text = "~"..frame_helper:itemNumFormat(value)

                cnt = cnt + 1
            end

            setInfo(NativeText.GetLangStr(9000011), self.data.hurtCannon)
            setInfo(NativeText.GetLangStr(9000012), self.data.hurtSoldier)
            setInfo(NativeText.GetLangStr(9000013), self.data.cityDur)
            setInfo(NativeText.GetLangStr(9000014), self.data.coreDur)
        else
            self.base_node:SetActive(false)
        end
    end,

    initBuild = function(self)
        if #self.data.builds > 0 then
            self.build_node:SetActive(true)

            local list = self.build_node:Find("List")
            local prefab = list:GetChild(0).gameObject

            for k, v in ipairs(self.data.builds) do
                local elem = nil

                if k > list.childCount then
                    elem = Object.Instantiate(prefab).transform
                    elem:SetParent(list)
                    resetTransformZ(elem)
                else
                    elem = list:GetChild(k-1)
                end

                elem:SetActive(true)

                local name = elem:Find("Name"):GetComponent(UnityEngine.UI.Text)
                local level = elem:Find("Count"):GetComponent(UnityEngine.UI.Text)

                local cfg = config.home_config_lists[v.cfgId]

                if cfg then
                    name.text = frame_helper:getLangCfg(cfg.name_id)
                    level.text = "Lv."..v.level
                end
            end

            for i=#self.data.builds, list.childCount - 1 do
                local child = list:GetChild(i)
                child:SetActive(false)
            end

            app:startCoroutine(
                function()
                    coroutine.yield(nil)
                    self.build_node.sizeDelta = Vector2(950, list.rect.height + 120)
                end
            )
        else
            self.build_node:SetActive(false)
        end
    end,

    isMonster = function(self)
        return self.data.objType == common_pb.WORLD_OBJ_MONSTER or self.data.objType == common_pb.WORLD_OBJ_RES_SHIP
    end,

    isMass = function(self)

    end,

    getName = function(self, info)
        local str = info.playerName

        if info.familyName and string.len(info.familyName) > 2 then
            str = string.format( "[%s]%s", info.familyName, str )
        end

        if self:isMonster() then
            local cfg = config.worldmap_monsters[info.cfgId]
            if cfg then
                str = frame_helper:getLangCfg(cfg.monstername)
            end
        end

        return str
    end,

    show = function(self)
        self.root:SetActive(true)
    end,

    hide = function(self)
        self.root:SetActive(false)
    end,
}