local super = require("ui.frame")
local log = logger.get("Game", "Step Frame")

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        self.owner = self.args.owner
        self.chapter_data = self.args.data

        -- 路径: root/login
        -- 事件: click_login 
        clickEvents = {
            "ButtonTemp/ButtonBack",
            --"bullet_box/Background/hori/attackBtn",
            --"bullet_box/Background/hori/resetBtn",
            --"bullet_box/Background/closeBox",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self.helper.hideTrans(self.owner.transform)

        --self.bullet_box = self.transform:Find("bullet_box")
        --self.helper.hideTrans(self.bullet_box)

        self.reset_evt = game_director.events:on("onResetMapInfo", functor.make(self.onResetMapInfo, self))

        self:init()
    end,

    onUnload = function (self)
        game_director.events:remove("onResetMapInfo", self.reset_evt)
    end,

    onResetMapInfo = function(self, msg)
        local level_trans = self.right_trans[msg.copyId]
        if level_trans then
            level_trans.transform:Find("Title/gamecopy_panel_16").gameObject:SetActive(false)
        end
    end,

    init = function(self)
        if self.chapter_data then
            self:findText("Windows/Left/Title/TextTemp/text1_Static_52").text = self.helper:getLangCfg(self.chapter_data.cfg.Chapter_Name_ID)
        end

        self.levels_cfg = {}
        config.game_copy_level_groups:foreach(function (k, v) 
            table.insert(self.levels_cfg, { __id = k, value = v})
        end)

        table.sort(self.levels_cfg, function(a, b)
            return a.__id < b.__id
        end)

        local level_group = self:getLevelCfgs(self.chapter_data.id)

        -- for k, v in ipairs(self.levels_cfg) do
        --     print(v.__id, v.value.Level_Name_ID)
        -- end

        -- print("#############:", #levels)
        -- for __, vl in pairs(levels) do
        --     for ___, detail in pairs(vl) do
        --         print("@:", __, detail.Level_Name_ID)
        --     end
        -- end

        local tplt = self.transform:Find("Windows/Left/Scroll View/Viewport/Content/tplt")
        tplt.gameObject:SetActive(false)

        self.native_data = {}
        self.left_trans = {}
        self.right_trans = {}

        local start_pos = 1
        local cur_pos = 1
        local open_id = 0
        for i = 1, #level_group do
            local item = Object.Instantiate(tplt.gameObject);
            item.transform.name = "level_" .. i
            -- item.transform.parent = tplt.transform.parent
            item.transform:SetParent(tplt.transform.parent)
            item.transform.localPosition = Vector3.zero
            item.transform.localScale = Vector3.one
            item:SetActive(true)

            local cfg = level_group[i]
            local split_levels = self.helper:split(cfg.value.level, ",")

            local image = item.transform:Find("gamecopy_picture_08"):GetComponent(UnityEngine.UI.Image)
            image:SetSprite(cfg.value.path_pic .. cfg.value.pic)

            local dis_image = item.transform:Find("dis")
            local is_open = self:isBlockOpen(split_levels, self.chapter_data.id)
            dis_image:SetActive(not is_open)

            if is_open then 
                open_id = i
            end

            --偷偷改
            --[[local text = item.transform:Find("TextTemp/text1_Static_55"):GetComponent(UnityEngine.UI.Text)
            if text then
                text.text = cur_pos .. " - " .. (cur_pos + #split_levels - 1)
                cur_pos = cur_pos + #split_levels
            end]]
            local text_cfg = config.game_copy_level_groups[i]
            local text = item.transform:Find("TextTemp/text1_Static_55"):GetComponent(UnityEngine.UI.Text)
            if text then
                text.text = self.helper:getLangCfg(text_cfg.group_name)
                --text.text = '111'
            end

            --改完了
            self:setScript("click", "Windows/Left/Scroll View/Viewport/Content/" .. item.transform.name, functor.rmake(self.click_left_level, i))
            table.insert(self.native_data, { i = i, cfg = cfg, trans = item.transform })
            table.insert(self.left_trans, item.transform)
        end

        if open_id ~= 0 then
            self:click_left_level(nil, open_id)
        end 

        -- 如果还没有出战英雄列表，向服务器请求
        local chapter_id = self.chapter_data.cfg.id
        if not game_info.chapter_attend_heros[chapter_id] then
            network_sender:reqChapterAttendHeros(chapter_id)
        end
    end,

    -- 是否开启
    isBlockOpen = function(self, split_levels, chapter_id)
        -- print("isBlockOpen:", split_levels)
         for __, str in ipairs(split_levels) do
            local level_id = tonumber(str)

            for k, v in ipairs(game_info.chapter.AreaMaps) do
                if chapter_id == v.CurAreaID then
                    for a, b in ipairs(v.AreaPosInfos) do
                        if b.AreaPosID == level_id then
                            return true
                        end
                    end
                    break
                end
            end
         end
        return false
    end,

    -- 点击左边的关卡
    click_left_level = function (self, handler, index)
        print("############# click_left_level: ", index, self.left_trans and #self.left_trans)
        if self.left_select_idx == index then
            return
        end

        local dt = self:getNativeData(index)
        local split_levels = self.helper:split(dt.cfg.value.level, ",")
        local rst = self:isBlockOpen(split_levels, self.chapter_data.id)
        if not rst then
            hint:setText(NativeText.GetLangStr(170007))
            return 
        end

        self.left_select_idx = index

        local level_group = self:getLevelCfgs(self.chapter_data.id)
        for i=1, 5 do
            local map = self.transform:Find("Windows/Right/Map"..i)
            map:SetActive(i==tonumber(level_group[index].value.path))
        end
        log.info("map index is ", level_group[index].value.path)
        local ui_map = self.transform:Find("Windows/Right/Map"..level_group[index].value.path)

        -- 设置选择状态
        for __, vl in pairs(self.left_trans) do
            if vl then
                --vl:Find("select"):GetComponent(UnityEngine.UI.Image).enabled = index == __
                local btn = vl:GetComponent(UnityEngine.UI.Button)
                if index == __ then
                    btn.targetGraphic.sprite = btn.spriteState.disabledSprite--SetSprite("Assets/Arts/ui/atlas/common/gamecopy_button_01_Pressed.png")
                else
                    btn.targetGraphic.sprite = btn.spriteState.pressedSprite--btn:SetSprite("Assets/Arts/ui/atlas/common/gamecopy_button_01_normal.png")
                end
            end
        end

        local cur_chapter_id, cur_level_id = self:getCurChapterId()
        if dt then
            local i = 1
            local levels = self.helper:split(dt.cfg.value.level, ",")
            
            for __, v in ipairs(levels) do
                local item = ui_map:GetChild(i-1)
                if item then
                    local level_id = tonumber(v)
                    local cfg = config.game_copy_levels[level_id]
                    item.gameObject:SetActive(true)
                    local image = item:Find("gamecopy_picture_09"):GetComponent(UnityEngine.UI.Image)
                    image:SetSprite(cfg.path_icon .. cfg.icon)
                    local text = item:Find("Title/gamecopy_panel_13"):GetChild(0):GetComponent(UnityEngine.UI.Text)
                    text.text = self.helper:getLangCfg(cfg.Level_Name_ID)
                    item:Find("Title/gamecopy_panel_16").gameObject:SetActive(cur_level_id == level_id)
                    self:levelOver(item, level_id, index)
                    --item:Find("Lock"):SetActive(not self:isLevelOpen(level_id))
                    print("__:", level_id)
                    self:clearScript("single", "Windows/Right/"..ui_map.name.."/"..item.name.."/gamecopy_picture_09")
                    self:setScript("single", "Windows/Right/"..ui_map.name.."/"..item.name.."/gamecopy_picture_09", functor.rmake(self.click_right_level, level_id))
                    i = i + 1
                    local _, cur_level = self:getCurChapterId()
                    local select_img = item:Find("gamecopy_panel_18")
                    select_img:SetActive(cur_level == level_id)
                    self.right_trans[level_id] = item
                end
            end

            for x = i, 5 do
                ui_map:GetChild(x-1).gameObject:SetActive(false)
            end
        end

        -- 如果通关，会自动回到关卡选择界面。然后重置关卡
        if level_info.level_over then
            level_info:reset()
        end
    end, 

    -- 获得当前进行的章节id (不一定有)
    getCurChapterId = function(self)
        for k, v in ipairs(game_info.chapter.AreaMaps) do
            if v.copyId and v.copyId ~= 0 then
                return config.game_copy_levels[v.copyId].Chapter, v.copyId
            end
        end
        return nil, nil
    end,

    -- 关卡是否开启
    isLevelOpen = function(self, level_id)
        for k, v in ipairs(game_info.chapter.AreaMaps) do
            for __, vl in ipairs(v.AreaPosInfos) do
                if vl.AreaPosID == level_id then
                    return true
                end
            end
        end
        return false        
    end,

    -- 获取最远的的章节id
    getFarGroupId = function(self)
        return game_info.chapter.AreaMaps[#game_info.chapter.AreaMaps].CurAreaID
    end,

    -- 点击右边的关卡
    click_right_level = function(self, handler, data, index)
        print("############# click_right_level: ", index)

        local rst = self:isLevelOpen(index)
        if not rst then
            hint:setText(NativeText.GetLangStr(170007))
            return
        end

        self.attack_level_id = index

        local cur_chapter, cur_level = self:getCurChapterId()

        if cur_level and cur_level ~= 0 and index ~= cur_level then
            local cfg = config.game_copy_levels[cur_level]
            -- 剧情关卡向服务器请求关卡出战列表
            if cfg.level_type == 2 then
                if not game_info.level_attend_heros[index] then
                    network_sender:reqChapterAttendHeros(cur_chapter, index)
                end
            end

            local tb = {
                title = "",
                content = string.format(NativeText.GetLangStr(170006),self.helper:getLangCfg(cfg.Level_Name_ID)),
                buttons =
                {
                    {
                        btn_name = NativeText.GetLangStr(110522),
                        callback = function() 
                            frame_manager:hide('common_box')
                        end
                    },
                    {
                        btn_name = NativeText.GetLangStr(110521),
                        callback = function()
                            network_sender:resetMap(cur_level )
                            frame_manager:hide('common_box')
                        end
                    },
                },
            }

            frame_manager:show('common_box', nil, tb)
            return
        end

        frame_manager:show("level_info", self, {level_id = self.attack_level_id, chapter_id = self.chapter_data.cfg.id})
        --[[self.helper.showTrans(self.bullet_box)
        local cfg = config.game_copy_levels[index]
        local ps_cost_text = self:findText("bullet_box/Background/ps/numText")
        local tmpArgs = self.helper:getJsonTab(cfg.Cost)
        for itemId, num in pairs(tmpArgs) do
            ps_cost_text.text = num
        end

        local image = self.transform:Find("bullet_box/Background/Icon"):GetComponent(UnityEngine.UI.Image)
        image:SetSprite("Assets/Arts/ui/atlas/level/" .. cfg.icon .. ".png")
        local text = self.transform:Find("bullet_box/Background/Icon/Text"):GetComponent(UnityEngine.UI.Text)
        text.text = self.helper:getLangCfg(cfg.Level_Name_ID)

        local cur_chapter_id, cur_level_id = self:getCurChapterId()
        self.transform:Find("bullet_box/Background/hori/resetBtn").gameObject:SetActive(cur_level_id == index)]]
    end,


    click_attackBtn = function(self)
        print("攻击", self.attack_level_id)
        if self.attack_level_id then
            local chapter_id = self.chapter_data.cfg.id
            -- 如果还没有出战英雄列表，向服务器请求
            if not game_info.chapter_attend_heros[chapter_id] then
                network_sender:reqChapterAttendHeros(chapter_id)
            end

            network_sender:enterMap(self.attack_level_id)
        end
    end,

    click_resetBtn = function(self)
        print("重置")
    end,

    click_closeBox = function(self)
        self.helper.hideTrans(self.bullet_box)
    end,

    -- 获取本地数据
    getNativeData = function(self, index)
        for __, vl in pairs(self.native_data) do
            if vl.i == index then
                return vl
            end
        end
        return nil
    end,

    -- 根据章节id 获取对应的关卡 5个1组
    getLevelCfgs = function(self, chapter_id)
        local levels = {}

        for k, v in ipairs(self.levels_cfg) do
            if v.value.chapter == chapter_id then
                print("v.value:", v.value.chapter)
                table.insert(levels, v)
            end
        end

        return levels      
    end,

    click_ButtonBack = function(self)
        frame_manager:hide('step')
        self.helper.showTrans(self.owner.transform)
    end,

    levelOver = function(self,item, level_id, index)
        if level_info.level_over then
            local unload = false
            for k, v in ipairs(level_info.unload_level) do
                log.info("###### level over unload ", v, level_id)
                if level_id == v then
                    unload = true
                    break
                end
            end
            if unload then
                app:startCoroutine(function()
                    coroutine.yield(WaitForSeconds(0.5))
                    item:Find("UI_StepFrame_dajiesuo_concat"):SetActive(true)
                    coroutine.yield(WaitForSeconds(0.5))
                    item:Find("Lock"):SetActive(false)
                end)
            else
                item:Find("Lock"):SetActive(not self:isLevelOpen(level_id))
            end
        else
            item:Find("Lock"):SetActive(not self:isLevelOpen(level_id))
        end
    end,
}

