local super = require("ui.frame")

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)
        
        self.owner = self.args.owner
        self.msg = self.args.data

        -- 路径: root/login
        -- 事件: click_login 
        clickEvents = {

            "buy/Background/showProperty",
            "buy/Background/buyProperty",
            
            "buy/Background/Content/Add/lv1",
            "buy/Background/Content/Add/lv2",
            "buy/Background/Content/Add/lv3",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self.buyTrans = self.transform:Find('buy')
        self.stmtTrans = self.transform:Find('stmt')
        self.bg = self.transform:Find('bg')

        self.helper.hideTrans(self.buyTrans)
        self.helper.hideTrans(self.stmtTrans)

        self:setSelect(1)

        self:click_openBuyProperty()
    end,

    onUnload = function (self)
        
    end,

    -- 打开购买属性界面
    click_openBuyProperty = function(self)
        self.helper.showTrans(self.buyTrans)
        self:setProperty(self.msg)
    end,

    -- 打开属性界面（发送请求给服务器）
    click_showProperty = function(self) 
        -- 发送获取属性请求
        if level_info.map_info then 
            network_sender:getBoostPropInfo(level_info.map_info.mapId)
        end
    end,

    -- 显示属性界面
    showProperty = function(self)
        frame_manager:show('boost', nil, function()
            self.helper.showTrans(self.buyTrans)
        end)
    end,

    -- 购买属性
    click_buyProperty = function(self)
        local vl = self.msg.boostList[self.selectIdx]
        local cfg = config.game_copy_boosts[vl.boostId]
        local tmpArgs = {}
        if vl.boostLevel == 1 then
            tmpArgs = self.helper:getJsonTab(cfg.High_Cost)
        elseif vl.boostLevel == 2 then
            tmpArgs = self.helper:getJsonTab(cfg.Medium_Cost)
        elseif vl.boostLevel == 3 then
            tmpArgs = self.helper:getJsonTab(cfg.Low_Cost)
        end
        local cost
        for __, num in pairs(tmpArgs) do
            cost = num
        end
        local have = bag_manager:getItemNum(consts.boostId)--level_info:getResouce(consts.boostId)
        if have >= cost then
            network_sender:buyBoost(vl.boostLevel, vl.boostId)
        else
            hint:setText("购买失败!")
        end

        print("stmt_frame click_buyProperty", vl.boostLevel, vl.boostId)
    end,

    -- 选择属性
    click_lv1 = function(self)  
        self:setSelect(1)
    end,

    -- 选择属性
    click_lv2 = function(self)   
        self:setSelect(2)
    end,

    -- 选择属性
    click_lv3 = function(self)   
        self:setSelect(3)
    end,

    setSelect = function(self, idx)
        self.selectIdx = idx
        self:findImage("buy/Background/Content/Add/lv1/select").enabled = idx == 1
        self:findImage("buy/Background/Content/Add/lv2/select").enabled = idx == 2
        self:findImage("buy/Background/Content/Add/lv3/select").enabled = idx == 3
    end,

    setBattleUI = function(self, msg)
        self:setProperty(msg)
    end,

    -- required uint32 boostLevel   = 1;   //boost等级 1:高级属性 2：中级属性 3:低级属性
    -- required uint32 boostId      = 2;  //boostID
    setProperty = function(self, msg)
        table.sort(msg.boostList, function(a,b)
            return a.boostLevel > b.boostLevel
        end)

        local resText = self:findText('buy/Background/Content/Res/numText')
        local res = bag_manager:getItemNum(consts.boostId) --level_info:getResouce(consts.boostId)
        if res then
            resText.text = res
        end

        --[[local boostTsf = self.transform:Find('bg/boostText')

        if msg.copyRes then
            frameHelper.log("copyRes count:" .. #msg.copyRes)
            for k, v in ipairs(msg.copyRes) do
                frameHelper.log("copyRes:" .. v.ItemTemId .. ' ' .. v.Count)
                if v.ItemTemId == consts.boostId then
                    self.helper:makeNumEffect(boostTsf, v.Count, 'boost +')
                    break
                end
            end
        end]]

        -- frameHelper.log("setProperty:" .. #msg.boostList)

        for k, v in ipairs(msg.boostList) do
            if v and v.boostId and v.boostLevel then
                -- frameHelper.log("setProperty:" .. v.boostLevel .. ' ' .. v.boostId)
                local cfg = config.game_copy_boosts[v.boostId]
                local icon = self.transform:Find("buy/Background/Content/Add/lv" .. k):GetComponent(UnityEngine.UI.Image)
                icon:SetSprite(cfg.path_icon .. cfg.Icon)

                local nameText = self.transform:Find("buy/Background/Content/Add/lv" .. k .. "/nameText"):GetComponent(UnityEngine.UI.Text)
                nameText.text = self.helper:getLangCfg(cfg.Name_ID)

                local desText = self.transform:Find("buy/Background/Content/Add/lv" .. k .. "/desText"):GetComponent(UnityEngine.UI.Text)
                
                local quality_border = self.transform:Find("buy/Background/Content/Add/lv" .. k .. "/out"):GetComponent(UnityEngine.UI.Image)
                quality_border.color = gui_helper:quality2UnityColor(5-v.boostLevel)

                local tmpArgs = {}
                local tmpEffect = {}
                if v.boostLevel == 1 then
                    tmpArgs = self.helper:getJsonTab(cfg.High_Cost)
                    tmpEffect = self.helper:getJsonTab(cfg.High_Effect)
                elseif v.boostLevel == 2 then
                    tmpArgs = self.helper:getJsonTab(cfg.Medium_Cost)
                    tmpEffect = self.helper:getJsonTab(cfg.Medium_Effect)
                elseif v.boostLevel == 3 then
                    tmpArgs = self.helper:getJsonTab(cfg.Low_Cost)
                    tmpEffect = self.helper:getJsonTab(cfg.Low_Effect)
                end

                local resText = self.transform:Find("buy/Background/Content/Add/lv" .. k .. "/res/Text"):GetComponent(UnityEngine.UI.Text)
                for __, num in pairs(tmpArgs) do
                    resText.text = num
                end

                for __, num in pairs(tmpEffect) do
                    desText.text = string.format(self.helper:getLangCfg(cfg.des_id), num / 100)
                end
                
            end
        end
    end,

    setLevelUI = function(self, msg)
        --self.bg.gameObject:SetActive(false)
        --self.helper.showTrans(self.buyTrans)

         if #msg.boostList ~= 0 then
            self:setProperty(msg)
         end
    end,
}
