local super = require("ui.frame")

local state = {
    resident = 1,
    un_resident = 2
}

return class {
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        self.home_map = self.args.owner  

        -- 路径: root/login
        -- 事件: click_login 
        clickEvents = {
            "content/bar layout/tagBtn1",
            "content/bar layout/tagBtn2",
            "content/bar layout/tagBtn3",
            "content/bg/btn/toggleBtn",
        }

        -- 注册点击事件
        frame_helper:regClickEvents("click", self, clickEvents)

        self.tick_event_list = {
            "BUILD_UPGRADE_TICK",
            "BARRACK_TRAIN_TICK",
            "HOSPITAL_CURE_TICK",
            "TRAP_TRAIN_TICK",
            "TRAP_CURE_TICK",
            "WALL_CURE_TICK",
            "TECH_UPGRADE_TICK",
            "FORGE_UPGRADE_TICK",
            "ARSENAL_CREATE_TICK",
        }

        for __, event_name in ipairs(self.tick_event_list) do
            home_info.events:on(event_name, function(build_data, elapsed, total_time)
                self:onTimeTick(event_name, build_data, elapsed, total_time)
            end)
        end

        self.BUILD_INFO_CHANGE = home_info.events:on("BUILD_INFO_CHANGE", function(build_id) 
            self:refreshQueue() 
        end)

        self.home_scroll = self.transform:Find("content/bg/content/Scroll View")
        self.army_scroll = self.transform:Find("content/bg/content/ArmyList")
        self.army_content = self.transform:Find("content/bg/content/ArmyList/Viewport/Content")
        self.army_scroll:SetActive(false)
        self.army_list = {}
        self.army_del_list = {}
        self.army_del_list[1] = self.army_content:GetChild(0)
        self.prefab = self.army_del_list[1].gameObject

        -- 初始化数据 start
        self.expend = false
        self.cur_tag_idx = game_director:isWorldMapState() and 3 or 1
        -- 初始化数据 end
        
        self:refreshQueue()
        self:initArmyQueue()
        self:updateArmyCnt()

        self.army_add_evt = army_queue_manager.events:on("ARMYQUEUEAdd", functor.make(self.onArmyStart, self))
        self.army_remove_evt = army_queue_manager.events:on("ARMYQUEUEREMOVED", functor.make(self.onArmyFinished, self))
        self.army_count_evt = army_queue_manager.events:on("ARMYQUEUECOUNTUPDATE", functor.make(self.updateArmyCnt, self))

    end,

    onUnload = function (self)
        home_info.events:off("BUILD_INFO_CHANGE", self.BUILD_INFO_CHANGE)
        for __, event_name in ipairs(self.tick_event_list) do
            home_info.events:removeAll(event_name)
        end

        army_queue_manager.events:off("ARMYQUEUEAdd", self.army_add_evt)
        army_queue_manager.events:off("ARMYQUEUEREMOVED", self.army_remove_evt)
        army_queue_manager.events:off("ARMYQUEUECOUNTUPDATE", self.army_count_evt)
    end,

    refreshQueue = function(self)
        self.cfg = {}

        local temp_list = {}
        table.insert(self.cfg, temp_list)

        -- 建筑
        local builder_num = home_info:getBuilderNum()
        local upgrading_builds = home_info:getAllUpgradingEntity()
        for __, build_data in ipairs(upgrading_builds) do
            table.insert(temp_list, {
                queue_type = home_speed_up_type.build_upgrade,
                state = state.resident,
                cfg_id = 1,
                event_name = "BUILD_UPGRADE_TICK",
                build_id = build_data.id
            })
        end
        for i = #upgrading_builds + 1, builder_num do
            table.insert(temp_list, {
                queue_type = home_speed_up_type.build_upgrade,
                state = state.resident,
                cfg_id = 1,
                event_name = "BUILD_UPGRADE_TICK",
                build_id = 0
            })
        end

        -- 研究
        table.insert(temp_list, {
            queue_type = home_speed_up_type.tech_upgrade,
            state = state.resident,
            cfg_id = 1,
            build_cfg_id = 2,
            event_name = "TECH_UPGRADE_TICK",
            build_btn_type = 3,
        })

        -- 城墙
        table.insert(temp_list, {
            queue_type = home_speed_up_type.wall_reqair,
            state = state.un_resident,
            cfg_id = 1,
            build_cfg_id = 24,
            event_name = "WALL_CURE_TICK",
            build_btn_type = 25,
        })

        -- 兵工厂
        table.insert(temp_list, {
            queue_type = home_speed_up_type.equip_create,
            state = state.un_resident,
            cfg_id = 1,
            build_cfg_id = 11,
            event_name = "ARSENAL_CREATE_TICK",
            build_btn_type = 12,
        })

        -- 锻造厂
        table.insert(temp_list, {
            queue_type = home_speed_up_type.quantum_create,
            state = state.un_resident,
            cfg_id = 1,
            build_cfg_id = 22,
            event_name = "FORGE_UPGRADE_TICK",
            build_btn_type = 27,
        })


        local temp_list = {}
        table.insert(self.cfg, temp_list)
        -- 士兵生产队列1
        table.insert(temp_list, {
            queue_type = home_speed_up_type.soldier_create,
            state = state.resident,
            cfg_id = 1,
            build_cfg_id = 3,
            event_name = "BARRACK_TRAIN_TICK",
            build_btn_type = 20,
        })
        -- 士兵生产队列1
        table.insert(temp_list, {
            queue_type = home_speed_up_type.soldier_create,
            state = state.resident,
            cfg_id = 1,
            build_cfg_id = 4,
            event_name = "BARRACK_TRAIN_TICK",
            build_btn_type = 20,
        })
        -- 士兵生产队列1
        table.insert(temp_list, {
            queue_type = home_speed_up_type.soldier_create,
            state = state.resident,
            cfg_id = 1,
            build_cfg_id = 5,
            event_name = "BARRACK_TRAIN_TICK",
            build_btn_type = 20,
        })
        -- 士兵生产队列1
        table.insert(temp_list, {
            queue_type = home_speed_up_type.soldier_create,
            state = state.resident,
            cfg_id = 1,
            build_cfg_id = 6,
            event_name = "BARRACK_TRAIN_TICK",
            build_btn_type = 20,
        })

        -- 陷阱生产
        table.insert(temp_list, {
            queue_type = home_speed_up_type.trap_create,
            state = state.un_resident,
            cfg_id = 1,
            build_cfg_id = 24,
            event_name = "TRAP_TRAIN_TICK",
            build_btn_type = 24,
        })
        -- 陷阱修复
        table.insert(temp_list, {
            queue_type = home_speed_up_type.trap_repair,
            state = state.un_resident,
            cfg_id = 1,
            build_cfg_id = 24,
            event_name = "TRAP_CURE_TICK",
            build_btn_type = 24,
        })
        -- 医院
        table.insert(temp_list, {
            queue_type = home_speed_up_type.soldier_repair,
            state = state.un_resident,
            cfg_id = 1,
            build_cfg_id = 7,
            event_name = "HOSPITAL_CURE_TICK",
            build_btn_type = 23,
        })

        local temp_list = {}
        table.insert(self.cfg, temp_list)

        self.cache_tags = {
            self:findButton("content/bar layout/tagBtn1"),
            self:findButton("content/bar layout/tagBtn2"),
            self:findButton("content/bar layout/tagBtn3"),
        }

        self:refreshBarNum()

        self:click_toggleBtn()
    end,

    click_tagBtn1 = function(self)
        self:click_toggleBtn(true, 1)
    end,

    click_tagBtn2 = function(self)
        self:click_toggleBtn(true, 2)
    end,

    click_tagBtn3 = function(self)
        self:click_toggleBtn(true, 3)
    end,

    getQueueIcon = function(self, queue_type)
        local icon_name = ""
        config.home_config_queues:foreach(function(id, cfg)
            if cfg.queue_type == queue_type then
                icon_name = cfg.icon
            end
        end)
        return icon_name
    end,

    clickTag = function(self, tag_idx)
        if not self.cache_tags then
            return
        end

        for __, btn in ipairs(self.cache_tags) do
            if not self.expend then
                btn.interactable = true
            else
                btn.interactable = __ ~= tag_idx
            end
        end

        self.army_scroll:SetActive(tag_idx == 3)
        self.home_scroll:SetActive(tag_idx ~= 3)

        self:clearEvent()
        local list = self.cfg[tag_idx]
        if not list then
            return
        end

        local idx = 0

        local path = "content/bg/content/Scroll View/Viewport/Content"
        frame_helper:hideAllChild(self, path, true)

        for __, value in ipairs(list) do
            local elapsed, total_time = self:getLeftTime(value, value.queue_type, value.build_id)
            local visible = false
            if value.state == state.resident then
                visible = true
            else
                visible = not (elapsed <= 0 and total_time <= 0)
            end

            if visible then
                local is_un_upgrade = (elapsed <= 0 and total_time <= 0)
                if not is_un_upgrade then
                    idx = idx + 1
                    local child = frame_helper:getChild(self, idx, path)
                    child:SetActive(true)

                    local un_upgrade_trans = child:Find("un_upgrade")
                    local upgrading_trans = child:Find("upgrading")
                    frame_helper:setActive(un_upgrade_trans, is_un_upgrade)
                    frame_helper:setActive(upgrading_trans, not is_un_upgrade)

                    local slider = child:Find("upgrading/slider"):GetComponent(UnityEngine.UI.Slider)
                    local name = child:Find("upgrading/mask/name"):GetComponent(UnityEngine.UI.Text)
                    local icon = child:Find("icon"):GetComponent(UnityEngine.UI.Image)
                    local time = child:Find("upgrading/time"):GetComponent(UnityEngine.UI.Text)
                    local help_btn = child:Find("upgrading/helpBtn"):GetComponent(UnityEngine.UI.Button)
                    local speed_up_btn = child:Find("upgrading/speedUpBtn"):GetComponent(UnityEngine.UI.Button)
                    local free_btn = child:Find("upgrading/freeBtn"):GetComponent(UnityEngine.UI.Button)
                    local rcv_btn = child:Find("upgrading/rcvBtn"):GetComponent(UnityEngine.UI.Button)
                    local out_fire_btn = child:Find("upgrading/outFireBtn"):GetComponent(UnityEngine.UI.Button)
                    local go_btn = child:Find("upgrading/goBtn"):GetComponent(UnityEngine.UI.Button)

                    help_btn.onClick:RemoveAllListeners()
                    speed_up_btn.onClick:RemoveAllListeners()
                    free_btn.onClick:RemoveAllListeners()
                    rcv_btn.onClick:RemoveAllListeners()
                    out_fire_btn.onClick:RemoveAllListeners()
                    go_btn.onClick:RemoveAllListeners()

                    frame_helper:setActive(help_btn.transform, false)
                    frame_helper:setActive(speed_up_btn.transform, false)
                    frame_helper:setActive(free_btn.transform, false)
                    frame_helper:setActive(rcv_btn.transform, false)
                    frame_helper:setActive(out_fire_btn.transform, false)
                    frame_helper:setActive(go_btn.transform, false)

                    -- name.text = 
                    self:setName(value, child:Find("upgrading/mask/name"))

                    icon:SetSprite(self:getQueueIcon(value.queue_type), function()
                        icon.color = Color.white
                    end)
                    icon.color = Color.white

                    if value.queue_type == home_speed_up_type.build_upgrade then
                        help_btn.onClick:AddListener(function()
                            network_sender:reqHelpFastBuild(value.build_id, common_pb.HT_BUILD)
                        end)

                        free_btn.onClick:AddListener(function()
                            network_sender:perSonBaseSpeedUp(value.build_id, common_pb.EPB_BUILD, 1, 0, {})
                        end)

                        speed_up_btn.onClick:AddListener(function()
                            frame_manager:show("home_speed_up", nil, {
                                speed_up_type = home_speed_up_type.build_upgrade,
                                id = value.build_id
                            })
                        end)
                    elseif value.queue_type == home_speed_up_type.tech_upgrade then
                        local build_data = home_info:getBuildDataByCfgId(value.build_cfg_id)
                        local cfg_id = build_data.studio.cfgId

                        help_btn.onClick:AddListener(function()
                            network_sender:reqHelpFastBuild(cfg_id, common_pb.HT_RES)
                        end)

                        speed_up_btn.onClick:AddListener(function()
                            frame_manager:show("home_speed_up", nil, {
                                speed_up_type = home_speed_up_type.tech_upgrade,
                                id = build_data.id,
                                tech_cfg_id = cfg_id,
                            })
                        end)
                    elseif value.queue_type == home_speed_up_type.soldier_create then
                        local build_data = home_info:getBuildDataByCfgId(value.build_cfg_id)
                        -- 全部训练完成
                        if build_data.barrack.canRecv and build_data.barrack.reamintime <= 0 then
                            frame_helper:setActive(rcv_btn.transform, true)
                            time.text = NativeText.GetLangStr(650009)
                            slider.value = 1
                        else
                            frame_helper:setActive(speed_up_btn.transform, true)
                        end

                        speed_up_btn.onClick:AddListener(function()
                            local build_data = home_info:getBuildDataByCfgId(value.build_cfg_id)
                            frame_manager:show("home_speed_up", nil, {
                                speed_up_type = home_speed_up_type.soldier_create,
                                id = build_data.id
                            })
                        end)

                        rcv_btn.onClick:AddListener(function()
                            local build_data = home_info:getBuildDataByCfgId(value.build_cfg_id)
                            network_sender:receiveSoldier(build_data.id)
                        end)
                    elseif value.queue_type == home_speed_up_type.soldier_repair then
                        local build_data = home_info:getBuildDataByCfgId(value.build_cfg_id)

                        local whole_rcv = true
                        local rcv_cfg_id = nil

                        for __, s in ipairs(build_data.hospital.soldiers) do
                            if not (s.canRecv and s.reamintime <= 0) then
                                whole_rcv = false
                            end
                            if s.canRecv then
                                rcv_cfg_id = s.cfgId
                            end
                        end

                        if whole_rcv then
                            frame_helper:setActive(go_btn.transform, false)
                            frame_helper:setActive(rcv_btn.transform, true)
                            time.text = NativeText.GetLangStr(650015)
                            slider.value = 1
                        elseif rcv_cfg_id then
                            frame_helper:setActive(go_btn.transform, false)
                            frame_helper:setActive(rcv_btn.transform, true)
                        else
                            frame_helper:setActive(go_btn.transform, true)
                            frame_helper:setActive(rcv_btn.transform, false)
                        end

                        rcv_btn.onClick:AddListener(function()
                            if whole_rcv then
                                network_sender:recvTreatedSoldier(0)
                            elseif rcv_cfg_id then
                                network_sender:recvTreatedSoldier(rcv_cfg_id)
                            end
                        end)

                        go_btn.onClick:AddListener(function()
                            self:click_goBtn(value)
                        end)

                    elseif value.queue_type == home_speed_up_type.trap_create then
                        local build_data = home_info:getBuildDataByCfgId(value.build_cfg_id)
                        if build_data.guardsystem.build.canRecv and build_data.guardsystem.build.reamintime <= 0 then
                            frame_helper:setActive(speed_up_btn.transform, false)
                            frame_helper:setActive(rcv_btn.transform, true)
                            time.text = NativeText.GetLangStr(650010)
                            slider.value = 1
                        else
                            frame_helper:setActive(speed_up_btn.transform, true)
                            frame_helper:setActive(rcv_btn.transform, false)
                        end
                        speed_up_btn.onClick:AddListener(function()
                            frame_manager:show("home_speed_up", nil, {
                                speed_up_type = home_speed_up_type.trap_create,
                                id = build_data.id,
                                soldier_id = build_data.cfgId,
                            })
                        end)
                        rcv_btn.onClick:AddListener(function()
                            network_sender:receiveSoldier(build_data.id)
                        end)
                    elseif value.queue_type == home_speed_up_type.trap_repair then
                        local build_data = home_info:getBuildDataByCfgId(value.build_cfg_id)
                        local whole_rcv = true
                        local rcv_cfg_id = nil

                        for __, s in ipairs(build_data.guardsystem.repair) do
                            if not (s.canRecv and s.reamintime <= 0) then
                                whole_rcv = false
                            end
                            if s.canRecv then
                                rcv_cfg_id = s.cfgId
                            end
                        end

                        if whole_rcv then
                            frame_helper:setActive(go_btn.transform, false)
                            frame_helper:setActive(rcv_btn.transform, true)
                            time.text = NativeText.GetLangStr(650016)
                            slider.value = 1
                        elseif rcv_cfg_id then
                            frame_helper:setActive(go_btn.transform, false)
                            frame_helper:setActive(rcv_btn.transform, true)
                        else
                            frame_helper:setActive(go_btn.transform, true)
                            frame_helper:setActive(rcv_btn.transform, false)
                        end

                        rcv_btn.onClick:AddListener(function()
                            if whole_rcv then
                                network_sender:recvRepairedTrap(0)
                            elseif rcv_cfg_id then
                                network_sender:recvRepairedTrap(rcv_cfg_id)
                            end
                        end)

                        go_btn.onClick:AddListener(function()
                            self:click_goBtn(value)
                        end)

                    elseif value.queue_type == home_speed_up_type.wall_reqair then
                        speed_up_btn.onClick:AddListener(function()
                            local build_data = home_info:getBuildDataByCfgId(value.build_cfg_id)
                            frame_manager:show("home_speed_up", nil, {
                                speed_up_type = home_speed_up_type.wall_reqair,
                                id = build_data.id,
                            })
                        end)

                        out_fire_btn.onClick:AddListener(function()
                            frame_manager:show('home_wall_repair', self)
                        end)
                    elseif value.queue_type == home_speed_up_type.quantum_create then
                        speed_up_btn.onClick:AddListener(function()
                            local build_data = home_info:getBuildDataByCfgId(value.build_cfg_id)
                            frame_manager:show("home_speed_up", nil, {
                                speed_up_type = home_speed_up_type.quantum_create,
                                id = build_data.id,
                                forge_block_id = build_data.trainCenter.cfgId,
                            })
                        end)
                    elseif value.queue_type == home_speed_up_type.equip_create then
                        local e = home_info:getBuildDataByCfgId(value.build_cfg_id)
                        if e and e.armory.state == common_pb.MMAKE_MAKING then
                            frame_helper:setActive(speed_up_btn.transform, true)
                        elseif e and e.armory.state == common_pb.MMAKE_MAKE_DONE then
                            frame_helper:setActive(rcv_btn.transform, true)
                            time.text = NativeText.GetLangStr(650011)
                            slider.value = 1
                        end
                        speed_up_btn.onClick:AddListener(function()
                            local build_data = home_info:getBuildDataByCfgId(value.build_cfg_id)
                            local item_cfg = frame_helper:getItemCfg(build_data.armory.itemId)
                            config.home_config_arsenals:foreach(function(id, cfg)
                                if cfg.weapon_type == item_cfg.family then
                                    frame_manager:show("home_speed_up", nil, {
                                        speed_up_type = home_speed_up_type.equip_create,
                                        id = build_data.id,
                                        arsenal_cfg = cfg,
                                    })
                                end
                            end)
                        end)
                        rcv_btn.onClick:AddListener(function()
                            network_sender:getArmoryEquip()
                        end)
                    end
                    self:registerEvent(value.event_name, time, slider, help_btn, speed_up_btn, free_btn, rcv_btn, out_fire_btn, go_btn, value)
                end
            end
        end

        for __, value in ipairs(list) do
            local elapsed, total_time = self:getLeftTime(value, value.queue_type, value.build_id)
            local visible = false
            if value.state == state.resident then
                visible = true
            else
                visible = not (elapsed <= 0 and total_time <= 0)
            end

            if visible then
                local is_un_upgrade = (elapsed <= 0 and total_time <= 0)
                if is_un_upgrade then
                    idx = idx + 1
                    local child = frame_helper:getChild(self, idx, path)
                    child:SetActive(true)
                    
                    local icon = child:Find("icon"):GetComponent(UnityEngine.UI.Image)

                    local un_upgrade_trans = child:Find("un_upgrade")
                    local upgrading_trans = child:Find("upgrading")
                    frame_helper:setActive(un_upgrade_trans, is_un_upgrade)
                    frame_helper:setActive(upgrading_trans, not is_un_upgrade)

                    icon:SetSprite(self:getQueueIcon(value.queue_type), function()
                        icon.color = Color.white
                    end)
                    icon.color = Color.white

                    local des = child:Find("un_upgrade/mask/no_constrution"):GetComponent(UnityEngine.UI.Text)
                    local btn = child:Find("un_upgrade/goBtn"):GetComponent(UnityEngine.UI.Button)
                    btn.onClick:RemoveAllListeners()

                    if value.queue_type == home_speed_up_type.build_upgrade then
                        des.text = NativeText.GetLangStr(650004)
                        frame_helper.hideTrans(btn.transform)
                    elseif value.queue_type == home_speed_up_type.tech_upgrade then
                        frame_helper.showTrans(btn.transform)
                        local build_data = home_info:getBuildDataByCfgId(value.build_cfg_id)
                        if build_data then
                            des.text = NativeText.GetLangStr(650005)
                        else
                            des.text = NativeText.GetLangStr(650006)
                        end
                        btn.onClick:AddListener(function()
                            self:click_goBtn(value)
                        end)
                    elseif value.queue_type == home_speed_up_type.soldier_create then
                        frame_helper.showTrans(btn.transform)
                        local build_data = home_info:getBuildDataByCfgId(value.build_cfg_id)
                        if build_data then
                            des.text = string.format(NativeText.GetLangStr(650007), frame_helper:getLangCfg(config.home_config_lists[value.build_cfg_id].name_id)) 
                        else
                            des.text = string.format(NativeText.GetLangStr(650008), frame_helper:getLangCfg(config.home_config_lists[value.build_cfg_id].name_id)) 
                        end
                        btn.onClick:AddListener(function()
                            self:click_goBtn(value)
                        end)
                    end
                end
            end
        end

        home_info:onTick()
    end,

    refreshBarNum = function(self)
        local cur_num, max_num = self:getStateNum(self.cfg[1])
        local num = self:findText("content/bar layout/tagBtn1/num")
        num.text = cur_num .. "/" .. max_num
        local cur_num, max_num = self:getStateNum(self.cfg[2])
        local num = self:findText("content/bar layout/tagBtn2/num")
        num.text = cur_num .. "/" .. max_num
    end,

    getStateNum = function(self, list)
        local max_num = 0
        local cur_num = 0
        for __, value in ipairs(list) do
            local elapsed, total_time = self:getLeftTime(value, value.queue_type, value.build_id)
            local visible = false
            if value.state == state.resident then
                visible = true
            else
                visible = not (elapsed <= 0 and total_time <= 0)
            end
            if visible then
                max_num = max_num + 1
                local is_un_upgrade = (elapsed <= 0 and total_time <= 0)
                if not is_un_upgrade then
                    cur_num = cur_num + 1
                end
            end
        end
        return cur_num, max_num
    end,

    registerEvent = function(self, event_name, time, slider, help_btn, speed_up_btn, free_btn, rcv_btn, out_fire_btn, go_btn, custom_data)
        self.cache_progress[event_name] = self.cache_progress[event_name] or {}
        local list = self.cache_progress[event_name]
        table.insert(list, {
            event_name = event_name,
            time = time,
            slider = slider,
            help_btn = help_btn,
            speed_up_btn = speed_up_btn,
            free_btn = free_btn,
            rcv_btn = rcv_btn,
            out_fire_btn = out_fire_btn,
            go_btn = go_btn,
            custom_data = custom_data,
        })
    end,

    clearEvent = function(self)
        self.cache_progress = {}
    end,

    onTimeTick = function(self, event_name, build_data, elapsed, total_time)
        if self.cache_progress then
            local list = self.cache_progress[event_name]
            if list then
                for __, value in ipairs(list) do
                    if value.custom_data.queue_type == home_speed_up_type.build_upgrade then 
                        if value.custom_data.build_id == build_data.id then
                            value.time.text = frame_helper:timeFormat(elapsed)
                            value.slider.value = 1 - elapsed / total_time

                            -- 免费时间
                            if home_info:isBuildInFreeTime(build_data.id) then
                                frame_helper:setActive(value.help_btn.transform, false)
                                frame_helper:setActive(value.speed_up_btn.transform, false)
                                frame_helper:setActive(value.free_btn.transform, true)
                                frame_helper:setActive(value.rcv_btn.transform, false)
                                frame_helper:setActive(value.out_fire_btn.transform, false)
                                frame_helper:setActive(value.go_btn.transform, false)
                            -- 工会帮助
                            elseif build_data.canHelp then
                                frame_helper:setActive(value.help_btn.transform, true)
                                frame_helper:setActive(value.speed_up_btn.transform, false)
                                frame_helper:setActive(value.free_btn.transform, false)
                                frame_helper:setActive(value.rcv_btn.transform, false)
                                frame_helper:setActive(value.out_fire_btn.transform, false)
                                frame_helper:setActive(value.go_btn.transform, false)
                            -- 其他
                            else
                                frame_helper:setActive(value.help_btn.transform, false)
                                frame_helper:setActive(value.speed_up_btn.transform, true)
                                frame_helper:setActive(value.free_btn.transform, false)
                                frame_helper:setActive(value.rcv_btn.transform, false)
                                frame_helper:setActive(value.out_fire_btn.transform, false)
                                frame_helper:setActive(value.go_btn.transform, false)
                            end
                        end
                    elseif value.custom_data.queue_type == home_speed_up_type.soldier_create then 
                        if value.custom_data.build_cfg_id == build_data.cfgId then
                            frame_helper:setActive(value.help_btn.transform, false)
                            frame_helper:setActive(value.free_btn.transform, false)
                            frame_helper:setActive(value.out_fire_btn.transform, false)
                            frame_helper:setActive(value.go_btn.transform, false)

                            -- 全部训练完成
                            if build_data.barrack.canRecv and build_data.barrack.reamintime <= 0 then
                                frame_helper:setActive(value.rcv_btn.transform, true)
                                frame_helper:setActive(value.speed_up_btn.transform, false)
                                value.time.text = NativeText.GetLangStr(650009)
                                value.slider.value = 1
                            elseif build_data.barrack.canRecv then
                                frame_helper:setActive(value.rcv_btn.transform, true)
                                frame_helper:setActive(value.speed_up_btn.transform, false)
                                value.time.text = frame_helper:timeFormat(elapsed)
                                value.slider.value = 1 - elapsed / total_time
                            else
                                frame_helper:setActive(value.rcv_btn.transform, false)
                                frame_helper:setActive(value.speed_up_btn.transform, true)
                                value.time.text = frame_helper:timeFormat(elapsed)
                                value.slider.value = 1 - elapsed / total_time
                            end
                        end
                    elseif value.custom_data.queue_type == home_speed_up_type.wall_reqair then
                        frame_helper.hideTrans(value.help_btn.transform)
                        frame_helper.hideTrans(value.free_btn.transform)
                        frame_helper.hideTrans(value.rcv_btn.transform)
                        frame_helper:setActive(value.go_btn.transform, false)
                        if build_data.guardsystem.bFire then
                            frame_helper.hideTrans(value.speed_up_btn.transform)
                            frame_helper.showTrans(value.out_fire_btn.transform)
                        else
                            frame_helper.showTrans(value.speed_up_btn.transform)
                            frame_helper.hideTrans(value.out_fire_btn.transform)
                        end
                        value.time.text = frame_helper:timeFormat(elapsed)
                        value.slider.value = 1 - elapsed / total_time
                    elseif value.custom_data.queue_type == home_speed_up_type.equip_create then
                        value.time.text = frame_helper:timeFormat(elapsed)
                        value.slider.value = 1 - elapsed / total_time
                    elseif value.custom_data.queue_type == home_speed_up_type.trap_create then

                        if value.custom_data.build_cfg_id == build_data.cfgId then
                            frame_helper:setActive(value.help_btn.transform, false)
                            frame_helper:setActive(value.free_btn.transform, false)
                            frame_helper:setActive(value.out_fire_btn.transform, false)
                            frame_helper:setActive(value.go_btn.transform, false)

                            -- 全部训练完成
                            if build_data.guardsystem.build.canRecv and build_data.guardsystem.build.reamintime <= 0 then
                                frame_helper:setActive(value.rcv_btn.transform, true)
                                frame_helper:setActive(value.speed_up_btn.transform, false)
                                value.time.text = NativeText.GetLangStr(650010)
                                value.slider.value = 1
                            elseif build_data.guardsystem.build.canRecv then
                                frame_helper:setActive(value.rcv_btn.transform, true)
                                frame_helper:setActive(value.speed_up_btn.transform, false)
                                value.time.text = frame_helper:timeFormat(elapsed)
                                value.slider.value = 1 - elapsed / total_time
                            else
                                frame_helper:setActive(value.rcv_btn.transform, false)
                                frame_helper:setActive(value.speed_up_btn.transform, true)
                                value.time.text = frame_helper:timeFormat(elapsed)
                                value.slider.value = 1 - elapsed / total_time
                            end
                        end

                    elseif value.custom_data.queue_type == home_speed_up_type.soldier_repair then
                        frame_helper:setActive(value.help_btn.transform, false)
                        frame_helper:setActive(value.speed_up_btn.transform, false)
                        frame_helper:setActive(value.free_btn.transform, false)
                        frame_helper:setActive(value.out_fire_btn.transform, false)

                        local whole_rcv = true
                        local rcv_soldier_cfg_id = nil

                        for __, s in ipairs(build_data.hospital.soldiers) do
                            if not (s.canRecv and s.reamintime <= 0) then
                                whole_rcv = false
                            end
                            if s.canRecv then
                                rcv_soldier_cfg_id = s.cfgId
                            end
                        end

                        if whole_rcv then
                            frame_helper:setActive(value.go_btn.transform, false)
                            frame_helper:setActive(value.rcv_btn.transform, true)
                            value.time.text = NativeText.GetLangStr(650015)
                            value.slider.value = 1
                        elseif rcv_soldier_cfg_id then
                            frame_helper:setActive(value.go_btn.transform, false)
                            frame_helper:setActive(value.rcv_btn.transform, true)
                            value.time.text = frame_helper:timeFormat(elapsed)
                            value.slider.value = 1 - elapsed / total_time
                        else
                            frame_helper:setActive(value.go_btn.transform, true)
                            frame_helper:setActive(value.rcv_btn.transform, false)
                            value.time.text = frame_helper:timeFormat(elapsed)
                            value.slider.value = 1 - elapsed / total_time
                        end

                    elseif value.custom_data.queue_type == home_speed_up_type.trap_repair then
                        frame_helper:setActive(value.help_btn.transform, false)
                        frame_helper:setActive(value.speed_up_btn.transform, false)
                        frame_helper:setActive(value.free_btn.transform, false)
                        frame_helper:setActive(value.out_fire_btn.transform, false)

                        local whole_rcv = true
                        local rcv_cfg_id = nil

                        for __, s in ipairs(build_data.guardsystem.repair) do
                            if not (s.canRecv and s.reamintime <= 0) then
                                whole_rcv = false
                            end
                            if s.canRecv then
                                rcv_cfg_id = s.cfgId
                            end
                        end

                        if whole_rcv then
                            frame_helper:setActive(value.go_btn.transform, false)
                            frame_helper:setActive(value.rcv_btn.transform, true)
                            value.time.text = NativeText.GetLangStr(650015)
                            value.slider.value = 1
                        elseif rcv_cfg_id then
                            frame_helper:setActive(value.go_btn.transform, false)
                            frame_helper:setActive(value.rcv_btn.transform, true)
                            value.time.text = frame_helper:timeFormat(elapsed)
                            value.slider.value = 1 - elapsed / total_time
                        else
                            frame_helper:setActive(value.go_btn.transform, true)
                            frame_helper:setActive(value.rcv_btn.transform, false)
                            value.time.text = frame_helper:timeFormat(elapsed)
                            value.slider.value = 1 - elapsed / total_time
                        end
                    else
                        if build_data.cfgId == value.custom_data.build_cfg_id then
                            frame_helper.hideTrans(value.help_btn.transform)
                            frame_helper.showTrans(value.speed_up_btn.transform)
                            frame_helper.hideTrans(value.free_btn.transform)
                            frame_helper.hideTrans(value.rcv_btn.transform)
                            frame_helper.hideTrans(value.out_fire_btn.transform)
                            value.time.text = frame_helper:timeFormat(elapsed)
                            value.slider.value = 1 - elapsed / total_time
                        end
                    end
                end
            end
        end
    end,

    getQueueCfg = function(self, queue_type)
        local rst = nil
        config.home_config_queues:foreach(function(id, cfg)
            if cfg.queue_type == queue_type then
                rst = cfg
            end
        end)
        return rst
    end,

    setName = function(self, custom_data, name_trans)
        local queue_type = custom_data.queue_type
        local build_id = custom_data.build_id
        local queue_cfg = self:getQueueCfg(queue_type)
        local switch_text = name_trans:GetComponent("SwitchText")

        local temp_name = ""
        if queue_type == home_speed_up_type.build_upgrade then
            local build_data = home_info:getBuildDataById(build_id)
            local build_cfg = config.home_config_lists[build_data.cfgId]
            -- temp_name = "Lv." .. (build_data.level + 1) .. " " .. frame_helper:getLangCfg(build_cfg.name_id)
            temp_name = frame_helper:getLangCfg(build_cfg.name_id)
        elseif queue_type == home_speed_up_type.tech_upgrade then
            local e = home_info:getBuildDataByCfgId(2)
            local cfg_id = e.studio.cfgId
            local cfg = config.home_config_research_lists[cfg_id]
            local cur_level =  home_info:getTechLevel(cfg_id)
            -- temp_name = "Lv." .. (cur_level + 1) .. " " .. frame_helper:getLangCfg(cfg.name_id)
            temp_name = frame_helper:getLangCfg(cfg.name_id)
        elseif queue_type == home_speed_up_type.soldier_create then
            local build_data = home_info:getBuildDataByCfgId(custom_data.build_cfg_id)
            local build_cfg = config.home_config_lists[build_data.cfgId]
            temp_name = frame_helper:getLangCfg(build_cfg.name_id)
            
            -- 士兵全部训练完成
            if build_data.barrack.canRecv and build_data.barrack.reamintime <= 0 then
                switch_text:SetSingleContent("")
            else
                if queue_cfg.is_wrap == 1 then
                    switch_text:SetContent({
                        temp_name,
                        frame_helper:getLangCfg(queue_cfg.des)
                    })
                else
                    switch_text:SetSingleContent(temp_name)
                end
            end
            return
        elseif queue_type == home_speed_up_type.soldier_repair then
            temp_name = NativeText.GetLangStr(650012)
            local build_data = home_info:getBuildDataByCfgId(custom_data.build_cfg_id)
            local whole_rcv = true
            for __, s in ipairs(build_data.hospital.soldiers) do
                if not (s.canRecv and s.reamintime <= 0) then
                    whole_rcv = false
                end
            end
            -- 士兵全部治疗完成
            if whole_rcv then
                switch_text:SetSingleContent("")
            else
                if queue_cfg.is_wrap == 1 then
                    switch_text:SetContent({
                        temp_name,
                        frame_helper:getLangCfg(queue_cfg.des)
                    })
                else
                    switch_text:SetSingleContent(temp_name)        
                end
            end
            return
        elseif queue_type == home_speed_up_type.trap_create then
            local build_data = home_info:getBuildDataByCfgId(custom_data.build_cfg_id)
            local soldier_cfg = config.soldier_soldiers[build_data.guardsystem.build.cfgId]
            local soldier_name = frame_helper:getLangCfg(soldier_cfg.soldiername)
            temp_name = soldier_name
            
            -- 全部陷阱制造完成
            if build_data.guardsystem.build.canRecv and build_data.guardsystem.build.reamintime <= 0 then
                switch_text:SetSingleContent("")
            else
                if queue_cfg.is_wrap == 1 then
                    switch_text:SetContent({
                        temp_name,
                        frame_helper:getLangCfg(queue_cfg.des)
                    })
                else
                    switch_text:SetSingleContent(temp_name)
                end
            end
            return

        elseif queue_type == home_speed_up_type.trap_repair then
            temp_name = NativeText.GetLangStr(650013)

            local build_data = home_info:getBuildDataByCfgId(custom_data.build_cfg_id)
            local whole_rcv = true
            for __, s in ipairs(build_data.guardsystem.repair) do
                if not (s.canRecv and s.reamintime <= 0) then
                    whole_rcv = false
                end
            end
             -- 全部陷阱修理完成
             if whole_rcv then
                switch_text:SetSingleContent("")
            else
                if queue_cfg.is_wrap == 1 then
                    switch_text:SetContent({
                        temp_name,
                        frame_helper:getLangCfg(queue_cfg.des)
                    })
                else
                    switch_text:SetSingleContent(temp_name)        
                end
            end
            return
        elseif queue_type == home_speed_up_type.wall_reqair then
            local build_data = home_info:getBuildDataByCfgId(custom_data.build_cfg_id)
            if build_data.guardsystem.bFire then
                temp_name = NativeText.GetLangStr(650014)
            else
                temp_name = NativeText.GetLangStr(570006)
            end
        elseif queue_type == home_speed_up_type.quantum_create then
            local e = home_info:getBuildDataByCfgId(22)
            local cfg = config.crew_blocks[e.trainCenter.cfgId]
            temp_name = frame_helper:getLangCfg(cfg.name_id)
        elseif queue_type == home_speed_up_type.equip_create then
            local e = home_info:getBuildDataByCfgId(11)
            local item_cfg = frame_helper:getItemCfg(e.armory.itemId)
            local name = ""
            config.home_config_arsenals:foreach(function(id, cfg)
                if cfg.weapon_type == item_cfg.family then
                    name = frame_helper:getLangCfg(cfg.name)
                end
            end)
            temp_name = name
        end

        if queue_cfg.is_wrap == 1 then
            switch_text:SetContent({
                temp_name,
                frame_helper:getLangCfg(queue_cfg.des)
            })
        else
            switch_text:SetSingleContent(temp_name)
        end

        -- log.error("加速类型：", queue_type, "没有写if语句！！！")
    end,

    getLeftTime = function(self, custom_data, queue_type, build_id)
        if queue_type == home_speed_up_type.build_upgrade then
            local build_data = home_info:getBuildDataById(build_id)
            if build_data then
                return home_info:getBuildUpgradeLeftTime(build_id), build_data.totalTime
            else
                return 0, 0
            end
        elseif queue_type == home_speed_up_type.soldier_create then
            local build_data = home_info:getBuildDataByCfgId(custom_data.build_cfg_id)
            if build_data then
                local elapsed = build_data.barrack.reamintime - ( Time.realtimeSinceStartup - build_data.rmbTime )
                elapsed = elapsed >= 0 and math.floor(elapsed) or 0
                return elapsed, build_data.barrack.totaltime
            else
                return 0, 0
            end
        elseif queue_type == home_speed_up_type.soldier_repair then
            local build_data = home_info:getBuildDataByCfgId(custom_data.build_cfg_id)
            if build_data and #build_data.hospital.soldiers > 0 then
                return 0, 1
            else
                return 0, 0
            end
        elseif queue_type == home_speed_up_type.trap_create then
            local build_data = home_info:getBuildDataByCfgId(custom_data.build_cfg_id)
            if build_data then
                local pass_time = Time.realtimeSinceStartup - build_data.rmbTime
                local data = build_data.guardsystem.build
                if data and data.bTran then
                    if data.canRecv then
                        return 0, 1
                    else
                        local elapsed = data.reamintime - pass_time
                        elapsed = elapsed >= 0 and math.floor(elapsed) or 0
                        return elapsed, data.totaltime
                    end
                else
                    return 0, 0
                end
            else
                return 0, 0
            end
        elseif queue_type == home_speed_up_type.trap_repair then
            local build_data = home_info:getBuildDataByCfgId(custom_data.build_cfg_id)
            if build_data and #build_data.guardsystem.repair > 0 then
                return 0, 1
            else
                return 0, 0
            end
        elseif queue_type == home_speed_up_type.wall_reqair then
            local e = home_info:getBuildDataByCfgId(24)
            if e and e.guardsystem and e.guardsystem.bFire then
                local total_time = config.home_config_configs[1].fire_duration
                local elapsed = e.guardsystem.fire_remaintime - Time.realtimeSinceStartup + e.rmbTime
                elapsed = elapsed >= 0 and math.floor(elapsed) or 0
                return elapsed, total_time
            elseif e and e.guardsystem and e.guardsystem.repairWallRemainTime > 0 then
                local total_time = math.floor(e.guardsystem.repairWallRemainTime / ( 1 - e.guardsystem.WallHp / e.guardsystem.WallMaxHp ))
                local elapsed = e.guardsystem.repairWallRemainTime - Time.realtimeSinceStartup + e.rmbTime
                elapsed = elapsed >= 0 and math.floor(elapsed) or 0
                return elapsed, total_time
            else
                return 0, 0
            end
        elseif queue_type == home_speed_up_type.tech_upgrade then
            local e = home_info:getBuildDataByCfgId(2)
            if e and e.studio.tLevelUp > 0 then
                local elapsed = e.studio.tLevelUp - Time.realtimeSinceStartup + e.rmbTime
                local total_time = e.studio.totalTime
                return elapsed, total_time
            else
                return 0, 0
            end
        elseif queue_type == home_speed_up_type.quantum_create then
            local e = home_info:getBuildDataByCfgId(22)
            if e and e.trainCenter.remainTime > 0 and e.trainCenter.bWork then
                local elapsed = e.trainCenter.remainTime - Time.realtimeSinceStartup + e.rmbTime
                local total_time = e.trainCenter.totaltime
                return elapsed, total_time
            else
                return 0, 0
            end
        elseif queue_type == home_speed_up_type.equip_create then
            local e = home_info:getBuildDataByCfgId(11)
            if e and e.armory.state == common_pb.MMAKE_MAKING then
                local elapsed = e.armory.leftTime - Time.realtimeSinceStartup + e.rmbTime
                local total_time = e.armory.makeTime
                return elapsed, total_time
            elseif e and e.armory.state == common_pb.MMAKE_MAKE_DONE then
                return 0, 1
            else
                return 0, 0
            end
        end

        return 0, 0
    end,

    -- 参数类型不能传 table （要不然逻辑出问题)(改得我都看不懂)
    -- 参数类型不能传 table （要不然逻辑出问题)(改得我都看不懂)
    -- 参数类型不能传 table （要不然逻辑出问题)(改得我都看不懂)
    click_toggleBtn = function(self, force, tag_idx)
        -- if force ~= nil and type(force) == "boolean" then
        --     self.expend = force
        -- else
        --     if self.expend == nil then
        --         self.expend = true
        --     end
            
        -- end

        -- 消息驱动界面刷新
        if force == nil and tag_idx == nil then
            self:clickTag(self.cur_tag_idx)
        -- 手动点击的按钮事件
        elseif type(tag_idx) == "table" and type(force) == "table" then
            self.expend = not self.expend
            self:clickTag(self.cur_tag_idx)
        else
            if tag_idx then
                self.expend = true
                self.cur_tag_idx = tag_idx
                self:clickTag(self.cur_tag_idx)
            end
        end

        local content_trans = self.transform:Find("content/bg/content")
        local btn_trans = self.transform:Find("content/bg/btn")
        frame_helper:setActive(content_trans, self.expend)
        btn_trans.sizeDelta = Vector2(self.expend and 496 or -14.31, 288)
    end,

    getGridIdByBuildCfgId = function(self, build_cfg_id)
        local grid_id = nil
        config.home_config_grids:foreach(function(id, cfg)
            local has_build = home_info:isGridHasBuild(id)
            if not has_build then
                local list = loadstring("return " .. cfg.can_build)()
                for idx, cfg_id in ipairs(list) do
                    if build_cfg_id == cfg_id then 
                        grid_id = id
                    end
                end
            end
        end)
        return grid_id
    end,

    click_goBtn = function(self, custom_data)
        local build_data = home_info:getBuildDataByCfgId(custom_data.build_cfg_id)
        local grid_id = build_data and build_data.grid_id or self:getGridIdByBuildCfgId(custom_data.build_cfg_id)
        if not grid_id then
            return
        end

        local is_lock = home_info:isGridLock(grid_id)
        if is_lock then
            hint:setText(NativeText.GetLangStr(999015))
            return
        end

        -- 在个人基地（否贼在世界地图）
        if self.home_map then
            self.home_map:clickEntityByGridId(grid_id, custom_data.build_btn_type or 1)
        else
            game_scheduler:addHomeMapEvent(grid_id, custom_data.build_btn_type)
            game_director:changeState('loading', {scene_name = 'home', next_state = 'home'})
        end
    end,

    onArmyStart = function(self, data)
        self:addArmyQueue(data)
    end,

    onArmyFinished = function(self, id)
        self:removeArmyQueue(id)
    end,

    initArmyQueue = function(self)
        for k, v in pairs(army_queue_manager.list) do
            self:addArmyQueue(v)
        end
    end,

    addArmyQueue = function(self, data)
        local elem = self.army_list[data.id] and self.army_list[data.id].ui

        if elem then
            self:setArmyInfo(elem, data)
        else
            elem = self.army_del_list[1]

            if not elem then
                elem = Object.Instantiate(self.prefab).transform
                elem:SetParent(self.army_content)
                resetTransformZ(elem)
            else
                table.remove( self.army_del_list, 1 )
            end

            elem:SetActive(true)
            self:setArmyInfo(elem, data)
        end

        self.army_list[data.id] = {ui = elem, data = data}
    end,

    removeArmyQueue = function(self, id)
        local ui = self.army_list[id] and self.army_list[id].ui

        if ui then
            ui:SetActive(false)
            self.army_list[id] = nil
            table.insert( self.army_del_list, ui )
        end
    end,

    setArmyInfo = function(self, elem, data)
        local icon = elem:Find("icon"):GetComponent(UnityEngine.UI.Image)
        local name = elem:Find("upgrading/mask/name"):GetComponent(UnityEngine.UI.Text)
        local speed_up_btn = elem:Find("upgrading/speedUpBtn"):GetComponent(UnityEngine.UI.Button)
        local go_btn = elem:Find("upgrading/GoBtn"):GetComponent(UnityEngine.UI.Button)
        local btn = elem:Find("bg"):GetComponent(UnityEngine.UI.Button)

        icon:SetSprite(data.icon_path)
        name.text = data.str

        speed_up_btn.transform:SetActive(data.show_up)
        go_btn.transform:SetActive(not data.show_up and data.show_go)

        if data.show_up then
            speed_up_btn.onClick:RemoveAllListeners()
            speed_up_btn.onClick:AddListener(
                function()
                    army_queue_manager:armyLineSpeedUp(data.march_info.instId, data.left_time, data.total_time, data.marchAction)
                end
            )
        else
            if data.show_go then
                go_btn.onClick:RemoveAllListeners()
                go_btn.onClick:AddListener(
                    function()
                        world_map_helper:jumpToHold(data.dest_id, data.pos.x, data.pos.y)
                    end
                )
            end
        end

        btn.onClick:RemoveAllListeners()
        btn.onClick:AddListener(
            function()
                frame_manager:show("army_queue_details")
            end
        )
    end,

    updateArmyCnt = function(self)
        local count = self.transform:Find("content/bar layout/tagBtn3/num"):GetComponent(UnityEngine.UI.Text)
        count.text = army_queue_manager.cur_count.."/"..army_queue_manager.max_count
    end,

    update = function(self, dt)
        if self.army_list then
            for k, v in pairs(self.army_list) do
                if v.data.left_time > 0 then
                    local time_text = v.ui:Find("upgrading/time"):GetComponent(UnityEngine.UI.Text)
                    local slider = v.ui:Find("upgrading/slider"):GetComponent(UnityEngine.UI.Slider)

                    time_text.text = gui_helper:timeFormat(v.data.left_time)
                    slider.value = 1 - v.data.left_time / v.data.total_time

                    v.data.left_time = v.data.left_time - dt
                    v.data.left_time = v.data.left_time > 0 and v.data.left_time or 0
                end
            end
        end
    end,
}

