local log = logger.get("Game", "Formation Adjust Helper")

local cls = class{
    ctor = function(self)
        self.teams = {}
        self.team_heros = {} --MHeroFormation
        self.idle_heros = {} --hero.lua
        self.curr_team = nil
        self.events = require("base.event").new()
    end,

    clear = function(self)
        self.team_heros = {}
        self.idle_heros = {}
        self.curr_team = nil

        for i=#self.teams, 1, -1 do
            self:removeTeam(i)
        end
        self.teams = {}
    end,

    inTeams = function(self, team_id)
        for k, v in ipairs(self.teams) do
            if v.teamId == team_id then
                return true, k
            end
        end

        return false
    end,

    setIdleHero = function(self, heros)
        self.idle_heros = heros
    end,

    setCurrTeam = function(self, team)
        local in_teams, idx = self:inTeams(team.teamId)
        if self.curr_team and in_teams then
            self:setHeroIdx(self.curr_team, 0)

            self.curr_team = team
            self:setHeroIdx(self.curr_team, idx)

            self.teams[idx] = self.curr_team
        else
            self.curr_team = team
        end
    end,

    getCurrTeam = function(self)
        return self.curr_team
    end,

    selectHero = function(self, hero, location)
        local loc_hero = self:findLocationHero(self.curr_team, location)
        local flag = true
        
        if self.team_heros[hero.id] and not self:heroInCurrTeam(hero.id) then
            if loc_hero then
                local hero_inst = hero_manager:getHero(loc_hero.heroId)

                if hero_inst then
                    local hero_location = self.team_heros[hero.id].location
                    local idx = self.team_heros[hero.id].hero.team_idx
                    hero_inst.team_idx = idx

                    self.team_heros[hero_inst.id] = {hero = hero_inst, location = hero_location}
    
                    loc_hero.location = 0
                    self:currTeamAddHero(hero.id, location)
                end
            else
                self:currTeamAddHero(hero.id, location)
            end
        else
            local in_team, team_hero = self:heroInCurrTeam(hero.id)

            if in_team then
                if loc_hero then
                    if loc_hero.heroId == team_hero.heroId then
                        team_hero.location = 0
                        self:resetHeroIdx(hero.id)
                        flag = false
                    else
                        loc_hero.location = team_hero.location
                        team_hero.location = location
                    end
                else
                    team_hero.location = location
                end
            else
                if loc_hero then
                    loc_hero.location = 0
                    self:resetHeroIdx(loc_hero.heroId)
                    --self.team_heros[loc_hero.heroId] = nil
                end

                if team_hero then
                    team_hero.location = location
                else
                    self:currTeamAddHero(hero.id, location)
                end
            end
        end
        
        if flag then
            --self.team_heros[hero.id] = {hero = hero, location = location}
        end
    end,

    getHeroFormationId = function(self, hero_id)
        local formation_id = 0
        for k, v in ipairs(self.teams) do
            local h = self:findHeroInTeam(v, hero_id)
            if h then
                formation_id = v.formationId
                break
            end
        end

        return formation_id
    end,

    locationCheck = function(self, formation_id, location)
        
    end,

    currTeamAddHero = function(self, id, location)
        local curr_heros = self.curr_team.formationHero
        local team_hero = curr_heros:add()
        team_hero.heroId = id
        team_hero.location = location
    end,

    refreshAllTeam = function(self)
        local len = #self.teams
        local list = {}
        for i=1, len do
            local team = self.teams[i]
            list[i] = common_pb.MFormation()
            list[i].formationId = team.formationId
            list[i].teamId = team.teamId or 0
            list[i].sword = team.sword
            list[i].contain = team.contain
        end

        for k, v in pairs(self.team_heros) do
            local idx = v.hero.team_idx
            if idx ~= 0 and v.location > 0 then
                local heros = list[idx].formationHero
                local hero = heros:add()
                hero.heroId = v.hero.id
                hero.location = v.location
            end
        end

        for i=#list, 1, -1 do
            local del = true
            for k, v in ipairs(list[i].formationHero) do
                if v.location > 0 then
                    del = false
                end
            end

            if del then
                table.remove( list, i )
            end
        end

        self.teams = list

        self:refreshIdleHeros()
    end,

    refreshIdleHeros = function(self)
        local list = {}
        for k, v in ipairs(self.idle_heros) do
            if not self.team_heros[v.id] then
                table.insert( list, v )
            end
        end

        self.idle_heros = list
    end,

    setAllHeroIdx = function(self)
        for k, v in ipairs(self.teams) do
            self:setHeroIdx(v, k)
        end
    end,

    addTeam = function(self, team)
        table.insert( self.teams, team)
        self:setHeroIdx(team, #self.teams)
        self:refreshAllTeam()
    end,

    removeTeam = function(self, idx)
        local team = self.teams[idx]

        if team  then
            self:setHeroIdx(team, 0)
            table.remove(self.teams, idx)
        end
        
        self:setAllHeroIdx()
        self:refreshAllTeam()
    end,

    setHeroIdx = function(self, team, idx)
        for k, v in ipairs(team.formationHero) do
            local hero = hero_manager:getHero(v.heroId)
            if hero then
                if idx == 0 then
                    v.location = 0
                    self:resetHeroIdx(hero.id)
                elseif v.location > 0 then
                    hero.team_idx = idx
                    self.team_heros[hero.id] = {hero = hero, location = v.location}
                end
            end
        end
    end,

    resetHeroIdx = function(self, id)
        local hero = hero_manager:getHero(id)
        if hero then
            hero.team_idx = 0
            self.team_heros[hero.id] = nil
            self:addIdleHero(hero)
        end
    end,

    addIdleHero = function(self, hero)
        for k, v in ipairs(self.idle_heros) do
            if hero.id == v.id then
                return
            end
        end
        
        table.insert(self.idle_heros, hero)
    end,
    
    getHeroList = function(self)
        local heros = hero_manager:getUnDispatchedHeros()
        
        table.sort( heros, function(a, b) return a.officer.cfg.color > b.officer.cfg.color end )

        return heros
    end,

    heroInCurrTeam = function(self, id)
        if not self.curr_team then
            return false, nil
        end

        for _, v in ipairs(self.curr_team.formationHero) do
            if v.heroId == id then
                return v.location > 0, v
            end
        end

        return false, nil
    end,

    findHeroInTeam = function(self, team, hero_id)
        if not team then
            return nil
        end

        for _, v in ipairs(team.formationHero) do
            if v.heroId == hero_id and v.location > 0 then
                return v
            end
        end

        return nil
    end,

    findLocationHero = function(self, team, location)
        if not team then
            return nil
        end

        for _, v in ipairs(team.formationHero) do
            if v.location == location then
                return v
            end
        end

        return nil
    end,

    getBestTeam = function(self, heros)
        local list = heros or self.idle_heros

        if #list == 0 then
            return nil
        end
        
        local formations = level_formation:getEnableFormations()
        local team = nil
        for k, v in ipairs(formations) do
            local mf = self:generateFormation(v, list)
            if team then
                if team.sword < mf.sword then
                    team = mf
                end
            else
                team = mf
            end
        end

        return team
    end,

    generateFormation = function(self, formation_id, heros)
        local fmt_cfg = level_formation:getFormationCfg(formation_id)

        table.sort( heros, function(a,b) return a.battle_force > b.battle_force end )

        local used_heros = {}

        local filterHero = function(tp)
            for k, v in ipairs(heros) do
                if (v.tank_cfg.type == ("type" .. tp) or tp == 0 ) and not used_heros[v.id] then
                    used_heros[v.id] = true
                    return v
                end
            end

            return nil
        end

        local mf = common_pb.MFormation()
        mf.formationId = formation_id

        for location, tp in pairs(fmt_cfg) do
            if location ~= 0 then
                local hero = filterHero(tp)
                if hero then
                    local h = mf.formationHero:add()
                    h.heroId = hero.id
                    h.location = location
                    mf.sword = (mf.sword or 0 ) + hero.battle_force
                end
            end
        end

        return mf
    end,
}

formation_adjust_helper = formation_adjust_helper or cls()

return formation_adjust_helper