local log = logger.get("Game", "frame_helper")
localization_manager = require('game.localization_manager')
_cached_recttransform_pos = {}
frame_helper =  {

    max_number = 2^54,

    -- 注册事件
    regClickEvents = function(self, regType, frame, clickEvents)
        if type(clickEvents) == "table" then
            for _, path in pairs(clickEvents) do
                local rst = string.find(path, "/")
                if rst then
                    local temp = self.lastStringOf(self, path, "/")
                    if temp then
                        local event = frame[regType .. "_" .. temp]
                        if event then
                            frame:setScript(regType, path, functor.make(event, frame))
                        else
                            logger.error("register event failed:", path)
                        end
                    else
                        logger.error("register event failed:", path)
                    end
                else
                    local event = frame[regType .. "_"  .. path]
                    if event then
                        frame:setScript(regType, path, functor.make(event, frame))
                    else
                        logger.error("register event failed:", path)
                    end
                end
            end
        elseif type(clickEvents) == "string" then
            local rst = string.find(clickEvents, "/")
            if rst then
                local temp = self.lastStringOf(self, path, "/")
                if temp then
                    local event = frame[regType .. "_" .. temp]
                    if event then
                        frame:setScript(regType, path, functor.make(event, frame))
                    else
                        logger.error("register event failed:", path)
                    end
                else
                    logger.error("register event failed:", path)
                end
            else
                local event = frame[regType .. "_" .. path]
                if event then
                    frame:setScript(regType, path, functor.make(event, frame))
                else
                    logger.error("register event failed:", path)
                end
            end
        else
            logger.error("register event failed:", path)
        end
    end, 
    
    -- lastStringOf("123/456", "/") 返回456
    lastStringOf = function(self, path, mark)
        for i = 0, string.len(path) do
            local idx = string.len(path) - i
            if idx > 0 then
                if string.sub(path, idx, idx) == mark then
                    return string.sub(path, idx + 1)
                end
            end
        end
        return nil        
    end,

    -- 设置GameObject的layermask
    setChildLayer = function(self, t, layer)
        if type(layer) == "string" then
            layer = UnityEngine.LayerMask.NameToLayer(layer)
        end

        if type(layer) ~= "number" then
            logger.error("setChildLayer error", layer)
            return
        end
        t.gameObject.layer = layer;
        for i = 0, t.childCount - 1 do 
            local child = t:GetChild(i);
            child.gameObject.layer = layer;
            self:setChildLayer(child, layer);
        end
	end,

    -- 加载配置文件
    loadConfig = function()

        dofile('gui/gui_enums.lua')

        -- lang文件夹下的配置文件
        local names =
        {
            "item.lua",
            "attribute_config.lua",
            "garage.lua",
            "partscale.lua",
            "monster_group.lua",
            "worldmap.lua",
            "home_config.lua",
            "dropbag.lua",
            "task.lua",
            "formation.lua",
            "announcement.lua",
            "exp.lua",
            "dialog.lua",
            "union.lua",
            "union_gift.lua",
            "union_building.lua",
            "store.lua",
            "camp.lua",
            "passive_skill.lua",
            "mail.lua",
            "system_config.lua",
            "buff.lua",
            "captain.lua",
            "crew.lua",
            "tavern.lua",
            "active.lua",
            "system_unlock.lua",
        }

        for __,name in pairs(names) do
            local fullPath = "config/" .. name
            dofile(fullPath)
        end

        if not lang_cfg then
            lang_cfg = {}
            dofile("config/lang/lang_config.lua")
            config.lang_lists:foreach(function(k, v)
                local fullPath = "config/lang/" .. v.sheet_id .. ".lua"
                dofile(fullPath)

                local cfg = gConfig[v.sheet_id .. "_langs"]
                for k, v in pairs(cfg) do
                    if lang_cfg[k] then
                        log.error('Repeat key in lang cfg:'..k)
                    else
                        lang_cfg[k] = v
                    end
                end 
            end)
        end

        -- for __,name in pairs(lang_names) do
        --     local fullPath = "config/lang/" .. name .. ".lua"
        --     dofile(fullPath)
        -- end

        -- if not lang_cfg then
        --     lang_cfg = {}
        --     for i = 1, #lang_names - 1 do
        --         local cfg = gConfig[lang_names[i] .. "_langs"]
        --         for k, v in pairs(cfg) do
        --             if lang_cfg[k] then
        --                 log.error('Repeat key in lang cfg:'..k)
        --             else
        --                 lang_cfg[k] = v
        --             end
        --         end 
        --     end

        --     local cfg = gConfig.lang_configs
        --     for k, v in pairs(cfg) do
        --         if lang_cfg[k] then
        --             log.error('Repeat key in lang cfg:'..k)
        --         else
        --             lang_cfg[k] = v
        --         end
        --     end 
        -- end

        local item_names =
        {
            "item_resources",
            "item_useables",
            "item_pieces",
        }

        if not item_cfg then
            item_cfg = {}
            for i = 1, #item_names do
                local cfg = gConfig[item_names[i]]
                for k, v in pairs(cfg) do
                    if item_cfg[k] then
                        log.error('Repeat key in item cfg:'..k)
                    else
                        item_cfg[k] = v
                    end
                end 
            end
        end
    end,

    -- 设置labeltext
    setText = function(self, frame, path, id, params)
        if frame == nil or type(path) ~= "string" or type(id) ~= "number" then
            return
        end

        local text = frame:findText(path)
        if text == nil then
            return
        end
        
        text.fontSize = config.gametext_gametexts[id].fontSize
        local rst, tempColor = UnityEngine.ColorUtility.TryParseHtmlString(config.gametext_gametexts[id].color, Slua.out);
        text.color = tempColor

        if params then
            text.text = string.replace_var(config.gametext_gametexts[id].content, function (name)
                return params[tonumber(name)]
            end)
        else
            text.text = config.gametext_gametexts[id].content
        end
    end,

    copyTrans = function(self, frame, path)
        if frame == nil or path == nil then
            return nil
        end

        if type(path) == "string" then
            local target = frame.transform:Find(path) 
            if target == nil then
                return nil
            else
                local newObj = Object.Instantiate(target)
                newObj:SetParent(target.parent.transform)
                local targetRect = target.transform:GetComponent(UnityEngine.RectTransform)
                local newObjRect = newObj.transform:GetComponent(UnityEngine.RectTransform)
                newObjRect.offsetMax = targetRect.offsetMax
                newObjRect.offsetMin = targetRect.offsetMin
                newObj.transform.localScale = targetRect.localScale
                newObj.transform.localPosition = targetRect.localPosition
                return newObj.transform
            end
        else
            print("copyTrans failed:", path)
            return nil
        end
    end,

    reverseTable = function(tab)
        local tmp = {}
        for i = 1, #tab do
            local key = #tab
            tmp[i] = table.remove(tab)
        end

        return tmp
    end,

    -- 遍历配置文件
    containKeyCfg = function(tb, key)
        local rst = false

        if tb == nil or key == nil then
            return rst
        end

        tb:foreach(function (k, v)
            if k == key then
                rst = true
            end
        end)

        return rst
    end,

    isTransShow = function(obj)
        if obj then
            local sr = obj.transform:GetComponentInChildren(UnityEngine.UI.ScrollRect)
            local srs = obj.transform:GetComponentInChildren(UnityEngine.UI.ScrollRectSliderLoad)
            local rect = obj:GetComponent(UnityEngine.RectTransform)
            if sr or srs then
                return rect.anchoredPosition == Vector2.zero
            else
                if rect then
                    return rect.localScale == Vector3.one
                end
            end
        end
    end,

    showTrans = function(obj)
        if obj then
            local sr = obj.transform:GetComponentInChildren(UnityEngine.UI.ScrollRect)
            local srs = obj.transform:GetComponentInChildren(UnityEngine.UI.ScrollRectSliderLoad)
            local rect = obj:GetComponent(UnityEngine.RectTransform)
            if sr or srs then
                rect.anchoredPosition = Vector2.zero
            else
                if rect then
                    rect.localScale = Vector3.one
                    return
                end
            end
        end
    end,

    hideTrans = function(obj)
        if obj then
            local sr = obj.transform:GetComponentInChildren(UnityEngine.UI.ScrollRect)
            local srs = obj.transform:GetComponentInChildren(UnityEngine.UI.ScrollRectSliderLoad)
            local rect = obj:GetComponent(UnityEngine.RectTransform)
            if sr or srs then
                rect.anchoredPosition = Vector2(-10000, 0)
            else
                if rect then
                    rect.localScale = Vector3.zero
                    return
                end
            end
        end
    end,

    setActive = function(self, obj, vsb)
        if vsb then
            self.showTrans(obj)
        else
            self.hideTrans(obj)
        end
    end,

    --[[
    -- 显示界面
    showTrans = function(obj)
        if obj == nil then
            return
        end

        local rect = obj:GetComponent(UnityEngine.RectTransform)
        if rect ~= nil then
            if rect.anchoredPosition.x == -10000 then
                if _cached_recttransform_pos[rect] then
                    rect.anchoredPosition = _cached_recttransform_pos[rect]
                else
                    rect.anchoredPosition = Vector2.zero
                end
            end
            -- rect.offsetMax = Vector2.zero
            -- rect.offsetMin = Vector2.zero
        end

        local tsf = obj:GetComponent(UnityEngine.Transform)
        if tsf ~= nil then
            tsf.localPosition = Vector3.zero
            return
        end
    end,

    -- 隐藏界面
    hideTrans = function(obj)
        if obj == nil then
            return
        end

        local rect = obj:GetComponent(UnityEngine.RectTransform)
        if rect ~= nil then
            if rect.anchoredPosition.x ~= -10000 then
                _cached_recttransform_pos[rect] = rect.anchoredPosition
            end

            rect.anchoredPosition = Vector2(-10000, 0)
            -- rect.offsetMax = Vector2(-10000, 0)
            -- rect.offsetMin = Vector2(-10000, 0)
            return
        end

        local tsf = obj:GetComponent(UnityEngine.Transform)
        if tsf ~= nil then
            tsf.localPosition = Vector3(10000, 0, 0)
            return
        end
    end,
    ]]

    -- 获得道具的icon配置
    getItemCfg = function(self, cfg_id)
        if type(cfg_id) == "string" then
            cfg_id = tonumber(cfg_id)
        end
        return item_helper:getItemCfg(cfg_id)
    end,

    -- 获得道具的icon路径
    getItemIcon = function(self, item_id)
        if type(item_id) ~= "number" then
            item_id = tonumber(item_id)
        end

        local cfg = self:getItemCfg(item_id)
        if cfg then
            return cfg.path .. cfg.icon
        end

        log.error("item找不到配置!!!", item_id)
        return ""
    end,

    -- 获得道具名字
    getItemName = function(self, item_id)
        if type(item_id) ~= "number" then
            item_id = tonumber(item_id)
        end

        local cfg = self:getItemCfg(item_id)
        if cfg then
            return self:getLangCfg(cfg.name_id)
        end

        log.error("item找不到配置!!!", item_id)
        return ""
    end,

    -- 获得道具描述
    getItemDes = function(self, item_id)
        if type(item_id) ~= "number" then
            item_id = tonumber(item_id)
        end

        local cfg = self:getItemCfg(item_id)
        if cfg then
            return self:getLangCfg(cfg.des_id)
        end

        log.error("item找不到配置!!!", item_id)
        return ""
    end,

    -- 获得道具品质名字
    getItemQualityIcon = function(self, item_id)
        if type(item_id) ~= "number" then
            item_id = tonumber(item_id)
        end

        local cfg = self:getItemCfg(item_id)
        if cfg then
            return gui_helper:getQualityIconPath(cfg.color)
        end

        log.error("item找不到配置!!!", item_id)
        return ""
    end,

    getItemNumFormat = function(self, item_id, need_num)
        local own_num = self:getItemNum(item_id)
        local rst = own_num >= need_num
        need_num = self:itemNumFormat(need_num)
        own_num = self:itemNumFormat(own_num)
        if rst then
            return string.format("<color=#00FF00FF>%s</color>/<color=#FFFFFFFF>%s</color>", own_num, need_num), rst
        else
            return string.format("<color=#FF0000FF>%s</color>/<color=#FFFFFFFF>%s</color>", own_num, need_num), rst
        end
    end,

    sortItem = function(a, b)
        local a_num = bag_manager:getItemNum(a.id)
        local b_num = bag_manager:getItemNum(b.id)

        if (a_num > 0 and b_num > 0) or (a_num == 0 and b_num == 0) then 
            if a.type > b.type then
                return true
            elseif a.type == b.type then
                return a.id < b.id
            else
                return false
            end
        else
            return a_num > b_num
        end
    end,

    getDiamondNum = function(self)
        return self:getItemNum(30001)
    end,

    getResToDiamondNum = function(self, item_id, need_num)
        local item_cfg = self:getItemCfg(item_id)
        local item_type = item_cfg.type

        local cost_cfg = nil
        config.item_speed_ups:foreach(function(id, cfg)
            if cfg.type == item_type then
                cost_cfg = cfg
            end
        end)

        if cost_cfg then
            local single_price = cost_cfg.cost
            need_num = need_num / cost_cfg.unit
            local total_price = single_price * need_num
            if total_price <= 0 then
                return 0
            else
                total_price = math.floor(total_price)
                return total_price == 0 and 1 or total_price
            end
        else
            log.error('config : config.item_speed_ups not exist cfg', item_type)
            return 0
        end
    end,

    getSpeedUpCost = function(self, item_type, time)
        return gui_helper:getSpeedUpCost(item_type, time)
    end,

    getItemNum = function(self, item_id)
        local DIAMOND    = 30001 -- 钻石
        local GOLD       = 30002 -- 金币
        local STEEL      = 32001 -- 钢材
        local WOOD       = 32002 -- 木头
        local BRICK      = 32003 -- 砖头
        local NUCLEAR    = 32004 -- 核能

        local COPYENERGY = 30004 -- 行动力
        local GUILD_COIN    = 30009 -- 公会币

        -- 领主经验
        local LORD_EXP = local_lord.EXP_ID

        if item_id == STEEL then
            return local_player.props[protocol_pb.PROR_STEEL]
        elseif item_id == WOOD then
            return local_player.props[protocol_pb.PROP_WOOD]
        elseif item_id == BRICK then
            return local_player.props[protocol_pb.PROP_BRICK]
        elseif item_id == NUCLEAR then
            local rst = local_player.props[protocol_pb.PROP_FOOD] - (Time.realtimeSinceStartup - local_player.food_rmb_time) * local_player.props[protocol_pb.PROP_REDUCEFOOD] 
            rst = rst < 0 and 0 or math.floor(rst)
            return rst
        elseif item_id == DIAMOND then
            return local_player.props[protocol_pb.PROP_DIAMOND]
        elseif item_id == GOLD then
            return local_player.props[protocol_pb.PROP_GOLD]
        elseif item_id == COPYENERGY then
            return local_player.props[protocol_pb.PROP_COPYENERGY]
        elseif item_id == GUILD_COIN then
            return local_player.props[protocol_pb.PROP_FAMILY_COIN]
        elseif item_id == LORD_EXP then
            return local_lord.exp
        end

        local rst = (bag_manager:getItemNum(item_id) or 0) + (crew_manager:getItemNumByCfgId(item_id) or 0)
        return rst
    end,

    -- 获得lang的配置数据
    getLangCfg = function(self, id)
        local lang_value = localization_manager:get()
        if lang_cfg[id] then
            return lang_cfg[id][lang_value]
        end
        log.warn("找不到lang配置:", tostring(id), debug.traceback())
        -- log.info(debug.traceback())
        return string.format("lang %s 未配置", tostring(id))
    end,

    -- 解析json格式的字符串
    getJsonTab = function(self, content)
        return loadstring('return ' .. content)()
    end,

    getTable = function(self, str)
        return loadstring('return ' .. str)()
    end,

    getItemBuyPrice = function(self, item_id)
        local cfg = item_helper:getItemCfg(item_id)
        local tab = self:getTable(cfg.allprice)
        for item_id, price in pairs(tab) do
            if item_id == 30001 then
                return price
            end
        end
        log.error(string.format("物品id:%s 没有allprice字段", item_id))
        return 0
    end,

    -- 置灰
    setImageGray = function(self, frame, imagePath, isGray)
        if not frame then
            return
        end
        local image = frame:findImage(imagePath)
        if not image then
            return
        end

        if not self.gray_mat then
            local sd = UnityEngine.Resources.Load("Frame Gray Shader")
            self.gray_mat = UnityEngine.Material(sd)
        end

        -- if image.material.name ~= 'Gray Material' then -- 没有添加材质球
            local mat = UnityEngine.Material.Instantiate(self.gray_mat)
            mat.name = 'Gray Material'
            image.material = mat
        -- end
        image.material:SetFloat("_UseUIGrayScale", isGray and 1 or 0);
    end,

    -- 置灰
    setImageGray1 = function(self, image, isGray)
        if not image then
            return
        end
        if not self.gray_mat then
            local sd = UnityEngine.Resources.Load("Frame Gray Shader")
            self.gray_mat = UnityEngine.Material(sd)
        end
        -- if image.material.name ~= 'Gray Material' then -- 没有添加材质球
            local mat = UnityEngine.Material.Instantiate(self.gray_mat)
            mat.name = 'Gray Material'
            image.material = mat
        -- end
        image.material:SetFloat("_UseUIGrayScale", isGray and 1 or 0);
    end,
    
    -- 置灰
    setImageGrayPlus = function(self, image, isGray)
        if not image then
            return
        end

        if image.material.name ~= 'Gray Material' then -- 没有添加材质球
            local sd = UnityEngine.Resources.Load("Frame Gray Shader")
            local mat = UnityEngine.Material(sd)
            mat.name = 'Gray Material'
            image.material = mat
        end

        image.material:SetFloat("_UseUIGrayScale", isGray and 1 or 0);
    end,

    -- 把秒数转成 时分秒
    -- timeFormat = function(self, dt)
    --     dt = math.floor(dt % (3600 * 24))
    --     local hour = math.floor(dt / 3600)
    --     dt = math.floor(dt % 3600)
    --     local min = math.floor(dt / 60)
    --     dt = math.floor(dt % 60)
        
    --     local timeFormat = "%H:%M:%S"
    --     local timeString = string.format("%02d:%02d:%02d", hour, min, dt)
    --     return timeString
    -- end,

    -- 数组分割
    split = function(self, s, delim)

        assert (type (delim) == "string" and string.len (delim) > 0,
            "bad delimiter")

        local start = 1
        local t = {}  -- results table

        -- find each instance of a string followed by the delimiter

        while true do
            local pos = string.find (s, delim, start, true) -- plain find

            if not pos then
                break
            end

            table.insert (t, string.sub (s, start, pos - 1))
            start = pos + string.len (delim)
        end -- while

        -- insert final one (after last delimiter)

        table.insert (t, string.sub (s, start))

        return t

    end, -- function split

    -- 递归gameObject
    findChild = function(self, tf, name)
        if tf == nil or name == nil then
            return nil
        end

        for i = 0, tf.childCount - 1 do
            local child = tf:GetChild(i)
            print("child name:", child.name)
            if child.name == name then
                -- FindTransform
                return child
            else
                child = self:findChild(child.transform, name)
                if child then 
                    return child
                end
            end
        end

        return nil
    end,

    -- 数字跳动动画
    makeNumEffect = function(self, tf, num, prefix, suffix)
        local mono = tf:GetComponent(NumberEffect)
        if not mono then
            mono = tf:AddComponent(NumberEffect)
        end

        if prefix then
            mono.prefix = prefix
        end

        if suffix then
            mono.suffix = suffix
        end

        mono:make(num)
    end,

    -- 坐标转镜头坐标
    worldToScreenPoint = function(self, position)
        if not self.ui_camera then
            self.ui_camera = UnityEngine.GameObject.Find('gui/UICamera'):GetComponent(UnityEngine.Camera)
        end
        return self.ui_camera:WorldToScreenPoint(position)
    end,

    -- 道具数量格式化  
    itemNumFormat = function(self, num)
        num = tonumber(num)
        
        if num < 100000 then
            return tostring(num)
        -- elseif num <= 999999 then
        --     num = math.floor(num / 10^3)
        --     return tostring(num) .. 'K'
        elseif num < 10^6 then
            num = num / 10^3
            return string.format("%.1f", num) .. 'K'
        elseif num < 10^9 then
            num = num / 10^6
            return string.format("%.1f", num) .. 'M'
        else
            num = num / 10^9
            return string.format("%.1f", num) .. 'B'
        end
    end,

    -- 逗号格式化
    commaFormat = function(self, num)
        local rst = ""
        local length = string.len(num)
        local temp = 0
        for i = 1, length do
            temp = temp + 1
            if temp > 3 then
                temp = 0
                rst = "," .. rst
            end
            local num = string.sub(str, length - i + 1, length - i + 1)
            rst = num .. rst
        end
        return rst
    end,

    -- 时间格式化
    timeFormat = function(self, time)
        --[[
        local tb = {}
        local hour = math.floor(time/3600);
        if hour > 0 then
            if hour < 10 then
                table.insert(tb, tostring(hour) .. 'h')
            else
                table.insert(tb, tostring(hour) .. 'h')
            end
        end
        local minute = math.fmod(math.floor(time/60), 60)
        if minute > 0 then
            if minute < 10 then
                table.insert(tb, "0" .. tostring(minute) .. 'm')
            else
                table.insert(tb, tostring(minute) .. 'm')
            end
        end
        local second = math.fmod(time, 60)
        if second >= 0 then
            if second < 10 then
                table.insert(tb, "0" .. tostring(second) .. 's')
            else
                table.insert(tb, tostring(second) .. 's')
            end
        end
        ]]

        time = math.floor(time)
        --[[
        makeStr = function(value)
            local temp_str = "00"
            if value > 0 then
                if value < 10 then
                    temp_str = "0" .. tostring(value)
                else
                    temp_str = tostring(value)
                end
            end
            return temp_str
        end
        local str_rst = ""
        local day = math.floor(time/86400)

        if day > 0 then
            str_rst = str_rst .. tostring(day) .. "d "
        end
        local hour = math.floor(math.fmod(time, 86400) / 3600)
        str_rst = str_rst .. makeStr(hour) .. ":"

        local minute = math.floor(math.fmod(time, 3600) / 60)
        str_rst = str_rst .. makeStr(minute) .. ":"

        local second = math.fmod(time, 60)
        str_rst = str_rst .. makeStr(second)
        return str_rst
        ]]

        local d = math.floor(time / 86400)
        local h = math.floor(math.fmod(time, 86400) / 3600)
        local m = math.floor(math.fmod(time, 3600) / 60)
        local s = math.fmod(time, 60)

        if d > 0 then
            return string.format( "%sd %02d:%02d:%02d", d, h, m, s)
        else
            return string.format( "%02d:%02d:%02d", h, m, s)
        end
    end,

    
    -- "8_100" 解析类似这种属性配置
    getPropertyDes = function(self, s)
        local tmp = self:getJsonTab(s)
        local t = {}
        for id, num in pairs(tmp) do
            local cfg = config.attribute_config_attrs[tonumber(id)]
            if cfg == nil then 
                cfg = config.player_config_attrs[tonumber(id)]
            end
            if cfg == nil then
                log.error("属性解析失败 ! ! ! 找不到属性id", id)
                break
            end
            local name_str = self:getLangCfg(cfg.name_id)
            local num_str = ""
            local propValue = tonumber(num)
            if cfg.showpercent == 1 then num_str = propValue * 0.01 .. "%" else num_str = propValue end

            table.insert(t, { 
                id = id, 
                name_str = name_str, 
                num_str = num_str, 
                value = propValue,
                is_percent = cfg.showpercent == 1 } )
        end
        return t
    end,

    getProptyName = function(self, prop_id)
        local cfg = config.attribute_config_attrs[prop_id]
        if cfg == nil then 
            cfg = config.player_config_attrs[prop_id]
        end
        if cfg == nil then
            log.error("属性解析失败 ! ! ! 找不到属性id", prop_id)
            return ""
        else
            return self:getLangCfg(cfg.name_id)
        end
    end,

    isPropPercentage = function(self, prop_id)
        local cfg = config.attribute_config_attrs[tonumber(prop_id)]
        if cfg == nil then 
            cfg = config.player_config_attrs[tonumber(prop_id)]
        end
        if cfg == nil then
            log.error("属性解析失败 ! ! ! 找不到属性id", prop_id)
            return
        end
        return cfg.showpercent == 1
    end,

    rateFormat = function(self, cur, max)
        return string.format("%0.1f", (cur / max * 100)) .. "%"
    end,

    hideAllChild = function(self, frame, parent_path, force)
        -- print("parent_path:",parent_path)
        if frame[parent_path] then
            for __, child in ipairs(frame[parent_path]) do
                child.gameObject:SetActive(false)
            end
        elseif frame then
            if force then
                local parent_trans = frame.transform:Find(parent_path)
                for i=0, parent_trans.childCount - 1 do
                    local child = parent_trans:GetChild(i)
                    child:SetActive(false)
                end
            end
        end
    end,

    getChildByTemplatePath = function(self, frame, index, parent_path, template_path)
        if frame and index and parent_path and template_path then
            local parent_trans = frame.transform:Find(parent_path)
            if parent_trans.childCount == 0 then
                return
            end
            if not frame[parent_path] then
                frame[parent_path] = {}
                local template_child = frame.transform:Find(template_path)
                table.insert(frame[parent_path], template_child)
            end
            if frame[parent_path] then 
                if index <= #frame[parent_path] then
                    return frame[parent_path][index]
                else
                    local template = frame.transform:Find(template_path)
                    for i = 1, index - #frame[parent_path] do
                        local inst_obj = Object.Instantiate(template.gameObject)
                        inst_obj.transform.name = i + #frame[parent_path]
                        -- inst_obj:AttachTo(parent_trans)
                        inst_obj.transform:SetParent(parent_trans)
                        inst_obj.transform.localScale = Vector3.one

                        local tplt_rect = template.transform:GetComponent(UnityEngine.RectTransform)
                        local inst_rect = inst_obj.transform:GetComponent(UnityEngine.RectTransform)

                        inst_rect.anchoredPosition = tplt_rect.anchoredPosition
                        inst_rect.anchoredPosition3D = tplt_rect.anchoredPosition3D
                        inst_rect.anchorMax = tplt_rect.anchorMax
                        inst_rect.anchorMin = tplt_rect.anchorMin
                        inst_rect.offsetMax = tplt_rect.offsetMax
                        inst_rect.offsetMin = tplt_rect.offsetMin
                        inst_rect.pivot = tplt_rect.pivot
                        inst_rect.sizeDelta = tplt_rect.sizeDelta

                        table.insert(frame[parent_path], inst_obj.transform)
                    end         
                end
            end
            return frame[parent_path][index]
        else
            log.error("请检查传递参数...")
        end
    end,

    getChild = function(self, frame, index, parent_path)
        if frame and index and parent_path then
            local parent_trans = frame.transform:Find(parent_path)
            if parent_trans.childCount == 0 then
                return
            end
            if not frame[parent_path] then
                frame[parent_path] = {}
                for i = 0, parent_trans.childCount - 1 do
                    local child = parent_trans:GetChild(i)
                    table.insert(frame[parent_path], child)
                    child.name = #frame[parent_path]
                end
            end
            if frame[parent_path] then 
                if index <= #frame[parent_path] then
                    return frame[parent_path][index]
                else
                    local template = parent_trans:GetChild(0)
                    for i = 1, index - #frame[parent_path] do
                        local inst_obj = Object.Instantiate(template.gameObject)
                        inst_obj.transform.name = i + #frame[parent_path]
                        -- inst_obj:AttachTo(parent_trans)
                        inst_obj.transform:SetParent(parent_trans)
                        inst_obj.transform.localScale = Vector3.one

                        local tplt_rect = template.transform:GetComponent(UnityEngine.RectTransform)
                        local inst_rect = inst_obj.transform:GetComponent(UnityEngine.RectTransform)

                        inst_rect.anchoredPosition = tplt_rect.anchoredPosition
                        inst_rect.anchoredPosition3D = tplt_rect.anchoredPosition3D
                        inst_rect.anchorMax = tplt_rect.anchorMax
                        inst_rect.anchorMin = tplt_rect.anchorMin
                        inst_rect.offsetMax = tplt_rect.offsetMax
                        inst_rect.offsetMin = tplt_rect.offsetMin
                        inst_rect.pivot = tplt_rect.pivot
                        inst_rect.sizeDelta = tplt_rect.sizeDelta

                        table.insert(frame[parent_path], inst_obj.transform)
                    end         
                end
            end
            return frame[parent_path][index]
        else
            log.error("请检查传递参数...")
        end
    end,

    -- index从1开始
    getChildPlus = function(self, parent_trans, index)
        local lack = index - parent_trans.childCount 
        if lack > 0 then
            for i = 1, lack do
                local template = parent_trans:GetChild(0)
                local inst_obj = Object.Instantiate(template.gameObject)
                inst_obj.transform.name = index
                inst_obj.transform:SetParent(parent_trans)
                inst_obj.transform.localScale = Vector3.one
                local tplt_rect = template.transform:GetComponent(UnityEngine.RectTransform)
                local inst_rect = inst_obj.transform:GetComponent(UnityEngine.RectTransform)
                inst_rect.anchoredPosition = tplt_rect.anchoredPosition
                inst_rect.anchoredPosition3D = tplt_rect.anchoredPosition3D
                inst_rect.anchorMax = tplt_rect.anchorMax
                inst_rect.anchorMin = tplt_rect.anchorMin
                inst_rect.offsetMax = tplt_rect.offsetMax
                inst_rect.offsetMin = tplt_rect.offsetMin
                inst_rect.pivot = tplt_rect.pivot
                inst_rect.sizeDelta = tplt_rect.sizeDelta
            end
        end
        return parent_trans:GetChild(index - 1)
    end,

    hideAllChildPlus = function(self, parent_trans)
        for i=0, parent_trans.childCount - 1 do
            local child = parent_trans:GetChild(i)
            child:SetActive(false)
        end
    end,

    resetTransform = function(self, trans)
        trans.transform.localPosition = Vector3.zero
        trans.transform.localScale = Vector3.one
        trans.transform.localRotation = Quaternion.identity
    end,

    showCommonBox = function(self, des, yes_callback, no_callback)
        des = des:gsub("\\n", "\n")
        local tb = {
            title = "",
            content = des,
            buttons =
            {
                {
                    btn_name = NativeText.GetLangStr(110042),
                    callback = function()
                        frame_manager:hide('common_box')
                        if no_callback then
                            no_callback()
                        end
                    end
                },
                {
                    btn_name = NativeText.GetLangStr(110043),
                    callback = function() 
                        frame_manager:hide('common_box')
                        if yes_callback then
                            yes_callback()
                        end
                    end
                },
            },
        }
        frame_manager:show('common_box', nil, tb)
    end,

    showConfirmBox = function(self, des, yes_callback)
        des = des:gsub("\\n", "\n")
        local tb = {
            title = "",
            content = des,
            buttons =
            {
                {
                    btn_name = NativeText.GetLangStr(110043),
                    callback = function() 
                        frame_manager:hide('common_box')
                        if yes_callback then
                            yes_callback()
                        end
                    end
                },
            },
        }
        frame_manager:show('common_box', nil, tb)
    end,

    showNoticeBox = function(self, notice_type, des, yes_callback)
        des = des:gsub("\\n", "\n")

        local tb = {
            content = des,
            callback = yes_callback,
        }
        frame_manager:show('notice_box', nil, tb)
    end,

    isEnoughDiamond = function(self, need_num)
        local DIAMOND    = 30001 -- 钻石
        local own_num = self:getItemNum(DIAMOND)
        local rst = own_num >= need_num
        if not rst then
            hint:setText(NativeText.GetLangStr(290024))
        end
        return rst
    end,

    getDiamondPriceFromStr = function(self, str)
        local tab = loadstring("return " .. str)()
        local DIAMOND    = 30001 -- 钻石
        return tab[DIAMOND]
    end,

    onScrollViewLocate = function(self, content_trans, target_trans, is_horizontal)
        -- app:startCoroutine(function()
        --     coroutine.yield(nil)
        --     if is_horizontal then
        --     else
        --         local height = content_trans.sizeDelta.y
        --         local invers_pos = content_trans:InverseTransformPoint(target_trans.position)
        --         content_trans.anchoredPosition = Vector2(content_trans.anchoredPosition.x, content_trans.anchoredPosition.y + invers_pos.y)
        --     end
        -- end) 
    end,

    shuffle = function(self, t)
        if type(t)~="table" then
            return nil
        end
        local tab={}
        local index=1
        while #t~=0 do
            local n=math.random(0,#t)
            if t[n]~=nil then
                tab[index]=t[n]
                table.remove(t,n)
                index=index+1
            end
        end
        return tab
    end,

    --深度拷贝
    deepCopy = function(self, object)    
        local lookup_table = {}
        local function _copy(object)
            if type(object) ~= "table" then
                return object
            elseif lookup_table[object] then

                return lookup_table[object]
            end  -- if        
            local new_table = {}
            lookup_table[object] = new_table

            for index, value in pairs(object) do
                new_table[_copy(index)] = _copy(value)
            end 
            return setmetatable(new_table, getmetatable(object))    
        end     
        return _copy(object)
    end,

    serialize = function (self, obj)
        local lua = ""
        local t = type(obj)
        if t == "number" then
            lua = lua .. obj
        elseif t == "boolean" then
            lua = lua .. tostring(obj)
        elseif t == "string" then
            lua = lua .. string.format("%q", obj)
        elseif t == "table" then
            lua = lua .. "{\n"
        for k, v in pairs(obj) do
            lua = lua .. "[" .. self:serialize(k) .. "]=" .. self:serialize(v) .. ",\n"
        end
        local metatable = getmetatable(obj)
            if metatable ~= nil and type(metatable.__index) == "table" then
            for k, v in pairs(metatable.__index) do
                lua = lua .. "[" .. self:serialize(k) .. "]=" .. self:serialize(v) .. ",\n"
            end
        end
            lua = lua .. "}"
        elseif t == "nil" then
            return nil
        else
            error("can not serialize a " .. t .. " type.")
        end
        return lua
    end,
    
    unserialize = function (self, lua)
        local t = type(lua)
        if t == "nil" or lua == "" then
            return nil
        elseif t == "number" or t == "string" or t == "boolean" then
            lua = tostring(lua)
        else
            error("can not unserialize a " .. t .. " type.")
        end
        lua = "return " .. lua
        local func = loadstring(lua)
        if func == nil then
            return nil
        end
        return func()
    end,

    log = function(args)
        -- UnityEngine.Debug.LogError("frame log:  " .. args)
    end,
}

return frame_helper

