local log = logger.get("Game", "Long Press")

return class{
    ctor = function(self, frame)
        self.frame = frame
        self.durationThreshold = 1
        self.isPointerDown = false
        self.longPressTriggered = false
        self.timePressStarted = 0
        self.cd = 0
    end,

    update = function(self, dt)
        if self.isPointerDown then
			if Time.time - self.timePressStarted > self.durationThreshold then

				if self.press_cb then
					local duration = Time.time - self.timePressStarted - self.durationThreshold
                    self.cd = self.cd + dt

                    if self.cd > 0.1 then

                        local count = 0

                        if duration < 1 then
                            count = 1
                        elseif duration < 2 then
                            count = 10
                        elseif duration < 3 then
                            count = 50
                        elseif duration < 4 then
                            count = 100
                        else
                            count = 1000
                        end

                        self.cd = 0
                        self.press_cb(self.frame, self.updown_param, count)
                    end
                end
            end
        end	
    end,

    addClickListener = function(self, path, cb, param)
        self.click_cb = cb
        self.frame:clearScript("click", path)
        self.frame:setScript("click", path, functor.rmake(self.onClick, {obj = self, var = param}))
    end,

    addUpDownListener = function(self, path, cb, param)
        self.press_cb = cb
        self.frame:clearScript("updown", path)
        self.frame:setScript("updown", path, functor.rmake(self.onUpDown, {obj = self, var = param}))
    end,

    onClick = function(self, handler, data)
        if data.obj.click_cb then
            data.obj.click_cb(data.obj.frame, data.var)
        end
    end,

    onUpDown = function(self, handler, data, updown, param)
        local obj = param.obj

        if updown == "down" then
            obj.cd = 0
            obj.updown_param = param.var
            obj.timePressStarted = Time.time
            obj.isPointerDown = true
            
            obj.coroutine = app:startCoroutine(
                function()
                    while true do
                        obj:update(Time.deltaTime)
                        coroutine.yield(nil)
                    end
                end
            )
        else
            app:stopCoroutine(obj.coroutine)
            obj.isPointerDown = false
        end
    end,
}
