local log = logger.get("Game", "Speed Up Helper")

speedUpTypes = {
    common = 42,
    building = 43,
    research = 44,
    produce = 45,
    repair = 46,
    remould = 47,
    march = 48,
    dispatch = 49,
    mass = 50,
    repair_bag = 52,
}

local value_type = {
    building = 43,
    research = 44,
    produce = 45,
    repair = 46,
    remould = 47,
    repair_bag = 52,
}

local percent_type = {
    dispatch = 49,
    mass = 50,
}

local march_lines = {
    march = 48,
    dispatch = 49,
    mass = 50,
}

local speed_up_titles = {
    [43] = "建造加速",
    [44] = "科研加速",
    [45] = "生产加速",
    [46] = "修理加速",
    [47] = "改造加速",
    [48] = "行军加速",
    [49] = "派遣加速",
    [50] = "集结加速",
    [52] = "修理包加速",
}

local speed_up_cls = class{
    ctor = function(self)
        self.items = {}

        for k, v in pairs(speedUpTypes) do
            self.items[v] = {}
        end

        self:initItems()
    end,

    initItems = function(self)
        config.item_resources:foreach(
            function (k, v)
                if self.items[v.type] then
                    table.insert(self.items[v.type], v)
                end
            end
        )

        for k, v in pairs(self.items) do
            table.sort( v, function(a, b) return tonumber(a.para1) < tonumber(b.para1) end )
        end
    end,

    isValueType = function(self, type)
        for k, v in pairs(value_type) do
            if v == type then
                return true
            end
        end

        return false
    end,

    isPercentType = function(self, type)
        for k, v in pairs(percent_type) do
            if v == type then
                return true
            end
        end

        return false
    end,

    isMarchLine = function(self, type)
        for k, v in pairs(march_lines) do
            if v == type then
                return true
            end
        end

        return false
    end,

    getTitle = function(self, type)
        return speed_up_titles[type] or ""
    end,

    getCostDiamond = function(self, time, type)
        local items = self.items[type]
        if not items then
            return 0
        end

        local tab = loadstring("return ".. items[1].buyprice)() 
        local id = 0
        for item_id, num in pairs(tab) do
            id = item_id
            break
        end

        local cost = gui_helper:getSpeedUpCost(type, time)
        return cost, id
    end,

    getItems = function(self, type)
        if not self.items[type] then
            log.error("#### get item error ! type ", type)
            return {}
        end

        local items = {}
        local flag = false

        local sort_func = function(a, b)
            if a.count >= 0 and b.count >= 0 then
                return tonumber(a.para1) < tonumber(b.para1)
            else
                return a.count > b.count
            end
        end

        for k, v in ipairs(self.items[type]) do
            v.count = frame_helper:getItemNum(v.id)
            if v.count > 0 then
                flag = true
            end

            table.insert( items, v )
            table.sort( items, sort_func )
        end

        local common_items = {}

        local getCommonItems = function(t)
            for k, v in ipairs(self.items[t]) do
                local num = frame_helper:getItemNum(v.id)
                if num > 0 then
                    v.count = num
                    table.insert( common_items, v )
                end
            end
        end

        if self:isValueType(type) then
            getCommonItems(speedUpTypes.common)
        end

        if self:isPercentType(type) then
            getCommonItems(speedUpTypes.march)
        end

        local src = items
        local dst = common_items

        if flag then
            src = common_items
            dst = items
        end

        for k, v in ipairs(src) do
            table.insert( dst, v )
        end

        return dst
    end,
}

speedup_helper = speedup_helper or speed_up_cls()

return speedup_helper