local log = logger.get("Game", "ui tank")

-- 界面用的坦克，支持部件拼装，动画播放
return class{
    ctor = function(self, cfg_id, weapon_id, wheel_id, attachments_id, events)
        self.events = events
        self.cfg_id = cfg_id
        self.cfg = config.tank_basic_ids[self.cfg_id]

        self.weapon_id = weapon_id
        self.wheel_id = wheel_id
        self.attachments_id = attachments_id

        --log.info("#### weapon_id:"..tostring(weapon_id))
        --log.info("#### wheel_id:"..tostring(wheel_id))
        self.wheel_cfg = config.weapon_weapons[self.wheel_id]
        self.prefab_path = self.cfg.show_prefab -- 使用展示用的 prefab

        self.main_weapon = nil
        self.wheel = nil
        self.attachments = {}   -- [slot] - attachment
        self.wheel_package = nil    -- 轮胎材质的 ab 包
        self:init()
    end,

    init = function(self)

        local callback = function (go)
            -- callback
            if go ~= nil then
                log.info("##### ui_tank init !!")
                self.cs_inst = go:GetComponentInChildren("Tank", true)
                if not self.cs_inst then
                    log.error("Failed to find Tank component, tank_id:"..self.cfg_id)
                    return
                end

                self.cs_inst.gameObject:SetActive(true)
                self.cs_inst.enabled = false
                self.minor_weapons = {}
                self:setMainWeapon(self.weapon_id)
                -- 取消轮胎材质
                --self:setWheel(self.wheel_id)

                if not go:GetComponentInChildren(TrailerBody) then
                    self:setTrailerBody()
                end

                self.is_initialized = true
                if self.events then
                    self.events:dispatch("actor_loaded", self)
                end

                go:SetLayer(core.layers["UI"])

                if self.temp_cube then
                    GameObject.Destroy(self.temp_cube)
                    self.temp_cube = nil
                end
            else
                log.error('loadResource go is nil')
            end
        end

        self.pool = game_director.pool
        --[[ 
        -- 优先使用 level 的 pool, 否则使用 game_director 的 pool
        if level_script then
            self.pool = level_script.pool
        else
            self.pool = game_director.pool
        end]]

        self.gameObject = self:_LoadPrefab(self.prefab_path, callback)
        if not self.gameObject or not self.gameObject.isLoaded then
            if battle_helper and battle_helper.temp_cube then
                self.temp_cube = GameObject.Instantiate(battle_helper.temp_cube)
                self.temp_cube:SetActive(true)
                if self.gameObject then
                    self.temp_cube.transform.position = self.gameObject.transform.position
                    self.temp_cube.transform.rotation = self.gameObject.transform.rotation
                    self.temp_cube.transform:SetParent(self.gameObject.transform)
                else
                    self.gameObject = self.temp_cube
                end
            
            end
        end

        if self.gameObject then
            self.transform = self.gameObject.transform
        end
    end,

    _LoadPrefab = function(self, prefab_path, callback)
        local uobject = self.pool:instantiate(prefab_path)
        if not uobject then
            log.error('Failed to load prefab:'..prefab_path)
            return nil
        end

        if uobject.isLoaded then 
            callback(uobject)
        else 
            uobject.onLoaded = callback
        end

        return uobject
    end,
    --
    setMainWeapon = function(self, weapon_id)
        self.weapon_id = weapon_id or self.cfg.default_weapon_id
        self.weapon_cfg = config.weapon_weapons[self.weapon_id]

        if not self.weapon_cfg then
            log.error('no main weapon cfg!! weapon id', self.weapon_id)
            return
        end
        local attach_point = self.cs_inst:FindChild('z_wq')
        local callback = function(go)
            -- callback
            if go ~= nil then
                --log.info("##### main weapon init !!")
                go.name = "main_weapon_"..self.weapon_id
                go.transform:SetParent(attach_point)
                --go.transform.localScale = Vector3.one
                go.transform.localPosition = Vector3.zero
                go.transform.localRotation = Quaternion.identity 

                local rts_helper = require('RTS.rts_helper')
                local scale = rts_helper:getWeaponScale(self.cfg.prefab1, self.weapon_id)
                go.transform.localScale = Vector3(scale, scale, scale)

                go:SetLayer(core.layers["UI"])

                if self.main_weapon then
                    --log.info("### destroy old main_weapon")
                    self.main_weapon:destroy()
                    self.main_weapon = nil
                end
                self.main_weapon = go

                -- 主武器初始化完成后设置副武器, 有些副武器插槽在主武器上
                --[[
                for slot, item in pairs(self.attachments_id) do
                    self:setAttachment(slot, item.cfg_id)
                end]]

            else
                log.error('initMainWeapon go is nil')
            end
        end

        local weapon_prefab_path = self.weapon_cfg.show_prefab
        self:_LoadPrefab(weapon_prefab_path, callback)
    end,

    --[[
    setWheel = function(self, wheel_id)
        --log.info("setWheel:"..wheel_id)
        self.wheel_id = wheel_id or self.cfg.initial_wheel_id
        self.wheel_cfg = config.weapon_weapons[self.wheel_id]

        local wheel_trans = self.cs_inst:FindChild("Wheel")
        if not wheel_trans or not wheel_trans.gameObject then
            log.error("## Failed to find Wheel node:"..self.cfg_id)
            return
        end
        if not self.wheel_cfg then
            log.error('no wheel cfg, :'..wheel_id)
            return
        end

        log.info("### loadWheelMaterial:"..self.wheel_id)
        self:loadWheelMaterial(wheel_trans.gameObject, self.wheel_id)

    end,
    
    loadWheelMaterial = function(self, go, wheel_id)

        local callback = function(material)
            if not go then
                return
            end
            -- callback
            if material ~= nil then
               -- log.info("#### 333333333344444444444")
                local renderer = go:GetComponent(UnityEngine.Renderer)
                if renderer then
                    --log.info("set wheel material:"..wheel_id)
                    renderer.material = material
                else
                    log.error("Failed to find renderer in Wheel:"..wheel_id)
                end
            else
                log.error('failed to load wheel material:'..wheel_id)
            end
        end

        local wheel_cfg = config.weapon_weapons[wheel_id]
        local path = wheel_cfg.show_material
        log.info("### load material:"..path)
        if path == "" then
            log.error('Failed to load wheel material:'..wheel_id)
        else
            --log.info("#### 1111122222")
            Fenix.Assets.Helper.LoadAsset(path, function(asset) 
                local obj = asset:Get()
                local mat = UnityEngine.Material(obj)

                if self.wheel_package then
                    self.wheel_package:RemoveRef()
                end
                self.wheel_package = asset.package
                if self.wheel_package then
                    self.wheel_package:AddRef()
                end

                callback(mat)
            end)
        end
    end,]]
    
    -- 初始化拖挂
    setTrailerBody = function(self)
        if self.cfg.relatetank and self.cfg.relatetank > 0 then
            self.taiiler_cfg = config.tank_basic_ids[self.cfg.relatetank]
            --local zoom = self.taiiler_cfg.zoom / 10000

            local prefab_path = self.taiiler_cfg.prefab1

            local uobject = self.pool:instantiate(prefab_path)
            if not uobject then
                log.error('Failed to load TrailerBody:'..prefab_path)
                return
            end

            local callback = function(go)
                self.trailer_body = go:GetComponentInChildren(TrailerBody)
                if not self.trailer_body then
                    log.error("Failed to find TrailerBody component")
                    return
                end
                go.transform.localScale = Vector3.one
                self.trailer_body:ConnectTo(self.cs_inst)
                --self.trailer_body.transform.localScale = Vector3(zoom, zoom, zoom)
                --go:SetLayer(core.layers["UI"])
                self.trailer_body.gameObject:SetLayer(core.layers["UI"])
                
                go.name = 'tank_trailer'
                
                --self:setTrailerWheel()
            end

            if uobject.isLoaded then 
                callback(uobject)
            else 
                uobject.onLoaded = callback
            end
        else
            --log.info("# No relatetank:"..tostring(self.cfg_relatetank))
        end
    end,

    --[[
    -- 初始化挂载轮胎
    setTrailerWheel = function(self, cfg)
        if self.wheel_cfg and self.wheel_cfg.relatewheel > 0 then
            local wheel_trans = self.trailer_body:FindChild('Wheel')
            if wheel_trans then
                self:loadWheelMaterial(wheel_trans.gameObject, self.wheel_id)
            end
        end
    end,]]

    -- 设置一个副武器或者装饰物
    setAttachment = function(self, slot, attahcment_id)
    end,

    destroy = function(self)
        if self.wheel_package then
            self.wheel_package:RemoveRef()
            self.wheel_package = nil
        end

        if self.wheel then
            --log.info("#### ui_tank wheel destroy")
            self.wheel:destroy()
            self.wheel = nil
        else

        end

        if self.main_weapon then
            self.main_weapon:destroy()
            self.main_weapon = nil
        else
        end
        if self.temp_cube then
            GameObject.Destroy(self.temp_cube)
        else
            self.gameObject:destroy()
        end
    end,

}
