local log = logger.get("Game", "Hold Team Frame")
local super = require("ui.frame")

return class{
    super = super,

    onLoaded = function( self )
        super.onLoaded(self)

        self.owner = self.args.owner
        self.factory = self.args.data.hold

        clickEvents = {
            "Close",
            "BG/Auto",
            "BG/Confirm",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self.amount = self.transform:Find("BG/Amount"):GetComponent(UnityEngine.UI.Text)
        self.res_panel = self.transform:Find("BG/Center")
    
        self.amount_max = 100000
        self.resources = {}
        self.move_res = {}
        self.over_weigh = false

        self:loadRes()
        self:setMoveTime(0)

        self.evts = {}
        self.evts["WORLDMAPQUERYTIME"] = game_director.events:on("WORLDMAPQUERYTIME", functor.make(self.onGetMoveTime, self))
        self.evts["WORLDMAPMOVERES"] = game_director.events:on("WORLDMAPMOVERES", functor.make(self.onTransport, self))

        local from_hold = self.factory
        local dest_hold = self.owner.my_hold
        network_sender:queryMarchTime(from_hold.coordX, from_hold.coordY, dest_hold.coordX, dest_hold.coordY, protocol_pb.EQUERY_PRETEAM)
    end,

    onUnload = function(self)
        if self.evts then
            for k, v in pairs(self.evts) do
                game_director.events:remove(k, v)
            end
        end
    end,

    onGetMoveTime = function(self, msg)
        self:setMoveTime(msg.marchTime)
    end,

    onTransport = function(self, msg)
        self:click_Close()
    end,

    update = function(self)
        self:setResCount()
    end,

    loadRes = function(self)
        self.amount.text = string.format("%s  <color=#ffffffff>%d/%d</color>", NativeText.GetLangStr(280603), 0, 0)
        local res_list = self.factory.queryInfo.curReses
        for k, v in ipairs(res_list) do
            if k <= self.res_panel.childCount then
                local child = self.res_panel:GetChild(k-1)
                child:SetActive(true)
                local count = child:Find("Count"):GetComponent(UnityEngine.UI.Text)
                local icon = child:Find("Icon/Image"):GetComponent(UnityEngine.UI.Image)
                
                local max_str = self.helper:itemNumFormat(v.Count)
                count.text = string.format( "0/%s", max_str)

                local cfg = item_helper:getItemCfg(v.ItemTemId)
                if cfg then
                    icon:SetSprite(gui_helper:getItemIconPath(cfg))
                end
                table.insert( self.resources, {ui = child, max = v.Count, curr = 0, id = v.ItemTemId} )
            end
        end

        for i=#res_list, self.res_panel.childCount - 1 do
            local child = self.res_panel:GetChild(i)
            child:SetActive(false)
        end
    end,

    setResCount = function(self)
        if self.resources then
            local total = 0
            self.over_weigh = false
            self.move_res = {}
            for k, v in ipairs(self.resources) do
                local slider = v.ui:GetComponent(UnityEngine.UI.Slider)
                local count = v.ui:Find("Count"):GetComponent(UnityEngine.UI.Text)
                local cur_cnt = slider.value * v.max
                if cur_cnt + total > self.amount_max then
                    cur_cnt = self.amount_max - total
                    slider.value = cur_cnt / v.max
                end
                
                local cur_str = self.helper:itemNumFormat(math.floor(cur_cnt))
                local max_str = self.helper:itemNumFormat(v.max)
                count.text = string.format( "%s/%s", cur_str, max_str)

                v.curr = cur_cnt
                total = total + cur_cnt
                if cur_cnt ~= 0 then
                    table.insert( self.move_res, {item_id = v.id, count = cur_cnt} )
                end
            end
            local color = "#ffffffff"
            if total > self.amount_max then
                color = "#D20000FF"
                self.over_weigh = true
            end
            local cur_str = self.helper:itemNumFormat(math.floor(total))
            local max_str = self.helper:itemNumFormat(self.amount_max)
            self.amount.text = string.format( "%s  <color=%s>%s</color><color=#ffffffff>/%s</color>", NativeText.GetLangStr(280603), color, cur_str, max_str)
        end
    end,

    setMoveTime = function(self, dt)
        local move_time = self.transform:Find("BG/MoveTime/Text"):GetComponent(UnityEngine.UI.Text)
        move_time.text = gui_helper:timeFormat(dt)
    end,

    setMaxMineCount = function(self)
        if self.resources then
            local total = 0
            self.over_weigh = false
            self.move_res = {}
            for k, v in ipairs(self.resources) do
                local slider = v.ui:GetComponent(UnityEngine.UI.Slider)
                local count = v.ui:Find("Count"):GetComponent(UnityEngine.UI.Text)
                local left_amount = self.amount_max - total
                local cur_cnt = left_amount > v.max and v.max or left_amount
                slider.value = cur_cnt / v.max

                local cur_str = self.helper:itemNumFormat(math.floor(cur_cnt))
                local max_str = self.helper:itemNumFormat(v.max)
                count.text = string.format( "%s/%s", cur_str, max_str)

                v.curr = cur_cnt
                total = total + cur_cnt
                if cur_cnt ~= 0 then
                    table.insert( self.move_res, {item_id = v.id, count = cur_cnt} )
                end
            end
            local color = "#ffffffff"
            if total > self.amount_max then
                color = "#D20000FF"
                self.over_weigh = true
            end
            local cur_str = self.helper:itemNumFormat(math.floor(total))
            local max_str = self.helper:itemNumFormat(self.amount_max)
            self.amount.text = string.format( "%s  <color=%s>%s</color><color=#ffffffff>/%s</color>", NativeText.GetLangStr(280603), color, cur_str, max_str)
        end
    end,

    click_Close = function(self)
        frame_manager:hide("transport_res")
    end,

    click_Auto = function(self)
        self:setMaxMineCount()
    end,
    
    click_Confirm = function(self)
        if #self.move_res == 0 then
            hint:setText(NativeText.GetLangStr(280092))
            return
        end

        if self.over_weigh then
            hint:setText(NativeText.GetLangStr(280093))
            return
        end

        network_sender:transportRes(self.factory.id, 0, self.move_res, 1)
    end,
}