local log = logger.get("Game", "Warming Info Frame")
local super = require("ui.frame")

return class{
    super = super,

    onLoaded = function( self )
        super.onLoaded(self)

        clickEvents = {
            "Close",
            "DetailPanel/DetailClose",
            "CounterPanel/CounterClose",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self.content = self.transform:Find("BG/Scroll View/Viewport/Content")
        self.prefab = self.content:GetChild(0).gameObject

        self.detail_panel = self.transform:Find("DetailPanel")
        self.detail_panel:SetActive(false)

        self.counter_panel = self.transform:Find("CounterPanel")
        self.counter_panel:SetActive(false)

        local team_node = self.detail_panel:Find("BG/Scroll View/Viewport/Content/Team")
        team_node:SetActive(false)
        self.team_ui = {}
        self.team_ui[1] = team_node

        self.army_list = {}
        self.lt_fills = {}
        self.ui_list = {}

        self:init()

        self.warn_evt = game_director.events:on("WARNNOTIFY", functor.make(self.init, self))
    end,

    onUnload = function(self)
        game_director.events:remove("WARNNOTIFY", self.warn_evt)
    end,

    update = function(self, dt)
        if self.lt_fills then
            for k, v in pairs(self.lt_fills) do
                local detla = os.time() - v.receive_time
                local lt = v.left_time - detla
                if lt > 0 then
                    v.arrive_time.text = gui_helper:timeFormat(lt)
                    v.fill.fillAmount = 1 - lt / (v.total_time > 0 and v.total_time or 1)
                else
                    self.ui_list[k]:SetActive(false)
                    self.lt_fills[k] = nil
                end
            end
        end

        if self.detail_fill then
            local detla = os.time() - self.detail_fill.receive_time
            local lt = self.detail_fill.left_time - detla
            if lt > 0 then
                self.detail_fill.arrive_time.text = gui_helper:timeFormat(lt)
                self.detail_fill.fill.fillAmount = 1 - lt / (self.detail_fill.total_time > 0 and self.detail_fill.total_time or 1)
            else
                self.detail_fill = nil
                self.detail_panel:SetActive(false)
            end
        end
    end,

    init = function(self)
        for k, v in pairs(game_info.warning_info) do
            table.insert( self.army_list, v )
        end

        table.sort( self.army_list, function(a, b) return a.leftTime - a.receive_time < b.leftTime - b.receive_time end )

        for k, v in ipairs(self.army_list) do
            local elem = nil

            if k > self.content.childCount then
                elem = Object.Instantiate(self.prefab).transform
                elem:SetParent(self.content)
                resetTransformZ(elem)
            else
                elem = self.content:GetChild(k-1)
            end

            elem:SetActive(true)
            elem.name = "Army_"..k

            self:setArmyInfo(elem, v)

            local btn = elem:GetComponent(UnityEngine.UI.Button)

            if v.type ~= common_pb.EMARCH_CREATE_MASS and v.type ~= common_pb.EMARCH_HELP then
                btn.onClick:RemoveAllListeners()
                btn.onClick:AddListener(
                    function ()
                        self:onClickArmy(v)
                    end
                )
            end

            btn = elem:Find("Counter"):GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(
                function ()
                    self:onClickCounter(v)
                end
            )

            btn = elem:Find("SwitchBtn"):GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(
                function ()
                    self:onClickSwitchBtn({ui = elem, info = v})
                end
            )

            self.ui_list[v.instId] = elem
        end
    end,

    setArmyInfo = function(self, elem, data)
        local bg = elem:Find("Image"):GetComponent(UnityEngine.UI.Image)
        local type_bg = elem:Find("NameBG"):GetComponent(UnityEngine.UI.Image)
        local army_type = elem:Find("NameBG/Text"):GetComponent(UnityEngine.UI.Text)
        local left_pos = elem:Find("Origin/LeftPos"):GetComponent(UnderLineText)
        local right_pos = elem:Find("Target/Pos"):GetComponent(UnderLineText)
        local player_name = elem:Find("Name"):GetComponent(UnityEngine.UI.Text)
        local army_size = elem:Find("ArmySize"):GetComponent(UnityEngine.UI.Text)
        local soldier_type = elem:Find("SoldierType"):GetComponent(UnityEngine.UI.Text)
        local type_icons = elem:Find("SoldierType/Icons")
        local arrive_text = elem:Find("FillBG/Armying"):GetComponent(UnityEngine.UI.Text)
        local arrive_time = elem:Find("FillBG/FillTime"):GetComponent(UnityEngine.UI.Text)
        local fill_bg = elem:Find("FillBG")
        local fill = elem:Find("FillBG/Fill"):GetComponent(UnityEngine.UI.Image)
        local target_name = elem:Find("Target/Name"):GetComponent(UnityEngine.UI.Text)
        local target_icon = elem:Find("Target"):GetComponent(UnityEngine.UI.Image)
        local on_img = elem:Find("SwitchBtn/On")
        local off_img = elem:Find("SwitchBtn/Off")
        local text = elem:Find("SwitchBtn/Text"):GetComponent(UnityEngine.UI.Text)

        if data.is_off then
            on_img.localScale = Vector3.zero
            off_img.localScale = Vector3.one
            text.text = NativeText.GetLangStr(9000057)
        else
            on_img.localScale = Vector3.one
            off_img.localScale = Vector3.zero
            text.text = NativeText.GetLangStr(9000058)
        end

        local cb = function()
            gui_helper:positionJump(data.baseIfno.posX, data.baseIfno.posY)
        end
        left_pos:SetText(gui_helper:positionFormat(data.baseIfno.posX, data.baseIfno.posY), cb)

        cb = function()
            gui_helper:positionJump(data.dstInfo.posX, data.dstInfo.posY)
        end
        right_pos:SetText(gui_helper:positionFormat(data.dstInfo.posX, data.dstInfo.posY), cb)

        arrive_time.text = gui_helper:timeFormat(data.leftTime - data.receive_time)
        player_name.text = data.baseIfno.playerName

        if string.len(data.baseIfno.familyName) > 2 then
            player_name.text = string.format( "[%s]%s", data.baseIfno.familyName, data.baseIfno.playerName )
        end

        player_name.color = Color(1, 80/255, 80/255, 1)
        soldier_type.color = Color(1, 80/255, 80/255, 1)
        arrive_text.color = Color(1, 80/255, 80/255, 1)
        army_size.color = Color(1, 80/255, 80/255, 1)

        self.lt_fills[data.instId] = {fill = fill, arrive_time = arrive_time, left_time = data.leftTime, receive_time = data.receive_time, total_time = data.totalTime}

        if data.dstInfo.objType == common_pb.WORLD_OBJ_PLAYER then
            local cfg = config.worldmap_base_configs[data.dstInfo.cfgId]

            if cfg then
                target_name.text = self.helper:getLangCfg(cfg.name_id)
                target_icon:SetSprite(cfg.iconpath..cfg.iconid)
            else
                log.error("######################################## worldmap_base_configs error ! id ", data.dstInfo.cfgId)
            end
        else
            local cfg = config.worldmap_titaniums[data.dstInfo.cfgId]

            if cfg then
                target_name.text = self.helper:getLangCfg(cfg.name_id)
                target_icon:SetSprite(cfg.iconpath..cfg.iconid)
            else
                log.error("######################################## worldmap_titaniums error ! id ", data.dstInfo.cfgId)
            end
        end

        local is_help = false
        local is_spy = false
        local have_team = true
        local is_mass = false
        arrive_text.text = NativeText.GetLangStr(9000059)

        if data.type == common_pb.EMARCH_CREATE_MASS then
            arrive_text.text = NativeText.GetLangStr(9000060)
            have_team = false
            is_mass = true
        elseif data.type == common_pb.EMARCH_MASS_ATK then
            is_help = true
            is_mass = true
        elseif data.type == common_pb.EMARCH_SPY then
            have_team = false
            is_spy = true
        elseif data.type == common_pb.EMARCH_HELP then
            is_help = true
        end

        if is_help then
            type_bg:SetSprite(res_path.res_path_113)
            bg:SetSprite(res_path.res_path_115)
            army_type.text = NativeText.GetLangStr(9000061)

            player_name.color = Color(1, 60/255, 200/255, 1)
            soldier_type.color = Color(1, 60/255, 200/255, 1)
            arrive_text.color = Color(1, 60/255, 200/255, 1)
            army_size.color = Color(1, 60/255, 200/255, 1)
        elseif is_spy then
            type_bg:SetSprite(res_path.res_path_112)
            bg:SetSprite(res_path.res_path_114)
            army_type.text = NativeText.GetLangStr(9000062)
        elseif is_mass then
            type_bg:SetSprite(res_path.res_path_111)
            bg:SetSprite(res_path.res_path_114)
            army_type.text = NativeText.GetLangStr(9000063)
        else
            type_bg:SetSprite(res_path.res_path_111)
            bg:SetSprite(res_path.res_path_114)
            army_type.text = NativeText.GetLangStr(9000064)
        end

        if have_team then
            army_size.transform.localScale = Vector3.one
            soldier_type.transform.localScale = Vector3.one
            fill_bg.localPosition = Vector3(-124, -72.5, 0)
        else
            army_size.transform.localScale = Vector3.zero
            soldier_type.transform.localScale = Vector3.zero
            fill_bg.localPosition = Vector3(-124, 15, 0)
        end

        local soldier_type_list = data.teamInfo.soldierInfo.soldierType
        local soldier_count = data.teamInfo.soldierInfo.count

        if is_mass then
            soldier_type_list = data.mass.massInfo.soldierType
            soldier_count = data.mass.massInfo.count
        end

        army_size.text = string.format( "%s <color=#FFFFFFFF>%d</color>", NativeText.GetLangStr(65), soldier_count )

        for k, v in ipairs(soldier_type_list) do
            if k <= type_icons.childCount then
                local elem = type_icons:GetChild(k-1)
                local icon = elem:GetComponent(UnityEngine.UI.Image)
                local path = gui_helper:getSoldierTypeIcon(v)
                if path then
                    icon:SetSprite(path)
                else
                    log.error("####################################### find soldier type icon error ! type ", v)
                end
            end
        end

        for i=#data.teamInfo.soldierInfo.soldierType, type_icons.childCount - 1 do
            local child = type_icons:GetChild(i)
            child:SetActive(false)
        end
    end,

    showDetail = function(self, data)
        self.detail_panel:SetActive(true)

        local name = self.detail_panel:Find("BG/Head/Name"):GetComponent(UnityEngine.UI.Text)
        local time_text = self.detail_panel:Find("BG/FillBG/Time"):GetComponent(UnityEngine.UI.Text)
        local fill = self.detail_panel:Find("BG/FillBG/Fill"):GetComponent(UnityEngine.UI.Image)
        local arrive_text = self.detail_panel:Find("BG/FillBG/Text"):GetComponent(UnityEngine.UI.Text)

        name.text = data.baseIfno.playerName

        if string.len(data.baseIfno.familyName) > 2 then
            name.text = string.format( "[%s]%s", data.baseIfno.familyName, data.baseIfno.playerName )
        end

        if data.type == common_pb.EMARCH_HELP then
            name.color = Color(1, 60/255, 200/255, 1)
            arrive_text.color = Color(1, 60/255, 200/255, 1)
        else
            name.color = Color(1, 80/255, 80/255, 1)
            arrive_text.color = Color(1, 80/255, 80/255, 1)
        end

        self.detail_fill = {fill = fill, arrive_time = time_text, left_time = data.leftTime, receive_time = data.receive_time, total_time = data.totalTime}

        local defender_node = self.detail_panel:Find("BG/Scroll View/Viewport/Content/Defender")
        local mass_node = self.detail_panel:Find("BG/Scroll View/Viewport/Content/Mass")
        defender_node:SetActive(false)
        mass_node:SetActive(false)

        if data.type == common_pb.EMARCH_CREATE_MASS then
            self:initMass(mass_node, data)
        else
            self:initDefender(defender_node, data)
        end
    end,

    initDefender = function(self, node, data)
        node:SetActive(true)

        local total_cnt = node:Find("Total/Count"):GetComponent(UnityEngine.UI.Text)

        total_cnt.text = "~"..frame_helper:itemNumFormat(data.teamInfo.soldierInfo.count)

        local list = node:Find("List")
        self:loadSoldiers(list, data.teamInfo.soldierInfo.soldierCounts)

        local hero_node = node:Find("Heros")
        self:loadTeamHeros(hero_node, data.teamInfo.heros)

        app:startCoroutine(
            function()
                coroutine.yield(nil)

                if #data.teamInfo.heros == 0 then
                    hero_node:SetActive(false)
                    node.sizeDelta = Vector2(910, list.rect.height + 140)
                    list.localPosition = Vector3(0, -100, 0)
                else
                    hero_node:SetActive(true)
                    node.sizeDelta = Vector2(910, list.rect.height + 240)
                    list.localPosition = Vector3(0, -200, 0)
                end
            end
        )

        self:loadTeam()
    end,
    
    initMass = function(self, node, data)
        if #data.mass > 0 then
            node:SetActive(true)

            local total_cnt = node:Find("Total/Count"):GetComponent(UnityEngine.UI.Text)

            total_cnt.text = "~"..frame_helper:itemNumFormat(data.mass.massInfo.count)

            local list = node:Find("List")

            self:loadSoldiers(list, data.mass.massInfo.soldierCounts)

            app:startCoroutine(
                function()
                    coroutine.yield(nil)
                    node.sizeDelta = Vector2(910, list.rect.height + 150)
                end
            )

            self:loadTeam(data.mass.teams)
        else
            node:SetActive(false)
        end
    end,

    loadTeam = function(self, teams)
        local content = self.detail_panel:Find("BG/Scroll View/Viewport/Content")
        local prefab = self.team_ui[1].gameObject
        local cnt = 1

        if teams then
            for k, v in ipairs(teams) do
                local elem = nil
                if k > #self.team_ui then
                    elem = Object.Instantiate(prefab).transform
                    elem:SetParent(content)
                    resetTransformZ(elem)

                    local idx = prefab.transform:GetSiblingIndex()
                    elem:SetSiblingIndex(idx + k - 1)

                    table.insert( self.team_ui, elem )
                else
                    elem = self.team_ui[k]
                end

                elem:SetActive(true)

                local name = elem:Find("Top/Name"):GetComponent(UnityEngine.UI.Text)
                name.text = v.playerName

                local hero_node = elem:Find("Heros")
                self:loadTeamHeros(hero_node, v.heros)

                local soldier_node = elem:Find("List")
                self:loadSoldiers(soldier_node, v.soldierInfo.soldierCounts)

                cnt = cnt + 1

                app:startCoroutine(
                    function()
                        coroutine.yield(nil)
                        if #v.heros == 0 then
                            hero_node:SetActive(false)
                            elem.sizeDelta = Vector2(910, soldier_node.rect.height + 78)
                            soldier_node.localPosition = Vector3(0, -75, 0)
                        else
                            hero_node:SetActive(true)
                            elem.sizeDelta = Vector2(910, soldier_node.rect.height + 178)
                            soldier_node.localPosition = Vector3(0, -175, 0)
                        end
                    end
                )
            end
        end

        for i=cnt, #self.team_ui do
            local idx = prefab.transform:GetSiblingIndex()
            idx = idx + i - 1
            local child = content:GetChild(idx)
            child:SetActive(false)
        end
    end,

    loadSoldiers = function(self, list, soldiers)
        local prefab = list:GetChild(0).gameObject

        for k, v in ipairs(soldiers) do
            local elem = nil
            if k > list.childCount then
                elem = Object.Instantiate(prefab).transform
                elem:SetParent(list)
                resetTransformZ(elem)
            else
                elem = list:GetChild(k-1)
            end

            elem:SetActive(true)

            local icon = elem:Find("Icon"):GetComponent(UnityEngine.UI.Image)
            local count = elem:Find("Count"):GetComponent(UnityEngine.UI.Text)
            local name = elem:Find("Name"):GetComponent(UnityEngine.UI.Text)

            local cfg = config.soldier_soldiers[v.cfgId]

            if cfg then
                icon:SetSprite(cfg.iconpath..cfg.soldiericon)
                name.text = frame_helper:getLangCfg(cfg.soldiername)
                count.text = "~"..frame_helper:itemNumFormat(v.count)
            end
        end

        for i=#soldiers, list.childCount - 1 do
            local child = list:GetChild(i)
            child:SetActive(false)
        end
    end,

    loadTeamHeros = function(self, node, heros) 
        local prefab = node:GetChild(0).gameObject

        for k, v in ipairs(heros) do
            local elem = nil
            if k > node.childCount then
                elem = Object.Instantiate(prefab).transform
                elem:SetParent(node)
                resetTransformZ(elem)
            else
                elem = node:GetChild(k-1)
            end

            elem:SetActive(true)

            local icon = elem:Find("Icon"):GetComponent(UnityEngine.UI.Image)
            local grade = elem:Find("Grade"):GetComponent(UnityEngine.UI.Image)
            local level = elem:Find("Image/Level"):GetComponent(UnityEngine.UI.Text)
            local stars = elem:Find("Image/Stars")

            local cfg = config.commander_basics[v.officeCfgId]

            if cfg then
                icon:SetSprite(cfg.path..cfg.icon)
                grade:SetSprite(gui_helper:getNewQualityIcon(v.quality))
                level.text = v.officeLev
                gui_helper:setOfficerStars(v.starLev, stars)
            else
                log.error("##################################### find commander cfg error ! id ", v.officeCfgId)
            end
        end

        for i=#heros, node.childCount - 1 do
            local child = node:GetChild(i)
            child:SetActive(false)
        end
    end,

    showCounter = function(self, data)
        local cfg = {}

        local getCfg = function(id)
            config.home_config_counters:foreach(
                function(k, v)
                    if v.event == id then
                        table.insert(cfg, v)
                    end
                end
            )
        end

        if data.type == common_pb.EMARCH_ATK or data.type == common_pb.EMARCH_MASS_ATK then
            getCfg(1)
        elseif data.type == common_pb.EMARCH_SPY then
            getCfg(2)
        elseif data.type == common_pb.EMARCH_GATHER_TITAN then
            getCfg(3)
        end

        local content = self.counter_panel:Find("Content/Scroll View/Viewport/Content")
        local prefab = content:GetChild(0).gameObject

        for k, v in ipairs(cfg) do
            local elem = nil
            if k > content.childCount then
                elem = Object.Instantiate(prefab).transform
                elem:SetParent(content)
                resetTransformZ(elem)
            else
                elem = content:GetChild(k-1)
            end

            elem:SetActive(true)
            elem.name = "Elem_"..k

            local icon = elem:Find("Icon"):GetComponent(UnityEngine.UI.Image)
            local name = elem:Find("Name"):GetComponent(UnityEngine.UI.Text)
            local des = elem:Find("Des"):GetComponent(UnityEngine.UI.Text)
            local btn = elem:GetComponent(UnityEngine.UI.Button)

            icon:SetSprite(v.icon)
            name.text = self.helper:getLangCfg(v.name)
            des.text = self.helper:getLangCfg(v.des)

            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(
                function ()
                    self:counterJump(v, data)
                end
            )
        end

        for i=#cfg, content.childCount - 1 do
            local child = content:GetChild(i)
            child:SetActive(false)
        end

        self.counter_panel:SetActive(true)
    end,

    counterJump = function(self, cfg, data)
        if cfg.counter == 1 then
            frame_manager:show('home_buff')
        elseif cfg.counter == 2 then
            if game_state_home then
                game_state_home:jumpToTrainingBuild()
            else
                game_director:changeState('loading', {next_state = 'home', scene_name = 'home', next_state_ui_name = 'home', jumpToTrain = true})
            end
        elseif cfg.counter == 3 then
            frame_manager:show('home_buff')
        elseif cfg.counter == 4 then
            frame_manager:show('home_buff')
        elseif cfg.counter == 5 then
            frame_manager:show('home_buff')
        elseif cfg.counter == 6 then
            frame_manager:show('home_buff')
        elseif cfg.counter == 7 then
            world_map_helper:counterJumpToHold(data.baseIfno.posX, data.baseIfno.posY)
        elseif cfg.counter == 8 then
            world_map_helper:counterJumpToHold(data.dstInfo.posX, data.dstInfo.posY)
        else
            log.error("################################################# counter not find !", cfg.id)
        end

        self:click_Close()
    end,

    onClickArmy = function(self, data)
        self:showDetail(data)
    end,

    onClickCounter = function(self, data)
        self:showCounter(data)
    end,

    onClickSwitchBtn = function(self, data)
        local on_img = data.ui:Find("SwitchBtn/On")
        local off_img = data.ui:Find("SwitchBtn/Off")
        local text = data.ui:Find("SwitchBtn/Text"):GetComponent(UnityEngine.UI.Text)

        local key = "Warning_"..data.info.instId

        if data.info.is_off then
            PlayerPrefs.DeleteKey(key)
            on_img.localScale = Vector3.one
            off_img.localScale = Vector3.zero
            text.text = NativeText.GetLangStr(9000058)
        else
            PlayerPrefs.SetString(key, key)
            on_img.localScale = Vector3.zero
            off_img.localScale = Vector3.one
            text.text = NativeText.GetLangStr(9000057)
        end

        game_info:warnStateChanged(data.info.instId)
    end,

    click_Close = function(self)
        frame_manager:hide("warn_info")
    end,

    click_DetailClose = function(self)
        self.detail_panel:SetActive(false)
    end,

    click_CounterClose = function(self)
        self.counter_panel:SetActive(false)
    end,
}