local log = logger.get("Game", "Exp Material Panel")

return class{
    ctor = function(self, info_panel, owner)
        self.panel = info_panel
        self.owner = owner
        self.helper = require("gui.tool.frame_helper")

        self.icon = info_panel:Find('Panel/Top/Grade/Icon'):GetComponent(UnityEngine.UI.Image)
        self.name = info_panel:Find('Panel/Top/Name'):GetComponent(UnityEngine.UI.Text)
        self.count_text = info_panel:Find('Panel/Top/Count'):GetComponent(UnityEngine.UI.Text)
        self.des_text = info_panel:Find('Panel/Top/Des'):GetComponent(UnityEngine.UI.Text)

        self.text_exp = self.panel:Find("Panel/Center/ExpBarBG/ExpValue"):GetComponent(UnityEngine.UI.Text)
        self.curr_exp_bar = self.panel:Find("Panel/Center/ExpBarBG/ExpBarGreen"):GetComponent(UnityEngine.UI.Image)
        self.add_exp_bar = self.panel:Find("Panel/Center/ExpBarBG/ExpBarWhite"):GetComponent(UnityEngine.UI.Image)
        self.amount_text = self.panel:Find("Panel/Center/AmountSlider/Amount"):GetComponent(UnityEngine.UI.Text)

        self.level_from = self.panel:Find("Panel/Center/ExpBarBG/TextLevelFrom"):GetComponent(UnityEngine.UI.Text)
        self.level_to = self.panel:Find("Panel/Center/ExpBarBG/TextLevelTo"):GetComponent(UnityEngine.UI.Text)

        self.slider = self.panel:Find("Panel/Center/AmountSlider"):GetComponent(UnityEngine.UI.Slider)
        
        local path_root = "Frame/MaterialInfo/Panel/Center/AmountSlider/"
        owner:setScript("click", path_root.."Minus", functor.rmake(self.onClick_Minus, self))
        owner:setScript("click", path_root.."Plus", functor.rmake(self.onClick_Plus, self))

        self:hide()

    end,

    -- 在面板上显示 item 信息
    showItem = function(self, item, weapon, count, exp)
        self.helper.showTrans(self.panel)
        self.item = item
        self.weapon = weapon
        self.count = count
        self.exp = exp
        local icon_path = gui_helper:getItemIconPath(item.cfg)
        self.icon:SetSprite(icon_path)
        self.name.text = self.helper:getLangCfg(item.cfg.name_id)
        self.count_text.text = tostring(item.count)
        self.des_text.text = self.helper:getLangCfg(item.cfg.des_id)
        self.amount_text.text = string.format("%d/%d", self.count, item.count)
        self.slider.value = self.count / self.item.count

        self.slider.onValueChanged:RemoveAllListeners()
        self.slider.onValueChanged:AddListener(
            function ()
                self:onValueChanged()
            end
        )

        local add_exp = self.count * self.item.cfg.para1
        self:refreshExpBar(add_exp)
    end,

    onValueChanged = function(self)
        self.count = math.ceil(self.slider.value * self.item.count)
        self.amount_text.text = string.format("%d/%d", self.count, self.item.count)
        local add_exp = self.count * self.item.cfg.para1
        self:refreshExpBar(add_exp)
    end,

    -- 刷新经验条
    refreshExpBar = function(self, add_exp)
        local levelup_cfg = config.weapon_levelups[self.weapon.level]
        if not levelup_cfg then
            log.error('Failed to find item level up cfg, level:'..self.weapon.level)
            return
        end

        -- 经验数值 和 经验条
        add_exp = add_exp + self.exp
        self.text_exp.text = (self.weapon.exp + add_exp)..'/'..levelup_cfg.exp
        self.curr_exp_bar.fillAmount = self.weapon.exp / levelup_cfg.exp
        self.add_exp_bar.fillAmount = (self.weapon.exp + add_exp) / levelup_cfg.exp

        -- 等级变化
        self.level_from.text = tostring(self.weapon.level)
        local to_level = self:getToLevel(self.weapon, add_exp)
        self.level_to.text = tostring(to_level)

    end,

    -- item 加了 add_exp 后能到达的等级
    getToLevel = function(self, item, add_exp)
        local level = item.level
        local exp = item.exp + add_exp
        while true do 
            local cfg = config.weapon_levelups[level]
            if not cfg then
                return level
            end
            if exp >= cfg.exp then
                exp = exp - cfg.exp
                level = level + 1
            else
                break
            end
        end

        return level
    end,

    getMaterial = function(self)
        return {count = self.count, inst = self.item}
    end,

    changeCount = function(self, val)
        self.slider.value = val/self.item.count + self.slider.value
    end,

    onClick_Minus = function(self)
        self.material_info_panel:changeCount(-1)
    end,

    onClick_Plus = function(self)
        self.material_info_panel:changeCount(1)
    end,

    hide = function(self)
        self.helper.hideTrans(self.panel)
    end,

}