local log = logger.get("Game", "Weapon Upgrade result Panel")
local item_helper = require('RTS.prop_sys.prop_enum')

-- 武器升级结构 panel
return class{
    ctor = function(self, panel, owner)
        self.panel = panel
        self.owner = owner

        self.helper = require("gui.tool.frame_helper")

        self.toggle_root = self.panel:Find('Panel/Center')
        self.toggle_white = panel:Find('Panel/Center/Toggle_White'):GetComponent(UnityEngine.UI.Toggle)
        self.toggle_green = panel:Find('Panel/Center/Toggle_Green'):GetComponent(UnityEngine.UI.Toggle)
        self.toggle_blue = panel:Find('Panel/Center/Toggle_Blue'):GetComponent(UnityEngine.UI.Toggle)
        self.toggle_purple = panel:Find('Panel/Center/Toggle_Purple'):GetComponent(UnityEngine.UI.Toggle)
        self.toggle_orange = panel:Find('Panel/Center/Toggle_Orange'):GetComponent(UnityEngine.UI.Toggle)
        self.toggle_all = panel:Find('Panel/Center/Toggle_All'):GetComponent(UnityEngine.UI.Toggle)
        self.toggle_exp_material = panel:Find('Panel/Center/Toggle_Exp_Material'):GetComponent(UnityEngine.UI.Toggle)

        self.toggles = {}
        self.toggles[1] = self.toggle_white
        self.toggles[2] = self.toggle_green
        self.toggles[3] = self.toggle_blue
        self.toggles[4] = self.toggle_purple
        self.toggles[5] = self.toggle_orange
        self.toggles[6] = self.toggle_exp_material

        for k, v in ipairs(self.toggles) do
            v.onValueChanged:AddListener(
                function()
                    local bg = v.transform:Find("BG")
                    bg:SetActive(v.isOn)
                end
            )
        end

        self.toggle_all.onValueChanged:AddListener(
            function()
                for k, v in pairs(self.toggles) do
                    v.isOn = self.toggle_all.isOn
                end

                local bg = self.toggle_all.transform:Find("BG")
                bg:SetActive(self.toggle_all.isOn)
            end
        )

        self.is_visible = false

        self:hide()
    end,

    show = function(self)
        self.helper.showTrans(self.panel)
        self.is_visible = true
        gui_helper:setToggleState(self.toggle_root, "fast_upgrade_toggles")
        self.toggle_white.isOn = true
        self.toggle_green.isOn = true
        --self.select_all = self.toggle_all.isOn
    end,

    --[[ update = function(self)
        if self.is_visible then
            for k, v in pairs(self.toggles) do
                if self.select_all ~= self.toggle_all.isOn then
                    v.isOn = self.toggle_all.isOn
                    self.select_all = self.toggle_all.isOn
                end
                local bg = v.transform:Find("BG")
                bg:SetActive(v.isOn)
            end
            if self.select_all ~= self.toggle_all.isOn then
                for k, v in pairs(self.toggles) do
                    v.isOn = self.toggle_all.isOn
                    local bg = v.transform:Find("BG")
                    bg:SetActive(v.isOn)
                end
                self.select_all = self.toggle_all.isOn
            end
        end
    end, ]]

    hide = function(self)
        self.helper.hideTrans(self.panel)
        self.is_visible = false
    end,

    -- 获取选择品质的道具
    getSelectedItems = function(self, item_list)
        local items = {}
        local materials = {}
        local weapon_bag_type = 1
        local bag = bag_manager:getBag(weapon_bag_type)

        -- 白绿蓝紫橙 1,2,3,4,5
        if self.toggle_white.isOn then
            log.info('### white is on!!')
            bag:getItemsByQualityFromList(1, items, item_list)
        end
        if self.toggle_green.isOn then
            log.info('### green is on!!')
            bag:getItemsByQualityFromList(2, items, item_list)
        end
        if self.toggle_blue.isOn then
            log.info('### blue is on!!')
            bag:getItemsByQualityFromList(3, items, item_list)
        end
        if self.toggle_purple.isOn then
            log.info('### purple is on!!')
            bag:getItemsByQualityFromList(4, items, item_list)
        end
        if self.toggle_orange.isOn then
            log.info('### purple is on!!')
            bag:getItemsByQualityFromList(5, items, item_list)
        end
        if self.toggle_exp_material.isOn then
            local type = gui_helper.current_item.cfg.family + 23
            materials = bag_manager:getItemsByType(type)
        end

        gui_helper:saveToggleState(self.toggle_root, "fast_upgrade_toggles")

        return items, materials
    end,

}