local log = logger.get("Game", "Weapon Upgrade Panel")
local item_helper = require('RTS.prop_sys.prop_enum')

-- 武器升级 panel
return class{
    ctor = function(self, info_panel)
        self.panel = info_panel
        self.helper = require("gui.tool.frame_helper")

        self.icon = info_panel:Find('Top/Grade/Icon'):GetComponent(UnityEngine.UI.Image)
        self.grade = info_panel:Find('Top/Grade'):GetComponent(UnityEngine.UI.Image)
        self.name = info_panel:Find('Top/Name'):GetComponent(UnityEngine.UI.Text)
        self.level = info_panel:Find('Top/Grade/Level'):GetComponent(UnityEngine.UI.Text)
        self.skill_1_icon = info_panel:Find('Bottom/Skill_1'):GetComponent(UnityEngine.UI.Image)
        self.skill_2_icon = info_panel:Find('Bottom/Skill_2'):GetComponent(UnityEngine.UI.Image)
        self.skill_des_1 = info_panel:Find('Bottom/Skill_1/Des'):GetComponent(UnityEngine.UI.Text)
        self.skill_des_2 = info_panel:Find('Bottom/Skill_2/Des'):GetComponent(UnityEngine.UI.Text)
        self.type_text = info_panel:Find('Top/Type'):GetComponent(UnityEngine.UI.Text)

        self.text_exp = self.panel:Find("Center/ExpBarBG/ExpValue"):GetComponent(UnityEngine.UI.Text)
        self.curr_exp_bar = self.panel:Find("Center/ExpBarBG/ExpBarGreen"):GetComponent(UnityEngine.UI.Image)
        self.add_exp_bar = self.panel:Find("Center/ExpBarBG/ExpBarWhite"):GetComponent(UnityEngine.UI.Image)

        self.level_from = self.panel:Find("Center/TextLevelFrom"):GetComponent(UnityEngine.UI.Text)
        self.level_to = self.panel:Find("Center/TextLevelTo"):GetComponent(UnityEngine.UI.Text)

        self.max_level = self.panel:Find("Center/MaxLevel"):GetComponent(UnityEngine.UI.Text)

        --self.rand_prop_up =self.panel:Find("Bottom/TextRandPropUp"):GetComponent(UnityEngine.UI.Text)
        self.skill_1_icon.transform:SetActive(false)
        self.skill_2_icon.transform:SetActive(false)

        self.skill_trans = {}
        self.skill_trans[1] = {icon = self.skill_1_icon, text = self.skill_des_1}
        self.skill_trans[2] = {icon = self.skill_2_icon, text = self.skill_des_2}

        self.cfg_max = 0

        config.weapon_levelups:foreach(
            function(k, v)
                self.cfg_max = self.cfg_max + 1
            end
        )

    end,

    -- 在面板上显示 item 信息
    showItem = function(self, item)
        local icon_path = gui_helper:getItemIconPath(item.cfg)
        self.icon:SetSprite(icon_path)
        self.grade:SetSprite(gui_helper:getQualityIconPath(item.cfg.color))
        self.name.text = self.helper:getLangCfg(item.cfg.name_id)
        self.level.text = "Lv."..item.level

        if item.cfg.family == 1 then
            self:loadSkills(item)
        elseif item.cfg.family == 2 then   
            self:loadPassiveSkills(item)
        end

        local prop_panel = self.panel:Find('Props')
        gui_helper:showUpgradeProps(prop_panel, item.props, nil, true)

        --self:refreshExpBar(item, 0)

        self.name.color = gui_helper:quality2UnityColor(item.cfg.color)

        self.type_text.text = gui_helper:getWeaponType(item.cfg)

        self.lv_max = 0

        if item.cfg.family == 1 then
            self.lv_max = local_player:getWeaponMaxLv()
        elseif item.cfg.family == 2 then
            self.lv_max = local_player:getEquipMaxLv()
        end

        self.max_level.text = "当前最高Lv."..self.lv_max
    end,

    loadSkills = function(self, equipment)
        local skill_id = equipment.cfg.skill_id
        local des = gui_helper:getSkillDes(skill_id)
        self.skill_des_1.text = des or ""
        self.skill_1_icon:SetSprite(gui_helper:getSkillIconPath(skill_id))

        skill_id = equipment.cfg.opening_skill
        des = gui_helper:getSkillDes(skill_id)
        self.skill_des_2.text = des or ""
        self.skill_2_icon:SetSprite(gui_helper:getSkillIconPath(skill_id))
    end,

    loadPassiveSkills = function(self, equipment)
        local skills = equipment.cfg.passive_skill
        local list = loadstring("return "..skills)()

        if type(list) == "table" then
            local idx = 1
            for k, v in pairs(list) do
                local icon = self.skill_trans[idx].icon
                local text = self.skill_trans[idx].text
                icon.transform:SetActive(true)

                local cfg = passive_skill_helper:getSkill(k, v)

                if cfg then
                    text.text = self.helper:getLangCfg(cfg.skill_des)
                    icon:SetSprite(cfg.skill_icon_path..cfg.skill_icon_name)
                else
                    log.error("###### passive skill cfg error ! id "..k.." level "..v)
                end

                idx = idx + 1
            end
        end
    end,

    -- 刷新经验条
    refreshExpBar = function(self, item, add_exp)
        local levelup_cfg = config.weapon_levelups[item.level]
        if not levelup_cfg then
            log.error('Failed to find item level up cfg, level:'..item.level)
            return
        end

        -- 等级变化
        self.level_from.text = "Lv "..tostring(item.level)
        local to_level = self:getToLevel(item, add_exp)

        if to_level >= self.cfg_max then
            self.level_to.text = string.format( "Lv.%d(Max)", self.cfg_max )
        elseif to_level == item.level and item.level ~= self.lv_max then
            self.level_to.text = NativeText.GetLangStr(110556)
        else
            self.level_to.text = "Lv "..tostring(to_level)

            if to_level >= self.lv_max then
                self.level_to.text = string.format( "%s(Max)", self.level_to.text )
            end
        end

        -- 经验数值 和 经验条
        self.text_exp.text = (item.exp + add_exp)..'/'..levelup_cfg.exp
        self.text_exp.transform:SetActive(item.level < self.cfg_max)

        if to_level >= self.cfg_max then
            self.curr_exp_bar.fillAmount = 1
            self.add_exp_bar.fillAmount = 0
        else
            self.curr_exp_bar.fillAmount = item.exp / levelup_cfg.exp
            self.add_exp_bar.fillAmount = (item.exp + add_exp) / levelup_cfg.exp
        end


        -- 属性变化
        --local atk_add = (to_level - item.level) * item.cfg.ATK_INC
        local add_props = loadstring("return "..item.cfg.attr_grow)()
        local change_props = {}

        if type(add_props) == "table" then
            for k, v in pairs(add_props) do
                local value = item:getProp(k)
                change_props[k] = value + v*(to_level - item.level)
            end
        end

        local prop_panel = self.panel:Find('Props')
        if to_level > item.level then
            gui_helper:showUpgradeProps(prop_panel, item.props, change_props, true)
        else
            gui_helper:showUpgradeProps(prop_panel, item.props, nil, true)
        end

        -- 将获得随机属性提示
        --[[ if math.floor(item.level/5) ~= math.floor(to_level/5) then
            self.rand_prop_up.transform:SetActive(true)
        else
            self.rand_prop_up.transform:SetActive(false)
        end ]]
    end,

    -- item 加了 add_exp 后能到达的等级
    getToLevel = function(self, item, add_exp)
        local level = item.level
        local exp = item.exp + add_exp
        while true do 
            local cfg = config.weapon_levelups[level]
            if not cfg then
                break
            end

            if exp >= cfg.exp then
                exp = exp - cfg.exp
                level = level + 1
            else
                break
            end
        end

        return level
    end,

}