local super = require("ui.frame")
local log = logger.get("Game", "Weapon Upgrade Frame")
local gui_helper = require('gui.gui_helper')
require('game.player')

return class
{
    super = super,
    
    onLoaded = function (self)
        super.onLoaded(self)

        self.owner = self.args.owner

        -- 路径: root/login
        -- 事件: click_login 
        clickEvents = {
            "Frame/Top/ButtonBack",
            "Frame/Top/ButtonHome",
            "Frame/Top/AddRes",
            -- 一键选择
            "Frame/Right/Bottom/ButtonEasySelect",
            -- 强化
            "Frame/Right/Bottom/ButtonUpgrade",
            -- 强化结果
            "Frame/UpgradeResult/ResultBG",
            "Frame/UpgradeResult/Frame/ResultConfirm",
            -- 快速选择
            "Frame/FastUpgrade/FastBG",
            -- 提升一级
            "Frame/FastUpgrade/Panel/ButtonUpgradeOne",
            -- 提升十级
            "Frame/FastUpgrade/Panel/ButtonUpgradeTen",
            -- 经验材料 panel
            "Frame/MaterialInfo/MaterialBG",
            "Frame/MaterialInfo/Panel/Bottom/ButtonCancel",
            "Frame/MaterialInfo/Panel/Bottom/ButtonConfirm",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        if self.owner then
            self.helper.hideTrans(self.owner.transform)
        end
        -- item prefab
        self.item_prefab = self.transform:Find('Prefabs/Slot/Item').gameObject
        self.slot_prefab = self.transform:Find('Prefabs/Slot').gameObject
        self.helper.hideTrans(self.transform:Find('Prefabs'))

        -- 显示材料的 scroll view
        self.scroll_view = self.transform:Find('Frame/Right/Scroll View'):GetComponent(UnityEngine.UI.ScrollRect)
        self.slot_root_path = 'Frame/Right/Scroll View/Viewport/SlotRoot'
        self.slot_root = self.transform:Find(self.slot_root_path)

        -- 升级信息 panel
        local panel = self.transform:Find('Frame/Left')
        self.upgrade_panel = require('gui.weapon_upgrade.weapon_upgrade_panel')(panel)

        -- 快速升级 panel
        panel = self.transform:Find('Frame/FastUpgrade')
        self.fast_upgrade_panel = require('gui.weapon_upgrade.fast_upgrade_panel')(panel, self)
        self.button_upgrade_one = self.transform:Find('Frame/FastUpgrade/Panel/ButtonUpgradeOne'):GetComponent(UnityEngine.UI.Button)
        self.button_upgrade_ten = self.transform:Find('Frame/FastUpgrade/Panel/ButtonUpgradeTen'):GetComponent(UnityEngine.UI.Button)
        self.fast_upgrade_level = 0 -- 0-无快速选择  1-快速升级一级  10-快速升级十级

        -- 升级结果 panel
        panel = self.transform:Find("Frame/UpgradeResult")
        self.result_panel = require('gui.weapon_upgrade.result_panel')(panel)

        -- 显示英雄所有装备的 scroll view
        --self.equip_scroll_view = self.transform:Find('Frame/ScrollViewEquips'):GetComponent(UnityEngine.UI.ScrollRect)
        --self.equip_slot_root = self.equip_scroll_view.content

        -- 强化材料栏
        --self.materials_layout = self.transform:Find('Frame/MaterialLayout')

        -- 经验材料信息 panel
        panel = self.transform:Find('Frame/MaterialInfo')
        self.material_info_panel = require('gui.weapon_upgrade.exp_material_panel')(panel, self)

        self.cfg_max = 0

        config.weapon_levelups:foreach(
            function(k, v)
                self.cfg_max = self.cfg_max + 1
            end
        )

        self.item = gui_helper.current_item -- 强化目标道具
        self.to_level = self.item.level
        self.upgrade_panel:showItem(self.item)
        self.upgrade_panel:refreshExpBar(self.item, 0)

        self.lv_max = 0

        if self.item.cfg.family == 1 then
            self.lv_max = local_player:getWeaponMaxLv()
        elseif self.item.cfg.family == 2 then
            self.lv_max = local_player:getEquipMaxLv()
        end

        self.selected_items = {}    -- 选择用作材料的道具
        self.selected_materials = {}    --选中的经验道具

        -- 消耗铜币
        self.text_cost = self.transform:Find('Frame/Right/Bottom/Cost/Count'):GetComponent(UnityEngine.UI.Text)
        self.text_cost.text = '0'

        -- 钻石
        --self.text_gem = self.transform:Find('Frame/TopRight/Gem/Num'):GetComponent(UnityEngine.UI.Text)

        -- 金币
        self.text_coin = self.transform:Find('Frame/Top/AddRes/Text'):GetComponent(UnityEngine.UI.Text)

        self.event_ids = {}
        -- 注册属性变化事件处理
        self.event_ids["onPropChange"] = local_player.events:on("onPropChange", functor.make(self.refreshResourceNum, self))
        self:refreshResourceNum()

        -- 注册道具事件
        self.event_ids["onBagChanged"] = bag_manager.events:on("onBagChanged", functor.make(self.onBagChanged, self))
        self.event_ids["onItemUpgrade"] = bag_manager.events:on("onItemUpgrade", functor.make(self.onItemUpgrade, self))
        
        -- 缓存所有已注册的 icon 事件，刷新时要清空
        self.icon_events = {}

        --self:refreshScrollViewEquips()

        -- 当前排序，背包类型
        self.sort =  nil
        self.sort_invert = false

        self.weapon_bag = 1

        self:refresh('level', false)

        self.is_dirty = false   -- 背包发生变化
    end,

    onUnload = function(self)
        local_player.events:remove("onPropChange", self.event_ids["onPropChange"] )
        bag_manager.events:remove("onBagChanged", self.event_ids["onBagChanged"])
        bag_manager.events:remove("onItemUpgrade", self.event_ids["onItemUpgrade"])
        self.upgrade_panel = nil
        self.fast_upgrade_panel = nil
        self.result_panel = nil

    end,

    -- 刷新金币，钻石
    refreshResourceNum = function(self)
        if self.text_coin then
            local str = self.helper:itemNumFormat(local_player.props[protocol_pb.PROP_GOLD])
            self.text_coin.text = str
        end
    end,

    -- 背包发生变化
    onBagChanged = function(self)
        log.info('#### onBagChanged!!')
        self.is_dirty = true
    end,

    -- 道具升级完成
    onItemUpgrade = function(self, item, src_level, src_props)
        self.result_panel:showItem(item, src_level, src_props, item.level, item.props)
        self:clearSelectedItems()
    end,

    update = function(self, dt)
        if self.is_dirty then
            self.selected_items = {}
            --gui_helper.clearSlotsAfter(self, 0)
            self:refresh(self.sort, self.sort_invert)
            local bag = bag_manager:getBag(self.item.cfg.bagtype)
            local item = bag:getItem(self.item.inst_id)
            --log.info('item in bag, level:'..item.level..' self.item level:'..self.item.level)

            self.upgrade_panel:showItem(self.item)
            self.upgrade_panel:refreshExpBar(self.item, 0)

            self.is_dirty = false
        end

        --[[ if self.fast_upgrade_panel then
            self.fast_upgrade_panel:update()
        end ]]
    end,

    click_ButtonBack = function(self)
        log.info('click_ButtonBack')
        frame_manager:hide('weapon_upgrade')
        if self.owner then
            self.helper.showTrans(self.owner.transform)
        end
    end,

    click_ButtonHome = function(self)
        frame_manager:clickHome()
    end,
    
    clearIconScripts = function(self)
        for path, evt in pairs(self.icon_events) do
            self:clearScript(evt, path)
        end
        self.icon_events = {}

    end,

    -- 根据当前的刷新背包
    refresh = function(self, sort, sort_invert)
        if (not self.is_dirty) and self.sort == sort and sort_invert == sort_invert then
            return
        end

        self.sort = sort
        self.sort_invert = sort_invert

        self.item_num = 0

        -- 清空原来的 icon 事件处理
        self:clearIconScripts()

        local mat_num = self:refreshMaterials()
        self.item_num = mat_num

        local bag = bag_manager:getBag(self.item.cfg.bagtype)
        if bag then

            local raw_items = bag:getItems(self.item.cfg.type)  
            self.items = {}    
            -- 再筛选一次
            if raw_items then
                for _, item in ipairs(raw_items) do
                    if self.item.cfg.family == item.cfg.family --
                    and item.inst_id ~= self.item.inst_id 
                    and not item.status then
                        table.insert(self.items, item)
                    end
                end
            end

            -- 筛选未上锁的道具
            self.items = item_helper:getUnlockedItems(self.items)

            function comp(a, b)
                if a.total_exp == b.total_exp then
                    if a.cfg.color == b.cfg.color then
                        if a.cfg_id == b.cfg_id then
                            return a.inst_id < b.inst_id
                        else
                            return a.cfg_id < b.cfg_id
                        end
                    else
                        return a.cfg.color < b.cfg.color
                    end
                end
                return a.total_exp < b.total_exp
            end

            table.sort( self.items, comp )

            -- 重置 slot 的数量
            --self:setSlotNum(math.max(#items, 5))
            self:setSlotNum(#self.items+mat_num)

            if self.items then
                log.info('item num:'..#self.items)
                local max_idx = mat_num
                for idx, item in ipairs(self.items) do
                    gui_helper.setItemToSlot(self, idx+mat_num, item)
                    max_idx = max_idx + 1
                end
            
                gui_helper:clearSlotsAfter(self.slot_root, max_idx)

                self.item_num = max_idx
                -- 滚动置顶
                --[[ local pos = self.scroll_view.content.localPosition
                pos.y = 0
                self.scroll_view.content.localPosition = pos ]]

            else
                log.error('No items!!')
            end
        else
            log.error('Failed to find bag:'..bag_type)
        end

        self.is_dirty = false
    end,

    -- 设置 slot 的数量 
    setSlotNum = function(self, num)
        log.info('set slot num:'..num)
        local idx = 0
        for idx=0, num-1, 1 do
            local slot = nil
            if idx < self.slot_root.childCount then
                -- slot 已经存在
                slot = self.slot_root:GetChild(idx)
                slot:SetActive(true)
            else
                -- slot 不存在，实例化一个
                slot = GameObject.Instantiate(self.slot_prefab)
                slot.name = 'slot_'..idx
                slot.transform:SetParent(self.slot_root)
                makeRectTransformIdentity(slot.transform)
            end
        end 

        if num == self.slot_root.childCount then
            return
        end

        -- 隐藏后续的 slot
        for idx = num, self.slot_root.childCount-1, 1 do
            local slot = self.slot_root:GetChild(idx)
            slot:SetActive(false)
        end

    end,

    -- 设置左侧的当前装备 Scroll view
    refreshScrollViewEquips = function(self)
        if not self.item.hero_id then
            self.hero_bag = nil
            -- 没有被装备的武器
            self:setEquipScrollViewIcon(self.item, 0)
            local idx = nil
            for idx=1,4,1 do
                self:setEquipScrollViewIcon(nil, idx)
            end
        else
            self.hero_bag = bag_manager:getHeroBag(self.item.hero_id)
            local weapon = self.hero_bag:get(common_pb.MAIN_WEAPON_SLOT)
            self:setEquipScrollViewIcon(weapon, 0)

            weapon = self.hero_bag:get(common_pb.MINOR_WEAPON_SLOT_1)
            self:setEquipScrollViewIcon(weapon, 1)

            weapon = self.hero_bag:get(common_pb.MINOR_WEAPON_SLOT_2)
            self:setEquipScrollViewIcon(weapon, 2)

            weapon = self.hero_bag:get(common_pb.MINOR_WEAPON_SLOT_3)
            self:setEquipScrollViewIcon(weapon, 3)

            weapon = self.hero_bag:get(common_pb.MINOR_WEAPON_SLOT_4)
            self:setEquipScrollViewIcon(weapon, 4)

        end
    end,

    -- 将 item icon 设置给 equip scroll view
    setEquipScrollViewIcon = function(self, item, idx)
        local icon_root = self.equip_slot_root:GetChild(idx)
        local icon = icon_root:GetChild(0)
        if item then
            icon_root:SetActive(true)
            icon:SetActive(true)
            local img = icon:GetComponent(UnityEngine.UI.Image)
            local icon_path = gui_helper:getItemIconPath(item.cfg)
            img:SetSprite(icon_path)

            -- 设置回调函数
            local path = "Frame/ScrollViewEquips/Viewport/Content/Image_"..idx
            --log.info('#### path:'..path)
            self:setScript("single", path , functor.rmake(self.onClick_EquipIcon, idx))
            self.icon_events[path] = "single"

        else
            icon_root:SetActive(false)
            icon:SetActive(false)
        end
    end,

    idxToSlot = function(self, idx)
        if idx == 0 then
            return common_pb.MAIN_WEAPON_SLOT
        elseif idx == 1 then
            return common_pb.MINOR_WEAPON_SLOT_1
        elseif idx == 2 then
            return common_pb.MINOR_WEAPON_SLOT_2
        elseif idx == 3 then
            return common_pb.MINOR_WEAPON_SLOT_3
        elseif idx == 4 then
            return common_pb.MINOR_WEAPON_SLOT_4
        end
    end,

    -- 点击了一个装备
    onClick_EquipIcon = function(self, handler, data, idx)
        if not self.hero_bag then return end

        local icon_root = self.equip_slot_root:GetChild(idx)

        local slot = self:idxToSlot(idx)
        self.item = self.hero_bag:get(slot)
        self.upgrade_panel:showItem(self.item)
        self.upgrade_panel:refreshExpBar(self.item, 0)

        self.selected_items = {}    -- 选择用作材料的道具
        self:refreshSelectedItems()

        for i=0, self.equip_slot_root.childCount, 1 do
            local child = self.equip_slot_root:GetChild(i)
            local img = child:GetComponent(UnityEngine.UI.Image)
            if i == idx then 
                --img:SetSprite("")
            else
                --img:SetSprite("")
            end
        end

    end,

    -- 点击了一个道具
    onClick_ItemIcon = function(self, handler, data, item)
        log.info('### click item:'..item.inst_id)

        local id = item.inst_id
        if self.selected_items[id] then
            -- 已选择过 item,取消选择 item
            self.selected_items[id] = nil
            local icon = item.ui:Find('Icon'):GetComponent(UnityEngine.UI.Image)
            --icon.color = Color.white
            item.ui:Find('Selected'):SetActive(false)
        else
            --选择  item
            if self:isMaxLevel(self.to_level) then
                return
            end
            self.selected_items[id] = item
            local icon = item.ui:Find('Icon'):GetComponent(UnityEngine.UI.Image)
            --icon.color = Color.green
            item.ui:Find('Selected'):SetActive(true)
        end

        self:refreshSelectedItems()

    end,

    refreshSelectedItems = function(self)
        local upgrade_cfg = config.weapon_upgradeconfigs[1]
        if not upgrade_cfg then
            log.error('Failed to find weapon_upgradeconfigs:'..self.item.cfg_id)
            return
        end

        local sum_exp = 0
        for _,item in pairs(self.selected_items) do
            sum_exp = sum_exp + item.total_exp* (upgrade_cfg.exppercent / 10000) + item.cfg.basicexp
        end

        for _,item in pairs(self.selected_materials) do
            sum_exp = sum_exp + item.inst.cfg.para1*item.count
        end

        -- 计算要消耗的 coin
        log.info('#### sum_exp:'..sum_exp)
        local coin = 0

        if sum_exp > 0 then
            coin = upgrade_cfg.p1 + upgrade_cfg.p2 * sum_exp + sum_exp*sum_exp*upgrade_cfg.p3
        end

        self.text_cost.text = tostring(math.floor(coin))
        if coin < local_player:getCoin() then
            self.text_cost.color = Color.white
        else
            self.text_cost.color = Color.red
        end

        self.upgrade_panel:refreshExpBar(self.item, sum_exp)
        self.to_level = self.upgrade_panel:getToLevel(self.item, sum_exp)
    end,

    -- 快速选择
    click_ButtonEasySelect = function(self)
        if self:isMaxLevel(self.to_level) then
            return
        end

        if self.item_num > 0 then
            self.fast_upgrade_panel:show()
        else
            hint:setText(NativeText.GetLangStr(110560))
        end
    end,

    -- 武器升级
    click_ButtonUpgrade = function(self)
        if self.to_level > self.lv_max then
            hint:setText("本次强化将导致大量经验浪费，无法继续")
            return true
        end

        local cb = function()
            local consume_items = {}
            for k,v in pairs(self.selected_items) do
                table.insert(consume_items, k)
            end
    
            local exp_items = {}
            for k,v in pairs(self.selected_materials) do
                local tab = {}
                tab.uId = v.inst.inst_id
                tab.itemId = 0
                tab.getNum = v.count
                table.insert( exp_items, tab )
            end
            network_sender:upgradeItem(self.item.hero_id, self.item.inst_id, consume_items, exp_items)
        end

        if self.to_level > self.cfg_max then
            gui_helper:showCommonTipBox(cb, "本次强化溢出的经验会被清零，是否继续？")
        else
            cb()
        end
    end,

    -- 购买齿轮
    click_AddRes = function(self)
        frame_manager:show('resource_convert')
    end,

    -- 关闭强化结果 panel
    click_ResultBG = function(self)
        self.result_panel:hide()
        self.text_cost.text = '0'
        
        if self.fast_upgrade_level == 1 then 
            self:click_ButtonUpgradeOne()
        elseif self.fast_upgrade_level == 10 then
            self:click_ButtonUpgradeTen()
        else

        end

    end,

    click_ResultConfirm = function(self)
        self.result_panel:hide()
        self.text_cost.text = '0'
        
        if self.fast_upgrade_level == 1 then 
            self:click_ButtonUpgradeOne()
        elseif self.fast_upgrade_level == 10 then
            self:click_ButtonUpgradeTen()
        else

        end
    end,

    click_FastBG = function(self)
        self.fast_upgrade_panel:hide()
    end,

    -- 提升一级
    click_ButtonUpgradeOne = function(self)
        self:clearSelectedItems()
        self:selectItemsToLevel(self.item.level + 1)
        self:refreshSelectedItems()
        self.fast_upgrade_level = 1

        --self.button_upgrade_one.targetGraphic.sprite = self.button_upgrade_one.spriteState.pressedSprite
        --self.button_upgrade_ten.targetGraphic.sprite = self.button_upgrade_ten.spriteState.disabledSprite

        self.fast_upgrade_panel:hide()
    end,

    -- 提升十级
    click_ButtonUpgradeTen = function(self)
        self:clearSelectedItems()

        local level = self.item.level + 10
        local max = self.lv_max > self.cfg_max and self.cfg_max or self.lv_max
        level = level > max and max or level
        self:selectItemsToLevel(level)
        self:refreshSelectedItems()
        self.fast_upgrade_level = 10

        --self.button_upgrade_ten.targetGraphic.sprite = self.button_upgrade_ten.spriteState.pressedSprite
        --self.button_upgrade_one.targetGraphic.sprite = self.button_upgrade_one.spriteState.disabledSprite
        
        self.fast_upgrade_panel:hide()
    end,

    -- 清除选择的道具
    clearSelectedItems = function(self)
        for k,item in pairs(self.selected_items) do
            --local icon = item.ui:Find('Icon'):GetComponent(UnityEngine.UI.Image)
            --icon.color = Color.white
            item.ui:Find('Selected'):SetActive(false)
        end

        for k, mat in pairs(self.selected_materials) do
            self:setMaterialNum(mat.inst, 0)
        end

        self.selected_items = {}
        self.selected_materials = {}
    end,

    selectItemsToLevel = function(self, to_level)
        local items, materials = self.fast_upgrade_panel:getSelectedItems(self.items)

        -- 判断升到目标等级，需要的经验
        local upgrade_cfg = config.weapon_upgradeconfigs[1]
        local expect_exp = 0    -- 期望经验
        local max_exp = 0
        local level = 0

        for level = self.item.level, to_level, 1 do
            local levelup_cfg = config.weapon_levelups[level]
            if levelup_cfg then
                if level < to_level then
                    expect_exp = expect_exp + levelup_cfg.exp
                end
                max_exp = max_exp + levelup_cfg.exp
            else
                break
            end
        end
        -- 减去已获得的经验，乘上经验因子
        expect_exp = (expect_exp - self.item.exp)

        -- 根据 item 的  total_exp 将 item 排序
        function comp(a, b)
            if a.total_exp == b.total_exp then
                if a.cfg.color == b.cfg.color then
                    if a.cfg_id == b.cfg_id then
                        return a.inst_id < b.inst_id
                    else
                        return a.cfg_id < b.cfg_id
                    end
                else
                    return a.cfg.color < b.cfg.color
                end
            end
            return a.total_exp < b.total_exp
        end

        table.sort(items, comp)

        self.selected_items = {}
        self.selected_materials = {}
        local exp_factor = upgrade_cfg.exppercent / 10000
        local total_exp = 0
        --local selected = {}

        -- 遍历 materials, 直到总经验大于 expect_exp
        local calculateMatExp = function()
            local num = 0
            for _,v in ipairs(materials) do
                if total_exp > expect_exp then
                    break
                end
                for i=1, v.count do
                    if total_exp > expect_exp then
                        break
                    end
                    num = num + 1
                    total_exp = total_exp + v.cfg.para1
                end
                self.selected_materials[v.inst_id] = {inst = v, count = num}
                self:setMaterialNum(v, num)
            end
        end
        calculateMatExp()

        -- 遍历 items, 直到总经验大于 expect_exp
        for _,v in ipairs(items) do
            if v.inst_id ~= self.item.inst_id then
                local item_exp = v.cfg.basicexp + v.total_exp * exp_factor
                if total_exp > expect_exp or (total_exp + item_exp) > max_exp then
                    break
                end

                total_exp = total_exp + item_exp
                --table.insert(selected, v)
                self.selected_items[v.inst_id] = v
                local icon = v.ui:Find('Icon'):GetComponent(UnityEngine.UI.Image)
                --icon.color = Color.green
                v.ui:Find('Selected'):SetActive(true)

                log.info('#### fast select add item:'..v.inst_id)
            end
        end
    end,

    refreshMaterials = function(self)
        local family = self.item.cfg.family
        local mat_cfg = item_helper:getWeaponExpMaterialCfg(family+23)
        local mats_in_bag = bag_manager:getItemsByType(family+23)
        local index = 0
        local mat_cfg_ids = {}

        --显示背包里的材料
        for idx, mat in ipairs(mats_in_bag) do
            self:setMaterialToSlot(idx, mat.cfg, mat)
            mat_cfg_ids[mat.cfg_id] = mat.cfg_id
            index = index + 1
        end

        --显示未获得的材料
        --[[ for idx, cfg in ipairs(mat_cfg) do
            if not mat_cfg_ids[cfg.id] then
                self:setMaterialToSlot(index, cfg)
                index = index + 1
            end
        end ]]

        return index
    end,

    setMaterialToSlot = function(self, idx, cfg, mat)
        local childCount = self.slot_root.childCount
        local slot = nil
        if idx > childCount then
            slot = GameObject.Instantiate(self.slot_prefab).transform
            slot:SetParent(self.slot_root)
            resetTransformZ(slot)
        else
            slot = self.slot_root:GetChild(idx-1)
            slot:SetActive(true)
        end

        if not slot then
            return
        end

        local item = slot:GetChild(0)
        slot.name = "mat_"..idx

        local grade = item:GetComponent(UnityEngine.UI.Image)
        local icon = item:Find('Icon')
        local num = item:Find('Num'):GetComponent(UnityEngine.UI.Text)
        local level = item:Find('Level')
        
        level:SetActive(false)
        num.transform:SetActive(false)
        --num.text = self.helper:getItemNum(mat.cfg_id)

        grade:SetSprite(gui_helper:getQualityIconPath(cfg.color))

        -- 设置 icon
        local img = icon:GetComponent(UnityEngine.UI.Image)
        local icon_path = gui_helper:getItemIconPath(cfg)
        img:SetSprite(icon_path)
        img.color = Color.gray

        -- 背包里有材料  注册点击事件
        if mat then
            img.color = Color.white
            mat.ui = item
            local path = self.slot_root_path.."/"..slot.name.."/"..item.name.."/Icon"
            --log.info('#### path:'..path)
            self:setScript("single", path , functor.rmake(self.onClick_MaterialIcon, mat))
            self.icon_events[path] = "single"
        end
    end,

    setMaterialNum = function(self, material, count)
        local num = material.ui:Find('Num'):GetComponent(UnityEngine.UI.Text)
        num.text = tostring(count)
        num.transform:SetActive(count > 0)
    end,

    getSelectedWeaponExp = function(self)
        local upgrade_cfg = config.weapon_upgradeconfigs[1]
        if not upgrade_cfg then
            log.error('Failed to find weapon_upgradeconfigs:'..self.item.cfg_id)
            return
        end
        local sum_exp = 0
        for _,item in pairs(self.selected_items) do
            sum_exp = sum_exp + item.total_exp* (upgrade_cfg.exppercent / 10000) + item.cfg.basicexp
        end

        return sum_exp
    end,

    onClick_MaterialIcon = function(self, handler, data, item)
        log.info("material id ",item.inst_id)
        local count = 0
        local exp = self:getSelectedWeaponExp()
        if self.selected_materials[item.inst_id] then
            count = self.selected_materials[item.inst_id].count
        end
        self.material_info_panel:showItem(item, self.item, count, exp)
    end,

    click_MaterialBG = function(self)
        self.material_info_panel:hide()
    end,

    click_ButtonCancel = function(self)
        self.material_info_panel:hide()
    end,

    click_ButtonConfirm = function(self)
        self.material_info_panel:hide()
        local material = self.material_info_panel:getMaterial()
        self.selected_materials[material.inst.inst_id] = material
        self:setMaterialNum(material.inst, material.count)
        self:refreshSelectedItems()
    end,

    isMaxLevel = function(self, level)
        if level >= self.lv_max or level >= self.cfg_max then
            hint:setText("装备已达到强化等级上限")
            return true
        end

        return false
    end,
}
