local super = require("ui.frame")
local log = logger.get("Game", "World Map Frame")
require('RTS.bag_sys.bag_manager')

return class {
    super = super,

    onLoaded = function(self)
        log.info('World Map Frame onLoaded!!')

        super.onLoaded(self)

        self.owner = self.args.owner

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        
        self.input_num_trans = self.transform:Find('inputNum')
        self.helper.hideTrans(self.input_num_trans)

        
        self.diamond_text = self:findText('new/topRight/Asset Gem/Text')

        -- 当前坐标显示框
        self.text_coordinate = self:findText("new/topRight/moveCoord/coordinateText")

        -- 世界地图进度ui
        --[[ local node = self.transform:Find("new/m_line")
        self.march_list = require("gui.march_list_panel")(node, self)

        self.march_line_ui = {} ]]

        self.enemy_warn_text = self:findText("new/WarningNode/EnemyWarn/Time")
        self.reinforce_text = self:findText("new/WarningNode/Reinforce/Time")
        self.warn_cnt_text = self:findText("new/WarningBtn/RedPoint/Count")

        self.enemy_warn_panel = self.transform:Find("new/WarningNode/EnemyWarn")
        self.reinforce_panel = self.transform:Find("new/WarningNode/Reinforce")
        self.warn_btn = self.transform:Find("new/WarningBtn")

        -- 路径: root/login
        -- 事件: click_login 
        clickEvents = {
            "new/buttomLeft/bg/homeBtn",
            "new/buttomLeft/bg/layout/emailBtn",
            "new/buttomLeft/bg/layout/carBtn",
            "new/buttomLeft/bg/layout/storeHouseBtn",
            "new/buttomLeft/bg/layout/dsfdsf/unionBtn",
            "new/buttomLeft/bg/layout/setBtn",

            "new/buttomLeft/bg/layout/dsfdsf/GameObject (4)/taskBtn",
            "new/buttomLeft/bg/layout/dsfdsf/GameObject (4)/dailyBtn",
            "new/buttomLeft/bg/layout/dsfdsf/GameObject (4)/achieveBtn",


            "new/buttomLeft/bg/chatBtn",
            -- "new/buttomLeft/bg/chatChannelBtn",
            "new/buttomLeft/bg/layout/robinBtn",


            "new/range/arrow/offset/lookAtHomeBtn",
            "new/buttomRight/SearchBtn",
            "new/buttomRight/GridBtn",
            "new/topLeft/bg/MiniMap",
       
            -- 修改界面
            "new/topRight/moveCoord",
            "new/topRight/collectCoord",
            "inputNum/left/positioningBtn",
            "inputNum/closeInputNum",

            -- 添加资源
            "new/topRight/res layout/Asset Gem (1)/addAssetBtn1",
            "new/topRight/res layout/Asset Gem (2)/addAssetBtn2",
            "new/topRight/res layout/Asset Gem (3)/addAssetBtn3",
            "new/topRight/res layout/Asset Gem (4)/addAssetBtn4",
            
            "new/WarningBtn",
            "new/WarningNode/Reinforce",
            "new/WarningNode/EnemyWarn",

            "fuel/closeFuelBtn",
            "new/topLeft/openFuleBtn",
            "fuel/left/confirmFuelBtn",

            "new/topRight/Asset Gem/addGemBtn",
        }

        world_map_info.ui = self

        self.helper:regClickEvents("click", self, clickEvents)

        for i = 0, 9 do 
            local path = "inputNum/right/layout/Button (" .. i .. ")"
            self:setScript("click", path, functor.rmake(self.click_number, i))
        end

        -- 当前选择的矿(包括 野矿 和 占有矿)
        self.cur_select_mine = nil
        -- 当前选择怪
        self.cur_select_mst = nil
        -- 当前选择行军
        self.cur_select_march = nil
        -- 当前选择玩家
        self.cur_select_player = nil

        self:refreshResourceNum()
        self.prop_change_event_id = local_player.events:on("onPropChange", functor.make(self.refreshResourceNum, self))

        self.warning_evt = game_director.events:on("WARNNOTIFY", functor.make(self.onWarnNotify, self))
        self.warn_changed = game_director.events:on("WARNSTATECHANGED", functor.make(self.onWarnNotify, self))
        self.FUEL_RECOVER_TICK = local_lord.events:on("FUEL_RECOVER_TICK", functor.make(self.onFelRecoverTick, self))
        self.LORD_INFO_CHANGE = local_lord.events:on("LORD_INFO_CHANGE", functor.make(self.onLordInfoChange, self))
        self:onLordInfoChange()
                
        self:levelOver()

        self:init()
        
        self:onWarnNotify()
        
        -- 如果是从战报播放回来，打开战报邮件
        if game_info.replay_battle_mail_id then
            frame_manager:show("mail", nil, {mails = local_player.mail_list, mail_id = game_info.replay_battle_mail_id})
            game_info.replay_battle_mail_id = nil
        end

        -- 关闭 loading
        frame_manager:hide('loading')
        Fenix.Assets.Helper.ms_SyncLoad = false

        self:click_closeFuelBtn()
        self:refreshGridState()

        self.arrowRect = self.arrowRect or self.transform:Find("new/range/arrow"):GetComponent(UnityEngine.RectTransform)
        self.arrowRect.localScale = Vector3.zero;
    end,

    onUnload = function(self)
        local_player.events:off("onPropChange", self.prop_change_event_id)
        local_chat.events:off("default_channel_change", self.default_channel_change_id)
        local_chat.events:off("chat_msg_change", self.chat_msg_change_id)
        game_director.events:off("WARNNOTIFY", self.warning_evt)
        game_director.events:off("WARNSTATECHANGED", self.warn_changed)
        local_lord.events:off("FUEL_RECOVER_TICK", self.FUEL_RECOVER_TICK)
        local_lord.events:off("LORD_INFO_CHANGE", self.LORD_INFO_CHANGE)
        world_map_info.ui = nil
    end,

    update = function(self, dt)
        --[[ if self.march_list then
            self.march_list:update(dt)
        end ]]

        self.interval_update = self.interval_update or 0.0
        self.interval_update = self.interval_update + dt
        if self.interval_update > 0.03 then
            self:onUpdate(interval)
            self:onUpdateRes(interval)
            self.interval_update = 0
        end

        if self.curr_enemy_warn and self.enemy_warn_text then
            local detal = os.time() - self.curr_enemy_warn.receive_time

            if self.curr_enemy_warn.leftTime - detal > 0 then
                --self.curr_enemy_warn.leftTime = self.curr_enemy_warn.leftTime - dt
                self.enemy_warn_text.text = gui_helper:timeFormat(self.curr_enemy_warn.leftTime - detal)
            else
                self:onWarnNotify()
            end
        end

        if self.curr_help_warn and self.reinforce_text then
            local detal = os.time() - self.curr_help_warn.receive_time

            if self.curr_help_warn.leftTime - detal > 0 then
                --self.curr_help_warn.leftTime = self.curr_help_warn.leftTime - dt
                self.reinforce_text.text = gui_helper:timeFormat(self.curr_help_warn.leftTime - detal)
            else
                self:onWarnNotify()
            end
        end
        
        if Input.GetMouseButtonDown(0) then
            local point = Input.mousePosition
            local click_on_ui = RTSHelper.IsPointerOverUI(point.x, point.y)
            if not click_on_ui then
                if self.task_vsb then
                    self.task_vsb = not self.task_vsb
                    local task_trans = self.transform:Find("new/buttomLeft/bg/layout/dsfdsf/GameObject (4)")
                    frame_helper:setActive(task_trans, self.task_vsb)
                end
            end
        end
    end,

    isBaseScreen = function(self)
        self.screen_rect = self.screen_rect or self.transform:Find("new"):GetComponent(UnityEngine.RectTransform)
        local buttom_left = Vector2(self.range_rect.rect.xMin, self.range_rect.rect.yMin)
        local buttom_right = Vector2(self.range_rect.rect.xMax, self.range_rect.rect.yMin) 
        local top_right = Vector2(self.range_rect.rect.xMax, self.range_rect.rect.yMax)
        local top_left = Vector2(self.range_rect.rect.xMin, self.range_rect.rect.yMax)

        local home_pos = self.owner.world_map:getMyholdPosition()
        local c1 = self:crossPoint(buttom_left, buttom_right, Vector2.zero, home_pos)
        local c2 = self:crossPoint(buttom_right, top_right, Vector2.zero,   home_pos)
        local c3 = self:crossPoint(top_right, top_left, Vector2.zero,       home_pos)
        local c4 = self:crossPoint(top_left, buttom_left, Vector2.zero,     home_pos)
        local ret = c1 or c2 or c3 or c4
        return ret and true or false
    end,

    onUpdate = function(self, dt)
        if not self.is_loaded or not self.transform or not self.owner.world_map:getMyholdPosition() or not self.owner.world_map.my_hold then
            return
        end

        self.range_rect = self.range_rect or self.transform:Find("new/range"):GetComponent(UnityEngine.RectTransform)
        self.arrowRect = self.arrowRect or self.transform:Find("new/range/arrow"):GetComponent(UnityEngine.RectTransform)
        self.offsetTrans = self.offsetTrans or self.transform:Find("new/range/arrow/offset")
        self.distance = self.distance or self:findText("new/range/arrow/offset/lookAtHomeBtn/Text")
        
        local buttom_left = Vector2(self.range_rect.rect.xMin, self.range_rect.rect.yMin)
        local buttom_right = Vector2(self.range_rect.rect.xMax, self.range_rect.rect.yMin) 
        local top_right = Vector2(self.range_rect.rect.xMax, self.range_rect.rect.yMax)
        local top_left = Vector2(self.range_rect.rect.xMin, self.range_rect.rect.yMax)

        local home_pos = self.owner.world_map:getMyholdPosition()
        self.center_go = self.center_go or UnityEngine.GameObject.Find("HexMap/Hex Map Camera/")

        self.screen_center = Vector2.zero
        self.main_camera = self.main_camera or Camera.main
        local temp = self.main_camera:WorldToScreenPoint(home_pos)

        local in_screen = temp.x >= 0 and temp.x <= UnityEngine.Screen.width and temp.y >= 0 and temp.y <= UnityEngine.Screen.height
        if not in_screen then
            self.home_pos = Vector2(temp.x - UnityEngine.Screen.width / 2, temp.y - UnityEngine.Screen.height / 2)
            local c1 = self:crossPoint(buttom_left, buttom_right, self.screen_center, self.home_pos)
            local c2 = self:crossPoint(buttom_right, top_right, self.screen_center, self.home_pos)
            local c3 = self:crossPoint(top_right, top_left, self.screen_center, self.home_pos)
            local c4 = self:crossPoint(top_left, buttom_left, self.screen_center, self.home_pos)
            local camera_pos = self.center_go.transform.position
            -- local d = math.ceil(Vector3.Distance(Vector3(camera_pos.x, 0, camera_pos.z), Vector3(home_pos.x, 0, home_pos.z)))
            local x, z = self.owner.world_map.cs_inst:GetXZ(camera_pos)
            local d = world_map_helper:getDistToBase({x=x, z=z})
            local ret = c1 or c2 or c3 or c4
            if ret then
                self.arrowRect.localScale = Vector3.one;
                self.arrowRect.anchoredPosition = Vector2(ret.x, ret.y)
                local dir = Vector3(ret.x, ret.y, 0)
                local q = Quaternion.identity
                q:SetFromToRotation(Vector3.up, dir)
                self.offsetTrans.rotation = q
                self.distance.transform.rotation = Quaternion.identity
                self.distance.text = d
            else
                self.arrowRect.localScale = Vector3.zero;
            end
        else
            self.arrowRect.localScale = Vector3.zero;
        end
    end,

    zero = function(self, x)
        local rst = x > 0 and x or (-x)
        return rst < 0.0000001
    end,

    parallel = function(self, u1, u2, v1, v2)
        return self:zero((u1.x-u2.x)*(v1.y-v2.y)-(v1.x-v2.x)*(u1.y-u2.y));
    end,

    intersect_in = function(self, u1, u2, v1, v2)
        if (not self:dots_inline(u1,u2,v1) and not self:dots_inline(u1,u2,v2)) then
            return not self:same_side(u1,u2,v1,v2) and not self:same_side(v1,v2,u1,u2);
        end

        return self:dot_online_in(u1,v1,v2) or self:dot_online_in(u2,v1,v2) or self:dot_online_in(v1,u1,u2) or self:dot_online_in(v2,u1,u2);
    end,

    dots_inline = function(self, p1, p2, p3)
        return self:zero(self:xmult(p1,p2,p3));
    end,

    xmult = function(self, p1, p2, p0)
        return (p1.x-p0.x)*(p2.y-p0.y)-(p2.x-p0.x)*(p1.y-p0.y);
    end,

    same_side = function(self, p1, p2, l1, l2)
        return self:xmult(l1,p1,l2)*self:xmult(l1,p2,l2)>0.0000001;
    end,

    intersection = function(self, u1, u2, v1, v2)
        local ret = {}
        ret.x = u1.x
        ret.y = u1.y

        local t=((u1.x-v1.x)*(v1.y-v2.y)-(u1.y-v1.y)*(v1.x-v2.x))
          /((u1.x-u2.x)*(v1.y-v2.y)-(u1.y-u2.y)*(v1.x-v2.x));

        ret.x=(u2.x-u1.x)*t + ret.x;
        ret.y=(u2.y-u1.y)*t + ret.y;
        return ret;
    end,

    crossPoint = function(self, u1, u2, v1, v2)
        if self:parallel(u1,u2,v1,v2) or not self:intersect_in(u1,u2,v1,v2) then
            return nil
        else
            local ans=self:intersection(u1,u2,v1,v2);
            return ans
        end
    end,

    -- 显示单位信息
    -- WORLD_OBJ_PLAYER = 1;
    -- WORLD_OBJ_MINE = 2;
    -- WORLD_OBJ_MARCH = 3;
    -- WORLD_OBJ_MONSTER = 4;
    showUnitInfo = function(self, msg, e_type, unit_id)
        e_type = world_map_info:getUnitType(unit_id)
        if e_type == 1 then
            
        elseif e_type == 2 then
            
        -- 点击车队不给服务器发送消息，从地图信息里面读取
        elseif e_type == 3 then
            local unit_msg = world_map_info.march_lines[unit_id]
            self:setMarchInfo(msg.marchLineQuerys[1].marchLineQuery)
        elseif e_type == 4 then
        end
    end,

    click_addAssetBtn1 = function(self)
        frame_manager:show('resource_exchange', nil, { 
            item_id = 32001,
            item_num = 0,
        })
    end,

    click_addAssetBtn2 = function(self)
        frame_manager:show('resource_exchange', nil, { 
            item_id = 30002,
            item_num = 0,
        })
    end,

    click_addAssetBtn3 = function(self)
        frame_manager:show('resource_exchange', nil, { 
            item_id = 32002,
            item_num = 0,
        })
    end,

    click_addAssetBtn4 = function(self)
        frame_manager:show('resource_exchange', nil, { 
            item_id = 32003,
            item_num = 0,
        })
    end,

    click_MiniMap = function(self)
        frame_manager:show("mini_map", self.owner)
    end,

    onUpdateRes = function(self, interval)
        -- if not self.can_be_gather then
        --     return
        -- end

        -- if not self.gather_total_time then
        --     self.gather_total_time = 0
        -- end

        -- self.gather_total_time = self.gather_total_time + interval

        -- if self.gather_total_time >= 1 then
        --     local text = self:findText("holdedMine/Fight Dialog/Background/icon/curAmountText")
        --     self.cur_gather_num = self.cur_gather_num + self.gather_speek
        --     if self.cur_gather_num > self.max_gather_num then
        --         self.cur_gather_num = self.max_gather_num
        --     end
        --     text.text = "已采集 " .. self.cur_gather_num .. "/" .. self.max_gather_num
        --     self.gather_total_time = 0

        --     text = self:findText("holdedMine/Fight Dialog/Background/icon/resAmountText")
        --     text.text = "剩余储量 " .. (self.max_gather_num - self.cur_gather_num)
        -- end
    end,

    click_ButtonBack = function(self)
        game_director:changeState('login')
    end,

    -- 摄像机定位到自己据点
    click_lookAtHomeBtn = function(self)
        --print("回家")
        self.owner:lookAtMyHold()
    end,

    -- 行军信息 加速
    onClickAccelerateMarchLine = function(self)
        self.target_march_line:refresh(
            self.target_march_line.src_pos,
            self.target_march_line.dest_pos,
            self.target_march_line.percent,
            20000
        )
    end,

    -- 行军信息 返回
    onClickMarchLineHoming = function(self)
        --self.target_march_line:backHome()
        if self.target_march_line then
            network_sender:worldMapGoHome(protocol_pb.WORLD_OBJ_MARCH, self.target_march_line.id, self.owner.map_type)
        else
            
        end
    end,

    -- 据点信息 玩家详细
    onClickDetailPlayerInfo = function(self)
    end,

    -- 据点信息 进入据点
    onClickEnterHold = function(self)
    end,

    -- 设置当前坐标
    setCameraCoordant = function(self, x, y)
        if not self.text_coordinate then
            return
        end

        if self.last_x ~= x or self.last_y ~= y then
            self.text_coordinate.text = string.format(NativeText.GetLangStr(100011), x, y)
            self.last_x = x
            self.last_y = y
        end
    end,

    -- march_line 是 MMarchLine
    addMarchLineUI = function(self, march_line)
        local id = march_line.marchLineId
        if not self.march_line_ui[id] then
            local tplt = self.transform:Find("new/topLeft/marchLineUI")
            tplt.gameObject:SetActive(false)
            local go = Object.Instantiate(tplt.gameObject);
            self.march_line_ui[id] = go

            go.transform:SetParent(self.left_form)
            go.transform.localPosition = Vector3.zero
            go.transform.name = id
            go.transform.localScale = Vector3.one
            go:SetActive(true)

            local leftTime = (march_line.totalTime - march_line.elapsedTime) * 0.000001

            self:setScript("click", "new/m_line/offset/marchs/" .. go.transform.name, functor.rmake(self.click_march_line, id))

            local des_text = go.transform:Find('desText').gameObject:GetComponent(UnityEngine.UI.Text)
            des_text.text = ""

            if march_line.goHome then
                des_text.text = self.helper:getLangCfg(config.worldmap_marchs[1].backtobaselang)
            else
                if march_line.objType == common_pb.WORLD_OBJ_PLAYER then

                elseif march_line.objType == common_pb.WORLD_OBJ_MINE then
                    local cfg = config.worldmap_resources[march_line.cfgId]
                    des_text.text = self.helper:getLangCfg(cfg.nameid)
                elseif march_line.objType == common_pb.WORLD_OBJ_MARCH then

                elseif march_line.objType == common_pb.WORLD_OBJ_MONSTER then
                    local cfg = config.worldmap_monsters[march_line.cfgId]
                    des_text.text = self.helper:getLangCfg(cfg.monstername)
                end
            end

            local time_text = go.transform:Find('timeText').gameObject:GetComponent(UnityEngine.UI.Text)
            time_text.text = self.helper:timeFormat(leftTime) 
            
            local clk_trans = go.transform:Find('timeText')
            local clk = clk_trans:GetComponent(Clock)
            if not clk then
                clk = clk_trans:AddComponent(Clock)
            end
            clk:Set(math.floor(leftTime))
        else
            local go = self.march_line_ui[id]
            local leftTime = (march_line.totalTime - march_line.elapsedTime) * 0.000001
            local clk_trans = go.transform:Find('timeText')
            local clk = clk_trans:GetComponent(Clock)
            if not clk then
                clk = clk_trans:AddComponent(Clock)
            end
            clk:Set(math.floor(leftTime))
        end

        self:tweenMarchLine()
    end,

    click_march_line = function (self, handler, id) 
        print("click_march_line:", id)
        self.owner:lookAtMarchLine(id)
        network_sender:queryWorldMap(id, self.owner.map_type)
        -- local unit_msg = world_map_info.march_lines[id]
        -- self:setMarchInfo(unit_msg)
    end,

    removeMarchLineUI = function(self, id)
        local go = self.march_line_ui[id]
        if go then
            self.march_line_ui[id] = nil
            go.transform:SetParent(nil)
            Object.DestroyImmediate(go)
        end

        self:tweenMarchLine()
    end,

    tweenMarchLine = function(self)
        if self.march_lines_visible == nil then
            self.march_lines_visible = false
        end
        local visible = false
        if self.march_line_ui then
            for __, value in pairs(self.march_line_ui) do
                if value then
                    visible = true
                    break
                end
            end
        end
        if self.march_lines_visible == visible then
            return
        else
            self.march_lines_visible = visible
            local offset = self.transform:Find('new/m_line/offset').gameObject
            LeanTween.cancel(offset)
            local lt = LeanTween.moveLocalX(offset, visible and 0 or -360, 0.25)
        end
    end,

    refreshResourceNum = function(self)
        self.helper:makeNumEffect(self.diamond_text.transform,  local_player.props[protocol_pb.PROP_DIAMOND])
        local fuel = self:findText("new/topLeft/Image/Text")
        fuel.text = local_player:getFuel() .. "/" .. local_player:getFuelMax() 
    end,

    -- ############### 换新界面 start

    -- 基地
    click_homeBtn = function(self)
        print("基地")
        game_director:changeState('loading', {scene_name = 'home', next_state = 'home'})
    end,

    click_taskBtn = function(self)
        frame_manager:show('growth_task')
        self:click_unionBtn()
    end,

    click_dailyBtn = function(self)
        frame_manager:show('daily_task', self)
        self:click_unionBtn()
    end,

    click_achieveBtn = function(self)
        frame_manager:show('chapter_achievement', self)
        self:click_unionBtn()
    end,
    
    -- 工会
    click_unionBtn = function(self)
        local task_trans = self.transform:Find("new/buttomLeft/bg/layout/dsfdsf/GameObject (4)")
        if not self.task_vsb then
            self.task_vsb = true
        else
            self.task_vsb = false
        end
        frame_helper:setActive(task_trans, self.task_vsb)
    end,

    click_setBtn = function(self)
        app:callJava("sdkSwitchAccount")
    end,

    -- 仓库
    click_storeHouseBtn = function(self)
        print("工会")
        if local_guild:inGuild(local_guild:getId()) then
            network_sender:getFamilyInfo(local_player:getGuildId())
        else
            network_sender:getFamilyList(0)
        end
    end,

    -- 车库
    click_carBtn = function(self)
        print("车库")
        frame_manager:show('hero_list', self)
    end,

    -- 进入关卡
    click_enterLevelBtn = function(self)
        frame_manager:show('chapter')
    end,

    -- 邮件
    click_emailBtn = function(self)
        print("邮件")
        network_sender:getMailListInfo()
    end,

    -- 地图
    click_mapBtn = function(self)
        print("地图")
        -- network_sender:getShopList()
    end,

    -- 战报
    click_recordBtn = function(self)
        print("战报")
    end,

    -- 罗宾
    click_robinBtn = function(self)
        -- hint:setText(NativeText.GetLangStr(900002))
        frame_manager:show('bag', self)
    end,

    showEditPos = function(self, collect_pos)
        frame_manager:show('collect_coord', self, {
            pos = collect_pos,
            is_edit = self.coord_edit_mode == "modify",
        })
        self.coord_edit_mode = nil
    end,

    click_remarkCoord = function(self)
        print("备注 收集")
        local input = self.transform:Find("editCoord/content/InputField"):GetComponent(UnityEngine.UI.InputField)
        -- input.isFocused = true
    end,

    click_collectCoord = function(self)
        frame_manager:show('coord_manager', self.owner)
    end,

    -- 定位功能 start
    click_moveCoord = function(self)
        -- self.cur_input_coord = { x = tostring(self.last_x), y = tostring(self.last_y), cur = 1 }

        self:findInputField("inputNum/left/InputFieldX").text = self.last_x
        self:findInputField("inputNum/left/InputFieldY").text = self.last_y

        self.helper.showTrans(self.input_num_trans)
    end,

    click_number = function (self, handler, index)
        local max_X, max_Z = self.owner:getWorldMapBorder()
        print(max_X, max_Z)

        if self.cur_input_coord.cur == 1 then
            local tmp = self.cur_input_coord.x .. tostring(index)
            if tonumber(tmp) <= max_X then
                self.cur_input_coord.x = tmp
            else
                hint:setText(string.format(NativeText.GetLangStr(9000015), max_X))
            end
        else
            local tmp = self.cur_input_coord.y .. tostring(index)
            if tonumber(tmp) <= max_Z then
                self.cur_input_coord.y = tmp
            else
                hint:setText(string.format(NativeText.GetLangStr(9000015), max_Z))
            end
        end

        local x_text = self:findText("inputNum/left/coordX/Text")
        local y_text = self:findText("inputNum/left/coordY/Text")
        x_text.text = tonumber(self.cur_input_coord.x)
        y_text.text = tonumber(self.cur_input_coord.y)
    end, 

    click_deleteNum = function(self)
        -- print("删除按钮")
        -- if self.cur_input_coord.cur == 1 then
        --     self.cur_input_coord.x = string.sub(self.cur_input_coord.x, 1, -2)
        --     if self.cur_input_coord.x == "" then
        --         self.cur_input_coord.x = "0"
        --     end
        -- else
        --     self.cur_input_coord.y = string.sub(self.cur_input_coord.y, 1, -2)
        --     if self.cur_input_coord.y == "" then
        --         self.cur_input_coord.y = "0"
        --     end
        -- end
        -- self:findText("inputNum/left/coordX/Text").text = self.cur_input_coord.x
        -- self:findText("inputNum/left/coordY/Text").text = self.cur_input_coord.y
    end,

    click_coordX = function(self)
        self.cur_input_coord.cur = 1
    end,

    click_coordY = function(self)
        self.cur_input_coord.cur = 2
    end,

    click_positioningBtn = function(self)
        local ix = self:findInputField("inputNum/left/InputFieldX").text
        local iy = self:findInputField("inputNum/left/InputFieldY").text

        if string.len(ix) <= 0 or string.len(iy) <= 0 then
            hint:setText(NativeText.GetLangStr(100049))
            return
        end

        self.owner:setCameraPos(tonumber(ix), tonumber(iy))
        self.helper.hideTrans(self.input_num_trans)
    end,

    click_closeInputNum = function(self)
        self.helper.hideTrans(self.input_num_trans)
    end,

    click_useRetreatItem = function (self, handler, item_id)
        print("使用召回道具Id:" .. item_id)
    end,

    init = function(self)
        self:setChatInfo()
        self.default_channel_change_id = local_chat.events:on('default_channel_change', function()
            self:setChatInfo()
        end)
        self.chat_msg_change_id = local_chat.events:on('chat_msg_change', function()
            self:setChatInfo()
        end)
    end,


    levelOver = function(self)
        if level_info.level_over then
            level_info.level_over = false
        end
    end,

    click_showBtn = function(self)
        self:setExtendState(true)
    end,

    click_hideBtn = function(self)
        self:setExtendState(false)
    end,

    setExtendState = function(self, extend)
        local duration = 0.25

        local buttom = self.transform:Find('new/buttomLeft/bg').gameObject
        LeanTween.cancel(buttom)
        local lt = LeanTween.moveLocalY(buttom, not extend and 0 or -180, duration)

        local fun_layout = self.transform:Find('new/fun layout'):GetComponent(UnityEngine.RectTransform)
        LeanTween.cancel(fun_layout)
        local lt = LeanTween.moveX(fun_layout, not extend and 0 or 130, duration)

        local m_line = self.transform:Find('new/m_line'):GetComponent(UnityEngine.RectTransform)
        LeanTween.cancel(m_line)
        local lt = LeanTween.moveX(m_line, not extend and 0 or -360, duration)

        local top_left = self.transform:Find('new/topLeft'):GetComponent(UnityEngine.RectTransform)
        LeanTween.cancel(top_left)
        local lt = LeanTween.moveY(top_left, not extend and 0 or 230 , duration)

        local top_right = self.transform:Find('new/topRight'):GetComponent(UnityEngine.RectTransform)
        LeanTween.cancel(top_right)
        local lt = LeanTween.moveY(top_right, not extend and 0 or 230 , duration)

        local bg = self.transform:Find('new/bg (1)'):GetComponent(UnityEngine.RectTransform)
        LeanTween.cancel(bg)
        local lt = LeanTween.moveY(bg, not extend and 0 or 125 , duration)

        local bg = self.transform:Find('new/bg (2)'):GetComponent(UnityEngine.RectTransform)
        LeanTween.cancel(bg)
        local lt = LeanTween.moveY(bg, not extend and 0 or -125 , duration)

        local show_trans = self.transform:Find('new/buttomRight/showBtn')
        local hide_trans = self.transform:Find('new/buttomRight/hideBtn')

        show_trans.gameObject:SetActive(not extend)
        hide_trans.gameObject:SetActive(extend)
    end,

    click_chatBtn = function(self)
        frame_manager:show('chat')
    end,

    click_chatChannelBtn = function(self)
        frame_manager:show('chat_channel')
    end,

    click_SearchBtn = function(self)
        frame_manager:show("world_map_search")
    end,

    click_GridBtn = function(self)
        local is_show = self.owner.world_map.cs_inst:ToggleGridShow()
        if is_show then
            self.owner.world_map:addMineCircle()
        else
            self.owner.world_map:removeMineCircle()
        end
        self:refreshGridState()
    end,

    refreshGridState = function(self)
        local is_show = self.owner.world_map.cs_inst:IsGridShow()
        local on = self:findImage("new/buttomRight/GridBtn/on")
        local off = self:findImage("new/buttomRight/GridBtn/off")
        on.enabled = not is_show
        off.enabled = is_show
    end,

    setChatInfo = function(self)
        local text = self:findText('new/buttomLeft/bg/chatChannelBtn/Text')
        text.text = local_chat:getDefaultChannel().acronym
        local text = self:findText('new/buttomLeft/bg/chatBtn/Text')
        local des = local_chat:getLatestContent(local_chat:getDefaultChannel())
        text.text = des and des or ""
    end,

    --[[ addMarchList = function(self, info, type)
        if self.march_list then
            self.march_list:addElem(info, type)
        end
    end,

    removeMarchList = function(self, id, type)
        if self.march_list then
            self.march_list:removeQueueInfo(type, id)
        end
    end, ]]

    click_holdListBtn = function(self)
        frame_manager:show("own_hold_list")
    end,

    click_WarningBtn = function(self)
        frame_manager:show("warn_info")
    end,

    click_Reinforce = function(self)
        frame_manager:show("warn_info")
    end,

    click_EnemyWarn = function(self)
        frame_manager:show("warn_info")
    end,

    onWarnNotify = function(self)
        self.warn_count = 0
        self.curr_enemy_warn = nil
        self.curr_help_warn = nil

        for k, v in pairs(game_info.warning_info) do
            if not v.is_off then
                local dt = os.time() - v.receive_time
    
                if v.type == common_pb.EMARCH_HELP then
                    if self.curr_help_warn then
                        if self.curr_help_warn.leftTime > v.leftTime - dt then
                            self.curr_help_warn = v
                        end
                    else
                        self.curr_help_warn = v
                    end
                else
                    if self.curr_enemy_warn then
                        if self.curr_enemy_warn.leftTime > v.leftTime - dt then
                            self.curr_enemy_warn = v
                        end
                    else
                        self.curr_enemy_warn = v
                    end
                end
            end

            self.warn_count = self.warn_count + 1
        end

        self.enemy_warn_panel:SetActive(self.curr_enemy_warn ~= nil)
        self.reinforce_panel:SetActive(self.curr_help_warn ~= nil)
        self.warn_btn:SetActive(self.warn_count > 0)
        self.warn_cnt_text.text = tostring(self.warn_count)

        if self.curr_enemy_warn then
            frame_manager:show('rim_red')
        else
            local is_display = frame_manager:isDisplay('rim_red')
            
            if is_display then
                frame_manager:hide('rim_red')
            end
        end
    end,

    click_confirmFuelBtn = function(self)
        self.fuel_trans = self.fuel_trans or self.transform:Find("fuel")
        frame_helper.hideTrans(self.fuel_trans)
    end,

    click_closeFuelBtn = function(self)
        self.fuel_trans = self.fuel_trans or self.transform:Find("fuel")
        frame_helper.hideTrans(self.fuel_trans)
    end,

    click_openFuleBtn = function(self)
        if local_player:getFuel() >= local_player:getFuelMax() then
            hint:setText(NativeText.GetLangStr(100051))
            return
        end

        self.fuel_trans = self.fuel_trans or self.transform:Find("fuel")
        frame_helper.showTrans(self.fuel_trans)
    end,

    onFelRecoverTick = function(self, elapsed)
        self.fuel_time = self.fuel_time or self:findText("fuel/left/time")
        self.fuel_time.text = frame_helper:timeFormat(elapsed)
    end,

    onLordInfoChange = function(self)
        local energy = self:findText("new/topLeft/apbg/Text")
        energy.text = local_lord.energy .. "/" .. local_lord.max_energy
    end,

    click_addGemBtn = function(self)
        hint:setText(NativeText.GetLangStr(999034))
    end,
}