local log = logger.get("Game", "World Map Search Frame")
local super = require("ui.frame")
local item_cls = require('RTS.bag_sys.item')

local res_lv_key = "SEARCHRESLEVEL"
local monster_lv_key = "SEARCHMONSTERLEVEL"

return class{
    super = super, 

    onLoaded = function (self)
        super.onLoaded(self)

        self.is_filled = false

        clickEvents = 
        {
            "Close",
            "BG/Bottom/Search",
            "BG/Resource/Right/Plus",
            "BG/Resource/Right/Minus",
            "BG/Monster/Right/Plus",
            "BG/Monster/Right/Minus",
            "BG/Pages/Page1",
            "BG/Pages/Page2",
            "BG/Pages/Page4",
        }

        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        --self.content = self.transform:Find("BG/Monster/Left/Scroll View/Viewport/Content")
        --self.prefab = self.transform:Find("BG/Monster/Left/Scroll View/Viewport/Content/Enemy").gameObject
        self.monster_panel = self.transform:Find("BG/Monster")
        self.res_panel = self.transform:Find("BG/Resource")
        self.res_slider = self.res_panel:Find("Right/Slider"):GetComponent(UnityEngine.UI.Slider)
        self.monster_slider = self.monster_panel:Find("Right/Slider"):GetComponent(UnityEngine.UI.Slider)
        self.search_btn = self.transform:Find("BG/Bottom/Search"):GetComponent(UnityEngine.UI.Button)

        self.search_btn.interactable = false

        self.elems = {}
        self.elems[1] = {ui = self.res_panel:Find("Left/Scroll View/Viewport/Content/Res")}
        self.elems[2] = {ui = self.res_panel:Find("Left/Scroll View/Viewport/Content/Res (1)")}
        self.elems[3] = {ui = self.res_panel:Find("Left/Scroll View/Viewport/Content/Res (2)")}
        self.elems[4] = {ui = self.res_panel:Find("Left/Scroll View/Viewport/Content/Res (3)")}
        self.elems[5] = {ui = self.res_panel:Find("Left/Scroll View/Viewport/Content/Res (4)")}

        for k, v in ipairs(self.elems) do
            self:setResIcon(k, v.ui)
        end

        self.page_btns = {}
        self.page_btns[common_pb.ESEARCH_MONSTER] = self.transform:Find("BG/Pages/Page1"):GetComponent(UnityEngine.UI.Button)
        --self.page_btns[common_pb.ESEARCH_BOSS] = self.transform:Find("BG/Pages/Page2"):GetComponent(UnityEngine.UI.Button)
        self.page_btns[common_pb.ESEARCH_RES] = self.transform:Find("BG/Pages/Page2"):GetComponent(UnityEngine.UI.Button)
        self.page_btns[4] = self.transform:Find("BG/Pages/Page4"):GetComponent(UnityEngine.UI.Button)

        self.page_btns[4].transform:SetActive(false)
        
        self.page_idx = common_pb.ESEARCH_MONSTER

        self.monster_ui = {}
        self.res_lv = PlayerPrefs.GetInt(res_lv_key, 1)

        self.res_slider.onValueChanged:RemoveAllListeners()
        self.res_slider.onValueChanged:AddListener(
            function ()
                self:onValueChanged()
            end
        )

        self.monster_slider.onValueChanged:RemoveAllListeners()
        self.monster_slider.onValueChanged:AddListener(
            function ()
                self:onValueChanged()
            end
        )

        self.evts = {}
        self.evts["WORLDMAPSEARCH"] = game_director.events:on("WORLDMAPSEARCH", functor.make(self.onSearched, self))

        network_sender:worldMapSearch(self.page_idx)
    end,

    onUnload = function(self)
        if self.evts then
        
            for k, v in pairs(self.evts) do
                game_director.events:remove(k, v)
            end
        end

    end,

    onSearched = function(self, idx)
        self.page_idx = idx

        self.monster_lv = self:getMonsterLv(search_info.monster_idxs[1])

        if idx == common_pb.ESEARCH_MONSTER then
            self.monster_idx = 0
            self:refreshMonster(search_info.monster_idxs[1])
            self:onClickMonster(search_info.monster_idxs[1])
            self:setSliderValue()
            self.res_idx = 0
        elseif idx == common_pb.ESEARCH_RES then
            self:refreshRes()
            self:onClickResHold(nil, 1)
            self:setSliderValue()
            self.monster_idx = 0
        end

        self:updatePages()
        self.search_btn.interactable = true
    end,

    onPageChanged = function(self, idx)
        self.page_idx = idx
    end,

    onValueChanged = function(self)
        if self.changed then
            self.changed = false
            return
        end
        if self.page_idx == common_pb.ESEARCH_RES then
            if self.select_res then
                local max_lv = #self.select_res - 1
                local level = math.floor(self.res_slider.value * max_lv + 0.5) + 1

                self:setResLv(level)
            end
        elseif self.page_idx == common_pb.ESEARCH_MONSTER then
            if self.select_monster then
                local max_lv = #self.select_monster - 1
                local level = math.floor(self.monster_slider.value * max_lv + 0.5) + 1

                self:setMonsterLv(level)
            end
        end
    end,

    setResLv = function(self, value)
        if self.res_lv ~= value then
            self.res_lv = value
            self:setResHoldInfo(self.res_idx)
            PlayerPrefs.SetInt(res_lv_key, self.res_lv)
        end
    end,

    setMonsterLv = function(self, value)
        if self.monster_lv ~= value then
            self.monster_lv = value
            self:setMonsterInfo(self.monster_idx)

            local cfg = self:getMonsterCfgByIdx(self.monster_idx)
            if cfg then
                local key = monster_lv_key.."_"..cfg.monster_type
                PlayerPrefs.SetInt(key, self.monster_lv)
            end
        end
    end,

    setSliderValue = function(self)
        if self.page_idx == common_pb.ESEARCH_RES then
            local max = #self.select_res - 1
            self.res_slider.value = (self.res_lv - 1) / (max > 0 and max or 1)
        elseif self.page_idx == common_pb.ESEARCH_MONSTER then
            local max = #self.select_monster - 1
            self.monster_slider.value = (self.monster_lv - 1) / (max > 0 and max or 1)
        end
    end,

    updatePages = function(self)
        for k, v in pairs(self.page_btns) do
            if k == self.page_idx then
                v.targetGraphic.sprite = v.spriteState.pressedSprite
            else
                v.targetGraphic.sprite = v.spriteState.disabledSprite
            end
        end

        self.is_filled = true
    end,

    setResIcon = function(self, idx, node)
        local icon = node:Find("Icon"):GetComponent(UnityEngine.UI.Image)
        local res_icon = node:Find("Title/Image/Image"):GetComponent(UnityEngine.UI.Image)
        local name = node:Find("Title/Name"):GetComponent(UnityEngine.UI.Text)
        local title_img = node:Find("Title"):GetComponent(UnityEngine.UI.Image)
        local light = node:Find("Light")

        light:SetActive(false)
        title_img:SetSprite(res_path.res_path_74)

        local cfg = nil

        if idx == 1 then
            cfg = config.worldmap_titaniums[201]
            name.text = NativeText.GetLangStr(9000016)
        elseif idx == 2 then
            cfg = config.worldmap_titaniums[301]
            name.text = NativeText.GetLangStr(9000017)
        elseif idx == 3 then
            cfg = config.worldmap_titaniums[401]
            name.text = NativeText.GetLangStr(9000018)
        elseif idx == 5 then
            cfg = config.worldmap_titaniums[101]
            name.text = NativeText.GetLangStr(9000019)
        elseif idx == 4 then
            cfg = config.worldmap_titaniums[501]
            name.text = NativeText.GetLangStr(9000020)
        end

        if cfg then
            icon:SetSprite(cfg.iconpath..cfg.iconid)
            local item_cfg = item_helper:getItemCfg(cfg.item)
            if item_cfg then
                res_icon:SetSprite(gui_helper:getItemIconPath(item_cfg))
            end
            self.elems[idx].cfg = cfg
        end

        local path = "BG/Resource/Left/Scroll View/Viewport/Content/"..node.name
        self:clearScript("click", path)
        self:setScript("click", path, functor.rmake(self.onClickResHold, idx))
    end,

    refreshMonster = function(self, idx)
        self.monster_panel:SetActive(true)
        self.res_panel:SetActive(false)
        self.monster_ui = {}
        
        self:loadMonsters()
        self:setMonsterInfo(idx)
    end,

    setMonsterInfo = function(self, idx)
        if not self.monster_ui[idx] then
            return
        end

        local level_text = self.monster_ui[idx].ui:Find("Title/Image/Level"):GetComponent(UnityEngine.UI.Text)
        level_text.text = self.monster_lv

        local info_node = self.monster_panel:Find("Right/Info/Info")
        local name = self.monster_panel:Find("Right/Info/Name"):GetComponent(UnityEngine.UI.Text)
        local level = self.monster_panel:Find("Right/Info/Info/Level/Value"):GetComponent(UnityEngine.UI.Text)
        local pos = self.monster_panel:Find("Right/Info/Info/Position/Value"):GetComponent(UnityEngine.UI.Text)
        local range = self.monster_panel:Find("Right/Info/Info/Range/Value"):GetComponent(UnityEngine.UI.Text)
        --local reserve = self.monster_panel:Find("Right/Info/Info/Reserve/Value"):GetComponent(UnityEngine.UI.Text)
        local icon = self.monster_panel:Find("Right/Info/Image/Icon"):GetComponent(UnityEngine.UI.Image)
        local not_find = self.monster_panel:Find("Right/Info/NotFind")
        local not_find_level = self.monster_panel:Find("Right/Info/NotFind/Level/Value"):GetComponent(UnityEngine.UI.Text)

        self.select_monster = search_info:getMonsters(idx)
        if not self.select_monster then
            return
        end

        local info = search_info:getDisplayMonsters(self.select_monster, self.monster_lv)
        info_node:SetActive(info ~= nil)
        not_find:SetActive(info == nil)

        if info then
            icon:SetSprite(info.cfg.iconpath..info.cfg.monstericon)
            name.text = self.helper:getLangCfg(info.name_id)
            level.text = info.level
            pos.text = gui_helper:positionFormat(info.coordX, info.coordY)
            range.text = info.distance.."km"
            --reserve.text = info.amount

            self.monster_lv_text = level
        else
            icon:SetSprite(self.monster_ui[idx].cfg.iconpath..self.monster_ui[idx].cfg.monstericon)
            name.text = self.helper:getLangCfg(self.monster_ui[idx].cfg.monstername)
            not_find_level.text = self.monster_lv

            self.monster_lv_text = not_find_level
        end

        self.curr_info = info
        search_info:setCurrSearchInfo(self.page_idx, self.curr_info)
    end,

    refreshRes = function(self)
        self.monster_panel:SetActive(false)
        self.res_panel:SetActive(true)

        self:setResHoldInfo(1)
    end,

    setResHoldInfo = function(self, idx)
        if not self.elems[idx] then
            return
        end

        local info_node = self.res_panel:Find("Right/Info/Info")
        local name = self.res_panel:Find("Right/Info/Name"):GetComponent(UnityEngine.UI.Text)
        local level = self.res_panel:Find("Right/Info/Info/Level/Value"):GetComponent(UnityEngine.UI.Text)
        local pos = self.res_panel:Find("Right/Info/Info/Position/Value"):GetComponent(UnityEngine.UI.Text)
        local range = self.res_panel:Find("Right/Info/Info/Range/Value"):GetComponent(UnityEngine.UI.Text)
        local reserve = self.res_panel:Find("Right/Info/Info/Reserve/Value"):GetComponent(UnityEngine.UI.Text)
        local icon = self.res_panel:Find("Right/Info/Image/Icon"):GetComponent(UnityEngine.UI.Image)
        local not_find = self.res_panel:Find("Right/Info/NotFind")
        local not_find_level = self.res_panel:Find("Right/Info/NotFind/Level/Value"):GetComponent(UnityEngine.UI.Text)

        self.select_res = search_info:getRess(idx)
        if not self.select_res then
            return
        end

        local info = search_info:getDisplayRes(self.select_res, self.res_lv)
        info_node:SetActive(info ~= nil)
        not_find:SetActive(info == nil)

        if info then
            icon:SetSprite(info.cfg.iconpath..info.cfg.iconid)
            name.text = self.helper:getLangCfg(info.name_id)
            level.text = info.level
            pos.text = gui_helper:positionFormat(info.coordX, info.coordY)
            range.text = info.distance.."km"
            reserve.text = info.amount

            self.res_lv_text = level
        else
            icon:SetSprite(self.elems[idx].cfg.iconpath..self.elems[idx].cfg.iconid)
            name.text = self.helper:getLangCfg(self.elems[idx].cfg.nameid)
            not_find_level.text = self.res_lv

            self.res_lv_text = not_find_level
        end

        self.curr_info = info
        search_info:setCurrSearchInfo(self.page_idx, self.curr_info)
    end,

    enabledLight = function(self, list, idx, enable)
        if list[idx] then
            local ui = list[idx].ui
            local light = ui:Find("Light")
            light:SetActive(enable)

            local title = ui:Find("Title")
            if title then
                local img = title:GetComponent(UnityEngine.UI.Image)

                if enable then
                    img:SetSprite(res_path.res_path_62)
                else
                    img:SetSprite(res_path.res_path_74)
                end
            end

            local setTextColor = function(path)
                local label = ui:Find(path)
                if label then
                    local text = label:GetComponent(UnityEngine.UI.Text)
                    
                    if enable then
                        text.color = Color(64/255, 199/255, 1, 1)
                    else
                        text.color = Color(1, 1, 1, 1)
                    end
                end
            end

            --setTextColor("Title/Name")
        end
    end,

    onClickMonster = function(self, idx)
        if self.monster_idx == idx then
            return
        end

        self.monster_lv = self:getMonsterLv(idx)
        self:enabledLight(self.monster_ui, self.monster_idx, false)
        self:enabledLight(self.monster_ui, idx, true)

        self.monster_idx = idx
        self:setMonsterInfo(idx)
        self:setSliderValue()
    end,

    onClickResHold = function(self, handler, idx)
        if self.res_idx == idx then
            return
        end

        self:enabledLight(self.elems, self.res_idx, false)
        self:enabledLight(self.elems, idx, true)

        self.res_idx = idx
        self:setResHoldInfo(idx)
    end,

    click_Close = function(self)
        search_info:setCurrSearchInfo(self.page_idx, nil)
        frame_manager:hide("world_map_search")
    end,

    click_Page1 = function(self)
        network_sender:worldMapSearch(common_pb.ESEARCH_MONSTER)
    end,

    click_Page2 = function(self)
        network_sender:worldMapSearch(common_pb.ESEARCH_RES)
    end,

    click_Page4 = function(self)

    end,

    click_Plus = function(self)
        if self.page_idx == common_pb.ESEARCH_RES then
            local max = #self.select_res
            if self.res_lv >= max then
                return
            end

            self:setResLv(self.res_lv + 1)
        elseif self.page_idx == common_pb.ESEARCH_MONSTER then
            local max = #self.select_monster
            if self.monster_lv >= max then
                return
            end

            self:setMonsterLv(self.monster_lv + 1)
        end

        self.changed = true

        self:setSliderValue()
    end,

    click_Minus = function(self)
        if self.page_idx == common_pb.ESEARCH_RES then
            if self.res_lv <= 1 then
                return
            end

            self:setResLv(self.res_lv - 1)
        elseif self.page_idx == common_pb.ESEARCH_MONSTER then
            if self.monster_lv <= 1 then
                return
            end

            self:setMonsterLv(self.monster_lv - 1)
        end

        self.changed = true

        self:setSliderValue()
    end,

    click_Search = function(self)
        if self.curr_info then
            frame_manager:hide("world_map_search")
            local cb = function()
                network_sender:queryWorldMap(self.curr_info.id, protocol_pb.MAP_WORLDMAP)
            end
            world_map_helper:lookAtPosition(self.curr_info.coordX, self.curr_info.coordY, cb)
        else
            hint:setText(NativeText.GetLangStr(9000047))
        end
    end,

    loadMonsters = function(self)
        local content = self.transform:Find("BG/Monster/Left/Scroll View/Viewport/Content")
        local prefab = content:GetChild(0).gameObject

        for k, v in ipairs(search_info.monster_idxs) do
            local elem = nil

            if k > content.childCount then
                elem = Object.Instantiate(prefab).transform
                elem:SetParent(content)
                resetTransformZ(elem)
            else
                elem = content:GetChild(k-1)
            end

            elem:SetActive(true)

            local icon = elem:Find("Icon"):GetComponent(UnityEngine.UI.Image)
            local level = elem:Find("Title/Image/Level"):GetComponent(UnityEngine.UI.Text)
            local type_icon = elem:Find("Title/Image"):GetComponent(UnityEngine.UI.Image)
            local name = elem:Find("Title/Name"):GetComponent(UnityEngine.UI.Text)

            local monster_cfg = nil

            config.worldmap_monsters:foreach(
                function(id, cfg)
                    if cfg.monster_index == v then
                        monster_cfg = cfg
                    end
                end
            )

            if monster_cfg then
                icon:SetSprite(monster_cfg.iconpath..monster_cfg.monstericon)
                name.text = self.helper:getLangCfg(monster_cfg.monstername)
                type_icon:SetSprite(self:monsterTypeIcon(monster_cfg.monster_type))
                level.text = tostring(1)

                local btn = elem:GetComponent(UnityEngine.UI.Button)
                btn.onClick:RemoveAllListeners()
                btn.onClick:AddListener(
                    function()
                        self:onClickMonster(monster_cfg.monster_index)
                    end
                )
            else
                log.error("######################################## find monster cfg error ! monster index", v)
            end

            self.monster_ui[v] = {ui = elem, cfg = monster_cfg}
        end

        for i=#search_info.monster_idxs, content.childCount - 1 do
            local child = content:GetChild(i)
            child:SetActive(false)
        end
    end,

    monsterTypeIcon = function(self, type)
        if type == 2 then
            return res_path.res_path_87
        elseif type == 4 then
            return res_path.res_path_137
        elseif type == 5 then
            return res_path.res_path_138
        elseif type == 7 then
            return res_path.res_path_136
        else
            return res_path.res_path_87
        end
    end,

    getMonsterCfgByIdx = function(self, idx)
        return search_info.idx_cfg[idx]
    end,

    getMonsterLv = function(self, idx)
        local cfg = self:getMonsterCfgByIdx(idx)
        local lv = 1

        if cfg then
            local key = monster_lv_key.."_"..cfg.monster_type
            lv = PlayerPrefs.GetInt(key, 1)
        end

        return lv
    end,
}