local log = logger.get("Game", "World Monster Info Frame")
local super = require("ui.frame")
local item_cls = require('RTS.bag_sys.item')

return class{
    super = super,

    onLoaded = function( self )
        super.onLoaded(self)

        self.owner = self.args.owner
        self.monster = self.args.data.monster

        clickEvents = {
            "Close",
            "BG/Top/Collect",
            "BG/Top/Info",
            "BG/Normal/Launch",
            "BG/Rally/Spy",
            "BG/Rally/StartRally",
            "BG/Energy/Attack",
            "BG/Energy/RepeatATK",
        }

        -- 注册点击事件
        self.helper = require("gui.tool.frame_helper")
        self.helper:regClickEvents("click", self, clickEvents)

        self.normal_panel = self.transform:Find("BG/Normal")
        self.rally_panel = self.transform:Find("BG/Rally")
        self.energy_panel = self.transform:Find("BG/Energy")

        self.normal_panel.localScale = Vector3.zero
        self.rally_panel.localScale = Vector3.zero
        self.energy_panel.localScale = Vector3.zero

        self.prefab = self.transform:Find("Prefab/Item").gameObject
        self.time_text = self.transform:Find("BG/Left/TimeBG/Text"):GetComponent(UnityEngine.UI.Text)

        self.left_time = self.monster.info.disappearTime
        self.cfg = self.monster.cfg

        local name = self.transform:Find("BG/Top/Name"):GetComponent(UnityEngine.UI.Text)
        local pos = self.transform:Find("BG/Top/Position"):GetComponent(UnityEngine.UI.Text)

        name.text = string.format( "%s <color=#ffffffff>Lv.%d</color>", self.helper:getLangCfg(self.cfg.monstername), self.cfg.level )
        pos.text = gui_helper:positionFormat(self.monster.coordX, self.monster.coordY)

        self:init()

        self.prop_change_event_id = local_player.events:on("onGlobalPropChange", functor.make(self.refreshResourceNum, self))
        self.search_evt_id = game_director.events:on("WORLDMAPSEARCH", functor.make(self.onSearched, self))
        self.LORD_INFO_CHANGE = local_lord.events:on('LORD_INFO_CHANGE', 
            function()
                self:updateEnergy()
            end)
    end,

    onUnload = function(self)
        local_player.events:remove("onGlobalPropChange", self.prop_change_event_id)
        game_director.events:remove("WORLDMAPSEARCH", self.search_evt_id)
        local_lord.events:remove("LORD_INFO_CHANGE", self.LORD_INFO_CHANGE)
    end,

    onSearched = function(self)
        local monster = search_info:getMonsterByIdxAndLv(self.search_info.type, self.search_info.lv)

        if monster then
            gui_helper:positionJump(monster.coordX, monster.coordY)
            self:click_Close()
        else
            hint:setText(NativeText.GetLangStr(9000021))
        end
    end,

    updateEnergy = function(self)
        if self.cfg.attack == 4 then
            self:energyMonster()
        end
    end,

    refreshResourceNum = function(self)
        local fill_img = self.energy_panel:Find("FillBG/Fill"):GetComponent(UnityEngine.UI.Image)
        local count = self.energy_panel:Find("FillBG/Count"):GetComponent(UnityEngine.UI.Text)
        local repeat_cost = self.energy_panel:Find("RepeatATK/BG/Count"):GetComponent(UnityEngine.UI.Text)
        local repeat_cnt = self.energy_panel:Find("RepeatATK/Text"):GetComponent(UnityEngine.UI.Text)

        local cur_energy = self.monster.info.cur_energy
        local max_energy = local_player:getEnergyMax()
        local percent = cur_energy / (max_energy > 0 and max_energy or 1)
        self.max_count = math.floor(cur_energy / self.monster.info.energy)

        fill_img.fillAmount = percent
        count.text = string.format( "%d/%d", cur_energy, max_energy )
        repeat_cost.text = self.monster.info.energy * self.max_count
        repeat_cnt.text = NativeText.GetLangStr(280208).." "..self.max_count
    end,

    init = function(self)
        local path = ""

        if self.cfg.attack == 1 then
            self.normal_panel.localScale = Vector3.one

            path = "BG/Normal/Reward/Scroll View"
            
            self:normalMonster()
            self:loadDrop(path)

        elseif self.cfg.attack == 2 or self.cfg.attack == 3 or self.cfg.attack == 5 then
            self.rally_panel.localScale = Vector3.one

            self:rallyMonster()

        elseif self.cfg.attack == 4 then
            self.energy_panel.localScale = Vector3.one

            path = "BG/Energy/Reward/Scroll View"

            self:energyMonster()
            self:loadDrop(path)
        end

        self:initLeftUI()
    end,

    update = function(self, dt)
        if self.time_text then
            if self.left_time > 0 then
                self.left_time = self.left_time - dt
                self:setTime(math.floor(self.left_time))
            else
                --self:click_Close()
            end
        end

        if self.boost_cd then
            local lt = self.monster.info.bufftime
            if lt > 0 then
                lt = lt - dt
                self.monster.info.bufftime = lt > 0 and lt or 0
                self.boost_cd.text = gui_helper:timeFormat(lt)
            else
                self:energyMonster()
            end
        end
    end,

    calculateFight = function(self)
        local fight = 0

        local group_cfg = config.monster_group_ids[self.monster.info.groupid]

        if group_cfg then
            local soldiers = gui_helper:getCorps(group_cfg)

            fight = fight + world_map_helper:calculateTeamPower({}, soldiers)
        end

        return fight
    end,

    monsterUnlock = function(self, cfg)
        local list = config.worldmap_others[1].MonsterLevelUnlock
        list = loadstring("return "..list)()

        local unlock_param = loadstring("return "..cfg.Unlock)()

        for k, v in ipairs(list) do
            if cfg.monster_type == v then
                local cur_lv = world_map_info:getMonsterAtkLv(v)

                if cur_lv then
                    return cur_lv >= unlock_param[2], cur_lv + 1
                else
                    return cfg.level <= 1, 1
                end
            end
        end

        if cfg.monster_type == 7 then
            local need_tech_id = unlock_param[2]
            local cur_tech_lv = home_info:getTechLevel(need_tech_id)

            return cur_tech_lv > 0, need_tech_id
        end

        return true, 0
    end,

    setTime = function(self, lt)
        self.time_text.text = string.format("%s%s", NativeText.GetLangStr(280201), gui_helper:timeFormat(lt))
    end,

    initLeftUI = function(self, msg)
        local icon = self.transform:Find("BG/Left/Icon"):GetComponent(UnityEngine.UI.Image)
        local hp = self.transform:Find("BG/Left/FillBG/Fill"):GetComponent(UnityEngine.UI.Image)
        local percent = self.transform:Find("BG/Left/FillBG/Percent"):GetComponent(UnityEngine.UI.Text)
        local power = self.transform:Find("BG/Left/Power/Value"):GetComponent(UnityEngine.UI.Text)
        local limit = self.transform:Find("BG/Left/Limit"):GetComponent(UnityEngine.UI.Text)
        local tip = self.transform:Find("BG/Left/Tip")

        if self.cfg.attack == 4 then
            tip.localScale = Vector3.one
        else
            tip.localScale = Vector3.zero
        end

        icon:SetSprite(self.cfg.iconpath..self.cfg.monstericon)

        local info = self.monster.info
        hp.fillAmount = info.curHp / info.maxHp
        percent.text = string.format( "%.1f%%", hp.fillAmount*100 )

        if self.cfg.monster_type == 5 or self.cfg.attack == 4 then
            power.text = "???"
        else
            self.monster.sword = self:calculateFight()
            power.text = self.helper:itemNumFormat(self.monster.sword)
        end

        local cnt_info = world_map_info:getAtkCnt(self.cfg.monster_type)
        
        if cnt_info then
            limit.text = string.format( "%s <color=#0BFF78FF>%d/%d</color>", NativeText.GetLangStr(280202), cnt_info.maxCount - cnt_info.curCount, cnt_info.maxCount )
        else
            limit.transform:SetActive(false)
        end
    end,

    normalMonster = function(self)
        
        local group_cfg = config.monster_group_ids[self.monster.info.groupid]

        if group_cfg then
            local soldiers = gui_helper:getCorps(group_cfg)
            local corps_info = self.normal_panel:Find("Soldiers")

            local cnt = 0
            for k, v in pairs(soldiers) do
                if cnt < corps_info.childCount then
                    local elem = corps_info:GetChild(cnt)
                    local icon = elem:Find("Icon"):GetComponent(UnityEngine.UI.Image)
                    local name = elem:Find("Name"):GetComponent(UnityEngine.UI.Text)
                    local count = elem:Find("Count"):GetComponent(UnityEngine.UI.Text)

                    local corps_cfg = config.soldier_soldiers[v.cfg_id]

                    if corps_cfg then
                        icon:SetSprite(corps_cfg.iconpath..corps_cfg.soldiericon)
                        name.text = self.helper:getLangCfg(corps_cfg.soldiername)
                        count.text = v.count
                    end

                    elem.localScale = Vector3.one

                    cnt = cnt + 1
                end
            end

            for i=cnt, corps_info.childCount - 1 do
                local elem = corps_info:GetChild(i)
                elem.localScale = Vector3.zero
            end
        end

        local tip = self.normal_panel:Find("Tip"):GetComponent(UnityEngine.UI.Text)
        local search_btn = self.normal_panel:Find("Search"):GetComponent(UnityEngine.UI.Button)
        local launch_btn = self.normal_panel:Find("Launch")
        local unlock, level = self:monsterUnlock(self.monster.cfg)

        search_btn.transform:SetActive(not unlock)
        launch_btn:SetActive(unlock)
        tip.transform:SetActive(not unlock)
        tip.text = string.format( NativeText.GetLangStr(280222), level)

        if not unlock then
            search_btn.onClick:RemoveAllListeners()
            search_btn.onClick:AddListener(
                function()
                    self.search_info = {type = self.monster.cfg.monster_index, lv = level}
                    network_sender:worldMapSearch(common_pb.ESEARCH_MONSTER)
                end
            )
        end
    end,

    rallyMonster = function(self)
        local scroll_view = self.rally_panel:Find("Reward/Scroll View")
        local tip = self.rally_panel:Find("Reward/Tip")

        if self.monster.cfg.monster_type == 5 then
            scroll_view:SetActive(false)
            tip.localScale = Vector3.one
        else
            scroll_view:SetActive(true)
            tip.localScale = Vector3.zero

            local path = "BG/Rally/Reward/Scroll View"
            self:loadDrop(path)
        end

        local spy_btn = self.rally_panel:Find("Spy")
        local rally_btn = self.rally_panel:Find("StartRally")
        local search_btn = self.rally_panel:Find("Search"):GetComponent(UnityEngine.UI.Button)
        local tip_text = self.rally_panel:Find("Tip/Text"):GetComponent(UnityEngine.UI.Text)
        local unlock, level = self:monsterUnlock(self.monster.cfg)

        spy_btn:SetActive(unlock)
        rally_btn:SetActive(unlock)
        search_btn.transform:SetActive(not unlock)

        if not unlock then
            tip_text.text = string.format( NativeText.GetLangStr(280222), level)

            search_btn.onClick:RemoveAllListeners()
            search_btn.onClick:AddListener(
                function()
                    self.search_info = {type = self.monster.cfg.monster_index, lv = level}
                    network_sender:worldMapSearch(common_pb.ESEARCH_MONSTER)
                end
            )
        else
            if self.monster.cfg.attack == 3 then
                tip_text.text = string.format( NativeText.GetLangStr(280209), 2)
            else
                tip_text.text = string.format( NativeText.GetLangStr(280226), 1)
            end
        end
    end,

    energyMonster = function(self)
        local tips = self.energy_panel:Find("Tip")
        local info = self.energy_panel:Find("Info")

        if self.monster.info.attacked_count == 0 or self.monster.info.bufftime <= 0 then
            tips.localScale = Vector3.one
            info.localScale = Vector3.zero

            self.boost_cd = nil
        else
            tips.localScale = Vector3.zero
            info.localScale = Vector3.one

            local atk_cnt = info:Find("Row/Value"):GetComponent(UnityEngine.UI.Text)
            local boost = info:Find("Boost/Value"):GetComponent(UnityEngine.UI.Text)
            self.boost_cd = info:Find("TimeLeft/Value"):GetComponent(UnityEngine.UI.Text)

            atk_cnt.text = self.monster.info.attacked_count
            boost.text = string.format("%.2f%%", self.monster.info.buff*0.01) 
        end

        local fill_img = self.energy_panel:Find("FillBG/Fill"):GetComponent(UnityEngine.UI.Image)
        local count = self.energy_panel:Find("FillBG/Count"):GetComponent(UnityEngine.UI.Text)
        local one_cost = self.energy_panel:Find("Attack/BG/Count"):GetComponent(UnityEngine.UI.Text)
        local repeat_cost = self.energy_panel:Find("RepeatATK/BG/Count"):GetComponent(UnityEngine.UI.Text)
        local repeat_cnt = self.energy_panel:Find("RepeatATK/Text"):GetComponent(UnityEngine.UI.Text)
        local atk_btn = self.energy_panel:Find("Attack"):GetComponent(UnityEngine.UI.Button)
        local repeat_btn = self.energy_panel:Find("RepeatATK"):GetComponent(UnityEngine.UI.Button)
        local repeat_bg = self.energy_panel:Find("RepeatATK/BG")

        local cur_energy = local_lord.energy
        local max_energy = local_player:getEnergyMax()
        local percent = cur_energy / (max_energy > 0 and max_energy or 1)
        self.max_count = math.floor(cur_energy / self.monster.info.energy)
        self.cost_energy = max_energy - cur_energy

        fill_img.fillAmount = percent
        count.text = string.format( "%d/%d", cur_energy, max_energy )
        one_cost.text = self.monster.info.energy

        if self.max_count < 1 then
            one_cost.color = Color.red
        else
            one_cost.color = Color.white
        end

        if self.max_count > 1 then
            repeat_bg.localScale = Vector3.one
            repeat_cost.text = self.monster.info.energy * self.max_count
            repeat_cnt.text = NativeText.GetLangStr(280208).." "..self.max_count
        else
            repeat_bg.localScale = Vector3.zero
            repeat_cnt.text = NativeText.GetLangStr(280218)
        end

        local path = "BG/Energy/FillBG/Plus"
        self:setScript("click", path, functor.make(self.onClickAddEnergy, self))

        local unlock, tech_id = self:monsterUnlock(self.monster.cfg)
        local tech_tip = self.energy_panel:Find("Tip/TechTip"):GetComponent(UnityEngine.UI.Text)
        local tip = self.energy_panel:Find("Tip/Text")
        local go_btn = self.energy_panel:Find("GoTo"):GetComponent(UnityEngine.UI.Button)
        local atk_btn = self.energy_panel:Find("Attack")
        local repeat_btn = self.energy_panel:Find("RepeatATK")

        tech_tip.transform:SetActive(not unlock)
        go_btn.transform:SetActive(not unlock)
        tip:SetActive(unlock)
        atk_btn:SetActive(unlock)
        repeat_btn:SetActive(unlock)

        if not unlock then
            local tech_cfg = config.home_config_research_lists[tech_id]

            if tech_cfg then
                tech_tip.text = string.format( NativeText.GetLangStr(280223), self.helper:getLangCfg(tech_cfg.name_id) )
            else
                log.error("###################################### find home research cfg error ! id ", tech_id)
            end

            go_btn.onClick:RemoveAllListeners()
            go_btn.onClick:AddListener(
                function()
                    local build_data = home_info:getBuildDataByCfgId(2)
                    if build_data then
                        local frame = frame_manager:show('home_entity_research', nil, build_data.id)
                        frame:addCommandBuff(function()
                            if tech_cfg then
                                frame:click_category(tech_cfg.type)
                                frame:click_tech(tech_cfg)
                            end
                        end)
                        self:click_Close()
                    else
                        hint:setText(NativeText.GetLangStr(9000022))
                    end
                end
            )
        end
    end,

    loadDrop = function(self, path)
        local scroll_view = self.transform:Find(path)
        self.scroll_rect = scroll_view.rect
        self.content = self.transform:Find(path.."/Viewport/Content")
        self.sizeFitter = self.content:GetComponent(UnityEngine.UI.ContentSizeFitter)

        local drops = loadstring("return "..self.cfg.personalreward)()

        if type(drops) == "table" then
            local idx = 0
            for k, v in pairs(drops) do
                local item = nil
                
                if idx < self.content.childCount then
                    item = self.content:GetChild(idx)
                else
                    item = Object.Instantiate(self.prefab).transform
                    item:SetParent(self.content)
                    resetTransformZ(item)
                end

                local item_cfg = item_helper:getItemCfg(k)
                if item and item_cfg then
                    item:SetActive(true)
                    item.name = "Item_"..idx

                    local grade = item:GetComponent(UnityEngine.UI.Image)
                    local icon = item:Find("Icon"):GetComponent(UnityEngine.UI.Image)

                    grade:SetSprite(gui_helper:getQualityIconPath(item_cfg.color))
                    icon:SetSprite(gui_helper:getItemIconPath(item_cfg))

                    local item_path = path.."/Viewport/Content/"..item.name
                    local item_inst = item_cls({cfgId = k, count = self.helper:getItemNum(k)})
                    self:clearScript("click", item_path)
                    self:setScript("click", item_path, functor.rmake(self.onClickItem, item_inst))

                    idx = idx + 1
                end
            end

            for i = idx, self.content.childCount-1 do
                local item = self.content:GetChild(i)
                item:SetActive(false)
            end
    
            app:startCoroutine(
                function()
                    coroutine.yield(nil)
                    self:setContentPosition()
                end
            )
        end
    end,

    setContentPosition = function(self)
        if self.content.rect.width > self.scroll_rect.width then
            self.sizeFitter.enabled = false
            self.content.pivot = Vector2(0, 0.5)
            local pos = self.content.localPosition
            pos.x = 0
            self.content.localPosition = pos
        end
    end,

    onClickItem = function(self, handler, item)
        frame_manager:show("itemInfo", self, {item = item})
    end,

    onClickAddEnergy = function(self)
        frame_manager:show('resource_exchange', nil, { 
            item_id = 50001,
            item_num = self.cost_energy,
        })
    end,

    click_Close = function(self)
        frame_manager:hide("world_monster_info")
    end,

    click_Info = function(self)
        
    end,

    click_Collect = function(self)
        local collection_pos = {}
        collection_pos.eTable = 0
        collection_pos.strDesc = world_map_helper:getEmptyName(self.monster.coordX, self.monster.coordY)
        collection_pos.posX = self.monster.coordX
        collection_pos.posY = self.monster.coordY
        collection_pos.iconId = 1
        world_map_info.ui:showEditPos(collection_pos)
    end,

    click_Launch = function(self)
        if not gui_helper:checkMonsterAtkCnt(self.cfg) then
            return
        end

        if army_queue_manager:queueMaxCheck() then
            return
        end

        local soldier_cnt = home_info:getAvailableSoldiers()

        if #soldier_cnt == 0 then
            hint:setText(NativeText.GetLangStr(900012))
            return
        end

        frame_manager:show("launch_rally", nil, {monster = self.monster, action = common_pb.EMARCH_MONSTER})
        self:hide()
    end,

    click_StartRally = function(self)
        --[[ if not gui_helper:checkMonsterAtkCnt(self.cfg) then
            return
        end

        if not local_guild:isSelfInGuild() then
            hint:setText(NativeText.GetLangStr(280106))
            return
        elseif not local_guild.is_remove then
            hint:setText(NativeText.GetLangStr(280107))
            return
        else
            if army_queue_manager:queueMaxCheck() then
                return
            end
            frame_manager:show("create_mass", self.owner, {hold = self.monster, before_frame = self})
        end ]]

        if not gui_helper:checkMonsterAtkCnt(self.cfg) then
            return
        end

        if not local_guild:isSelfInGuild() then
            hint:setText(NativeText.GetLangStr(289001))
            return
        end

        frame_manager:show("create_mass", self.owner, {hold = self.monster, before_frame = self})
    end,

    click_Spy = function(self)
        network_sender:startArmyLine(common_pb.EMARCH_SPY, nil, nil, self.monster.id)
        self:hide()
    end,

    click_Attack = function(self)
        if local_lord.energy < self.monster.info.energy then
            hint:setText(NativeText.GetLangStr(280219))
            return
        end

        local soldier_cnt = home_info:getAvailableSoldiers()

        if #soldier_cnt == 0 then
            hint:setText(NativeText.GetLangStr(900012))
            return
        end

        frame_manager:show("launch_rally", nil, {monster = self.monster, action = common_pb.EMARCH_MONSTER, atk_cnt = 1})
        self:hide()
    end,

    click_RepeatATK = function(self)
        local soldier_cnt = home_info:getAvailableSoldiers()

        if #soldier_cnt == 0 then
            hint:setText(NativeText.GetLangStr(900012))
            return
        end

        if self.max_count > 1 then
            frame_manager:show("launch_rally", nil, {monster = self.monster, action = common_pb.EMARCH_MONSTER, atk_cnt = self.max_count})
            self:hide()
        else
            self:onClickAddEnergy()
        end
    end,

    hide = function(self)
        self:click_Close()
    end,
}