--用来控制整个界面的打开和关闭
local log = logger.get("Game", "Guide Helper")

guide_helper = {
    -- 显示对话
    waitDialogFinished = function(self, id)
        guide_manager.waiting_tap_count_down = 5

        local finished = false

        dialog_manager:playScenario(id, function()
            finished = true
        end)

        self:disableScreenMask()

        while not finished do
            coroutine.yield(nil)
        end
        
        self:enableScreenMask()
    end,

    -- 显示 ‘点这里’ 的提示
    showTapHere = function(self, parent, layer, scale, is_3d)
        guide_manager.waiting_tap_count_down = 5

        if self.tap_here and self.tap_here.transform then
            self.tap_here.transform:SetParent(parent)


            if scale then
                self.tap_here.transform.localScale = Vector3(scale, scale, scale)
            else
                self.tap_here.transform.localScale = Vector3.one
            end
            if is_3d then
                self.tap_here_billboard = self.tap_here:AddComponent(Fenix.Unity.Billboard)
            elseif self.tap_here_billboard then
                Object.DestroyImmediate(self.tap_here_billboard)
                self.tap_here_billboard = nil
            end

            if layer then
                frame_helper:setChildLayer(self.tap_here.transform, layer)
            else
                frame_helper:setChildLayer(self.tap_here.transform, "UI")
            end

            self.tap_here.transform.localPosition = Vector3.zero
            self.tap_here.transform.anchoredPosition = Vector3.zero
            self.tap_here.transform.localRotation = Quaternion.identity
        else    
            app:startCoroutine(
            function()
                    game_director.pool:wait_prepare("Assets/Prefab/UI/TapHere.prefab", 5)
                    self.tap_here = game_director.pool:instantiate("Assets/Prefab/UI/TapHere.prefab")
                    if not self.tap_here.transform then
                        self.tap_here = game_director.pool:instantiate("Assets/Prefab/UI/TapHere.prefab")
                        if not self.tap_here.transform then
                            self.tap_here = game_director.pool:instantiate("Assets/Prefab/UI/TapHere.prefab")
                        end
                    end

                    self.tap_here.transform:SetParent(parent)

                    if layer then
                        frame_helper:setChildLayer(self.tap_here.transform, layer)
                    end
                    if scale then
                        self.tap_here.transform.localScale = Vector3(scale, scale, scale)
                    else
                        self.tap_here.transform.localScale = Vector3.one
                    end
                    if is_3d then
                        self.tap_here_billboard = self.tap_here:AddComponent(Fenix.Unity.Billboard)
                    elseif self.tap_here_billboard then
                        Object.DestroyImmediate(self.tap_here_billboard)
                        self.tap_here_billboard = nil
                    end

                    self.tap_here.transform.localPosition = Vector3.zero
                    self.tap_here.transform.anchoredPosition = Vector3.zero
                    self.tap_here.transform.localRotation = Quaternion.identity
            end)
        end
    end,

    removeTapHere = function(self)
        if self.tap_here then
            self.tap_here:destroy()
            if self.tap_here_billboard then
                Object.DestroyImmediate(self.tap_here_billboard)
                self.tap_here_billboard = nil
            end
        end
        self.tap_here = nil
        guide_manager.waiting_tap_count_down = -1
    end,

    -- 显示引导精灵,协程中调用
    waitGuideSpriteShow = function(self, pos)
        local is_finished = false

        if not self.guide_sprite then
            local path = 'Assets/Prefab/UI/GuideSprite.prefab'
            local parent = frame_manager.parent.transform
            coroutine.yield(WaitForInstantiate(path, parent, function(newObj)
                if newObj then
                    self.guide_sprite = newObj.transform
                    newObj.transform.anchoredPosition = pos
                    self.guide_sprite_skeleton = newObj.transform:Find("Spine"):GetComponent(Spine.Unity.SkeletonGraphic)
                    self.guide_sprite:Find('Bubble'):SetActive(false)
                    is_finished = true
                else
                    log.error("Failed to load spine:"..path)
                end
                is_finished = true                
            end))

        else
            self.guide_sprite:SetActive(true)
            self.guide_sprite:Find('Bubble'):SetActive(false)
            self.guide_sprite.anchoredPosition = pos
            is_finished = true
        end

        while not is_finished do
            coroutine.yield(nil)
        end

        self:guideSpritePlayAnim('birth', false)
        coroutine.yield(WaitForSeconds(0.5))
        self:guideSpritePlayAnim('idle', true)

    end,

    -- 隐藏引导精灵
    waitGuideSpriteHide = function(self)
        -- 播放消失动画
        if not self.guide_sprite then
            return
        end
        self:guideSpritePlayAnim('vanish', false)
        coroutine.yield(WaitForSeconds(0.5))
        self.guide_sprite:SetActive(false)
    end,

    guideSpriteHide = function(self)
        if not self.guide_sprite then
            return
        end
        self.guide_sprite:SetActive(false)
    end,
    
    -- 移动引导精灵到某位置
    guideSpriteMove = function(self, target_pos, time)
        if not self.guide_sprite then
            return
        end

        local curr_pos = self.guide_sprite.anchoredPosition

        if target_pos.x > curr_pos.x then
            -- 目标在右侧
            self:guideSpriteFacing(0)
        elseif target_pos.x < curr_pos.x then
            -- 目标在左侧
            self:guideSpriteFacing(1)
        end

        -- 播放移动
        self:guideSpritePlayAnim('idle', true)--run

        local lt = LeanTween.move(self.guide_sprite, target_pos, time or 1.0)
        lt.loopType = LeanTweenType.once
        lt:setEaseInOutQuart()

        lt:setOnComplete(
            function()
                -- 播放 idle
                self:guideSpritePlayAnim('idle', true)
            end
        )

    end,

    -- 移动引导精灵到某位置
    guideSpriteMoveToUI = function(self, frame_name, ui_path, offset, time)
        if not self.guide_sprite then
            return
        end

        -- 自动面朝方向
        -- 隐藏对话气泡
        local frame = frame_manager:getFrame(frame_name)
        if not frame then
            log.error('Failed to find frame:'..frame_name)
            return
        end
    
        local ui_control = frame.transform:Find(ui_path)
        if not ui_control then
            log.error("Failed to find ui control:"..ui_path)
        end

        local viewport_pos = frame_manager:getCamera():WorldToScreenPoint(ui_control.position)
        log.info("viewport_pos:"..tostring(viewport_pos))

        local canvas_rect = frame_manager.root.transform
        log.info("sizeDelta x:"..canvas_rect.sizeDelta.x..", y:"..canvas_rect.sizeDelta.y)
        local canvas_pos = Vector2(
            (viewport_pos.x) - (canvas_rect.sizeDelta.x*0.5), -- * canvas_rect.sizeDelta.x
            (viewport_pos.y) - (canvas_rect.sizeDelta.y*0.5)); --* canvas_rect.sizeDelta.y

        log.info("### canvas_pos:"..tostring(canvas_pos))

        local curr_pos = self.guide_sprite.anchoredPosition
        local target_pos = canvas_pos + offset

        if target_pos.x > curr_pos.x then
            -- 目标在右侧
            self:guideSpriteFacing(0)
        elseif target_pos.x < curr_pos.x then
            -- 目标在左侧
            self:guideSpriteFacing(1)
        end

        local lt = LeanTween.move(self.guide_sprite, target_pos, time or 1.0)
        lt.loopType = LeanTweenType.once
        lt:setEaseInOutQuart()

        lt:setOnComplete(
            function()
                -- 播放 idle
            end
        )

    end,

    -- 显示引导对话
    guideSpriteDialog = function(self, content, pos)
        if not self.guide_sprite then
            return
        end

        local bubble = self.guide_sprite:Find("Bubble")
        local text = self.guide_sprite:Find("Bubble/Text")

        local top_right = Vector3(93,37,0)
        local right = Vector3(125,-120,0)
        local bottom_right = Vector3(100,-282,0)
        local bottom_left = Vector3(-653,-282,0)
        local left = Vector3(-684,-125,0)
        local top_left = Vector3(-684,31,0)

        --log.error(tostring(bubble.transform.anchoredPosition))

        if pos == 'top_right' then
            bubble.transform.anchoredPosition = top_right
        elseif pos == 'right' then
            bubble.transform.anchoredPosition = right
        elseif pos == 'bottom_right' then
            bubble.transform.anchoredPosition = bottom_right
        elseif pos == 'bottom_left' then
            bubble.transform.anchoredPosition = bottom_left
        elseif pos == 'left' then
            bubble.transform.anchoredPosition = left
        elseif pos == 'top_left' then
            bubble.transform.anchoredPosition = top_left
        else
            bubble.transform.anchoredPosition = top_right
        end

        if content then
            bubble:SetActive(true)
            text:GetComponent(UnityEngine.UI.Text).text = content
        else
            bubble:SetActive(false)
        end
    end,

    -- 播放动画
    guideSpritePlayAnim = function(self, anim, is_loop)
        if not self.guide_sprite_skeleton then
            return
        end

        log.info("### guideSpritePlayAnim:"..anim)

        local anim_state = self.guide_sprite_skeleton.AnimationState
        if anim_state then
            if anim_state:GetAnimationName(0) ~= anim then
                anim_state:SetAnimation(0, anim, is_loop)
            end
        else
            log.error("Failed to find anim state")
        end   
    end,

    -- 面朝方向 0-默认的右边   1-左边
    guideSpriteFacing = function(self, facing)
        if facing == 0 then
            -- 目标在右侧
            self.guide_sprite_skeleton.transform.localRotation = Quaternion.identity
        elseif facing == 1 then
            -- 目标在左侧
            self.guide_sprite_skeleton.transform.localRotation = Quaternion.AngleAxis(180, Vector3.up)
        else
            log.error("Error in facing:"..tostring(facing)..", should be 0 or 1")
        end
    end,

    -- 打开界面
    showFrame = function(self, frame_name)
        local frame = frame_manager:show(frame_name)

        while not frame:isLoaded() do
            coroutine.yield(nil)
        end

        return
    end,

    -- 关闭界面
    hideFrame = function(self, frame_name)
       frame_manager:hide(frame_name)
    end,
    
    showSkipButtonCallback = function(self, is_active)
        -- skip 按钮的回调
        local trans = GameObject.Find("gui/root/game/SkipButton").transform
        trans:SetActive(is_active or false)

        if is_active then
            trans:SetAsLastSibling()           
        end

        if not self.is_skip_callback then
            self.is_skip_callback = true
            local btn = trans:GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(function()
                guide_manager:skipGuide()
            end)        
        end
    end,


    -- 显示屏幕遮罩
    enableScreenMask = function(self)
        frame_manager.full_screen_mask:SetActive(true)
        frame_manager:disableTouch()
    end,

    -- 关闭屏幕遮罩
    disableScreenMask = function(self)
        frame_manager.full_screen_mask:SetActive(false)
        frame_manager:enableTouch()
    end,

    -- 高亮 UI 控件
    beginHighlightUIControl = function(self, trans, layer, show_tap_here)
        local canvas = trans.gameObject:GetComponent(UnityEngine.Canvas)
        if not canvas then
            canvas = trans.gameObject:AddComponent(UnityEngine.Canvas)
            canvas.additionalShaderChannels = UnityEngine.AdditionalCanvasShaderChannels.TexCoord1;
        end

        canvas.overrideSorting = true
        canvas.sortingOrder = 1000
        canvas.sortingLayerName = "UI"

        if show_tap_here then

            local raycaster = trans.gameObject:GetComponent(UnityEngine.UI.GraphicRaycaster)
            if not raycaster then
                raycaster = trans:AddComponent(UnityEngine.UI.GraphicRaycaster)
            end
            self:showTapHere(trans, layer)
            return canvas, raycaster
        else
            return canvas
        end
    end,

    -- 停止高亮
    endHighlightUIControl = function(self ,canvas, raycaster)
        self:removeTapHere()
        if raycaster then
            Object.Destroy(raycaster)
        end
        Object.Destroy(canvas)
    end,

    -- 显示强引导并等待完成
    waitForceButtonGuide = function(self, frame_name, button_path, lang_id)
        local frame = frame_manager:getFrame(frame_name)
        if not frame then
            log.error("Failed to find frame:"..frame_name)
            return
        end

        if not frame.transform then
            log.error("frame not initialized:"..frame_name)
            return
        end

        local button_trans = frame.transform:Find(button_path)
        if not button_trans then
            log.error("Failed to find button:"..button_path.." of frame:"..frame_name)
            return
        end

        local canvas, raycaster = self:beginHighlightUIControl(button_trans, "Temp", true)

        local is_clicked = false
        self:registerWaitingClick(button_path, function()
            log.info("button callback")
            is_clicked = true
        end)

        while not is_clicked do
            coroutine.yield(nil)
        end

        self:unregisterWaitingClick(button_path)
        self:endHighlightUIControl(canvas, raycaster)

        return
    end,


    waitForceButtonOnClickGuide = function(self, frame_name, button_path, lang_id)
        local frame = frame_manager:getFrame(frame_name)
        if not frame then
            log.error("Failed to find frame:"..frame_name)
            return
        end

        if not frame.transform then
            log.error("frame not initialized:"..frame_name)
            return
        end

        local button_trans = frame.transform:Find(button_path)
        if not button_trans then
            log.error("Failed to find button:"..button_path.." of frame:"..frame_name)
            return
        end

        local canvas, raycaster = self:beginHighlightUIControl(button_trans, "Temp", true)
        
        local is_clicked = false
        local btn = button_trans:GetComponent(UnityEngine.UI.Button)
        btn.onClick:AddListener(
            function ()
                is_clicked = true
            end)

       -- log.error("## waitForceButtonOnClickGuide:"..button_path)
        while not is_clicked do
            coroutine.yield(nil)
        end
        self:endHighlightUIControl(canvas, raycaster)

        return  
    end,

    -- 等待一个面板被显示
    waitForTransformShow = function(self, frame_name, transform_path)
        local frame = frame_manager:getFrame(frame_name)
        if not frame then
            log.error("Failed to find frame:"..frame_name)
            return
        end

        if not frame.transform then
            log.error("frame not initialized:"..frame_name)
            return
        end

        local t = frame.transform:Find(transform_path)
        if not t then
            log.error("Failed to find:"..transform_path)
        end
        
        while not t.gameObject.activeInHierarchy do
            coroutine.yield(nil)
        end
    
        -- 界面优化的隐藏方式
        while not frame_helper.isTransShow(t.gameObject) do
            coroutine.yield(nil)
        end
        
        return
    end,

    -- 显示弱引导
    showWeakButtonGuide = function(self, frame_name, button_path)
       local frame = frame_manager:getFrame(frame_name)
        if not frame then
            log.error("Failed to find frame:"..frame_name)
            return
        end

        if not frame.transform then
            log.error("frame not initialized:"..frame_name)
            return
        end

        local button_trans = frame.transform:Find(button_path)
        if not button_trans then
            log.error("Failed to find button:"..button_path.." of frame:"..frame_name)
            return
        end

        local canvas, raycaster = self:beginHighlightUIControl(button_trans, "Temp", true)
        
        self:waitAnyTouch()

        self:endHighlightUIControl(canvas, raycaster)

        return
    end,


    -- 等待 frame 开启
    waitForFrame = function(self, frame_name)
        -- 等待界面开始加载
        local frame = nil
        while true do
            frame = frame_manager:getFrame(frame_name)
            if frame then
                break
            else
                coroutine.yield(nil)
            end
        end

        -- 等待界面加载完成
        while not frame:isLoaded() do
            coroutine.yield(nil)
        end

        coroutine.yield(nil)
    end,

    -- 等待用户点击按钮
    waitTouchButton = function(self, frame_name, button_path, lang_id)

        local is_callback = false
        local callback = function()
            is_callback = true
        end

        app:startCoroutine(
            function()
                coroutine.yield(WaitForSeconds(2.0))
                callback()
            end
        )

        -- 堵塞执行，等待回调
        while not is_callback do
            coroutine.yield(nil)
        end

        print("## Waiting finished!")
    end,

    -- 等待用户任何点击
    waitAnyTouch = function(self)
        guide_manager.waiting_tap_count_down = 5

        while true do
            coroutine.yield(nil)
            if Input.GetMouseButtonUp(0) then
                break
            end
        end
    end,

    -- 攝像機定位到個人基地的建築
    cameraFocusToHomeEntity = function(self, grid_id)
        local entity_obj = game_state_home.home_map:getEntityByGridId(grid_id)
        if not entity_obj then
            log.error("Failed to find entity obj:"..entity.id)
            return false
        end

        -- 隐藏操作按钮
        -- local obj = game_state_home.home_map:getOperator(entity_obj)
        -- game_state_home.home_map:hideOperatorBtns(obj)

        game_state_home.home_map:showOperator(nil)
        coroutine.yield(WaitForSeconds(0.1))
        game_state_home.home_map:cameraLookAt(entity_obj.transform)
        coroutine.yield(WaitForSeconds(0.1))
        game_state_home.home_map:cameraLookAt(entity_obj.transform)
        coroutine.yield(WaitForSeconds(0.1))
        game_state_home.home_map:cameraLookAt(entity_obj.transform)
        
        return entity_obj
    end,

    -- 摄像机定位到资源船
    cameraFocusToHomeResShip = function(self, chapter_id)
        if game_state_home then
            local obj = GameObject.Find("ResShipRoot/ResShipRoot"..chapter_id.."/AnimRoot")
            if obj then
                game_state_home.home_map:cameraLookAt(obj.transform)

                return obj
            end
        else
            log.error("cameraFocusToHomeResShip only call in home state!")
        end
    end,

    -- 等待用戶點擊 home entity
    waitForceHomeResShipGuide = function(self, ship_obj)
        -- 將 guide_camera 開啓，將 entity_obj 設置到 temp 層
        local guide_camera_go = self:getGuideCamera()
        guide_camera_go:SetActive(true)

        self.wait_tap_home_eneity = ship_obj  -- 当前等待点击的 home entity

        frame_helper:setChildLayer(ship_obj.transform, "Temp")
        self:showTapHere(ship_obj.transform, "Temp", 0.05, true)

        -- 判斷到用戶點擊
        while true do
            coroutine.yield(nil)
            if Input.GetMouseButtonUp(0) then
                local point = Input.mousePosition
                local rst, go = game_state_home.home_map:raycast(point, "Temp")
                if rst and go then
                    log.info('點擊到了。')
                    break
                else
                    log.info("Touch nothing")
                end
            end
        end

        -- 恢復
        self:removeTapHere()
        guide_camera_go:SetActive(false)
        frame_helper:setChildLayer(ship_obj.transform, "Home")
        self.wait_tap_home_eneity = nil
    end,


    waitHomeEntityLoaded = function(self, entity_obj)
        if not entity_obj then
            log.error("entity_obj is nil!!")
            return
        end

        while true do
            coroutine.yield(nil)
            if entity_obj.model then
                break
            end
        end

    end,

    getGuideCamera = function(self)
        if game_state_home then
            local home_map = game_state_home.home_map
            local home_camera = home_map.home_camera
            local guide_camera_go = home_camera.transform:Find("GuideCamera")  
            
            return guide_camera_go
        elseif game_state_world_map then
            local guide_camera_go = game_state_world_map.camera.transform:Find("GuideCamera")
            return guide_camera_go
        end
    end,

    -- 等待用戶點擊 home entity
    waitForceHomeEntityGuide = function(self, entity_obj)
        -- 將 guide_camera 開啓，將 entity_obj 設置到 temp 層
        local guide_camera_go = self:getGuideCamera()
        guide_camera_go:SetActive(true)
        while not entity_obj.model do
            coroutine.yield(nil)
        end

        guide_manager.waiting_tap_count_down = 5
        self.wait_tap_home_eneity = entity_obj  -- 当前等待点击的 home entity

        frame_helper:setChildLayer(entity_obj.transform, "Temp")
        self:showTapHere(entity_obj.transform, "Temp", 0.05, true)

        -- 判斷到用戶點擊
        while true do
            coroutine.yield(nil)
            if Input.GetMouseButtonUp(0) then
                local point = Input.mousePosition
                local rst, go = game_state_home.home_map:raycast(point, "Temp")
                if rst and go then
                    log.info('點擊到了。')

                    local home_entity = go:GetComponent(HomeEntity)
                    if home_entity then
                        log.info("home_map:onClickEntity:"..home_entity.id)
                        game_state_home.home_map:onClickEntity(home_entity.id, true)
                        break
                    else
                        log.info("## hit not entity:"..go.name)
                        -- 点中的不是 entity
                    end
                    
                else
                    log.info("Touch nothing")
                end
            end
        end

        -- 恢復
        self:removeTapHere()
        guide_camera_go:SetActive(false)
        frame_helper:setChildLayer(entity_obj.transform, "Home")
        self.wait_tap_home_eneity = nil
    end,

    -- 关闭所有兄弟结点的按钮
    enableAllSiblingButtons = function(self, trans, is_enabled)
        local parent = trans.parent;
        for i=0,parent.childCount-1 do
            local child = parent:GetChild(i)
            if child ~= trans then
                local btn = child:GetComponent(UnityEngine.UI.Button)
                btn.enabled = is_enabled
            end
        end
    end,

    -- 等待用戶點擊 home entity 按鈕
    waitForceHomeEntityButtonGuide = function(self, entity_obj, button_path)
        guide_manager.waiting_tap_count_down = 5
        
        local home_map = game_state_home.home_map
        local home_camera = home_map.home_camera
        local guide_camera_go = home_camera.transform:Find("GuideCamera")
        guide_camera_go:SetActive(true)
        --frame_helper:setChildLayer(entity_obj.model.transform, "Temp")
        local entity = home_info:getBuildDataByGridId(entity_obj.id)
        local ui_root = home_map:getOperator(entity_obj.id)
        local root_raycaster = ui_root:GetComponent(UnityEngine.UI.GraphicRaycaster)

        local button_trans = ui_root.transform:Find(button_path)
        if not button_trans then
            log.error("Failed to find button:"..button_path)
        end

        -- 遍历 button_trans 所有兄弟结点，把 button 关闭
        self:enableAllSiblingButtons(button_trans, false)

        local root_canvas = ui_root:GetComponent(UnityEngine.Canvas)
        root_canvas.worldCamera = guide_camera_go:GetComponent(UnityEngine.Camera)

        frame_helper:setChildLayer(button_trans, "Temp")

        local canvas, raycaster = self:beginHighlightUIControl(button_trans, "Temp", true)
  
        local is_clicked = false
        local callback = function()
            log.info("button callback")
            is_clicked = true
        end
        local btn = button_trans:GetComponent(UnityEngine.UI.Button)
        btn.onClick:AddListener(callback)

        -- 缓存
        self.waiting_home_entity_button = {
            button_trans = button_trans,
            root_canvas = root_canvas,
            canvas = canvas,
            raycaster = raycaster,
            guide_camera_go = guide_camera_go
        }

        -- 等待用戶點擊
        while not is_clicked do
            coroutine.yield(nil)
        end

        log.info("Clicked!!")
        self.waiting_home_entity_button = null

        -- 恢復
        self:enableAllSiblingButtons(button_trans, true)
        btn.onClick:RemoveListener(callback)
        root_canvas.worldCamera = nil
        frame_helper:setChildLayer(button_trans, "UI")
        self:endHighlightUIControl(canvas, raycaster)
        guide_camera_go:SetActive(false)

    end,

    -- Home Entity 操作按钮  非强制引导
    showHomeEntityButtonGuide = function(self, entity_obj, button_path)
        local home_map = game_state_home.home_map
        local home_camera = home_map.home_camera
        local guide_camera_go = home_camera.transform:Find("GuideCamera")
        guide_camera_go:SetActive(true)
        --frame_helper:setChildLayer(entity_obj.model.transform, "Temp")
        local entity = home_info:getBuildDataByGridId(entity_obj.id)
        local ui_root = home_map:getOperator(entity_obj.id)
        local root_raycaster = ui_root:GetComponent(UnityEngine.UI.GraphicRaycaster)

        local button_trans = ui_root.transform:Find(button_path)
        if not button_trans then
            log.error("Failed to find button:"..button_path)
        end

        -- 遍历 button_trans 所有兄弟结点，把 button 关闭
        self:enableAllSiblingButtons(button_trans, false)

        local root_canvas = ui_root:GetComponent(UnityEngine.Canvas)
        root_canvas.worldCamera = guide_camera_go:GetComponent(UnityEngine.Camera)

        frame_helper:setChildLayer(button_trans, "Temp")

        local canvas, raycaster = self:beginHighlightUIControl(button_trans, "Temp", true)
         
        self:waitAnyTouch()
        log.info("Clicked!!")

        -- 恢復
        self:enableAllSiblingButtons(button_trans, true)
        root_canvas.worldCamera = nil
        frame_helper:setChildLayer(button_trans, "UI")
        self:endHighlightUIControl(canvas, raycaster)
        guide_camera_go:SetActive(false)
    end,

    -- 等待用戶點擊 home entity 的气泡
    waitForceHomeEntityBubbleGuide = function(self, entity_obj)
        local home_map = game_state_home.home_map
        local home_camera = home_map.home_camera
        local guide_camera_go = home_camera.transform:Find("GuideCamera")
        guide_camera_go:SetActive(true)
        --frame_helper:setChildLayer(entity_obj.model.transform, "Temp")
        local entity = home_info:getBuildDataByGridId(entity_obj.id)
        local ui_root = home_map:getOperator(entity_obj.id)
      
        local button_path = "content/info/detail/bg/bubble layout"
        local parent = ui_root.transform:Find(button_path)
        local button_trans = parent:GetChild(0)
        if not button_trans then
            log.error("Failed to find button:"..button_path)
        end

        frame_helper:setChildLayer(button_trans, "Temp")

        local canvas, raycaster = self:beginHighlightUIControl(button_trans, "Temp", true)
  
        local is_clicked = false
        local callback = function()
            log.info("button callback")
            is_clicked = true
        end
        local btn = button_trans:GetComponent(UnityEngine.UI.Button)
        btn.onClick:AddListener(callback)

        -- 等待用戶點擊
        while not is_clicked do
            coroutine.yield(nil)
        end

        log.info("Clicked!!")

        -- 恢復
        btn.onClick:RemoveListener(callback)
        frame_helper:setChildLayer(button_trans, "UI")
        self:endHighlightUIControl(canvas, raycaster)
        guide_camera_go:SetActive(false)

    end,

    -- 英雄列表定位到某个英雄,返回该英雄的界面对象
    heroListFocusTo = function(self, hero_cfg_id)
        local hero_list_frame = frame_manager:getFrame("hero_list")
        if not hero_list_frame then
            log.error("Failed to find hero_list frame!")
            return false
        end

        local elem = nil
        for k,v in pairs(hero_list_frame.heros) do
            local officer_id = hero_list_frame:getCommandCfgId(v)
            if officer_id == hero_cfg_id then
                elem = v.hero_elem
                break
            end
        end

        if not elem then
            log.error("Failed to find hero:"..hero_cfg_id)
            return
        end

        local trans = hero_list_frame.transform:Find("BG/Scroll View")
        local scrollRect = trans.gameObject:GetComponent(UnityEngine.UI.ScrollRect)
        local content = scrollRect.content
        scrollRect.verticalNormalizedPosition = 1 + (elem.transform.localPosition.y + 300) / content.rect.height

        return elem
    end,

    
    -- 显示英雄点击的强制引导
    waitForceHeroGuide = function(self, hero_elem)
        local img_path = "BG/Scroll View/Viewport/Content/"..hero_elem.name

        -- 高亮该控件
        local hero_list_frame = frame_manager:getFrame("hero_list")
        local img_trans = hero_list_frame.transform:Find(img_path)

        local canvas, raycaster = self:beginHighlightUIControl(img_trans, "Temp", true)

        local is_clicked = false

        local btn = img_trans:GetComponent(UnityEngine.UI.Button)
        btn.onClick:AddListener(
            function()
                is_clicked = true
            end
        )
        while not is_clicked do
            coroutine.yield(nil)
        end
        
        --[[
        -- 当前等待点击的控件
        self:registerWaitingClick(img_path, 
            function()
                is_clicked = true
            end)

        while not is_clicked do
            coroutine.yield(nil)
        end
        self:unregisterWaitingClick(img_path)]]

        self:endHighlightUIControl(canvas, raycaster)

        return
    end,

    -- 更换装备界面点击武器强引导
    waitForceChangeEquipClickGuide = function(self, cfg_id)
        local frame = frame_manager:getFrame('changeEquipment')
        if not frame then
            log.error("no changeEquipment frame!!")
            return false
        end 
        
        local equip = nil
        for k,v in ipairs(frame.equipments) do
            if v.cfg.id == cfg_id then
                equip = v
                break
            end
        end

        if not equip then
            log.error("Failed to find equip:"..cfg_id)
            return false
        end

        local path = "BG/Right/Scroll View/Viewport/Content/"..equip.ui_item.name.."/Grade"
        
        print("111")
        coroutine.yield(self:waitForceButtonGuide('changeEquipment', path))
        print("2222")
        
        -- BG/Right/Scroll View/Viewport/Content/slot_1/Grade

    end,

    waitForPlayerHold = function(self)
        -- 等待玩家基地创建
        local world_map = world_map_helper.world_map

        -- 等待界面加载完成
        while not world_map.hold_manager.player_hold do
            coroutine.yield(nil)
        end

        coroutine.yield(nil)
    end,

    -- 注册一个控件点击的回调
    registerWaitingClick = function(self, path, callback)
        self.waiting_clicks = {[path] = callback}
    end,

    -- 反注册控件点击
    unregisterWaitingClick = function(self, path)
        self.waiting_clicks[path] = false
        self.waiting_clicks = nil
    end,

    -- 统一的界面点击回调
    onClick = function(self, path)
        log.info('click:'..path)
        if not self.waiting_clicks then
            return
        end

        local callback = self.waiting_clicks[path]
        if  callback then
            callback()
        else
            -- 点了引导之外打断引导
            guide_manager:onGuideBreak()

        end
    end,

    -- 获取道具数量
    getItemNum = function(self, item_cfg_id)
        return frame_helper:getItemNum(item_cfg_id)
    end,

    -- 获取个人基地建筑等级
    getHomeEntityLevel = function(self, cfg_id)
        local entity_data = home_info:getBuildDataByCfgId(cfg_id)
        if entity_data then
            return entity_data.level
        end
    end,

    -- 判断引导是否完成
    isGuideComplete = function(self, id)
        return guide_manager:isGuideComplete(id)
    end,

    -- 判断科技是否完成
    isTechFinished = function(self, id)
    end,

    -- 判断个人基地建筑能否有足够的资源升到下一级
    isEnoughResource = function(self, cfg_id)
        --id为建筑id
        --返回boolean
        getUpgradeCfg = function(cfg_id, level)
            local rst = nil
            local max_level = 1
            config.home_config_upgrades:foreach(
                function(k, v)
                    if v.building_id == cfg_id then
                        if v.level == level then rst = v end
                        if v.level > max_level then max_level = v.level end
                    end
                end)
            return rst, max_level
        end
        local entity_data = home_info:getBuildDataByCfgId(cfg_id)
        if entity_data then
            local cfg, max_level = getUpgradeCfg(cfg_id, entity_data.level)
            if entity_data.level < max_level then
                local cost_tab = loadstring("return " .. cfg.cost)()
                
                for item_id, need_num in pairs(cost_tab) do
                    local own_num = self:getItemNum(item_id)
                    if need_num > own_num then
                        return false
                    end
                end
                return true
            end
        end
    end,

    -- 获取英雄晋升等级 ￥￥￥￥￥英雄的初始晋升等级是1 ￥￥￥￥￥￥
    getHeroQuality = function(self, id)
        local hero = hero_manager:getHero(id)
        if hero and hero.officer then
            return hero.officer.quality
        else
            return nil
        end
    end,

    -- 获取英雄觉醒等级 英雄的初始觉醒等级，星级是0
    getHeroAwakeLevel = function(self, id)
        local hero = hero_manager:getHero(id)
        if hero and hero.officer then
            return hero.officer.awake_level, hero.officer.star_level
        else
            return nil
        end
    end,

    isInGuild = function(self)
        --有公会返回true，else false
        return local_guild:isSelfInGuild()
    end,

    -- 引导超时
    onGuideOverTime = function(self)
        if self.wait_tap_home_eneity then
            local guide_camera_go = self:getGuideCamera()
            guide_camera_go:SetActive(false)
            frame_helper:setChildLayer(self.wait_tap_home_eneity.transform, "Home")
            self.wait_tap_home_eneity = nil
            self.waiting_clicks = nil
        end
        self.waiting_clicks = nil   
        
        if self.waiting_home_entity_button then
            local cache = self.waiting_home_entity_button
            -- 恢復
            self:enableAllSiblingButtons(cache.button_trans, true)
            cache.root_canvas.worldCamera = nil
            frame_helper:setChildLayer(cache.button_trans, "UI")
            self:endHighlightUIControl(cache.canvas, cache.raycaster)
            cache.guide_camera_go:SetActive(false)
            self.waiting_home_entity_button = null            
        end

        self:guideFinished()
    end,

    guideFinished = function(self)
        guide_helper:removeTapHere()
        guide_helper:disableScreenMask()
        guide_helper:guideSpriteHide()  
        guide_helper:showSkipButtonCallback(false)
    end,

    -- 等待用戶點擊 monster
    waitForceMonsterGuide = function(self)
        local guide_camera_go = self:getGuideCamera()
        guide_camera_go:SetActive(true)
        local monster = search_info:getCurrHold()

        while not monster do
            coroutine.yield(nil)
        end

        local hold = world_map_helper:getHoldById(monster.id)

        while not hold or not hold.transform do
            coroutine.yield(nil)
        end

        frame_helper:setChildLayer(hold.transform, "Temp")
        self:showTapHere(hold.transform, "Temp", 0.05, true)

        -- 判斷到用戶點擊
        while true do
            coroutine.yield(nil)
            if Input.GetMouseButtonUp(0) then
                local point = Input.mousePosition
                local rst, go = world_map_helper.world_map:raycast(point, "Temp")
                if rst and go then
                    log.info('點擊到了。')
                    network_sender:queryWorldMap(hold.id, protocol_pb.MAP_WORLDMAP)
                    break
                end
            end
        end

        -- 恢復
        self:removeTapHere()
        guide_camera_go:SetActive(false)
        frame_helper:setChildLayer(hold.transform, "Tank")
    end,

    setSearchMonsterIdx = function(self, idx)
        local frame = frame_manager:getFrame("world_map_search")
        if not frame then
            log.error("Failed to find frame:"..frame_name)
            return
        end

        if not frame.transform then
            log.error("frame not initialized:"..frame_name)
            return
        end

        local is_valid = search_info:idxIsValid(idx)

        if is_valid then
            frame:onClickMonster(idx)
        end

        return is_valid
    end,

    setSearchMonsterLv = function(self, lv)
        local frame = frame_manager:getFrame("world_map_search")
        if not frame then
            log.error("Failed to find frame:"..frame_name)
            return
        end

        if not frame.transform then
            log.error("frame not initialized:"..frame_name)
            return
        end

        local cnt = search_info:getMonsterCntByLevel(frame.monster_idx, lv)

        if cnt > 0 then
            frame:setMonsterLv(lv)
            frame:setSliderValue()
        end

        return cnt
        --local count = guide_helper:setSearchMonsterLv(5)
    end,

    getSearchMonsterPath = function(self, idx)
        local frame = frame_manager:getFrame("world_map_search")
        if not frame then
            log.error("Failed to find frame:"..frame_name)
            return
        end

        if not frame.transform then
            log.error("frame not initialized:"..frame_name)
            return
        end

        local data = frame.monster_ui[idx]

        if data then
            return "BG/Monster/Left/Scroll View/Viewport/Content/"..data.ui.name
        end

        return false
    end,

    waitSearchFrameLoaded = function(self)
        local frame = frame_manager:getFrame("world_map_search")
        if not frame then
            log.error("Failed to find frame:"..frame_name)
            return
        end

        while true do
            coroutine.yield(nil)
            if frame.is_filled then
                break
            end
        end
    end,

    -- 找到可以建造 build_id 的 grid
    findAvailableGrid = function(self, build_id)
        -- 遍历所有 grid
        local found_entity = nil
        config.home_config_grids:foreach(
            function(grid_id, cfg)
                if found_entity then
                    return
                end

                local entity = 
                    game_state_home.home_map:getEntityByGridId(grid_id)
                if not entity:hasBuild() then
                    local tab_can_build = loadstring("return "..cfg.can_build)()
                    for _, _id in pairs(tab_can_build) do
                        if _id == build_id then
                            found_entity = entity
                            break
                        end
                    end
                end
            end
        )

        return found_entity
    end,
}

return guide_helper