local log = logger.get("Game", "emp manager")
-- emp 能量道具 31004

local emp_manager_cls  = class{
    ctor = function(self)
        self.events = require('base.event').new()
        self.level = 1  -- 初始为 1 级
        self.modules = {
            {id=1,level=1},
            {id=2,level=1},
            {id=3,level=1},
            }
        self.mod_effects = {}

        -- 消耗道具 id, 数量
        self.cost_item_id = nil
        self.cost_item_num = nil

        self.emp_use_count = 0
        self.duration = 0
        self.in_range_monsters = {}

        --self:print()
    end,

    init = function(self)
        self:refreshCostItem()
    end,

    --[[//EMP信息
    message MEMPInfo
    { 
        repeated MEMPModInfo empModInfo      = 1;  //EMP各个模块信息
        required uint32     level            = 2;  //等级
    }
    //EMP模块信息
    message MEMPModInfo
    { 
        required uint32     id               = 1;  //哪一模块 
        required uint32     level            = 2;  //等级
        required uint32     curExp           = 3;  //当前经验
    }]]
    -- 初始化，可能会被调用多次
    refresh = function(self, emp_info)
        self.level = emp_info.level                 -- 总的经验 
        self.modules = {}
        for _, mod_info in ipairs(emp_info.empModInfo) do
            self.modules[mod_info.id] = mod_info    -- 每个 emp 模块的详细信息 [id]- MEMPModInfo
        end

        self.events:dispatch("onRefreshed")

        -- 消耗道具 和 数量
        self:refreshCostItem()

        self:print()
    end,

    refreshCostItem = function(self)
        local tmp_str = config.game_copy_emp_upgrades[self.level].emp_cost
        local cost_tab = loadstring("return "..tmp_str)()
        for k,v in pairs(cost_tab) do 
            self.cost_item_id = k
            self.cost_item_num = v
            break
        end
    end,

    -- 获取 emp 等级。 0-主模块  1~3 子模块
    getModuleLevel = function(self, mod_id)
        if mod_id == 0 then
            return self.level   -- 主模块
        else
            if not self.modules[mod_id] then
                log.error("#### emp module not exist ! id ",mod_id)
                return
            end
            return self.modules[mod_id].level  -- 子模块
        end
    end,

    -- 获取模块升级所需材料
    getModuleUpgradeItem = function(self, mod_id, level)
        if mod_id == 0 then
            return nil
        end

        local cfg = config.game_copy_emp_upgrades[level]
        if not cfg then
            log.error('### Failed to find emp upgrades cfg, level:'..level)
            return nil
        end

        local str = nil
        if mod_id == 1 then
            str = cfg.mod1_upgrade
        elseif mod_id == 2 then
            str = cfg.mod2_upgrade
        elseif mod_id == 3 then
            str = cfg.mod3_upgrade
        end

        -- tab 里只有一个元素 
        local tab = loadstring("return "..str)()
        for id, num in pairs(tab) do
            return id, num
        end
    end,

    -- 获取模块的效果 {[level] - {[prop_id] - value}}
    getModuleEffects = function(self, mod_id)
        if self.mod_effects[mod_id] then
            return self.mod_effects[mod_id]
        end

        local key = "mod"..mod_id
        local effects = {}
        config.game_copy_emp_upgrades:foreach(function(k,v)
            effects[v.level] = loadstring("return ".. v[key])()
        end)

        self.mod_effects[mod_id] = effects
        return effects
    end,

    -- 获取使用 emp 需要消耗的道具 id
    getCostItem = function(self)
        return self.cost_item_id, self.cost_item_num
    end,

    -- 显示瞄准界面
    showAimFrame = function(self)
        if self.is_aimming then
            return
        end
        self.is_aimming = true

        -- 显示瞄准界面
        frame_manager:show('emp_aim')
        -- 关闭关卡界面
        local level_frame = frame_manager:getFrame('level')
        if level_frame then
            --level_frame.transform:SetActive(false)
            frame_helper.hideTrans(level_frame.transform)
        end

        game_state_level.ignore_tap = true  -- 忽略用户点击
        local level_manager = game_state_level.level.level_manager
        level_manager.fogOfWar.gameObject:SetActive(false)  -- 关闭战争迷雾

        -- 改变摄像机位置
        self.level_camera = level_manager.levelCamera
        local main_cam = level_manager.mainCamera
        local look_target = self.level_camera.transform.position - self.level_camera.m_Offset 
        local new_offset = self.level_camera.m_Offset * 2
        new_offset.z = new_offset.z * 0.2
        self.level_camera:LookPosWithOffset(look_target, new_offset)
        level_script:changeCameraFov(45)
        -- 绑定 EmpCannon 的模型
        local path = "Assets/Prefab/Level/EmpCannon.prefab"
        local cannon = game_director.pool:instantiate(path)
        if self.level_camera.transform then
            cannon.transform:SetParent(self.level_camera.transform)
        else
            log.error('can not find transform of level camera')
        end
        level_helper:makeTransformIdentity(cannon.transform)
        self.emp_cannon = cannon

        local main_cam = level_manager.mainCamera
        self.emp_image_effect = main_cam:GetComponent(EmpImageEffect) or main_cam:AddComponent(EmpImageEffect)
        self.emp_image_effect.enabled = true
        SoundEngine.PostEvent("wpn_skill_reload_17", self.gameObject)
    end,


    -- 关闭瞄准界面
    closeAimFrame = function(self)
        if self.is_aimming then
            self.is_aimming = false
        else
            return
        end

        -- 显示关卡界面
        local level_frame = frame_manager:getFrame('level')
        if level_frame and level_frame.transform then
            --level_frame.transform:SetActive(true)
            frame_helper.showTrans(level_frame.transform)
        end
        -- 关闭瞄准界面
        frame_manager:hide('emp_aim')

        game_state_level.ignore_tap = false -- 恢复用户点击
        local level_manager = game_state_level.level.level_manager
        level_manager.fogOfWar.gameObject:SetActive(true)   -- 开启战斗阴影

        -- 恢复摄像机
        --game_state_level:lookTank()
        if not self.level_camera then
            self.level_camera = level_manager.levelCamera
        end

        local look_target = self.level_camera.transform.position - self.level_camera.m_Offset 
        self.level_camera:LookPosWithOffset(look_target, self.level_camera.m_InitOffset)
        if self.emp_cannon then
            self.emp_cannon:destroy()
            self.emp_cannon = nil
        end

        self.emp_image_effect = level_manager.mainCamera:GetComponent(EmpImageEffect)
        if self.emp_image_effect then
            self.emp_image_effect.enabled = false
        end
    end,

    -- 检查范围内的怪物
    checkInRangeMonsters = function(self)
        local emp_range = 300
        if self.level_camera then
            self.emp_target_pos = self.level_camera.transform.position - self.level_camera.m_Offset 
            --log.info('target postion  '..self.emp_target_pos)
            self.in_range_monsters = {}
            for _,go in pairs(level_script.level_go_list) do
                if Vector3.Distance(go.transform.position, self.emp_target_pos) < emp_range then
                    table.insert(self.in_range_monsters, go)
                    go.is_in_emp_range = true
                    --log.info('inserted')
                else 
                    go.is_in_emp_range = false
                    --log.info('insert failed')
                end
            end
        end

    end,

    -- 发射 emp
    fireEmp = function(self)
        -- 发射点
        local left = level_helper:findNode(self.emp_cannon.transform, "Left")
        local right = level_helper:findNode(self.emp_cannon.transform, "Right")

        local left_point = left:Find("shoot")
        local right_point = right:Find("shoot")

        local left_anim = left:GetComponent(UnityEngine.Animation)
        local right_anim = right:GetComponent(UnityEngine.Animation)

        --local look_target = self.level_camera.transform.position - self.level_camera.m_Offset 

        --level_script:shootBullet(20002, left_point, look_target, 200)
        --level_script:shootBullet(20002, right_point, look_target, 200)

        app:startCoroutine(
            function()
                for i = 1, 8 do
                    local effect_id = 10017
                    local random_range = 30
                    
                    local soundCallBack = function()
                        SoundEngine.PostEvent("Play_explo_small", self.gameObject)
                        --log.error('11')
                    end

                    if i == 8 then

                        local fireCallBack = function()
                            self:useEmpReq()
                        end

                        left_anim:Play("attack")
                        local fire_target = Vector3.New(-348.5519, 0, -123.0076) + Vector3.New(math.random(-random_range, random_range), 0, math.random(-random_range, random_range))
                        level_script:shootBullet(effect_id, left_point, fire_target, 100, fireCallBack)
                        level_script:playCameraAnim('shake11')
                        SoundEngine.PostEvent("wpn_skill_launch_17", self.gameObject)
                        coroutine.yield(WaitForSeconds(0.1))

                        right_anim:Play("attack")
                        fire_target = Vector3.New(-348.5519, 0, -123.0076) + Vector3.New(math.random(-random_range, random_range), 0, math.random(-random_range, random_range))
                        level_script:shootBullet(effect_id, right_point, fire_target, 100, soundCallBack)
                        level_script:playCameraAnim('shake11')
                        SoundEngine.PostEvent("wpn_skill_launch_17", self.gameObject)
                        coroutine.yield(WaitForSeconds(0.1))
                    else
                        left_anim:Play("attack")
                        local fire_target = Vector3.New(-348.5519, 0, -123.0076) + Vector3.New(math.random(-random_range, random_range), 0, math.random(-random_range, random_range))
                        level_script:shootBullet(effect_id, left_point, fire_target, 100, soundCallBack)
                        level_script:playCameraAnim('shake11')
                        SoundEngine.PostEvent("wpn_skill_launch_17", self.gameObject)
                        coroutine.yield(WaitForSeconds(0.1))

                        right_anim:Play("attack")
                        fire_target = Vector3.New(-348.5519, 0, -123.0076) + Vector3.New(math.random(-random_range, random_range), 0, math.random(-random_range, random_range))
                        level_script:shootBullet(effect_id, right_point, fire_target, 100, soundCallBack)
                        level_script:playCameraAnim('shake11')
                        SoundEngine.PostEvent("wpn_skill_launch_17", self.gameObject)
                        coroutine.yield(WaitForSeconds(0.1))
                    end
                end
            end
        )

    end,

    useEmpReq = function(self)
        -- local entities = {}
        -- log.info("## emp fire:".. tostring(emp_manager.emp_target_pos))

        -- for k,entity in pairs(emp_manager.in_range_monsters) do
        --     table.insert(entities, entity.id)
        --     log.info("id:"..entity.id)
        -- end
        -- network_sender:useEmp(emp_manager.emp_target_pos, entities)

        app:startCoroutine(
            function()
                for index, go in pairs(self.in_range_monsters) do
                    go:SetActive(false)
                    local path = "Assets/Prefab/Level/level_boom.prefab"
                    level_script:createGameObject(path, go.transform.position.x, 0, go.transform.position.z, 0, 1)
                    coroutine.yield(WaitForSeconds(0.25))
                end
                self:closeAimFrame()
                level_script:changeCameraFov(31)
                level_script:cameraFocusToPlayer()
            end
        )
    end,

    -- emp开始生效
    useEmpResult = function(self, duration, weakenEntities)
        log.info("###emp result, duration:"..duration..", num:"..#weakenEntities)
        local level = game_state_level.level
        for _,id in ipairs(weakenEntities) do
            log.info("emp result id:"..id)
            local entity = level:getEntity(id)
            entity:beginEmp(duration)
            self.duration = duration
        end
        --[[
        self.emp_use_count = self.emp_use_count + 1

        local level = game_state_level.level
        if not level then
            log.error('use emp, no level!!')
            return
        end

        self.duration = duration

        local pos = level.player:getPosition()
        -- 玩家坦克上播放特效
        local path = "Assets/Arts/ui/Fx/ui_prefab/Effect_EMP_Click.prefab"
        local effect = 
            level_script:createGameObject(path, pos.x, pos.y, pos.z, 0, 1)
        battle_helper:destroyObjectAfter(effect, 8)

        self:showEmpEffect()

        -- duration 时间后关闭 emp
        app:startCoroutine(
                function()
                    coroutine.yield(WaitForSeconds(duration))
                    self:empFinished()
                    
                end)   
        ]]
        app:startCoroutine(
            function()
                coroutine.yield(WaitForSeconds(self.duration))
                self.duration = 0
            end
        )
        self.events:dispatch("onEmpUsed")
    end,

    showEmpEffect = function(self)
        -- 敌方坦克上显示 emp 效果图标
        local monsters = level_helper:getEntitiesByType(common_pb.MET_MONSTER)
        for _,monster in pairs(monsters) do
            monster:showIcon("Assets/Arts/ui/atlas/icon/home/button_12.png")
        end

        -- 播放 emp 屏幕特效
        if level_frame and self.emp_use_count == 1 then
            path = "Assets/Arts/ui/Fx/ui_prefab/UI_Effect_EMP_Fullscreen.prefab"
            self.screen_effect = 
                level_script:createGameObject(path, 0, 0, 0, 0, 1)
            self.screen_effect.transform:SetParent(level_frame.transform)
            self.screen_effect.transform.localPosition = Vector3.zero
            self.screen_effect.transform.localScale = Vector3.one
        end
    end,

    -- emp效果 结束
    empFinished = function(self)
        self.emp_use_count = self.emp_use_count - 1

        if self.emp_use_count <= 0 then
            if game_director.state_name == "level" then

                local monsters = level_helper:getEntitiesByType(common_pb.MET_MONSTER)
                for _,monster in pairs(monsters) do
                    monster:hideIcon()
                end
                if self.screen_effect ~= nil then
                    log.info("Destroy screen effect!!")
                    battle_helper:destroyObjectAfter(self.screen_effect, 0.01)
                    self.screen_effect = nil
                end
            else
                self.screen_effect = nil
            end
            self.duration = 0
        end
    end,

    print = function(self)
        local str = ""
        str = "level:"..self.level.."cost:"..self.cost_item_id..":"..self.cost_item_num.."\n"
        for k,v in ipairs(self.modules) do
            str = "mod:"..v.id..", level:"..v.level.."\n"
        end
        log.info(str)
    end,

    
}

emp_manager = emp_manager_cls()

return emp_manager