local log = logger.get("Game", "Level")

-- 关卡处理回合的扩展
return {
    -- 更新一个回合
    updateTurn = function(self, _target_point_id)
        log.info("### updateTurn")

        local target_point_id =_target_point_id or self.player.path_point_target
        level_info.step_count = level_info.step_count + 1   -- 增加步数
        -- 刷新界面
        game_state_level.gui:refreshHealthAndGasoline()

        self.path_point_manager:HideAllPoints();
        self.player.path_point_target = target_point_id

        -- 收集所有 tank 的起点 / 终点
        local starts = {}
        local ends = {}

        -- 执行所有 entity 的回合
        for k,v in pairs(self.entities) do
            if v and v.preUpdateTurn then
                v:preUpdateTurn(starts, ends)
            end
        end

        local move_half = {}
        local move_offset = {}

        -- 判断两 entity 起终点互换的情况
        local id_a = self.player.id
        local entity_a = self.player
        for id_b, entity_b in pairs(self.entities) do
            if entity_b and not entity_b:isHide() then
                if starts[id_a] == ends[id_b] and starts[id_b] == ends[id_a] then
                    move_half[id_a] = true
                    move_half[id_b] = true
                end
            end
        end    

        -- 判断两 entity 终点相同的情况
        for id_b, entity_b in pairs(self.entities) do
            if entity_b and not entity_b:isHide() then
                if id_a ~= id_b and ends[id_a] == ends[id_b] then
                    if not move_half[id_a] then
                        move_offset[id_a] = true
                    end

                    if not move_half[id_b] then
                        move_offset[id_b] = true
                    end
                end
            end
        end    


        -- 执行所有 entity 的回合
        for id, v in pairs(self.entities) do
            if v and v.updateTurn then
                v:updateTurn(move_half[id], move_offset[id])
            end
        end

        app:startCoroutine(
            function()
                coroutine.yield(nil)

                local start_time = Time.time
                -- 等待所有 entity 回合移动结束
                while true do
                    local is_move_finished = self:isTurnMoveFinished()
                    if is_move_finished or Time.time - start_time > 1.8 then
                        break
                    else
                        coroutine.yield(nil)
                    end
                end

                if not self.level_script then
                    return
                end

                self.level_script.curr_point_id = self.player.path_point_id

                -- 判断是否要进行战斗
                local to_battle_id = nil
                local to_battle_monster = nil
                local point_id = self.player.path_point_id
                for k,v in pairs(self.entities) do
                    if v then
                        
                        if v.entity_type == common_pb.MET_MONSTER and not v.is_hide then
                            -- 当非游民怪物与玩家站在同一点
                            if v.path_point_id == point_id and not v.is_tramp then
                                -- 进行战斗
                                to_battle_id = v.id
                                to_battle_monster = v
                                break
                            -- 玩家与怪物错肩
                            elseif v.path_point_id == self.player.last_path_point_id and 
                                    v.last_path_point_id == self.player.path_point_id then
                                to_battle_id = v.id
                                to_battle_monster = v
                                break
                            end
                        end
                    end
                end
                log.info("### 666666")
                -- 玩家进入一个点的逻辑
                if to_battle_id then
                    -- 优先进行战斗
                    log.info("## battle with monster:"..to_battle_id)
                    --level_helper:battleWithMonster(to_battle_id, false)
                    game_state_level:breakPlayerInput()
                    to_battle_monster:changeState("interact", {player = self.player})
                    coroutine.yield(WaitForSeconds(1.0))
                    game_state_level:resumePlayerInput()
                else
                    log.info("## show path point:"..target_point_id)
                    self.path_point_manager:ShowPathPoint(target_point_id)

                    -- 判断点的类型
                    --[[
                    public enum EType
                    {
                        None = 0,
                        RoadBlock = 1,  // 路障
                        SteelSafe = 2,  // 保险箱
                        Tramp = 3,      // 游民
                        Footprint = 4,  // 足迹
                        Book = 5,       // 书籍
                        Door = 6,       // 门
                    }]]
                    local point = self.path_point_manager:FindPathPoint(target_point_id)
                    -- 通知关卡脚本进入一个点
                    level_script:onEnterPoint(target_point_id)

                    -- 判断进入 op trigger


                    if point.m_Type == 0 then -- 无事件点

                    else
                        game_state_level:breakPlayerInput()

                        if point.m_Type == 1 then -- 路障
                            level_script:onMoveToRoadBlock(target_point_id)
                        elseif point.m_Type == 2 then -- 保险箱
                            level_script:onMoveToSteelSafe(target_point_id)
                        elseif point.m_Type == 3 then -- 游民
                            level_script:onMoveToTramp(target_point_id)
                        elseif point.m_Type == 4 then -- 足迹
                            level_script:onMoveToFootprint(target_point_id)
                        elseif point.m_Type == 5 then -- 书籍
                            level_script:onMoveToBook(target_point_id)
                        elseif point.m_Type == 6 then -- 门
                            level_script:onMoveToDoor(target_point_id)
                        elseif point.m_Type == 10 then  -- custom

                        end

                        coroutine.yield(WaitForSeconds(1.5))
                        game_state_level:resumePlayerInput()
                    end
                end
            end
        )

    end,

    -- 设置点的类型
    setPathPointType = function(self, id, type, block_point_id)
        -- 记录到 level info 中
        level_info.path_point_type[id] = type

        -- 修改关卡中点的类型
        local point = self.path_point_manager:FindPathPoint(id)
        point.m_Type = type

        -- 刷新点的显示
        self.path_point_manager:ShowPathPoint( self.player.path_point_id)
    end,

    -- 判断是否所有 entity 回合执行完成
    isTurnMoveFinished = function(self)
        for k,v in pairs(self.entities) do
            if  v then
                if v.is_turn_moving then
                    return false
                end
            end
        end
        return true
    end,

}