local log = logger.get("Game", "Level")
local level_player = require('level.level_player')
local level_monster = require('level.level_monster')
local level_npc = require('level.level_npc')
local level_mine = require("level.level_mine")
local level_award = require("level.level_award")

require('level.level_helper')

return class {
    extensions = {"level/level+turn.lua"}, 

    ctor = function (self, scene_name, level_id)
        log.info("##### level ctor!")
        self.extensions = {}
        self.scene_name = scene_name    -- 场景名
        self.id = level_id -- 关卡 id
        self.cfg = config.game_copy_levels[level_id]
        self.chapter_id = self.cfg.Chapter  -- 章 id
        self.scene = scene_name
        self.inited = false
        self.heros = {30,1,2,3,4,5}  -- 注意，这里其实是战车 id ，不是英雄 id
        self.level_points = {}
        self.player = nil
        self.entities = {}  -- 以 index 为下标的总表，包括 monster,npc,mine,award

        self.is_tap_entity = false -- 用户是否点了一个对象

        self.is_interaction_enabled = true

        local go = GameObject.Find('LevelManager')
        if go ~= nil then 
            self.level_manager = go:GetComponent('LevelManager')    -- cs 组件，预先缓存关卡中的重要对象
            self.path_point_manager = self.level_manager.m_PathPointManager
            self.shadow_projector = self.level_manager.shadowProjector
            self.tank_root = self.level_manager.levelTankRoot
            --self.soldier_root = self.level_manager.levelSoldierRoot
        else
            log.error('No LevelManager in level:'..tostring(self.id))
        end
        level = self    -- 全局 level
        
        -- 注册事件处理函数
        self.events = require("base.event").new()

        -- actor 资源加载完成回调
        self.events:on("actor_loaded", function (actor)
            --log.info("### actor_loaded:"..actor.id)
            self.shadow_projector:AddCaster(actor.transform)

            --log.info("### actor_loaded 11111:"..actor.id)

            -- 玩家角色
            if actor.team == 0 then
                --log.info("### actor_loaded 1.5:"..actor.id)
                game_state_level:lookTarget(actor.transform)
                self.level_manager:SetFogOfWarTransform(actor.transform, config.commonvars.LEVEL_SIGHT)   -- 视野
                --actor.cs_inst:SetDrawPath(true)
            end
            --log.info("### actor_loaded 222222222:"..actor.id)
            -- 玩家初始化完成后再初始化关卡脚本
            if self:isAllEntityLoaded() then
               log.info("##################### isAllEntityLoaded trur:"..actor.id)
                app:startCoroutine(
                    function()
                        game_state_level:onEntitiesLoaded()

                        -- 等待 ui 加载完成
                        while self.gui == nil do
                            coroutine.yield(nil)
                        end
                        --log.info("### initLevelScript!")
                        self:initLevelScript()
                        self.gui:refreshItemNum()
                        self.gui:refreshHealthAndGasoline()
                        for _,e in pairs(self.entities) do
                            e:setPaused(false)
                        end
                    end
                )
            else
                log.info("############### isAllEntityLoaded false:"..actor.id)
            end
        end)
        self.events:on("actor_unloaded", function (actor)
            self.shadow_projector:RemoveCaster(actor.transform)

            if actor.team == 0 then
                self.level_manager:SetFogOfWarTransform(nil, 0)   -- 视野
            end
        end)
        
        self.events:on("entity_touch_player_evt", function (entity)
            self:onPlayerTouchEntity(entity)
        end)
        self.events:on("entity_leave_player_evt", function (entity)
            self:onPlayerLeaveEntity(entity)
        end)

        self.events:on("monster_view_player", function(monster)
            self:onMonsterViewPlayer(monster)
        end)

        self.events:on("show_formation", function(monster_id)
            self:showFormation(monster_id)
        end)

        -- 关卡中的全局 battle_helper
        battle_helper = require("level.level_script.level_battle_helper")
        battle_helper:init(self)


        app:startCoroutine(
            function()
                coroutine.yield(nil)
                self:init()
            end
        )
    end,

    -- 初始化
    init = function(self)

        -- 如果没有英雄出战信息，向服务器请求
        network_sender:reqChapterAttendHeros(self.chapter_id, self.id, true)
        --[[
        self.attend_heros = game_info.chapter_attend_heros[self.chapter_id]
        if not self.attend_heros then
            network_sender:reqChapterAttendHeros(self.chapter_id)
        end
        ]]

        if level_info.map_info then
            self:initWithMapInfo(level_info.map_info)
        else
            log.error("Failed to find level_info.map_info to init level!")
        end

        -- 恢复 PathPoint 类型
        for id,type in pairs(level_info.path_point_type) do
            local point = self.path_point_manager:FindPathPoint(id)
            point.m_Type = type
        end
        
        -- 请求英雄血量百分比
        network_sender:reqLevelHeroHpPercent()

       --[[
        if emp_manager.emp_use_count > 0 then
            emp_manager:showEmpEffect()
        end]]
    end,

    initLevelScript = function(self)
        log.info("### initLevelScript")

        if self.level_script then
            return
        end

        if level_info.backup_level_script then
            level_script = level_info.backup_level_script
            level_script.level = self
            level_script:reload()
        else
            log.info("### before dofile 1111111111111111")
            -- 创建关卡脚本, 全局表
            level_script = dofile('level/level_script/'..self.id..'.lua')(self, self.id, nil)
            log.info("### after dofile 2222222222222222222")
        end

        if self.player then
            level_script.curr_point_id = self.player.path_point_id
        end
        self.path_point_manager:ShowPathPoint(level_script.curr_point_id)

        if not level_script then
            log.error('Failed to find level script:'..self.id)
        end
        self.level_script = level_script
    end,

    -- 用服务器发的地图信息初始化
    initWithMapInfo = function(self, map_info)
        log.info('### initWithMapInfo (from server)')
        -- 出生玩家
        if map_info.plr then
            local pos = level_info.player_pos or core.decVec3(map_info.plr.posX, 0, map_info.plr.posY)
            local rot = Quaternion.AngleAxis(map_info.plr.angle * 360/255, Vector3.up)
            local playerID = map_info.plr.entityId

            local player = self:spawnPlayer(playerID, pos, rot)
            self.entities[playerID] = player

        else
            log.error('No map_info.plr')
        end

        --
        -- 出生地图 entities
        if map_info.mapEntities then
            --log.info("map entities", #map_info.mapEntities)
            for k, entity_obj in ipairs(map_info.mapEntities) do
                local entity = self:spawnMapEntity(entity_obj)
                self.entities[entity_obj.entityId] = entity

                log.info("# add entity:"..entity_obj.entityId)
            end
        else
            log.error('No map_info.mapEntities')
        end

        if map_info.enhanceMon then 
            for k, entity_id in ipairs(map_info.enhanceMon) do 
                self:notifyMonsterEnhancement(entity_id)
            end
        end

        -- 迷雾数据
        if level_info.fog_data then
            log.info("### DeserializeFog from level_info.fog_data")
            self.level_manager:DeserializeFog(level_info.fog_data)
        elseif map_info.fogOfWar and #map_info.fogOfWar > 0 then
            --LuaArray.init(map_info.fogOfWar)
            log.info("### DeserializeFog from map_info.fog_data")
            self.level_manager:DeserializeFog(map_info.fogOfWar)
        else
            log.info("### Clear Fog")
            self.level_manager:ClearFog()
        end
    end,

    -- 是否所有 entity 都加载完成
    isAllEntityLoaded = function(self)
        for _,e in pairs(self.entities) do
            if not e.is_initialized then
                return false
            end
        end

        return true
    end,

    -- entity_id 就是配置表里的 index
    getMapEntity = function(self, entity_id)
        return self.entities[entity_id]
    end,

    -- entity 消失,有表现过程
    entityDisappear = function(self, entity_id)
        local entity = self.entities[entity_id]
        if entity then 
            if self.player then self.player:removeTouchingEntity(entity) end
            entity:disappear(function() 
                self:removeMapEntity(entity_id)
            end)
        end
    end,

    removeMapEntity = function (self, entity_id)
        if self.entities[entity_id] then
            self.entities[entity_id]:destroy()
            self.entities[entity_id] = false
        end
    end, 

    spawnMapEntity = function (self, entity_obj)
        -- MapEntity
        -- 公共数据
        local pos = core.decVec3(entity_obj.posX, 0, entity_obj.posY)
        --print("spawnMapEntity:", entity_obj.entity_type, entity_obj.entityId, "pos:", entity_obj.posX, entity_obj.posY, "dec:", pos.x, pos.y, pos.z)
        local rot = Quaternion.AngleAxis(entity_obj.angle * 360 / 255, Vector3.up)
        local id = entity_obj.entityId 
        local cfg_id = entity_obj.cfgId
        local team = 1
        local entity = false   -- 创建出来的关卡对象

        local obstacle_size = Vector3.New(entity_obj.ox*0.0001, entity_obj.oy*0.0001, entity_obj.oz*0.0001)
        local entity_type = entity_obj.met  -- 'met' terrible naming !!

        entity = level_helper:spawnMapEntity(cfg_id, team, id, pos, rot, obstacle_size, self.events, self.tank_root, entity_type)
        entity.eventGroupId = entity_obj.eventGroupId   -- 事件组 id
        entity.guarderIndex = entity_obj.guarderIndex   -- 守卫 id

        self.entities[id] = entity
        return entity
    end, 

    -- 出生玩家角色
    spawnPlayer = function(self, id, pos, rot)
        local tank_cfg_id = self.heros[1] or 6 -- 注意，这里其实是战车 id ，不是英雄 id
        local team = 0
        self.player = level_player(tank_cfg_id, team, id, pos, rot, Vector3.zero, self.events, self.tank_root)

        return self.player
    end,

    -- 玩家移动到一个位置
    moveTo = function(self, pos, speed, callback)
        self.is_tap_entity = false
        if self.player then
            self.player:setBreakInput(true)
            self.player:moveTo(pos, speed, callback)
        end
    end,
   
    -- 玩家移动到一个路点
    moveToPathPoint = function(self, point, speed, callback)
       -- local point = self.path_point_manager:FindPathPoint(id)
        if point then
            self:moveTo(point.position, speed, callback)
        else
            log.error("Failed to find path point:"..id)
        end
    end,

    -- 玩家向一个场景对象移动
    moveToEntity = function(self, entity_id)
        local entity = self.entities[entity_id]

        if entity then
            self.is_tap_entity = true
            self.moving_target_id = entity_id
            if entity.touch_player then
                self:onPlayerTouchEntity(entity)
            else
                self.player:moveTo(entity:getPosition())
                entity:onPlayerMoveToMe() -- 通知 entity player 向我移动了
            end
        else
            log.warn("Failed to find entity id:"..tostring(entity_id))
        end
    end,

    -- 玩家与一个场景对象接触
    onPlayerTouchEntity = function(self, entity)
        if not self.is_interaction_enabled then
            return
        end

        self.player:addTouchingEntity(entity)
    end,

    onPlayerLeaveEntity = function(self, entity)
        self.player:removeTouchingEntity(entity)
    end,

    -- 玩家进入怪物视野
    onMonsterViewPlayer = function(self, monster)
        -- 玩家停止移动
        self.player:stopMove()

        -- 怪物向玩家移动
        monster:moveToPlayer(self.player)
    end,


    notifyMonsterEnhancement = function (self, entity_id)
        local monster = self.getEntity(entity_id)
        if monster then 
            monster:onEnhancement()
        end
    end,    

    --[[
    saveLevelInfo = function(self)
        if self.to_reset then
            return
        end

        local fogData = self.level_manager:SerializeFog()
        -- LuaArray to table
        network_sender:saveLevelFog(level_info.map_id, fogData.Table)

        if self.player then
            local pos = self.player:getPosition()
            if level.player.path_point_id then
                local point = self.path_point_manager:FindPathPoint(level.player.path_point_id)
                pos = point.position
            end

            network_sender:saveLevelPlayer(level_info.map_id, pos)
        end
    end,]]

    getEntity = function(self, id)
        return self.entities[id]    
    end,

    -- 重置关卡
    resetLevel = function(self)
        self.to_reset = true
        network_sender:resetMap(level_info.map_id)
    end,

    -- 显示布阵界面
    showFormation = function(self)
    end,

    -- 显示布阵设置界面
    showFormationSet = function(self)
    end,

    isStoryLevel = function(self)
        local is_story = (self.cfg.level_type == 2)
        return is_story
    end,

    -- 显示路点,只显示当前路点和邻居路点
    showPathPoint = function(self, id)
        if self.path_point_manager then
            self.path_point_manager:ShowPathPoint(id)
        end
    end,

    -- 使用道具
    useItem = function(self, idx)
        log.info("click icon:"..idx)
        local slot = level_info:getItemSlot(idx)
        if not slot then
            return
        end

        if slot.id == 31005 then  -- 使用汽油
            if level_info.gasoline == level_info.max_gasoline then
                hint:setText('Your tank is full')
            else
                if level_info:consumeItemByIdx(idx) then
                    level_info.gasoline = level_info.max_gasoline
                    level_frame:refreshItemNum()
                    level_frame:refreshHealthAndGasoline()
                    hint:setText('You filled your tank with gasoline, time to roll out!')
                else
                    -- 道具数量不足
                    hint:setText('There is no more fuel')
                end
            end

        elseif slot.id == 31006 then  -- 使用工具箱
            if level_info.is_gasoline_leak then
                if level_info:consumeItemByIdx(idx) then
                    --level_info.is_gasoline_leak = false
                    level_script:setGasolineLeak(false)
                    hint:setText('你用工具箱修理了漏油！')
                    level_frame:refreshItemNum()
                else
                    hint:setText('你查了下库存，发现工具箱早就用光了！')
                end
            else
                hint:setText('你检查了一下爱车，觉得暂时还不必要使用工具箱！')
            end
        elseif slot.id == 31007 then  -- 使用食物
            if level_info:consumeItemByIdx(idx) then
                network_sender:levelChangeHeroProp(1, 0)
                hint:setText('Your troops ate some food and recovered health')
                level_frame:refreshItemNum()
            else
                hint:setText('You are out of food!')
            end
        elseif slot.id == 31008 then -- 使用铲子
            if level_info:consumeItemByIdx(idx) then
               -- hint:setText('你在地上挖了个洞，什么都没发现！')
                level_frame:refreshItemNum()
                level_script:onUseShovel(game_state_level.level.player.path_point_id)
            else
                hint:setText('Out of shove')
            end
        end       
    end,

    update = function(self, dt)
        battle_helper:update(dt)
        if self.level_script then
            self.level_script:update(dt)
        end

        -- 更新所有怪物
        for _,entity in pairs(self.entities) do
            if entity and entity.entity_type == common_pb.MET_MONSTER then
                entity:update(dt)
            end
        end

        if self.player then
            self.player:update(dt)
        end
    end,

    destroy = function(self)
        if level_script then
            level_script:destroy()
            level_script = nil
        end

        if battle_helper then
            battle_helper:destroy(self)
            battle_helper = nil
        end

         for idx,entity in pairs(self.entities) do
            if entity then
                entity:destroy()
                self.entities[idx] = false
            end
         end
         level = nil
    end,

}