-- 关卡掉落的物品
local log = logger.get("Game", "LevelDropItem")
local Widget = require("ui.widget")

DROP_ITEM_PREFAB = "Assets/Prefab/Level/DropItem.prefab"

return class{
    ctor = function(self, pos, item_cfg_id, item_num)
        self.cfg = item_helper:getItemCfg(item_cfg_id)
        self.item_num = item_num
        self.pos = pos
        self:init()
        self.start_time = Time.time
        self.can_pickup = false
        self.is_pickup = false
    end,

    init = function(self)
        self.gameObject = game_director.pool:instantiate(DROP_ITEM_PREFAB)
        self.transform = self.gameObject.transform
        self.gameObject.transform.position = self.pos
        self.gameObject.transform.rotation = Quaternion.AngleAxis(math.random(0, 360), Vector3.up)

        if self.cfg.type == 8 then
            -- 武器，使用武器的模型
            self.model = game_director.pool:instantiate(self.cfg.model_resource)
        end

        local callback = function(go)
            self.icon_bg = go.transform:Find("AnimRoot/IconBG")
            self.icon = go.transform:Find("AnimRoot/IconBG/Icon")
            self.icon_bg:SetActive(false)

            self.anim_root = go.transform:Find("AnimRoot")
            local case = self.anim_root:Find("Cube")

            if self.model then
                self.model:SetParent(self.anim_root)
                level_helper:makeTransformIdentity(self.model.transform)
                case:SetActive(false)
            else
                case:SetActive(true)
            end

            go.layer = UnityEngine.LayerMask.NameToLayer("Tank")
        end

        if self.gameObject.isLoaded then
            callback(self.gameObject)
        else
            self.gameObject.onLoaded = callback
        end

        -- 动画结束后显示 icon
        app:startCoroutine(
            function()
                coroutine.yield(WaitForSeconds(2.0))
                if self.icon_bg then
                    self.icon_bg:SetActive(true)

                    -- 设置 icon 
                    local icon_img = self.icon:GetComponent(UnityEngine.UI.Image)
                    icon_img:SetSprite(gui_helper:getItemIconPath(self.cfg))

                    local bg_img = self.icon_bg:GetComponent(UnityEngine.UI.Image)
                    if self.cfg.type == 8 then
                        -- 武器设置背景品质
                        bg_img:SetSprite(gui_helper:getQualityIconPath(self.cfg.color))

                    else
                        bg_img.sprite = nil
                    end

                    self:setIconTouchEvent()
                end
                self.can_pickup = true

                local pos = self.anim_root.position
                if self.cfg.color == 4 then
                    self.sfx = level_script:createGameObject(
                        "Assets/Arts/ui/Fx/ui_prefab/Prefab_wupinguang_purple.prefab", pos.x, pos.y+1, pos.z)
                elseif self.cfg.color == 5 then
                    self.sfx = level_script:createGameObject(
                        "Assets/Arts/ui/Fx/ui_prefab/Prefab_wupinguang_yellow.prefab", pos.x, pos.y+1, pos.z)
                end

                coroutine.yield(WaitForSeconds(5.0))
                if not self.is_pickup then
                    self:pickup()
                end
            end
        )
    end,

    -- 为 icon 设置回调
    setIconTouchEvent = function(self)
        local native_obj = self.icon:GetComponent(Fenix.UI.PointerHandler) or 
                            self.icon.gameObject:AddComponent(Fenix.UI.PointerHandler)

        self.widget_wrap = Widget.new(native_obj)
        self.widget_wrap:AddClickListener(function (eventData)
               log.info("Icon Clicked!!")
               self:pickup()
        end)
    end,

    destroy = function(self)

        local screen_pos = Vector3.New(10, 10, Camera.main.transform.position.y* 0.5)
        if level_frame then
            screen_pos = frame_helper:worldToScreenPoint(level_frame.bag_button.position)
        end
        level_helper:createFlyToScreenEffect(
            "Assets/Arts/ui/Fx/ui_prefab/Prefab_bag_lizi_yellow_starAdd.prefab", self.anim_root.position, screen_pos, 1.0)

        if self.widget_wrap then
            self.widget_wrap:ClearClickListener()
            self.widget_wrap = nil
        end

        if self.model then
            self.model:destroy()
        end

        if self.gameObject then
            self.gameObject:destroy()
        end

        if self.sfx then
            self.sfx:destroy()
        end
    end,

    pickup = function(self)
        -- todo 变成光点
        if not self.is_pickup and self.can_pickup then
            self.is_pickup = true
            self:destroy()
        end
    end,

}