local log = logger.get("Game", "Formation")

level_formation = {

    events = require("base.event").new(),
    

    -- 添加阵形id
    addEnableFormations = function(self, id)
        self.enable_formations = self.enable_formations or {}
        table.insert(self.enable_formations, id)
    end,

    -- 移除阵形id
    removeEnableFormations = function(self, formation_id)
    end,

    -- 阵形id列表
    getEnableFormations = function(self)
        return self.enable_formations
    end,

    -- 是否拥有这个阵型
    isFormationIdEnable = function(self, formation_id)
        if self.enable_formations then
            for __, id in ipairs(self.enable_formations) do
                if id == formation_id then
                    return true
                end
            end
        end
        return false
    end,

    -- 清理预设阵型
    clearPresetFormations = function(self)
        self.preset_formations = {}
    end,

    -- 设置预设阵型
    setPresetFormation = function(self, id, formation)
        self.preset_formations = self.preset_formations or {}
        self.preset_formations[id] = formation
    end,

    -- 获取预设阵型
    getPresetFormations = function(self)
        return self.preset_formations
    end,

    -- battle_type 0 全都可以 1 肉盾型 2 突击型 3 坦克型 4 火炮型
    -- 获取上阵的坦克
    getTanks = function(self, chapter_id, slot_type)
        local rst = game_info.chapter_attend_heros[chapter_id] -- SCChapterAttendHeroes
        if rst.formations then
            if slot_type == 0 then
                return rst.formations
            else
                local temp = {}
                for __, tank in ipairs(rst.formations) do
                    local cfg = config.tank_basic_ids[tank.heroId]
                    if cfg.type == ("type" .. slot_type) then
                        table.insert(temp, tank)
                    end
                end
                return temp
            end 
        end
    end,

    -- 获取配置id
    getFormationId = function(self, chapter_id)
        local rst = game_info.chapter_attend_heros[chapter_id] -- SCChapterAttendHeroes
        if rst.formationId then
            return rst.formationId
        end
    end,

    -- 获得阵容的槽位类型
    getFormationCfg = function(self, cfg_id)
        local cfg_str = config.formation_lists[cfg_id].list
        return loadstring("return ".. cfg_str)()
    end,

    -- 获取阵容
    getConfig = function(self)
        local cfg = {}
        config.formation_lists:foreach(
            function(k,v)
                table.insert(cfg, v)
            end
        )
        local sort = function(a, b)
            return a.id < b.id
        end,
        table.sort(cfg, sort)
        return cfg
    end,

    -- 随机生成阵容
    randomFormation = function(self, chapter_id, cfg_id)
        local helper = require("gui.tool.frame_helper")
        local cfg = config.formation_lists[cfg_id]
        local rst = {}
        local used_tb = {}    
        local tb = loadstring('return ' .. cfg.list)()
        for slot_id, slot_type in pairs(tb) do
            local temp = { slot_id = slot_id, slot_type = slot_type, tank_cfg_id = 0 }
            local tanks = self:getTanks(chapter_id, slot_type)
            for __, t in ipairs(tanks) do
                if not used_tb[t.heroId] then
                    used_tb[t.heroId] = ""
                    temp.tank_cfg_id = t.heroId
                    break
                end
            end
            table.insert(rst, temp)
        end
        return rst
    end,

    getHerosBySlotType = function(self, hero_list, slot_type)
        if hero_list then
            if slot_type == 0 then
                local rst = {}
                for cfg_id, hero in pairs(hero_list) do
                    table.insert(rst, hero)
                end
                return rst
            else
                local rst = {}
                for cfg_id, hero in pairs(hero_list) do
                    local cfg =  hero.tank_cfg --config.tank_basic_ids[cfg_id]
                    if cfg.type == ("type" .. slot_type) then
                        table.insert(rst, hero)
                    end
                end
                return rst
            end 
        end
    end,

    -- hero_list = pairs hero.lua
    -- return MFormation
    spawnServerData = function(self, cfg_id, hero_list)
        local mf = common_pb.MFormation()
        mf.formationId = cfg_id
        local helper = require("gui.tool.frame_helper")
        local cfg = config.formation_lists[cfg_id]
        local used_heros = {}    
        local tb = loadstring('return ' .. cfg.list)()
        for slot_id, slot_type in pairs(tb) do
            local heros = self:getHerosBySlotType(hero_list, slot_type)
            for __, h in ipairs(heros) do
                if not used_heros[h.id] then
                    used_heros[h.id] = h
                    local item = mf.formationHero:add()
                    item.heroId = h.id
                    item.location = slot_id
                    -- print("spawnServerData", item.heroId, item.location)
                    break
                end
            end
        end
        return mf
    end,

    spawnEmptyServerData = function(self, cfg_id, hero_list)
        local mf = common_pb.MFormation()
        mf.formationId = cfg_id
        for cfg_id, hero in pairs(hero_list) do
            local item = mf.formationHero:add()
            item.heroId = hero.id
            item.location = 0
        end
        return mf
    end,

    -- hero_list = repeated MHeroFormation iparis
    completeFormation = function(self, formation_id, MHeroFormation_list)
        local mf = common_pb.MFormation()
        mf.formationId = formation_id

        local unused_hero_list = {}
        for __, hero in ipairs(MHeroFormation_list) do
            local item = mf.formationHero:add()
            item.heroId = hero.heroId
            item.location = hero.location > 0 and hero.location or 0
            -- if hero.location == 0 then
            --     unused_hero_list[hero.heroId] = hero_manager:getHero(hero.heroId)
            -- end
        end

        isHeroOwnSlot = function(MFormation, location)
            for __, hero in ipairs(MFormation.formationHero) do
                if hero.location == location then
                    return true
                end
            end
            return false
        end

        getUnuseHero = function(MFormation, tp)
            --[[
            for __, hero in ipairs(MFormation.formationHero) do
                if hero.location == 0 then
                    if tp == 0 then
                        return hero
                    else
                        local hero_cfg = hero_manager:getHero(hero.heroId).tank_cfg
                        if hero_cfg.type == ("type" .. tp) then
                            return hero
                        end
                    end
                end
            end
            ]]

            -- 添加优先战斗力高的英雄
            local rst_list = {}

            for __, hero in ipairs(MFormation.formationHero) do
                if hero.location == 0 then
                    local hero_lua = hero_manager:getHero(hero.heroId)
                    if not hero_lua.team_idx or hero_lua.team_idx == 0 then
                        if tp == 0 then
                            table.insert(rst_list, { hero = hero, hero_lua = hero_lua })
                        else
                            if hero_lua.tank_cfg.type == ("type" .. tp) then
                                table.insert(rst_list, { hero = hero, hero_lua = hero_lua })
                            end
                        end
                    end
                end
            end

            sortFun = function(a, b)
                return a.hero_lua.battle_force > b.hero_lua.battle_force
            end

            table.sort(rst_list, sortFun)

            if #rst_list > 0 then
                return rst_list[1].hero
            else
                return nil
            end
        end

        local fmt_cfg = self:getFormationCfg(formation_id)
        for location, tp in pairs(fmt_cfg) do
            if location ~= 0 then
                local own = isHeroOwnSlot(mf, location)
                if not own then
                    local hero = getUnuseHero(mf, tp)
                    if hero then
                        hero.location = location
                    end
                end
            end
        end

        return mf
    end,

    -- callback 返回值 SCQueryTeamSword
    getForce = function(self, MFormation, enemy_formation_id, callback)
        local id = self:genId()
        log.info("######################################################### get force id", id)
        network_sender:queryTeamSword(id, MFormation, enemy_formation_id or 0)        
        self.force_callback = self.force_callback or {}
        self.force_callback[id] = callback
    end,

    setForce = function(self, id, SCQueryTeamSword)
        log.info("######################################################### set force id", id)
        if self.force_callback and self.force_callback[id] then
            self.force_callback[id](SCQueryTeamSword)
        end
    end,

    genId = function(self)
        self.queue_id = self.queue_id or 0
        self.queue_id = self.queue_id + 1
        return self.queue_id
    end,

    validMFormation = function(self, MFormation)
        local temp_location = {}
        local formation_cfg = self:getFormationCfg(MFormation.formationId)
        local rst = true
        for __, hero in ipairs(MFormation.formationHero) do
            if hero.location > 0 then
                if temp_location[hero.location] then
                    log.error('出现重复的location', hero.location)
                    for ___, h in ipairs(MFormation.formationHero) do
                        log.error('heroId:', h.heroId, 'location:', h.location)
                    end
                    return false
                else
                    temp_location[hero.location] = hero.location
                end
            end

            if hero.location > 0 then
                local slot_type = formation_cfg[hero.location]
                local hero_cfg = hero_manager:getHero(hero.heroId).tank_cfg --config.tank_basic_ids[hero.heroId]
                if hero_cfg.type ~= ("type" .. slot_type) and slot_type ~= 0 then
                    rst = false
                    log.error('validMFormation error', hero.heroId, hero.location)
                end
            end
        end
        return rst
    end,
}

formation_tool = level_formation