-- 关卡对象状态机
local log = logger.get("Game", "LevelGameObject")

-- 待机
local state_idle = class{
    super = require('common.state'),

    onEnter = function(self, args)
        --log.info('state onEnter:'..self.name)
        self.owner:playAnimation("idle")
        self.owner:stopMove()
    end,
}

-- 沿着路径移动
local state_move_in_path = class{
    super = require('common.state'),

    onEnter = function(self, args)
        if args.resume then
            -- 恢复状态
            if self.points then
                self:move()
            end
            return
        end

        self.points = args.points
        self.speed = args.speed
        self.move_elapsed = 0.0
        self.index = 1

        self:move()
    end,

    move = function(self)
        log.info("### move:"..self.index)
        local target = self.points[self.index]
        if not target then
            log.info("## move finished!!")
            self.owner:changeState('idle')
            return
        end

        if self.owner.is_broken then
            self.owner:playAnimation("brokenrun")
        else
            self.owner:playAnimation("run")
        end

        log.info("### move index:"..self.index)
        self.owner:moveTo(
            self.points[self.index],
            self.speed,
            function()
                self.index = self.index + 1
                self:move()
            end
        )
    end,

    onUpdate = function(self, dt)
        if not self.points then
            self.owner:changeState('idle')
        end
    end,

    onExit = function(self)
    end,
}

local state_hitten = class{
    super = require('common.state'),

    onEnter = function(self, args)
        --log.info('state onEnter:'..self.name)
        self.owner:playAnimation("hurt")
        self.owner:stopMove()
        self.elapsed = 0.0
        self.owner.hit_count = self.owner.hit_count + 1  
    end,

    canChangeTo = function(self, name, args)
        if self.elapsed > 0.25 then
            return true
        end

        return false
    end,

    onUpdate = function(self, dt)
        self.elapsed = self.elapsed + dt
        if self.elapsed > 0.5 then
            self.owner:changeState('move_in_path', {resume = true})
        end
    end,
}

local state_broken = class{
    super = require('common.state'),

    onEnter = function(self, args)
        --log.info('state onEnter:'..self.name)
        self.owner:playAnimation("broken")
        self.owner:stopMove()
        self.elapsed = 0.0

    end,

    onUpdate = function(self, dt)
        self.elapsed = self.elapsed + dt
        if self.elapsed > 5.0 then
            self.owner:changeState('move_in_path', {resume = true})
        end
    end,

    -- 是否能切到状态
    canChangeTo = function(self, name, args)
        if self.elapsed > 5.0 and name == 'move_in_path' then
            return true
        end

        return false
    end,
}


return{
    moveInPath = function(self, points, speed)
        self:changeState('move_in_path', {points = points, speed = speed})
    end,

    hit = function(self)
        if self.hit_count >= 10 then
            self:changeState('broken')
            self.is_broken = true
            self.hit_count = 0
        else
            self:changeState('hitten')
        end
    end,

    -- 初始化怪物状态
    initBossStates = function(self)
        self.state_manager = require("common/state_machine")()
        self.state_manager.states["idle"] = state_idle("idle", self)
        self.state_manager.states["move_in_path"] = state_move_in_path("move_in_path", self)
        self.state_manager.states["hitten"] = state_hitten("hitten", self)
        self.state_manager.states["broken"] = state_broken("broken", self)
        
        self.is_broken = false  -- 是否被打爆
        self.hit_count = 0

        self:changeState("idle")

    end,

    changeState = function(self, state_name, args)
        self.state_manager:changeState(state_name, args)
    end,    
}