local log = logger.get("Game", "Level Helper")
local drop_item_cls = require("level.level_drop_item")

level_helper = {
    battleWithMonster = function(self, monster_id, is_sneak)
        if level_info.map_info then
            log.info('battleWithMonster mapId', level_info.map_info.mapId --[[, monster_id, #tankIDs, #soldierIDs]])

            -- 预先缓存战争迷雾和玩家位置
            game_state_level:cacheLevelInfo()
            
            network_sender:attackMonster(level_info.map_info.mapId, monster_id, is_sneak)
            game_info.attack_monster_id = monster_id
        else
            log.error('map_info is nil!!')
        end
    end,

    -- 怪物是否存在
    isMonsterExist = function(self, monster_id)
        if game_state_level and game_state_level.level then
            return game_state_level.level.monsters[monster_id]
        else
            log.error("No game state level or level !!")
        end
    end,

    -- 显示怪物详细信息对话框
    showAttackMonsterDlg = function(self, monster_id)
        network_sender:getMonsterInfo(monster.id)
    end,

    -- 关卡对象类,如果增加了关卡对象类型，需要在这里添加
    entity_class = {
        [common_pb.MET_MONSTER] = require('level.level_monster'),
        [common_pb.MET_NPC] = require('level.level_npc'),
        [common_pb.MET_MINE] = require('level.level_mine'),
        [common_pb.MET_AWARD] = require('level.level_award'),
    },

    -- 出生关卡对象
    spawnMapEntity = function(self, cfg_id, team, id, pos, rot, obstacle_size, events, root, entity_type)
        local entity_cls = self.entity_class[entity_type]
        if entity_cls then
            return entity_cls(cfg_id, team, id, pos, rot, obstacle_size, events, root, entity_type)
        else
            log.error('Failed to spawn map entity:'..cfg_id)
        end
    end,

    -- 根据类型获取 entity
    getEntitiesByType = function(self, entity_type)
        local entities = {}
        if game_state_level and game_state_level.level then
            for _,entity in pairs(game_state_level.level.entities) do
                if entity and entity.entity_type == entity_type then
                    entities[entity.id] = entity
                end
            end
        end
        return entities
    end,

    -- 创建一个掉落物
    dropItem = function(self, pos, item_cfg_id, item_num)
        local item = drop_item_cls(pos, item_cfg_id, item_num)
        item.transform:SetParent(game_state_level.level.tank_root)

        return item
    end,

    makeTransformIdentity = function(self, transform)
        if transform then
            transform.localPosition = Vector3.zero
            transform.localRotation = Quaternion.identity
            transform.localScale = Vector3.one
        end
    end,

    -- 创建飞到某坐标的特效
    createFlyToScreenEffect = function(self, prefab, from, screen_pos, total_time)

        local screenToWorld = function (pos)
            local to = Camera.main:ScreenToWorldPoint(pos)
            return to
        end

        local to = screenToWorld(screen_pos)

        --log.info("from:"..tostring(from)..", to:"..tostring(to))
        local go = game_director.pool:instantiate(prefab)
        go.transform.position = from

        local desc = LeanTween.move(go, to, 1.0)
        desc.from = from
        desc.to = to
        desc:setEase(LeanTweenType.easeInQuad)

        local start_time = Time.time
        app:startCoroutine(
            function()
                while true do
                    desc:setTo(screenToWorld(screen_pos))
                    coroutine.yield(WaitForSeconds(0.05))
                    if Time.time - start_time >= (total_time or 1.0) then
                        break
                    end
                end
                go:destroy()
            end
        )
    
    end,

    findNode = function(self, root, name)
        local node = root:Find(name)
        if node then
            return node
        else
            for i=1,root.childCount do
                local node = self:findNode(root:GetChild(i-1), name)
                if node then
                    return node
                end
            end
        end
    end,
}