local log = logger.get("Game", "level_info")

-- 缓存的关卡信息
level_info = {
    fog_data = nil, -- 战争迷雾
    player_pos = nil, -- 玩家位置
    level = nil,    -- 关卡实例
    
    map_id = 1001,      -- 关卡 id
    map_info = nil,     -- 当前关卡地图信息 SCEnterMap
    level_cfg = nil,    -- 关卡配置
    map_attack_info = nil, -- 关卡战斗请求数据    => ATTACKMONSTER

    randomEvent = nil,  -- 随机事件 ( groupId, eventId, entityIndex, remainTime, rmbTime)
    entityChanged = {},


    formation_id = 13,   -- 当前的阵型

    cur_chapter = 0,
    unload_level = {},
    unload_chapter = nil,
    level_over = false,      -- 关卡通关
    is_win = false,         -- 是否胜利通关

    is_simple_battle = false,

    backup_level_script = nil, -- 进出战斗保存的关卡脚本
    level_monster_states = {}, -- 关卡怪物状态
    is_sneak = false,          -- 是否是偷袭攻击
    is_attack_from_back = false,
    is_formationed = false,
    hero_killed = false,        -- 战斗中是否有英雄死亡 
    path_point_type = {},       -- 被修改过的路点类型

    -- 关卡中的道具类型
    item_types = 
    {
        gasoline = 31005,
        toolkit = 31006,
        food = 31007,
        shovel = 31008,
    },

    level_items = nil,      -- 关卡道具(消耗品)

    step_count = 0,         -- 步数统计
    gasoline = 20,          -- 剩余汽油量
    max_gasoline = 20,      -- 汽油最大量
    hero_hp_percent = 1.0,  -- 英雄总血量百分比 [0, 1]
    is_gasoline_leak = false,   -- 是否在漏油

    -- 获取关卡道具数量
    getItem = function(self, id)
        local num = 0
        for idx,slot in pairs(self.level_items) do
            if slot.id == id then
                num = num + slot.num
            end
        end
        return num
    end,

    getItemSlot = function(self, idx)
        return self.level_items[idx]
    end,

    -- 找道具格子的索引
    getItemIndex = function(self, item_cfg_id)
        for idx,slot in pairs(self.level_items) do
            if slot.id == item_cfg_id then
                return idx
            end
        end
        return nil
    end,

    --modified by hong feng
    addItem = function(self, item_cfg_id, num, display_num)     --display_num 用于显示+1，不然只会显示1
        for idx,slot in pairs(self.level_items) do
            if slot.id == item_cfg_id then
                slot.num = slot.num + num
                
                if display_num then
                    local item_cfg = item_helper:getItemCfg(item_cfg_id)
                    local path = gui_helper:getItemIconPath(item_cfg)
                    level.player:showIcon(path, display_num, true)
                else
                    self:showConsumeItemIcon(slot.id, num)
                end

                return
            end
        end
    end,
    --modification end

    -- 根据 id 消耗道具
    consumeItem = function(self, id)
        for idx,slot in pairs(self.level_items) do
            if slot.id == id and slot.num > 0 then
                slot.num = slot.num - 1
                self:showConsumeItemIcon(slot.id, -1)   --added by Hong Feng
                return true
            end
        end
        return false
    end,

    -- 根据索引消耗道具
    consumeItemByIdx = function(self, idx)
        local slot = self:getItemSlot(idx)
        if slot and slot.num > 0 then
            slot.num = slot.num - 1
            self:showConsumeItemIcon(slot.id, -1)
            return true
        else
            return false
        end
    end,

    -- 在玩家头上显示消耗品图标
    showConsumeItemIcon = function(self, item_cfg_id, num)
        local item_cfg = item_helper:getItemCfg(item_cfg_id)
        local path = gui_helper:getItemIconPath(item_cfg)
        level.player:showIcon(path, tostring(num), true)
    end,


    -- 关卡重置
    reset = function(self)
        self.fog_data = nil -- 战争迷雾
        self.player_pos = nil -- 玩家位置
        self.level = nil    -- 关卡实例
        
        self.map_id = 1001     -- 关卡 id
        self.map_info = nil     -- 当前关卡地图信息 SCEnterMap
        self.level_cfg = nil    -- 关卡配置
        self.map_attack_info = nil -- 关卡战斗请求数据    => ATTACKMONSTER
    
        self.randomEvent = nil  -- 随机事件 ( groupId, eventId, entityIndex, remainTime, rmbTime)
        self.entityChanged = {}
        self.resource = {}      -- 关卡内的货币 id num
    
        self.boost_info = nil  -- boost 信息 SCBoostListInfo
    
        self.cur_chapter = 0
        self.unload_level = {}
        self.unload_chapter = nil
        self.level_over = false      -- 关卡通关
        self.is_win = false
        
        self.is_simple_battle = false
        self.backup_level_script = nil -- 进出战斗保存的关卡脚本
        self.is_sneak = false          -- 是否是偷袭攻击
        self.is_attack_from_back = false
        self.is_formationed = false
        self.hero_killed = false        -- 战斗中是否有英雄死亡 

        self.level_monster_states = {}
        self.path_point_type = {}
        self.level_res = {}             -- 关卡资源
        self.step_count = 0            -- 步数统计
        self.gasoline = 20            -- 剩余汽油量
        self.max_gasoline = 20        -- 最大汽油量
        self.hero_hp_percent = 1.0    -- 英雄总血量百分比
        self.is_gasoline_leak = false
        --self.level_items = nil  -- 关卡道具(消耗品)
    end,
}