local super = require('level.level_tank')
local log = logger.get("Game", "LevelMine")

return class{
    super = super,
    
    initCfg = function(self)
        self.cfg = config.game_copy_mines[self.cfg_id]
        if self.cfg == nil then
            -- 找不到配置
            log.error('config.game_copy_mines 中找不到矿配置:', self.cfg_id)
            return
        end

        self.prefab_path = self.cfg.path .. self.cfg.Prefab
    end,

    onResLoaded = function(self, pos, rot)
        local go = self.gameObject
        if not go then
            log.error("Failed to load:"..self.prefab_path)
            return
        else
            log.info("main loaded:"..self.prefab_path)
        end

        go.name = "mine_" .. self.id
        go.layer = core.layers["Tank"]
        self.is_mine = true
        
        --log("创建了矿对象", self.id, self.cfg_id)
        local col = self.gameObject:GetComponentInChildren(UnityEngine.Collider, true)
        self.cs_inst = col.gameObject:GetComponent(LevelMonster)
        if not self.cs_inst then
            self.cs_inst = col.gameObject:AddComponent(LevelMonster)
        end
        self.transform = self.gameObject.transform
        self.transform.position = pos
        self.transform.rotation = rot
        
        self.__base = self.cs_inst
        self.cs_inst:Init(common_pb.MET_MINE, self.team, self.id, self.obstacle_size,
            function () 
                --log.info('enter 矿对象!!')
                self.touch_player = true
                self.events:dispatch("entity_touch_player_evt", self)
            end, 
            function ()
                self.touch_player = false
                self.events:dispatch("entity_leave_player_evt", self)
            end)
    end,

    -- 与玩家发生交互
    interactWithPlayer = function(self, player)
        local is_collect = false
        if self.guarderIndex ~= 0 then 
            -- 矿有守卫
            if level_helper:isMonsterExist(self.guarderIndex) then
                -- 守卫还活着，显示攻击怪物对话框
                level_helper:showAttackMonsterDlg()
            else
                -- 守卫死了，直接采矿
                network_sender:levelInteraction(level_info.map_id, self.id)
                is_collect = true
            end
        else
            network_sender:levelInteraction(level_info.map_id, self.id)
            is_collect = true
        end

    end,

    --[[
    destroy = function (self)
        self:onCollect()
        -- super.destroy(self)
    end,]] 


    -- 消失效果
    disappear = function(self, callback)
        if self.is_disappear then
            return
        end

        self.is_disappear = true
        app:startCoroutine(
            function()
                -- 播放特效
                local path = "Assets/Arts/ui/Fx/ui_prefab/Prefab_jinkuang_xiaoshi.prefab"
                local pos = self:getPosition()
                local effect = level_script:createGameObject(path, pos.x, pos.y, pos.z, 0, 1)
                battle_helper:destroyObjectAfter(effect, 3)

                coroutine.yield(WaitForSeconds(1.0))

                -- 沉入地下
                local pos = self.transform.position
                pos.y = pos.y - 5.0
                LeanTween.move(self.transform.gameObject, pos, 0.5)
                coroutine.yield(WaitForSeconds(0.5))

                if callback then
                    callback()
                end
            end
        )

    end,

    destroy = function (self)

        if self.cs_inst then
            Object.DestroyImmediate(self.cs_inst)
            self.cs_inst = nil
        end

        super.destroy(self)
    end, 
}