-- 关卡怪物状态机
local log = logger.get("Game", "LevelMonster")
--[[
-- 待机
local state_idle = class{
    super = require('common.state'),

    onEnter = function(self, args)
        --log.info('state onEnter:'..self.name)
        self.owner:playAnimation("idle")
    end,
}

-- 警戒，当玩家进入进入视野开始追赶玩家
local state_alert = class{
    super = require('common.state'),

    onEnter = function(self, args)
        --log.info('state onEnter:'..self.name)
        self.owner:playAnimation("idle")

    end,

    -- 玩家进入视野,开始追踪玩家
    onViewPlayer = function(self, player)
        player:viewByMonster(self.owner)
        self.owner:changeState("tracing_player", {player=player, auto_fight_time = 2.0, from_state = self.name})
    end,
}

-- 追赶玩家
local state_tracing_player = class{
    super = require('common.state'),

    onEnter = function(self, args)
        log.info('state onEnter:'..self.name)
        self.player = args.player
        if args.auto_fight_time then
            self.auto_fight = true
            self.auto_fight_time = args.auto_fight_time
        else
            self.auto_fight = false
            self.auto_fight_time = false
        end
        self.from_state = args.from_state or "alert"
        log.info("### from state :"..self.from_state)

        self.move_elapsed = 1.0
        self.tracing_elapsed = 0.0
    end,

    onUpdate = function(self, dt)
        self.move_elapsed = self.move_elapsed + dt
        if self.move_elapsed > 0.5 then
            self.move_elapsed = 0.0
            self.owner:moveTo(self.player:getPosition(), self.owner.speed,
                function()  -- callback
                    self.player:stopMove()
                    self.owner:changeState("interact", {player=self.player})
                end)
        end

        self.tracing_elapsed = self.tracing_elapsed + dt
        if self.auto_fight and self.auto_fight_time then
            self.owner:changeState("interact", {player=self.player})
        end
    end,
    
    reload = function(self)
        --log.info("#### reload, from_state:"..self.from_state)
        self:changeState(self.from_state)
    end,

    onExit = function(self, to_state_name)
        log.info('state onExit'..self.name)
    end,
}

-- 巡逻
local state_patrol = class{
    super = state_alert,

    onEnter = function(self, args)
        --log.info('state onEnter:'..self.name)
        if args.points then
            self.points = args.points
        end

        if args.speed then
            self.speed = args.speed
        end

        self.move_elapsed = 0.0
        self:moveToPoint(1)

    end,

    moveToPoint = function(self, point_idx)
        --log.info("### patrol moveToPoint:"..point_idx)
        self.move_elapsed = 0.0
        self.move_finished = false
        self.idx = point_idx
        if self.idx > #self.points then
            self.idx = 1
        end
        self.owner:moveTo(self.points[self.idx], self.speed, 
            function()
                self.move_finished = true
            end)
    end,

    onUpdate = function(self, dt)
        self.move_elapsed = self.move_elapsed + dt
        self.pos = self.owner:getPosition()
        if self.move_elapsed > 20  or self.move_finished then
            self:moveToPoint(self.idx+1)
        end
    end,
    
    reload = function(self)
        log.info("## partol reload, speed:"..self.speed)
        if self.pos then
            self.owner.transform.position = self.pos
            self:moveToPoint(self.idx)
        end
    end,

    onExit = function(self, to_state_name)
        --log.info('state onExit'..self.name)
    end,
}

-- 与玩家交互
local state_interact = class{
    super = require('common.state'),

    onEnter = function(self, args)

        local mon = self.owner
        local player = args.player
        local from_player = args.from_player    -- 是否玩家发起的交互

        app:startCoroutine(
            function()
                --self:showIcon("Assets/Prefab/Level/Texture/Futoro_PersonSprites.jpg")
                -- 头顶播放遇敌特效
                local path = "Assets/Arts/ui/Fx/ui_prefab/Prefab_yudijingbao.prefab"
                if not mon.head_sfx then
                    mon.head_sfx = game_director.pool:instantiate(path)
                    mon.head_sfx.transform:SetParent(mon.head_node)
                    level_helper:makeTransformIdentity(mon.head_sfx.transform)
                else
                    mon:showIcon(path)
                end

                coroutine.yield(WaitForSeconds(0.2))

                if not from_player then
                    local from = mon.transform.rotation
                    local dir = player:getPosition()-mon:getPosition()
                    dir.y = 0
                    local to = Quaternion.LookRotation(dir)
                    local start_time = Time.time
                    local total_time = 0.3
                    while true do
                        mon.transform.rotation = Quaternion.Lerp(from, to, (Time.time-start_time)/total_time)
                        
                        coroutine.yield()
                        if Time.time - start_time > total_time then
                            break
                        end
                    end
                end
                
                mon:hideIcon()

                coroutine.yield(WaitForSeconds(0.1))
                level_helper:battleWithMonster(mon.id, from_player)
                coroutine.yield(WaitForSeconds(0.1))
            end
        )
    end,

    onUpdate = function(self, dt)
    end,

    canChangeTo = function(self, state_name, args)
        return true
    end,
    
    reload = function(self)
        local patrol_state = self.owner.state_manager.states["patrol"]
        if patrol_state.points then
            self.owner:changeState("patrol", {})
        else
            self.owner:changeState("alert")
        end
    end,

    onExit = function(self, to_state_name)
        self.owner:stopMove()
    end,
}

-- 消失 
local state_disappear = class{
    super = require('common.state'),

    onEnter = function(self, args)
        local callback = args.callback
        local mon = self.owner
        
        app:startCoroutine(
            function()
                coroutine.yield(WaitForSeconds(0.5))

                -- 播放特效
                local path = "Assets/Arts/fx8/1/prefab_guanqia/baozha.prefab"
                local pos = mon:getPosition()
                local effect = level_script:createGameObject(path, pos.x, pos.y, pos.z, 0, 1)
                battle_helper:destroyObjectAfter(effect, 3)

                -- 播放动画
                mon:playAnimation('die')

                coroutine.yield(WaitForSeconds(1.0))

                local pos = mon.transform.position
                pos.y = pos.y - 10
                LeanTween.move(mon.transform.gameObject, pos, 1)

                coroutine.yield(WaitForSeconds(1))

                if callback then
                    callback()
                end
            end
        )
    end,

    onUpdate = function(self, dt)

    end,

    canChangeTo = function(self, state_name, args)
        return false
    end,

    onExit = function(self, to_state_name)
        --log.info('state onExit'..self.name)
    end,
}

return{
    -- 初始化怪物状态
    initStates = function(self)
        if level_info.level_monster_states[self.id] then
            self.state_manager = level_info.level_monster_states[self.id]
            for _,state in pairs(self.state_manager.states) do
                state.owner = self
            end

            if self.state_manager.curr_state.reload then
                self.state_manager.curr_state:reload()
            end
        else
            self.state_manager = require("common/state_machine")()
            self.state_manager.states["idle"] = state_idle("idle", self)
            self.state_manager.states["alert"] = state_alert("alert", self)
            self.state_manager.states["tracing_player"] = state_tracing_player("tracing_player", self)
            self.state_manager.states["patrol"] = state_patrol("patrol", self)
            self.state_manager.states["interact"] = state_interact("interact", self)
            self.state_manager.states["disappear"] = state_disappear("disappear", self)

            self:changeState("idle")
            level_info.level_monster_states[self.id] = self.state_manager
        end
    end,

    changeState = function(self, state_name, args)
        self.state_manager:changeState(state_name, args)
    end,
}]]
