-- 关卡怪物状态机
local log = logger.get("Game", "LevelMonster")

-- 待机
local state_idle = class{
    super = require('common.state'),

    onEnter = function(self, args)
        --log.info('state onEnter:'..self.name)
        self.owner:playAnimation("idle")
    end,
}

-- 巡逻状态
local state_order_patrol = class{
    super = require('common.state'),

    onEnter = function(self, args)
        --log.info('state onEnter:'..self.name)
        if args.points then
            self.points = args.points
        end

        if args.speed then
            self.speed = args.speed
        end

        self.path_point_manager = game_state_level.path_point_manager
        self.move_elapsed = 0.0
        self.index = 1
    end,

    -- 更新回合
    updateTurn = function(self)
        self.index = self.index + 1
        if self.index > #self.points then
            self.index = 1
        end

        self.owner.path_point_target = self.points[self.index]

    end,

    moveToPoint = function(self, point_idx)
    end,

    onUpdate = function(self, dt)
    end,
    
    reload = function(self)
        local point_id = self.points[self.index]
        local curr_point = game_state_level:getPathPoint(point_id)

        self.owner.transform.position = curr_point.position
    end,

    onExit = function(self, to_state_name)
        log.info('state onExit'..self.name)
    end,
}

-- 巡逻状态
local state_patrol = class{
    super = require('common.state'),

    onEnter = function(self, args)
        --log.info('state onEnter:'..self.name)
        if args.points then
            self.points = args.points
        end

        if args.speed then
            self.speed = args.speed
        end

        self.path_point_manager = game_state_level.path_point_manager
        self.move_elapsed = 0.0

    end,

    -- 更新回合
    updateTurn = function(self)
        local curr_point_id = self.owner.path_point_id  -- 当前点
        local curr_point = game_state_level:getPathPoint(curr_point_id)

        -- 找到 points 中是当前点邻居的点
        local neighbors = {}
        table.insert(neighbors, curr_point_id)
        for _,id in ipairs(self.points) do
            if curr_point:IsNeighbor(id) then
                table.insert(neighbors, id)
            end
        end

        -- 从中随机一个点
        local idx = math.random(1, #neighbors)
        local target_id = neighbors[idx]

        self.owner.path_point_target = target_id

    end,

    onUpdate = function(self, dt)
    end,
    
    reload = function(self)
     end,

    onExit = function(self, to_state_name)
        log.info('state onExit'..self.name)
    end,
}

-- 与玩家交互
local state_interact = class{
    super = require('common.state'),

    onEnter = function(self, args)

        local mon = self.owner
        local player = args.player
        local from_player = args.from_player    -- 是否玩家发起的交互

        app:startCoroutine(
            function()
                --self:showIcon("Assets/Prefab/Level/Texture/Futoro_PersonSprites.jpg")
                -- 头顶播放遇敌特效
                local path = "Assets/Arts/ui/Fx/ui_prefab/Prefab_yudijingbao.prefab"
                if not mon.head_sfx then
                    mon.head_sfx = game_director.pool:instantiate(path)
                    mon.head_sfx.transform:SetParent(mon.head_node)
                    level_helper:makeTransformIdentity(mon.head_sfx.transform)
                else
                    mon:showIcon(path)
                end

                coroutine.yield(WaitForSeconds(0.2))

                if not from_player then
                    local from = mon.transform.rotation
                    local dir = player:getPosition()-mon:getPosition()
                    dir.y = 0
                    local to = Quaternion.LookRotation(dir)
                    local start_time = Time.time
                    local total_time = 0.3
                    while true do
                        mon.transform.rotation = Quaternion.Lerp(from, to, (Time.time-start_time)/total_time)
                        
                        coroutine.yield()
                        if Time.time - start_time > total_time then
                            break
                        end
                    end
                end
                
                mon:hideIcon()

                coroutine.yield(WaitForSeconds(0.1))
                level_helper:battleWithMonster(mon.id, from_player)
                coroutine.yield(WaitForSeconds(0.1))
            end
        )
    end,

    onUpdate = function(self, dt)
    end,

    canChangeTo = function(self, state_name, args)
        return true
    end,
    
    reload = function(self)
        local patrol_state = self.owner.state_manager.states["patrol"]
        if patrol_state.points then
            self.owner:changeState("patrol", {})
        else
            patrol_state = self.owner.state_manager.states["order_patrol"]
            if patrol_state.points then
                self.owner:changeState("order_patrol", {})
            end
        end


    end,

    onExit = function(self, to_state_name)
        self.owner:stopMove()
    end,
}

-- 消失 
local state_disappear = class{
    super = require('common.state'),

    onEnter = function(self, args)
        local callback = args.callback
        local mon = self.owner

        app:startCoroutine(
            function()
                coroutine.yield(WaitForSeconds(0.5))
                -- 播放特效
                --[[                local path = "Assets/Arts/fx8/1/prefab_guanqia/baozha.prefab"
                local pos = mon:getPosition()
                local effect = level_script:createGameObject(path, pos.x, pos.y, pos.z, 0, 1)
                battle_helper:destroyObjectAfter(effect, 3)]]

                -- 播放动画
               -- mon:playAnimation('die')

                coroutine.yield(WaitForSeconds(1.0))
                local pos = mon.transform.position
                pos.y = pos.y - 3
                LeanTween.move(mon.transform.gameObject, pos, 1)

                coroutine.yield(WaitForSeconds(1))

                if callback then
                    callback()
                end
            end
        )
    end,

    onUpdate = function(self, dt)

    end,

    canChangeTo = function(self, state_name, args)
        return false
    end,

    onExit = function(self, to_state_name)
        --log.info('state onExit'..self.name)
    end,
}


-- 隐藏
local state_hide = class{
    super = require('common.state'),

    onEnter = function(self, args)
        log.info('state onEnter'..self.name)
        self.owner.gameObject:SetActive(false)
        self.owner.is_hide = true
    end,

    reload = function(self)
        self.owner.gameObject:SetActive(false)
        self.owner.is_hide = true
    end,

    onUpdate = function(self, dt)

    end,

    canChangeTo = function(self, state_name, args)
        return true
    end,

    onExit = function(self, to_state_name)
        log.info('state onExit'..self.name)
        self.owner.gameObject:SetActive(true)
        self.owner.is_hide = false
    end,
}

return{
    -- 初始化怪物状态
    initStates = function(self)
        if level_info.level_monster_states[self.id] then
            self.state_manager = level_info.level_monster_states[self.id]
            for _,state in pairs(self.state_manager.states) do
                state.owner = self
            end

            if self.state_manager.curr_state.reload then
                self.state_manager.curr_state:reload()
            end
        else
            self.state_manager = require("common/state_machine")()
            self.state_manager.states["idle"] = state_idle("idle", self)
            self.state_manager.states["patrol"] = state_patrol("patrol", self)
            self.state_manager.states["order_patrol"] = state_order_patrol("order_patrol", self)
            self.state_manager.states["interact"] = state_interact("interact", self)
            self.state_manager.states["disappear"] = state_disappear("disappear", self)
            self.state_manager.states["hide"] = state_hide("hide", self)

            self:changeState("idle")
            level_info.level_monster_states[self.id] = self.state_manager
        end
    end,

    changeState = function(self, state_name, args)
        self.state_manager:changeState(state_name, args)
    end,
}
