local super = require('level.level_tank')
local log = logger.get("Game", "LevelMonster")

return class{
    super = super,
    extensions = { "level/level_monster+turn_states.lua"},

    initCfg = function (self)
        self.group_cfg = config.monster_group_ids[self.cfg_id]  -- 怪物战车组配置
        if not self.group_cfg then
            log.error("Failed to find group cfg:"..tostring(self.cfg_id))
        end

       -- log.info("##captain:"..self.group_cfg.captain)
       -- local captain_cfg = config.monster_stat_ids[self.group_cfg.captain] -- 单个怪物配置 
        self.tank_cfg_id = self.group_cfg.captain   -- 洪峰要求 captain 是 tank 表 id
        self.cfg = config.tank_basic_ids[self.tank_cfg_id]      -- 坦克基础配置

        if self.cfg == nil then
            -- 找不到配置
            log.error('config.monster_group_ids 中找不到战车配置:'..tostring(self.cfg_id))
            return
        end
        self.speed = 100
        self.acceleration = 100
        self.prefab_path = self.cfg.show_prefab --prefab2

        self.weapon_id = self.cfg.default_weapon_id
        self.wheel_id = self.cfg.initial_wheel_id

        self.move_to_elapsed = 0.0
        self.emp_duration = -1.0

    end, 

    onResLoaded = function(self, pos, rot)
        -- 初始化 CS 对象
        self.cs_inst = self.gameObject:GetComponentInChildren("Tank", true)
        if not self.cs_inst then
            log.error("Failed to find Tank Component in level monster:"..tostring(self.prefab_path))
            return
        end
        
        self.transform = self.cs_inst.transform

        self.transform.position = pos
        self.transform.rotation = rot
        self.cs_inst:Init(self.team, self.id, 0, true)
        self.cs_inst.gameObject:SetActive(true)
        self.cs_inst:SetAvoidancePriority(40)   -- 关卡坦克不希望被推动

        --self.cs_inst.enabled = false

        self.touch_player = false   -- 是否和玩家接触

        self.cs_monster = self.cs_inst:GetComponent(LevelMonster) 
        if not self.cs_monster then
            self.cs_monster = self.cs_inst:AddComponent(LevelMonster)
        end

        self.cs_monster:Init(self.entity_type, self.team, self.id, self.obstacle_size,
            function () 
                log.info('Lua enter trigger!!')

                if self.gameObject.activeInHierarchy then
                    self.touch_player = true
                    self.events:dispatch("entity_touch_player_evt", self)
                end
            end, 
            function ()
                if self.gameObject.activeInHierarchy then
                    self.touch_player = false
                    self.events:dispatch("entity_leave_player_evt", self)
                end
            end)

        self.emp_duration = -1.0

        self:initMonsterIcon()
        --self:initFeetHalo("Assets/Arts/ui/Fx/ui_prefab/Prefab_guaiwuquan_red.prefab")
       -- self:initHeadInfo()
        self:initStates()

        -- 初始化视野 
        self:initViewTrigger()
        --self:initSneakTrigger()
    end,

    -- 初始化头顶 icon
    initMonsterIcon = function(self)
        self.head_node = self.cs_inst:FindChild('head')
        if not self.head_node then
            return
        end

        self.monster_icon = self.cs_inst:FindChild("monster_icon")

        --[[
        if self.monster_icon then
            self.monster_icon.transform:SetActive(true)
            local text = self.monster_icon.gameObject:GetComponentInChildren(UnityEngine.UI.Text)
            text.text = "Lv. "..self.group_cfg.Group_Level

            self:setFormationIcon(self.group_cfg.formation)
        else
            local callback = function (go)
                -- callback
                if go ~= nil then
                    go.name = "monster_icon"
                    go.transform.position = self.head_node.position
                    go.transform.rotation = self.head_node.rotation
                    go.transform:SetParent(self.head_node)

                    --local text = go:GetComponentInChildren(UnityEngine.UI.Text)
                    --text.text = "Lv. "..self.group_cfg.Group_Level

                    self:setFormationIcon(self.group_cfg.formation)

                    app:startCoroutine(function()
                        coroutine.yield(nil)
                        go.transform.localScale = Vector3(1,1,1)
                    end)
                else
                    log.error('initIcon loadResource go is nil')
                end
            end

            local prefab_path = "Assets/Prefab/Level/MonsterIcon.prefab"

            self.monster_icon = game_director.pool:instantiate(prefab_path)
            if not self.monster_icon then
                log.error('Failed to load monster icon:'..prefab_path)
                return
            end

            if self.monster_icon.isLoaded then 
                callback(self.monster_icon)
            else 
                self.monster_icon.onLoaded = callback
            end
        end
        ]]
    end,

    bindToHeadPoint =  function (self, obj, rot)
        local rot = rot or {0,0,0}
        return self.cs_inst:BindToHead(obj, Vector3(rot[1], rot[2], rot[3]))
    end, 

    -- 头部血条
    initHeadInfo = function(self)
        self.head_info_go = battle_helper:loadPrefab(consts.assets.hp_slider_T1,
            function(go)
                self.head_info_go = go
                self.head_info_go:Reset()
                self.head_info_go:ScaleTo(0.1)
                self:bindToHeadPoint(self.head_info_go)
                self.head_info = self.head_info_go:GetComponentInChildren('UIHeadInfo');
                self.head_info.m_ActorID = self.id
                self.head_info:SetHPSliderValue(1)
                self.head_info:SetEnergySliderValue(0.0)
                self.head_info:SetEnable(false)  
                self.head_info_go:SetActive(false)
            end
        )
     
    end,

    setFormationIcon = function(self, formation_id)
        local path = nil
        if formation_id < 4 then
            path = "Assets/Arts/ui/atlas/icon/vehicle/type1.png"
        elseif formation_id < 7 then
            path = "Assets/Arts/ui/atlas/icon/vehicle/type2.png"
        elseif formation_id < 10 then
            path = "Assets/Arts/ui/atlas/icon/vehicle/type3.png"
        elseif formation_id < 13 then
            path = "Assets/Arts/ui/atlas/icon/vehicle/type4.png"
        else
            path = nil
        end

        local icon_trans = level_helper:findNode(self.monster_icon.transform, "Icon")--level_script:findChild(self.monster_icon.transform, "Icon")
        if icon_trans then
            local img = icon_trans:GetComponent(UnityEngine.UI.Image)
            if path then
                img:SetSprite(path)
            else
                img.gameObject:SetActive(false)
            end
            --log.info("set sformation:"..formation_id..", path:"..path)
        else
            log.error("Failed to find monster icon!")
        end

    end,

    -- 显示头顶图标
    showIcon = function(self, icon_path)
        self.monster_icon.gameObject:SetActive(false)
        super.showIcon(self, icon_path)
    end,

    -- 重写，不创建副武器
    initMinorWeapons = function(self)
    end,

    initViewTrigger = function(self)
        local obj = battle_helper:loadPrefab(
            "Assets/Prefab/Level/ViewTrigger.prefab", 
            function(go)
                local view_trigger = go:GetComponentInChildren(ViewTrigger)
                view_trigger:RegisterViewCallback(function(player) self:onViewPlayer(player) end)
            end
        )
        obj.transform.position = self.transform.position
        obj.transform.rotation = self.transform.rotation
        obj.transform:SetParent(self.transform)
        obj.transform.localPosition = Vector3.New(0, 1, 6)
        obj.transform.localScale = Vector3.New(20, 1, 30)
    end,

    --hong feng
    --[[
    initSneakTrigger = function(self)
        local obj = battle_helper:loadPrefab(
            "Assets/Prefab/Level/SneakTrigger.prefab", 
            function(go)
                local sneak_trigger = go:GetComponentInChildren(ViewTrigger)
                sneak_trigger:RegisterViewCallback(function(player) self:onViewPlayer(player) end)
            end
        )
        obj.transform.position = self.transform.position
        obj.transform.rotation = self.transform.rotation
        obj.transform:SetParent(self.transform)
        obj.transform.localPosition = Vector3.New(-0.15, 1, -23)
        obj.transform.localScale = Vector3.New(26, 1, 27)
    end,]]
    --hong feng


    onViewPlayer = function(self, player)
        log.info("onViewPlayer: "..player.m_ID)
        if self.state_manager.curr_state.onViewPlayer then
            local player_entity = battle_helper:getEntity(player.m_ID)
            if player_entity then
                self.state_manager.curr_state:onViewPlayer(player_entity)
            else
                log.error("Failed to find entity by id :"..player.m_ID)
            end
        end
    end,

    -- 判断怪物是否被隐藏
    isHide = function(self)
        if self.state_manager and self.state_manager.curr_state then
            return self.state_manager.curr_state.name == 'hide'
        else
            return false
        end
    end,

    onEnhancement = function (self)
        if not self.enhancement then 
            -- 这个特效路径已经失效了
            self.enhancement = game_director.pool:instantiate("Assets/Arts/fx/prefab/xh_jc.prefab")
            -- pos.y = self.transform.position.y
            self.enhancement.parent = self.transform
            log.info("level_monster.onEnhancement", self.id)
        end
    end, 

    -- 与玩家发生交互
    interactWithPlayer = function(self, player)
        return self:changeState("interact", {player = player, from_player = true})
    end,

    playAnimation = function(self, anim)
        self.cs_inst:PlayAnimation(anim)
    end,

    -- 开始追踪玩家 auto_fight_time 隔多久固定发生战斗，nil 表示无固定战斗
    beginTracePlayer = function(self, player, auto_fight_time)
        self:changeState("tracing_player", {player = player, auto_fight_time= auto_fight_time})
    end,

    -- 停止追踪玩家
    stopTracePlayer = function(self)
        self:changeState("idle")
    end,

    -- 开始巡逻
    beginPatrol = function(self, points, speed)
        self:changeState("patrol", {points=points, speed=speed})
    end,

    -- 开始顺序点巡逻
    beginOrderPatrol = function(self, points, speed)
        self:changeState("order_patrol", {points=points, speed=speed})
    end,

    preUpdateTurn = function(self, starts, ends)
        if self.state_manager.curr_state.updateTurn then
            self.state_manager.curr_state:updateTurn()
        end

        -- 记录起点，终点
        starts[self.id] = self.path_point_id
        ends [self.id] = self.path_point_target
    end,

    -- 
    update = function(self, dt)
        super.update(self, dt)

        -- 更新状态机 
        if self.state_manager then
            self.state_manager:update(dt)
        end

        if self.emp_duration > 0.0 then
            self.emp_duration = self.emp_duration - dt
            if self.emp_duration <= 0.0 then
                -- emp 结束
                self:endEmp()
                self.emp_duration = -1.0
            end
        end
    end,

    -- 受到 emp 影响 开始
    beginEmp = function(self, duration)

        log.info("level monster beginEmp!! id:"..self.id..' duration:'..duration)
        self.emp_duration = duration

        if not self.emp_effect then
                          
            local path = "Assets/Arts/ui/Fx/ui_prefab/prefab_EMP/Prefab_EMP_chetitanhuan.prefab"
            local pos = self:getPosition()
            self.emp_effect = 
                level_script:createGameObject(path, pos.x, pos.y, pos.z, 0, 1)
            self.emp_effect.transform.rotation = self.transform.rotation
            --log.info("## crete emp effect:"..tostring(self.emp_effect)..' pos:'..tostring(pos))
        end

        --if self.cs_tank then
        --    self.cs_tank:PlayAnimation("hitten")
        --end
    end,

    -- emp 结束
    endEmp = function(self)
        if self.emp_effect then
            self.emp_effect:destroy()
            self.emp_effect = nil
            --log.info("## destroy emp_effect!!")
        end
    end,

    -- 消失效果
    disappear = function(self, callback)
        self:changeState("disappear", {callback = callback})
    end,

    destroy = function (self)
        if self.enhancement then 
            self.enhancement:force_destroy()
            self.enhancement = nil
        end
        if self.cs_monster then 
            Object.DestroyImmediate(self.cs_monster)
            self.cs_monster = nil
        end

        if self.emp_effect then
            self.emp_effect:destroy()
            self.emp_effect = nil
        end

        if self.monster_icon then
            self.monster_icon.transform:SetActive(false)
            self.monster_icon = nil
        end

        if self.head_sfx then
            self.head_sfx:destroy()
            self.head_sfx = nil
        end

        super.destroy(self)
    end, 
}

