local super = require('level.level_monster')
local log = logger.get("Game", "LevelMonster")

return class{
    super = super,
    
    initCfg = function (self)
        self.cfg = config.game_copy_npc_npcs[self.cfg_id]
        --self.cfg = config.game_copy_random_events[self.cfg_id]
        if self.cfg == nil then
            -- 找不到配置
            log.error('config.game_copy_random_events 中找不到配置:'..tostring(self.cfg_id))
            return
        end
        self.prefab_path = self.cfg.prefab_path..self.cfg.prefab_name
        --log.error("wtf???", self.prefab_path, self.entity_type)
    end, 

    onResLoaded = function(self, pos, rot)
        local go = self.gameObject
        if not go then
            return
        end
        go.name = "npc_" .. self.id
        go.layer = core.layers["Tank"]
        log.info("创建了 NPC 对象", self.id, self.cfg_id)
        local col = self.gameObject:GetComponentInChildren(UnityEngine.Collider, true)
        if not col then
            log.error("Failed to find collider in npc!")
            return
        end
        self.cs_tank = self.gameObject:GetComponentInChildren("Tank", true)
        self.cs_inst = self.cs_tank
        if self.cs_inst then
            self.cs_inst.enabled = false
        end

        self.__base = self.cs_inst
        self.cs_monster = col.gameObject:AddComponent(LevelMonster)
        self.cs_inst = self.cs_inst or self.cs_monster

        self.transform = self.cs_monster.transform
        self.transform.position = pos
        self.transform.rotation = rot

        self.cs_monster:Init(common_pb.MET_NPC, self.team, self.id, self.obstacle_size,
            function () 
                self.touch_player = true
                self.events:dispatch("entity_touch_player_evt", self)
            end, 
            function ()
                self.touch_player = false
                self.events:dispatch("entity_leave_player_evt", self)
            end)

        self:initNpcIcon()
        self:initFeetHalo("Assets/Arts/ui/Fx/ui_prefab/Prefab_guaiwuquan_blue.prefab")
    end,

    -- 初始化头顶 icon
    initNpcIcon = function(self)
        self.head_node = self.cs_inst:FindChild('head')
        if not self.head_node then
            return
        end

        local callback = function (go)
            -- callback
            if go ~= nil then
                go.name = "npc_icon"
                go.transform.position = self.head_node.position
                go.transform.rotation = self.head_node.rotation
                go.transform:SetParent(self.head_node)
                local text = go:GetComponentInChildren(UnityEngine.UI.Text)
                text.text = frame_helper:getLangCfg(self.cfg.name_id)
            else
                log.error('initHeadIcon loadResource go is nil')
            end
        end

        local prefab_path = "Assets/Prefab/Level/NpcIcon.prefab"

        self.npc_icon = game_director.pool:instantiate(prefab_path)
        if not self.npc_icon then
            log.error('Failed to load level_tank:'..prefab_path)
            return
        end

        if self.npc_icon.isLoaded then 
            callback(self.npc_icon)
        else 
            self.npc_icon.onLoaded = callback
        end
    end,

    -- 与玩家发生交互
    interactWithPlayer = function(self, player)
        -- todo 显示 npc 对话
        frame_manager:show("gamble")
    end,

    destroy = function (self)
        if self.cs_monster then 
            Object.DestroyImmediate(self.cs_monster)
            self.cs_monster = nil
        end

        if self.npc_icon then
            Object.DestroyImmediate(self.npc_icon)
        end
        super.destroy(self)
    end, 

}