      -- 关卡怪物状态机
local log = logger.get("Game", "LevelPlayer")

-- 玩家控制的状态
local state_user_control = class{
    super = require('common.state'),

    onEnter = function(self, args)
        log.info('state onEnter:'..self.name)
        self.owner:setBreakInput(false)
        self.owner:playAnimation("idle")

    end,

    canChangeTo = function(self, state_name, args)
        return true
    end,

    onExit = function(self)
        log.info('state onExit:'..self.name)

    end,
}

--偷袭怪物
local state_sneak_attack = class{
    super = require('common.state'),

    onEnter = function(self, args)
        log.info('state onEnter:'..self.name)
        self.owner:setBreakInput(true)  -- 中断操作
        self.owner:stopMove()
        self.monster = args.monster
        self.elapsed = 0.0

        self.onMoveFinished = function()
            app:startCoroutine(
                function()
                    if self.is_rush_finishend then
                        return
                    end

                    self.is_rush_finishend = true

                    -- 播放怪物受击特效
                    local vfx = battle_helper:loadPrefab("Assets/Arts/fx/prefab/assult_rush.prefab")
                    local node = self.owner.cs_inst:FindChild("rush")
                    vfx.transform.position = node.position
                    vfx.transform.rotation = node.rotation
                    vfx.transform:SetParent(node)
                   -- vfx.transform.localScale = Vector3(10,10,10)

                   if self.monster.group_cfg.is_sneak_battle == 1 then

                        local tab = loadstring("return "..self.monster.group_cfg.sneak_hp_lost)()
                        local lost = math.random(tab[1], tab[2]) / 10000

                        self.monster.head_info_go:SetActive(true)
                        self.monster.head_info:SetEnable(true)

                        for i=1,10 do
                            coroutine.yield(WaitForSeconds(0.05))
                            self.monster.head_info:SetHPSliderValue(1-lost + (10-i) * 0.1 * lost)    
                        end
                    else
                        coroutine.yield(WaitForSeconds(0.5))
                    end

                    coroutine.yield(WaitForSeconds(0.1))
                    self.monster:interactWithPlayer(self)
                end
            )
        end

        local to = self.monster:getPosition() - self.owner:getPosition()
        local len = Vector3.Magnitude(to)
        to:Normalize()
        to = to * (len - 7)
        to = self.owner:getPosition() + to
        self.owner:moveTo(to, 200, self.onMoveFinished)
        self.owner.cs_tank:SetMoveParam(200, 2000, 3600, 1.0) -- 移动速度，加速度，旋转速度，操控缩放

        self.monster:changeState('idle')
        self.monster:stopMove()

        -- 播放冲锋特效
        local vfx = battle_helper:loadPrefab("Assets/Arts/fx/prefab/opening_assult.prefab")
        local node = self.owner.cs_inst:FindChild("rush")
        vfx.transform.position = node.position
        vfx.transform.rotation = node.rotation
        vfx.transform:SetParent(node)
        vfx.transform.localScale = Vector3.one
    end,

    onUpdate = function(self, dt)
        self.elapsed = self.elapsed + dt
        if self.elapsed > 0.5 then
            self.elapsed = -100
            self.onMoveFinished()
        end

        --[[
        self.move_elapsed = self.move_elapsed + dt
        if self.move_elapsed > 0.5 then
            self.move_elapsed = 0.0
            self.owner:moveTo(self.player:getPosition(), self.owner.speed,
                function()  -- callback
                    self.player:stopMove()
                    self.owner:changeState("interact", {player=self.player})
                end)
        end

        self.tracing_elapsed = self.tracing_elapsed + dt
        if self.auto_fight and self.auto_fight_time then
            self.owner:changeState("interact", {player=self.player})
        end]]
    end,
    
    canChangeTo = function(self, state_name, args)
        return false
    end,

    onExit = function(self, to_state_name)
        log.info('state onExit'..self.name)
    end,
}

local state_view_by_monster = class{
    super = require('common.state'),
    
    onEnter = function(self, args)
        log.info('state onEnter:'..self.name)
        self.monster = args.monster
        self.elapsed = 0.0
        self.over_time = false
        self.owner.cs_tank:SetMoveParam(config.commonvars.LEVEL_SPEED*0.2, -1, 270, 0.1)
    end,

    onUpdate = function(self, dt)
        self.elapsed = self.elapsed + dt
        if self.elapsed > 5.0 then
            self.over_time = true
            self.owner:changeState("user_control")
        end
    end,
    
    canChangeTo = function(self, state_name, args)
        return self.over_time
    end,

    onExit = function(self, to_state_name)
        log.info('state onExit'..self.name)
        self.owner:resetSpeed()
    end,   
}

-- 与entity交互
local state_interact = class{
    super = require('common.state'),

    onEnter = function(self, args)

    end,

    onUpdate = function(self, dt)
    end,

    canChangeTo = function(self, state_name, args)
        return true
    end,

    onExit = function(self, to_state_name)
        self.owner:stopMove()
    end,
}

return{
    -- 初始化玩家状态
    initStates = function(self)
        self.state_manager = require("common/state_machine")()
        self.state_manager.states["user_control"] = state_user_control("user_control", self)
        self.state_manager.states["sneak_attack"] = state_sneak_attack("sneak_attack", self)
        self.state_manager.states["view_by_monster"] = state_view_by_monster("view_by_monster", self)
        self.state_manager.states["interact"] = state_interact("interact", self)

        self:changeState("user_control")
    end,

    changeState = function(self, state_name, args)
        self.state_manager:changeState(state_name, args)
    end,
}
