local super = require('level.level_tank')
local log = logger.get("Game", "LevelPlayer")

return class {
    super = super,
    extensions = { "level/level_player+states.lua"},

    initCfg = function (self)
        self.cfg = config.tank_basic_ids[self.cfg_id]
        if self.cfg == nil then
            -- 找不到配置
            log.error('config.tank_basic_ids 中找不到战车配置:'..tostring(self.cfg_id))
            return
        end
        self.speed = self.cfg.speed
        self.acceleration = self.speed
        if self.team == 1 then
            self.prefab_path = self.cfg.show_prefab --self.cfg.prefab2--show_prefab
        else
            self.prefab_path = self.cfg.show_prefab --self.cfg.prefab2 --show_prefab
        end

        self.weapon_id = self.cfg.default_weapon_id
        self.wheel_id = self.cfg.initial_wheel_id
    end, 

    onResLoaded = function(self, pos, rot)
        log.info('#### Level Player onResLoaded, pos:'..tostring(pos))

        -- 初始化 CS 对象
        self.gameObject:SetLayer(core.layers["Tank"])

        self.cs_tank = self.gameObject:GetComponentInChildren("Tank", true)
        self.cs_inst = self.cs_tank
        self.cs_inst.gameObject:SetActive(true)
        self.cs_inst.enabled = true
        
        self.transform = self.cs_tank.transform
        -- print(self.transform, self.cs_tank)
        self.transform.position = pos
        self.transform.rotation = rot
        self.cs_tank:SetMoveParam(config.commonvars.LEVEL_SPEED, 60, 270, 0.5) -- 移动速度，加速度，旋转速度，操控缩放
        self.animation = self.cs_tank.m_Animation
        self.shoot_point = self.cs_tank.m_ShootPoint -- 射击点

        self.stop_elapsed = 1.0 -- 停了多久

        --self:initHeadLight()

        self.cs_inst:Init(self.team, self.id, 0, false)
        self.cs_inst:RegisterJoystickMove()

        self:initStates()

        local cam = game_state_level.level_camera
        if cam then
            cam:SetLookTarget(self.transform)
        else
            log.error('Failed to find LevelCamera!')
        end

        self.touching_entities = {} -- 当前与玩家接触的 entity
        self.view_by_monster = false -- 是否被敌人发现 
        self.operation_trigger = nil -- 与玩家接触的 操作区域，操作优先级高于 entity

        self:refreshInteractButton()
    end,

    onWheelLoaded = function(self, go)
        log.info("## onWheelLoaded!!")

        if not go then
            return
        end
        
        local attach_point_1 = level_helper:findNode(go.transform, "foot_1")
        local attach_point_2 = level_helper:findNode(go.transform, "foot_2")
        self.foot_1 = attach_point_1
        self.foot_2 = attach_point_2

        if attach_point_1 then
            self.tail_1 = 
                game_director.pool:instantiate("Assets/Arts/fx/prefab/Vehicle_tail.prefab")
            self.tail_1.transform.position = attach_point_1.position
            self.tail_1.transform:SetParent(attach_point_1)
        else
            log.error("## no foot_1")
        end

        if attach_point_2 then
            self.tail_2 = 
                game_director.pool:instantiate("Assets/Arts/fx/prefab/Vehicle_tail.prefab")
            self.tail_2.transform.position = attach_point_2.position
            self.tail_2.transform:SetParent(attach_point_2)
        else
            log.error("## no foot_2")
        end
    end,

    -- 添加一个与 player 接触的 entity, 放在列表最后
    addTouchingEntity = function(self, entity)
        --[[
        log.error("addTouchingEntity:"..entity.id)
        if entity.entity_type == 3 then
            -- 偷袭
            self:changeState("sneak_attack", {monster=entity})

        else
            table.insert(self.touching_entities, entity)
            if #self.touching_entities == 1 then
                self:refreshInteractButton()
            end
        end]]
    end,

    -- 设置接触的操作区域
    setTouchingOpTrigger = function(self, trigger)
        self.operation_trigger = trigger
        self:refreshInteractButton()
    end,

    -- 删除与玩家接触的 entity, 要遍历找到索引 
    removeTouchingEntity = function(self, entity)
        for k,v in ipairs(self.touching_entities) do
            if v == entity then
                table.remove(self.touching_entities, k)
                self:refreshInteractButton()
                break
            end
        end
    end,

    -- 刷新交互按钮，根据交互类型
    refreshInteractButton = function(self)
        if self.operation_trigger then
            if level_frame then
                if self.operation_trigger.operation_type == 1 then
                    level_frame:refreshInteractText(NativeText.GetLangStr(170054), true)    -- 交谈
                elseif self.operation_trigger.operation_type == 2 then
                    level_frame:refreshInteractText(NativeText.GetLangStr(170053), true)    -- 收集
                elseif self.operation_trigger.operation_type == 3 then
                    level_frame:refreshInteractText(NativeText.GetLangStr(170055), true)    -- 修理
                elseif self.operation_trigger.operation_type == 4 then
                    level_frame:refreshInteractText(NativeText.GetLangStr(170056), true)    -- 放置
                elseif self.operation_trigger.operation_type == 5 then
                    level_frame:refreshInteractText(NativeText.GetLangStr(170057), true)    -- 破坏
                elseif self.operation_trigger.operation_type == 6 then
                    level_frame:refreshInteractText(NativeText.GetLangStr(170057), true)    -- 破坏
                elseif self.operation_trigger.operation_type == 7 then
                    level_frame:refreshInteractText(NativeText.GetLangStr(170058), true)    -- 营救
                elseif self.operation_trigger.operation_type == 8 then
                    level_frame:refreshInteractText(NativeText.GetLangStr(170130), true)    -- 发射
                end
            end
        else
            local v = self:getFirstTouchingEntity()

            local lang_id = 170051
            local high_light = false
            if v then
                if v.entity_type == common_pb.MET_MONSTER then 
                    lang_id = 170052
                elseif v.entity_type == common_pb.MET_MINE then
                    lang_id = 170053
                elseif v.entity_type == common_pb.MET_NPC then
                    lang_id = 170054
                elseif v.entity_type == common_pb.MET_AWARD then
                    lang_id = 170053
                elseif v.entity_type == common_pb.MET_EVENT then
                    lang_id = 170053
                end
                high_light = true
            end

            local str = NativeText.GetLangStr(lang_id)
            if level_frame then
                level_frame:refreshInteractText(str, high_light)
            end
        end
    end,

    -- 与接触的 entity 交互
    interactWithTouchingEntity = function(self)
        if self.operation_trigger then
            self.operation_trigger:interactWithPlayer(self)
        else
            local v = self:getFirstTouchingEntity()
            if v then
                v:interactWithPlayer(self)
                log.info("interactWithTouchingEntity"..v.id)
            else
                log.info("No touching entity!")
            end
        end
    end,

    getFirstTouchingEntity = function(self)
        for k, v in ipairs(self.touching_entities) do
            if v then
                return v
            end
        end
    end,

    -- 被敌人发现 
    viewByMonster = function(self, monster)
        log.info("### player view by monster")
        self:changeState("view_by_monster", {monster = monster})
        --self.is_view_by_monster = true

        --[[
        self.cs_tank:SetMoveParam(config.commonvars.LEVEL_SPEED*0.2, -1, 270, 0.1)
        app:startCoroutine(
            function()
                coroutine.yield(WaitForSeconds(5.0))
                self:resetSpeed()
            end
        )]]
    end,

    playSmoke = function(self, parent, path)
        local smoke_go = game_director.pool:instantiate(path) 
        smoke_go.transform.position = parent.position
        smoke_go.transform.rotation = parent.rotation
        smoke_go.transform:SetParent(parent)
        smoke_go.transform.localScale = Vector3.one
        battle_helper:destroyObjectAfter(smoke_go, 3.0)
    end,

    update = function(self, dt)
        super.update(self, dt)
        if self.wheel then
            self.wheel:update()
        end

        if self.state_manager then
            self.state_manager:update(dt)
        end

        if self.cs_inst then
            if self.cs_inst.m_bIsMoving then
                if self.stop_elapsed ~= 0.0 then
                    self.stop_elapsed = 0

                    if not self.light_enabled then
                        self:setHeadLightEnable(true)
                        -- 
                        --[[
                        app:startCoroutine(
                            function()
                                AudioController.Instance:SoundPlay("engine_start", "SFX", 1, false, self.transform)
                                coroutine.yield(WaitForSeconds(1))
                                if self.engine_loop_sfx then
                                    AudioController.Instance:SoundStop(self.engine_loop_sfx)
                                    self.engine_loop_sfx = nil
                                end
                                self.engine_loop_sfx = 
                                    AudioController.Instance:SoundPlay("engine_loop", "SFX", 1, true, self.transform)
                            end
                        )]]
                    end

                    -- 播放启动烟雾
                    if self.foot_1 then
                        self:playSmoke(self.foot_1, "Assets/Arts/fx/prefab/start_yanwu_qibu.prefab")
                    end
                    if self.foot_2 then
                        self:playSmoke(self.foot_2, "Assets/Arts/fx/prefab/start_yanwu_qibu.prefab")
                    end
                end
            else
                if self.stop_elapsed < 5.0 then
                    self.stop_elapsed = self.stop_elapsed + dt
                    if self.stop_elapsed > 5.0 then
                        self:setHeadLightEnable(false)

                        --[[
                        if self.engine_loop_sfx then
                            AudioController.Instance:SoundStop(self.engine_loop_sfx)
                            self.engine_loop_sfx = nil
                        end
                        AudioController.Instance:SoundPlay("engine_off", "SFX", 1, false, self.transform)
                        ]]
                    end
                end
            end
        end
    end,

    moveTo = function(self, pos, speed, callback)
        if self.cs_tank then 
           -- log.info("### level player move to "..tostring(pos))
           -- log.info(debug.traceback())
            super.moveTo(self, pos, speed or 60, callback)

            --[[
            local prefab_path = "Assets/Arts/ui/Fx/ui_prefab/Prefab_shouzhidianji.prefab"
            local uobject = game_director.pool:instantiate(prefab_path)
            -- pos.y = self.transform.position.y
            uobject.transform.position = pos
            app:startCoroutine(function ()
                coroutine.yield(WaitForSeconds(3))
                uobject:destroy()
            end)]]
            --uobject:
        end
    end,

    preUpdateTurn = function(self, starts, ends)
        -- 记录起点，终点
        starts[self.id] = self.path_point_id
        ends [self.id] = self.path_point_target
    end,

    -- 移动到目标回调
    onMoveToDestination = function(self)
        --AudioController.Instance:SoundStop("engine_loop")
    end,

    resetSpeed = function(self)
        self.cs_tank:SetMoveParam(config.commonvars.LEVEL_SPEED, -1, 270, 0.5)
    end,

    stopMove = function(self)
        if self.cs_tank then 
            self.cs_tank:StopMove()
        end
    end,

    -- 设置是否中断操作
    setBreakInput = function(self, break_input)
        if self.cs_inst then
            self.cs_inst.m_IsInputBreak = break_input
        end
    end,
    
    destroy = function(self)
        if self.engine_loop_sfx then
            --AudioController.Instance:SoundStop(self.engine_loop_sfx)
            self.engine_loop_sfx = nil
        end

        if self.tail_1 then
            self.tail_1:destroy()
            self.tail_1 = nil
        end

        if self.tail_2 then
            self.tail_2:destroy()
            self.tail_2 = nil
        end

        super.destroy(self)
    end,


}