local log = logger.get("Game", "LevelQuestManager")

-- 关卡任务管理器
local level_quest_mgr_cls = class{
    ctor = function(self)
        self.level_id = nil     -- 当前关卡 id
        self.level_quests = {}  -- 当前关卡的任务
    end,

    -- 开始关卡的任务, 返回任务列表
    beginLevelQuest = function(self, level_id)
        self.level_id = level_id
        self.level_quests = self:getLevelQuestCfg(level_id)
        return self.level_quests
    end,

    -- 从配置表读取关卡任务
    getLevelQuestCfg = function(self, level_id)
        local tab = {}
        config.game_copy_scenario_quests:foreach(
            function(k, v)
                if v.quest_level == level_id then
                    local state = 0 -- 任务状态 0-未开启  1-进行中  2-完成未领奖 3-已领奖
                    if v.previous_quest == 0 then state = 1 end
                    local quest_inst = {id = k, cfg=v, progress=0, state = state}
                    quest_inst.quest_location = {}  -- 复制任务点
                    local tab_location = loadstring("return "..v.quest_location)()
                    for _, point_id in ipairs(tab_location) do
                        table.insert(quest_inst.quest_location, point_id)
                    end
                    table.insert(tab, quest_inst)
                end
            end)
        -- 对 tab 按照 id 排序
        table.sort(tab, function(a, b)
            if a.id < b.id then
                return true
            else
                return false
            end
        end
        )
        return tab
    end,

    -- 增加任务进度
    incQuestProgress = function(self, id, num)
        local quest = self:getQuest(id)
        
        if quest then
            quest.progress = quest.progress + num
            
            if quest.state == 1 and 
                quest.progress >= quest.cfg.quest_total_progress then
                -- 任务完成
                self:setQuestState(quest, 2)
            else
                -- 显示任务进度
                local hint_str = frame_helper:getLangCfg(quest.cfg.quest_name).." "..quest.progress.."/"..quest.cfg.quest_total_progress
                game_state_level.gui:showMissionProgress(hint_str)
            end
            -- 刷新界面
            game_state_level.gui:refreshMission()
            -- 删除任务对应路点
            self:removeQuestLocation(quest, game_state_level.level.player.path_point_id)
            return
        else
            log.error("Failed to find quest:"..id.." in level:"..self.level_id)
        end
    end,

    removeQuestLocation = function(self, quest, point_id)
        local idx = nil
        for i,id in ipairs(quest.quest_location) do
            if id == point_id then
                idx = i
                break
            end
        end

        if idx then
            table.remove(quest.quest_location, idx)
        end
    end,

    -- 设置任务状态
    -- 0-未开启, 1-进行中, 2-目标达成, 3-领取过奖励，任务结束
    setQuestState = function(self, quest, state)
        -- 任务完成
        quest.state = state
        -- 尝试开启后续任务
        if state == 2 then
            log.info("### 任务目标达成:"..quest.id)
            local next = self:getQuest(quest.cfg.next_quest)
            local hint_str = frame_helper:getLangCfg(quest.cfg.quest_name) .. " ".. NativeText.GetLangStr(170113)
            game_state_level.gui:showMissionProgress(hint_str)
            game_state_level.gui:showMissionRedPoint(true)

            if next and next.state == 0 then
                log.info("### 任务开启:"..next.id)
                app:startCoroutine(
                    function()
                        coroutine.yield(WaitForSeconds(2.0))
                        --log.info("next.quest_name:"..next.cfg.quest_name)
                        local hint_str = NativeText.GetLangStr(170108)..frame_helper:getLangCfg(next.cfg.quest_name)
                        game_state_level.gui:showMissionProgress(hint_str)
                    end
                )
                next.state = 1
            end
        elseif state == 3 then
            -- 任务领取过奖励
            if quest.ui_detail then
                quest.ui_detail.gameObject:SetActive(false)
                quest.ui_detail.gameObject.transform:SetParent(nil)
                GameObject.Destroy(quest.ui_detail.gameObject)
            end

            if game_state_level and game_state_level.gui then
                game_state_level.gui:showMissionRedPoint(self:isAnyQuestCanGetReward())
            end
        end
    end,

    -- 获取任务实例
    getQuest = function(self, id)
        for _, quest in ipairs(self.level_quests) do
            if quest.id == id then
                return quest
            end
        end
        return nil
    end,

    isAnyQuestCanGetReward = function(self)
        for _, quest in ipairs(self.level_quests) do
            if quest.state == 2 then
                return true
            end
        end
        return false
    end,
}

level_quest_manager = level_quest_mgr_cls()
return level_quest_manager

