local log = logger.get("Game", "LevelScript 1006")
local super = require('level.level_script.level_script_base')
local phase_base = require("level.level_script.level_phase_base")


-----------注意----------
--阶段phase是class，也就是类，而不同于之前的函数类型
--phase中的onEnter,onExit以及自己定义的函数都是类的内容，也就是table的key，所以不能在这些前面增加local 字段
--自定义函数前不能增加local
--可以在onEnter中定义额外函数并能使用local，调用类中全局函数前要使用self.调用，onEnter内部定义函数直接使用函数名调用

local boss = nil
local ship01 = nil
local ship02 = nil
local stone = nil
local boss_current_idx = 1 -- 当前路点
local boss_path_points = {20, 19, 18, 17, 3, 4, 5, 6, 7, 8, 9, 10, 11 ,12}    -- boss 路径索引

-- 第一个阶段，主角对话
local phase_1 = class{
    super = phase_base,

    onEnter = function(self)
        self.level_script:breakPlayerInput()
        self.level_script:playDialog(292,
            function()
                self.level_script:changePhase("phase_2")
            end
        )

    end,

    onEnterPoint = function(self, point_id)
       
    end,

    onBattleWin = function(self, id)
    end,

    onExit = function(self)
        --self.level_script:resumePlayerInput()
    end,
}


--第二个阶段，敌方 boss 进场
local phase_2 = class{
    super = phase_base,

    onEnter = function(self)

        app:startCoroutine(
            function()
                self.level_script:cameraFocusToTransform(boss.transform)
                coroutine.yield(WaitForSeconds(0.5))

                self.level_script:playDialog(297, function()
                        self.level_script:changePhase("phase_3")
                end)
            end
        )
    end,
}

-- 展示关卡目标，开始关卡
local phase_3 = class {
    super = phase_base,

    onEnter = function(self)

        local dialog_callback = function()
            app:startCoroutine(
                function()

                    self.level_script:cameraFocusToTransform(ship02.transform)

                    coroutine.yield(WaitForSeconds(1.5))

                    self.level_script:cameraFocusToPlayer()
                    coroutine.yield(WaitForSeconds(0.5))

                    self.level_script:resumePlayerInput();
                    self.level_script:changePhase("phase_4")
                end
            )
        end
        
        app:startCoroutine(
            function()
                self.level_script:cameraFocusToPlayer()
                coroutine.yield(WaitForSeconds(0.5))
                self.level_script:playDialog(299, dialog_callback)
            end
        )
    end,
}

local phase_4 = class{
    super = phase_base,

    onEnter = function(self)
    end,

    onEnterPoint = function(self, point_id)
        
    end,

    onBattleWin = function(self, id)

    end,
}

-- 第一次打船砸 boss
local phase_5 = class{
    super = phase_base,

    onEnter = function(self)
        -- boss 进入 3 号路点，提示玩家可以对山头的船开会阻碍 boss 行动
        self.level_script:playDialog(301, function()

            local pos = self.level_script.level.player:getPosition()
            pos.y = 20.5
            local rot = Quaternion.LookRotation(ship01.transform.position - pos, Vector3.up)
            game_state_level:setFPSMode(true, pos, rot, 
                function()
                    app:startCoroutine(
                        function()
                            game_state_level:setFPSMode(false)
                            self.level_script:cameraFocusToTransform(ship01.transform)
                            coroutine.yield(WaitForSeconds(0.5))
                            local boom =  self.level_script:createGameObject(self.level_script.prefabs.boom, -369.7, 39.6, -317.7, 0, 2)                 
                            -- 切换到射击模式
                            ship01:playAnimation("ship01_drop")
                            coroutine.yield(WaitForSeconds(0.5))
        
                            self.level_script:cameraFocusToTransform(boss.transform)
                            -- boss 倒车躲避
                            boss:moveTo(Vector3(-412.7, 0, -453.3), 200)
                            --boss:rotateTo(ship01.transform, 360)
                         
                            coroutine.yield(WaitForSeconds(1.5))
                            
                            self.level_script:removeGameObject(boom)
                            self.level_script:playDialog(303, function()
                                ship01:destroy()
                                self.level_script:cameraFocusToPlayer()
                                self.level_script:resumePlayerInput()  
                            end)
        
        
                        end)
                end)
            end
        )
    end,

    onEnterPoint = function(self, point_id)
        
    end,

    onBattleWin = function(self, id)

    end,
}

-- boss 反击，用石头砸玩家
local phase_6 = class{
    super = phase_base,

    onEnter = function(self)

        local dialog_callback = function()

            local shoot_point = boss:findChild("shoot")
            local to = self.level_script.level.player:getPosition() --Vector3(-124.22, 59.38, -275.59)
            -- 这里应该放一个音波攻击的效果
            local bullet = self.level_script:shootBullet(8, shoot_point, to, 300, function(x,y,z)
                app:startCoroutine(function()

                    local boom = self.level_script:createGameObject(self.level_script.prefabs.boom, x, y, z, 0, 2)         
                    coroutine.yield(WaitForSeconds(0.5))
                    stone:playAnimation("rock_drop")
                    coroutine.yield(WaitForSeconds(1.0))

                    self.level_script:playDialog(305, function()
                        -- 将6号路点设置为路障
                        self.level_script.level:setPathPointType(6, 1)
                        local player_point_id = self.level_script.level.player.path_point_id
                        self.level_script.level.path_point_manager:ShowPathPoint(player_point_id)
                        local point = self.level_script.level.path_point_manager:FindPathPoint(player_point_id)
                        if point.m_Type == 1 then   -- 路障
                            self.level_script:onMoveToRoadBlock(player_point_id)
                        end

                        self.level_script:cameraFocusToPlayer()
                        self.level_script:resumePlayerInput()

                    end)
                    coroutine.yield(WaitForSeconds(0.5))
                    boom:destroy()
                end)
            end)

            self.level_script:cameraFocusToTransform(bullet.transform)

        end

        app:startCoroutine(
            function()
                self.level_script:cameraFocusToTransform(boss.transform)
                boss:rotateTo(self.level_script.level.player.transform, 360)

                coroutine.yield(WaitForSeconds(0.5))

                self.level_script:playDialog(304, dialog_callback)
            end
        )
    end,

    onEnterPoint = function(self, point_id)
        
    end,

    onBattleWin = function(self, id)

    end,
}

local phase_7 = class{
    super = phase_base,

    onEnter = function(self)
    end,

    onEnterPoint = function(self, point_id)
        
    end,

    onBattleWin = function(self, id)

    end,
}

local phase_8 = class{
    super = phase_base,

    onEnter = function(self)
        local dialog_callback = function()

            local pos = self.level_script.level.player:getPosition()
            pos.y = 20.5
            local rot = Quaternion.LookRotation(ship02.transform.position - pos, Vector3.up)
            game_state_level:setFPSMode(true, pos, rot, function()
                app:startCoroutine(
                    function()
                        game_state_level:setFPSMode(false)
                        -- 播放船掉下来的动画
                        self.level_script:cameraFocusToTransform(ship02.transform)
                        coroutine.yield(WaitForSeconds(1.0))
                        local pos = ship02.transform.position
                        self.level_script:createGameObject(self.level_script.prefabs.boom, pos.x, pos.y, pos.z, 0, 3)                 
                        -- 切换到射击模式
                        coroutine.yield(WaitForSeconds(0.5))
    
                        ship02:playAnimation("ship02_drop")
                        self.level_script:cameraFocusToTransform(boss.transform)
    
                        pos = boss.transform.position
                        self.level_script:createGameObject(self.level_script.prefabs.boom, pos.x, pos.y, pos.z, 0, 5)
                        coroutine.yield(WaitForSeconds(2.0))
    
                        -- 漫画交代剧情
                        self.level_script:finishLevel(true)
                    end
                )
            
            end)


        end

        self.level_script:cameraFocusToPlayer()
        self.level_script:playDialog(307, dialog_callback)
    end,

    onEnterPoint = function(self, point_id)
        
    end,

    onBattleWin = function(self, id)

    end,
}

-- 关卡 3002 对应的脚本
return class{
    super = super,

    -- 初始化
    init = function(self)
        self.prefabs = {
            boss = 'Assets/Prefab/Level/level_prefab_kv2.prefab',     --丘吉尔坦克，解救英雄时用到
            ship01 = 'Assets/Prefab/Level/Chapter3_2/ship01.prefab',
            ship02 = 'Assets/Prefab/Level/Chapter3_2/ship02.prefab',
            stone = 'Assets/Prefab/Level/Chapter3_2/RockB.prefab',
            boom = 'Assets/Arts/fx/prefab/skill10010_hit.prefab',       --爆炸特效
        }

        -- 预加载
        app:startCoroutine( function() 
            for k, path in pairs(self.prefabs) do
                self.pool:wait_prepare(path, 1)
            end
        end)

        -- 创建 boss 对象
        if not boss then
            local point_id = boss_path_points[1]
            local point = self.level.path_point_manager:FindPathPoint(point_id)
            local pos = point.position
            boss = self:createGameObject(
                self.prefabs.boss ,pos.x, 0, pos.z, 290, 3.0)
        end

        -- 创建 ship01
        if not ship01 then
            ship01 = self:createGameObject(
                self.prefabs.ship01, -369.5, 39.6, -317.7, 0, 160, function(go)
                    go:setRotation(Quaternion.Euler(-4.8, 163.9, 8.48))
                end)
        end

        -- 创建 ship02
        if not ship02 then
            ship02 = self:createGameObject(
                self.prefabs.ship02, -316, 93.5, 154.4, 0, 1, function(go)
                end)
        end

        -- 创建石头
        if not stone then
            stone = self:createGameObject(
                self.prefabs.stone, -61.5, 48.2, -144, 0, 1)
        end

        self:addPhases("phase_1", phase_1(self))
        self:addPhases("phase_2", phase_2(self))
        self:addPhases("phase_3", phase_3(self))
        self:addPhases("phase_4", phase_4(self))
        self:addPhases("phase_5", phase_5(self))
        self:addPhases("phase_6", phase_6(self))
        self:addPhases("phase_7", phase_7(self))
        self:addPhases("phase_8", phase_8(self))

        self:changePhase("phase_1")
    end,

    onEnterPoint = function(self, point_id)
        super.onEnterPoint(self)


        self:bossTurn()
    end,

    -- boss 回合
    bossTurn = function(self)
        log.info("## bossTurn!")
        if not boss then
            return
        end

        local player_point_id = self.level.player.path_point_id

        log.info("### Player point id:"..player_point_id)

        self:breakPlayerInput()
        app:startCoroutine(
            function()
                self:cameraFocusToTransform(boss.transform)
                coroutine.yield(WaitForSeconds(1.0))
                -- 这里可以插入个 boss 唱歌对话

                self:bossMoveToNextPoint(
                    function()
                        self:cameraFocusToPlayer()

                        -- 判断 boss 和 玩家是否相邻
                        if self:isBossNeighborPlayer() then
                            -- 和 boss 直接发起战斗
                            network_sender:levelChangeHeroProp(-0.8, 0)
                            self:battleWithMonster(87)  
                        else
                            if boss_current_idx == 3 and player_point_id == 4 then  -- 注意，这里 boss 是路点的索引，玩家是路点 id
                                --log.info("#### changePhase('phase_5')")
                                self:changePhase('phase_5')
                            elseif boss_current_idx == 5 and (player_point_id == 5 or player_point_id == 6) then
                                self:changePhase('phase_6') -- boss 反击
                                --self:resumePlayerInput()
                            elseif player_point_id == 12 then
                                self:changePhase('phase_8') -- 打船2，结束
                            else
                                self:resumePlayerInput()
                            end
                        end


                    end
                )
            end
        )
    end,

    -- boss 和 玩家 是否相邻
    isBossNeighborPlayer = function(self)
        local boss_point_id = boss_path_points[boss_current_idx]
        --log.info("boss point id:"..boss_point_id)
        local boss_point = self.level.path_point_manager:FindPathPoint(boss_point_id)
        
        local player_point_id = self.level.player.path_point_id
        --log.info("player point id:"..player_point_id)

        local is_neighbor = boss_point:IsNeighbor(player_point_id)
        --log.info("is neighbor:"..tostring(is_neighbor))

        return is_neighbor or (boss_point_id == player_point_id)
    end,

    -- boss 移动到下一点
    bossMoveToNextPoint = function(self, callback)
        boss_current_idx = boss_current_idx + 1
        local point_id = boss_path_points[boss_current_idx]
        local point = self.level.path_point_manager:FindPathPoint(point_id)

        boss:rotateTo(point.transform, 360)
        boss:moveTo(point.position, 100, function()
            if callback then
                callback()
            end
        end)               
    end,

    -- 战斗胜利
    onBattleWin = function(self, id)
        super.onBattleWin(self, id)

        -- 干掉 86 号怪物，关卡结束
        if id == 87 then
            self:finishLevel(true)
        end
    end,

    onBattleLose = function(self, id)
        super.onBattleLose(self, id)

        -- boss 战失败
        --if id == 87 then
        self:finishLevel(false)
        --end
    end,

    -- 清除了路障
    onClearRoadBlock = function(self, id)
        if id == 6 and stone then
            stone:destroy()
        end
    end,
}
