local log = logger.get("Game", "BattleHelper")

-- 为了使用子弹 兼容一些 battle_helper 的接口 (比如 loadRes)
level_battle_helper_inst = 
{
    -- 初始化
    init = function(self, level)
        self.level = level
        self.level_script = level.level_script
        self.pool = game_director.pool

        self.scheduler = require("base.scheduler").new()

        self.temp_cube = self:loadRes("Assets/Prefab/Level/Cube.prefab")
        self.temp_cube:SetActive(false)
    end,

    -- 读取一个资源
    loadRes = function(self, prefab_path, need_reset_vfx)
        local res = self.pool:instantiate(prefab_path)
        if not res then
            log.warning('pool:instantiate got nil:'..prefab_path)
        end

        if need_reset_vfx then
            local vfx = res:GetComponent('VFX')
            if vfx then
                vfx:Reset()
            end
        end

        return res
    end,

    -- 加载一个prefab
    loadPrefab = function(self, prefab_path, callback)
        local uobject = self.pool:instantiate(prefab_path)
        if not uobject then
            log.error('Failed to load prefab:'..prefab_path)
            return nil
        end

        if callback then
            if uobject.isLoaded then 
                callback(uobject)
            else 
                uobject.onLoaded = callback
            end
        end
        return uobject
    end,

    -- 定时销毁指定对象
    destroyObjectAfter = function (self, obj, time)
        self.scheduler:create(time, 1, functor.make(obj.destroy, obj))
    end, 

    -- 根据 id 获取 entity
    getEntity = function(self, id)
        return self.level:getEntity(id)
    end,

    update = function(self, dt)
        self.scheduler:update(dt)
    end,

    getCameraForward = function(self)
        local cam = Camera.main
        local cam_forward = cam.transform.forward
        cam_forward.y = 0.0
        cam_forward:Normalize()
        return cam_forward
    end,

    destroy = function(self)
        self.scheduler:destroy()
    end,


}

return level_battle_helper_inst