local log = logger.get("Game", "LevelScript")
local bullet_class = require("RTS.rts_bullet")
local PlayerPrefs = UnityEngine.PlayerPrefs
local level_game_object_cls = require('level.level_game_object')
local formation_helper = require('systems.formation_helper')

-- 关卡脚本基类
local level_script_base = class{

    ctor = function(self, owner, id, marks)
        self.level = owner  -- level 实例
        self.level_manager = self.level.level_manager
        
        self.id = id        -- 关卡 id
        self.marks = marks or {}
        self.pool = game_director.pool

        self.in_battle = false
        
        --[[
        self.pool = require("base.pool"){
            size = 5, 
            transform = app.gameObject.transform:CreateChild("pool"),
        }]]

        self.op_triggers = {}

        self.bullets = require("base.list").new(true)   -- 子弹列表

        -- 关卡的所有阶段
        self.phases = {}
        self.curr_phase = nil

        self.level_go_list = {} -- 所有动态创建的 level go
        self.curr_point_id = 0  -- 当前 path point 
        self.mission_text = {}

        level_script = self
        self:init()

        -- 判断是否有战斗胜利/失败
        if game_info.attack_monster_id ~= 0 then
            if game_info.battle_result ~= 0  then
                self:onBattleWin(game_info.attack_monster_id)
                local entity = self.level:getEntity(game_info.attack_monster_id)
                -- 如果是游民，对应 path_point 清空
                if entity.path_point_id then
                    self:setPathPointType(entity.path_point_id, 0)
                end

            else
                self:onBattleLose(game_info.attack_monster_id)
            end
            game_info.attack_monster_id = 0
        end

        math.randomseed(Time.time * 10000)
        log.info("#### level script ctro!!")
    end,

    -- 恢复关卡状态
    reload = function(self)
        self.level_manager = self.level.level_manager

        for _,level_go in pairs(self.level_go_list) do
            if level_go then
                level_go:reload()
            end
        end

        for _,trigger in pairs(self.op_triggers) do
            if trigger then
                trigger:reload()
            end
        end

        self.bullets = require("base.list").new(true) 
        level_script = self
        self.cam_animator = nil

        local isLevelGameObjectLoaded = function()
            for _, level_go in pairs(self.level_go_list) do
                if not level_go.is_loaded then
                    return false
                end
            end
            return true
        end

        app:startCoroutine(
            function()
                while true do
                    if isLevelGameObjectLoaded() then
                        log.info("### all level gameobject loaded!!")
                        break
                    else
                        coroutine.yield(nil)
                    end
                end
                
                -- 判断是否有战斗胜利/失败
                if game_info.attack_monster_id ~= 0 then
                    log.info("## onBattle:"..game_info.attack_monster_id)

                    if game_info.battle_result ~= 0  then
                        self:onBattleWin(game_info.attack_monster_id)
                    else
                        self:onBattleLose(game_info.attack_monster_id)
                    end
                    game_info.attack_monster_id = 0
                end
            end
        )

        self:battleEnd()
    end,

    destroy = function(self)
        self.level = nil
        for _,level_go in pairs(self.level_go_list) do
            if level_go then
                level_go:destroy()
            end
        end

        for _,trigger in pairs(self.op_triggers) do
            if trigger then
                trigger:destroy()
            end
        end
    end,

    -- 初始化
    init = function(self)
    end,

    -- 逻辑更新
    update = function(self, dt)
        self.bullets:update(dt)
        if self.curr_phase and self.curr_phase.onUpdate then
            self.curr_phase:onUpdate(dt)
        end

        if self.onUpdate then
            self:onUpdate(dt)
        end
    end,

    ------------[[事件]]----------------
    onEnterTrigger = function(self, trigger_name)
        log.info("##### onEnterTrigger"..trigger_name)
        if self.curr_phase and self.curr_phase.onEnterTrigger then
            self.curr_phase:onEnterTrigger(trigger_name)
        end
    end,

    onExitTrigger = function(self, trigger_name)
        log.info("##### onExitTrigger"..trigger_name)
        if self.curr_phase and self.curr_phase.onExitTrigger then
            self.curr_phase:onExitTrigger(trigger_name)
        end
    end,

    -- 一场战斗胜利   
    onBattleWin = function(self, index)     --index为entity的索引编号
        if self.curr_phase and self.curr_phase.onBattleWin then
            log.info("### cur_phase onBattleWin"..index)
            self.curr_phase:onBattleWin(index)
        end

        local point_id = self.level.player.path_point_id    
        log.info("## win point id:"..point_id)    
        self:onBackLevelEnterPoint(point_id)
    end,

    -- 一场战斗失败
    onBattleLose = function(self, index)
        if self.curr_phase and self.curr_phase.onBattleLose then
            self.curr_phase:onBattleLose(index)
        end

        local point_id = self.level.player.path_point_id     
        log.info("## lose point id:"..point_id)       
        self:onBackLevelEnterPoint(point_id)
    end,

    -- 从战斗回来，进入当前点
    onBackLevelEnterPoint = function(self, point_id)
        self:onEnterPoint(point_id)

        local point = self.level.path_point_manager:FindPathPoint(point_id)
        if point.m_Type == 1 then -- 路障
            self:onMoveToRoadBlock(point_id)
        elseif point.m_Type == 2 then -- 保险箱
            self:onMoveToSteelSafe(point_id)
        elseif point.m_Type == 3 then -- 游民
            self:onMoveToTramp(point_id)
        elseif point.m_Type == 4 then -- 足迹
            self:onMoveToFootprint(point_id)
        elseif point.m_Type == 5 then -- 书籍
            self:onMoveToBook(point_id)
        end
    end,

    -- 进入路点
    onEnterPoint = function(self, point_id)
        --log.info("## onEnterPoint:"..point_id)
        -- 判断是否进入 op trigger
        local trigger = nil
        for k, t in pairs(self.op_triggers) do
            if t and t.point_id == point_id then
                --log.info("## find trigger:"..t.point_id)
                trigger = t
                break
            end
        end
        if trigger then
            self.level.player:setTouchingOpTrigger(trigger)
        else
            self.level.player:setTouchingOpTrigger(nil)
        end
    
        self.curr_point_id = point_id
        if self.curr_phase and self.curr_phase.onEnterPoint then
            self.curr_phase:onEnterPoint(point_id)
        end
    end,

    ------------[[接口函数]]-------------
    -- 创建一个游戏对象
    createGameObject = function(self, prefab_path, x, y, z, yaw, scale, callback)
        local level_go = level_game_object_cls(prefab_path, Vector3.New(x, y, z), yaw, scale, callback)
        table.insert(self.level_go_list, level_go)
        return level_go
    end,

    -- 创建一个游戏对象，attach 到另一个对象上
    createGameObjectAttach = function(self, prefab_path, parent_go, attach_point_path, scale, callback)

        local parent = parent_go.transform:Find(attach_point_path)
        if not parent then
            log.error('Failed to find attach_point_path:'..attach_point_path)
            return nil
        end

        local pos = parent.position

        self:createGameObject(prefab_path, pos.x, pos.y, pos.z, 0, scale, function(go)
            go.transform.position = parent.position
            go.transform.rotation = parent.rotation
                --log.info('info')
            if scale then
                go.transform.localScale = Vector3.New(scale, scale, scale)
            end
            go.transform:SetParent(parent)
            
            callback(go)
        end)
    end,

    -- 删除游戏对象
    removeGameObject = function(self, level_go)
        if level_go then
            level_go:destroy()
            for idx,go in pairs(self.level_go_list) do
                if go == level_go then
                    table.remove(self.level_go_list, idx)
                    break
                end
            end
        end
    end,

    -- 游戏对象移动到目标
    gameObjectMoveTo = function(self, level_go, to, speed, callback)
        if not speed or speed == 0 then
            log.error('gameObjectMoveTo invalid speed:'..tostring(speed))
            return false
        end

        level_go:moveTo(to, speed, callback)
    end,

    -- 场景中非自己创建的gameobject对象的移动
    sceneGameObjectMoveTo = function(self, go, to, speed, callback)
        local from = go.transform.position
        local start_time = Time.time
        local dist = Vector3.Distance(from, to)
        local total_time = dist / speed
        
        app:startCoroutine(
                function()
                    while true do
                        coroutine.yield(nil)
                        local percent = (Time.time-start_time)/total_time
                        go.transform.position = Vector3.Lerp(from, to, percent)
                        if percent >= 1 then
                            break
                        end
                    end

                    if callback then
                        callback()
                    end
                end)   

    end,

    --level manger中取出的gameObject如果要播放动画要使用这个接口
    sceneGameObjectPlayAnimation = function(self, go, anim_name)
        local anim_comps = go:GetComponentsInChildren(UnityEngine.Animation)

        for _,anim in pairs(anim_comps.Table) do
            anim:CrossFade(anim_name)
        end
    end,

    -- 关卡士兵播放动画
    sceneGameObjectPlaySoldierAnimation = function(self, go, anim_name)
        local soldier = go:GetComponentInChildren(Soldier)

        if soldier then
            local soldier_cls = require('RTS.rts_soldier')
            local prefix = soldier_cls.getAnimPostfix(soldier)
            soldier:PlayAnimation(prefix..anim_name)
        else
            log.error("Failed to find soldier component!")
        end
    end,

    -- 关卡 entity 对象播放动画
    entityPlayAnimation = function(self, entity_id, anim_name)
        local entity = self.level:getEntity(entity_id)
        if entity then
            entity.cs_inst:PlayAnimation(anim_name)
        else
            log.error("Failed to find entity:"..tostring(entity_id))
        end
    end,

    -- 根据名称找预先缓存在 LevelManager 中的游戏对象
    --
    --local go = self:findGameObject('smoke')
    --go:SetActive(true)

    findGameObject = function(self, obj_name)
        return self.level_manager:FindGameObject(obj_name)
    end,

    -- 从 trans 中找到一个 名为 name 的子结点
    -- 注意！！， 尽量使用 level_help:findNode
    findChild = function(self, trans, name)
        if not trans then
            return nil
        end

        if trans.name == name then
            return trans
        end

        for i=0, trans.childCount-1 do
            local child = self:findChild(trans:GetChild(i), name)
            if child then
                return child
            end
        end

        return nil
    end,

    -- 播放动画
    playAnimation = function(self, level_go, anim_name)
        if level_go and  anim_name then
            level_go:playAnimation(anim_name)
        end
    end,

    
    -- 播放射击效果，包括枪口特效，子弹飞行过程，击中特效
    shootBullet = function(self, shoot_effect_id, shoot_point, to, speed, callback)
        local shoot_cfg = config.shoot_effect_shoots[shoot_effect_id]
        if not shoot_cfg then
            log.error('Failed to find shoot_cfg:'..shoot_effect_id)
        end

        -- 播放枪口火花特效
        if shoot_cfg.start > 0 then
            local start_cfg = config.shoot_effect_effect_libs[shoot_cfg.start]
            if start_cfg then
                local start_effect = self.pool:instantiate(start_cfg.effect_path)
                start_effect.transform.position = shoot_point.position
                start_effect.transform.rotation = shoot_point.rotation
                start_effect.transform:SetParent(shoot_point)
                
                -- 注意！这里的 battle_helper 是 level_battle_helper!!
                battle_helper:destroyObjectAfter(start_effect, start_cfg.effect_time)
            end

            -- 发射子弹
            local loop_cfg = config.shoot_effect_effect_libs[shoot_cfg.loop]
            local body_cfg = config.shoot_effect_effect_libs[shoot_cfg.loop_1]
            local body_effect_path = nil
            if body_cfg then
                body_effect_path = body_cfg.effect_path
            end

            if loop_cfg then
                local curve = nil
                if curve_name then
                    --log.info(curve_name)
                    curve = battle_helper:loadCurve(curve_name)
                end

                local from = shoot_point.position
                local rot = Quaternion.LookRotation(to-from, Vector3.up)
                local stop_cfg = config.shoot_effect_effect_libs[shoot_cfg.stop]
                --ctor = function (self, owner, target, prefab_path, pos, destination, speed, hit_callback)
                local bullet = bullet_class(self, to, rot, loop_cfg.effect_path, body_effect_path, from, to, speed, curve,
                    -- 击中回调
                    function(b) 
                        self.bullets:remove(b)

                         -- 播放击中特效
                        if stop_cfg then
                            local stop_effect_go = battle_helper:loadRes(stop_cfg.effect_path)
                            stop_effect_go.transform.position = to
                            --local stop_effect_go = self:createGameObject(stop_cfg.effect_path, to.x, to.y, to.z, 0)
                            battle_helper:destroyObjectAfter(stop_effect_go, stop_cfg.effect_time)
                        end

                        if callback then
                            callback(to.x, to.y, to.z)
                        end
                    end)
                self.bullets:add(bullet)  

                return bullet
            end
        end
    end,

    -- 等待 xxx 秒
    wait = function(self, seconds)
        coroutine.yield(WaitForSeconds(seconds))
    end,

    -- 进入战斗
    battle = function(self, index)
        -- 与关卡怪物发生战斗
        self.in_battle = true
        self:removePlayerChat()
    end,

    -- 结束战斗
    battleEnd = function(self)
        self.in_battle = false
    end,

    -- 获取玩家当前位置 返回 Vector3
    getPlayerPos = function(self)
        if self.level.player then
            return self.level.player:getPosition()
        else
            log.error('Failed to find player pos!!')
            return nil
        end
    end,

    --Hong Feng
    --获取entity位置，返回Vector3
    getEntityPos = function(self,entity)
        if self.level.entities[entity] then
            return self.level.entities[entity]:getPosition()
        else
            log.error('Failed to find entity'..entity..'pos')
            return nil
        end
    end,
    
    -- 玩家中断操作，进入剧情
    breakPlayerInput = function(self)
        game_state_level:breakPlayerInput()
    end,

    -- 玩家恢复操作
    resumePlayerInput = function(self)
        game_state_level:resumePlayerInput()
    end,

    -- 摄像机跟随玩家
    cameraFocusToPlayer = function(self)
        game_state_level:lookTarget(self.level.player.transform)
        self.level_manager:SetFogOfWarTransform(self.level.player.transform, config.commonvars.LEVEL_SIGHT)
    end,

    -- 摄像机跟随关卡点 index - level point 索引   is_open_fog - 是否开启关卡点的战争迷雾
    cameraFocusToEntity = function(self, index, is_open_fog)
        local entity = self.level:getMapEntity(index)
        if entity then
            self:cameraFocusToTransform(entity.transform, is_open_fog)
        else
            log.error('Failed to find map entity to look at, index'..index)
        end
    end,

    cameraFocusToTransform = function(self, trans, is_open_fog, sight)
        game_state_level:lookTarget(trans)

        if is_open_fog then
            if sight then
                self.level_manager:SetFogOfWarTransform(trans, sight)
            else
                self.level_manager:SetFogOfWarTransform(trans, config.commonvars.LEVEL_SIGHT)
            end
        end
    end,

    -- 玩家移动到某位置
    --调用这个接口前，一定要中断玩家操作！！！非常重要
    playerMoveTo = function(self, pos, speed, callback)     
        self.level.player:moveTo(pos, speed, callback)
    end,

    --重置玩家速度，在调用移动玩家的函数后要使用
    playerResetSpeed = function(self)
        self.level.player:resetSpeed()
    end,

    -- 玩家停止移动
    playerStopMove= function(self)
        self.level.player:stopMove()
    end,

    -- 玩家寻找附近的路点作为当前路点
    playerFindNearestPathPoint = function(self)
        local id = self.level.player:initPathPoint()
        if id then
            self.level.path_point_manager:ShowPathPoint(id)
        end
    end,

    -- 玩家发射一颗子弹
    playerShootBullet = function(self, shoot_effect_id, to, speed, callback)
        local entity = self.level.player
        if entity then
            local shoot_point = entity:findChild('shoot')
            if not shoot_point then
                log.error('entityShootBullet Failed to shoot point!!')
                return
            end
            return self:shootBullet(shoot_effect_id, shoot_point, to, speed, callback)
        end
    end,

    -- 关卡对象移动到
    entityMoveTo = function(self, index, pos, speed, callback)
        local entity = self.level:getMapEntity(index)
        if entity then
            entity:moveTo(pos, speed, callback)
        end
    end,

    -- 关卡对象停止移动
    entityStopMove = function(self, index)
        if not self.level then
            return
        end
        local entity = self.level:getMapEntity(index)
        if entity then
            entity:stopMove()
        end
    end,

    -- 游戏对象发射子弹
    entityShootBullet = function(self, index, shoot_effect_id, to, speed, callback)
        if not self.level then
            return 
        end
        local entity = self.level:getMapEntity(index)
        if entity then
            local shoot_point = entity:findChild('shoot')
            if not shoot_point then
                log.error('entityShootBullet Failed to shoot point!!')
                return
            end
            return self:shootBullet(shoot_effect_id, shoot_point, to, speed, callback)
        end
    end,

    --[[
    -- 隐藏 entity
    hideEntity = function(self, index)
        local entity = self.level:getMapEntity(index)
        if entity and entity.gameObject then
            entity.gameObject:SetActive(false)
        end
    end,

    -- 显示 entity
    showEntity = function(self, index)
        local entity = self.level:getMapEntity(index)
        if entity and entity.gameObject then
            entity.gameObject:SetActive(true)
        end
    end,]]

    -- 摄像机缩放
    changeCameraFov = function(self, fov)
        game_state_level.level_camera:ChangeFov(fov)
    end,

    -- 关卡怪物开始追踪玩家,auto_fight_time为空时，按照正常走到时间战斗
    monsterBeginTracePlayer = function(self, monster_id, auto_fight_time)
        local entity = self.level:getMapEntity(monster_id)
        if entity then
            self.level.player:viewByMonster(entity)
            entity:beginTracePlayer(self.level.player, auto_fight_time or 2.0)
        else
        end
    end,

    -- 关卡怪物停止追踪玩家
    monsterStopTracePlayer = function(self, monster_id)
        local entity = self.level:getMapEntity(monster_id)
        if entity then
            entity:stopTracePlayer()
        end

    end,

    -- 设置关卡标记
    setMark = function(self, index, value)
        PlayerPrefs.SetInt("mark_"..tostring(self.id).."_"..index, value or 0)
    end,

    -- 获取关卡标记
    getMark = function(self, index)
        return PlayerPrefs.GetInt("mark_"..tostring(self.id).."_"..index, 0)
    end,

    -- 重置所有 mark
    resetMarks = function(self)
        for i=1,64 do
             PlayerPrefs.SetInt("mark_"..tostring(self.id).."_"..i, 0)
        end
    end,

    -- 关闭与关卡对象交互
    disableEntityInteraction = function(self)
        self.level.is_interaction_enabled = false
    end,

    -- 开启与关卡对象交互
    enableEntityInteraction = function(self)
        self.level.is_interaction_enabled = true
    end,

    -- 获取上次战斗信息 怪物 id, 是否胜利
    getLastBattleInfo = function(self)
        return game_info.attack_monster_id, game_info.battle_result
    end,

    -- 清除上次的战斗信息
    clearLastBattleInfo = function(self)
        game_info.attack_monster_id = 0
        game_info.battle_result = 0
    end,

    -- 是否存在激活的英雄
    isHeroNew = function(self, hero_id)
        return hero_manager.new_heros[hero_id] ~= nil
    end,

    -- 显示新增加的英雄
    performNewHero = function(self, hero_id)
        local hero_info = hero_manager.new_heros[hero_id]
        if hero_info then
            local hero = hero_manager:createHero(hero_info.HeroInfo, hero_info.ParPropIdValues)
            local officer = hero.officer
            local officer_name = frame_helper:getLangCfg(officer.cfg.commander_name)

            local str = NativeText.GetLangStr(160017)
            str = string.format(str, officer_name)
            hint:setText(str)
            hero_manager.new_heros[hero_id] = false
        end
    end,

    -- 播放背景音乐
    playBGM = function(self, bgm_name, category)
        --AudioController.Instance:BGMPlay(bgm_name, category, 1)
    end,

    pauseBGM = function(self)
         --AudioController.Instance:BGMPause()
    end,

    -- 播放音效
    playSFX = function(self, sfx_name, category)
        --AudioController.Instance:SoundPlay(sfx_name, category, 1, false, nil)
    end,

    ---------------------------------
    ---------对话脚本说明-------------
    ---------------------------------

    --lang_id为对话内容的lang编号
    --time为这段对话显示的时间，单位是秒，time的值永远大于0，到达时间后，对话完成，调用callback（如果有callback函数）

    --玩家对话，气泡在玩家头上
    playerChat = function(self,lang_id,time,callback)
        if self.in_battle then
            return
        end
        local bubble = self.level.player.cs_inst:FindChild("HeadBubble")
        if bubble then
            self:showExistChat(bubble, lang_id, time, callback)
        else
            self:showChat(self.level.player.cs_inst, lang_id, time, callback)
        end
    end,

    --entity头上气泡
    entityChat = function(self,entity_id,lang_id,time,callback)
        if self.in_battle then
            return
        end
        local entity = self.level:getMapEntity(entity_id)
        if entity and entity.cs_inst then
            local bubble = entity.cs_inst:FindChild("HeadBubble")
            if bubble then
                self:showExistChat(bubble, lang_id, time, callback)
            else
                self:showChat(entity.cs_inst, lang_id, time, callback)
            end
        else
            log.error("##### entity not exist !")
        end
    end,

    --GO头上气泡
    gameObjectChat = function(self,gameObject,lang_id,time,callback)
        if self.in_battle then
            return
        end
        self:showChat(gameObject, lang_id, time, callback, true)
    end,

    -- 对象头顶已经有气泡
    showExistChat = function(self, bubble, lang_id, time, callback)
        bubble:SetActive(true)
        local text = bubble.gameObject:GetComponentInChildren(UnityEngine.UI.Text, true)
        text.text = frame_helper:getLangCfg(lang_id)
        app:startCoroutine(
            function()
                coroutine.yield(WaitForSeconds(time))
                if self.in_battle or game_state_level.leave_map then
                    return
                end
                bubble:SetActive(false)
                if callback then
                    callback()
                end
            end
        )
    end,

    -- 显示对象头顶气泡
    showChat = function(self, target, lang_id, time, callback, is_go)
        local parent = nil
        if is_go then
            parent = self:findChild(target.transform, "head")
        else
            parent = target:FindChild("head")
        end
        if not parent then
            log.error('Failed to find head:'..target.transform.name)
            return
        end

        local _cb = function (go)
            -- callback
            if go ~= nil then
                go.name = "HeadBubble"
                go.transform.position = parent.position
                go.transform.rotation = parent.rotation
                go.transform.localScale = Vector3.one
                go.transform:SetActive(true)
                if is_go then
                    go.transform:SetParent(self.level.tank_root)
                else
                    go.transform:SetParent(parent)
                end
                local text = go:GetComponentInChildren(UnityEngine.UI.Text, true)
                text.text = frame_helper:getLangCfg(lang_id)

                app:startCoroutine(
                    function()
                        coroutine.yield(WaitForSeconds(time))
                        if self.in_battle or game_state_level.leave_map then
                            return
                        end

                        if go and go.transform then
                            go.transform:SetActive(false)
                        end

                        if callback then
                            callback()
                        end

                    end
                )
            else
                log.error('show bubble loadResource go is nil')
            end
        end

        local prefab_path = "Assets/Prefab/Level/HeadBubble.prefab"

        local uobject = self.pool:instantiate(prefab_path)
        if not uobject then
            log.error('Failed to load bubble:'..prefab_path)
            return
        end

        if uobject.isLoaded then 
            _cb(uobject)
        else 
            uobject.onLoaded = _cb
        end


    end,

    -- 隐藏Player头顶气泡
    removePlayerChat = function(self)
        local bubble = self.level.player.cs_inst:FindChild("HeadBubble")
        if bubble then
            bubble:SetActive(false)
        end
    end,

    -- 显示对话选项 options = {lang_id, lang_id, lang_id}
    showDialogOption = function(self, title, options, callback)
        frame_manager:show("dialog_option", nil, {title = title, options = options, callback = callback})
    end,

    -- 开始巡逻
    beginPatrol = function(self, entity_id, points, speed)
        local entity = self.level:getMapEntity(entity_id)
        if entity and entity.cs_inst then
            entity:beginPatrol(points, speed)
        end
    end,

    -- 开始巡逻
    beginOrderPatrol = function(self, entity_id, points, speed)
        local entity = self.level:getMapEntity(entity_id)
        if entity and entity.cs_inst then
            entity:beginOrderPatrol(points, speed)
        end
    end,

    -- 创建操作区域
    -- id - 区域的编号
    -- op_type - 操作类型，用于对应按钮上的文字
    -- prefab - 触发区域资源路径。应该用单位大小的碰撞
    -- pos - 位置
    -- r - 半径，会进行缩放
    -- callback - 用户按了按钮进行操作的回调
    createOpTrigger = function(self, id, op_type, point_id, callback)
        local cls = require("level.level_operation_trigger")
        local trigger = cls(id, op_type, point_id, callback)

        table.insert(self.op_triggers, trigger)
    end,

    -- 根据 id 删除交互区域
    removeOpTrigger = function(self, id)
        for k,v in pairs(self.op_triggers) do
            if v and v.id == id then
                v:destroy()
                self.op_triggers[k] = false
                break
            end
        end
    end,

    -- 是否所有交互区域都被用户交互过了
    isAllOpTriggerInteracted = function(self)
        for k,v in pairs(self.op_triggers) do
            if v then
                if not v.is_interacted then
                    return false
                end
            end
        end
        return true
    end,

    --添加关卡的阶段
    addPhases = function(self, name, phase)
        self.phases[name] = phase
    end,

    -- 改变关卡的阶段
    changePhase = function(self, name)
        if self.phases[name] then
            if self.curr_phase then
                self.curr_phase:onExit()
            end
            self.curr_phase = self.phases[name]
            self.curr_phase:onEnter()
        else

        end
    end,

    -- 播放纸片对话
    playDialog = function(self, id, callback)
        dialog_manager:playScenario(id, callback)
    end,

    -- 播放摄像机动画
    playCameraAnim = function(self, anim)
        if not self.cam_animator then
            self.cam_animator = self.level_manager.mainCamera:GetComponent(UnityEngine.Animation)
        end

        if self.cam_animator then
            self.cam_animator:Play(anim)
        else
            log.error("Failed to find cam animator!!")
        end

    end,

    -- 播放关卡摄像机动画(绝对坐标动画),必须在
    playLevelCameraAnim = function(self, anim, is_hide_fog)
        if not self.level_camera_anim then
            self.level_camera_anim = game_state_level.level_camera:GetComponent(UnityEngine.Animation)
        end

        app:startCoroutine(
            function()

                if self.level_camera_anim then
                    game_state_level.level_camera.enabled = false
                    self.level_camera_anim.enabled = true

                    self.level_camera_anim:Play(anim)
                    local len = self.level_camera_anim:GetClip(anim).length
                    log.info("len:"..len)
                    coroutine.yield(WaitForSeconds(len+1))
                    self.level_camera_anim.enabled = false
                    game_state_level.level_camera.enabled = true


                else
                    log.error("Failed to find level camera animator!!")
                end
            end)
    end,

    -- 添加剧情关卡指挥官
    addStoryLevelHero = function(self, story_hero_id, _formation_id)
        local story_hero_cfg = config.monster_stat_heros[story_hero_id]
        local tank_cfg_id = story_hero_cfg.tank_id
        network_sender:addStoryLevelHero(tank_cfg_id)

        -- 更新阵型
        local heros = level_frame:getCurrentHeros()
        table.insert(heros, tank_cfg_id)
        local formation_id = _formation_id or level_info.formation_id

        local hero_list = formation_helper:autoFormation(heros, formation_id)
        network_sender:copyEmbattle(formation_id, hero_list)

    end,

    -- 移除剧情关卡指挥官
    removeStoryLevelHero = function(self, story_hero_id, _formation_id)
        local story_hero_cfg = config.monster_stat_heros[story_hero_id]
        local tank_cfg_id = story_hero_cfg.tank_id
        network_sender:removeStoryLevelHero(tank_cfg_id)

        -- 更新阵型
        local heros = level_frame:getCurrentHeros()
        for idx,id in ipairs(heros) do
            if id == tank_cfg_id then
                table.remove(heros, idx)
                break
            end
        end

        local formation_id = _formation_id or level_info.formation_id
        local hero_list = formation_helper:autoFormation(heros, formation_id)
        network_sender:copyEmbattle(formation_id, hero_list)
    end,

    -- 自定义阵型
    customFormation = function(self, formation_id, _hero_list)
        local hero_list = {}
        for pos, hero_id in pairs(_hero_list) do
            local story_hero_cfg = config.monster_stat_heros[hero_id]
            table.insert(hero_list, {heroId=story_hero_cfg.tank_id, location=pos})
        end
        network_sender:copyEmbattle(formation_id, hero_list)
    end,

    -- 恢复默认阵型
    defaultFormation = function(self)
        self.gui:autoFormation(self.id, self.level.cfg.default_formation)
    end,

    -- 与某个关卡怪物发生战斗
    battleWithMonster = function(self, entity_id, is_sneak)
        level_helper:battleWithMonster(entity_id, is_sneak)
    end,

    -- 关卡结束
    finishLevel = function(self, is_success)
        network_sender:finishLevel(is_success)
    end,

    --移动至路障
    onMoveToRoadBlock = function(self, point_id)

        local item_shove_num = 999        --获取铲子物品数量
        local choiceCallBack = function(var)
            choice = var
            if choice == 1 then         --使用工具清理
                if level_info:consumeItem(level_info.item_types.shovel) then --如果有铲子才有效，如果没有铲子则回到三个选项
                    item_shove_num = item_shove_num - 1
                    hint:setText('Road block cleared!')

                    self.level:setPathPointType(point_id, 0)
                    self:onClearRoadBlock(point_id)
                    game_state_level.gui:refreshItemNum()
                else
                    hint:setText('You do not have enough shove')
                    self:onMoveToRoadBlock(point_id)
                end
            elseif choice == 2 then     --撞开路障
                --判断汽油是否足够，如果true
                if level_info.gasoline >= 2 then
                    local dice = math.random()
                    --汽油量 -2
                    level_info.gasoline = level_info.gasoline - 2
                    if dice >= 0.8 then
                         --全体血量 -10%
                        network_sender:levelChangeHeroProp(-0.1, 0)
                        hint:setText('你驾驶战车冲过了路障，但铁皮还是在你的战车上留下了很多明显的划痕')
                    else
                        --油箱漏
                        --汽油 -2
                        self:setGasolineLeak(true)
                        hint:setText('你驾驶战车冲过了路障，但是嘲笑你的铁皮插入了战车油箱，大量的油料泄露出来')
                    end
                    game_state_level.gui:refreshHealthAndGasoline()
                    self.level:setPathPointType(point_id, 0)
                    self:onClearRoadBlock(point_id)
                else
                    --如果没有足够汽油
                    hint:setText('你没有足够的汽油了')
                end
            elseif choice == 3 then     --离开
                hint:setText('你转身离开了这里，在开回去的时候似乎听见了路障的嘲笑声越发明显')
            end
        end
        --hint:setText('前面的路被障碍堵住了，该怎么办呢')
        --self:wait(1.5)
        --82 使用铲子  83 撞开路障  84 离开
        self:showDialogOption('level_script_97',{'level_script_82','level_script_83','level_script_84'},choiceCallBack)
    end,
    
    -- 移动到门
    onMoveToDoor = function(self, target_point_id)
    end,

    -- 设置是否漏油
    setGasolineLeak = function(self, is_leak)
        level_info.is_gasoline_leak = is_leak
        game_state_level.gui:refreshHealthAndGasoline()
        if is_leak then
            game_state_level.gui:animLeakIcon()
        end
    end,

    --移动至保险箱
    onMoveToSteelSafe = function(self, point_id)
        --获取工具箱数量

        local choiceCallBack = function(var)
            choice = var
            if choice == 1 then
                if level_info:consumeItem(level_info.item_types.toolkit) then
                    --if 有足够的工具箱
                    --工具箱数量 -1
                    hint:setText('你使用工具打开了保险箱，获取了全部财宝')
                    --获得物品
                    game_state_level:refreshItemNum()
                    self.level:setPathPointType(point_id, 0)
                else
                    --else 没有足够的工具箱，回到三个选项
                    hint:setText('你摸了摸行囊，并没有找到工具箱')
                    self:onMoveToSteelSafe(point_id)
                end

            elseif choice == 2 then
                local dice = math.random()
                if dice >= 0.5 then
                    --获得部分物品
                    hint:setText('你使出全身力气砸开了保险箱，不过部分物品在过程中被损坏了')
                else
                    --保险箱损毁
                    hint:setText('你的力气太大了，整个保险箱都陷入了地底，这下再也拿不出来了！！')
                end 
                self.level:setPathPointType(point_id, 0)
            elseif choice == 3 then
                --离开
                hint:setText('你想了想还是决定离开，也许下次多带些工具箱是个好主意')
            end
        end

        --hint:setText('前面似乎有个保险箱，这下发财了')
        --self:wait(1.5)
        --85 用工具打开  86 强行打开  84 离开
        self:showDialogOption('level_script_98',{'level_script_85','level_script_86','level_script_84'},choiceCallBack)
    end,

    --移动至游民（野外NPC）
    onMoveToTramp = function(self, point_id)

        local choiceCallBack = function(var)
            choice = var
            local path_point = game_state_level:getPathPoint(point_id)
            if choice == 1 then --选择战斗
                --战斗
                log.info("## battle with tramp:"..path_point.m_EntityID)
                level_helper:battleWithMonster(path_point.m_EntityID)
                -- 胜利后消失
                --self.level:setPathPointType(point_id, 0)

            elseif choice == 2 then    --贿赂给食物
                --如果食物足够
                if level_info:consumeItem(level_info.item_types.food) then
                    local dice = math.random()
                    if dice >= 0.8 then
                        --给食物成功
                        --游民离开
                        --获取一些物品
                        hint:setText('你分了点食物给他，他留下了一下物品并且离开了')
                        -- 隐藏 entity
                        local entity = self.level:getEntity(path_point.m_EntityID)
                        self:hideEntity(entity.id)
                        self.level:setPathPointType(point_id, 0)
                    else
                        --给食物失败
                        --返回3个选项
                        hint:setText('你分了点食物给他，可他似乎没有要走的意思，真是可恶！')
                        self:onMoveToTramp(point_id)
                    end
                else
                    --如果食物不够
                    --返回3个选项
                    self:onMoveToTramp(point_id)
                end
            elseif choice == 3 then
                --离开
                hint:setText('你想了想还是决定离开')
            end
        end


        --87 攻击  88 贿赂  84 离开
        self:showDialogOption('level_script_99',{'level_script_87','level_script_88','level_script_84'},choiceCallBack)        
    end,

    --移动到足迹
    onMoveToFootprint = function(self,point_id)
        --89 查看足迹  84 离开
        local choiceCallBack = function(var)
            choice = var
            if choice == 1 then
                local dice = math.random()
                if dice <= 0.33 then
                    --触发战斗
                    hint:setText('你顺着足迹找到了盗贼老巢！')
                    self:wait(1.0)
                    level_helper:battleWithMonster(path_point.m_EntityID)
                elseif dice > 0.33 and dice <= 0.67 then
                    --获得物品
                    hint:setText('顺着足迹你找到了一些被遗留的物品')
                else
                    --汽油 -1
                    --什么都没
                    hint:setText('你顺着足迹找了一个小时，一无所获，唉，真倒霉！')
                    level_info.gasoline = level_info.gasoline - 1
                    game_state_level.gui:refreshHealthAndGasoline()

                end
                self.level:setPathPointType(point_id, 0)
            elseif choice == 2 then
                --离开
            end
        end 
        self:showDialogOption('level_script_100',{'level_script_89','level_script_84'},choiceCallBack)
    end,

    --移动至书籍
    onMoveToBook = function(self,point_id)

        local choiceCallBack = function(var)
            choice = var
            if choice == 1 then --查看书籍
                local dice = math.random()
                if dice <= 1 then
                    --获得增益buff
                    hint:setText('你在书籍中学习到了一些战斗的技巧')
                    network_sender:levelChangeHeroProp(0.0, 0.1)
                else
                    hint:setText('书籍中记载着核战前的花花世界，你突然对现在的生活索然无味')
                    network_sender:levelChangeHeroProp(0.0, 0.1)
                    --获得减益buff

                end

                self.level:setPathPointType(point_id, 0)
            end
        end
        --hint:setText('前面地上有好多书籍，要不要去看看')
        self:wait(1.5)
        --90 查看信息  84 离开
        self:showDialogOption('level_script_101',{'level_script_90','level_script_84'},choiceCallBack)
    end,

    -- 清除了路障
    onClearRoadBlock = function(self, id)

    end,

    -- 显示道具
    showGetItemUI = function(self, items)
        local item_tab = {}
        for id,num in pairs(items) do
            local item = {
                inst_id = id,
                cfg_id = id,
                count = num,
                level = 1,
                cfg = item_helper:getItemCfg(id)
            }
            table.insert(item_tab, item)
        end
        frame_manager:show('getItems', self, {items = item_tab, has_Confirm_Btn = true})
    end,

    -- 设置任务文字
    setMissionText = function(self, idx, text)
        log.error("setMissionText 接口失效！")
        -- 接口失效
        --[[
        self.mission_text[idx] = text
        game_state_level.gui:setMissionText(idx, text)
        ]]
    end,

    -- 增加任务进度
    incQuestProgress = function(self, id, num)
        level_quest_manager:incQuestProgress(id, num or 1)
    end,

    -- 设置任务进度
    setQuestProgress = function(self, id, progress)

    end,

    -- 隐藏 entity
    hideEntity = function(self, id)
        local entity = self.level:getEntity(id)
        if entity then
            entity:changeState("hide")
        else
            log.error("Failed to find entity:"..id)
        end
    end,

    -- 显示 entity
    showEntity = function(self, id)
        local entity = self.level:getEntity(id)
        if entity then
            entity:changeState("idle")
        else
            log.error("Failed to find entity:"..id)
        end
    end,

    -- 使用铲子回调
    onUseShovel = function(self, point_id)
    end,

    -- 设置路点的类型
    setPathPointType = function(self, point_id, type)
        self.level:setPathPointType(point_id, type)
    end,

    -- 获取关卡任务奖励
    getQuestReward = function(self, quest_id)
        network_sender:getCopyTaskReward(self.id, quest_id)
    end,

    -- 获取关卡箱子奖励
    getGiftBagReward = function(self, gift_id)
        network_sender:getCopyGiftBag(self.id, gift_id)
    end,

    --增加补给品 31005 汽油， 31007 饼干， 31008 铲子
    addSupplyItem = function(self, item_cfg_id, num, display_num)
        level_info:addItem(item_cfg_id, num, display_num)
        game_state_level.gui:refreshItemNum()
    end,

    --fps mode fire
    fpsFire = function(self)
        log.error('fire!')
    end,

}

local phase_base = require("level.level_script.level_phase_base")
-- 第一阶段找萝宾
local phase_find_robin = class{
    super = phase_base,

    onEnterTrigger = function(self, trigger_name)
        if trigger_name == "trigger1" then
            self.level_script:changePhase("phase2")
        end
    end,

    onExit = function(self)
        log.info("exit phase1")
    end,
}

-- 第二阶段初战
local phase_first_fight = class{
    onEnter = function(self)
        log.info("onEnter phase2")
        self.phase_elapsed = 0
    end,

    onUpdate = function(self, dt)
        self.phase_elapsed = self.phase_elapsed + dt
    end,
}

-- 使用阶段的关卡脚本
local simple_level_script = class{
    super = level_script_base,
    init = function(self)
        super.init(self)
        self:addPhases("find_robin", phase_find_robin(self))
        self:addPhases("first_fight", phase_first_fight(self))

        self:changePhase("find_robin")
    end,
}

return level_script_base
