local super = require('level.level_actor')
local log = logger.get("Game", "LevelMonster")
local attachment_class = require('RTS.attachment.attachment')
local main_weapon_class = require('RTS.attachment.main_weapon')

return class{
    super = super,

    -- 初始化，加载资源
    init = function(self, pos, rot, root)
        --self:preinit()
        local callback = function (go)
            if not level then
                -- 如果这时 level 被销毁了，不处理后续逻辑
                return
            end
            
            -- callback
            if go ~= nil then
                self.gameObject = go
                self.gameObject:SetActive(true)

                self.root = root
                go.transform:SetParent(root)
                level_helper:makeTransformIdentity(go.transform)
                go.layer = UnityEngine.LayerMask.NameToLayer("Tank")

                go.name = 'tank_'..self.team..'_'..self.id
                --log.info("*** callback ##### create level tank:"..go.name)

                -- 资源加载完成回调，子类实现
                self:onResLoaded(pos, rot)
                local scale = self.cfg.level_scale * 0.0001
                self.cs_inst.transform.localScale = Vector3.New(scale,scale,scale)
                self.cs_inst:SetAvoidanceQuality(0)

                self.minor_weapons = {}
                self:initMainWeapon()
                self:initWheel()
                self:initHeadIcon()
                self:initTrailerBody()
                self:initPathPoint()    -- 初始化路点

                self.is_initialized = true
                self.events:dispatch("actor_loaded", self)
            else
                log.error('loadResource go is nil')
            end
        end

        if not self.prefab_path or self.prefab_path == "" then
            self.prefab_path = "Assets/Prefab/Vehicle/Prefab_Vehicle_Assult_Pickup.prefab"
        end

        --log.info("#### Create tank prefab:"..self.prefab_path..', root:'..root.name)
        local uobject = game_director.pool:instantiate(self.prefab_path)
        if not uobject then
            log.error('Failed to load level_tank:'..self.prefab_path)
            return
        end

        if uobject.isLoaded then 
            app:startCoroutine(
                function()
                    -- 回调要延迟一帧调用，保证一致性
                    coroutine.yield(nil)
                    callback(uobject)
                end
            )
        else 
            uobject.onLoaded = callback
        end

        self.head_light_1 = nil
        self.head_light_2 = nil
    end,

    -- 初始化主武器
    initMainWeapon = function(self)
        if self.weapon_id and self.weapon_id > 0 then
            local attach_point = self.cs_inst:FindChild('z_wq')
            self.main_weapon = main_weapon_class(self, attach_point, self.weapon_id, function(go)
                self:onAttachmentLoaded(go)
            end)
        end
    end,

    -- 初始化轮胎
    initWheel = function(self)
        if self.wheel_id and self.wheel_id > 0 then
            local attach_point = "Wheel"--self.cs_inst:FindChild('lt_1')
            self.wheel = attachment_class(self, attach_point, self.wheel_id, function(go)
                self:onAttachmentLoaded(go)
               
                -- 将轮胎的动画组件设置给 cs_inst
                --[[
                local anim = go:GetComponentInChildren(UnityEngine.Animation, true)
                if anim then
                   self.cs_inst:AddAnimationComp(anim)
                end]]

                self:onWheelLoaded(go)
            end)
        end
    end,

    -- 初始化拖挂
    initTrailerBody = function(self)
        if self.cfg.relatetank and self.cfg.relatetank > 0 then
            self.taiiler_cfg = config.tank_basic_ids[self.cfg.relatetank]

            local prefab_path = self.taiiler_cfg.prefab1
            if self.team == 1 then
                prefab_path = self.taiiler_cfg.prefab2
            end

            local uobject = game_director.pool:instantiate(prefab_path)
            if not uobject then
                log.error('Failed to load TrailerBody:'..prefab_path)
                return
            end

            local callback = function(go)
                self.trailer_body = go:GetComponentInChildren(TrailerBody)
                if not self.trailer_body then
                    log.error("Failed to find TrailerBody component")
                    return
                end
                go.transform.localScale = Vector3.New(level_tank_scale,level_tank_scale,level_tank_scale)
                self.trailer_body:ConnectTo(self.cs_inst)
                --go.transform:SetParent(self.root)
                --self.trailer_body.transform.localScale = Vector3.New(level_tank_scale,level_tank_scale,level_tank_scale)
                go.name = 'tank_trailer'
                
                self:initTrailerWheel()
            end

            if uobject.isLoaded then 
                callback(uobject)
            else 
                uobject.onLoaded = callback
            end
        else
            --log.info("# No relatetank:"..tostring(self.cfg_relatetank))
        end
    end,

    -- 初始化挂载轮胎
    initTrailerWheel = function(self)
        if self.wheel.cfg.relatewheel and self.wheel.cfg.relatewheel > 0 then
            local attach_point = "Wheel" --self.trailer_body:FindChild('lt_1')
            self.trailer_wheel = attachment_class(self, attach_point, self.wheel.cfg.relatewheel, function(go)
                -- 将轮胎的动画组件设置给 cs_inst
                local anim = go:GetComponentInChildren(UnityEngine.Animation, true)
                if anim then
                   self.cs_inst:AddAnimationComp(anim)
                end
            end)
        end
    end,

    onWheelLoaded = function(self, go)

    end,

    -- 初始化所有副武器
    initMinorWeapons = function(self)
        return
        --[[
        local _init = function(self, idx, id)
            local attach_point = self:getMinorWeaponAttachPoint(idx)
            if attach_point then
                self.minor_weapons[idx] = attachment_class(self, attach_point, id, function(go)
                    self:onAttachmentLoaded(go)
                end)
            else
                log.error('Failed to find minor weapon attach point:'..idx)
            end
        end

        -- 先使用坦克表配置的默认副武器
        if self.cfg.point1 > 0 then _init(self, 1, self.cfg.point1) end
        if self.cfg.point2 > 0 then _init(self, 2, self.cfg.point2) end
        if self.cfg.point3 > 0 then _init(self, 3, self.cfg.point3) end
        if self.cfg.point4 > 0 then _init(self, 4, self.cfg.point4) end]]
    end,

    -- 初始化路点信息，找一个最近的路点作为当前点
    initPathPoint = function(self)

        local pp = game_state_level:getNearestPathPoint(self.transform.position)
        self.path_point_id = pp.m_ID    -- 当前路点 id
        self.path_point_target = pp.m_ID    -- 目标路点，有值说明正在移动中
        self.last_path_point_id = pp.m_ID   -- 上一个 path point
        self.is_moving = false          -- 是否在移动中

        --log.info("## current point:"..self.path_point_id)
        --[[
        // 点的类型
        public enum EType
        {
            None = 0,
            RoadBlock = 1,  // 路障
            SteelSafe = 2,  // 保险箱
            Tramp = 3,      // 游民
            Footprint = 4,  // 足迹
            Book = 5,       // 书籍
        }]]
        if pp.m_Type == 3 and pp.m_EntityID == self.id then
            self.is_tramp = true    -- 设定成游民怪
        end

        return self.path_point_id
    end,

    onAttachmentLoaded = function(self, go)
        if go then
            go.layer = UnityEngine.LayerMask.NameToLayer("Tank")
        end
    end,

    -- 找到副武器挂载点
    getMinorWeaponAttachPoint = function(self, idx)
        local point_name = 'f_wq_'..idx
        return self.cs_inst:FindChild(point_name)
    end,

    -- 初始化装饰物
    initDecoration = function(self)
    end,

    -- 初始化头顶 icon
    initHeadIcon = function(self)
        self.head_node = self.cs_inst:FindChild('head')
        if not self.head_node then
            return
        end

        local icon_trans = self.cs_inst:FindChild("head_icon")
        
        if icon_trans then
            self.head_icon = icon_trans.gameObject:GetComponentInChildren(UnityEngine.UI.Image, true)
            self.head_icon.gameObject:SetActive(false)
            self.head_icon_text = icon_trans:Find("Icon/Text"):GetComponent(UnityEngine.UI.Text)
            return
        end

        local callback = function (go)
            -- callback
            if go ~= nil then
                go.name = "head_icon"
                go.transform.position = self.head_node.position
                go.transform.rotation = self.head_node.rotation
                go.transform:SetParent(self.head_node)
                self.head_icon = go:GetComponentInChildren(UnityEngine.UI.Image, true)
                self.head_icon.gameObject:SetActive(false)
                self.head_icon_text = go:GetComponentInChildren(UnityEngine.UI.Text, true)
            else
                log.error('initHeadIcon loadResource go is nil')
            end
        end

        local prefab_path = "Assets/Prefab/Level/HeadIcon.prefab"

        local uobject = game_director.pool:instantiate(prefab_path)
        if not uobject then
            log.error('Failed to load level_tank:'..prefab_path)
            return
        end

        if uobject.isLoaded then 
            callback(uobject)
        else 
            uobject.onLoaded = callback
        end
    end,

    moveTo = function(self, pos, speed, callback)
        if self.cs_inst then 
            self.cs_inst.enabled = true
            self.cs_inst:PlayAnimation("run")

            if speed then
                self.cs_inst:SetMoveParam(speed, speed, 360, 1.0)
            end
            --print("set dest", pos, debug.traceback())
            self.cs_inst:SetDestination(pos)
            self.move_callback = callback
            self.move_target = pos

            log.info("moveTo:"..tostring(pos)..', speed:'..speed)
        end
    end,

    stopMove = function(self)
        if self.cs_inst then 
            self.cs_inst:StopMove()
            self.cs_inst.enabled = false
        end
    end,

    -- 移动到路点
    moveToPathPoint = function(self, id, speed, callback, is_move_half, is_move_offset)
        local p0 = game_state_level:getPathPoint(self.path_point_id).position   -- 当前点
        local p1 = game_state_level:getPathPoint(self.path_point_target).position   -- 目标点

        local pos = p1
        if is_move_half then
            pos = Vector3.Lerp(p0, p1, 0.35)
        elseif is_move_offset then
            pos = Vector3.Lerp(p0, p1, 0.7)
        end

        self.last_path_point_id = self.path_point_id
        self.path_point_target = id
        self.path_point_id = id

        self:moveTo(pos, speed, 
            function()
                if callback then
                    callback()
                end
            end)

    end,

    -- 更新一个回合
    updateTurn = function(self, is_move_half, is_move_offset)
        if self.path_point_id == self.path_point_target then
            return
        end

        self.is_turn_moving = true

        --log.info("## tank:"..self.id.." move to point:"..self.path_point_target)
        self.cs_inst.m_IsInputBreak = true
        self:moveToPathPoint(self.path_point_target, 100, 
        function()
            log.info("## tank:"..self.id.." move callback")
            self.is_turn_moving = false
            app:startCoroutine(
                function()
                    self:playAnimation("stop")
                    coroutine.yield(WaitForSeconds(1.0))
                    self:playAnimation("idle")
                    self.cs_inst.m_IsInputBreak = false
                end
            )

        end, 
        is_move_half, is_move_offset)
    end,

    -- 根据名称搜索子结点
    findChild = function(self, bone_name)
        return self.cs_inst:FindChild(bone_name)
    end,

    -- 显示头顶图标
    showIcon = function(self, icon_path, text, fade_fly)
        self.head_icon.transform.localPosition = Vector3.zero
        self.head_icon.gameObject:SetActive(true)
        self.head_icon:SetSprite(icon_path)
        self.head_icon.color = Color.white
        self.head_icon_text.color = Color.white

        if text then
            self.head_icon_text.text = text
            self.head_icon_text.gameObject:SetActive(true)
        else
            self.head_icon_text.text = text
            self.head_icon_text.gameObject:SetActive(false)
        end

        if fade_fly then
            local y = self.head_icon.transform.position.y + 20
            local lt = LeanTween.moveY(self.head_icon.gameObject, y, 1.0)
            lt.loopType = LeanTweenType.once
            lt:setOnComplete(function()
                self:hideIcon()
            end)

            lt = LeanTween.color(self.head_icon.transform, Color(1,1,1,0), 0.5)
            lt.delay = 0.5
            lt.loopType = LeanTweenType.once

            lt = LeanTween.textColor(self.head_icon_text.transform, Color(1,1,1,0), 0.5)
            lt.delay = 0.5
            lt.loopType = LeanTweenType.once
        end

    end,

    -- 隐藏头顶图标
    hideIcon = function(self)
        self.head_icon.gameObject:SetActive(false)
    end,

    -- 播放动画
    playAnimation = function(self, anim_name)
        if self.cs_inst then
            self.cs_inst:PlayAnimation(anim_name)
        end
    end,

    update = function(self, dt)
        -- 如果有移动回调，判断是否移动到目标点
        if  self.move_target then
            if not self.cs_inst.m_bIsMoving then
                self.move_target = nil

                app:startCoroutine(
                    function()
                        log.info("## Play Stop!!!")
                        coroutine.yield(nil)
                        if self.move_callback then
                            --log.info("## callback")
                            self.move_callback()
                            --self.move_callback = nil
                        else
                           -- log.info("## No callback")
                        end

                        self.cs_inst:PlayAnimation("stop")
                        coroutine.yield(WaitForSeconds(1.0))
                        self.cs_inst:PlayAnimation("idle")
                    end
                )

                if self.onMoveToDestination then
                    self:onMoveToDestination()
                end

            end
        end
    end,

        -- 销毁该对象
    destroy = function (self)
        if self.is_initialized then
            if self.main_weapon then
                self.main_weapon:destroy()
            end

            if self.wheel then
                self.wheel:destroy()
            end

            for _,weapon in pairs(self.minor_weapons) do
                weapon:destroy()
            end

            if self.head_light_1 then
                self.head_light_1:destroy()
                self.head_light_1 = nil
            end

            if self.head_light_2 then
                self.head_light_2:destroy()
                self.head_light_2 = nil
            end
        end

        super.destroy(self)
    end, 

    -- 脚底光圈
    initFeetHalo = function(self, path)
        self.head_node = self.cs_inst:FindChild('head')
        if not self.head_node then
            return
        end

        self.feet_halo = self.cs_inst:FindChild("feet_halo")

        if self.feet_halo then
            return
        end

        local callback = function (go)
            -- callback
            if go ~= nil then
                go.name = "feet_halo"
                local pos = self.head_node.position
                pos.y = 0.1
                go.transform.position = pos
                go.transform.rotation = self.head_node.rotation
                go.transform:SetParent(self.head_node)
            else
                log.error('initIcon loadResource go is nil')
            end
        end

        self.feet_halo = game_director.pool:instantiate(path)
        if not self.feet_halo then
            log.error('Failed to load monster icon:'..path)
            return
        end

        if self.feet_halo.isLoaded then 
            callback(self.feet_halo)
        else 
            self.feet_halo.onLoaded = callback
        end
    end,

    -- 初始化车头灯
    initHeadLight = function(self)
        -- Assets/Prefab/Level/Vehicle_Headlight.prefab
        local head_light_node_1 = self.cs_inst:FindChild('Headlight_1')
        local head_light_node_2 = self.cs_inst:FindChild('Headlight_2')

        local loadHeadLight = function(parent, path)
            local callback = function (go)
                -- callback
                if go ~= nil then
                    go.transform.position = parent.position
                    go.transform.rotation = parent.rotation
                    go.transform:SetParent(parent)
                else
                    log.error('initIcon loadResource go is nil')
                end
            end

            local go = game_director.pool:instantiate(path)    
            if go.isLoaded then
                callback(go)
            else
                go.onLoaded = callback
            end
            return go
        end

        if head_light_node_1 then
            self.head_light_1 = loadHeadLight(head_light_node_1, "Assets/Prefab/HeadLight/Vehicle_Headlight_left.prefab")
        else
            log.error("Failed to find Headlight_1")
        end

        if head_light_node_2 then
            self.head_light_2 = loadHeadLight(head_light_node_2, "Assets/Prefab/HeadLight/Vehicle_Headlight_right.prefab")
        else
            --log.error("Failed to find Headlight_2")
        end
    end,

    -- 玩家向我移动了
    onPlayerMoveToMe = function(self)
        if self.feet_halo then
            app:startCoroutine(function()
                self.feet_halo:SetActive(false)
                coroutine.yield()
                self.feet_halo:SetActive(true)
            end)
        end
    end,

    setHeadLightEnable = function(self, enabled)
        if enabled then
            if self.head_light_1 then
                self.head_light_1:SetActive(true)
                self.head_light_2:SetActive(true)
            else
                self:initHeadLight()
            end
            self.light_enabled = true
        else
            if self.head_light_1 then
                self.head_light_1:SetActive(false)
            end

            if self.head_light_2 then
                self.head_light_2:SetActive(false)
            end
            self.light_enabled = false
        end
    end,

    isHide = function(self)
        return false
    end,

}
