-- 全局消息处理器
local log = logger.get("Game", "Message handler")
require ('RTS.bag_sys.bag_manager')
require ('RTS.hero_sys.hero_manager')
require ('game.player')
require ('RTS.task_sys.task_manager')
local item_cls = require('RTS.bag_sys.item')
require('systems.shop_sys.shop_manager')
local fame_manager = require('systems.fame_sys.fame_manager')
require('systems.announce_sys.announce_manager')
require('systems.dialog_sys.dialog_manager')
require("level.level_quest_manager")
-- 公会
require('game.guild')
-- 领主
require('game.lord')
require('RTS.crew_sys.crew_manager')
-- BUFF
require('RTS.home_sys.buff_info')
-- 关卡布阵
require('level.level_formation')
-- 世界布阵
require('world_map.world_map_formation')
require('systems.chapter_achievement_manager')
-- 世界战报战斗
require('config.soldier')
require('systems.chapter_manager')
require ('RTS.redpoint_sys.red_point_manager')

-- 活动管理
activity_manager = require("game.activity_manager")

local msg_handlers = {
    [protocol_pb.CLIENTGATEWAYHEARTBEAT] = function(self, msg)
        game_director:onPing(msg.time)
    end,

    --[[
    //英雄信息通知
    message SCHeroInfo
    {
      repeated MFullHeroInfo FullHeroInfos 		= 1; //英雄信息
      repeated MNoneOfficerInfo noneOfficers	= 2; //未拥有的指挥官
    }]]
    [protocol_pb.HEROINFONTF] = function (self, msg)
        -- 已激活的英雄列表 
        if msg.FullHeroInfos then 
            -- 正常通知英雄信息
            for heroInfoIndex, heroInfoValue in ipairs(msg.FullHeroInfos) do 
                local hero_info = heroInfoValue.HeroInfo -- 英雄信息
                local hero_props = heroInfoValue.ParPropIdValues -- 英雄属性列表 
                local global_props = heroInfoValue.playerCfgPropVal -- playerCfg属性列表 
                if heroInfoValue.isNewHero and heroInfoValue.isNewHero == 1 then
                    --log.info("add new hero!")
                    hero_manager.new_heros[hero_info.Id] = heroInfoValue
                    hero_manager:createHero(hero_info, hero_props, global_props)
                else
                    -- 创建英雄
                    --log.info("create old hero!")
                    hero_manager:createHero(hero_info, hero_props, global_props)
                end
                --[[
                 if hero_props then 
                     for ppk, ppv in ipairs(hero_props) do 
                         local propId = ppv.PropId
                         local propValue = ppv.PropValue
                         print(heroInfoIndex .. ">     (" .. ppk .. ") 属性", propId, "=", propValue)
                     end
                 end]]

                --print(heroInfoIndex .. "> 英雄id", hero_info.Id)
                --print(heroInfoIndex .. "> 英雄状态", hero_info.HeroStatus)
            end
        end  -- msg.FullHeroInfos

        --[[
        message MNoneOfficerInfo
        {
            required uint32 heroId	= 1;//英雄唯一id
            required uint32 tankId	= 2;//坦克id
            required uint32 officerId=3;//指挥官id
        }]]
        -- 未激活的英雄列表
        if msg.noneOfficers then
            for _, info in ipairs(msg.noneOfficers) do
                hero_manager:createUnactivedHero(info.heroId, info.tankId, info.officerId)
            end
        end
    end, 

    -- 英雄数值或者数值资源发生变化
    --[[
    message Notify_PlayerProp
    {
        required PLAYER_PROP propId = 1;
        required uint32 propValue = 2;
    }]]
    [protocol_pb.SVRPROPCHANGE] = function(self, msg)
        for _,v in ipairs(msg.proplist) do
            local_player:onPropChange(v.propId, v.propValue)
        end
        local_player:dispatchEvents()
    end,

    -- player_config 属性
    [protocol_pb.GLOBALPROPNOTIFY] = function(self, msg)
        for _,v in ipairs(msg.propList) do
            local_player:onGlobalPropChange(v.PropId, v.PropValue)
        end
    end,

    -- 进入地图
    [protocol_pb.ENTERMAP] = function (self, msg)
        -- 登陆成功 
        if msg.RespCode == 0 then
            level_info:reset()
            level_info.map_info = msg    -- 保存地图信息
            local level_cfg = config.game_copy_levels[msg.mapId]
            level_info.map_id = msg.mapId
            level_info.level_cfg = level_cfg
            level_info.fog_data = nil
            level_info.player_pos = nil
            
            -- 重置关卡任务
            level_quest_manager:beginLevelQuest(msg.mapId)
            level_info.finished_quest = msg.doneCopyTask
            for _,id in ipairs(level_info.finished_quest) do 
                log.info("### finished quest:"..id)
            end

            game_director:changeState('loading', {next_state = 'level', scene_name = level_cfg.Map, level_id = msg.mapId})

            if game_info.chapter and game_info.chapter.AreaMaps then
                for k, v in ipairs(game_info.chapter.AreaMaps) do
                    v.copyId = msg.mapId
                end
            end

            -- 关卡内的资源数据保存成本地数据
            --[[level_info:setResouce(consts.boostId, msg.boostCoin)
            level_info:setResouce(consts.psId, msg.copyEnergy)
            for k, v in ipairs(msg.copyResource) do
                frameHelper.log(v.ItemTemId .. '   '  ..v.Count)
                level_info:setResouce(v.ItemTemId, v.Count)
            end]]
            -- for __, entity in ipairs(msg.mapEntities) do 
            -- end
        else
            log.info('enter failed, RespCode:'..msg.RespCode)
        end
    end, 

    -- message SCMapEntityChange
    -- {
    -- 	repeated MapEntity appearEntities = 1; //新出现的实体
    -- 	repeated uint32 disappearEntities = 2; //消失的实体
    -- }
    -- 地图实体变化 (可能在战斗结算时收到，也可能在关卡内收到)
    [protocol_pb.ENTITYCHANGE] = function (self, msg)
        -- for __,entity in ipairs(msg.appearEntities) do
        -- end
        table.insert(level_info.entityChanged, msg)
        game_director.events:dispatch("ENTITY_CHANGED", msg)
    end, 

    [protocol_pb.CHAT] = function (self, msg)
        print("CHAT_RSP:", msg.RespCode, msg.ChatChannel)
    end, 

    -- #################################################### 章节 start

    [protocol_pb.AREAMAPNTF] = function (self, msg)
        --[[
        message SCAreaMap
        {
          required uint32   CurAreaID          = 1;       //章节ID
          required EChapterState chapterState  = 2;		  //章节状态
          optional uint32 copyId               = 3;       //当前未通过的关卡 0表示已全部通过
        }]]

        log.info("### protocol_pb.AREAMAPNTF")

        -- 服务器只会发已解锁的章(资源船)
        for _, area_map in ipairs(msg.AreaMaps) do
            local chapter_id= area_map.CurAreaID
            chapter_manager:initChapterInfo(chapter_id, area_map.chapterState, area_map.copyId)
        end

        --[[for __, map in ipairs(msg.AreaMaps) do
            frame_helper.log("CurAreaID:" .. map.CurAreaID .. "  copyId:" .. map.copyId .. "  ")
            for ___, area in ipairs(map.AreaPosInfos) do
                frame_helper.log("AreaPosID:" .. area.AreaPosID .. "  TodayPassNum:" .. area.TodayPassNum)
            end
        end]]
    end, 

    -- 设置 chapter 状态
    [protocol_pb.SETCHAPTERSTATE] = function(self, msg)
        if msg.respCode == 0 then
            --[[
            common_pb.
            ECHAPTER_NO_OPEN = 0
            ECHAPTER_NO_TRIGGER = 1
            ECHAPTER_TRIGGER = 2            
            ECHAPTER_OCCUPY = 3
            ECHAPTER_OVER = 4

            ]]

            log.info("### chapter_manager:setChapterInfo:"..msg.chapterId..", "..msg.state..","..msg.curLevel)
            chapter_manager:setChapterInfo(msg.chapterId, msg.state, msg.curLevel)
        end
    end,

    -- 资源船战斗/侦察回应
    [protocol_pb.CHAPTERFIGHT] = function(self, msg)
        if msg.respCode == 0 then
            if msg.action == protocol_pb.ECHAPTER_FIGHT then
                -- 调用 battle_record_frame 的函数
                local battle_record_frame_cls = require('gui.battle_record_frame')
                battle_record_frame_cls.replayBattle({mail={mailId=nil}}, msg.batResult)
            end
        end
    end,

    -- 章解锁关卡
    [protocol_pb.UNLOCKCOPY] = function(self, msg)
        if msg.respCode == 0 then
            log.info("### receive  unlock level:"..msg.chapterId..", "..msg.copyId)
            local chapter_id = msg.chapterId
            local chapter_info = chapter_manager:getChapterInfo(chapter_id)
            if chapter_info then
                chapter_info.curr_level = msg.copyId
            else
                log.error('Failed to find chapter info:'..chapter_id)
            end
            if level_select_frame then
                level_select_frame:onLevelUnlock(chapter_id, msg.copyId)
            end
        else
            log.error("# unlock level failed:"..msg.respCode)
        end
    end,

    -- 章节任务 由于没有主界面,暂时放这里
    [protocol_pb.CHAPTERTASKINFO] = function (self, msg)
        game_info.chapterTask = msg
        local rst = frame_manager:isDisplay('chapterTask')
        if rst then
            game_director.events:dispatch("CHAPTERTASKINFO")
        else
            frame_manager:show('chapterTask')
        end
    end, 

    -- 章节任务 由于没有主界面,暂时放这里
    [protocol_pb.GETCHAPTERREWARD] = function (self, msg)
        if msg.respCode == 0 then
            network_sender:getChapterTask(1)
            frame_manager:show('reward', nil, config.game_copy_chapter_quests[msg.taskId].Reward)
        else
            log.info("get chapter task award failed ! code ",msg.respCode)
        end
    end, 

    -- #################################################### 章节 end

    -- ################道具系统 start ###################
    --[[
    message MItemInfo
    {
        required string         instId      = 1;  //道具唯一ID
        optional uint32        cfgId      = 2;  //道具模板ID
        optional ITEM_STATUS   status      = 3;  //道具是已装备还是在背包
        optional uint32        count       = 4;  //道具个数
    }]]
    [protocol_pb.ITEMINFONTF] = function (self, msg)
    --log.info('############## item info!!')
        for _, item in ipairs(msg.ItemInfos) do
            --log.info('############## add item:'..item.cfgId)
            bag_manager:setItem(item)
        end
    end, 

    [protocol_pb.SVRITEMCHANG] = function (self, msg)
        --log.info('############## item change!!')

        local add_items = {}
        
        for _, change in ipairs(msg.ItemChanges) do
            local item = change.ItemInfo
            log.info('item change, cfgId:'..item.cfgId..', instId:'..item.instId..', count:'..item.count..', status:'..tostring(item.status))
            
            if change.ChangeType == protocol_pb.IU_ADD then
                bag_manager:setItem(item)

            elseif change.ChangeType == protocol_pb.IU_DELETE then
                if item.count <= 0 then
                    if item.cfgId ~= 0 then
                        bag_manager:removeItem(item.cfgId, item.instId, item.count)
                    else
                        log.error('handle protocol_pb.SVRITEMCHAN IU_DELETE failed!! cfgId:'..item.cfgId..', instId:'..item.instId)
                    end
                else
                    bag_manager:setItem(item)
                end
            elseif change.ChangeType == protocol_pb.IU_UPDATE then
                 --bag_manager:setItem(item)
                 bag_manager:refreshItem(item)
            end

            if msg.reason == common_pb.RESET_EQUIP or  msg.reason == common_pb.RESOLVE_ITEM then
                if change.changeNum > 0 then
                    local tmp_item = item_cls(item)
                    tmp_item.count = change.changeNum
                    table.insert(add_items, tmp_item)
                end
            end
        end 

        if msg.reason == common_pb.RESET_EQUIP then
            -- 重置
            frame_manager:show('getItems', self, {items = add_items, has_Confirm_Btn = true})
        elseif msg.reason == common_pb.RESOLVE_ITEM then
            -- 分解
            local msg = NativeText.GetLangStr(110523)
            frame_manager:show('getItems', self, {items = add_items, has_Confirm_Btn = true,message = msg})
        end

    end, 

    -- 替换装备
    [protocol_pb.HEROEQUIP] = function(self, msg)
        --log.info('############## receive hero equip!!')
        if msg.respCode == 0 then
            bag_manager:onHeroEquip(msg)
        else
            log.info("equip failed ! code: ",msg.respCode)
        end
    end,

    -- 道具升级 (主武器副武器)
    [protocol_pb.UPGRADEITEM] = function(self, msg)
        --log.info('############# receive upgrade item!!')
        item_helper:handleUpgradeItem(msg)
    end,

    -- 使用道具
    [protocol_pb.USEITEM] = function(self, msg)
        --log.info('############ receive use item!!', msg.respCode)
        bag_manager.events:dispatch("onUseItem", msg)
    end,

    -- 装备分解
    [protocol_pb.RESOLVEEQUIP] = function(self, msg)
        if msg.respCode == 0 then
            bag_manager.events:dispatch("onWeaponResolve")
        else
            log.info("resolve equipment error ! code: ",msg.respCode)
        end
    end,

    -- ################道具系统 end #####################

    --########### 指挥官养成 start ####################
    --[[//指挥官经验改变通知
    message SCOfficerExpChange
    {
        repeated MOfficerExInfo expInfo = 1; //指挥官经验、等级信息
    }
    //指挥官经验等级信息
    message MOfficerExInfo
    {
        required uint32  id           = 1;  //指挥官id
        required uint32  Lv        = 2;  //当前等级
        required uint32  Exp       =	3;  //当前经验
    }]]
    [protocol_pb.OFFICEREXPCHANGE] = function(self, msg)
        for _,v in ipairs(msg.expInfo) do
            local officer = hero_manager:getOfficer(v.id)
            if officer then
                log.info('### officer exp change, lv:'..v.curLevel..', exp:'..v.exp)
                local old_level = officer.level
                officer:expChange(v.exp, v.curLevel)

                local lv_changed = v.Lv ~= old_level
                if lv_changed then
                    --frame_manager:show('officer_level_up', nil, officer)
                end

                hero_manager.events:dispatch("OFFICEREXPCHANGED", {id = v.id, changed = lv_changed})
            else
                log.error('OFFICEREXPCHANGE Failed to find officer:'..v.id)
            end
        end
    end,

    --[[message SCOfficerPromote
    {
        required int32   respCode     = 1;
        optional int32   id           = 2;  //指挥官Id
        optional int32   quality      = 3;  //军衔
        
    }]]
    [protocol_pb.OFFICERPROMOTE] = function(self, msg)
        if msg.respCode == 0 then
            local officer = hero_manager:getOfficer(msg.id)
            if officer then
                officer:promote(msg.quality)
                hero_manager.events:dispatch("OFFICERPROMOTE", msg)
            else
                log.error('## Failed to find officer to promote:'..msg.id)
            end
        else
            log.info('# Officer Promote failed, code:'..msg.respCode)
        end
    end,

    
    [protocol_pb.OFFICERSTARUP] = function(self, msg)
        if msg.respCode == 0 then
            hero_manager.events:dispatch("OFFICERSTARUP")
        else
            log.info('# Officer Star Up failed, code:'..msg.respCode)
        end
    end,

    -- 指挥官属性变化
    [protocol_pb.PARTNERPROPCHANGE] = function(self, msg)
        local hero = hero_manager.heros[msg.heroId]
        if hero then
            hero:updateHeroProps(msg.ParPropIdValues)
            hero:updateState(msg.heroState)
            hero:updateStrength(msg.strength)
            hero:updateBattleForce(msg.sword)
            hero.officer:updateSkill(msg.skill)
        end
        hero_manager.events:dispatch("PARTNERPROPCHANGE", msg)
    end,

    ---------------任务系统----------------
    --[[
    message MTaskInfo
    {
        required uint32        Id      = 1;  //任务ID
        required TASK_STATUS   Ts      = 2;  //任务状态
        repeated MTASKPROGRE   taskProger   = 4;  //任务进度
    }
    ]]
    -- 任务信息下发通知
    [protocol_pb.NOTIFYTASKINFO] = function(self, msg)
        task_manager:refresh(msg.taskInfos)
    end,

    --[[
    message SCActivePointInfo
    {
        required uint32   activeValue   = 1;  //活跃值
        repeated uint32   getPhase   = 2;  //已经领取过奖励的阶段
    }
    ]]
    -- 活跃值下发
    [protocol_pb.NOTIFYACTIVEPOIOT] = function(self, msg)
        task_manager:refreshActiveness(msg)
    end,

    --[[enum Task_RewardType
    {
        Norm_Task  = 1;   //普通任务奖励
        ACTIVE_POINT = 2; //活跃点奖励
        Rese_Task  = 3;   //研发任务奖励
        Rese_Phase = 4;   //研发任务阶段奖励
    }]]
    -- 领取奖励
    [protocol_pb.GETTASKREWARD] = function(self, msg)
        if msg.respCode == 0 then
            if msg.awardType == common_pb.Norm_Task then
                task_manager:onGetRewards(msg.taskId)
            elseif msg.awardType == common_pb.Growth_Task then
                log.info("### GetTaskReward:"..msg.taskId)
                task_manager:onGetGrowthTaskRewards(msg.taskId)
            end
        else
            log.info('#### task get reward error! code:'..msg.respCode)
        end
    end,

    ------------------------------------

    -------------获得道具通知(包括金币 钻石等资源)-------------
     --[[
    message SCGetItem
    {
        required ITEM_CHANGE_REASON     reason   = 1;  //道具来源
        repeated MGETITEM               getItems   = 2;  //道具列表
    }

    message MGETITEM
    {
        required string uId   = 1;  //inst id 
        required int32  itemId   = 2;  //config id
        required int32  getNum = 3; //获得数量
    }

    menu ITEM_CHANGE_REASON
    {
        "NONE",
        "USE_ITEM",
        "BUY_ITEM",
        "RESOLVE_ITEM",
        "RESET_EQUIP",
        "TASK_REWARD",
        "COPY_DROP",
        "EQUIP_UPGRADE",
        "OFFICER_AWAKE",
        "OFFICER_PROMOTE",
        "ACTIVE_REWARRD"
    }
    ]]
    [protocol_pb.NOTIFYGETITEM] = function(self, msg)
        log.info("##### get item notify ", msg.reason)
        gui_helper:showGetItemsPanel(msg)

        -- 缓存战斗胜利获得的道具
        if msg.reason == common_pb.COMBAT_WIN then
            table.insert(game_info.drop_items, msg)
        end

        if msg.reason == common_pb.TASK_REWARD then
            local items = gui_helper.drop_list[msg.reason]
            if items and items[1] then
                local cfg = task_manager:getTaskCfg(msg.reasonId)--config.task_tasks[msg.reasonId]
                if not cfg then
                    --log.error("Failed to find task id:"..msg.reasonId)
                else
                    local build_lv = guide_helper:getHomeEntityLevel(1)
                    
                    if build_lv and build_lv > 2 then
                        local str = string.format( NativeText.GetLangStr(9000083), frame_helper:getLangCfg(cfg.title), items[1].count )
                        hint:setText(str)
                    end                    
                end
            end
        end

        game_director.events:dispatch("NOTIFYGETITEM", msg)
    end,

    [protocol_pb.POSTPOSINFO] = function(self, msg)
        if msg.respCode == 0 then
            log.info("###@@@@### pos info:"..msg.posX..", "..msg.posY)
            level_info.player_pos = Vector3(msg.posX*0.0001, 0, msg.posY*0.0001)
        end
    end,

    -------------------------------------

    -------- 章节英雄出战，战前布阵----------
    -- 请求章节出战英雄列表
    [protocol_pb.CHAPTERATTENDINFO] = function(self, msg)
        
        log.info("### 章节出战英雄列表 message_handler", msg.chapterId, msg.copyId, msg.formationId)
        --[[
        for _,f in ipairs(msg.formations) do
            log.info("## heroId:"..f.heroId..", tankId:"..f.tankId..", officerid:"..f.officerId..", location:"..f.location, ", isDeath:"..f.isDeath)
        end]]

        -- 保存章节的出战英雄

        -- 根据 level 类型，保存不同容器
        local level_id = msg.copyId
        if not level_id then
            game_info.chapter_attend_heros[msg.chapterId] = msg
        else
            local level_cfg = config.game_copy_levels[level_id]
            if not level_cfg or level_cfg.level_type == 1 then  -- 普通副本，出战英雄每章固定
                game_info.chapter_attend_heros[msg.chapterId] = msg
            elseif level_cfg.level_type == 2 then       -- 剧情关卡
                game_info.level_attend_heros[msg.copyId] = msg
                story_hero_manager:clear()
                -- 将英雄添加到 story_hero_manager
                for k, v in ipairs(msg.formations) do
                    -- 创建剧情英雄
                    story_hero_manager:createStoryHero(v.heroId, v.tankId, v.officerId)
                    local officer_inst = {
                        cfg_id = v.officerId, 
                        star_level = 1, 
                        quality = 1,
                        res_id = v.officerId
                    }
                end

            end
        end

        game_info.events:dispatch('attend_heros_msg')

        if embattle_frame then
            embattle_frame:refresh()
        end

        if level_info_frame then
            level_info_frame:refresh()
        end
        
        --log.info("#### CHAPTERATTENDINFO chapter:"..msg.chapterId..', skillId:'..msg.skillId)
        for _,hero in ipairs(msg.formations) do
            --log.info("## hero_id:"..hero.heroId..', location:'..hero.location..', isDeath:'..hero.isDeath)
        end
    end,

    -- 选择章节出战英雄
    [protocol_pb.SELECTCHAPTERATTENDHEROES] = function(self, msg)
        if msg.respCode ~= 0 then
            log.error('select chapter attend heros failed!! errCode:'..msg.respCode)
        else
            log.info("#### protocol_pb.SELECTCHAPTERATTENDHEROES")
        end
    end,

    -- 关卡战斗前布阵
    [protocol_pb.LEVELEMBATTLE] = function(self, msg)
        if msg.respCode ~= 0 then
            log.error('level embattle failed!! errCode:'..msg.respCode)
        else
            log.info("#### protocol_pb.LEVELEMBATTLE")
        end
    end,

    -- 战斗开始
    [protocol_pb.FIGHTSTART] = function(self, msg)
        if msg.respCode == 0 then
            if embattle_frame then
                embattle_frame:onFightStart()
            end
        end
    end,

    ---------------------------------------

    ------------重置关卡----------------
    [protocol_pb.RESETMAPINFO] = function(self, msg)
        log.info("reset map info resp code ", msg.respCode)
        if msg.respCode == 0 then
            if game_info.chapter and game_info.chapter.AreaMaps then
                for k, v in ipairs(game_info.chapter.AreaMaps) do
                    v.copyId = 0
                end
            end
            -- 重置关卡
            level_info:reset()
            game_director.events:dispatch("onResetMapInfo", msg)

        else
            log.info("reset map info error! code ", msg.respCode)
        end
    end,
    ------------------------------

    ---------------商店-------------------
    -- 收到商品列表
    [protocol_pb.SHOPITEMLIST] = function(self, msg)
        if msg.respCode == 0 then
            log.info('SHOPITEMLIST type:'..msg.shopType)

            if msg.currency then
                log.info("### currency:".. (#msg.currency > 0 and msg.currency[1] or ""))
            else
                log.error("### no currency!!")
            end
            shop_manager:refreshShop(msg.shopType, msg.shopItems, msg.currency)
        else
            log.info("SHOPITEMLIST! code ", msg.respCode)
        end
    end,

    -- 购买商品
    [protocol_pb.BUY_GOODS] = function(self, msg)
        if msg.respCode == 0 then
            log.info('BUY_GOODS respCode:'..msg.respCode..' shop type:'..msg.shopType..', idx:'..msg.index)
        else
            log.info("BUY_GOODS failed! code ", msg.respCode)
        end
    end,

   --------------------------------------
   -- 声望 item id:39501
    [protocol_pb.FAMEINFO] = function(self, msg)
        log.info('protocol_pb.FAMEINFO')

        fame_manager:refresh(msg.fameInfo)
    end,

    -- 被踢下线
    [protocol_pb.SVR_CLI_KICK_PLAYER] = function(self, msg)
        if not reconnector.is_reconnecting then
            log.info('###protocol_pb.SVR_CLI_KICK_PLAYER')
            gui_helper:showMessageBox(NativeText.GetLangStr(999008), function() game_director:changeState('login') end)
        end
    end,

    -- 公告
    [protocol_pb.ANNOUCEMENTNOTIFY] = function(self, msg)
        log.info('###protocol_pb.ANNOUCEMENTNOTIFY')
        for k, v in ipairs(msg.annouces) do
            announce_manager:add(v)
        end
    end,

    -- 通关
    [protocol_pb.OVERCOPY] = function(self, msg)
        log.info('######protocol_pb.OVERCOPY')
        level_info.level_over = true

        for k, v in ipairs(msg.activeCopies) do
            log.info("########### active copy id", v)
        end
        level_info.unload_level = msg.activeCopies
        level_info.unload_chapter = msg.nextActiveChapter

        if game_state_level.gui then
            if msg.respCode == 0 then
                game_state_level.gui:levelOver(true)
                level_info.is_win = true
            else
                game_state_level.gui:levelOver(false)
                level_info.is_win = false
            end
        end
    end,

    -- 邀请加入工会
    --[[[protocol_pb.INVITEDTOFAMILY] = function(self, msg)
        log.info("####### INVITEDTOFAMILY ", msg.respCode)
    end,]]

    -- 工会邀请通知
    --[[[protocol_pb.NOTIFYINVITEDFAMILY] = function(self, msg)
        log.info("####### NOTIFYINVITEDFAMILY ")
        if not frame_manager:isDisplay("guild_invite") then
            frame_manager:show("guild_invite", self, {info = msg.familyInfo})
        end
    end,]]

    -- 获取邮件
    [protocol_pb.MAILLISTINFO] = function(self, msg)
        log.info("####### MAILLISTINFO ")
        if msg.mailinfos then
            local_player.mail_list = msg.mailinfos
            
            if mail_frame then
                mail_frame:init(msg.mailinfos)
            else
                frame_manager:show("mail", self, {mails = msg.mailinfos})
            end
        end
    end,

    -- 读取邮件
    [protocol_pb.READMAIL] = function(self, msg)
        if msg.respCode == 0 then
            game_director.events:dispatch("READMAIL", msg.mail)
        end
    end,

    -- 提取邮件附件
    [protocol_pb.GETMAILITEMS] = function(self, msg)
        if msg.respCode == 0 then
            game_director.events:dispatch("GETMAILITEMS")
        end
    end,

    -- 删除邮件
    [protocol_pb.DELMAIL] = function(self, msg)
        if msg.respCode == 0 then
            game_director.events:dispatch("DELMAIL")
        end
    end,

    -- 一键读取
    [protocol_pb.ALLREASMAIL] = function(self, msg)
        if msg.respCode == 0 then
            game_director.events:dispatch("ALLREASMAIL", msg.mailinfos)
        end
    end,

    -- 收藏邮件
    [protocol_pb.COLLECTMAIL] = function(self, msg)
        if msg.respCode == 0 then
            game_director.events:dispatch("COLLECTMAIL", msg.mailId)
        end
    end,

    -- 聊天通知
    [protocol_pb.CHATNOTIFY] = function(self, msg)
        print("### 聊天通知 CHATNOTIFY")
        if not local_chat then require('game.chat') end
        local_chat:setNtf(msg)
    end,

    -- 聊天列表
    [protocol_pb.GETCHATCHANNELLIST] = function(self, msg)
        print("### 聊天列表 GETCHATCHANNELLIST")
        if not local_chat then require('game.chat') end
        local_chat:setBasicInfo(msg)
    end,

    -- ############ 公会 start ############
     -- 创建工会
    [protocol_pb.CREATEFAMILY] = function(self, msg) 
        if msg.RespCode == 0 then
            frame_manager:hide('guild_apply')
            network_sender:getFamilyInfo(msg.familyId)
        end
    end,

    -- 获得工会列表
    [protocol_pb.GETFAMILYLIST] = function(self, msg) 
        frame_manager:show('guild_apply', self.owner, {msg = msg})
    end,

    -- 申请 or 取消
    [protocol_pb.FAMILYJOINORCANCEL] = function(self, msg)
        if msg.RespCode == 0 then
            local vsb, frame = frame_manager:isDisplay('guild_apply')
            if vsb then
                if msg.status ~= 0 then
                    frame:addCommandBuff(function()
                        frame:refreshApplyGuildList(msg)
                    end)

                    if msg.status == 2 then
                        hint:setText(NativeText.GetLangStr(9000105))
                    end
                else
                    frame_manager:hide('guild_apply')
                    hint:setText(NativeText.GetLangStr(190155))
                    network_sender:getFamilyInfo(msg.familyid)
                end
            end

            local vsb, frame = frame_manager:isDisplay('guild_info')
            if vsb then
                if msg.status == 0 then
                    frame_manager:hide('guild_info')
                    hint:setText(NativeText.GetLangStr(190155))
                    network_sender:getFamilyInfo(msg.familyid)
                end
            end
        end
    end,
    
    -- 工会详情
    [protocol_pb.GETFAMILYINFO] = function(self, msg) 
        print("工会详情消息")
        if msg.respCode == 0 then
            local vsb, frame = frame_manager:isDisplay('guild_info')
            if vsb then
                frame:addCommandBuff(function()
                    frame:refreshInfo(msg)
                end)
            else
                frame_manager:show('guild_info', self, {msg = msg})
            end
        end
    end,
    
    -- 工会通知
    [protocol_pb.NOTIFYFAMILYINFO] = function(self, msg) 
        local_player:setGuildId(msg.familyId)
        local_guild:setId(msg.familyId, msg.familyInfo.memberInfo)
        local_guild:setInfo(msg.familyInfo)

        local vsb, frame = frame_manager:isDisplay('guild_info')
        if vsb then
            frame:addCommandBuff(function()
                frame:refreshInfo(msg)
            end)
        end
    end,
    
    -- 工会成员
    [protocol_pb.GETMEMBERLIST] = function(self, msg) 
        if msg.respCode == 0 then
            --frame_manager:show('guild_members', self, {msg = msg})
            local_guild.events:dispatch("GETMEMBERLIST", msg)
        end
    end,
    
    -- 获取邀请我的工会列表
    --[[ [protocol_pb.GETINVITEDFAMILYLIST] = function(self, msg) 
        -- print("##### 获取邀请我的工会列表:", #msg.familyInfo, msg.respCode)
        if msg.respCode == 0 then
            if self.guild_apply then
                self.guild_apply:showInviteGuildList(msg)
            end
        end
    end, ]]
    
    -- 获取我申请的工会
    [protocol_pb.GETMYAPPLYFAMILY] = function(self, msg) 
        local vsb, frame = frame_manager:isDisplay('guild_apply')
        if vsb then
            frame:addCommandBuff(function()
                frame:showInviteGuildList(msg)
            end)
        end
    end,
    
    -- 工会成员升阶降阶
    [protocol_pb.FAMILYMEMBERPROMOTE] = function(self, msg) 
        -- print("FAMILYMEMBERPROMOTE respCode:", msg.respCode)
        if msg.respCode == 0 then
            game_director.events:dispatch("FAMILYMEMBERPROMOTE", msg)
        end
    end,
    
    -- 开除工会成员
    [protocol_pb.EXPELFAMILYMEMBER] = function(self, msg) 
        -- print("EXPELFAMILYMEMBER respCode:", msg.respCode)
        if msg.respCode == 0 then
            game_director.events:dispatch("EXPELFAMILYMEMBER", msg)
        end
    end,
    
    -- 获取工会日志
    [protocol_pb.GETFAMILYOG] = function(self, msg) 
        -- print("收到服务器消息 获取工会日志")
        if msg.respCode == 0 then
            self.guild_logs = frame_manager:show('guild_logs', self, {msg = msg})
        end
    end,
    
    -- 获取工会留言
    [protocol_pb.GETFAMILYLEAVEWORD ] = function(self, msg) 
        if msg.respCode == 0 then
            frame_manager:show('guild_notes', self, {msg = msg})
        end
    end,
    
    -- 通知玩家新留言
    [protocol_pb.NOTIFYNEWLEAVEWORD ] = function(self, msg) 
        local vsb, frame = frame_manager:isDisplay('guild_notes')
        if vsb then
            frame:addCommandBuff(function()
                frame:addWordList(msg.leaveWord)
            end)
        end
    end,
    
    -- 工会留言
    [protocol_pb.FAMILYLEAVEWORD ] = function(self, msg) 
        if msg.respCode == 0 then
            -- network_sender:getFamilyLeaveWord(local_player:getGuildId())
        end
    end,
    
    -- 修改工会信息
    [protocol_pb.MODIFYFAMILYINFO ] = function(self, msg) 
        if msg.respCode == 0 then
            network_sender:getFamilyInfo(local_guild:getId())
        end
    end,

    -- 工会转让
    [protocol_pb.TRANSFERLEADER ] = function(self, msg) 
        if msg.respCode == 0 then
            hint:setText(NativeText.GetLangStr(900007))
            network_sender:getFamilyInfo(local_player:getGuildId())
        end
    end,
    
    -- 退出公会
    [protocol_pb.QUITFAMILY ] = function(self, msg) 
        if msg.respCode == 0 then
            frame_manager:hide('guild_set')
            frame_manager:hide('guild_info')
            local_player:setGuildId("")
            local_guild:clear()

            if msg.bDismiss then
                hint:setText(NativeText.GetLangStr(9000106))
            end
        end
    end,
    
    -- 搬迁返回结果
    [protocol_pb.MOVEPERSONALBASE ] = function(self, msg) 
        if msg.respCode == 0 then
            frame_manager:hideAll()
            game_director:changeState('loading', {next_state = 'world_map', scene_name = 'hex_map', next_state_ui_name = 'worldMap'})
        end
    end,
    
     -- 联盟礼物信息
    [protocol_pb.GETFAMILYGIFTINFO ] = function(self, msg) 
        if msg.respCode == 0 then
            if frame_manager:isDisplay("guild_gifts") then
                self.guild_gifts:refresh(msg)
            elseif msg.isActiveNotf ~= 1 then
                self.guild_gifts = frame_manager:show('guild_gifts', self, {msg = msg})
            end
        end
    end,
    
    -- 领取联盟礼物
    [protocol_pb.GETFAMILYGIFTREWARD ] = function(self, msg)
        if msg.respCode == 0 then 
            local_guild.events:dispatch("GETFAMILYGIFTREWARD", msg)
        end
    end,
    
    -- 删除联盟礼物
    [protocol_pb.DELFAMILYGIFT ] = function(self, msg) 
        if msg.respCode == 0 then
            local_guild.events:dispatch("DELFAMILYGIFT", msg)
        end
    end,
    
    -- 联盟礼物经验通知
    [protocol_pb.NOTIFYFAMILYGIFTEXP ] = function(self, msg)
        local_guild.events:dispatch("NOTIFYFAMILYGIFTEXP", msg)
    end,
    
    -- 联盟礼物刷新通知
    [protocol_pb.NOTIFYNEWFAMILYGIFT ] = function(self, msg)
        local_guild.events:dispatch("NOTIFYNEWFAMILYGIFT", msg)
    end,
    
    -- 获取工会玩家申请列表
    [protocol_pb.GETAPPLYLIST ] = function(self, msg)
        if msg.respCode == 0 then
            frame_manager:show('guild_member_apply', self, {msg = msg})
        end   
    end,
    
    -- 会长或r4处理工会的申请列表           
    [protocol_pb.DISPOSEFAMILYAPPLY ] = function(self, msg) 
        if msg.respCode == 0 then
        end
    end,
    
    -- 联盟礼物领取信息
    [protocol_pb.GETGIFTGAININFO ] = function(self, msg) 
        if msg.respCode == 0 then
            frame_manager:show('guild_gift_info', self, {msg = msg})
        end
    end,
    
    -- 发送联盟邮件
    [protocol_pb.SENDFAMILYMAIL ] = function(self, msg) 
        if msg.respCode == 0 then
            local_guild.events:dispatch("SENDFAMILYMAIL")
        end
    end,
    
    -- 联盟帮助列表
    [protocol_pb.GETFAMILYHELPLIST ] = function(self, msg) 
        if msg.respCode == 0 then
            local vsb, frame = frame_manager:isDisplay('guild_info')
            if vsb then
                frame:addCommandBuff(function()
                    frame:onGetHelpList(msg)
                end)
            end
        end
    end,
    
    -- 联盟帮助成功
    [protocol_pb.HELPFASTBUILD ] = function(self, msg) 
        log.info("####### HELPFASTBUILD ", msg.respCode)
        if msg.respCode == 0 then
            local vsb, frame = frame_manager:isDisplay("guild_info")
            if vsb then
                frame:addCommandBuff(function()
                    frame:onHelpSuccess(msg)
                end)
            end
        end
    end,
    
    -- 联盟帮助更新通知
    [protocol_pb.UPDATEHELPLIST ] = function(self, msg) 
        log.info("####### UPDATEHELPLIST ", msg.respCode)
        local vsb, frame = frame_manager:isDisplay("guild_info")
        if vsb then
            frame:addCommandBuff(function()
                frame:refreshHelpList(msg)
            end)
        end
    end,
    
    -- 联盟建筑列表
    [protocol_pb.GETFAMILYBUILD ] = function(self, msg) 
        if msg.respCode == 0 then
            local vsb, frame = frame_manager:isDisplay('guild_info')
            if vsb then
                frame:addCommandBuff(function()
                    frame:onGetFamilyBuild(msg)
                end)
            end
        end
    end,
    
    -- 加速建造工会建筑
    [protocol_pb.FAMILYBUILDFAST ] = function(self, msg) 
        
    end,
    
    -- 升级工会建筑
    [protocol_pb.UPGRADEFAMILYBUDILING ] = function(self, msg) 
        
    end,
    
    -- 通知工会建筑信息
    [protocol_pb.NOTIFYFAMILYBUILDING ] = function(self, msg)
        local vsb, frame = frame_manager:isDisplay('guild_info')
        if vsb then
            frame:addCommandBuff(function()
                frame:refreshBuild(msg)
            end)
        end
    end,
    
    -- 加入联盟邀请
    [protocol_pb.INVITEDTOFAMILY ] = function(self, msg) 
        log.info("####### INVITEDTOFAMILY ", msg.respCode)
    end,
    
    -- 联盟邀请通知
    [protocol_pb.NOTIFYINVITEDFAMILY ] = function(self, msg) 
        log.info("####### NOTIFYINVITEDFAMILY ")
        if not frame_manager:isDisplay("guild_invite") then
            frame_manager:show("guild_invite", self, {info = msg.familyInfo})
        end
    end,

    -- 获取集结列表
    [protocol_pb.WORLDMAPGETMASSLIST ] = function(self, msg) 
        local vsb, frame = frame_manager:isDisplay('guild_info')
        if vsb then
            frame:addCommandBuff(function()
                frame:onGetMassList(msg)
            end)
        end
    end,

    -- 查询集结信息
    [protocol_pb.WORLDMAPGETMASSINFO ] = function(self, msg)
        if msg.respCode == 0 then
            frame_manager:show("mass_info", nil, {info = msg.massInfo})
        end
    end,

    -- 创建集结
    [protocol_pb.WORLDMAPSTARTMASS ] = function(self, msg)
        if msg.respCode == 0 then
            hint:setText(NativeText.GetLangStr(9000084))
        end
    end,

    -- 获取驻扎队伍
    [protocol_pb.WORLDMAPGETEAMS ] = function(self, msg)
        if msg.respCode == 0 then
            game_director.events:dispatch("WORLDMAPGETEAMS", msg)
        end
    end,

    -- 获取玩家自己的建筑
    [protocol_pb.WORLDMAPGETOWNERBUILD ] = function(self, msg)
        if msg.respCode == 0 then
            own_hold_info:clear()

            for _,hold in ipairs(msg.appears.playerHolds) do
                if hold.playerId == game_info.role_info.playerid then
                    --own_hold_info:addPlayerHold(hold)
                end
            end

            -- 工会基地
            for __, hold in ipairs(msg.appears.familyBuildings) do
                local building_cfg = config.camp_buildings[hold.cfgId] 
                if building_cfg.type == 5 and hold.familyId == local_guild:getId() then
                    own_hold_info:addGuildHold(hold)
                end
            end

            -- 公共基地
            for _, hold in ipairs(msg.appears.shareCity) do
                local cfg = config.worldmap_publicbases[hold.cityCfgId]
                if cfg then
                    own_hold_info:addPublicBaseHold(hold)
                else
                    log.error("#### create public base error ! id", hold.cityCfgId)
                end
            end

            -- 工厂
            for _, hold in ipairs(msg.appears.factoryHolds) do
                own_hold_info:addFactoryHold(hold)
            end

            -- 军营
            for _, hold in ipairs(msg.appears.campHolds) do
                own_hold_info:addCampHold(hold)
            end

            game_director.events:dispatch("WORLDMAPGETOWNERBUILD", msg)
        end
    end,

    --取消集结
    [protocol_pb.WORLDMAPGETMASSDESTORY] = function(self, msg) 
        if msg.respCode == 0 then
            game_director.events:dispatch("WORLDMAPGETMASSDESTORY", msg)
        end
    end,

    --集结消失通知
    [protocol_pb.WORLDMAPMASSDEL] = function(self, msg) 
        game_director.events:dispatch("WORLDMAPMASSDEL", msg)
    end,

    --集结队伍变化通知
    [protocol_pb.WORLDMAPMASSNOTIFY] = function(self, msg) 
        game_director.events:dispatch("WORLDMAPMASSNOTIFY", msg)
    end,

    -- ############ 公会 end ############

    -- ############ 关卡布阵 start ############

    -- 阵形信息 SCNotifyFormationList
    [protocol_pb.FORMATIONLIST] = function(self, msg) 
        -- log.error("阵形信息 message_handler", #msg.formationList)
        for __, id in ipairs(msg.formationList) do
            level_formation:addEnableFormations(id)
        end
        network_sender:getPresetFormation()
    end,

    -- 开启阵形通知 SCNotifyAddFormation
    [protocol_pb.NOTIFYADDFORMATION] = function(self, msg) 
        level_formation:addEnableFormations(msg.formationList)
    end,

    -- 副本布阵 SCCopyEmbattle
    [protocol_pb.COPYEMBATTLE] = function(self, msg) 
        -- log.error("副本布阵 message_handler respCode", msg.respCode)
        if msg.respCode == 0 then
            log.info("##### formation id:"..msg.formationId)
            game_info.level_attend_heros[level_info.map_id].formationId = msg.formationId
        end
    end,

    -- 添加预设编队 SCAddPresetFormation
    [protocol_pb.ADDPRESETFORMATION] = function(self, msg) 
        -- log.error("添加预设编队 message_handler respCode", msg.respCode)
        if msg.respCode == 0 then
            network_sender:getPresetFormation()
        end
    end,

    -- 删除预设编队 SCDelPressetFormation
    [protocol_pb.DELPRESETFORMATION] = function(self, msg) 
        -- log.error("删除预设编队 message_handler respCode", msg.respCode)
        if msg.respCode == 0 then
            network_sender:getPresetFormation()
        end
    end,

    -- 获取预设编队 SCGetPresetFormation
    [protocol_pb.GETPRESETFORMATION] = function(self, msg) 
        -- log.error("获取预设编队 message_handler", #msg.presetFormation)
        level_formation:clearPresetFormations()
        for __, content in ipairs(msg.presetFormation) do
            level_formation:setPresetFormation(content.presetId, content.formation)
            -- log.error("获取预设编队 message_handler", content.formation.formationId)
        end
        level_formation.events:dispatch('preset_formation_change')
    end,

    -- 获取队伍战力
    [protocol_pb.GETTEAMSWORD] = function(self, msg) 
        level_formation:setForce(msg.queryId, msg)
    end,
    -- ############ 关卡布阵 end ############

    -- 世界地图战报通知
    [protocol_pb.WORLDMAPBATRECNOTIFY] = function(self, msg) 
        --local url = msg.battleRes.link
        --log.info("battle result link:"..url)

        -- 打开界面
        frame_manager:show('battle_tip', self, msg.battleRes)
    end,

    -- 模拟战斗
    [protocol_pb.TESTFIGHTPROP] = function(self, msg)
        log.info("### receive TESTFIGHTPROP")

         -- 浮点型属性要除 10000
         local function convertPropToFloat(props)
            local str = "\r\n"
            for _, prop in ipairs(props.ParPropIdValues) do
                if config.attribute_config_attrs[prop.PropId].isfloat > 0 then
                    prop.PropValue = prop.PropValue * 0.0001
                end
                str = str .. ('prop:id'..prop.PropId..' value:'..prop.PropValue).."\r\n"
            end
            log.info(str)
        end

        level_info.map_attack_info = {}
        level_info.formation_t0 = {}    -- 己方的阵型
        level_info.formation_t1 = {}    -- 敌方的阵型
        level_info.map_attack_info.playerFormation = msg.playerFormation.formationId
        level_info.map_attack_info.monsterFormation = msg.enemyFormation.formationId
        level_info.map_attack_info.monsterBuff = {}

        -- 己方
        for k, props in ipairs(msg.playerFormation.formationHero) do
            log.info("### hero:"..props.fightProp.HeroId..", pos:"..props.fightProp.pos)
            convertPropToFloat(props.fightProp)
            level_info.formation_t0[props.fightProp.pos] = props.fightProp
        end

        -- 敌方
        for k, props in ipairs(msg.enemyFormation.formationHero) do
            log.info("### hero:"..props.fightProp.HeroId..", pos:"..props.fightProp.pos)
            convertPropToFloat(props.fightProp)
            level_info.formation_t1[props.fightProp.pos] = props.fightProp
        end

        level_info.is_simple_battle = true
        -- 进入战斗
        game_director:changeState('battle', {stage_id=1, battle_id="1"})

    end,

    -- 招募战斗
    [protocol_pb.STARTRECRUITFIGHT] = function(self, msg)
        print("招募战斗 message handle...")

         -- 浮点型属性要除 10000
         local function convertPropToFloat(props)
            local str = "\r\n"
            for _, prop in ipairs(props.ParPropIdValues) do
                if config.attribute_config_attrs[prop.PropId].isfloat > 0 then
                    prop.PropValue = prop.PropValue * 0.0001
                end
                str = str .. ('prop:id'..prop.PropId..' value:'..prop.PropValue).."\r\n"
            end
            log.info(str)
        end

        local recruit_data = {}
        recruit_data.recruit_hero_id = msg.instId
        recruit_data.formation_t0 = {}    -- 己方的阵型
        recruit_data.formation_t1 = {}    -- 敌方的阵型

        recruit_data.fmt_cfg_id_t0 = msg.playerForm.formationId
        recruit_data.fmt_cfg_id_t1 = msg.monsterForm.formationId

        if #msg.playerForm.formationHero <= 0 or #msg.monsterForm.formationHero <= 0 then
            -- print("招募战斗####:", msg.playerForm.formationHero and #msg.playerForm.formationHero)
            -- print("招募战斗####:", msg.monsterForm.formationHero and #msg.monsterForm.formationHero)
            return
        end

        -- 己方
        for k, props in ipairs(msg.playerForm.formationHero) do
            -- log.info("### 己方 招募战斗 hero:"..props.fightProp.HeroId..", pos:"..props.fightProp.pos)
            convertPropToFloat(props.fightProp)
            recruit_data.formation_t0[props.fightProp.pos] = props.fightProp
        end

        -- 敌方
        for k, props in ipairs(msg.monsterForm.formationHero) do
            -- log.info("### 敌方 招募战斗 hero:"..props.fightProp.HeroId..", pos:"..props.fightProp.pos)
            convertPropToFloat(props.fightProp)
            recruit_data.formation_t1[props.fightProp.pos] = props.fightProp
        end

        -- 进入战斗
        game_director:changeState('recruit_battle', recruit_data)
    end,

    -- 获取修理厂信息
    --[[
    [protocol_pb.GETREPAIRFACTORYINFO] = function(self, msg) 
    end,]]

    --- 兵工厂
    [protocol_pb.GETARMORYINFO] = function(self, msg)
        if msg.armoryInfo.state == common_pb.MMAKE_NONE then
            frame_manager:show("blueprint_select", nil, {info = msg.armoryInfo})
        else
            if config.home_config_produces[msg.armoryInfo.drawingId].type == 1 then
                network_sender:buildingExpend(msg.armoryInfo.drawingId, common_pb.EPRODUCT_WEAPON)
            else
                network_sender:buildingExpend(msg.armoryInfo.drawingId, common_pb.EPRODUCT_ARMOUR)
            end
        end
    end,

    [protocol_pb.PRODUCTEQUIP] = function(self, msg) 
        if msg.respCode == 0 then
            game_director.events:dispatch("REFRESHARMORYINFO", msg.armoryInfo)
        end
    end,

    [protocol_pb.GETEQUIP] = function(self, msg) 
        if msg.respCode == 0 then
            game_director.events:dispatch("REFRESHARMORYINFO", msg.armoryInfo)
        end
    end,

    [protocol_pb.GETDRAWINGINFO] = function(self, msg) 
        if msg.respCode == 0 then
            game_director.events:dispatch("GETDRAWINGINFO", msg.drawingInfo)
        end
    end,

    [protocol_pb.SPEEDUPPRODUCTEQUIP] = function(self, msg) 
        if msg.respCode == 0 then
            game_director.events:dispatch("REFRESHARMORYINFO", msg.armoryInfo)
        end
    end,

    ----------

    ----- 世界地图使用修理包
    [protocol_pb.WORLDUSEREPAIR] = function(self, msg) 
        game_director.events:dispatch("WORLDUSEREPAIR")
    end,

    -- 章节目标
    [protocol_pb.CHAPTERACHIEVEGETLIST] = function(self, msg) 
        chapter_achievement_manager:onChapterAchievementList(msg)
    end,

    -- 通知减少消耗信息
    [protocol_pb.NOTIFYBUILDEXPEND] = function(self, msg) 
    end,

    -- 工会科技信息获取
    [protocol_pb.GETFAMILYTECH] = function(self, msg)
        print("-- 工会科技信息获取")
        if msg.respCode == 0 then
            local vsb, frame = frame_manager:isDisplay('guild_info')
            if vsb then
                frame:addCommandBuff(function()
                    frame:refreshTech(msg)
                end)
            end
        end
    end,

    -- 设置推荐科技
    [protocol_pb.SETRECOMMENDTECH] = function(self, msg)
        if msg.respCode == 0 then
            local vsb, frame = frame_manager:isDisplay('guild_info')
            if vsb then
                frame:addCommandBuff(function()
                    frame:recommendInfo(msg.techs)
                end)
            end
        end
    end,

    -- 工会科技捐献
    [protocol_pb.FAMILYTECHDONATE] = function(self, msg)
        if msg.respCode == 0 then
            local vsb, frame = frame_manager:isDisplay('guild_info')
            if vsb then
                network_sender:getFamilyTech(local_guild:getId())
                network_sender:getFamilyInfo(local_guild:getId())
            end
        end
    end,

    [protocol_pb.CHAPTERACHIEVEGETREWARD] = function(self, msg) 
        chapter_achievement_manager:onReward(msg)
    end,

    [protocol_pb.CHAPTERACHIEVESTART] = function(self, msg)
        if respCode == 0 then
            -- todo 显示章节开启
            log.info("#### receive CHAPTERACHIEVESTART")
        else
        end
    end,

    -- 收到引导信息
    [protocol_pb.NOTIFYGUIDE] = function(self, msg)
        log.info("#### protocol_pb.NOTIFYGUIDE")
        guide_manager:loadGuideData(msg)
    end,

    -- 集结设置
    [protocol_pb.WORLDMAPMASSEDITSET] = function(self, msg)
        if msg.respCode == 0 then
            game_director.events:dispatch("WORLDMAPMASSEDITSET")
        end
    end,

    -- 怪物攻打次数
    [protocol_pb.WORLDMAPATKCOUNTNOTIFY] = function(self, msg)
        world_map_info:setMonsterAtkCnt(msg.atkCount)
    end,

    -- buff
    [protocol_pb.NOTIFYBUFF] = function(self, msg)
        buff_info:clearInfo()
        for __, value in ipairs(msg.buffList) do
            buff_info:setBuff(value.buffId, value.contTime, value.totalTime)
        end
        buff_info:dispatch()
    end,

    -- 商店 start
    [protocol_pb.GETSHOPLIST] = function(self, msg)
        local_shop:setShopTypes(msg.shops)
        local_shop:clear()
        for __, shop_type in ipairs(msg.shops) do
            network_sender:reqShopItemList(shop_type)
        end
    end,

    [protocol_pb.SHOPITEMLIST] = function(self, msg)
        if msg.respCode == 0 then
            local_shop:setShopItems(msg.shopType, msg.shopItems)
            local_shop:setRefreshInfo(msg.shopType, msg.refreshInfo)
            local_shop:setRefreshCountTime(msg.refreshTime)
        end
        if local_shop:isReady() then
            local vsb, frame = frame_manager:isDisplay('shop')
            if not vsb then
                frame_manager:show('shop')
            end
        end
    end,

    [protocol_pb.BUY_GOODS] = function(self, msg)
        if msg.respCode == 0 then
            local_shop:setItemSold(msg.shopType, msg.index)
        end
    end,

    [protocol_pb.GAMESHOPREFRESH] = function(self, msg)
        if msg.respCode == 0 then
            local_shop:setShopItems(msg.shopType, msg.shopItems)
            local_shop:setRefreshInfo(msg.shopType, msg.refreshInfo)
        end
    end,
    -- 商店 end

    [protocol_pb.HELPNOTICE] = function(self, msg)
        hint:setText(string.format(NativeText.GetLangStr(190172), msg.helperName))
    end,

    -- 世界地图战报战斗通知
    [protocol_pb.WORLDMAPBATMSGNOTIFY] = function(self, msg)
        game_director.events:dispatch("WORLDMAPBATMSGNOTIFY", msg)

    end,

    -- 个人基地 ------------------------------------- start

    -- 建筑新增
    [protocol_pb.PERSONBASEAPPEAR] = function(self, msg)
        -- 本地缓存
        for k, v in ipairs(msg.groups) do
            print("组 id:", v.groupId, "状态:", not v.bLock)
            home_info:setGroup(v.groupId, v.bLock)
        end
        for k, v in ipairs(msg.objs) do
            home_info:setBuild(v.gridId, v)
        end

        -- 广播
        home_info:dispatchGroupInfos()
        home_info:dispatchGridInfos()

        for k, v in ipairs(msg.objs) do
            home_info:dispatchBuildInfos(v.instId)
        end

        --[[ 测试代码
        app:startCoroutine(
            function()
                -- coroutine.yield(WaitForSeconds(1))
                coroutine.yield(nil)
                for k, v in ipairs(msg.objs) do
                    if v.gridId == 4 then
                        v.tLevelUp = 0
                        v.level = v.level + 1
                        home_info:setBuild(v.gridId, v)
                    end
                end

                -- 广播
                home_info:dispatchGroupInfos()
                home_info:dispatchGridInfos()

                for k, v in ipairs(msg.objs) do
                    home_info:dispatchBuildInfos(v.instId)
                end
            end
        )]]
    end,

    -- 建筑删除
    [protocol_pb.PERSONBASEDISAPPEAR] = function(self, msg)
        -- 移除建筑数据
        for k, id in ipairs(msg.instIds) do
            home_info:removeBuild(id)
        end

        -- 广播消息
        home_info:dispatchGridInfos()

        for k, id in ipairs(msg.instIds) do
            home_info:dispatchBuildInfos(id)
        end
    end,

    -- 建筑升级返回消息
    [protocol_pb.PERSONBASELEVELUP] = function(self, msg)
        if msg.respCode == 0 then
            home_info:setBuild(msg.obj.gridId, msg.obj)
            home_info:dispatchBuildInfos(msg.obj.instId)
        end
    end,

    -- 建筑完成升级广播
    [protocol_pb.PERSONBASELEVELUPNOTIFY] = function(self, msg)
        home_info:setBuild(msg.obj.gridId, msg.obj)
        home_info:dispatchBuildInfos(msg.obj.instId)
    end,

    -- 建筑升级加速
    [protocol_pb.PERSONBASEBUILDINGSPEEDUP] = function(self, msg)
        if msg.respCode == 0 then 
            home_info:setBuild(msg.obj.gridId, msg.obj)
            home_info:dispatchBuildInfos(msg.obj.instId)
        end
    end,

    -- 创建联盟建筑
    [protocol_pb.BUILDFAMILYBUILDING] = function(self, msg)
        if msg.respCode == 0 then 
            world_map_helper:destroyTempHold()
        end
    end,

    -- 个人基地 ------------------------------------- end

    -- 预警通知
    [protocol_pb.WARNNOTIFY] = function(self, msg)
        for k, v in ipairs(msg.warns) do
            game_info:createWarnInfo(v)
        end

        game_director.events:dispatch("WARNNOTIFY")
    end,

    -- 预警消失
    [protocol_pb.WARNDEL] = function(self, msg)
        for k, v in ipairs(msg.instIds) do
            game_info:deleteWarnInfo(v)
        end

        game_director.events:dispatch("WARNNOTIFY")
    end,
    
    -- 飞回联盟领地
    [protocol_pb.WORLDMAPRETURNFAMILYFORTRESS] = function(self, msg)
        if msg.respCode == 0 then
            game_director.events:dispatch("WORLDMAPRETURNFAMILYFORTRESS")
        end
    end,

    -- 领主 ------------------------------------- start

    [protocol_pb.GETCAPTAINGIFT] = function(self, msg)
        if msg.respCode == 0 then 
            local vsb, frame = frame_manager:isDisplay('lord_talent')
            if vsb then
                frame:addCommandBuff(function()
                    frame:refresh(msg)
                end)
            else
                frame_manager:show('lord_talent', nil, msg)
            end
        end
    end,

    [protocol_pb.NATURALGIFTLEVELUP] = function(self, msg)
        if msg.respCode == 0 then 
            local vsb, frame = frame_manager:isDisplay('lord_talent')
            if vsb then
                network_sender:getCaptainGift()
            end
        end
    end,

    [protocol_pb.NATURALGIFTRESET] = function(self, msg)
        if msg.respCode == 0 then 
            local vsb, frame = frame_manager:isDisplay('lord_talent')
            if vsb then
                network_sender:getCaptainGift()
            end
        end
    end,


    [protocol_pb.GETGLOBALPROPINFO] = function(self, msg)
        if msg.respCode == 0 then 
            frame_manager:show('lord_prop', nil, {
                is_pvp = false,
                msg = msg
            })
        end
    end,

    [protocol_pb.GETPLAYERPVPDATA] = function(self, msg)
        if msg.respCode == 0 then 
            frame_manager:show('lord_prop', nil, {
                is_pvp = true,
                msg = msg
            })
        end
    end,
 
    [protocol_pb.NOTIFYCAPTAININFO] = function(self, msg)
        local pre_level = local_lord.level
        local pre_force = local_lord.fight_force

        local_lord.name = msg.PlayerName
        local_lord.level = msg.captain_level
        local_lord.exp = msg.curExp
        local_lord.max_exp = msg.maxExp
        local_lord.energy = msg.energy   
        local_lord.max_energy = msg.maxEnergy

        local_lord.vip_level = msg.vip_level
        local_lord.fight_force = msg.fightforce
        local_lord.kill_num = msg.killNum
        local_lord.icon_id = msg.iconId
        local_lord.nickname = msg.nickName

        local_lord.rmb_time = Time.realtimeSinceStartup
        local_lord.fuel_recover_time = msg.fuel_recoverytime
        local_lord.events:dispatch("LORD_INFO_CHANGE")

        if local_lord.rcv_server_msg then
            if local_lord.level > pre_level then
                frame_manager:show('lord_level_up', nil, pre_level)
            end

            if local_lord.fight_force > pre_force then
                -- log.error("战力", pre_force, local_lord.fight_force, local_lord.fight_force - pre_force) 
                frame_manager:show('effect_power_up', nil, {
                    pre_power = pre_force,
                    cur_power = local_lord.fight_force,
                })
            end
        else
            local_lord.rcv_server_msg = true
        end
    end,

    [protocol_pb.NOTIFYPROFESSIONINFO] = function(self, msg)
            -- 装备中的船员
        if msg.profession then
            for _,item_info in ipairs(msg.profession) do
                crew_manager:equip(item_info)
            end
        end
    end,

    [protocol_pb.HANDLEPROFESSION] = function(self, msg)

        --[[
        message SCHandleProfession	
        {
            required uint32        respCode      = 1;
            optional MItemChange   equipItem     = 2;        //装备的装备
           optional MItemChange	 unequipItem   = 3;        //卸下的装备
        }]]
--[[
        app:startCoroutine(
            function()
                log.info('## receive protocol_pb.HANDLEPROFESSION')   

                if msg.respCode == 0 then
                    log.info('## receive protocol_pb.HANDLEPROFESSION')
                    -- 先卸下装备
                    if msg.unequipItem then
                        local slot = msg.unequipItem.ItemInfo.status
                        log.info('#### 11111111111111111111')
                        crew_manager:unequip(slot)
                        log.info('#### 22222222222222222')
                        local item = bag_manager:setItem(msg.unequipItem.ItemInfo)
                        item.slot = 0
                    end

                    if msg.equipItem then
                        local info = msg.equipItem.ItemInfo
                        coroutine.yield(nil)
                        log.info('#### 3333333333333333')
                        bag_manager:removeItem(info.cfgId, info.instId, info.count)
                        coroutine.yield(nil)
                        log.info('#### 444444444444444444')
                        crew_manager:equip(msg.equipItem.ItemInfo)
                    end    
                    
                    coroutine.yield(nil)
                    log.info('#### 5555555555555555555')
                    crew_manager.events:dispatch('onEquipChanged')
                    coroutine.yield(nil)
                    log.info('#### 6666666666666666666')
                end            
            end
        )]]

        if msg.respCode == 0 then
            if msg.unequipItem then
                if string.len(msg.unequipItem.ItemInfo.instId) > 0 then
                    local slot = msg.unequipItem.ItemInfo.status
                    crew_manager:unequip(slot)
                    bag_manager:setItem(msg.unequipItem.ItemInfo)
                else 
                end
            end

            if msg.equipItem then
                if string.len(msg.equipItem.ItemInfo.instId) > 0 then
                    if frame_manager:isDisplay('crewEquip') then
                        frame_manager:hide('crewEquip')
                    end 

                    local info = msg.equipItem.ItemInfo
                    bag_manager:removeItem(info.cfgId, info.instId, info.count)
                    crew_manager:equip(msg.equipItem.ItemInfo)
                end
            end    
            
            crew_manager.events:dispatch('onEquipChanged')
        end
    end,
    -- 领主 ------------------------------------- end


    -- 游戏内活动 start -----------------------------------------------
    [protocol_pb.GETACTIVELIST] = function(self, msg)
        log.info("活动通知 GETACTIVELIST")
        activity_manager:setActives(msg.actives)
    end,

    [protocol_pb.DELACTIVE] = function(self, msg)
        log.info("活动通知 DELACTIVE")
        activity_manager:deleteActives(msg.instIds)
    end,

    [protocol_pb.NOTIFYACTIVE] = function(self, msg)
        log.info("活动通知 NOTIFYACTIVE")
        activity_manager:refreshActives(msg.actives)
    end,
    -- 游戏内活动 end -----------------------------------------------

    [protocol_pb.WORLDMAPMYMARCHNOTIFY] = function(self, msg)
        for k, v in ipairs(msg.marchlines) do
            log.info("################################### queue add ", v.instId, v.action, v.pre_action)
            army_queue_manager:addQueue(v)
        end
        army_queue_manager:setQueueCount(msg.curQueueCount, msg.maxQueueCount)
    end,

    [protocol_pb.WORLDMAPMYMARCHDELNOTIFY] = function(self, msg)
        for k, v in ipairs(msg.ids) do
            log.info("################################### queue remove id ", v)
            army_queue_manager:removeQueue(v)
        end
        army_queue_manager:setQueueCount(msg.curQueueCount, msg.maxQueueCount)
    end,

    [protocol_pb.WORLDMAPSPEEDUP] = function(self, msg)
        army_queue_manager:updateSpeedUpArmy(msg)
    end,

    [protocol_pb.WORLDMAPNOTIFYATKRECORD] = function(self, msg)
        world_map_info:updateMonsterUnlock(msg.atkRecord)
    end,

    [protocol_pb.REDPOINTAPPEAR] = function(self, msg)
        red_point_manager:appear(msg)
    end,

    [protocol_pb.REDPOINTDISAPPEAR] = function(self, msg)
        red_point_manager:disappear(msg)
    end,

    [protocol_pb.GETPLAYERINFO] = function(self, msg)
        -- log.error("收到获取玩家信息", playerId)
        if msg.playerId == local_player.id then
            return
        end
        frame_manager:show('other_lord_info', nil, msg)
        -- log.error("playerName   :" .. msg.playerName   )
        -- log.error("strength	    :" .. msg.strength	    )
        -- log.error("vipLv	    :" .. msg.vipLv	    )
        -- log.error("lordLv		:" .. msg.lordLv		)
        -- log.error("familyId:    " .. msg.familyId)
        -- log.error("killNum		:" .. msg.killNum		)
        -- log.error("familyPost	:" .. msg.familyPost	)
        -- log.error("playerId	    :" .. msg.playerId	    )

        -- log.error("dsfkljdsklf:", #msg.professionId)
        
        -- for __, abc in ipairs(msg.professionId) do
        --     log.error("crew:", abc)
        -- end

    end,

    [protocol_pb.NOTIFYBEHELPED] = function(self, msg)
        hint:setServerNotify(string.format( NativeText.GetLangStr(190247), msg.name ))
    end,
}

network:registerMessageHandler({
    onMessage = function(self, cmd, msg)
        local handler = msg_handlers[cmd]
        if handler then 
            handler(msg_handlers, msg)
        end
    end,
})

--print("register generic msg handler!!!")