local log = logger.get("Game", "Network")
addRequire = require
require "protocol_pb"
require('network.reconnector')

local super = require("base.tcp")
network_sender = nil    

-- 单件 
local network_cls = class{
    inst = nil, -- 唯一实例 
    super = super,
    ctor = function(self)
        super.ctor(self)
        network_sender = require("network.sender")(self)
        self.message_handlers = {}  -- 消息处理器容器
        self.inst = self

        self.wait_ack_messages = {}    -- 等待回复的消息列表 {[cmd],  obj,  elapsed}

        self:on("close", function (refused)
            if refused then 
                log.info('connect server failed!!')
            else
                log.info("connection closed")
                log.info(debug.traceback)
            end
        end)

        self:on("connect", function ()
            --print("连接成功")
        end)

        self:on("message", function (cmd, msg)
            --print("收到协议", cmd, msg)
            self:onMessage(cmd, msg)
        end)
        --log.info("network ctor!")
    end,

    -- 从 base.tcp 的 ctor 调用过来的，只执行一次
    onInit = function(self, args)
        self.map = {
            [protocol_pb.CLIENTGATEWAYHEARTBEAT] = protocol_pb.HeartBeat,   -- 心跳
            [protocol_pb.LOGIN] = protocol_pb.SCLogin,
            [protocol_pb.CREATEROLE] = protocol_pb.SCCreateRole,
            [protocol_pb.SELECTROLE] = protocol_pb.SCSelectRole,
            [protocol_pb.DELROLE] = protocol_pb.SCDelRole,
            [protocol_pb.HEROINFONTF] = protocol_pb.SCHeroInfo,     -- HEROINFONTF = 10; //英雄信息通知 SCHeroInfo	
            [protocol_pb.GETHEROREPAIRINFO] = protocol_pb.SCGetHeroNeedFixInfo,     --	= 26;//获取英雄需要修理的信息 CSGetHeroNeedFixInfo SCGetHeroNeedFixInfo
            [protocol_pb.GLOBALPROPNOTIFY] = protocol_pb.SCGlobalPlayerPropChange,     --	= 29;//玩家全局属性通知SCGlobalPlayerPropChange
            [protocol_pb.CHECKFAMILYNAMESTR] = protocol_pb.SCCheckFamilyNameStr,     --	= 36;//检测字符串合法	CSCheckFamilyNameStr SCCheckFamilyNameStr
            
            -- 指挥官升级晋升觉醒
            [protocol_pb.OFFICEREXPCHANGE] = protocol_pb.SCOfficerExpChange,   -- OFFICEREXPCHANGE =152; //指挥官经验改变  SCOfficerExpChange
            [protocol_pb.OFFICERPROMOTE] = protocol_pb.SCOfficerPromote,   -- 	OFFICERPROMOTE   =153;  //指挥官晋升 CSOfficerPromote SCOfficerPromote
            [protocol_pb.OFFICERSTARUP] = protocol_pb.SCOfficerStarUp,   -- 		OFFICERSTARUP     =154;  //指挥官升星 CSOfficerStarUp SCOfficerStarUp
            [protocol_pb.PARTNERPROPCHANGE] = protocol_pb.SCPartnerPropChange,  -- PARTNERPROPCHANGE =21;  //指挥官属性变化通知 SCPartnerPropChange
            [protocol_pb.ADDHEROEXP] = protocol_pb.SCAddHeroExp,  -- 		    = 35;//使用物品给指挥官增加经验 CSAddHeroExp SCAddHeroExp

            [protocol_pb.SVRPROPCHANGE] = protocol_pb.SCPlayerPropChange,  --SVRPROPCHANGE = 13; //玩家属性变化通知 SCPlayerPropChange
            [protocol_pb.ITEMINFONTF] = protocol_pb.SCItemInfo,     -- ITEMINFONTF   = 11; //道具信息通知 SCItemInfo
            [protocol_pb.SVRITEMCHANG] = protocol_pb.SCItemChange,  -- SVRITEMCHANG = 12; //道具增删改信息 SCItemChange
            [protocol_pb.HEROEQUIP] = protocol_pb.SCHeroEquip,  -- 	HEROEQUIP   =150;   //英雄装备和卸下装备 CSHeroEquip SCHeroEquip
            [protocol_pb.UPGRADEITEM] = protocol_pb.SCUpGradeItem,  -- 		UPGRADEITEM =151;   //升级道具 SCUpGradeItem
            [protocol_pb.ENTERMAP] = protocol_pb.SCEnterMap,
            [protocol_pb.ATTACKMONSTER] = protocol_pb.SCAttackMonster,  -- ATTACKMONSTER = 51; //攻击怪物 CSAttackMonster SCAttackMonster
            [protocol_pb.POSTFOGINFO] = protocol_pb.SCCommonAck,
            [protocol_pb.POSTPOSINFO] = protocol_pb.SCPosChange,
            [protocol_pb.ENTITYCHANGE] = protocol_pb.SCMapEntityChange, -- ENTITYCHANGE = 56; //地图实体变化 SCMapEntityChange	
            [protocol_pb.RESETMAPINFO] = protocol_pb.SCResetMapInfo,    -- RESETMAPINFO = 55; //重置关卡 CSResetMapInfo SCResetMapInfo
            [protocol_pb.LEAVEMAP] = protocol_pb.SCLeaveMap,
            [protocol_pb.COMMONLEVELINTERATION] = protocol_pb.SCCommonLevelInteraction, -- 	COMMONLEVELINTERATION  =64; //关卡交互CSCommonLevelInteraction SCommonLevelInteraction
            [protocol_pb.KILLMONSTER] = protocol_pb.SCKillMonster, --  KILLMONSTER = 52; //攻击怪物结果 CSKillMonster SCKillMonster
            [protocol_pb.CHAT] = protocol_pb.CSChatReq,         -- CHAT                = 20; //聊天请求  CSChatReq  SCChatReq
            [protocol_pb.CHATNOTIFY] = protocol_pb.SCChatNotify,
            [protocol_pb.GETCHATCHANNELLIST] = protocol_pb.SCGetChatChannelList,
           -- [protocol_pb.RESEACCHTASKINFO] = protocol_pb.SCNotifyResearchTaskInfo, -- RESEACCHTASKINFO=61;//发送研发任务信息 SCNotifyResearchTaskInfo  CSGetTaskInfo
            --[protocol_pb.RESEACCHTASkUPDATE] = protocol_pb.SCNotifyResearchTaskUpdate, --研发任务进度更新 SCNotifyResearchTaskUpdate
            --[protocol_pb.COPYRESCHANGE] = protocol_pb.SCNotifyCopyResChange, --玩家关卡资源改变  SCNotifyCopyResChange
            [protocol_pb.LEVELCHANGEHEROPROP] = protocol_pb.SCLevelChangeHeroProp, --改变出战英雄属性 CSLevelChangeHeroProp SCLevelChangeHeroProp
           -- [protocol_pb.BOOSTLISTINFO] = protocol_pb.SCBoostListInfo, --下发boost列表信息 SCBoostListInfo
          --  [protocol_pb.BUYBOOST] = protocol_pb.SCCommonAck,  -- BUYBOOST     =59;  //购买boost  CSBuyBoost
           -- [protocol_pb.BOOSTPROPINFO] = protocol_pb.SCBoostPropInfo,  -- 请求.回应boost属性信息 CSBoostPropInfo  SCBoostPropInfo
           -- [protocol_pb.NotifyRandEvent] = protocol_pb.SCNotifyRandEventInfo, --随机事件通知   SCNotifyRandEventInfo
           -- [protocol_pb.NotityMonEnhance] = protocol_pb.SCNotifyEnhanceMonster, --随机事件对怪物属性增强通知   SCNotifyEnhanceMonster
          -- [protocol_pb.CHECKMONSTERINFO] = protocol_pb.SCGetMonsterInfo, --查看怪物信息  CSGetMonsterInfo SCGetMonsterInfo
            [protocol_pb.ENTERWORLDMAP ] = protocol_pb.SCEnterWorldMap, --  101 进入世界地图 无 SCEnterWorldMap
            [protocol_pb.WORLDMAPOBJSAPPEAR ] = protocol_pb.SCWorldObjsAppear, -- 102 世界地图物件出现 SCWorldObjsAppear
            [protocol_pb.WORLDMAPOBJSDISAPPEAR  ] = protocol_pb.SCWorldObjsDisappear, -- 103; //世界地图物件消失 SCWorldObjsDisappear
            [protocol_pb.WORLDMAPSTARTMARCH  ] = protocol_pb.SCStartMarchLine,  -- 104; //开始行军 CSStartMarchLine SCStartMarchLine
            [protocol_pb.AREAMAPNTF  ] = protocol_pb.SCAreaMapNtf, -- 副本信息通知 SCAreaMapNtf
            [protocol_pb.CHAPTERTASKINFO] = protocol_pb.SCNotifyChapterTaskInfo, -- 章节任务信息 CSGetChapterTaskInfo SCNotifyChapterTaskInfo
            [protocol_pb.GETCHAPTERREWARD] = protocol_pb.SCGetChapterTaskReward, -- 获取章节任务奖励 CSGetChapterTaskReward SCGetChapterTaskReward
            [protocol_pb.WORLDMAPQUERYOBJ] = protocol_pb.SCWorldMapQuery, -- 查询信息
            [protocol_pb.WORLDMAPCOLLECTIONADD] = protocol_pb.SCCollectionAdd, -- 107; //收藏坐标 CSCollectionAdd SCCollectionAdd	
            [protocol_pb.WORLDMAPCOLLECTIONMOD] = protocol_pb.SCCollectionMod, -- 108; //收藏编辑 CSCollectionMod SCCollectionMod
            [protocol_pb.WORLDMAPCOLLECTIONDEL] = protocol_pb.SCCollectionDel, -- 109; //收藏删除 CSCollectionDel SCCollectionDel
            [protocol_pb.WORLDMAPGOHOME] = protocol_pb.SCWorldMapGoHome, -- 105; //回家CSWorldMapGoHome SCWorldMapGoHome	
            [protocol_pb.WORLDMAPSPEEDUP] = protocol_pb.SCSpeedUp, -- 110;//战车加速 CSSpeedUp SCSpeedUp
            [protocol_pb.WORLDUSEREPAIR] = protocol_pb.SCWMUseRepair, -- = 3000;//CSWMUseRepair  SCWMUseRepair
            [protocol_pb.WORLDMAPMYMARCHNOTIFY] = protocol_pb.SCWMMyMarchNotify, -- = 3002;//SCWMMyMarchNotify;
            [protocol_pb.WORLDMAPMYMARCHDELNOTIFY] = protocol_pb.SCWMMyMarchDelNotify, -- = 3003;//SCWMMyMarchDelNotify;
            [protocol_pb.WORLDMAPOBJQUERY] = protocol_pb.SCWMObjQuery, -- = 3004;//CSWMObjQuery, SCWMObjQuery;	
            [protocol_pb.WORLDMAPGATHERCARNOTIFY] = protocol_pb.SCGatherCarNotify, -- = 3005;// SCGatherCarNotify
            [protocol_pb.WARNNOTIFY] = protocol_pb.SCWarnNotify, --               = 240;//预警通知 SCWarnNotify	
            [protocol_pb.WARNDEL] = protocol_pb.SCWarnDel, --                 = 241;//预警消失 SCWarnDel	
            [protocol_pb.BATSHOWNOTIFY] = protocol_pb.SCBatShowNotify, --         			= 3022;//战斗特效广播 SCBatShowNotify
            [protocol_pb.WORLDMAPNOTIFYATKRECORD] = protocol_pb.SCNotifyAtkMonsterRecord, --			= 3021;//通知攻击怪物记录 SCNotifyAtkMonsterRecord

            -- 个人基地消息
            [protocol_pb.ENTERPERSONBASE] = protocol_pb.SCEnterPersonBase,
            [protocol_pb.PERSONBASEADD] = protocol_pb.SCPersonBaseAdd,
            [protocol_pb.PERSONBASEMOVE] = protocol_pb.SCPersonBaseMove,
            [protocol_pb.PERSONBASEDEL] = protocol_pb.SCPersonBaseDel,
            [protocol_pb.PERSONBASEQUERY] = protocol_pb.SCPersonBaseQuery,
            [protocol_pb.PERSONBASEAPPEAR] = protocol_pb.SCPersonBaseAppear,
            [protocol_pb.PERSONBASEDISAPPEAR] = protocol_pb.SCPersonBaseDisappear,
            [protocol_pb.PERSONBASELEVELUP] = protocol_pb.SCPersonBaseLevelUp,
            [protocol_pb.PERSONBASELEVELUPNOTIFY] = protocol_pb.SCPersonBaseLevelUpNotify,
            [protocol_pb.PERSONBASEBUILDINGSPEEDUP] = protocol_pb.SCBuildingSpeedUp,
            [protocol_pb.PERSONBASETECHNOLOGYUP] = protocol_pb.SCTechnologyUp,
            [protocol_pb.PERSONBASETECHNOLOGYUPNOTIFY] = protocol_pb.SCTechnologyUpNotify,
            [protocol_pb.TECHNOLYGYSPEEDUP] = protocol_pb.SCTechnologySpeedUp,
            [protocol_pb.CLICKBUBBLE] = protocol_pb.SCCLICKBUBBLIE,
            [protocol_pb.GETCELLINFO] = protocol_pb.SCGetCellInfo,
            [protocol_pb.EXILEPRISONER] = protocol_pb.SCExilePrisoner,
            [protocol_pb.SUMMONPRISONER] = protocol_pb.SCSummonPrisoner,
            [protocol_pb.RELEASEPRISONER] = protocol_pb.SCReleasePrisoner,
            [protocol_pb.STARTRECRUITFIGHT] = protocol_pb.SCStartRecuritFight,
            [protocol_pb.ENDRECRUITFIGHT] = protocol_pb.SCEndRecuritFight,
            [protocol_pb.NOTIFYBUILDEXPEND] = protocol_pb.SCNotifyBuildExpend,
            [protocol_pb.QUERYBUILDEXPEND] = protocol_pb.SCBuildingExpend, -- 查询建筑消耗
            [protocol_pb.PERSONBASEADDOFFICE] = protocol_pb.SCPBAddOffice,
            [protocol_pb.PERSONBASEREMOVEOFFICE] = protocol_pb.SCPBRemoveOffice,
            [protocol_pb.PERSONBASEDISPATCHOFFICE] = protocol_pb.SCPBDispatchOffice,
            [protocol_pb.PERSONBASESTARTPROJECT] = protocol_pb.SCPBStartProject,
            [protocol_pb.PERSONBASEADDPARTNER] = protocol_pb.SCPBAddPartner,
            [protocol_pb.PERSONBASEREMOVEPARTNER] = protocol_pb.SCPBRemovePartner,
            [protocol_pb.PERSONBASEFREETIMENOTIFY] = protocol_pb.SCPBBuildFreeTimeNotify,
            [protocol_pb.PERSONBASEBUILDCLEARTIME] = protocol_pb.SCPBBuildClearTime,
            [protocol_pb.PERSONBASESPEEDUP] = protocol_pb.SCPerSonBaseSpeedUp,
            [protocol_pb.NOTIFYBUFF] = protocol_pb.SCNotifyBuffList,
            [protocol_pb.GETSKININFO] = protocol_pb.SCGetSkinInfo,
            [protocol_pb.SKININFOCHANGE] = protocol_pb.SCSkinInfoChange,
            [protocol_pb.USESKIN] = protocol_pb.SCUseSkin,
            [protocol_pb.TRAINSOLDIER] = protocol_pb.SCTrainSoldier,
            [protocol_pb.RECEIVESOLDIER] = protocol_pb.SCReceiveSoldier,
            [protocol_pb.NotifyPbUpdate] = protocol_pb.SCNotifyPbUpdate,
            [protocol_pb.GETBUILDDETAILS] = protocol_pb.SCGetBuildDetails,
            [protocol_pb.TreatSoldier] = protocol_pb.SCTreatSoldier,
            [protocol_pb.RECVTREATEDSOLDIER] = protocol_pb.SCRecvTreatedSoldier,
            [protocol_pb.DELETESOLDIER] = protocol_pb.SCDeleteSoldier,
            [protocol_pb.FIREFIGHT] = protocol_pb.SCFireFight,
            [protocol_pb.REPAIRCORE] = protocol_pb.SCRepairCore,
            [protocol_pb.REPAIRTRAP] = protocol_pb.SCRepairTrap,
            [protocol_pb.RECVREPAIREDTRAP] = protocol_pb.SCRecvRepairedTrap,
            [protocol_pb.CHANGEGARRISON] = protocol_pb.SCChangeGarrison,

            [protocol_pb.USEITEM] = protocol_pb.SCUseItem,  --74; //使用道具 CSUseItem SCUseItem
            [protocol_pb.SALEITEM] = protocol_pb.CSSaleItem,  --75; //出售道具 CSSaleItem

            -- 任务
            [protocol_pb.NOTIFYTASKINFO] = protocol_pb.SCTaskInfo,  -- 76  //任务信息 SCTaskInfo
            [protocol_pb.NOTIFYACTIVEPOIOT] = protocol_pb.SCActivePointInfo,  -- 77  //日常任务活跃度 SCActivePointInfo
            [protocol_pb.GETTASKREWARD] = protocol_pb.SCGetTaskReward, --获取任务奖励  CSGetTaskReward       SCGetTaskReward
           -- [protocol_pb.GETNORTASKREWARD] = protocol_pb.SCGetNorTaskAward,  -- 78  //领取奖励 SCGetNorTaskAward CSGetNorTaskAward

            -- 获得道具 金币 钻石等资源通知
            [protocol_pb.NOTIFYGETITEM] = protocol_pb.SCGetItem,    -- 79 //获得道具 SCGetItem
            -- 装备分解
            [protocol_pb.RESOLVEEQUIP] = protocol_pb.SCResolveEquip,      --81   //分解武器 CSResolveEquip SCResolveEquip

            -- 章节出战英雄和关卡布阵
            [protocol_pb.CHAPTERATTENDINFO] = protocol_pb.SCChapterAttendHeroes,  --  =111; //获取玩家出战信息 CSReqChapterAttendHeroes SCChapterAttendHeroes
            [protocol_pb.SELECTCHAPTERATTENDHEROES] = protocol_pb.SCSelectChapterAttendHeroes,  --  =111; //获取玩家出战信息 CSReqChapterAttendHeroes SCChapterAttendHeroes
            [protocol_pb.LEVELEMBATTLE] = protocol_pb.SCLevelEmbattle,  --  =111; //获取玩家出战信息 CSReqChapterAttendHeroes SCChapterAttendHeroes

            -- emp
            --[protocol_pb.NOTIFYEMPINFO] = protocol_pb.SCEMPInfo,  --  OTIFYEMPINFO  = 300; //通知EMP信息 SCEMPInfo
            -- [protocol_pb.EMPMODUPGRADE] = protocol_pb.SCEMPModuleUpgrade,  --  EMPMODUPGRADE  = 301; //EMP模块升级 CSEMPModuleUpgrade SCEMPModuleUpgrade
            --[protocol_pb.USEEMP] = protocol_pb.SCUseEMP,  --  	USEEMP         = 302; //使用EMP CSUseEMP SCUseEMP

            -- 商店
            [protocol_pb.SHOPITEMLIST] = protocol_pb.SCShopItemList,
            [protocol_pb.BUY_GOODS] = protocol_pb.SCBuyGoods,
            [protocol_pb.GETSHOPLIST] = protocol_pb.SCGetShopList,
            [protocol_pb.GAMESHOPREFRESH] = protocol_pb.SCGameShopRefresh,

            -- 声望
            [protocol_pb.FAMEINFO] = protocol_pb.SCFameInfo,	-- FAMEINFO  =145; //声望信息 SCFameInfo

            -- 被踢下线
            [protocol_pb.SVR_CLI_KICK_PLAYER] = protocol_pb.SCKickPlayer,	-- SVR_CLI_KICK_PLAYER  =65502; //踢下线 SCKickPlayer

            -- 公告
            [protocol_pb.ANNOUCEMENTNOTIFY] = protocol_pb.SCAnnouceNotify,  -- ANNOUCEMENTNOTIFY = 400; //公告通知 SCAnnouceNotify

            -- 关卡
            [protocol_pb.OVERCOPY] = protocol_pb.SCOverCopy,    -- OVERCOPY = 72; //通关关卡 SCOverCopy
            [protocol_pb.GETCOPYTASKREWARD] = protocol_pb.SCGetCopyTaskReward, -- GETCOPYTASKREWARD= 90;//领取副本任务奖励 CSGetCopyTaskReward SCGetCopyTaskReward
            [protocol_pb.GETCOPYGIFTBAG] = protocol_pb.SCGetCopyGiftBag, -- GETCOPYGIFTBAG	 = 91;//领取副本礼包 CSGetCopyGiftBag SCGetCopyGiftBag

            [protocol_pb.CREATEFAMILY] = protocol_pb.SCCreateFamily,            -- 创建工会
            [protocol_pb.NOTIFYFAMILYINFO] = protocol_pb.SCNotifyFamilyInfo,    -- 通知工会信息
            [protocol_pb.GETFAMILYLIST] = protocol_pb.SCGetFamilyList,           -- 获取工会列表
            [protocol_pb.FAMILYJOINORCANCEL] = protocol_pb.SCFamilyJoinOrCancel,           
            [protocol_pb.GETMEMBERLIST] = protocol_pb.SCGetFamilyMemberList, -- 获取工会成员列表
            [protocol_pb.GETFAMILYINFO] = protocol_pb.SCGetFamilyInfo, -- 获取工会信息
            [protocol_pb.INVITEDTOFAMILY] = protocol_pb.SCInvitePlayerToFamily, -- 邀请加入工会
            [protocol_pb.NOTIFYINVITEDFAMILY] = protocol_pb.SCNotifyInvitedToFamily, -- 通知玩家有人邀请进工会
            [protocol_pb.GETINVITEDFAMILYLIST] = protocol_pb.SCGetInvitedFamilyList, -- 获取邀请我的工会列表
            [protocol_pb.DISPOSEFAMILYINVITE] = protocol_pb.SCDisposeFamilyInviting, -- 处理工会申请信息
            [protocol_pb.FAMILYMEMBERPROMOTE] = protocol_pb.SCFamilyMemberPromote, -- 工会成员升阶降阶
            [protocol_pb.EXPELFAMILYMEMBER] = protocol_pb.SCExpelFamilyMember, -- 开除工会成员
            [protocol_pb.GETFAMILYOG] = protocol_pb.SCGetFamilyLog, -- 获取工会日志
            [protocol_pb.MOVEPERSONALBASE] = protocol_pb.SCMovePersonalBase, -- 工会搬迁
            [protocol_pb.FAMILYLEAVEWORD] = protocol_pb.SCFamilyLeaveWord, -- 工会留言
            [protocol_pb.GETFAMILYLEAVEWORD] = protocol_pb.SCGetFamilyLeaveWord, -- 获取工会留言
            [protocol_pb.NOTIFYNEWLEAVEWORD] = protocol_pb.SCNotifyFamilyLeaveWord, -- 通知新的留言
            [protocol_pb.MODIFYFAMILYINFO] = protocol_pb.SCModifyFamilyInfo, -- 修改工会信息
            [protocol_pb.QUITFAMILY] = protocol_pb.SCQuitFamily, -- 退出工会
            [protocol_pb.GETAPPLYLIST] = protocol_pb.SCGetFamilyApplyList, 		-- 504;//获取工会玩家申请列表 CSGetFamilyApplyList SCGetFamilyApplyList
            [protocol_pb.DISPOSEFAMILYAPPLY] = protocol_pb.SCDisposeFamilyApply, 	            -- DISPOSEFAMILYAPPLY	= 523;//会长或r4处理工会的申请列表 CSDisposeFamilyApply SCDisposeFamilyApply	
            [protocol_pb.GETFAMILYGIFTINFO] = protocol_pb.SCGetFamilyGiftInfo, -- 工会礼物信息
            [protocol_pb.GETFAMILYGIFTREWARD] = protocol_pb.SCGetFamilyGiftReward, -- 领取工会礼物
            [protocol_pb.GETGIFTGAININFO] = protocol_pb.SCGetFamilyGiftGainInfo, -- 工会礼物领取信息
            [protocol_pb.TRANSFERLEADER] = protocol_pb.SCTransferFamilyLeader, -- = 526;//转让会长 CSTransferFamilyLeader SCTransferFamilyLeader
            -- [protocol_pb.MEMBERINFOCHANGE] = protocol_pb.SCNotifyMemberInfoChange, -- = 527;//工会成员信息改变 SCNotifyMemberInfoChange
            [protocol_pb.GETMYAPPLYFAMILY] = protocol_pb.SCGetMyApplyFamily, -- 	= 541;//获取我申请中的工会列表 CSGetMyApplyFamily SCGetMyApplyFamily
            [protocol_pb.DELFAMILYGIFT] = protocol_pb.SCDelFamilyGift, --		= 542;//删除公会礼物 CSDelFamilyGift SCDelFamilyGift
            [protocol_pb.NOTIFYFAMILYGIFTEXP] = protocol_pb.SCNotifyFamilyGiftExp, --	= 544;//通知工会礼物经验相关 SCNotifyFamilyGiftExp
            [protocol_pb.NOTIFYNEWFAMILYGIFT] = protocol_pb.SCNotifyNewFamilyGift, --	= 545;//通知新的礼物 SCNotifyNewFamilyGift
            [protocol_pb.DELFAMILYBUILD] = protocol_pb.SCDelFamilyBuild, --		= 548;//销毁工会建筑 CSDelFamilyBuild SCDelFamilyBuild
            [protocol_pb.WORLDMAPRETURNFAMILYFORTRESS] = protocol_pb.SCWorldMapReturnFamilyFortress, -- = 3019;//飞回联盟基地 CSWorldMapReturnFamilyFortress SCWorldMapReturnFamilyFortress
            [protocol_pb.SETRECOMMENDTECH] = protocol_pb.SCSetRecommendTech, --	= 553;//设置推荐科技 CSSetRecommendTech SCSetRecommendTech
            [protocol_pb.RECRUITHERO] = protocol_pb.SCRecruitHero, -- 指挥官招募

            -- 邮件
            [protocol_pb.MAILLISTINFO] = protocol_pb.SCGetMailListInfo, -- 600;//获取邮件列表信息 CSGetMailListInfo SCGetMailListInfo
            [protocol_pb.READMAIL] = protocol_pb.SCReadMail,  --601; //读邮件 CSReadMail SCReadMail
            [protocol_pb.DELMAIL] = protocol_pb.SCDelMail,  --603;//删除邮件 CSDelMail SCDelMail
            [protocol_pb.GETMAILITEMS] = protocol_pb.SCGetMailItems,  --602;//提取邮件附件 CSGetMailItems SCGetMailItems 
            [protocol_pb.SENDFAMILYMAIL] = protocol_pb.SCSendFamilyMail,  --525;//发送工会邮件 CSSendFamilyMail SCSendFamilyMail
            [protocol_pb.ALLREASMAIL] = protocol_pb.SCReadMoreMails,  --= 604;//一键阅读邮件 CSReadMoreMails  SCReadMoreMails
            [protocol_pb.COLLECTMAIL] = protocol_pb.SCCollectMail,  --			= 605;//收藏邮件 CSCollectMail SCCollectMail

            [protocol_pb.FIGHTSTART] = protocol_pb.SCFightStart,  -- =86; //战斗开始 CSFightStart SCFightStart

            [protocol_pb.REQHELPFASTBUILD] = protocol_pb.SCReqHelpFastBuild,  -- = 527;//请求加速建造帮助 CSReqHelpFastBuild SCReqHelpFastBuild
            [protocol_pb.GETFAMILYHELPLIST] = protocol_pb.SCGetFamilyHelpList,  -- = 528;//获取需要帮助列表 CSGetFamilyHelpList SCGetFamilyHelpList
            [protocol_pb.HELPFASTBUILD] = protocol_pb.SCHelpFastBuild,  -- = 529;//帮助加速建造 CSHelpFastBuild SCHelpFastBuild 
            [protocol_pb.GETFAMILYBUILD] = protocol_pb.SCGetFamilyBuildList,  -- = 530;//获取工会建筑 CSGetFamilyBuildList SCGetFamilyBuildList
            [protocol_pb.UPDATEHELPLIST] = protocol_pb.SCUpdateHelpList,  -- = 533;//更新工会帮助列表 SCUpdateHelpList

            [protocol_pb.BUILDFAMILYBUILDING] = protocol_pb.SCBuildFamilyBuilding, -- 创建工会建筑
            [protocol_pb.UPGRADEFAMILYBUDILING] = protocol_pb.SCUpgradeFamilyBuilding, -- 升级工会建筑
            [protocol_pb.FAMILYBUILDFAST] = protocol_pb.SCFamilyFastBuild, -- = 534;//加速建造工会建筑 CSFamilyFastBuild SCFamilyFastBuild
            [protocol_pb.NOTIFYFAMILYBUILDING] = protocol_pb.SCNotifyFamilyBuilding, -- = 535;//通知工会建筑信息 SCNotifyFamilyBuilding
            [protocol_pb.WORLDMAPSELECITY] = protocol_pb.SCWMSelecity, -- = 111;//选择公共城CSWMSelectCity, SCWMSelecity
            [protocol_pb.WORLDMAPGETBUILDCOUNT] = protocol_pb.SCGetBuildCount, -- = 114;//获取世界地图建造CSWMGetBuildCount, SCGetBuildCount
            [protocol_pb.WORLDMAPQUERYTIME] = protocol_pb.SCWMQueryTime, -- = 112;//查询行军时间CSWMQueryTime, SCWMQueryTime
            [protocol_pb.WORLDMAPGATHERCARS] = protocol_pb.SCWMGetGatherCar, -- = 115;//获取矿厂下面的矿车列表 CSWMGetGatherCar, SCWMGetGatherCar
            [protocol_pb.WORLDMAPGETEAMS] = protocol_pb.SCWMGetTeams, -- = 116;//获取队伍列表 CSWMGetTeams,  SCWMGetTeams
            [protocol_pb.WORLDMAPMOVERES] = protocol_pb.SCWMMoveRes, --  = 117;//资源运输 CSWMMoveRes,  SCWMMoveRes
            [protocol_pb.WORLDMAPDESTORY] = protocol_pb.SCWMDestory, --  = 118;//建筑销毁 CSWMDestory,  SCWMDestory
            [protocol_pb.WORLDMAPDESTORYNOTIFY] = protocol_pb.SCWMBuildDestory, --  = 119;//建筑销毁通知 SCWMBuildDestory
            [protocol_pb.WORLDMAPBUILDLEVELUP] = protocol_pb.SCWMBuildLevelUp, --  = 120;// 建筑升级 CSWMBuildLevelUp SCWMBuildLevelUp
            [protocol_pb.WORLDMAPREPARIR] = protocol_pb.SCWMRepair, --  = 121;// 修理建筑 CSWMRepair SCWMRepair
            [protocol_pb.WORLDMAPGETHELPS] = protocol_pb.SCWMGetHelps, --  = 122;//获取支援队伍列表 CSWMGetHelps,  SCWMGetHelps
            [protocol_pb.WORLDMAPBATRECNOTIFY] = protocol_pb.SCWMBatRecNotify, -- 123;//战报通知   SCWMBatRecNotify
            [protocol_pb.WORLDMAPGETMASSLIST] = protocol_pb.SCWMGetMassList, --  = 124;//获取集结列表  CSWMGetMassList SCWMGetMassList
            [protocol_pb.WORLDMAPGETMASSINFO] = protocol_pb.SCWMGetMassInfo, --   = 125;//集结详情  CSWMGetMassInfo SCWMGetMassInfo
            [protocol_pb.WORLDMAPGETMASSDESTORY] = protocol_pb.SCWMGetMassDestory, --   = 126;//集结销毁  CSWMGetMassDestory SCWMGetMassDestory
            [protocol_pb.WORLDMAPGETOWNERBUILD] = protocol_pb.SCWMGetOwnerBuild, --  = 127;//获取跟人建筑 CSWMGetOwnerBuild SCWMGetOwnerBuild
            [protocol_pb.WORLDMAPMASSDEL] = protocol_pb.SCWMMassDel, --   = 129;//集结消失通知 SCWMMassDel
            [protocol_pb.WORLDMAPMASSNOTIFY] = protocol_pb.SCWMMassNotify, --  = 130;//集结队伍变化通知 SCWMMassNotify
            [protocol_pb.WORLDMAPMASSDELMEM] = protocol_pb.SCWMMassDelMem, --  = 128;//移除集结队伍 CSWMMassDelMem SCWMMassDelMem
            [protocol_pb.WORLDMAPEDITTEAM] = protocol_pb.SCWMEditTeam, --    = 131;//CSWMEditTeam SCWMEditTeam
            [protocol_pb.WORLDMAPBUILDTEAMCOUNT] = protocol_pb.SCWMBuildTeamCount, --  = 132;//SCWMBuildTeamCount
            [protocol_pb.WORLDMAPOBJOUT] = protocol_pb.SCWorldObjsOut, --  = 134;//移动到目标点 SCWorldObjsOut
            [protocol_pb.WORLDMAPMINIMAP] = protocol_pb.SCWorldMapMiniMap, -- = 3007;//世界地图缩略图 CSWorldMapMiniMap SCWorldMapMiniMap
            [protocol_pb.WORLDMAPMASSEDITSET] = protocol_pb.SCWorldMapMassEditSet, -- = 3008;//集结设置编辑 CSWorldMapMassEditSet SCWorldMapMassEditSet 
            [protocol_pb.WORLDMAPATKCOUNTNOTIFY] = protocol_pb.SCWorldMapAtkCountNotify, -- = 3009;//打怪次数通知 SCWorldMapAtkCountNotify
            [protocol_pb.WORLDMAPCHANGENAME] = protocol_pb.SCWorldMapChangeName, -- = 3010;//修改建造名称 CSWorldMapChangeName SCWorldMapChangeName
            [protocol_pb.WORLDMAPLEVIATHANMOVE] = protocol_pb.SCWorldMapLeviathanMove, -- = 3011;	//个人基地移动 CSWorldMapLeviathanMove SCWorldMapLeviathanMove
            [protocol_pb.WORLDMAPSTARTARMY] = protocol_pb.SCStartArmyLine, -- = 3013;		//CSStartArmyLine SCStartArmyLine
            [protocol_pb.WORLDMAPSTARTTRANSITION] = protocol_pb.SCStartTransition, --  = 3014;		//CSStartTransition SCStartTransition
            [protocol_pb.WORLDMAPRANDTRANSITION] = protocol_pb.SCRandTransition, -- = 3015;		//CSRandTransition SCRandTransition
            [protocol_pb.WORLDMAPSTARTMASS] = protocol_pb.SCStartMass, -- = 3016;			//CSStartMass SCStartMass
            [protocol_pb.WORLDMAPMASSATK] = protocol_pb.SCMassAtk, -- = 3017;				//CSMassAtk SCMassAtk

            [protocol_pb.BUYITEM] = protocol_pb.SCBuyItem, --			=139; //购买道具 CSBuyItem SCBuyItem

            -- 阵形，布阵相关
            [protocol_pb.FORMATIONLIST] = protocol_pb.SCNotifyFormationList,
            [protocol_pb.NOTIFYADDFORMATION] = protocol_pb.SCNotifyAddFormation,
            [protocol_pb.COPYEMBATTLE] = protocol_pb.SCCopyEmbattle,
            [protocol_pb.ADDPRESETFORMATION] = protocol_pb.SCAddPresetFormation,
            [protocol_pb.DELPRESETFORMATION] = protocol_pb.SCDelPressetFormation,
            [protocol_pb.GETPRESETFORMATION] = protocol_pb.SCGetPresetFormation,
            [protocol_pb.GETTEAMSWORD] = protocol_pb.SCQueryTeamSword,
            [protocol_pb.ADDSTORYLEVELHERO] = protocol_pb.SCAddStoryLevelHero,
            [protocol_pb.DELSTORYLEVELHERO] = protocol_pb.SCDelStoryLevelHero,
            [protocol_pb.TESTFIGHTPROP] = protocol_pb.SCTestFightProp,

            -- 改装车体
            --[protocol_pb.REFITCAEBODY] = protocol_pb.SCRefitCarBody, -- 161;//改装车体 CSRefitCarBody SCRefitCarBody
            --[protocol_pb.CANCELREFITCARBODY] = protocol_pb.SCCancelRefitCarBody, -- 162;//取消改造车体 CSCancelRefitCarBody SCCancelRefitCarBody
           -- [protocol_pb.SPEEDUPREFIT] = protocol_pb.SCSpeedUpRefit, -- 163;//加速改造 CSSpeedUpRefit SCSpeedUpRefit
           -- [protocol_pb.GETCARBODYINFO] = protocol_pb.SCGetCarBodyInfo, -- 24;//获取车体信息 CSGetCarBodyInfo SCGetCarBodyInfo

            -- 修理厂
            --[[
            [protocol_pb.GETREPAIRFACTORYINFO] = protocol_pb.SCGetRepairFactoryInfo, -- = 200;//获取修理厂信息 CSGetRepairFactoryInfo SCGetRepairFactoryInfo	
            [protocol_pb.PERSONBASEADDREPAIRHERO] = protocol_pb.SCAddRepairHero, -- = 201;//添加修理英雄 CSAddRepairHero SCAddRepairHero	
            [protocol_pb.PERSONBASECANCELREPAIRHERO] = protocol_pb.SCCancelRepairHero, -- = 202;//取消英雄修理 CSCancelRepairHero SCCancelRepairHero
            [protocol_pb.GETNEEDREPAIRHERO] = protocol_pb.SCGetNeedRepairHero, -- = 203;//获取需要修理的英雄 CSGetNeedRepairHero SCGetNeedRepairHero
            [protocol_pb.RELIVEHERO] = protocol_pb.SCReliveHero,  -- 25;//复活英雄
            [protocol_pb.SPEEDUPREPAIRHERO]            = protocol_pb.SCSpeedUpRepairHero,        --加速修理战车
            [protocol_pb.GETFIXFACTORYMAKEIFNO]          = protocol_pb.SCGetFactoryMakeInfo,    --获取修理厂制造信息
            [protocol_pb.MAKEREPAIRPACKAGE]           = protocol_pb.SCMakeRepairPackage,    -- 生产修理包
            [protocol_pb.GETREPAIRPACKAGE]           = protocol_pb.SCGetRepairPackage,    -- 领取修理包
            [protocol_pb.SPEEDUPMAKEREPACKAGE]           = protocol_pb.SCSpeedUpMakeRePackage,    -- 加速制作修理包
            [protocol_pb.USEREPAIRPACKAGE]           = protocol_pb.SCUseRepairPackage,    -- 使用修理包
            ]]
            -- 兵工厂
            [protocol_pb.GETARMORYINFO] = protocol_pb.SCGetArmoryInfo,  -- = 210;//获取兵工厂信息 CSGetArmoryInfo SCGetArmoryInfo
            [protocol_pb.PRODUCTEQUIP] = protocol_pb.SCProductEquip,    -- = 211;//生产装备 CSProductEquip SCProductEquip
            [protocol_pb.GETEQUIP] = protocol_pb.SCGetEquip,    -- = 212;//领取生产的装备 CSGetEquip SCGetEquip
            [protocol_pb.SPEEDUPPRODUCTEQUIP] = protocol_pb.SCSpeedUpProductEquip,    -- = 213;//加速生产装备 CSSpeedUpProductEquip SCSpeedUpProductEquip
            [protocol_pb.GETRESOURCE] = protocol_pb.SCGetMineFieldMineral,   -- 领取矿场矿石
            [protocol_pb.GETDRAWINGINFO] = protocol_pb.SCGetDrawingInfo,   -- = 215;//获取制造图纸信息 CSGetDrawingInfo SCGetDrawingInfo

            -- 章节目标
            [protocol_pb.CHAPTERACHIEVEGETLIST] = protocol_pb.SCGetChapterAchievements,  -- = 3501;//获取章节成就列表CSGetChapterAchivements SCGetChapterAchivements 
            [protocol_pb.CHAPTERACHIEVEGETREWARD] = protocol_pb.SCGetChapterAchieveReward,  -- = 3502;//获取章节成就奖励CSGetChapterArhiveReward SCGetChapterArhiveReward
            [protocol_pb.CHAPTERACHIEVESTART] = protocol_pb.SCChapterAchieveStart, -- = 3503;//开启章节成就任务 CSChapterAchieveStart SCChapterAchieveStart

            -- 新手引导
            [protocol_pb.NOTIFYGUIDE] = protocol_pb.SCGuideInfo,  -- = 27;//通知新手引导 CSGuideInfo SCGuideInfo
            [protocol_pb.SAVEGUIDE] = protocol_pb.SCSaveGuideInfo,  -- = 28;//保存新手引导 CSSaveGuideInfo SCSaveGuideInfo

            -- 世界地图搜索
            [protocol_pb.WORLDMAPSEARCH] = protocol_pb.SCWorldMapSearch,  -- = 3006;//搜素请求 CSWorldMapSearch SCWorldMapSearch

            [protocol_pb.FAMILYTECHDONATE] = protocol_pb.SCFamilyTechDonate, -- 工会科技捐献
            [protocol_pb.GETFAMILYTECH] = protocol_pb.SCGetFamilyTech, -- 工会科技信息获取
            [protocol_pb.HELPNOTICE] = protocol_pb.SCHelpNotice, -- 公会帮助通知

            -- 世界地图战斗
            [protocol_pb.WORLDMAPBATMSGNOTIFY] = protocol_pb.SCBatResultNotify, -- 公会帮助通知

            -- 领主
            [protocol_pb.GETCAPTAINGIFT] = protocol_pb.SCGetCaptainGift,
            [protocol_pb.NATURALGIFTLEVELUP] = protocol_pb.SCNaturalGiftLevelup,
            [protocol_pb.NATURALGIFTRESET] = protocol_pb.SCNaturalGiftReset,
            [protocol_pb.GETGLOBALPROPINFO] = protocol_pb.SCGetGlobalPropInfo,
            [protocol_pb.GETPLAYERPVPDATA] = protocol_pb.SCGetPlayerPvpData,

            [protocol_pb.TRAINPROFESSION] = protocol_pb.SCTrainProfession,
            [protocol_pb.RETIREPROFESSION] = protocol_pb.SCRetireProfession,
            [protocol_pb.RECVPROFESSION] = protocol_pb.SCRecvProfession,
            [protocol_pb.NOTIFYCAPTAININFO] = protocol_pb.SCNotifyCaptainInfo,
            [protocol_pb.NOTIFYPROFESSIONINFO] = protocol_pb.SCNotifyProfessionInfo,
            [protocol_pb.HANDLEPROFESSION] = protocol_pb.SCHandleProfession,

            [protocol_pb.DISMANTLEMATERIAL] = protocol_pb.SCDismantleMaterial,
            [protocol_pb.COMPOSEMATERIAL] = protocol_pb.SCComposeMaterial,
            [protocol_pb.GETDEPORTINFO] = protocol_pb.SCGetDepotPropList,
            [protocol_pb.DRAWCARD] = protocol_pb.SCDrawCard,
            [protocol_pb.MODIFYROLENAME] =  protocol_pb.SCModifyRoleName,

            -- 资源船解锁
            [protocol_pb.SETCHAPTERSTATE] = protocol_pb.SCSetChapterState,
            [protocol_pb.CHAPTERFIGHT] = protocol_pb.SCStartChapterFight,
            [protocol_pb.UNLOCKCOPY] =  protocol_pb.SCUnLockCopy,

            -- 游戏活动
            [protocol_pb.GETACTIVELIST] =  protocol_pb.SCGetActiveList,
            [protocol_pb.DELACTIVE] =  protocol_pb.SCDelActive,
            [protocol_pb.NOTIFYACTIVE] =  protocol_pb.SCNotifyActive,

            -- 红点
            [protocol_pb.REDPOINTAPPEAR] = protocol_pb.SCRedPointAppear,
            [protocol_pb.REDPOINTDISAPPEAR] = protocol_pb.SCRedPointDisappear,

            -- 获取玩家信息
            [protocol_pb.GETPLAYERINFO] = protocol_pb.SCGetPlayerInfo,

            -- 被帮助通知
            [protocol_pb.NOTIFYBEHELPED] = protocol_pb.SCNotifyBeHelped, --		= 555;//SCNotifyBeHelped
        }
    end,

    -- 根据协议 cmd 创建协议对象
    createInstance = function (self, cmd)
        local pb = self.map and self.map[cmd]
        return pb and pb()
    end, 

    connect = function(self, ip, port)
        if super.connect(self, ip, port) then
            self.ip = ip
            self.port = port
            return true
        else
            return false
        end
    end,

    -- 是否在连接中
    isConnecting = function(self)
        return self.state == "begin"
    end,

    -- 是否连接成功
    isConnected = function(self)
        return self.state == "connect"
    end,

    -- 是否连接关闭
    isClosed = function(self)
        return self.state == "close"
    end,

    -- 是否是安全包
    isSafeMessage = function(self, cmd)
        if not self.safe_cmds then
            self.safe_cmds = {}
            self.safe_cmds[protocol_pb.CREATEROLE] = true
            self.safe_cmds[protocol_pb.SELECTROLE] = true
            self.safe_cmds[protocol_pb.ENTERPERSONBASE] = true
            self.safe_cmds[protocol_pb.ENTERMAP] = true
            self.safe_cmds[protocol_pb.ENTERWORLDMAP] = true
            self.safe_cmds[protocol_pb.OVERCOPY] = true
            self.safe_cmds[protocol_pb.ATTACKMONSTER] = true
            self.safe_cmds[protocol_pb.KILLMONSTER] = true
            self.safe_cmds[protocol_pb.ENTERWORLDMAP] = true
            self.safe_cmds[protocol_pb.LEAVEWORLDMAP] = true
            self.safe_cmds[protocol_pb.READMAIL] = true
            self.safe_cmds[protocol_pb.NOTIFYGUIDE] = true
            self.safe_cmds[protocol_pb.SAVEGUIDE] = true
            self.safe_cmds[protocol_pb.WORLDMAPQUERYOBJ] = true
            self.safe_cmds[protocol_pb.WORLDMAPSEARCH] = true
            self.safe_cmds[protocol_pb.CHAPTERACHIEVEGETLIST] = true
            self.safe_cmds[protocol_pb.CHAPTERACHIEVESTART] = true
            self.safe_cmds[protocol_pb.GETACTIVELIST] = true
            self.safe_cmds[protocol_pb.PERSONBASEADD] = true
            self.safe_cmds[protocol_pb.PRODUCTEQUIP] = true
            self.safe_cmds[protocol_pb.TRAINSOLDIER] = true
            self.safe_cmds[protocol_pb.WORLDMAPSTARTARMY] = true
            self.safe_cmds[protocol_pb.UNLOCKCOPY] = true
            self.safe_cmds[protocol_pb.SETCHAPTERSTATE] = true
            self.safe_cmds[protocol_pb.CHAPTERACHIEVEGETREWARD] = true
            
        end

        return self.safe_cmds[cmd]
    end,

    -- 添加一个消息处理 
    registerMessageHandler = function(self, handler)
        table.insert(self.message_handlers, handler)
    end,

    -- 移除消息处理
    unregisterMessageHandler = function(self, handler)
        local i = 1
        while i <= #self.message_handlers do
            if self.message_handlers[i] == handler then
                table.remove(self.message_handlers, i)
            else
                i = i + 1
            end
        end
    end,

    -- 添加一个等待回包的消息
    addWaitingMessage = function(self, cmd, obj)
        table.insert(self.wait_ack_messages, {
            cmd = cmd,
            obj = obj,
            elapsed = 0.0,
        })
    end,

    -- 发送包
    send = function(self, cmd, obj, is_resend)
        --log.info('network send:'..cmd)

        -- 如果是安全包，放到等待回包列表中
        if self:isSafeMessage(cmd) then
            self:addWaitingMessage(cmd, obj) 
        end

        if is_resend then
            log.info("Resend:"..cmd)
            super.send(self, cmd, obj, is_resend)    -- 先全都认为是重发消息
        else
            super.send(self, cmd, obj, false)
        end
    end,

    -- 收到消息 
    onMessage = function(self, cmd, msg)
        --log.info('## receive :'..tostring(cmd))

        if UnityEngine.Application.isEditor then
            app:startCoroutine(function()
                coroutine.yield(nil)
                for _,handler in pairs(self.message_handlers) do
                    handler:onMessage(cmd, msg)
                end    
                --[[
                xpcall(
                    function()
                        for _,handler in pairs(self.message_handlers) do
                            handler:onMessage(cmd, msg)
                        end    
                    end, 
                    function()
                        log.error("onMessage exception:"..debug.traceback())
                    end)]]
            end) 
        else

            xpcall(
                function()
                    for _,handler in pairs(self.message_handlers) do
                        handler:onMessage(cmd, msg)
                    end    
                end, 
                function()
                    log.error("onMessage exception:"..debug.traceback())
                end)

        end
        
        gui_helper:showErrorBox(msg.respCode or msg.RespCode or msg.errorCode)

        -- 从等待回包列表中去掉
        if self:isSafeMessage(cmd) then
            for k,v in pairs(self.wait_ack_messages) do
                if v and v.cmd == cmd then
                    table.remove(self.wait_ack_messages, k)
                    break
                end
            end
        end
    end,

    -- 逻辑更新
    update = function(self, dt)
        local resend_msg = nil
        local show_waiting = false

        for k,v in pairs(self.wait_ack_messages) do
            if v then
                v.elapsed = v.elapsed + dt
                if v.elapsed > 0.1 then
                    show_waiting = true
                end

                if v.elapsed > 4.0 then -- 超时重发
                   resend_msg = v
                   table.remove(self.wait_ack_messages, k)
                   break
                end
            end
        end

        if resend_msg then 
            self:send(resend_msg.cmd, resend_msg.obj, true)
        end

        if show_waiting then
            if not frame_manager:isDisplay('waiting') then
                frame_manager:show('waiting')
            end
        else
            if frame_manager:isDisplay('waiting') then
                frame_manager:hide('waiting')
            end
        end
    end,


}

network = network_cls()
return network


