local log = logger.get("Game", "Network Sender")
require("network.protocol_pb")

return class {
    inst = nil,
    ctor = function(self, _network)
        self.network = _network
        self.inst = self
    end,

    send = function(self, cmd, msg)
        --[[
        if cmd ~= 10020 then
            log.error("### send:"..cmd)
            log.error(debug.traceback())
        end]]

        self.network:send(cmd, msg)
    end,

    heartBeat = function(self)
        local msg = protocol_pb.HeartBeat()
        msg.time = math.floor(Time.unscaledTime * 1000)
        self:send(protocol_pb.CLIENTGATEWAYHEARTBEAT, msg)
    end,

    login = function(self, account, is_reconnect)
        local msg = protocol_pb.CSLogin()
        msg.account = account
        msg.gs_id = 999
        msg.token = 0
        msg.cli_version = 0
        if is_reconnect then
            msg.bReConnect = 1
        else
            msg.bReConnect = 0
        end

        self:send(protocol_pb.LOGIN, msg)
    end,

    -- 创建角色
    createRole = function(self, role_name)
        local msg = protocol_pb.CSCreateRole()
        msg.rolename = role_name
        msg.account = ''
    
        self:send(protocol_pb.CREATEROLE, msg)
    end,

    -- 选择角色
    selectRole = function(self, player_id)
        local msg = protocol_pb.CSSelectRole() 
        msg.playerid = tostring(player_id)

        log.info('select role:'..msg.playerid)
        self:send(protocol_pb.SELECTROLE, msg)
    end,

    -- 删除角色
    deleteRole = function(self, player_id)
        local msg = protocol_pb.CSDelRole() 
        msg.playerid = tostring(player_id)

        log.info('delete role:'..msg.playerid)
        self:send(protocol_pb.DELROLE, msg)
    end,

    -- 请求进地图
    enterMap = function(self, id, cost)
        log.info("##### enterMap:"..id)
        local level_cfg = config.game_copy_levels[id]
        level_info.level_items = loadstring('return '..level_cfg.default_supply)()

        level_info.cur_chapter = config.game_copy_levels[id].Chapter
        level_info.map_id = id

        --log.error("### level_info:"..level_info.cur_chapter..","..level_info.map_id)
        local msg = protocol_pb.CSEnterMap () 
        msg.mapId = id
        msg.costGold = cost
        self:send(protocol_pb.ENTERMAP, msg)        
    end,

    --[[
    -- 保存关卡战争迷雾
    saveLevelFog = function(self, map_id, data)
        local msg = protocol_pb.CSPostFogInfo() 
        msg.copyId = map_id
        for k,v in ipairs(data) do
           msg.fogOfWar:append(v)
        end
        self:send(protocol_pb.POSTFOGINFO, msg)     
    end,

    -- 保存关卡玩家位置
    saveLevelPlayer = function(self, map_id, pos)
        --log.info('### saveLevelPlayer, map_id'..map_id..', pos:'..tostring(pos))
        local msg = protocol_pb.CSPosChange() 
        msg.copyId = map_id
        msg.posX = pos.x * 10000
        msg.posY = pos.z * 10000
        self:send(protocol_pb.POSTPOSINFO, msg)
    end,]]

    -- 重置关卡
    resetMap = function(self, map_id)
        local msg = protocol_pb.CSResetMapInfo() 
        msg.copyId = map_id
        self:send(protocol_pb.RESETMAPINFO, msg)        
    end,

    -- ATTACKMONSTER = 51; //攻击怪物 CSAttackMonster SCAttackMonster
    attackMonster = function (self, mapId, monsterId, is_sneak)
        local msg = protocol_pb.CSAttackMonster()
        msg.mapId = mapId
        msg.monsterId = monsterId
        if is_sneak then
            --log.info("### is sneak!!")
            msg.isSneakRaid = 1
        else
           -- log.info("### not sneak!!")
        end

        --[[
        有了布阵，这个部分可以不发
        for k, v in ipairs(heroIDs) do 
            msg.inHeroIDs:append(v)
        end
        for k, v in ipairs(soldierIDs) do 
            msg.inPetIds:append(v)
        end]]

        self:send(protocol_pb.ATTACKMONSTER, msg)
    end, 

    -- 结束关卡
    finishLevel = function(self, is_success)
        local msg = protocol_pb.CSOverCopy()
        if is_success then
            msg.isSuccceed = 1
        else
            msg.isSuccceed = 0
        end
        
        self:send(protocol_pb.OVERCOPY, msg)
    end,

    -- 关卡中添加剧情英雄
    addStoryLevelHero = function(self, hero_id)
        local msg = protocol_pb.CSAddStoryLevelHero()
        msg.heroId = hero_id
        self:send(protocol_pb.ADDSTORYLEVELHERO, msg)
    end,

    -- 关卡中移除剧情英雄
    removeStoryLevelHero = function(self, hero_id)
        local msg = protocol_pb.CSDelStoryLevelHero()
        msg.heroId = hero_id
        self:send(protocol_pb.DELSTORYLEVELHERO, msg)
    end,

    -- 离开关卡
    leaveMap = function(self, map_id)
        local msg = protocol_pb.CSLeaveMap()
        msg.mapId = map_id
        log.info('###send leave map:'..tostring(map_id))
        self:send(protocol_pb.LEAVEMAP, msg)
    end,

    -- 请求领取关卡任务奖励
    getCopyTaskReward = function(self, copy_id, task_id)
        local msg = protocol_pb.CSGetCopyTaskReward()
        msg.copyId = copy_id
        msg.taskId = task_id
        self:send(protocol_pb.GETCOPYTASKREWARD, msg)
    end,

    getCopyGiftBag = function(self, copy_id, gift_id)
        local msg = protocol_pb.CSGetCopyGiftBag()
        msg.copyId = copy_id
        msg.giftId = gift_id
        self:send(protocol_pb.GETCOPYGIFTBAG, msg)
    end,

    chat = function (self, text)
        if string.sub(text, 1, 1) == "@" then 
            return self:gm(string.sub(text, 2))
        end
        print("unimplemented chat service")
    end, 

    gm = function (self, cmd)
        local msg = protocol_pb.CSChatReq()
        -- print("send chat", (string.sub(cmd, 1, 1) == "@") and string.sub(cmd, 2) or cmd)
        msg.chatChannel = protocol_pb.GM
        msg.playerId = ""
        msg.context = (string.sub(cmd, 1, 1) == "@") and string.sub(cmd, 2) or cmd
        print("send chat", msg.context)
        self:send(protocol_pb.CHAT, msg)
    end, 

    -- 通用关卡交互，包括采矿，随机事件领奖励
    levelInteraction = function (self, mapId, entity_id)
        log.info("### send level interaction, mapId:"..mapId..', entity id:'..entity_id)
        
        local msg = protocol_pb.CSCommonLevelInteraction()
        msg.mapId = mapId
        msg.entityId = entity_id

        self:send(protocol_pb.COMMONLEVELINTERATION, msg)
    end, 

    -- KILLMONSTER = 52; //攻击怪物结果 CSKillMonster SCKillMonster
    killMonster = function (self, fightUId, result, herosHp)
        -- print(fightUId, result, #herosHp)
        local msg = protocol_pb.CSKillMonster()
        msg.fightUId = fightUId
        msg.result = result
        level_info.hero_killed = fales
        for __, info in ipairs(herosHp) do
            if info.camp == 0 then
                log.info("### hero id:"..info.hero_id..", hp:"..info.curHp)

                local hpObj = msg.herosHp:add()
                hpObj.heroId = info.hero_id
                hpObj.heroHp = info.curHp
                if hpObj.heroHp <= 0 then
                    level_info.hero_killed = true
                end
            end
        end

        self:send(protocol_pb.KILLMONSTER, msg)
    end, 

    -- 请求改变英雄属性
    levelChangeHeroProp = function(self, hpChange, atkChange)
        local msg = protocol_pb.CSLevelChangeHeroProp()
        msg.hpChange = hpChange * 10000
        msg.atkChange = atkChange * 10000

        self:send(protocol_pb.LEVELCHANGEHEROPROP, msg)
    end,

    -- 请求英雄血量百分比
    reqLevelHeroHpPercent = function(self)
        self:levelChangeHeroProp(0,0)
    end,

    --[[
    -- RESEACCHTASKINFO=61;//发送研发任务信息 SCNotifyResearchTaskInfo  CSGetTaskInfo
    -- mapId 如果不指定，就自动获取当前所在关卡的 mapId
    getTaskInfo = function (self, mapId)
        local msg = protocol_pb.CSGetTaskInfo()
        msg.copyId = mapId or level_info.map_info.mapId
        self:send(protocol_pb.RESEACCHTASKINFO, msg)
    end, 

    -- GETTASKREWARD =62;  //获取任务奖励  CSGetTaskAward
    getTaskReward = function (self, rewardType, phase, taskId, taskIndex)
        -- print("send msg getTaskReward")
        local msg = protocol_pb.CSGetTaskAward()
        msg.rewardType = rewardType -- 奖励类型
	    msg.phase = phase           -- 第几阶段
	    msg.taskId = taskId         -- 任务ID，
	    msg.taskIndex = taskIndex   -- 研发任务索引，因为可能有相同ID任务	
        self:send(protocol_pb.GETTASKREWARD, msg)
    end, 

    -- BUYBOOST     =59;  //购买boost  CSBuyBoost
    buyBoost = function (self, boostLevel, boostId)
        local msg = protocol_pb.CSBuyBoost()
        msg.boost.boostLevel = boostLevel
        msg.boost.boostId = boostId
        self:send(protocol_pb.BUYBOOST, msg)
    end, 

    -- 获得boost的属性 add by zhanghao
    getBoostPropInfo = function(self, copyId)
        local msg = protocol_pb.CSBoostPropInfo()
        msg.copyId = copyId
        self:send(protocol_pb.BOOSTPROPINFO, msg)
    end,]]

    -- 查看怪物信息  CSGetMonsterInfo SCGetMonsterInfo
    getMonsterInfo = function(self, entityIndex)
        -- log.error("sender@",entityIndex)
        local msg = protocol_pb.CSGetMonsterInfo()
        msg.entityIndex = entityIndex
        self:send(protocol_pb.CHECKMONSTERINFO, msg)
    end,

    -- 章节任务信息
    getChapterTask = function(self, chapter_id)
        local msg = protocol_pb.CSGetChapterTaskInfo()
        msg.chapterId = chapter_id
        self:send(protocol_pb.CHAPTERTASKINFO, msg)
    end,

    -- 获取章节任务奖励
    getChapterTaskReward = function(self, task_id)
        local msg = protocol_pb.CSGetChapterTaskReward()
        msg.taskId = task_id
        self:send(protocol_pb.GETCHAPTERREWARD, msg)
    end,

    -- 请求进入世界地图
    enterWorldMap = function(self, map_type, x, y)
        map_type = map_type or protocol_pb.MAP_WORLDMAP
        local msg = protocol_pb.CSEnterWorldMap()
        msg.mapType = map_type

        if x then
            msg.posX = x
        end

        if y then
            msg.posY = y
        end

        self:send(protocol_pb.ENTERWORLDMAP, msg)
    end,

    -- 请求离开世界地图
    leaveWorldMap = function(self)
        map_type = map_type or protocol_pb.MAP_WORLDMAP
        local msg = protocol_pb.CSLeaveWorldMap()
        msg.mapType = map_type
        self:send(protocol_pb.LEAVEWORLDMAP, msg)
    end,

    -- 世界地图移动
    moveWorldMap = function(self, x, y)
        local msg = protocol_pb.CSWMMoveTo()
        msg.posX = x
        msg.posY = y
        self:send(protocol_pb.WORLDMAPMOVETO, msg)
    end,

    -- 开始行军 目标类型 目标实例id 出征英雄列表
    startMarchLine = function(self, map_type, target_type, march_param, team_ids, formations)
        map_type = map_type or protocol_pb.MAP_WORLDMAP
        local msg = protocol_pb.CSStartMarchLine()
        msg.marchAction = target_type  -- EMarchAction
        msg.maptype = map_type
        
        if target_type == common_pb.EMARCH_BUILD then
            local param = msg.buildParam
            param.srcInstId = march_param.srcInstId
            param.posX = march_param.posX
            param.posY = march_param.posY
            param.cfgId = march_param.cfgId
            param.buildName = march_param.buildName
            param.level = march_param.level

        elseif target_type == common_pb.EMARCH_GATHER then
            local param = msg.gatherParam
            param.mineInstId = march_param.mineInstId
            param.carId = march_param.carId

        elseif target_type == common_pb.EMARCH_LOCAL then
            local param = msg.localParm
            param.srcInstId = march_param.srcInstId
            param.dstInstId = march_param.dstInstId

        elseif target_type == common_pb.EMARCH_ATK then
            local param = msg.atkParam
            param.srcInstId = march_param.srcInstId
            param.dstInstId = march_param.dstInstId

        elseif target_type == common_pb.EMARCH_HELP then
            local param = msg.helpParam
            param.srcInstId = march_param.srcInstId
            param.dstInstId = march_param.dstInstId

        elseif target_type == common_pb.EMARCH_PRE_TEAM then
            local param = msg.preTeam
            param.srcInstId = march_param.srcInstId
            param.dstInstId = march_param.dstInstId

            local item = param.rewardItem
            item.ItemTemId = march_param.rewardItem.ItemTemId
            item.Count = march_param.rewardItem.Count

            for k, v in ipairs(march_param.heroId) do
                param.heroId:append(v)
            end
        elseif target_type == common_pb.EMARCH_CREATE_MASS then
            local param = msg.createMass
            param.srcInstId = march_param.srcInstId
            param.dstInstId = march_param.dstInstId
            param.sets.memStart = world_map_info.mass_sets.memStart
            param.sets.pwd = world_map_info.mass_sets.pwd
            param.sets.maxAuto = world_map_info.mass_sets.maxAuto
            param.sets.shared = world_map_info.mass_sets.shared

        elseif target_type == common_pb.EMARCH_JOIN_MASS then
            local param = msg.joinMass
            param.srcInstId = march_param.srcInstId
            param.massInstId = march_param.massInstId
            
        elseif target_type == common_pb.EMARCH_MASS_ATK then
            local param = msg.massAtk
            param.massInstId = march_param.massInstId

        elseif target_type == common_pb.EMARCH_GATHER_TITAN then
            local param = msg.gatherTitanParam
            param.srcInstId = march_param.srcInstId
            param.titanInstId = march_param.titanInstId
            param.isOpenProtect = march_param.isOpenProtect

        elseif target_type == common_pb.EMARCH_ROB_TITAN then
            local param = msg.atkParam
            param.srcInstId = march_param.srcInstId
            param.dstInstId = march_param.dstInstId
        end

        if team_ids then
            for k, v in ipairs(team_ids) do
                msg.teamIds:append(v)
            end
        end

        if formations then
            for k, v in ipairs(formations) do
                local formation = msg.formations:add()
                formation.formationId = v.formationId
                for k1, v1 in ipairs(v.formationHero) do
                    if v1.location > 0 then
                        local hero = formation.formationHero:add()
                        hero.heroId = v1.heroId
                        hero.location = v1.location
                    end
                end
            end
        end
        self:send(protocol_pb.WORLDMAPSTARTMARCH, msg)
    end,

    --[[ 
    enum ITEM_UPDATE
    {
        IU_ADD    = 0;  //增加道具
        IU_DELETE = 1;  //删除道具
        IU_UPDATE = 2;  //更新道具
        IU_EQUIP  = 3;  //装备装备
        IU_UNEQUIP= 4;  //卸下装备
    }]]
    -- 英雄装备
    heroEquip = function(self, hero_id, item_inst_id, slot_id, item_update_type)
        local msg = protocol_pb.CSHeroEquip()
        msg.HeroCfgId = hero_id
        msg.instId = item_inst_id
        msg.slotId = slot_id
        msg.handleType = item_update_type

        self:send(protocol_pb.HEROEQUIP, msg)
    end,

    -- 升级道具，目前是主副武器
    upgradeItem = function(self, hero_id, inst_id, consume_items, exp_items)
        local msg = protocol_pb.CSUpGradeItem()
        if hero_id then msg.HeroCfgId = hero_id end
        msg.instId = inst_id
        for k,v in ipairs(consume_items) do
            msg.consumeItems:append(v)
        end
        for k, v in pairs(exp_items) do
            local item = msg.expItems:add()
            item.uId = v.uId
            item.itemId = v.itemId
            item.getNum = v.getNum
        end
        self:send(protocol_pb.UPGRADEITEM, msg)
    end,

    -- 查询地图单位信息
    queryWorldMap = function(self, objId, map_type)
        -- WORLD_OBJ_PLAYER = 1;
        -- WORLD_OBJ_MINE = 2;
        -- WORLD_OBJ_MARCH = 3;
        -- WORLD_OBJ_MONSTER = 4;
        print("queryWorldMap:", objId, map_type)
        map_type = map_type or protocol_pb.MAP_WORLDMAP
        local msg = protocol_pb.CSWorldMapQuery()
        msg.objId = objId
        msg.mapType = map_type
        self:send(protocol_pb.WORLDMAPQUERYOBJ, msg)
    end,

    -- 查询地图建筑信息
    queryWorldObj = function(self, objId)
        local msg = protocol_pb.CSWMObjQuery()
        msg.objId = objId
        self:send(protocol_pb.WORLDMAPOBJQUERY, msg)
    end,
    
    -- 指挥官晋升
    officerPromote = function(self, officer_id)
        local msg = protocol_pb.CSOfficerPromote()
        msg.id = officer_id
        self:send(protocol_pb.OFFICERPROMOTE, msg)
    end,

    -- 指挥官升星
    officerStarUp = function(self, officer_id)
        local msg = protocol_pb.CSOfficerStarUp()
        msg.id = officer_id
        self:send(protocol_pb.OFFICERSTARUP, msg)
    end,

    -- 请求添加收藏
    worldMapCollectionAdd = function(self, eTab, desc, iconId, x, y, server_id)
        local msg = protocol_pb.CSCollectionAdd()
        msg.collectionPos.serverid = server_id
        msg.collectionPos.id = 0
        msg.collectionPos.eTable = eTab
        msg.collectionPos.strDesc = desc
        msg.collectionPos.iconId = iconId
        msg.collectionPos.posX = x
        msg.collectionPos.posY = y
        msg.collectionPos.marchDist = 0
        msg.collectionPos.marchTime = 0
        msg.collectionPos.objType = 0
        self:send(protocol_pb.WORLDMAPCOLLECTIONADD, msg)
    end,

    -- 请求修改收藏
    worldMapCollectionMod = function(self, id, eTab, desc)
        local msg = protocol_pb.CSCollectionMod()
        msg.collectionPos.id = id
        msg.collectionPos.eTable = eTab
        msg.collectionPos.strDesc = desc
        msg.collectionPos.iconId = 0
        msg.collectionPos.posX = 0
        msg.collectionPos.posY = 0
        msg.collectionPos.marchDist = 0
        msg.collectionPos.marchTime = 0
        msg.collectionPos.objType = 0
        self:send(protocol_pb.WORLDMAPCOLLECTIONMOD, msg)
    end,

    -- 请求删除收藏
    worldMapCollectionDel = function(self, id)
         local msg = protocol_pb.CSCollectionDel()
         msg.collectionId = id
         self:send(protocol_pb.WORLDMAPCOLLECTIONDEL, msg)
    end,

    -- 召回车队
    worldMapGoHome = function(self, type, team_id, item_id)
        local msg = protocol_pb.CSWorldMapGoHome()
        msg.opType = type
        msg.teamId = tostring(team_id)
        if item_id then
            msg.instId = item_id    --option
        end
        self:send(protocol_pb.WORLDMAPGOHOME, msg)
    end,

    -- 加速车队
    speedUp = function(self, marchLineId, instId)
        map_type = map_type or protocol_pb.MAP_WORLDMAP
        local msg = protocol_pb.CSSpeedUp()
        msg.marchLineId = marchLineId
        msg.itemId = instId
        self:send(protocol_pb.WORLDMAPSPEEDUP, msg)
    end,

    -- 个人基地消息 start
    enterPersonBase = function(self, playerId)
        local msg = protocol_pb.CSEnterPersonBase()
        msg.playerId = playerId
        self:send(protocol_pb.ENTERPERSONBASE, msg)
    end,

    --[[
    personBaseAdd = function(self, cfgId, posX, posY)
        -- print("发送消息 添加建筑:", cfgId, posX, posY)
        local msg = protocol_pb.CSPersonBaseAdd()
        msg.cfgId = cfgId
        msg.posX = posX
        msg.posY = posY
        self:send(protocol_pb.PERSONBASEADD, msg)
    end,
    ]]

    personBaseAdd = function(self, cfgId, gridid)
        print("添加建筑:", cfgId, gridid)
        local msg = protocol_pb.CSPersonBaseAdd()
        msg.cfgId = cfgId
        msg.gridid = gridid
        self:send(protocol_pb.PERSONBASEADD, msg)
    end,

    personBaseMove = function(self, instId, eObjType, posX, posY)
        print("发送消息 移动坐标:", instId, eObjType, posX, posY)
        local msg = protocol_pb.CSPersonBaseMove()
        msg.instId = instId
        msg.eObjType = eObjType
        msg.posX = posX
        msg.posY = posY
        self:send(protocol_pb.PERSONBASEMOVE, msg)
    end,

    personBaseDel = function(self, instId)
        print("发送消息 删除建筑:", instId, eObjType)
        local msg = protocol_pb.CSPersonBaseDel()
        msg.instId = instId
        self:send(protocol_pb.PERSONBASEDEL, msg)
    end,

    personBaseQuery = function(self, instId)
        local msg = protocol_pb.CSPersonBaseQuery()
        msg.instId = instId
        self:send(protocol_pb.PERSONBASEQUERY, msg)
    end,

    personBaseLevelUp = function(self, instId, isPromFinish)
        print("发送升级请求:", instId, "isPromFinish:",isPromFinish)
        local msg = protocol_pb.CSPersonBaseLevelUp()
        msg.instId = instId
        msg.isPromFinish = isPromFinish
        self:send(protocol_pb.PERSONBASELEVELUP, msg)
    end,

    buildingSpeedUp = function(self, instId, ItemTemId, Count, isPromFinish)
        local msg = protocol_pb.CSBuildingSpeedUp()
        msg.instId = instId
        msg.quickenItem.ItemTemId = ItemTemId
        msg.quickenItem.Count = Count
        msg.isPromFinish = isPromFinish
        self:send(protocol_pb.PERSONBASEBUILDINGSPEEDUP, msg)
    end,

    technologyUp = function(self, cfgId, isPromFinish)
        local msg = protocol_pb.CSTechnologyUp()
        msg.cfgId = cfgId
        msg.isPromFinish = isPromFinish
        self:send(protocol_pb.PERSONBASETECHNOLOGYUP, msg)
    end,

    technologySpeedUp = function(self, cfgId, ItemTemId, Count, isPromFinish)
        local msg = protocol_pb.CSTechnologySpeedUp()
        msg.cfgId = cfgId
        msg.quickenItem.ItemTemId = ItemTemId
        msg.quickenItem.Count = Count
        msg.isPromFinish = isPromFinish
        self:send(protocol_pb.TECHNOLYGYSPEEDUP, msg)
    end,

    clickBubblie = function(self, bubbleType, techId)
        local msg = protocol_pb.CSCLICKBUBBLIE()
        msg.bubbleType = bubbleType
        msg.techId = techId
        self:send(protocol_pb.CLICKBUBBLE, msg)
    end,

    -- 获取监狱俘虏信息
    getCellInfo = function(self)
        print("发送服务器请求: 获取监狱俘虏信息")
        local msg = protocol_pb.CSGetCellInfo()
        self:send(protocol_pb.GETCELLINFO, msg)
    end,

    -- 流放俘虏请求
    exilePrisoner = function(self, instIds)
        local msg = protocol_pb.CSExilePrisoner()
        for __, id in ipairs(instIds) do
            msg.instIds:append(id)
        end
        self:send(protocol_pb.EXILEPRISONER, msg)
    end,

    -- 招降俘虏请求
    summonPrisoner = function(self, instIds)
        local msg = protocol_pb.CSSummonPrisoner()
        for __, id in ipairs(instIds) do
            msg.instIds:append(id)
        end
        self:send(protocol_pb.SUMMONPRISONER, msg)
    end,

    -- 释放俘虏请求
    releasePrisoner = function(self, instIds)
        local msg = protocol_pb.CSReleasePrisoner()
        for __, id in ipairs(instIds) do
            msg.instIds:append(id)
        end
        self:send(protocol_pb.RELEASEPRISONER, msg)
    end,

    -- 开始招募战斗
    startRecuritFight = function(self, instId, fmt_cfg_id, hero_list)
        local msg = protocol_pb.CSStartRecuritFight()
        msg.instId = instId
        msg.playerForm.formationId = fmt_cfg_id
        for __, hero in ipairs(hero_list) do
            if hero.location > 0 then
                local item = msg.playerForm.formationHero:add()
                item.heroId = hero.heroId
                item.location = hero.location
                print("开始招募战斗", __, item.heroId, item.location)
            end
        end
        self:send(protocol_pb.STARTRECRUITFIGHT, msg)
    end,

    -- 结束招募战斗
    endRecuritFight = function(self, result, instId)
        local msg = protocol_pb.CSEndRecuritFight()
        msg.result = result
        msg.instId = instId
        print(result, instId)
        self:send(protocol_pb.ENDRECRUITFIGHT, msg)
    end,

    pbStartProject = function(self, itemId)
        local msg = protocol_pb.CSPBStartProject()
        msg.itemId = itemId
        self:send(protocol_pb.PERSONBASESTARTPROJECT, msg)
    end,

    pbAddPartner = function(self, posIndex, partnerId)
        local msg = protocol_pb.CSPBAddPartner()
        msg.posIndex = posIndex
        msg.partnerId = partnerId
        self:send(protocol_pb.PERSONBASEADDPARTNER, msg)
    end,

    pbRemovePartner = function(self, posIndex)
        local msg = protocol_pb.CSPBRemovePartner()
        msg.posIndex = posIndex
        self:send(protocol_pb.PERSONBASEREMOVEPARTNER, msg)
    end,

    pbBuildClearTime = function(self, instId)
        local msg = protocol_pb.CSPBBuildClearTime()
        msg.instId = instId
        self:send(protocol_pb.PERSONBASEBUILDCLEARTIME, msg)
    end,

    -- 个人基地消息 end

    pbAddOffice = function(self, officeId)
        local msg = protocol_pb.CSPBAddOffice()
        msg.officeId = officeId
        self:send(protocol_pb.PERSONBASEADDOFFICE, msg)
    end,

    pbRemoveOffice = function(self, officeId)
        local msg = protocol_pb.CSPBRemoveOffice()
        msg.officeId = officeId
        self:send(protocol_pb.PERSONBASEREMOVEOFFICE, msg)
    end,

    pbDispatchOffice = function(self, officeId, playerId)
        local msg = protocol_pb.CSPBDispatchOffice()
        msg.officeId = officeId
        msg.playerId = playerId
        self:send(protocol_pb.PERSONBASEDISPATCHOFFICE, msg)
    end,

    -- 船员 - 装备
    crewEquip = function(self, inst_id, slot_id)
        log.info('### send crewEquip inst_id:'..inst_id..'slot_id:'..slot_id)
        local msg = protocol_pb.CSHandleProfession()
        msg.instId = inst_id
        msg.slotId = slot_id
        msg.handleType = protocol_pb.IU_EQUIP
        self:send(protocol_pb.HANDLEPROFESSION, msg)
    end,

    -- 船员 - 卸下
    crewUnequip = function(self, inst_id, slot_id)
        log.info('### send crewUnequip inst_id:'..inst_id..'slot_id:'..slot_id)
         local msg = protocol_pb.CSHandleProfession()
         msg.instId = inst_id
         msg.slotId = slot_id
         msg.handleType = protocol_pb.IU_UNEQUIP
         self:send(protocol_pb.HANDLEPROFESSION, msg)
    end,

    --出售道具
    sellItem = function(self, inst_id,  count)
        local msg = protocol_pb.CSSaleItem()
        msg.ItemId = inst_id
        msg.Count = count
        self:send(protocol_pb.SALEITEM, msg)
    end,

    --使用道具
    useItem = function(self, cfg_id, count)
        print("使用道具:", cfg_id, count)
        local msg = protocol_pb.CSUseItem()
        msg.itemId = cfg_id
        msg.itemCount = count
        self:send(protocol_pb.USEITEM, msg)
    end,
    
    --指挥官经验道具
    useExpItem = function(self, cfg_id, count, hero_id)
        local msg = protocol_pb.CSAddHeroExp()
        msg.useItemId = cfg_id
        msg.itemCount = count
        msg.heroId = hero_id
        self:send(protocol_pb.ADDHEROEXP, msg)
    end,

    -- 购买道具
    buyItem = function(self, data, isUse)
        local msg = protocol_pb.CSBuyItem()
        msg.isUse = isUse
        for __, value in ipairs(data) do
            local item = msg.buyItem:add()
            item.ItemTemId = value.item_id
            item.Count = value.count
        end
        self:send(protocol_pb.BUYITEM, msg)
    end,

    --[[enum Task_RewardType
    {
        Norm_Task  = 1;   //普通任务奖励
        ACTIVE_POINT = 2; //活跃点奖励
        Rese_Task  = 3;   //研发任务奖励
        Rese_Phase = 4;   //研发任务阶段奖励
    }]]
    -- 领取任务奖励
    getNorTaskRewards = function(self, task_id, reward_Type)
        local msg = protocol_pb.CSGetTaskReward()
        msg.taskId = task_id
        msg.awardType = reward_Type
        self:send(protocol_pb.GETTASKREWARD, msg)
    end,  

    -- 装备锁定
    lockedEquipments = function(self, equipments)
        local msg = protocol_pb.CSLockEquip()
        for k,v in pairs(equipments) do
           msg.lockEquips:append(v.inst_id)
        end
        self:send(protocol_pb.LOCKEQUIP, msg)
    end,  

    -- 装备分解
    resolveEquipments = function(self, equipments)
        local msg = protocol_pb.CSResolveEquip()
        for k,v in pairs(equipments) do
           msg.resolveEquips:append(v.inst_id)
        end
        self:send(protocol_pb.RESOLVEEQUIP, msg)
    end, 

    -------------------- 关卡出战，布阵-----------------
    -- 选择章节的出战英雄
    selectChapterAttendHeros = function(self, chapter_id, level_id, heroes)
        local msg = protocol_pb.CSSelectChapterAttendHeroes()
        for k,hero_id in ipairs(heroes) do
            --log.info("$$$ hero_id:"..hero_id)
            msg.heroes:append(hero_id)
        end
        msg.chapterId = chapter_id
        msg.copyId = level_id
        self:send(protocol_pb.SELECTCHAPTERATTENDHEROES, msg)
    end,

    -- 请求章节出战英雄
    reqChapterAttendHeros = function(self, chapter_id, level_id, is_in_copy)
        --log.info("### reqChapterAttendHeros:"..chapter_id..", "..tostring(level_id))

        local msg = protocol_pb.CSReqChapterAttendHeroes()
        msg.chapterId = chapter_id
        if level_id then
            msg.copyId = level_id
        end
        if is_in_copy then
            msg.isInCopy = 1
        end
        self:send(protocol_pb.CHAPTERATTENDINFO, msg)
    end,

    -- 关卡战斗中布阵, 现在不用
    --[[
    levelEmbattle = function(self, chapter_id, heros, officer_skill_id)
        local msg = protocol_pb.CSLevelEmbattle()
        msg.chapterId = chapter_id
        msg.skillId = officer_skill_id or 0

        for location, hero_id in ipairs(heros) do
            if hero_id then
                local formation = msg.formations:add()
                formation.location = location
                formation.heroId = hero_id
                formation.isDeath = 0
            end
        end

        self:send(protocol_pb.LEVELEMBATTLE, msg)
    end,]]

    -- 副本布阵
    copyEmbattle = function(self, formationId, hero_list)
        log.info('### copyEmbatle:'..formationId..", num:"..#hero_list)

        local msg = protocol_pb.CSCopyEmbattle()
        msg.formation.formationId = formationId
        local count = 0
        for __, hero in ipairs(hero_list) do
            if hero.location > 0 then
                local item = msg.formation.formationHero:add()
                item.heroId = hero.heroId
                item.location = hero.location
                count = count + 1
            end
        end
       --level_info.is_req_embattle = true
        self:send(protocol_pb.COPYEMBATTLE, msg)
        if count < 8 then
            hint:setText(NativeText.GetLangStr(170109))
        end
    end,

    -- 添加预设编队
    addPresetFormation = function(self, MFormation)
        print("添加预设编队 sender", MFormation.formationId)
        -- formationId
        -- formationHero
        local msg = protocol_pb.CSAddPresetFormation()
        msg.formation.formationId = MFormation.formationId
        for __, hero in ipairs(MFormation.formationHero) do
            if hero.location > 0 then
                local item = msg.formation.formationHero:add()
                item.heroId = hero.heroId
                item.location = hero.location
            end
        end
        self:send(protocol_pb.ADDPRESETFORMATION, msg)
    end,

    -- 删除预设编队
    delPressetFormation = function(self, presetId)
        local msg = protocol_pb.CSDelPressetFormation()
        msg.presetId = presetId
        self:send(protocol_pb.DELPRESETFORMATION, msg)
    end,

    -- 获取预设编队
    getPresetFormation = function(self)
        local msg = protocol_pb.CSGetPresetFormation()
        self:send(protocol_pb.GETPRESETFORMATION, msg)
    end,

    -- 获取队伍战力
    queryTeamSword = function(self, id, self_MFormation, enemy_formation_id)
        local msg = protocol_pb.CSQueryTeamSword()
        msg.queryId = id
        msg.formation.formationId = self_MFormation.formationId
        for __, hero in ipairs(self_MFormation.formationHero) do
            if hero.location > 0 then
                local item = msg.formation.formationHero:add()
                item.heroId = hero.heroId
                item.location = hero.location
            end
        end
        msg.enemyFormation = enemy_formation_id
        self:send(protocol_pb.GETTEAMSWORD, msg)
    end,

    -- 战斗开始
    fightStart = function(self, uid)
        local msg = protocol_pb.CSFightStart()
        msg.fightUId = uid
        self:send(protocol_pb.FIGHTSTART, msg)
    end,
--------------------------------------------------------------
----------------EMP---------------
    -- emp 模块升级， 0-主模块  1,2,3 - 子模块
    empModUpgrade = function(self, mod_id)
        local msg = protocol_pb.CSEMPModuleUpgrade()
        msg.id = mod_id

        self:send(protocol_pb.EMPMODUPGRADE, msg)
    end,

    -- 使用 emp
    useEmp = function(self, pos, targets)
        local msg = protocol_pb.CSUseEMP()
        msg.posx = pos.x * 10000    -- float to int
        msg.posy = pos.z * 10000
        for _,entity_id in ipairs(targets) do
            msg.entityId:append(entity_id)
        end

        self:send(protocol_pb.USEEMP, msg)
    end,    
--------------------------------------------------------------
----------------- 商店 ---------------------
    -- 请求商店列表
    reqShopItemList = function(self, shop_type)
        local msg = protocol_pb.CSShopItemList()
        msg.shopType = shop_type
        
        self:send(protocol_pb.SHOPITEMLIST, msg)
    end,

    -- 购买商品
    buyGoods = function(self, shop_type, index)
        local msg = protocol_pb.CSBuyGoods()
        msg.shopType = shop_type
        msg.index = index -- 商品在商店中的索引

        self:send(protocol_pb.BUY_GOODS, msg)
    end,

    -- 刷新商品
    gameShopRefresh = function(self, shop_type)
        local msg = protocol_pb.CSGameShopRefresh()
        msg.eShopType = shop_type
        self:send(protocol_pb.GAMESHOPREFRESH, msg)
    end,

--------------------------------------------

--------------------------------------------------------------
----------------- 工会 ---------------------
    -- 创建工会
    createFamily = function(self, familyName, familyAcronym, familyLanguage, emble_bg, emble_color, emble_icon, innotice, outnotice)
        print("发送 创建工会 消息", familyName, familyAcronym, familyLanguage, emble_bg, emble_color, emble_icon)
        local msg = protocol_pb.CSCreateFamily()
        msg.familyName = familyName
        msg.familyAcronym = familyAcronym
        msg.familyLanguage = familyLanguage
        msg.familyEmble.grounding = emble_color
        msg.familyEmble.iconFrame = emble_bg
        msg.familyEmble.icon = emble_icon
        msg.innotice = innotice
        msg.outnotice = outnotice
        self:send(protocol_pb.CREATEFAMILY, msg)
    end,

    -- 申请或取消加入工会
    familyJoinOrCancel = function(self, enter_or_cancel, familyid)
        local msg = protocol_pb.CSFamilyJoinOrCancel()
        msg.action = enter_or_cancel
        msg.familyid = familyid
        self:send(protocol_pb.FAMILYJOINORCANCEL, msg)
    end,

    -- 获取工会列表
    getFamilyList = function(self, page)
        local msg = protocol_pb.CSGetFamilyList()
        msg.page = page
        self:send(protocol_pb.GETFAMILYLIST, msg)
    end,

    -- 获取工会玩家申请列表
    getFamilyApplyList = function(self)
        local msg = protocol_pb.CSGetFamilyApplyList()
        self:send(protocol_pb.GETAPPLYLIST, msg)
    end,
    
    -- 获取工会成员列表
    getFamilyMemberList = function(self, family_id)
        local msg = protocol_pb.CSGetFamilyMemberList()
        msg.familyId = family_id
        self:send(protocol_pb.GETMEMBERLIST, msg)
    end,
    
    -- 获取工会信息
    getFamilyInfo = function(self, familyId)
        print("发送获取工会详情")
        local msg = protocol_pb.CSGetFamilyInfo()
        msg.familyId = familyId
        self:send(protocol_pb.GETFAMILYINFO, msg)
    end,

    -- 邀请加入工会
    familyInvite = function(self, player_id)
        local msg = protocol_pb.CSInvitePlayerToFamily()
        msg.playerId = player_id
        self:send(protocol_pb.INVITEDTOFAMILY, msg)
    end,

    -- 回应工会邀请
    disposeInvite = function(self, agree, family_id)
        local msg = protocol_pb.CSDisposeFamilyInviting()
        msg.isAgree = agree  -- //0拒绝，1同意，2忽略
        msg.familyId = family_id
        self:send(protocol_pb.DISPOSEFAMILYINVITE, msg)
    end,

    -- 获取邀请我的工会列表
    getInvitedFamilyList = function(self)
        local msg = protocol_pb.CSGetInvitedFamilyList()
        self:send(protocol_pb.GETINVITEDFAMILYLIST, msg)
    end,

    -- 工会成员升阶降阶
    familyMemberPromote = function(self, player_id, status)
        local msg = protocol_pb.CSFamilyMemberPromote()
        msg.playerId = player_id
        msg.status = status --//0表示降职、1表示升职
        self:send(protocol_pb.FAMILYMEMBERPROMOTE, msg)
    end,

    -- 开除工会成员
    expelFamilyMember = function(self, player_id)
        local msg = protocol_pb.CSExpelFamilyMember()
        msg.playerId = player_id
        self:send(protocol_pb.EXPELFAMILYMEMBER, msg)
    end,

    -- 获取工会日志
    getFamilyLog = function(self, familyId)
        local msg = protocol_pb.CSGetFamilyLog()
        msg.familyId = familyId
        self:send(protocol_pb.GETFAMILYOG, msg)
    end,

    -- 工会搬迁
    movePersonalBase = function(self)
        local msg = protocol_pb.CSMovePersonalBase()
        self:send(protocol_pb.MOVEPERSONALBASE, msg)
    end,

    -- 工会留言
    familyLeaveWord = function(self, familyId, content)
        local msg = protocol_pb.CSFamilyLeaveWord()
        msg.familyId = familyId
        msg.content = content
        self:send(protocol_pb.FAMILYLEAVEWORD, msg)
    end,

    -- 获取工会留言
    getFamilyLeaveWord = function(self, familyId)
        local msg = protocol_pb.CSGetFamilyLeaveWord()
        msg.familyId = familyId
        self:send(protocol_pb.GETFAMILYLEAVEWORD, msg)
    end,

    -- 关闭留言板
    closeFamilyLeaveWord = function(self, id)
        local msg = protocol_pb.CSCloseFamilyLeaveWord()
        msg.familyId = id
        self:send(protocol_pb.CLOSELEAVEWORDBOARD, msg)
    end,

    makeDefaultFamilyInfo = function(self)
        local msg = protocol_pb.CSModifyFamilyInfo()
        msg.modifyType  	    = common_pb.MT_FAMILYNAME
        msg.familyName			= ""
        msg.familyAcronym		= ""
        msg.emblem.icon		    = 1
        msg.emblem.grounding    = 1
        msg.emblem.iconFrame    = 1
        msg.familyLang			= 1
        msg.inNotice			= ""
        msg.outNotice			= ""
        msg.joinLimit			= 1
        return msg
    end,

    -- 修改工会信息 名字
    modifyFamilyInfoName = function(self, familyName)
        local msg = self:makeDefaultFamilyInfo()
        msg.modifyType = common_pb.MT_FAMILYNAME
        msg.familyName = familyName
        self:send(protocol_pb.MODIFYFAMILYINFO, msg)
    end,

    -- 修改工会信息 缩写
    modifyFamilyInfoAcronym = function(self, familyAcronym)
        local msg = self:makeDefaultFamilyInfo()
        msg.modifyType = common_pb.MT_FAMILYACRONYM
        msg.familyAcronym = familyAcronym
        self:send(protocol_pb.MODIFYFAMILYINFO, msg)
    end,

    -- 修改工会信息 语言
    modifyFamilyInfoLanguage = function(self, familyLang)
        local msg = self:makeDefaultFamilyInfo()
        msg.modifyType = common_pb.MT_FAMILYLANG
        msg.familyLang = familyLang
        self:send(protocol_pb.MODIFYFAMILYINFO, msg)
    end,

    -- 修改工会信息 logo
    modifyFamilyInfoLogo = function(self, iconFrame, grounding, icon)
        local msg = self:makeDefaultFamilyInfo()
        msg.modifyType = common_pb.MT_FAMILYEMBLEM
        msg.emblem.icon		    = icon
        msg.emblem.grounding    = grounding
        msg.emblem.iconFrame    = iconFrame
        self:send(protocol_pb.MODIFYFAMILYINFO, msg)
    end,

    -- 修改工会信息 内公告
    modifyFamilyInfoInNotice = function(self, inNotice)
        local msg = self:makeDefaultFamilyInfo()
        msg.modifyType = common_pb.MT_INNOTICE
        msg.inNotice		    = inNotice
        self:send(protocol_pb.MODIFYFAMILYINFO, msg)
    end,

    -- 修改工会信息 外公告
    modifyFamilyInfoOutNotice = function(self, outNotice)
        local msg = self:makeDefaultFamilyInfo()
        msg.modifyType = common_pb.MT_OUTNOTICE
        msg.outNotice		    = outNotice
        self:send(protocol_pb.MODIFYFAMILYINFO, msg)
    end,
    
    -- 修改 招募设定
    modifyFamilyInfoEnlist = function(self, joinLimit)
        local msg = self:makeDefaultFamilyInfo()
        msg.modifyType = common_pb.MT_CONSCRIBE
        msg.joinLimit		    = joinLimit

        self:send(protocol_pb.MODIFYFAMILYINFO, msg)
    end,

    -- 退出工会
    quitFamily = function(self)
        local msg = protocol_pb.CSQuitFamily()
        self:send(protocol_pb.QUITFAMILY, msg)
    end,

	-- required uint32	isAgree			= 1;//0拒绝，1同意
	-- required string playerId		= 2;//0表示拒绝所有

    -- 处理加入公会申请列表
    disposeListFamily = function(self, isAgree, playerId)
        local msg = protocol_pb.CSDisposeFamilyApply()
        msg.isAgree     = isAgree
        msg.playerId    = playerId
        self:send(protocol_pb.DISPOSEFAMILYAPPLY, msg)
    end,

    -- 工会礼物信息
    getFamilyGiftInfo = function(self)
        local msg = protocol_pb.CSGetFamilyGiftInfo()
        self:send(protocol_pb.GETFAMILYGIFTINFO, msg)
    end,

    -- 领取工会礼物
    getFamilyGiftAward = function(self, gift_id)
        local msg = protocol_pb.CSGetFamilyGiftReward()
        msg.giftId = gift_id
        self:send(protocol_pb.GETFAMILYGIFTREWARD, msg)
    end,

    -- 工会礼物领取信息
    familyGiftGainInfo = function(self, gift_id)
        local msg = protocol_pb.CSGetFamilyGiftGainInfo()
        msg.giftId = gift_id
        self:send(protocol_pb.GETGIFTGAININFO, msg)
    end,

    -- 盟主转让
    transferFamilyLeader = function(self, player_id)
        local msg = protocol_pb.CSTransferFamilyLeader()
        msg.desPlayerId = player_id
        self:send(protocol_pb.TRANSFERLEADER, msg)
    end,

    -- 个人建筑请求帮助加速
    reqHelpFastBuild = function(self, build_id, type)
        local msg = protocol_pb.CSReqHelpFastBuild()
        msg.buildId = tostring(build_id)
        msg.helpType = type
        self:send(protocol_pb.REQHELPFASTBUILD, msg)
    end,

    -- 获取帮助加速列表
    getFamilyHelpList = function(self)
        local msg = protocol_pb.CSGetFamilyHelpList()
        self:send(protocol_pb.GETFAMILYHELPLIST, msg)
    end,

    -- 帮助加速
    helpFastBuild = function(self, inst_id, type)
        local msg = protocol_pb.CSHelpFastBuild()
        msg.instId = inst_id
        msg.helptype = type
        self:send(protocol_pb.HELPFASTBUILD, msg)
    end,

    -- 获取工会建筑
    getFamilyBuildList = function(self)
        local msg = protocol_pb.CSGetFamilyBuildList()
        self:send(protocol_pb.GETFAMILYBUILD, msg)
    end,

    -- 创建工会建筑
    buildFamilyBuilding = function(self, cfgId, posX, posY, type)
        local msg = protocol_pb.CSBuildFamilyBuilding()
        msg.cfgId = cfgId
        msg.posX = posX
        msg.posY = posY
        msg.type = type
        self:send(protocol_pb.BUILDFAMILYBUILDING, msg)
    end,

    -- 升级工会建筑
    upgradeFamilyBuilding = function(self, buildingId)
        local msg = protocol_pb.CSUpgradeFamilyBuilding()
        msg.buildingId = buildingId
        self:send(protocol_pb.UPGRADEFAMILYBUDILING, msg)
    end,

    -- 加速建造工会建筑
    familyFastBuild = function(self, buildingId)
        local msg = protocol_pb.CSFamilyFastBuild()
        msg.buildingId = buildingId
        self:send(protocol_pb.FAMILYBUILDFAST, msg)
    end,

--------------------------------------------

    -- 招募英雄
    recruitHero = function(self, heroId)
        local msg = protocol_pb.CSRecruitHero()
        msg.heroId = heroId
        self:send(protocol_pb.RECRUITHERO, msg)
    end,

    -- 邮件
    getMailListInfo = function(self)
        local msg = protocol_pb.CSGetMailListInfo()
        self:send(protocol_pb.MAILLISTINFO, msg)
    end,

    -- 读取邮件
    readMail = function(self, mail_id)
        local msg = protocol_pb.CSReadMail()
        msg.mailId = mail_id
        self:send(protocol_pb.READMAIL, msg)
    end,

    -- 提取邮件附件
    getMailItems = function(self, mail_ids)  
        local msg = protocol_pb.CSGetMailItems()
        for k,v in ipairs(mail_ids) do
           msg.mailId:append(v)
        end
        self:send(protocol_pb.GETMAILITEMS, msg)
    end,

    -- 删除邮件
    deleteMail = function(self, mail_ids)  
        local msg = protocol_pb.CSDelMail()
        for k,v in ipairs(mail_ids) do
            msg.mailId:append(v)
        end
        self:send(protocol_pb.DELMAIL, msg)
    end,

    -- 一键读取
    readMoreMails = function(self, list)  
        local msg = protocol_pb.CSReadMoreMails()
        for k, v in ipairs(list) do
            msg.mailIdList:append(v)
        end
        self:send(protocol_pb.ALLREASMAIL, msg)
    end,

    -- 发送工会邮件
    sendFamilyMail = function(self, title, content)
        local msg = protocol_pb.CSSendFamilyMail()
        msg.mailTitle = title
        msg.mailcontex = content
        self:send(protocol_pb.SENDFAMILYMAIL, msg)
    end,

    -- 聊天请求
    -- CSGetChatChannelList
    getChatChannelList = function(self)
        local msg = protocol_pb.CSGetChatChannelList()
        self:send(protocol_pb.GETCHATCHANNELLIST, msg)
    end,

    chatReq = function(self, chatChannel, playerId, context)
        local msg = protocol_pb.CSChatReq()
        msg.chatChannel = chatChannel
        msg.playerId = playerId or ""
        msg.context = context or ""
        self:send(protocol_pb.CHAT, msg)
    end,

    -- 获取商店列表
    getShopList = function(self)
        local msg = protocol_pb.CSGetShopList()
        self:send(protocol_pb.GETSHOPLIST, msg)
    end,

    -- 选择公共基地
    selectCity = function(self, id)
        local msg = protocol_pb.CSWMSelectCity()
        msg.cityId = id
        self:send(protocol_pb.WORLDMAPSELECITY, msg)
    end,

    -- 获取个人建筑信息
    getBuildCount = function(self)
        local msg = protocol_pb.CSWMGetBuildCount()
        self:send(protocol_pb.WORLDMAPGETBUILDCOUNT, msg)
    end,

    -- 查询行军时间
    queryMarchTime = function(self, start_x, start_y, end_x, end_y, type, team_ids, formations)
        local msg = protocol_pb.CSWMQueryTime()
        msg.startPosX = start_x
        msg.startPosY = start_y
        msg.endPosX = end_x
        msg.endPosY = end_y
        msg.queryType = type

        if team_ids then
            for k, v in ipairs(team_ids) do
                msg.teamIds:append(v)
            end
        end

        if formations then
            for k, v in ipairs(formations) do
                local formation = msg.formations:add()
                formation.formationId = v.formationId
                for k1, v1 in ipairs(v.formationHero) do
                    if v1.location > 0 then
                        local hero = formation.formationHero:add()
                        hero.heroId = v1.heroId
                        hero.location = v1.location
                    end
                end
            end
        end
        self:send(protocol_pb.WORLDMAPQUERYTIME, msg)
    end,

    -- 获取矿厂下面的矿车列表
    getGatherCar = function(self, inst_id)
        local msg = protocol_pb.CSWMGetGatherCar()
        msg.instId = inst_id
        self:send(protocol_pb.WORLDMAPGATHERCARS, msg)
    end,

    -- 获取建筑队伍
    getHoldTeams = function(self, inst_id)
        local msg = protocol_pb.CSWMGetTeams()
        msg.instId = tostring(inst_id)
        self:send(protocol_pb.WORLDMAPGETEAMS, msg)
    end,

    -- 运输资源
    transportRes = function(self, inst_id, team_id, res_list, back)
        local msg = protocol_pb.CSWMMoveRes()
        msg.instId = inst_id
        msg.teamId = team_id
        msg.bReturn = back
        for k, v in ipairs(res_list) do
            local res = msg.reses:add()
            res.ItemTemId = v.item_id
            res.Count = v.count
        end
        self:send(protocol_pb.WORLDMAPMOVERES, msg)
    end,

    -- 销毁建筑
    destroyHold = function(self, inst_id)
        local msg = protocol_pb.CSWMDestory()
        msg.instId = inst_id
        self:send(protocol_pb.WORLDMAPDESTORY, msg)
    end,

    -- 建筑升级
    levelupHold = function(self, inst_id)
        local msg = protocol_pb.CSWMBuildLevelUp()
        msg.instId = inst_id
        self:send(protocol_pb.WORLDMAPBUILDLEVELUP, msg)
    end,

    -- 建筑修理
    repairHold = function(self, inst_id)
        local msg = protocol_pb.CSWMRepair()
        msg.instId = inst_id
        self:send(protocol_pb.WORLDMAPREPARIR, msg)
    end,

    -- 支援队伍
    supportTeams = function(self, inst_id)
        local msg = protocol_pb.CSWMGetHelps()
        msg.instId = inst_id
        self:send(protocol_pb.WORLDMAPGETHELPS, msg)
    end,

    -- 调整队伍
    editHoldTeam = function(self, inst_id, teams)
        local msg = protocol_pb.CSWMEditTeam()
        msg.instId = inst_id
        for k, v in ipairs(teams) do
            local formation = msg.formations:add()
            formation.formationId = v.formationId
            formation.teamId = v.teamId
            for k1, v1 in ipairs(v.formationHero) do
                if v1.location > 0 then
                    local hero = formation.formationHero:add()
                    hero.heroId = v1.heroId
                    hero.location = v1.location
                end
            end
        end
        self:send(protocol_pb.WORLDMAPEDITTEAM, msg)
    end,

    -- 获取集结列表
    getMassList = function(self, id)
        local msg = protocol_pb.CSWMGetMassList()
        msg.familyInstId = id
        self:send(protocol_pb.WORLDMAPGETMASSLIST, msg)
    end,

    -- 集结详情
    getMassInfo = function(self, id)
        local msg = protocol_pb.CSWMGetMassInfo()
        msg.massInstId = id
        self:send(protocol_pb.WORLDMAPGETMASSINFO, msg)
    end,

    -- 取消集结
    destoryMass = function(self, id)
        local msg = protocol_pb.CSWMGetMassDestory()
        msg.massInstId = id
        self:send(protocol_pb.WORLDMAPGETMASSDESTORY, msg)
    end,

    -- 移除集结队伍
    removeMassTeam = function(self, mass_id, player_id, army_id)
        local msg = protocol_pb.CSWMMassDelMem()
        msg.massInstId = mass_id
        msg.playerid = player_id
        msg.armyId = army_id
        self:send(protocol_pb.WORLDMAPMASSDELMEM, msg)
    end,

    -- 获取玩家自己的建筑
    getOwnHolds = function(self)
        local msg = protocol_pb.CSWMGetOwnerBuild()
        self:send(protocol_pb.WORLDMAPGETOWNERBUILD, msg)
    end,
    
    -- 改造车体
    refitCaebody = function(self, hero_id)
        local msg = protocol_pb.CSRefitCarBody()
        msg.heroId = hero_id
        self:send(protocol_pb.REFITCAEBODY, msg)
    end,
    
    -- 取消改造
    cancelRefitCarBody = function(self, hero_id)
        local msg = protocol_pb.CSCancelRefitCarBody()
        msg.heroId = hero_id
        self:send(protocol_pb.CANCELREFITCARBODY, msg)
    end,

    -- 加速改造
    speedUpRefit = function(self, hero_id, items, isPromFinish)
        local msg = protocol_pb.CSSpeedUpRefit()
        msg.heroId = hero_id
        msg.quickenItem.ItemTemId = items.item_id
        msg.quickenItem.Count = items.count
        msg.isPromFinish = isPromFinish
        self:send(protocol_pb.SPEEDUPREFIT, msg)
    end,

    -- 获取车体信息
    getCarBodyInfo = function(self, hero_id)
        local msg = protocol_pb.CSGetCarBodyInfo()
        msg.heroId = hero_id
        self:send(protocol_pb.GETCARBODYINFO, msg)
    end,

    -- 进行模拟战斗
    simBattle = function(self, msg)
        self:send(protocol_pb.TESTFIGHTPROP, msg)
    end,

    -- 获取修理厂信息
    getRepairInfo = function(self)
        print("发送修理厂消息...")
        local msg = protocol_pb.CSGetRepairFactoryInfo()
        self:send(protocol_pb.GETREPAIRFACTORYINFO, msg)
    end,

    -- 添加修理英雄
    addRepairHero = function(self, hero_id_list, isAllRepair)
        local msg = protocol_pb.CSAddRepairHero()
        for __, id in ipairs(hero_id_list) do
            msg.heroId:append(id)
        end
        msg.isAllRepair = isAllRepair or 0
        self:send(protocol_pb.PERSONBASEADDREPAIRHERO, msg)
    end,

    -- 取消英雄修理
    removeRepairHero = function(self, hero_id)
        local msg = protocol_pb.CSCancelRepairHero()
        msg.heroId = hero_id
        self:send(protocol_pb.PERSONBASECANCELREPAIRHERO, msg)
    end,

    -- 获取需要修理的英雄
    getNeedRepairHero = function(self)
        local msg = protocol_pb.CSGetNeedRepairHero()
        self:send(protocol_pb.GETNEEDREPAIRHERO, msg)
    end,

    -- 加速修理战车
    speedUpRepairHero = function(self, heroId, ItemTemId, Count, isPromFinish)
        local msg = protocol_pb.CSSpeedUpRepairHero()
        msg.heroId = heroId
        msg.quickenItem.ItemTemId = ItemTemId
        msg.quickenItem.Count = Count
        msg.isPromFinish = isPromFinish
        self:send(protocol_pb.SPEEDUPREPAIRHERO, msg)
    end,

    -- 获取修理厂制造信息
    getFactoryMakeInfo = function(self)
        local msg = protocol_pb.CSGetFactoryMakeInfo()
        self:send(protocol_pb.GETFIXFACTORYMAKEIFNO, msg)
    end,

    makeRepairPackage = function(self, count, isPromFinish)
        local msg = protocol_pb.CSMakeRepairPackage()
        msg.count = count
        msg.isPromFinish = isPromFinish
        self:send(protocol_pb.MAKEREPAIRPACKAGE, msg)
    end,

    reliveHero = function(self, reliveHero)
        local msg = protocol_pb.CSReliveHero()
        for __, id in ipairs(reliveHero) do
            msg.reliveHero:append(id)
        end
        self:send(protocol_pb.RELIVEHERO, msg)
    end,

    -- 领取修理包
    getRepairPackage = function(self, finishAndGet)
        local msg = protocol_pb.CSGetRepairPackage()
        msg.finishAndGet = finishAndGet
        self:send(protocol_pb.GETREPAIRPACKAGE, msg)
    end,

    -- 加速制作修理包
    speedUpMakeRePackage = function(self, ItemTemId, Count, isPromFinish)
        local msg = protocol_pb.CSSpeedUpMakeRePackage()
        msg.quickenItem.ItemTemId = ItemTemId
        msg.quickenItem.Count = Count
        msg.isPromFinish = isPromFinish
        self:send(protocol_pb.SPEEDUPMAKEREPACKAGE, msg)
    end,

    -- 使用修理包
    useRepairPackage = function(self, heroId, ItemTemId, Count, repair_all)
        local msg = protocol_pb.CSUseRepairPackage()
        for k, v in ipairs(heroId) do
            msg.heroId:append(v)
        end
        msg.useItem.ItemTemId = ItemTemId
        msg.useItem.Count = Count
        msg.allRepair = repair_all
        self:send(protocol_pb.USEREPAIRPACKAGE, msg)
    end,

    -- 获取兵工厂信息
    getArmoryInfo = function(self)
        local msg = protocol_pb.CSGetArmoryInfo()
        self:send(protocol_pb.GETARMORYINFO, msg)
    end,

    -- 获取制造图纸信息
    getDrawingInfo = function(self, type)
        local msg = protocol_pb.CSGetDrawingInfo()
        msg.drawingType = type
        self:send(protocol_pb.GETDRAWINGINFO, msg)
    end,

    -- 生产装备
    armoryProductEquip = function(self, drawingType, isMultiple)
        local msg = protocol_pb.CSProductEquip()
        msg.drawingType = drawingType
        msg.isMultiple = isMultiple
        self:send(protocol_pb.PRODUCTEQUIP, msg)
    end,

    -- 领取生产的装备
    getArmoryEquip = function(self)
        local msg = protocol_pb.CSGetEquip()
        self:send(protocol_pb.GETEQUIP, msg)
    end,

    -- 加速生产装备
    speedUpProductEquip = function(self, item_id, count, finish)
        local msg = protocol_pb.CSSpeedUpProductEquip()
        msg.quickenItem.ItemTemId = item_id
        msg.quickenItem.Count = count
        msg.isPromFinish = finish
        self:send(protocol_pb.SPEEDUPPRODUCTEQUIP, msg)
    end,

    -- 获取英雄需要修理的信息
    getHeroNeedFixInfo = function(self, hero_id)
        local msg = protocol_pb.CSGetHeroNeedFixInfo()
        for k, v in ipairs(hero_id) do
            msg.heroId:append(v)
        end
        self:send(protocol_pb.GETHEROREPAIRINFO, msg)
    end,

    -- 领取矿场矿石
    getMineFieldMineral = function(self, buildId)
        local msg = protocol_pb.CSGetResource()
        msg.buildId = buildId
        self:send(protocol_pb.GETRESOURCE, msg)
        -- local msg = protocol_pb.CSGetMineFieldMineral()
        -- msg.mineFieldId = mineFieldId
        -- self:send(protocol_pb.GETMINERALS, msg)
    end,

    -- 世界地图使用修理包
    useRepairKitInWorldMap = function(self, hold_id, heros, count, type)
        local msg = protocol_pb.CSWMUseRepair()
        msg.instId = hold_id
        msg.type = type
        msg.count = count
        for k, v in ipairs(heros) do
            msg.heroId:append(v)
        end
        self:send(protocol_pb.WORLDUSEREPAIR, msg)
    end,

    -- 请求章节目标数据
    reqChapterAchievementList = function(self)
        local msg = protocol_pb.CSGetChapterAchievements()
        self:send(protocol_pb.CHAPTERACHIEVEGETLIST, msg)
    end,

    -- 请求章节目标奖励
    reqChapterAchievementReward = function(self, id)
        local msg = protocol_pb.CSGetChapterAchieveReward()
        msg.id = id
        self:send(protocol_pb.CHAPTERACHIEVEGETREWARD, msg)
    end,

    -- 请求开启章节目标
    reqChapterAchievementStart = function(self, id)
        log.info("#### reqChapterAchievementStart:"..tostring(id))
        local msg = protocol_pb.CSChapterAchieveStart()
        msg.chapterId = id
        self:send(protocol_pb.CHAPTERACHIEVESTART, msg)
    end,

    -- 请求引导状态
    reqNotifyGuide = function(self)
        local msg = protocol_pb.CSGuideInfo()
        self:send(protocol_pb.NOTIFYGUIDE, msg)
    end,

    -- 请求保存引导
    -- order_guide_id - 顺序引导完成的 id
    -- random_guides - 随机引导 {id-true/false}, 需要 pack 成整数数组
    reqSaveGuide = function(self, order_guide_id, random_guides)
        local msg = protocol_pb.CSSaveGuideInfo()
        msg.guide.orderGuide = order_guide_id      -- MGuideInfo

        if random_guides then
            local unpacked = {}
            for id,v in pairs(random_guides) do
                if v then
                    table.insert(unpacked, id-10000)
                end
            end
            -- 打包
            local packed = GuideInfoPacker.pack(unpacked)
            if packed.Table then
                for k,v in ipairs(packed.Table) do
                    msg.guide.randGuide:append(v)
                end
            end
        end

        self:send(protocol_pb.SAVEGUIDE, msg)
    end,

    -- 搜索
    worldMapSearch = function(self, searchType)
        local msg = protocol_pb.CSWorldMapSearch()
        msg.searchType = searchType
        self:send(protocol_pb.WORLDMAPSEARCH, msg)
    end,

    -- 查询建筑消耗
    buildingExpend = function(self, id, queryType)
        print('发送请求 查询建筑消耗', id, queryType)
        local msg = protocol_pb.CSBuildingExpend()
        msg.id = id
        msg.queryType = queryType
        self:send(protocol_pb.QUERYBUILDEXPEND, msg)
    end,

    -- 缩略图
    requestMiniMap = function(self)
        local msg = protocol_pb.CSWorldMapMiniMap()
        self:send(protocol_pb.WORLDMAPMINIMAP, msg)
    end,

    -- 集结设置
    massSetting = function(self, mass_id)
        local msg = protocol_pb.CSWorldMapMassEditSet()
        msg.instId = mass_id
        msg.set.memStart = world_map_info.mass_sets.memStart
        msg.set.pwd = world_map_info.mass_sets.pwd
        msg.set.maxAuto = world_map_info.mass_sets.maxAuto
        msg.set.shared = world_map_info.mass_sets.shared
        self:send(protocol_pb.WORLDMAPMASSEDITSET, msg)
    end,
    
    -- 工会科技捐献
    familyTechDonate = function(self, id, tech_id)
        local msg = protocol_pb.CSFamilyTechDonate()
        msg.id = id
        msg.tech_id = tech_id
        self:send(protocol_pb.FAMILYTECHDONATE, msg)
    end,

    -- 工会科技信息获取
    getFamilyTech = function(self, id)
        local msg = protocol_pb.CSGetFamilyTech()
        msg.familyId = id
        self:send(protocol_pb.GETFAMILYTECH, msg)
    end,

    -- 世界地图建筑改名
    changeHoldName = function(self, id, name)
        local msg = protocol_pb.CSWorldMapChangeName()
        msg.instId = id
        msg.newName = name
        self:send(protocol_pb.WORLDMAPCHANGENAME, msg)
    end,

    -- 主基地移动
    leviathanMove = function(self, id, x, y)
        local msg = protocol_pb.CSWorldMapLeviathanMove()
        msg.playerId = id
        msg.posX = x
        msg.posY = y
        self:send(protocol_pb.WORLDMAPLEVIATHANMOVE, msg)
    end,

    -- 主基地移动
    startArmyLine = function(self, type, heros, soldiers, dest_id, mass_id, atk_cnt, pos_x, pos_y)
        local msg = protocol_pb.CSStartArmyLine()
        msg.marchAction = type
        msg.dstInstId = dest_id

        if heros then
            for k, v in ipairs(heros) do
                msg.armys.heros:append(v)
            end
        end

        if soldiers then
            for k, v in pairs(soldiers) do
                if v.count > 0 then
                    local soldier = msg.armys.soldiers:add()
                    soldier.cfgid = v.cfg_id
                    soldier.count = v.count
                end
            end
        end

        if mass_id then
            msg.massParam.massid = mass_id
        end

        if atk_cnt then
            msg.atkmonsterparam.atk_count = atk_cnt
        end

        if pos_x and pos_y then
            msg.dstPos.posX = pos_x
            msg.dstPos.posY = pos_y
        end

        self:send(protocol_pb.WORLDMAPSTARTARMY, msg)
    end,

    -- 主基地跃迁
    baseStartTransition = function(self, x, y)
        local msg = protocol_pb.CSStartTransition()
        msg.posx = x
        msg.posy = y

        self:send(protocol_pb.WORLDMAPSTARTTRANSITION, msg)
    end,

    -- 主基地随机跃迁
    baseRandTransition = function(self)
        local msg = protocol_pb.CSRandTransition()

        self:send(protocol_pb.WORLDMAPRANDTRANSITION, msg)
    end,

    -- 发起集结
    startMass = function(self, dst_id, duration, heros, soldiers, pos_x, pos_y)
        local msg = protocol_pb.CSStartMass()
        msg.dstId = dst_id
        msg.duration = duration

        if pos_x and pos_y then
            msg.dstpos.posX = pos_x
            msg.dstpos.posY = pos_y
        end
        
        for k, v in ipairs(heros) do
            msg.armys.heros:append(v)
        end

        for k, v in pairs(soldiers) do
            if v.count > 0 then
                local soldier = msg.armys.soldiers:add()
                soldier.cfgid = v.cfg_id
                soldier.count = v.count
            end
        end

        self:send(protocol_pb.WORLDMAPSTARTMASS, msg)
    end,

    -- 个人基地加速
    perSonBaseSpeedUp = function(self, instId, speedUpType, isPromFinish, isBuyItem, items, subId)
        print(string.format("建筑加速 建筑实例id:%s 加速类型:%s 是否立即完成:%s 是否购买道具加速:%s subId:%s", instId, speedUpType, isPromFinish, isBuyItem, subId))
        for __, item in ipairs(items) do
            print(string.format("道具 id:%s 数量:%s", item.item_id, item.item_num))
        end
        local msg = protocol_pb.CSPerSonBaseSpeedUp()
        msg.instId = instId
        msg.speedUpType = speedUpType
        msg.isPromFinish = isPromFinish
        msg.isBuyItem = isBuyItem
        msg.subId = subId or 0

        for __, item in ipairs(items) do
            local ri = msg.quickenItem:add()
            ri.ItemTemId = item.item_id
            ri.Count = item.item_num
        end
        self:send(protocol_pb.PERSONBASESPEEDUP, msg)
    end,

    --集结出发
    massAttack = function(self, id)
        local msg = protocol_pb.CSMassAtk()
        msg.massId = id

        self:send(protocol_pb.WORLDMAPMASSATK, msg)
    end,

    -- 获取皮肤信息
    getSkinInfo = function(self)
        local msg = protocol_pb.CSGetSkinInfo()
        self:send(protocol_pb.GETSKININFO, msg)
    end,

    -- 更换皮肤
    useSkin = function(self, skinId)
        local msg = protocol_pb.CSUseSkin()
        msg.skinId = skinId
        self:send(protocol_pb.USESKIN, msg)
    end,

    trainSoldier = function(self, instId, soldierId, soldierCount, bPromFinish)
        local msg = protocol_pb.CSTrainSoldier()
        msg.instId = instId
        msg.soldierId = soldierId
        msg.soldierCount = soldierCount
        msg.bPromFinish = bPromFinish
        self:send(protocol_pb.TRAINSOLDIER, msg)
    end,

    receiveSoldier = function(self, instId)
        local msg = protocol_pb.CSReceiveSoldier()
        msg.instId = instId
        self:send(protocol_pb.RECEIVESOLDIER, msg)
    end,

    getBuildDetails = function(self, buildId)
        local msg = protocol_pb.CSGetBuildDetails()
        msg.buildId = buildId
        self:send(protocol_pb.GETBUILDDETAILS, msg)
    end,

    deleteSoldier = function(self, list)
        local msg = protocol_pb.CSDeleteSoldier()
        for __, value in ipairs(list) do
            local s = msg.soldiers:add()
            s.cfgId = value.cfg_id
            s.count = value.count
            s.state = value.state
        end
        self:send(protocol_pb.DELETESOLDIER, msg)
    end,

    getBattleReport = function(self, player_id, obj_id, type)
        local msg = protocol_pb.CSGetReport()
        msg.type = type
        msg.playerId = player_id
        msg.targetInstId = obj_id
        self:send(protocol_pb.WORLDMAPBATMSGNOTIFY, msg)
    end,

    treatSoldier = function(self, list, bPromFinish)
        local msg = protocol_pb.CSTreatSoldier()
        msg.bPromFinish = bPromFinish
        for __, value in ipairs(list) do
            local s = msg.tasks:add()
            s.soldierId = value.soldierId
            s.soldierCount = value.soldierCount
        end
        self:send(protocol_pb.TreatSoldier, msg)
    end,

    repairTrap = function(self, list, bPromFinish)
        local msg = protocol_pb.CSRepairTrap()
        msg.bPromFinish = bPromFinish
        for __, value in ipairs(list) do
            local s = msg.tasks:add()
            s.soldierId = value.soldierId
            s.soldierCount = value.soldierCount
        end
        self:send(protocol_pb.REPAIRTRAP, msg)
    end,

    recvRepairedTrap = function(self, cfgId)
        local msg = protocol_pb.CSRecvRepairedTrap()
        msg.cfgId = cfgId
        self:send(protocol_pb.RECVREPAIREDTRAP, msg)
    end,

    recvTreatedSoldier = function(self, cfgId)
        local msg = protocol_pb.CSRecvTreatedSoldier()
        msg.cfgId = cfgId
        self:send(protocol_pb.RECVTREATEDSOLDIER, msg)
    end,

    fireFight = function(self)
        local msg = protocol_pb.CSFireFight()
        self:send(protocol_pb.FIREFIGHT, msg)
    end,

    repairCore = function(self)
        local msg = protocol_pb.CSRepairCore()
        self:send(protocol_pb.REPAIRCORE, msg)
    end,

    changeGarrison = function(self, list)
        local msg = protocol_pb.CSChangeGarrison()
        for __, hero_id in ipairs(list) do
            table.insert(msg.heros, hero_id)
        end
        self:send(protocol_pb.CHANGEGARRISON, msg)
    end,

    getCaptainGift = function(self)
        local msg = protocol_pb.CSGetCaptainGift()
        self:send(protocol_pb.GETCAPTAINGIFT, msg)
    end,

    naturalGiftReset = function(self, isBuyItem)
        local msg = protocol_pb.CSNaturalGiftReset()
        msg.isBuyItem = isBuyItem
        self:send(protocol_pb.NATURALGIFTRESET, msg)
    end,

    naturalGiftLevelup = function(self, listId)
        local msg = protocol_pb.CSNaturalGiftLevelup()
        msg.listId = listId
        self:send(protocol_pb.NATURALGIFTLEVELUP, msg)
    end,

    getGlobalPropInfo = function(self, listId)
        local msg = protocol_pb.CSGetGlobalPropInfo()
        self:send(protocol_pb.GETGLOBALPROPINFO, msg)
    end,

    getPlayerPvpData = function(self, playerId)
        local msg = protocol_pb.CSGetPlayerPvpData()
        msg.playerId = playerId
        self:send(protocol_pb.GETPLAYERPVPDATA, msg)
    end,

    trainProfession = function(self, cfgId, bPromFinish)
        local msg = protocol_pb.CSTrainProfession()
        msg.cfgId = cfgId
        msg.bPromFinish = bPromFinish
        self:send(protocol_pb.TRAINPROFESSION, msg)
    end,

    retireProfession = function(self, instId)
        local msg = protocol_pb.CSRetireProfession()
        msg.instId = instId
        self:send(protocol_pb.RETIREPROFESSION, msg)
    end,

    recvProfession = function(self)
        local msg = protocol_pb.CSRecvProfession()
        self:send(protocol_pb.RECVPROFESSION, msg)
    end,

    checkFamilyNameStr = function(self, flag, str)
        local msg = protocol_pb.CSCheckFamilyNameStr()
        msg.strContext = str
        msg.flag = flag
        self:send(protocol_pb.CHECKFAMILYNAMESTR, msg)
    end,

    getMyApplyFamily = function(self)
        local msg = protocol_pb.CSGetMyApplyFamily()
        self:send(protocol_pb.GETMYAPPLYFAMILY, msg)
    end,

    dismantleMaterial = function(self, itemId, bOneKey)
        local msg = protocol_pb.CSDismantleMaterial()
        msg.itemId = itemId
        msg.bOneKey = bOneKey
        self:send(protocol_pb.DISMANTLEMATERIAL, msg)
    end,

    composeMaterial = function(self, itemId, bOneKey)
        local msg = protocol_pb.CSComposeMaterial()
        msg.itemId = itemId
        msg.bOneKey = bOneKey
        self:send(protocol_pb.COMPOSEMATERIAL, msg)
    end,

    setChapterState = function(self, chapter_id, state)
        log.info("### send setChapterState:"..chapter_id..', '..state)

        local msg = protocol_pb.CSSetChapterState()
        msg.chapterId = chapter_id
        msg.state = state

        self:send(protocol_pb.SETCHAPTERSTATE, msg)
    end,

    -- 开始资源船争夺战(或侦察)
    startChapterFight = function(self, chapter_id, heros, soldiers, is_spy)
        local msg = protocol_pb.CSStartChapterFight()
        msg.chapterId = chapter_id
        log.info("### startChapterFight chapter_id:"..chapter_id)
        if is_spy then
            msg.action = protocol_pb.ECHAPTER_SPY
        else
            msg.action = protocol_pb.ECHAPTER_FIGHT

            if heros then
                for k, v in ipairs(heros) do
                    msg.armys.heros:append(v)
                end
            end
    
            if soldiers then
                for k, v in pairs(soldiers) do
                    if v.count > 0 then
                        local soldier = msg.armys.soldiers:add()
                        soldier.cfgid = v.cfg_id
                        soldier.count = v.count
                    end
                end
            end     
        end

        self:send(protocol_pb.CHAPTERFIGHT, msg)
    end,

    delFamilyGift = function(self, id, bOneKey)
        local msg = protocol_pb.CSDelFamilyGift()
        msg.instId = id
        msg.bOneKey = bOneKey
        self:send(protocol_pb.DELFAMILYGIFT, msg)
    end,

    -- 请求解锁关卡，可以开始打了
    unlockLevel = function(self, chapter_id, level_id)
        log.info("### req unlockLevel chapter:"..chapter_id..", level:"..level_id)

        local msg = protocol_pb.CSUnLockCopy()
        msg.chapterId = chapter_id
        msg.copyId = level_id
        self:send(protocol_pb.UNLOCKCOPY, msg)
    end,    

    getDepotPropList = function(self, instId)
        local msg = protocol_pb.CSGetDepotPropList()
        msg.instId = instId
        self:send(protocol_pb.GETDEPORTINFO, msg)
    end,

    --[[成长任务]]
    -- 请求获得成长任务信息
    reqGrowthTaskInfo = function(self)
        local msg = protocol_pb.CSTaskInfo()
        msg.taskType = common_pb.Growth_Task
        self:send(protocol_pb.NOTIFYTASKINFO, msg)
    end,

    -- 请求获得成长任务的奖励
    reqGrowthTaskReward = function(self, task_id)
        local msg = protocol_pb.CSGetTaskReward() 
        msg.awardType = common_pb.Growth_Task
        msg.taskId = task_id

        self:send(protocol_pb.GETTASKREWARD, msg)
    end,

    drawCard = function(self, drawCardType)
        local msg = protocol_pb.CSDrawCard()
        msg.drawCardType = drawCardType
        self:send(protocol_pb.DRAWCARD, msg)
    end,

    delFamilyBuild = function(self, inst_id)
        local msg = protocol_pb.CSDelFamilyBuild()
        msg.instId = inst_id
        self:send(protocol_pb.DELFAMILYBUILD, msg)
    end,

    backToFamilyFortress = function(self, inst_id)
        local msg = protocol_pb.CSWorldMapReturnFamilyFortress()
        msg.instId = inst_id
        self:send(protocol_pb.WORLDMAPRETURNFAMILYFORTRESS, msg)
    end,

    modifyRoleName = function(self, modifyType, roleName)
        print(" modifyRoleName 发送消息:", modifyType, roleName)
        local msg = protocol_pb.CSModifyRoleName()
        msg.modifyType = modifyType
        msg.roleName = roleName
        self:send(protocol_pb.MODIFYROLENAME, msg)
    end,

    getActiveList = function(self)
        local msg = protocol_pb.CSGetActiveList()
        self:send(protocol_pb.GETACTIVELIST, msg)
    end,

    setRecommendTech = function(self, set_id, cancel_id)
        local msg = protocol_pb.CSSetRecommendTech()
        msg.setId = set_id
        msg.cancelId = cancel_id
        self:send(protocol_pb.SETRECOMMENDTECH, msg)
    end,

    queryAllArmyLine = function(self)
        local msg = protocol_pb.CSQueryAllArmyLine()
        self:send(protocol_pb.WORLDMAPQUERYALLARMYLINE, msg)
    end,

    collectMail = function(self, id)
        local msg = protocol_pb.CSCollectMail()
        msg.mailId = id
        self:send(protocol_pb.COLLECTMAIL, msg)
    end,

    getPlayerInfo = function(self, playerId)
        -- log.error("发送 获取玩家信息", playerId)
        local msg = protocol_pb.CSGetPlayerInfo()
        msg.playerId = playerId
        self:send(protocol_pb.GETPLAYERINFO, msg)
    end,
}
