local log = logger.get("Game", "Announce Manager")
local announce_cls = require("systems.announce_sys.announce")

local announce_manager_cls = class{

    ctor = function(self)
        self.announce_list = {}
        self.battle_announce = {}
        self.isBattle = false
        self.event = require('base.event').new()
    end,

    add = function(self, message)
        local announce = announce_cls(message)
        table.insert(self.announce_list, announce)

        if announce.cfg.isdisturb ~= 0 then
            table.insert(self.battle_announce, announce)
        end

        self:sort()

        self.event:dispatch("announceNotify")
    end,

    getAnnounce = function(self)
        local str = nil
        if self.isBattle then
            str = self:getBattleNounce()
        else
            str = self:getNormalNounce()
        end
        return str
    end,

    getNormalNounce = function(self)
        local str = nil
        if self.announce_list[1] then
            str = self.announce_list[1].content
            self:remove(self.announce_list.id)
        end

        return str
    end,

    getBattleNounce = function(self)
        local str = nil
        if self.battle_announce[1] then
            str = self.battle_announce[1].content
            self:remove(self.battle_announce.id)
        end

        return str
    end,

    remove = function(self, id)
        local idx = 0
        for k, v in ipairs(self.announce_list) do
            if v.id == id then
                idx = k
                break
            end
        end
        if idx ~= 0 then
            table.remove( self.announce_list, idx )
        end

        idx = 0
        for k, v in ipairs(self.battle_announce) do
            if v.id == id then
                idx = k
                break
            end
        end
        if idx ~= 0 then
            table.remove( self.battle_announce, idx )
        end
    end,

    canPlay = function(self)
        return self.announce_list[1]
    end,

    sort = function(self)
        local func = function(a, b)
            return a.adv < b.adv
        end
        table.sort(self.announce_list, func)
        table.sort( self.battle_announce, func )
    end,

    stateChanged = function(self, isBattle)
        self.isBattle = isBattle
    end,
}

announce_manager = announce_manager_cls()

return announce_manager