local log = logger.get("Game", "Chapter Achievement")
require('config.chapter_achievement')

-- 章节目标管理器
local cls = class{
    extensions = {"systems/chapter_achievement_manager+go.lua"},

    ctor = function(self)
        self.chapter_id = -1   -- 当前章
        self.achievement_info_list = nil -- 当前章任务数据,服务器下发的
    end,

    -- 获取某章任务完成情况
    getChapterAchievementList = function(self)
        if self.chapter_id == -1 then
            --log.error("Chapter achievement not initialized!")
            return nil
        end

        local achievement_info = {}
        for k,v in ipairs(self.achievement_info_list) do
            achievement_info[v.id] = v  -- 转换成用 id 做 key
        end

        -- 遍历章目标配置，
        local result = {}
        config.chapter_achievement_quests:foreach(
            function(id,cfg)
                if cfg.chapter == self.chapter_id then
                    -- 配置表数据
                    local quest = {
                        id = cfg.id,
                        is_final = cfg.is_final,
                        desc = cfg.quest_des,
                        reward = loadstring('return '..cfg.quest_reward)()
                    }

                    -- 服务器下发数据
                    local info = achievement_info[id]
                    if info then
                        quest.current_value = info.curVal
                        quest.max_value = info.maxVal
                        --[[enum EAchievementState
                        {
                        ACHIEVEMENT_NONE = 1;//初始状态
                        ACHIEVEMENT_UNLOCK = 2;//已解锁
                        ACHIEVEMENT_FINISH = 3;//完成未领取
                        ACHIEVEMENT_GETAWARD = 4;//领取
                        }]]
                        quest.state = info.state
                    else
                        -- 没找到相同 id 的 info
                        quest.current_value = 0
                        quest.max_value = 0
                        quest.state = common_pb.ACHIEVEMENT_NONE

                        log.error("Failed to find quest info:"..tostring(id))
                    end

                    table.insert(result, quest)
                end
            end
        )
        -- 为 result 排序
        table.sort(result, function(a, b)
            return a.id < b.id
        end)
        return result
    end,

    -- 获取主目标（在主界面上显示的）
    getPrimaryAchievement = function(self)
        local list = self:getChapterAchievementList()
        if not list then
            return
        end
        local rst = nil
        -- 优先已完成未领奖的
        for idx, item in ipairs(list) do
            if item.state == common_pb.ACHIEVEMENT_FINISH then
                rst = item
                break
            end
        end
        if rst then
            return rst
        end

        -- 
        for idx, item in ipairs(list) do
            if item.state == common_pb.ACHIEVEMENT_UNLOCK then
                rst = item
                break
            end
        end

        return rst
    end,

    -- 消息刷新
    onChapterAchievementList = function(self, msg)
        if msg.respCode == 0 then
            log.info("####onChapterAchievementList respCode == 0")
            if msg.chapterAchive then
                self.chapter_id = msg.chapterAchive.chapterId  -- 当前章 id
                self.achievement_info_list = msg.chapterAchive.achievements

                -- log 输出
                --[[
                log.info("current chapter:"..tostring(self.chapter_id))
                for k,v in ipairs(msg.chapterAchive.achievements) do
                    log.info("id:"..v.id..","..v.curVal..","..v.maxVal..","..v.state)
                end]]
            else
                log.info("#### No chapterAchive")
            end
        else
            if msg.respCode == 10003 then
                -- 所有任务都完成了
                log.info("#### all achievement completed!")
                msg.respCode = 0
            else
                log.error("### get chapter achievement list failed:"..msg.respCode)
            end
        end

        -- 刷新界面
        if chapter_achievement_frame  then
            local achievement_list = self:getChapterAchievementList()
            chapter_achievement_frame:refresh(self.chapter_id, achievement_list)
        end

        -- 刷新个人基地上的章节成就快捷按钮
        if home_frame then
            home_frame:refreshChapterAchievementShortcut()
        end

        if self.chapter_id <= 0 then
            self:checkChapterStart()
        elseif not frame_manager:isDisplay('chapter_achievement_cutscene') and self:isAllAchievementFinished() then
            frame_manager:show('chapter_achievement_cutscene', nil, self.chapter_id)
            local next_chapter_id = self.chapter_id + 1
            log.info("### show chapter_start:"..next_chapter_id)     
            app:startCoroutine(
                function()
                    coroutine.yield(WaitForSeconds(4.0))
                    frame_manager:show('chapter_start', nil, next_chapter_id)                    
                end
            )       

        end

        log.info("### waiting_start_id, chapter_id :"..tostring(self.waiting_start_id)..", "..self.chapter_id)

        if self.waiting_start_id == self.chapter_id then
            self.waiting_start_id = nil
            if self.chapter_id ~= 1 then    -- 第一章不自动显示
                frame_manager:show('chapter_start', nil, self.chapter_id)
            end
            --frame_manager:show('chapter_start', nil, self.chapter_id)
        end
    end,

    -- 是否所有目标都达成了
    isAllAchievementFinished = function(self)
        local list = self:getChapterAchievementList()
        if not list then
            return
        end

        for idx, item in ipairs(list) do
            if item.state ~= common_pb.ACHIEVEMENT_GETAWARD then
                -- 有任务未完成的
                return false
            end
        end    
        
        return true
    end,

    -- 检查章节开启
    checkChapterStart = function(self)
        log.info("#### checkChapterStart")

        if self.chapter_id <= 0 then    -- 第一章都没开启
            -- 请求开启第一章
            self.waiting_start_id = 1
            network_sender:reqChapterAchievementStart(1)
        else
            -- 判断所有目标都达成
            if self:isAllAchievementFinished() then
                -- 请求开启下一章
                network_sender:reqChapterAchievementStart(self.chapter_id + 1)

                self.waiting_start_id = self.chapter_id + 1
                log.info("### waiting start id:"..self.waiting_start_id)   

                return true
            else
                log.info("### not all finished")
            end
        end
    end,

    -- 获得奖励
    onReward = function(self, msg)
    end,
}

chapter_achievement_manager = cls()

return chapter_achievement_manager