local log = logger.get("Game", "Chapter Manager")

chapter_manager = nil

-- 每一章满足触发条件时调用的对话
local trigger_dialogs = 
{
    [1] = function(self, chapter_id)
        self:showDialogOption('level_script_133', {'level_script_134', 'level_script_135'}, function(selected)
            if selected == 1 then
                -- 用户选择触发
                network_sender:setChapterState(chapter_id, common_pb.ECHAPTER_TRIGGER)
                -- 播放资源船靠近动画
               
            elseif selected == 2 then
                -- 暂先不触发
                -- 显示图标
                home_frame.transform:Find('new/buttomRight/triggerResShipBtn'):SetActive(true)
            end

        end)
    end,
    [2] = function(self, chapter_id)
        self:showDialogOption('level_script_136', {'level_script_137', 'level_script_138'}, function(selected)
            if selected == 1 then
                -- 用户选择触发
                network_sender:setChapterState(chapter_id, common_pb.ECHAPTER_TRIGGER)

            elseif selected == 2 then
                -- 暂先不触发
                -- 显示图标
                home_frame.transform:Find('new/buttomRight/triggerResShipBtn'):SetActive(true)
            end

        end)
    end,
    [3] = function()
        self:showDialogOption('level_script_139', {'level_script_140', 'level_script_141'}, function(selected)
            if selected == 1 then
                -- 用户选择触发
                network_sender:setChapterState(chapter_id, common_pb.ECHAPTER_TRIGGER)

            elseif selected == 2 then
                -- 暂先不触发
                -- 显示图标
                home_frame.transform:Find('new/buttomRight/triggerResShipBtn'):SetActive(true)
            end

        end)
    end,
    [4] = function()
        self:showDialogOption('level_script_142', {'level_script_143', 'level_script_144'}, function(selected)
            if selected == 1 then
                -- 用户选择触发
                network_sender:setChapterState(chapter_id, common_pb.ECHAPTER_TRIGGER)

            elseif selected == 2 then
                -- 暂先不触发
                -- 显示图标
                home_frame.transform:Find('new/buttomRight/triggerResShipBtn'):SetActive(true)
            end

        end)
    end,
    [5] = function()
        self:showDialogOption('level_script_145', {'level_script_146', 'level_script_147'}, function(selected)
            if selected == 1 then
                -- 用户选择触发
                network_sender:setChapterState(chapter_id, common_pb.ECHAPTER_TRIGGER)

            elseif selected == 2 then
                -- 暂先不触发
                -- 显示图标
                home_frame.transform:Find('new/buttomRight/triggerResShipBtn'):SetActive(true)
            end

        end)
    end,

    showDialogOption = function(self, title, options, callback)
        frame_manager:show("dialog_option", nil, {title = title, options = options, callback = callback})
    end,

}

-- 关卡章节管理器
local chapter_manager_cls = class{
    ctor = function(self)
        self.chapters = {} -- 所有章

        for k=1,5 do
            local v =  config.game_copy_chapters[k]
            self:initChapterInfo(v.id, common_pb.ECHAPTER_NO_OPEN, 0)
        end
        self.curr_chapter = nil -- 当前章

        home_info.events:on("BUILD_INFO_CHANGE", function(build_id)
            local build_data = home_info:getBuildDataById(build_id)
            --[[
            if build_data.cfgId == home_info.main_entity_cfg_id then
                self:initChapterResShip()
            end
            ]]
        end)

        self.events = require("base.event").new()
    end,

    initChapterInfo = function(self, chapter_id, state, curr_level)
        log.info("## initChapterInfo:".."id:"..chapter_id..", "..state..", curr_level:"..tostring(curr_level))
        --log.info(debug.traceback())
        self.chapters[chapter_id] = {state = state, curr_level = curr_level}

        -- 判断当前状态
        if state > common_pb.ECHAPTER_NO_OPEN and state < common_pb.ECHAPTER_OVER then
            if not self.curr_chapter then
                self.curr_chapter = chapter_id
            elseif self.curr_chapter ~= chapter_id then
                log.error('### chapter state error! id:'..chapter_id..', curr id:'..self.curr_chapter)
            end
        end
    end,

    -- 设置章信息
    setChapterInfo = function(self, chapter_id, state, curr_level)
        local info = self:getChapterInfo(chapter_id)
        if info then
            if info.state == common_pb.ECHAPTER_NO_OPEN and state == common_pb.ECHAPTER_NO_TRIGGER then
                if home_frame then
                    -- 满足触发条件，显示对话
                    trigger_dialogs[chapter_id](trigger_dialogs, chapter_id)
                end
                self.curr_chapter = chapter_id
            elseif info.state == common_pb.ECHAPTER_NO_TRIGGER and state == common_pb.ECHAPTER_TRIGGER then
                log.info('## From ECHAPTER_NO_TRIGGER to ECHAPTER_TRIGGER!')
                -- 触发资源船靠近
                if home_frame then
                    home_frame.transform:Find('new/buttomRight/triggerResShipBtn'):SetActive(false)

                    -- 隐藏界面
                    frame_manager:setFloorVisibility(0, false)
                    local chapter_cfg = config.game_copy_chapters[chapter_id]
                    self:spawnResShip(chapter_id, 'born', function(newObj)

                        -- 摄像机跟随或动画
                        --local screen_center = Vector3(Screen.width / 2, Screen.height / 2, 0)
                        --game_state_home.home_map.home_camera:LookAt(newObj.transform, screen_center, 0.2)

                        app:startCoroutine(
                            function()
                                -- 等待动画完成
                                coroutine.yield(nil)
                                local screen_center = Vector3(Screen.width / 2, Screen.height / 2, 0)
                                local entity_id = chapter_cfg.entity_id
                                local grid_id = chapter_cfg.grid_id                                
                                log.info('### personBaseAdd:'..entity_id..","..grid_id)
                                network_sender:personBaseAdd(entity_id, grid_id)   

                                for i = 1, 75 do
                                    game_state_home.home_map.home_camera:LookAt(newObj.transform, screen_center, 80, nil)
                                    coroutine.yield(WaitForSeconds(0.1))                                    
                                end

                                local entity = game_state_home.home_map:getEntityByGridId(grid_id)
                                if not entity or not entity:hasBuild() then
                                    log.info("## resend personBaseAdd")
                                    network_sender:personBaseAdd(entity_id, grid_id)  
                                end

                                 -- 恢复界面
                                frame_manager:setFloorVisibility(0, true)

                                if chapter_id == 1 then
                                    guide_manager:tryStartGuide()
                                end
                            end
                        )

                    end)
                end
            elseif info.state == common_pb.ECHAPTER_TRIGGER and state == common_pb.ECHAPTER_OCCUPY then
                -- 从已触发变为占领
            elseif info.state == common_pb.ECHAPTER_OCCUPY and state == common_pb.ECHAPTER_OVER then
                -- 关卡通关
                self.chapters[chapter_id] = {state = state, curr_level = curr_level}
                self.events:dispatch("CHAPTER_OVER", chapter_id)
            end
        else
            log.error('Failed to find chapter_info:'..tostring(chapter_id))
        end

        log.info('### manager SetChapterInfo:'..chapter_id..", state:"..state.."curr_level:"..curr_level)
        self.chapters[chapter_id] = {state = state, curr_level = curr_level}
    end,

    -- 获取一章信息
    getChapterInfo = function(self, chapter_id)
        return self.chapters[chapter_id]
    end,

    -- 出生资源船
    spawnResShip = function(self, chapter_id, anim, callback)
        log.info("### spawnResShip, chapter:"..chapter_id..", anim:"..anim)

        local ship_root = GameObject.Find('ResShipRoot/ResShipRoot'..chapter_id.."/AnimRoot")
        
        if ship_root.transform:Find("resship_model") then    -- 防止重复创建
            return
        end

        local chapter_cfg = config.game_copy_chapters[chapter_id]
        local prefab_path = chapter_cfg.res_ship

        app:startCoroutine(
            function()
                coroutine.yield(WaitForInstantiate(prefab_path, ship_root.transform, function(newObj)
                    log.info('### ResShip Loaded:'..prefab_path)
                    -- 资源船播放动画
                    local anim_comp = ship_root:GetComponent(UnityEngine.Animation)
                    if anim_comp then
                        anim_comp:Play(anim)
                    else
                        log.error("Failed to find animation component:"..prefab_path)
                    end

                    newObj.name = "resship_model"
                    if callback then
                        callback(newObj)
                    end
                end))
            end)
    end,

    -- 初始化资源船状态，并尝试触发新的章
    initChapterResShip = function(self)
        -- 找到第一个未解锁的章，判断是否触发
        local unlock_chapter = nil

        -- 章节是依次解锁的，只处理第一个非 ECHAPTER_OVER 的章，
        for id=1,5 do
            local chapter_info = self.chapters[id]
            if not chapter_info then
                break
            end

            log.info('#### chapter_info:'..id..", state:"..chapter_info.state)
            if chapter_info.state == common_pb.ECHAPTER_NO_OPEN then
                log.info("#### 尝试解锁:"..id)
                if not unlock_chapter then
                    unlock_chapter = id
                end
                break
            elseif chapter_info.state == common_pb.ECHAPTER_NO_TRIGGER then
                -- 显示 icon
                log.info("#### 显示 retrigger button!!")
                if home_frame then
                    home_frame.transform:Find('new/buttomRight/triggerResShipBtn'):SetActive(true)
                else
                    log.error('### no home_frame')
                end
                break
            elseif chapter_info.state == common_pb.ECHAPTER_TRIGGER then
                -- 创建资源船，播放 idle 动画
                self:spawnResShip(id, 'idle', function()
                    -- 
                    local chapter_cfg = config.game_copy_chapters[id]
                    local entity_id = chapter_cfg.entity_id
                    local grid_id = chapter_cfg.grid_id  

                    app:startCoroutine(
                        function()
                            coroutine.yield(WaitForSeconds(1.0))

                            if game_state_home then
                                local entity = game_state_home.home_map:getEntityByGridId(grid_id)
                                if not entity or not entity:hasBuild() then
                                    log.info("## resend personBaseAdd")
                                    network_sender:personBaseAdd(entity_id, grid_id)  
                                end
                            end
                        end
                    )
                end)

                break
            elseif chapter_info.state == common_pb.ECHAPTER_OCCUPY then
                -- 创建资源船，播放 idle 动画
                self:spawnResShip(id, 'idle')
                break
            elseif chapter_info.state == common_pb.ECHAPTER_OVER then
                -- 创建资源船，播放 idle 动画
                self:spawnResShip(id, 'idle')
                -- 章已完成
                -- continue
            else

            end             
        end

        if unlock_chapter then
            local cfg = config.game_copy_chapters[unlock_chapter]
            if cfg then
                app:startCoroutine(
                    function()
                        local main_level = 0
                        while main_level <= 0 do
                            main_level = home_info:getMainEntityLv()
                            -- 等待 mian_level 有效
                            coroutine.yield(nil)
                        end

                        log.info('## main_level:'..main_level..", cfg.unlock:"..cfg.unlock)
                        if main_level >= cfg.unlock then
                            -- 请求解锁
                            log.info('### 请求解锁章:'..unlock_chapter)
                            network_sender:setChapterState(unlock_chapter, common_pb.ECHAPTER_NO_TRIGGER)
                        end                        
                    end
                )

                return true                
            else
                log.error('Failed to find chapter cfg:'..unlock_chapter)
            end
        end
        return false
    end,

    -- 点了资源船触发按钮(状态 1 -> 状态 2)
    buttonTriggerResShip = function(self)
        for id,chapter_info in ipairs(self.chapters) do
            log.info('id:'..id..", chapter_info state:"..chapter_info.state)

            if chapter_info.state == common_pb.ECHAPTER_NO_OPEN then

            elseif chapter_info.state == common_pb.ECHAPTER_OVER then
                -- 章已完成
            elseif chapter_info.state == common_pb.ECHAPTER_NO_TRIGGER then
                -- 触发对话
                trigger_dialogs[id](trigger_dialogs, id)
                --network_sender:setChapterState(id, common_pb.ECHAPTER_TRIGGER)
                return true
            else
            end 
        end
        return false
    end,

    -- 点了资源船本体 (状态3)，开启关卡界面
    onClickResShip = function(self, id)
        local chapter_info = self.chapters[id]
        if not chapter_info then
            log.error('### Failed to find chapter info:'..id)
        end

        if chapter_info.state == common_pb.ECHAPTER_OCCUPY then
            frame_manager:show('level_select', nil, {chapter_id = id})
        end
    end,

    -- 根据 grid_id 获取 chapter id
    getChapterIdByGridId = function(self, grid_id)
        local chapter_id = nil
        config.game_copy_chapters:foreach(
            function(k,v)
                if v.grid_id == grid_id then
                    chapter_id = v.id
                end
            end
        )
        return chapter_id
    end,

    -- 侦察资源船
    spyResShip = function(self, grid_id)
        local chapter_id = self:getChapterIdByGridId(grid_id)
        network_sender:startChapterFight(chapter_id, nil, nil, true)
    end,

    -- 攻打资源船
    attackResShip = function(self, grid_id)
        local chapter_id = self:getChapterIdByGridId(grid_id)
        local chapter_cfg = config.game_copy_chapters[chapter_id]
        local monster_cfg_id = chapter_cfg.monster_config_id

        -- 显示派兵界面
        local hold = {}
        hold.id = 0
        hold.type = 0 --self.detail.dstPlace.objType
        hold.cfg_id = monster_cfg_id
        hold.player_name = ''
        hold.short_name = ''
        hold.coordX = 0
        hold.coordY = 0
        frame_manager:show("launch_rally", self.owner, {monster = hold, action = common_pb.EMARCH_ATK, chapter_id = chapter_id})

    end,
}

chapter_manager = chapter_manager or chapter_manager_cls()

return chapter_manager
