local log = logger.get("Game", "UnitTest")
local actor_props = require('RTS.prop_sys.actor_props')
local buff_handler = require('RTS.buff.buff_handler')

local atk_cfg = {
    ATK_INC = 10,
    DEF_INC = 7,
    MAX_HP_INC = 100,
    ATK = 100,
    DEF = 70,
    MAX_HP = 1000,
    ATK_FACTOR = 0.3,
    DEF_FACTOR = 0.3,
    MAX_HP_FACTOR = 0.2,
    DAMAGE_ADD = 100,
    DAMAGE_FACTOR = 0.2,
    BE_DAMAGED_REDUCE = 50,
    BE_DAMAGED_REDUCE_FACTOR = 0.2,
    TRUE_DAMAGE_ADD = 500,
    TRUE_DAMAGE_ADD_FACTOR = 0.2,
    ATK_SPEED = 1,
    ATK_SPEED_FACTOR = 0.2,
    ACCURACY = 100,
    DODGE = 100,
    CRIT = 200,
    CRIT_FACTOR = 0.5,
    ANTI_CRIT = 150,
}

local def_cfg = {
    ATK_INC = 10,
    DEF_INC = 7,
    MAX_HP_INC = 100,
    ATK = 100,
    DEF = 70,
    MAX_HP = 1000,
    ATK_FACTOR = 0.1,
    DEF_FACTOR = 0.4,
    MAX_HP_FACTOR = 0.5,
    DAMAGE_ADD = 200,
    DAMAGE_FACTOR = 0.1,
    BE_DAMAGED_REDUCE = 75,
    BE_DAMAGED_REDUCE_FACTOR = 0.1,
    TRUE_DAMAGE_ADD = 250,
    TRUE_DAMAGE_ADD_FACTOR = 1,
    ATK_SPEED = 2,
    ATK_SPEED_FACTOR = 0.3,
    ACCURACY = 100,
    DODGE = 100,
    CRIT = 200,
    CRIT_FACTOR = 0.5,
    ANTI_CRIT = 150,
}

local property_test = class
{
    typename = "Property Test.",
    
    __features = {
        debug.testable {
            run = function(self)
                --print("Property Test Run!!")
            end,

            failed = function(self)
                --print("Property Test Failed!!!")
            end,
        
            check = function(self)
                --assert(self:attackTest(), "普通攻击测试错误")
                -- assert(self:buffTest(), "buff 测试错误")
                --assert(self:test3(), "测试3错误")
            end,
        },
    },

    attackTest = function(self)
        --print("test1")
        local atk_level = 15
        local attacker = actor_props(atk_cfg, atk_level)
        attacker:computeStatic()
        attacker:computeFinal()

        local def_level = 10
        local defender = actor_props(def_cfg, def_level)
        defender:computeStatic()
        defender:computeFinal()

        local skill_cfg = {
            skill_factor = 1.0
        }

        attacker:getFinalGroup():log()
        defender:getFinalGroup():log()
        
        -- 取得伤害公式
        local calculator = require('RTS.prop_sys.damage_calculator')
        local name = 'base'

        for i=1, 20, 1 do 
            local dmg, hit, crit = calculator.calculate(calculator, {props = attacker}, {props = defender}, skill_cfg)
            log.info('dmg:'..dmg..", hit:"..tostring(hit)..', crit:'..tostring(crit))
        end

        return true
    end,

    -- 测试 buff
    buffTest = function(self)
        local atk_level = 11
        local actor_props = actor_props(atk_cfg, atk_level)       
        local actor = {props = actor_props, hp = 0, max = 0, }
        actor_props:computeStatic()
        actor_props:computeFinal()
        actor.buff_handler = buff_handler(actor)
        log:info('before')
        actor_props:log()
        --[ 1 ]={"test_激发肾上腺", "加攻 10 点,持续12秒","{[4] = 10}", 12.0, 0.0},
        --[ 2 ]={"test_治疗", "每2秒加10点HP,持续20秒","{[7] = 10}", 20.0, 2.0},
        --[ 3 ]={"test_怒吼", "每2秒加5点攻击,持续20秒","{[4] = 5}", 20.0, 2.0, true},

        actor.buff_handler:addBuff(actor, 1, 1)
        actor.buff_handler:addBuff(actor, 3, 2)

        log.info('buff:')
        actor_props:log()

        for i=1,12,1 do
            actor.buff_handler:update(1)
        end

        log.info('finished 01')
        actor_props:log()

        for i=1, 8, 1 do
            actor.buff_handler:update(1)
        end
        log.info('finished 02')
        actor_props:log()

        actor.buff_handler:update(1)
        log.info('finished 03')
        actor_props:log()
        
        return true
    end,

    test3 = function(self)
        --print("test3")
        return true
    end,
}

local state_machine_test = class
{
    typename = "State Machine Test.",
    
    __features = {
        debug.testable {
            run = function(self)
               --print("--- FSM Test!!---")
            end,

            failed = function(self)
                --log.error("---FSM Test Failed!!!---")
            end,
        
            check = function(self)
                local state1_class = class {
                    super = require('common/state'),
                }

                local state2_class = class {
                    super = require('common/state'),
                }
                
                local fsm_class = class{
                    super = require("common/state_machine"), 
                    init = function(self)
                        self.states['state1'] = state1_class.new('state1', self)
                        self.states['state2'] = state2_class.new('state2', self)
                        self:changeState('state1')
                    end,
                }

                local fsm = fsm_class.new()
                fsm:init()
                fsm:update()
                assert(fsm.curr_state ~= nil, "状态机初始化失败！！")
                assert(fsm.curr_state.name == 'state1', "状态1名称不正确！");
                fsm:changeState('state2');
                fsm:update()
                assert(fsm.curr_state.name == 'state2', "转换状态失败！");

            end,    -- check
        },
    },
}

--require('test.unit_test_bt')
require('test.unit_test_bag')




