local _log = logger.get("Game", "UnitTest")

local bt_test = class
{
    typename = "BT Test.",
    
    __features = {
        debug.testable {
            run = function(self)
            end,

            failed = function(self)
            end,

            check = function(self)
                self:sequenceTest()
                self:selectTest()
            end,    -- check


        },
    },  

    
    -- 顺序测试
    sequenceTest = function(self)

                local root = require('bt.root')
                local sequence = require("bt.sequence")
                local condition = require("bt.condition")
                local action = require("bt.action")
                local delay = require("bt.delay")

                local states = require("bt.node").states

                -- 条件结点，奇数次返回失败，偶数次返回成功
                local cond1_class = class {
                    super = condition,

                    ctor = function(self, ...)
                        condition.ctor(self, ...)
                        self.update_count = 0
                    end,

                    update = function(self, dt)
                        self.update_count = self.update_count + 1
                        if self.update_count % 2 == 0 then
                            return states.SUCCESS
                        else
                            return states.FAILURE
                        end
                    end,
                }
                
                -- 行为结点
                local action1_class = class{
                    super = action,

                    ctor = function(self, ...)
                        action.ctor(self, ...)
                        self.update_count = 0

                    end,
                    update = function(self, dt)
                        --_log.info('action update!!')
                        self.update_count = self.update_count + 1

                        return states.SUCCESS
                    end,
                }

                local cond1 = cond1_class() 
                local action1 = action1_class()
                local bt = root():setChild(
                    sequence()
                        :add(cond1)
                        :add(action1)
                )
                
                -- 检查 bt update 的结果
                local check_bt_tick = function(bt, tick, bt_rst, cond_update_count, action_update_count)
                    local rst = bt:update(0.1)
                    assert(rst == bt_rst, "tick"..tick.." 不正确")
                    assert(cond1.update_count == cond_update_count, "condition update count:"..tick.." 不正确")
                    assert(action1.update_count == action_update_count, "action update count:"..tick.." 不正确")
                end

                check_bt_tick(bt, 1, states.FAILURE, 1, 0)
                check_bt_tick(bt, 2, states.SUCCESS, 2, 1)
                check_bt_tick(bt, 3, states.FAILURE, 3, 1)
                check_bt_tick(bt, 4, states.SUCCESS, 4, 2)

    end,

    selectTest = function()
        local root = require('bt.root')
        local sequence = require("bt.sequence")
        local select = require("bt.select")
        local condition = require("bt.condition")
        local action = require("bt.action")
        local delay = require("bt.delay")

        local states = require("bt.node").states
                
        -- 行为结点, 第偶数次 执行成功
        local action1_class = class{
            super = action,

            ctor = function(self, ...)
                action.ctor(self, ...)
                self.update_count = 0

            end,

            update = function(self, dt)
                self.update_count = self.update_count + 1
                if self.update_count % 2 == 0 then
                    return states.SUCCESS
                else
                    return states.FAILURE
                end
            end,
         }

        -- 行为结点, 总是执行失败
        local action2_class = class{
            super = action,

            ctor = function(self, ...)
                action.ctor(self, ...)
                self.update_count = 0

            end,

            update = function(self, dt)
                self.update_count = self.update_count + 1
                return states.FAILURE
            end,
         }

        -- 行为结点, 总是执行成功
        local action3_class = class{
            super = action,

            ctor = function(self, ...)
                action.ctor(self, ...)
                self.update_count = 0

            end,

            update = function(self, dt)
                self.update_count = self.update_count + 1
                return states.SUCCESS
            end,
         }

        local action1 = action1_class()
        local action2 = action2_class()
        local action3 = action3_class()

        -- 构建行为树
        local bt = root():setChild(
            sequence()
                :add(select()
                    :add(action1)
                    :add(action2)
                    :add(action3)))

        -- 检查函数                        
        local check_bt_tick = function(bt, tick, rst, count1, count2, count3)
            local r = bt:update(0.1)
            assert(r == rst, "返回值不正确，Tick:"..tick)
            assert(action1.update_count == count1, "action1 count 不正确, Tick:"..tick)
            assert(action2.update_count == count2, "action2 count 不正确, Tick:"..tick)
            assert(action3.update_count == count3, "action3 count 不正确, Tick:"..tick)

        end

        check_bt_tick(bt, 1, states.RUNNING, 1, 0, 0)
        check_bt_tick(bt, 2, states.RUNNING, 1, 1, 0)
        check_bt_tick(bt, 3, states.SUCCESS, 1, 1, 1)
        check_bt_tick(bt, 4, states.SUCCESS, 2, 1, 1)
        check_bt_tick(bt, 5, states.RUNNING, 3, 1, 1)
        check_bt_tick(bt, 6, states.RUNNING, 3, 2, 1)
        check_bt_tick(bt, 7, states.SUCCESS, 3, 2, 2)
    end,
}
