--[[
    无法保证当前运行的updater脚本是哪一个版本的, 如果删除存储目录中的脚本包, 则会运行最老版本的updater.lua (来自StreamingAssets)
    为了避免混乱, 尽量保证updater.lua 不引用其他lua脚本, 否则需要在脚本包替换后, 重载已经载入的脚本
    甚至应该最差情况假定 updater 几乎等同于程序包一样, 随包更新, 而无法热更. 因为根本无法保证当前运行的 updater.lua 是哪一个版本.
]]
-- 这是第一个执行的 lua 文件
if jit then
    -- 关闭 jit, 使用 interpreter 模式
    jit.off();jit.flush()
end

local app_id = Fenix.App.GetInstance().appId
local getaddress_url = "http://139.196.142.89/getaddress.php?appid=" .. app_id

-- 如果 base_url == nil 那么会先请求 getaddress_url 获取 base_url 
local base_url = "http://space2017.oss-cn-shanghai.aliyuncs.com/v1/"
local updater_ui_path = "Assets/Arts/updater/updater.prefab"
local retry_on_failure = 3
local gui_root_path = "/gui/root/game"
local ReadableProgress = function (value)
    return string.format("%.2f%%", value * 100)
end
local ReadableBytes = function (size)
    if size < 1024 then 
        return size .. " B"
    elseif size < 1024 * 1024 then 
        return string.format("%.2f KB", size / (1024)) 
    elseif size < 1024 * 1024 * 1024 then 
        return string.format("%.2f MB", size / (1024 * 1024)) 
    end
    return string.format("%.2f GB", size / (1024 * 1024 * 1024)) 
end

local wrap_updater_ui = function ()
    local methods = {
        init = function (self, view, path)
            if view then 
                self.view = view

                local t2 = view:GetComponent(UnityEngine.RectTransform)
                local Vector2 = UnityEngine.Vector2
                t2.anchorMax = Vector2.one
                t2.anchorMin = Vector2.zero
                t2.offsetMax = Vector2.zero
                t2.offsetMin = Vector2.zero
                t2.pivot = Vector2.one * 0.5
                self.text_bytes = self.view.transform:FindComponent("Text Bytes", UnityEngine.UI.Text)
                self.text_files = self.view.transform:FindComponent("Text Files", UnityEngine.UI.Text)
                self.slider_obj = self.view.transform:FindComponent("Slider", UnityEngine.UI.Slider)
                if self.text_bytes then 
                    self.text_bytes.text = ""
                end
                if self.text_files then 
                    self.text_files.gameObject:SetActive(false) 
                end
            else
                UnityEngine.Debug.LogWarning("fail to load updater_ui: " .. path)
            end
        end, -- end function init

        setProgress = function (self, nSize, aSize)
            if self.slider_obj and self.text_bytes then 
                self.slider_obj.value = aSize ~= 0 and nSize / aSize or 1
                self.text_bytes.text = ReadableBytes(nSize) .. "/" .. ReadableBytes(aSize) .. "(" .. ReadableProgress(self.slider_obj.value) .. ")"
            end
        end, 

        setHint = function (self, text)
            if self.text_files then 
                self.text_files.text = text
                self.text_files.gameObject:SetActive(true)
            end
        end,

        setFile = function (self, fileName)
            self:setHint("Downloading: " .. (fileName or "?"))
        end,

        close = function (self)
            if self.view then 
                -- print("close", self.view)
                UnityEngine.Object.Destroy(self.view)
            end
        end,  
    }

    return methods
end

if Fenix.App.GetPackageManager().useAssetDatabase then -- 使用 Assets 下的项目资源
    print("use AssetDatabase, skip updater.")
    dofile("main.lua")
else    -- 使用 StreamingAssets 下的打包资源

    local app = Fenix.App.GetInstance()

    app:StartCoroutine(function ()
        print("run updater...")
        -- 加载更新时显示的界面  
        --[[              
        local updater_ui = wrap_updater_ui()
        coroutine.yield(Fenix.Assets.Helper.WaitForInstantiate(
            Fenix.App.GetFileSystem(),  -- IAssetProvider
            updater_ui_path,            -- asset path
            UnityEngine.GameObject.Find(gui_root_path).transform, -- parent
            function (gameObject)       -- callback
               -- print("load ui finished.")
                updater_ui:init(gameObject, updater_ui_path) 
            end)
        )]]

        local storagePath = Fenix.App.GetStoragePath()
        local verFilePath = storagePath .. Fenix.Assets.VersionData.VERSION_FILE
        local vd_bytes -- version data 字节流 
        local local_vd -- json 格式的 version data

        -- 读 version.json
        if UnityEngine.Application.isEditor then 
            -- 先读取 StreamingAssets 确定当前运行平台
            -- 读 version.json, 打包时生成的所有包信息，包依赖关系
            local platform
            coroutine.yield(Fenix.Assets.Helper.ReadStreamingAssets(Fenix.Assets.VersionData.VERSION_FILE, function (fileBytes)
                vd_bytes = fileBytes
                local_vd = Fenix.JsonUtils.FromBytes(vd_bytes, Fenix.Assets.VersionData)
                platform = local_vd.platform

            end))


            -- 如果 storage 目录下有 version data 并且平台一致，用 storage 目录下的版本
            -- 本地存在记录，并且平台符合 （编辑器中所有平台均在同一目录运行，所以需要这样判定）
            if Fenix.FileUtils.Exists(verFilePath) then 
               --print("updater kkk")
                local store_vd_bytes = Fenix.FileUtils.ReadAllBytes(verFilePath)
                local store_local_vd = Fenix.JsonUtils.FromBytes(vd_bytes, Fenix.Assets.VersionData)

                if store_local_vd ~= nil and store_local_vd.platform == platform then 
                    vd_bytes = store_vd_bytes
                    local_vd = store_local_vd
                end
            end
        else -- Not inEditor
            -- 先判断 storage 下有没有
            if Fenix.FileUtils.Exists(verFilePath) then 
                vd_bytes = Fenix.FileUtils.ReadAllBytes(verFilePath)
                local_vd = Fenix.JsonUtils.FromBytes(vd_bytes, Fenix.Assets.VersionData)
            end

            -- 没有就到 StreamingAssets 下读
            if vd_bytes == nil or local_vd == nil then 
                -- coroutine.yield(Fenix.Assets.Helper.CopyStreamingAssets(Fenix.Assets.VersionData.VERSION_FILE, nil))
                coroutine.yield(Fenix.Assets.Helper.ReadStreamingAssets(Fenix.Assets.VersionData.VERSION_FILE, function (fileBytes)
                    vd_bytes = fileBytes
                    local_vd = Fenix.JsonUtils.FromBytes(vd_bytes, Fenix.Assets.VersionData)
                end))
            end
        end
        -- print(local_vd...)
        coroutine.yield(nil)
        
        if app.standalone then -- 不更新
            print("run as standalone mode")
            --updater_ui:close()
            Fenix.App.GetPackageManager():SetVersionData(local_vd)
            dofile("main.lua")
            return
        end

        -- 获取 base_url
        if base_url == nil then 
            print("fetch update url...", getaddress_url)
            while true do 
                local req = Fenix.Async.HttpRequest(getaddress_url)
                coroutine.yield(req)
                if req.error == nil then 
                    base_url = req.text
                    if base_url ~= nil and base_url ~= "" then 
                        break
                    end
                end 
                updater_ui:setHint("network unreachable... retry...")
                coroutine.yield(UnityEngine.WaitForSeconds(retry_on_failure))
            end
        end

        -- 比较本地 version data  和 最新版本的 hash 值
        local local_hash = Fenix.Assets.Helper.ComputeHash(vd_bytes)
        local latest_hash 
        local platform_url = base_url .. Fenix.Assets.VersionData.PACKAGES_PATH .. local_vd.platform .. "/"
        
        print("update from", platform_url)
        while true do 
            local hash_url = platform_url .. Fenix.Assets.VersionData.HASHCODE_FILE
            local req = Fenix.Async.HttpRequest(hash_url)
            coroutine.yield(req)
            if req.error == nil then 
                latest_hash = req.text
                break
            end 
            UnityEngine.Debug.LogErrorFormat("failed version hash {0}", hash_url)
            updater_ui:setHint("network unreachable... retry...")
            coroutine.yield(UnityEngine.WaitForSeconds(retry_on_failure))
        end

        -- 下载最新的 version.json
        print("local hash:", local_hash, "latest hash:", latest_hash)
        if latest_hash ~= local_hash then  
            while true do 
                local vd_url = platform_url .. Fenix.Assets.VersionData.VERSION_FILE
                local req = Fenix.Async.HttpRequest(vd_url)
                coroutine.yield(req)
                if req.error == nil then 
                    -- 下载成功，解析
                    local req_bytes = req.bytes
                    local req_hash = Fenix.Assets.Helper.ComputeHash(req_bytes)
                    if req_hash == latest_hash then 
                        local_vd = Fenix.JsonUtils.FromBytes(req_bytes, Fenix.Assets.VersionData)
                        print("update hash", vd_url, local_hash, "=>", latest_hash, "req:", req_hash)
                        Fenix.FileUtils.WriteAllBytes(storagePath .. Fenix.Assets.VersionData.VERSION_FILE, req_bytes)
                        break
                    end
                end
                UnityEngine.Debug.LogError("failed version file " .. vd_url)
                UnityEngine.Debug.LogError(req.error)
                updater_ui:setHint("network unreachable... retry...")
                coroutine.yield(UnityEngine.WaitForSeconds(retry_on_failure))
            end
        end 

        Fenix.App.GetPackageManager():SetVersionData(local_vd)
        Fenix.App.GetPackageManager():SetDownloader(Fenix.Assets.SimpleDownloader(platform_url))
        print("begin download, generate filelist...")
        local download_task
        coroutine.yield(Fenix.Assets.Helper.GetDownloadTask(local_vd, function (task)
            download_task = task
        end))
        print("you have", download_task.count, "(", ReadableBytes(download_task.fullSize), ") packages to download")
        coroutine.yield(Fenix.Assets.Helper.DownloadStartup(download_task, function() 
            updater_ui:setProgress(download_task.loadSize, download_task.fullSize)
            -- print("loading:", download_task.loaded, "/", download_task.count, " bytes:",
            --     download_task.loadSize / 1024, "kb / ", download_task.fullSize / 1024, "kb")
        end))
        if download_task.isError then 
            updater_ui:setHint("download error!")
            coroutine.yield(UnityEngine.WaitForSeconds(99999999))
        end
        updater_ui:close()
        print("download finished, start main.")
        dofile("main.lua")
    end)
end
