local log = logger.get("Game", "Base Hold")
local frame_helper = require("gui.tool.frame_helper")
local hold_name_ui_path = "Assets/Prefab/WorldMap/HoldName.prefab"
local hold_capture_ui_path = "Assets/Prefab/WorldMap/HoldCaptureIcon.prefab"
local monster_name_ui_path = "Assets/Prefab/WorldMap/MonsterName.prefab"
local boss_name_ui_path = "Assets/Prefab/WorldMap/BossName.prefab"
local guild_base_name_ui_path = "Assets/Prefab/WorldMap/GuildBaseName.prefab"
local res_icon_ui_path = "Assets/Prefab/WorldMap/ResHoldIcon.prefab"
local hold_info_ui_path = "Assets/Prefab/WorldMap/HoldInfo.prefab"
local hold_operate_ui_path = "Assets/Prefab/WorldMap/HoldOperate.prefab"

return class {
    ctor = function(self, id, cfg_id, x, z)
        self.old_state = hold_states.normal
        self.state = hold_states.normal

        self.events:dispatch('set_hold_cells', id, x, z, self.cfg.gridnum or 1, self)
    end,

    refresh = function(self)
        self.events:dispatch("hold_refreshed", self)
    end,

    initUI = function(self, pos, root)
        --log.info("#### base hold initUI")
        
        self.transform = self.gameObject.transform
        self.transform.parent = root.transform
        self.transform.position = pos
        self.name_node = self.transform:Find("NameNode") or self.transform:GetChild(0):Find("NameNode")
        self.level_node = self.transform:Find("LevelNode") or self.transform:GetChild(0):Find("LevelNode")
        self.capture_node = self.transform:Find("CapturedNode") or self.transform:GetChild(0):Find("CapturedNode")
        self.operate_node = self.transform:Find("OperateNode") or self.transform:GetChild(0):Find("OperateNode")

        self.renderer = self.gameObject:GetComponentInChildren(UnityEngine.Renderer)

        self:setUIEnable(false)

        --self:enabledInfo(false)
    end,

    nameChanged = function(self, name)
        self.info.name = name
        self.name = name
        self.events:dispatch("hold_name_changed", self)
    end,

    getOperators = function(self)
        return {}
    end,
    
    addWorldMapActorMono = function(self, go, prefab_path)
        --frame_helper:setChildLayer(go.transform, 'Tank')
        local bc_mono = go:GetComponentInChildren(UnityEngine.BoxCollider)
        if bc_mono then
            local cs = bc_mono.gameObject:GetComponent(WorldMapActor)
            if not cs then
                cs = bc_mono.gameObject:AddComponent(WorldMapActor)
            end
            return cs
        else
            log.error("prefab not exist add BoxCollider Script !!! path:", prefab_path)
            return nil
        end
    end,

    _LoadPrefab = function(self, prefab_path, callback)
        local uobject = game_director.pool:instantiate(prefab_path)
        if not uobject then
            log.error('Failed to load prefab:'..prefab_path)
            return nil
        end

        if uobject.isLoaded then 
            callback(uobject)
        else 
            uobject.onLoaded = callback
        end

        return uobject
    end,

    setNameText = function(self)
        if self.name_node then
            self.name_node:SetActive(true)

            if self.type == hold_types.monster then
                self:setMonsterName()
            elseif self.type == hold_types.mine or self.type == hold_types.titanMine then
                self:setResName()
            elseif self.type == hold_types.player then
                self:setBaseName()
            elseif self.type == hold_types.guild then
                self:setGuildBaseName()
            end
        end
    end,

    setMonsterName = function(self)
        local cb = function(go, inited)
            if not inited then
                go.transform:SetParent(self.name_node)
                go:Reset()
            end

            local str = frame_helper:getLangCfg(self.cfg.monstername)
            local lv = self.cfg.level

            local name_trans = go.transform:Find("Name/Value") or go.transform:GetChild(0):Find("Name/Value")
            local lv_trans = go.transform:Find("Level/Value") or go.transform:GetChild(0):Find("Level/Value")
            local lv_text = lv_trans:GetComponent(UnityEngine.UI.Text)
            local name_text = name_trans:GetComponent(UnityEngine.UI.Text)

            name_text.text = str
            lv_text.text = lv

            --if self.cfg.attack < 2 then
                local name_bg = go.transform:Find("Name") or go.transform:GetChild(0):Find("Name")
                local lv_bg = go.transform:Find("Level") or go.transform:GetChild(0):Find("Level")
                local name_img = name_bg:GetComponent(UnityEngine.UI.Image)
                local lv_img = lv_bg:GetComponent(UnityEngine.UI.Image)

                --name_img:SetSprite(res_path.res_path_92)
                lv_img:SetSprite(gui_helper:monsterTypeIcon(self.cfg.monster_type))
            --end
        end

        if self.name_node.childCount > 0 then
            local name_prefab = self.name_node:GetChild(0)
            cb(name_prefab.gameObject, true)
        else
            --if self.cfg.attack < 2 then
                self:_LoadPrefab(monster_name_ui_path, cb)
            --else
            --    self:_LoadPrefab(boss_name_ui_path, cb)
            --end
        end
    end,

    setResName = function(self)
        local cb = function(go, inited)
            if not inited then
                go.transform:SetParent(self.name_node)
                go:Reset()
            end

            local lv = self.cfg.level or 1

            local name_trans = go.transform:Find("Name/Value") or go.transform:GetChild(0):Find("Name/Value")
            local lv_trans = go.transform:Find("Level/Value") or go.transform:GetChild(0):Find("Level/Value")
            local lv_text = lv_trans:GetComponent(UnityEngine.UI.Text)
            local name_text = name_trans:GetComponent(UnityEngine.UI.Text)

            name_text.text = self.name
            lv_text.text = lv

            local name_bg = go.transform:Find("Name") or go.transform:GetChild(0):Find("Name")
            local lv_bg = go.transform:Find("Level") or go.transform:GetChild(0):Find("Level")
            local name_img = name_bg:GetComponent(UnityEngine.UI.Image)
            local lv_img = lv_bg:GetComponent(UnityEngine.UI.Image)

            if self.type == hold_types.titanMine then
                local is_own = world_map_helper:isOwnHold(self)
                local is_friend = not is_own and world_map_helper:isOwnGuild(self)
                local is_neutral = world_map_helper:isNeutralHold(self)

                if is_own then
                    name_img:SetSprite(res_path.res_path_94)
                    lv_img:SetSprite(res_path.res_path_88)
                elseif is_friend then
                    name_img:SetSprite(res_path.res_path_93)
                    lv_img:SetSprite(res_path.res_path_89)
                elseif is_neutral then
                    name_img:SetSprite(res_path.res_path_91)
                    lv_img:SetSprite(res_path.res_path_90)
                else
                    name_img:SetSprite(res_path.res_path_92)
                    lv_img:SetSprite(res_path.res_path_87)
                end
            else
                name_img:SetSprite(res_path.res_path_91)
                lv_img:SetSprite(res_path.res_path_90)
            end
        end

        if self.name_node.childCount > 0 then
            local name_prefab = self.name_node:GetChild(0)
            cb(name_prefab.gameObject, true)
        else
            self:_LoadPrefab(monster_name_ui_path, cb)
        end

        if self.type == hold_types.titanMine then
            self:setCapture()
        end

        --self:setResIcon()
    end,

    setBaseName = function(self)
        local cb = function(go, inited)
            if not inited then
                go.transform:SetParent(self.name_node)
                go:Reset()
            end

            local str = self.info.playerName
            if self.info.familyId and string.len(self.info.familyId) > 1 then
                local short_name = self.info.familyShortName or self.info.shortName
                str = string.format( "[%s]%s", short_name, str )
            end
            
            local name_trans = go.transform:Find("Name/Value") or go.transform:GetChild(0):Find("Name/Value")
            local name_text = name_trans:GetComponent(UnityEngine.UI.Text)
            local lv_trans = go.transform:Find("Level/Value") or go.transform:GetChild(0):Find("Level/Value")
            local lv_text = lv_trans:GetComponent(UnityEngine.UI.Text)

            name_text.text = str
            lv_text.text = self.info.homeLevel

            local name_bg = go.transform:Find("Name") or go.transform:GetChild(0):Find("Name")
            local name_img = name_bg:GetComponent(UnityEngine.UI.Image)
            local lv_bg = go.transform:Find("Level") or go.transform:GetChild(0):Find("Level")
            local lv_img = lv_bg:GetComponent(UnityEngine.UI.Image)

            local is_own = world_map_helper:isOwnHold(self)
            local is_friend = not is_own and world_map_helper:isOwnGuild(self)

            if is_own then
                name_img:SetSprite(res_path.res_path_94)
                lv_img:SetSprite(res_path.res_path_88)
            elseif is_friend then
                name_img:SetSprite(res_path.res_path_93)
                lv_img:SetSprite(res_path.res_path_89)
            else
                name_img:SetSprite(res_path.res_path_92)
                lv_img:SetSprite(res_path.res_path_87)
            end

        end

        if self.name_node.childCount > 0 then
            local name_prefab = self.name_node:GetChild(0)
            cb(name_prefab.gameObject, true)
        else
            self:_LoadPrefab(hold_name_ui_path, cb)
        end
    end,

    setGuildBaseName = function(self)
        local cb = function(go, inited)
            if not inited then
                go.transform:SetParent(self.name_node)
                go:Reset()
            end

            local cfg_name = frame_helper:getLangCfg(self.name_id)
            local short_name = self.info.shortName
            local state_str = ""

            if self.info.state == 0 then
                state_str = NativeText.GetLangStr(9000085)
            elseif self.info.state == 1 then
                state_str = NativeText.GetLangStr(9000086)
            elseif self.info.state == 3 then
                state_str = NativeText.GetLangStr(9000087)
            end

            local str = string.format( "[%s]%s%s", short_name, cfg_name, state_str )
            
            local name_trans = go.transform:Find("Name/Value") or go.transform:GetChild(0):Find("Name/Value")
            local name_text = name_trans:GetComponent(UnityEngine.UI.Text)

            name_text.text = str

            local name_bg = go.transform:Find("Name") or go.transform:GetChild(0):Find("Name")
            local name_img = name_bg:GetComponent(UnityEngine.UI.Image)

            local is_own = world_map_helper:isOwnGuild(self)

            if is_own then
                name_img:SetSprite(res_path.res_path_93)
            else
                name_img:SetSprite(res_path.res_path_92)
            end
        end

        if self.name_node.childCount > 0 then
            local name_prefab = self.name_node:GetChild(0)
            cb(name_prefab.gameObject, true)
        else
            self:_LoadPrefab(guild_base_name_ui_path, cb)
        end
    end,

    setResIcon = function(self)
        if self.level_node then
            self.level_node:SetActive(true)

            local cb = function(go, inited)
                if not inited then
                    go.transform:SetParent(self.level_node)
                    go:Reset()
                end
    
                local icon = go.transform:Find("Level/Image") or go.transform:GetChild(0):Find("Level/Image")
                local img = icon:GetComponent(UnityEngine.UI.Image)

                img:SetSprite(gui_helper:getItemIconPath(self.main_res))
            end

            if self.level_node.childCount > 0 then
                local level_prefab = self.level_node:GetChild(0)
                cb(level_prefab.gameObject, true)
            else
                self:_LoadPrefab(res_icon_ui_path, cb)
            end
        end
    end,

    setCapture = function(self)
        if self.capture_node then
            self.capture_node:SetActive(true)

            local cb = function(go, inited)
                if not inited then
                    go.transform:SetParent(self.capture_node)
                    go:Reset()
                end
    
                go.transform:SetActive(true)

                local img = go.transform:Find("Icon"):GetComponent(UnityEngine.UI.Image)

                local is_own = world_map_helper:isOwnHold(self)
                local is_friend = not is_own and world_map_helper:isOwnGuild(self)
                local is_enemy = world_map_helper:isEnemyHold(self)

                if is_friend then
                    img:SetSprite(res_path.res_path_79)
                elseif is_enemy then
                    img:SetSprite(res_path.res_path_80)
                else
                    go.transform:SetActive(false)
                end
            end

            if self.capture_node.childCount > 0 then
                local capture_prefab = self.capture_node:GetChild(0)
                cb(capture_prefab.gameObject, true)
            else
                self:_LoadPrefab(hold_capture_ui_path, cb)
            end
        end
    end,

    setState = function(self, state)
        self.old_state = self.state
        self.state = state
    end,

    revertState = function(self)
        if self.left_time <= 0 then
            self.state = hold_states.normal
            self.old_state = hold_states.normal
        else
            self.state = self.old_state
        end
    end,

    tearDown = function(self)
        self.destroy_time = 10
        if self.cancel_btn then
            self.cancel_btn.transform:SetActive(true)
        end
        self:setState(hold_states.destroy)
    end,

    unTearDown = function(self)
        if self.time_ui then
            self.time_ui:SetActive(false)
        end
        self.destroy_time = 10
        self:revertState()
    end,

    isBuilding = function(self)
        return self.state == hold_states.building
    end,

    isLevelUp = function(self)
        return self.state == hold_states.levelup
    end,

    isRepair = function(self)
        return self.state == hold_states.repair
    end,

    isDestroy = function(self)
        return self.state == hold_states.destroy
    end,

    getHoldType = function(self)
        return self.type
    end,

    setQueryInfo = function(self, info)
        self.queryInfo = info
    end,

    refreshQueryInfo = function(self, info)
        if self.queryInfo then
            self.queryInfo.durable = info.curDur
            self.queryInfo.durableMax = info.maxDur
            self.queryInfo.maxTeams = info.maxTeam
            self.info.curTeams = info.curTeam
        end
    end,

    setUIEnable = function(self, enable)
        if self.name_node then
            self.name_node:SetActive(enable)
        end
        if self.level_node then
            self.level_node:SetActive(enable)
        end
        if self.capture_node then
            self.capture_node:SetActive(enable)
        end
    end,

    enabledInfo = function(self, enable)
        if self.info_ui and self.operate then
            self.info_ui:SetActive(enable)
            self.operate:SetActive(enable)
        end
    end,

    update = function(self, dt)
        if self.time_ui then
            local flag = false
            if world_map_helper:isGuildHold(self.type) then
                if not world_map_helper:isOwnGuild(self) then
                    flag = true
                end
            else
                if not self.info.playerId or self.info.playerId ~= local_player.id then
                    flag = true
                end
            end

            if flag then
                return
            end
            
            if self.left_time > 0 then
                self.left_time = self.left_time - dt
            end

            if self:isDestroy() then
                self.destroy_time = self.destroy_time - dt
                self:leftTimeUI(self.destroy_time, 10, NativeText.GetLangStr(280402))
            elseif self:isBuilding() then
                self:leftTimeUI(self.left_time, self.total_time, NativeText.GetLangStr(280403))
            elseif self:isLevelUp() then
                self:leftTimeUI(self.left_time, self.total_time, NativeText.GetLangStr(280404))
            elseif self:isRepair() then
                self:leftTimeUI(self.left_time, self.total_time, NativeText.GetLangStr(280405))
            end
        end
    end,

    leftTimeUI = function(self, left_time, total_time, str)
        if left_time <= 0 then
            if self:isDestroy() then
                network_sender:destroyHold(self.id)
            end

            self:setState(hold_states.normal)
            self.time_ui:SetActive(false)

            return
        end
        self.time_ui:SetActive(true)
        self.time_image.fillAmount = 1 - left_time / total_time
        self.time_label.text = string.format( "%s %s", str, gui_helper:timeFormat(math.ceil(left_time)) )
    end,

    getDetailCfg = function(self, cfg, id)
        local detail = {}
        cfg:foreach(
            function(k, v)
                if v.id == id then
                    detail[v.level] = v
                end
            end
        )
        return detail
    end,

    destroy = function(self)
        if self.gameObject then
            self.gameObject:destroy()
        end
        self.gameObject = nil
        self.transform = nil
        self.info_ui = nil
        self.operate = nil
        self.time_ui = nil
        self.name_node = nil
        self.level_node = nil
        self.capture_node = nil
        self.inited = false
        self.renderer = nil
    end,
}