local log = logger.get("Game", "Guild Hold")
local super = require('world_map.base_hold')

-- 工会据点
return class
{
    super = super,

    ctor = function(self, pos, info, events, root)
        self.info = info.familybaseinfo
        self.id = info.objinfo.instid
        self.coordX = info.objinfo.posx
        self.coordY = info.objinfo.posy
        self.left_time = tonumber(info.familybaseinfo.remaintime)
        self.guild_id = info.familybaseinfo.familyId
        self.family_name = info.familybaseinfo.familyName
        self.short_name = info.familybaseinfo.shortName
        self.events = events
        self.type = hold_types.guild
        self.root = root
        self.pos = pos

        self.cfg_id = info.objinfo.cfgid or 1
        self.cfg = config.union_union_buildings[self.cfg_id] or {}
        self.name_id = self.cfg.name_id
        self.icon_path = self.cfg.IconPath..self.cfg.Icon

        self.range = {}

        log.info("################################## left time ", self.left_time)

        super.ctor(self, self.id, self.cfg_id, self.coordX, self.coordY)
    end,

    refresh = function(self, info)
        self.info = info.familybaseinfo
        self.left_time = tonumber(info.familybaseinfo.remaintime)

        self:setNameText()
        super.refresh(self)
    end,

    setQueryInfo = function(self, info)
        self.info = info.familybaseinfo
        self.left_time = tonumber(info.familybaseinfo.remaintime)
    end,

    init = function(self, pos, root)
        local prefab_path = self:getPrefabPath()
        local uobject = game_director.pool:instantiate(prefab_path)
        self.inited = true

        local callback = function (go)
           -- callback
            if go ~= nil then
                if not game_director:isWorldMapState() then
                    go:SetActive(false)
                    return
                end
                self.gameObject = go
                self.gameObject.name = 'guild_hold_' .. self.id
                self:initUI(pos, root)
                self:setNameText()
                self.cs_inst = self:addWorldMapActorMono(go, prefab_path)
                if self.cs_inst then
                    self.cs_inst.id = tostring(self.id)
                    self.is_initialized = true
                    self.cs_inst:setType(hold_types.guild)
                    self.cs_inst:SetColor(Color(1,1,1,1))
                end
                --self.events:dispatch("guild_hold_loaded", self)
                self:setState(self.info.buildState)
            else
                log.error('loadResource go is nil')
            end
        end

        if uobject.isLoaded then 
            callback(uobject)
        else 
            uobject.onLoaded = callback
        end

    end,

    getPrefabPath = function(self)
        if self.info.state == 0 then
            return self.cfg.BuildUnfinishPrefab
        elseif self.info.state == 1 then
            return self.cfg.BuildUnfinishPrefab
        elseif self.info.state == 2 then
            return self.cfg.Prefab
        elseif self.info.state == 3 then
            return self.cfg.Prefab
        end

        return self.cfg.Prefab
    end,

    addRange = function(self, idx)
        self.range[idx] = true
    end,

    removeRange = function(self, idx)
        self.range[idx] = false
    end,

    destroy = function(self)
        super.destroy(self)

        self.range = {}
    end,
}