local log = logger.get("Game", "Hold Manager")
local player_hold = require('world_map.player_hold')
local mine_hold = require('world_map.mine_hold')
local march_line = require("world_map.march_line")
local monster_hold = require('world_map.monster_hold')
local guild_hold = require('world_map.guild_hold')
local titanium_hold = require('world_map.titanium_hold')

return class {
    ctor = function (self, events)
        self.block_holds = {}
        self.all_hold = {}
        self.own_hold = {}
        self.hold_list = {}
        self.events = events
        for k, v in pairs(hold_types) do
            self.all_hold[v] = {}
        end
        
        self.root = GameObject.Find('Root').transform

        self.counter = 1
    end,

    initBlockHolds = function(self, num)
        for i=1, num do
            self.block_holds[i] = {}
        end
    end,

    addBlockHold = function(self, idx, hold)
        if self.block_holds[idx] then
            self.block_holds[idx][hold.id] = hold
        end
    end,

    removeBlockHold = function(self, idx, id)
        if idx and self.block_holds[idx] then
            self.block_holds[idx][id] = nil
        end
    end,

    update = function(self, dt)
        local army_lines = self.all_hold[hold_types.armyLine]
        if army_lines then
            for _,v in pairs(army_lines) do
                v:update(dt)
            end
        end

        local factory = self.all_hold[hold_types.factory]
        if factory then
            for _,v in pairs(factory) do
                v:update(dt)
            end
        end

        local camp = self.all_hold[hold_types.camp]
        if camp then
            for _,v in pairs(camp) do
                v:update(dt)
            end
        end

        local guild = self.all_hold[hold_types.guild]
        if guild then
            for _,v in pairs(guild) do
                v:update(dt)
            end
        end

        local hold = self.all_hold[hold_types.player]
        if hold then
            for _,v in pairs(hold) do
                v:update(dt)
            end
        end

        local cnt = 0
        for i=self.counter, #self.hold_list do
            cnt = cnt + 1
            if cnt > 10 then
                break
            end

            self.counter = self.counter + 1
            if self.counter > #self.hold_list then
                self.counter = 1
            end

            local hold = self.hold_list[i]
            if hold.renderer then
                hold:setUIEnable(hold.renderer.isVisible)
            end
        end
    end,

    createHold = function(self, type, info, pos1, pos2)
        if type == hold_types.monster then
            self:createMonsterHold(pos1, info)
        elseif type == hold_types.guild then
            self:createGuildHold(pos1, info)
        elseif type == hold_types.player then
            self:createPlayerHold(pos1, info)
        elseif type == hold_types.titanMine then
            self:createTitanMine(pos1, info)
        elseif type == hold_types.armyLine then
            self:createArmyLine(pos1, pos2, info)
        else
            log.error("##### create hold error ! unknown type ", type)
        end
    end,

    refreshHold = function(self, type, id, info)
        local hold = self:getHold(type, id)
        if hold then
            hold:refresh(info)
        end
    end,

    removeHold = function(self, hold, block_idx)
        if hold then
            self:removeBlockHold(block_idx, hold.id)
            self:removeOwnHold(hold)
            self:removeHoldList(hold.id)
            self:clearHold(hold.type, hold.id)
            self.events:dispatch('releas_hold_cells', hold.coordX, hold.coordY, hold.cfg.gridnum or 1, hold)
            hold:destroy()
        else
            --log.error("##### remove hold error ! ".."  block idx "..tostring(block_idx))
        end
    end,

    addHoldList = function(self, hold)
        local add = true

        for k, v in ipairs(self.hold_list) do
            if v.id == hold.id then
                add = false
                break
            end
        end

        if add then
            table.insert( self.hold_list, hold )
        end
    end,

    removeHoldList = function(self, id)
        for k, v in ipairs(self.hold_list) do
            if v.id == id then
                table.remove( self.hold_list, k )
                break
            end
        end
    end,

    hideHold = function(self, type, id)
        local hold = self:getHold(type, id)
        if hold then
            hold:destroy()
            return true
        end
        return false
    end,

    storHold = function(self, type, hold)
        if self.all_hold[type] then
            self.all_hold[type][hold.id] = hold
            if not self:isOwnHold(hold) and hold.type ~= hold_types.marchLine then
                self.events:dispatch("CREATEHOLD", hold)
            end
        else
            log.error("##### set hold error ! type ", type)
        end

        self:addHoldList(hold)
    end,

    clearHold = function(self, type, id)
        if self.all_hold[type] then
            self.all_hold[type][id] = nil
        else
            log.error("##### clear hold error ! type ", type)
        end
    end,

    isOwnHold = function(self, hold)
        if hold.info and hold.info.playerId then
            if hold.info.playerId == local_player.id then
                return true
            end
        end
        return false
    end,
    
    storOwnHold = function(self, hold, add)
        if add or self:isOwnHold(hold) then
            hold:init(hold.pos, hold.root)
            table.insert( self.own_hold, hold )
        end
    end,

    removeOwnHold = function(self, hold)
        local idx = 0
        for k, v in ipairs(self.own_hold) do
            if v.id == hold.id then
                idx = k
                break
            end
        end

        if idx ~= 0 then
            table.remove( self.own_hold, idx )
        end
    end,

    getHold = function(self, type, id)
        if self.all_hold[type] then
            return self.all_hold[type][id]
        else
            log.error("##### get hold error ! type ", type)
            return nil
        end
    end,

    getHoldFromId = function(self, id)
        for k, v in pairs(self.all_hold) do
            for k1, hold in pairs(v) do
                if k1 == id then
                    return hold
                end
            end
        end

        return nil
    end,

    getHoldFromPos = function(self, x, y)
        for k, v in pairs(self.all_hold) do
            for k1, hold in pairs(v) do
                if hold.coordX == x and hold.coordY == y then
                    return hold
                end
            end
        end

        return nil
    end,

    -- 创建或者刷新玩家据点
    createPlayerHold = function(self, pos, hold_info)
        if pos ~= Vector3.zero then
            log.info("############################################## create player hold")
            local hold = self:getHold(hold_types.player, hold_info.objinfo.instid)
            if hold then
                self:guildChanged(hold, hold_info)
                hold:refresh(hold_info)
            else
                hold = player_hold(pos, hold_info, self.events, self.root)
                self:storHold(hold_types.player, hold)
                self:storOwnHold(hold)
            end

            if not hold.inited then
                hold:init(hold.pos, hold.root)
            end
        else
            log.error("wrong pos to create player hold!! :"..hold_info.posX..','..hold_info.posY)
        end
    end,

    -- 创建或刷新行军路线
    createArmyLine = function(self, src, dest, march_info)
        local percent = march_info.curPos * 0.001
        if percent > 1 then percent = 1 end

        log.info('########################### march_info curPos:'..march_info.curPos..', marchSpeed:'..march_info.speed..', percent:'..percent
        ..', total time:'..march_info.totalTime..', elapsed time:'..march_info.elapsedTime)


        if src == Vector3.zero or dest == Vector3.zero then
            log.error("wrong pos to create march line!! :"..
                march_info.srcPosX..','..march_info.srcPosY..','..march_info.dstPosX..','..march_info.dstPosY)
        else

            local line = self:getHold(hold_types.armyLine, march_info.instId)
            if line then
                line:refresh(src, dest, percent, march_info.speed, march_info)
            else
                line = march_line(march_info, src, dest, percent, self.events, self.root)
                self:storHold(hold_types.armyLine, line)
            end

            if not line.inited then
                local cur_pos = Vector3.Lerp(src, dest, percent)
                line:init(cur_pos, self.root)
            end
        end
    end,

    -- 创建怪物据点
    createMonsterHold = function(self, pos, monster_info)
        if pos ~= Vector3.zero then

            local hold = self:getHold(hold_types.monster, monster_info.objinfo.instid)
            if hold then
                -- 刷新 monster
            else
                hold = monster_hold(pos, monster_info, self.events, self.root)
                self:storHold(hold_types.monster, hold)
            end

            if not hold.inited then
                hold:init(hold.pos, hold.root)
            end
        else
            log.error("wrong pos to create mine hold!! :"..monster_info.objinfo.posx..','..monster_info.objinfo.posy)
        end
    end,

    -- 创建工会建筑
    createGuildHold = function(self, pos, guild_info)
        if pos ~= Vector3.zero then
            local hold = self:getHold(hold_types.guild, guild_info.objinfo.instid)
            if hold then
                self:guildChanged(hold, guild_info.familybaseinfo)
                hold:refresh(guild_info)
            else
                hold = guild_hold(pos, guild_info, self.events, self.root)
                self:storHold(hold_types.guild, hold)
            end

            if not hold.inited then
                hold:init(hold.pos, hold.root)
            end

        else
            log.error("wrong pos to create guild hold!! :"..guild_info.objinfo.posx..','..guild_info.objinfo.posy)
        end
    end,

    -- 创建钛矿
    createTitanMine = function(self, pos, info)
        if pos ~= Vector3.zero then
            local hold = self:getHold(hold_types.titanMine, info.objinfo.instid)
            if hold then
                hold:refresh(info)
            else
                hold = titanium_hold(pos, info, self.events, self.root)
                self:storHold(hold_types.titanMine, hold)
            end

            if not hold.inited then
                hold:init(hold.pos, hold.root)
            end
        else
            log.error("wrong pos to create camp hold!! :"..pos.x..','..pos.y)
        end
    end,

    findNeighborGuild = function(self, hold, is_late)
        local list = {}

        for k, v in pairs(self.all_hold[hold_types.guild]) do
            local dis = world_map_helper:getCoordDistance({x = hold.coordX, z = hold.coordY}, {x = v.coordX, z = v.coordY})

            log.info("######################################## guild hold distance ", dis)
            if dis < 9 then

                if is_late and v.info.finishtime > hold.info.finishtime then
                    table.insert( list, v )
                else
                    table.insert( list, v )
                end
            end
        end

        return list
    end,

    guildChanged = function(self, hold, info)
        if hold.info.familyId ~= info.familyId then
            for k, v in pairs(self.all_hold[hold_types.player]) do
                v:setNameText()
            end

            for k, v in pairs(self.all_hold[hold_types.guild]) do
                v:setNameText()
                world_map_helper:drawGuildBaseRegions(v)
            end
        end
    end,

    clear = function(self)
        for k, v in pairs(self.all_hold) do
            for _, hold in pairs(v) do
                self:removeHold(hold)
            end
        end
    end,

    destroy = function(self)
        for k, v in ipairs(self.own_hold) do
            v = nil
        end

        for k, v in pairs(self.block_holds) do
            for _, hold in pairs(v) do
                hold = nil
            end
        end

        for k, v in pairs(self.all_hold) do
            for _, hold in pairs(v) do
                hold:destroy()
                hold = nil
            end
        end
    end,

}