local log = logger.get("Game", "Build Opreator")
opreator_type = require("world_map.world_map_enums").opreator_type

return class{

    ctor = function(self, owner)
        self.owner = owner
        self.buildTypes = config.worldmap_operatelists

        self.functions = {
            [1] = function(hold) self:collected(hold) end,
            [2] = function(hold) self:showInfo(hold) end,
            [3] = function(hold) self:build(hold) end,
            [4] = function(hold) self:unbuild(hold) end,
            [5] = function(hold) self:adjustTeam(hold) end,
            [6] = function(hold) self:repair(hold) end,
            [7] = function(hold) self:transport(hold) end,
            [8] = function(hold) self:levelup(hold) end,
            [9] = function(hold) self:tearDown(hold) end,
            [10] = function(hold) self:untearDown(hold) end,
            [11] = function(hold) self:support(hold) end,
            [12] = function(hold) self:gather(hold) end,
            [13] = function(hold) self:detect(hold) end,
            [14] = function(hold) self:plunder(hold) end,
            [15] = function(hold) self:attack(hold) end,
            [16] = function(hold) self:mass(hold) end,
            [17] = function(hold) self:formation(hold) end,
            [18] = function(hold) self:goHome(hold) end,
            [19] = function(hold) self:speedUp(hold) end,
            [20] = function(hold) self:goToBase(hold) end,
            [21] = function(hold) self:moveHold(hold) end,
            [22] = function(hold) self:contribute(hold) end,
            [23] = function(hold) self:transportHeros(hold) end,
            [24] = function(hold) self:seize(hold) end,
            [25] = function(hold) self:leviathanMove(hold) end,
            [26] = function(hold) self:leviathanJump(hold) end,
            [27] = function(hold) self:share(hold) end,
            [28] = function(hold) self:randJump(hold) end,
            [29] = function(hold) self:viewGuildBaseInfo(hold) end,
            [30] = function(hold) self:backToGuildBase(hold) end,
        }

        self:addFunc()
    end,

    addFunc = function(self)
        self.opreators = {}
        config.worldmap_operates:foreach(
            function(k, v)
                v.cb = self.functions[k]
                self.opreators[k] = v
            end
        )
    end,

    refresh = function(self)
        self:setUI(self.obj, self.build_type, self.hold)
    end,

    setUI = function(self, obj, build_type, hold)
        self.obj = obj
        self.build_type = build_type
        self.hold = hold

        self.is_show = true

        obj.transform:SetAsLastSibling()

        self.layout = obj.transform:Find("Buttons") or obj.transform:GetChild(0):Find("Buttons")
        local des = obj.transform:Find("Info/List") or obj.transform:GetChild(0):Find("Info/List")
        local tip = obj.transform:Find("Info/Tip") or obj.transform:GetChild(0):Find("Info/Tip")
        local bg_trans = obj.transform:Find("Info/List") or go.transform:GetChild(0):Find("Info/List")
        local bg_img = bg_trans:GetComponent(UnityEngine.UI.Image)

        if hold then
            local is_own = world_map_helper:isOwnHold(hold)
            local is_friend = not is_own and world_map_helper:isOwnGuild(hold)
            local is_neutral = world_map_helper:isNeutralHold(hold)

            if is_own then
                bg_img:SetSprite(res_path.res_path_94)
            elseif is_friend then
                bg_img:SetSprite(res_path.res_path_93)
            elseif is_neutral then
                bg_img:SetSprite(res_path.res_path_91)
            else
                bg_img:SetSprite(res_path.res_path_92)
            end

            tip:SetActive(hold.type == hold_types.titanMine and hold.cfg.type ~= 2)

            local btn = tip:GetComponent(UnityEngine.UI.Button)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(
                function()
                    self:showTips(hold)
                end
            )
        else
            bg_img:SetSprite(res_path.res_path_91)
            tip:SetActive(false)
        end

        if des then
            for i=0, des.childCount - 1 do
                local child = des:GetChild(i)
                child:SetActive(false)
            end

            self:setText(hold, des)
        end

        if self.layout then
            self:addListener(build_type, hold)
        end
    end,

    setText = function(self, hold, des)
        if hold then
            if hold.type == hold_types.guild then
                self:setGuildHoldInfo(hold, des)
            elseif hold.type == hold_types.titanMine then
                self:setTitanMineInfo(hold, des)
            elseif hold.type == hold_types.player then
                self:setPlayerBaseInfo(hold, des)
            elseif hold.type == hold_types.armyLine then
                self:setArmyLineInfo(hold, des)
            end
        else
            local str = world_map_helper:getEmptyName(self.owner.click_cell.x, self.owner.click_cell.z)
            self:setHoldPos(str, self.owner.click_cell.x, self.owner.click_cell.z, des)
        end

        app:startCoroutine(
            function()
                coroutine.yield(nil)
                local bg = des:GetParent()
                bg.sizeDelta = Vector2(des.rect.width, des.rect.height)
            end
        )
    end,

    setGuildHoldInfo = function(self, hold, des)
        local pos_str = gui_helper:positionFormat(hold.coordX, hold.coordY)
        local name = frame_helper:getLangCfg(hold.name_id)

        local des_1 = des:GetChild(0):GetComponent(UnityEngine.UI.Text)
        local des_2 = des:GetChild(1):GetComponent(UnityEngine.UI.Text)

        des_1.text = string.format("[%s]", pos_str)
        des_2.text = NativeText.GetLangStr(280097)..hold.info.familyName

        des_1.transform:SetActive(true)
        des_2.transform:SetActive(true)
    end,

    setHoldPos = function(self, str, x, y, des)
        local pos_str = gui_helper:positionFormat(x, y)

        local des_1 = des:GetChild(0):GetComponent(UnityEngine.UI.Text)
        des_1.text = string.format("%s %s", str, pos_str)
        des_1.transform:SetActive(true)
    end,

    setMineInfo = function(self, hold, des)
        local name = frame_helper:getLangCfg(hold.cfg.nameid)
        local res_name = frame_helper:getLangCfg(hold.main_res.name_id)
        local left_count = hold.queryInfo.resAmount
        local pos_str = gui_helper:positionFormat(hold.coordX, hold.coordY)

        local des_1 = des:GetChild(0):GetComponent(UnityEngine.UI.Text)
        local des_2 = des:GetChild(1):GetComponent(UnityEngine.UI.Text)
        local des_3 = des:GetChild(2):GetComponent(UnityEngine.UI.Text)

        des_1.text = string.format("[%s]", pos_str)
        des_2.text = NativeText.GetLangStr(280099).." "..left_count

        des_1.transform:SetActive(true)
        des_2.transform:SetActive(true)
    end,

    setArmyLineInfo = function(self, hold, des)
        local player_name = local_player.name

        if local_guild:isSelfInGuild() then
            player_name = string.format( "[%s]%s", local_guild.info.familycronym, player_name ) 
        end

        local pos_str = gui_helper:positionFormat(hold.info.dstPosX, hold.info.dstPosY)

        local des_1 = des:GetChild(0):GetComponent(UnityEngine.UI.Text)
        local des_2 = des:GetChild(1):GetComponent(UnityEngine.UI.Text)
        local des_3 = des:GetChild(2):GetComponent(UnityEngine.UI.Text)

        des_1.text = player_name
        des_2.text = string.format("%s %s", NativeText.GetLangStr(9000089), pos_str)
        
        des_1.transform:SetActive(true)
        des_2.transform:SetActive(true)
        des_3.transform:SetActive(true)

        self.army_coroutione_id = app:startCoroutine(
            function()
                local lt = (hold.info.totalTime - hold.info.elapsedTime)*0.001
                while lt > 0 and self.is_show do
                    des_3.text = gui_helper:timeFormat(lt)
                    coroutine.yield(WaitForSeconds(1))
                    lt = lt - 1
                end
            end
        )
    end,

    setPlayerBaseInfo = function(self, hold, des)
        local name = frame_helper:getLangCfg(hold.name_id)
        local player_name = hold.info.playerName
        local guild_name = hold.info.familyShortName

        local x, z = world_map_helper:getBaseCurrPos()
        local pos_str = gui_helper:positionFormat(x, z)

        local des_1 = des:GetChild(0):GetComponent(UnityEngine.UI.Text)
        local des_2 = des:GetChild(1):GetComponent(UnityEngine.UI.Text)
        local des_3 = des:GetChild(2):GetComponent(UnityEngine.UI.Text)

        if guild_name and string.len(guild_name) > 2 then
            des_1.text = string.format("[%s]%s", guild_name, player_name)
        else
            des_1.text = player_name
        end

        if hold.info.playerId == local_player.id then
            des_2.text = string.format("[%s]", pos_str)
            des_3.text = string.format( "%s <color=#0BFF78FF>%s</color>", NativeText.GetLangStr(9000090), frame_helper:itemNumFormat(hold.info.strength) )
        else
            des_2.text = string.format( "%s <color=#0BFF78FF>%s</color>", NativeText.GetLangStr(9000090), frame_helper:itemNumFormat(hold.info.strength) )
        end
        
        des_1.transform:SetActive(true)
        des_2.transform:SetActive(true)
        des_3.transform:SetActive(hold.info.playerId == local_player.id)
    end,

    setTitanMineInfo = function(self, hold, des)
        local name = frame_helper:getLangCfg(hold.name_id)
        local res_amount = hold.info.resAmmount
        local player_name = hold.info.playerName
        local short_name = hold.info.familyShortName
        local pos_str = gui_helper:positionFormat(hold.coordX, hold.coordY)

        local des_1 = des:GetChild(0):GetComponent(UnityEngine.UI.Text)
        local des_2 = des:GetChild(1):GetComponent(UnityEngine.UI.Text)
        local des_3 = des:GetChild(2):GetComponent(UnityEngine.UI.Text)

        des_1.text = string.format("[%s]", pos_str)
        des_2.text = string.format("%s <color=#0BFF78FF>%d</color>", NativeText.GetLangStr(9000091), res_amount)
        
        if string.len(hold.info.playerId) > 1 then
            local color = "#0BFF78FF"

            if world_map_helper:isEnemyHold(hold) then
                color = "#F84949FF"
            end

            if string.len(hold.info.familyShortName) > 2 then
                des_3.text = string.format("%s <color=%s>[%s]%s</color>", NativeText.GetLangStr(9000092), color, short_name, player_name)
            else
                des_3.text = string.format("%s <color=%s>%s</color>", NativeText.GetLangStr(9000092), color, player_name)
            end
        end

        des_1.transform:SetActive(true)
        des_2.transform:SetActive(true)
        des_3.transform:SetActive(string.len(hold.info.playerId) > 1)

        self.gather_coroutione_id = app:startCoroutine(
            function()
                local dt = 0
                while hold.info.remainTime > 0 and self.is_show do
                    local res_amount = hold.info.resAmmount - dt*hold.info.gatherRate / 60
                    des_2.text = string.format("%s <color=#0BFF78FF>%d</color>", NativeText.GetLangStr(9000091), res_amount)
                    coroutine.yield(WaitForSeconds(1))
                    dt = dt + 1
                end
            end
        )
    end,
    
    addListener = function(self, build_type, hold)
        local cfg = self.buildTypes[build_type]

        if not cfg then
            log.error("##### operate list error ! id", build_type)
            return
        end

        local operatelist = loadstring("return "..cfg.operatelist)()
        local count = #operatelist
        local btns = {}

        for i=1, count do
            if i <= self.layout.childCount then
                local index = operatelist[i]
                if self.opreators[index] then
                    local btn = self.layout:GetChild(i-1):GetComponent(UnityEngine.UI.Button)
                    local label = btn.transform:GetChild(0):GetComponent(UnityEngine.UI.Text)
                    btn.transform:SetActive(true)
                    btn.onClick:RemoveAllListeners()

                    if index == 23 and not self:hasCamp() then
                        btn.transform:SetActive(false)
                    else
                        local img = btn.transform:GetComponent(UnityEngine.UI.Image)
                        img:SetSprite(self.opreators[index].path..self.opreators[index].name)

                        label.text = frame_helper:getLangCfg(self.opreators[index].operate)

                        btn.onClick:AddListener(functor.rmake(self.opreators[index].cb, hold))
                        table.insert(btns, btn)
                    end

                    if index == 1 then
                        if self:collectePosCheck() then
                            label.text = NativeText.GetLangStr(280102)
                        end
                    elseif index == 25 then
                        local cnt = world_map_helper:getFuelExpend(self.owner.click_cell)
                        label.text = string.format( "%s\n(%s%d)", label.text, NativeText.GetLangStr(9000093), cnt)
                    elseif index == 26 then
                        local cnt = world_map_helper:getTransitionCost(self.owner.click_cell)
                        label.text = string.format( "%s\n(%s%d)", label.text, NativeText.GetLangStr(9000094), cnt)
                    end
                end
            end
        end

        for i=count, self.layout.childCount-1 do
            local btn = self.layout:GetChild(i)
            btn:SetActive(false)
        end

        self:setButtonPos()
    end,

    hide = function(self)
        self.is_show = false

        if self.obj and self.obj.transform then
            self.obj.transform:SetActive(false)
        end

        if self.army_coroutione_id then
            app:stopCoroutine(self.army_coroutione_id)
            self.army_coroutione_id = nil
        end

        if self.gather_coroutione_id then
            app:stopCoroutine(self.gather_coroutione_id)
            self.gather_coroutione_id = nil
        end

        self.owner:clickOther()
    end,

    hasCamp = function(self)
        return own_hold_info:haveCamp()
    end,

    collectePosCheck = function(self)
        local x = self.owner.click_cell.x
        local y = self.owner.click_cell.z
        local collected = self.owner.owner:collectePosCheck(x, y)
        return collected
    end,

    collected = function(self, hold)
        log.info("##### collected")
        local collected = self:collectePosCheck()

        if collected then
            network_sender:worldMapCollectionDel(collected.id)
        else
            local str = ""

            if hold then
                if hold.type == hold_types.player then
                    str = hold.info.playerName
                elseif hold.type == hold_types.titanMine then
                    str = frame_helper:getLangCfg(hold.name_id)
                elseif hold.type == hold_types.monster then
                    str = world_map_helper:getEmptyName(hold.coordX, hold.coordY)
                end
            else
                str = world_map_helper:getEmptyName(self.owner.click_cell.x, self.owner.click_cell.z)
            end

            local collection_pos = {}
            collection_pos.eTable = 0
            collection_pos.strDesc = str
            collection_pos.posX = self.owner.click_cell.x
            collection_pos.posY = self.owner.click_cell.z
            collection_pos.iconId = 1
            world_map_info.ui:showEditPos(collection_pos)
        end
        self:hide()
    end,

    showInfo = function(self, hold)
        if hold then
            if hold.type == hold_types.titanMine then
                
            end
        end
        self:hide()
    end,

    build = function(self, hold)
        frame_manager:show("guild_build_dispatch", nil, {info = hold})
        self:hide()
    end,

    unbuild = function(self, hold)
        log.info("##### unbuild")
        cb = function()
            network_sender:destroyHold(hold.id)
            self:hide()
        end
        
        self:holdTip(cb, NativeText.GetLangStr(280104))
        self:hide()
    end,

    adjustTeam = function(self, hold)
        log.info("##### adjustTeam")
        self:hide()
    end,

    repair = function(self, hold)
        log.info("##### repair")
        frame_manager:show("guild_build_dispatch", nil, {info = hold})
        self:hide()
    end,

    transport = function(self, hold)
        log.info("##### transport")
        frame_manager:show("transport_res", self.owner, {hold = hold})
        self:hide()
    end,

    levelup = function(self, hold)
        log.info("##### levelup")
        self:hide()
    end,

    tearDown = function(self, hold)
        log.info("##### tearDown")
        hold:tearDown()
        self:hide()
    end,

    untearDown = function(self, hold)
        log.info("##### untearDown")
        hold:unTearDown()
        self:hide()
    end,

    support = function(self, hold)
        log.info("##### support")
        self:hide()
    end,

    gather = function(self, hold)
        log.info("##### gather")
        local soldier_cnt = home_info:getAvailableSoldiers()

        if #soldier_cnt == 0 then
            hint:setText(NativeText.GetLangStr(900012))
            return
        end

        frame_manager:show("launch_rally", nil, {monster = hold, action = common_pb.EMARCH_GATHER_TITAN})
        self:hide()
    end,

    detect = function(self, hold)
        log.info("##### detect")
        local pos_x = hold.coordX
        local pos_y = hold.coordY

        if hold.type == hold_types.player then
            pos_x, pos_y = hold:getCurrPos()
        end

        network_sender:startArmyLine(common_pb.EMARCH_SPY, nil, nil, hold.id, nil, nil, pos_x, pos_y)
        self:hide()
    end,

    plunder = function(self, hold)
        log.info("##### plunder")
        local soldier_cnt = home_info:getAvailableSoldiers()

        if #soldier_cnt == 0 then
            hint:setText(NativeText.GetLangStr(900012))
            return
        end

        frame_manager:show("launch_rally", nil, {monster = hold, action = common_pb.EMARCH_ROB_TITAN})
        self:hide()
    end,

    attack = function(self, hold)
        log.info("##### attack")
        local soldier_cnt = home_info:getAvailableSoldiers()

        if #soldier_cnt == 0 then
            hint:setText(NativeText.GetLangStr(900012))
            return
        end

        frame_manager:show("launch_rally", nil, {monster = hold, action = common_pb.EMARCH_ATK, spy = true})
        self:hide()
    end,

    mass = function(self, hold)
        log.info("##### mass")

        if not local_guild:isSelfInGuild() then
            hint:setText(NativeText.GetLangStr(289001))
            return
        end
        
        frame_manager:show("create_mass", self.owner, {hold = hold})
        self:hide()
    end,

    formation = function(self, hold)
        log.info("##### formation")
        self:hide()
    end,

    goHome = function(self, hold)
        log.info("##### goHome")
        if hold.type == hold_types.guild then
            local armyline_id = nil

            for k, v in ipairs(hold.info.teams) do
                if v.playerId == local_player.id then
                    armyline_id = v.armylineId
                end
            end

            if armyline_id then
                network_sender:worldMapGoHome(3, armyline_id)
            else
                log.error("################################## guild team go home error !")
            end
        elseif hold.type == hold_types.titanMine then
            local cb = function()
                network_sender:worldMapGoHome(3, hold.info.armylineId)
                self:hide()
            end
    
            gui_helper:showCommonTipBox(cb, NativeText.GetLangStr(9000095))
        elseif hold.type == hold_types.armyLine then
            local cb = function()
                network_sender:worldMapGoHome(3, hold.id)
            end
    
            gui_helper:showCommonTipBox(cb, NativeText.GetLangStr(9000096))
        end
        self:hide()
    end,

    speedUp = function(self, hold)
        local army_id = hold.id
        local left_time = (hold.info.totalTime - hold.info.elapsedTime) * 0.001
        local total_time = hold.info.totalTime * 0.001
        local action = hold.info.action

        army_queue_manager:armyLineSpeedUp(army_id, left_time, total_time, action)

        self:hide()
    end,

    goToBase = function(self, hold)
        log.info("##### goToBase")
        game_director:changeState('loading', {scene_name = 'home', next_state = 'home'})
        self:hide()
    end,

    moveHold = function(self, hold)
        log.info("##### moveHold")
        self:hide()
    end,

    contribute = function(self, hold)
        log.info("##### contribute")
        self:hide()
    end,

    transportHeros = function(self, hold)
        self:hide()
    end,

    seize = function(self, hold)
        self:hide()
    end,

    leviathanMove = function(self, hold)
        local cnt = world_map_helper:getFuelExpend(self.owner.click_cell)
        
        if cnt > local_player:getFuel() then
            hint:setText(NativeText.GetLangStr(9000097))
            return
        end

        network_sender:leviathanMove(local_player.id, self.owner.click_cell.x, self.owner.click_cell.z)
        self:hide()
    end,

    leviathanJump = function(self, hold)
        local cb = function()
            network_sender:baseStartTransition(self.owner.click_cell.x, self.owner.click_cell.z)
        end

        local cnt = world_map_helper:getTransitionCost(self.owner.click_cell)
        gui_helper:showCommonBox3(NativeText.GetLangStr(9000098), NativeText.GetLangStr(9000099), 30001, cnt, cb)

        --[[ local cnt = world_map_helper:getTransitionCost(self.owner.click_cell)
        
        if cnt > local_player:getGem() then
            hint:setText("钻石不足")
            return
        end ]]

        self:hide()
    end,

    share = function(self, hold)

    end,

    randJump = function(self, hold)
        local cb = function()
            network_sender:baseRandTransition()
        end

        gui_helper:showCommonBox3(NativeText.GetLangStr(9000100), NativeText.GetLangStr(9000101), 65017, 1, cb)
        self:hide()
    end,

    viewGuildBaseInfo = function(self, hold)
        frame_manager:show("guild_base_info", nil, {hold = hold})
        self:hide()
    end,

    backToGuildBase = function(self, hold)
        local cb = function()
            network_sender:backToFamilyFortress(hold.id)
        end

        gui_helper:showCommonBox3(NativeText.GetLangStr(9000102), NativeText.GetLangStr(9000103), 65015, 1, cb)
        self:hide()
    end,

    onGetOwnHold = function(self, data)
    end,

    setButtonPos = function(self)
        local circle_layout = self.layout:GetComponent(CircleLayout)
        circle_layout:Show()
    end,
    
    holdTip = function(self, cb, content)
        local tb = {
            title = "",
            content = content,
            buttons =
            {
                {
                    btn_name = NativeText.GetLangStr(110522),
                    callback = function() 
                        frame_manager:hide('common_box')
                    end
                },
                {
                    btn_name = NativeText.GetLangStr(110521),
                    callback = function()
                        cb()
                        frame_manager:hide('common_box')
                    end
                },
            },
        }

        frame_manager:show('common_box', nil, tb)
    end,

    showTips = function(self, hold)
        if hold.type == hold_types.titanMine and hold.cfg.type ~= 2 then
            local content = frame_helper:getLangCfg(hold.cfg.GatherAwardDes)
            content = content:gsub("\\n", "\n")

            local tb = {
                title = NativeText.GetLangStr(9000104),
                content = content,
            }
    
            frame_manager:show('common_box_2', nil, tb)
        end
    end,
}