local log = logger.get("Game", "March Line")

-- 玩家据点
return class
{
    ctor = function(self, march_line_info, src_pos, dest_pos, percent, events, root)
        self.info = march_line_info
        self.id = march_line_info.instId
        --log.info('#### march_line_info ctro id:'..tostring(self.id))
        self.player_id = march_line_info.playerId
        self.achieved = false   -- 是否到达目标
        self.events = events
        self.is_back_home = false
        self.type = hold_types.armyLine
        self.action = march_line_info.action
        self.before_action = march_line_info.pre_action
        self.cfg = {}
        self.coordX = 0
        self.coordY = 0

        self.src_pos = src_pos  -- 世界坐标位置
        self.dest_pos = dest_pos
        self.fixed_src = self.src_pos
        self.fixed_dest = self.dest_pos

        self.is_show = true

        -- 刷新位置，速度等参数
        self.distance = 0
        self.remain_distance = 0
        self.speed = 0
        self.percent = percent
        self.server_speed = march_line_info.speed
        self.curr_time = march_line_info.elapsedTime * 0.001
        self.total_time = march_line_info.totalTime * 0.001

        self:setFixedPos()
        self:updateSpeed(self.curr_time)

        --local cur_pos = Vector3.Lerp(src_pos, dest_pos, percent)
        --self:init(cur_pos, root)
    end,

    setQueryInfo = function(self, info)
        self.queryInfo = info
    end,

    setColor = function(self)
        local relation = self:marchLineRelation()
        if relation == "own" then
            self.cs_inst:SetLineColor(Color.green)
        elseif relation == "friend" then
            self.cs_inst:SetLineColor(Color.blue)
        elseif relation == "enemy" then
            self.cs_inst:SetLineColor(Color.red)
        else
            self.cs_inst:SetLineColor(Color.white)
        end
    end,

    isAttackArmy = function(self)
        if self.action == common_pb.EMARCH_ATK or
            self.action == common_pb.EMARCH_MASS_ATK or
            self.action == common_pb.EMARCH_MONSTER then
                return true
        end

        return false
    end,

    setFixedPos = function(self)
        local src_offset = 5
        local dest_offset = 5

        if self.is_player_base then
            src_offset = 0
            dest_offset = 0
        end

        if self:isAttackArmy() then
            dest_offset = 15
        end

        local dis = Vector3.Distance(self.dest_pos, self.src_pos)

        if dis < src_offset + dest_offset then
            src_offset = 0
            dest_offset = dis / 2
        end

        local dir = self.dest_pos - self.src_pos
        self.fixed_src = self.src_pos + dir.normalized * src_offset
        self.fixed_dest = self.dest_pos - dir.normalized * dest_offset
        self.fixed_distance = Vector3.Distance(self.fixed_src, self.fixed_dest)

        self.remain_distance = self.fixed_distance * (1 - self.percent)
    end,

    updateSpeed = function(self, elapsed_time)
        self.speed = self.remain_distance / (self.total_time - elapsed_time)
        log.info("################################################ total time  curr time", self.total_time, elapsed_time)
    end,

    -- 刷新参数
    refresh = function(self, src_pos, dest_pos, percent, speed, info)
        self.info = info
        self.src_pos = src_pos  -- 世界坐标位置
        self.dest_pos = dest_pos
        --self.fixed_src = self.src_pos
        --self.fixed_dest = self.dest_pos
        self.total_time = info.totalTime * 0.001
        self.distance = Vector3.Distance(src_pos, dest_pos)

        --self:setFixedPos()

        if self.server_speed ~= speed then
            self:updateSpeed(info.elapsedTime * 0.001)
        end

        -- 总距离
        if self.distance > 0.1 then
            -- 判断是用自己的 percent 还是服务器发来的 
            --local d = math.abs(percent - self.percent)
            --d = self.distance * d
            --self.speed = speed * 0.000001    -- int  2 double
            -- if self.percent <= 0 or d > 10 or self.percent >= 1 then
            --     self.percent = percent
            -- else
                -- 使用客户端本身的 percent
                -- 需要修正 speed
                --local speed_factor =  ((1.0 - self.percent) / (1.0-percent > 0 and 1.0-percent or 1))
                --log.info('### speed_factor:'..speed_factor)
                --self.speed = self.speed * speed_factor
            --end

            -- 完成比例
            self.achieved = false

            if self.transform then
                --self:refreshRotation()
            end

            -- 绘制线
            if self.cs_inst then
                self.cs_inst:DrawLineTo(dest_pos, src_pos)
            end
            
            if self.transform then
                self:playAnimation(self.anim_names.move, true)
            end
        else
            if self.transform then
                self.transform.position = dest_pos
            end
            self.percent = 1
            self.achieved = true
        end
    end,

    backHome = function(self)
        --[[
        if not self.is_back_home then
            self:refresh(self.dest_pos, self.src_pos, 1.0-self.percent, self.speed*10000)
            local r = self.transform.rotation
            r:SetLookRotation(self.dest_pos - self.src_pos, Vector3.up)
            self.transform.rotation = r

            self.is_back_home = true
        end]]
    end,

    update = function(self, dt)
        if not self.achieved then
            if self.is_show then
                --self.percent = (self.speed * self.curr_time) / self.fixed_distance
                
                -- if self.percent >= 1 then   -- 到达目标 
                --     self.percent = 1
                --     self.achieved = true
                -- end


                if self.transform then
                    self.curr_time = self.curr_time + dt

                    local ds = self.speed * dt
                    self.remain_distance = self.remain_distance - ds

                    local src_pos = self.fixed_src
                    local dest_pos = self.fixed_dest

                    if self.info.srcid then
                        local src_obj = world_map_helper:getHold(self.info.srcid)
                        if src_obj and src_obj.transform and src_obj.is_move then
                            src_pos = src_obj.transform.position
                            self.src_pos = src_pos
                        end
                    end

                    if self.info.dstid then
                        local dest_obj = world_map_helper:getHold(self.info.dstid)
                        if dest_obj and dest_obj.transform and dest_obj.is_move then
                            dest_pos = dest_obj.transform.position
                            self.dest_pos = dest_pos
                        end
                    end

                    self:playAnimation(self.anim_names.move, true)
                    self.cs_inst:DrawLineTo(self.dest_pos, self.src_pos)


                    local percent = 1 - (self.remain_distance / self.fixed_distance)
                    self.transform.position = Vector3.Lerp(src_pos, dest_pos, math.min(1, percent))
                    --local dir = dest_pos - src_pos
                    --self.transform.position = self.transform.position + dir.normalized*ds
                    
                    if percent >= 1 then
                        self.percent = 1
                        self.achieved = true
                    end
                end
            end

            self.info.elapsedTime = self.info.elapsedTime + dt*1000
        else
            if self.transform then
                --self.cs_inst:ResetLine()
            end
        end
    end,

    refreshRotation = function(self)
        local r = self.transform.rotation
        r:SetLookRotation(self.dest_pos - self.src_pos, Vector3.up)
        self.transform.rotation = r

        r = Quaternion.identity
        --self.operate.rotation = r
    end,

    getLeftTime = function(self)
        return (self.info.totalTime - self.info.elapsedTime) * 0.001
    end,

    getTotalTime = function(self)
        return self.info.totalTime * 0.001
    end,

    marchLineRelation = function(self)
        local relation = "unknow"
        if world_map_helper:isOwnMarchLine(self.info) then
            relation = "own"
        elseif world_map_helper:isFriendMarchLine(self.info) then
            relation = "friend"
        elseif world_map_helper:isEnemyMarchLine(self.info) then
            relation = "enemy"
        end

        return relation
    end,

    enabledInfo = function(self, enable)
        if self.operate then
            self.operate:SetActive(enable)
        end
    end,

    -- 播放动画
    playAnimation = function(self, anim, loop)
        if self.anim_group_cs then
            return
        end

        local _anim_name = self:getAnimPostfix()

        --log.info("########################################### anim name ", _anim_name)
        if self.anim_loop and self.anim_name == _anim_name then
            return
        end
        
        self.cs_inst:PlayAnimation(_anim_name)

        self.orig_anim_name = anim
        self.anim_name = _anim_name
        self.anim_loop = loop
    end,

    -- 获取动画后缀 sin(22.5) = 0.383, cos(22.5) = 0.924
    getAnimPostfix = function(self)
        local a = world_map_helper:getCameraForward()
        local b = self.dest_pos - self.src_pos
        --local dot = Vector3.Dot(a, b)
        local angle = Vector3.Angle(a, b.normalized)
        local axis = Vector3.Cross(a, b)

        if axis.y < 0 then
            angle = 360 - angle
        end

        --log.info("a:"..tostring(a)..", b:"..tostring(b))
        --log.info(', angle:'..tostring(angle)..", axis:"..tostring(axis))

        local postfix = nil;
        if angle < 11.25 or angle > 360 - 11.25 then
            postfix = "1"
        elseif angle < 11.25*3 then
            postfix = "2"
        elseif angle < 11.25*5 then
            postfix = "3"
        elseif angle < 11.25*7 then
            postfix = "4"
        elseif angle < 11.25*9 then
            postfix = "5"
        elseif angle < 11.25*11 then
            postfix = "6"
        elseif angle < 11.25*13 then
            postfix = "7"
        elseif angle < 11.25*15 then
            postfix = "8"
        elseif angle < 11.25*17 then
            postfix = "9"
        elseif angle < 11.25*19 then
            postfix = "10"
        elseif angle < 11.25*21 then
            postfix = "11"
        elseif angle < 11.25*23 then
            postfix = "12"
        elseif angle < 11.25*25 then
            postfix = "13"
        elseif angle < 11.25*27 then
            postfix = "14"
        elseif angle < 11.25*29 then
            postfix = "15"
        else
            postfix = "16"
        end
        return postfix
    end,

    destroy = function(self)
        self.is_destroied = true

        if self.gameObject then
            self.gameObject:destroy()
        end

        self.operate = nil
    end,

}