local log = logger.get("Game", "March Line")
local super = require("world_map.march_base")

-- 玩家据点
return class
{
    super = super,

    ctor = function(self, march_line_info, src_pos, dest_pos, percent, events, root)
        super.ctor(self, march_line_info, src_pos, dest_pos, percent, events, root)

        self.anim_names =       -- 动画名称配置 
        {
            march = 'march',
            attack = 'attack',
            scatter = 'scatter',
            over = 'over',
        }  

        self.is_group = true
        self:refresh(src_pos, dest_pos, percent, march_line_info.speed, march_line_info)
        
        --local cur_pos = Vector3.Lerp(src_pos, dest_pos, percent)
        --self:init(cur_pos, root)
        
        self.inited = false
        self.attacking = false

        self.bat_evt = world_map_battle.events:on("WORLDMAPBATTLE", 
            function(id)
                self:onBattle(id)
            end
        )
        --functor.make(self.onBattle, self))
    end,

    init = function(self, pos, root)
        local prefab_path = self:getPrefabPath()
        local uobject = game_director.pool:instantiate(prefab_path)

        local callback = function (go)
            -- callback
            if go ~= nil then
                if not game_director:isWorldMapState() then
                    go:SetActive(false)
                    return
                end
                go.parent = root
                go.name = 'ArmyLine'..self.id
                self.gameObject = go
                self.cs_inst = go:GetComponentInChildren(MarchLineActor)
                self.cs_inst.id = tostring(self.id)
                self.cs_inst:setType(hold_types.armyLine)
                self.transform = go.transform

                self.anim_group_cs = go:GetComponentInChildren(SpriteAnimatorGroup)

                self.anim = self.cs_inst.m_Animation
                go.transform.position = pos
                self.cs_inst:DrawLineTo(self.dest_pos, self.src_pos)
                self:setColor()
                self.is_initialized = true
                self:playGroupAnimation(self.anim_names.march)
                
                self.operate_node = self.transform:Find("operate") or self.transform:GetChild(0):Find("operate")

                self:refreshRotation()

                if self.is_destroied then
                    self.gameObject:destroy()   -- 是否加载完成前就被 删除
                else     
                    self:battleCheck()
                end

            else
                log.error('loadResource go is nil')
            end
        end

        if uobject.isLoaded then 
            callback(uobject)
        else 
            uobject.onLoaded = callback
        end

        self.inited = true
    end,

    update = function(self, dt)
        super.update(self, dt)

        if self.attacking then

        end
    end,

    onBattle = function(self, bat_id)
        if self.anim then
            if self.info.battleId == bat_id then
                self.attacking = true
                self:scatter()

                self.coroutine_id = app:startCoroutine(
                    function()
                        coroutine.yield(WaitForSeconds(1))
                        self:attack()
                        coroutine.yield(WaitForSeconds(1.833))
                        self:over()
                        coroutine.yield(WaitForSeconds(0.333))
                        self:battleFinished()
                    end
                )
            end
        end
    end,

    scatter = function(self)
        self.scatter_playing = true
        self:playGroupAnimation(self.anim_names.scatter)
    end,

    attack = function(self)
        self.attack_playing = true

        if self.anim_group_cs then
            local dis = Vector3.Distance(self.cs_inst.m_ShootPoint.position, self.dest_pos)
            self.anim_group_cs:SetTargetOffset(dis)

            log.info("#################################### attack dis ", dis)
        end

        self:playGroupAnimation(self.anim_names.attack)
    end,

    over = function(self)
        self.over_playing = true
        self:playGroupAnimation(self.anim_names.over)
    end,

    playGroupAnimation = function(self, name)
        if self.anim_group_cs then
            self.anim:Play(name)
        end
    end,

    getPrefabPath = function(self)
        if self.action == common_pb.EMARCH_SPY or self.before_action == common_pb.EMARCH_SPY then
            return res_path.res_path_10026
        elseif self.action == common_pb.EMARCH_MASS_ATK then
            return res_path.res_path_10029
        else
            return res_path.res_path_10028
        end
    end,

    battleCheck = function(self)
        local is_bat = world_map_battle:isBattleObj(self.info.battleId)

        if is_bat then
            self:hide()
            world_map_battle:addWaiteAppear(self.info.battleId, self)
        end
    end,

    hide = function(self)
        self.is_show = false
        self.transform:SetActive(false)
    end,

    show = function(self)
        self.is_show = true

        if self.info.elapsedTime >= self.info.totalTime then
            self:destroy()
        else
            self:updateSpeed(self.info.elapsedTime * 0.001)
            self.transform:SetActive(true)
        end
    end,

    battleFinished = function(self)
        self.attacking = false
        world_map_battle:battleFinished(self.info.battleId)
    end,

    destroy = function(self, immediate)
        if not self.attacking or immediate then
            if self.coroutine_id then
                app:stopCoroutine(self.coroutine_id)
            end
            world_map_battle.events:remove("WORLDMAPBATTLE", self.bat_evt)
            super.destroy(self)
        else
            world_map_battle:addWaiteDisappear(self.info.battleId, self)
        end
    end,
}