local log = logger.get("Game", "Own Hold Info")

-- 服务器发来的世界地图信息
local cls = class{
    ctor = function(self)
        self.player_hold = nil
        self.guild_base_hold = nil
        self.public_base_hold = nil
        self.own_base = nil
        self.frame_opened = false
        self.holds = {}
    end,

    clear = function(self)
        self.player_hold = nil
        self.guild_base_hold = nil
        self.public_base_hold = nil
        self.own_base = nil
        self.frame_opened = false
        self.holds = {}
    end,

    createHold = function(self, data, type)
        local hold = {}
        hold.info = data
        hold.id = data.instId or data.buildingId or data.playerId or tostring(data.cityCfgId)
        hold.coordX = data.posX
        hold.coordY = data.posY
        hold.level = data.level
        hold.name = data.name
        hold.type = type

        return hold
    end,

    addCampHold = function(self, data)
        local hold = self:createHold(data, hold_types.camp)
        
        if data.cfgId then
            hold.cfg = config.camp_buildings[data.cfgId]
            hold.name_id = hold.cfg.name_id
            hold.icon_path = hold.cfg.iconpath..hold.cfg.iconid
        end

        table.insert(self.holds, hold)
    end,

    addFactoryHold = function(self, data)
        local hold = self:createHold(data, hold_types.factory)

        if data.cfgId then
            hold.cfg = config.camp_buildings[data.cfgId]
            hold.name_id = hold.cfg.name_id
            hold.icon_path = hold.cfg.iconpath..hold.cfg.iconid
        end

        table.insert(self.holds, hold)
    end,

    addGuildHold = function(self, data)
        self.guild_base_hold = self:createHold(data, hold_types.guild)

        if data.cfgId then
            self.guild_base_hold.cfg = config.camp_buildings[data.cfgId]
            self.guild_base_hold.name_id = self.guild_base_hold.cfg.name_id
            self.guild_base_hold.icon_path = self.guild_base_hold.cfg.iconpath..self.guild_base_hold.cfg.iconid
            if data.cfgId ~= 7 then
                self.guild_base_hold.name = frame_helper:getLangCfg(self.guild_base_hold.name_id)
            end
        end

        if local_guild.is_remove then
            table.insert(self.holds, self.guild_base_hold)
            self.own_base = self.guild_base_hold
        end
    end,

    addPlayerHold = function(self, data)
        self.player_hold = self:createHold(data, hold_types.player)
        table.insert(self.holds, self.player_hold)
    end,

    addPublicBaseHold = function(self, data)
        self.public_base_hold = self:createHold(data, hold_types.publicBase)

        if data.cityCfgId then
            self.public_base_hold.cfg = config.worldmap_publicbases[data.cityCfgId]
            self.public_base_hold.name_id = self.public_base_hold.cfg.name_id
            self.public_base_hold.icon_path = self.public_base_hold.cfg.iconpath..self.public_base_hold.cfg.iconid
            self.public_base_hold.coordX = self.public_base_hold.cfg.x
            self.public_base_hold.coordY = self.public_base_hold.cfg.y
        end

        if not local_guild.is_remove then
            table.insert(self.holds, self.public_base_hold)
            self.own_base = self.public_base_hold
        end
    end,

    getOwnBase = function(self)
        return self.own_base
    end,

    isOwnBase = function(self, hold_id)
        return hold_id == self.own_base.id
    end,

    haveCamp = function(self)
        for k, v in ipairs(self.holds) do
            if v.type == hold_types.camp then
                return true
            end
        end

        return false
    end,

    setFrameOpened = function(self, opened)
        self.frame_opened = opened
    end,

    waitFrameOpened = function(self)
        while not self.frame_opened do
            coroutine.yield(nil)
        end
        coroutine.yield(nil)
    end,
}

own_hold_info = cls()

return own_hold_info
