local log = logger.get("Game", "Player Hold")
local super = require('world_map.base_hold')
local march_line = require('world_map.march_base')

-- 玩家据点
return class
{
    super = super,

    ctor = function(self, pos, info, events, root)
        self.info = info
        self.id = info.objinfo.instid
        self.level = info.homeLevel
        self.coordX = info.objinfo.posx
        self.coordY = info.objinfo.posy
        self.is_move = info.isMove
        self.events = events
        self.type = hold_types.player
        self.root = root
        self.pos = pos
        self.player_name = info.playerName
        self.short_name = info.familyShortName

        self.cfg_id = info.objinfo.cfgid
        log.info("############################################## cfg id ", self.cfg_id)
        self.cfg = config.worldmap_base_configs[self.cfg_id]
        self.name_id = self.cfg.name_id
        self.icon_path = self.cfg.iconpath..self.cfg.iconid

        super.ctor(self, self.id, self.cfg_id, self.coordX, self.coordY)

        --self:init(pos, root)
    end,

    refresh = function(self, info)
        local old_skin = self.info.cityDress
        local pos_changed = self.coordX ~= info.objinfo.posx or self.coordY ~= info.objinfo.posy

        if pos_changed then
            self.events:dispatch('releas_hold_cells', self.coordX, self.coordY, self.cfg.gridnum or 1, self)
        end

        self.info = info
        self.level = info.homeLevel
        self.is_move = info.isMove
        self.coordX = info.objinfo.posx
        self.coordY = info.objinfo.posy

        if old_skin ~= info.cityDress then
            self:destroy()
            self:init(self.pos, self.root)
        end

        if self.transform then
            if self.is_move then
                if self.march then
                    self:refreshArmyLine(info.armyLine)
                else
                    self:initMarch(info.armyLine)
                end
            else
                if pos_changed then
                    self:changePos()
                end
            end
        end

        self:setNameText()
        super.refresh(self)
    end,

    init = function(self, pos, root)
        local prefab_path = self:getMainSkin()
        local uobject = game_director.pool:instantiate(prefab_path)
        self.inited = true

        local callback = function (go)
            -- callback
            if go ~= nil then
                if not game_director:isWorldMapState() then
                    go:SetActive(false)
                    return
                end
                self.gameObject = go
                self.gameObject.name = 'player_base_' .. self.id
                self:initUI(pos, root)
                self:setNameText()
                self.actor_cs = self:addWorldMapActorMono(go, prefab_path)
                self.front = go.transform:Find("Front") or go.transform:GetChild(0):Find("Front")
                self.back = go.transform:Find("Back") or go.transform:GetChild(0):Find("Back")
                if self.actor_cs then
                    self.actor_cs.id = tostring(self.id)
                    self.is_initialized = true
                    self.actor_cs:setType(hold_types.player)
                end

                if self.is_move then
                    self:initMarch(self.info.armyLine)
                end

                if self.info.playerId == local_player.id then
                    self.events:dispatch("player_hold_loaded", self)
                end
            else
                log.error('loadResource go is nil')
            end
        end

        if uobject.isLoaded then 
            callback(uobject)
        else 
            uobject.onLoaded = callback
        end

    end,

    initMarch = function(self, march_info)
        self.march_id = march_info.instId
        local percent = march_info.curPos * 0.001

        local src = world_map_helper:getCellPosition(march_info.srcPosX, march_info.srcPosY)
        local dst = world_map_helper:getCellPosition(march_info.dstPosX, march_info.dstPosY)

        self.march = march_line(march_info, src, dst, percent, self.events, self.root)
        self.march.is_player_base = true
        self.march.transform = self.transform
        self.march.gameObject = self.gameObject
        self.march.operate = self.transform:Find("operate") or self.transform:GetChild(0):Find("operate")

        self.march.anim_names =       -- 动画名称配置 
        {
            idle = 'luke_idle',
            move = 'luke_move',
            attack = 'luke_attack',
            die = 'luke_die',
            escape = 'luke_escape',
        }   

        self.march.cs_inst = self.gameObject:GetComponentInChildren(MarchLineActor)
        self.march.cs_inst.id = tostring(self.id)
        self.march.cs_inst:setType(hold_types.player)
        self.march:setColor()

        self:refreshArmyLine(march_info)
    end,

    refreshArmyLine = function(self, info)
        if self.front then
            self.front:SetActive(true)
        end

        if self.back then
            self.back:SetActive(true)
        end

        local src = world_map_helper:getCellPosition(info.srcPosX, info.srcPosY)
        if info.instId ~= self.march_id then
            src = self.transform.position
            self.march_id = info.instId
        end
        local dst = world_map_helper:getCellPosition(info.dstPosX, info.dstPosY)
        local percent = info.curPos * 0.001
        if percent > 1 then percent = 1 end

        self.march:refresh(src, dst, percent, info.speed, info)

        self.events:dispatch('releas_hold_cells', self.coordX, self.coordY, self.cfg.gridnum or 1, self)
    end,

    update = function(self, dt)
        super.update(self, dt)

        if self.is_move and self.march then
            self.march:update(dt)
        end

        if not self.is_move and self.march then
            self:stop()
            self.march = nil
        end
    end,

    isMove = function(self)
        return self.is_move
    end,

    changePos = function(self)
        if self.transform then
            local new_pos = world_map_helper:getCellPosition(self.coordX, self.coordY)
            self.transform.position = new_pos

            if local_player.id == self.info.playerId then
                world_map_helper.world_map:lookAtMyHold()
            end
        end
    end,

    getMainSkin = function(self)
        local prefab_path = self.cfg.respath..self.cfg.resname
        local cfg = config.home_config_skins[self.info.cityDress]

        if cfg then
            prefab_path = cfg.prefab_real
        end

        return prefab_path
    end,

    stop = function(self)
        if self.front then
            self.front:SetActive(false)
        end

        if self.back then
            self.back:SetActive(false)
        end
    end,

    getCurrPos = function(self)
        local x = self.coordX
        local z = self.coordY

        if self.transform then
            x, z = world_map_helper:positionToXZ(self.transform.position)
            x = x or self.coordX
            z = z or self.coordY
        end

        return x, z
    end,
}