local log = logger.get("Game", "Search Info")

local cls = {

    init = function (self, idxs)
        self.monsters = {}
        self.idx_cfg = {}
        self.ress = {}
        for i=1, 5 do
            self.ress[i] = {}
            self:initResList(i)
        end

        self.monster_idxs = idxs

        for k, v in ipairs(idxs) do
            self.monsters[v] = {}
            self:initMonsterList(v)
        end

        self.searching = false
    end,

    initResList = function(self, idx)
        local cfg = config.worldmap_titaniums
        cfg:foreach(
            function(k, v)
                if v.type == idx then
                    self.ress[idx][v.level] = {}
                end
            end
        )
    end,

    initMonsterList = function(self, idx)
        config.worldmap_monsters:foreach(
            function(k, v)
                if v.monster_index == idx then
                    self.monsters[idx][v.level] = {}
                    self.idx_cfg[idx] = v
                end
            end
        )
    end,

    getDefaultCfg = function(self)
        local idx = self.monster_idxs[1]
        return self.idx_cfg[idx]
    end,

    monster = function(self, info)
        local inst = {}
        inst.info = info
        inst.id = info.objinfo.instid
        inst.coordX = info.objinfo.posx
        inst.coordY = info.objinfo.posy
        inst.hp = info.curHp
        inst.max_hp = info.maxHp
        inst.left_time = info.disappearTime
        inst.index = 0

        inst.cfg_id = info.objinfo.cfgid
        inst.cfg = config.worldmap_monsters[inst.cfg_id]
        
        if inst.cfg then
            inst.name_id = inst.cfg.monstername
            inst.level = inst.cfg.level
        else
            log.error("########## monster hold cfg error ! id", info.objinfo.instid)
        end

        inst.distance = world_map_helper:getDistToBase({x = inst.coordX, z = inst.coordY})

        return inst
    end,

    titanium = function(self, info)
        local inst = {}
        inst.info = info
        inst.id = info.objinfo.instid
        inst.coordX = info.objinfo.posx
        inst.coordY = info.objinfo.posy
        inst.amount = info.resAmmount
        inst.index = 0

        inst.cfg_id = info.objinfo.cfgid
        inst.cfg = config.worldmap_titaniums[inst.cfg_id] or {}
        inst.type = inst.cfg.type

        inst.level = inst.cfg.level
        inst.name_id = inst.cfg.nameid

        inst.distance = world_map_helper:getDistToBase({x = inst.coordX, z = inst.coordY})

        return inst
    end,

    addMonsters = function(self, list)
        for k, v in ipairs(list) do
            local m = self:monster(v)
            table.insert( self.monsters[m.cfg.monster_index][m.level], m )
            m.index = #self.monsters[m.cfg.monster_index][m.level]
        end
    end,

    addTitanium = function(self, ress)
        for k, v in ipairs(ress) do
            local m = self:titanium(v)
            table.insert( self.ress[m.cfg.type][m.level], m )
            m.index = #self.ress[m.cfg.type][m.level]
        end
    end,

    getMonsters = function(self, idx)
        return self.monsters[idx]
    end,

    getRess = function(self, idx)
        return self.ress[idx]
    end,

    getMonsterByIdxAndLv = function(self, idx, level)
        local list = self.monsters[idx]
        
        if list then
            lv_list = list[level] or {}

            return lv_list[1]
        end

        return nil
    end,

    getDisplayMonsters = function(self, monsters, level)
        local list = monsters[level]

        if list then
            return self:getNearestHold(list)
        end

        return nil
    end,

    getDisplayRes = function(self, ress, level)
        local list = ress[level]

        if list then
            return self:getNearestHold(list)
        end

        return nil
    end,

    getNearestHold = function(self, holds)
        local hold = nil

        for k, v in pairs(holds) do
            if not hold then
                hold = v
            else
                if v.distance < hold.distance then
                    hold = v
                end
            end
        end

        return hold
    end,

    setCurrSearchInfo = function(self, type, info)
        self.search_list = nil

        if info then
            self.search_idx = info.index

            if type == common_pb.ESEARCH_MONSTER then
                self.search_list = self.monsters[info.cfg.monster_index][info.level]
            elseif type == common_pb.ESEARCH_RES then
                self.search_list = self.ress[info.type][info.level]
            end
            
            if self.search_list then
                local tab = {}
                for i=self.search_idx, #self.search_list do
                    table.insert( tab, self.search_list[i] )
                end

                for i=1, self.search_idx - 1 do
                    table.insert( tab, self.search_list[i] )
                end

                self.search_list = tab
                self.search_idx = 1

                if type == common_pb.ESEARCH_MONSTER then
                    local world_map = world_map_helper.world_map
                    world_map:createHold(hold_types.monster, info.info)
                end
            end
        end

        self.searching = self.search_list ~= nil
    end,

    getCurrHold = function(self)
        if self.search_list then
            return self.search_list[self.search_idx]
        end

        return nil
    end,

    getNextHold = function(self)
        if self.search_idx < #self.search_list then
            self.search_idx = self.search_idx + 1
            return self.search_list[self.search_idx]
        end

        return nil
    end,

    getPreHold = function(self)
        if self.search_idx > 1 then
            self.search_idx = self.search_idx - 1
            return self.search_list[self.search_idx]
        end

        return nil
    end,

    isLastHold = function(self)
        return self.search_idx >= #self.search_list
    end,

    isSearched = function(self)
        return self.search_list ~= nil
    end,

    getMonsterCntByLevel = function(self, idx, lv)
        local list = self.monsters[idx]

        if list then
            local monsters = list[lv] or {}

            return #monsters
        end

        return 0
    end,

    idxIsValid = function(self, idx)
        for k, v in ipairs(self.monster_idxs) do
            if v == idx then
                return true
            end
        end

        return false
    end,
}

search_info = search_info or cls

return search_info