local log = logger.get("Game", "World Map")
local temp_hold_cls = require('world_map.temp_hold')
local world_map_cell_cls = require('world_map.world_map_cell')
require('world_map.world_map_helper')
require('world_map.world_map_battle')

hold_types = require('world_map.world_map_enums').hold_type
opreator_type = require('world_map.world_map_enums').opreator_type
hold_states = require('world_map.world_map_enums').hold_states
progress_bar = require('world_map.world_map_enums').progress_bar

local operator_path = "Assets/Prefab/WorldMap/HoldOperator.prefab"
local hold_progress_path = "Assets/Prefab/WorldMap/HoldBuildProgress.prefab"
local hold_create_path = "Assets/Prefab/WorldMap/HoldCreateOperator.prefab"
local build_operator_path = "Assets/Prefab/WorldMap/WorldBuildOperator.prefab"
local search_ui_path = "Assets/Prefab/WorldMap/SearchChangeUI.prefab"
local hold_name_ui_path = "Assets/Prefab/WorldMap/HoldName.prefab"
local hold_capture_ui_path = "Assets/Prefab/WorldMap/HoldCaptureIcon.prefab"
local monster_name_ui_path = "Assets/Prefab/WorldMap/MonsterName.prefab"
local boss_name_ui_path = "Assets/Prefab/WorldMap/BossName.prefab"
local res_icon_ui_path = "Assets/Prefab/WorldMap/ResHoldIcon.prefab"
local hold_operate_path = "Assets/Prefab/WorldMap/HoldOperate.prefab"

return class {
    ctor = function (self, owner)
        self.inited = false
        self.owner = owner      -- game_state_world_map
        self.my_hold = nil      -- 玩家自己的据点
        self.cs_inst = nil  -- HexMapLua 组件
        self.click_cell = {}   --当前点击的地块的坐标
        self.target_block = nil
        self.regions = {}
        self.guild_base_regions = {}
        self.temp_regions = {}

        -- 注册事件处理函数
        self.events = require("base.event").new()

        self.hold_manager = require("world_map.hold_manager")(self.events)
        self.hold_operator = require("world_map.hold_opreator")(self)
        world_map_helper:setWorldMap(self)
        world_map_battle:setWorldMap(self)

        -- 玩家据点资源加载完成回调
        self.events:on("player_hold_loaded", function (hold)
            -- 玩家自己的据点
            if hold.info.playerId == local_player.id then
                self.my_hold = hold
            end
        end)

        --[[ self.events:on("public_base_loaded", function (hold)
            if world_map_helper:isOwnPublicBase(hold) then
                self.my_hold = hold
            end
        end) ]]

        self.events:on("guild_temp_hold_loaded", function (hold)
            self:showHoldOperator(hold)
        end)

        self.events:on("set_hold_cells", function (id, x, z, gridnum, hold)
            self.cells:setHoldCells(id, x, z, gridnum)
            if hold.type == hold_types.guild then
                local repeat_cells = world_map_helper:setRealGuildBaseRange(hold)
                self:removGuildBaseRegions(repeat_cells)
                self:drawGuildBaseRegions(hold)
            end
        end)

        self.events:on("releas_hold_cells", function (x, z, gridnum, hold)
            self.cells:releasHoldCells(x, z, gridnum)
            if hold.type == hold_types.guild then
                self:guildBaseDestroyed(hold)
            end
        end)

        self.events:on("create_hold", function (hold)
            local x, z = self.cs_inst:GetXZ(hold.transform.position)
            network_sender:buildFamilyBuilding(hold.cfg_id, x, z, hold.cfg_type)
        end)

        --[[ self.events:on("CREATEHOLD", function (hold)
            local idx = self.cells:getBlockIdx(hold.coordX, hold.coordY)
            self.hold_manager:addBlockHold(idx, hold)
            if self.cells:inBlocks(idx) then
                hold:init(hold.pos, hold.root)
            end
        end) ]]

        self.events:on("destroy_hold", function (hold)
            self:hideHoldOperator()
            hold:destroy()
            self.drag_hold = nil
            if self.temp_hold == hold then
                self:destroyTempHold()
            end
        end)

        self.events:on("hold_refreshed", function (hold)
            if self.click_hold and self.click_hold.id == hold.id then
                self:clickOther()
                network_sender:queryWorldMap(hold.id, protocol_pb.MAP_WORLDMAP)
            end
        end)
    end,

    -- 初始化
    init = function(self)
        local go = GameObject.Find('HexGrid')
        self.cs_inst = go:GetComponent(HexMapLua)
        self:setCameraRange()
        self:setCameraMoveSpeed()

        self.root = GameObject.Find('Root').transform
        -- syz
        -- self.cs_inst:GenerateMap(30, 10);
        -- zh
        --self.cs_inst:GenerateMap(2, 2);
        self.owner:initDecorateCfg()

        self.cs_inst:GenerateMap(game_info.player_pos.x, game_info.player_pos.y, 
            function()
                if not game_director:isWorldMapState() then
                    return
                end
                -- 创建格子
                local max_x, max_z = self:getWorldMapBorder()
                log.info("## x, z"..max_x..","..max_z)
                self.cells = world_map_cell_cls(max_x, max_z)
                self.hold_manager:initBlockHolds(self.cells.block_num_x*self.cells.block_num_z)

                self:addMineCircle()

                --[[ self.pool_parent = app.gameObject.transform:CreateChild("world_map_pool")

                if not self.pool_parent then
                    log.error("############## self.pool_parent is null !!!")
                end ]]

                -- 创建对象池
                --[[ self.pool = require("base.pool"){
                    size = 1, 
                    transform = self.pool_parent,
                } ]]
                self.pool = game_director.pool

                self.pool_coroutine = app:startCoroutine(function() 
                    if not self.pool then
                        return
                    end

                   -- log.error("## begin preload!!")
                    -- self.pool:wait_prepare(hold_progress_path, 5)
                    self.pool:wait_prepare(operator_path, 1)
                    self.pool:wait_prepare(search_ui_path, 1)
                    self.pool:wait_prepare(build_operator_path, 1)
                    self.pool:wait_prepare(hold_name_ui_path, 30)
                    self.pool:wait_prepare(hold_capture_ui_path, 30)
                    self.pool:wait_prepare(monster_name_ui_path, 30)
                    self.pool:wait_prepare(boss_name_ui_path, 30)
                    self.pool:wait_prepare(hold_operate_path, 1)
                    --self.pool:wait_prepare(res_icon_ui_path, 5)

                    --log.error("## end preload")

                    network:registerMessageHandler(self.owner)
                    self:sendEnterWorldMapMsg()
                end)
            end
        )

        -- 创建测试据点
        local id = 1
        if game_info and game_info.role_info then
            id = game_info.role_info.playerid
        end
    end,

    sendEnterWorldMapMsg = function(self)
        local pos = world_map_info.enter_pos
        if pos and pos.x and pos.y then
            network_sender:enterWorldMap(self.owner.map_type, pos.x, pos.y)
        else
            network_sender:enterWorldMap(self.owner.map_type)
        end
        world_map_info.enter_pos = nil
    end,

    update = function(self, dt)
        if self.hold_manager then
            self.hold_manager:update(dt)
        end

        if self.cs_inst and self.cells then
            local _, _, hit = self:raycast(Vector3(UnityEngine.Screen.width/2, UnityEngine.Screen.height/2, 0), 'Ground')
            if hit then
                local pos = self.cs_inst:GetCellPos(hit.point)
                local x, z = self.cs_inst:GetXZ(pos)
                local changed, block_idx = self.cells:updateCenterBlock(x, z)
                if changed then
                    if not self.target_block then
                        network_sender:moveWorldMap(x, z)
                    else
                        if block_idx == self.target_block then
                            self.target_block = nil
                            network_sender:moveWorldMap(x, z)
                        end
                    end
                end
            end
        end

        if self.begain_build then
            self:updateTempHold()
        end
    end,

    getBlockIdx = function(self, x, z)
        local idx = self.cells:getBlockIdx(x, z)
        return idx
    end,

    -- 摄像机定位到自己家
    lookAtMyHold = function(self)
        if self.my_hold then
            self.target_block = self:getBlockIdx(self.my_hold.coordX, self.my_hold.coordY)
            local x, z = self.cs_inst:GetXZ(self.my_hold.transform.position)
            self.cs_inst:LookAtPos(x, z, nil)
        end
    end,

    getMyholdPosition = function(self)
        local pos = Vector3.zero
        if self.my_hold then
            pos = self.cs_inst:GetCellPos(self.my_hold.coordX, self.my_hold.coordY)
        end
        return pos
    end,

    -- 摄像机跟随 target
    lookAtTarget = function(self, target, cb)
        self.cs_inst:LookAt(target, cb)
    end,

    -- 摄像机跟随 MarchLine
    lookAtMarchLine= function(self, id, cb)
        local march_line = self.hold_manager:getHold(hold_types.armyLine, id)
        if march_line then
            self.cs_inst:LookAt(march_line.transform, cb)
        end
    end,

    -- 摄像机定位到某个坐标
    lookAtPosition = function(self, x, z, cb)
        self.target_block = self:getBlockIdx(x, z)
        self.cs_inst:LookAtPos(x, z, cb)
    end,

    -- 设置摄像机位置
    setCameraPos = function(self, iX, iZ)
        self.target_block = self:getBlockIdx(iX, iZ)
        self.cs_inst:SetCameraPos(iX, iZ)
    end,

    -- 获得世界地图的最大范围
    getWorldMapBorder = function(self)
        return self.cs_inst.CellCountX, self.cs_inst.CellCountZ
        --return 1024, 1024
    end,

    isFloor = function(self, pos)
        local region = self.cs_inst:GetCellType(pos)
        if region >= 0 then
            log.info("## region:"..region)
            return region --and not cell.IsUnderWater
        else
            log.info("## region:"..tostring(region))
            return false
        end
    end,

    -- 点击
    click = function(self, point)
        print('点击')
        local r = self.owner.camera:ScreenPointToRay(point)
        local rst, hit = UnityEngine.Physics.Raycast(r, hit, 999)
        if rst and hit then
            local go = hit.transform.gameObject
            if go.layer == core.layers['Ground'] then
                self:clickGround(hit.point)
            end
        end
    end,

    -- 开始拖拽
    dragStart = function(self, point)
        -- print('开始拖拽')
        self:clickOther()
        --self:hideHoldOperator()
        self.drag_hold = nil
        self.target_block = nil
        local rst, go = self:raycast(point, 'Temp')
        if rst and go then
            local cs = go:GetComponentInChildren(WorldMapActor)
            if self.hold_temp_id and tostring(self.hold_temp_id) == cs.id then
                self.drag_hold = self.temp_hold
            end
        end
    end,

    -- 拖拽中...
    drag = function(self, point)
        -- print('拖拽中...', self.drag_hold and "ok" or "fuck!")
        if self.drag_hold then
            local rst, go, hit = self:raycast(point, 'Ground')
            if hit then
                local pos = self.cs_inst:GetCellPos(hit.point)
                self.drag_hold:setPos(pos)
                local x, z = self.cs_inst:GetXZ(pos)
                local rst = self.cells:isValid(self.drag_hold.cfg_id, x, z, self.drag_hold:getHoldType())
                self.drag_hold:setValid(rst)
                print("x:", x, "z:", z, "valid:", rst)
            end
        end
    end,

    -- 拖拽结束
    dragEnd = function(self, point)
        -- print('拖拽结束')
        if self.drag_hold then
            self:showHoldOperator(self.drag_hold)
        end
    end,

    createHold = function(self, type, info)
        if self.hold_manager then
            local pos_1 = nil
            local pos_2 = nil
            if type == hold_types.armyLine then
                pos_1 = self.cs_inst:GetCellPos(info.srcPosX, info.srcPosY)
                pos_2 = self.cs_inst:GetCellPos(info.dstPosX, info.dstPosY)
            else
                pos_1 = self.cs_inst:GetCellPos( info.objinfo.posx, info.objinfo.posy)
            end
            self.hold_manager:createHold(type, info, pos_1, pos_2)
        end
    end,

    -- 返回是否成功删除
    removeHold = function(self, type, id)
        if self.hold_manager then
            local hold = self.hold_manager:getHold(type, id)
            if hold then
                local block_idx = nil
                
                if hold.type ~= hold_types.marchLine then
                    block_idx = self.cells:getBlockIdx(hold.coordX, hold.coordY)
                end

                self.hold_manager:removeHold(hold, block_idx)
                return true
            end
        end

        return false
    end,

    hideHold = function(self, type, id)
        if self.hold_manager then
            self.hold_manager:hideHold(type, id)
        end
    end,

    -- 创建建筑
    createNewHold = function(self, cfg_id, type)
        self.hold_temp_id = 0
        print("创建新建筑...")
        if not self.temp_hold then
            local camera_look_pos = self.cs_inst.hexCamera.transform.position
            local pos = self.cs_inst:GetCellPos(camera_look_pos)
            self.temp_hold = temp_hold_cls(self.hold_temp_id, cfg_id, type, pos, self.events, self.root)
            self:drawTempRegion(pos)
            self.begain_build = true
        end
    end,

    updateTempHold = function(self)
        if self.temp_hold.transform then
            local camera_look_pos = self.cs_inst.hexCamera.transform.position
            local pos = self.cs_inst:GetCellPos(camera_look_pos)

            if self.temp_hold.transform.position == pos then
                return
            end

            self.temp_hold.transform.position = pos
            
            self:drawTempRegion(pos)
        end
    end,

    drawTempRegion = function(self, pos)
        self:removeTempRegions()

        local x, z = self.cs_inst:GetXZ(pos)
        local all_index = self.cells:getBuildingHoldIndexs(self.temp_hold.cfg_id, x, z, hold_types.guild)
        local _, idx_list = self.cells:isBuildingOnCell(all_index)

        for _, idx in pairs(all_index) do
            local is_valid = idx_list[idx] == nil
            
            local color = Color(167/255, 0, 13/255, 0.25)
            if is_valid then
                color = Color(9/255, 177/255, 80/255, 0.25)
            end
            
            x, z = self.cells:getXZ(idx)
            local id = self.cs_inst:DrawRegion({Vector2(x, z)}, color)
            self.temp_regions[idx] = id
        end
    end,

    removeTempRegions = function(self)
        for k, v in pairs(self.temp_regions) do
            self.cs_inst:RemoveRegion(v)
        end

        self.temp_regions = {}
    end,

    -- 射线
    raycast = function(self, point, layer)
        local mask = core.mask(layer)
        local r = self.owner.camera:ScreenPointToRay(point)
        local rst, hit = UnityEngine.Physics.Raycast(r, hit, 999, mask)
        if rst then
            local go = hit.transform.gameObject
            return rst, go, hit
        else
            return rst, nil, nil
        end
    end,

    -- 显示操作
    showHoldOperator = function(self, hold)
        self.show_operator_hold = hold
        local max = 6   -- 最多6个操作选项
        local rst = hold:getOperators()
        if rst and #rst > 0 then
            local uobject = game_director.pool:instantiate(operator_path)

            local callback = function (go)
                -- callback
                if go ~= nil then
                    self.operator_trans = go
                    self.operator_trans:Reset()
                    self.operator_trans:ScaleTo(0.1)
                    self.operator_trans.transform:SetParent(hold.transform)
                    self.operator_trans.transform.localPosition = Vector3(0, 8.2, -32)

                    local node = self.operator_trans.transform:Find("info/des_text") or self.operator_trans.transform:GetChild(0):Find("info/des_text")
                    local des_text = node:GetComponent(UnityEngine.UI.Text)
                    des_text.text = NativeText.GetLangStr(9000088)

                    for __, fun in ipairs(rst) do
                        local btn_trans = self.operator_trans.transform:Find("layout/Button (" .. __ .. ")") or self.operator_trans.transform:GetChild(0):Find("layout/Button (" .. __ .. ")")
                        if btn_trans then
                            btn_trans.gameObject:SetActive(true)
                            local icon = btn_trans:GetComponent(UnityEngine.UI.Image)
                            icon:SetSprite(fun.icon_name)
                            local btn = btn_trans:GetComponent(UnityEngine.UI.Button)
                            btn.onClick:RemoveAllListeners()
                            btn.onClick:AddListener(function ()
                                fun.click_callback()
                            end)
                        end
                    end

                    for __ = #rst + 1, max do
                        local btn_trans = self.operator_trans.transform:Find("layout/Button (" .. __ .. ")") or self.operator_trans.transform:GetChild(0):Find("layout/Button (" .. __ .. ")")
                        if btn_trans then
                            btn_trans.gameObject:SetActive(false)
                        end
                    end
                else
                    log.error('loadResource go is nil')
                end
            end

            if uobject.isLoaded then 
                callback(uobject)
            else 
                uobject.onLoaded = callback
            end
        end
    end,

    -- 隐藏操作
    hideHoldOperator = function(self)
        self.show_operator_hold = nil
        if self.operator_trans then
            self.operator_trans:destroy()
            self.operator_trans = nil
        end
    end,

    showOperators = function(self, pos, hold_type, hold)
        local type = hold_type

        if type then
            if not self.hold_operate then
                self.hold_operate = self.pool:instantiate(hold_operate_path)
                self.hold_operate:Reset()
            else
                self.hold_operate:SetActive(true)
            end

            if hold and (hold.type == hold_types.armyLine or hold.type == hold_types.player) then
                self.hold_operate.transform:SetParent(hold.operate_node)
                self.hold_operate.transform.localPosition = Vector3.zero
            else
                self.hold_operate.transform:SetParent(self.root)
                self.hold_operate.transform.position = pos
            end
            
            self.hold_operator:setUI(self.hold_operate, type, hold)
        end

        if search_info.searching and hold ~= nil then
            if not self.search_change_ui then
                self.search_change_ui = self.pool:instantiate(search_ui_path)
                self.search_change_ui.transform:SetParent(self.root)
                self.search_change_ui:Reset()
                self:addSearchBtnListener()
            else
                self.search_change_ui:SetActive(true)
            end
            
            local position = self.cs_inst:GetCellPos(hold.coordX, hold.coordY)
            position.z = position.z - 5
            position.y = 8
            self.search_change_ui.transform.localPosition = position
            self:refreshSearchUI()
        end
    end,

    clickHold = function(self, hold)
        local type = world_map_helper:getOperateType(hold)
        
        log.info("#### click hold type ", tostring(type))
        if not type then
            return
        end

        local x, z = self.cs_inst:GetXZ(hold.transform.position)
        
        if self.click_hold and self.click_hold.id == hold.id then
            return
        end

        self:clickOther()

        local pos = self.cs_inst:GetCellPos(hold.transform.position)

        if hold.type == hold_types.guild then
            pos = hold.operate_node.position
        end
        
        self.click_hold = hold
        self.click_cell.x = x
        self.click_cell.z = z

        hold.transform:SetAsLastSibling()
        self:showOperators(pos, type, hold)

        if hold.type ~= hold_types.armyLine and not hold.is_move then
            local all_index = self.cells:getBuildingHoldIndexs(hold.cfg_id, x, z, hold.type)
            local cell_pos = {}

            for _, idx in pairs(all_index) do
                local x, z = self.cells:getXZ(idx)
                table.insert(cell_pos, {x = x, z = z})
            end

            self:drawRegion(cell_pos)
        end
    end,

    clickGround = function(self, point)
        if self.drag_hold then
            return
        end

        self:clickOther()

        local x, z = self.cs_inst:GetXZ(point)

        if self.cells:cellIsEmpty(x,z) then
            network_sender:queryWorldMap(self.cells:cellIsEmpty(x,z), protocol_pb.MAP_WORLDMAP)
            return
        end
            
        if not self:isFloor(point) then
            log.info("#### cell not is empty")
            return
        end

        if self.click_cell.x == x and self.click_cell.z == z then
            --return
        end

        local pos = self.cs_inst:GetCellPos(point)

        self.click_cell.x = x
        self.click_cell.z = z
        
        local cb = function()
            self:showOperators(pos, opreator_type.space)
        end
        self:lookAtPosition(x, z, cb)

        self:resetSearchInfo()

        local all_index = self.cells:getAllIndex(1, x, z)
        local cell_pos = {}

        for _, idx in pairs(all_index) do
            local x, z = self.cells:getXZ(idx)
            table.insert(cell_pos, {x = x, z = z})
        end

        self:drawRegion(cell_pos)
    end,

    clickOther = function(self)
        --[[ if self.hold_operate then
            self.hold_operate.transform:SetActive(false)
        end ]]

        if self.hold_operator.is_show then
            self.hold_operator:hide()
        end

        if self.click_hold then
            --self.click_hold:enabledInfo(false)
            self.click_hold = nil
        end
        --self.click_cell = {}
        
        self:removeRegions()
    end,

    queryHoldInfo = function(self, msg)
        local hold = nil
        if world_map_helper:isGuildHold(msg.eObjType) then
            hold = self.hold_manager:getHold(hold_types.guild, msg.objId)
        else
            hold = self.hold_manager:getHold(msg.eObjType, msg.objId)
        end
        
        if not hold then
            log.error("##### query hold error ! id  type", msg.objId, msg.eObjType)
            return
        end

        if msg.eObjType == common_pb.WORLD_OBJ_MONSTER then
            hold:onQuery(msg.monsterHoldQuerys)
        elseif msg.eObjType == common_pb.WORLD_OBJ_WILD_MINE then
            hold:setQueryInfo(msg.wildMineQuery)
        elseif msg.eObjType == common_pb.WORLD_OBJ_FAMILY then
            hold:setQueryInfo(msg.familybase)
        end

        local cb = function()
            if msg.eObjType == common_pb.WORLD_OBJ_MONSTER then
                local search_hold = search_info:getCurrHold()
                if search_info.searching then
                    self:showOperators(nil, nil, hold)
                    search_info.searching = false
                else
                    if not search_hold or search_hold.id ~= hold.id then
                        self:resetSearchInfo()
                    end
                    frame_manager:show("world_monster_info", self, {monster = hold})
                end
                self:clickOther()
                self.click_hold = hold
                if hold.transform then
                    hold.transform:SetAsLastSibling()
                end
                self:drawRegion({x = hold.coordX, z = hold.coordY})
            else
                local search_hold = search_info:getCurrHold()
                if not search_hold or search_hold.id ~= hold.id then
                    self:resetSearchInfo()
                end
                self:clickHold(hold)
            end
        end

        if msg.eObjType == common_pb.WORLD_OBJ_ARMY then
            self:lookAtMarchLine(hold.id, cb)
        elseif msg.eObjType == common_pb.WORLD_OBJ_PLAYER and hold.is_move then
            self:lookAtTarget(hold.transform, cb)
        else
            self:lookAtPosition(hold.coordX, hold.coordY, cb)
        end
    end,

    gatherMine = function(self, mine, car_id)
        local param = {}
        param.mineInstId = mine.id
        param.carId = car_id
        network_sender:startMarchLine(protocol_pb.MAP_WORLDMAP, common_pb.EMARCH_GATHER, param)
        self:clickOther()
    end,

    refreshQueryInfo = function(self, info)
        local hold = self.hold_manager:getHoldFromId(info.instId)
        if hold then
            hold:refreshQueryInfo(info)
            game_director.events:dispatch("WORLDMAPBUILDTEAMCOUNT", info.instId)
        end
    end,

    setCameraRange = function(self)
        local cfg = config.worldmap_regions[99]
        self.cs_inst:SetCameraMoveRange(cfg.bottomleftx, cfg.uprightx, cfg.bottomlefty, cfg.uprighty)
    end,

    setCameraMoveSpeed = function(self)
        local cfg = config.worldmap_others[1]
        self.cs_inst:SetCameraMoveSpeed(cfg.screenmovespeed, cfg.screenmovespeed)
    end,

    addMineCircle = function(self)
        self:removeMineCircle()
        if self.cs_inst:IsGridShow() then
            self.mine_regions = {}
            local color = Color(12/255, 243/255, 110/255, 10/255)
            for k, v in pairs(self.cells.mine_circles) do
                local id = self.cs_inst:DrawRaingRegion(
                    v.outerbottomleftx, v.outerbottomlefty, v.outeruprightx, v.outeruprighty,
                    v.innerbottomleftx, v.innerbottomlefty, v.inneruprightx, v.inneruprighty, color)
                table.insert(self.mine_regions, id)
            end
        end
    end,

    removeMineCircle = function(self)
        if self.mine_regions then
            for k,v in ipairs(self.mine_regions) do
                self.cs_inst:RemoveRegion(v)
            end
            self.mine_regions = nil
        end
    end,

    refreshSearchUI = function(self)
        local pre_btn = self.search_change_ui.transform:Find("layout/Button (0)") or self.search_change_ui.transform:GetChild(0):Find("layout/Button (0)")
        local next_btn = self.search_change_ui.transform:Find("layout/Button (1)") or self.search_change_ui.transform:GetChild(0):Find("layout/Button (1)")

        if search_info.searching then
            pre_btn:SetActive(search_info.search_idx ~= 1)
            next_btn:SetActive(not search_info:isLastHold())
        end
    end,

    addSearchBtnListener = function(self)
        local node_1 = self.search_change_ui.transform:Find("layout/Button (0)") or self.search_change_ui.transform:GetChild(0):Find("layout/Button (0)")
        local node_2 = self.search_change_ui.transform:Find("layout/Button (1)") or self.search_change_ui.transform:GetChild(0):Find("layout/Button (1)")
        local pre_btn = node_1:GetComponent(UnityEngine.UI.Button)
        local next_btn = node_2:GetComponent(UnityEngine.UI.Button)

        local addListener = function(btn, cb)
            btn.onClick:RemoveAllListeners()
            btn.onClick:AddListener(cb)
        end

        addListener(pre_btn, function() self:onClickPreBtn() end)
        addListener(next_btn, function() self:onClickNextBtn() end)
    end,

    onClickPreBtn = function(self)
        local hold = search_info:getPreHold()
        if hold then
            local cb = function()
                network_sender:queryWorldMap(hold.id, protocol_pb.MAP_WORLDMAP)
            end
            search_info.searching = true
            self:lookAtPosition(hold.coordX, hold.coordY, cb)
        end
    end,

    onClickNextBtn = function(self)
        local hold = search_info:getNextHold()
        if hold then
            local cb = function()
                network_sender:queryWorldMap(hold.id, protocol_pb.MAP_WORLDMAP)
            end
            search_info.searching = true
            self:lookAtPosition(hold.coordX, hold.coordY, cb)
        end
    end,

    resetSearchInfo = function(self)
        search_info:setCurrSearchInfo()
        if self.search_change_ui then
            self.search_change_ui:SetActive(false)
        end

        self:removeRegions()
    end,

    drawRegion = function(self, cells)
        for k, v in ipairs(cells) do
            local pos = Vector2(v.x, v.z)
            local color = Color(9/255, 177/255, 80/255, 0.25)
            local id = self.cs_inst:DrawRegion({pos}, color)
            table.insert( self.regions, id )
        end
    end,

    removeRegions = function(self)
        for k, v in ipairs(self.regions) do
            self.cs_inst:RemoveRegion(v)
        end

        self.regions = {}
    end,

    drawGuildBaseRegions = function(self, hold)
        for idx, _ in pairs(hold.range) do
            local x, z = self.cells:getXZ(idx)
            local pos = Vector2(x, z)
            local color = Color(167/255, 0, 13/255, 0.25)
            if world_map_helper:isOwnGuild(hold) then
                color = Color(9/255, 177/255, 80/255, 0.25)
            end
            local id = self.cs_inst:DrawRegion({pos}, color)
            self.guild_base_regions[idx] = id
        end
    end,

    drawExtendRegions = function(self, hold, list)
        for _, idx in pairs(list) do
            local x, z = self.cells:getXZ(idx)
            local pos = Vector2(x, z)
            local color = Color(167/255, 0, 13/255, 0.25)
            if world_map_helper:isOwnGuild(hold) then
                color = Color(9/255, 177/255, 80/255, 0.25)
            end
            local id = self.cs_inst:DrawRegion({pos}, color)
            self.guild_base_regions[idx] = id
        end
    end,

    removGuildBaseRegions = function(self, list)
        for idx, _ in pairs(list) do
            local id = self.guild_base_regions[idx]
            self.cs_inst:RemoveRegion(id)
            self.guild_base_regions[idx] = nil
        end
    end, 

    guildBaseDestroyed = function(self, hold)
        self:removGuildBaseRegions(hold.range)
        world_map_helper:resetRealGuildBaseRange(hold)
    end,

    holdNameChanged = function(self, id, name)
        local hold = self.hold_manager:getHoldFromId(id)
        if hold then
            hold:nameChanged(name)
        end
    end,

    destroyTempHold = function(self)
        if self.temp_hold then
            self.temp_hold:destroy()
            self.temp_hold = nil
            self.drag_hold = nil
            self:hideHoldOperator()
            self.begain_build = false
            self:removeTempRegions()
        end
    end,

    reload = function(self)
        if self.hold_manager then
            self.hold_manager:clear()
        end
    end,

    onExit = function(self)
        if self.hold_operator.is_show then
            self.hold_operator:hide()
        end

        if self.hold_manager then
            self.hold_manager:destroy() 
        end

        if self.pool_coroutine then
            app:stopCoroutine(self.pool_coroutine)
        end

        if self.pool then
            self.pool:destroy()
        end

        if self.pool_parent then
            UnityEngine.Object.Destroy(self.pool_parent.gameObject)
        end

        self.pool = nil
        self.pool_parent = nil
        self.cs_inst = nil
        self.root = nil
        self.cells = nil
        self.owner = nil
        self.my_hold = nil
        self.cs_inst = nil
        self.click_cell = nil
        self.events = nil
        self.hold_manager = nil
        self.hold_operator = nil
        world_map_helper.world_map = nil
    end,
}