local log = logger.get("Game", "World Map Battle")

local cls = class{

    ctor = function(self)
        self.waite_disappear = {}
        self.waite_appear = {}
        self.wm_bat_info = {}

        self.events = require("base.event").new()
    end,

    clear = function(self)
        self.wm_bat_info = {}

        for _, objs in pairs(self.waite_appear) do
            for _, obj in pairs(objs) do
                obj:destroy(true)
            end
        end

        for _, objs in pairs(self.waite_disappear) do
            for _, obj in pairs(objs) do
                obj:destroy(true)
            end
        end

        self.waite_disappear = {}
        self.waite_appear = {}
    end,

    setWorldMap = function(self, world_map)
        self.world_map = world_map
        self.hold_manager = world_map.hold_manager
    end,

	addBattleInfo = function(self, info)
        self.wm_bat_info[info.battleId] = info
        self.waite_appear[info.battleId] = {}
        self.waite_disappear[info.battleId] = {}

        self.events:dispatch("WORLDMAPBATTLE", info.battleId)
	end,

	removeBattleInfo = function(self, id)
		self.wm_bat_info[id] = nil
        self.waite_appear[id] = {}
        self.waite_disappear[id] = {}
    end,

    addWaiteDisappear = function(self, bat_id, obj)
        local list = self.waite_disappear[bat_id]

        if list then
            list[obj.id] = obj
        end
    end,

    removeWaiteDisappear = function(self, bat_id, id)
        local list = self.waite_disappear[bat_id]

        if list then
            list[id] = nil
        end
	end,

    addWaiteAppear = function(self, bat_id, obj)
        local list = self.waite_appear[bat_id]

        if list then
            list[obj.id] = obj
        end
    end,

    removeWaiteAppear = function(self, bat_id, id)
        local list = self.waite_appear[bat_id]

        if list then
            list[id] = nil
        end
	end,
    
    isBattleObj = function(self, id)
        return self.wm_bat_info[id] ~= nil
    end,

    battleFinished = function(self, id)
        local appear_list = self.waite_appear[id]
        local disappear_list = self.waite_disappear[id]

        if appear_list then
            for k, v in pairs(appear_list) do
                if v.info.totalTime - v.info.elapsedTime > 0 then
                    army_queue_manager:addDataByArmyId(v.id)
                else
                    army_queue_manager:removeDataByArmyId(v.id)
                end

                v:show()
            end
        end

        if disappear_list then
            for k, v in pairs(disappear_list) do
                army_queue_manager:removeQueue(v.id, true)
                v:destroy()
            end
        end

        self:removeBattleInfo(id)
    end,
}

world_map_battle = cls()

return world_map_battle