local log = logger.get("Game", "World Map")

local valid = "valid"
local invalid = "invalid"

return class {

    -- 索引从0开始
    ctor = function(self, max_x, max_z)
        self.max_x = max_x
        self.max_z = max_z

        self.buildings = {}
        

        --可以建造工厂的环
        self.mine_circles = {}

        self.block_idx = 0
        self.neighbors = {}
        self.blocks = {}
        self.length = 8
        self.block_num_x = math.ceil( max_x / 16 )
        self.block_num_z = math.ceil( max_z / 16 )
        for i=1, self.block_num_z do
            for j=1, self.block_num_x do
                local tab = {}
                tab.bottom_left_x = (i-1)*self.length
                tab.bottom_left_z = (j-1)*self.length
                tab.top_right_x = math.min(i*self.length, max_x)
                tab.top_right_z = math.min(j*self.length, max_z)
                table.insert(self.blocks, tab)
            end
        end
    end,

    getNeighborBlocks = function(self, idx)
        local tab = {}

        if idx > 0 then
            local addNeighbor = function(num)
                if num > 0 and num <= #self.blocks then
                    table.insert( tab, num, self.blocks[num] )
                end
            end

            local idxs = {}
            local length = self.block_num_x
            
            idxs[1] = idx - length - 1
            idxs[2] = idx - length
            idxs[3] = idx - length + 1
            idxs[4] = idx - 1
            idxs[5] = idx
            idxs[6] = idx + 1
            idxs[7] = idx + length - 1
            idxs[8] = idx + length
            idxs[9] = idx + length + 1

            for k, v in ipairs(idxs) do
                addNeighbor(v)
            end
        end

        return tab
    end,

    getBlockIdx = function(self, x, z)
        if x < 0 or z < 0 then
            return 0
        end

        local idx = math.floor(x/self.length)*self.block_num_x + math.floor(z/self.length) + 1
        return idx
    end,

    inBlocks = function(self, idx)
        for k, v in pairs(self.neighbors) do
            if idx == k then
                return true
            end
        end
        
        return false
    end,

    updateCenterBlock = function(self, x, z)
        local idx = self:getBlockIdx(x, z)
        if self.block_idx ~= idx then
            local new_neighbors = self:getNeighborBlocks(idx)
            local add_neighbors = {}
            local remove_neighbors = {}
            
            for k, v in pairs(new_neighbors) do
                if not self.neighbors[k] then
                    table.insert( add_neighbors, k )
                end
            end
            for k, v in pairs(self.neighbors) do
                if not new_neighbors[k] then
                    table.insert( remove_neighbors, k )
                end
            end
            self.block_idx = idx
            self.neighbors = new_neighbors

            return true, idx
        end

        return false
    end,

    -- 是否在圈圈内
    isValidInCircle = function(self, all_index)
        -- print("~~")
        for __, idx in ipairs(all_index) do
            local x, z = self:getXZ(idx)
            -- print('####', x, z, self.circle[idx])
            if self.circle[idx] == invalid then
                -- print('####', x, z, self.circle[idx])
            end
        end

        for __, idx in ipairs(all_index) do
            if self.circle[idx] == invalid then
                return false
            end
        end
        return true
    end,

    -- 是否在工会范围内
    isValidInGuildRange = function(self, all_index)
        local cfg = {}
        config.union_building_buildingdetailss:foreach(function(k, v)
            if v.level == world_map_info.guild_main_level and v.id == world_map_info.guild_main_cfg_id then
                cfg = v
            end
        end)
        local half_range = cfg.functionpara1
        local main_x = world_map_info.guild_main_x
        local main_z = world_map_info.guild_main_z
        for __, idx in ipairs(all_index) do
            local x, z = self:getXZ(idx)
            local v_x = main_x - x
            local v_z = main_z - z
            if math.abs(main_x - x) > half_range or math.abs(main_z - z) > half_range then
                return false
            end
        end
        return true
    end,

    -- 获取建筑会占有的格子
    getBuildingHoldIndexs = function(self, cfg_id, x, z, hold_type)
        local size = 0

        if hold_type == hold_types.guild then
            local cfg = config.union_union_buildings[cfg_id]
            size = cfg.gridnum
        elseif hold_type == hold_types.titanMine then
            local cfg = config.worldmap_titaniums[cfg_id]
            size = cfg.gridnum
        elseif hold_type == hold_types.monster then
            local cfg = config.worldmap_monsters[cfg_id]
            size = cfg.gridnum
        elseif hold_type == hold_types.player then
            local cfg = config.worldmap_base_configs[cfg_id]
            size = cfg.gridnum
        elseif hold_type == hold_types.temp then
            local cfg = config.union_union_buildings[cfg_id]
            size = cfg.gridnum
        end

        if size == 0 then
            log.error("未知建筑类型!!! << config.camp_buildings >>", cfg_id, hold_type)
        end

        local all_index = self:getAllIndex(size, x, z)
        
        return all_index
    end,

    getAllIndex = function(self, size, x, z)
        local all_index = {}
        if size == 1 then
            table.insert(all_index, self:getIndex(x, z))
        elseif size == 3 then
            table.insert(all_index, self:getIndex(x, z))
            if z % 2 == 0 then
                table.insert(all_index, self:getIndex(x - 1, z - 1))
                table.insert(all_index, self:getIndex(x, z - 1))
            else
                table.insert(all_index, self:getIndex(x, z - 1))
                table.insert(all_index, self:getIndex(x + 1, z - 1))
            end
        elseif size == 7 then
            table.insert(all_index, self:getIndex(x, z))
            if z % 2 == 0 then
                table.insert(all_index, self:getIndex(x + 1, z))
                table.insert(all_index, self:getIndex(x, z + 1))
                table.insert(all_index, self:getIndex(x, z - 1))
                table.insert(all_index, self:getIndex(x - 1, z))
                table.insert(all_index, self:getIndex(x - 1, z + 1))
                table.insert(all_index, self:getIndex(x - 1, z - 1))
            else
                table.insert(all_index, self:getIndex(x + 1, z))
                table.insert(all_index, self:getIndex(x + 1, z + 1))
                table.insert(all_index, self:getIndex(x + 1, z - 1))
                table.insert(all_index, self:getIndex(x - 1, z))
                table.insert(all_index, self:getIndex(x, z + 1))
                table.insert(all_index, self:getIndex(x, z - 1))
            end
        else
            log.error("未知建筑类型!!! size ", size)
        end

        return all_index
    end,

    getRingCfgId = function(self)
        local cfg = nil
        config.camp_buildings:foreach(function(k, v)
            if v.type == 5 then
                cfg = v
            end
        end)
        return cfg.buildareapara1
    end,

    -- 设置被建筑占有的格子
    setHoldCells = function(self, id, x, z, size)
        local all_index = self:getAllIndex(size, x, z)
        for __, idx in ipairs(all_index) do
            self.buildings[idx] = id
        end
    end,

    releasHoldCells = function(self, x, z, size)
        local all_index = self:getAllIndex(size, x, z)
        for __, idx in ipairs(all_index) do
            self.buildings[idx] = nil
        end
    end,

    isBuildingOnCell = function(self, all_index)
        local list = {}

        for __, idx in ipairs(all_index) do
            if self.buildings[idx] then
                list[idx] = true
            end
        end

        return #list == 0, list
    end,

    getIndex = function(self, x, z)
        if x < 0 or z < 0 then
            log.error("世界地图坐标 error: getIndex", x, z)
            return
        end
        local rst = x + z * self.max_x
        return rst
    end,

    getXZ = function(self, index)
        local x = index % self.max_x
        local z = index / self.max_x
        return x, z
    end,

    cellIsEmpty = function(self, x, z)
        local idx = self:getIndex(x, z)
        return self.buildings[idx]
    end,
}