local log = logger.get("Game", "world_map_info")

-- 服务器发来的世界地图信息
world_map_info = {
    clear = function(self)
		self.self_hold = nil
        self.player_holds = {}
        self.mine_holds = {}
        self.march_lines = {}
		self.monster_holds = {} -- MMonsterHold
		self.titanium_holds = {}
		self.guild_base_holds = {}
    end,
    
    self_hold = nil,    -- 本地玩家自己的据点
    player_holds = {},  -- MPlayerHold
    mine_holds = {},    -- MMineHold
    march_lines = {},   -- MMarchLine
	monster_holds = {}, -- MMonsterHold
	titanium_holds = {},
	guild_base_holds = {},

	guild_main_level = 0, -- 工会主基地等级
	guild_main_x = 0,
	guild_main_z = 0,
	guild_main_cfg_id = 0,

	enter_pos = nil,  -- 进入世界地图的初始坐标

	mass_sets = {},
	monster_atk_cnt = {},
	monster_unlock = {},

	setMassSetting = function(self, start, pwd, auto, share)
		self.mass_sets = {}
		self.mass_sets.memStart = start
		self.mass_sets.pwd = pwd
		self.mass_sets.maxAuto = auto
		self.mass_sets.shared = share
	end,

	setMonsterAtkCnt = function(self, data)
		for k, v in ipairs(data) do
			self.monster_atk_cnt[v.type] = v
		end
	end,

	updateMonsterUnlock = function(self, data)
		for k, v in ipairs(data) do
			self.monster_unlock[v.ItemTemId] = v.Count
		end
	end,

	getMonsterAtkLv = function(self, type)
		return self.monster_unlock[type]
	end,

	getAtkCnt = function(self, type)
		return self.monster_atk_cnt[type]
	end,

	getUnitType = function(self, id)
		if self.player_holds[id] then
			return 1
		elseif self.mine_holds[id] then
			return 2
		elseif self.march_lines[id] then
			return 3
		elseif self.monster_holds[id] then
			return 4
		end
	end,

	-- 设置基地等级
	setGuildMainInfo = function(self, level, cfg_id, x, z)
		if level <= 0 then
			log.error("set guild main level is invalid", self.guild_main_level)
			return
		end
		self.guild_main_level = level
		self.guild_main_x = x
		self.guild_main_z = z
		self.guild_main_cfg_id = cfg_id
	end,

	-- 获取基地等级
	getGuildMainLevel = function(self)
		if self.guild_main_level <= 0 then
			log.error("get guild main level is invalid", self.guild_main_level)
		else
			return self.guild_main_level
		end
	end,

	-- 获取工会建筑的建造时间
	getGuildBuildTime = function(self, id)
		if self.guild_holds[id] then
			return self.guild_holds[id].costTime
		else
			log.error("获取工会建筑的建造时间 error !!!", id)
		end
	end,
    
	ui = nil,   -- world map frame

	getGatherCarMarch = function(self, car_id)
		for k, v in pairs(self.march_lines) do
			if v.gatherCarId and v.gatherCarId == car_id then
				return v
			end
		end
	end,

    getServerId = function(self)
        return self.server_id or 1
    end,
}

--[[
//角色信息
message RoleInfo
{
    required string playerid = 1; //角色id
    required string rolename = 2; //角色名
    required uint32 level = 3; //	
}

//世界地图玩家据点
message MPlayerHold
{
	required string playerId = 1; //玩家id
	required uint32 homeLevel = 2; //基地等级
	required string playerName = 3; //玩家名字
	required uint32 posX = 4;
	required uint32 posY = 5;
}
//世界地图资源据点
message MMineHold
{
	required string resHoldId = 1;
	required uint32 resCfgId = 2;
	required uint32 resLevel = 3;
	required uint32 posX = 4;
	required uint32 posY = 5;
	required string ownerId = 6;
}
//行军线
message MMarchLine
{
	required string marchLineId = 1;
	required string playerId = 2;
	required uint32 srcPosX = 3;
	required uint32 srcPosY = 4;
	required uint32 dstPosX = 5;
	required uint32 dstPosY = 6;
	required uint32 marchSpeed = 7;
	required uint32 totalTime = 8;
	required uint32 curPos = 9;
	required uint32 elapsedTime = 10;
}
]]
