-- 所有的技能行为树定义
local log = logger.get("Game", "SKILL BT")
local actor_types = require('RTS.enums').actor_types
local buff_targets = require('RTS.enums').buff_targets

local root = require('bt.root')
local sequence = require("bt.sequence")
local sequence_success = require("bt.sequence_success")
local loop = require("bt.loop")
local loop_until = require("bt.loop_until")
local select = require("bt.select")
local parallel = require("bt.parallel")
local delay = require("bt.delay")
local action = require("bt.action")
local condition = require("bt.condition")
local random_select = require("bt.random_select")
local random_node = require("bt.random_node")

local shoot_bullet = nil
local shoot_bullet_simple = nil
local shoot_bullet_forward = nil
local facing_target = nil
local play_animation = nil
local facing_skill_direction = nil
local facing_skill_position = nil
local play_vfx = nil
local shoot_bullet_damage_in_area = nil
local shoot_bullet_to_enemy_field = nil
local do_damage_in_rect = nil
local do_damage_in_circle = nil
local rush = nil
local stop_rush = nil
local rush_forward = nil
local move_to_enemy = nil
local move_forward = nil
local casting_complete = nil
local add_buff = nil
local delay_px = nil
local finished = nil
local is_enemy_type = nil
local shake_camera = nil
local run_to_and_hit = nil
local stop_run_and_hit = nil
local is_hit_enemy = nil
local is_enemy_in_attack_range = nil
local begin_cd = nil
local set_camera_size_factor = nil
local set_avoidance_quality = nil
local set_avoidance_priority = nil
local select_multiple_enemy_by_range = nil
local select_multiple_enemy_randomly = nil
local set_enemy_from_multiple = nil
local show_skill_break_bar = nil
local shoot_laser = nil
local set_is_turning_weapon = nil
local call_func = nil
local cast_post_skill = nil
local play_sfx = nil
local play_vfx_if_have_bone = nil
local sneak = nil

if nil == battle_core_flag then
   -- 面向选定的敌人 ([容差角度] or 5， [超时时间] or 2)
   facing_target = require('RTS.skill_lib.facing_target')
   -- 面向技能选择方向 ([容差角度] or 5， [超时时间] or 2)
   facing_skill_direction = require('RTS.skill_lib.facing_skill_direction')
   -- 面向技能选择位置 ([容差角度] or 5， [超时时间] or 2)
   facing_skill_position = require('RTS.skill_lib.facing_skill_position')
   --[[ 发射一颗子弹 
   <1>shoot_effect_id, 
   <2> 骨骼名称, 
   <3> 飞行速度 or p(x), 
   <4> 弹道曲线 [nil],
   <5> 伤害半径 [0] or p(x)(对目标区域造成范围伤害),
   <6> 是否穿透[false]，
   <7> 伤害长度or p(x)[0],
   <8> 高度参数[0]（0：等比缩放，其他数：缩放比例）,
   <9> 摄像机动画 [nil]
   <10> stop 特效是否跟随目标 [false])
   --]] 
   shoot_bullet = require('RTS.skill_lib.shoot_bullet')
   shoot_bullet_simple = require('RTS.skill_lib.shoot_bullet_simple')
   -- 向战场正前方发射穿透子弹 (shoot_effect_id, 骨骼名称, 飞行速度 or p(x), 弹道曲线, 飞行距离, 伤害半径 or p(x)
   shoot_bullet_forward = require('RTS.skill_lib.shoot_bullet_forward')
   -- 播放动画 ([动画名称] or 默认攻击动画，是否调用方向)参数（弹道，发射点，速度，曲线文件，最大距离，伤害半径）
   play_animation = require('RTS.skill_lib.play_animation')
   -- 播放特效 (effect_libs_id, 绑定的骨骼名称,[参数1为路径名时采用，代表特效持续时间，如果参数1用的effectid则不使用],是否强制播放)
   play_vfx = require('RTS.skill_lib.play_vfx')
   -- 发射一颗子弹，在技能选择位置进行范围伤害 (shoot_effect_id, 骨骼名称, 飞行速度 or p(x), 弹道曲线, 伤害半径 or p(x),高度参数（0：等比缩放，其他数：固定高度数值）,震屏方式)
   shoot_bullet_damage_in_area = require('RTS.skill_lib.shoot_bullet_damage_in_area')
   -- 向敌人阵地随机位置发射子弹 (shoot_effect_id, 骨骼名称, 飞行速度 or p(x), 弹道曲线, 伤害半径 or p(x),高度参数（0：等比缩放，其他数：固定高度数值）,震屏方式)
   shoot_bullet_to_enemy_field = require('RTS.skill_lib.shoot_bullet_to_enemy_field')
   -- 以技能选择方向或玩家方向进行矩形范围伤害 (向前的偏移 or p(x), 范围长度 or p(x), 范围宽度 or p(x))
   do_damage_in_rect = require('RTS.skill_lib.do_damage_in_rect')    -- fOffset, fWidth, fLength
   -- 以技能选择位置或玩家位置进行圆形范围伤害 (向前的偏移 or p(x), 伤害半径 or p(x))
   do_damage_in_circle = require('RTS.skill_lib.do_damage_in_circle')
   -- 冲锋 (shoot_effect_id, 骨骼名称，速度，伤害半径，距离，超时时间，击退速度)
   rush = require('RTS.skill_lib.rush')
   -- 停止冲锋
   stop_rush = require('RTS.skill_lib.stop_rush')
   -- 向初始的前方向冲锋
   rush_forward = require('RTS.skill_lib.rush_forward')
   -- 向敌人移动
   move_to_enemy = require('RTS.skill_lib.move_to_enemy')
   -- 向前移动 参数 - 移动距离
   move_forward =  require('RTS.skill_lib.move_forward')
   -- 前摇完成，技能处于不可被打断状态
   casting_complete = require('RTS.skill_lib.casting_complete')
   -- 添加 buff (buff 的目标类型 见buff_targets枚举, buff id )
   add_buff = require('RTS.skill_lib.add_buff')
   -- delay p(x) 的时间, Example: delay_px('p3') // delay 技能表 p3 字段上配的时间长度
   delay_px = require('RTS.skill_lib.delay_px')
   -- 技能结束
   finished = require('RTS.skill_lib.finished')
   -- 判断选定敌人的类型 (类型整数)
   is_enemy_type = require('RTS.skill_lib.is_enemy_type')
   -- 震动屏幕
   shake_camera = require('RTS.skill_lib.shake_camera')
   -- 冲过去打一下(shoot_effect_id, 骨骼名称，速度，伤害半径，距离)
   run_to_and_hit = require('RTS.skill_lib.run_to_and_hit')
   -- 停止冲过去
   stop_run_and_hit = require('RTS.skill_lib.stop_run_and_hit')
   -- 是否打到人了
   is_hit_enemy = require('RTS.skill_lib.is_hit_enemy')
   -- 敌人是否在射程内
   is_enemy_in_attack_range = require('RTS.skill_lib.is_enemy_in_attack_range')
   -- 开始 CD 计算
   begin_cd = require('RTS.skill_lib.begin_cd')
   -- 设置摄像机尺寸
   set_camera_size_factor = require('RTS.skill_lib.set_camera_size_factor')
   -- 设置避让品质 0-不避让 1-low 4 -highest
   set_avoidance_quality = require('RTS.skill_lib.set_avoidance_quality')
   -- 设置避让优先级 值越小越硬
   set_avoidance_priority = require('RTS.skill_lib.set_avoidance_priority')
   -- 根据距离选择多个敌人 num-数量 only_attackable-只选择可攻击的
   select_multiple_enemy_by_range = require('RTS.skill_lib.select_multiple_enemy_by_range')
   -- 随机选择多个敌人 num-数量 only_attackable-只选择可攻击的
   select_multiple_enemy_randomly = require('RTS.skill_lib.select_multiple_enemy_randomly')
   -- 从多个敌人中选择一个作为当前敌人 index - 索引
   set_enemy_from_multiple = require('RTS.skill_lib.set_enemy_from_multiple')
   -- 显示技能打断条 total_time, left_to_right, inc_or_dec, is_in_order, skill_types
   -- 吟唱条的总时间，从左往右还是从右往左，填充还是渐消，是否按照顺序，要求的技能顺序
   -- show_skill_break_bar(10, true, true, true, {1,2,3,2})
   show_skill_break_bar = require('RTS.skill_lib.show_skill_break_bar')
   -- 发射激光 shoot_effect_id, bind_point, 
   shoot_laser = require('RTS.skill_lib.shoot_laser')
   --  设置武器是否旋转 is_turning - trun/false
   set_is_turning_weapon = require('RTS.skill_lib.set_is_turning_weapon')
   -- 调用一个自定义函数 func 
   call_func = require('RTS.skill_lib.call_func')
    -- 调用后续技能 skill_id 技能 id
   cast_post_skill = require('RTS.skill_lib.cast_post_skill')
   -- 播放音效 evt_name - 音效事件, on_self - 是否在自己身上播
   play_sfx = require('RTS.skill_lib.play_sfx')
   -- 参数同 play_vfx,当 bone 找不到时不报错
   play_vfx_if_have_bone = require('RTS.skill_lib.play_vfx_if_have_bone')
   -- 潜行 参数 - is_sneak 是否在潜行
   sneak = require('RTS.skill_lib.sneak')
else
   shoot_bullet = require('RTS.skill_lib.svr_shoot_bullet')
   facing_target = require('RTS.skill_lib.svr_facing_target')
   facing_skill_direction = require('RTS.skill_lib.svr_facing_skill_direction')
   facing_skill_position = require('RTS.skill_lib.svr_facing_skill_position')
   play_animation = require('RTS.skill_lib.svr_play_animation')
   play_vfx = require('RTS.skill_lib.svr_play_vfx')
   shoot_bullet_damage_in_area = require('RTS.skill_lib.svr_shoot_bullet_damage_in_area')
   shoot_bullet_to_enemy_field = require('RTS.skill_lib.svr_shoot_bullet_to_enemy_field')
   do_damage_in_rect = require('RTS.skill_lib.svr_do_damage_in_rect')    -- fOffset, fWidth, fLength
   do_damage_in_circle = require('RTS.skill_lib.svr_do_damage_in_circle')
   rush = require('RTS.skill_lib.svr_rush')
   move_to_enemy = require('RTS.skill_lib.svr_move_to_enemy')
   casting_complete = require('RTS.skill_lib.svr_casting_complete')
   add_buff = require('RTS.skill_lib.svr_add_buff')
   finished = require('RTS.skill_lib.svr_finished')
   is_enemy_type = require('RTS.skill_lib.svr_is_enemy_type')
   shake_camera = require('RTS.skill_lib.svr_shake_camera')
   run_to_and_hit = require('RTS.skill_lib.svr_run_to_and_hit')
   stop_run_and_hit = require('RTS.skill_lib.svr_stop_run_and_hit')
   is_hit_enemy = require('RTS.skill_lib.svr_is_hit_enemy')
   is_enemy_in_attack_range = require('RTS.skill_lib.svr_is_enemy_in_attack_range')
   begin_cd = require('RTS.skill_lib.svr_begin_cd')
   set_camera_size_factor = require('RTS.skill_lib.svr_set_camera_size_factor')

end

return {

    -- 坦克普攻
    [1] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   
                    :add(play_animation('attack'))  -- 播放指定名称动画
                    :add(shoot_bullet(1, 'shoot', 200, consts.assets.curves.curve))        -- 射击子弹 prefab, speed, damage
                    :add(delay(1.2))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,

	
    -- 挖掘机杂兵普攻
    [3] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   
                    :add(play_vfx('Assets/Arts/fx/prefab/3_Prefab_Heavy_Excavator/skill3_huichen.prefab', 'lt_1', 1))        -- 播放灰尘特效，1秒消失
                    :add(play_vfx('Assets/Arts/fx/prefab/3_Prefab_Heavy_Excavator/skill3_zhengqi.prefab', 'Dummy002', 1))        -- 播放蒸汽特效，1秒消失
                    :add(play_animation('attack'))  -- 播放指定名称动作
                    :add(delay(1.0))
                    :add(play_vfx('Assets/Arts/fx/prefab/3_Prefab_Heavy_Excavator/skill3_start.prefab', 'shoot', 1))        -- 播放启动特效，1秒消失
                    --:add(play_vfx(33, 'shoot', 1))        -- 播放受击特效，1秒消失
                    :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                    :add(delay(1.0))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,
	
	
    -- 挖掘机杂兵技能攻击
    [10003] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   
                    :add(play_vfx('Assets/Arts/fx/prefab/3_Prefab_Heavy_Excavator/skill10003_focus.prefab', 'shoot', 1,true))	-- 播放聚焦特效，1秒消失
                    :add(play_vfx('Assets/Arts/fx/prefab/3_Prefab_Heavy_Excavator/skill10003_huichen.prefab', 'lt_1', 1,true))	-- 播放灰尘特效，1秒消失
                    :add(play_animation('attack_1_rush'))	-- 播放指定名称动画
                    :add(delay(1.0))
                    :add(casting_complete())	--施法动作播放完毕，之后不能被打断
                    :add(play_vfx('Assets/Arts/fx/prefab/3_Prefab_Heavy_Excavator/skill10003_zhengqi.prefab', 'Chimney', 1,true))	-- 播放灰尘特效，1秒消失
                    :add(delay(0.17))
                    :add(play_animation('attack_1_loop'))  -- 播放指定名称动画
                    :add(play_vfx('Assets/Arts/fx/prefab/3_Prefab_Heavy_Excavator/skill10003_start.prefab', 'shoot', 1,true))	-- 播放灰尘特效，1秒消失
                    :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                    :add(delay(0.13))
                    --:add(play_vfx(100033, 'shoot', 1))	-- 播放命中特效，1秒消失
                    :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                    :add(delay(0.2))
                    --:add(play_vfx(100033, 'shoot', 1))	-- 播放命中特效，1秒消失
                    :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                    :add(delay(0.2))
                    :add(play_animation('attack_1_back'))  -- 播放指定名称动画
                    :add(delay(0.5))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,

    -- 挖掘机BOSS技能攻击
    [11003] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   
                    :add(play_vfx(100034, 'shoot', 1))	-- 播放聚焦特效，1秒消失
                    :add(play_vfx(100035, 'lt_1', 1))	-- 播放灰尘特效，1秒消失
                    :add(play_animation('attack_1_rush'))	-- 播放指定名称动画
                    :add(delay(1.0))
                    :add(show_skill_break_bar(8, true, true, false, {4,4}))
                    :add(delay(8.0))
                    :add(casting_complete())	--施法动作播放完毕，之后不能被打断
                    :add(play_animation('attack_1_loop'))  -- 播放指定名称动画
                    :add(play_vfx(100036, 'Dummy002', 1))	-- 播放灰尘特效，1秒消失
                    :add(delay(0.17))
                    :add(play_vfx(100031, 'shoot', 1))	-- 播放发动特效，1秒消失
                    :add(delay(0.03))
                    --:add(play_vfx(100033, 'shoot', 1))	-- 播放命中特效，1秒消失
                    :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                    :add(delay(0.15))
                    --:add(play_vfx(100033, 'shoot', 1))	-- 播放命中特效，1秒消失
                    :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                    :add(delay(0.15))
                    --:add(play_vfx(100033, 'shoot', 1))	-- 播放命中特效，1秒消失
                    :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                    :add(delay(0.15))
                    --:add(play_vfx(100033, 'shoot', 1))	-- 播放命中特效，1秒消失
                    :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                    :add(delay(0.15))
                    --:add(play_vfx(100033, 'shoot', 1))	-- 播放命中特效，1秒消失
                    :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                    :add(delay(0.15))
                    --:add(play_vfx(100033, 'shoot', 1))	-- 播放命中特效，1秒消失
                    :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                    :add(delay(0.05))
                    :add(play_animation('attack_1_back'))  -- 播放指定名称动画
                    :add(delay(0.5))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,
    
		-- 弱智圈圈的普攻
    [1002] = function(actor)
            if not actor then 
                log.error('actor is nil!!')
                return nil
            end
    
            local bt = root():setChild(
                    sequence()   -- 选择
                        :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   

                        :add(finished())            -- 技能结束
                        )
    
            bt:setContext(actor)
            --log.info('create common bt! agent:'..tostring(agent.id))
            return bt
    end,
	
    -- 挖掘机杂兵技能攻击
    --[[[10003] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   
                    :add(play_animation('attack1'))  -- 播放指定名称动画
					:add(casting_complete())	--施法动作播放完毕，之后不能被打断
					:add(delay(0.3))
                    :add(play_animation('loop'))  -- 播放指定名称动画
					:add(casting_complete())	--施法动作播放完毕，之后不能被打断
                    :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
					:add(delay(0.2))
                    :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
					:add(delay(0.2))
                    :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
					:add(delay(0.1))
					:add(play_animation('back'))  -- 播放指定名称动画
					:add(casting_complete())	--施法动作播放完毕，之后不能被打断
					:add(delay(0.7))  
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,]]
	
	
	[101] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时 
                    :add(play_animation('attack'))  -- 播放指定名称动画
                    :add(shoot_bullet(1, 'shoot', 200, consts.assets.curves.curve))        -- 射击子弹 prefab, speed, damage
                    :add(delay(1.2))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,

	    -- 并列式喷火器普攻
    [6] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时
                    :add(play_animation('attack'))  -- 播放指定名称动画
                    :add(shoot_bullet(6, 'shoot', 200, consts.assets.curves.curve))        -- 射击子弹 prefab, speed, damage
					:add(delay(0.8))
					:add(play_animation('attack'))  -- 播放指定名称动画
                    :add(shoot_bullet(6, 'shoot', 200, consts.assets.curves.curve))        -- 射击子弹 prefab, speed, damage
					:add(delay(0.8))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,

	-- 向战车真前方喷射火焰，对被波及的单位每秒造成(p1%)攻击的伤害，持续4秒。y
	-- 战车动作为晃动喷火管子，形成扇形的火焰区域，火焰区域的角度为60%，长度为15码
    [10006] = function(actor)
        if not actor then 
           log.error('actor is nil!!')
           return nil
       end

       local bt = root():setChild(
               sequence()   -- 测试
                   :add(facing_target(5, 4))   -- 最大差1度开始执行后续逻辑并且转向过程不超过2秒   对象技能的朝向
                   :add(casting_complete())	--施法动作播放完毕，之后不能被打断	                    
                   :add(play_animation('attack_1'))  -- 播放指定攻击动作
                   :add(play_vfx('Assets/Arts/fx/prefab/start_yanwu_10006.prefab', 'y1', 1))
                   :add(play_vfx('Assets/Arts/fx/prefab/focus_6.prefab', 'shoot',3))
                   :add(delay(2.35))
                   :add(play_vfx(100061, 'shoot'))        -- 播放特效
                   --:add(shoot_bullet(10007, 'shoot', 20, nil,'p3', false, 'p4')) 					-- 发射一颗子弹 (shoot_effect_id, 骨骼名称, 飞行速度 or p(x), 弹道曲线,伤害半径 or p(x)(对目标区域造成范围伤害),是否穿透，伤害长度or p(x))
                   :add(loop(4)
                       :add(do_damage_in_rect(1, 'p3', 'p4'))
                       :add(delay(0.8))
                       )
                   :add(delay(0.5))
                   :add(finished())            -- 技能结束
                   )

       bt:setContext(actor)
       --log.info('create common bt! agent:'..tostring(agent.id))
       return bt
    end,	
    
    [106] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时
                    :add(play_animation('attack'))  -- 播放指定名称动画
                    :add(shoot_bullet(6, 'shoot', 200, consts.assets.curves.curve))        -- 射击子弹 prefab, speed, damage
                    :add(delay(1.2))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,

	
	    -- 肉盾战车普攻
    [7] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时
                    :add(play_animation('attack'))  -- 播放指定名称动画
                    :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                    :add(delay(1.2))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,
	
    [107] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时
                    :add(play_animation('attack'))  -- 播放指定名称动画
                    :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                    :add(delay(1.2))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,
	
	    -- KV2坦克炮塔的普攻
    [8] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时
                    :add(play_animation('attack',true))  -- 播放指定名称动画
					:add(delay(0.1))
                    :add(shoot_bullet(8, 'shoot', 500, consts.assets.curves.curve8))        -- 射击子弹 prefab, speed, damage
                    :add(play_vfx('Assets/Arts/fx/prefab/start_yanwu_pugong.prefab', 'lt_1'))
					:add(delay(1.2))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,	

    -- KV2坦克炮塔的技能：强力射击:对目标发射一次强力的穿甲弹，造成(p1)%攻击的穿甲伤害，如果目标是肉盾型战车，则造成额外(p2)%的伤害
    [10008] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(select_multiple_enemy_randomly(3,false))
					:add(set_enemy_from_multiple(1))
					:add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时
					:add(casting_complete())	--施法动作播放完毕，之后不能被打断
					:add(play_vfx('Assets/Arts/fx/prefab/focus_6.prefab', 'shoot'))        -- 播放特效					
                    :add(play_animation('attack_1',true))  -- 播放指定攻击动作
					:add(play_vfx('Assets/Arts/fx/prefab/start_yanwu_qibu_daoc_2.prefab', 'lt_1'))        -- 播放特效	
					:add(delay(1.8))
                    :add(shoot_bullet(10008, 'shoot', 450, consts.assets.curves.curve10008,0, false, 0,0,"zhenping_1"))        -- 发射一颗子弹 (shoot_effect_id, 骨骼名称, 飞行速度 or p(x), 弹道曲线,伤害半径 or p(x)(对目标区域造成范围伤害),是否穿透，伤害长度or p(x))
                    :add(play_vfx('Assets/Arts/fx/prefab/start_yanwu.prefab', 'shoot'))        -- 播放特效
					:add(set_enemy_from_multiple(2))
					:add(facing_target(5, 4))
					:add(play_animation('attack_1',true))  -- 播放指定攻击动作
					:add(play_vfx('Assets/Arts/fx/prefab/start_yanwu_qibu_daoc_2.prefab', 'lt_1'))        -- 播放特效	
					:add(delay(1.8))
					:add(shoot_bullet(10008, 'shoot', 450, consts.assets.curves.curve10008,0, false, 0,0,"zhenping_1"))
					:add(set_enemy_from_multiple(3))
					:add(facing_target(5, 4))
					:add(play_animation('attack_1',true))  -- 播放指定攻击动作
					:add(play_vfx('Assets/Arts/fx/prefab/start_yanwu_qibu_daoc_2.prefab', 'lt_1'))        -- 播放特效	
					:add(delay(1.8))
					:add(shoot_bullet(10008, 'shoot', 450, consts.assets.curves.curve10008,0, false, 0,0,"zhenping_1"))
					:add(add_buff(buff_targets.SELF,7))					
					:add(delay(1.6))            --等待一段时间后清除特效
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,

    [108] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时
                    :add(play_animation('attack'))  -- 播放指定名称动画
                    :add(shoot_bullet(8, 'shoot', 150, consts.assets.curves.curve))        -- 射击子弹 prefab, speed, damage
                    :add(delay(1.2))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,		
	
	
	[110] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   
                    :add(play_animation('attack'))  -- 播放指定名称动画
                    --:add(play_vfx('Assets/Arts/fx/prefab/10_Prefab_Artillery_German42/skill10_start.prefab', 'shoot'))        -- 播放特效
					:add(delay(0.8))
                    :add(shoot_bullet(10, 'shoot', 150, consts.assets.curves.curve1))        -- 射击子弹 prefab, speed, damage
                    :add(delay(1.2))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,
	
	--普通攻击随机调用动作
    [12] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

		local i=math.random(1,100)
	
		if i>=50 then
		
			local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时 					
                    :add(play_animation('attack'))  -- 播放指定名称动画
					:add(delay(0.3))
                    :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
					:add(delay(0.1))
					:add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
					:add(delay(0.1))
					:add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                    :add(delay(2.33))
                    :add(finished())            -- 技能结束
                    )
					
					bt:setContext(actor)
					return bt
			
			else
			local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时 					
                    :add(play_animation('attack_r1'))  -- 播放指定名称动画
					:add(delay(0.3))
                    :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
					:add(delay(0.1))
					:add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
					:add(delay(0.1))
					:add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                    :add(delay(2.33))

					-- 播放back1
					--:add(play_animation('back1'))  -- 播放指定名称动画
                    --:add(delay(3.16))

                    :add(finished())            -- 技能结束
                    )
					
			bt:setContext(actor)
        return bt
		end
    end,	

    [112] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时 					
                    :add(play_animation('attack_2'))  -- 播放指定名称动画
					:add(delay(0.3))
                    :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
					:add(delay(0.1))
					:add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
					:add(delay(0.1))
					:add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                    :add(delay(0.5))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,
	
		-- 毒筒投射器：普攻
    [14] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   
                    :add(play_animation('attack'))  -- 播放指定名称动画
					:add(delay(0.2))
                    :add(shoot_bullet(14, 'shoot', 80, consts.assets.curves.curve14))        -- 射击子弹 prefab, speed, damage
                    :add(delay(1))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,
    
    -- 毒筒投射器：毒云密布-向敌方后排发射一枚大型毒气桶，对敌方后排造成一次(p1%)攻击的毒性伤害并附加一个持续造成伤害的DOT
    [10014] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 测试
                    :add(facing_target(5, 4))   -- 最大差1度开始执行后续逻辑并且转向过程不超过2秒   对象技能的朝向
                    :add(casting_complete())	--施法动作播放完毕，之后不能被打断
                    :add(play_animation('attack_1'))  -- 播放指定攻击动作					
                    :add(play_vfx('Assets/Arts/fx/prefab/focus_2.prefab', 'shoot',3))
                    :add(play_vfx('Assets/Arts/fx/prefab/start_yanwu_focus_10014.prefab', 'shoot', 3))        --播放聚焦动作伴随烟雾特效
                    -- :add(delay(1.2)) --时间根据动作时间来填
                    :add(play_vfx('Assets/Arts/fx/prefab/14_Prefab_Artillery_Nvpu/skill10014_start.prefab', 'shoot', 3))        --播放毒球掉落特效
                    :add(delay(1.5)) --时间根据动作时间来填                    
                    :add(select_multiple_enemy_randomly(3,false))
                    :add(set_enemy_from_multiple(1))
                    :add(shoot_bullet(10014, 'shoot', 80, consts.assets.curves.curve10014))
                    :add(facing_target())
                    :add(delay(0.5))
                    :add(set_enemy_from_multiple(2))
                    :add(shoot_bullet(10014, 'shoot', 80, consts.assets.curves.curve10014))
                    :add(facing_target())
                    :add(delay(0.5))
                    :add(set_enemy_from_multiple(3))
                    :add(shoot_bullet(10014, 'shoot', 80, consts.assets.curves.curve10014))
                    :add(delay(0.8))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt 	
    end,

		-- 信标发射器：普攻
    [15] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   
                    :add(play_animation('attack'))  -- 播放指定名称动画
					:add(delay(0.15))
                    :add(shoot_bullet(15, 'shoot', 150, consts.assets.curves.curve10015))        -- 射击子弹 prefab, speed, damage
                    :add(delay(1))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,

    -- 信标发射器：信标投射-向敌方阵地的随机3个目标发射攻击信标，延迟p3秒后召唤导弹对其进行造成(p1%)的爆炸伤害。
    [10015] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   
                    :add(play_animation('attack_1'))  -- 播放指定攻击动作
					:add(casting_complete())	--施法动作播放完毕，之后不能被打断					
                    :add(shoot_bullet(10015, 'shoot', 150, consts.assets.curves.curve1,0, false, 0))        --发射一颗子弹 (shoot_effect_id, 骨骼名称, 飞行速度 or p(x), 弹道曲线,伤害半径 or p(x)(对目标区域造成范围伤害),是否穿透，伤害长度or p(x))
                    :add(delay(1.2))            --等待一段时间后清除特效
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,
	

		-- 歼星者反装甲导弹：普攻
    [17] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   
                    :add(play_animation('attack'))  -- 播放指定名称动画
                    :add(delay(0.2))
                    :add(play_vfx('Assets/Arts/fx/prefab/17_Prefab_Artillery_Moxi/skill17_smoke.prefab', 'lt_1', 1))        -- 播放蒸汽特效，1秒消失
                    :add(delay(0.05))
                    :add(shoot_bullet(17, 'shoot', 100, consts.assets.curves.curve17))        -- 射击子弹
                    :add(delay(1))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,

	-- 歼星者反装甲导弹:技能
    [10017] = function(actor)
        if not actor then
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(select_multiple_enemy_randomly(3,false))
                    :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   
                    :add(casting_complete())	--施法动作播放完毕，之后不能被打断
                    :add(play_animation('attack_1'))  -- 播放指定名称动画
                    :add(play_vfx('Assets/Arts/fx/prefab/17_Prefab_Artillery_Moxi/skill10017_fuzhu_zhengqi.prefab', 'Chimney_1',5,true))        -- 播放特效
                    :add(play_vfx('Assets/Arts/fx/prefab/17_Prefab_Artillery_Moxi/skill10017_fuzhu_zhengqi.prefab', 'Chimney_2',5,true))        -- 播放特效
                    :add(play_vfx('Assets/Arts/fx/prefab/17_Prefab_Artillery_Moxi/skill10017_fuzhu_zhengqi.prefab', 'Chimney_3',5,true))        -- 播放特效
                    :add(play_vfx('Assets/Arts/fx/prefab/17_Prefab_Artillery_Moxi/skill10017_fuzhu_zhengqi.prefab', 'Chimney_4',5,true))        -- 播放特效
                    :add(set_enemy_from_multiple(1))
                    :add(delay(0.73))
                    :add(shoot_bullet(100171, 'shoot_3', 80, consts.assets.curves.curve10017_1))        -- 射击子弹
                    :add(delay(0.02))
                    :add(shoot_bullet(100171, 'shoot_2', 100, consts.assets.curves.curve10017_1))        -- 射击子弹
                    :add(delay(0.05))
                    :add(shoot_bullet(100171, 'shoot_5', 120, consts.assets.curves.curve10017_2))        -- 射击子弹
                    :add(delay(0.04))
                    :add(shoot_bullet(100171, 'shoot_1', 95, consts.assets.curves.curve10017_4))        -- 射击子弹
                    :add(delay(0.04))
                    :add(shoot_bullet(100171, 'shoot_5', 90, consts.assets.curves.curve10017_3))        -- 射击子弹
                    :add(delay(0.09))
                    :add(shoot_bullet(100171, 'shoot_4', 85, consts.assets.curves.curve10017_1))        -- 射击子弹
                    :add(set_enemy_from_multiple(2))
                    :add(delay(0.06))
                    :add(shoot_bullet(100171, 'shoot_3', 90, consts.assets.curves.curve10017_2))        -- 射击子弹
                    :add(delay(0.04))
                    :add(shoot_bullet(100171, 'shoot_2', 110, consts.assets.curves.curve10017_2))        -- 射击子弹
                    :add(delay(0.05))
                    :add(shoot_bullet(100171, 'shoot_4', 115, consts.assets.curves.curve10017_2))        -- 射击子弹
                    :add(delay(0.04))
                    :add(shoot_bullet(100171, 'shoot_1', 95, consts.assets.curves.curve10017_3))        -- 射击子弹
                    :add(delay(0.06))
                    :add(shoot_bullet(100171, 'shoot_5', 100, consts.assets.curves.curve10017_1))        -- 射击子弹
                    :add(delay(0.05))
                    :add(shoot_bullet(100171, 'shoot_4', 90, consts.assets.curves.curve10017_3))        -- 射击子弹
                    :add(set_enemy_from_multiple(3))
                    :add(delay(0.06))
                    :add(shoot_bullet(100171, 'shoot_1', 85, consts.assets.curves.curve10017_1))        -- 射击子弹
                    :add(delay(0.05))
                    :add(shoot_bullet(100171, 'shoot_5', 95, consts.assets.curves.curve10017_4))        -- 射击子弹
                    :add(delay(0.08))
                    :add(shoot_bullet(100171, 'shoot_3', 105, consts.assets.curves.curve10017_3))        -- 射击子弹
                    :add(delay(0.08))
                    :add(shoot_bullet(100171, 'shoot_2', 90, consts.assets.curves.curve10017_3))        -- 射击子弹
                    :add(delay(0.05))
                    :add(shoot_bullet(100171, 'shoot_4', 110, consts.assets.curves.curve10017_4))        -- 射击子弹
                    :add(delay(0.06))
                    :add(shoot_bullet(10017, 'shoot', 120, consts.assets.curves.curve10017))        -- 射击子弹
                    :add(play_vfx('Assets/Arts/fx/prefab/17_Prefab_Artillery_Moxi/skill10017_smoke.prefab', 'lt_1',5,true))        -- 播放特效
                    :add(delay(1.8))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,

    -- 向上发炮
    [18] = function(actor)
        if not actor then
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
            sequence()   -- 选择
                :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   
                :add(casting_complete())	--施法动作播放完毕，之后不能被打断
                :add(play_animation('attack'))  -- 播放指定名称动画
                :add(play_vfx('Assets/Arts/fx/prefab/28_Prefab_Tank_Luke/skill10028_xuli_cheti.prefab', 'lt_1',5,true))        -- 播放特效
                :add(delay(1.53))
                :add(shoot_bullet_to_enemy_field(10028, 'shoot', 10, consts.assets.curves.curve10028, 1,140))
                :add(delay(1.5))
                :add(add_buff(buff_targets.ALL_TEAMATE_VEHICLE,8))	
                :add(finished())            -- 技能结束
                )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,

    -- 转轮火神炮：普攻
    [19] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   
                    :add(play_animation('attack'))  -- 播放指定名称动画
                    -- :add(play_vfx('Assets/Arts/fx/prefab/17_Prefab_Artillery_Moxi/skill17_smoke.prefab', 'lt_1', 1))        -- 播放蒸汽特效，1秒消失
                    :add(shoot_bullet(19, 'shoot', 200, consts.assets.curves.curve19))        -- 射击子弹
                    :add(delay(0.1))
                    :add(shoot_bullet(19, 'shoot', 200, consts.assets.curves.curve19))        -- 射击子弹
                    :add(delay(0.2))
                    :add(shoot_bullet(19, 'shoot', 200, consts.assets.curves.curve19))        -- 射击子弹
                    :add(delay(0.2))
                    :add(shoot_bullet(19, 'shoot', 200, consts.assets.curves.curve19))        -- 射击子弹
                    :add(delay(0.2))
                    :add(shoot_bullet(99919, 'shoot', 200, consts.assets.curves.curve19))        -- 射击子弹
                    :add(delay(0.2))
                    :add(shoot_bullet(99919, 'shoot', 200, consts.assets.curves.curve19))        -- 射击子弹
                    :add(delay(0.2))
                    :add(shoot_bullet(99919, 'shoot', 200, consts.assets.curves.curve19))        -- 射击子弹
                    :add(delay(1))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,

    -- 转轮火神炮:技能火神怒吼
    [10019] = function(actor)
        if not actor then
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   
                    :add(casting_complete())	--施法动作播放完毕，之后不能被打断
                    :add(play_animation('attack_1'))  -- 播放指定名称动画
                    :add(play_vfx('Assets/Arts/fx/prefab/19_Prefab_Assult_Palisitong/skill10019_cheyan.prefab', 'foot_1',3,true))        -- 播放特效
                    :add(play_vfx('Assets/Arts/fx/prefab/19_Prefab_Assult_Palisitong/skill10019_cheyan.prefab', 'foot_2',3,true))        -- 播放特效
                    :add(play_vfx('Assets/Arts/fx/prefab/19_Prefab_Assult_Palisitong/skill10019_xuanfeng.prefab', 'y_1',3,true))        -- 播放特效
                    :add(play_vfx('Assets/Arts/fx/prefab/19_Prefab_Assult_Palisitong/skill10019_xuli.prefab', '112',3,true))        -- 播放特效
                    :add(play_vfx('Assets/Arts/fx/prefab/19_Prefab_Assult_Palisitong/skill10019_xuanfeng.prefab', 'y_2',3,true))        -- 播放特效
                    :add(delay(1))
                    :add(shoot_bullet(10019, 'shoot', 200, consts.assets.curves.curve10019))        -- 射击子弹
                    :add(play_vfx('Assets/Arts/fx/prefab/19_Prefab_Assult_Palisitong/skill10019_loop.prefab', 'shoot',3,true))        -- 播放特效
                    :add(delay(0.1))
                    :add(shoot_bullet(99919, 'shoot', 200, consts.assets.curves.curve19))        -- 射击子弹
                    :add(delay(0.1))
                    :add(shoot_bullet(99919, 'shoot', 200, consts.assets.curves.curve19))        -- 射击子弹
                    :add(delay(0.1))
                    :add(shoot_bullet(99919, 'shoot', 200, consts.assets.curves.curve19))        -- 射击子弹
                    :add(delay(0.1))
                    :add(shoot_bullet(99919, 'shoot', 200, consts.assets.curves.curve19))        -- 射击子弹
                    :add(delay(0.1))
                    :add(shoot_bullet(99919, 'shoot', 200, consts.assets.curves.curve19))        -- 射击子弹
                    :add(delay(0.1))
                    :add(shoot_bullet(99919, 'shoot', 200, consts.assets.curves.curve19))        -- 射击子弹
                    :add(delay(0.1))
                    :add(shoot_bullet(99919, 'shoot', 200, consts.assets.curves.curve19))        -- 射击子弹
                    :add(delay(0.1))
                    :add(shoot_bullet(99919, 'shoot', 200, consts.assets.curves.curve19))        -- 射击子弹
                    :add(delay(0.1))
                    :add(shoot_bullet(99919, 'shoot', 200, consts.assets.curves.curve19))        -- 射击子弹
                    :add(delay(0.1))
                    :add(shoot_bullet(99919, 'shoot', 200, consts.assets.curves.curve19))        -- 射击子弹
                    :add(delay(0.1))
                    :add(shoot_bullet(99919, 'shoot', 200, consts.assets.curves.curve19))        -- 射击子弹
                    :add(delay(0.1))
                    :add(shoot_bullet(99919, 'shoot', 200, consts.assets.curves.curve19))        -- 射击子弹
                    :add(delay(0.1))
                    :add(shoot_bullet(99919, 'shoot', 200, consts.assets.curves.curve19))        -- 射击子弹
                    :add(delay(0.1))
                    :add(shoot_bullet(99919, 'shoot', 200, consts.assets.curves.curve19))        -- 射击子弹
                    :add(delay(0.1))
                    :add(shoot_bullet(99919, 'shoot', 200, consts.assets.curves.curve19))        -- 射击子弹
                    :add(delay(0.1))
                    :add(shoot_bullet(99919, 'shoot', 200, consts.assets.curves.curve19))        -- 射击子弹
                    :add(delay(0.1))
                    :add(shoot_bullet(99919, 'shoot', 200, consts.assets.curves.curve19))        -- 射击子弹
                    :add(delay(0.1))
                    :add(shoot_bullet(99919, 'shoot', 200, consts.assets.curves.curve19))        -- 射击子弹
                    :add(delay(0.1))
                    :add(shoot_bullet(99919, 'shoot', 200, consts.assets.curves.curve19))        -- 射击子弹
                    :add(delay(0.1))
                    :add(shoot_bullet(99919, 'shoot', 200, consts.assets.curves.curve19))        -- 射击子弹
                    :add(delay(0.1))
                    :add(shoot_bullet(99919, 'shoot', 200, consts.assets.curves.curve19))        -- 射击子弹
                    :add(delay(0.1))
                    :add(shoot_bullet(99919, 'shoot', 200, consts.assets.curves.curve19))        -- 射击子弹
                    :add(delay(0.1))
                    :add(shoot_bullet(99919, 'shoot', 200, consts.assets.curves.curve19))        -- 射击子弹
                    :add(delay(1))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,
    
	-- 收割者滚木：普攻
    [20] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时
                    :add(play_animation('attack'))  -- 播放指定名称动画
                    :add(play_vfx('Assets/Arts/fx/prefab/20_Prefab_Heavy_Manghan/skill20_start_mts.prefab', 'rightwuqi', 1))        -- 播放蒸汽特效，1秒消失
                    :add(play_vfx('Assets/Arts/fx/prefab/20_Prefab_Heavy_Manghan/skill20_start_mts.prefab', 'leftwuqi', 1))        -- 播放蒸汽特效，1秒消失
                    :add(play_vfx('Assets/Arts/fx/prefab/20_Prefab_Heavy_Manghan/skill20_start_mtb.prefab', 'midwuqi', 1))        -- 播放蒸汽特效，1秒消失
                    :add(delay(0.73))
                    :add(play_vfx('Assets/Arts/fx/prefab/20_Prefab_Heavy_Manghan/skill20_fuzhu_smoke.prefab', 'lt_1', 1))        -- 播放蒸汽特效，1秒消失
                    :add(play_vfx('Assets/Arts/fx/prefab/20_Prefab_Heavy_Manghan/skill20_gongji.prefab', 'ground', 1))        -- 播放蒸汽特效，1秒消失
                    :add(play_vfx('Assets/Arts/fx/prefab/20_Prefab_Heavy_Manghan/skill20_hit.prefab', 'ground', 1))        -- 播放蒸汽特效，1秒消失
                    :add(shoot_bullet(90001, 'shoot', 200)) 
                    :add(delay(0.8))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt 
    end,	

	-- 收割者滚木的技能
    [10020] = function(actor)
        if not actor then
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选光
                    :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   
                    :add(casting_complete())	--施法动作播放完毕，之后不能被打断
                    :add(play_animation('attack_1'))  -- 播放指定名称动画
                    :add(play_vfx('Assets/Arts/fx/prefab/20_Prefab_Heavy_Manghan/skill10020_fuzhu_zhengqi.prefab', 'Chimney_5',5,true))        -- 播放特效
                    :add(play_vfx('Assets/Arts/fx/prefab/20_Prefab_Heavy_Manghan/skill10020_fuzhu_zhengqi.prefab', 'Chimney_2',5,true))        -- 播放特效
                    :add(play_vfx('Assets/Arts/fx/prefab/20_Prefab_Heavy_Manghan/skill10020_start_mts.prefab', 'leftwuqi',5,true))        -- 播放特效
                    :add(play_vfx('Assets/Arts/fx/prefab/20_Prefab_Heavy_Manghan/skill10020_start_mts.prefab', 'rightwuqi',5,true))        -- 播放特效
                    :add(play_vfx('Assets/Arts/fx/prefab/20_Prefab_Heavy_Manghan/skill10020_fuzhu_zhengqi.prefab', 'Chimney_3',5,true))        -- 播放特效
                    :add(play_vfx('Assets/Arts/fx/prefab/20_Prefab_Heavy_Manghan/skill10020_fuzhu_zhengqi.prefab', 'Chimney_4',5,true))        -- 播放特效
                    :add(play_vfx('Assets/Arts/fx/prefab/20_Prefab_Heavy_Manghan/skill10020_start_mtb.prefab', 'midwuqi',5,true))        -- 播放特效
                    :add(play_vfx('Assets/Arts/fx/prefab/20_Prefab_Heavy_Manghan/skill10020_fuzhu_huichen.prefab', 'dahezi',5,true))        -- 播放特效
                    :add(play_vfx('Assets/Arts/fx/prefab/20_Prefab_Heavy_Manghan/skill10020_fuzhu_zhengqi.prefab', 'Chimney_1',5,true))        -- 播放特效
                    :add(play_vfx('Assets/Arts/fx/prefab/20_Prefab_Heavy_Manghan/skill10020_fuzhu_zhengqi.prefab', 'Chimney',5,true))        -- 播放特效
                    :add(play_vfx('Assets/Arts/fx/prefab/20_Prefab_Heavy_Manghan/skill10020_fuzhu_huichen.prefab', 'xiaohezi',5,true))        -- 播放特效
                    :add(delay(2.73))
                    :add(play_vfx('Assets/Arts/fx/prefab/20_Prefab_Heavy_Manghan/skill10020_fuzhu_smoke.prefab', 'lt_1',5))        -- 播放特效
                    :add(play_vfx('Assets/Arts/fx/prefab/20_Prefab_Heavy_Manghan/skill10020_gongji.prefab', 'ground',5))        -- 播放特效
                    :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                    :add(play_vfx(100203, 'shoot', 1))        -- 播放蒸汽特效，1秒消失
                    :add(shake_camera('zhenping_1')) --调用震屏
                    :add(delay(0.1))
                    :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                    :add(play_vfx(100203, 'shoot', 1))        -- 播放蒸汽特效，1秒消失
                    :add(shake_camera('zhenping_1')) --调用震屏
                    :add(delay(0.1))
                    :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                    :add(play_vfx(100203, 'shoot', 1))        -- 播放蒸汽特效，1秒消失
                    :add(shake_camera('zhenping_1')) --调用震屏
                    :add(delay(0.1))
                    :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                    :add(play_vfx(100203, 'shoot', 1))        -- 播放蒸汽特效，1秒消失
                    :add(shake_camera('zhenping_1')) --调用震屏
                    :add(delay(0.1))
                    :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                    :add(play_vfx(100203, 'shoot', 1))        -- 播放蒸汽特效，1秒消失
                    :add(shake_camera('zhenping_1')) --调用震屏
                    :add(delay(0.1))
                    :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                    :add(play_vfx(100203, 'shoot', 1))        -- 播放蒸汽特效，1秒消失
                    :add(shake_camera('zhenping_1')) --调用震屏
                    :add(delay(0.1))
                    :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                    :add(play_vfx(100203, 'shoot', 1))        -- 播放蒸汽特效，1秒消失
                    :add(shake_camera('zhenping_1')) --调用震屏
                    :add(delay(0.1))
                    :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                    :add(play_vfx(100203, 'shoot', 1))        -- 播放蒸汽特效，1秒消失
                    :add(shake_camera('zhenping_1')) --调用震屏
                    :add(delay(0.1))
                    :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                    :add(play_vfx(100203, 'shoot', 1))        -- 播放蒸汽特效，1秒消失
                    :add(shake_camera('zhenping_1')) --调用震屏
                    :add(delay(0.1))
                    :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                    :add(play_vfx(100203, 'shoot', 1))        -- 播放蒸汽特效，1秒消失
                    :add(shake_camera('zhenping_1')) --调用震屏
                    :add(delay(0.1))
                    :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                    :add(play_vfx(100203, 'shoot', 1))        -- 播放蒸汽特效，1秒消失
                    :add(shake_camera('zhenping_1')) --调用震屏
                    :add(delay(1.8))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,

    -- 大锯子：普攻
    [21] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   
                    :add(play_animation('attack'))  -- 播放指定名称动画
                    :add(play_vfx('Assets/Arts/fx/prefab/21_Prefab_Heavy_Kuangxintu/skill21_start_left_1.prefab', 'shangl', 3,true))        -- 播放蒸汽特效，1秒消失
                    :add(play_vfx('Assets/Arts/fx/prefab/21_Prefab_Heavy_Kuangxintu/skill21_start_right_1.prefab', 'shangr', 3,true))        -- 播放蒸汽特效，1秒消失
                    :add(play_vfx('Assets/Arts/fx/prefab/21_Prefab_Heavy_Kuangxintu/skill21_start_right_2.prefab', 'xiar', 3,true))        -- 播放蒸汽特效，1秒消失
                    :add(play_vfx('Assets/Arts/fx/prefab/21_Prefab_Heavy_Kuangxintu/skill21_start_left_1.prefab', 'shangl', 3,true))        -- 播放蒸汽特效，1秒消失
                    :add(delay(0.8))
                    :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                    :add(play_vfx('Assets/Arts/fx/prefab/22_Prefab_Heavy_Dazong/skill22_hit.prefab', 'shoot_1', 3,true))        -- 播放蒸汽特效，1秒消失
                    :add(play_vfx('Assets/Arts/fx/prefab/22_Prefab_Heavy_Dazong/skill22_hit.prefab', 'shoot', 3,true))        -- 播放蒸汽特效，1秒消失
                    :add(delay(1))
                    -- :add(add_buff(buff_targets.SELF,7))		
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,	

	-- 锯子的技能
    [10021] = function(actor)
        if not actor then
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选光
                    :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   
                    :add(casting_complete())	--施法动作播放完毕，之后不能被打断
                    :add(play_animation('attack_1'))  -- 播放指定名称动画
                    :add(play_vfx('Assets/Arts/fx/prefab/21_Prefab_Heavy_Kuangxintu/skill10021_fuzhu_yan_2.prefab', 'Chimney_3',5,true))        -- 播放特效
                    :add(play_vfx('Assets/Arts/fx/prefab/21_Prefab_Heavy_Kuangxintu/skill10021_fuzhu_yan.prefab', 'Chimney_2',5,true))        -- 播放特效
                    :add(play_vfx('Assets/Arts/fx/prefab/21_Prefab_Heavy_Kuangxintu/skill10021_start_right_1.prefab', 'shangr',5,true))        -- 播放特效
                    :add(play_vfx('Assets/Arts/fx/prefab/21_Prefab_Heavy_Kuangxintu/skill10021_start_left_1.prefab', 'shangl',5,true))        -- 播放特效
                    :add(play_vfx('Assets/Arts/fx/prefab/21_Prefab_Heavy_Kuangxintu/skill10021_start_left_2.prefab', 'xial',5,true))        -- 播放特效
                    :add(play_vfx('Assets/Arts/fx/prefab/21_Prefab_Heavy_Kuangxintu/skill10021_fuzhu_yan_2.prefab', 'Chimney_4',5,true))        -- 播放特效
                    :add(play_vfx('Assets/Arts/fx/prefab/21_Prefab_Heavy_Kuangxintu/skill10021_fuzhu_yan.prefab', 'Chimney_1',5,true))        -- 播放特效
                    :add(play_vfx('Assets/Arts/fx/prefab/21_Prefab_Heavy_Kuangxintu/skill10021_start_right_2.prefab', 'xiar',5,true))        -- 播放特效
                    :add(delay(2.6))
                    :add(play_vfx('Assets/Arts/fx/prefab/21_Prefab_Heavy_Kuangxintu/skill10021_hit.prefab', 'shoot_1',5,true))        -- 播放特效
                    :add(delay(0.1))
                    :add(play_vfx('Assets/Arts/fx/prefab/21_Prefab_Heavy_Kuangxintu/skill10021_gongji.prefab', 'ground',5,true))        -- 播放特效
                    :add(play_vfx('Assets/Arts/fx/prefab/21_Prefab_Heavy_Kuangxintu/skill10021_hit.prefab', 'shoot',5,true))        -- 播放特效
                    :add(delay(0.3))
                    :add(play_vfx('Assets/Arts/fx/prefab/21_Prefab_Heavy_Kuangxintu/skill10021_hit.prefab', 'shoot',5,true))        -- 播放特效
                    :add(play_vfx('Assets/Arts/fx/prefab/21_Prefab_Heavy_Kuangxintu/skill10021_hit.prefab', 'shoot_1',5,true))        -- 播放特效
                    :add(shake_camera('zhenping_1')) --调用震屏
                    :add(delay(0.2))
                    :add(play_vfx('Assets/Arts/fx/prefab/21_Prefab_Heavy_Kuangxintu/skill10021_hit.prefab', 'shoot',5,true))        -- 播放特效
                    :add(play_vfx('Assets/Arts/fx/prefab/21_Prefab_Heavy_Kuangxintu/skill10021_hit.prefab', 'shoot_1',5,true))        -- 播放特效
                    :add(shake_camera('zhenping_1')) --调用震屏
                    :add(delay(0.2))
                    :add(play_vfx('Assets/Arts/fx/prefab/21_Prefab_Heavy_Kuangxintu/skill10021_hit.prefab', 'shoot',5,true))        -- 播放特效
                    :add(play_vfx('Assets/Arts/fx/prefab/21_Prefab_Heavy_Kuangxintu/skill10021_hit.prefab', 'shoot_1',5,true))        -- 播放特效
                    :add(shake_camera('zhenping_1')) --调用震屏
                    :add(delay(0.2))
                    :add(play_vfx('Assets/Arts/fx/prefab/21_Prefab_Heavy_Kuangxintu/skill10021_hit.prefab', 'shoot',5,true))        -- 播放特效
                    :add(play_vfx('Assets/Arts/fx/prefab/21_Prefab_Heavy_Kuangxintu/skill10021_hit.prefab', 'shoot_1',5,true))        -- 播放特效
                    :add(shake_camera('zhenping_1')) --调用震屏
                    :add(delay(0.2))
                    :add(play_vfx('Assets/Arts/fx/prefab/21_Prefab_Heavy_Kuangxintu/skill10021_hit.prefab', 'shoot',5,true))        -- 播放特效
                    :add(play_vfx('Assets/Arts/fx/prefab/21_Prefab_Heavy_Kuangxintu/skill10021_hit.prefab', 'shoot_1',5,true))        -- 播放特效
                    :add(shake_camera('zhenping_1')) --调用震屏
                    :add(delay(0.2))
                    :add(play_vfx('Assets/Arts/fx/prefab/21_Prefab_Heavy_Kuangxintu/skill10021_hit.prefab', 'shoot',5,true))        -- 播放特效
                    :add(play_vfx('Assets/Arts/fx/prefab/21_Prefab_Heavy_Kuangxintu/skill10021_hit.prefab', 'shoot_1',5,true))        -- 播放特效
                    :add(shake_camera('zhenping_1')) --调用震屏
                    :add(delay(1))
                    :add(play_vfx('Assets/Arts/fx/prefab/21_Prefab_Heavy_Kuangxintu/skill10021_hit.prefab', 'shoot',5,true))        -- 播放特效
                    :add(play_vfx('Assets/Arts/fx/prefab/21_Prefab_Heavy_Kuangxintu/skill10021_hit.prefab', 'shoot_1',5,true))        -- 播放特效
                    :add(shake_camera('zhenping_1')) --调用震屏
                    :add(delay(1.8))
                    :add(finished())            -- 技能结束
                    )
        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,

    -- 喷射锤：普攻
    [22] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   
                    :add(play_animation('attack'))  -- 播放指定名称动画
                    :add(play_vfx('Assets/Arts/fx/prefab/22_Prefab_Heavy_Dazong/skill22_zhengqi.prefab', 'Zhengqi_1', 1,true))        -- 播放蒸汽特效，1秒消失
                    :add(play_vfx('Assets/Arts/fx/prefab/22_Prefab_Heavy_Dazong/skill22_zhengqi.prefab', 'Zhengqi_2', 1,true))        -- 播放蒸汽特效，1秒消失
                    :add(play_vfx('Assets/Arts/fx/prefab/22_Prefab_Heavy_Dazong/skill22_deng.prefab', 'd_3', 1))        -- 播放灯特效，1秒消失
                    :add(delay(0.73))
                    :add(play_vfx('Assets/Arts/fx/prefab/22_Prefab_Heavy_Dazong/skill22_start.prefab', 'ground', 1,true))        -- 播放蒸汽特效，1秒消失
                    :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                    :add(delay(0.77))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,	
    
    -- 喷射锤：技能

    [10022] = function(actor)
        
            -- 获得敌人数量
        local getEnemyNum = function()
            return actor.enemy_list and #actor.enemy_list or 0
        end

        -- 选择下一个敌人
        local enemy_index = 0
        local nextEnemy = function()
            enemy_index = enemy_index + 1
            return enemy_index
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(select_multiple_enemy_randomly(8,false)) 
                    :add(casting_complete())	--施法动作播放完毕，之后不能被打断
                    :add(play_animation('attack_1'))  -- 播放指定名称动画
                    :add(play_vfx('Assets/Arts/fx/prefab/22_Prefab_Heavy_Dazong/skill10022_xuli.prefab', 'lt_1', 2,true))        -- 播放蒸汽特效，1秒消失
                    :add(play_vfx('Assets/Arts/fx/prefab/22_Prefab_Heavy_Dazong/skill10022_xuli_chuizi.prefab', 'shoot_1', 2,true))        -- 播放蒸汽特效，1秒消失
                    :add(play_vfx('Assets/Arts/fx/prefab/22_Prefab_Heavy_Dazong/skill10022_xuli_chuizi.prefab', 'shoot', 2,true))   
                    :add(play_vfx('Assets/Arts/fx/prefab/22_Prefab_Heavy_Dazong/skill10022_deng.prefab', 'd_3', 2))        -- 播放蒸汽特效，1秒消失
                    :add(play_vfx('Assets/Arts/fx/prefab/22_Prefab_Heavy_Dazong/skill10022_zhengqi.prefab', 'Zhengqi_1', 2,true))        -- 播放蒸汽特效，1秒消失
                    :add(play_vfx('Assets/Arts/fx/prefab/22_Prefab_Heavy_Dazong/skill10022_zhengqi.prefab', 'Zhengqi_2', 2,true))        -- 播放蒸汽特效，1秒消失
                    :add(delay(2.63))
                    :add(play_vfx('Assets/Arts/fx/prefab/22_Prefab_Heavy_Dazong/skill10022_start.prefab', 'ground', 5,true))        -- 播放蒸汽特效，1秒消失
                    :add(shake_camera('zhenping_1')) --调用震屏
                    :add(loop(getEnemyNum)
                        :add(set_enemy_from_multiple(nextEnemy))
                        :add(shoot_bullet(90001, 'shoot', 180))        -- 射击子弹 prefab, speed, damage
                        )
                    :add(delay(0.5))
                    :add(call_func( function() enemy_index = 0 end )) --临时 匿名函数
                    :add(play_vfx('Assets/Arts/fx/prefab/22_Prefab_Heavy_Dazong/skill10022_start.prefab', 'ground', 5,true))        -- 播放蒸汽特效，1秒消失
                    :add(shake_camera('zhenping_1')) --调用震屏
                    :add(loop(getEnemyNum)
                        :add(set_enemy_from_multiple(nextEnemy))
                        :add(shoot_bullet(90001, 'shoot', 180))        -- 射击子弹 prefab, speed, damage
                        )
                    :add(delay(0.5))
                    :add(call_func( function() enemy_index = 0 end )) --临时 匿名函数
                    :add(play_vfx('Assets/Arts/fx/prefab/22_Prefab_Heavy_Dazong/skill10022_start.prefab', 'ground', 5,true))        -- 播放蒸汽特效，1秒消失
                    :add(shake_camera('zhenping_1')) --调用震屏
                    :add(loop(getEnemyNum)
                        :add(set_enemy_from_multiple(nextEnemy))
                        :add(shoot_bullet(90001, 'shoot', 180))        -- 射击子弹 prefab, speed, damage
                        )
                    :add(delay(0.3))
                    :add(finished())            -- 技能结束
                )
        bt:setContext(actor)
        return bt
    end,	

    -- 狙击炮：普攻
    [23] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   
                    :add(play_animation('attack'))  -- 播放指定名称动画
                    :add(delay(0.16))
                    :add(play_vfx('Assets/Arts/fx/prefab/23_Prefab_Tank_Liudemila/skill23_smoke.prefab', 'lt_1', 5,true))        -- 播放蒸汽特效，1秒消失
                    :add(shoot_bullet(23, 'shoot', 400, consts.assets.curves.curve23)        -- 射击子弹 prefab, speed, damage
                    :add(delay(1.5))
                    :add(finished())            -- 技能结束
                    ))

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,

    -- 大锯子：普攻
    [24] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   
                    :add(play_animation('attack'))  -- 播放指定名称动画
                    :add(play_vfx('Assets/Arts/fx/prefab/24_Prefab_Heavy_Shangren/skill24_start_left.prefab', 'Dummy004', 3,true))        -- 播放蒸汽特效，1秒消失
                    :add(play_vfx('Assets/Arts/fx/prefab/24_Prefab_Heavy_Shangren/skill24_start_right.prefab', 'Dummy003', 3,true))        -- 播放蒸汽特效，1秒消失
                    :add(delay(1.2))
                    :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                    :add(delay(0.05))
                    :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                    :add(delay(0.05))
                    :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                    :add(delay(0.05))
                    :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                    :add(delay(0.05))
                    :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                    :add(delay(1))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,	
    
    -- 大锯子：技能
    [10024] = function(actor)
        if not actor then
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
            sequence()   -- 选择
                :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   
                :add(casting_complete())	--施法动作播放完毕，之后不能被打断
                :add(play_animation('attack_1'))  -- 播放指定名称动画
                :add(play_vfx('Assets/Arts/fx/prefab/24_Prefab_Heavy_Shangren/skill10024_start_right.prefab', 'Dummy003',5,true))        -- 播放特效
                :add(play_vfx('Assets/Arts/fx/prefab/24_Prefab_Heavy_Shangren/skill10024_start_left.prefab', 'Dummy004',5,true))        -- 播放特效
                :add(delay(2.55))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(play_vfx('Assets/Arts/fx/prefab/24_Prefab_Heavy_Shangren/skill10024_fuzhu_huoxing.prefab', 'Dummy005',5,true))        -- 播放特效
                :add(play_vfx('Assets/Arts/fx/prefab/24_Prefab_Heavy_Shangren/skill10024_fuzhu_huoxing.prefab', 'Dummy002',5,true))        -- 播放特效
                :add(delay(0.05))
                :add(play_vfx('Assets/Arts/fx/prefab/24_Prefab_Heavy_Shangren/skill10024_hit.prefab', 'right_hit',5,true))        -- 播放特效
                :add(play_vfx('Assets/Arts/fx/prefab/24_Prefab_Heavy_Shangren/skill10024_hit.prefab', 'left_hit',5,true))        -- 播放特效
                :add(play_vfx('Assets/Arts/fx/prefab/24_Prefab_Heavy_Shangren/skill10024_smoke.prefab', 'lt_1',5,true))        -- 播放特效
                :add(delay(0.05))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.1))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(play_vfx('Assets/Arts/fx/prefab/24_Prefab_Heavy_Shangren/skill10024_hit.prefab', 'right_hit',5,true))        -- 播放特效
                :add(play_vfx('Assets/Arts/fx/prefab/24_Prefab_Heavy_Shangren/skill10024_hit.prefab', 'left_hit',5,true))        -- 播放特效
                :add(delay(0.05))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.1))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(play_vfx('Assets/Arts/fx/prefab/24_Prefab_Heavy_Shangren/skill10024_hit.prefab', 'right_hit',5,true))        -- 播放特效
                :add(play_vfx('Assets/Arts/fx/prefab/24_Prefab_Heavy_Shangren/skill10024_hit.prefab', 'left_hit',5,true))        -- 播放特效
                :add(delay(0.1))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.1))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(play_vfx('Assets/Arts/fx/prefab/24_Prefab_Heavy_Shangren/skill10024_hit.prefab', 'right_hit',5,true))        -- 播放特效
                :add(play_vfx('Assets/Arts/fx/prefab/24_Prefab_Heavy_Shangren/skill10024_hit.prefab', 'left_hit',5,true))        -- 播放特效
                :add(delay(0.05))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.1))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(play_vfx('Assets/Arts/fx/prefab/24_Prefab_Heavy_Shangren/skill10024_hit.prefab', 'right_hit',5,true))        -- 播放特效
                :add(play_vfx('Assets/Arts/fx/prefab/24_Prefab_Heavy_Shangren/skill10024_hit.prefab', 'left_hit',5,true))        -- 播放特效
                :add(delay(0.1))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.1))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.05))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(play_vfx('Assets/Arts/fx/prefab/24_Prefab_Heavy_Shangren/skill10024_hit.prefab', 'right_hit',5,true))        -- 播放特效
                :add(play_vfx('Assets/Arts/fx/prefab/24_Prefab_Heavy_Shangren/skill10024_hit.prefab', 'left_hit',5,true))        -- 播放特效
                :add(delay(0.5))
                :add(play_vfx('Assets/Arts/fx/prefab/24_Prefab_Heavy_Shangren/skill10024_hit.prefab', 'right_hit',5,true))        -- 播放特效
                :add(play_vfx('Assets/Arts/fx/prefab/24_Prefab_Heavy_Shangren/skill10024_hit.prefab', 'left_hit',5,true))        -- 播放特效
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(1.5))
                :add(finished())            -- 技能结束
                )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,

    -- 爆炸长矛：普攻
    [27] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   
                    :add(play_animation('attack'))  -- 播放指定名称动画
                    :add(delay(2))
                    :add(shoot_bullet(27, 'shoot', 100, consts.assets.curves.curve1))        -- 射击子弹 prefab, speed, damage
                    :add(delay(1))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,
    
    -- 乔力：普攻
    [28] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   
                    :add(play_animation('attack'))  -- 播放指定名称动画
                    :add(delay(0.23))
                    :add(shoot_bullet(28, 'shoot', 180, consts.assets.curves.curve28))        -- 射击子弹 prefab, speed, damage
                    :add(delay(1))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,

    -- 乔力：技能
    [10028] = function(actor)
        if not actor then
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
            sequence()   -- 选择
                :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   
                :add(casting_complete())	--施法动作播放完毕，之后不能被打断
                :add(play_animation('attack_1'))  -- 播放指定名称动画
                :add(play_vfx('Assets/Arts/fx/prefab/28_Prefab_Tank_Luke/skill10028_xuli_cheti.prefab', 'lt_1',5,true))        -- 播放特效
                :add(delay(1.53))
                :add(shoot_bullet_to_enemy_field(10028, 'shoot', 10, consts.assets.curves.curve10028, 1,140))
                :add(delay(1.5))
                :add(add_buff(buff_targets.ALL_TEAMATE_VEHICLE,8))	
                :add(finished())            -- 技能结束
                )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,

    -- 蛇影普攻
    [29] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   
                    :add(play_vfx('Assets/Arts/fx/prefab/29_Prefab_Assult_Laola/skill29_smoke.prefab', 'lt_1',5,true))        -- 播放特效
                    :add(play_animation('attack'))  -- 播放指定名称动作
                    :add(delay(0.16))
                    --:add(play_vfx(301, 'shoot', 1))        -- 播放开始特效，1秒消失
                    :add(shoot_bullet(29, 'shoot', 120, consts.assets.curves.curve29))        -- 射击子弹 prefab, speed, damage
                    :add(delay(1.27))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,
    
        
    -- 蛇影技能
    [10029] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(select_multiple_enemy_by_range(1,false))
                    :add(play_animation('attack_1'))  -- 播放指定名称动作
                    :add(play_vfx('Assets/Arts/fx/prefab/29_Prefab_Assult_Laola/skill10029_fuzhu_huoxing.prefab', 'Dummy012', 3))        -- 播放开始特效，1秒消失
                    :add(play_vfx('Assets/Arts/fx/prefab/29_Prefab_Assult_Laola/skill10029_fuzhu_huoxing.prefab', 'Dummy013', 3))        -- 播放开始特效，1秒消失
                    :add(play_vfx('Assets/Arts/fx/prefab/29_Prefab_Assult_Laola/skill10029_start_xuli.prefab', 'Dummy014', 3))        -- 播放开始特效，1秒消失
                    :add(casting_complete())	--施法动作播放完毕，之后不能被打断
                    :add(delay(1.5))
                    :add(set_enemy_from_multiple(1))
                    --:add(play_animation('attack_1_loop'))  -- 播放指定名称动作
                    :add(play_vfx('Assets/Arts/fx/prefab/29_Prefab_Assult_Laola/skill10029_smoke.prefab', 'lt_1', 3))        -- 播放开始特效，1秒消失
                    :add(shoot_bullet(10029, 'shoot', 120, consts.assets.curves.curve10029))        -- 射击子弹 prefab, speed, damage
                    :add(delay(0.25))
                    :add(shoot_bullet(10029, 'shoot', 120, consts.assets.curves.curve10029))        -- 射击子弹 prefab, speed, damage
                    :add(delay(0.25))
                    :add(shoot_bullet(10029, 'shoot', 120, consts.assets.curves.curve10029))        -- 射击子弹 prefab, speed, damage
                    :add(delay(0.25))
                    :add(shoot_bullet(10029, 'shoot', 120, consts.assets.curves.curve10029))        -- 射击子弹 prefab, speed, damage
                    :add(delay(0.25))
                    :add(shoot_bullet(10029, 'shoot', 120, consts.assets.curves.curve10029))        -- 射击子弹 prefab, speed, damage
                    :add(delay(0.25))
                    :add(shoot_bullet(10029, 'shoot', 120, consts.assets.curves.curve10029))        -- 射击子弹 prefab, speed, damage
                    -- :add(set_enemy_from_multiple(2))
                    -- :add(play_animation('attack_1_loop'))  -- 播放指定名称动作
                    -- :add(play_vfx('Assets/Arts/fx/prefab/29_Prefab_Assult_Laola/skill10029_smoke.prefab', 'lt_1', 3))        -- 播放开始特效，1秒消失
                    -- :add(shoot_bullet(10029, 'shoot', 150, consts.assets.curves.curve10029))        -- 射击子弹 prefab, speed, damage
                    -- :add(delay(0.25))
                    -- :add(shoot_bullet(10029, 'shoot', 150, consts.assets.curves.curve10029))        -- 射击子弹 prefab, speed, damage
                    -- :add(delay(0.25))
                    -- :add(shoot_bullet(10029, 'shoot', 150, consts.assets.curves.curve10029))        -- 射击子弹 prefab, speed, damage
                    -- :add(delay(0.5))
                    -- :add(set_enemy_from_multiple(3))
                    -- :add(play_animation('attack_1_loop'))  -- 播放指定名称动作
                    -- :add(play_vfx('Assets/Arts/fx/prefab/29_Prefab_Assult_Laola/skill10029_smoke.prefab', 'lt_1', 3))        -- 播放开始特效，1秒消失
                    -- :add(shoot_bullet(10029, 'shoot', 150, consts.assets.curves.curve10029))        -- 射击子弹 prefab, speed, damage
                    -- :add(delay(0.25))
                    -- :add(shoot_bullet(10029, 'shoot', 150, consts.assets.curves.curve10029))        -- 射击子弹 prefab, speed, damage
                    -- :add(delay(0.25))
                    -- :add(shoot_bullet(10029, 'shoot', 150, consts.assets.curves.curve10029))        -- 射击子弹 prefab, speed, damage
                    --:add(play_animation('attack_1_back'))  -- 播放指定名称动作
                    :add(delay(1.5))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,

    -- 榴弹炮普攻
    [30] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   
                    :add(play_animation('attack'))  -- 播放指定名称动作
                    :add(delay(0.16))
                    --:add(play_vfx(301, 'shoot', 1))        -- 播放开始特效，1秒消失
                    :add(shoot_bullet(30, 'shoot', 120, consts.assets.curves.curve30))        -- 射击子弹 prefab, speed, damage
                    :add(delay(1.27))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,
    
        
    -- 榴弹炮技能
    [10030] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(select_multiple_enemy_by_range(3,false))
                    :add(play_animation('attack_1_rush'))  -- 播放指定名称动作
                    :add(play_vfx('Assets/Arts/fx/prefab/30_Prefab_Assult_Haonvhai/skill10030_focus.prefab', 'shoot', 1))        -- 播放开始特效，1秒消失
                    :add(play_vfx('Assets/Arts/fx/prefab/30_Prefab_Assult_Haonvhai/skill10030_chanrao.prefab', 'Dummy010', 1))        -- 播放开始特效，1秒消失
                    :add(casting_complete())	--施法动作播放完毕，之后不能被打断
                    :add(delay(1.33))
                    :add(set_enemy_from_multiple(1))
                    :add(play_animation('attack_1_loop'))  -- 播放指定名称动作
                    :add(shoot_bullet(10030, 'shoot', 200, consts.assets.curves.curve30))        -- 射击子弹 prefab, speed, damage
                    :add(delay(0.25))
                    :add(shoot_bullet(10030, 'shoot', 200, consts.assets.curves.curve30))        -- 射击子弹 prefab, speed, damage
                    :add(delay(0.25))
                    :add(shoot_bullet(10030, 'shoot', 200, consts.assets.curves.curve30))        -- 射击子弹 prefab, speed, damage
                    :add(delay(0.5))
                    :add(set_enemy_from_multiple(2))
                    :add(play_animation('attack_1_loop'))  -- 播放指定名称动作
                    :add(shoot_bullet(10030, 'shoot', 200, consts.assets.curves.curve30))        -- 射击子弹 prefab, speed, damage
                    :add(delay(0.25))
                    :add(shoot_bullet(10030, 'shoot', 200, consts.assets.curves.curve30))        -- 射击子弹 prefab, speed, damage
                    :add(delay(0.25))
                    :add(shoot_bullet(10030, 'shoot', 200, consts.assets.curves.curve30))        -- 射击子弹 prefab, speed, damage
                    :add(delay(0.5))
                    :add(set_enemy_from_multiple(3))
                    :add(play_animation('attack_1_loop'))  -- 播放指定名称动作
                    :add(shoot_bullet(10030, 'shoot', 200, consts.assets.curves.curve30))        -- 射击子弹 prefab, speed, damage
                    :add(delay(0.25))
                    :add(shoot_bullet(10030, 'shoot', 200, consts.assets.curves.curve30))        -- 射击子弹 prefab, speed, damage
                    :add(delay(0.25))
                    :add(shoot_bullet(10030, 'shoot', 200, consts.assets.curves.curve30))        -- 射击子弹 prefab, speed, damage
                    :add(play_animation('attack_1_back'))  -- 播放指定名称动作
                    :add(delay(0.5))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,
    
    -- 巨型粒子炮普攻
    [32] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   
                    :add(play_animation('attack'))  -- 播放指定名称动作
                    :add(delay(0.23))
                    :add(play_vfx('Assets/Arts/fx/prefab/32_Prefab_Tank_ShuangZi/skill32_start.prefab', 'shoot_2', 1,true))        -- 播放开始特效，1秒消失，强制播放
                    :add(play_vfx('Assets/Arts/fx/prefab/32_Prefab_Tank_ShuangZi/skill32_start.prefab', 'shoot_1', 1,true))        -- 播放开始特效，1秒消失，强制播放
                    :add(play_vfx('Assets/Arts/fx/prefab/32_Prefab_Tank_ShuangZi/skill32_huichen.prefab', 'foot_1', 1,true))        -- 播放光柱特效，1秒消失，强制播放
                    :add(play_vfx('Assets/Arts/fx/prefab/32_Prefab_Tank_ShuangZi/skill32_huichen.prefab', 'foot_2', 1,true))        -- 播放光柱特效，1秒消失，强制播放
                    :add(play_vfx('Assets/Arts/fx/prefab/32_Prefab_Tank_ShuangZi/skill32_guangzhu.prefab', 'shoot_2', 1,true))        -- 播放开始特效，1秒消失，强制播放
                    :add(play_vfx('Assets/Arts/fx/prefab/32_Prefab_Tank_ShuangZi/skill32_guangzhu.prefab', 'shoot_1', 1,true))        -- 播放开始特效，1秒消失，强制播放
                    :add(shoot_laser(32,'shoot_2')) 
                    :add(shoot_laser(32,'shoot_1')) 
                    :add(delay(0.5))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,
        
    -- 巨型粒子炮技能
    [10032] = function(actor)

        -- 获得敌人数量
        local getEnemyNum = function()
            return actor.enemy_list and #actor.enemy_list or 0
        end

        -- 选择下一个敌人
        local enemy_index = 0
        local nextEnemy = function()
            enemy_index = enemy_index + 1
            return enemy_index
        end

        local getDistance = function()
            if battle_core_flag then
                local from = actor.tk:getPosition()
                local to = actor.tk.target_actor:getPosition()
                
                local distance_1 = (from - to):magnitude()
                return distance_1

            else
                local from = actor:getPosition()
                local to = actor.target_actor:getPosition()
                
                local distance_1 = Vector3.Distance(from,to)
                return distance_1
            end
        end


        local bt = root():setChild(
                sequence()   -- 选择
                    :add(select_multiple_enemy_randomly(8,false))
                    :add(play_animation('attack_1'))  -- 播放指定名称动作
                    :add(casting_complete())	--施法动作播放完毕，之后不能被打断
                    :add(play_vfx('Assets/Arts/fx/prefab/32_Prefab_Tank_ShuangZi/skill10032_yancong.prefab', 'Chimney', 3,true))        -- 播放灰尘特效，1秒消失，强制播放
                    :add(play_vfx('Assets/Arts/fx/prefab/32_Prefab_Tank_ShuangZi/skill10032_yancong.prefab', 'Chimney_1', 3,true))        -- 播放灰尘特效，1秒消失，强制播放
                    :add(play_vfx('Assets/Arts/fx/prefab/32_Prefab_Tank_ShuangZi/skill10032_yancong.prefab', 'Chimney_2', 3,true))        -- 播放灰尘特效，1秒消失，强制播放
                    :add(delay(0.23))
                    :add(play_vfx('Assets/Arts/fx/prefab/32_Prefab_Tank_ShuangZi/skill10032_start.prefab', 'shoot_2', 3,true))        -- 播放启动特效，1秒消失，强制播放
                    :add(play_vfx('Assets/Arts/fx/prefab/32_Prefab_Tank_ShuangZi/skill10032_start.prefab', 'shoot_1', 3,true))        -- 播放启动特效，1秒消失，强制播放
                    :add(delay(0.77))
                    :add(play_vfx('Assets/Arts/fx/prefab/32_Prefab_Tank_ShuangZi/skill10032_lengjing.prefab', 'shoot', 2))        -- 播放棱镜特效，2秒消失，强制播放
                    --:add(play_vfx(100326, 'shoot_2', 2))        -- 播放棱镜特效，2秒消失
                    :add(set_is_turning_weapon(false))  --关闭武器转头
                    :add(delay(0.5))
                    :add(loop(getEnemyNum)
                        :add(set_enemy_from_multiple(nextEnemy))
                        :add(play_animation('attack_1'))  -- 播放指定名称动作
                        :add(play_vfx('Assets/Arts/fx/prefab/32_Prefab_Tank_ShuangZi/skill10032_guangzhu.prefab', 'shoot_2', 1,true))        -- 播放射击棱镜特效，1秒消失，强制播放
                        :add(play_vfx('Assets/Arts/fx/prefab/32_Prefab_Tank_ShuangZi/skill10032_guangzhu.prefab', 'shoot_1', 1,true))        -- 播放射击棱镜特效，1秒消失，强制播放
                        :add(shoot_laser(10032,'shoot')) 
                        :add(play_vfx('Assets/Arts/fx/prefab/32_Prefab_Tank_ShuangZi/skill10032_lengjing.prefab', 'shoot', 2))        -- 播放棱镜特效，2秒消失，强制播放
                        :add(delay(0.1))
                        )
                    :add(delay(2.0))
                    :add(set_is_turning_weapon(true))  --开启武器转头
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,


    -- 卫星炮普攻
    [33] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   
                    :add(play_animation('attack'))  -- 播放指定名称动作
                    :add(delay(0.16))
                    --:add(play_vfx(331, 'shoot', 1))        -- 播放开始特效，1秒消失
                    :add(shoot_bullet(33, 'shoot', 200, consts.assets.curves.curve33))        -- 射击子弹 prefab, speed, damage
                    :add(delay(1.34))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,

    -- 卫星炮技能
    [10033] = function(actor)

        -- 获得敌人数量
        local getEnemyNum = function()
            return actor.enemy_list and #actor.enemy_list or 0
        end

        -- 选择下一个敌人
        local enemy_index = 0
        local nextEnemy = function()
            enemy_index = enemy_index + 1
            return enemy_index
        end

        local total_loop_count = 18 -- 总的循环次数
        local dist_loop_idx = 0     -- 循环索引
        local dist_factor = {0.85,1,1.15,1.3,1.45,1.6,1.75,1.9,2.05,2.2,2.35,2.5,2.65,2.8,2.95,3.1,3.25,3.4} -- 时间数组
        local getDistance = function()
            dist_loop_idx = dist_loop_idx + 1

            local result = 0
            if battle_core_flag then
                local from = actor.tk:getPosition()
                local to = actor.tk.target_actor:getPosition()
                
                local distance_1 = (from - to):magnitude()
                result = distance_1 / dist_factor[dist_loop_idx]

            else
                local from = actor:getPosition()
                local ta = actor:getTargetActor()
                if ta then
                    local to = actor:getTargetActor():getPosition()
                    local distance_1 = Vector3.Distance(from,to)
                    result = distance_1 / dist_factor[dist_loop_idx]
                else
                    result = 100
                end
            end

            if dist_loop_idx >= total_loop_count then
                dist_loop_idx = 0
            end

            return result
        end

        local bt = root():setChild(
            
                sequence()   -- 选择
                    :add(select_multiple_enemy_randomly(8,false))
                    :add(play_animation('attack_1'))  -- 播放指定名称动作
                    :add(casting_complete())	--施法动作播放完毕，之后不能被打断
                    :add(play_vfx('Assets/Arts/fx/prefab/33_Prefab_Artillery_Aisimeilada/skill10033_focus.prefab', 'shoot', 2))        -- 播放聚焦特效，1秒消失
                    :add(set_is_turning_weapon(false))  --关闭武器转头
                    :add(delay(1.53))
                    :add(play_vfx('Assets/Arts/fx/prefab/33_Prefab_Artillery_Aisimeilada/skill10033_huichen.prefab', 'lt_1', 1))        -- 播放灰尘特效，1秒消失
                    :add(play_vfx('Assets/Arts/fx/prefab/33_Prefab_Artillery_Aisimeilada/skill10033_start.prefab', 'shoot', 1))        -- 播放开始特效，1秒消失
                    :add(delay(0.07))
                    :add(loop(getEnemyNum)
                        :add(set_enemy_from_multiple(nextEnemy))
                        :add(shoot_bullet(10033, 'shoot', 200, nil, 0, false, 0, 0, nil, true))        -- 射击子弹 prefab, speed, damage
                        :add(loop(total_loop_count)
                            :add(shoot_bullet(90001, 'shoot', getDistance)) )
                        :add(delay(0.2,0.5))
                    )
                    :add(set_is_turning_weapon(true))  --开启武器转头
                    :add(delay(1.34))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,

	-- 丘吉尔的普攻
    [36] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
            sequence()   -- 选择
                :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   
                :add(play_animation('attack'))  -- 播放指定名称动画
                :add(delay(0.2))
                :add(play_vfx('Assets/Arts/fx/prefab/36_Prefab_Tank_Guigongzi/skill36_smoke.prefab', 'lt_1',5,true))        -- 播放特效
                :add(shoot_bullet(36, 'shoot', 250, consts.assets.curves.curve36))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.8))
                :add(finished())            -- 技能结束
                )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,

	-- 丘吉尔的技能
    [10036] = function(actor)
        if not actor then
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
            sequence()   -- 选择
                :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时。
                :add(casting_complete())	--施法动作播放完毕，之后不能被打断
                :add(play_animation('attack_1'))  -- 播放指定名称动画
                :add(play_vfx('Assets/Arts/fx/prefab/36_Prefab_Tank_Guigongzi/skill10036_fuzhu_zhengqi.prefab', 'Chimney_2',5,true))        -- 播放特效
                :add(play_vfx('Assets/Arts/fx/prefab/36_Prefab_Tank_Guigongzi/skill10036_fuzhu_zhengqi.prefab', 'Chimney_1',5,true))        -- 播放特效
                :add(play_vfx('Assets/Arts/fx/prefab/36_Prefab_Tank_Guigongzi/skill10036_xuli.prefab', 'shoot',5,true))        -- 播放特效
                :add(delay(1.5))
                :add(play_vfx('Assets/Arts/fx/prefab/36_Prefab_Tank_Guigongzi/skill10036_smoke.prefab', 'lt_1',5,true))        -- 播放特效
                :add(shoot_bullet(10036, 'shoot', 350, consts.assets.curves.curve10036))        -- 射击子弹 prefab, speed, damage
                :add(shoot_bullet(100361, 'shoot', 350, consts.assets.curves.curve10036))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.2))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.2))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(play_vfx('Assets/Arts/fx/prefab/36_Prefab_Tank_Guigongzi/skill10036_xuli.prefab', 'shoot',5,true))        -- 播放特效
                :add(delay(0.2))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.2))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.2))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.2))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.2))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(play_vfx('Assets/Arts/fx/prefab/36_Prefab_Tank_Guigongzi/skill10036_xuli.prefab', 'shoot',5,true))        -- 播放特效
                :add(delay(0.2))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.2))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.2))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.2))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.2))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(play_vfx('Assets/Arts/fx/prefab/36_Prefab_Tank_Guigongzi/skill10036_xuli.prefab', 'shoot',5,true))        -- 播放特效
                :add(delay(0.2))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.2))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.2))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.2))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.2))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(play_vfx('Assets/Arts/fx/prefab/36_Prefab_Tank_Guigongzi/skill10036_xuli.prefab', 'shoot',5,true))        -- 播放特效
                :add(delay(0.2))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.2))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.2))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.2))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.2))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.2))
                :add(shoot_bullet(10036, 'shoot', 350, consts.assets.curves.curve10036))        -- 射击子弹 prefab, speed, damage
                :add(shoot_bullet(100362, 'shoot', 350, consts.assets.curves.curve10036))        -- 射击子弹 prefab, speed, damage
                :add(delay(1.5))
                :add(finished())            -- 技能结束
                )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,

    -- 复仇虎的普攻
    [38] = function(actor)
        if not actor then
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
            sequence()   -- 选择
                :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   
                :add(play_animation('attack'))  -- 播放指定名称动画
                :add(delay(0.2))
                :add(play_vfx('Assets/Arts/fx/prefab/38_Prefab_Vehicle_Tank_Aisha/skill38_somke.prefab', 'lt_1',5))        -- 播放特效
                :add(play_vfx('Assets/Arts/fx/prefab/38_Prefab_Vehicle_Tank_Aisha/skill38_start_penqi.prefab', 'shoot',5))        -- 播放特效
                :add(shoot_bullet(38, 'shoot', 280, consts.assets.curves.curve38))        -- 射击子弹 prefab, speed, damage
                -- :add(shoot_bullet(10036, 'shoot', 350, consts.assets.curves.curve10036))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.2))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.2))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.5))
                :add(finished())            -- 技能结束
                )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,

	-- 复仇虎的技能
    [10038] = function(actor)
        if not actor then
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
            sequence()   -- 选择
                :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时
                :add(casting_complete())	--施法动作播放完毕，之后不能被打断
                :add(play_animation('attack_1'))  -- 播放指定名称动画
                :add(play_vfx('Assets/Arts/fx/prefab/38_Prefab_Vehicle_Tank_Aisha/skill10038_start_xuli.prefab', 'shoot',5,true))        -- 播放特效
                :add(delay(1.5))
                :add(shoot_bullet(10038, 'shoot', 320, consts.assets.curves.curve10038))        -- 射击子弹 prefab, speed, damage
                :add(play_vfx('Assets/Arts/fx/prefab/38_Prefab_Vehicle_Tank_Aisha/skill10038_start_penqi.prefab', 'shoot',5,true))        -- 播放特效
                :add(play_vfx('Assets/Arts/fx/prefab/38_Prefab_Vehicle_Tank_Aisha/skill10038_somke.prefab', 'lt_1',5,true))        -- 播放特效
                :add(delay(2.8))
                :add(finished())            -- 技能结束
                )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,

	-- 地火明夷的普攻
    [39] = function(actor)
        if not actor then
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
            sequence()   -- 选择
                :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   
                :add(play_animation('attack'))  -- 播放指定名称动画
                :add(delay(0.2))
                :add(play_vfx('Assets/Arts/fx/prefab/39_Prefab_Assult_Helilian/skill39_fuzhu_maoyan.prefab', 'Chimney_3',5))        -- 播放特效
                :add(play_vfx('Assets/Arts/fx/prefab/39_Prefab_Assult_Helilian/skill39_start.prefab', 'shoot',5))        -- 播放特效
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.2))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.2))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.5))
                :add(finished())            -- 技能结束
                )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,
    
	-- 地火明夷的技能
    [10039] = function(actor)
        if not actor then
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
            sequence()   -- 选择
                :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时
                :add(casting_complete())	--施法动作播放完毕，之后不能被打断
                :add(play_animation('attack_1'))  -- 播放指定名称动画
                :add(play_vfx('Assets/Arts/fx/prefab/39_Prefab_Assult_Helilian/skill10039_fuzhu_huoxing.prefab', 'Ejector_1',5,true))        -- 播放特效
                :add(play_vfx('Assets/Arts/fx/prefab/39_Prefab_Assult_Helilian/skill10039_fuzhu_maoyan.prefab', 'Chimney_1',5,true))        -- 播放特效
                :add(play_vfx('Assets/Arts/fx/prefab/39_Prefab_Assult_Helilian/skill10039_fuzhu_maoyan.prefab', 'Chimney_2',5,true))        -- 播放特效
                :add(play_vfx('Assets/Arts/fx/prefab/39_Prefab_Assult_Helilian/skill10039_fuzhu_maoyan.prefab', 'Chimney_3',5,true))        -- 播放特效
                :add(play_vfx('Assets/Arts/fx/prefab/39_Prefab_Assult_Helilian/skill10039_fuzhu_huoxing.prefab', 'Ejector_2',5,true))        -- 播放特效
                -- :add(play_vfx('Assets/Arts/fx/prefab/39_Prefab_Assult_Helilian/skill10039_fuzhu_zhengqi', 'Zhengqi',5,true))        -- 播放特效
                :add(play_vfx('Assets/Arts/fx/prefab/39_Prefab_Assult_Helilian/skill10039_fuzhu_smoke.prefab', 'lt_1',5,true))
                :add(play_vfx('Assets/Arts/fx/prefab/39_Prefab_Assult_Helilian/skill10039_start_xuli.prefab', 'shoot',5,true))
                :add(play_vfx('Assets/Arts/fx/prefab/39_Prefab_Assult_Helilian/skill10039_start.prefab', 'shoot',5,true))  -- 播放特效
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.2))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.2))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.2))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.2))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.2))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.2))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.2))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.2))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.2))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.2))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.2))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.2))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.2))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.2))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.8))
                :add(finished())            -- 技能结束
                )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,
    

    --喷气式骑枪普攻
    [41] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end
        
        local bt = root():setChild(
            sequence()   -- 选择
                :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时 					
                :add(play_animation('attack'))  -- 播放指定名称动画
                :add(delay(0.73))
                :add(play_vfx('Assets/Arts/fx/prefab/41_Prefab_Heavy_Taluo/skill41_start.prefab', 'right_hit',3))        -- 播放特效
                :add(shoot_bullet(90001, 'right_hit', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(1.33))
                :add(finished())            -- 技能结束
                )
                    
        bt:setContext(actor)
        return bt
    end,	
    
    -- 喷气式骑枪技能
    [10041] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
            sequence()   -- 选择
                :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   
                :add(casting_complete())	--施法动作播放完毕，之后不能被打断
                :add(play_animation('attack_1'))	-- 播放指定名称动画
                :add(play_vfx('Assets/Arts/fx/prefab/41_Prefab_Heavy_Taluo/skill10041_fuzhu_yanchen.prefab', 'lt_1', 5,true))	-- 播放辅助蒸汽特效，1秒消失
                :add(play_vfx('Assets/Arts/fx/prefab/41_Prefab_Heavy_Taluo/skill10041_fuzhu_huoxing.prefab', 'Ejector_4', 5,true))	-- 播放辅助蒸汽特效，1秒消失
                :add(play_vfx('Assets/Arts/fx/prefab/41_Prefab_Heavy_Taluo/skill10041_xuli_qiang.prefab', 'left_hit', 5,true))	-- 播放辅助蒸汽特效，1秒消失
                :add(play_vfx('Assets/Arts/fx/prefab/41_Prefab_Heavy_Taluo/skill10041_xuli_qiang.prefab', 'right_hit', 5,true))	-- 播放辅助蒸汽特效，1秒消失
                :add(play_vfx('Assets/Arts/fx/prefab/41_Prefab_Heavy_Taluo/skill10041_fuzhu_huoxing.prefab', 'Ejector_3', 5,true))	-- 播放辅助蒸汽特效，1秒消失
                :add(play_vfx('Assets/Arts/fx/prefab/41_Prefab_Heavy_Taluo/skill10041_fuzhu_zhengqi.prefab', 'Chimney_2', 5,true))	-- 播放辅助蒸汽特效，1秒消失
                :add(play_vfx('Assets/Arts/fx/prefab/41_Prefab_Heavy_Taluo/skill10041_fuzhu_zhengqi.prefab', 'Chimney_1', 5,true))	-- 播放辅助蒸汽特效，1秒消失
                :add(play_vfx('Assets/Arts/fx/prefab/41_Prefab_Heavy_Taluo/skill10041_fuzhu_huoxing.prefab', 'Ejector_2', 5,true))	-- 播放辅助蒸汽特效，1秒消失
                :add(play_vfx('Assets/Arts/fx/prefab/41_Prefab_Heavy_Taluo/skill10041_fuzhu_huoxing.prefab', 'Ejector_1', 5,true))	-- 播放辅助蒸汽特效，1秒消失
                :add(delay(2.57))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(shake_camera('zhenping_1')) --调用震屏
                :add(play_vfx('Assets/Arts/fx/prefab/41_Prefab_Heavy_Taluo/skill10041_start.prefab', 'left_hit', 5,true))	-- 播放蓄力强特效，1秒消失
                :add(play_vfx('Assets/Arts/fx/prefab/41_Prefab_Heavy_Taluo/skill10041_start.prefab', 'right_hit', 5,true))	-- 播放蓄力强特效，1秒消失
                :add(delay(0.53))
                :add(play_vfx('Assets/Arts/fx/prefab/41_Prefab_Heavy_Taluo/skill10041_start_chixu.prefab', 'right_hit', 200,true))        -- 射击子弹 prefab, speed, damage
                :add(play_vfx('Assets/Arts/fx/prefab/41_Prefab_Heavy_Taluo/skill10041_start_chixu.prefab', 'left_hit', 200,true))        -- 射击子弹 prefab, speed, damage
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.15))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.15))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.15))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.15))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.15))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.15))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.15))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.15))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(delay(0.15))
                :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                :add(play_vfx('Assets/Arts/fx/prefab/41_Prefab_Heavy_Taluo/skill10041_hit_baozha.prefab', 'lt_1', 200,true))        -- 射击子弹 prefab, speed, damage
                :add(cast_post_skill(999998))
                :add(delay(1.0))
                :add(finished())            -- 技能结束
                )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,

    -- 灌顶式反坦克导弹：普攻
    [42] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
            sequence()   -- 选择
                :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   
                :add(play_animation('attack'))  -- 播放指定名称动画
                :add(delay(0.23))
                :add(play_vfx('Assets/Arts/fx/prefab/42_Prefab_Artillery_Kajielinna/skill42_smoke.prefab', 'lt_1', 5))        -- 播放蒸汽特效，1秒消失
                :add(shoot_bullet(42, 'shoot', 80, consts.assets.curves.curve42))        -- 射击子弹
                :add(delay(1))
                :add(finished())            -- 技能结束
                )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,

    -- 灌顶式反坦克导弹:技能
    [10042] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
            
                sequence()   -- 选择
                    :add(select_multiple_enemy_randomly(8,false))
                    :add(play_animation('attack_1'))  -- 播放指定名称动作
                    :add(play_vfx('Assets/Arts/fx/prefab/42_Prefab_Artillery_Kajielinna/skill10042_smoke.prefab', 'lt_1', 5,true))        -- 播放开始特效，5秒消失
                    :add(play_vfx('Assets/Arts/fx/prefab/42_Prefab_Artillery_Kajielinna/skill10042_start.prefab', '330', 5,true))        -- 播放开始特效，5秒消失
                    :add(play_vfx('Assets/Arts/fx/prefab/42_Prefab_Artillery_Kajielinna/skill10042_fuzhu_penhuo.prefab', 'Ejector_2', 5,true))        -- 播放开始特效，5秒消失
                    :add(play_vfx('Assets/Arts/fx/prefab/42_Prefab_Artillery_Kajielinna/skill10042_fuzhu_penhuo.prefab', 'Ejector_1', 5,true))        -- 播放开始特效，5秒消失
                    :add(casting_complete())	--施法动作播放完毕，之后不能被打断
                   
                    --[[
                    :add(delay(1.56))
                    :add(set_enemy_from_multiple(1))
                    :add(shoot_bullet(10042, 'shoot_3', 100, consts.assets.curves.curve10042_3))        -- 射击子弹 prefab, speed, damage
                    :add(delay(0.55))
                    :add(set_enemy_from_multiple(2))
                    :add(shoot_bullet(10042, 'shoot_2', 100, consts.assets.curves.curve10042_2))        -- 射击子弹 prefab, speed, damage
                    :add(delay(0.1))
                    :add(set_enemy_from_multiple(3))
                    :add(shoot_bullet(10042, 'shoot_4', 100, consts.assets.curves.curve10042_4))        -- 射击子弹 prefab, speed, damage
                    :add(delay(0.55))
                    :add(set_enemy_from_multiple(4))
                    :add(shoot_bullet(10042, 'shoot', 100, consts.assets.curves.curve10042))        -- 射击子弹 prefab, speed, damage
                    :add(delay(0.1))
                    :add(set_enemy_from_multiple(5))
                    :add(shoot_bullet(10042, 'shoot_5', 100, consts.assets.curves.curve10042_5))        -- 射击子弹 prefab, speed, damage
                    :add(delay(0.25))
                    :add(set_enemy_from_multiple(6))
                    :add(shoot_bullet(10042, 'shoot_1', 100, consts.assets.curves.curve10042_1))        -- 射击子弹 prefab, speed, damage
                    :add(delay(0.1))
                    :add(set_enemy_from_multiple(7))
                    :add(shoot_bullet(10042, 'shoot_6', 100, consts.assets.curves.curve10042_6))        -- 射击子弹 prefab, speed, damage
                    :add(delay(0.4))
                    :add(set_enemy_from_multiple(8))
                    :add(shoot_bullet(10042, 'shoot_7', 100, consts.assets.curves.curve10042_7))        -- 射击子弹 prefab, speed, damage
                    :add(delay(2.5))]]

                    :add(finished())            -- 技能结束
            )
        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,

    -- 冷锋普攻
    [44] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   
                    :add(play_animation('attack'))  -- 播放指定名称动作
                    :add(delay(0.23))
                    :add(play_vfx('Assets/Arts/fx/prefab/44_Prefab_Tank_Lengfeng/skill44_smoke.prefab', 'lt_1',3,true))        -- 播放开始特效，1秒消失
                    --:add(play_vfx(301, 'shoot', 1))        -- 播放开始特效，1秒消失
                    :add(shoot_bullet(44, 'shoot', 200, consts.assets.curves.curve44))        -- 射击子弹 prefab, speed, damage
                    :add(delay(1.27))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,
    
        
    -- 冷锋技能
    [10044] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时
                    :add(play_animation('attack_1'))  -- 播放指定名称动作
                    :add(play_vfx('Assets/Arts/fx/prefab/44_Prefab_Tank_Lengfeng/skill10044_start2.prefab', 'shoot',3,true))        -- 播放开始特效，1秒消失
                    :add(play_vfx('Assets/Arts/fx/prefab/44_Prefab_Tank_Lengfeng/skill10044_start1.prefab', 'lt_1',3,true))        -- 播放开始特效，1秒消失
                    :add(play_vfx('Assets/Arts/fx/prefab/44_Prefab_Tank_Lengfeng/skill10044_juqi.prefab', 'shoot',3,true))        -- 播放开始特效，1秒消失
                    :add(casting_complete())	--施法动作播放完毕，之后不能被打断
                    :add(delay(0.23))
                    :add(play_vfx('Assets/Arts/fx/prefab/44_Prefab_Tank_Lengfeng/skill10044_smoke.prefab', 'lt_1',3,true))        -- 播放开始特效，1秒消失
                    :add(delay(1.27))
                    :add(shoot_bullet(10044, 'shoot', 250, consts.assets.curves.curve10044))        -- 射击子弹 prefab, speed, damage
                    -- :add(delay(0.27))
                    -- :add(shoot_bullet(10044, 'shoot', 300, consts.assets.curves.curve10044))        -- 射击子弹 prefab, speed, damage
                    -- :add(delay(0.27))
                    -- :add(shoot_bullet(10044, 'shoot', 300, consts.assets.curves.curve10044))        -- 射击子弹 prefab, speed, damage
                    :add(delay(1.5))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,

    -- 普通攻击(飞行器)
    [20001] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 2))   -- 最大偏差 5 度 2 秒超时   
                    :add(play_animation('attack'))      -- 播放动画
                    :add(delay(0.1))
                    :add(shoot_bullet_simple(20002, 'shoot', 100,consts.assets.curves.curve))         -- 射击子弹
					:add(delay(0.1))
					:add(shoot_bullet_simple(20002, 'shoot', 100,consts.assets.curves.curve))         -- 射击子弹
					:add(delay(0.1))
					:add(shoot_bullet_simple(20002, 'shoot', 100,consts.assets.curves.curve))         -- 射击子弹
                    :add(delay(0.2))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,

	    -- 突击兵普攻
    [20002] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 2))   -- 最大偏差 5 度 2 秒超时   
                    :add(play_animation('attack', true))      -- 播放动画
                    :add(delay(0.1))
                    :add(play_vfx('Assets/Arts/fx/prefab/jqb/jqb_loop.prefab', 'shoot',3,true))        -- 播放开始特效，1秒消失
                    :add(shoot_bullet_simple(20002, 'shoot', 200,0))         -- 射击子弹
					-- :add(delay(0.15))
                    -- :add(shoot_bullet(19, 'shoot', 200, consts.assets.curves.curve19))        -- 射击子弹
					-- :add(delay(0.15))
                    -- :add(shoot_bullet(19, 'shoot', 200, consts.assets.curves.curve19))        -- 射击子弹
                    :add(delay(0.5,1.0))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,

    -- 狙击兵普攻
    [20003] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 2))   -- 最大偏差 5 度 2 秒超时   
                    :add(play_animation('attack', true))      -- 播放动画
                    :add(delay(0.4))
                    :add(shoot_bullet_simple(20003, 'shoot', 800,1))         -- 射击子弹
					-- :add(delay(0.2))
					-- :add(shoot_bullet(20004, 'shoot', 200,consts.assets.curves.curve))         -- 射击子弹
					-- :add(delay(0.2))
					-- :add(shoot_bullet(20004, 'shoot', 200,consts.assets.curves.curve))         -- 射击子弹
                    :add(delay(0.5,1.5))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,
 
        
    -- 火箭兵普攻
    [20004] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 2))   -- 最大偏差 5 度 2 秒超时   
                    :add(play_animation('attack', true))      -- 播放动画
                    :add(shoot_bullet_simple(20004, 'shoot', 200,10))         -- 射击子弹
                    :add(delay(1.0,12.0))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,

    -- 掷弹兵普攻
    [20005] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 2))   -- 最大偏差 5 度 2 秒超时   
                    :add(play_animation('attack', true))      -- 播放动画
                    :add(shoot_bullet_simple(20005, 'shoot', 50,25))         -- 射击子弹
                    :add(delay(1.0,10.0))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,

	    -- 普通攻击(机械师)
    [20006] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 2))   -- 最大偏差 5 度 2 秒超时   
                    :add(play_animation('attack'))      -- 播放动画
                    --:add(delay(0.2))
                    :add(shoot_bullet_simple(20006, 'shoot', 200,consts.assets.curves.curve))         -- 射击子弹
                    :add(delay(1.2))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,	
	
    -- 强力射击1:对目标发射一次强力的穿甲弹，造成(p1)%攻击的穿甲伤害，如果目标是肉盾型战车，则造成额外(p2)%的伤害
    [10002] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   
                    :add(play_animation('attack_1'))  -- 播放指定攻击动作
					:add(casting_complete())	--施法动作播放完毕，之后不能被打断
                    :add(shoot_bullet(5, 'shoot', 200, nil,0, false, 0))        -- 发射一颗子弹 (shoot_effect_id, 骨骼名称, 飞行速度 or p(x), 弹道曲线,伤害半径 or p(x)(对目标区域造成范围伤害),是否穿透，伤害长度or p(x))
                    :add(delay(1.2))            --等待一段时间后清除特效
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,

    -- 10002的前置行为树
    [910002] = function(actor)
          if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
				sequence()   -- 测试
					:add(set_camera_size_factor(0.8,0.05))  --摄像机推近
                    :add(play_vfx(100105, 'hit', 1))        -- 播放聚焦特效，1秒消失
					:add(delay(0.1))    
					:add(play_animation('focus'))  -- 播放聚焦的动作
					:add(delay(0.5)) --时间根据动作时间来填，后面接持续的施法
					:add(play_vfx(100106, 'lt_1', 1))        --播放持续的施法特效，车底
					:add(play_vfx(100107, 'shoot', 1))        --播放持续的施法特效，炮口
					:add(play_animation('cast'))  --播放持续的施法特效
					:add(delay(99))  --保证在施法状态中
                    )

        bt:setContext(actor)
        return bt           
    end,

	
	--电钻的大招
    --[[[10003] = function(actor)
         if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
				sequence()   -- 测试
                    :add(facing_target(1, 2))   -- 最大差1度开始执行后续逻辑并且转向过程不超过2秒   对象技能的朝向
                    :add(play_animation('attack_1_rush'))  -- 播放指定攻击动作
					:add(casting_complete())	--施法动作播放完毕，之后不能被打断						
					:add(delay(0.2))
					:add(play_vfx(12,'shoot'))
					:add(delay(0.1))
					:add(play_animation('attack_1_loop'))  -- 播放指定攻击动作
					:add(shoot_bullet(90001,'shoot',200))
					:add(delay(0.6))
					:add(shoot_bullet(90001,'shoot',200))
					:add(delay(0.6))
					:add(shoot_bullet(90001,'shoot',200))
					:add(play_vfx(12,'shoot'))
					:add(delay(0.8))
					:add(play_animation('attack_1_back'))  -- 播放指定攻击动作
					:add(delay(0.5))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,]]
	
    -- 10003的前置行为树
    [910003] = function(actor)
          if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
				sequence()   -- 测试
					:add(set_camera_size_factor(0.8,0.05))  --摄像机推近
                    :add(play_vfx(100105, 'hit', 1))        -- 播放聚焦特效，1秒消失
					:add(delay(0.1))    
					:add(play_animation('focus'))  -- 播放聚焦的动作
					:add(delay(0.5)) --时间根据动作时间来填，后面接持续的施法
					:add(play_vfx(100106, 'lt_1', 1))        --播放持续的施法特效，车底
					:add(play_vfx(100107, 'shoot', 1))        --播放持续的施法特效，炮口
					:add(play_animation('cast'))  --播放持续的施法特效
					:add(delay(99))  --保证在施法状态中
                    )

        bt:setContext(actor)
        return bt           
    end,
	
	
	-- 烟雾弹：攻击目标圆形区域的坦克，造成(p1)%的高爆伤害，并降低(p2)点攻击力，区域半径(p3)码，最远攻击距离(p4)码
	-- 弹道飞行速度为p5，打击半径为p3
    [10004] = function(actor)
         if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
				sequence()   -- 测试
                    :add(facing_target(5, 4))   -- 最大差1度开始执行后续逻辑并且转向过程不超过2秒   对象技能的朝向
                    :add(play_animation('attack_1'))  -- 播放指定攻击动作
					:add(casting_complete())	--施法动作播放完毕，之后不能被打断						
                    :add(shoot_bullet_damage_in_area(3, 'shoot', 'p5', consts.assets.curves.curve2, 'p3', 'zhenping_1'))      --向目标范围发射导弹，指定固定范围计算伤害，显示范围和实际伤害范围有一定差别
                    :add(delay(0.2))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt 	
    end,
	
    -- 10004的前置行为树
    [910004] = function(actor)
          if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
				sequence()   -- 测试
					:add(set_camera_size_factor(0.8,0.05))  --摄像机推近
                    :add(play_vfx(100105, 'hit', 1))        -- 播放聚焦特效，1秒消失
					:add(delay(0.1))    
					:add(play_animation('focus'))  -- 播放聚焦的动作
					:add(delay(0.5)) --时间根据动作时间来填，后面接持续的施法
					:add(play_vfx(100106, 'lt_1', 1))        --播放持续的施法特效，车底
					:add(play_vfx(100107, 'shoot', 1))        --播放持续的施法特效，炮口
					:add(play_animation('cast'))  --播放持续的施法特效
					:add(delay(99))  --保证在施法状态中
                    )

        bt:setContext(actor)
        return bt           
    end,
		
	
    -- 暴击射击:发射一次强力的穿甲弹，造成(p1)%攻击的穿甲伤害，如果目标生命值低于(p2)%，则此次攻击必定暴击
	-- 选择p3范围内的一辆战车进行攻击，造成伤害
    [10005] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   
                    :add(play_animation('attack_1'))  -- 播放指定攻击动作
					:add(casting_complete())	--施法动作播放完毕，之后不能被打断					
                    :add(shoot_bullet(5, 'shoot', 200, nil,0, false, 0))        --发射一颗子弹 (shoot_effect_id, 骨骼名称, 飞行速度 or p(x), 弹道曲线,伤害半径 or p(x)(对目标区域造成范围伤害),是否穿透，伤害长度or p(x))
                    :add(delay(1.2))            --等待一段时间后清除特效
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,
	
    -- 10005的前置行为树
    [910005] = function(actor)
          if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
				sequence()   -- 测试
					:add(set_camera_size_factor(0.8,0.05))  --摄像机推近
                    :add(play_vfx(100105, 'hit', 1))        -- 播放聚焦特效，1秒消失
					:add(delay(0.1))    
					:add(play_animation('focus'))  -- 播放聚焦的动作
					:add(delay(0.5)) --时间根据动作时间来填，后面接持续的施法
					:add(play_vfx(100106, 'lt_1', 1))        --播放持续的施法特效，车底
					:add(play_vfx(100107, 'shoot', 1))        --播放持续的施法特效，炮口
					:add(play_animation('cast'))  --播放持续的施法特效
					:add(delay(99))  --保证在施法状态中
                    )

        bt:setContext(actor)
        return bt           
    end,	

    --BOSS喷火的技能
    [11006] = function(actor)
        if not actor then 
           log.error('actor is nil!!')
           return nil
       end

       local bt = root():setChild(
               sequence()   -- 测试
                   :add(facing_target(5, 4))   -- 最大差1度开始执行后续逻辑并且转向过程不超过2秒   对象技能的朝向
                   :add(show_skill_break_bar(8, true, true, false, {4,4}))
                   :add(delay(8.0))
                   :add(casting_complete())	--施法动作播放完毕，之后不能被打断	                 
                   :add(play_animation('attack_1'))  -- 播放指定攻击动作
                   :add(play_vfx('Assets/Arts/fx/prefab/start_yanwu_10006.prefab', 'y1', 1))
                   :add(play_vfx('Assets/Arts/fx/prefab/focus_6.prefab', 'shoot',3))
                   :add(delay(2.35))
                   :add(play_vfx(100061, 'shoot'))        -- 播放特效
                   --:add(shoot_bullet(10007, 'shoot', 20, nil,'p3', false, 'p4')) 					-- 发射一颗子弹 (shoot_effect_id, 骨骼名称, 飞行速度 or p(x), 弹道曲线,伤害半径 or p(x)(对目标区域造成范围伤害),是否穿透，伤害长度or p(x))
                   :add(loop(4)
                       :add(do_damage_in_rect(1, 'p3', 'p4'))
                       :add(delay(0.8))
                       )
                   :add(delay(0.5))
                   :add(finished())            -- 技能结束
                   )

       bt:setContext(actor)
       --log.info('create common bt! agent:'..tostring(agent.id))
       return bt
   end,	

    -- 10006的前置行为树
    [910006] = function(actor)
          if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
				sequence()   -- 测试
					:add(set_camera_size_factor(0.8,0.05))  --摄像机推近 
					:add(play_animation('focus'))  -- 播放聚焦的动作
					:add(play_vfx('Assets/Arts/fx/prefab/focus_6.prefab', 'shoot',3))
					:add(play_vfx('Assets/Arts/fx/prefab/start_yanwu_10006.prefab', 'y1', 1))        
					:add(delay(2.3)) --时间根据动作时间来填，后面接持续的施法
					:add(play_animation('cast'))  --播放持续的施法特效
					:add(delay(99))  --保证在施法状态中
                    )

        bt:setContext(actor)
        return bt           
    end,	
	
	
	-- 战车冲撞：向指定方向冲撞p3码，对沿途单位造成80%攻击力的冲撞伤害，并击退敌人，冲撞后会就近攻击敌人
	-- 战车的碰撞半径为p2
    [10007] = function(actor)
         if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
				sequence()   -- 测试
                    :add(facing_target(5, 4))   -- 最大差1度开始执行后续逻辑并且转向过程不超过2秒   对象技能的朝向
                    :add(play_animation('attack'))  -- 播放指定攻击动作
					:add(casting_complete())	--施法动作播放完毕，之后不能被打断
					:add(play_vfx(13, 'foot_1', 1))        --轮胎特效
					:add(play_vfx(13, 'foot_2', 1))        --轮胎特效					
                    :add(rush(7, 'shoot', 150, 'p2', 'p3', 2, 30))			--冲锋 (shoot_effect_id, 骨骼名称，速度，伤害半径，距离，超时时间，击退速度)
					:add(delay(0.5)) --延迟结束，播放rush之后的倒回动作
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,

    -- 10007的前置行为树
    [910007] = function(actor)
          if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
				sequence()   -- 测试
					:add(set_camera_size_factor(0.8,0.05))  --摄像机推近
                    :add(play_vfx(100105, 'hit', 1))        -- 播放聚焦特效，1秒消失
					:add(delay(0.1))    
					:add(play_animation('focus'))  -- 播放聚焦的动作
					:add(delay(0.5)) --时间根据动作时间来填，后面接持续的施法
					:add(play_vfx(100106, 'lt_1', 1))        --播放持续的施法特效，车底
					:add(play_vfx(100107, 'shoot', 1))        --播放持续的施法特效，炮口
					:add(play_animation('cast'))  --播放持续的施法特效
					:add(delay(99))  --保证在施法状态中
                    )

        bt:setContext(actor)
        return bt           
    end,	
	

    -- KV2坦克BOSS炮塔的技能：强力射击:对目标发射一次强力的穿甲弹，造成(p1)%攻击的穿甲伤害，如果目标是肉盾型战车，则造成额外(p2)%的伤害
    [11008] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(select_multiple_enemy_randomly(3,false))
					:add(set_enemy_from_multiple(1))
                    :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时
                    :add(show_skill_break_bar(8, true, true, false, {4,4}))
                    :add(delay(8.0))
                    :add(casting_complete())	--施法动作播放完毕，之后不能被打断
					:add(play_vfx('Assets/Arts/fx/prefab/focus_6.prefab', 'shoot'))        -- 播放特效					
                    :add(play_animation('attack_1',true))  -- 播放指定攻击动作
					:add(play_vfx('Assets/Arts/fx/prefab/start_yanwu_qibu_daoc_2.prefab', 'lt_1'))        -- 播放特效	
					:add(delay(1.8))
                    :add(shoot_bullet(10008, 'shoot', 450, consts.assets.curves.curve10008,0, false, 0,0,"zhenping_1"))        -- 发射一颗子弹 (shoot_effect_id, 骨骼名称, 飞行速度 or p(x), 弹道曲线,伤害半径 or p(x)(对目标区域造成范围伤害),是否穿透，伤害长度or p(x))
                    :add(play_vfx('Assets/Arts/fx/prefab/start_yanwu.prefab', 'shoot'))        -- 播放特效
					:add(set_enemy_from_multiple(2))
					:add(facing_target(5, 4))
					:add(play_animation('attack_1',true))  -- 播放指定攻击动作
					:add(play_vfx('Assets/Arts/fx/prefab/start_yanwu_qibu_daoc_2.prefab', 'lt_1'))        -- 播放特效	
					:add(delay(1.8))
					:add(shoot_bullet(10008, 'shoot', 450, consts.assets.curves.curve10008,0, false, 0,0,"zhenping_1"))
					:add(set_enemy_from_multiple(3))
					:add(facing_target(5, 4))
					:add(play_animation('attack_1',true))  -- 播放指定攻击动作
					:add(play_vfx('Assets/Arts/fx/prefab/start_yanwu_qibu_daoc_2.prefab', 'lt_1'))        -- 播放特效	
					:add(delay(1.8))
					:add(shoot_bullet(10008, 'shoot', 450, consts.assets.curves.curve10008,0, false, 0,0,"zhenping_1"))
					:add(add_buff(buff_targets.SELF,7))					
					:add(delay(1.6))            --等待一段时间后清除特效
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,

    -- 10008的前置行为树
    [910008] = function(actor)
          if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
				sequence()   -- 测试
					:add(set_camera_size_factor(0.8,0.05))  --摄像机推近
					:add(play_animation('focus',true))  -- 播放指定攻击动作
					:add(play_vfx('Assets/Arts/fx/prefab/focus_6.prefab', 'shoot',3))
					:add(play_vfx('Assets/Arts/fx/prefab/start_y anwu_qibu_daoc.prefab', 'lt_1'))        -- 播放特效	
					:add(delay(1.4))  --让战车的聚焦动作播放完毕
					:add(play_animation('cast',true))  --播放持续的施法特效
					:add(play_vfx('Assets/Arts/fx/prefab/start_yanwu_qibu_daoc_3.prefab', 'lt_1'))        -- 播放特效	
					:add(delay(99))  --保证在施法状态中
                    )
				
        bt:setContext(actor)
        return bt           
    end,	
    
	    -- 火箭炮的普攻
    [10] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   
                    :add(play_animation('attack'))  -- 播放指定名称动画
                    :add(delay(0.16))  --让战车的聚焦动作播放完毕
                    :add(shoot_bullet(10, 'shoot', 160, consts.assets.curves.curve10))        -- 射击子弹 prefab, speed, damage
                    --:add(shake_camera('zhenping_1')) 调用震屏
                    --:add(delay(0.45))
                    --:add(play_animation('attack'))  -- 播放指定名称动画
                    --:add(shoot_bullet(10, 'shoot', 180, consts.assets.curves.curve10_0))        -- 射击子弹 prefab, speed, damage
                    --:add(delay(0.45))
                    --:add(play_animation('attack'))  -- 播放指定名称动画
                    --:add(shoot_bullet(10, 'shoot', 180, consts.assets.curves.curve10))        -- 射击子弹 prefab, speed, damage
                    :add(delay(1.2))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,
	
    -- 必杀技：向目标圆形区域连续发射p1枚导弹攻击敌方坦克，圆形伤害半径为p2，每枚坦克造成(p3)%攻击的高爆伤害,最远攻击距离(p4)码
	-- 爆炸半径为p2，导弹的飞行速度为p5。每枚导弹的发射间隔为p6
    [10010] = function(actor)
         if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
				sequence()   -- 测试
					:add(select_multiple_enemy_randomly(1,false))
					:add(set_enemy_from_multiple(1))
                    :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   
                    :add(casting_complete())	--施法动作播放完毕，之后不能被打断
                    :add(play_animation('attack_1_rush'))  -- 播放指定攻击动作
					:add(play_vfx('Assets/Arts/fx/prefab/focus_1.prefab', 'shoot'))
					:add(delay(2))
					:add(play_vfx('Assets/Arts/fx/prefab/dianhuo_10010.prefab', 'shoot_1'))
					:add(delay(1))				
                    :add(play_animation('attack_1_loop'))  -- 播放指定攻击动作
                    :add(shoot_bullet_damage_in_area(10010, 'shoot', 130, consts.assets.curves.curve10010_1, 'p2',100, "zhenping_1"))                     -- 向目标范围发射导弹，指定固定范围计算伤害，显示范围和实际伤害范围有一定差别, 
                    :add(play_vfx('Assets/Arts/fx/prefab/start_yanwu.prefab', 'shoot'))
					:add(delay(0.4))
                    :add(play_animation('attack_1_loop'))  -- 播放指定攻击动作
                    :add(shoot_bullet_damage_in_area(10010, 'shoot_1', 130, consts.assets.curves.curve10010_3, 'p2',100, "zhenping_1"))                     -- 向目标范围发射导弹，指定固定范围计算伤害，显示范围和实际伤害范围有一定差别, 
					:add(play_vfx('Assets/Arts/fx/prefab/start_yanwu.prefab', 'shoot_1'))
					:add(delay(0.3))
                    :add(play_animation('attack_1_loop'))  -- 播放指定攻击动作
                    :add(shoot_bullet_damage_in_area(10010, 'shoot_2', 130, consts.assets.curves.curve10010_2, 'p2',100, "zhenping_1"))                     -- 向目标范围发射导弹，指定固定范围计算伤害，显示范围和实际伤害范围有一定差别, 
                    :add(play_vfx('Assets/Arts/fx/prefab/start_yanwu.prefab', 'shoot_2'))
					:add(delay(0.3))
                    :add(play_animation('attack_1_loop'))  -- 播放指定攻击动作
                    :add(shoot_bullet_damage_in_area(10010, 'shoot_3', 130, consts.assets.curves.curve10010_1, 'p2',100, "zhenping_1"))                     -- 向目标范围发射导弹，指定固定范围计算伤害，显示范围和实际伤害范围有一定差别, 
                    :add(play_vfx('Assets/Arts/fx/prefab/start_yanwu.prefab', 'shoot_3'))
					:add(delay(0.3))
                    :add(play_animation('attack_1_loop'))  -- 播放指定攻击动作
                    :add(shoot_bullet_damage_in_area(10010, 'shoot', 130, consts.assets.curves.curve10010_2, 'p2',100, "zhenping_1"))                     -- 向目标范围发射导弹，指定固定范围计算伤害，显示范围和实际伤害范围有一定差别, 
                    :add(play_vfx('Assets/Arts/fx/prefab/start_yanwu.prefab', 'shoot'))
					:add(delay(0.3))
                    :add(play_animation('attack_1_loop'))  -- 播放指定攻击动作
                    :add(shoot_bullet_damage_in_area(10010, 'shoot_1', 130, consts.assets.curves.curve10010_3, 'p2',100, "zhenping_1"))                     -- 向目标范围发射导弹，指定固定范围计算伤害，显示范围和实际伤害范围有一定差别, 
                    :add(play_vfx('Assets/Arts/fx/prefab/start_yanwu.prefab', 'shoot_1'))
					:add(delay(0.4))
					:add(play_animation('attack_1_back'))  -- 播放指定攻击动作
					:add(delay(1))
                    -- shoot_effect_id, bone, speed, damage_radius
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt       
    end,
	

    [11010] = function(actor)
         if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
				sequence()   -- 测试
					:add(select_multiple_enemy_randomly(1,false))
					:add(set_enemy_from_multiple(1))
                    :add(facing_target(1, 2))   -- 最大差1度开始执行后续逻辑并且转向过程不超过2秒   对象技能的朝向
					:add(play_vfx('Assets/Arts/fx/prefab/focus_1.prefab', 'shoot'))
                    :add(delay(2))
					:add(play_vfx('Assets/Arts/fx/prefab/dianhuo_10010.prefab', 'shoot'))
                    :add(delay(1))
                    :add(show_skill_break_bar(12, true, true, true, {4,4}))
					:add(delay(12))
                    :add(casting_complete())	--施法动作播放完毕，之后不能被打断				
                    :add(play_animation('attack_1'))  -- 播放指定攻击动作
                    :add(shoot_bullet_damage_in_area(10010, 'shoot', 100, consts.assets.curves.curve10010_1, 'p2',100, "zhenping_1"))                     -- 向目标范围发射导弹，指定固定范围计算伤害，显示范围和实际伤害范围有一定差别, 
                    :add(play_vfx('Assets/Arts/fx/prefab/start_yanwu.prefab', 'lt_1'))
					:add(delay(0.4))
                    :add(play_animation('attack_1'))  -- 播放指定攻击动作
                    :add(shoot_bullet_damage_in_area(10010, 'shoot', 100, consts.assets.curves.curve10010_3, 'p2',100, "zhenping_1"))                     -- 向目标范围发射导弹，指定固定范围计算伤害，显示范围和实际伤害范围有一定差别, 
					:add(play_vfx('Assets/Arts/fx/prefab/start_yanwu.prefab', 'lt_1'))
					:add(delay(0.2))
                    :add(play_animation('attack_1'))  -- 播放指定攻击动作
                    :add(shoot_bullet_damage_in_area(10010, 'shoot', 100, consts.assets.curves.curve10010_2, 'p2',100, "zhenping_1"))                     -- 向目标范围发射导弹，指定固定范围计算伤害，显示范围和实际伤害范围有一定差别, 
                    :add(play_vfx('Assets/Arts/fx/prefab/start_yanwu.prefab', 'lt_1'))
					:add(delay(0.2))
                    :add(play_animation('attack_1'))  -- 播放指定攻击动作
                    :add(shoot_bullet_damage_in_area(10010, 'shoot', 100, consts.assets.curves.curve10010_1, 'p2',100, "zhenping_1"))                     -- 向目标范围发射导弹，指定固定范围计算伤害，显示范围和实际伤害范围有一定差别, 
                    :add(play_vfx('Assets/Arts/fx/prefab/start_yanwu.prefab', 'lt_1'))
					:add(delay(0.3))
                    :add(play_animation('attack_1'))  -- 播放指定攻击动作
                    :add(shoot_bullet_damage_in_area(10010, 'shoot', 100, consts.assets.curves.curve10010_2, 'p2',100, "zhenping_1"))                     -- 向目标范围发射导弹，指定固定范围计算伤害，显示范围和实际伤害范围有一定差别, 
                    :add(play_vfx('Assets/Arts/fx/prefab/start_yanwu.prefab', 'lt_1'))
					:add(delay(0.3))
                    :add(play_animation('attack_1'))  -- 播放指定攻击动作
                    :add(shoot_bullet_damage_in_area(10010, 'shoot', 100, consts.assets.curves.curve10010_3, 'p2',100, "zhenping_1"))                     -- 向目标范围发射导弹，指定固定范围计算伤害，显示范围和实际伤害范围有一定差别, 
                    :add(play_vfx('Assets/Arts/fx/prefab/start_yanwu.prefab', 'lt_1'))
					:add(delay(0.2))
					:add(play_animation('attack_1_1'))  -- 播放指定攻击动作
					:add(delay(1))
                    -- shoot_effect_id, bone, speed, damage_radius
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt       
    end,
	
	
    -- 10010的前置行为树
    [910010] = function(actor)
          if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
				sequence()   -- 测试
					:add(set_camera_size_factor(0.8,0.05))  --摄像机推近
					:add(play_animation('focus',true))  -- 播放聚焦的动作
					:add(play_vfx('Assets/Arts/fx/prefab/focus_1.prefab', 'shoot',3))
					:add(delay(2)) --等待聚焦动作播放完毕
					:add(play_vfx(100105, 'shoot',1))        --播放炮口的点火特效		
					:add(delay(1)) --等待点火特效播放完毕					
					:add(play_animation('cast',true))  --播放持续的施法动作
					:add(delay(99))  --保证在施法状态中
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt           
    end,


	
	-- 横扫冲锋：将前方的圆锯轮盘横置，持续旋转，同时车体缓慢前进，对沿途的单位造成伤害（沿途触碰到的车体会被推开）
	-- 战车的碰撞半径为p2
    [10012] = function(actor)
         if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
				sequence()   -- 测试
                    :add(facing_target(5, 4))   -- 最大差1度开始执行后续逻辑并且转向过程不超过2秒   对象技能的朝向					
					:add(casting_complete())	--施法动作播放完毕，之后不能被打断
					:add(play_animation('attack_1'))  -- 播放指定攻击动
					:add(play_vfx('Assets/Arts/fx/prefab/focus_6.prefab', 'shoot', 3))        --播放聚焦动作伴随特效
					:add(play_vfx('Assets/Arts/fx/prefab/start_yanwu_qibu_daoc.prefab', 'lt_1', 3))        --播放聚焦动作伴随特效
					:add(delay(1.1))
					:add(play_vfx('Assets/Arts/fx/prefab/skill10012_cast_1.prefab', 'shoot',3))
					:add(delay(1.2))
					:add(play_vfx('Assets/Arts/fx/prefab/start_yanwu_qibu_daoc_2.prefab', 'lt_1', 3))        --播放聚焦动作伴随特效		
					:add(play_vfx(100122, 'shoot', 1))        --轮盘处的持续播放特效					
                    --:add(rush(10012, 'shoot', 20, 'p2',30, 10, 1))			--冲锋 (shoot_effect_id, 骨骼名称，速度，伤害半径，距离，超时时间，击退速度)
					:add(play_animation('attack_1_1'))  -- 播放结束动作
					:add(delay(0.3))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,

    -- 10012的前置行为树
    [910012] = function(actor)
          if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
				sequence()   -- 测试
					:add(set_camera_size_factor(0.8,0.05))  --摄像机推近					
					:add(play_animation('focus'))  -- 播放聚焦的动作
					:add(play_vfx('Assets/Arts/fx/prefab/focus_6.prefab', 'shoot',3))
					:add(play_vfx('Assets/Arts/fx/prefab/start_yanwu_qibu_daoc.prefab', 'lt_1', 3))        --播放聚焦动作伴随特效
					:add(delay(1.1))
					:add(play_animation('cast'))  --播放持续的施法特效
					:add(play_vfx('Assets/Arts/fx/prefab/skill10012_cast_1.prefab', 'shoot',99))        --播放聚焦动作伴随特效
					:add(delay(1.4))
					:add(play_vfx('Assets/Arts/fx/prefab/start_yanwu_qibu_daoc_2', 'lt_1', 3))        --播放聚焦动作伴随特效
					:add(delay(99))  --保证在施法状态中
                    )

        bt:setContext(actor)
        return bt           
    end,	

	
    -- 10014的前置行为树
    [910014] = function(actor)
          if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
				sequence()   -- 测试
					:add(set_camera_size_factor(0.8,0.05))  --摄像机推近  
					:add(play_animation('focus'))  -- 播放聚焦的动作
					:add(play_vfx('Assets/Arts/fx/prefab/focus_2.prefab', 'shoot',3))
					:add(play_vfx('Assets/Arts/fx/prefab/start_yanwu_focus_10014.prefab', 'shoot', 3))        --播放聚焦动作伴随烟雾特效
					--Assets/Arts/fx/prefab/start_yanwu_focus_10014.prefab
					:add(delay(1.2)) --时间根据动作时间来填
					:add(play_vfx('Assets/Arts/fx/prefab/14_Prefab_Artillery_Nvpu/skill10014_start.prefab', 'shoot', 3))        --播放毒球掉落特效
					:add(delay(1.8)) --时间根据动作时间来填
					:add(play_animation('cast'))  --播放持续的施法特效
					:add(delay(99))  --保证在施法状态中
                    )

        bt:setContext(actor)
        return bt           
    end,	

    -- 10015的前置行为树
    [910015] = function(actor)
          if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
				sequence()   -- 测试
					:add(set_camera_size_factor(0.8,0.05))  --摄像机推近
                    :add(play_vfx(100105, 'hit', 1))        -- 播放聚焦特效，1秒消失
					:add(delay(0.1))    
					:add(play_animation('focus'))  -- 播放聚焦的动作
					:add(delay(0.5)) --时间根据动作时间来填，后面接持续的施法
					:add(play_vfx(100106, 'lt_1', 1))        --播放持续的施法特效，车底
					:add(play_vfx(100107, 'shoot', 1))        --播放持续的施法特效，炮口
					:add(play_animation('cast'))  --播放持续的施法特效
					:add(delay(99))  --保证在施法状态中
                    )

        bt:setContext(actor)
        return bt           
    end,
	
    -- 爆炸长矛组：杀人锋 将所有五根爆炸长矛向敌方阵地发射，对敌方单体造成一次强力的伤害，每根长矛造成(p1%)攻击的爆炸伤害
    [10027] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   
                    :add(play_animation('attack_1'))  -- 播放指定攻击动作
					:add(casting_complete())	--施法动作播放完毕，之后不能被打断					
                    :add(shoot_bullet(10027, 'shoot', 100, consts.assets.curves.curve1,0, false, 0))        --发射一颗子弹 (shoot_effect_id, 骨骼名称, 飞行速度 or p(x), 弹道曲线,伤害半径 or p(x)(对目标区域造成范围伤害),是否穿透，伤害长度or p(x))
                    :add(delay(1.2))            --等待一段时间后清除特效
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,
	
    -- 10027的前置行为树
    [910027] = function(actor)
          if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
				sequence()   -- 测试
					:add(set_camera_size_factor(0.8,0.05))  --摄像机推近
                    :add(play_vfx(100105, 'hit', 1))        -- 播放聚焦特效，1秒消失
					:add(delay(0.1))    
					:add(play_animation('focus'))  -- 播放聚焦的动作
					:add(delay(0.5)) --时间根据动作时间来填，后面接持续的施法
					:add(play_vfx(100106, 'lt_1', 1))        --播放持续的施法特效，车底
					:add(play_vfx(100107, 'shoot', 1))        --播放持续的施法特效，炮口
					:add(play_animation('cast'))  --播放持续的施法特效
					:add(delay(99))  --保证在施法状态中
                    )

        bt:setContext(actor)
        return bt           
    end,
		
    -- -- 丘吉尔：爆裂射击  向敌方后排单体进行一次强力射击，造成一次攻击(p1)%的爆炸伤害。本次攻击必定暴击，且战车的每1%的暴击率会提高1%的伤害
    -- [10036] = function(actor)
    --     if not actor then 
    --         log.error('actor is nil!!')
    --         return nil
    --     end

    --     local bt = root():setChild(
    --             sequence()   -- 选择
    --                 :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   
    --                 :add(play_animation('attack_1'))  -- 播放指定攻击动作
	-- 				:add(casting_complete())	--施法动作播放完毕，之后不能被打断					
    --                 :add(shoot_bullet(10036, 'shoot', 300, nil,0, false, 0))        --发射一颗子弹 (shoot_effect_id, 骨骼名称, 飞行速度 or p(x), 弹道曲线,伤害半径 or p(x)(对目标区域造成范围伤害),是否穿透，伤害长度or p(x))
    --                 :add(delay(1.2))            --等待一段时间后清除特效
    --                 :add(finished())            -- 技能结束
    --                 )

    --     bt:setContext(actor)
    --     --log.info('create common bt! agent:'..tostring(agent.id))
    --     return bt
    -- end,
	
    -- 10036的前置行为树
    [910036] = function(actor)
          if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
				sequence()   -- 测试
					:add(set_camera_size_factor(0.8,0.05))  --摄像机推近
                    :add(play_vfx(100105, 'hit', 1))        -- 播放聚焦特效，1秒消失
					:add(delay(0.1))    
					:add(play_animation('focus'))  -- 播放聚焦的动作
					:add(delay(0.5)) --时间根据动作时间来填，后面接持续的施法
					:add(play_vfx(100106, 'lt_1', 1))        --播放持续的施法特效，车底
					:add(play_vfx(100107, 'shoot', 1))        --播放持续的施法特效，炮口
					:add(play_animation('cast'))  --播放持续的施法特效
					:add(delay(99))  --保证在施法状态中
                    )

        bt:setContext(actor)
        return bt           
    end,
	
	
	-- 火炮型战车的战前火力轰炸
    [50010] = function(actor)
         if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
				sequence()   -- 测试
                    :add(play_animation('attack_1'))  -- 播放指定攻击动作
                    :add(shoot_bullet_to_enemy_field(10010, 'shoot', 50, consts.assets.curves.curve10010_1, 200,80, "zhenping_1"))     --向敌人阵地随机位置发射子弹 (shoot_effect_id, 骨骼名称, 飞行速度 or p(x), 弹道曲线, 伤害半径 or p(x),高度参数（0：等比缩放，其他数：缩放比例）,震屏方式) 
                    :add(delay(0.3))
                    :add(play_animation('attack_1'))  -- 播放指定攻击动作
                    :add(shoot_bullet_to_enemy_field(10010, 'shoot', 50, consts.assets.curves.curve10010_1, 200,80, "zhenping_1"))     --向敌人阵地随机位置发射子弹 (shoot_effect_id, 骨骼名称, 飞行速度 or p(x), 弹道曲线, 伤害半径 or p(x),高度参数（0：等比缩放，其他数：缩放比例）,震屏方式)  
					:add(delay(0.5))
                    :add(play_animation('attack_1'))  -- 播放指定攻击动作
                    :add(shoot_bullet_to_enemy_field(10010, 'shoot', 50, consts.assets.curves.curve10010_1, 200,80, "zhenping_1"))     --向敌人阵地随机位置发射子弹 (shoot_effect_id, 骨骼名称, 飞行速度 or p(x), 弹道曲线, 伤害半径 or p(x),高度参数（0：等比缩放，其他数：缩放比例）,震屏方式) 
                    :add(delay(0.1))
                    :add(play_animation('attack_1'))  -- 播放指定攻击动作
                    :add(shoot_bullet_to_enemy_field(10010, 'shoot', 50, consts.assets.curves.curve10010_1, 200,80, "zhenping_1"))     --向敌人阵地随机位置发射子弹 (shoot_effect_id, 骨骼名称, 飞行速度 or p(x), 弹道曲线, 伤害半径 or p(x),高度参数（0：等比缩放，其他数：缩放比例）,震屏方式) 
                    --[[:add(delay(0.1))
                    :add(play_animation('attack_1'))  -- 播放指定攻击动作
                    :add(shoot_bullet_to_enemy_field(10010, 'shoot', 50, consts.assets.curves.curve2, 200,50, "zhenping_1"))     --向敌人阵地随机位置发射子弹 (shoot_effect_id, 骨骼名称, 飞行速度 or p(x), 弹道曲线, 伤害半径 or p(x),高度参数（0：等比缩放，其他数：缩放比例）,震屏方式) 
                    :add(delay(0.5))
                    :add(play_animation('attack_1'))  -- 播放指定攻击动作
                    :add(shoot_bullet_to_enemy_field(10010, 'shoot', 50, consts.assets.curves.curve2, 200,50))      --向敌人阵地随机位置发射子弹 (shoot_effect_id, 骨骼名称, 飞行速度 or p(x), 弹道曲线, 伤害半径 or p(x),高度参数（0：等比缩放，其他数：缩放比例）,震屏方式) 
                    :add(delay(0.1))
                    :add(play_animation('attack_1'))  -- 播放指定攻击动作
                    :add(shoot_bullet_to_enemy_field(10010, 'shoot', 50, consts.assets.curves.curve2, 200,50))      --向敌人阵地随机位置发射子弹 (shoot_effect_id, 骨骼名称, 飞行速度 or p(x), 弹道曲线, 伤害半径 or p(x),高度参数（0：等比缩放，其他数：缩放比例）,震屏方式) 
                    :add(delay(0.1))
                    :add(play_animation('attack_1'))  -- 播放指定攻击动作
                    :add(shoot_bullet_to_enemy_field(10010, 'shoot', 50, consts.assets.curves.curve2, 200,50, "zhenping_1"))      --向敌人阵地随机位置发射子弹 (shoot_effect_id, 骨骼名称, 飞行速度 or p(x), 弹道曲线, 伤害半径 or p(x),高度参数（0：等比缩放，其他数：缩放比例）,震屏方式) 
                    :add(delay(0.5))
                    :add(play_animation('attack_1'))  -- 播放指定攻击动作
                    :add(shoot_bullet_to_enemy_field(10010, 'shoot', 50, consts.assets.curves.curve2, 200,50))      --向敌人阵地随机位置发射子弹 (shoot_effect_id, 骨骼名称, 飞行速度 or p(x), 弹道曲线, 伤害半径 or p(x),高度参数（0：等比缩放，其他数：缩放比例）,震屏方式) 
					:add(delay(0.5))]]
                    -- shoot_effect_id, bone, speed, damage_radius
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt       
    end,
	
	-- 坦克型战车的移动射击

    [50008] = function(actor)
           if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                    sequence()
						:add(parallel()   -- 并行结点
							:add(move_forward(100))
							:add(sequence()
								:add(loop(5)  --最多loop5次 怕万一直线行走没有在射程中的单位就会一直移动
									:add(play_animation('attack'))  -- 播放指定攻击动作
									:add(shoot_bullet_forward(50008, 'shoot', 'p10', consts.assets.curves.curve, 200,10))   
									:add(delay(1.5))
									)
								:add(finished())
								)
							:add(sequence()
                                :add(loop_until(is_enemy_in_attack_range())
                                    :add(delay(0.05)))
                                :add(finished())
                                )			
						    )
                    :add(finished())            -- 技能结束
					)
        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
						
        return bt          
    end,

	
	-- 肉盾型战车的冲锋
    [50007] = function(actor)
           if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                    sequence()
                        :add(parallel()   -- 并行结点
                            :add(sequence()
								:add(set_avoidance_quality(0))    --设定避让品质，它不会避让任何单位
								:add(set_avoidance_priority(100))  --设定避让优先级，任何单位都不会避让它
								--:add(play_vfx('Assets/Arts/fx/prefab/assult_rush_wheel.prefab', 'lt_1')) 
								:add(rush_forward(90003, 'rush', 30,15,100,10,1))    -- 向目标冲(shoot_effect_id, 骨骼名称，速度，伤害半径，距离, 超时时间，击退速度)
								:add(add_buff(buff_targets.SELF ,7))
								:add(finished()))
                            :add(sequence()
                                :add(loop_until(is_hit_enemy())
                                    :add(delay(0.05)))
                                :add(stop_rush())
								:add(set_avoidance_quality(4))
								:add(set_avoidance_priority())
								:add(add_buff(buff_targets.SELF ,7))
								:add(finished()))
                                )
                        :add(finished())
                        )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt          
    end,
	


	-- 突击型战车的冲锋
    [50006] = function(actor)
           if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                    sequence()
                        :add(parallel()   -- 并行结点
                            :add(sequence()
                            :add(add_buff(buff_targets.SELF ,9))
								:add(delay(1.5))
								:add(set_avoidance_quality(0))
								:add(set_avoidance_priority(100))
								:add(play_vfx('Assets/Arts/fx/prefab/assult_rush_wheel.prefab', 'lt_1'))
								:add(rush_forward(90003, 'rush', 120 , 10 , 210 , 10 , 1))    -- 向目标冲(shoot_effect_id, 骨骼名称，速度，伤害半径，距离, 超时时间，击退速度)								 
								:add(add_buff(buff_targets.SELF ,9))
								:add(finished()))
                            :add(sequence()
                                :add(loop_until(is_hit_enemy())
                                    :add(delay(0.05)))
                                :add(stop_rush())
								--:add(play_vfx('Assets/Arts/fx/prefab/assult_rush.prefab', 'shoot',1)) 
								:add(set_avoidance_quality(4))
								:add(set_avoidance_priority())
								:add(add_buff(buff_targets.SELF ,9))
								:add(finished()))
                                )
							)
        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt          
    end,

	-- 肉盾型战车的冲锋
    [50012] = function(actor)
           if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                    sequence()
                        :add(parallel()   -- 并行结点
                            :add(sequence()
								:add(set_avoidance_quality(0))    --设定避让品质，它不会避让任何单位
								:add(set_avoidance_priority(100))  --设定避让优先级，任何单位都不会避让它
								--:add(play_vfx('Assets/Arts/fx/prefab/assult_rush_wheel.prefab', 'lt_1')) 
								:add(rush_forward(90003, 'rush', 30,15,100,10,1))    -- 向目标冲(shoot_effect_id, 骨骼名称，速度，伤害半径，距离, 超时时间，击退速度)
								:add(add_buff(buff_targets.SELF ,7))
								:add(finished()))
                            :add(sequence()
                                :add(loop_until(is_hit_enemy())
                                    :add(delay(0.05)))
                                :add(stop_rush())
								:add(set_avoidance_quality(4))
								:add(set_avoidance_priority())
								:add(add_buff(buff_targets.SELF ,7))
								:add(finished()))
                                )
                        :add(finished())
                        )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt          
    end,

	--什么都不干的开场技能
	[999999] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,

   	--蜘蛛开场技能
	[101101] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(add_buff(buff_targets.SELF ,9))
                    --:add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   
                    --:add(delay(5.2))   
                    :add(play_animation('born'))  -- 播放指定攻击动作
                    :add(delay(0.2))       
                    :add(play_vfx('Assets/Arts/fx/prefab/Prefab_Boss_Spider/skill1011_born_smoke.prefab', 'd',5,true)) 
                    :add(delay(1.45))
                    :add(play_vfx('Assets/Arts/fx/prefab/Prefab_Boss_Spider/skill1011_born_dilie.prefab', 'IK Chain001',5,true)) 
                    :add(delay(1.35))
                    :add(play_vfx('Assets/Arts/fx/prefab/Prefab_Boss_Spider/skill1011_born_dilie.prefab', 'IK Chain006',5,true)) 
                    :add(delay(0.1))
                    :add(play_vfx('Assets/Arts/fx/prefab/Prefab_Boss_Spider/skill1011_born_dilie.prefab', 'IK Chain001',5,true)) 
                    :add(delay(1.9))
                    :add(play_vfx('Assets/Arts/fx/prefab/Prefab_Boss_Spider/skill1011_born_nuhou.prefab', 'Bone034',5,true)) 
                    :add(delay(1.2))            --等待一段时间后清除特效
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,

   	--蜘蛛死亡技能
    [101102] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    --:add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时    
                    :add(play_animation('broken'))  -- 播放指定攻击动作    
                    :add(play_vfx('Assets/Arts/fx/prefab/Prefab_Boss_Spider/skill1011_broken_loudian.prefab', 'Bone011',5,true)) 
                    :add(play_vfx('Assets/Arts/fx/prefab/Prefab_Boss_Spider/skill1011_broken_tuibu_zuo.prefab', 'Bone012',5,true)) 
                    :add(play_vfx('Assets/Arts/fx/prefab/Prefab_Boss_Spider/skill1011_broken_baozha.prefab', 'Bone011',5,true)) 
                    :add(play_vfx('Assets/Arts/fx/prefab/Prefab_Boss_Spider/skill1011_broken_loudian_2.prefab', 'Bone015',5,true)) 
                    :add(play_vfx('Assets/Arts/fx/prefab/Prefab_Boss_Spider/skill1011_broken_baozha.prefab', 'Bone015',5,true)) 
                    :add(play_vfx('Assets/Arts/fx/prefab/Prefab_Boss_Spider/skill1011_broken_tuibu_you.prefab', 'Bone004',5,true)) 
                    :add(play_vfx('Assets/Arts/fx/prefab/Prefab_Boss_Spider/skill1011_broken_baozha.prefab', 'Bone003',5,true)) 
                    :add(play_vfx('Assets/Arts/fx/prefab/Prefab_Boss_Spider/skill1011_broken_loudian.prefab', 'Bone003',5,true)) 
                    :add(play_vfx('Assets/Arts/fx/prefab/Prefab_Boss_Spider/skill1011_broken_loudian_2.prefab', 'Bone019',5,true)) 
                    :add(delay(0.3))
                    :add(play_vfx(' Assets/Arts/fx/prefab/Prefab_Boss_Spider/skill1011_broken_baozha.prefab', 'Bone019',5,true)) 
                    :add(play_vfx(' Assets/Arts/fx/prefab/Prefab_Boss_Spider/skill1011_broken_baozha.prefab', 'Bone003',5,true)) 
                    :add(delay(0.1))
                    :add(play_vfx('Assets/Arts/fx/prefab/Prefab_Boss_Spider/skill1011_broken_baozha.prefab', 'Bone011',5,true)) 
                    :add(delay(0.4))
                    :add(play_vfx('Assets/Arts/fx/prefab/Prefab_Boss_Spider/skill1011_broken_baozha.prefab', 'Bone003',5,true)) 
                    :add(play_vfx('Assets/Arts/fx/prefab/Prefab_Boss_Spider/skill1011_broken_baozha.prefab', 'Bone011',5,true)) 
                    :add(delay(0.2))
                    :add(play_vfx('Assets/Arts/fx/prefab/Prefab_Boss_Spider/skill1011_broken_baozha.prefab', 'd',5,true)) 
                    :add(delay(1.2))            --等待一段时间后清除特效
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,

   	--蜘蛛导弹技能
    [101103] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(select_multiple_enemy_randomly(4,false))
                    --:add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   
                    :add(casting_complete())	--施法动作播放完毕，之后不能被打断
                    :add(play_animation('attack'))  -- 播放指定名称动画
                    :add(play_vfx('Assets/Arts/fx/prefab/Prefab_Boss_Spider/skill21011_suoding.prefab', 'lt_1',5,true)) 
                    :add(delay(1.9))
                    :add(set_enemy_from_multiple(1))
                    :add(shoot_bullet(21011, 'shoot_1', 200, consts.assets.curves.curve_zhizhu_1,0, false, 0,0,"zhenping_1"))        -- 射击子弹 prefab, speed, damage
                    :add(shoot_bullet(21011, 'shoot_4', 200, consts.assets.curves.curve_zhizhu_2,0, false, 0,0,"zhenping_1"))        -- 射击子弹 prefab, speed, damage
                    :add(shoot_bullet(21011, 'shoot_7', 200, consts.assets.curves.curve_zhizhu_1,0, false, 0,0,"zhenping_1"))        -- 射击子弹 prefab, speed, damage
                    :add(shoot_bullet(21011, 'shoot_14', 200, consts.assets.curves.curve_zhizhu_3,0, false, 0,0,"zhenping_1"))        -- 射击子弹 prefab, speed, damage
                    :add(shake_camera('zhenping_1')) --调用震屏
                    :add(delay(0.2))
                    :add(set_enemy_from_multiple(2))
                    :add(shoot_bullet(21011, 'shoot_2', 200, consts.assets.curves.curve_zhizhu_4,0, false, 0,0,"zhenping_1"))        -- 射击子弹 prefab, speed, damage
                    :add(shoot_bullet(21011, 'shoot_6', 200, consts.assets.curves.curve_zhizhu_1,0, false, 0,0,"zhenping_1"))        -- 射击子弹 prefab, speed, damage
                    :add(shoot_bullet(21011, 'shoot_10', 200, consts.assets.curves.curve_zhizhu_2,0, false, 0,0,"zhenping_1"))        -- 射击子弹 prefab, speed, damage
                    :add(shoot_bullet(21011, 'shoot_16', 200, consts.assets.curves.curve_zhizhu_1,0, false, 0,0,"zhenping_1"))        -- 射击子弹 prefab, speed, damage
                    :add(shake_camera('zhenping_1')) --调用震屏
                    :add(delay(0.3))
                    :add(set_enemy_from_multiple(3))
                    :add(shoot_bullet(21011, 'shoot_3', 200, consts.assets.curves.curve_zhizhu_3,0, false, 0,0,"zhenping_1"))        -- 射击子弹 prefab, speed, damage
                    :add(shoot_bullet(21011, 'shoot_8', 200, consts.assets.curves.curve_zhizhu_1,0, false, 0,0,"zhenping_1"))        -- 射击子弹 prefab, speed, damage
                    :add(shoot_bullet(21011, 'shoot_11', 200, consts.assets.curves.curve_zhizhu_1,0, false, 0,0,"zhenping_1"))        -- 射击子弹 prefab, speed, damage
                    :add(shoot_bullet(21011, 'shoot_13', 200, consts.assets.curves.curve_zhizhu_4,0, false, 0,0,"zhenping_1"))        -- 射击子弹 prefab, speed, damage
                    :add(shake_camera('zhenping_1')) --调用震屏
                    :add(delay(0.2))
                    :add(set_enemy_from_multiple(4))
                    :add(shoot_bullet(21011, 'shoot_5', 200, consts.assets.curves.curve_zhizhu_2,0, false, 0,0,"zhenping_1"))        -- 射击子弹 prefab, speed, damage
                    :add(shoot_bullet(21011, 'shoot_9', 200, consts.assets.curves.curve_zhizhu_1,0, false, 0,0,"zhenping_1"))        -- 射击子弹 prefab, speed, damage
                    :add(shoot_bullet(21011, 'shoot_12', 200, consts.assets.curves.curve_zhizhu_1,0, false, 0,0,"zhenping_1"))        -- 射击子弹 prefab, speed, damage
                    :add(shoot_bullet(21011, 'shoot_15', 200, consts.assets.curves.curve_zhizhu_3,0, false, 0,0,"zhenping_1"))        -- 射击子弹 prefab, speed, damage
                    :add(shake_camera('zhenping_1')) --调用震屏
                    :add(delay(1.2))            --等待一段时间后清除特效
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,

   	--蜘蛛飞扑技能
       [101104] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    --:add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   
                    :add(play_animation('skill1'))  -- 播放指定攻击动作
                    :add(play_vfx('Assets/Arts/fx/prefab/Prefab_Boss_Spider/skill1011_feipu_tuibu_you.prefab', 'Bone004',5,true)) 
                    :add(play_vfx('Assets/Arts/fx/prefab/Prefab_Boss_Spider/skill1011_feipu_tuibu_zuo.prefab', 'Bone012',5,true)) 
                    :add(delay(1))
                    :add(play_vfx('Assets/Arts/fx/prefab/Prefab_Boss_Spider/skill1011_feipu_yanchen.prefab', 'd_1',5,true)) 
                    :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                    :add(delay(1.2))            --等待一段时间后清除特效
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,    

    --蜘蛛喷毒技能
        [101105] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(select_multiple_enemy_randomly(8,false))   --在全场范围内选择敌人
                    --:add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   
                    :add(play_animation('skill2'))  -- 播放指定攻击动作
                    :add(play_vfx('Assets/Arts/fx/prefab/Prefab_Boss_Spider/skill11011_xuli.prefab', 'Bone034',5,true)) 
                    :add(delay(1.1))
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(1))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(2))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(3))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(4))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(5))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(6))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(7))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(8))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(play_vfx('Assets/Arts/fx/prefab/Prefab_Boss_Spider/skill11011_pendu.prefab', 'Bone034',5,true)) 
                    :add(delay(0.1))
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(1))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(2))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(3))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(4))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(5))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(6))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(7))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(8))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(play_vfx('Assets/Arts/fx/prefab/Prefab_Boss_Spider/skill11011_dimian.prefab', 'd_2',5,true)) 
                    :add(delay(0.4))
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(1))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(2))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(3))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(4))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(5))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(6))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(7))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(8))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(delay(0.4))
                    :add(play_vfx('Assets/Arts/fx/prefab/Prefab_Boss_Spider/skill11011_dimian.prefab', 'd_2',5,true)) 
                    :add(delay(0.4))
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(1))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(2))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(3))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(4))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(5))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(6))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(7))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(8))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(delay(0.4))
                    :add(play_vfx('Assets/Arts/fx/prefab/Prefab_Boss_Spider/skill11011_dimian.prefab', 'd_2',5,true)) 
                    :add(delay(0.3))
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(1))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(2))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(3))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(4))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(5))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(6))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(7))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(8))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(delay(0.4))
                    :add(play_vfx('Assets/Arts/fx/prefab/Prefab_Boss_Spider/skill11011_dimian.prefab', 'd_2',5,true)) 
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(1))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(2))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(3))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(4))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(5))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(6))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(7))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(sequence_success()
                        :add(set_enemy_from_multiple(8))
                        :add(shoot_bullet(90001, 'shoot', 200)))        -- 射击子弹 prefab, speed, damage
                    :add(delay(1.5))            --等待一段时间后清除特效
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,  
        
    --蜘蛛轧车技能
    [101106] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    --:add(facing_target(5, 4))   -- 最大偏差 8 度 4 秒超时   
                    :add(play_vfx('Assets/Arts/fx/prefab/Prefab_Boss_Spider/skill1011_zhache_tuibu_zuo.prefab', 'Bone012',5,true)) 
                    :add(delay(0.6))
                    :add(play_vfx('Assets/Arts/fx/prefab/Prefab_Boss_Spider/skill1011_zhache_dilie.prefab', 'IK Chain005',5,true)) 
                    :add(delay(0.2))
                    :add(play_vfx('Assets/Arts/fx/prefab/Prefab_Boss_Spider/skill1011_zhache_tuibu_you.prefab', 'Bone004',5,true)) 
                    :add(delay(0.8))
                    :add(play_vfx('Assets/Arts/fx/prefab/Prefab_Boss_Spider/skill11011_dimian.prefab', 'd_2',5,true)) 
                    :add(delay(1.0))
                    :add(play_vfx('Assets/Arts/fx/prefab/Prefab_Boss_Spider/skill1011_zhache_dilie.prefab', 'IK Chain001',5,true)) 
                    :add(delay(1.7))
                    :add(play_vfx('Assets/Arts/fx/prefab/Prefab_Boss_Spider/skill1011_zhache_bachu.prefab', 'd_2',5,true)) 
                    :add(delay(1.2))            --等待一段时间后清除特效
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,  

	-- 副武器攻击
    [30001] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择 
					:add(delay(0.8))
                    :add(shoot_bullet(10, 'shoot', 50, consts.assets.curves.curve1))        -- 射击子弹 prefab, speed, damage
                    :add(delay(1.2))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,
	
	-- 副武器攻击
    [30001] = function(actor)
        if not actor then 
            log.error('actor is nil!!')
            return nil
        end

        local bt = root():setChild(
				sequence()   -- 选择 
					:add(select()   -- 选择
						:add(sequence() -- 敌人在攻击范围内
                                :add(is_enemy_in_attack_range())
								:add(begin_cd())
								:add(delay(0,2))
                                :add(play_animation('attack'))  -- 播放指定名称动画
								:add(shoot_bullet(30001, 'shoot', 100, consts.assets.curves.curve))        -- 射击子弹 prefab, speed, damage
								:add(delay(0.5)))
						:add(sequence() -- 敌人不在范围内
                                :add(delay(0.5,1))))
					:add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,

    [999998] = function(actor)
        if not actor then 
            return nil
        end

        local bt = root():setChild(
                sequence()   -- 选择
                    :add(shoot_bullet(90001, 'shoot', 200))        -- 射击子弹 prefab, speed, damage
                    :add(delay(1.0))
                    :add(finished())            -- 技能结束
                    )

        bt:setContext(actor)
        --log.info('create common bt! agent:'..tostring(agent.id))
        return bt
    end,
	
}